package com.avaje.ebean;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

























































































































































public final class RawSql
{
  private final Sql sql;
  private final ColumnMapping columnMapping;
  
  protected RawSql(Sql sql, ColumnMapping columnMapping)
  {
    this.sql = sql;
    this.columnMapping = columnMapping;
  }
  


  public Sql getSql()
  {
    return sql;
  }
  


  public ColumnMapping getColumnMapping()
  {
    return columnMapping;
  }
  


  public int queryHash()
  {
    return 31 * sql.queryHash() + columnMapping.queryHash();
  }
  


  public static final class Sql
  {
    private final boolean parsed;
    

    private final String unparsedSql;
    

    private final String preFrom;
    

    private final String preWhere;
    

    private final boolean andWhereExpr;
    
    private final String preHaving;
    
    private final boolean andHavingExpr;
    
    private final String orderBy;
    
    private final int queryHashCode;
    

    protected Sql(String unparsedSql)
    {
      queryHashCode = unparsedSql.hashCode();
      parsed = false;
      this.unparsedSql = unparsedSql;
      preFrom = null;
      preHaving = null;
      preWhere = null;
      andHavingExpr = false;
      andWhereExpr = false;
      orderBy = null;
    }
    




    protected Sql(int queryHashCode, String preFrom, String preWhere, boolean andWhereExpr, String preHaving, boolean andHavingExpr, String orderBy)
    {
      this.queryHashCode = queryHashCode;
      parsed = true;
      unparsedSql = null;
      this.preFrom = preFrom;
      this.preHaving = preHaving;
      this.preWhere = preWhere;
      this.andHavingExpr = andHavingExpr;
      this.andWhereExpr = andWhereExpr;
      this.orderBy = orderBy;
    }
    


    public int queryHash()
    {
      return queryHashCode;
    }
    
    public String toString() {
      if (!parsed) {
        return "unparsed[" + unparsedSql + "]";
      }
      return "select[" + preFrom + "] preWhere[" + preWhere + "] preHaving[" + preHaving + "] orderBy[" + orderBy + "]";
    }
    







    public boolean isParsed()
    {
      return parsed;
    }
    


    public String getUnparsedSql()
    {
      return unparsedSql;
    }
    


    public String getPreFrom()
    {
      return preFrom;
    }
    


    public String getPreWhere()
    {
      return preWhere;
    }
    



    public boolean isAndWhereExpr()
    {
      return andWhereExpr;
    }
    


    public String getPreHaving()
    {
      return preHaving;
    }
    



    public boolean isAndHavingExpr()
    {
      return andHavingExpr;
    }
    


    public String getOrderBy()
    {
      return orderBy;
    }
  }
  


  public static final class ColumnMapping
  {
    private final LinkedHashMap<String, Column> dbColumnMap;
    

    private final Map<String, String> propertyMap;
    

    private final Map<String, Column> propertyColumnMap;
    
    private final boolean parsed;
    
    private final boolean immutable;
    
    private final int queryHashCode;
    

    protected ColumnMapping(List<Column> columns)
    {
      queryHashCode = 0;
      immutable = false;
      parsed = true;
      propertyMap = null;
      propertyColumnMap = null;
      dbColumnMap = new LinkedHashMap();
      for (int i = 0; i < columns.size(); i++) {
        Column c = (Column)columns.get(i);
        dbColumnMap.put(c.getDbColumn(), c);
      }
    }
    


    protected ColumnMapping()
    {
      queryHashCode = 0;
      immutable = false;
      parsed = false;
      propertyMap = null;
      propertyColumnMap = null;
      dbColumnMap = new LinkedHashMap();
    }
    


    protected ColumnMapping(boolean parsed, LinkedHashMap<String, Column> dbColumnMap)
    {
      immutable = true;
      this.parsed = parsed;
      this.dbColumnMap = dbColumnMap;
      
      int hc = ColumnMapping.class.getName().hashCode();
      
      HashMap<String, Column> pcMap = new HashMap();
      HashMap<String, String> pMap = new HashMap();
      
      for (Column c : dbColumnMap.values()) {
        pMap.put(c.getPropertyName(), c.getDbColumn());
        pcMap.put(c.getPropertyName(), c);
        
        hc = 31 * hc + c.getPropertyName() == null ? 0 : c.getPropertyName().hashCode();
        hc = 31 * hc + c.getDbColumn() == null ? 0 : c.getDbColumn().hashCode();
      }
      propertyMap = Collections.unmodifiableMap(pMap);
      propertyColumnMap = Collections.unmodifiableMap(pcMap);
      queryHashCode = hc;
    }
    






    protected ColumnMapping createImmutableCopy()
    {
      for (Column c : dbColumnMap.values()) {
        c.checkMapping();
      }
      
      return new ColumnMapping(parsed, dbColumnMap);
    }
    
    protected void columnMapping(String dbColumn, String propertyName)
    {
      if (immutable) {
        throw new IllegalStateException("Should never happen");
      }
      if (!parsed) {
        int pos = dbColumnMap.size();
        dbColumnMap.put(dbColumn, new Column(pos, dbColumn, null, propertyName, null));
      } else {
        Column column = (Column)dbColumnMap.get(dbColumn);
        if (column == null) {
          String msg = "DB Column [" + dbColumn + "] not found in mapping. Expecting one of [" + dbColumnMap.keySet() + "]";
          
          throw new IllegalArgumentException(msg);
        }
        column.setPropertyName(propertyName);
      }
    }
    


    public int queryHash()
    {
      if (queryHashCode == 0) {
        throw new RuntimeException("Bug: queryHashCode == 0");
      }
      return queryHashCode;
    }
    








    public boolean isParsed()
    {
      return parsed;
    }
    


    public int size()
    {
      return dbColumnMap.size();
    }
    


    protected Map<String, Column> mapping()
    {
      return dbColumnMap;
    }
    


    public Map<String, String> getMapping()
    {
      return propertyMap;
    }
    


    public int getIndexPosition(String property)
    {
      Column c = (Column)propertyColumnMap.get(property);
      return c == null ? -1 : c.getIndexPos();
    }
    


    public Iterator<Column> getColumns()
    {
      return dbColumnMap.values().iterator();
    }
    


    public static class Column
    {
      private final int indexPos;
      

      private final String dbColumn;
      
      private final String dbAlias;
      
      private String propertyName;
      

      public Column(int indexPos, String dbColumn, String dbAlias)
      {
        this(indexPos, dbColumn, dbAlias, null);
      }
      
      private Column(int indexPos, String dbColumn, String dbAlias, String propertyName) {
        this.indexPos = indexPos;
        this.dbColumn = dbColumn;
        this.dbAlias = dbAlias;
        if ((propertyName == null) && (dbAlias != null)) {
          this.propertyName = dbAlias;
        } else {
          this.propertyName = propertyName;
        }
      }
      
      private void checkMapping() {
        if (propertyName == null) {
          String msg = "No propertyName defined (Column mapping) for dbColumn [" + dbColumn + "]";
          throw new IllegalStateException(msg);
        }
      }
      
      public String toString() {
        return dbColumn + "->" + propertyName;
      }
      


      public int getIndexPos()
      {
        return indexPos;
      }
      


      public String getDbColumn()
      {
        return dbColumn;
      }
      


      public String getDbAlias()
      {
        return dbAlias;
      }
      


      public String getPropertyName()
      {
        return propertyName;
      }
      
      private void setPropertyName(String propertyName) {
        this.propertyName = propertyName;
      }
    }
  }
}
