package com.avaje.ebean.common;

import com.avaje.ebean.Ebean;
import com.avaje.ebean.ExpressionList;
import com.avaje.ebean.bean.BeanCollection;
import com.avaje.ebean.bean.BeanCollection.ModifyListenMode;
import com.avaje.ebean.bean.BeanCollectionLoader;
import com.avaje.ebean.bean.BeanCollectionTouched;
import com.avaje.ebean.bean.EntityBean;
import com.avaje.ebean.bean.EntityBeanIntercept;
import java.util.Set;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import javax.persistence.PersistenceException;




















































public abstract class AbstractBeanCollection<E>
  implements BeanCollection<E>
{
  private static final long serialVersionUID = 3365725236140187588L;
  protected int state;
  protected transient BeanCollectionLoader loader;
  protected transient ExpressionList<?> filterMany;
  protected int loaderIndex;
  protected String ebeanServerName;
  protected transient BeanCollectionTouched beanCollectionTouched;
  protected transient Future<Integer> fetchFuture;
  protected final Object ownerBean;
  protected final String propertyName;
  protected boolean finishedFetch = true;
  

  protected boolean hasMoreRows;
  

  protected ModifyHolder<E> modifyHolder;
  

  protected BeanCollection.ModifyListenMode modifyListenMode;
  
  protected boolean modifyAddListening;
  
  protected boolean modifyRemoveListening;
  
  protected boolean modifyListening;
  

  public AbstractBeanCollection()
  {
    ownerBean = null;
    propertyName = null;
  }
  


  public AbstractBeanCollection(BeanCollectionLoader loader, Object ownerBean, String propertyName)
  {
    this.loader = loader;
    ebeanServerName = loader.getName();
    this.ownerBean = ownerBean;
    this.propertyName = propertyName;
    
    if ((ownerBean instanceof EntityBean)) {
      EntityBeanIntercept ebi = ((EntityBean)ownerBean)._ebean_getIntercept();
      int parentState = ebi.getState();
      if (parentState != 0) {
        state = parentState;
      }
    }
  }
  
  public Object getOwnerBean() {
    return ownerBean;
  }
  
  public String getPropertyName() {
    return propertyName;
  }
  
  public int getLoaderIndex() {
    return loaderIndex;
  }
  
  public ExpressionList<?> getFilterMany() {
    return filterMany;
  }
  
  public void setFilterMany(ExpressionList<?> filterMany) {
    this.filterMany = filterMany;
  }
  
  protected void lazyLoadCollection(boolean onlyIds) {
    if (loader == null) {
      loader = ((BeanCollectionLoader)Ebean.getServer(ebeanServerName));
    }
    if (loader == null) {
      String msg = "Lazy loading but LazyLoadEbeanServer is null? The LazyLoadEbeanServer needs to be set after deserialization to support lazy loading.";
      

      throw new PersistenceException(msg);
    }
    
    loader.loadMany(this, onlyIds);
    checkEmptyLazyLoad();
  }
  
  protected void touched() {
    if (beanCollectionTouched != null)
    {
      beanCollectionTouched.notifyTouched(this);
      beanCollectionTouched = null;
    }
  }
  
  public void setBeanCollectionTouched(BeanCollectionTouched notify) {
    beanCollectionTouched = notify;
  }
  
  public void setLoader(int beanLoaderIndex, BeanCollectionLoader loader) {
    loaderIndex = beanLoaderIndex;
    this.loader = loader;
    ebeanServerName = loader.getName();
  }
  
  public boolean isSharedInstance()
  {
    return state == 3;
  }
  
  public void setSharedInstance() {
    state = 3;
  }
  
  public boolean isReadOnly()
  {
    return state >= 2;
  }
  
  public void setReadOnly(boolean readOnly) {
    if (state == 3) {
      if (!readOnly) {
        String msg = "This collection is a sharedInstance and must always be read only";
        throw new IllegalStateException(msg);
      }
    } else {
      state = (readOnly ? 2 : 0);
    }
  }
  









  public boolean hasMoreRows()
  {
    return hasMoreRows;
  }
  




  public void setHasMoreRows(boolean hasMoreRows)
  {
    this.hasMoreRows = hasMoreRows;
  }
  



  public boolean isFinishedFetch()
  {
    return finishedFetch;
  }
  



  public void setFinishedFetch(boolean finishedFetch)
  {
    this.finishedFetch = finishedFetch;
  }
  
  public void setBackgroundFetch(Future<Integer> fetchFuture) {
    this.fetchFuture = fetchFuture;
  }
  
  public void backgroundFetchWait(long wait, TimeUnit timeUnit) {
    if (fetchFuture != null) {
      try {
        fetchFuture.get(wait, timeUnit);
      } catch (Exception e) {
        throw new PersistenceException(e);
      }
    }
  }
  
  public void backgroundFetchWait() {
    if (fetchFuture != null) {
      try {
        fetchFuture.get();
      } catch (Exception e) {
        throw new PersistenceException(e);
      }
    }
  }
  
  protected void checkReadOnly()
  {
    if (state >= 2) {
      String msg = "This collection is in ReadOnly mode";
      throw new IllegalStateException(msg);
    }
  }
  










  public void setModifyListening(BeanCollection.ModifyListenMode mode)
  {
    modifyListenMode = mode;
    modifyAddListening = BeanCollection.ModifyListenMode.ALL.equals(mode);
    modifyRemoveListening = ((modifyAddListening) || (BeanCollection.ModifyListenMode.REMOVALS.equals(mode)));
    modifyListening = ((modifyRemoveListening) || (modifyAddListening));
    if (modifyListening)
    {
      modifyHolder = null;
    }
  }
  


  public BeanCollection.ModifyListenMode getModifyListenMode()
  {
    return modifyListenMode;
  }
  
  protected ModifyHolder<E> getModifyHolder() {
    if (modifyHolder == null) {
      modifyHolder = new ModifyHolder();
    }
    return modifyHolder;
  }
  
  public void modifyAddition(E bean) {
    if (modifyAddListening) {
      getModifyHolder().modifyAddition(bean);
    }
  }
  
  public void modifyRemoval(Object bean) {
    if (modifyRemoveListening) {
      getModifyHolder().modifyRemoval(bean);
    }
  }
  
  public void modifyReset() {
    if (modifyHolder != null) {
      modifyHolder.reset();
    }
  }
  
  public Set<E> getModifyAdditions() {
    if (modifyHolder == null) {
      return null;
    }
    return modifyHolder.getModifyAdditions();
  }
  
  public Set<E> getModifyRemovals()
  {
    if (modifyHolder == null) {
      return null;
    }
    return modifyHolder.getModifyRemovals();
  }
}
