package com.avaje.ebean.common;

import com.avaje.ebean.bean.BeanCollectionLoader;
import com.avaje.ebean.bean.SerializeControl;
import java.io.ObjectStreamException;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;


























public final class BeanMap<K, E>
  extends AbstractBeanCollection<E>
  implements Map<K, E>
{
  private Map<K, E> map;
  
  public BeanMap(Map<K, E> map)
  {
    this.map = map;
  }
  


  public BeanMap()
  {
    this(new LinkedHashMap());
  }
  
  public BeanMap(BeanCollectionLoader ebeanServer, Object ownerBean, String propertyName) {
    super(ebeanServer, ownerBean, propertyName);
  }
  
  Object readResolve() throws ObjectStreamException {
    if (SerializeControl.isVanillaCollections()) {
      return map;
    }
    return this;
  }
  
  Object writeReplace() throws ObjectStreamException {
    if (SerializeControl.isVanillaCollections()) {
      return map;
    }
    return this;
  }
  
  public void internalAdd(Object bean) {
    throw new RuntimeException("Not allowed for map");
  }
  



  public boolean isPopulated()
  {
    return map != null;
  }
  



  public boolean isReference()
  {
    return map == null;
  }
  
  public boolean checkEmptyLazyLoad() {
    if (map == null) {
      map = new LinkedHashMap();
      return true;
    }
    return false;
  }
  
  private void initClear()
  {
    synchronized (this) {
      if (map == null) {
        if (modifyListening) {
          lazyLoadCollection(true);
        } else {
          map = new LinkedHashMap();
        }
      }
      touched();
    }
  }
  
  private void init() {
    synchronized (this) {
      if (map == null) {
        lazyLoadCollection(false);
      }
      touched();
    }
  }
  



  public void setActualMap(Map<?, ?> map)
  {
    this.map = map;
  }
  


  public Map<K, E> getActualMap()
  {
    return map;
  }
  






  public Collection<E> getActualDetails()
  {
    return map.values();
  }
  


  public Object getActualCollection()
  {
    return map;
  }
  
  public String toString() {
    StringBuffer sb = new StringBuffer();
    sb.append("BeanMap ");
    if (isSharedInstance()) {
      sb.append("sharedInstance ");
    } else if (isReadOnly()) {
      sb.append("readOnly ");
    }
    if (map == null) {
      sb.append("deferred ");
    }
    else {
      sb.append("size[").append(map.size()).append("]");
      sb.append(" hasMoreRows[").append(hasMoreRows).append("]");
      sb.append(" map").append(map);
    }
    return sb.toString();
  }
  


  public boolean equals(Object obj)
  {
    init();
    return map.equals(obj);
  }
  
  public int hashCode() {
    init();
    return map.hashCode();
  }
  
  public void clear() {
    checkReadOnly();
    initClear();
    if (modifyRemoveListening) {
      for (K key : map.keySet()) {
        E o = map.remove(key);
        modifyRemoval(o);
      }
    }
    map.clear();
  }
  
  public boolean containsKey(Object key) {
    init();
    return map.containsKey(key);
  }
  
  public boolean containsValue(Object value) {
    init();
    return map.containsValue(value);
  }
  
  public Set<Map.Entry<K, E>> entrySet()
  {
    init();
    if (isReadOnly()) {
      return Collections.unmodifiableSet(map.entrySet());
    }
    if (modifyListening) {
      Set<Map.Entry<K, E>> s = map.entrySet();
      return new ModifySet(this, s);
    }
    return map.entrySet();
  }
  
  public E get(Object key) {
    init();
    return map.get(key);
  }
  
  public boolean isEmpty() {
    init();
    return map.isEmpty();
  }
  
  public Set<K> keySet() {
    init();
    if (isReadOnly()) {
      return Collections.unmodifiableSet(map.keySet());
    }
    
    return map.keySet();
  }
  
  public E put(K key, E value) {
    checkReadOnly();
    init();
    if (modifyListening) {
      Object o = map.put(key, value);
      modifyAddition(value);
      modifyRemoval(o);
    }
    return map.put(key, value);
  }
  
  public void putAll(Map<? extends K, ? extends E> t)
  {
    checkReadOnly();
    init();
    if (modifyListening) {
      Iterator it = t.entrySet().iterator();
      while (it.hasNext()) {
        Map.Entry entry = (Map.Entry)it.next();
        Object o = map.put(entry.getKey(), entry.getValue());
        modifyAddition(entry.getValue());
        modifyRemoval(o);
      }
    }
    map.putAll(t);
  }
  
  public E remove(Object key) {
    checkReadOnly();
    init();
    if (modifyRemoveListening) {
      E o = map.remove(key);
      modifyRemoval(o);
      return o;
    }
    return map.remove(key);
  }
  
  public int size() {
    init();
    return map.size();
  }
  
  public Collection<E> values() {
    init();
    if (isReadOnly()) {
      return Collections.unmodifiableCollection(map.values());
    }
    if (modifyListening) {
      Collection<E> c = map.values();
      return new ModifyCollection(this, c);
    }
    return map.values();
  }
}
