package com.avaje.ebean.config.lucene;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.document.Field.Index;
import org.apache.lucene.document.Field.Store;
















public class IndexFieldDefn
{
  protected final String name;
  protected String propertyName;
  protected Field.Index index;
  protected Field.Store store;
  protected Sortable sortable;
  
  public static enum Sortable
  {
    YES, 
    




    DEFAULT;
    




    private Sortable() {}
  }
  




  protected int precisionStep = -1;
  
  protected float boost;
  
  protected Analyzer queryAnalyzer;
  
  protected Analyzer indexAnalyzer;
  protected String[] properties;
  
  public IndexFieldDefn(String name)
  {
    this.name = name;
    propertyName = name;
  }
  
  public IndexFieldDefn(String name, Field.Store store, Field.Index index, Sortable sortable) {
    this(name);
    this.store = store;
    this.index = index;
    this.sortable = sortable;
  }
  
  public String toString() {
    return name;
  }
  






  public IndexFieldDefn copyField(String name)
  {
    IndexFieldDefn copy = new IndexFieldDefn(name, store, index, sortable);
    copy.setPropertyName(name);
    
    copy.setIndexAnalyzer(indexAnalyzer);
    copy.setQueryAnalyzer(queryAnalyzer);
    copy.setPrecisionStep(precisionStep);
    copy.setBoost(boost);
    
    return copy;
  }
  


  public IndexFieldDefn copyFieldConcat(String fieldName, String[] properties)
  {
    IndexFieldDefn copy = copyField(fieldName);
    copy.setPropertyName(null);
    copy.setPropertyNames(properties);
    return copy;
  }
  


  public String getName()
  {
    return name;
  }
  


  public String getPropertyName()
  {
    return propertyName;
  }
  


  public IndexFieldDefn setPropertyName(String propertyName)
  {
    this.propertyName = propertyName;
    return this;
  }
  


  public Field.Index getIndex()
  {
    return index;
  }
  


  public IndexFieldDefn setIndex(Field.Index index)
  {
    this.index = index;
    return this;
  }
  


  public Field.Store getStore()
  {
    return store;
  }
  


  public IndexFieldDefn setStore(Field.Store store)
  {
    this.store = store;
    return this;
  }
  


  public Sortable getSortable()
  {
    return sortable;
  }
  


  public IndexFieldDefn setSortable(Sortable sortable)
  {
    this.sortable = sortable;
    return this;
  }
  


  public int getPrecisionStep()
  {
    return precisionStep;
  }
  


  public IndexFieldDefn setPrecisionStep(int precisionStep)
  {
    this.precisionStep = precisionStep;
    return this;
  }
  


  public float getBoost()
  {
    return boost;
  }
  


  public void setBoost(float boost)
  {
    this.boost = boost;
  }
  


  public Analyzer getQueryAnalyzer()
  {
    return queryAnalyzer;
  }
  


  public Analyzer getIndexAnalyzer()
  {
    return indexAnalyzer;
  }
  


  public IndexFieldDefn setQueryAnalyzer(Analyzer queryAnalyzer)
  {
    this.queryAnalyzer = queryAnalyzer;
    return this;
  }
  


  public IndexFieldDefn setIndexAnalyzer(Analyzer indexAnalyzer)
  {
    this.indexAnalyzer = indexAnalyzer;
    return this;
  }
  


  public IndexFieldDefn setBothAnalyzers(Analyzer analyzer)
  {
    queryAnalyzer = analyzer;
    indexAnalyzer = analyzer;
    return this;
  }
  


  public String[] getPropertyNames()
  {
    return properties;
  }
  


  public void setPropertyNames(String[] properties)
  {
    this.properties = properties;
  }
}
