package com.avaje.ebean.enhance.agent;

import com.avaje.ebean.enhance.asm.AnnotationVisitor;
import com.avaje.ebean.enhance.asm.ClassAdapter;
import com.avaje.ebean.enhance.asm.ClassVisitor;
import com.avaje.ebean.enhance.asm.FieldVisitor;
import com.avaje.ebean.enhance.asm.MethodVisitor;












public class ClassAdpaterEntity
  extends ClassAdapter
  implements EnhanceConstants
{
  private final EnhanceContext enhanceContext;
  private final ClassLoader classLoader;
  private final ClassMeta classMeta;
  private boolean firstMethod = true;
  
  public ClassAdpaterEntity(ClassVisitor cv, ClassLoader classLoader, EnhanceContext context) {
    super(cv);
    this.classLoader = classLoader;
    enhanceContext = context;
    classMeta = context.createClassMeta();
  }
  


  public void logEnhanced()
  {
    classMeta.logEnhanced();
  }
  
  public boolean isLog(int level) {
    return classMeta.isLog(level);
  }
  
  public void log(String msg) {
    classMeta.log(msg);
  }
  




  public void visit(int version, int access, String name, String signature, String superName, String[] interfaces)
  {
    classMeta.setClassName(name, superName);
    
    int n = 1 + interfaces.length;
    String[] c = new String[n];
    for (int i = 0; i < interfaces.length; i++) {
      c[i] = interfaces[i];
      if (c[i].equals("com/avaje/ebean/bean/EntityBean")) {
        classMeta.setEntityBeanInterface(true);
      }
      if (c[i].equals("scala/ScalaObject")) {
        classMeta.setScalaInterface(true);
      }
      if (c[i].equals("groovy/lang/GroovyObject")) {
        classMeta.setGroovyInterface(true);
      }
    }
    
    if (classMeta.hasEntityBeanInterface())
    {
      c = interfaces;
    }
    else {
      c[(c.length - 1)] = "com/avaje/ebean/bean/EntityBean";
    }
    
    if (!superName.equals("java/lang/Object"))
    {
      if (classMeta.isLog(7)) {
        classMeta.log("read information about superClasses " + superName + " to see if it is entity/embedded/mappedSuperclass");
      }
      
      ClassMeta superMeta = enhanceContext.getSuperMeta(superName, classLoader);
      if ((superMeta != null) && (superMeta.isEntity()))
      {
        classMeta.setSuperMeta(superMeta);
        if (classMeta.isLog(1)) {
          classMeta.log("entity extends " + superMeta.getDescription());
        }
      }
      else if (classMeta.isLog(7)) {
        if (superMeta == null) {
          classMeta.log("unable to read superMeta for " + superName);
        } else {
          classMeta.log("superMeta " + superName + " is not an entity/embedded/mappedsuperclass " + superMeta.getClassAnnotations());
        }
      }
    }
    

    super.visit(version, access, name, signature, superName, c);
  }
  
  public AnnotationVisitor visitAnnotation(String desc, boolean visible)
  {
    classMeta.addClassAnnotation(desc);
    return super.visitAnnotation(desc, visible);
  }
  







  private boolean isEbeanFieldMarker(String name, String desc, String signature)
  {
    if (name.equals("_EBEAN_MARKER")) {
      if (!desc.equals("Ljava/lang/String;")) {
        String m = "Error: _EBEAN_MARKER field of wrong type? " + desc;
        classMeta.log(m);
      }
      return true;
    }
    return false;
  }
  
  private boolean isPropertyChangeListenerField(String name, String desc, String signature) {
    if (desc.equals("Ljava/beans/PropertyChangeSupport;")) {
      return true;
    }
    return false;
  }
  





  public FieldVisitor visitField(int access, String name, String desc, String signature, Object value)
  {
    if ((access & 0x8) != 0)
    {
      if (isEbeanFieldMarker(name, desc, signature)) {
        classMeta.setAlreadyEnhanced(true);
        if (isLog(2)) {
          log("Found ebean marker field " + name + " " + value);
        }
      }
      else if (isLog(2)) {
        log("Skip intercepting static field " + name);
      }
      


      return super.visitField(access, name, desc, signature, value);
    }
    
    if (isPropertyChangeListenerField(name, desc, signature))
    {
      if (isLog(1)) {
        classMeta.log("Found existing PropertyChangeSupport field " + name);
      }
      
      return super.visitField(access, name, desc, signature, value);
    }
    
    if ((access & 0x80) != 0) {
      if (isLog(2)) {
        log("Skip intercepting transient field " + name);
      }
      
      return super.visitField(access, name, desc, signature, value);
    }
    



    FieldVisitor fv = super.visitField(access, name, desc, signature, value);
    
    return classMeta.createLocalFieldVisitor(cv, fv, name, desc);
  }
  




  public MethodVisitor visitMethod(int access, String name, String desc, String signature, String[] exceptions)
  {
    if (firstMethod) {
      if (!classMeta.isEntityEnhancementRequired())
      {
        throw new NoEnhancementRequiredException();
      }
      
      if (classMeta.hasEntityBeanInterface()) {
        log("Enhancing when EntityBean interface already exists!");
      }
      

      String marker = MarkerField.addField(cv, classMeta.getClassName());
      if (isLog(4)) {
        log("... add marker field \"" + marker + "\"");
      }
      
      if (!classMeta.isSuperClassEntity())
      {

        if (isLog(4)) {
          log("... add intercept and identity fields");
        }
        InterceptField.addField(cv, enhanceContext.isTransientInternalFields());
        MethodEquals.addIdentityField(cv);
      }
      
      firstMethod = false;
    }
    

    classMeta.addExistingMethod(name, desc);
    
    if (isDefaultConstructor(name, desc))
    {
      MethodVisitor mv = super.visitMethod(1, name, desc, signature, exceptions);
      
      return new ConstructorAdapter(mv, classMeta, desc);
    }
    
    MethodVisitor mv = super.visitMethod(access, name, desc, signature, exceptions);
    
    if (interceptEntityMethod(access, name, desc, signature, exceptions))
    {

      return new MethodFieldAdapter(mv, classMeta, name + " " + desc);
    }
    

    return mv;
  }
  




  public void visitEnd()
  {
    if (!classMeta.isEntityEnhancementRequired()) {
      throw new NoEnhancementRequiredException();
    }
    
    if (!classMeta.hasDefaultConstructor()) {
      DefaultConstructor.add(cv, classMeta);
    }
    
    MarkerField.addGetMarker(cv, classMeta.getClassName());
    
    if (!classMeta.isSuperClassEntity())
    {
      InterceptField.addGetterSetter(cv, classMeta.getClassName());
      

      MethodPropertyChangeListener.addMethod(cv, classMeta);
    }
    


    classMeta.addFieldGetSetMethods(cv);
    

    IndexFieldWeaver.addMethods(cv, classMeta);
    
    MethodSetEmbeddedLoaded.addMethod(cv, classMeta);
    MethodIsEmbeddedNewOrDirty.addMethod(cv, classMeta);
    MethodNewInstance.addMethod(cv, classMeta);
    

    enhanceContext.addClassMeta(classMeta);
    
    super.visitEnd();
  }
  
  private boolean isDefaultConstructor(String name, String desc)
  {
    if (name.equals("<init>")) {
      if (desc.equals("()V")) {
        classMeta.setHasDefaultConstructor(true);
      }
      return true;
    }
    
    return false;
  }
  

  private boolean interceptEntityMethod(int access, String name, String desc, String signature, String[] exceptions)
  {
    if ((access & 0x8) != 0)
    {
      if (isLog(2)) {
        log("Skip intercepting static method " + name);
      }
      return false;
    }
    
    if ((name.equals("hashCode")) && (desc.equals("()I"))) {
      classMeta.setHasEqualsOrHashcode(true);
      return true;
    }
    
    if ((name.equals("equals")) && (desc.equals("(Ljava/lang/Object;)Z"))) {
      classMeta.setHasEqualsOrHashcode(true);
      return true;
    }
    
    if ((name.equals("toString")) && (desc.equals("()Ljava/lang/String;")))
    {

      return false;
    }
    
    return true;
  }
}
