package com.avaje.ebean.enhance.agent;

import com.avaje.ebean.enhance.asm.MethodAdapter;
import com.avaje.ebean.enhance.asm.MethodVisitor;
import com.avaje.ebean.enhance.asm.Opcodes;
import java.io.PrintStream;









public class ConstructorAdapter
  extends MethodAdapter
  implements EnhanceConstants, Opcodes
{
  private final ClassMeta meta;
  private final String className;
  private final String constructorDesc;
  private boolean constructorInitializationDone;
  
  public ConstructorAdapter(MethodVisitor mv, ClassMeta meta, String constructorDesc)
  {
    super(mv);
    this.meta = meta;
    className = meta.getClassName();
    this.constructorDesc = constructorDesc;
  }
  

  public void visitMethodInsn(int opcode, String owner, String name, String desc)
  {
    super.visitMethodInsn(opcode, owner, name, desc);
    addInitialisationIfRequired(opcode, owner, name, desc);
  }
  







  public void addInitialisationIfRequired(int opcode, String owner, String name, String desc)
  {
    if ((opcode == 183) && (name.equals("<init>")) && (desc.equals("()V"))) {
      if (meta.isSuperClassEntity()) {
        if (meta.isLog(3)) {
          meta.log("... skipping intercept <init> ... handled by super class... CONSTRUCTOR:" + constructorDesc);
        }
      }
      else if (owner.equals(meta.getClassName())) {
        if (meta.isLog(3)) {
          meta.log("... skipping intercept <init> ... handled by other constructor... CONSTRUCTOR:" + constructorDesc);
        }
      }
      else if (owner.equals(meta.getSuperClassName())) {
        if (meta.isLog(2)) {
          meta.log("... adding intercept <init> in CONSTRUCTOR:" + constructorDesc + " OWNER/SUPER:" + owner);
        }
        
        if (constructorInitializationDone)
        {

          String msg = "Error in Enhancement. Only expecting to add <init> of intercept object once but it is trying to add it twice for " + meta.getClassName() + " CONSTRUCTOR:" + constructorDesc + " OWNER:" + owner;
          

          System.err.println(msg);
        }
        else
        {
          super.visitVarInsn(25, 0);
          super.visitTypeInsn(187, "com/avaje/ebean/bean/EntityBeanIntercept");
          super.visitInsn(89);
          super.visitVarInsn(25, 0);
          
          super.visitMethodInsn(183, "com/avaje/ebean/bean/EntityBeanIntercept", "<init>", "(Ljava/lang/Object;)V");
          super.visitFieldInsn(181, className, "_ebean_intercept", "Lcom/avaje/ebean/bean/EntityBeanIntercept;");
          
          constructorInitializationDone = true;
        }
      }
      else if (meta.isLog(3)) {
        meta.log("... skipping intercept <init> ... incorrect type " + owner);
      }
    }
  }
}
