package com.avaje.ebean.enhance.agent;

import com.avaje.ebean.enhance.asm.ClassVisitor;
import com.avaje.ebean.enhance.asm.Label;
import com.avaje.ebean.enhance.asm.MethodVisitor;
import com.avaje.ebean.enhance.asm.Opcodes;
import com.avaje.ebean.enhance.asm.Type;
import java.io.PrintStream;
import java.util.HashSet;






public class FieldMeta
  implements Opcodes, EnhanceConstants
{
  private static final Type BOOLEAN_OBJECT_TYPE = Type.getType(Boolean.class);
  
  private final ClassMeta classMeta;
  
  private final String fieldClass;
  private final String fieldName;
  private final String fieldDesc;
  private final HashSet<String> annotations = new HashSet();
  
  private final Type asmType;
  
  private final boolean primativeType;
  
  private final boolean objectType;
  
  private final String getMethodName;
  
  private final String getMethodDesc;
  
  private final String setMethodName;
  
  private final String setMethodDesc;
  
  private final String getNoInterceptMethodName;
  
  private final String setNoInterceptMethodName;
  private final String publicSetterName;
  private final String publicGetterName;
  
  public FieldMeta(ClassMeta classMeta, String name, String desc, String fieldClass)
  {
    this.classMeta = classMeta;
    fieldName = name;
    fieldDesc = desc;
    this.fieldClass = fieldClass;
    
    asmType = Type.getType(desc);
    
    int sort = asmType.getSort();
    primativeType = ((sort > 0) && (sort <= 8));
    objectType = (sort == 10);
    
    getMethodName = ("_ebean_get_" + name);
    getMethodDesc = ("()" + desc);
    
    setMethodName = ("_ebean_set_" + name);
    setMethodDesc = ("(" + desc + ")V");
    
    getNoInterceptMethodName = ("_ebean_getni_" + name);
    setNoInterceptMethodName = ("_ebean_setni_" + name);
    
    if ((classMeta != null) && (classMeta.hasScalaInterface()))
    {
      publicSetterName = (name + "_$eq");
      publicGetterName = name;
    }
    else {
      String publicFieldName = getFieldName(name, asmType);
      
      String initCap = Character.toUpperCase(publicFieldName.charAt(0)) + publicFieldName.substring(1);
      publicSetterName = ("set" + initCap);
      
      if (fieldDesc.equals("Z")) {
        publicGetterName = ("is" + initCap);
      } else {
        publicGetterName = ("get" + initCap);
      }
    }
    
    if ((classMeta != null) && (classMeta.isLog(6))) {
      classMeta.log(" ... public getter [" + publicGetterName + "]");
      classMeta.log(" ... public setter [" + publicSetterName + "]");
    }
  }
  


  private String getFieldName(String name, Type asmType)
  {
    if (((BOOLEAN_OBJECT_TYPE.equals(asmType)) || (Type.BOOLEAN_TYPE.equals(asmType))) && (name.startsWith("is")) && (name.length() > 2))
    {


      char c = name.charAt(2);
      if (Character.isUpperCase(c)) {
        if (classMeta.isLog(6)) {
          classMeta.log("trimming off \"is\" from boolean field name " + name + "]");
        }
        return name.substring(2);
      }
    }
    return name;
  }
  
  public String toString() {
    return fieldName;
  }
  


  public String getFieldName()
  {
    return fieldName;
  }
  


  public boolean isPrimativeType()
  {
    return primativeType;
  }
  





  public String getPublicGetterName()
  {
    return publicGetterName;
  }
  





  public String getPublicSetterName()
  {
    return publicSetterName;
  }
  



  public boolean isPersistentSetter(String methodDesc)
  {
    return (setMethodDesc.equals(methodDesc)) && (isInterceptSet());
  }
  



  public boolean isPersistentGetter(String methodDesc)
  {
    return (getMethodDesc.equals(methodDesc)) && (isInterceptGet());
  }
  


  protected void addAnnotationDesc(String desc)
  {
    annotations.add(desc);
  }
  


  public String getName()
  {
    return fieldName;
  }
  


  public String getDesc()
  {
    return fieldDesc;
  }
  
  private boolean isInterceptGet() {
    if (isId()) {
      return false;
    }
    if (isTransient()) {
      return false;
    }
    if (isMany()) {
      return true;
    }
    return true;
  }
  
  private boolean isInterceptSet() {
    if (isId()) {
      return false;
    }
    if (isTransient()) {
      return false;
    }
    if (isMany()) {
      return false;
    }
    return true;
  }
  





  public boolean isObjectArray()
  {
    if ((fieldDesc.charAt(0) == '[') && 
      (fieldDesc.length() > 2)) {
      if (!isTransient()) {
        System.err.println("ERROR: We can not support Object Arrays... for field: " + fieldName);
      }
      return true;
    }
    
    return false;
  }
  


  public boolean isPersistent()
  {
    return !isTransient();
  }
  


  public boolean isTransient()
  {
    return annotations.contains("Ljavax/persistence/Transient;");
  }
  





  public boolean isId()
  {
    boolean idField = (annotations.contains("Ljavax/persistence/Id;")) || (annotations.contains("Ljavax/persistence/EmbeddedId;"));
    


    return idField;
  }
  


  public boolean isMany()
  {
    return (annotations.contains("Ljavax/persistence/OneToMany;")) || (annotations.contains("Ljavax/persistence/ManyToMany;"));
  }
  
  public boolean isManyToMany()
  {
    return annotations.contains("Ljavax/persistence/ManyToMany;");
  }
  


  public boolean isEmbedded()
  {
    return (annotations.contains("Ljavax/persistence/Embedded;")) || (annotations.contains("Lcom/avaje/ebean/annotation/EmbeddedColumns;"));
  }
  




  private boolean isLocalField(ClassMeta classMeta)
  {
    return fieldClass.equals(classMeta.getClassName());
  }
  


  public void appendGetPrimitiveIdValue(MethodVisitor mv, ClassMeta classMeta)
  {
    if (classMeta.isSubclassing()) {
      mv.visitMethodInsn(182, classMeta.getSuperClassName(), publicGetterName, getMethodDesc);
    } else {
      mv.visitMethodInsn(182, classMeta.getClassName(), getMethodName, getMethodDesc);
    }
  }
  


  public void appendCompare(MethodVisitor mv, ClassMeta classMeta)
  {
    if (primativeType) {
      if (classMeta.isLog(4)) {
        classMeta.log(" ... getIdentity compare primitive field[" + fieldName + "] type[" + fieldDesc + "]");
      }
      if (fieldDesc.equals("J"))
      {
        mv.visitInsn(9);
        mv.visitInsn(148);
      }
      else if (fieldDesc.equals("D"))
      {
        mv.visitInsn(14);
        mv.visitInsn(151);
      }
      else if (fieldDesc.equals("F"))
      {
        mv.visitInsn(11);
        mv.visitInsn(149);
      }
    }
  }
  









  public void appendValueOf(MethodVisitor mv, ClassMeta classMeta)
  {
    if (primativeType)
    {
      Type objectWrapperType = PrimitiveHelper.getObjectWrapper(asmType);
      
      String objDesc = objectWrapperType.getInternalName();
      String primDesc = asmType.getDescriptor();
      
      mv.visitMethodInsn(184, objDesc, "valueOf", "(" + primDesc + ")L" + objDesc + ";");
    }
  }
  



  public void addFieldCopy(MethodVisitor mv, ClassMeta classMeta)
  {
    if (classMeta.isSubclassing()) {
      String copyClassName = classMeta.getSuperClassName();
      mv.visitMethodInsn(183, copyClassName, publicGetterName, getMethodDesc);
      mv.visitMethodInsn(182, copyClassName, publicSetterName, setMethodDesc);
    }
    else if (isLocalField(classMeta)) {
      mv.visitFieldInsn(180, fieldClass, fieldName, fieldDesc);
      mv.visitFieldInsn(181, fieldClass, fieldName, fieldDesc);
    } else {
      if (classMeta.isLog(4)) {
        classMeta.log(" ... addFieldCopy on non-local field [" + fieldName + "] type[" + fieldDesc + "]");
      }
      mv.visitMethodInsn(182, classMeta.getClassName(), getNoInterceptMethodName, getMethodDesc);
      mv.visitMethodInsn(182, classMeta.getClassName(), setNoInterceptMethodName, setMethodDesc);
    }
  }
  




  public void appendSwitchGet(MethodVisitor mv, ClassMeta classMeta, boolean intercept)
  {
    if (classMeta.isSubclassing())
    {
      if (intercept)
      {
        mv.visitMethodInsn(182, classMeta.getClassName(), publicGetterName, getMethodDesc);
      }
      else {
        mv.visitMethodInsn(183, classMeta.getSuperClassName(), publicGetterName, getMethodDesc);
      }
    }
    else if (intercept)
    {
      mv.visitMethodInsn(182, classMeta.getClassName(), getMethodName, getMethodDesc);
    }
    else if (isLocalField(classMeta)) {
      mv.visitFieldInsn(180, classMeta.getClassName(), fieldName, fieldDesc);
    }
    else {
      mv.visitMethodInsn(182, classMeta.getClassName(), getNoInterceptMethodName, getMethodDesc);
    }
    


    if (primativeType) {
      appendValueOf(mv, classMeta);
    }
  }
  
  public void appendSwitchSet(MethodVisitor mv, ClassMeta classMeta, boolean intercept)
  {
    if (primativeType)
    {
      Type objectWrapperType = PrimitiveHelper.getObjectWrapper(asmType);
      
      String primDesc = asmType.getDescriptor();
      String primType = asmType.getClassName();
      String objInt = objectWrapperType.getInternalName();
      mv.visitTypeInsn(192, objInt);
      
      mv.visitMethodInsn(182, objInt, primType + "Value", "()" + primDesc);
    }
    else {
      mv.visitTypeInsn(192, asmType.getInternalName());
    }
    
    if (classMeta.isSubclassing()) {
      if (intercept)
      {
        mv.visitMethodInsn(182, classMeta.getClassName(), publicSetterName, setMethodDesc);
      }
      else {
        mv.visitMethodInsn(183, classMeta.getSuperClassName(), publicSetterName, setMethodDesc);
      }
      
    }
    else if (intercept)
    {
      mv.visitMethodInsn(182, classMeta.getClassName(), setMethodName, setMethodDesc);

    }
    else if (isLocalField(classMeta)) {
      mv.visitFieldInsn(181, fieldClass, fieldName, fieldDesc);
    } else {
      mv.visitMethodInsn(182, classMeta.getClassName(), setNoInterceptMethodName, setMethodDesc);
    }
  }
  





  public void addPublicGetSetMethods(ClassVisitor cv, ClassMeta classMeta, boolean checkExisting)
  {
    if (isPersistent())
    {
      if (isId())
      {

        addPublicSetMethod(cv, classMeta, checkExisting);
      }
      else {
        addPublicGetMethod(cv, classMeta, checkExisting);
        addPublicSetMethod(cv, classMeta, checkExisting);
      }
    }
  }
  
  private void addPublicGetMethod(ClassVisitor cv, ClassMeta classMeta, boolean checkExisting)
  {
    if ((checkExisting) && (!classMeta.isExistingSuperMethod(publicGetterName, getMethodDesc))) {
      if (classMeta.isLog(1)) {
        classMeta.log("excluding " + publicGetterName + " as not on super object");
      }
      return;
    }
    
    addPublicGetMethod(new VisitMethodParams(cv, 1, publicGetterName, getMethodDesc, null, null), classMeta);
  }
  
  private void addPublicGetMethod(VisitMethodParams params, ClassMeta classMeta)
  {
    MethodVisitor mv = params.visitMethod();
    int iReturnOpcode = asmType.getOpcode(172);
    
    mv.visitCode();
    Label l0 = new Label();
    mv.visitLabel(l0);
    mv.visitLineNumber(1, l0);
    mv.visitVarInsn(25, 0);
    mv.visitFieldInsn(180, classMeta.getClassName(), "_ebean_intercept", "Lcom/avaje/ebean/bean/EntityBeanIntercept;");
    mv.visitLdcInsn(fieldName);
    mv.visitMethodInsn(182, "com/avaje/ebean/bean/EntityBeanIntercept", "preGetter", "(Ljava/lang/String;)V");
    Label l1 = new Label();
    mv.visitLabel(l1);
    mv.visitLineNumber(1, l1);
    mv.visitVarInsn(25, 0);
    mv.visitMethodInsn(183, classMeta.getSuperClassName(), params.getName(), params.getDesc());
    mv.visitInsn(iReturnOpcode);
    Label l2 = new Label();
    mv.visitLabel(l2);
    mv.visitLocalVariable("this", "L" + classMeta.getClassName() + ";", null, l0, l2, 0);
    mv.visitMaxs(2, 1);
    mv.visitEnd();
  }
  
  private void addPublicSetMethod(ClassVisitor cv, ClassMeta classMeta, boolean checkExisting)
  {
    if ((checkExisting) && (!classMeta.isExistingSuperMethod(publicSetterName, setMethodDesc))) {
      if (classMeta.isLog(1)) {
        classMeta.log("excluding " + publicSetterName + " as not on super object");
      }
      return;
    }
    
    addPublicSetMethod(new VisitMethodParams(cv, 1, publicSetterName, setMethodDesc, null, null), classMeta);
  }
  
  private void addPublicSetMethod(VisitMethodParams params, ClassMeta classMeta)
  {
    MethodVisitor mv = params.visitMethod();
    
    String publicGetterName = getPublicGetterName();
    
    String preSetterArgTypes = "Ljava/lang/Object;Ljava/lang/Object;";
    if (primativeType)
    {
      preSetterArgTypes = fieldDesc + fieldDesc;
    }
    

    int iLoadOpcode = asmType.getOpcode(21);
    

    int iPosition = asmType.getSize();
    

    String className = classMeta.getClassName();
    String superClassName = classMeta.getSuperClassName();
    
    mv.visitCode();
    Label l0 = new Label();
    mv.visitLabel(l0);
    mv.visitLineNumber(1, l0);
    mv.visitVarInsn(25, 0);
    mv.visitFieldInsn(180, className, "_ebean_intercept", "Lcom/avaje/ebean/bean/EntityBeanIntercept;");
    if (isInterceptSet()) {
      mv.visitInsn(4);
    }
    else {
      mv.visitInsn(3);
    }
    
    String preSetterMethod = "preSetter";
    if (isMany()) {
      preSetterMethod = "preSetterMany";
    }
    
    mv.visitLdcInsn(fieldName);
    mv.visitVarInsn(25, 0);
    mv.visitMethodInsn(182, className, publicGetterName, getMethodDesc);
    mv.visitVarInsn(iLoadOpcode, 1);
    mv.visitMethodInsn(182, "com/avaje/ebean/bean/EntityBeanIntercept", preSetterMethod, "(ZLjava/lang/String;" + preSetterArgTypes + ")Ljava/beans/PropertyChangeEvent;");
    mv.visitVarInsn(58, 1 + iPosition);
    
    Label l1 = new Label();
    mv.visitLabel(l1);
    mv.visitLineNumber(1, l1);
    mv.visitVarInsn(25, 0);
    mv.visitVarInsn(iLoadOpcode, 1);
    mv.visitMethodInsn(183, superClassName, params.getName(), params.getDesc());
    
    Label levt = new Label();
    mv.visitLabel(levt);
    mv.visitLineNumber(3, levt);
    
    mv.visitVarInsn(25, 0);
    mv.visitFieldInsn(180, className, "_ebean_intercept", "Lcom/avaje/ebean/bean/EntityBeanIntercept;");
    mv.visitVarInsn(25, 1 + iPosition);
    mv.visitVarInsn(25, 0);
    mv.visitMethodInsn(183, superClassName, publicGetterName, getMethodDesc);
    if (primativeType) {
      appendValueOf(mv, classMeta);
    }
    mv.visitMethodInsn(182, "com/avaje/ebean/bean/EntityBeanIntercept", "postSetter", "(Ljava/beans/PropertyChangeEvent;Ljava/lang/Object;)V");
    
    Label l2 = new Label();
    mv.visitLabel(l2);
    mv.visitLineNumber(1, l2);
    mv.visitInsn(177);
    
    Label l3 = new Label();
    mv.visitLabel(l3);
    mv.visitLocalVariable("this", "L" + className + ";", null, l0, l3, 0);
    mv.visitLocalVariable("newValue", fieldDesc, null, l0, l3, 1);
    mv.visitLocalVariable("evt", "Ljava/beans/PropertyChangeEvent;", null, l1, l3, 2);
    mv.visitMaxs(5, 3);
    mv.visitEnd();
  }
  




  public void addGetSetMethods(ClassVisitor cv, ClassMeta classMeta)
  {
    if (!isLocalField(classMeta)) {
      String msg = "ERROR: " + fieldClass + " != " + classMeta.getClassName() + " for field " + fieldName + " " + fieldDesc;
      throw new RuntimeException(msg);
    }
    

    addGet(cv, classMeta);
    addSet(cv, classMeta);
    


    addGetNoIntercept(cv, classMeta);
    addSetNoIntercept(cv, classMeta);
  }
  
  private String getEbeanCollectionClass() {
    if (fieldDesc.equals("Ljava/util/List;")) {
      return "com/avaje/ebean/common/BeanList";
    }
    if (fieldDesc.equals("Ljava/util/Set;")) {
      return "com/avaje/ebean/common/BeanSet";
    }
    if (fieldDesc.equals("Ljava/util/Map;")) {
      return "com/avaje/ebean/common/BeanMap";
    }
    return null;
  }
  



  private boolean isInterceptMany()
  {
    if ((isMany()) && (!isTransient()))
    {
      String ebCollection = getEbeanCollectionClass();
      if (ebCollection != null) {
        return true;
      }
      classMeta.log("Error unepxected many type " + fieldDesc);
    }
    
    return false;
  }
  



  private void addGet(ClassVisitor cw, ClassMeta classMeta)
  {
    if (classMeta.isLog(3)) {
      classMeta.log(getMethodName + " " + getMethodDesc + " intercept:" + isInterceptGet() + " " + annotations);
    }
    
    MethodVisitor mv = cw.visitMethod(4, getMethodName, getMethodDesc, null, null);
    mv.visitCode();
    
    if (isInterceptMany()) {
      addGetForMany(mv);
      return;
    }
    

    int iReturnOpcode = asmType.getOpcode(172);
    
    String className = classMeta.getClassName();
    
    Label labelEnd = new Label();
    Label labelStart = null;
    
    if (isInterceptGet()) {
      labelStart = new Label();
      mv.visitLabel(labelStart);
      mv.visitLineNumber(4, labelStart);
      mv.visitFrame(3, 0, null, 0, null);
      mv.visitVarInsn(25, 0);
      mv.visitFieldInsn(180, className, "_ebean_intercept", "Lcom/avaje/ebean/bean/EntityBeanIntercept;");
      mv.visitLdcInsn(fieldName);
      mv.visitMethodInsn(182, "com/avaje/ebean/bean/EntityBeanIntercept", "preGetter", "(Ljava/lang/String;)V");
    }
    if (labelStart == null) {
      labelStart = labelEnd;
    }
    mv.visitLabel(labelEnd);
    mv.visitLineNumber(5, labelEnd);
    mv.visitFrame(3, 0, null, 0, null);
    mv.visitVarInsn(25, 0);
    mv.visitFieldInsn(180, className, fieldName, fieldDesc);
    mv.visitInsn(iReturnOpcode);
    Label labelEnd1 = new Label();
    mv.visitLabel(labelEnd1);
    mv.visitLocalVariable("this", "L" + className + ";", null, labelStart, labelEnd1, 0);
    mv.visitMaxs(2, 1);
    mv.visitEnd();
  }
  
  private void addGetForMany(MethodVisitor mv)
  {
    String className = classMeta.getClassName();
    String ebCollection = getEbeanCollectionClass();
    
    Label l0 = new Label();
    mv.visitLabel(l0);
    mv.visitLineNumber(1, l0);
    mv.visitVarInsn(25, 0);
    mv.visitFieldInsn(180, className, "_ebean_intercept", "Lcom/avaje/ebean/bean/EntityBeanIntercept;");
    mv.visitLdcInsn(fieldName);
    mv.visitMethodInsn(182, "com/avaje/ebean/bean/EntityBeanIntercept", "preGetter", "(Ljava/lang/String;)V");
    
    Label l4 = new Label();
    if (classMeta.getEnhanceContext().isCheckNullManyFields())
    {
      if (classMeta.isLog(3)) {
        classMeta.log("... add Many null check on " + fieldName + " ebtype:" + ebCollection);
      }
      
      Label l3 = new Label();
      mv.visitLabel(l3);
      mv.visitLineNumber(2, l3);
      mv.visitVarInsn(25, 0);
      mv.visitFieldInsn(180, className, fieldName, fieldDesc);
      
      mv.visitJumpInsn(199, l4);
      Label l5 = new Label();
      mv.visitLabel(l5);
      mv.visitLineNumber(3, l5);
      mv.visitVarInsn(25, 0);
      mv.visitTypeInsn(187, ebCollection);
      mv.visitInsn(89);
      mv.visitMethodInsn(183, ebCollection, "<init>", "()V");
      mv.visitFieldInsn(181, className, fieldName, fieldDesc);
      
      if (isManyToMany())
      {
        if (classMeta.isLog(3)) {
          classMeta.log("... add ManyToMany modify listening to " + fieldName);
        }
        
        Label l6 = new Label();
        mv.visitLabel(l6);
        mv.visitLineNumber(4, l6);
        mv.visitVarInsn(25, 0);
        mv.visitFieldInsn(180, className, fieldName, fieldDesc);
        mv.visitTypeInsn(192, "com/avaje/ebean/bean/BeanCollection");
        mv.visitFieldInsn(178, "com/avaje/ebean/bean/BeanCollection$ModifyListenMode", "ALL", "Lcom/avaje/ebean/bean/BeanCollection$ModifyListenMode;");
        mv.visitMethodInsn(185, "com/avaje/ebean/bean/BeanCollection", "setModifyListening", "(Lcom/avaje/ebean/bean/BeanCollection$ModifyListenMode;)V");
      }
    }
    
    mv.visitLabel(l4);
    mv.visitLineNumber(5, l4);
    mv.visitFrame(3, 0, null, 0, null);
    mv.visitVarInsn(25, 0);
    mv.visitFieldInsn(180, className, fieldName, fieldDesc);
    mv.visitInsn(176);
    Label l7 = new Label();
    mv.visitLabel(l7);
    mv.visitLocalVariable("this", "L" + className + ";", null, l0, l7, 0);
    mv.visitMaxs(3, 1);
    mv.visitEnd();
  }
  







  private void addGetNoIntercept(ClassVisitor cw, ClassMeta classMeta)
  {
    int iReturnOpcode = asmType.getOpcode(172);
    
    if (classMeta.isLog(3)) {
      classMeta.log(getNoInterceptMethodName + " " + getMethodDesc);
    }
    
    MethodVisitor mv = cw.visitMethod(4, getNoInterceptMethodName, getMethodDesc, null, null);
    mv.visitCode();
    
    Label l0 = new Label();
    mv.visitLabel(l0);
    mv.visitLineNumber(1, l0);
    mv.visitVarInsn(25, 0);
    mv.visitFieldInsn(180, fieldClass, fieldName, fieldDesc);
    mv.visitInsn(iReturnOpcode);
    Label l2 = new Label();
    mv.visitLabel(l2);
    mv.visitLocalVariable("this", "L" + fieldClass + ";", null, l0, l2, 0);
    mv.visitMaxs(2, 1);
    mv.visitEnd();
  }
  











  private void addSet(ClassVisitor cw, ClassMeta classMeta)
  {
    String preSetterArgTypes = "Ljava/lang/Object;Ljava/lang/Object;";
    if (!objectType)
    {
      preSetterArgTypes = fieldDesc + fieldDesc;
    }
    

    int iLoadOpcode = asmType.getOpcode(21);
    

    int iPosition = asmType.getSize();
    
    if (classMeta.isLog(3)) {
      classMeta.log(setMethodName + " " + setMethodDesc + " intercept:" + isInterceptSet() + " opCode:" + iLoadOpcode + "," + iPosition + " preSetterArgTypes" + preSetterArgTypes);
    }
    

    MethodVisitor mv = cw.visitMethod(4, setMethodName, setMethodDesc, null, null);
    mv.visitCode();
    
    Label l0 = new Label();
    mv.visitLabel(l0);
    mv.visitLineNumber(1, l0);
    mv.visitVarInsn(25, 0);
    mv.visitFieldInsn(180, fieldClass, "_ebean_intercept", "Lcom/avaje/ebean/bean/EntityBeanIntercept;");
    if (isInterceptSet()) {
      mv.visitInsn(4);
    }
    else {
      mv.visitInsn(3);
    }
    mv.visitLdcInsn(fieldName);
    mv.visitVarInsn(25, 0);
    mv.visitMethodInsn(182, fieldClass, getMethodName, getMethodDesc);
    mv.visitVarInsn(iLoadOpcode, 1);
    String preSetterMethod = "preSetter";
    if (isMany()) {
      preSetterMethod = "preSetterMany";
    }
    mv.visitMethodInsn(182, "com/avaje/ebean/bean/EntityBeanIntercept", preSetterMethod, "(ZLjava/lang/String;" + preSetterArgTypes + ")Ljava/beans/PropertyChangeEvent;");
    mv.visitVarInsn(58, 1 + iPosition);
    Label l1 = new Label();
    mv.visitLabel(l1);
    mv.visitLineNumber(2, l1);
    mv.visitVarInsn(25, 0);
    mv.visitVarInsn(iLoadOpcode, 1);
    mv.visitFieldInsn(181, fieldClass, fieldName, fieldDesc);
    
    Label l2 = new Label();
    mv.visitLabel(l2);
    mv.visitLineNumber(3, l2);
    mv.visitVarInsn(25, 0);
    mv.visitFieldInsn(180, fieldClass, "_ebean_intercept", "Lcom/avaje/ebean/bean/EntityBeanIntercept;");
    mv.visitVarInsn(25, 1 + iPosition);
    mv.visitMethodInsn(182, "com/avaje/ebean/bean/EntityBeanIntercept", "postSetter", "(Ljava/beans/PropertyChangeEvent;)V");
    
    Label l3 = new Label();
    mv.visitLabel(l3);
    mv.visitLineNumber(4, l3);
    mv.visitInsn(177);
    Label l4 = new Label();
    mv.visitLabel(l4);
    mv.visitLocalVariable("this", "L" + fieldClass + ";", null, l0, l4, 0);
    mv.visitLocalVariable("newValue", fieldDesc, null, l0, l4, 1);
    mv.visitLocalVariable("evt", "Ljava/beans/PropertyChangeEvent;", null, l1, l4, 2);
    mv.visitMaxs(5, 3);
    mv.visitEnd();
  }
  








  private void addSetNoIntercept(ClassVisitor cw, ClassMeta classMeta)
  {
    int iLoadOpcode = asmType.getOpcode(21);
    

    int iPosition = asmType.getSize();
    
    if (classMeta.isLog(3)) {
      classMeta.log(setNoInterceptMethodName + " " + setMethodDesc + " opCode:" + iLoadOpcode + "," + iPosition);
    }
    
    MethodVisitor mv = cw.visitMethod(4, setNoInterceptMethodName, setMethodDesc, null, null);
    mv.visitCode();
    Label l0 = new Label();
    

    mv.visitLabel(l0);
    mv.visitLineNumber(1, l0);
    mv.visitVarInsn(25, 0);
    mv.visitVarInsn(iLoadOpcode, 1);
    
    mv.visitFieldInsn(181, fieldClass, fieldName, fieldDesc);
    
    Label l2 = new Label();
    mv.visitLabel(l2);
    mv.visitLineNumber(1, l2);
    mv.visitInsn(177);
    Label l3 = new Label();
    mv.visitLabel(l3);
    mv.visitLocalVariable("this", "L" + fieldClass + ";", null, l0, l3, 0);
    mv.visitLocalVariable("_newValue", fieldDesc, null, l0, l3, 1);
    mv.visitMaxs(4, 2);
    mv.visitEnd();
  }
}
