package com.avaje.ebean.enhance.agent;

import com.avaje.ebean.enhance.asm.AnnotationVisitor;
import com.avaje.ebean.enhance.asm.Label;
import com.avaje.ebean.enhance.asm.MethodAdapter;
import com.avaje.ebean.enhance.asm.MethodVisitor;
import com.avaje.ebean.enhance.asm.Opcodes;










public class MethodFieldAdapter
  extends MethodAdapter
  implements Opcodes
{
  final ClassMeta meta;
  final String className;
  final String methodDescription;
  boolean transientAnnotation = false;
  
  public MethodFieldAdapter(MethodVisitor mv, ClassMeta meta, String methodDescription) {
    super(mv);
    this.meta = meta;
    className = meta.getClassName();
    this.methodDescription = methodDescription;
  }
  







  public AnnotationVisitor visitAnnotation(String desc, boolean visible)
  {
    if (desc.equals("Ljavax/persistence/Transient;")) {
      transientAnnotation = true;
    }
    return super.visitAnnotation(desc, visible);
  }
  

  public void visitLocalVariable(String name, String desc, String signature, Label start, Label end, int index)
  {
    super.visitLocalVariable(name, desc, signature, start, end, index);
  }
  

  public void visitMethodInsn(int opcode, String owner, String name, String desc)
  {
    super.visitMethodInsn(opcode, owner, name, desc);
  }
  
  public void visitFieldInsn(int opcode, String owner, String name, String desc)
  {
    if (transientAnnotation)
    {
      super.visitFieldInsn(opcode, owner, name, desc);
      return;
    }
    
    if ((opcode == 178) || (opcode == 179)) {
      if (meta.isLog(3)) {
        meta.log(" ... info: skip static field " + owner + " " + name + " in " + methodDescription);
      }
      super.visitFieldInsn(opcode, owner, name, desc);
      return;
    }
    
    if (!meta.isFieldPersistent(name)) {
      if (meta.isLog(2)) {
        meta.log(" ... info: non-persistent field " + owner + " " + name + " in " + methodDescription);
      }
      super.visitFieldInsn(opcode, owner, name, desc);
      return;
    }
    

    if (opcode == 180) {
      String methodName = "_ebean_get_" + name;
      String methodDesc = "()" + desc;
      if (meta.isLog(4)) {
        meta.log("GETFIELD method:" + methodDescription + " field:" + name + " > " + methodName + " " + methodDesc);
      }
      
      super.visitMethodInsn(182, className, methodName, methodDesc);
    }
    else if (opcode == 181) {
      String methodName = "_ebean_set_" + name;
      String methodDesc = "(" + desc + ")V";
      if (meta.isLog(4)) {
        meta.log("PUTFIELD method:" + methodDescription + " field:" + name + " > " + methodName + " " + methodDesc);
      }
      
      super.visitMethodInsn(182, className, methodName, methodDesc);
    }
    else {
      meta.log("Warning adapting method:" + methodDescription + "; unexpected static access to a persistent field?? " + name + " opCode not GETFIELD or PUTFIELD??  opCode:" + opcode + "");
      


      super.visitFieldInsn(opcode, owner, name, desc);
    }
  }
}
