package com.avaje.ebean.enhance.agent;

import com.avaje.ebean.enhance.asm.AnnotationVisitor;
import com.avaje.ebean.enhance.asm.MethodVisitor;
import com.avaje.ebean.enhance.asm.Type;
import com.avaje.ebean.enhance.asm.commons.MethodAdviceAdapter;
import java.util.ArrayList;










public class ScopeTransAdapter
  extends MethodAdviceAdapter
  implements EnhanceConstants
{
  private static final Type txScopeType = Type.getType("Lcom/avaje/ebean/TxScope;");
  private static final Type scopeTransType = Type.getType("Lcom/avaje/ebeaninternal/api/ScopeTrans;");
  private static final Type helpScopeTrans = Type.getType("Lcom/avaje/ebeaninternal/api/HelpScopeTrans;");
  
  private final AnnotationInfo annotationInfo;
  
  private final ClassAdapterTransactional owner;
  
  private boolean transactional;
  private int posTxScope;
  private int posScopeTrans;
  
  public ScopeTransAdapter(ClassAdapterTransactional owner, MethodVisitor mv, int access, String name, String desc)
  {
    super(mv, access, name, desc);
    this.owner = owner;
    

    AnnotationInfo parentInfo = classAnnotationInfo;
    

    AnnotationInfo interfaceInfo = owner.getInterfaceTransactionalInfo(name, desc);
    if (parentInfo == null) {
      parentInfo = interfaceInfo;
    } else {
      parentInfo.setParent(interfaceInfo);
    }
    

    annotationInfo = new AnnotationInfo(parentInfo);
    


    transactional = (parentInfo != null);
  }
  
  public boolean isTransactional() {
    return transactional;
  }
  
  public AnnotationVisitor visitAnnotation(String desc, boolean visible)
  {
    if (desc.equals("Lcom/avaje/ebean/annotation/Transactional;")) {
      transactional = true;
    }
    AnnotationVisitor av = super.visitAnnotation(desc, visible);
    return new AnnotationInfoVisitor(null, annotationInfo, av);
  }
  
  private void setTxType(Object txType)
  {
    mv.visitVarInsn(25, posTxScope);
    mv.visitLdcInsn(txType.toString());
    mv.visitMethodInsn(184, "com/avaje/ebean/TxType", "valueOf", "(Ljava/lang/String;)Lcom/avaje/ebean/TxType;");
    mv.visitMethodInsn(182, "com/avaje/ebean/TxScope", "setType", "(Lcom/avaje/ebean/TxType;)Lcom/avaje/ebean/TxScope;");
    mv.visitInsn(87);
  }
  
  private void setTxIsolation(Object txIsolation)
  {
    mv.visitVarInsn(25, posTxScope);
    mv.visitLdcInsn(txIsolation.toString());
    mv.visitMethodInsn(184, "com/avaje/ebean/TxIsolation", "valueOf", "(Ljava/lang/String;)Lcom/avaje/ebean/TxIsolation;");
    mv.visitMethodInsn(182, "com/avaje/ebean/TxScope", "setIsolation", "(Lcom/avaje/ebean/TxIsolation;)Lcom/avaje/ebean/TxScope;");
    mv.visitInsn(87);
  }
  
  private void setServerName(Object serverName)
  {
    mv.visitVarInsn(25, posTxScope);
    mv.visitLdcInsn(serverName.toString());
    mv.visitMethodInsn(182, "com/avaje/ebean/TxScope", "setServerName", "(Ljava/lang/String;)Lcom/avaje/ebean/TxScope;");
    mv.visitInsn(87);
  }
  
  private void setReadOnly(Object readOnlyObj)
  {
    boolean readOnly = ((Boolean)readOnlyObj).booleanValue();
    mv.visitVarInsn(25, posTxScope);
    if (readOnly) {
      mv.visitInsn(4);
    } else {
      mv.visitInsn(3);
    }
    mv.visitMethodInsn(182, "com/avaje/ebean/TxScope", "setReadOnly", "(Z)Lcom/avaje/ebean/TxScope;");
  }
  



  private void setNoRollbackFor(Object noRollbackFor)
  {
    ArrayList<?> list = (ArrayList)noRollbackFor;
    
    for (int i = 0; i < list.size(); i++)
    {
      Type throwType = (Type)list.get(i);
      
      mv.visitVarInsn(25, posTxScope);
      mv.visitLdcInsn(throwType);
      mv.visitMethodInsn(182, txScopeType.getInternalName(), "setNoRollbackFor", "(Ljava/lang/Class;)Lcom/avaje/ebean/TxScope;");
      mv.visitInsn(87);
    }
  }
  



  private void setRollbackFor(Object rollbackFor)
  {
    ArrayList<?> list = (ArrayList)rollbackFor;
    
    for (int i = 0; i < list.size(); i++)
    {
      Type throwType = (Type)list.get(i);
      
      mv.visitVarInsn(25, posTxScope);
      mv.visitLdcInsn(throwType);
      mv.visitMethodInsn(182, txScopeType.getInternalName(), "setRollbackFor", "(Ljava/lang/Class;)Lcom/avaje/ebean/TxScope;");
      mv.visitInsn(87);
    }
  }
  

  protected void onMethodEnter()
  {
    if (!transactional) {
      return;
    }
    

    owner.transactionalMethod(methodName, methodDesc, annotationInfo);
    
    posTxScope = newLocal(txScopeType);
    posScopeTrans = newLocal(scopeTransType);
    
    mv.visitTypeInsn(187, txScopeType.getInternalName());
    mv.visitInsn(89);
    mv.visitMethodInsn(183, txScopeType.getInternalName(), "<init>", "()V");
    mv.visitVarInsn(58, posTxScope);
    
    Object txType = annotationInfo.getValue("type");
    if (txType != null) {
      setTxType(txType);
    }
    
    Object txIsolation = annotationInfo.getValue("isolation");
    if (txIsolation != null) {
      setTxIsolation(txIsolation);
    }
    
    Object readOnly = annotationInfo.getValue("readOnly");
    if (readOnly != null) {
      setReadOnly(readOnly);
    }
    
    Object noRollbackFor = annotationInfo.getValue("noRollbackFor");
    if (noRollbackFor != null) {
      setNoRollbackFor(noRollbackFor);
    }
    
    Object rollbackFor = annotationInfo.getValue("rollbackFor");
    if (rollbackFor != null) {
      setRollbackFor(rollbackFor);
    }
    
    Object serverName = annotationInfo.getValue("serverName");
    if ((serverName != null) && (!serverName.equals(""))) {
      setServerName(serverName);
    }
    
    mv.visitVarInsn(25, posTxScope);
    mv.visitMethodInsn(184, helpScopeTrans.getInternalName(), "createScopeTrans", "(" + txScopeType.getDescriptor() + ")" + scopeTransType.getDescriptor());
    
    mv.visitVarInsn(58, posScopeTrans);
  }
  

  protected void onMethodExit(int opcode)
  {
    if (!transactional) {
      return;
    }
    
    if (opcode == 177) {
      visitInsn(1);
    }
    else if ((opcode == 176) || (opcode == 191)) {
      dup();
    }
    else {
      if ((opcode == 173) || (opcode == 175)) {
        dup2();
      } else {
        dup();
      }
      box(Type.getReturnType(methodDesc));
    }
    visitIntInsn(17, opcode);
    loadLocal(posScopeTrans);
    
    visitMethodInsn(184, helpScopeTrans.getInternalName(), "onExitScopeTrans", "(Ljava/lang/Object;I" + scopeTransType.getDescriptor() + ")V");
  }
}
