package com.avaje.ebean.enhance.asm;











final class AnnotationWriter
  implements AnnotationVisitor
{
  private final ClassWriter cw;
  









  private int size;
  









  private final boolean named;
  









  private final ByteVector bv;
  









  private final ByteVector parent;
  









  private final int offset;
  









  AnnotationWriter next;
  









  AnnotationWriter prev;
  










  AnnotationWriter(ClassWriter cw, boolean named, ByteVector bv, ByteVector parent, int offset)
  {
    this.cw = cw;
    this.named = named;
    this.bv = bv;
    this.parent = parent;
    this.offset = offset;
  }
  



  public void visit(String name, Object value)
  {
    size += 1;
    if (named) {
      bv.putShort(cw.newUTF8(name));
    }
    if ((value instanceof String)) {
      bv.put12(115, cw.newUTF8((String)value));
    } else if ((value instanceof Byte)) {
      bv.put12(66, cw.newInteger(((Byte)value).byteValue()).index);
    } else if ((value instanceof Boolean)) {
      int v = ((Boolean)value).booleanValue() ? 1 : 0;
      bv.put12(90, cw.newInteger(v).index);
    } else if ((value instanceof Character)) {
      bv.put12(67, cw.newInteger(((Character)value).charValue()).index);
    } else if ((value instanceof Short)) {
      bv.put12(83, cw.newInteger(((Short)value).shortValue()).index);
    } else if ((value instanceof Type)) {
      bv.put12(99, cw.newUTF8(((Type)value).getDescriptor()));
    } else if ((value instanceof byte[])) {
      byte[] v = (byte[])value;
      bv.put12(91, v.length);
      for (int i = 0; i < v.length; i++) {
        bv.put12(66, cw.newInteger(v[i]).index);
      }
    } else if ((value instanceof boolean[])) {
      boolean[] v = (boolean[])value;
      bv.put12(91, v.length);
      for (int i = 0; i < v.length; i++) {
        bv.put12(90, cw.newInteger(v[i] != 0 ? 1 : 0).index);
      }
    } else if ((value instanceof short[])) {
      short[] v = (short[])value;
      bv.put12(91, v.length);
      for (int i = 0; i < v.length; i++) {
        bv.put12(83, cw.newInteger(v[i]).index);
      }
    } else if ((value instanceof char[])) {
      char[] v = (char[])value;
      bv.put12(91, v.length);
      for (int i = 0; i < v.length; i++) {
        bv.put12(67, cw.newInteger(v[i]).index);
      }
    } else if ((value instanceof int[])) {
      int[] v = (int[])value;
      bv.put12(91, v.length);
      for (int i = 0; i < v.length; i++) {
        bv.put12(73, cw.newInteger(v[i]).index);
      }
    } else if ((value instanceof long[])) {
      long[] v = (long[])value;
      bv.put12(91, v.length);
      for (int i = 0; i < v.length; i++) {
        bv.put12(74, cw.newLong(v[i]).index);
      }
    } else if ((value instanceof float[])) {
      float[] v = (float[])value;
      bv.put12(91, v.length);
      for (int i = 0; i < v.length; i++) {
        bv.put12(70, cw.newFloat(v[i]).index);
      }
    } else if ((value instanceof double[])) {
      double[] v = (double[])value;
      bv.put12(91, v.length);
      for (int i = 0; i < v.length; i++) {
        bv.put12(68, cw.newDouble(v[i]).index);
      }
    } else {
      Item i = cw.newConstItem(value);
      bv.put12(".s.IFJDCS".charAt(type), index);
    }
  }
  



  public void visitEnum(String name, String desc, String value)
  {
    size += 1;
    if (named) {
      bv.putShort(cw.newUTF8(name));
    }
    bv.put12(101, cw.newUTF8(desc)).putShort(cw.newUTF8(value));
  }
  


  public AnnotationVisitor visitAnnotation(String name, String desc)
  {
    size += 1;
    if (named) {
      bv.putShort(cw.newUTF8(name));
    }
    
    bv.put12(64, cw.newUTF8(desc)).putShort(0);
    return new AnnotationWriter(cw, true, bv, bv, bv.length - 2);
  }
  
  public AnnotationVisitor visitArray(String name) {
    size += 1;
    if (named) {
      bv.putShort(cw.newUTF8(name));
    }
    
    bv.put12(91, 0);
    return new AnnotationWriter(cw, false, bv, bv, bv.length - 2);
  }
  
  public void visitEnd() {
    if (parent != null) {
      byte[] data = parent.data;
      data[offset] = ((byte)(size >>> 8));
      data[(offset + 1)] = ((byte)size);
    }
  }
  








  int getSize()
  {
    int size = 0;
    AnnotationWriter aw = this;
    while (aw != null) {
      size += bv.length;
      aw = next;
    }
    return size;
  }
  





  void put(ByteVector out)
  {
    int n = 0;
    int size = 2;
    AnnotationWriter aw = this;
    AnnotationWriter last = null;
    while (aw != null) {
      n++;
      size += bv.length;
      aw.visitEnd();
      prev = last;
      last = aw;
      aw = next;
    }
    out.putInt(size);
    out.putShort(n);
    aw = last;
    while (aw != null) {
      out.putByteArray(bv.data, 0, bv.length);
      aw = prev;
    }
  }
  










  static void put(AnnotationWriter[] panns, int off, ByteVector out)
  {
    int size = 1 + 2 * (panns.length - off);
    for (int i = off; i < panns.length; i++) {
      size += (panns[i] == null ? 0 : panns[i].getSize());
    }
    out.putInt(size).putByte(panns.length - off);
    for (int i = off; i < panns.length; i++) {
      AnnotationWriter aw = panns[i];
      AnnotationWriter last = null;
      int n = 0;
      while (aw != null) {
        n++;
        aw.visitEnd();
        prev = last;
        last = aw;
        aw = next;
      }
      out.putShort(n);
      aw = last;
      while (aw != null) {
        out.putByteArray(bv.data, 0, bv.length);
        aw = prev;
      }
    }
  }
}
