package com.avaje.ebean.enhance.asm;










final class FieldWriter
  implements FieldVisitor
{
  FieldWriter next;
  








  private final ClassWriter cw;
  








  private final int access;
  








  private final int name;
  








  private final int desc;
  








  private int signature;
  








  private int value;
  








  private AnnotationWriter anns;
  







  private AnnotationWriter ianns;
  







  private Attribute attrs;
  








  FieldWriter(ClassWriter cw, int access, String name, String desc, String signature, Object value)
  {
    if (firstField == null) {
      firstField = this;
    } else {
      lastField.next = this;
    }
    lastField = this;
    this.cw = cw;
    this.access = access;
    this.name = cw.newUTF8(name);
    this.desc = cw.newUTF8(desc);
    if (signature != null) {
      this.signature = cw.newUTF8(signature);
    }
    if (value != null) {
      this.value = newConstItemindex;
    }
  }
  









  public AnnotationVisitor visitAnnotation(String desc, boolean visible)
  {
    ByteVector bv = new ByteVector();
    
    bv.putShort(cw.newUTF8(desc)).putShort(0);
    AnnotationWriter aw = new AnnotationWriter(cw, true, bv, bv, 2);
    if (visible) {
      next = anns;
      anns = aw;
    } else {
      next = ianns;
      ianns = aw;
    }
    return aw;
  }
  
  public void visitAttribute(Attribute attr) {
    next = attrs;
    attrs = attr;
  }
  





  public void visitEnd() {}
  




  int getSize()
  {
    int size = 8;
    if (value != 0) {
      cw.newUTF8("ConstantValue");
      size += 8;
    }
    if (((access & 0x1000) != 0) && ((cw.version & 0xFFFF) < 49))
    {

      cw.newUTF8("Synthetic");
      size += 6;
    }
    if ((access & 0x20000) != 0) {
      cw.newUTF8("Deprecated");
      size += 6;
    }
    if (signature != 0) {
      cw.newUTF8("Signature");
      size += 8;
    }
    if (anns != null) {
      cw.newUTF8("RuntimeVisibleAnnotations");
      size += 8 + anns.getSize();
    }
    if (ianns != null) {
      cw.newUTF8("RuntimeInvisibleAnnotations");
      size += 8 + ianns.getSize();
    }
    if (attrs != null) {
      size += attrs.getSize(cw, null, 0, -1, -1);
    }
    return size;
  }
  




  void put(ByteVector out)
  {
    out.putShort(access).putShort(name).putShort(desc);
    int attributeCount = 0;
    if (value != 0) {
      attributeCount++;
    }
    if (((access & 0x1000) != 0) && ((cw.version & 0xFFFF) < 49))
    {

      attributeCount++;
    }
    if ((access & 0x20000) != 0) {
      attributeCount++;
    }
    if (signature != 0) {
      attributeCount++;
    }
    if (anns != null) {
      attributeCount++;
    }
    if (ianns != null) {
      attributeCount++;
    }
    if (attrs != null) {
      attributeCount += attrs.getCount();
    }
    out.putShort(attributeCount);
    if (value != 0) {
      out.putShort(cw.newUTF8("ConstantValue"));
      out.putInt(2).putShort(value);
    }
    if (((access & 0x1000) != 0) && ((cw.version & 0xFFFF) < 49))
    {

      out.putShort(cw.newUTF8("Synthetic")).putInt(0);
    }
    if ((access & 0x20000) != 0) {
      out.putShort(cw.newUTF8("Deprecated")).putInt(0);
    }
    if (signature != 0) {
      out.putShort(cw.newUTF8("Signature"));
      out.putInt(2).putShort(signature);
    }
    if (anns != null) {
      out.putShort(cw.newUTF8("RuntimeVisibleAnnotations"));
      anns.put(out);
    }
    if (ianns != null) {
      out.putShort(cw.newUTF8("RuntimeInvisibleAnnotations"));
      ianns.put(out);
    }
    if (attrs != null) {
      attrs.put(cw, null, 0, -1, -1, out);
    }
  }
}
