package com.avaje.ebean.enhance.asm;








class MethodWriter
  implements MethodVisitor
{
  static final int ACC_CONSTRUCTOR = 262144;
  






  static final int SAME_FRAME = 0;
  





  static final int SAME_LOCALS_1_STACK_ITEM_FRAME = 64;
  





  static final int RESERVED = 128;
  





  static final int SAME_LOCALS_1_STACK_ITEM_FRAME_EXTENDED = 247;
  





  static final int CHOP_FRAME = 248;
  





  static final int SAME_FRAME_EXTENDED = 251;
  





  static final int APPEND_FRAME = 252;
  





  static final int FULL_FRAME = 255;
  





  private static final int FRAMES = 0;
  





  private static final int MAXS = 1;
  





  private static final int NOTHING = 2;
  





  MethodWriter next;
  





  final ClassWriter cw;
  





  private int access;
  





  private final int name;
  





  private final int desc;
  





  private final String descriptor;
  





  String signature;
  





  int classReaderOffset;
  





  int classReaderLength;
  





  int exceptionCount;
  





  int[] exceptions;
  





  private ByteVector annd;
  





  private AnnotationWriter anns;
  





  private AnnotationWriter ianns;
  





  private AnnotationWriter[] panns;
  





  private AnnotationWriter[] ipanns;
  





  private int synthetics;
  





  private Attribute attrs;
  





  private ByteVector code = new ByteVector();
  






  private int maxStack;
  






  private int maxLocals;
  






  private int frameCount;
  






  private ByteVector stackMap;
  






  private int previousFrameOffset;
  





  private int[] previousFrame;
  





  private int frameIndex;
  





  private int[] frame;
  





  private int handlerCount;
  





  private Handler firstHandler;
  





  private Handler lastHandler;
  





  private int localVarCount;
  





  private ByteVector localVar;
  





  private int localVarTypeCount;
  





  private ByteVector localVarType;
  





  private int lineNumberCount;
  





  private ByteVector lineNumber;
  





  private Attribute cattrs;
  





  private boolean resize;
  





  private int subroutines;
  





  private final int compute;
  





  private Label labels;
  





  private Label previousBlock;
  





  private Label currentBlock;
  





  private int stackSize;
  





  private int maxStackSize;
  






  MethodWriter(ClassWriter cw, int access, String name, String desc, String signature, String[] exceptions, boolean computeMaxs, boolean computeFrames)
  {
    if (firstMethod == null) {
      firstMethod = this;
    } else {
      lastMethod.next = this;
    }
    lastMethod = this;
    this.cw = cw;
    this.access = access;
    this.name = cw.newUTF8(name);
    this.desc = cw.newUTF8(desc);
    descriptor = desc;
    
    this.signature = signature;
    
    if ((exceptions != null) && (exceptions.length > 0)) {
      exceptionCount = exceptions.length;
      this.exceptions = new int[exceptionCount];
      for (int i = 0; i < exceptionCount; i++) {
        this.exceptions[i] = cw.newClass(exceptions[i]);
      }
    }
    compute = (computeMaxs ? 1 : computeFrames ? 0 : 2);
    if ((computeMaxs) || (computeFrames)) {
      if ((computeFrames) && ("<init>".equals(name))) {
        this.access |= 0x40000;
      }
      
      int size = getArgumentsAndReturnSizes(descriptor) >> 2;
      if ((access & 0x8) != 0) {
        size--;
      }
      maxLocals = size;
      
      labels = new Label();
      labels.status |= 0x8;
      visitLabel(labels);
    }
  }
  






  public AnnotationVisitor visitAnnotationDefault()
  {
    annd = new ByteVector();
    return new AnnotationWriter(cw, false, annd, null, 0);
  }
  





  public AnnotationVisitor visitAnnotation(String desc, boolean visible)
  {
    ByteVector bv = new ByteVector();
    
    bv.putShort(cw.newUTF8(desc)).putShort(0);
    AnnotationWriter aw = new AnnotationWriter(cw, true, bv, bv, 2);
    if (visible) {
      next = anns;
      anns = aw;
    } else {
      next = ianns;
      ianns = aw;
    }
    return aw;
  }
  






  public AnnotationVisitor visitParameterAnnotation(int parameter, String desc, boolean visible)
  {
    ByteVector bv = new ByteVector();
    if ("Ljava/lang/Synthetic;".equals(desc))
    {

      synthetics = Math.max(synthetics, parameter + 1);
      return new AnnotationWriter(cw, false, bv, null, 0);
    }
    
    bv.putShort(cw.newUTF8(desc)).putShort(0);
    AnnotationWriter aw = new AnnotationWriter(cw, true, bv, bv, 2);
    if (visible) {
      if (panns == null) {
        panns = new AnnotationWriter[Type.getArgumentTypes(descriptor).length];
      }
      next = panns[parameter];
      panns[parameter] = aw;
    } else {
      if (ipanns == null) {
        ipanns = new AnnotationWriter[Type.getArgumentTypes(descriptor).length];
      }
      next = ipanns[parameter];
      ipanns[parameter] = aw;
    }
    return aw;
  }
  
  public void visitAttribute(Attribute attr) {
    if (attr.isCodeAttribute()) {
      next = cattrs;
      cattrs = attr;
    } else {
      next = attrs;
      attrs = attr;
    }
  }
  



  public void visitCode() {}
  



  public void visitFrame(int type, int nLocal, Object[] local, int nStack, Object[] stack)
  {
    if (compute == 0) {
      return;
    }
    
    if (type == -1) {
      startFrame(code.length, nLocal, nStack);
      for (int i = 0; i < nLocal; i++) {
        if ((local[i] instanceof String)) {
          frame[(frameIndex++)] = (0x1700000 | cw.addType((String)local[i]));
        }
        else if ((local[i] instanceof Integer)) {
          frame[(frameIndex++)] = ((Integer)local[i]).intValue();
        } else {
          frame[(frameIndex++)] = (0x1800000 | cw.addUninitializedType("", position));
        }
      }
      

      for (int i = 0; i < nStack; i++) {
        if ((stack[i] instanceof String)) {
          frame[(frameIndex++)] = (0x1700000 | cw.addType((String)stack[i]));
        }
        else if ((stack[i] instanceof Integer)) {
          frame[(frameIndex++)] = ((Integer)stack[i]).intValue();
        } else {
          frame[(frameIndex++)] = (0x1800000 | cw.addUninitializedType("", position));
        }
      }
      

      endFrame();
    } else { int delta;
      int delta;
      if (stackMap == null) {
        stackMap = new ByteVector();
        delta = code.length;
      } else {
        delta = code.length - previousFrameOffset - 1;
      }
      
      switch (type) {
      case 0: 
        stackMap.putByte(255).putShort(delta).putShort(nLocal);
        

        for (int i = 0; i < nLocal; i++) {
          writeFrameType(local[i]);
        }
        stackMap.putShort(nStack);
        for (int i = 0; i < nStack; i++) {
          writeFrameType(stack[i]);
        }
        break;
      case 1: 
        stackMap.putByte(251 + nLocal).putShort(delta);
        
        for (int i = 0; i < nLocal; i++) {
          writeFrameType(local[i]);
        }
        break;
      case 2: 
        stackMap.putByte(251 - nLocal).putShort(delta);
        
        break;
      case 3: 
        if (delta < 64) {
          stackMap.putByte(delta);
        } else {
          stackMap.putByte(251).putShort(delta);
        }
        break;
      case 4: 
        if (delta < 64) {
          stackMap.putByte(64 + delta);
        } else {
          stackMap.putByte(247).putShort(delta);
        }
        
        writeFrameType(stack[0]);
      }
      
      
      previousFrameOffset = code.length;
      frameCount += 1;
    }
  }
  
  public void visitInsn(int opcode)
  {
    code.putByte(opcode);
    

    if (currentBlock != null) {
      if (compute == 0) {
        currentBlock.frame.execute(opcode, 0, null, null);
      }
      else {
        int size = stackSize + Frame.SIZE[opcode];
        if (size > maxStackSize) {
          maxStackSize = size;
        }
        stackSize = size;
      }
      
      if (((opcode >= 172) && (opcode <= 177)) || (opcode == 191))
      {

        noSuccessor();
      }
    }
  }
  
  public void visitIntInsn(int opcode, int operand)
  {
    if (currentBlock != null) {
      if (compute == 0) {
        currentBlock.frame.execute(opcode, operand, null, null);
      } else if (opcode != 188)
      {

        int size = stackSize + 1;
        if (size > maxStackSize) {
          maxStackSize = size;
        }
        stackSize = size;
      }
    }
    
    if (opcode == 17) {
      code.put12(opcode, operand);
    } else {
      code.put11(opcode, operand);
    }
  }
  
  public void visitVarInsn(int opcode, int var)
  {
    if (currentBlock != null) {
      if (compute == 0) {
        currentBlock.frame.execute(opcode, var, null, null);

      }
      else if (opcode == 169)
      {
        currentBlock.status |= 0x100;
        

        currentBlock.inputStackTop = stackSize;
        noSuccessor();
      } else {
        int size = stackSize + Frame.SIZE[opcode];
        if (size > maxStackSize) {
          maxStackSize = size;
        }
        stackSize = size;
      }
    }
    
    if (compute != 2) {
      int n;
      int n;
      if ((opcode == 22) || (opcode == 24) || (opcode == 55) || (opcode == 57))
      {

        n = var + 2;
      } else {
        n = var + 1;
      }
      if (n > maxLocals) {
        maxLocals = n;
      }
    }
    
    if ((var < 4) && (opcode != 169)) { int opt;
      int opt;
      if (opcode < 54)
      {
        opt = 26 + (opcode - 21 << 2) + var;
      }
      else {
        opt = 59 + (opcode - 54 << 2) + var;
      }
      code.putByte(opt);
    } else if (var >= 256) {
      code.putByte(196).put12(opcode, var);
    } else {
      code.put11(opcode, var);
    }
    if ((opcode >= 54) && (compute == 0) && (handlerCount > 0)) {
      visitLabel(new Label());
    }
  }
  
  public void visitTypeInsn(int opcode, String type) {
    Item i = cw.newClassItem(type);
    
    if (currentBlock != null) {
      if (compute == 0) {
        currentBlock.frame.execute(opcode, code.length, cw, i);
      } else if (opcode == 187)
      {

        int size = stackSize + 1;
        if (size > maxStackSize) {
          maxStackSize = size;
        }
        stackSize = size;
      }
    }
    
    code.put12(opcode, index);
  }
  




  public void visitFieldInsn(int opcode, String owner, String name, String desc)
  {
    Item i = cw.newFieldItem(owner, name, desc);
    
    if (currentBlock != null) {
      if (compute == 0) {
        currentBlock.frame.execute(opcode, 0, cw, i);
      }
      else
      {
        char c = desc.charAt(0);
        int size; switch (opcode) {
        case 178: 
          size = stackSize + ((c == 'D') || (c == 'J') ? 2 : 1);
          break;
        case 179: 
          size = stackSize + ((c == 'D') || (c == 'J') ? -2 : -1);
          break;
        case 180: 
          size = stackSize + ((c == 'D') || (c == 'J') ? 1 : 0);
          break;
        
        default: 
          size = stackSize + ((c == 'D') || (c == 'J') ? -3 : -2);
        }
        
        
        if (size > maxStackSize) {
          maxStackSize = size;
        }
        stackSize = size;
      }
    }
    
    code.put12(opcode, index);
  }
  




  public void visitMethodInsn(int opcode, String owner, String name, String desc)
  {
    boolean itf = opcode == 185;
    Item i = cw.newMethodItem(owner, name, desc, itf);
    int argSize = intVal;
    
    if (currentBlock != null) {
      if (compute == 0) {
        currentBlock.frame.execute(opcode, 0, cw, i);



      }
      else
      {



        if (argSize == 0)
        {

          argSize = getArgumentsAndReturnSizes(desc);
          

          intVal = argSize; }
        int size;
        int size;
        if (opcode == 184) {
          size = stackSize - (argSize >> 2) + (argSize & 0x3) + 1;
        } else {
          size = stackSize - (argSize >> 2) + (argSize & 0x3);
        }
        
        if (size > maxStackSize) {
          maxStackSize = size;
        }
        stackSize = size;
      }
    }
    
    if (itf) {
      if (argSize == 0) {
        argSize = getArgumentsAndReturnSizes(desc);
        intVal = argSize;
      }
      code.put12(185, index).put11(argSize >> 2, 0);
    } else {
      code.put12(opcode, index);
    }
  }
  
  public void visitJumpInsn(int opcode, Label label) {
    Label nextInsn = null;
    
    if (currentBlock != null) {
      if (compute == 0) {
        currentBlock.frame.execute(opcode, 0, null, null);
        
        getFirst()status |= 0x10;
        
        addSuccessor(0, label);
        if (opcode != 167)
        {
          nextInsn = new Label();
        }
      }
      else if (opcode == 168) {
        if ((status & 0x200) == 0) {
          status |= 0x200;
          subroutines += 1;
        }
        currentBlock.status |= 0x80;
        addSuccessor(stackSize + 1, label);
        
        nextInsn = new Label();




      }
      else
      {



        stackSize += Frame.SIZE[opcode];
        addSuccessor(stackSize, label);
      }
    }
    

    if (((status & 0x2) != 0) && (position - code.length < -32768))
    {








      if (opcode == 167) {
        code.putByte(200);
      } else if (opcode == 168) {
        code.putByte(201);
      }
      else
      {
        if (nextInsn != null) {
          status |= 0x10;
        }
        code.putByte(opcode <= 166 ? (opcode + 1 ^ 0x1) - 1 : opcode ^ 0x1);
        

        code.putShort(8);
        code.putByte(200);
      }
      label.put(this, code, code.length - 1, true);


    }
    else
    {


      code.putByte(opcode);
      label.put(this, code, code.length - 1, false);
    }
    if (currentBlock != null) {
      if (nextInsn != null)
      {



        visitLabel(nextInsn);
      }
      if (opcode == 167) {
        noSuccessor();
      }
    }
  }
  
  public void visitLabel(Label label)
  {
    resize |= label.resolve(this, code.length, code.data);
    
    if ((status & 0x1) != 0) {
      return;
    }
    if (compute == 0) {
      if (currentBlock != null) {
        if (position == currentBlock.position)
        {
          currentBlock.status |= status & 0x10;
          frame = currentBlock.frame;
          return;
        }
        
        addSuccessor(0, label);
      }
      
      currentBlock = label;
      if (frame == null) {
        frame = new Frame();
        frame.owner = label;
      }
      
      if (previousBlock != null) {
        if (position == previousBlock.position) {
          previousBlock.status |= status & 0x10;
          frame = previousBlock.frame;
          currentBlock = previousBlock;
          return;
        }
        previousBlock.successor = label;
      }
      previousBlock = label;
    } else if (compute == 1) {
      if (currentBlock != null)
      {
        currentBlock.outputStackMax = maxStackSize;
        addSuccessor(stackSize, label);
      }
      
      currentBlock = label;
      
      stackSize = 0;
      maxStackSize = 0;
      
      if (previousBlock != null) {
        previousBlock.successor = label;
      }
      previousBlock = label;
    }
  }
  
  public void visitLdcInsn(Object cst) {
    Item i = cw.newConstItem(cst);
    
    if (currentBlock != null) {
      if (compute == 0) {
        currentBlock.frame.execute(18, 0, cw, i);
      } else {
        int size;
        int size;
        if ((type == 5) || (type == 6))
        {
          size = stackSize + 2;
        } else {
          size = stackSize + 1;
        }
        
        if (size > maxStackSize) {
          maxStackSize = size;
        }
        stackSize = size;
      }
    }
    
    int index = index;
    if ((type == 5) || (type == 6)) {
      code.put12(20, index);
    } else if (index >= 256) {
      code.put12(19, index);
    } else {
      code.put11(18, index);
    }
  }
  
  public void visitIincInsn(int var, int increment) {
    if ((currentBlock != null) && 
      (compute == 0)) {
      currentBlock.frame.execute(132, var, null, null);
    }
    
    if (compute != 2)
    {
      int n = var + 1;
      if (n > maxLocals) {
        maxLocals = n;
      }
    }
    
    if ((var > 255) || (increment > 127) || (increment < -128)) {
      code.putByte(196).put12(132, var).putShort(increment);
    }
    else
    {
      code.putByte(132).put11(var, increment);
    }
  }
  





  public void visitTableSwitchInsn(int min, int max, Label dflt, Label[] labels)
  {
    int source = code.length;
    code.putByte(170);
    code.length += (4 - code.length % 4) % 4;
    dflt.put(this, code, source, true);
    code.putInt(min).putInt(max);
    for (int i = 0; i < labels.length; i++) {
      labels[i].put(this, code, source, true);
    }
    
    visitSwitchInsn(dflt, labels);
  }
  




  public void visitLookupSwitchInsn(Label dflt, int[] keys, Label[] labels)
  {
    int source = code.length;
    code.putByte(171);
    code.length += (4 - code.length % 4) % 4;
    dflt.put(this, code, source, true);
    code.putInt(labels.length);
    for (int i = 0; i < labels.length; i++) {
      code.putInt(keys[i]);
      labels[i].put(this, code, source, true);
    }
    
    visitSwitchInsn(dflt, labels);
  }
  
  private void visitSwitchInsn(Label dflt, Label[] labels)
  {
    if (currentBlock != null) {
      if (compute == 0) {
        currentBlock.frame.execute(171, 0, null, null);
        
        addSuccessor(0, dflt);
        getFirst()status |= 0x10;
        for (int i = 0; i < labels.length; i++) {
          addSuccessor(0, labels[i]);
          getFirst()status |= 0x10;
        }
      }
      else {
        stackSize -= 1;
        
        addSuccessor(stackSize, dflt);
        for (int i = 0; i < labels.length; i++) {
          addSuccessor(stackSize, labels[i]);
        }
      }
      
      noSuccessor();
    }
  }
  
  public void visitMultiANewArrayInsn(String desc, int dims) {
    Item i = cw.newClassItem(desc);
    
    if (currentBlock != null) {
      if (compute == 0) {
        currentBlock.frame.execute(197, dims, cw, i);
      }
      else
      {
        stackSize += 1 - dims;
      }
    }
    
    code.put12(197, index).putByte(dims);
  }
  




  public void visitTryCatchBlock(Label start, Label end, Label handler, String type)
  {
    handlerCount += 1;
    Handler h = new Handler();
    start = start;
    end = end;
    handler = handler;
    desc = type;
    type = (type != null ? cw.newClass(type) : 0);
    if (lastHandler == null) {
      firstHandler = h;
    } else {
      lastHandler.next = h;
    }
    lastHandler = h;
  }
  






  public void visitLocalVariable(String name, String desc, String signature, Label start, Label end, int index)
  {
    if (signature != null) {
      if (localVarType == null) {
        localVarType = new ByteVector();
      }
      localVarTypeCount += 1;
      localVarType.putShort(position).putShort(position - position).putShort(cw.newUTF8(name)).putShort(cw.newUTF8(signature)).putShort(index);
    }
    



    if (localVar == null) {
      localVar = new ByteVector();
    }
    localVarCount += 1;
    localVar.putShort(position).putShort(position - position).putShort(cw.newUTF8(name)).putShort(cw.newUTF8(desc)).putShort(index);
    



    if (compute != 2)
    {
      char c = desc.charAt(0);
      int n = index + ((c == 'J') || (c == 'D') ? 2 : 1);
      if (n > maxLocals) {
        maxLocals = n;
      }
    }
  }
  
  public void visitLineNumber(int line, Label start) {
    if (lineNumber == null) {
      lineNumber = new ByteVector();
    }
    lineNumberCount += 1;
    lineNumber.putShort(position);
    lineNumber.putShort(line);
  }
  
  public void visitMaxs(int maxStack, int maxLocals) {
    if (compute == 0)
    {
      Handler handler = firstHandler;
      while (handler != null) {
        Label l = start.getFirst();
        Label h = handler.getFirst();
        Label e = end.getFirst();
        
        String t = desc == null ? "java/lang/Throwable" : desc;
        

        int kind = 0x1700000 | cw.addType(t);
        
        status |= 0x10;
        
        while (l != e)
        {
          Edge b = new Edge();
          info = kind;
          successor = h;
          
          next = successors;
          successors = b;
          
          l = successor;
        }
        handler = next;
      }
      

      Frame f = labels.frame;
      Type[] args = Type.getArgumentTypes(descriptor);
      f.initInputFrame(cw, access, args, this.maxLocals);
      visitFrame(f);
      






      int max = 0;
      Label changed = labels;
      while (changed != null)
      {
        Label l = changed;
        changed = next;
        next = null;
        f = frame;
        
        if ((status & 0x10) != 0) {
          status |= 0x20;
        }
        
        status |= 0x40;
        
        int blockMax = inputStack.length + outputStackMax;
        if (blockMax > max) {
          max = blockMax;
        }
        
        Edge e = successors;
        while (e != null) {
          Label n = successor.getFirst();
          boolean change = f.merge(cw, frame, info);
          if ((change) && (next == null))
          {

            next = changed;
            changed = n;
          }
          e = next;
        }
      }
      this.maxStack = max;
      

      Label l = labels;
      while (l != null) {
        f = frame;
        if ((status & 0x20) != 0) {
          visitFrame(f);
        }
        if ((status & 0x40) == 0)
        {
          Label k = successor;
          int start = position;
          int end = (k == null ? code.length : position) - 1;
          
          if (end >= start)
          {
            for (int i = start; i < end; i++) {
              code.data[i] = 0;
            }
            code.data[end] = -65;
            
            startFrame(start, 0, 1);
            frame[(frameIndex++)] = (0x1700000 | cw.addType("java/lang/Throwable"));
            
            endFrame();
          }
        }
        l = successor;
      }
    } else if (compute == 1)
    {
      Handler handler = firstHandler;
      while (handler != null) {
        Label l = start;
        Label h = handler;
        Label e = end;
        
        while (l != e)
        {
          Edge b = new Edge();
          info = 2147483647;
          successor = h;
          
          if ((status & 0x80) == 0) {
            next = successors;
            successors = b;

          }
          else
          {
            next = successors.next.next;
            successors.next.next = b;
          }
          
          l = successor;
        }
        handler = next;
      }
      
      if (subroutines > 0)
      {





        int id = 0;
        labels.visitSubroutine(null, 1L, subroutines);
        
        Label l = labels;
        while (l != null) {
          if ((status & 0x80) != 0)
          {
            Label subroutine = successors.next.successor;
            
            if ((status & 0x400) == 0)
            {
              id++;
              subroutine.visitSubroutine(null, id / 32L << 32 | 1L << id % 32, subroutines);
            }
          }
          
          l = successor;
        }
        
        l = labels;
        while (l != null) {
          if ((status & 0x80) != 0) {
            Label L = labels;
            while (L != null) {
              status &= 0xFFFFFBFF;
              L = successor;
            }
            
            Label subroutine = successors.next.successor;
            subroutine.visitSubroutine(l, 0L, subroutines);
          }
          l = successor;
        }
      }
      










      int max = 0;
      Label stack = labels;
      while (stack != null)
      {
        Label l = stack;
        stack = next;
        
        int start = inputStackTop;
        int blockMax = start + outputStackMax;
        
        if (blockMax > max) {
          max = blockMax;
        }
        
        Edge b = successors;
        if ((status & 0x80) != 0)
        {
          b = next;
        }
        while (b != null) {
          l = successor;
          
          if ((status & 0x8) == 0)
          {
            inputStackTop = (info == 2147483647 ? 1 : start + info);
            

            status |= 0x8;
            next = stack;
            stack = l;
          }
          b = next;
        }
      }
      this.maxStack = max;
    } else {
      this.maxStack = maxStack;
      this.maxLocals = maxLocals;
    }
  }
  







  public void visitEnd() {}
  







  static int getArgumentsAndReturnSizes(String desc)
  {
    int n = 1;
    int c = 1;
    for (;;) {
      char car = desc.charAt(c++);
      if (car == ')') {
        car = desc.charAt(c);
        return n << 2 | ((car == 'D') || (car == 'J') ? 2 : car == 'V' ? 0 : 1);
      }
      if (car == 'L') {
        while (desc.charAt(c++) != ';') {}
        
        n++;
      } else if (car == '[') {
        while ((car = desc.charAt(c)) == '[') {
          c++;
        }
        if ((car == 'D') || (car == 'J')) {
          n--;
        }
      } else if ((car == 'D') || (car == 'J')) {
        n += 2;
      } else {
        n++;
      }
    }
  }
  






  private void addSuccessor(int info, Label successor)
  {
    Edge b = new Edge();
    info = info;
    successor = successor;
    
    next = currentBlock.successors;
    currentBlock.successors = b;
  }
  



  private void noSuccessor()
  {
    if (compute == 0) {
      Label l = new Label();
      frame = new Frame();
      frame.owner = l;
      l.resolve(this, code.length, code.data);
      previousBlock.successor = l;
      previousBlock = l;
    } else {
      currentBlock.outputStackMax = maxStackSize;
    }
    currentBlock = null;
  }
  









  private void visitFrame(Frame f)
  {
    int nTop = 0;
    int nLocal = 0;
    int nStack = 0;
    int[] locals = inputLocals;
    int[] stacks = inputStack;
    

    for (int i = 0; i < locals.length; i++) {
      int t = locals[i];
      if (t == 16777216) {
        nTop++;
      } else {
        nLocal += nTop + 1;
        nTop = 0;
      }
      if ((t == 16777220) || (t == 16777219)) {
        i++;
      }
    }
    

    for (i = 0; i < stacks.length; i++) {
      int t = stacks[i];
      nStack++;
      if ((t == 16777220) || (t == 16777219)) {
        i++;
      }
    }
    
    startFrame(owner.position, nLocal, nStack);
    for (i = 0; nLocal > 0; nLocal--) {
      int t = locals[i];
      frame[(frameIndex++)] = t;
      if ((t == 16777220) || (t == 16777219)) {
        i++;
      }
      i++;
    }
    




    for (i = 0; i < stacks.length; i++) {
      int t = stacks[i];
      frame[(frameIndex++)] = t;
      if ((t == 16777220) || (t == 16777219)) {
        i++;
      }
    }
    endFrame();
  }
  








  private void startFrame(int offset, int nLocal, int nStack)
  {
    int n = 3 + nLocal + nStack;
    if ((frame == null) || (frame.length < n)) {
      frame = new int[n];
    }
    frame[0] = offset;
    frame[1] = nLocal;
    frame[2] = nStack;
    frameIndex = 3;
  }
  



  private void endFrame()
  {
    if (previousFrame != null) {
      if (stackMap == null) {
        stackMap = new ByteVector();
      }
      writeFrame();
      frameCount += 1;
    }
    previousFrame = frame;
    frame = null;
  }
  



  private void writeFrame()
  {
    int clocalsSize = frame[1];
    int cstackSize = frame[2];
    if ((cw.version & 0xFFFF) < 50) {
      stackMap.putShort(frame[0]).putShort(clocalsSize);
      writeFrameTypes(3, 3 + clocalsSize);
      stackMap.putShort(cstackSize);
      writeFrameTypes(3 + clocalsSize, 3 + clocalsSize + cstackSize);
      return;
    }
    int localsSize = previousFrame[1];
    int type = 255;
    int k = 0;
    int delta;
    int delta; if (frameCount == 0) {
      delta = frame[0];
    } else {
      delta = frame[0] - previousFrame[0] - 1;
    }
    if (cstackSize == 0) {
      k = clocalsSize - localsSize;
      switch (k) {
      case -3: 
      case -2: 
      case -1: 
        type = 248;
        localsSize = clocalsSize;
        break;
      case 0: 
        type = delta < 64 ? 0 : 251;
        break;
      case 1: 
      case 2: 
      case 3: 
        type = 252;
      }
    }
    else if ((clocalsSize == localsSize) && (cstackSize == 1)) {
      type = delta < 63 ? 64 : 247;
    }
    

    if (type != 255)
    {
      int l = 3;
      for (int j = 0; j < localsSize; j++) {
        if (frame[l] != previousFrame[l]) {
          type = 255;
          break;
        }
        l++;
      }
    }
    switch (type) {
    case 0: 
      stackMap.putByte(delta);
      break;
    case 64: 
      stackMap.putByte(64 + delta);
      writeFrameTypes(3 + clocalsSize, 4 + clocalsSize);
      break;
    case 247: 
      stackMap.putByte(247).putShort(delta);
      
      writeFrameTypes(3 + clocalsSize, 4 + clocalsSize);
      break;
    case 251: 
      stackMap.putByte(251).putShort(delta);
      break;
    case 248: 
      stackMap.putByte(251 + k).putShort(delta);
      break;
    case 252: 
      stackMap.putByte(251 + k).putShort(delta);
      writeFrameTypes(3 + localsSize, 3 + clocalsSize);
      break;
    
    default: 
      stackMap.putByte(255).putShort(delta).putShort(clocalsSize);
      

      writeFrameTypes(3, 3 + clocalsSize);
      stackMap.putShort(cstackSize);
      writeFrameTypes(3 + clocalsSize, 3 + clocalsSize + cstackSize);
    }
    
  }
  







  private void writeFrameTypes(int start, int end)
  {
    for (int i = start; i < end; i++) {
      int t = frame[i];
      int d = t & 0xF0000000;
      if (d == 0) {
        int v = t & 0xFFFFF;
        switch (t & 0xFF00000) {
        case 24117248: 
          stackMap.putByte(7).putShort(cw.newClass(cw.typeTable[v].strVal1));
          
          break;
        case 25165824: 
          stackMap.putByte(8).putShort(cw.typeTable[v].intVal);
          break;
        default: 
          stackMap.putByte(v);
        }
      } else {
        StringBuffer buf = new StringBuffer();
        d >>= 28;
        while (d-- > 0) {
          buf.append('[');
        }
        if ((t & 0xFF00000) == 24117248) {
          buf.append('L');
          buf.append(cw.typeTable[(t & 0xFFFFF)].strVal1);
          buf.append(';');
        } else {
          switch (t & 0xF) {
          case 1: 
            buf.append('I');
            break;
          case 2: 
            buf.append('F');
            break;
          case 3: 
            buf.append('D');
            break;
          case 9: 
            buf.append('Z');
            break;
          case 10: 
            buf.append('B');
            break;
          case 11: 
            buf.append('C');
            break;
          case 12: 
            buf.append('S');
            break;
          case 4: case 5: case 6: case 7: case 8: default: 
            buf.append('J');
          }
        }
        stackMap.putByte(7).putShort(cw.newClass(buf.toString()));
      }
    }
  }
  
  private void writeFrameType(Object type) {
    if ((type instanceof String)) {
      stackMap.putByte(7).putShort(cw.newClass((String)type));
    } else if ((type instanceof Integer)) {
      stackMap.putByte(((Integer)type).intValue());
    } else {
      stackMap.putByte(8).putShort(position);
    }
  }
  








  final int getSize()
  {
    if (classReaderOffset != 0) {
      return 6 + classReaderLength;
    }
    if (resize)
    {

      resizeInstructions();
    }
    


    int size = 8;
    if (code.length > 0) {
      cw.newUTF8("Code");
      size += 18 + code.length + 8 * handlerCount;
      if (localVar != null) {
        cw.newUTF8("LocalVariableTable");
        size += 8 + localVar.length;
      }
      if (localVarType != null) {
        cw.newUTF8("LocalVariableTypeTable");
        size += 8 + localVarType.length;
      }
      if (lineNumber != null) {
        cw.newUTF8("LineNumberTable");
        size += 8 + lineNumber.length;
      }
      if (stackMap != null) {
        boolean zip = (cw.version & 0xFFFF) >= 50;
        cw.newUTF8(zip ? "StackMapTable" : "StackMap");
        size += 8 + stackMap.length;
      }
      if (cattrs != null) {
        size += cattrs.getSize(cw, code.data, code.length, maxStack, maxLocals);
      }
    }
    



    if (exceptionCount > 0) {
      cw.newUTF8("Exceptions");
      size += 8 + 2 * exceptionCount;
    }
    if (((access & 0x1000) != 0) && ((cw.version & 0xFFFF) < 49))
    {

      cw.newUTF8("Synthetic");
      size += 6;
    }
    if ((access & 0x20000) != 0) {
      cw.newUTF8("Deprecated");
      size += 6;
    }
    if (signature != null) {
      cw.newUTF8("Signature");
      cw.newUTF8(signature);
      size += 8;
    }
    if (annd != null) {
      cw.newUTF8("AnnotationDefault");
      size += 6 + annd.length;
    }
    if (anns != null) {
      cw.newUTF8("RuntimeVisibleAnnotations");
      size += 8 + anns.getSize();
    }
    if (ianns != null) {
      cw.newUTF8("RuntimeInvisibleAnnotations");
      size += 8 + ianns.getSize();
    }
    if (panns != null) {
      cw.newUTF8("RuntimeVisibleParameterAnnotations");
      size += 7 + 2 * (panns.length - synthetics);
      for (int i = panns.length - 1; i >= synthetics; i--) {
        size += (panns[i] == null ? 0 : panns[i].getSize());
      }
    }
    if (ipanns != null) {
      cw.newUTF8("RuntimeInvisibleParameterAnnotations");
      size += 7 + 2 * (ipanns.length - synthetics);
      for (int i = ipanns.length - 1; i >= synthetics; i--) {
        size += (ipanns[i] == null ? 0 : ipanns[i].getSize());
      }
    }
    if (attrs != null) {
      size += attrs.getSize(cw, null, 0, -1, -1);
    }
    return size;
  }
  





  final void put(ByteVector out)
  {
    out.putShort(access).putShort(name).putShort(desc);
    if (classReaderOffset != 0) {
      out.putByteArray(cw.cr.b, classReaderOffset, classReaderLength);
      return;
    }
    int attributeCount = 0;
    if (code.length > 0) {
      attributeCount++;
    }
    if (exceptionCount > 0) {
      attributeCount++;
    }
    if (((access & 0x1000) != 0) && ((cw.version & 0xFFFF) < 49))
    {

      attributeCount++;
    }
    if ((access & 0x20000) != 0) {
      attributeCount++;
    }
    if (signature != null) {
      attributeCount++;
    }
    if (annd != null) {
      attributeCount++;
    }
    if (anns != null) {
      attributeCount++;
    }
    if (ianns != null) {
      attributeCount++;
    }
    if (panns != null) {
      attributeCount++;
    }
    if (ipanns != null) {
      attributeCount++;
    }
    if (attrs != null) {
      attributeCount += attrs.getCount();
    }
    out.putShort(attributeCount);
    if (code.length > 0) {
      int size = 12 + code.length + 8 * handlerCount;
      if (localVar != null) {
        size += 8 + localVar.length;
      }
      if (localVarType != null) {
        size += 8 + localVarType.length;
      }
      if (lineNumber != null) {
        size += 8 + lineNumber.length;
      }
      if (stackMap != null) {
        size += 8 + stackMap.length;
      }
      if (cattrs != null) {
        size += cattrs.getSize(cw, code.data, code.length, maxStack, maxLocals);
      }
      



      out.putShort(cw.newUTF8("Code")).putInt(size);
      out.putShort(maxStack).putShort(maxLocals);
      out.putInt(code.length).putByteArray(code.data, 0, code.length);
      out.putShort(handlerCount);
      if (handlerCount > 0) {
        Handler h = firstHandler;
        while (h != null) {
          out.putShort(start.position).putShort(end.position).putShort(handler.position).putShort(type);
          


          h = next;
        }
      }
      attributeCount = 0;
      if (localVar != null) {
        attributeCount++;
      }
      if (localVarType != null) {
        attributeCount++;
      }
      if (lineNumber != null) {
        attributeCount++;
      }
      if (stackMap != null) {
        attributeCount++;
      }
      if (cattrs != null) {
        attributeCount += cattrs.getCount();
      }
      out.putShort(attributeCount);
      if (localVar != null) {
        out.putShort(cw.newUTF8("LocalVariableTable"));
        out.putInt(localVar.length + 2).putShort(localVarCount);
        out.putByteArray(localVar.data, 0, localVar.length);
      }
      if (localVarType != null) {
        out.putShort(cw.newUTF8("LocalVariableTypeTable"));
        out.putInt(localVarType.length + 2).putShort(localVarTypeCount);
        out.putByteArray(localVarType.data, 0, localVarType.length);
      }
      if (lineNumber != null) {
        out.putShort(cw.newUTF8("LineNumberTable"));
        out.putInt(lineNumber.length + 2).putShort(lineNumberCount);
        out.putByteArray(lineNumber.data, 0, lineNumber.length);
      }
      if (stackMap != null) {
        boolean zip = (cw.version & 0xFFFF) >= 50;
        out.putShort(cw.newUTF8(zip ? "StackMapTable" : "StackMap"));
        out.putInt(stackMap.length + 2).putShort(frameCount);
        out.putByteArray(stackMap.data, 0, stackMap.length);
      }
      if (cattrs != null) {
        cattrs.put(cw, code.data, code.length, maxLocals, maxStack, out);
      }
    }
    if (exceptionCount > 0) {
      out.putShort(cw.newUTF8("Exceptions")).putInt(2 * exceptionCount + 2);
      
      out.putShort(exceptionCount);
      for (int i = 0; i < exceptionCount; i++) {
        out.putShort(exceptions[i]);
      }
    }
    if (((access & 0x1000) != 0) && ((cw.version & 0xFFFF) < 49))
    {

      out.putShort(cw.newUTF8("Synthetic")).putInt(0);
    }
    if ((access & 0x20000) != 0) {
      out.putShort(cw.newUTF8("Deprecated")).putInt(0);
    }
    if (signature != null) {
      out.putShort(cw.newUTF8("Signature")).putInt(2).putShort(cw.newUTF8(signature));
    }
    

    if (annd != null) {
      out.putShort(cw.newUTF8("AnnotationDefault"));
      out.putInt(annd.length);
      out.putByteArray(annd.data, 0, annd.length);
    }
    if (anns != null) {
      out.putShort(cw.newUTF8("RuntimeVisibleAnnotations"));
      anns.put(out);
    }
    if (ianns != null) {
      out.putShort(cw.newUTF8("RuntimeInvisibleAnnotations"));
      ianns.put(out);
    }
    if (panns != null) {
      out.putShort(cw.newUTF8("RuntimeVisibleParameterAnnotations"));
      AnnotationWriter.put(panns, synthetics, out);
    }
    if (ipanns != null) {
      out.putShort(cw.newUTF8("RuntimeInvisibleParameterAnnotations"));
      AnnotationWriter.put(ipanns, synthetics, out);
    }
    if (attrs != null) {
      attrs.put(cw, null, 0, -1, -1, out);
    }
  }
  


















  private void resizeInstructions()
  {
    byte[] b = code.data;
    


























    int[] allIndexes = new int[0];
    int[] allSizes = new int[0];
    


    boolean[] resize = new boolean[code.length];
    

    int state = 3;
    int label;
    do { if (state == 3) {
        state = 2;
      }
      u = 0;
      while (u < b.length) {
        int opcode = b[u] & 0xFF;
        int insert = 0;
        int newOffset;
        switch (ClassWriter.TYPE[opcode]) {
        case 0: 
        case 4: 
          u++;
          break;
        case 8:  int label;
          if (opcode > 201)
          {


            opcode = opcode < 218 ? opcode - 49 : opcode - 20;
            label = u + readUnsignedShort(b, u + 1);
          } else {
            label = u + readShort(b, u + 1);
          }
          newOffset = getNewOffset(allIndexes, allSizes, u, label);
          if ((newOffset < -32768) || (newOffset > 32767))
          {

            if (resize[u] == 0) {
              if ((opcode == 167) || (opcode == 168))
              {




                insert = 2;


              }
              else
              {


                insert = 5;
              }
              resize[u] = true;
            }
          }
          u += 3;
          break;
        case 9: 
          u += 5;
          break;
        case 13: 
          if (state == 1)
          {






            newOffset = getNewOffset(allIndexes, allSizes, 0, u);
            insert = -(newOffset & 0x3);
          } else if (resize[u] == 0)
          {


            insert = u & 0x3;
            resize[u] = true;
          }
          
          u = u + 4 - (u & 0x3);
          u += 4 * (readInt(b, u + 8) - readInt(b, u + 4) + 1) + 12;
          break;
        case 14: 
          if (state == 1)
          {
            int newOffset = getNewOffset(allIndexes, allSizes, 0, u);
            insert = -(newOffset & 0x3);
          } else if (resize[u] == 0)
          {
            insert = u & 0x3;
            resize[u] = true;
          }
          
          u = u + 4 - (u & 0x3);
          u += 8 * readInt(b, u + 4) + 8;
          break;
        case 16: 
          opcode = b[(u + 1)] & 0xFF;
          if (opcode == 132) {
            u += 6;
          } else {
            u += 4;
          }
          break;
        case 1: 
        case 3: 
        case 10: 
          u += 2;
          break;
        case 2: 
        case 5: 
        case 6: 
        case 11: 
        case 12: 
          u += 3;
          break;
        case 7: 
          u += 5;
          break;
        case 15: 
        default: 
          u += 4;
        }
        
        if (insert != 0)
        {

          int[] newIndexes = new int[allIndexes.length + 1];
          int[] newSizes = new int[allSizes.length + 1];
          System.arraycopy(allIndexes, 0, newIndexes, 0, allIndexes.length);
          



          System.arraycopy(allSizes, 0, newSizes, 0, allSizes.length);
          newIndexes[allIndexes.length] = u;
          newSizes[allSizes.length] = insert;
          allIndexes = newIndexes;
          allSizes = newSizes;
          if (insert > 0) {
            state = 3;
          }
        }
      }
      if (state < 3) {
        state--;
      }
    } while (state != 0);
    




    ByteVector newCode = new ByteVector(code.length);
    
    int u = 0;
    int label; int newOffset; while (u < code.length) {
      int opcode = b[u] & 0xFF;
      int v; int j; switch (ClassWriter.TYPE[opcode]) {
      case 0: 
      case 4: 
        newCode.putByte(opcode);
        u++;
        break;
      case 8: 
        if (opcode > 201)
        {


          opcode = opcode < 218 ? opcode - 49 : opcode - 20;
          label = u + readUnsignedShort(b, u + 1);
        } else {
          label = u + readShort(b, u + 1);
        }
        newOffset = getNewOffset(allIndexes, allSizes, u, label);
        if (resize[u] != 0)
        {




          if (opcode == 167) {
            newCode.putByte(200);
          } else if (opcode == 168) {
            newCode.putByte(201);
          } else {
            newCode.putByte(opcode <= 166 ? (opcode + 1 ^ 0x1) - 1 : opcode ^ 0x1);
            

            newCode.putShort(8);
            newCode.putByte(200);
            
            newOffset -= 3;
          }
          newCode.putInt(newOffset);
        } else {
          newCode.putByte(opcode);
          newCode.putShort(newOffset);
        }
        u += 3;
        break;
      case 9: 
        label = u + readInt(b, u + 1);
        newOffset = getNewOffset(allIndexes, allSizes, u, label);
        newCode.putByte(opcode);
        newCode.putInt(newOffset);
        u += 5;
        break;
      
      case 13: 
        v = u;
        u = u + 4 - (v & 0x3);
        
        newCode.putByte(170);
        length += (4 - length % 4) % 4;
        label = v + readInt(b, u);
        u += 4;
        newOffset = getNewOffset(allIndexes, allSizes, v, label);
        newCode.putInt(newOffset);
        j = readInt(b, u);
        u += 4;
        newCode.putInt(j);
        j = readInt(b, u) - j + 1;
        u += 4;
        newCode.putInt(readInt(b, u - 4));
      case 14: case 16: case 1: case 3: case 10: case 2: case 5: case 6: case 11: case 12: case 7: case 15: default:  while (j > 0) {
          label = v + readInt(b, u);
          u += 4;
          newOffset = getNewOffset(allIndexes, allSizes, v, label);
          newCode.putInt(newOffset);j--; continue;
          



          v = u;
          u = u + 4 - (v & 0x3);
          
          newCode.putByte(171);
          length += (4 - length % 4) % 4;
          label = v + readInt(b, u);
          u += 4;
          newOffset = getNewOffset(allIndexes, allSizes, v, label);
          newCode.putInt(newOffset);
          j = readInt(b, u);
          u += 4;
          newCode.putInt(j);
          while (j > 0) {
            newCode.putInt(readInt(b, u));
            u += 4;
            label = v + readInt(b, u);
            u += 4;
            newOffset = getNewOffset(allIndexes, allSizes, v, label);
            newCode.putInt(newOffset);j--; continue;
            


            opcode = b[(u + 1)] & 0xFF;
            if (opcode == 132) {
              newCode.putByteArray(b, u, 6);
              u += 6;
            } else {
              newCode.putByteArray(b, u, 4);
              u += 4;
              
              break;
              


              newCode.putByteArray(b, u, 2);
              u += 2;
              break;
              




              newCode.putByteArray(b, u, 3);
              u += 3;
              break;
              
              newCode.putByteArray(b, u, 5);
              u += 5;
              break;
              

              newCode.putByteArray(b, u, 4);
              u += 4;
            }
          }
        }
      }
    }
    if (frameCount > 0) {
      if (compute == 0) {
        frameCount = 0;
        stackMap = null;
        previousFrame = null;
        frame = null;
        Frame f = new Frame();
        owner = this.labels;
        Type[] args = Type.getArgumentTypes(descriptor);
        f.initInputFrame(cw, access, args, maxLocals);
        visitFrame(f);
        Label l = this.labels;
        while (l != null)
        {



          u = position - 3;
          if (((status & 0x20) != 0) || ((u >= 0) && (resize[u] != 0)))
          {
            getNewOffset(allIndexes, allSizes, l);
            
            visitFrame(frame);
          }
          l = successor;



        }
        




      }
      else
      {




        cw.invalidFrames = true;
      }
    }
    
    Handler h = firstHandler;
    while (h != null) {
      getNewOffset(allIndexes, allSizes, start);
      getNewOffset(allIndexes, allSizes, end);
      getNewOffset(allIndexes, allSizes, handler);
      h = next;
    }
    

    for (int i = 0; i < 2; i++) {
      ByteVector bv = i == 0 ? localVar : localVarType;
      if (bv != null) {
        b = data;
        u = 0;
        while (u < length) {
          label = readUnsignedShort(b, u);
          newOffset = getNewOffset(allIndexes, allSizes, 0, label);
          writeShort(b, u, newOffset);
          label += readUnsignedShort(b, u + 2);
          newOffset = getNewOffset(allIndexes, allSizes, 0, label) - newOffset;
          
          writeShort(b, u + 2, newOffset);
          u += 10;
        }
      }
    }
    if (lineNumber != null) {
      b = lineNumber.data;
      u = 0;
      while (u < lineNumber.length) {
        writeShort(b, u, getNewOffset(allIndexes, allSizes, 0, readUnsignedShort(b, u)));
        


        u += 4;
      }
    }
    
    Attribute attr = cattrs;
    while (attr != null) {
      Label[] labels = attr.getLabels();
      if (labels != null) {
        for (i = labels.length - 1; i >= 0; i--) {
          getNewOffset(allIndexes, allSizes, labels[i]);
        }
      }
      attr = next;
    }
    

    code = newCode;
  }
  






  static int readUnsignedShort(byte[] b, int index)
  {
    return (b[index] & 0xFF) << 8 | b[(index + 1)] & 0xFF;
  }
  






  static short readShort(byte[] b, int index)
  {
    return (short)((b[index] & 0xFF) << 8 | b[(index + 1)] & 0xFF);
  }
  






  static int readInt(byte[] b, int index)
  {
    return (b[index] & 0xFF) << 24 | (b[(index + 1)] & 0xFF) << 16 | (b[(index + 2)] & 0xFF) << 8 | b[(index + 3)] & 0xFF;
  }
  







  static void writeShort(byte[] b, int index, int s)
  {
    b[index] = ((byte)(s >>> 8));
    b[(index + 1)] = ((byte)s);
  }
  

























  static int getNewOffset(int[] indexes, int[] sizes, int begin, int end)
  {
    int offset = end - begin;
    for (int i = 0; i < indexes.length; i++) {
      if ((begin < indexes[i]) && (indexes[i] <= end))
      {
        offset += sizes[i];
      } else if ((end < indexes[i]) && (indexes[i] <= begin))
      {
        offset -= sizes[i];
      }
    }
    return offset;
  }
  



















  static void getNewOffset(int[] indexes, int[] sizes, Label label)
  {
    if ((status & 0x4) == 0) {
      position = getNewOffset(indexes, sizes, 0, position);
      status |= 0x4;
    }
  }
}
