package com.avaje.ebean.meta;

import com.avaje.ebean.bean.ObjectGraphOrigin;
import java.io.Serializable;
import java.util.Arrays;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.Transient;












@Entity
public class MetaAutoFetchStatistic
  implements Serializable
{
  private static final long serialVersionUID = -6640406753257176803L;
  @Id
  private String id;
  private ObjectGraphOrigin origin;
  private String beanType;
  private int counter;
  @Transient
  private List<QueryStats> queryStats;
  @Transient
  private List<NodeUsageStats> nodeUsageStats;
  
  public MetaAutoFetchStatistic() {}
  
  public MetaAutoFetchStatistic(ObjectGraphOrigin origin, int counter, List<QueryStats> queryStats, List<NodeUsageStats> nodeUsageStats)
  {
    this.origin = origin;
    beanType = (origin == null ? null : origin.getBeanType());
    id = (origin == null ? null : origin.getKey());
    this.counter = counter;
    this.queryStats = queryStats;
    this.nodeUsageStats = nodeUsageStats;
  }
  


  public String getId()
  {
    return id;
  }
  


  public String getBeanType()
  {
    return beanType;
  }
  


  public ObjectGraphOrigin getOrigin()
  {
    return origin;
  }
  


  public int getCounter()
  {
    return counter;
  }
  


  public List<QueryStats> getQueryStats()
  {
    return queryStats;
  }
  


  public List<NodeUsageStats> getNodeUsageStats()
  {
    return nodeUsageStats;
  }
  

  public static class QueryStats
    implements Serializable
  {
    private static final long serialVersionUID = -5517935732867671387L;
    
    private final String path;
    
    private final int exeCount;
    
    private final int totalBeanLoaded;
    
    private final int totalMicros;
    
    public QueryStats(String path, int exeCount, int totalBeanLoaded, int totalMicros)
    {
      this.path = path;
      this.exeCount = exeCount;
      this.totalBeanLoaded = totalBeanLoaded;
      this.totalMicros = totalMicros;
    }
    



    public String getPath()
    {
      return path;
    }
    


    public int getExeCount()
    {
      return exeCount;
    }
    


    public int getTotalBeanLoaded()
    {
      return totalBeanLoaded;
    }
    


    public int getTotalMicros()
    {
      return totalMicros;
    }
    
    public String toString() {
      long avgMicros = exeCount == 0 ? 0L : totalMicros / exeCount;
      
      return "queryExe path[" + path + "] count[" + exeCount + "] totalBeansLoaded[" + totalBeanLoaded + "] avgMicros[" + avgMicros + "] totalMicros[" + totalMicros + "]";
    }
  }
  


  public static class NodeUsageStats
    implements Serializable
  {
    private static final long serialVersionUID = 1786787832374844739L;
    
    private final String path;
    
    private final int profileCount;
    
    private final int profileUsedCount;
    
    private final String[] usedProperties;
    

    public NodeUsageStats(String path, int profileCount, int profileUsedCount, String[] usedProperties)
    {
      this.path = (path == null ? "" : path);
      this.profileCount = profileCount;
      this.profileUsedCount = profileUsedCount;
      this.usedProperties = usedProperties;
    }
    



    public String getPath()
    {
      return path;
    }
    


    public int getProfileCount()
    {
      return profileCount;
    }
    







    public int getProfileUsedCount()
    {
      return profileUsedCount;
    }
    


    public String[] getUsedProperties()
    {
      return usedProperties;
    }
    


    public Set<String> getUsedPropertiesSet()
    {
      LinkedHashSet<String> s = new LinkedHashSet();
      for (int i = 0; i < usedProperties.length; i++) {
        s.add(usedProperties[i]);
      }
      return s;
    }
    
    public String toString() {
      return "path[" + path + "] profileCount[" + profileCount + "] used[" + profileUsedCount + "] props" + Arrays.toString(usedProperties);
    }
  }
}
