package com.avaje.ebean.text;

import com.avaje.ebean.Query;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;







































public class PathProperties
{
  private final Map<String, Props> pathMap;
  private final Props rootProps;
  
  public static PathProperties parse(String source)
  {
    return PathPropertiesParser.parse(source);
  }
  


  public PathProperties()
  {
    rootProps = new Props(this, null, null, null);
    pathMap = new LinkedHashMap();
    pathMap.put(null, rootProps);
  }
  


  private PathProperties(PathProperties orig)
  {
    rootProps = rootProps.copy(this);
    pathMap = new LinkedHashMap(pathMap.size());
    Set<Map.Entry<String, Props>> entrySet = pathMap.entrySet();
    for (Map.Entry<String, Props> e : entrySet) {
      pathMap.put(e.getKey(), ((Props)e.getValue()).copy(this));
    }
  }
  









  public PathProperties copy()
  {
    return new PathProperties(this);
  }
  


  public boolean isEmpty()
  {
    return pathMap.isEmpty();
  }
  
  public String toString() {
    return pathMap.toString();
  }
  


  public boolean hasPath(String path)
  {
    Props props = (Props)pathMap.get(path);
    return (props != null) && (!props.isEmpty());
  }
  


  public Set<String> get(String path)
  {
    Props props = (Props)pathMap.get(path);
    return props == null ? null : props.getProperties();
  }
  
  public void addToPath(String path, String property) {
    Props props = (Props)pathMap.get(path);
    if (props == null) {
      props = new Props(this, null, path, null);
      pathMap.put(path, props);
    }
    props.getProperties().add(property);
  }
  


  public void put(String path, Set<String> properties)
  {
    pathMap.put(path, new Props(this, null, path, properties, null));
  }
  


  public Set<String> remove(String path)
  {
    Props props = (Props)pathMap.remove(path);
    return props == null ? null : props.getProperties();
  }
  


  public Set<String> getPaths()
  {
    return new LinkedHashSet(pathMap.keySet());
  }
  
  public Collection<Props> getPathProps() {
    return pathMap.values();
  }
  



  public void apply(Query<?> query)
  {
    for (Map.Entry<String, Props> entry : pathMap.entrySet()) {
      String path = (String)entry.getKey();
      String props = ((Props)entry.getValue()).getPropertiesAsString();
      
      if ((path == null) || (path.length() == 0)) {
        query.select(props);
      } else {
        query.fetch(path, props);
      }
    }
  }
  
  protected Props getRootProperties() {
    return rootProps;
  }
  

  public static class Props
  {
    private final PathProperties owner;
    private final String parentPath;
    private final String path;
    private final Set<String> propSet;
    
    private Props(PathProperties owner, String parentPath, String path, Set<String> propSet)
    {
      this.owner = owner;
      this.path = path;
      this.parentPath = parentPath;
      this.propSet = propSet;
    }
    
    private Props(PathProperties owner, String parentPath, String path) {
      this(owner, parentPath, path, new LinkedHashSet());
    }
    


    public Props copy(PathProperties newOwner)
    {
      return new Props(newOwner, parentPath, path, new LinkedHashSet(propSet));
    }
    
    public String getPath() {
      return path;
    }
    
    public String toString() {
      return propSet.toString();
    }
    
    public boolean isEmpty() {
      return propSet.isEmpty();
    }
    


    public Set<String> getProperties()
    {
      return propSet;
    }
    



    public String getPropertiesAsString()
    {
      StringBuilder sb = new StringBuilder();
      
      Iterator<String> it = propSet.iterator();
      boolean hasNext = it.hasNext();
      while (hasNext) {
        sb.append((String)it.next());
        hasNext = it.hasNext();
        if (hasNext) {
          sb.append(",");
        }
      }
      return sb.toString();
    }
    


    protected Props getParent()
    {
      return (Props)owner.pathMap.get(parentPath);
    }
    



    protected Props addChild(String subpath)
    {
      subpath = subpath.trim();
      addProperty(subpath);
      

      String p = path + "." + subpath;
      Props nested = new Props(owner, path, p);
      owner.pathMap.put(p, nested);
      return nested;
    }
    


    protected void addProperty(String property)
    {
      propSet.add(property.trim());
    }
  }
}
