package com.avaje.ebean.validation.factory;

import java.util.Date;

public class FutureValidatorFactory implements ValidatorFactory
{
  Validator DATE;
  Validator CALENDAR;
  
  public FutureValidatorFactory()
  {
    DATE = new DateValidator(null);
    
    CALENDAR = new CalendarValidator(null);
  }
  
  public Validator create(java.lang.annotation.Annotation annotation, Class<?> type) { if (Date.class.isAssignableFrom(type)) {
      return DATE;
    }
    if (java.util.Calendar.class.isAssignableFrom(type)) {
      return CALENDAR;
    }
    String msg = "Can not use @Future on type " + type;
    throw new RuntimeException(msg);
  }
  
  private static class DateValidator extends NoAttributesValidator
  {
    public String getKey() {
      return "future";
    }
    
    public boolean isValid(Object value) {
      if (value == null) {
        return true;
      }
      
      Date date = (Date)value;
      return date.after(new Date());
    }
  }
  
  private static class CalendarValidator extends NoAttributesValidator
  {
    public String getKey() {
      return "future";
    }
    
    public boolean isValid(Object value) {
      if (value == null) {
        return true;
      }
      
      java.util.Calendar cal = (java.util.Calendar)value;
      return cal.after(java.util.Calendar.getInstance());
    }
  }
}
