package com.avaje.ebeaninternal.api;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map.Entry;
import java.util.Set;




















public class BindParams
  implements Serializable
{
  private static final long serialVersionUID = 4541081933302086285L;
  private ArrayList<Param> positionedParameters;
  private HashMap<String, Param> namedParameters;
  private int queryPlanHash;
  private String preparedSql;
  
  public BindParams()
  {
    positionedParameters = new ArrayList();
    
    namedParameters = new HashMap();
    



    queryPlanHash = 1;
  }
  








  public BindParams copy()
  {
    BindParams copy = new BindParams();
    for (Param p : positionedParameters) {
      positionedParameters.add(p.copy());
    }
    Iterator<Map.Entry<String, Param>> it = namedParameters.entrySet().iterator();
    while (it.hasNext()) {
      Map.Entry<String, Param> entry = (Map.Entry)it.next();
      namedParameters.put(entry.getKey(), ((Param)entry.getValue()).copy());
    }
    return copy;
  }
  
  public int queryBindHash() {
    int hc = namedParameters.hashCode();
    for (int i = 0; i < positionedParameters.size(); i++) {
      hc = hc * 31 + ((Param)positionedParameters.get(i)).hashCode();
    }
    return hc;
  }
  
  public int hashCode() {
    int hc = getClass().hashCode();
    hc = hc * 31 + namedParameters.hashCode();
    for (int i = 0; i < positionedParameters.size(); i++) {
      hc = hc * 31 + ((Param)positionedParameters.get(i)).hashCode();
    }
    hc = hc * 31 + (preparedSql == null ? 0 : preparedSql.hashCode());
    return hc;
  }
  
  public boolean equals(Object o) {
    if (o == null) {
      return false;
    }
    if (o == this) {
      return true;
    }
    if ((o instanceof BindParams)) {
      return hashCode() == o.hashCode();
    }
    return false;
  }
  


  public boolean isEmpty()
  {
    return (positionedParameters.isEmpty()) && (namedParameters.isEmpty());
  }
  
  public int size() {
    return positionedParameters.size();
  }
  




  public boolean requiresNamedParamsPrepare()
  {
    return (!namedParameters.isEmpty()) && (positionedParameters.isEmpty());
  }
  


  public void setNullParameter(int position, int jdbcType)
  {
    Param p = getParam(position);
    p.setInNullType(jdbcType);
  }
  



  public void setParameter(int position, Object value, int outType)
  {
    addToQueryPlanHash(String.valueOf(position), value);
    
    Param p = getParam(position);
    p.setInValue(value);
    p.setOutType(outType);
  }
  




  public void setParameter(int position, Object value)
  {
    addToQueryPlanHash(String.valueOf(position), value);
    
    Param p = getParam(position);
    p.setInValue(value);
  }
  


  public void registerOut(int position, int outType)
  {
    Param p = getParam(position);
    p.setOutType(outType);
  }
  
  private Param getParam(String name) {
    Param p = (Param)namedParameters.get(name);
    if (p == null) {
      p = new Param();
      namedParameters.put(name, p);
    }
    return p;
  }
  
  private Param getParam(int position) {
    int more = position - positionedParameters.size();
    if (more > 0) {
      for (int i = 0; i < more; i++) {
        positionedParameters.add(new Param());
      }
    }
    return (Param)positionedParameters.get(position - 1);
  }
  



  public void setParameter(String name, Object value, int outType)
  {
    addToQueryPlanHash(name, value);
    
    Param p = getParam(name);
    p.setInValue(value);
    p.setOutType(outType);
  }
  


  public void setNullParameter(String name, int jdbcType)
  {
    Param p = getParam(name);
    p.setInNullType(jdbcType);
  }
  



  public Param setParameter(String name, Object value)
  {
    addToQueryPlanHash(name, value);
    
    Param p = getParam(name);
    p.setInValue(value);
    return p;
  }
  


  private void addToQueryPlanHash(String name, Object value)
  {
    if ((value != null) && 
      ((value instanceof Collection))) {
      queryPlanHash = (queryPlanHash * 31 + name.hashCode());
      queryPlanHash = (queryPlanHash * 31 + ((Collection)value).size());
    }
  }
  








  public int getQueryPlanHash()
  {
    return queryPlanHash;
  }
  





  public Param setEncryptionKey(String name, Object value)
  {
    Param p = getParam(name);
    p.setEncryptionKey(value);
    return p;
  }
  


  public void registerOut(String name, int outType)
  {
    Param p = getParam(name);
    p.setOutType(outType);
  }
  



  public Param getParameter(int position)
  {
    return getParam(position);
  }
  


  public Param getParameter(String name)
  {
    return getParam(name);
  }
  


  public List<Param> positionedParameters()
  {
    return positionedParameters;
  }
  


  public void setPreparedSql(String preparedSql)
  {
    this.preparedSql = preparedSql;
  }
  



  public String getPreparedSql()
  {
    return preparedSql;
  }
  



  public static final class OrderedList
  {
    final List<BindParams.Param> paramList;
    

    final StringBuilder preparedSql;
    


    public OrderedList()
    {
      this(new ArrayList());
    }
    
    public OrderedList(List<BindParams.Param> paramList) {
      this.paramList = paramList;
      preparedSql = new StringBuilder();
    }
    


    public void add(BindParams.Param param)
    {
      paramList.add(param);
    }
    


    public int size()
    {
      return paramList.size();
    }
    


    public List<BindParams.Param> list()
    {
      return paramList;
    }
    


    public void appendSql(String parsedSql)
    {
      preparedSql.append(parsedSql);
    }
    
    public String getPreparedSql() {
      return preparedSql.toString();
    }
  }
  


  public static final class Param
  {
    private boolean encryptionKey;
    

    private boolean isInParam;
    

    private boolean isOutParam;
    

    private int type;
    

    private Object inValue;
    

    private Object outValue;
    

    private int textLocation;
    


    public Param copy()
    {
      Param copy = new Param();
      isInParam = isInParam;
      isOutParam = isOutParam;
      type = type;
      inValue = inValue;
      outValue = outValue;
      return copy;
    }
    
    public int hashCode() {
      int hc = getClass().hashCode();
      hc = hc * 31 + (isInParam ? 0 : 1);
      hc = hc * 31 + (isOutParam ? 0 : 1);
      hc = hc * 31 + type;
      hc = hc * 31 + (inValue == null ? 0 : inValue.hashCode());
      return hc;
    }
    
    public boolean equals(Object o) {
      if (o == null) {
        return false;
      }
      if (o == this) {
        return true;
      }
      if ((o instanceof Param)) {
        return hashCode() == o.hashCode();
      }
      return false;
    }
    



    public boolean isInParam()
    {
      return isInParam;
    }
    



    public boolean isOutParam()
    {
      return isOutParam;
    }
    



    public int getType()
    {
      return type;
    }
    


    public void setOutType(int type)
    {
      this.type = type;
      isOutParam = true;
    }
    


    public void setInValue(Object in)
    {
      inValue = in;
      isInParam = true;
    }
    


    public void setEncryptionKey(Object in)
    {
      inValue = in;
      isInParam = true;
      encryptionKey = true;
    }
    



    public void setInNullType(int type)
    {
      this.type = type;
      inValue = null;
      isInParam = true;
    }
    



    public Object getOutValue()
    {
      return outValue;
    }
    



    public Object getInValue()
    {
      return inValue;
    }
    



    public void setOutValue(Object out)
    {
      outValue = out;
    }
    


    public int getTextLocation()
    {
      return textLocation;
    }
    



    public void setTextLocation(int textLocation)
    {
      this.textLocation = textLocation;
    }
    


    public boolean isEncryptionKey()
    {
      return encryptionKey;
    }
  }
}
