package com.avaje.ebeaninternal.api;

import java.util.logging.Level;
import java.util.logging.Logger;





























class ClassLoadContext
{
  private static final Logger logger = Logger.getLogger(ClassLoadContext.class.getName());
  
  private final ClassLoader callerLoader;
  
  private final ClassLoader contextLoader;
  
  private final boolean preferContext;
  private boolean ambiguous;
  
  public static ClassLoadContext of(Class<?> caller, boolean preferContext)
  {
    return new ClassLoadContext(caller, preferContext);
  }
  



  ClassLoadContext(Class<?> caller, boolean preferContext)
  {
    if (caller == null) {
      throw new IllegalArgumentException("caller is null");
    }
    callerLoader = caller.getClassLoader();
    contextLoader = Thread.currentThread().getContextClassLoader();
    this.preferContext = preferContext;
  }
  
  public Class<?> forName(String name) throws ClassNotFoundException
  {
    ClassLoader defaultLoader = getDefault(preferContext);
    try
    {
      return Class.forName(name, true, defaultLoader);
    } catch (ClassNotFoundException e) {
      if (callerLoader == defaultLoader)
        throw e;
    }
    return Class.forName(name, true, callerLoader);
  }
  










  public ClassLoader getDefault(boolean preferContext)
  {
    if (contextLoader == null) {
      if (logger.isLoggable(Level.FINE)) {
        logger.fine("No Context ClassLoader, using " + callerLoader.getClass().getName());
      }
      return callerLoader;
    }
    if (contextLoader == callerLoader) {
      if (logger.isLoggable(Level.FINE)) {
        logger.fine("Context and Caller ClassLoader's same instance of " + contextLoader.getClass().getName());
      }
      return callerLoader;
    }
    
    if (isChild(contextLoader, callerLoader)) {
      if (logger.isLoggable(Level.FINE)) {
        logger.info("Caller ClassLoader " + callerLoader.getClass().getName() + " child of ContextLoader " + contextLoader.getClass().getName());
      }
      
      return callerLoader;
    }
    if (isChild(callerLoader, contextLoader)) {
      if (logger.isLoggable(Level.FINE)) {
        logger.info("Context ClassLoader " + contextLoader.getClass().getName() + " child of Caller ClassLoader " + callerLoader.getClass().getName());
      }
      
      return contextLoader;
    }
    

    logger.info("Ambiguous ClassLoader choice preferContext:" + preferContext + " Context:" + contextLoader.getClass().getName() + " Caller:" + callerLoader.getClass().getName());
    
    ambiguous = true;
    return preferContext ? contextLoader : callerLoader;
  }
  



  public boolean isAmbiguous()
  {
    return ambiguous;
  }
  


  public ClassLoader getCallerLoader()
  {
    return callerLoader;
  }
  


  public ClassLoader getContextLoader()
  {
    return contextLoader;
  }
  


  public ClassLoader getThisLoader()
  {
    return getClass().getClassLoader();
  }
  
  private boolean isChild(ClassLoader loader1, ClassLoader loader2)
  {
    for (; 
        














        loader2 != null; loader2 = loader2.getParent()) {
      if (loader2 == loader1) {
        return true;
      }
    }
    
    return false;
  }
}
