package com.avaje.ebeaninternal.api;

import com.avaje.ebeaninternal.server.core.PersistRequestBean;
import com.avaje.ebeaninternal.server.deploy.BeanDescriptor;
import com.avaje.ebeaninternal.server.transaction.BeanDelta;
import com.avaje.ebeaninternal.server.transaction.DeleteByIdMap;
import com.avaje.ebeaninternal.server.transaction.IndexInvalidate;
import java.io.PrintStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;









































public class TransactionEvent
  implements Serializable
{
  private static final long serialVersionUID = 7230903304106097120L;
  private transient boolean local;
  private boolean invalidateAll;
  private TransactionEventTable eventTables;
  private transient TransactionEventBeans eventBeans;
  private transient List<BeanDelta> beanDeltas;
  private transient DeleteByIdMap deleteByIdMap;
  private transient Set<IndexInvalidate> indexInvalidations;
  private transient Set<String> pauseIndexInvalidate;
  
  public TransactionEvent()
  {
    local = true;
  }
  


  public void setInvalidateAll(boolean isInvalidateAll)
  {
    invalidateAll = isInvalidateAll;
  }
  



  public boolean isInvalidateAll()
  {
    return invalidateAll;
  }
  


  public void pauseIndexInvalidate(Class<?> beanType)
  {
    if (pauseIndexInvalidate == null) {
      pauseIndexInvalidate = new HashSet();
    }
    pauseIndexInvalidate.add(beanType.getName());
  }
  


  public void resumeIndexInvalidate(Class<?> beanType)
  {
    if (pauseIndexInvalidate != null) {
      pauseIndexInvalidate.remove(beanType.getName());
    }
  }
  


  public void addIndexInvalidate(IndexInvalidate indexEvent)
  {
    if ((pauseIndexInvalidate != null) && (pauseIndexInvalidate.contains(indexEvent.getIndexName()))) {
      System.out.println("--- IGNORE Invalidate on " + indexEvent.getIndexName());
      return;
    }
    if (indexInvalidations == null) {
      indexInvalidations = new HashSet();
    }
    indexInvalidations.add(indexEvent);
  }
  
  public void addDeleteById(BeanDescriptor<?> desc, Object id) {
    if (deleteByIdMap == null) {
      deleteByIdMap = new DeleteByIdMap();
    }
    deleteByIdMap.add(desc, id);
  }
  
  public void addDeleteByIdList(BeanDescriptor<?> desc, List<Object> idList) {
    if (deleteByIdMap == null) {
      deleteByIdMap = new DeleteByIdMap();
    }
    deleteByIdMap.addList(desc, idList);
  }
  
  public DeleteByIdMap getDeleteByIdMap() {
    return deleteByIdMap;
  }
  
  public void addBeanDelta(BeanDelta delta) {
    if (beanDeltas == null) {
      beanDeltas = new ArrayList();
    }
    beanDeltas.add(delta);
  }
  
  public List<BeanDelta> getBeanDeltas() {
    return beanDeltas;
  }
  



  public boolean isLocal()
  {
    return local;
  }
  


  public TransactionEventBeans getEventBeans()
  {
    return eventBeans;
  }
  
  public TransactionEventTable getEventTables() {
    return eventTables;
  }
  
  public Set<IndexInvalidate> getIndexInvalidations() {
    return indexInvalidations;
  }
  
  public void add(String tableName, boolean inserts, boolean updates, boolean deletes) {
    if (eventTables == null) {
      eventTables = new TransactionEventTable();
    }
    eventTables.add(tableName, inserts, updates, deletes);
  }
  
  public void add(TransactionEventTable table) {
    if (eventTables == null) {
      eventTables = new TransactionEventTable();
    }
    eventTables.add(table);
  }
  



  public void add(PersistRequestBean<?> request)
  {
    if (request.isNotify(this))
    {
      if (eventBeans == null) {
        eventBeans = new TransactionEventBeans();
      }
      eventBeans.add(request);
    }
  }
  







  public void notifyCache()
  {
    if (eventBeans != null) {
      eventBeans.notifyCache();
    }
    if (deleteByIdMap != null) {
      deleteByIdMap.notifyCache();
    }
  }
}
