package com.avaje.ebeaninternal.api;

import com.avaje.ebeaninternal.server.cluster.BinaryMessage;
import com.avaje.ebeaninternal.server.cluster.BinaryMessageList;
import java.io.DataInput;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.Serializable;
import java.util.Map;

public final class TransactionEventTable implements Serializable
{
  private static final long serialVersionUID = 2236555729767483264L;
  private final Map<String, TableIUD> map;
  
  public TransactionEventTable()
  {
    map = new java.util.HashMap();
  }
  
  public String toString() { return "TransactionEventTable " + map.values(); }
  
  public void writeBinaryMessage(BinaryMessageList msgList)
    throws IOException
  {
    for (TableIUD tableIud : map.values()) {
      tableIud.writeBinaryMessage(msgList);
    }
  }
  
  public void readBinaryMessage(DataInput dataInput) throws IOException
  {
    TableIUD tableIud = TableIUD.readBinaryMessage(dataInput);
    map.put(tableIud.getTable(), tableIud);
  }
  

  public void add(TransactionEventTable table)
  {
    for (TableIUD iud : table.values()) {
      add(iud);
    }
  }
  
  public void add(String table, boolean insert, boolean update, boolean delete)
  {
    table = table.toUpperCase();
    
    add(new TableIUD(table, insert, update, delete, null));
  }
  
  public void add(TableIUD newTableIUD)
  {
    TableIUD existingTableIUD = (TableIUD)map.put(newTableIUD.getTable(), newTableIUD);
    if (existingTableIUD != null) {
      newTableIUD.add(existingTableIUD);
    }
  }
  
  public boolean isEmpty() {
    return map.isEmpty();
  }
  
  public java.util.Collection<TableIUD> values() {
    return map.values();
  }
  
  public static class TableIUD implements Serializable
  {
    private static final long serialVersionUID = -1958317571064162089L;
    private String table;
    private boolean insert;
    private boolean update;
    private boolean delete;
    
    private TableIUD(String table, boolean insert, boolean update, boolean delete)
    {
      this.table = table;
      this.insert = insert;
      this.update = update;
      this.delete = delete;
    }
    
    public static TableIUD readBinaryMessage(DataInput dataInput) throws IOException
    {
      String table = dataInput.readUTF();
      boolean insert = dataInput.readBoolean();
      boolean update = dataInput.readBoolean();
      boolean delete = dataInput.readBoolean();
      
      return new TableIUD(table, insert, update, delete);
    }
    
    public void writeBinaryMessage(BinaryMessageList msgList) throws IOException
    {
      BinaryMessage msg = new BinaryMessage(table.length() + 10);
      DataOutputStream os = msg.getOs();
      os.writeInt(2);
      os.writeUTF(table);
      os.writeBoolean(insert);
      os.writeBoolean(update);
      os.writeBoolean(delete);
      
      msgList.add(msg);
    }
    
    public String toString() {
      return "TableIUD " + table + " i:" + insert + " u:" + update + " d:" + delete;
    }
    
    private void add(TableIUD other) {
      if (insert) {
        insert = true;
      }
      if (update) {
        update = true;
      }
      if (delete) {
        delete = true;
      }
    }
    
    public String getTable() {
      return table;
    }
    
    public boolean isInsert() {
      return insert;
    }
    
    public boolean isUpdate() {
      return update;
    }
    
    public boolean isDelete() {
      return delete;
    }
    
    public boolean isUpdateOrDelete() {
      return (update) || (delete);
    }
  }
}
