package com.avaje.ebeaninternal.server.autofetch;

import com.avaje.ebean.bean.ObjectGraphOrigin;
import com.avaje.ebean.meta.MetaAutoFetchTunedQueryInfo;
import com.avaje.ebeaninternal.api.SpiQuery;
import com.avaje.ebeaninternal.server.querydefn.OrmQueryDetail;
import java.io.Serializable;















public class TunedQueryInfo
  implements Serializable
{
  private static final long serialVersionUID = 7381493228797997282L;
  private final ObjectGraphOrigin origin;
  private OrmQueryDetail tunedDetail;
  private int profileCount;
  private Long lastTuneTime = Long.valueOf(0L);
  
  private final String rateMonitor = new String();
  

  private transient int tunedCount;
  

  private transient int rateTotal;
  
  private transient int rateHits;
  
  private transient double lastRate;
  

  public TunedQueryInfo(ObjectGraphOrigin queryPoint, OrmQueryDetail tunedDetail, int profileCount)
  {
    origin = queryPoint;
    this.tunedDetail = tunedDetail;
    this.profileCount = profileCount;
  }
  



  public boolean isPercentageProfile(double rate)
  {
    synchronized (rateMonitor)
    {
      if (lastRate != rate)
      {
        lastRate = rate;
        rateTotal = 0;
        rateHits = 0;
      }
      
      rateTotal += 1;
      if (rate > rateHits / rateTotal) {
        rateHits += 1;
        return true;
      }
      return false;
    }
  }
  



  public MetaAutoFetchTunedQueryInfo createPublicMeta()
  {
    return new MetaAutoFetchTunedQueryInfo(origin, tunedDetail.toString(), profileCount, tunedCount, lastTuneTime.longValue());
  }
  




  public void setProfileCount(int profileCount)
  {
    this.profileCount = profileCount;
  }
  



  public void setTunedDetail(OrmQueryDetail tunedDetail)
  {
    this.tunedDetail = tunedDetail;
    lastTuneTime = Long.valueOf(System.currentTimeMillis());
  }
  


  public boolean isSame(OrmQueryDetail newQueryDetail)
  {
    if (tunedDetail == null) {
      return false;
    }
    return tunedDetail.isAutoFetchEqual(newQueryDetail);
  }
  




  public boolean autoFetchTune(SpiQuery<?> query)
  {
    if (tunedDetail == null) {
      return false;
    }
    
    boolean tuned = false;
    
    if (query.isDetailEmpty()) {
      tuned = true;
      
      query.setDetail(tunedDetail.copy());
    }
    else {
      tuned = query.tuneFetchProperties(tunedDetail);
    }
    if (tuned) {
      query.setAutoFetchTuned(true);
      
      tunedCount += 1;
    }
    return tuned;
  }
  


  public Long getLastTuneTime()
  {
    return lastTuneTime;
  }
  


  public int getTunedCount()
  {
    return tunedCount;
  }
  



  public int getProfileCount()
  {
    return profileCount;
  }
  
  public OrmQueryDetail getTunedDetail() {
    return tunedDetail;
  }
  
  public ObjectGraphOrigin getOrigin() {
    return origin;
  }
  
  public String getLogOutput(OrmQueryDetail newQueryDetail)
  {
    boolean changed = newQueryDetail != null;
    
    StringBuilder sb = new StringBuilder(150);
    sb.append(changed ? "\"Changed\"," : "\"New\",");
    sb.append("\"").append(origin.getBeanType()).append("\",");
    sb.append("\"").append(origin.getKey()).append("\",");
    if (changed) {
      sb.append("\"to: ").append(newQueryDetail.toString()).append("\",");
      sb.append("\"from: ").append(tunedDetail.toString()).append("\",");
    } else {
      sb.append("\"to: ").append(tunedDetail.toString()).append("\",");
      sb.append("\"\",");
    }
    sb.append("\"").append(origin.getFirstStackElement()).append("\"");
    
    return sb.toString();
  }
  
  public String toString() {
    return origin.getBeanType() + " " + origin.getKey() + " " + tunedDetail;
  }
}
