package com.avaje.ebeaninternal.server.cache;

import com.avaje.ebean.annotation.CacheTuning;
import com.avaje.ebean.cache.ServerCache;
import com.avaje.ebean.cache.ServerCacheFactory;
import com.avaje.ebean.cache.ServerCacheOptions;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.concurrent.ConcurrentHashMap;




public class DefaultCacheHolder
{
  private final ConcurrentHashMap<Class<?>, ServerCache> concMap = new ConcurrentHashMap();
  
  private final HashMap<Class<?>, ServerCache> synchMap = new HashMap();
  
  private final Object monitor = new Object();
  



  private final ServerCacheFactory cacheFactory;
  



  private final ServerCacheOptions defaultOptions;
  



  private final boolean useBeanTuning;
  



  public DefaultCacheHolder(ServerCacheFactory cacheFactory, ServerCacheOptions defaultOptions, boolean useBeanTuning)
  {
    this.cacheFactory = cacheFactory;
    this.defaultOptions = defaultOptions;
    this.useBeanTuning = useBeanTuning;
  }
  


  public ServerCacheOptions getDefaultOptions()
  {
    return defaultOptions;
  }
  



  public ServerCache getCache(Class<?> beanType)
  {
    ServerCache cache = (ServerCache)concMap.get(beanType);
    if (cache != null) {
      return cache;
    }
    synchronized (monitor) {
      cache = (ServerCache)synchMap.get(beanType);
      if (cache == null) {
        ServerCacheOptions options = getCacheOptions(beanType);
        cache = cacheFactory.createCache(beanType, options);
        synchMap.put(beanType, cache);
        concMap.put(beanType, cache);
      }
      return cache;
    }
  }
  


  public boolean isCaching(Class<?> beanType)
  {
    return concMap.containsKey(beanType);
  }
  
  public void clearAll() {
    Iterator<ServerCache> it = concMap.values().iterator();
    while (it.hasNext()) {
      ServerCache serverCache = (ServerCache)it.next();
      serverCache.clear();
    }
  }
  



  private ServerCacheOptions getCacheOptions(Class<?> beanType)
  {
    if (useBeanTuning)
    {
      CacheTuning cacheTuning = (CacheTuning)beanType.getAnnotation(CacheTuning.class);
      if (cacheTuning != null) {
        ServerCacheOptions o = new ServerCacheOptions(cacheTuning);
        o.applyDefaults(defaultOptions);
        return o;
      }
    }
    
    return defaultOptions.copy();
  }
}
