package com.avaje.ebeaninternal.server.cache;

import com.avaje.ebean.EbeanServer;
import com.avaje.ebean.cache.ServerCache;
import com.avaje.ebean.cache.ServerCacheFactory;
import com.avaje.ebean.cache.ServerCacheManager;
import com.avaje.ebean.cache.ServerCacheOptions;








public class DefaultServerCacheManager
  implements ServerCacheManager
{
  private final DefaultCacheHolder beanCache;
  private final DefaultCacheHolder queryCache;
  private final ServerCacheFactory cacheFactory;
  
  public DefaultServerCacheManager(ServerCacheFactory cacheFactory, ServerCacheOptions defaultBeanOptions, ServerCacheOptions defaultQueryOptions)
  {
    this.cacheFactory = cacheFactory;
    beanCache = new DefaultCacheHolder(cacheFactory, defaultBeanOptions, true);
    queryCache = new DefaultCacheHolder(cacheFactory, defaultQueryOptions, false);
  }
  
  public void init(EbeanServer server)
  {
    cacheFactory.init(server);
  }
  



  public void clear(Class<?> beanType)
  {
    if (isBeanCaching(beanType)) {
      getBeanCache(beanType).clear();
    }
    if (isQueryCaching(beanType)) {
      getQueryCache(beanType).clear();
    }
  }
  
  public void clearAll()
  {
    beanCache.clearAll();
    queryCache.clearAll();
  }
  



  public ServerCache getQueryCache(Class<?> beanType)
  {
    return queryCache.getCache(beanType);
  }
  


  public ServerCache getBeanCache(Class<?> beanType)
  {
    return beanCache.getCache(beanType);
  }
  



  public boolean isBeanCaching(Class<?> beanType)
  {
    return beanCache.isCaching(beanType);
  }
  
  public boolean isQueryCaching(Class<?> beanType)
  {
    return queryCache.isCaching(beanType);
  }
}
