package com.avaje.ebeaninternal.server.cluster.mcast;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;
import java.util.logging.Logger;



































public class IncomingPacketsProcessed
{
  private final ConcurrentHashMap<String, GotAllPoint> mapByMember = new ConcurrentHashMap();
  private final int maxResendIncoming;
  
  public IncomingPacketsProcessed(int maxResendIncoming)
  {
    this.maxResendIncoming = maxResendIncoming;
  }
  
  public void removeMember(String memberKey) {
    mapByMember.remove(memberKey);
  }
  




  public boolean isProcessPacket(String memberKey, long packetId)
  {
    GotAllPoint memberPackets = getMemberPackets(memberKey);
    return memberPackets.processPacket(packetId);
  }
  






  public AckResendMessages getAckResendMessages(IncomingPacketsLastAck lastAck)
  {
    AckResendMessages response = new AckResendMessages();
    
    for (GotAllPoint member : mapByMember.values())
    {
      MessageAck lastAckMessage = lastAck.getLastAck(member.getMemberKey());
      
      member.addAckResendMessages(response, lastAckMessage);
    }
    
    return response;
  }
  



  private GotAllPoint getMemberPackets(String memberKey)
  {
    GotAllPoint memberGotAllPoint = (GotAllPoint)mapByMember.get(memberKey);
    if (memberGotAllPoint == null) {
      memberGotAllPoint = new GotAllPoint(memberKey, maxResendIncoming);
      mapByMember.put(memberKey, memberGotAllPoint);
    }
    return memberGotAllPoint;
  }
  








  public static class GotAllPoint
  {
    private static final Logger logger = Logger.getLogger(GotAllPoint.class.getName());
    

    private final String memberKey;
    

    private final int maxResendIncoming;
    
    private long gotAllPoint;
    
    private long gotMaxPoint;
    
    private ArrayList<Long> outOfOrderList = new ArrayList();
    
    private HashMap<Long, Integer> resendCountMap = new HashMap();
    
    public GotAllPoint(String memberKey, int maxResendIncoming) {
      this.memberKey = memberKey;
      this.maxResendIncoming = maxResendIncoming;
    }
    



    public void addAckResendMessages(AckResendMessages response, MessageAck lastAckMessage)
    {
      synchronized (this) {
        if ((lastAckMessage == null) || (lastAckMessage.getGotAllPacketId() < gotAllPoint))
        {


          response.add(new MessageAck(memberKey, gotAllPoint));
        }
        
        if (getMissingPacketCount() > 0)
        {
          List<Long> missingPackets = getMissingPackets();
          response.add(new MessageResend(memberKey, missingPackets));
        }
      }
    }
    
    public String getMemberKey() {
      return memberKey;
    }
    
    public long getGotAllPoint() {
      synchronized (this) {
        return gotAllPoint;
      }
    }
    
    public long getGotMaxPoint() {
      synchronized (this) {
        return gotMaxPoint;
      }
    }
    
    private int getMissingPacketCount() {
      if (gotMaxPoint <= gotAllPoint) {
        if (!resendCountMap.isEmpty()) {
          resendCountMap.clear();
        }
        return 0;
      }
      return (int)(gotMaxPoint - gotAllPoint) - outOfOrderList.size();
    }
    
    public List<Long> getMissingPackets()
    {
      synchronized (this) {
        ArrayList<Long> missingList = new ArrayList();
        



        boolean lostPacket = false;
        
        for (long i = gotAllPoint + 1L; i < gotMaxPoint; i += 1L) {
          Long packetId = Long.valueOf(i);
          if (!outOfOrderList.contains(packetId)) {
            if (incrementResendCount(packetId))
            {
              missingList.add(packetId);
            } else {
              lostPacket = true;
            }
          }
        }
        
        if (lostPacket) {
          checkOutOfOrderList();
        }
        
        return missingList;
      }
    }
    


    private boolean incrementResendCount(Long packetId)
    {
      Integer resendCount = (Integer)resendCountMap.get(packetId);
      if (resendCount != null) {
        int i = resendCount.intValue() + 1;
        if (i > maxResendIncoming)
        {
          logger.warning("Exceeded maxResendIncoming[" + maxResendIncoming + "] for packet[" + packetId + "]. Giving up on requesting it.");
          resendCountMap.remove(packetId);
          outOfOrderList.add(packetId);
          return false;
        }
        resendCount = Integer.valueOf(i);
        resendCountMap.put(packetId, resendCount);
      } else {
        resendCountMap.put(packetId, ONE);
      }
      return true;
    }
    
    private static final Integer ONE = Integer.valueOf(1);
    
    public boolean processPacket(long packetId) {
      synchronized (this)
      {
        if (gotAllPoint == 0L) {
          gotAllPoint = packetId;
          return true;
        }
        if (packetId <= gotAllPoint)
        {
          return false;
        }
        
        if (!resendCountMap.isEmpty()) {
          resendCountMap.remove(Long.valueOf(packetId));
        }
        
        if (packetId == gotAllPoint + 1L) {
          gotAllPoint = packetId;
        } else {
          if (packetId > gotMaxPoint) {
            gotMaxPoint = packetId;
          }
          outOfOrderList.add(Long.valueOf(packetId));
        }
        checkOutOfOrderList();
        return true;
      }
    }
    
    private void checkOutOfOrderList()
    {
      if (outOfOrderList.size() == 0) {
        return;
      }
      boolean continueCheck;
      do
      {
        continueCheck = false;
        long nextPoint = gotAllPoint + 1L;
        
        Iterator<Long> it = outOfOrderList.iterator();
        while (it.hasNext()) {
          Long id = (Long)it.next();
          if (id.longValue() == nextPoint)
          {
            it.remove();
            gotAllPoint = nextPoint;
            continueCheck = true;
            break;
          }
        }
      } while (continueCheck);
    }
  }
}
