package com.avaje.ebeaninternal.server.cluster.mcast;

import com.avaje.ebeaninternal.server.cluster.Packet;
import com.avaje.ebeaninternal.server.cluster.PacketMessages;
import java.io.DataInput;
import java.io.IOException;
import java.util.HashSet;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;



























public class McastPacketControl
{
  private static final Logger logger = Logger.getLogger(McastPacketControl.class.getName());
  
  private final String localSenderHostPort;
  
  private final McastClusterManager owner;
  
  private final HashSet<String> groupMembers = new HashSet();
  
  private final OutgoingPacketsAcked outgoingPacketsAcked = new OutgoingPacketsAcked();
  private final IncomingPacketsProcessed incomingPacketsProcessed;
  
  public McastPacketControl(McastClusterManager owner, String localSenderHostPort, int maxResendIncoming)
  {
    this.owner = owner;
    this.localSenderHostPort = localSenderHostPort;
    incomingPacketsProcessed = new IncomingPacketsProcessed(maxResendIncoming);
  }
  



  protected void onListenerTimeout()
  {
    if (groupMembers.size() == 0) {
      owner.fromListenerTimeoutNoMembers();
    }
  }
  
  protected void processMessagesPacket(String senderHostPort, Packet header, DataInput dataInput, long totalPacketsReceived, long totalBytesReceived, long totalTransEventsReceived)
    throws IOException
  {
    PacketMessages packetMessages = PacketMessages.forRead(header);
    packetMessages.read(dataInput);
    List<Message> messages = packetMessages.getMessages();
    
    if (logger.isLoggable(Level.FINER)) {
      logger.finer("INCOMING Messages " + messages);
    }
    



    MessageControl control = null;
    MessageAck ack = null;
    MessageResend resend = null;
    

    for (int i = 0; i < messages.size(); i++) {
      Message message = (Message)messages.get(i);
      if (message.isControlMessage())
      {
        control = (MessageControl)message;
      }
      else if (localSenderHostPort.equals(message.getToHostPort())) {
        if ((message instanceof MessageAck)) {
          ack = (MessageAck)message;
        } else if ((message instanceof MessageResend)) {
          resend = (MessageResend)message;
        } else {
          logger.log(Level.SEVERE, "Expecting a MessageAck or MessageResend but got a " + message.getClass().getName());
        }
      }
    }
    

    if (control != null) {
      if (control.getControlType() == 2) {
        groupMembers.remove(senderHostPort);
        logger.info("Cluster member leaving [" + senderHostPort + "] " + groupMembers.size() + " other members left");
        
        outgoingPacketsAcked.removeMember(senderHostPort);
        incomingPacketsProcessed.removeMember(senderHostPort);
      } else {
        groupMembers.add(senderHostPort);
      }
    }
    
    long newMin = 0L;
    if (ack != null) {
      newMin = outgoingPacketsAcked.receivedAck(senderHostPort, ack);
    }
    
    if ((newMin > 0L) || (control != null) || (resend != null)) {
      int groupSize = groupMembers.size();
      
      owner.fromListener(newMin, control, resend, groupSize, totalPacketsReceived, totalBytesReceived, totalTransEventsReceived);
    }
  }
  





  public boolean isProcessPacket(String memberKey, long packetId)
  {
    return incomingPacketsProcessed.isProcessPacket(memberKey, packetId);
  }
  
  public AckResendMessages getAckResendMessages(IncomingPacketsLastAck lastAck)
  {
    return incomingPacketsProcessed.getAckResendMessages(lastAck);
  }
}
