package com.avaje.ebeaninternal.server.cluster.socket;

import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.util.logging.Level;
import java.util.logging.Logger;






















class SocketClient
{
  private static final Logger logger = Logger.getLogger(SocketClient.class.getName());
  
  private final InetSocketAddress address;
  
  private final String hostPort;
  
  private boolean online;
  
  private Socket socket;
  
  private OutputStream os;
  
  private ObjectOutputStream oos;
  
  public SocketClient(InetSocketAddress address)
  {
    this.address = address;
    hostPort = (address.getHostName() + ":" + address.getPort());
  }
  
  public String getHostPort() {
    return hostPort;
  }
  
  public int getPort() {
    return address.getPort();
  }
  
  public boolean isOnline() {
    return online;
  }
  
  public void setOnline(boolean online) throws IOException {
    if (online) {
      setOnline();
    } else {
      disconnect();
    }
  }
  


  private void setOnline()
    throws IOException
  {
    connect();
    online = true;
  }
  
  public void reconnect() throws IOException {
    disconnect();
    connect();
  }
  
  private void connect() throws IOException {
    if (socket != null) {
      throw new IllegalStateException("Already got a socket connection?");
    }
    Socket s = new Socket();
    s.setKeepAlive(true);
    s.connect(address);
    
    socket = s;
    os = socket.getOutputStream();
  }
  
  public void disconnect() {
    online = false;
    if (socket != null)
    {
      try {
        socket.close();
      } catch (IOException e) {
        String msg = "Error disconnecting from Cluster member " + hostPort;
        logger.log(Level.INFO, msg, e);
      }
      
      os = null;
      oos = null;
      socket = null;
    }
  }
  
  public boolean register(SocketClusterMessage registerMsg)
  {
    try {
      setOnline();
      send(registerMsg);
      return true;
    } catch (IOException e) {
      disconnect(); }
    return false;
  }
  
  public boolean send(SocketClusterMessage msg)
    throws IOException
  {
    if (online) {
      writeObject(msg);
      return true;
    }
    
    return false;
  }
  
  private void writeObject(Object object)
    throws IOException
  {
    if (oos == null) {
      oos = new ObjectOutputStream(os);
    }
    oos.writeObject(object);
    oos.flush();
  }
}
