package com.avaje.ebeaninternal.server.cluster.socket;

import com.avaje.ebeaninternal.server.lib.thread.ThreadPool;
import com.avaje.ebeaninternal.server.lib.thread.ThreadPoolManager;
import java.io.IOException;
import java.io.InterruptedIOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketException;
import java.util.logging.Level;
import java.util.logging.Logger;































class SocketClusterListener
  implements Runnable
{
  private static final Logger logger = Logger.getLogger(SocketClusterListener.class.getName());
  



  private final int port;
  



  private final int listenTimeout = 60000;
  



  private final ServerSocket serverListenSocket;
  



  private final Thread listenerThread;
  


  private final ThreadPool threadPool;
  


  private final SocketClusterBroadcast owner;
  


  boolean doingShutdown;
  


  boolean isActive;
  



  public SocketClusterListener(SocketClusterBroadcast owner, int port)
  {
    this.owner = owner;
    threadPool = ThreadPoolManager.getThreadPool("EbeanClusterMember");
    this.port = port;
    try
    {
      serverListenSocket = new ServerSocket(port);
      serverListenSocket.setSoTimeout(60000);
      listenerThread = new Thread(this, "EbeanClusterListener");
    }
    catch (IOException e) {
      String msg = "Error starting cluster socket listener on port " + port;
      throw new RuntimeException(msg, e);
    }
  }
  


  public int getPort()
  {
    return port;
  }
  

  public void startListening()
    throws IOException
  {
    listenerThread.setDaemon(true);
    listenerThread.start();
  }
  


  public void shutdown()
  {
    doingShutdown = true;
    try {
      if (isActive) {
        synchronized (listenerThread) {
          try {
            listenerThread.wait(1000L);
          }
          catch (InterruptedException e) {}
        }
      }
      

      listenerThread.interrupt();
      serverListenSocket.close();
    } catch (IOException e) {
      logger.log(Level.SEVERE, null, e);
    }
  }
  




  public void run()
  {
    while (!doingShutdown) {
      try {
        synchronized (listenerThread) {
          Socket clientSocket = serverListenSocket.accept();
          
          isActive = true;
          
          Runnable request = new RequestProcessor(owner, clientSocket);
          threadPool.assign(request, true);
          
          isActive = false;
        }
      } catch (SocketException e) {
        if (doingShutdown) {
          String msg = "doingShutdown and accept threw:" + e.getMessage();
          logger.info(msg);
        }
        else {
          logger.log(Level.SEVERE, null, e);
        }
        
      }
      catch (InterruptedIOException e)
      {
        logger.fine("Possibly expected due to accept timeout?" + e.getMessage());
      }
      catch (IOException e)
      {
        logger.log(Level.SEVERE, null, e);
      }
    }
  }
}
