package com.avaje.ebeaninternal.server.core;

import com.avaje.ebean.annotation.LdapDomain;
import com.avaje.ebean.config.CompoundType;
import com.avaje.ebean.config.ScalarTypeConverter;
import com.avaje.ebean.config.lucene.IndexDefn;
import com.avaje.ebean.event.BeanFinder;
import com.avaje.ebean.event.BeanPersistController;
import com.avaje.ebean.event.BeanPersistListener;
import com.avaje.ebean.event.BeanQueryAdapter;
import com.avaje.ebeaninternal.server.type.ScalarType;
import com.avaje.ebeaninternal.server.util.ClassPathSearchMatcher;
import java.lang.annotation.Annotation;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.persistence.Embeddable;
import javax.persistence.Entity;
import javax.persistence.Table;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlType;

























public class BootupClasses
  implements ClassPathSearchMatcher
{
  private static final Logger logger = Logger.getLogger(BootupClasses.class.getName());
  
  private ArrayList<Class<?>> xmlBeanList = new ArrayList();
  
  private ArrayList<Class<?>> embeddableList = new ArrayList();
  
  private ArrayList<Class<?>> entityList = new ArrayList();
  
  private ArrayList<Class<?>> scalarTypeList = new ArrayList();
  
  private ArrayList<Class<?>> scalarConverterList = new ArrayList();
  
  private ArrayList<Class<?>> compoundTypeList = new ArrayList();
  
  private ArrayList<Class<?>> beanControllerList = new ArrayList();
  
  private ArrayList<Class<?>> beanFinderList = new ArrayList();
  
  private ArrayList<Class<?>> beanListenerList = new ArrayList();
  
  private ArrayList<Class<?>> beanQueryAdapterList = new ArrayList();
  
  private ArrayList<Class<?>> luceneIndexList = new ArrayList();
  
  private List<BeanPersistController> persistControllerInstances = new ArrayList();
  private List<BeanPersistListener<?>> persistListenerInstances = new ArrayList();
  private List<BeanQueryAdapter> queryAdapterInstances = new ArrayList();
  private List<IndexDefn<?>> luceneIndexInstances = new ArrayList();
  
  public BootupClasses() {}
  
  public BootupClasses(List<Class<?>> list)
  {
    if (list != null) {
      process(list.iterator());
    }
  }
  
  private BootupClasses(BootupClasses parent) {
    xmlBeanList.addAll(xmlBeanList);
    embeddableList.addAll(embeddableList);
    entityList.addAll(entityList);
    scalarTypeList.addAll(scalarTypeList);
    scalarConverterList.addAll(scalarConverterList);
    compoundTypeList.addAll(compoundTypeList);
    beanControllerList.addAll(beanControllerList);
    beanFinderList.addAll(beanFinderList);
    beanListenerList.addAll(beanListenerList);
    beanQueryAdapterList.addAll(beanQueryAdapterList);
    luceneIndexList.addAll(luceneIndexList);
  }
  
  private void process(Iterator<Class<?>> it) {
    while (it.hasNext()) {
      Class<?> cls = (Class)it.next();
      isMatch(cls);
    }
  }
  


  public BootupClasses createCopy()
  {
    return new BootupClasses(this);
  }
  
  public void addIndexDefns(List<IndexDefn<?>> indexInstances) {
    if (indexInstances != null) {
      luceneIndexInstances.addAll(indexInstances);
    }
  }
  
  public void addQueryAdapters(List<BeanQueryAdapter> queryAdapterInstances) {
    if (queryAdapterInstances != null) {
      this.queryAdapterInstances.addAll(queryAdapterInstances);
    }
  }
  


  public void addPersistControllers(List<BeanPersistController> beanControllerInstances)
  {
    if (beanControllerInstances != null) {
      persistControllerInstances.addAll(beanControllerInstances);
    }
  }
  
  public void addPersistListeners(List<BeanPersistListener<?>> listenerInstances) {
    if (listenerInstances != null) {
      persistListenerInstances.addAll(listenerInstances);
    }
  }
  

  public List<BeanQueryAdapter> getBeanQueryAdapters()
  {
    for (Class<?> cls : beanQueryAdapterList) {
      try {
        BeanQueryAdapter newInstance = (BeanQueryAdapter)cls.newInstance();
        queryAdapterInstances.add(newInstance);
      } catch (Exception e) {
        String msg = "Error creating BeanQueryAdapter " + cls;
        logger.log(Level.SEVERE, msg, e);
      }
    }
    
    return queryAdapterInstances;
  }
  

  public List<BeanPersistListener<?>> getBeanPersistListeners()
  {
    for (Class<?> cls : beanListenerList) {
      try {
        BeanPersistListener<?> newInstance = (BeanPersistListener)cls.newInstance();
        persistListenerInstances.add(newInstance);
      } catch (Exception e) {
        String msg = "Error creating BeanPersistController " + cls;
        logger.log(Level.SEVERE, msg, e);
      }
    }
    
    return persistListenerInstances;
  }
  

  public List<BeanPersistController> getBeanPersistControllers()
  {
    for (Class<?> cls : beanControllerList) {
      try {
        BeanPersistController newInstance = (BeanPersistController)cls.newInstance();
        persistControllerInstances.add(newInstance);
      } catch (Exception e) {
        String msg = "Error creating BeanPersistController " + cls;
        logger.log(Level.SEVERE, msg, e);
      }
    }
    
    return persistControllerInstances;
  }
  



  public List<IndexDefn<?>> getLuceneIndexInstances()
  {
    for (Class<?> cls : luceneIndexList) {
      try {
        IndexDefn<?> indexDefn = (IndexDefn)cls.newInstance();
        luceneIndexInstances.add(indexDefn);
      } catch (Exception e) {
        String msg = "Error creating BeanPersistController " + cls;
        logger.log(Level.SEVERE, msg, e);
      }
    }
    
    return luceneIndexInstances;
  }
  


  public ArrayList<Class<?>> getEmbeddables()
  {
    return embeddableList;
  }
  


  public ArrayList<Class<?>> getEntities()
  {
    return entityList;
  }
  


  public ArrayList<Class<?>> getScalarTypes()
  {
    return scalarTypeList;
  }
  


  public ArrayList<Class<?>> getScalarConverters()
  {
    return scalarConverterList;
  }
  


  public ArrayList<Class<?>> getCompoundTypes()
  {
    return compoundTypeList;
  }
  


  public ArrayList<Class<?>> getBeanControllers()
  {
    return beanControllerList;
  }
  


  public ArrayList<Class<?>> getBeanFinders()
  {
    return beanFinderList;
  }
  


  public ArrayList<Class<?>> getBeanListeners()
  {
    return beanListenerList;
  }
  


  public ArrayList<Class<?>> getXmlBeanList()
  {
    return xmlBeanList;
  }
  
  public void add(Iterator<Class<?>> it) {
    while (it.hasNext()) {
      Class<?> clazz = (Class)it.next();
      isMatch(clazz);
    }
  }
  
  public boolean isMatch(Class<?> cls)
  {
    if (isEmbeddable(cls)) {
      embeddableList.add(cls);
    }
    else if (isEntity(cls)) {
      entityList.add(cls);
    }
    else if (isXmlBean(cls)) {
      entityList.add(cls);
    }
    else {
      if (isInterestingInterface(cls)) {
        return true;
      }
      
      return false;
    }
    
    return true;
  }
  






  private boolean isInterestingInterface(Class<?> cls)
  {
    boolean interesting = false;
    
    if (BeanPersistController.class.isAssignableFrom(cls)) {
      beanControllerList.add(cls);
      interesting = true;
    }
    
    if (ScalarType.class.isAssignableFrom(cls)) {
      scalarTypeList.add(cls);
      interesting = true;
    }
    
    if (ScalarTypeConverter.class.isAssignableFrom(cls)) {
      scalarConverterList.add(cls);
      interesting = true;
    }
    
    if (CompoundType.class.isAssignableFrom(cls)) {
      compoundTypeList.add(cls);
      interesting = true;
    }
    
    if (BeanFinder.class.isAssignableFrom(cls)) {
      beanFinderList.add(cls);
      interesting = true;
    }
    
    if (BeanPersistListener.class.isAssignableFrom(cls)) {
      beanListenerList.add(cls);
      interesting = true;
    }
    
    if (BeanQueryAdapter.class.isAssignableFrom(cls)) {
      beanQueryAdapterList.add(cls);
      interesting = true;
    }
    
    if (IndexDefn.class.isAssignableFrom(cls)) {
      luceneIndexList.add(cls);
      interesting = true;
    }
    
    return interesting;
  }
  
  private boolean isEntity(Class<?> cls)
  {
    Annotation ann = cls.getAnnotation(Entity.class);
    if (ann != null) {
      return true;
    }
    ann = cls.getAnnotation(Table.class);
    if (ann != null) {
      return true;
    }
    ann = cls.getAnnotation(LdapDomain.class);
    if (ann != null) {
      return true;
    }
    return false;
  }
  
  private boolean isEmbeddable(Class<?> cls)
  {
    Annotation ann = cls.getAnnotation(Embeddable.class);
    if (ann != null) {
      return true;
    }
    return false;
  }
  
  private boolean isXmlBean(Class<?> cls)
  {
    Annotation ann = cls.getAnnotation(XmlRootElement.class);
    if (ann != null) {
      return true;
    }
    ann = cls.getAnnotation(XmlType.class);
    if (ann != null)
    {
      return !cls.isEnum();
    }
    return false;
  }
}
