package com.avaje.ebeaninternal.server.core;

import com.avaje.ebean.bean.BeanCollection;
import com.avaje.ebean.common.BeanList;
import com.avaje.ebean.common.BeanMap;
import com.avaje.ebean.common.BeanSet;
import com.avaje.ebeaninternal.server.deploy.BeanDescriptor;
import com.avaje.ebeaninternal.server.deploy.CopyContext;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;























public class CopyBeanCollection<T>
{
  private final BeanCollection<T> bc;
  private final BeanDescriptor<T> desc;
  private final CopyContext ctx;
  private final int maxDepth;
  
  public CopyBeanCollection(BeanCollection<T> bc, BeanDescriptor<T> desc, CopyContext ctx, int maxDepth)
  {
    this.bc = bc;
    this.desc = desc;
    this.ctx = ctx;
    this.maxDepth = maxDepth;
  }
  
  public BeanCollection<T> copy() {
    if ((bc instanceof BeanList)) {
      return copyList();
    }
    if ((bc instanceof BeanSet)) {
      return copySet();
    }
    if ((bc instanceof BeanMap)) {
      return copyMap();
    }
    
    String msg = "Invalid beanCollection type " + bc.getClass().getName();
    throw new RuntimeException(msg);
  }
  
  private BeanCollection<T> copyList()
  {
    BeanList<T> newList = new BeanList();
    List<T> actualList = ((BeanList)bc).getActualList();
    
    for (int i = 0; i < actualList.size(); i++) {
      T t = actualList.get(i);
      newList.add(desc.createCopy(t, ctx, maxDepth));
    }
    return newList;
  }
  
  private BeanCollection<T> copySet() {
    BeanSet<T> newSet = new BeanSet();
    Set<T> actualSet = ((BeanSet)bc).getActualSet();
    for (T t : actualSet) {
      newSet.add(desc.createCopy(t, ctx, maxDepth));
    }
    return newSet;
  }
  
  private BeanCollection<T> copyMap()
  {
    BeanMap<Object, T> newMap = new BeanMap();
    Map<Object, T> actualMap = ((BeanMap)bc).getActualMap();
    Iterator<Map.Entry<Object, T>> iterator = actualMap.entrySet().iterator();
    while (iterator.hasNext()) {
      Map.Entry<Object, T> entry = (Map.Entry)iterator.next();
      newMap.put(entry.getKey(), desc.createCopy(entry.getValue(), ctx, maxDepth));
    }
    return newMap;
  }
}
