package com.avaje.ebeaninternal.server.core;

import com.avaje.ebean.Query.Type;
import com.avaje.ebean.QueryIterator;
import com.avaje.ebean.QueryResultVisitor;
import com.avaje.ebean.RawSql;
import com.avaje.ebean.RawSql.ColumnMapping;
import com.avaje.ebean.bean.BeanCollection;
import com.avaje.ebean.bean.PersistenceContext;
import com.avaje.ebean.event.BeanFinder;
import com.avaje.ebean.event.BeanQueryRequest;
import com.avaje.ebeaninternal.api.BeanIdList;
import com.avaje.ebeaninternal.api.LoadContext;
import com.avaje.ebeaninternal.api.SpiEbeanServer;
import com.avaje.ebeaninternal.api.SpiQuery;
import com.avaje.ebeaninternal.api.SpiQuery.Mode;
import com.avaje.ebeaninternal.api.SpiTransaction;
import com.avaje.ebeaninternal.server.deploy.BeanDescriptor;
import com.avaje.ebeaninternal.server.deploy.BeanProperty;
import com.avaje.ebeaninternal.server.deploy.BeanPropertyAssocMany;
import com.avaje.ebeaninternal.server.deploy.CopyContext;
import com.avaje.ebeaninternal.server.deploy.DeployParser;
import com.avaje.ebeaninternal.server.deploy.DeployPropertyParserMap;
import com.avaje.ebeaninternal.server.loadcontext.DLoadContext;
import com.avaje.ebeaninternal.server.lucene.LIndex;
import com.avaje.ebeaninternal.server.query.CQueryPlan;
import com.avaje.ebeaninternal.server.query.CancelableQuery;
import java.util.List;
import java.util.Map;
import java.util.Set;
import javax.persistence.PersistenceException;















































public final class OrmQueryRequest<T>
  extends BeanRequest
  implements BeanQueryRequest<T>, SpiOrmQueryRequest<T>
{
  private final BeanDescriptor<T> beanDescriptor;
  private final OrmQueryEngine queryEngine;
  private final SpiQuery<T> query;
  private final boolean vanillaMode;
  private final BeanFinder<T> finder;
  private final LoadContext graphContext;
  private final int parentState;
  private final RawSql rawSql;
  private PersistenceContext persistenceContext;
  private Integer cacheKey;
  private int queryPlanHash;
  private boolean backgroundFetching;
  private boolean useBeanCache;
  private boolean useBeanCacheReadOnly;
  private LuceneOrmQueryRequest luceneQueryRequest;
  
  public OrmQueryRequest(SpiEbeanServer server, OrmQueryEngine queryEngine, SpiQuery<T> query, BeanDescriptor<T> desc, SpiTransaction t)
  {
    super(server, t);
    
    beanDescriptor = desc;
    rawSql = query.getRawSql();
    finder = beanDescriptor.getBeanFinder();
    this.queryEngine = queryEngine;
    this.query = query;
    vanillaMode = query.isVanillaMode(server.isVanillaMode());
    parentState = determineParentState(query);
    
    int defaultBatchSize = server.getLazyLoadBatchSize();
    graphContext = new DLoadContext(ebeanServer, beanDescriptor, defaultBatchSize, parentState, query);
    
    graphContext.registerSecondaryQueries(query);
  }
  
  private int determineParentState(SpiQuery<T> query) {
    if (query.isSharedInstance()) {
      return 3;
    }
    
    Boolean queryReadOnly = query.isReadOnly();
    if (queryReadOnly != null)
    {
      if (Boolean.TRUE.equals(queryReadOnly)) {
        return 2;
      }
      return 1;
    }
    
    if (query.getMode().equals(SpiQuery.Mode.NORMAL))
    {
      if (beanDescriptor.calculateReadOnly(query.isReadOnly())) {
        return 2;
      }
    }
    
    return 0;
  }
  
  public void executeSecondaryQueries(int defaultQueryBatch) {
    graphContext.executeSecondaryQueries(this, defaultQueryBatch);
  }
  







  public int getSecondaryQueriesMinBatchSize(int defaultQueryBatch)
  {
    return graphContext.getSecondaryQueriesMinBatchSize(this, defaultQueryBatch);
  }
  


  public int getParentState()
  {
    return parentState;
  }
  


  public BeanDescriptor<T> getBeanDescriptor()
  {
    return beanDescriptor;
  }
  


  public LIndex getLuceneIndex()
  {
    return beanDescriptor.getLuceneIndex();
  }
  


  public LoadContext getGraphContext()
  {
    return graphContext;
  }
  


  public void calculateQueryPlanHash()
  {
    queryPlanHash = query.queryPlanHash(this);
  }
  
  public boolean isRawSql() {
    return rawSql != null;
  }
  
  public DeployParser createDeployParser() {
    if (rawSql != null) {
      return new DeployPropertyParserMap(rawSql.getColumnMapping().getMapping());
    }
    return beanDescriptor.createDeployPropertyParser();
  }
  




  public boolean isSqlSelect()
  {
    return (query.isSqlSelect()) && (query.getRawSql() == null);
  }
  


  public PersistenceContext getPersistenceContext()
  {
    return persistenceContext;
  }
  










  public void initTransIfRequired()
  {
    if (query.createOwnTransaction())
    {
      transaction = ebeanServer.createQueryTransaction();
      createdTransaction = true;
    }
    else if (transaction == null)
    {
      transaction = ebeanServer.getCurrentServerTransaction();
      if (transaction == null)
      {
        transaction = ebeanServer.createQueryTransaction();
        createdTransaction = true;
      }
    }
    persistenceContext = getPersistenceContext(query, transaction);
    graphContext.setPersistenceContext(persistenceContext);
  }
  




  private PersistenceContext getPersistenceContext(SpiQuery<?> query, SpiTransaction t)
  {
    PersistenceContext ctx = query.getPersistenceContext();
    if (ctx == null) {
      ctx = t.getPersistenceContext();
    }
    return ctx;
  }
  






  public void endTransIfRequired()
  {
    if ((createdTransaction) && (!backgroundFetching))
    {
      transaction.rollback();
    }
  }
  


  public void setBackgroundFetching()
  {
    backgroundFetching = true;
  }
  


  public boolean isFindById()
  {
    return query.getType() == Query.Type.BEAN;
  }
  
  public boolean isVanillaMode() {
    return vanillaMode;
  }
  
  public LuceneOrmQueryRequest getLuceneOrmQueryRequest() {
    return luceneQueryRequest;
  }
  
  public void setLuceneOrmQueryRequest(LuceneOrmQueryRequest luceneQueryRequest) {
    this.luceneQueryRequest = luceneQueryRequest;
  }
  
  public boolean isLuceneQuery() {
    return luceneQueryRequest != null;
  }
  


  public Object findId()
  {
    return queryEngine.findId(this);
  }
  
  public int findRowCount() {
    return queryEngine.findRowCount(this);
  }
  
  public List<Object> findIds() {
    BeanIdList idList = queryEngine.findIds(this);
    return idList.getIdList();
  }
  
  public void findVisit(QueryResultVisitor<T> visitor) {
    QueryIterator<T> it = queryEngine.findIterate(this);
    try {
      while (it.hasNext()) {
        if (!visitor.accept(it.next())) {
          break;
        }
      }
    } finally {
      it.close();
    }
  }
  
  public QueryIterator<T> findIterate() {
    return queryEngine.findIterate(this);
  }
  



  public List<T> findList()
  {
    BeanCollection<T> bc = queryEngine.findMany(this);
    return (List)(vanillaMode ? bc.getActualCollection() : bc);
  }
  



  public Set<?> findSet()
  {
    BeanCollection<T> bc = queryEngine.findMany(this);
    return (Set)(vanillaMode ? bc.getActualCollection() : bc);
  }
  


  public Map<?, ?> findMap()
  {
    String mapKey = query.getMapKey();
    if (mapKey == null) {
      BeanProperty[] ids = beanDescriptor.propertiesId();
      if (ids.length == 1) {
        query.setMapKey(ids[0].getName());
      } else {
        String msg = "No mapKey specified for query";
        throw new PersistenceException(msg);
      }
    }
    BeanCollection<T> bc = queryEngine.findMany(this);
    return (Map)(vanillaMode ? bc.getActualCollection() : bc);
  }
  
  public Query.Type getQueryType() {
    return query.getType();
  }
  


  public BeanFinder<T> getBeanFinder()
  {
    return finder;
  }
  


  public SpiQuery<T> getQuery()
  {
    return query;
  }
  



  public BeanPropertyAssocMany<?> getManyProperty()
  {
    return beanDescriptor.getManyProperty(query);
  }
  



  public CQueryPlan getQueryPlan()
  {
    return beanDescriptor.getQueryPlan(Integer.valueOf(queryPlanHash));
  }
  







  public int getQueryPlanHash()
  {
    return queryPlanHash;
  }
  


  public void putQueryPlan(CQueryPlan queryPlan)
  {
    beanDescriptor.putQueryPlan(Integer.valueOf(queryPlanHash), queryPlan);
  }
  
  public boolean isUseBeanCache() {
    return useBeanCache;
  }
  
  public boolean isUseBeanCacheReadOnly() {
    return useBeanCacheReadOnly;
  }
  
  private boolean calculateUseBeanCache() {
    useBeanCache = beanDescriptor.calculateUseCache(query.isUseBeanCache());
    if (useBeanCache) {
      useBeanCacheReadOnly = beanDescriptor.calculateReadOnly(query.isReadOnly());
    }
    return useBeanCache;
  }
  




  public T getFromPersistenceContextOrCache()
  {
    if (query.isLoadBeanCache())
    {

      return null;
    }
    
    SpiTransaction t = transaction;
    if (t == null) {
      t = ebeanServer.getCurrentServerTransaction();
    }
    if (t != null)
    {
      PersistenceContext context = t.getPersistenceContext();
      if (context != null) {
        Object o = context.get(beanDescriptor.getBeanType(), query.getId());
        if (o != null) {
          return o;
        }
      }
    }
    
    if (!calculateUseBeanCache())
    {
      return null;
    }
    
    Object cachedBean = beanDescriptor.cacheGet(query.getId());
    if (cachedBean == null)
    {
      return null;
    }
    if (useBeanCacheReadOnly)
    {

      return cachedBean;
    }
    

    return beanDescriptor.createCopyForUpdate(cachedBean, vanillaMode);
  }
  




  public BeanCollection<T> getFromQueryCache()
  {
    if (!query.isUseQueryCache()) {
      return null;
    }
    
    if (query.getType() == null)
    {
      cacheKey = Integer.valueOf(query.queryHash());
    }
    else
    {
      cacheKey = Integer.valueOf(31 * query.queryHash() + query.getType().hashCode());
    }
    
    BeanCollection<T> bc = beanDescriptor.queryCacheGet(cacheKey);
    if ((bc != null) && (Boolean.FALSE.equals(query.isReadOnly())))
    {
      CopyContext ctx = new CopyContext(vanillaMode, false);
      return new CopyBeanCollection(bc, beanDescriptor, ctx, 5).copy();
    }
    return bc;
  }
  
  public void putToQueryCache(BeanCollection<T> queryResult) {
    beanDescriptor.queryCachePut(cacheKey, queryResult);
  }
  



  public void setCancelableQuery(CancelableQuery cancelableQuery)
  {
    query.setCancelableQuery(cancelableQuery);
  }
  


  public void logSql(String sql)
  {
    if (transaction.isLogSql()) {
      transaction.logInternal(sql);
    }
  }
}
