package com.avaje.ebeaninternal.server.core;

import com.avaje.ebean.InvalidValue;
import com.avaje.ebean.ValidationException;
import com.avaje.ebean.bean.EntityBean;
import com.avaje.ebean.bean.EntityBeanIntercept;
import com.avaje.ebean.event.BeanPersistController;
import com.avaje.ebean.event.BeanPersistListener;
import com.avaje.ebean.event.BeanPersistRequest;
import com.avaje.ebeaninternal.api.SpiEbeanServer;
import com.avaje.ebeaninternal.api.SpiTransaction;
import com.avaje.ebeaninternal.api.TransactionEvent;
import com.avaje.ebeaninternal.server.deploy.BeanDescriptor;
import com.avaje.ebeaninternal.server.deploy.BeanManager;
import com.avaje.ebeaninternal.server.deploy.BeanProperty;
import com.avaje.ebeaninternal.server.persist.BatchControl;
import com.avaje.ebeaninternal.server.persist.PersistExecute;
import com.avaje.ebeaninternal.server.persist.dml.GenerateDmlRequest;
import com.avaje.ebeaninternal.server.transaction.BeanDelta;
import com.avaje.ebeaninternal.server.transaction.BeanPersistIdMap;
import java.sql.SQLException;
import java.util.Set;
import javax.persistence.OptimisticLockException;






































































public class PersistRequestBean<T>
  extends PersistRequest
  implements BeanPersistRequest<T>
{
  protected final BeanManager<T> beanManager;
  protected final BeanDescriptor<T> beanDescriptor;
  protected final BeanPersistListener<T> beanPersistListener;
  protected final BeanPersistController controller;
  protected final EntityBeanIntercept intercept;
  protected final Object parentBean;
  protected final boolean isDirty;
  protected final boolean vanilla;
  protected final T bean;
  protected T oldValues;
  protected ConcurrencyMode concurrencyMode;
  protected final Set<String> loadedProps;
  protected Object idValue;
  protected Integer beanHash;
  protected Integer beanIdentityHash;
  protected final Set<String> changedProps;
  protected boolean notifyCache;
  private boolean statelessUpdate;
  private boolean deleteMissingChildren;
  private boolean updateNullProperties;
  
  public PersistRequestBean(SpiEbeanServer server, T bean, Object parentBean, BeanManager<T> mgr, SpiTransaction t, PersistExecute persistExecute, Set<String> updateProps, ConcurrencyMode concurrencyMode)
  {
    super(server, t, persistExecute);
    beanManager = mgr;
    beanDescriptor = mgr.getBeanDescriptor();
    beanPersistListener = beanDescriptor.getPersistListener();
    this.bean = bean;
    this.parentBean = parentBean;
    
    controller = beanDescriptor.getPersistController();
    this.concurrencyMode = beanDescriptor.getConcurrencyMode();
    
    this.concurrencyMode = concurrencyMode;
    loadedProps = updateProps;
    changedProps = updateProps;
    
    vanilla = true;
    isDirty = true;
    oldValues = bean;
    if ((bean instanceof EntityBean)) {
      intercept = ((EntityBean)bean)._ebean_getIntercept();
    } else {
      intercept = null;
    }
  }
  


  public PersistRequestBean(SpiEbeanServer server, T bean, Object parentBean, BeanManager<T> mgr, SpiTransaction t, PersistExecute persistExecute)
  {
    super(server, t, persistExecute);
    beanManager = mgr;
    beanDescriptor = mgr.getBeanDescriptor();
    beanPersistListener = beanDescriptor.getPersistListener();
    this.bean = bean;
    this.parentBean = parentBean;
    
    controller = beanDescriptor.getPersistController();
    concurrencyMode = beanDescriptor.getConcurrencyMode();
    
    if ((bean instanceof EntityBean)) {
      intercept = ((EntityBean)bean)._ebean_getIntercept();
      if (intercept.isReference())
      {

        concurrencyMode = ConcurrencyMode.NONE;
      }
      
      isDirty = intercept.isDirty();
      if (!isDirty) {
        changedProps = intercept.getChangedProps();
      }
      else {
        Set<String> beanChangedProps = intercept.getChangedProps();
        Set<String> dirtyEmbedded = beanDescriptor.getDirtyEmbeddedProperties(bean);
        changedProps = mergeChangedProperties(beanChangedProps, dirtyEmbedded);
      }
      loadedProps = intercept.getLoadedProps();
      oldValues = intercept.getOldValues();
      vanilla = false;
    }
    else
    {
      vanilla = true;
      isDirty = true;
      loadedProps = null;
      changedProps = null;
      intercept = null;
      

      if (concurrencyMode.equals(ConcurrencyMode.ALL)) {
        concurrencyMode = ConcurrencyMode.NONE;
      }
    }
  }
  


  private Set<String> mergeChangedProperties(Set<String> beanChangedProps, Set<String> embChanged)
  {
    if (embChanged == null)
      return beanChangedProps;
    if (beanChangedProps == null) {
      return embChanged;
    }
    beanChangedProps.addAll(embChanged);
    return beanChangedProps;
  }
  
  public boolean isNotify(TransactionEvent txnEvent)
  {
    return (notifyCache) || (isNotifyPersistListener()) || (beanDescriptor.isNotifyLucene(txnEvent));
  }
  
  public boolean isNotifyCache() {
    return notifyCache;
  }
  
  public boolean isNotifyPersistListener() {
    return beanPersistListener != null;
  }
  
  public void notifyCache() {
    if (notifyCache) {
      if (type == PersistRequest.Type.INSERT) {
        beanDescriptor.queryCacheClear();
      }
      else {
        beanDescriptor.cacheRemove(idValue);
      }
    }
  }
  
  public void pauseIndexInvalidate() {
    transaction.getEvent().pauseIndexInvalidate(beanDescriptor.getBeanType());
  }
  
  public void resumeIndexInvalidate() {
    transaction.getEvent().resumeIndexInvalidate(beanDescriptor.getBeanType());
  }
  
  public void addToPersistMap(BeanPersistIdMap beanPersistMap)
  {
    beanPersistMap.add(beanDescriptor, type, idValue);
  }
  
  public boolean notifyLocalPersistListener() {
    if (beanPersistListener == null) {
      return false;
    }
    
    switch (1.$SwitchMap$com$avaje$ebeaninternal$server$core$PersistRequest$Type[type.ordinal()]) {
    case 1: 
      return beanPersistListener.inserted(bean);
    
    case 2: 
      return beanPersistListener.updated(bean, getUpdatedProperties());
    
    case 3: 
      return beanPersistListener.deleted(bean);
    }
    
    return false;
  }
  

  public boolean isParent(Object o)
  {
    return o == parentBean;
  }
  





  private Integer getBeanHash()
  {
    if (beanHash == null) {
      Object id = beanDescriptor.getId(bean);
      int hc = 31 * bean.getClass().getName().hashCode();
      if (id != null) {
        hc += id.hashCode();
      }
      beanHash = Integer.valueOf(hc);
    }
    return beanHash;
  }
  
  private Integer getBeanIdentityHash() {
    if (beanIdentityHash == null) {
      beanIdentityHash = Integer.valueOf(System.identityHashCode(bean));
    }
    return beanIdentityHash;
  }
  






  public void registerVanillaBean()
  {
    if (intercept == null) {
      transaction.registerBean(getBeanIdentityHash());
    }
  }
  



  public boolean isRegisteredVanillaBean()
  {
    if ((intercept == null) && (transaction != null)) {
      return transaction.isRegisteredBean(getBeanIdentityHash());
    }
    return false;
  }
  




  public void registerBean()
  {
    transaction.registerBean(getBeanHash());
  }
  


  public void unregisterBean()
  {
    transaction.unregisterBean(getBeanHash());
  }
  


  public boolean isRegistered()
  {
    if (transaction == null) {
      return false;
    }
    return transaction.isRegisteredBean(getBeanHash());
  }
  




  public void setType(PersistRequest.Type type)
  {
    this.type = type;
    if ((type == PersistRequest.Type.DELETE) || (type == PersistRequest.Type.UPDATE)) {
      if (oldValues == null) {
        oldValues = bean;
      }
      notifyCache = beanDescriptor.isCacheNotify(false);
    } else {
      notifyCache = beanDescriptor.isCacheNotify(true);
    }
  }
  
  public BeanManager<T> getBeanManager() {
    return beanManager;
  }
  


  public BeanDescriptor<T> getBeanDescriptor()
  {
    return beanDescriptor;
  }
  


  public boolean isStatelessUpdate()
  {
    return statelessUpdate;
  }
  



  public boolean isDeleteMissingChildren()
  {
    return deleteMissingChildren;
  }
  



  public boolean isUpdateNullProperties()
  {
    return updateNullProperties;
  }
  







  public void setStatelessUpdate(boolean statelessUpdate, boolean deleteMissingChildren, boolean updateNullProperties)
  {
    this.statelessUpdate = statelessUpdate;
    this.deleteMissingChildren = deleteMissingChildren;
    this.updateNullProperties = updateNullProperties;
  }
  



  public boolean isDirty()
  {
    return isDirty;
  }
  


  public ConcurrencyMode getConcurrencyMode()
  {
    return concurrencyMode;
  }
  



  public void setLoadedProps(Set<String> additionalProps)
  {
    if (intercept != null) {
      intercept.setLoadedProps(additionalProps);
    }
  }
  
  public Set<String> getLoadedProperties() {
    return loadedProps;
  }
  







  public String getFullName()
  {
    return beanDescriptor.getFullName();
  }
  


  public T getBean()
  {
    return bean;
  }
  


  public Object getBeanId()
  {
    return beanDescriptor.getId(bean);
  }
  
  public BeanDelta createDeltaBean() {
    return new BeanDelta(beanDescriptor, getBeanId());
  }
  



  public T getOldValues()
  {
    return oldValues;
  }
  



  public Object getParentBean()
  {
    return parentBean;
  }
  



  public BeanPersistController getBeanController()
  {
    return controller;
  }
  


  public EntityBeanIntercept getEntityBeanIntercept()
  {
    return intercept;
  }
  



  public void validate()
  {
    InvalidValue errs = beanDescriptor.validate(false, bean);
    if (errs != null) {
      throw new ValidationException(errs);
    }
  }
  



  public boolean isLoadedProperty(BeanProperty prop)
  {
    if (loadedProps == null) {
      return true;
    }
    return loadedProps.contains(prop.getName());
  }
  

  public int executeNow()
  {
    switch (1.$SwitchMap$com$avaje$ebeaninternal$server$core$PersistRequest$Type[type.ordinal()]) {
    case 1: 
      persistExecute.executeInsertBean(this);
      return -1;
    
    case 2: 
      persistExecute.executeUpdateBean(this);
      return -1;
    
    case 3: 
      persistExecute.executeDeleteBean(this);
      return -1;
    }
    
    throw new RuntimeException("Invalid type " + type);
  }
  


  public int executeOrQueue()
  {
    boolean batch = transaction.isBatchThisRequest();
    
    BatchControl control = transaction.getBatchControl();
    if (control != null) {
      return control.executeOrQueue(this, batch);
    }
    if (batch) {
      control = persistExecute.createBatchControl(transaction);
      return control.executeOrQueue(this, batch);
    }
    
    return executeNow();
  }
  




  public void setGeneratedKey(Object idValue)
  {
    if (idValue != null)
    {


      idValue = beanDescriptor.convertSetId(idValue, bean);
      

      this.idValue = idValue;
    }
  }
  



  public void setBoundId(Object idValue)
  {
    this.idValue = idValue;
  }
  

  public final void checkRowCount(int rowCount)
    throws SQLException
  {
    if (rowCount != 1) {
      String m = Message.msg("persist.conc2", "" + rowCount);
      throw new OptimisticLockException(m, null, bean);
    }
  }
  


  public void postExecute()
    throws SQLException
  {
    if (controller != null) {
      controllerPost();
    }
    
    if (intercept != null)
    {
      intercept.setLoaded();
    }
    
    addEvent();
    
    if (isLogSummary()) {
      logSummary();
    }
  }
  
  private void controllerPost() {
    switch (1.$SwitchMap$com$avaje$ebeaninternal$server$core$PersistRequest$Type[type.ordinal()]) {
    case 1: 
      controller.postInsert(this);
      break;
    case 2: 
      controller.postUpdate(this);
      break;
    case 3: 
      controller.postDelete(this);
      break;
    }
    
  }
  

  private void logSummary()
  {
    String name = beanDescriptor.getName();
    switch (1.$SwitchMap$com$avaje$ebeaninternal$server$core$PersistRequest$Type[type.ordinal()]) {
    case 1: 
      transaction.logInternal("Inserted [" + name + "] [" + idValue + "]");
      break;
    case 2: 
      transaction.logInternal("Updated [" + name + "] [" + idValue + "]");
      break;
    case 3: 
      transaction.logInternal("Deleted [" + name + "] [" + idValue + "]");
      break;
    }
    
  }
  





  private void addEvent()
  {
    TransactionEvent event = transaction.getEvent();
    if (event != null) {
      event.add(this);
    }
  }
  







  public ConcurrencyMode determineConcurrencyMode()
  {
    if (loadedProps != null)
    {
      if (concurrencyMode.equals(ConcurrencyMode.VERSION))
      {
        BeanProperty prop = beanDescriptor.firstVersionProperty();
        if ((prop == null) || (!loadedProps.contains(prop.getName())))
        {

          concurrencyMode = ConcurrencyMode.ALL;
        }
      }
    }
    return concurrencyMode;
  }
  





  public boolean isDynamicUpdateSql()
  {
    return ((!vanilla) && (beanDescriptor.isUpdateChangesOnly())) || (loadedProps != null);
  }
  






  public GenerateDmlRequest createGenerateDmlRequest(boolean emptyStringAsNull)
  {
    if (beanDescriptor.isUpdateChangesOnly()) {
      return new GenerateDmlRequest(emptyStringAsNull, changedProps, loadedProps, oldValues);
    }
    return new GenerateDmlRequest(emptyStringAsNull, loadedProps, loadedProps, oldValues);
  }
  




  public Set<String> getUpdatedProperties()
  {
    if (changedProps != null) {
      return changedProps;
    }
    return loadedProps;
  }
  




  public boolean hasChanged(BeanProperty prop)
  {
    return changedProps.contains(prop.getName());
  }
}
