package com.avaje.ebeaninternal.server.core;

import com.avaje.ebean.EbeanServer;
import com.avaje.ebean.Query.Type;
import com.avaje.ebean.SqlQuery;
import com.avaje.ebean.SqlRow;
import com.avaje.ebean.Transaction;
import com.avaje.ebeaninternal.api.SpiEbeanServer;
import com.avaje.ebeaninternal.api.SpiSqlQuery;
import com.avaje.ebeaninternal.api.SpiTransaction;
import java.util.List;
import java.util.Map;
import java.util.Set;































public final class RelationalQueryRequest
{
  private final SpiSqlQuery query;
  private final RelationalQueryEngine queryEngine;
  private final SpiEbeanServer ebeanServer;
  private SpiTransaction trans;
  private boolean createdTransaction;
  private Query.Type queryType;
  
  public RelationalQueryRequest(SpiEbeanServer server, RelationalQueryEngine engine, SqlQuery q, Transaction t)
  {
    ebeanServer = server;
    queryEngine = engine;
    query = ((SpiSqlQuery)q);
    trans = ((SpiTransaction)t);
  }
  


  public void rollbackTransIfRequired()
  {
    if (createdTransaction) {
      trans.rollback();
    }
  }
  


  public void initTransIfRequired()
  {
    if (trans == null) {
      trans = ebeanServer.getCurrentServerTransaction();
      if ((trans == null) || (!trans.isActive()))
      {
        trans = ebeanServer.createServerTransaction(false, -1);
        



        createdTransaction = true;
      }
    }
  }
  


  public void endTransIfRequired()
  {
    if (createdTransaction)
    {
      trans.rollback();
    }
  }
  
  public List<SqlRow> findList()
  {
    queryType = Query.Type.LIST;
    return (List)queryEngine.findMany(this);
  }
  
  public Set<SqlRow> findSet()
  {
    queryType = Query.Type.SET;
    return (Set)queryEngine.findMany(this);
  }
  
  public Map<?, SqlRow> findMap()
  {
    queryType = Query.Type.MAP;
    return (Map)queryEngine.findMany(this);
  }
  


  public SpiSqlQuery getQuery()
  {
    return query;
  }
  


  public Query.Type getQueryType()
  {
    return queryType;
  }
  
  public EbeanServer getEbeanServer() {
    return ebeanServer;
  }
  
  public SpiTransaction getTransaction() {
    return trans;
  }
  
  public boolean isLogSql() {
    return trans.isLogSql();
  }
  
  public boolean isLogSummary() {
    return trans.isLogSummary();
  }
}
