package com.avaje.ebeaninternal.server.deploy;

import com.avaje.ebean.InvalidValue;
import com.avaje.ebean.Query;
import com.avaje.ebean.Query.UseIndex;
import com.avaje.ebean.SqlUpdate;
import com.avaje.ebean.Transaction;
import com.avaje.ebean.bean.BeanCollection;
import com.avaje.ebean.bean.EntityBean;
import com.avaje.ebean.bean.EntityBeanIntercept;
import com.avaje.ebean.cache.ServerCache;
import com.avaje.ebean.cache.ServerCacheManager;
import com.avaje.ebean.config.EncryptKey;
import com.avaje.ebean.config.dbplatform.IdGenerator;
import com.avaje.ebean.config.dbplatform.IdType;
import com.avaje.ebean.config.lucene.IndexDefn;
import com.avaje.ebean.event.BeanFinder;
import com.avaje.ebean.event.BeanPersistController;
import com.avaje.ebean.event.BeanPersistListener;
import com.avaje.ebean.event.BeanQueryAdapter;
import com.avaje.ebean.text.TextException;
import com.avaje.ebean.text.json.JsonWriteBeanVisitor;
import com.avaje.ebean.validation.factory.Validator;
import com.avaje.ebeaninternal.api.SpiEbeanServer;
import com.avaje.ebeaninternal.api.SpiQuery;
import com.avaje.ebeaninternal.api.SpiUpdatePlan;
import com.avaje.ebeaninternal.api.TransactionEvent;
import com.avaje.ebeaninternal.api.TransactionEventTable.TableIUD;
import com.avaje.ebeaninternal.server.core.ConcurrencyMode;
import com.avaje.ebeaninternal.server.core.DefaultSqlUpdate;
import com.avaje.ebeaninternal.server.core.InternString;
import com.avaje.ebeaninternal.server.core.ReferenceOptions;
import com.avaje.ebeaninternal.server.deploy.id.IdBinder;
import com.avaje.ebeaninternal.server.deploy.meta.DeployBeanDescriptor;
import com.avaje.ebeaninternal.server.deploy.meta.DeployBeanPropertyLists;
import com.avaje.ebeaninternal.server.el.ElComparator;
import com.avaje.ebeaninternal.server.el.ElComparatorCompound;
import com.avaje.ebeaninternal.server.el.ElComparatorProperty;
import com.avaje.ebeaninternal.server.el.ElPropertyChainBuilder;
import com.avaje.ebeaninternal.server.el.ElPropertyDeploy;
import com.avaje.ebeaninternal.server.el.ElPropertyValue;
import com.avaje.ebeaninternal.server.ldap.LdapPersistenceException;
import com.avaje.ebeaninternal.server.lucene.LIndex;
import com.avaje.ebeaninternal.server.persist.DmlUtil;
import com.avaje.ebeaninternal.server.query.CQueryPlan;
import com.avaje.ebeaninternal.server.query.SplitName;
import com.avaje.ebeaninternal.server.querydefn.OrmQueryDetail;
import com.avaje.ebeaninternal.server.reflect.BeanReflect;
import com.avaje.ebeaninternal.server.text.json.ReadJsonContext;
import com.avaje.ebeaninternal.server.text.json.ReadJsonContext.ReadBeanState;
import com.avaje.ebeaninternal.server.text.json.WriteJsonContext;
import com.avaje.ebeaninternal.server.text.json.WriteJsonContext.WriteBeanState;
import com.avaje.ebeaninternal.server.transaction.IndexInvalidate;
import com.avaje.ebeaninternal.server.type.DataBind;
import com.avaje.ebeaninternal.server.type.TypeManager;
import com.avaje.ebeaninternal.util.SortByClause;
import com.avaje.ebeaninternal.util.SortByClause.Property;
import com.avaje.ebeaninternal.util.SortByClauseParser;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.naming.InvalidNameException;
import javax.naming.directory.Attributes;
import javax.naming.directory.BasicAttribute;
import javax.naming.directory.BasicAttributes;
import javax.naming.ldap.LdapName;
import javax.persistence.PersistenceException;
























public class BeanDescriptor<T>
{
  private static final Logger logger = Logger.getLogger(BeanDescriptor.class.getName());
  
  private final ConcurrentHashMap<Integer, SpiUpdatePlan> updatePlanCache = new ConcurrentHashMap();
  
  private final ConcurrentHashMap<Integer, CQueryPlan> queryPlanCache = new ConcurrentHashMap();
  
  private final ConcurrentHashMap<String, ElPropertyValue> elGetCache = new ConcurrentHashMap();
  
  private final ConcurrentHashMap<String, ElComparator<T>> comparatorCache = new ConcurrentHashMap();
  
  private final ConcurrentHashMap<String, BeanFkeyProperty> fkeyMap = new ConcurrentHashMap();
  private final String serverName;
  
  public static enum EntityType { ORM, 
    EMBEDDED, 
    SQL, 
    META, 
    LDAP, 
    XMLELEMENT;
    



    private EntityType() {}
  }
  



  private final EntityType entityType;
  


  private final IdType idType;
  


  private final IdGenerator idGenerator;
  


  private final String sequenceName;
  


  private final String ldapBaseDn;
  


  private final String[] ldapObjectclasses;
  


  private final String selectLastInsertedId;
  


  private final boolean autoFetchTunable;
  


  private final String lazyFetchIncludes;
  


  private final ConcurrencyMode concurrencyMode;
  


  private final String[] dependantTables;
  


  private final CompoundUniqueContraint[] compoundUniqueConstraints;
  


  private final Map<String, String> extraAttrMap;
  


  private final String baseTable;
  


  private final BeanReflect beanReflect;
  


  private final LinkedHashMap<String, BeanProperty> propMap;
  


  private final LinkedHashMap<String, BeanProperty> propMapByDbColumn;
  


  private final Class<T> beanType;
  


  private final BeanDescriptorMap owner;
  


  private final Class<?> factoryType;
  


  private final boolean enhancedBean;
  


  private volatile BeanPersistController persistController;
  


  private volatile BeanPersistListener<T> persistListener;
  


  private volatile BeanQueryAdapter queryAdapter;
  


  private final BeanFinder<T> beanFinder;
  


  private final IndexDefn<?> luceneIndexDefn;
  


  private final TableJoin[] derivedTableJoins;
  


  private final InheritInfo inheritInfo;
  


  private final BeanProperty[] propertiesId;
  


  private final BeanProperty[] propertiesVersion;
  


  private final BeanProperty[] propertiesLocal;
  

  private final BeanPropertyAssocOne<?> unidirectional;
  

  private final int namesOfManyPropsHash;
  

  private final Set<String> namesOfManyProps;
  

  private final BeanPropertyAssocMany<?>[] propertiesMany;
  

  private final BeanPropertyAssocMany<?>[] propertiesManySave;
  

  private final BeanPropertyAssocMany<?>[] propertiesManyDelete;
  

  private final BeanPropertyAssocMany<?>[] propertiesManyToMany;
  

  private final BeanPropertyAssocOne<?>[] propertiesOne;
  

  private final BeanPropertyAssocOne<?>[] propertiesOneImported;
  

  private final BeanPropertyAssocOne<?>[] propertiesOneImportedSave;
  

  private final BeanPropertyAssocOne<?>[] propertiesOneImportedDelete;
  

  private final BeanPropertyAssocOne<?>[] propertiesOneExported;
  

  private final BeanPropertyAssocOne<?>[] propertiesOneExportedSave;
  

  private final BeanPropertyAssocOne<?>[] propertiesOneExportedDelete;
  

  private final BeanPropertyAssocOne<?>[] propertiesEmbedded;
  

  private final BeanProperty[] propertiesBaseScalar;
  

  private final BeanPropertyCompound[] propertiesBaseCompound;
  

  private final BeanProperty[] propertiesTransient;
  

  final BeanProperty[] propertiesNonTransient;
  

  private final BeanProperty propertyFirstVersion;
  

  private final BeanProperty propertySingleId;
  

  private final String fullName;
  

  private final Map<String, DeployNamedQuery> namedQueries;
  

  private final Map<String, DeployNamedUpdate> namedUpdates;
  

  private final boolean hasLocalValidation;
  

  private final boolean hasCascadeValidation;
  

  private final BeanProperty[] propertiesValidationLocal;
  

  private final BeanProperty[] propertiesValidationCascade;
  

  private final Validator[] beanValidators;
  

  private boolean saveRecurseSkippable;
  

  private boolean deleteRecurseSkippable;
  

  private final TypeManager typeManager;
  

  private final IdBinder idBinder;
  

  private String idBinderInLHSSql;
  

  private String idBinderIdSql;
  

  private String deleteByIdSql;
  

  private String deleteByIdInSql;
  

  private final String name;
  

  private final String baseTableAlias;
  

  private final boolean updateChangesOnly;
  

  private final ServerCacheManager cacheManager;
  

  private final ReferenceOptions referenceOptions;
  

  private final String defaultSelectClause;
  

  private final Set<String> defaultSelectClauseSet;
  

  private final String[] defaultSelectDbArray;
  

  private final String descriptorId;
  

  private final Query.UseIndex useIndex;
  

  private SpiEbeanServer ebeanServer;
  

  private ServerCache beanCache;
  

  private ServerCache queryCache;
  

  private LIndex luceneIndex;
  

  private Set<IndexInvalidate> luceneIndexInvalidations;
  

  public BeanDescriptor(BeanDescriptorMap owner, TypeManager typeManager, DeployBeanDescriptor<T> deploy, String descriptorId)
  {
    this.owner = owner;
    cacheManager = owner.getCacheManager();
    serverName = owner.getServerName();
    luceneIndexDefn = deploy.getIndexDefn();
    entityType = deploy.getEntityType();
    name = InternString.intern(deploy.getName());
    baseTableAlias = "t0";
    fullName = InternString.intern(deploy.getFullName());
    this.descriptorId = descriptorId;
    
    useIndex = deploy.getUseIndex();
    this.typeManager = typeManager;
    beanType = deploy.getBeanType();
    factoryType = deploy.getFactoryType();
    enhancedBean = beanType.equals(factoryType);
    namedQueries = deploy.getNamedQueries();
    namedUpdates = deploy.getNamedUpdates();
    
    inheritInfo = deploy.getInheritInfo();
    
    beanFinder = deploy.getBeanFinder();
    persistController = deploy.getPersistController();
    persistListener = deploy.getPersistListener();
    queryAdapter = deploy.getQueryAdapter();
    referenceOptions = deploy.getReferenceOptions();
    
    defaultSelectClause = deploy.getDefaultSelectClause();
    defaultSelectClauseSet = deploy.parseDefaultSelectClause(defaultSelectClause);
    defaultSelectDbArray = deploy.getDefaultSelectDbArray(defaultSelectClauseSet);
    
    idType = deploy.getIdType();
    idGenerator = deploy.getIdGenerator();
    ldapBaseDn = deploy.getLdapBaseDn();
    ldapObjectclasses = deploy.getLdapObjectclasses();
    sequenceName = deploy.getSequenceName();
    selectLastInsertedId = deploy.getSelectLastInsertedId();
    lazyFetchIncludes = InternString.intern(deploy.getLazyFetchIncludes());
    concurrencyMode = deploy.getConcurrencyMode();
    updateChangesOnly = deploy.isUpdateChangesOnly();
    
    dependantTables = deploy.getDependantTables();
    compoundUniqueConstraints = deploy.getCompoundUniqueConstraints();
    
    extraAttrMap = deploy.getExtraAttributeMap();
    
    baseTable = InternString.intern(deploy.getBaseTable());
    
    beanReflect = deploy.getBeanReflect();
    
    autoFetchTunable = ((EntityType.ORM.equals(entityType)) && (beanFinder == null));
    

    DeployBeanPropertyLists listHelper = new DeployBeanPropertyLists(owner, this, deploy);
    
    propMap = listHelper.getPropertyMap();
    propMapByDbColumn = getReverseMap(propMap);
    propertiesTransient = listHelper.getTransients();
    propertiesNonTransient = listHelper.getNonTransients();
    propertiesBaseScalar = listHelper.getBaseScalar();
    propertiesBaseCompound = listHelper.getBaseCompound();
    propertiesId = listHelper.getId();
    propertiesVersion = listHelper.getVersion();
    propertiesEmbedded = listHelper.getEmbedded();
    propertiesLocal = listHelper.getLocal();
    unidirectional = listHelper.getUnidirectional();
    propertiesOne = listHelper.getOnes();
    propertiesOneExported = listHelper.getOneExported();
    propertiesOneExportedSave = listHelper.getOneExportedSave();
    propertiesOneExportedDelete = listHelper.getOneExportedDelete();
    propertiesOneImported = listHelper.getOneImported();
    propertiesOneImportedSave = listHelper.getOneImportedSave();
    propertiesOneImportedDelete = listHelper.getOneImportedDelete();
    
    propertiesMany = listHelper.getMany();
    propertiesManySave = listHelper.getManySave();
    propertiesManyDelete = listHelper.getManyDelete();
    propertiesManyToMany = listHelper.getManyToMany();
    
    namesOfManyProps = deriveManyPropNames();
    namesOfManyPropsHash = namesOfManyProps.hashCode();
    
    derivedTableJoins = listHelper.getTableJoin();
    propertyFirstVersion = listHelper.getFirstVersion();
    
    if (propertiesId.length == 1) {
      propertySingleId = propertiesId[0];
    } else {
      propertySingleId = null;
    }
    




    saveRecurseSkippable = (0 == propertiesOneExportedSave.length + propertiesOneImportedSave.length + propertiesManySave.length);
    

    deleteRecurseSkippable = (0 == propertiesOneExportedDelete.length + propertiesOneImportedDelete.length + propertiesManyDelete.length);
    
    propertiesValidationLocal = listHelper.getPropertiesWithValidators(false);
    propertiesValidationCascade = listHelper.getPropertiesWithValidators(true);
    beanValidators = listHelper.getBeanValidators();
    hasLocalValidation = ((propertiesValidationLocal.length > 0) || (beanValidators.length > 0));
    hasCascadeValidation = ((propertiesValidationCascade.length > 0) || (beanValidators.length > 0));
    

    idBinder = owner.createIdBinder(propertiesId);
  }
  
  private LinkedHashMap<String, BeanProperty> getReverseMap(LinkedHashMap<String, BeanProperty> propMap)
  {
    LinkedHashMap<String, BeanProperty> revMap = new LinkedHashMap(propMap.size() * 2);
    
    for (BeanProperty prop : propMap.values()) {
      if (prop.getDbColumn() != null) {
        revMap.put(prop.getDbColumn(), prop);
      }
    }
    
    return revMap;
  }
  


  public void setEbeanServer(SpiEbeanServer ebeanServer)
  {
    this.ebeanServer = ebeanServer;
    for (int i = 0; i < propertiesMany.length; i++)
    {
      propertiesMany[i].setLoader(ebeanServer);
    }
  }
  









  public T createCopy(Object source, CopyContext ctx, int maxDepth)
  {
    Object destBean = createBean(ctx.isVanillaMode());
    for (int j = 0; j < propertiesId.length; j++) {
      propertiesId[j].copyProperty(source, destBean, ctx, maxDepth);
    }
    
    Object destId = getId(destBean);
    Object existing = ctx.putIfAbsent(destId, destBean);
    if (existing != null) {
      return existing;
    }
    for (int i = 0; i < propertiesNonTransient.length; i++) {
      propertiesNonTransient[i].copyProperty(source, destBean, ctx, maxDepth);
    }
    
    if ((destBean instanceof EntityBean)) {
      EntityBeanIntercept copyEbi = ((EntityBean)destBean)._ebean_getIntercept();
      if ((source instanceof EntityBean)) {
        EntityBeanIntercept origEbi = ((EntityBean)source)._ebean_getIntercept();
        origEbi.copyStateTo(copyEbi);
      }
      copyEbi.setBeanLoader(0, ebeanServer);
      copyEbi.setPersistenceContext(ctx.getPersistenceContext());
      
      if (ctx.isSharing()) {
        copyEbi.setSharedInstance();
      }
    }
    
    return destBean;
  }
  
  public T createCopyForUpdate(Object orig, boolean vanilla) {
    return createCopy(orig, new CopyContext(false, false), 3);
  }
  
  public T createCopyForSharing(Object orig) {
    return createCopy(orig, new CopyContext(false, true), 3);
  }
  




  public ConcurrencyMode determineConcurrencyMode(Object bean)
  {
    if (propertyFirstVersion == null) {
      return ConcurrencyMode.NONE;
    }
    Object v = propertyFirstVersion.getValue(bean);
    return v == null ? ConcurrencyMode.NONE : ConcurrencyMode.VERSION;
  }
  



  public Set<String> getDirtyEmbeddedProperties(Object bean)
  {
    HashSet<String> dirtyProperties = null;
    
    for (int i = 0; i < propertiesEmbedded.length; i++) {
      Object embValue = propertiesEmbedded[i].getValue(bean);
      if ((embValue instanceof EntityBean)) {
        if (((EntityBean)embValue)._ebean_getIntercept().isDirty())
        {
          if (dirtyProperties == null) {
            dirtyProperties = new HashSet();
          }
          dirtyProperties.add(propertiesEmbedded[i].getName());
        }
      }
      else {
        if (dirtyProperties == null) {
          dirtyProperties = new HashSet();
        }
        dirtyProperties.add(propertiesEmbedded[i].getName());
      }
    }
    
    return dirtyProperties;
  }
  



  public Set<String> determineLoadedProperties(Object bean)
  {
    HashSet<String> nonNullProps = new HashSet();
    
    for (int j = 0; j < propertiesId.length; j++) {
      if (propertiesId[j].getValue(bean) != null) {
        nonNullProps.add(propertiesId[j].getName());
      }
    }
    for (int i = 0; i < propertiesNonTransient.length; i++) {
      if (propertiesNonTransient[i].getValue(bean) != null) {
        nonNullProps.add(propertiesNonTransient[i].getName());
      }
    }
    return nonNullProps;
  }
  


  public SpiEbeanServer getEbeanServer()
  {
    return ebeanServer;
  }
  


  public EntityType getEntityType()
  {
    return entityType;
  }
  


  public IndexDefn<?> getLuceneIndexDefn()
  {
    return luceneIndexDefn;
  }
  



  public Query.UseIndex getUseIndex()
  {
    return useIndex;
  }
  


  public LIndex getLuceneIndex()
  {
    return luceneIndex;
  }
  


  public void setLuceneIndex(LIndex luceneIndex)
  {
    this.luceneIndex = luceneIndex;
  }
  
  public void addIndexInvalidate(IndexInvalidate e) {
    if (luceneIndexInvalidations == null) {
      luceneIndexInvalidations = new HashSet();
    }
    luceneIndexInvalidations.add(e);
  }
  
  public boolean isNotifyLucene(TransactionEvent txnEvent) {
    if (luceneIndexInvalidations != null) {
      for (IndexInvalidate invalidate : luceneIndexInvalidations) {
        txnEvent.addIndexInvalidate(invalidate);
      }
    }
    return luceneIndex != null;
  }
  







  public void initialiseId()
  {
    if (logger.isLoggable(Level.FINER)) {
      logger.finer("BeanDescriptor initialise " + fullName);
    }
    
    if (inheritInfo != null) {
      inheritInfo.setDescriptor(this);
    }
    
    if (isEmbedded())
    {
      Iterator<BeanProperty> it = propertiesAll();
      while (it.hasNext()) {
        BeanProperty prop = (BeanProperty)it.next();
        prop.initialise();
      }
    }
    else {
      BeanProperty[] idProps = propertiesId();
      for (int i = 0; i < idProps.length; i++) {
        idProps[i].initialise();
      }
    }
  }
  



  public void initialiseOther()
  {
    if (!isEmbedded())
    {
      Iterator<BeanProperty> it = propertiesAll();
      while (it.hasNext()) {
        BeanProperty prop = (BeanProperty)it.next();
        if (!prop.isId()) {
          prop.initialise();
        }
      }
    }
    
    if (unidirectional != null) {
      unidirectional.initialise();
    }
    
    idBinder.initialise();
    idBinderInLHSSql = idBinder.getBindIdInSql(baseTableAlias);
    idBinderIdSql = idBinder.getBindIdSql(baseTableAlias);
    String idBinderInLHSSqlNoAlias = idBinder.getBindIdInSql(null);
    String idEqualsSql = idBinder.getBindIdSql(null);
    
    deleteByIdSql = ("delete from " + baseTable + " where " + idEqualsSql);
    deleteByIdInSql = ("delete from " + baseTable + " where " + idBinderInLHSSqlNoAlias + " ");
    
    if (!isEmbedded())
    {
      for (DeployNamedUpdate namedUpdate : namedUpdates.values()) {
        DeployUpdateParser parser = new DeployUpdateParser(this);
        namedUpdate.initialise(parser);
      }
    }
  }
  
  public void initInheritInfo()
  {
    if (inheritInfo != null)
    {
      if (saveRecurseSkippable) {
        saveRecurseSkippable = inheritInfo.isSaveRecurseSkippable();
      }
      if (deleteRecurseSkippable) {
        deleteRecurseSkippable = inheritInfo.isDeleteRecurseSkippable();
      }
    }
  }
  



  public void cacheInitialise()
  {
    if ((referenceOptions != null) && (referenceOptions.isUseCache())) {
      beanCache = cacheManager.getBeanCache(beanType);
    }
  }
  
  protected boolean hasInheritance() {
    return inheritInfo != null;
  }
  
  protected boolean isDynamicSubclass() {
    return !beanType.equals(factoryType);
  }
  



  public void setLdapObjectClasses(Attributes attributes)
  {
    if (ldapObjectclasses != null) {
      BasicAttribute ocAttrs = new BasicAttribute("objectclass");
      for (int i = 0; i < ldapObjectclasses.length; i++) {
        ocAttrs.add(ldapObjectclasses[i]);
      }
      attributes.put(ocAttrs);
    }
  }
  



  public Attributes createAttributes()
  {
    Attributes attrs = new BasicAttributes(true);
    setLdapObjectClasses(attrs);
    return attrs;
  }
  
  public String getLdapBaseDn() {
    return ldapBaseDn;
  }
  
  public LdapName createLdapNameById(Object id) throws InvalidNameException
  {
    LdapName baseDn = new LdapName(ldapBaseDn);
    idBinder.createLdapNameById(baseDn, id);
    return baseDn;
  }
  
  public LdapName createLdapName(Object bean)
  {
    try {
      LdapName name = new LdapName(ldapBaseDn);
      if (bean != null) {
        idBinder.createLdapNameByBean(name, bean);
      }
      return name;
    }
    catch (InvalidNameException e) {
      throw new LdapPersistenceException(e);
    }
  }
  
  public SqlUpdate deleteById(Object id, List<Object> idList) {
    if (id != null) {
      return deleteById(id);
    }
    return deleteByIdList(idList);
  }
  





  private SqlUpdate deleteByIdList(List<Object> idList)
  {
    StringBuilder sb = new StringBuilder(deleteByIdInSql);
    String inClause = idBinder.getIdInValueExprDelete(idList.size());
    sb.append(inClause);
    
    DefaultSqlUpdate delete = new DefaultSqlUpdate(sb.toString());
    for (int i = 0; i < idList.size(); i++) {
      idBinder.bindId(delete, idList.get(i));
    }
    return delete;
  }
  




  private SqlUpdate deleteById(Object id)
  {
    DefaultSqlUpdate sqlDelete = new DefaultSqlUpdate(deleteByIdSql);
    
    Object[] bindValues = idBinder.getBindValues(id);
    for (int i = 0; i < bindValues.length; i++) {
      sqlDelete.addParameter(bindValues[i]);
    }
    
    return sqlDelete;
  }
  



  public void add(BeanFkeyProperty fkey)
  {
    fkeyMap.put(fkey.getName(), fkey);
  }
  
  public void initialiseFkeys() {
    for (int i = 0; i < propertiesOneImported.length; i++) {
      propertiesOneImported[i].addFkey();
    }
  }
  
  public boolean calculateUseCache(Boolean queryUseCache) {
    if (queryUseCache != null) {
      return queryUseCache.booleanValue();
    }
    if (referenceOptions != null) {
      return referenceOptions.isUseCache();
    }
    return false;
  }
  

  public boolean calculateReadOnly(Boolean queryReadOnly)
  {
    if (queryReadOnly != null) {
      return queryReadOnly.booleanValue();
    }
    if (referenceOptions != null) {
      return referenceOptions.isReadOnly();
    }
    return false;
  }
  




  public ReferenceOptions getReferenceOptions()
  {
    return referenceOptions;
  }
  









  public EncryptKey getEncryptKey(BeanProperty p)
  {
    return owner.getEncryptKey(baseTable, p.getDbColumn());
  }
  


  public EncryptKey getEncryptKey(String tableName, String columnName)
  {
    return owner.getEncryptKey(tableName, columnName);
  }
  



  public void runCacheWarming()
  {
    if (referenceOptions == null) {
      return;
    }
    String warmingQuery = referenceOptions.getWarmingQuery();
    if ((warmingQuery != null) && (warmingQuery.trim().length() > 0)) {
      Query<T> query = ebeanServer.createQuery(beanType, warmingQuery);
      query.setUseCache(true);
      query.setReadOnly(true);
      query.setLoadBeanCache(true);
      List<T> list = query.findList();
      if (logger.isLoggable(Level.INFO)) {
        String msg = "Loaded " + beanType + " cache with [" + list.size() + "] beans";
        logger.info(msg);
      }
    }
  }
  



  public boolean hasDefaultSelectClause()
  {
    return defaultSelectClause != null;
  }
  


  public String getDefaultSelectClause()
  {
    return defaultSelectClause;
  }
  


  public Set<String> getDefaultSelectClauseSet()
  {
    return defaultSelectClauseSet;
  }
  



  public String[] getDefaultSelectDbArray()
  {
    return defaultSelectDbArray;
  }
  



  public boolean isInheritanceRoot()
  {
    return (inheritInfo == null) || (inheritInfo.isRoot());
  }
  


  public boolean isQueryCaching()
  {
    return queryCache != null;
  }
  


  public boolean isBeanCaching()
  {
    return beanCache != null;
  }
  



  public boolean isCacheNotify(boolean isInsertRequest)
  {
    if (isInsertRequest)
    {
      return queryCache != null;
    }
    return (beanCache != null) || (queryCache != null);
  }
  



  public boolean isUsingL2Cache()
  {
    return (beanCache != null) || (luceneIndex != null);
  }
  


  public boolean isLuceneIndexed()
  {
    return luceneIndex != null;
  }
  



  public void cacheNotify(TransactionEventTable.TableIUD tableIUD)
  {
    if (tableIUD.isUpdateOrDelete()) {
      cacheClear();
    }
    
    queryCacheClear();
  }
  


  public void queryCacheClear()
  {
    if (queryCache != null) {
      queryCache.clear();
    }
  }
  



  public BeanCollection<T> queryCacheGet(Object id)
  {
    if (queryCache == null) {
      return null;
    }
    return (BeanCollection)queryCache.get(id);
  }
  



  public void queryCachePut(Object id, BeanCollection<T> query)
  {
    if (queryCache == null) {
      queryCache = cacheManager.getQueryCache(beanType);
    }
    queryCache.put(id, query);
  }
  


  public void cacheClear()
  {
    if (beanCache != null) {
      beanCache.clear();
    }
  }
  
  public T cachePutObject(Object bean)
  {
    Object cacheBean = createCopyForSharing(bean);
    return cachePut(cacheBean, true);
  }
  



  public T cachePut(T bean, boolean alreadyShared)
  {
    if (beanCache == null) {
      beanCache = cacheManager.getBeanCache(beanType);
    }
    
    if (!alreadyShared) {
      bean = createCopyForSharing(bean);
    }
    Object id = getId(bean);
    return beanCache.put(id, bean);
  }
  



  public T cacheGet(Object id)
  {
    if (beanCache == null) {
      return null;
    }
    return beanCache.get(id);
  }
  



  public void cacheRemove(Object id)
  {
    if (beanCache != null) {
      beanCache.remove(id);
    }
  }
  



  public String getBaseTableAlias()
  {
    return baseTableAlias;
  }
  


  public boolean refreshFromCache(EntityBeanIntercept ebi, Object id)
  {
    if ((ebi.isUseCache()) && (ebi.isReference()))
    {
      Object cacheBean = cacheGet(id);
      if (cacheBean != null)
      {

        String lazyLoadProperty = ebi.getLazyLoadProperty();
        Set<String> loadedProps = ((EntityBean)cacheBean)._ebean_getIntercept().getLoadedProps();
        if ((loadedProps == null) || (loadedProps.contains(lazyLoadProperty)))
        {
          refreshFromCacheBean(ebi, cacheBean, true);
          return true;
        }
      }
    }
    return false;
  }
  


  private void refreshFromCacheBean(EntityBeanIntercept ebi, Object cacheBean, boolean isLazyLoad)
  {
    new BeanRefreshFromCacheHelp(this, ebi, cacheBean, isLazyLoad).refresh();
  }
  
  public void preAllocateIds(int batchSize) {
    if (idGenerator != null) {
      idGenerator.preAllocateIds(batchSize);
    }
  }
  
  public Object nextId(Transaction t) {
    if (idGenerator != null) {
      return idGenerator.nextId(t);
    }
    return null;
  }
  
  public DeployPropertyParser createDeployPropertyParser()
  {
    return new DeployPropertyParser(this);
  }
  



  public String convertOrmUpdateToSql(String ormUpdateStatement)
  {
    return new DeployUpdateParser(this).parse(ormUpdateStatement);
  }
  


  public void clearQueryStatistics()
  {
    Iterator<CQueryPlan> it = queryPlanCache.values().iterator();
    while (it.hasNext()) {
      CQueryPlan queryPlan = (CQueryPlan)it.next();
      queryPlan.resetStatistics();
    }
  }
  



  public void postLoad(Object bean, Set<String> includedProperties)
  {
    BeanPersistController c = persistController;
    if (c != null) {
      c.postLoad(bean, includedProperties);
    }
  }
  


  public Iterator<CQueryPlan> queryPlans()
  {
    return queryPlanCache.values().iterator();
  }
  
  public CQueryPlan getQueryPlan(Integer key) {
    return (CQueryPlan)queryPlanCache.get(key);
  }
  
  public void putQueryPlan(Integer key, CQueryPlan plan) {
    queryPlanCache.put(key, plan);
  }
  


  public SpiUpdatePlan getUpdatePlan(Integer key)
  {
    return (SpiUpdatePlan)updatePlanCache.get(key);
  }
  


  public void putUpdatePlan(Integer key, SpiUpdatePlan plan)
  {
    updatePlanCache.put(key, plan);
  }
  


  public TypeManager getTypeManager()
  {
    return typeManager;
  }
  



  public boolean isUpdateChangesOnly()
  {
    return updateChangesOnly;
  }
  



  public boolean isSaveRecurseSkippable()
  {
    return saveRecurseSkippable;
  }
  



  public boolean isDeleteRecurseSkippable()
  {
    return deleteRecurseSkippable;
  }
  


  public boolean hasLocalValidation()
  {
    return hasLocalValidation;
  }
  


  public boolean hasCascadeValidation()
  {
    return hasCascadeValidation;
  }
  
  public InvalidValue validate(boolean cascade, Object bean)
  {
    if (!hasCascadeValidation)
    {
      return null;
    }
    
    List<InvalidValue> errList = null;
    
    Set<String> loadedProps = null;
    if ((bean instanceof EntityBean)) {
      EntityBeanIntercept ebi = ((EntityBean)bean)._ebean_getIntercept();
      loadedProps = ebi.getLoadedProps();
    }
    if (loadedProps != null)
    {
      Iterator<String> propIt = loadedProps.iterator();
      while (propIt.hasNext()) {
        String propName = (String)propIt.next();
        BeanProperty property = getBeanProperty(propName);
        

        if ((property != null) && (property.hasValidationRules(cascade))) {
          Object value = property.getValue(bean);
          List<InvalidValue> errs = property.validate(cascade, value);
          if (errs != null) {
            if (errList == null) {
              errList = new ArrayList();
            }
            errList.addAll(errs);
          }
        }
      }
    }
    else {
      BeanProperty[] props = cascade ? propertiesValidationCascade : propertiesValidationLocal;
      

      for (int i = 0; i < props.length; i++) {
        BeanProperty prop = props[i];
        Object value = prop.getValue(bean);
        List<InvalidValue> errs = prop.validate(cascade, value);
        if (errs != null) {
          if (errList == null) {
            errList = new ArrayList();
          }
          errList.addAll(errs);
        }
      }
    }
    
    for (int i = 0; i < beanValidators.length; i++) {
      if (!beanValidators[i].isValid(bean)) {
        if (errList == null) {
          errList = new ArrayList();
        }
        Validator v = beanValidators[i];
        errList.add(new InvalidValue(v.getKey(), v.getAttributes(), getFullName(), null, bean));
      }
    }
    
    if (errList == null) {
      return null;
    }
    
    return new InvalidValue(null, getFullName(), bean, InvalidValue.toArray(errList));
  }
  



  public BeanPropertyAssocMany<?> getManyProperty(SpiQuery<?> query)
  {
    OrmQueryDetail detail = query.getDetail();
    for (int i = 0; i < propertiesMany.length; i++) {
      if (detail.includes(propertiesMany[i].getName())) {
        return propertiesMany[i];
      }
    }
    
    return null;
  }
  



  public IdBinder getIdBinder()
  {
    return idBinder;
  }
  



  public String getIdBinderIdSql()
  {
    return idBinderIdSql;
  }
  


  public String getIdBinderInLHSSql()
  {
    return idBinderInLHSSql;
  }
  




  public void bindId(DataBind dataBind, Object idValue)
    throws SQLException
  {
    idBinder.bindId(dataBind, idValue);
  }
  





  public Object[] getBindIdValues(Object idValue)
  {
    return idBinder.getBindValues(idValue);
  }
  


  public DeployNamedQuery getNamedQuery(String name)
  {
    return (DeployNamedQuery)namedQueries.get(name);
  }
  
  public DeployNamedQuery addNamedQuery(DeployNamedQuery deployNamedQuery) {
    return (DeployNamedQuery)namedQueries.put(deployNamedQuery.getName(), deployNamedQuery);
  }
  


  public DeployNamedUpdate getNamedUpdate(String name)
  {
    return (DeployNamedUpdate)namedUpdates.get(name);
  }
  


  public Object createBean(boolean vanillaMode)
  {
    return vanillaMode ? createVanillaBean() : createEntityBean();
  }
  





  public Object createVanillaBean()
  {
    return beanReflect.createVanillaBean();
  }
  


  public EntityBean createEntityBean()
  {
    try
    {
      return (EntityBean)beanReflect.createEntityBean();

    }
    catch (Exception ex)
    {
      throw new PersistenceException(ex);
    }
  }
  



  public T createReference(boolean vanillaMode, Object id, Object parent, ReferenceOptions options)
  {
    try
    {
      Object bean = createBean(vanillaMode);
      
      convertSetId(id, bean);
      
      if (!vanillaMode) {
        EntityBean eb = (EntityBean)bean;
        
        EntityBeanIntercept ebi = eb._ebean_getIntercept();
        ebi.setBeanLoader(0, ebeanServer);
        
        if (parent != null)
        {

          ebi.setParentBean(parent);
        }
        
        if (options != null) {
          ebi.setUseCache(options.isUseCache());
          ebi.setReadOnly(options.isReadOnly());
        }
        

        ebi.setReference();
      }
      
      return bean;
    }
    catch (Exception ex) {
      throw new PersistenceException(ex);
    }
  }
  


  public BeanProperty getBeanPropertyFromDbColumn(String dbColumn)
  {
    return (BeanProperty)propMapByDbColumn.get(dbColumn);
  }
  




  public BeanProperty getBeanPropertyFromPath(String path)
  {
    String[] split = SplitName.splitBegin(path);
    if (split[1] == null) {
      return _findBeanProperty(split[0]);
    }
    BeanPropertyAssoc<?> assocProp = (BeanPropertyAssoc)_findBeanProperty(split[0]);
    BeanDescriptor<?> targetDesc = assocProp.getTargetDescriptor();
    
    return targetDesc.getBeanPropertyFromPath(split[1]);
  }
  



  public BeanDescriptor<?> getBeanDescriptor(String path)
  {
    if (path == null) {
      return this;
    }
    String[] splitBegin = SplitName.splitBegin(path);
    
    BeanProperty beanProperty = (BeanProperty)propMap.get(splitBegin[0]);
    if ((beanProperty instanceof BeanPropertyAssoc)) {
      BeanPropertyAssoc<?> assocProp = (BeanPropertyAssoc)beanProperty;
      return assocProp.getTargetDescriptor().getBeanDescriptor(splitBegin[1]);
    }
    
    throw new RuntimeException("Error getting BeanDescriptor for path " + path + " from " + getFullName());
  }
  



  public <U> BeanDescriptor<U> getBeanDescriptor(Class<U> otherType)
  {
    return owner.getBeanDescriptor(otherType);
  }
  







  public BeanPropertyAssocOne<?> getUnidirectional()
  {
    if (unidirectional != null) {
      return unidirectional;
    }
    if ((inheritInfo != null) && (!inheritInfo.isRoot())) {
      return inheritInfo.getParent().getBeanDescriptor().getUnidirectional();
    }
    return null;
  }
  


  public Object getValue(Object bean, String property)
  {
    return getBeanProperty(property).getValue(bean);
  }
  






  public boolean isUseIdGenerator()
  {
    return idGenerator != null;
  }
  



  public String getDescriptorId()
  {
    return descriptorId;
  }
  


  public Class<T> getBeanType()
  {
    return beanType;
  }
  


  public Class<?> getFactoryType()
  {
    return factoryType;
  }
  






  public String getFullName()
  {
    return fullName;
  }
  


  public String getName()
  {
    return name;
  }
  


  public String toString()
  {
    return fullName;
  }
  






  public Object getId(Object bean)
  {
    if (propertySingleId != null) {
      if ((inheritInfo != null) && (!enhancedBean))
      {
        return propertySingleId.getValueViaReflection(bean);
      }
      
      return propertySingleId.getValue(bean);
    }
    



    LinkedHashMap<String, Object> idMap = new LinkedHashMap();
    for (int i = 0; i < propertiesId.length; i++)
    {
      Object value = propertiesId[i].getValue(bean);
      idMap.put(propertiesId[i].getName(), value);
    }
    return idMap;
  }
  



  public boolean isComplexId()
  {
    return idBinder.isComplexId();
  }
  



  public String getDefaultOrderBy()
  {
    return idBinder.getDefaultOrderBy();
  }
  


  public Object convertId(Object idValue)
  {
    return idBinder.convertSetId(idValue, null);
  }
  






  public Object convertSetId(Object idValue, Object bean)
  {
    return idBinder.convertSetId(idValue, bean);
  }
  


  public BeanProperty getBeanProperty(String propName)
  {
    return (BeanProperty)propMap.get(propName);
  }
  
  public void sort(List<T> list, String sortByClause)
  {
    ElComparator<T> comparator = getElComparator(sortByClause);
    Collections.sort(list, comparator);
  }
  
  public ElComparator<T> getElComparator(String propNameOrSortBy) {
    ElComparator<T> c = (ElComparator)comparatorCache.get(propNameOrSortBy);
    if (c == null) {
      c = createComparator(propNameOrSortBy);
      comparatorCache.put(propNameOrSortBy, c);
    }
    return c;
  }
  




  public boolean lazyLoadMany(EntityBeanIntercept ebi)
  {
    String lazyLoadProperty = ebi.getLazyLoadProperty();
    BeanProperty lazyLoadBeanProp = getBeanProperty(lazyLoadProperty);
    
    if ((lazyLoadBeanProp instanceof BeanPropertyAssocMany)) {
      BeanPropertyAssocMany<?> manyProp = (BeanPropertyAssocMany)lazyLoadBeanProp;
      manyProp.createReference(ebi.getOwner());
      Set<String> loadedProps = ebi.getLoadedProps();
      HashSet<String> newLoadedProps = new HashSet();
      if (loadedProps != null) {
        newLoadedProps.addAll(loadedProps);
      }
      newLoadedProps.add(lazyLoadProperty);
      ebi.setLoadedProps(newLoadedProps);
      ebi.setLoadedLazy();
      return true;
    }
    
    return false;
  }
  







  private ElComparator<T> createComparator(String sortByClause)
  {
    SortByClause sortBy = SortByClauseParser.parse(sortByClause);
    if (sortBy.size() == 1)
    {
      return createPropertyComparator((SortByClause.Property)sortBy.getProperties().get(0));
    }
    

    ElComparator<T>[] comparators = new ElComparator[sortBy.size()];
    
    List<SortByClause.Property> sortProps = sortBy.getProperties();
    for (int i = 0; i < sortProps.size(); i++) {
      SortByClause.Property sortProperty = (SortByClause.Property)sortProps.get(i);
      comparators[i] = createPropertyComparator(sortProperty);
    }
    
    return new ElComparatorCompound(comparators);
  }
  
  private ElComparator<T> createPropertyComparator(SortByClause.Property sortProp)
  {
    ElPropertyValue elGetValue = getElGetValue(sortProp.getName());
    
    Boolean nullsHigh = sortProp.getNullsHigh();
    if (nullsHigh == null) {
      nullsHigh = Boolean.TRUE;
    }
    return new ElComparatorProperty(elGetValue, sortProp.isAscending(), nullsHigh.booleanValue());
  }
  


  public ElPropertyValue getElGetValue(String propName)
  {
    return getElPropertyValue(propName, false);
  }
  





  public ElPropertyDeploy getElPropertyDeploy(String propName)
  {
    ElPropertyDeploy fk = (ElPropertyDeploy)fkeyMap.get(propName);
    if (fk != null) {
      return fk;
    }
    return getElPropertyValue(propName, true);
  }
  
  private ElPropertyValue getElPropertyValue(String propName, boolean propertyDeploy) {
    ElPropertyValue elGetValue = (ElPropertyValue)elGetCache.get(propName);
    if (elGetValue == null)
    {
      elGetValue = buildElGetValue(propName, null, propertyDeploy);
      if (elGetValue == null) {
        return null;
      }
      if ((elGetValue instanceof BeanFkeyProperty)) {
        fkeyMap.put(propName, (BeanFkeyProperty)elGetValue);
      } else {
        elGetCache.put(propName, elGetValue);
      }
    }
    return elGetValue;
  }
  
  protected ElPropertyValue buildElGetValue(String propName, ElPropertyChainBuilder chain, boolean propertyDeploy)
  {
    if ((propertyDeploy) && (chain != null)) {
      BeanFkeyProperty fk = (BeanFkeyProperty)fkeyMap.get(propName);
      if (fk != null) {
        return fk.create(chain.getExpression());
      }
    }
    
    int basePos = propName.indexOf('.');
    if (basePos > -1)
    {
      String baseName = propName.substring(0, basePos);
      String remainder = propName.substring(basePos + 1);
      
      BeanProperty assocProp = _findBeanProperty(baseName);
      if (assocProp == null) {
        return null;
      }
      return assocProp.buildElPropertyValue(propName, remainder, chain, propertyDeploy);
    }
    
    BeanProperty property = _findBeanProperty(propName);
    if (chain == null) {
      return property;
    }
    if (property == null) {
      throw new PersistenceException("No property found for [" + propName + "] in expression " + chain.getExpression());
    }
    
    if (property.containsMany()) {
      chain.setContainsMany(true);
    }
    return chain.add(property).build();
  }
  






  public BeanProperty findBeanProperty(String propName)
  {
    int basePos = propName.indexOf('.');
    if (basePos > -1)
    {
      String baseName = propName.substring(0, basePos);
      return _findBeanProperty(baseName);
    }
    
    return _findBeanProperty(propName);
  }
  
  private BeanProperty _findBeanProperty(String propName) {
    BeanProperty prop = (BeanProperty)propMap.get(propName);
    if ((prop == null) && (inheritInfo != null))
    {
      return inheritInfo.findSubTypeProperty(propName);
    }
    return prop;
  }
  
  protected Object getBeanPropertyWithInheritance(Object bean, String propName)
  {
    BeanDescriptor<?> desc = getBeanDescriptor(bean.getClass());
    BeanProperty beanProperty = desc.findBeanProperty(propName);
    return beanProperty.getValue(bean);
  }
  


  public String getServerName()
  {
    return serverName;
  }
  


  public boolean isAutoFetchTunable()
  {
    return autoFetchTunable;
  }
  



  public InheritInfo getInheritInfo()
  {
    return inheritInfo;
  }
  


  public boolean isEmbedded()
  {
    return EntityType.EMBEDDED.equals(entityType);
  }
  
  public boolean isBaseTableType() {
    return EntityType.ORM.equals(entityType);
  }
  


  public ConcurrencyMode getConcurrencyMode()
  {
    return concurrencyMode;
  }
  



  public String[] getDependantTables()
  {
    return dependantTables;
  }
  


  public CompoundUniqueContraint[] getCompoundUniqueConstraints()
  {
    return compoundUniqueConstraints;
  }
  


  public BeanPersistListener<T> getPersistListener()
  {
    return persistListener;
  }
  


  public BeanFinder<T> getBeanFinder()
  {
    return beanFinder;
  }
  


  public BeanQueryAdapter getQueryAdapter()
  {
    return queryAdapter;
  }
  




  public void deregister(BeanPersistListener<?> listener)
  {
    BeanPersistListener<T> currListener = persistListener;
    if (currListener != null)
    {

      BeanPersistListener<T> deregListener = listener;
      if ((currListener instanceof ChainedBeanPersistListener))
      {
        persistListener = ((ChainedBeanPersistListener)currListener).deregister(deregListener);
      } else if (currListener.equals(deregListener)) {
        persistListener = null;
      }
    }
  }
  



  public void deregister(BeanPersistController controller)
  {
    BeanPersistController c = persistController;
    if (c != null)
    {

      if ((c instanceof ChainedBeanPersistController))
      {
        persistController = ((ChainedBeanPersistController)c).deregister(controller);
      } else if (c.equals(controller)) {
        persistController = null;
      }
    }
  }
  




  public void register(BeanPersistListener<?> newPersistListener)
  {
    if (PersistListenerManager.isRegisterFor(beanType, newPersistListener))
    {

      BeanPersistListener<T> newListener = newPersistListener;
      
      BeanPersistListener<T> currListener = persistListener;
      if (currListener == null) {
        persistListener = newListener;
      }
      else if ((currListener instanceof ChainedBeanPersistListener))
      {
        persistListener = ((ChainedBeanPersistListener)currListener).register(newListener);
      }
      else {
        persistListener = new ChainedBeanPersistListener(currListener, newListener);
      }
    }
  }
  




  public void register(BeanPersistController newController)
  {
    if (newController.isRegisterFor(beanType))
    {


      BeanPersistController c = persistController;
      if (c == null) {
        persistController = newController;
      }
      else if ((c instanceof ChainedBeanPersistController))
      {
        persistController = ((ChainedBeanPersistController)c).register(newController);
      }
      else {
        persistController = new ChainedBeanPersistController(c, newController);
      }
    }
  }
  



  public BeanPersistController getPersistController()
  {
    return persistController;
  }
  







  public boolean isSqlSelectBased()
  {
    return EntityType.SQL.equals(entityType);
  }
  


  public boolean isLdapEntityType()
  {
    return EntityType.LDAP.equals(entityType);
  }
  



  public String getBaseTable()
  {
    return baseTable;
  }
  


  public String getExtraAttribute(String key)
  {
    return (String)extraAttrMap.get(key);
  }
  


  public IdType getIdType()
  {
    return idType;
  }
  


  public String getSequenceName()
  {
    return sequenceName;
  }
  






  public String getSelectLastInsertedId()
  {
    return selectLastInsertedId;
  }
  


  public IdGenerator getIdGenerator()
  {
    return idGenerator;
  }
  


  public String getLazyFetchIncludes()
  {
    return lazyFetchIncludes;
  }
  





  public TableJoin[] tableJoins()
  {
    return derivedTableJoins;
  }
  



  public Iterator<BeanProperty> propertiesAll()
  {
    return propMap.values().iterator();
  }
  






  public BeanProperty[] propertiesId()
  {
    return propertiesId;
  }
  


  public BeanProperty[] propertiesNonTransient()
  {
    return propertiesNonTransient;
  }
  


  public BeanProperty[] propertiesTransient()
  {
    return propertiesTransient;
  }
  



  public BeanProperty getSingleIdProperty()
  {
    return propertySingleId;
  }
  



  public BeanPropertyAssocOne<?>[] propertiesEmbedded()
  {
    return propertiesEmbedded;
  }
  



  public BeanPropertyAssocOne<?>[] propertiesOne()
  {
    return propertiesOne;
  }
  





  public BeanPropertyAssocOne<?>[] propertiesOneImported()
  {
    return propertiesOneImported;
  }
  


  public BeanPropertyAssocOne<?>[] propertiesOneImportedSave()
  {
    return propertiesOneImportedSave;
  }
  


  public BeanPropertyAssocOne<?>[] propertiesOneImportedDelete()
  {
    return propertiesOneImportedDelete;
  }
  





  public BeanPropertyAssocOne<?>[] propertiesOneExported()
  {
    return propertiesOneExported;
  }
  


  public BeanPropertyAssocOne<?>[] propertiesOneExportedSave()
  {
    return propertiesOneExportedSave;
  }
  


  public BeanPropertyAssocOne<?>[] propertiesOneExportedDelete()
  {
    return propertiesOneExportedDelete;
  }
  
  private Set<String> deriveManyPropNames()
  {
    LinkedHashSet<String> names = new LinkedHashSet();
    for (int i = 0; i < propertiesMany.length; i++) {
      names.add(propertiesMany[i].getName());
    }
    
    return Collections.unmodifiableSet(names);
  }
  




  public int getNamesOfManyPropsHash()
  {
    return namesOfManyPropsHash;
  }
  


  public Set<String> getNamesOfManyProps()
  {
    return namesOfManyProps;
  }
  


  public BeanPropertyAssocMany<?>[] propertiesMany()
  {
    return propertiesMany;
  }
  


  public BeanPropertyAssocMany<?>[] propertiesManySave()
  {
    return propertiesManySave;
  }
  


  public BeanPropertyAssocMany<?>[] propertiesManyDelete()
  {
    return propertiesManyDelete;
  }
  


  public BeanPropertyAssocMany<?>[] propertiesManyToMany()
  {
    return propertiesManyToMany;
  }
  






  public BeanProperty firstVersionProperty()
  {
    return propertyFirstVersion;
  }
  


  public boolean isVanillaInsert(Object bean)
  {
    if (propertyFirstVersion == null) {
      return true;
    }
    Object versionValue = propertyFirstVersion.getValue(bean);
    return DmlUtil.isNullOrZero(versionValue);
  }
  



  public boolean isStatelessUpdate(Object bean)
  {
    if (propertyFirstVersion == null) {
      Object versionValue = getId(bean);
      return !DmlUtil.isNullOrZero(versionValue);
    }
    Object versionValue = propertyFirstVersion.getValue(bean);
    return !DmlUtil.isNullOrZero(versionValue);
  }
  





  public BeanProperty[] propertiesVersion()
  {
    return propertiesVersion;
  }
  


  public BeanProperty[] propertiesBaseScalar()
  {
    return propertiesBaseScalar;
  }
  





  public BeanPropertyCompound[] propertiesBaseCompound()
  {
    return propertiesBaseCompound;
  }
  


  public BeanProperty[] propertiesLocal()
  {
    return propertiesLocal;
  }
  
  public void jsonWrite(WriteJsonContext ctx, Object bean)
  {
    if (bean != null)
    {
      ctx.appendObjectBegin();
      WriteJsonContext.WriteBeanState prevState = ctx.pushBeanState(bean);
      
      if (inheritInfo != null) {
        InheritInfo localInheritInfo = inheritInfo.readType(bean.getClass());
        String discValue = localInheritInfo.getDiscriminatorStringValue();
        String discColumn = localInheritInfo.getDiscriminatorColumn();
        ctx.appendKeyValue(discColumn, "\"" + discValue + "\"");
        
        BeanDescriptor<?> localDescriptor = localInheritInfo.getBeanDescriptor();
        localDescriptor.jsonWriteProperties(ctx, bean);
      }
      else {
        jsonWriteProperties(ctx, bean);
      }
      
      ctx.pushPreviousState(prevState);
      ctx.appendObjectEnd();
    }
  }
  


  private void jsonWriteProperties(WriteJsonContext ctx, Object bean)
  {
    boolean referenceBean = ctx.isReferenceBean();
    
    JsonWriteBeanVisitor<T> beanVisitor = ctx.getBeanVisitor();
    
    Set<String> props = ctx.getIncludeProperties();
    boolean explicitAllProps;
    boolean explicitAllProps;
    if (props == null) {
      explicitAllProps = false;
    } else {
      explicitAllProps = props.contains("*");
      if ((explicitAllProps) || (props.isEmpty())) {
        props = null;
      }
    }
    
    for (int i = 0; i < propertiesId.length; i++) {
      Object idValue = propertiesId[i].getValue(bean);
      if ((idValue != null) && (
        (props == null) || (props.contains(propertiesId[i].getName())))) {
        propertiesId[i].jsonWrite(ctx, bean);
      }
    }
    

    if ((!explicitAllProps) && (props == null)) {
      props = ctx.getLoadedProps();
    }
    if (props != null) {
      for (String prop : props) {
        BeanProperty p = getBeanProperty(prop);
        if (!p.isId()) {
          p.jsonWrite(ctx, bean);
        }
        
      }
    } else if (!referenceBean) {
      for (int j = 0; j < propertiesNonTransient.length; j++) {
        propertiesNonTransient[j].jsonWrite(ctx, bean);
      }
    }
    

    if (beanVisitor != null) {
      beanVisitor.visit(bean, ctx);
    }
  }
  
  public T jsonReadBean(ReadJsonContext ctx, String path)
  {
    ReadJsonContext.ReadBeanState beanState = jsonRead(ctx, path);
    if (beanState == null) {
      return null;
    }
    beanState.setLoadedState();
    return beanState.getBean();
  }
  
  public ReadJsonContext.ReadBeanState jsonRead(ReadJsonContext ctx, String path)
  {
    if (!ctx.readObjectBegin())
    {
      return null;
    }
    
    if (inheritInfo == null) {
      return jsonReadObject(ctx, path);
    }
    

    String discColumn = inheritInfo.getRoot().getDiscriminatorColumn();
    
    if (!ctx.readKeyNext()) {
      String msg = "Error reading inheritance discriminator - expected [" + discColumn + "] but no json key?";
      throw new TextException(msg);
    }
    String propName = ctx.getTokenKey();
    
    if (!propName.equalsIgnoreCase(discColumn)) {
      String msg = "Error reading inheritance discriminator - expected [" + discColumn + "] but read [" + propName + "]";
      throw new TextException(msg);
    }
    
    String discValue = ctx.readScalarValue();
    if (!ctx.readValueNext()) {
      String msg = "Error reading inheritance discriminator [" + discColumn + "]. Expected more json name values?";
      throw new TextException(msg);
    }
    

    InheritInfo localInheritInfo = inheritInfo.readType(discValue);
    BeanDescriptor<?> localDescriptor = localInheritInfo.getBeanDescriptor();
    return localDescriptor.jsonReadObject(ctx, path);
  }
  


  private ReadJsonContext.ReadBeanState jsonReadObject(ReadJsonContext ctx, String path)
  {
    T bean = createEntityBean();
    ctx.pushBean(bean, path, this);
    

    while (ctx.readKeyNext())
    {


      String propName = ctx.getTokenKey();
      BeanProperty p = getBeanProperty(propName);
      if (p != null) {
        p.jsonRead(ctx, bean);
        ctx.setProperty(propName);
      }
      else {
        ctx.readUnmappedJson(propName);
      }
      
      if (!ctx.readValueNext()) {
        break;
      }
    }
    

    return ctx.popBeanState();
  }
  



  public void setLoadedProps(EntityBeanIntercept ebi, Set<String> loadedProps)
  {
    if (isLoadedReference(loadedProps)) {
      ebi.setReference();
    } else {
      ebi.setLoadedProps(loadedProps);
    }
  }
  




  public boolean isLoadedReference(Set<String> loadedProps)
  {
    if ((loadedProps != null) && 
      (loadedProps.size() == propertiesId.length)) {
      for (int i = 0; i < propertiesId.length; i++) {
        if (!loadedProps.contains(propertiesId[i].getName())) {
          return false;
        }
      }
      return true;
    }
    

    return false;
  }
}
