package com.avaje.ebeaninternal.server.deploy;

import com.avaje.ebean.InvalidValue;
import com.avaje.ebean.bean.EntityBean;
import com.avaje.ebean.config.EncryptKey;
import com.avaje.ebean.config.dbplatform.DbEncryptFunction;
import com.avaje.ebean.config.dbplatform.DbType;
import com.avaje.ebean.config.ldap.LdapAttributeAdapter;
import com.avaje.ebean.config.lucene.LuceneIndex;
import com.avaje.ebean.text.StringFormatter;
import com.avaje.ebean.text.StringParser;
import com.avaje.ebean.validation.factory.Validator;
import com.avaje.ebeaninternal.server.core.InternString;
import com.avaje.ebeaninternal.server.deploy.generatedproperty.GeneratedProperty;
import com.avaje.ebeaninternal.server.deploy.meta.DeployBeanProperty;
import com.avaje.ebeaninternal.server.el.ElPropertyChainBuilder;
import com.avaje.ebeaninternal.server.el.ElPropertyValue;
import com.avaje.ebeaninternal.server.ldap.LdapPersistenceException;
import com.avaje.ebeaninternal.server.lib.util.StringHelper;
import com.avaje.ebeaninternal.server.query.SqlBeanLoad;
import com.avaje.ebeaninternal.server.reflect.BeanReflectGetter;
import com.avaje.ebeaninternal.server.reflect.BeanReflectSetter;
import com.avaje.ebeaninternal.server.text.json.ReadJsonContext;
import com.avaje.ebeaninternal.server.text.json.WriteJsonContext;
import com.avaje.ebeaninternal.server.type.DataBind;
import com.avaje.ebeaninternal.server.type.ScalarType;
import com.avaje.ebeaninternal.util.ValueUtil;
import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import javax.naming.NamingException;
import javax.naming.directory.Attribute;
import javax.naming.directory.BasicAttribute;
import javax.persistence.PersistenceException;



























































































































































































public class BeanProperty
  implements ElPropertyValue
{
  public static final String EXCLUDE_FROM_UPDATE_WHERE = "EXCLUDE_FROM_UPDATE_WHERE";
  public static final String EXCLUDE_FROM_DELETE_WHERE = "EXCLUDE_FROM_DELETE_WHERE";
  public static final String EXCLUDE_FROM_INSERT = "EXCLUDE_FROM_INSERT";
  public static final String EXCLUDE_FROM_UPDATE = "EXCLUDE_FROM_UPDATE";
  final boolean id;
  final boolean unidirectionalShadow;
  final boolean embedded;
  final boolean version;
  final boolean nullable;
  final boolean unique;
  final boolean dbRead;
  final boolean dbInsertable;
  final boolean dbUpdatable;
  final boolean secondaryTable;
  final TableJoin secondaryTableJoin;
  final String secondaryTableJoinPrefix;
  final boolean inherited;
  final Class<?> owningType;
  final boolean local;
  final boolean lob;
  final boolean fetchEager;
  final boolean isTransient;
  final String name;
  final Field field;
  final Class<?> propertyType;
  final String dbBind;
  final String dbColumn;
  final String elPlaceHolder;
  final String elPlaceHolderEncrypted;
  final String sqlFormulaSelect;
  final String sqlFormulaJoin;
  final boolean formula;
  final boolean dbEncrypted;
  final boolean localEncrypted;
  final int dbEncryptedType;
  final int dbType;
  final Object defaultValue;
  final Map<String, String> extraAttributeMap;
  final Method readMethod;
  final Method writeMethod;
  final GeneratedProperty generatedProperty;
  final BeanReflectGetter getter;
  final BeanReflectSetter setter;
  final BeanDescriptor<?> descriptor;
  final ScalarType scalarType;
  final LdapAttributeAdapter ldapAttributeAdapter;
  final Validator[] validators;
  final boolean hasLocalValidators;
  boolean cascadeValidate;
  final int dbLength;
  final int dbScale;
  final String dbColumnDefn;
  final String dbConstraintExpression;
  final DbEncryptFunction dbEncryptFunction;
  final boolean dynamicSubclassWithInheritance;
  int deployOrder;
  
  public BeanProperty(DeployBeanProperty deploy)
  {
    this(null, null, deploy);
  }
  
  public BeanProperty(BeanDescriptorMap owner, BeanDescriptor<?> descriptor, DeployBeanProperty deploy)
  {
    this.descriptor = descriptor;
    name = InternString.intern(deploy.getName());
    if (descriptor != null) {
      dynamicSubclassWithInheritance = ((descriptor.isDynamicSubclass()) && (descriptor.hasInheritance()));
    } else {
      dynamicSubclassWithInheritance = false;
    }
    unidirectionalShadow = deploy.isUndirectionalShadow();
    localEncrypted = deploy.isLocalEncrypted();
    dbEncrypted = deploy.isDbEncrypted();
    dbEncryptedType = deploy.getDbEncryptedType();
    dbEncryptFunction = deploy.getDbEncryptFunction();
    dbBind = deploy.getDbBind();
    dbRead = deploy.isDbRead();
    dbInsertable = deploy.isDbInsertable();
    dbUpdatable = deploy.isDbUpdateable();
    
    secondaryTable = deploy.isSecondaryTable();
    if (secondaryTable) {
      secondaryTableJoin = new TableJoin(deploy.getSecondaryTableJoin(), null);
      secondaryTableJoinPrefix = deploy.getSecondaryTableJoinPrefix();
    } else {
      secondaryTableJoin = null;
      secondaryTableJoinPrefix = null;
    }
    fetchEager = deploy.isFetchEager();
    isTransient = deploy.isTransient();
    nullable = deploy.isNullable();
    unique = deploy.isUnique();
    dbLength = deploy.getDbLength();
    dbScale = deploy.getDbScale();
    dbColumnDefn = InternString.intern(deploy.getDbColumnDefn());
    dbConstraintExpression = InternString.intern(deploy.getDbConstraintExpression());
    
    inherited = false;
    owningType = deploy.getOwningType();
    local = deploy.isLocal();
    
    version = deploy.isVersionColumn();
    embedded = deploy.isEmbedded();
    id = deploy.isId();
    generatedProperty = deploy.getGeneratedProperty();
    readMethod = deploy.getReadMethod();
    writeMethod = deploy.getWriteMethod();
    getter = deploy.getGetter();
    if ((descriptor != null) && (getter == null) && 
      (!unidirectionalShadow)) {
      String m = "Null Getter for: " + getFullBeanName();
      throw new RuntimeException(m);
    }
    
    setter = deploy.getSetter();
    
    dbColumn = tableAliasIntern(descriptor, deploy.getDbColumn(), false, null);
    sqlFormulaJoin = InternString.intern(deploy.getSqlFormulaJoin());
    sqlFormulaSelect = InternString.intern(deploy.getSqlFormulaSelect());
    formula = (sqlFormulaSelect != null);
    
    extraAttributeMap = deploy.getExtraAttributeMap();
    defaultValue = deploy.getDefaultValue();
    dbType = deploy.getDbType();
    scalarType = deploy.getScalarType();
    ldapAttributeAdapter = deploy.getLdapAttributeAdapter();
    lob = isLobType(dbType);
    propertyType = deploy.getPropertyType();
    field = deploy.getField();
    validators = deploy.getValidators();
    hasLocalValidators = (validators.length > 0);
    
    BeanDescriptor.EntityType et = descriptor == null ? null : descriptor.getEntityType();
    elPlaceHolder = tableAliasIntern(descriptor, deploy.getElPlaceHolder(et), false, null);
    elPlaceHolderEncrypted = tableAliasIntern(descriptor, deploy.getElPlaceHolder(et), dbEncrypted, dbColumn);
  }
  
  private String tableAliasIntern(BeanDescriptor<?> descriptor, String s, boolean dbEncrypted, String dbColumn) {
    if (descriptor != null) {
      s = StringHelper.replaceString(s, "${ta}.", "${}");
      s = StringHelper.replaceString(s, "${ta}", "${}");
      
      if (dbEncrypted) {
        s = dbEncryptFunction.getDecryptSql(s);
        String namedParam = ":encryptkey_" + descriptor.getBaseTable() + "___" + dbColumn;
        s = StringHelper.replaceString(s, "?", namedParam);
      }
    }
    return InternString.intern(s);
  }
  







  public BeanProperty(BeanProperty source, BeanPropertyOverride override)
  {
    descriptor = descriptor;
    name = InternString.intern(source.getName());
    dynamicSubclassWithInheritance = dynamicSubclassWithInheritance;
    
    dbColumn = InternString.intern(override.getDbColumn());
    sqlFormulaJoin = InternString.intern(override.getSqlFormulaJoin());
    sqlFormulaSelect = InternString.intern(override.getSqlFormulaSelect());
    formula = (sqlFormulaSelect != null);
    
    fetchEager = fetchEager;
    unidirectionalShadow = unidirectionalShadow;
    localEncrypted = source.isLocalEncrypted();
    isTransient = source.isTransient();
    secondaryTable = source.isSecondaryTable();
    secondaryTableJoin = secondaryTableJoin;
    secondaryTableJoinPrefix = secondaryTableJoinPrefix;
    
    dbBind = source.getDbBind();
    dbEncrypted = source.isDbEncrypted();
    dbEncryptedType = source.getDbEncryptedType();
    dbEncryptFunction = dbEncryptFunction;
    dbRead = source.isDbRead();
    dbInsertable = source.isDbInsertable();
    dbUpdatable = source.isDbUpdatable();
    nullable = source.isNullable();
    unique = source.isUnique();
    dbLength = source.getDbLength();
    dbScale = source.getDbScale();
    dbColumnDefn = InternString.intern(source.getDbColumnDefn());
    dbConstraintExpression = InternString.intern(source.getDbConstraintExpression());
    
    inherited = source.isInherited();
    owningType = owningType;
    local = owningType.equals(descriptor.getBeanType());
    
    version = source.isVersion();
    embedded = source.isEmbedded();
    id = source.isId();
    generatedProperty = source.getGeneratedProperty();
    readMethod = source.getReadMethod();
    writeMethod = source.getWriteMethod();
    getter = getter;
    setter = setter;
    extraAttributeMap = extraAttributeMap;
    defaultValue = source.getDefaultValue();
    dbType = source.getDbType();
    scalarType = scalarType;
    ldapAttributeAdapter = ldapAttributeAdapter;
    lob = isLobType(dbType);
    propertyType = source.getPropertyType();
    field = source.getField();
    validators = source.getValidators();
    hasLocalValidators = (validators.length > 0);
    
    elPlaceHolder = override.replace(elPlaceHolder, dbColumn);
    elPlaceHolderEncrypted = override.replace(elPlaceHolderEncrypted, dbColumn);
  }
  





  public void initialise()
  {
    if ((!isTransient) && (scalarType == null)) {
      String msg = "No ScalarType assigned to " + descriptor.getFullName() + "." + getName();
      throw new RuntimeException(msg);
    }
  }
  


  public int getDeployOrder()
  {
    return deployOrder;
  }
  


  public void setDeployOrder(int deployOrder)
  {
    this.deployOrder = deployOrder;
  }
  
  public ElPropertyValue buildElPropertyValue(String propName, String remainder, ElPropertyChainBuilder chain, boolean propertyDeploy)
  {
    throw new PersistenceException("Not valid on scalar bean property " + getFullBeanName());
  }
  


  public BeanDescriptor<?> getBeanDescriptor()
  {
    return descriptor;
  }
  


  public boolean isScalar()
  {
    return true;
  }
  


  public boolean isFormula()
  {
    return formula;
  }
  
  public boolean hasChanged(Object bean, Object oldValues) {
    Object value = getValue(bean);
    Object oldVal = getValue(oldValues);
    
    return !ValueUtil.areEqual(value, oldVal);
  }
  
  public void copyProperty(Object sourceBean, Object destBean) {
    Object value = getValue(sourceBean);
    setValue(destBean, value);
  }
  
  public void copyProperty(Object sourceBean, Object destBean, CopyContext ctx, int maxDepth) {
    Object value = getValue(sourceBean);
    setValue(destBean, value);
  }
  


  public EncryptKey getEncryptKey()
  {
    return descriptor.getEncryptKey(this);
  }
  
  public String getDecryptProperty() {
    return dbEncryptFunction.getDecryptSql(getName());
  }
  
  public String getDecryptProperty(String propertyName) {
    return dbEncryptFunction.getDecryptSql(propertyName);
  }
  
  public String getDecryptSql() {
    return dbEncryptFunction.getDecryptSql(getDbColumn());
  }
  
  public String getDecryptSql(String tableAlias) {
    return dbEncryptFunction.getDecryptSql(tableAlias + "." + getDbColumn());
  }
  



  public void appendFrom(DbSqlContext ctx, boolean forceOuterJoin)
  {
    if ((formula) && (sqlFormulaJoin != null)) {
      ctx.appendFormulaJoin(sqlFormulaJoin, forceOuterJoin);
    }
    else if (secondaryTableJoin != null)
    {
      String relativePrefix = ctx.getRelativePrefix(secondaryTableJoinPrefix);
      secondaryTableJoin.addJoin(forceOuterJoin, relativePrefix, ctx);
    }
  }
  



  public String getSecondaryTableJoinPrefix()
  {
    return secondaryTableJoinPrefix;
  }
  
  public void appendSelect(DbSqlContext ctx, boolean subQuery) {
    if (formula) {
      ctx.appendFormulaSelect(sqlFormulaSelect);
    }
    else if (!isTransient)
    {
      if (secondaryTableJoin != null) {
        String relativePrefix = ctx.getRelativePrefix(secondaryTableJoinPrefix);
        ctx.pushTableAlias(relativePrefix);
      }
      
      if (dbEncrypted) {
        String decryptSql = getDecryptSql(ctx.peekTableAlias());
        ctx.appendRawColumn(decryptSql);
        ctx.addEncryptedProp(this);
      }
      else {
        ctx.appendColumn(dbColumn);
      }
      
      if (secondaryTableJoin != null) {
        ctx.popTableAlias();
      }
    }
  }
  
  public boolean isAssignableFrom(Class<?> type) {
    return owningType.isAssignableFrom(type);
  }
  
  public Object readSetOwning(DbReadContext ctx, Object bean, Class<?> type) throws SQLException
  {
    try {
      Object value = scalarType.read(ctx.getDataReader());
      if ((value != null) && (bean != null))
      {

        if (owningType.equals(type)) {
          setValue(bean, value);
        }
      }
      return value;
    } catch (Exception e) {
      String msg = "Error readSet on " + descriptor + "." + name;
      throw new PersistenceException(msg, e);
    }
  }
  
  public void loadIgnore(DbReadContext ctx) {
    scalarType.loadIgnore(ctx.getDataReader());
  }
  
  public void load(SqlBeanLoad sqlBeanLoad) throws SQLException {
    sqlBeanLoad.load(this);
  }
  
  public void buildSelectExpressionChain(String prefix, List<String> selectChain) {
    if (prefix == null) {
      selectChain.add(name);
    } else {
      selectChain.add(prefix + "." + name);
    }
  }
  
  public Object read(DbReadContext ctx) throws SQLException {
    return scalarType.read(ctx.getDataReader());
  }
  
  public Object readSet(DbReadContext ctx, Object bean, Class<?> type) throws SQLException
  {
    try {
      Object value = scalarType.read(ctx.getDataReader());
      if ((bean != null) && ((type == null) || (owningType.isAssignableFrom(type))))
      {

        setValue(bean, value);
      }
      return value;
    } catch (Exception e) {
      String msg = "Error readSet on " + descriptor + "." + name;
      throw new PersistenceException(msg, e);
    }
  }
  






  public Object toBeanType(Object value)
  {
    return scalarType.toBeanType(value);
  }
  
  public void bind(DataBind b, Object value) throws SQLException
  {
    scalarType.bind(b, value);
  }
  
  public void writeData(DataOutput dataOutput, Object value) throws IOException {
    scalarType.writeData(dataOutput, value);
  }
  
  public Object readData(DataInput dataInput) throws IOException {
    return scalarType.readData(dataInput);
  }
  
  Validator[] getValidators() {
    return validators;
  }
  
  public boolean isCascadeValidate() {
    return cascadeValidate;
  }
  
  public boolean hasLocalValidators() {
    return hasLocalValidators;
  }
  
  public boolean hasValidationRules(boolean cascade) {
    return (hasLocalValidators) || ((cascade) && (cascadeValidate));
  }
  






  public boolean isValueLoaded(Object value)
  {
    return true;
  }
  


  public InvalidValue validateCascade(Object value)
  {
    return null;
  }
  









  public final List<InvalidValue> validate(boolean cascade, Object value)
  {
    if (!isValueLoaded(value)) {
      return null;
    }
    
    ArrayList<InvalidValue> list = null;
    for (int i = 0; i < validators.length; i++) {
      if (!validators[i].isValid(value)) {
        if (list == null) {
          list = new ArrayList();
        }
        Validator v = validators[i];
        list.add(new InvalidValue(v.getKey(), v.getAttributes(), descriptor.getFullName(), name, value));
      }
    }
    
    if ((list == null) && (cascade) && (cascadeValidate))
    {
      InvalidValue recursive = validateCascade(value);
      if (recursive != null) {
        return InvalidValue.toList(recursive);
      }
    }
    
    return list;
  }
  
  public BeanProperty getBeanProperty() {
    return this;
  }
  


  public Method getReadMethod()
  {
    return readMethod;
  }
  


  public Method getWriteMethod()
  {
    return writeMethod;
  }
  


  public boolean isInherited()
  {
    return inherited;
  }
  


  public boolean isLocal()
  {
    return local;
  }
  
  public Attribute createAttribute(Object bean) {
    Object v = getValue(bean);
    if (v == null) {
      return null;
    }
    if (ldapAttributeAdapter != null) {
      return ldapAttributeAdapter.createAttribute(v);
    }
    Object ldapValue = scalarType.toJdbcType(v);
    return new BasicAttribute(dbColumn, ldapValue);
  }
  
  public void setAttributeValue(Object bean, Attribute attr) {
    try {
      if (attr != null) { Object beanValue;
        Object beanValue;
        if (ldapAttributeAdapter != null) {
          beanValue = ldapAttributeAdapter.readAttribute(attr);
        } else {
          beanValue = scalarType.toBeanType(attr.get());
        }
        
        setValue(bean, beanValue);
      }
    } catch (NamingException e) {
      throw new LdapPersistenceException(e);
    }
  }
  


  public void setValue(Object bean, Object value)
  {
    try
    {
      if ((bean instanceof EntityBean)) {
        setter.set(bean, value);
      } else {
        Object[] args = new Object[1];
        args[0] = value;
        writeMethod.invoke(bean, args);
      }
    } catch (Exception ex) {
      String beanType = bean == null ? "null" : bean.getClass().getName();
      String msg = "set " + name + " on [" + descriptor + "] arg[" + value + "] type[" + beanType + "] threw error";
      
      throw new RuntimeException(msg, ex);
    }
  }
  

  public void setValueIntercept(Object bean, Object value)
  {
    try
    {
      if ((bean instanceof EntityBean)) {
        setter.setIntercept(bean, value);
      } else {
        Object[] args = new Object[1];
        args[0] = value;
        writeMethod.invoke(bean, args);
      }
    } catch (Exception ex) {
      String beanType = bean == null ? "null" : bean.getClass().getName();
      String msg = "setIntercept " + name + " on [" + descriptor + "] arg[" + value + "] type[" + beanType + "] threw error";
      
      throw new RuntimeException(msg, ex);
    }
  }
  
  private static Object[] NO_ARGS = new Object[0];
  
  private ArrayList<LuceneIndex> luceneIndexes;
  
  public Object getValueWithInheritance(Object bean)
  {
    if (dynamicSubclassWithInheritance) {
      return descriptor.getBeanPropertyWithInheritance(bean, name);
    }
    return getValue(bean);
  }
  

  public Object getValue(Object bean)
  {
    try
    {
      if ((bean instanceof EntityBean)) {
        return getter.get(bean);
      }
      return readMethod.invoke(bean, NO_ARGS);
    }
    catch (Exception ex) {
      String beanType = bean == null ? "null" : bean.getClass().getName();
      String msg = "get " + name + " on [" + descriptor + "] type[" + beanType + "] threw error.";
      throw new RuntimeException(msg, ex);
    }
  }
  

  public Object getValueViaReflection(Object bean)
  {
    try
    {
      return readMethod.invoke(bean, NO_ARGS);
    } catch (Exception ex) {
      String beanType = bean == null ? "null" : bean.getClass().getName();
      String msg = "get " + name + " on [" + descriptor + "] type[" + beanType + "] threw error.";
      throw new RuntimeException(msg, ex);
    }
  }
  
  public Object getValueIntercept(Object bean) {
    try {
      if ((bean instanceof EntityBean)) {
        return getter.getIntercept(bean);
      }
      return readMethod.invoke(bean, NO_ARGS);
    }
    catch (Exception ex) {
      String beanType = bean == null ? "null" : bean.getClass().getName();
      String msg = "getIntercept " + name + " on [" + descriptor + "] type[" + beanType + "] threw error.";
      throw new RuntimeException(msg, ex);
    }
  }
  
  public Object elConvertType(Object value) {
    if (value == null) {
      return null;
    }
    return convertToLogicalType(value);
  }
  
  public void elSetReference(Object bean) {
    throw new RuntimeException("Should not be called");
  }
  
  public void elSetValue(Object bean, Object value, boolean populate, boolean reference) {
    if (bean != null) {
      setValueIntercept(bean, value);
    }
  }
  
  public Object elGetValue(Object bean) {
    if (bean == null) {
      return null;
    }
    return getValueIntercept(bean);
  }
  
  public Object elGetReference(Object bean) {
    throw new RuntimeException("Not expected to call this");
  }
  


  public String getName()
  {
    return name;
  }
  
  public String getElName() {
    return name;
  }
  


  public boolean isDeployOnly()
  {
    return false;
  }
  
  public boolean containsManySince(String sinceProperty) {
    return containsMany();
  }
  
  public boolean containsMany() {
    return false;
  }
  
  public Object[] getAssocOneIdValues(Object bean)
  {
    return null;
  }
  
  public String getAssocOneIdExpr(String prefix, String operator)
  {
    return null;
  }
  
  public String getAssocIdInExpr(String prefix)
  {
    return null;
  }
  
  public String getAssocIdInValueExpr(int size)
  {
    return null;
  }
  
  public boolean isAssocId()
  {
    return false;
  }
  
  public boolean isAssocProperty()
  {
    return false;
  }
  
  public String getElPlaceholder(boolean encrypted) {
    return encrypted ? elPlaceHolderEncrypted : elPlaceHolder;
  }
  
  public String getElPrefix() {
    return secondaryTableJoinPrefix;
  }
  


  public String getFullBeanName()
  {
    return descriptor.getFullName() + "." + name;
  }
  


  public ScalarType<?> getScalarType()
  {
    return scalarType;
  }
  
  public StringFormatter getStringFormatter() {
    return scalarType;
  }
  
  public StringParser getStringParser() {
    return scalarType;
  }
  
  public boolean isDateTimeCapable() {
    return (scalarType != null) && (scalarType.isDateTimeCapable());
  }
  
  public int getJdbcType() {
    return scalarType == null ? 0 : scalarType.getJdbcType();
  }
  
  public Object parseDateTime(long systemTimeMillis) {
    return scalarType.parseDateTime(systemTimeMillis);
  }
  


  public int getDbLength()
  {
    return dbLength;
  }
  


  public int getDbScale()
  {
    return dbScale;
  }
  


  public String getDbColumnDefn()
  {
    return dbColumnDefn;
  }
  





  public String getDbConstraintExpression()
  {
    return dbConstraintExpression;
  }
  


  public String renderDbType(DbType dbType)
  {
    if (dbColumnDefn != null) {
      return dbColumnDefn;
    }
    return dbType.renderType(dbLength, dbScale);
  }
  


  public Field getField()
  {
    return field;
  }
  


  public GeneratedProperty getGeneratedProperty()
  {
    return generatedProperty;
  }
  


  public boolean isNullable()
  {
    return nullable;
  }
  



  public boolean isDDLNotNull()
  {
    return (isVersion()) || ((generatedProperty != null) && (generatedProperty.isDDLNotNullable()));
  }
  


  public boolean isUnique()
  {
    return unique;
  }
  


  public boolean isTransient()
  {
    return isTransient;
  }
  


  public boolean isVersion()
  {
    return version;
  }
  
  public String getDeployProperty() {
    return dbColumn;
  }
  


  public String getDbColumn()
  {
    return dbColumn;
  }
  


  public int getDbType()
  {
    return dbType;
  }
  


  public Object convertToLogicalType(Object value)
  {
    if (scalarType != null) {
      return scalarType.toBeanType(value);
    }
    return value;
  }
  

  public void registerLuceneIndex(LuceneIndex luceneIndex)
  {
    if (luceneIndexes == null) {
      luceneIndexes = new ArrayList();
    }
    luceneIndexes.add(luceneIndex);
  }
  
  public boolean isDeltaRequired() {
    return luceneIndexes != null;
  }
  



  public boolean isFetchEager()
  {
    return fetchEager;
  }
  



  public boolean isLob()
  {
    return lob;
  }
  
  private boolean isLobType(int type) {
    switch (type) {
    case 2005: 
      return true;
    case 2004: 
      return true;
    case -4: 
      return true;
    case -1: 
      return true;
    }
    
    return false;
  }
  




  public String getDbBind()
  {
    return dbBind;
  }
  


  public boolean isLocalEncrypted()
  {
    return localEncrypted;
  }
  


  public boolean isDbEncrypted()
  {
    return dbEncrypted;
  }
  
  public int getDbEncryptedType() {
    return dbEncryptedType;
  }
  


  public boolean isDbInsertable()
  {
    return dbInsertable;
  }
  


  public boolean isDbUpdatable()
  {
    return dbUpdatable;
  }
  


  public boolean isDbRead()
  {
    return dbRead;
  }
  



  public boolean isSecondaryTable()
  {
    return secondaryTable;
  }
  


  public Class<?> getPropertyType()
  {
    return propertyType;
  }
  


  public boolean isId()
  {
    return id;
  }
  



  public boolean isEmbedded()
  {
    return embedded;
  }
  


  public String getExtraAttribute(String key)
  {
    return (String)extraAttributeMap.get(key);
  }
  


  public Object getDefaultValue()
  {
    return defaultValue;
  }
  
  public String toString() {
    return name;
  }
  

  public void jsonWrite(WriteJsonContext ctx, Object bean)
  {
    Object value = getValueIntercept(bean);
    if (value == null) {
      ctx.appendNull(name);
    } else {
      String jv = scalarType.jsonToString(value, ctx.getValueAdapter());
      ctx.appendKeyValue(name, jv);
    }
  }
  
  public void jsonRead(ReadJsonContext ctx, Object bean)
  {
    String jsonValue = ctx.readScalarValue();
    Object objValue;
    Object objValue;
    if (jsonValue == null) {
      objValue = null;
    } else {
      objValue = scalarType.jsonFromString(jsonValue, ctx.getValueAdapter());
    }
    setValue(bean, objValue);
  }
}
