package com.avaje.ebeaninternal.server.deploy;

import com.avaje.ebean.bean.EntityBean;
import com.avaje.ebean.bean.EntityBeanIntercept;
import com.avaje.ebeaninternal.server.core.InternString;
import com.avaje.ebeaninternal.server.deploy.id.IdBinder;
import com.avaje.ebeaninternal.server.deploy.id.ImportedId;
import com.avaje.ebeaninternal.server.deploy.id.ImportedIdEmbedded;
import com.avaje.ebeaninternal.server.deploy.id.ImportedIdMultiple;
import com.avaje.ebeaninternal.server.deploy.id.ImportedIdSimple;
import com.avaje.ebeaninternal.server.deploy.meta.DeployBeanPropertyAssoc;
import com.avaje.ebeaninternal.server.el.ElPropertyChainBuilder;
import com.avaje.ebeaninternal.server.el.ElPropertyValue;
import java.util.ArrayList;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.persistence.PersistenceException;























public abstract class BeanPropertyAssoc<T>
  extends BeanProperty
{
  private static final Logger logger = Logger.getLogger(BeanPropertyAssoc.class.getName());
  


  BeanDescriptor<T> targetDescriptor;
  


  IdBinder targetIdBinder;
  


  InheritInfo targetInheritInfo;
  


  String targetIdProperty;
  


  final BeanCascadeInfo cascadeInfo;
  


  final TableJoin tableJoin;
  

  final Class<T> targetType;
  

  final BeanTable beanTable;
  

  final String mappedBy;
  

  final boolean isOuterJoin;
  

  String extraWhere;
  

  boolean saveRecurseSkippable;
  

  boolean deleteRecurseSkippable;
  


  public BeanPropertyAssoc(BeanDescriptorMap owner, BeanDescriptor<?> descriptor, DeployBeanPropertyAssoc<T> deploy)
  {
    super(owner, descriptor, deploy);
    extraWhere = InternString.intern(deploy.getExtraWhere());
    isOuterJoin = deploy.isOuterJoin();
    beanTable = deploy.getBeanTable();
    mappedBy = InternString.intern(deploy.getMappedBy());
    
    tableJoin = new TableJoin(deploy.getTableJoin(), null);
    
    targetType = deploy.getTargetType();
    cascadeInfo = deploy.getCascadeInfo();
  }
  





  public void initialise()
  {
    if (!isTransient) {
      targetDescriptor = descriptor.getBeanDescriptor(targetType);
      targetIdBinder = targetDescriptor.getIdBinder();
      targetInheritInfo = targetDescriptor.getInheritInfo();
      
      saveRecurseSkippable = targetDescriptor.isSaveRecurseSkippable();
      deleteRecurseSkippable = targetDescriptor.isDeleteRecurseSkippable();
      
      cascadeValidate = cascadeInfo.isValidate();
      
      if (!targetIdBinder.isComplexId()) {
        targetIdProperty = targetIdBinder.getIdProperty();
      }
    }
  }
  




  protected ElPropertyValue createElPropertyValue(String propName, String remainder, ElPropertyChainBuilder chain, boolean propertyDeploy)
  {
    BeanDescriptor<?> embDesc = getTargetDescriptor();
    
    if (chain == null) {
      chain = new ElPropertyChainBuilder(isEmbedded(), propName);
    }
    chain.add(this);
    if (containsMany()) {
      chain.setContainsMany(true);
    }
    return embDesc.buildElGetValue(remainder, chain, propertyDeploy);
  }
  


  public boolean addJoin(boolean forceOuterJoin, String prefix, DbSqlContext ctx)
  {
    return tableJoin.addJoin(forceOuterJoin, prefix, ctx);
  }
  


  public boolean addJoin(boolean forceOuterJoin, String a1, String a2, DbSqlContext ctx)
  {
    return tableJoin.addJoin(forceOuterJoin, a1, a2, ctx);
  }
  


  public void addInnerJoin(String a1, String a2, DbSqlContext ctx)
  {
    tableJoin.addInnerJoin(a1, a2, ctx);
  }
  


  public boolean isScalar()
  {
    return false;
  }
  



  public String getMappedBy()
  {
    return mappedBy;
  }
  





  public String getTargetIdProperty()
  {
    return targetIdProperty;
  }
  


  public BeanDescriptor<T> getTargetDescriptor()
  {
    return targetDescriptor;
  }
  
  public boolean isSaveRecurseSkippable(Object bean) {
    if (!saveRecurseSkippable)
    {

      return false;
    }
    if ((bean instanceof EntityBean)) {
      return !((EntityBean)bean)._ebean_getIntercept().isNewOrDirty();
    }
    
    return false;
  }
  









  public boolean isSaveRecurseSkippable()
  {
    return saveRecurseSkippable;
  }
  


  public boolean isDeleteRecurseSkippable()
  {
    return deleteRecurseSkippable;
  }
  



  public boolean hasId(Object bean)
  {
    BeanDescriptor<?> targetDesc = getTargetDescriptor();
    
    BeanProperty[] uids = targetDesc.propertiesId();
    for (int i = 0; i < uids.length; i++)
    {
      Object value = uids[i].getValue(bean);
      if (value == null) {
        return false;
      }
    }
    
    return true;
  }
  






  public Class<?> getTargetType()
  {
    return targetType;
  }
  



  public String getExtraWhere()
  {
    return extraWhere;
  }
  


  public boolean isOuterJoin()
  {
    return isOuterJoin;
  }
  


  public boolean isUpdateable()
  {
    if (tableJoin.columns().length > 0) {
      return tableJoin.columns()[0].isUpdateable();
    }
    
    return true;
  }
  


  public boolean isInsertable()
  {
    if (tableJoin.columns().length > 0) {
      return tableJoin.columns()[0].isInsertable();
    }
    
    return true;
  }
  


  public TableJoin getTableJoin()
  {
    return tableJoin;
  }
  






  public BeanTable getBeanTable()
  {
    return beanTable;
  }
  


  public BeanCascadeInfo getCascadeInfo()
  {
    return cascadeInfo;
  }
  




  protected ImportedId createImportedId(BeanPropertyAssoc<?> owner, BeanDescriptor<?> target, TableJoin join)
  {
    BeanProperty[] props = target.propertiesId();
    BeanProperty[] others = target.propertiesBaseScalar();
    
    if (descriptor.isSqlSelectBased()) {
      String dbColumn = owner.getDbColumn();
      return new ImportedIdSimple(owner, dbColumn, props[0], 0);
    }
    
    TableJoinColumn[] cols = join.columns();
    
    if (props.length == 1) {
      if (!props[0].isEmbedded())
      {
        if (cols.length != 1) {
          String msg = "No Imported Id column for [" + props[0] + "] in table [" + join.getTable() + "]";
          logger.log(Level.SEVERE, msg);
          return null;
        }
        return createImportedScalar(owner, cols[0], props, others);
      }
      

      BeanPropertyAssocOne<?> embProp = (BeanPropertyAssocOne)props[0];
      BeanProperty[] embBaseProps = embProp.getTargetDescriptor().propertiesBaseScalar();
      ImportedIdSimple[] scalars = createImportedList(owner, cols, embBaseProps, others);
      
      return new ImportedIdEmbedded(owner, embProp, scalars);
    }
    


    ImportedIdSimple[] scalars = createImportedList(owner, cols, props, others);
    return new ImportedIdMultiple(owner, scalars);
  }
  

  private ImportedIdSimple[] createImportedList(BeanPropertyAssoc<?> owner, TableJoinColumn[] cols, BeanProperty[] props, BeanProperty[] others)
  {
    ArrayList<ImportedIdSimple> list = new ArrayList();
    
    for (int i = 0; i < cols.length; i++) {
      list.add(createImportedScalar(owner, cols[i], props, others));
    }
    
    return ImportedIdSimple.sort(list);
  }
  
  private ImportedIdSimple createImportedScalar(BeanPropertyAssoc<?> owner, TableJoinColumn col, BeanProperty[] props, BeanProperty[] others)
  {
    String matchColumn = col.getForeignDbColumn();
    String localColumn = col.getLocalDbColumn();
    
    for (int j = 0; j < props.length; j++) {
      if (props[j].getDbColumn().equalsIgnoreCase(matchColumn)) {
        return new ImportedIdSimple(owner, localColumn, props[j], j);
      }
    }
    
    for (int j = 0; j < others.length; j++) {
      if (others[j].getDbColumn().equalsIgnoreCase(matchColumn)) {
        return new ImportedIdSimple(owner, localColumn, others[j], j + props.length);
      }
    }
    
    String msg = "Error with the Join on [" + getFullBeanName() + "]. Could not find the local match for [" + matchColumn + "] " + " Perhaps an error in a @JoinColumn";
    

    throw new PersistenceException(msg);
  }
}
