package com.avaje.ebeaninternal.server.deploy;

import com.avaje.ebean.EbeanServer;
import com.avaje.ebean.ExpressionList;
import com.avaje.ebean.InvalidValue;
import com.avaje.ebean.Query;
import com.avaje.ebean.SqlUpdate;
import com.avaje.ebean.Transaction;
import com.avaje.ebean.bean.EntityBean;
import com.avaje.ebean.bean.EntityBeanIntercept;
import com.avaje.ebean.bean.PersistenceContext;
import com.avaje.ebeaninternal.server.core.DefaultSqlUpdate;
import com.avaje.ebeaninternal.server.core.ReferenceOptions;
import com.avaje.ebeaninternal.server.deploy.id.IdBinder;
import com.avaje.ebeaninternal.server.deploy.id.ImportedId;
import com.avaje.ebeaninternal.server.deploy.meta.DeployBeanPropertyAssocOne;
import com.avaje.ebeaninternal.server.el.ElPropertyChainBuilder;
import com.avaje.ebeaninternal.server.el.ElPropertyValue;
import com.avaje.ebeaninternal.server.query.SplitName;
import com.avaje.ebeaninternal.server.query.SqlBeanLoad;
import com.avaje.ebeaninternal.server.text.json.ReadJsonContext;
import com.avaje.ebeaninternal.server.text.json.WriteJsonContext;
import com.avaje.ebeaninternal.server.type.DataReader;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import javax.persistence.PersistenceException;



































public class BeanPropertyAssocOne<T>
  extends BeanPropertyAssoc<T>
{
  private final boolean oneToOne;
  private final boolean oneToOneExported;
  private final boolean embeddedVersion;
  private final boolean importedPrimaryKey;
  private final BeanPropertyAssocOne<T>.LocalHelp localHelp;
  private final BeanProperty[] embeddedProps;
  private final HashMap<String, BeanProperty> embeddedPropsMap;
  private ImportedId importedId;
  private ExportedProperty[] exportedProperties;
  private String deleteByParentIdSql;
  private String deleteByParentIdInSql;
  
  public BeanPropertyAssocOne(BeanDescriptorMap owner, DeployBeanPropertyAssocOne<T> deploy)
  {
    this(owner, null, deploy);
  }
  




  public BeanPropertyAssocOne(BeanDescriptorMap owner, BeanDescriptor<?> descriptor, DeployBeanPropertyAssocOne<T> deploy)
  {
    super(owner, descriptor, deploy);
    
    importedPrimaryKey = deploy.isImportedPrimaryKey();
    oneToOne = deploy.isOneToOne();
    oneToOneExported = deploy.isOneToOneExported();
    
    if (embedded)
    {
      BeanEmbeddedMeta overrideMeta = BeanEmbeddedMetaFactory.create(owner, deploy, descriptor);
      embeddedProps = overrideMeta.getProperties();
      if (id) {
        embeddedVersion = false;
      } else {
        embeddedVersion = overrideMeta.isEmbeddedVersion();
      }
      embeddedPropsMap = new HashMap();
      for (int i = 0; i < embeddedProps.length; i++) {
        embeddedPropsMap.put(embeddedProps[i].getName(), embeddedProps[i]);
      }
    }
    else {
      embeddedProps = null;
      embeddedPropsMap = null;
      embeddedVersion = false;
    }
    localHelp = createHelp(embedded, oneToOneExported);
  }
  
  public void initialise()
  {
    super.initialise();
    if ((!isTransient) && 
      (!embedded))
    {
      if (!oneToOneExported) {
        importedId = createImportedId(this, targetDescriptor, tableJoin);
      } else {
        exportedProperties = createExported();
        
        String delStmt = "delete from " + targetDescriptor.getBaseTable() + " where ";
        
        deleteByParentIdSql = (delStmt + deriveWhereParentIdSql(false));
        deleteByParentIdInSql = (delStmt + deriveWhereParentIdSql(true));
      }
    }
  }
  



  public ElPropertyValue buildElPropertyValue(String propName, String remainder, ElPropertyChainBuilder chain, boolean propertyDeploy)
  {
    if (embedded) {
      BeanProperty embProp = (BeanProperty)embeddedPropsMap.get(remainder);
      if (embProp == null) {
        String msg = "Embedded Property " + remainder + " not found in " + getFullBeanName();
        throw new PersistenceException(msg);
      }
      if (chain == null) {
        chain = new ElPropertyChainBuilder(true, propName);
      }
      chain.add(this);
      return chain.add(embProp).build();
    }
    
    return createElPropertyValue(propName, remainder, chain, propertyDeploy);
  }
  
  public String getElPlaceholder(boolean encrypted)
  {
    return encrypted ? elPlaceHolderEncrypted : elPlaceHolder;
  }
  

  public void copyProperty(Object sourceBean, Object destBean, CopyContext ctx, int maxDepth)
  {
    localHelp.copyProperty(sourceBean, destBean, ctx, maxDepth);
  }
  
  public SqlUpdate deleteByParentId(Object parentId, List<Object> parentIdist) {
    if (parentId != null) {
      return deleteByParentId(parentId);
    }
    return deleteByParentIdList(parentIdist);
  }
  

  private SqlUpdate deleteByParentIdList(List<Object> parentIdist)
  {
    StringBuilder sb = new StringBuilder(100);
    sb.append(deleteByParentIdInSql);
    
    String inClause = targetIdBinder.getIdInValueExpr(parentIdist.size());
    sb.append(inClause);
    
    DefaultSqlUpdate delete = new DefaultSqlUpdate(sb.toString());
    for (int i = 0; i < parentIdist.size(); i++) {
      targetIdBinder.bindId(delete, parentIdist.get(i));
    }
    
    return delete;
  }
  
  private SqlUpdate deleteByParentId(Object parentId)
  {
    DefaultSqlUpdate delete = new DefaultSqlUpdate(deleteByParentIdSql);
    if (exportedProperties.length == 1) {
      delete.addParameter(parentId);
    } else {
      targetDescriptor.getIdBinder().bindId(delete, parentId);
    }
    return delete;
  }
  
  public List<Object> findIdsByParentId(Object parentId, List<Object> parentIdist, Transaction t) {
    if (parentId != null) {
      return findIdsByParentId(parentId, t);
    }
    return findIdsByParentIdList(parentIdist, t);
  }
  

  private List<Object> findIdsByParentId(Object parentId, Transaction t)
  {
    String rawWhere = deriveWhereParentIdSql(false);
    
    EbeanServer server = getBeanDescriptor().getEbeanServer();
    Query<?> q = server.find(getPropertyType()).where().raw(rawWhere).query();
    

    bindWhereParendId(q, parentId);
    return server.findIds(q, t);
  }
  
  private List<Object> findIdsByParentIdList(List<Object> parentIdist, Transaction t)
  {
    String rawWhere = deriveWhereParentIdSql(true);
    String inClause = targetIdBinder.getIdInValueExpr(parentIdist.size());
    
    String expr = rawWhere + inClause;
    
    EbeanServer server = getBeanDescriptor().getEbeanServer();
    Query<?> q = (Query)server.find(getPropertyType()).where().raw(expr);
    

    for (int i = 0; i < parentIdist.size(); i++) {
      bindWhereParendId(q, parentIdist.get(i));
    }
    
    return server.findIds(q, t);
  }
  
  private void bindWhereParendId(Query<?> q, Object parentId)
  {
    if (exportedProperties.length == 1) {
      q.setParameter(1, parentId);
    }
    else {
      int pos = 1;
      for (int i = 0; i < exportedProperties.length; i++) {
        Object embVal = exportedProperties[i].getValue(parentId);
        q.setParameter(pos++, embVal);
      }
    }
  }
  
  public void addFkey() {
    if (importedId != null) {
      importedId.addFkeys(name);
    }
  }
  
  public boolean isValueLoaded(Object value)
  {
    if ((value instanceof EntityBean)) {
      return ((EntityBean)value)._ebean_getIntercept().isLoaded();
    }
    return true;
  }
  

  public InvalidValue validateCascade(Object value)
  {
    BeanDescriptor<?> target = getTargetDescriptor();
    return target.validate(true, value);
  }
  
  private boolean hasChangedEmbedded(Object bean, Object oldValues)
  {
    Object embValue = getValue(oldValues);
    if ((embValue instanceof EntityBean))
    {
      return ((EntityBean)embValue)._ebean_getIntercept().isNewOrDirty();
    }
    if (embValue == null) {
      return getValue(bean) != null;
    }
    return false;
  }
  

  public boolean hasChanged(Object bean, Object oldValues)
  {
    if (embedded) {
      return hasChangedEmbedded(bean, oldValues);
    }
    Object value = getValue(bean);
    Object oldVal = getValue(oldValues);
    if (oneToOneExported)
    {
      return false;
    }
    if (value == null)
      return oldVal != null;
    if (oldValues == null) {
      return true;
    }
    
    return importedId.hasChanged(value, oldVal);
  }
  




  public BeanProperty[] getProperties()
  {
    return embeddedProps;
  }
  
  public void buildSelectExpressionChain(String prefix, List<String> selectChain)
  {
    prefix = SplitName.add(prefix, name);
    
    if (!embedded) {
      targetIdBinder.buildSelectExpressionChain(prefix, selectChain);
    }
    else {
      for (int i = 0; i < embeddedProps.length; i++) {
        embeddedProps[i].buildSelectExpressionChain(prefix, selectChain);
      }
    }
  }
  



  public boolean isOneToOne()
  {
    return oneToOne;
  }
  


  public boolean isOneToOneExported()
  {
    return oneToOneExported;
  }
  


  public boolean isEmbeddedVersion()
  {
    return embeddedVersion;
  }
  


  public boolean isImportedPrimaryKey()
  {
    return importedPrimaryKey;
  }
  



  public Class<?> getTargetType()
  {
    return getPropertyType();
  }
  



  public Object[] getAssocOneIdValues(Object bean)
  {
    return targetDescriptor.getIdBinder().getIdValues(bean);
  }
  


  public String getAssocOneIdExpr(String prefix, String operator)
  {
    return targetDescriptor.getIdBinder().getAssocOneIdExpr(prefix, operator);
  }
  



  public String getAssocIdInValueExpr(int size)
  {
    return targetDescriptor.getIdBinder().getIdInValueExpr(size);
  }
  



  public String getAssocIdInExpr(String prefix)
  {
    return targetDescriptor.getIdBinder().getAssocIdInExpr(prefix);
  }
  
  public boolean isAssocId()
  {
    return !embedded;
  }
  
  public boolean isAssocProperty()
  {
    return !embedded;
  }
  




  public Object createEmbeddedId()
  {
    return getTargetDescriptor().createVanillaBean();
  }
  


  public Object createEmptyReference()
  {
    return targetDescriptor.createEntityBean();
  }
  
  public void elSetReference(Object bean) {
    Object value = getValueIntercept(bean);
    if (value != null) {
      ((EntityBean)value)._ebean_getIntercept().setReference();
    }
  }
  
  public Object elGetReference(Object bean)
  {
    Object value = getValueIntercept(bean);
    if (value == null) {
      value = targetDescriptor.createEntityBean();
      setValueIntercept(bean, value);
    }
    return value;
  }
  
  public ImportedId getImportedId() {
    return importedId;
  }
  
  private String deriveWhereParentIdSql(boolean inClause)
  {
    StringBuilder sb = new StringBuilder();
    
    for (int i = 0; i < exportedProperties.length; i++) {
      String fkColumn = exportedProperties[i].getForeignDbColumn();
      if (i > 0) {
        String s = inClause ? "," : " and ";
        sb.append(s);
      }
      sb.append(fkColumn);
      if (!inClause) {
        sb.append("=? ");
      }
    }
    return sb.toString();
  }
  



  private ExportedProperty[] createExported()
  {
    BeanProperty[] uids = descriptor.propertiesId();
    
    ArrayList<ExportedProperty> list = new ArrayList();
    
    if ((uids.length == 1) && (uids[0].isEmbedded()))
    {
      BeanPropertyAssocOne<?> one = (BeanPropertyAssocOne)uids[0];
      BeanDescriptor<?> targetDesc = one.getTargetDescriptor();
      BeanProperty[] emIds = targetDesc.propertiesBaseScalar();
      try {
        for (int i = 0; i < emIds.length; i++) {
          ExportedProperty expProp = findMatch(true, emIds[i]);
          list.add(expProp);
        }
      }
      catch (PersistenceException e) {
        e.printStackTrace();
      }
    }
    else {
      for (int i = 0; i < uids.length; i++) {
        ExportedProperty expProp = findMatch(false, uids[i]);
        list.add(expProp);
      }
    }
    
    return (ExportedProperty[])list.toArray(new ExportedProperty[list.size()]);
  }
  



  private ExportedProperty findMatch(boolean embeddedProp, BeanProperty prop)
  {
    String matchColumn = prop.getDbColumn();
    
    String searchTable = tableJoin.getTable();
    TableJoinColumn[] columns = tableJoin.columns();
    
    for (int i = 0; i < columns.length; i++) {
      String matchTo = columns[i].getLocalDbColumn();
      
      if (matchColumn.equalsIgnoreCase(matchTo)) {
        String foreignCol = columns[i].getForeignDbColumn();
        return new ExportedProperty(embeddedProp, foreignCol, prop);
      }
    }
    
    String msg = "Error with the Join on [" + getFullBeanName() + "]. Could not find the matching foreign key for [" + matchColumn + "] in table[" + searchTable + "]?" + " Perhaps using a @JoinColumn with the name/referencedColumnName attributes swapped?";
    

    throw new PersistenceException(msg);
  }
  

  public void appendSelect(DbSqlContext ctx, boolean subQuery)
  {
    if (!isTransient) {
      localHelp.appendSelect(ctx, subQuery);
    }
  }
  
  public void appendFrom(DbSqlContext ctx, boolean forceOuterJoin)
  {
    if (!isTransient) {
      localHelp.appendFrom(ctx, forceOuterJoin);
    }
  }
  
  public Object readSet(DbReadContext ctx, Object bean, Class<?> type) throws SQLException
  {
    boolean assignable = (type == null) || (owningType.isAssignableFrom(type));
    return localHelp.readSet(ctx, bean, assignable);
  }
  




  public Object read(DbReadContext ctx)
    throws SQLException
  {
    return localHelp.read(ctx);
  }
  
  public void loadIgnore(DbReadContext ctx)
  {
    localHelp.loadIgnore(ctx);
  }
  
  public void load(SqlBeanLoad sqlBeanLoad) throws SQLException
  {
    Object dbVal = sqlBeanLoad.load(this);
    if ((embedded) && (sqlBeanLoad.isLazyLoad()) && 
      ((dbVal instanceof EntityBean))) {
      ((EntityBean)dbVal)._ebean_getIntercept().setLoaded();
    }
  }
  
  private BeanPropertyAssocOne<T>.LocalHelp createHelp(boolean embedded, boolean oneToOneExported)
  {
    if (embedded)
      return new Embedded(null);
    if (oneToOneExported) {
      return new ReferenceExported(null);
    }
    return new Reference(this);
  }
  
  private abstract class LocalHelp
  {
    private LocalHelp() {}
    
    abstract void copyProperty(Object paramObject1, Object paramObject2, CopyContext paramCopyContext, int paramInt);
    
    abstract void loadIgnore(DbReadContext paramDbReadContext);
    
    abstract Object read(DbReadContext paramDbReadContext)
      throws SQLException;
    
    abstract Object readSet(DbReadContext paramDbReadContext, Object paramObject, boolean paramBoolean)
      throws SQLException;
    
    abstract void appendSelect(DbSqlContext paramDbSqlContext, boolean paramBoolean);
    
    abstract void appendFrom(DbSqlContext paramDbSqlContext, boolean paramBoolean);
    
    void copy(Object sourceBean, Object destBean, CopyContext ctx, int maxDepth)
    {
      Object value = getValue(sourceBean);
      if (value != null) {
        Class<?> valueClass = value.getClass();
        BeanDescriptor<?> refDesc = descriptor.getBeanDescriptor(valueClass);
        Object refId = refDesc.getId(value);
        Object destRef = ctx.get(valueClass, refId);
        if (destRef == null)
        {
          if (maxDepth > 1)
          {
            destRef = refDesc.createCopy(value, ctx, maxDepth - 1);
          }
          else
            destRef = refDesc.createReference(ctx.isVanillaMode(), refId, destBean, null);
        }
        setValue(destBean, destRef);
      }
    }
  }
  
  private final class Embedded extends BeanPropertyAssocOne.LocalHelp { private Embedded() { super(null); }
    
    void copyProperty(Object sourceBean, Object destBean, CopyContext ctx, int maxDepth) {
      Object srcEmb = getValue(sourceBean);
      if (srcEmb != null) {
        Object dstEmb = targetDescriptor.createBean(ctx.isVanillaMode());
        for (int i = 0; i < embeddedProps.length; i++) {
          embeddedProps[i].copyProperty(srcEmb, dstEmb, ctx, maxDepth);
        }
        setValue(destBean, dstEmb);
      }
    }
    
    void loadIgnore(DbReadContext ctx) {
      for (int i = 0; i < embeddedProps.length; i++) {
        embeddedProps[i].loadIgnore(ctx);
      }
    }
    
    Object readSet(DbReadContext ctx, Object bean, boolean assignable) throws SQLException
    {
      Object dbVal = read(ctx);
      if ((bean != null) && (assignable))
      {
        setValue(bean, dbVal);
        ctx.propagateState(dbVal);
        return dbVal;
      }
      
      return null;
    }
    
    Object read(DbReadContext ctx)
      throws SQLException
    {
      EntityBean embeddedBean = targetDescriptor.createEntityBean();
      
      boolean notNull = false;
      for (int i = 0; i < embeddedProps.length; i++) {
        Object value = embeddedProps[i].readSet(ctx, embeddedBean, null);
        if (value != null) {
          notNull = true;
        }
      }
      if (notNull) {
        ctx.propagateState(embeddedBean);
        return embeddedBean;
      }
      return null;
    }
    


    void appendFrom(DbSqlContext ctx, boolean forceOuterJoin) {}
    

    void appendSelect(DbSqlContext ctx, boolean subQuery)
    {
      for (int i = 0; i < embeddedProps.length; i++) {
        embeddedProps[i].appendSelect(ctx, subQuery);
      }
    }
  }
  
  private final class Reference
    extends BeanPropertyAssocOne.LocalHelp
  {
    private final BeanPropertyAssocOne<?> beanProp;
    
    Reference()
    {
      super(null);
      this.beanProp = beanProp;
    }
    
    void copyProperty(Object sourceBean, Object destBean, CopyContext ctx, int maxDepth) {
      copy(sourceBean, destBean, ctx, maxDepth);
    }
    
    void loadIgnore(DbReadContext ctx) {
      targetIdBinder.loadIgnore(ctx);
      if (targetInheritInfo != null) {
        ctx.getDataReader().incrementPos(1);
      }
    }
    
    Object readSet(DbReadContext ctx, Object bean, boolean assignable) throws SQLException {
      Object val = read(ctx);
      if ((bean != null) && (assignable)) {
        setValue(bean, val);
        ctx.propagateState(val);
      }
      return val;
    }
    



    Object read(DbReadContext ctx)
      throws SQLException
    {
      BeanDescriptor<?> rowDescriptor = null;
      Class<?> rowType = targetType;
      if (targetInheritInfo != null)
      {
        InheritInfo rowInheritInfo = targetInheritInfo.readType(ctx);
        if (rowInheritInfo != null) {
          rowType = rowInheritInfo.getType();
          rowDescriptor = rowInheritInfo.getBeanDescriptor();
        }
      }
      

      Object id = targetIdBinder.read(ctx);
      if (id == null) {
        return null;
      }
      

      Object existing = ctx.getPersistenceContext().get(rowType, id);
      
      if (existing != null) {
        return existing;
      }
      

      Object parent = null;
      Object ref = null;
      
      boolean vanillaMode = ctx.isVanillaMode();
      int parentState = ctx.getParentState();
      
      ReferenceOptions options = ctx.getReferenceOptionsFor(beanProp);
      if ((options != null) && (options.isUseCache())) {
        ref = targetDescriptor.cacheGet(id);
        if (ref != null) {
          if (vanillaMode) {
            ref = targetDescriptor.createCopy(ref, new CopyContext(true), 5);
          }
          else if (parentState == 1) {
            ref = targetDescriptor.createCopy(ref, new CopyContext(false), 5);
          }
          else if ((parentState == 0) && (!options.isReadOnly())) {
            ref = targetDescriptor.createCopy(ref, new CopyContext(false), 5);
          }
        }
      }
      
      boolean createReference = false;
      if (ref == null)
      {
        createReference = true;
        if (targetInheritInfo != null)
        {
          ref = rowDescriptor.createReference(vanillaMode, id, parent, options);
        } else {
          ref = targetDescriptor.createReference(vanillaMode, id, parent, options);
        }
      }
      
      Object existingBean = ctx.getPersistenceContext().putIfAbsent(id, ref);
      if (existingBean != null)
      {


        ref = existingBean;
        createReference = false;
      }
      
      if (!vanillaMode) {
        EntityBeanIntercept ebi = ((EntityBean)ref)._ebean_getIntercept();
        
        if (createReference) {
          if (parentState != 0) {
            ebi.setState(parentState);
          }
          ctx.register(name, ebi);
        }
      }
      
      return ref;
    }
    

    void appendFrom(DbSqlContext ctx, boolean forceOuterJoin)
    {
      if (targetInheritInfo != null)
      {
        String relativePrefix = ctx.getRelativePrefix(name);
        tableJoin.addJoin(forceOuterJoin, relativePrefix, ctx);
      }
    }
    




    void appendSelect(DbSqlContext ctx, boolean subQuery)
    {
      if ((!subQuery) && (targetInheritInfo != null))
      {
        String relativePrefix = ctx.getRelativePrefix(getName());
        String tableAlias = ctx.getTableAlias(relativePrefix);
        ctx.appendColumn(tableAlias, targetInheritInfo.getDiscriminatorColumn());
      }
      importedId.sqlAppend(ctx);
    }
  }
  
  private final class ReferenceExported extends BeanPropertyAssocOne.LocalHelp
  {
    private ReferenceExported() {
      super(null);
    }
    
    void copyProperty(Object sourceBean, Object destBean, CopyContext ctx, int maxDepth) { copy(sourceBean, destBean, ctx, maxDepth); }
    

    void loadIgnore(DbReadContext ctx)
    {
      targetDescriptor.getIdBinder().loadIgnore(ctx);
    }
    



    Object readSet(DbReadContext ctx, Object bean, boolean assignable)
      throws SQLException
    {
      Object dbVal = read(ctx);
      if ((bean != null) && (assignable)) {
        setValue(bean, dbVal);
        ctx.propagateState(dbVal);
      }
      return dbVal;
    }
    

    Object read(DbReadContext ctx)
      throws SQLException
    {
      IdBinder idBinder = targetDescriptor.getIdBinder();
      Object id = idBinder.read(ctx);
      if (id == null) {
        return null;
      }
      
      PersistenceContext persistCtx = ctx.getPersistenceContext();
      Object existing = persistCtx.get(targetType, id);
      
      if (existing != null) {
        return existing;
      }
      boolean vanillaMode = ctx.isVanillaMode();
      Object parent = null;
      Object ref = targetDescriptor.createReference(vanillaMode, id, parent, null);
      
      if (!vanillaMode) {
        EntityBeanIntercept ebi = ((EntityBean)ref)._ebean_getIntercept();
        if (ctx.getParentState() != 0) {
          ebi.setState(ctx.getParentState());
        }
        persistCtx.put(id, ref);
        ctx.register(name, ebi);
      }
      return ref;
    }
    







    void appendSelect(DbSqlContext ctx, boolean subQuery)
    {
      String relativePrefix = ctx.getRelativePrefix(getName());
      ctx.pushTableAlias(relativePrefix);
      
      IdBinder idBinder = targetDescriptor.getIdBinder();
      idBinder.appendSelect(ctx, subQuery);
      
      ctx.popTableAlias();
    }
    

    void appendFrom(DbSqlContext ctx, boolean forceOuterJoin)
    {
      String relativePrefix = ctx.getRelativePrefix(getName());
      tableJoin.addJoin(forceOuterJoin, relativePrefix, ctx);
    }
  }
  

  public void jsonWrite(WriteJsonContext ctx, Object bean)
  {
    Object value = getValueIntercept(bean);
    if (value == null) {
      ctx.beginAssocOneIsNull(name);

    }
    else if (!ctx.isParentBean(value))
    {


      ctx.pushParentBean(bean);
      ctx.beginAssocOne(name);
      BeanDescriptor<?> refDesc = descriptor.getBeanDescriptor(value.getClass());
      refDesc.jsonWrite(ctx, value);
      ctx.endAssocOne();
      ctx.popParentBean();
    }
  }
  


  public void jsonRead(ReadJsonContext ctx, Object bean)
  {
    T assocBean = targetDescriptor.jsonReadBean(ctx, name);
    setValue(bean, assocBean);
  }
}
