package com.avaje.ebeaninternal.server.deploy;

import com.avaje.ebean.bean.EntityBean;
import com.avaje.ebean.bean.EntityBeanIntercept;
import java.util.HashSet;
import java.util.Set;
































public class BeanRefreshFromCacheHelp
{
  private final BeanDescriptor<?> desc;
  private final EntityBeanIntercept ebi;
  private final EntityBean bean;
  private final Object cacheBean;
  private final Object originalOldValues;
  private final boolean isLazyLoad;
  private final boolean readOnly;
  private final boolean sharedInstance;
  private final int parentState;
  private final Set<String> excludes;
  private final Set<String> cacheBeanLoadedProps;
  private final Set<String> loadedProps;
  private final boolean setOriginalOldValues;
  
  public BeanRefreshFromCacheHelp(BeanDescriptor<?> desc, EntityBeanIntercept ebi, Object cacheBean, boolean isLazyLoad)
  {
    this.desc = desc;
    this.ebi = ebi;
    bean = ebi.getOwner();
    this.cacheBean = cacheBean;
    cacheBeanLoadedProps = ((EntityBean)cacheBean)._ebean_getIntercept().getLoadedProps();
    
    if (cacheBeanLoadedProps != null) {
      loadedProps = new HashSet();
    } else {
      loadedProps = null;
    }
    
    this.isLazyLoad = isLazyLoad;
    readOnly = ebi.isReadOnly();
    sharedInstance = ebi.isSharedInstance();
    if (sharedInstance) {
      parentState = 1;
    } else if (readOnly) {
      parentState = 2;
    } else {
      parentState = 0;
    }
    
    excludes = (isLazyLoad ? ebi.getLoadedProps() : null);
    if (excludes != null)
    {


      originalOldValues = ebi.getOldValues();
    } else {
      originalOldValues = null;
    }
    setOriginalOldValues = (originalOldValues != null);
  }
  
  private boolean includeProperty(BeanProperty prop) {
    String name = prop.getName();
    if ((excludes != null) && (excludes.contains(name)))
    {
      return false;
    }
    if ((cacheBeanLoadedProps != null) && (!cacheBeanLoadedProps.contains(name))) {
      return false;
    }
    if (loadedProps != null) {
      loadedProps.add(name);
    }
    return true;
  }
  
  private void propagateParentState(Object bean) {
    if ((bean != null) && (parentState > 0)) {
      ((EntityBean)bean)._ebean_getIntercept().setState(parentState);
    }
  }
  






  public void refresh()
  {
    ebi.setIntercepting(false);
    
    BeanProperty[] props = desc.propertiesBaseScalar();
    for (int i = 0; i < props.length; i++) {
      BeanProperty prop = props[i];
      if (includeProperty(prop)) {
        Object val = prop.getValue(cacheBean);
        if (isLazyLoad) {
          prop.setValue(bean, val);
        } else {
          prop.setValueIntercept(bean, val);
        }
        if (setOriginalOldValues)
        {
          prop.setValue(originalOldValues, val);
        }
      }
    }
    
    BeanPropertyAssocOne<?>[] ones = desc.propertiesOne();
    for (int i = 0; i < ones.length; i++) {
      BeanPropertyAssocOne<?> prop = ones[i];
      if (includeProperty(prop))
      {
        Object val = prop.getValue(cacheBean);
        if (!sharedInstance)
        {
          val = prop.getTargetDescriptor().createCopyForUpdate(val, false);
        }
        if (isLazyLoad) {
          prop.setValue(bean, val);
        } else {
          prop.setValueIntercept(bean, val);
        }
        if (setOriginalOldValues)
        {
          prop.setValue(originalOldValues, val);
        }
        propagateParentState(val);
      }
    }
    
    refreshEmbedded();
    

    BeanPropertyAssocMany<?>[] manys = desc.propertiesMany();
    for (int i = 0; i < manys.length; i++) {
      BeanPropertyAssocMany<?> prop = manys[i];
      if (includeProperty(prop))
      {
        prop.createReference(bean);
      }
    }
    
    ebi.setLoadedProps(loadedProps);
    

    ebi.setLoaded();
  }
  



  private void refreshEmbedded()
  {
    BeanPropertyAssocOne<?>[] embeds = desc.propertiesEmbedded();
    for (int i = 0; i < embeds.length; i++) {
      BeanPropertyAssocOne<?> prop = embeds[i];
      if (includeProperty(prop))
      {
        Object oEmb = prop.getValue(bean);
        

        Object cacheEmb = prop.getValue(cacheBean);
        
        if (oEmb == null)
        {

          if (cacheEmb == null) {
            prop.setValueIntercept(bean, null);
          }
          else {
            Object copyEmb = prop.getTargetDescriptor().createCopyForUpdate(cacheEmb, false);
            prop.setValueIntercept(bean, copyEmb);
            propagateParentState(copyEmb);
          }
          
        }
        else
        {
          if ((oEmb instanceof EntityBean))
          {

            ((EntityBean)oEmb)._ebean_getIntercept().setIntercepting(false);
          }
          
          BeanProperty[] props = prop.getProperties();
          for (int j = 0; j < props.length; j++) {
            Object v = props[j].getValue(cacheEmb);
            props[j].setValueIntercept(oEmb, v);
          }
        }
      }
    }
  }
}
