package com.avaje.ebeaninternal.server.deploy;

import com.avaje.ebeaninternal.server.core.InternString;
import com.avaje.ebeaninternal.server.deploy.id.IdBinder;
import com.avaje.ebeaninternal.server.deploy.parse.DeployInheritInfo;
import com.avaje.ebeaninternal.server.query.SqlTreeProperties;
import com.avaje.ebeaninternal.server.subclass.SubClassUtil;
import com.avaje.ebeaninternal.server.type.DataReader;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import javax.persistence.PersistenceException;































public class InheritInfo
{
  private final String discriminatorStringValue;
  private final Object discriminatorValue;
  private final String discriminatorColumn;
  private final int discriminatorType;
  private final int discriminatorLength;
  private final String where;
  private final Class<?> type;
  private final ArrayList<InheritInfo> children = new ArrayList();
  

  private final HashMap<String, InheritInfo> discMap;
  

  private final HashMap<String, InheritInfo> typeMap;
  

  private final InheritInfo parent;
  

  private final InheritInfo root;
  

  private BeanDescriptor<?> descriptor;
  

  public InheritInfo(InheritInfo r, InheritInfo parent, DeployInheritInfo deploy)
  {
    this.parent = parent;
    type = deploy.getType();
    discriminatorColumn = InternString.intern(deploy.getDiscriminatorColumn(parent));
    discriminatorValue = deploy.getDiscriminatorObjectValue();
    discriminatorStringValue = deploy.getDiscriminatorStringValue();
    
    discriminatorType = deploy.getDiscriminatorType(parent);
    discriminatorLength = deploy.getDiscriminatorLength(parent);
    where = InternString.intern(deploy.getWhere());
    
    if (r == null)
    {
      root = this;
      discMap = new HashMap();
      typeMap = new HashMap();
      registerWithRoot(this);
    }
    else {
      root = r;
      
      discMap = null;
      typeMap = null;
      root.registerWithRoot(this);
    }
  }
  



  public void visitChildren(InheritInfoVisitor visitor)
  {
    for (int i = 0; i < children.size(); i++) {
      InheritInfo child = (InheritInfo)children.get(i);
      visitor.visit(child);
      child.visitChildren(visitor);
    }
  }
  



  public boolean isSaveRecurseSkippable()
  {
    return root.isNodeSaveRecurseSkippable();
  }
  
  private boolean isNodeSaveRecurseSkippable() {
    if (!descriptor.isSaveRecurseSkippable()) {
      return false;
    }
    for (int i = 0; i < children.size(); i++) {
      InheritInfo child = (InheritInfo)children.get(i);
      if (!child.isNodeSaveRecurseSkippable()) {
        return false;
      }
    }
    return true;
  }
  



  public boolean isDeleteRecurseSkippable()
  {
    return root.isNodeDeleteRecurseSkippable();
  }
  
  private boolean isNodeDeleteRecurseSkippable() {
    if (!descriptor.isDeleteRecurseSkippable()) {
      return false;
    }
    for (int i = 0; i < children.size(); i++) {
      InheritInfo child = (InheritInfo)children.get(i);
      if (!child.isNodeDeleteRecurseSkippable()) {
        return false;
      }
    }
    return true;
  }
  



  public void setDescriptor(BeanDescriptor<?> descriptor)
  {
    this.descriptor = descriptor;
  }
  


  public BeanDescriptor<?> getBeanDescriptor()
  {
    return descriptor;
  }
  



  public BeanProperty findSubTypeProperty(String propertyName)
  {
    BeanProperty prop = null;
    
    int i = 0; for (int x = children.size(); i < x; i++) {
      InheritInfo childInfo = (InheritInfo)children.get(i);
      

      prop = childInfo.getBeanDescriptor().findBeanProperty(propertyName);
      
      if (prop != null) {
        return prop;
      }
    }
    
    return null;
  }
  



  public void addChildrenProperties(SqlTreeProperties selectProps)
  {
    int i = 0; for (int x = children.size(); i < x; i++) {
      InheritInfo childInfo = (InheritInfo)children.get(i);
      selectProps.add(descriptor.propertiesLocal());
      
      childInfo.addChildrenProperties(selectProps);
    }
  }
  


  public InheritInfo readType(DbReadContext ctx)
    throws SQLException
  {
    String discValue = ctx.getDataReader().getString();
    return readType(discValue);
  }
  



  public InheritInfo readType(String discValue)
  {
    if (discValue == null) {
      return null;
    }
    
    InheritInfo typeInfo = root.getType(discValue);
    if (typeInfo == null) {
      String m = "Inheritance type for discriminator value [" + discValue + "] was not found?";
      throw new PersistenceException(m);
    }
    
    return typeInfo;
  }
  



  public InheritInfo readType(Class<?> beanType)
  {
    InheritInfo typeInfo = root.getTypeByClass(beanType);
    if (typeInfo == null) {
      String m = "Inheritance type for bean type [" + beanType.getName() + "] was not found?";
      throw new PersistenceException(m);
    }
    
    return typeInfo;
  }
  


  public Object createBean(boolean vanillaMode)
  {
    return descriptor.createBean(vanillaMode);
  }
  


  public IdBinder getIdBinder()
  {
    return descriptor.getIdBinder();
  }
  


  public Class<?> getType()
  {
    return type;
  }
  





  public InheritInfo getRoot()
  {
    return root;
  }
  


  public InheritInfo getParent()
  {
    return parent;
  }
  


  public boolean isAbstract()
  {
    return discriminatorValue == null;
  }
  


  public boolean isRoot()
  {
    return parent == null;
  }
  


  public InheritInfo getType(String discValue)
  {
    return (InheritInfo)discMap.get(discValue);
  }
  


  private InheritInfo getTypeByClass(Class<?> beanType)
  {
    String clsName = SubClassUtil.getSuperClassName(beanType.getName());
    return (InheritInfo)typeMap.get(clsName);
  }
  
  private void registerWithRoot(InheritInfo info) {
    if (info.getDiscriminatorStringValue() != null) {
      String stringDiscValue = info.getDiscriminatorStringValue();
      discMap.put(stringDiscValue, info);
    }
    String clsName = SubClassUtil.getSuperClassName(info.getType().getName());
    typeMap.put(clsName, info);
  }
  


  public void addChild(InheritInfo childInfo)
  {
    children.add(childInfo);
  }
  



  public String getWhere()
  {
    return where;
  }
  


  public String getDiscriminatorColumn()
  {
    return discriminatorColumn;
  }
  


  public int getDiscriminatorType()
  {
    return discriminatorType;
  }
  



  public int getDiscriminatorLength()
  {
    return discriminatorLength;
  }
  


  public String getDiscriminatorStringValue()
  {
    return discriminatorStringValue;
  }
  
  public Object getDiscriminatorValue() {
    return discriminatorValue;
  }
  
  public String toString() {
    return "InheritInfo[" + type.getName() + "] disc[" + discriminatorStringValue + "]";
  }
}
