package com.avaje.ebeaninternal.server.deploy;

import com.avaje.ebeaninternal.server.core.InternString;
import com.avaje.ebeaninternal.server.deploy.meta.DeployBeanProperty;
import com.avaje.ebeaninternal.server.deploy.meta.DeployTableJoin;
import com.avaje.ebeaninternal.server.deploy.meta.DeployTableJoinColumn;
import com.avaje.ebeaninternal.server.query.SplitName;
import com.avaje.ebeaninternal.server.query.SqlBeanLoad;
import java.sql.SQLException;
import java.util.LinkedHashMap;























































public final class TableJoin
{
  public static final String NEW_LINE = "\n";
  public static final String LEFT_OUTER = "left outer join";
  public static final String JOIN = "join";
  private final boolean importedPrimaryKey;
  private final String table;
  private final String type;
  private final BeanCascadeInfo cascadeInfo;
  private final BeanProperty[] properties;
  private final TableJoinColumn[] columns;
  
  public TableJoin(DeployTableJoin deploy, LinkedHashMap<String, BeanProperty> propMap)
  {
    importedPrimaryKey = deploy.isImportedPrimaryKey();
    table = InternString.intern(deploy.getTable());
    type = InternString.intern(deploy.getType());
    cascadeInfo = deploy.getCascadeInfo();
    
    DeployTableJoinColumn[] deployCols = deploy.columns();
    columns = new TableJoinColumn[deployCols.length];
    for (int i = 0; i < deployCols.length; i++) {
      columns[i] = new TableJoinColumn(deployCols[i]);
    }
    
    DeployBeanProperty[] deployProps = deploy.properties();
    if ((deployProps.length > 0) && (propMap == null)) {
      throw new NullPointerException("propMap is null?");
    }
    
    properties = new BeanProperty[deployProps.length];
    for (int i = 0; i < deployProps.length; i++) {
      BeanProperty prop = (BeanProperty)propMap.get(deployProps[i].getName());
      properties[i] = prop;
    }
  }
  




  public TableJoin createWithAlias(String localAlias, String foreignAlias)
  {
    return new TableJoin(this, localAlias, foreignAlias);
  }
  




  private TableJoin(TableJoin join, String localAlias, String foreignAlias)
  {
    importedPrimaryKey = importedPrimaryKey;
    table = table;
    type = type;
    cascadeInfo = cascadeInfo;
    properties = properties;
    columns = columns;
  }
  
  public String toString()
  {
    StringBuilder sb = new StringBuilder(30);
    sb.append(type).append(" ").append(table).append(" ");
    for (int i = 0; i < columns.length; i++) {
      sb.append(columns[i]).append(" ");
    }
    return sb.toString();
  }
  
  public void appendSelect(DbSqlContext ctx, boolean subQuery) {
    int i = 0; for (int x = properties.length; i < x; i++) {
      properties[i].appendSelect(ctx, subQuery);
    }
  }
  
  public void load(SqlBeanLoad sqlBeanLoad) throws SQLException {
    int i = 0; for (int x = properties.length; i < x; i++) {
      properties[i].load(sqlBeanLoad);
    }
  }
  
  public Object readSet(DbReadContext ctx, Object bean, Class<?> type) throws SQLException {
    int i = 0; for (int x = properties.length; i < x; i++) {
      properties[i].readSet(ctx, bean, type);
    }
    return null;
  }
  


  public boolean isImportedPrimaryKey()
  {
    return importedPrimaryKey;
  }
  


  public BeanCascadeInfo getCascadeInfo()
  {
    return cascadeInfo;
  }
  


  public TableJoinColumn[] columns()
  {
    return columns;
  }
  



  public BeanProperty[] properties()
  {
    return properties;
  }
  


  public String getTable()
  {
    return table;
  }
  


  public String getType()
  {
    return type;
  }
  


  public boolean isOuterJoin()
  {
    return type.equals("left outer join");
  }
  
  public boolean addJoin(boolean forceOuterJoin, String prefix, DbSqlContext ctx)
  {
    String[] names = SplitName.split(prefix);
    String a1 = ctx.getTableAlias(names[0]);
    String a2 = ctx.getTableAlias(prefix);
    
    return addJoin(forceOuterJoin, a1, a2, ctx);
  }
  
  public boolean addJoin(boolean forceOuterJoin, String a1, String a2, DbSqlContext ctx)
  {
    ctx.addJoin(forceOuterJoin ? "left outer join" : type, table, columns(), a1, a2);
    
    return (forceOuterJoin) || ("left outer join".equals(type));
  }
  


  public void addInnerJoin(String a1, String a2, DbSqlContext ctx)
  {
    ctx.addJoin("join", table, columns(), a1, a2);
  }
}
