package com.avaje.ebeaninternal.server.deploy.meta;

import com.avaje.ebean.Query.UseIndex;
import com.avaje.ebean.config.TableName;
import com.avaje.ebean.config.dbplatform.IdGenerator;
import com.avaje.ebean.config.dbplatform.IdType;
import com.avaje.ebean.config.lucene.IndexDefn;
import com.avaje.ebean.event.BeanFinder;
import com.avaje.ebean.event.BeanPersistController;
import com.avaje.ebean.event.BeanPersistListener;
import com.avaje.ebean.event.BeanQueryAdapter;
import com.avaje.ebean.meta.MetaAutoFetchStatistic;
import com.avaje.ebeaninternal.server.core.ConcurrencyMode;
import com.avaje.ebeaninternal.server.core.ReferenceOptions;
import com.avaje.ebeaninternal.server.deploy.BeanDescriptor.EntityType;
import com.avaje.ebeaninternal.server.deploy.ChainedBeanPersistController;
import com.avaje.ebeaninternal.server.deploy.ChainedBeanPersistListener;
import com.avaje.ebeaninternal.server.deploy.ChainedBeanQueryAdapter;
import com.avaje.ebeaninternal.server.deploy.CompoundUniqueContraint;
import com.avaje.ebeaninternal.server.deploy.DRawSqlMeta;
import com.avaje.ebeaninternal.server.deploy.DeployNamedQuery;
import com.avaje.ebeaninternal.server.deploy.DeployNamedUpdate;
import com.avaje.ebeaninternal.server.deploy.InheritInfo;
import com.avaje.ebeaninternal.server.reflect.BeanReflect;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;























public class DeployBeanDescriptor<T>
{
  static class PropOrder
    implements Comparator<DeployBeanProperty>
  {
    public int compare(DeployBeanProperty o1, DeployBeanProperty o2)
    {
      int v2 = o1.getSortOrder();
      int v1 = o2.getSortOrder();
      return v1 == v2 ? 0 : v1 < v2 ? -1 : 1;
    } }
  
  private static final PropOrder PROP_ORDER = new PropOrder();
  
  private static final String I_SCALAOBJECT = "scala.ScalaObject";
  
  private static final Logger logger = Logger.getLogger(DeployBeanDescriptor.class.getName());
  
  private static final String META_BEAN_PREFIX = MetaAutoFetchStatistic.class.getName().substring(0, 20);
  



  private LinkedHashMap<String, DeployBeanProperty> propMap = new LinkedHashMap();
  

  private final Class<T> beanType;
  

  private BeanDescriptor.EntityType entityType;
  

  private final Map<String, DeployNamedQuery> namedQueries = new LinkedHashMap();
  
  private final Map<String, DeployNamedUpdate> namedUpdates = new LinkedHashMap();
  
  private final Map<String, DRawSqlMeta> rawSqlMetas = new LinkedHashMap();
  


  private DeployBeanPropertyAssocOne<?> unidirectional;
  


  private IdType idType;
  


  private String idGeneratorName;
  


  private IdGenerator idGenerator;
  


  private String sequenceName;
  

  private String ldapBaseDn;
  

  private String[] ldapObjectclasses;
  

  private String selectLastInsertedId;
  

  private String lazyFetchIncludes;
  

  private ConcurrencyMode concurrencyMode = ConcurrencyMode.ALL;
  


  private boolean updateChangesOnly;
  


  private String[] dependantTables;
  

  private List<CompoundUniqueContraint> compoundUniqueConstraints;
  

  private HashMap<String, String> extraAttrMap = new HashMap();
  


  private String baseTable;
  


  private TableName baseTableFull;
  


  private BeanReflect beanReflect;
  


  private Class<?> factoryType;
  


  private List<BeanPersistController> persistControllers = new ArrayList();
  private List<BeanPersistListener<T>> persistListeners = new ArrayList();
  private List<BeanQueryAdapter> queryAdapters = new ArrayList();
  


  private ReferenceOptions referenceOptions;
  

  private BeanFinder<T> beanFinder;
  

  private Query.UseIndex useIndex;
  

  private IndexDefn<?> indexDefn;
  

  private ArrayList<DeployTableJoin> tableJoinList = new ArrayList();
  

  private InheritInfo inheritInfo;
  

  private String name;
  

  private boolean processedRawSqlExtend;
  


  public DeployBeanDescriptor(Class<T> beanType)
  {
    this.beanType = beanType;
  }
  


  public boolean isAbstract()
  {
    return Modifier.isAbstract(beanType.getModifiers());
  }
  


  public Query.UseIndex getUseIndex()
  {
    return useIndex;
  }
  


  public void setUseIndex(Query.UseIndex useIndex)
  {
    this.useIndex = useIndex;
  }
  
  public IndexDefn<?> getIndexDefn() {
    return indexDefn;
  }
  
  public void setIndexDefn(IndexDefn<?> indexDefn) {
    this.indexDefn = indexDefn;
  }
  
  public boolean isScalaObject() {
    Class<?>[] interfaces = beanType.getInterfaces();
    for (int i = 0; i < interfaces.length; i++) {
      String iname = interfaces[i].getName();
      if ("scala.ScalaObject".equals(iname)) {
        return true;
      }
    }
    return false;
  }
  
  public Collection<DRawSqlMeta> getRawSqlMeta() {
    if (!processedRawSqlExtend) {
      rawSqlProcessExtend();
      processedRawSqlExtend = true;
    }
    return rawSqlMetas.values();
  }
  




  private void rawSqlProcessExtend()
  {
    for (DRawSqlMeta rawSqlMeta : rawSqlMetas.values()) {
      String extend = rawSqlMeta.getExtend();
      if (extend != null) {
        DRawSqlMeta parentQuery = (DRawSqlMeta)rawSqlMetas.get(extend);
        if (parentQuery == null) {
          throw new RuntimeException("parent query [" + extend + "] not found for sql-select " + rawSqlMeta.getName());
        }
        rawSqlMeta.extend(parentQuery);
      }
    }
  }
  

  public DeployBeanTable createDeployBeanTable()
  {
    DeployBeanTable beanTable = new DeployBeanTable(getBeanType());
    beanTable.setBaseTable(baseTable);
    beanTable.setIdProperties(propertiesId());
    
    return beanTable;
  }
  




  public boolean checkReadAndWriteMethods()
  {
    if (isMeta()) {
      return true;
    }
    boolean missingMethods = false;
    
    Iterator<DeployBeanProperty> it = propMap.values().iterator();
    while (it.hasNext()) {
      DeployBeanProperty prop = (DeployBeanProperty)it.next();
      if (!prop.isTransient()) {
        String m = "";
        if (prop.getReadMethod() == null) {
          m = m + " missing readMethod ";
        }
        if (prop.getWriteMethod() == null) {
          m = m + " missing writeMethod ";
        }
        if (!"".equals(m)) {
          m = m + ". Should it be transient?";
          String msg = "Bean property " + getFullName() + "." + prop.getName() + " has " + m;
          logger.log(Level.SEVERE, msg);
          missingMethods = true;
        }
      }
    }
    return !missingMethods;
  }
  
  public void setEntityType(BeanDescriptor.EntityType entityType) {
    this.entityType = entityType;
  }
  
  public boolean isEmbedded() {
    return BeanDescriptor.EntityType.EMBEDDED.equals(entityType);
  }
  
  public boolean isBaseTableType() {
    BeanDescriptor.EntityType et = getEntityType();
    return BeanDescriptor.EntityType.ORM.equals(et);
  }
  
  public BeanDescriptor.EntityType getEntityType() {
    if (entityType == null) {
      entityType = (isMeta() ? BeanDescriptor.EntityType.META : BeanDescriptor.EntityType.ORM);
    }
    return entityType;
  }
  






  private boolean isMeta()
  {
    return beanType.getName().startsWith(META_BEAN_PREFIX);
  }
  
  public void add(DRawSqlMeta rawSqlMeta) {
    rawSqlMetas.put(rawSqlMeta.getName(), rawSqlMeta);
    if ("default".equals(rawSqlMeta.getName())) {
      setEntityType(BeanDescriptor.EntityType.SQL);
    }
  }
  
  public void add(DeployNamedUpdate namedUpdate) {
    namedUpdates.put(namedUpdate.getName(), namedUpdate);
  }
  
  public void add(DeployNamedQuery namedQuery) {
    namedQueries.put(namedQuery.getName(), namedQuery);
    if ("default".equals(namedQuery.getName())) {
      setEntityType(BeanDescriptor.EntityType.SQL);
    }
  }
  
  public Map<String, DeployNamedQuery> getNamedQueries() {
    return namedQueries;
  }
  
  public Map<String, DeployNamedUpdate> getNamedUpdates() {
    return namedUpdates;
  }
  
  public BeanReflect getBeanReflect() {
    return beanReflect;
  }
  


  public Class<T> getBeanType()
  {
    return beanType;
  }
  


  public Class<?> getFactoryType()
  {
    return factoryType;
  }
  





  public void setFactoryType(Class<?> factoryType)
  {
    this.factoryType = factoryType;
  }
  



  public void setBeanReflect(BeanReflect beanReflect)
  {
    this.beanReflect = beanReflect;
  }
  



  public InheritInfo getInheritInfo()
  {
    return inheritInfo;
  }
  


  public void setInheritInfo(InheritInfo inheritInfo)
  {
    this.inheritInfo = inheritInfo;
  }
  


  public ReferenceOptions getReferenceOptions()
  {
    return referenceOptions;
  }
  


  public void setReferenceOptions(ReferenceOptions referenceOptions)
  {
    this.referenceOptions = referenceOptions;
  }
  
  public DeployBeanPropertyAssocOne<?> getUnidirectional() {
    return unidirectional;
  }
  
  public void setUnidirectional(DeployBeanPropertyAssocOne<?> unidirectional) {
    this.unidirectional = unidirectional;
  }
  


  public ConcurrencyMode getConcurrencyMode()
  {
    return concurrencyMode;
  }
  


  public void setConcurrencyMode(ConcurrencyMode concurrencyMode)
  {
    this.concurrencyMode = concurrencyMode;
  }
  
  public String getLdapBaseDn() {
    return ldapBaseDn;
  }
  
  public void setLdapBaseDn(String ldapBaseDn) {
    this.ldapBaseDn = ldapBaseDn;
  }
  
  public String[] getLdapObjectclasses() {
    return ldapObjectclasses;
  }
  
  public void setLdapObjectclasses(String[] ldapObjectclasses) {
    this.ldapObjectclasses = ldapObjectclasses;
  }
  
  public boolean isUpdateChangesOnly() {
    return updateChangesOnly;
  }
  
  public void setUpdateChangesOnly(boolean updateChangesOnly) {
    this.updateChangesOnly = updateChangesOnly;
  }
  



  public String[] getDependantTables()
  {
    return dependantTables;
  }
  


  public void addCompoundUniqueConstraint(CompoundUniqueContraint c)
  {
    if (compoundUniqueConstraints == null) {
      compoundUniqueConstraints = new ArrayList();
    }
    compoundUniqueConstraints.add(c);
  }
  


  public CompoundUniqueContraint[] getCompoundUniqueConstraints()
  {
    if (compoundUniqueConstraints == null) {
      return null;
    }
    return (CompoundUniqueContraint[])compoundUniqueConstraints.toArray(new CompoundUniqueContraint[compoundUniqueConstraints.size()]);
  }
  




  public void setDependantTables(String[] dependantTables)
  {
    this.dependantTables = dependantTables;
  }
  


  public BeanFinder<T> getBeanFinder()
  {
    return beanFinder;
  }
  



  public void setBeanFinder(BeanFinder<T> beanFinder)
  {
    this.beanFinder = beanFinder;
  }
  


  public BeanPersistController getPersistController()
  {
    if (persistControllers.size() == 0)
      return null;
    if (persistControllers.size() == 1) {
      return (BeanPersistController)persistControllers.get(0);
    }
    return new ChainedBeanPersistController(persistControllers);
  }
  



  public BeanPersistListener<T> getPersistListener()
  {
    if (persistListeners.size() == 0)
      return null;
    if (persistListeners.size() == 1) {
      return (BeanPersistListener)persistListeners.get(0);
    }
    return new ChainedBeanPersistListener(persistListeners);
  }
  
  public BeanQueryAdapter getQueryAdapter()
  {
    if (queryAdapters.size() == 0)
      return null;
    if (queryAdapters.size() == 1) {
      return (BeanQueryAdapter)queryAdapters.get(0);
    }
    return new ChainedBeanQueryAdapter(queryAdapters);
  }
  



  public void addPersistController(BeanPersistController controller)
  {
    persistControllers.add(controller);
  }
  
  public void addPersistListener(BeanPersistListener<T> listener) {
    persistListeners.add(listener);
  }
  
  public void addQueryAdapter(BeanQueryAdapter queryAdapter) {
    queryAdapters.add(queryAdapter);
  }
  






  public boolean isUseIdGenerator()
  {
    return idType == IdType.GENERATOR;
  }
  



  public String getBaseTable()
  {
    return baseTable;
  }
  


  public TableName getBaseTableFull()
  {
    return baseTableFull;
  }
  



  public void setBaseTable(TableName baseTableFull)
  {
    this.baseTableFull = baseTableFull;
    baseTable = (baseTableFull == null ? null : baseTableFull.getQualifiedName());
  }
  
  public void sortProperties()
  {
    ArrayList<DeployBeanProperty> list = new ArrayList();
    list.addAll(propMap.values());
    
    Collections.sort(list, PROP_ORDER);
    
    propMap = new LinkedHashMap(list.size());
    for (int i = 0; i < list.size(); i++) {
      addBeanProperty((DeployBeanProperty)list.get(i));
    }
  }
  


  public DeployBeanProperty addBeanProperty(DeployBeanProperty prop)
  {
    return (DeployBeanProperty)propMap.put(prop.getName(), prop);
  }
  


  public DeployBeanProperty getBeanProperty(String propName)
  {
    return (DeployBeanProperty)propMap.get(propName);
  }
  
  public Map<String, String> getExtraAttributeMap() {
    return extraAttrMap;
  }
  


  public String getExtraAttribute(String key)
  {
    return (String)extraAttrMap.get(key);
  }
  







  public void setExtraAttribute(String key, String value)
  {
    extraAttrMap.put(key, value);
  }
  






  public String getFullName()
  {
    return beanType.getName();
  }
  


  public String getName()
  {
    return name;
  }
  


  public void setName(String name)
  {
    this.name = name;
  }
  



  public IdType getIdType()
  {
    return idType;
  }
  


  public void setIdType(IdType idType)
  {
    this.idType = idType;
  }
  


  public String getSequenceName()
  {
    return sequenceName;
  }
  


  public void setSequenceName(String sequenceName)
  {
    this.sequenceName = sequenceName;
  }
  





  public String getSelectLastInsertedId()
  {
    return selectLastInsertedId;
  }
  


  public void setSelectLastInsertedId(String selectLastInsertedId)
  {
    this.selectLastInsertedId = selectLastInsertedId;
  }
  



  public String getIdGeneratorName()
  {
    return idGeneratorName;
  }
  



  public void setIdGeneratorName(String idGeneratorName)
  {
    this.idGeneratorName = idGeneratorName;
  }
  


  public IdGenerator getIdGenerator()
  {
    return idGenerator;
  }
  


  public void setIdGenerator(IdGenerator idGenerator)
  {
    this.idGenerator = idGenerator;
    if ((idGenerator != null) && (idGenerator.isDbSequence())) {
      setSequenceName(idGenerator.getName());
    }
  }
  


  public String getLazyFetchIncludes()
  {
    return lazyFetchIncludes;
  }
  




  public void setLazyFetchIncludes(String lazyFetchIncludes)
  {
    if ((lazyFetchIncludes != null) && (lazyFetchIncludes.length() > 0)) {
      this.lazyFetchIncludes = lazyFetchIncludes;
    }
  }
  


  public String toString()
  {
    return getFullName();
  }
  


  public void addTableJoin(DeployTableJoin join)
  {
    tableJoinList.add(join);
  }
  
  public List<DeployTableJoin> getTableJoins() {
    return tableJoinList;
  }
  


  public Iterator<DeployBeanProperty> propertiesAll()
  {
    return propMap.values().iterator();
  }
  



  public String getDefaultSelectClause()
  {
    StringBuilder sb = new StringBuilder();
    
    boolean hasLazyFetch = false;
    
    Iterator<DeployBeanProperty> it = propMap.values().iterator();
    while (it.hasNext()) {
      DeployBeanProperty prop = (DeployBeanProperty)it.next();
      if (!prop.isTransient())
      {
        if (!(prop instanceof DeployBeanPropertyAssocMany))
        {

          if (prop.isFetchEager()) {
            sb.append(prop.getName()).append(",");
          } else {
            hasLazyFetch = true;
          }
        }
      }
    }
    if (!hasLazyFetch) {
      return null;
    }
    String selectClause = sb.toString();
    return selectClause.substring(0, selectClause.length() - 1);
  }
  



  public String[] getDefaultSelectDbArray(Set<String> defaultSelect)
  {
    ArrayList<String> list = new ArrayList();
    for (DeployBeanProperty p : propMap.values()) {
      if (defaultSelect != null) {
        if (defaultSelect.contains(p.getName()))
        {
          list.add(p.getDbColumn());
        }
      } else if ((!p.isTransient()) && (p.isDbRead()))
      {
        list.add(p.getDbColumn());
      }
    }
    return (String[])list.toArray(new String[list.size()]);
  }
  



  public Set<String> parseDefaultSelectClause(String rawList)
  {
    if (rawList == null) {
      return null;
    }
    
    String[] res = rawList.split(",");
    
    LinkedHashSet<String> set = new LinkedHashSet(res.length + 3);
    
    String temp = null;
    for (int i = 0; i < res.length; i++) {
      temp = res[i].trim();
      if (temp.length() > 0) {
        set.add(temp);
      }
    }
    return Collections.unmodifiableSet(set);
  }
  



  public String getSinglePrimaryKeyColumn()
  {
    List<DeployBeanProperty> ids = propertiesId();
    if (ids.size() == 1) {
      DeployBeanProperty p = (DeployBeanProperty)ids.get(0);
      if ((p instanceof DeployBeanPropertyAssoc))
      {
        return null;
      }
      return p.getDbColumn();
    }
    
    return null;
  }
  







  public List<DeployBeanProperty> propertiesId()
  {
    ArrayList<DeployBeanProperty> list = new ArrayList(2);
    
    Iterator<DeployBeanProperty> it = propMap.values().iterator();
    while (it.hasNext()) {
      DeployBeanProperty prop = (DeployBeanProperty)it.next();
      if (prop.isId()) {
        list.add(prop);
      }
    }
    
    return list;
  }
  
  public DeployBeanPropertyAssocOne<?> findJoinToTable(String tableName)
  {
    List<DeployBeanPropertyAssocOne<?>> assocOne = propertiesAssocOne();
    for (DeployBeanPropertyAssocOne<?> prop : assocOne) {
      DeployTableJoin tableJoin = prop.getTableJoin();
      if ((tableJoin != null) && (tableJoin.getTable().equalsIgnoreCase(tableName))) {
        return prop;
      }
    }
    return null;
  }
  




  public List<DeployBeanPropertyAssocOne<?>> propertiesAssocOne()
  {
    ArrayList<DeployBeanPropertyAssocOne<?>> list = new ArrayList();
    
    Iterator<DeployBeanProperty> it = propMap.values().iterator();
    while (it.hasNext()) {
      DeployBeanProperty prop = (DeployBeanProperty)it.next();
      if (((prop instanceof DeployBeanPropertyAssocOne)) && 
        (!prop.isEmbedded())) {
        list.add((DeployBeanPropertyAssocOne)prop);
      }
    }
    

    return list;
  }
  




  public List<DeployBeanPropertyAssocMany<?>> propertiesAssocMany()
  {
    ArrayList<DeployBeanPropertyAssocMany<?>> list = new ArrayList();
    
    Iterator<DeployBeanProperty> it = propMap.values().iterator();
    while (it.hasNext()) {
      DeployBeanProperty prop = (DeployBeanProperty)it.next();
      if ((prop instanceof DeployBeanPropertyAssocMany)) {
        list.add((DeployBeanPropertyAssocMany)prop);
      }
    }
    
    return list;
  }
  





  public List<DeployBeanProperty> propertiesVersion()
  {
    ArrayList<DeployBeanProperty> list = new ArrayList();
    
    Iterator<DeployBeanProperty> it = propMap.values().iterator();
    while (it.hasNext()) {
      DeployBeanProperty prop = (DeployBeanProperty)it.next();
      
      if (!(prop instanceof DeployBeanPropertyAssoc))
      {

        if ((!prop.isId()) && (prop.isVersionColumn())) {
          list.add(prop);
        }
      }
    }
    
    return list;
  }
  



  public List<DeployBeanProperty> propertiesBase()
  {
    ArrayList<DeployBeanProperty> list = new ArrayList();
    
    Iterator<DeployBeanProperty> it = propMap.values().iterator();
    while (it.hasNext()) {
      DeployBeanProperty prop = (DeployBeanProperty)it.next();
      
      if (!(prop instanceof DeployBeanPropertyAssoc))
      {

        if (!prop.isId()) {
          list.add(prop);
        }
      }
    }
    
    return list;
  }
}
