package com.avaje.ebeaninternal.server.deploy.meta;

import com.avaje.ebean.annotation.CreatedTimestamp;
import com.avaje.ebean.annotation.UpdatedTimestamp;
import com.avaje.ebean.config.ScalarTypeConverter;
import com.avaje.ebean.config.dbplatform.DbEncrypt;
import com.avaje.ebean.config.dbplatform.DbEncryptFunction;
import com.avaje.ebean.config.ldap.LdapAttributeAdapter;
import com.avaje.ebean.validation.factory.Validator;
import com.avaje.ebeaninternal.server.core.InternString;
import com.avaje.ebeaninternal.server.deploy.BeanDescriptor.EntityType;
import com.avaje.ebeaninternal.server.deploy.generatedproperty.GeneratedProperty;
import com.avaje.ebeaninternal.server.reflect.BeanReflectGetter;
import com.avaje.ebeaninternal.server.reflect.BeanReflectSetter;
import com.avaje.ebeaninternal.server.type.ScalarType;
import com.avaje.ebeaninternal.server.type.ScalarTypeEnum;
import com.avaje.ebeaninternal.server.type.ScalarTypeWrapper;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import javax.persistence.EmbeddedId;
import javax.persistence.FetchType;
import javax.persistence.Id;
import javax.persistence.Version;






























































public class DeployBeanProperty
{
  private static final int ID_ORDER = 1000000;
  private static final int UNIDIRECTIONAL_ORDER = 100000;
  private static final int AUDITCOLUMN_ORDER = -1000000;
  private static final int VERSIONCOLUMN_ORDER = -1000000;
  public static final String EXCLUDE_FROM_UPDATE_WHERE = "EXCLUDE_FROM_UPDATE_WHERE";
  public static final String EXCLUDE_FROM_DELETE_WHERE = "EXCLUDE_FROM_DELETE_WHERE";
  public static final String EXCLUDE_FROM_INSERT = "EXCLUDE_FROM_INSERT";
  public static final String EXCLUDE_FROM_UPDATE = "EXCLUDE_FROM_UPDATE";
  private boolean id;
  private boolean embedded;
  private boolean versionColumn;
  private boolean fetchEager = true;
  



  private boolean nullable = true;
  

  private boolean unique;
  

  private LdapAttributeAdapter ldapAttributeAdapter;
  
  private int dbLength;
  
  private int dbScale;
  
  private String dbColumnDefn;
  
  private boolean isTransient;
  
  private boolean localEncrypted;
  
  private boolean dbEncrypted;
  
  private DbEncryptFunction dbEncryptFunction;
  
  private int dbEncryptedType;
  
  private String dbBind = "?";
  



  private boolean dbRead;
  



  private boolean dbInsertable;
  



  private boolean dbUpdateable;
  



  private DeployTableJoin secondaryTableJoin;
  



  private String secondaryTableJoinPrefix;
  


  private String secondaryTable;
  


  private Class<?> owningType;
  


  private boolean lob;
  


  private String name;
  


  private Field field;
  


  private Class<?> propertyType;
  


  private ScalarType<?> scalarType;
  


  private String dbColumn;
  


  private String sqlFormulaSelect;
  


  private String sqlFormulaJoin;
  


  private int dbType;
  


  private Object defaultValue;
  


  private HashMap<String, String> extraAttributeMap = new HashMap();
  


  private Method readMethod;
  


  private Method writeMethod;
  


  private BeanReflectGetter getter;
  

  private BeanReflectSetter setter;
  

  private GeneratedProperty generatedProperty;
  

  private List<Validator> validators = new ArrayList();
  
  private final DeployBeanDescriptor<?> desc;
  
  private boolean undirectionalShadow;
  private int sortOrder;
  
  public DeployBeanProperty(DeployBeanDescriptor<?> desc, Class<?> propertyType, ScalarType<?> scalarType, ScalarTypeConverter<?, ?> typeConverter)
  {
    this.desc = desc;
    this.propertyType = propertyType;
    this.scalarType = wrapScalarType(propertyType, scalarType, typeConverter);
  }
  



  private ScalarType<?> wrapScalarType(Class<?> propertyType, ScalarType<?> scalarType, ScalarTypeConverter<?, ?> typeConverter)
  {
    if (typeConverter == null) {
      return scalarType;
    }
    return new ScalarTypeWrapper(propertyType, scalarType, typeConverter);
  }
  
  public int getSortOverride() {
    if (field == null) {
      return 0;
    }
    if (field.getAnnotation(Id.class) != null)
      return 1000000;
    if (field.getAnnotation(EmbeddedId.class) != null)
      return 1000000;
    if (undirectionalShadow)
      return 100000;
    if (field.getAnnotation(CreatedTimestamp.class) != null)
      return -1000000;
    if (field.getAnnotation(UpdatedTimestamp.class) != null)
      return -1000000;
    if (field.getAnnotation(Version.class) != null) {
      return -1000000;
    }
    return 0;
  }
  


  public boolean isScalar()
  {
    return true;
  }
  
  public String getFullBeanName() {
    return desc.getFullName() + "." + name;
  }
  





  public boolean isNullablePrimitive()
  {
    if ((nullable) && (propertyType.isPrimitive())) {
      return true;
    }
    return false;
  }
  







  public int getDbLength()
  {
    if ((dbLength == 0) && (scalarType != null)) {
      return scalarType.getLength();
    }
    
    return dbLength;
  }
  


  public int getSortOrder()
  {
    return sortOrder;
  }
  


  public void setSortOrder(int sortOrder)
  {
    this.sortOrder = sortOrder;
  }
  


  public boolean isUndirectionalShadow()
  {
    return undirectionalShadow;
  }
  


  public void setUndirectionalShadow(boolean undirectionalShadow)
  {
    this.undirectionalShadow = undirectionalShadow;
  }
  


  public boolean isLocalEncrypted()
  {
    return localEncrypted;
  }
  


  public void setLocalEncrypted(boolean localEncrypted)
  {
    this.localEncrypted = localEncrypted;
  }
  


  public void setDbLength(int dbLength)
  {
    this.dbLength = dbLength;
  }
  


  public int getDbScale()
  {
    return dbScale;
  }
  


  public void setDbScale(int dbScale)
  {
    this.dbScale = dbScale;
  }
  


  public String getDbColumnDefn()
  {
    return dbColumnDefn;
  }
  


  public void setDbColumnDefn(String dbColumnDefn)
  {
    if ((dbColumnDefn == null) || (dbColumnDefn.trim().length() == 0)) {
      this.dbColumnDefn = null;
    } else {
      this.dbColumnDefn = InternString.intern(dbColumnDefn);
    }
  }
  
  public String getDbConstraintExpression() {
    if ((scalarType instanceof ScalarTypeEnum))
    {
      ScalarTypeEnum etype = (ScalarTypeEnum)scalarType;
      

      return "check (" + dbColumn + " in " + etype.getContraintInValues() + ")";
    }
    return null;
  }
  


  public void addValidator(Validator validator)
  {
    validators.add(validator);
  }
  







  public boolean containsValidatorType(Class<?> type)
  {
    Iterator<Validator> it = validators.iterator();
    while (it.hasNext()) {
      Validator validator = (Validator)it.next();
      if (validator.getClass().equals(type)) {
        return true;
      }
    }
    return false;
  }
  


  public Validator[] getValidators()
  {
    return (Validator[])validators.toArray(new Validator[validators.size()]);
  }
  



  public ScalarType<?> getScalarType()
  {
    return scalarType;
  }
  
  public void setScalarType(ScalarType<?> scalarType) {
    this.scalarType = scalarType;
  }
  
  public BeanReflectGetter getGetter() {
    return getter;
  }
  
  public BeanReflectSetter getSetter() {
    return setter;
  }
  


  public Method getReadMethod()
  {
    return readMethod;
  }
  


  public Method getWriteMethod()
  {
    return writeMethod;
  }
  


  public void setOwningType(Class<?> owningType)
  {
    this.owningType = owningType;
  }
  
  public Class<?> getOwningType() {
    return owningType;
  }
  


  public boolean isLocal()
  {
    return (owningType == null) || (owningType.equals(desc.getBeanType()));
  }
  


  public void setGetter(BeanReflectGetter getter)
  {
    this.getter = getter;
  }
  


  public void setSetter(BeanReflectSetter setter)
  {
    this.setter = setter;
  }
  


  public String getName()
  {
    return name;
  }
  


  public void setName(String name)
  {
    this.name = InternString.intern(name);
  }
  


  public Field getField()
  {
    return field;
  }
  


  public void setField(Field field)
  {
    this.field = field;
  }
  



  public boolean isGenerated()
  {
    return generatedProperty != null;
  }
  


  public GeneratedProperty getGeneratedProperty()
  {
    return generatedProperty;
  }
  


  public void setGeneratedProperty(GeneratedProperty generatedValue)
  {
    generatedProperty = generatedValue;
  }
  


  public boolean isNullable()
  {
    return nullable;
  }
  


  public void setNullable(boolean isNullable)
  {
    nullable = isNullable;
  }
  


  public boolean isUnique()
  {
    return unique;
  }
  


  public void setUnique(boolean unique)
  {
    this.unique = unique;
  }
  


  public LdapAttributeAdapter getLdapAttributeAdapter()
  {
    return ldapAttributeAdapter;
  }
  


  public void setLdapAttributeAdapter(LdapAttributeAdapter ldapAttributeAdapter)
  {
    this.ldapAttributeAdapter = ldapAttributeAdapter;
  }
  


  public boolean isVersionColumn()
  {
    return versionColumn;
  }
  


  public void setVersionColumn(boolean isVersionColumn)
  {
    versionColumn = isVersionColumn;
  }
  


  public boolean isFetchEager()
  {
    return fetchEager;
  }
  


  public void setFetchType(FetchType fetchType)
  {
    fetchEager = FetchType.EAGER.equals(fetchType);
  }
  


  public String getSqlFormulaSelect()
  {
    return sqlFormulaSelect;
  }
  
  public String getSqlFormulaJoin() {
    return sqlFormulaJoin;
  }
  


  public void setSqlFormula(String formulaSelect, String formulaJoin)
  {
    sqlFormulaSelect = formulaSelect;
    sqlFormulaJoin = (formulaJoin.equals("") ? null : formulaJoin);
    dbRead = true;
    dbInsertable = false;
    dbUpdateable = false;
  }
  
  public String getElPlaceHolder(BeanDescriptor.EntityType et) {
    if (sqlFormulaSelect != null)
      return sqlFormulaSelect;
    if (BeanDescriptor.EntityType.LDAP.equals(et)) {
      return getDbColumn();
    }
    if (secondaryTableJoinPrefix != null) {
      return "${" + secondaryTableJoinPrefix + "}" + getDbColumn();
    }
    
    return "${}" + getDbColumn();
  }
  



  public String getDbColumn()
  {
    if (sqlFormulaSelect != null) {
      return sqlFormulaSelect;
    }
    return dbColumn;
  }
  


  public void setDbColumn(String dbColumn)
  {
    this.dbColumn = InternString.intern(dbColumn);
  }
  


  public int getDbType()
  {
    return dbType;
  }
  


  public void setDbType(int dbType)
  {
    this.dbType = dbType;
    lob = isLobType(dbType);
  }
  



  public boolean isLob()
  {
    return lob;
  }
  
  private boolean isLobType(int type) {
    switch (type) {
    case 2005: 
      return true;
    case 2004: 
      return true;
    case -4: 
      return true;
    case -1: 
      return true;
    }
    
    return false;
  }
  



  public boolean isSecondaryTable()
  {
    return secondaryTable != null;
  }
  


  public String getSecondaryTable()
  {
    return secondaryTable;
  }
  


  public void setSecondaryTable(String secondaryTable)
  {
    this.secondaryTable = secondaryTable;
    dbInsertable = false;
    dbUpdateable = false;
  }
  


  public String getSecondaryTableJoinPrefix()
  {
    return secondaryTableJoinPrefix;
  }
  
  public DeployTableJoin getSecondaryTableJoin() {
    return secondaryTableJoin;
  }
  
  public void setSecondaryTableJoin(DeployTableJoin secondaryTableJoin, String prefix) {
    this.secondaryTableJoin = secondaryTableJoin;
    secondaryTableJoinPrefix = prefix;
  }
  



  public String getDbBind()
  {
    return dbBind;
  }
  


  public void setDbBind(String dbBind)
  {
    this.dbBind = dbBind;
  }
  


  public boolean isDbEncrypted()
  {
    return dbEncrypted;
  }
  






  public DbEncryptFunction getDbEncryptFunction()
  {
    return dbEncryptFunction;
  }
  
  public void setDbEncryptFunction(DbEncryptFunction dbEncryptFunction, DbEncrypt dbEncrypt, int dbLen) {
    this.dbEncryptFunction = dbEncryptFunction;
    dbEncrypted = true;
    dbBind = dbEncryptFunction.getEncryptBindSql();
    
    dbEncryptedType = (isLob() ? 2004 : dbEncrypt.getEncryptDbType());
    if (dbLen > 0) {
      setDbLength(dbLen);
    }
  }
  



  public int getDbEncryptedType()
  {
    return dbEncryptedType;
  }
  


  public void setDbEncryptedType(int dbEncryptedType)
  {
    this.dbEncryptedType = dbEncryptedType;
  }
  


  public boolean isDbRead()
  {
    return dbRead;
  }
  


  public void setDbRead(boolean isDBRead)
  {
    dbRead = isDBRead;
  }
  
  public boolean isDbInsertable() {
    return dbInsertable;
  }
  
  public void setDbInsertable(boolean insertable) {
    dbInsertable = insertable;
  }
  
  public boolean isDbUpdateable() {
    return dbUpdateable;
  }
  
  public void setDbUpdateable(boolean updateable) {
    dbUpdateable = updateable;
  }
  


  public boolean isTransient()
  {
    return isTransient;
  }
  


  public void setTransient(boolean isTransient)
  {
    this.isTransient = isTransient;
  }
  






  public void setReadMethod(Method readMethod)
  {
    this.readMethod = readMethod;
  }
  






  public void setWriteMethod(Method writeMethod)
  {
    this.writeMethod = writeMethod;
  }
  


  public Class<?> getPropertyType()
  {
    return propertyType;
  }
  


  public boolean isId()
  {
    return id;
  }
  


  public void setId(boolean id)
  {
    this.id = id;
  }
  



  public boolean isEmbedded()
  {
    return embedded;
  }
  


  public void setEmbedded(boolean embedded)
  {
    this.embedded = embedded;
  }
  
  public Map<String, String> getExtraAttributeMap() {
    return extraAttributeMap;
  }
  


  public String getExtraAttribute(String key)
  {
    return (String)extraAttributeMap.get(key);
  }
  


  public void setExtraAttribute(String key, String value)
  {
    extraAttributeMap.put(key, value);
  }
  


  public Object getDefaultValue()
  {
    return defaultValue;
  }
  


  public void setDefaultValue(Object defaultValue)
  {
    this.defaultValue = defaultValue;
  }
  
  public String toString() {
    return desc.getFullName() + "." + name;
  }
}
