package com.avaje.ebeaninternal.server.deploy.meta;

import com.avaje.ebean.bean.BeanCollection.ModifyListenMode;
import com.avaje.ebean.validation.factory.Validator;
import com.avaje.ebeaninternal.server.deploy.BeanCascadeInfo;
import com.avaje.ebeaninternal.server.deploy.BeanDescriptor;
import com.avaje.ebeaninternal.server.deploy.BeanDescriptorMap;
import com.avaje.ebeaninternal.server.deploy.BeanProperty;
import com.avaje.ebeaninternal.server.deploy.BeanPropertyAssocMany;
import com.avaje.ebeaninternal.server.deploy.BeanPropertyAssocOne;
import com.avaje.ebeaninternal.server.deploy.BeanPropertyCompound;
import com.avaje.ebeaninternal.server.deploy.BeanPropertySimpleCollection;
import com.avaje.ebeaninternal.server.deploy.TableJoin;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;

























public class DeployBeanPropertyLists
{
  private BeanProperty derivedFirstVersionProp;
  private final BeanDescriptor<?> desc;
  private final LinkedHashMap<String, BeanProperty> propertyMap;
  private final ArrayList<BeanProperty> ids = new ArrayList();
  
  private final ArrayList<BeanProperty> version = new ArrayList();
  
  private final ArrayList<BeanProperty> local = new ArrayList();
  
  private final ArrayList<BeanProperty> manys = new ArrayList();
  
  private final ArrayList<BeanProperty> ones = new ArrayList();
  
  private final ArrayList<BeanProperty> onesExported = new ArrayList();
  
  private final ArrayList<BeanProperty> onesImported = new ArrayList();
  
  private final ArrayList<BeanProperty> embedded = new ArrayList();
  
  private final ArrayList<BeanProperty> baseScalar = new ArrayList();
  
  private final ArrayList<BeanPropertyCompound> baseCompound = new ArrayList();
  
  private final ArrayList<BeanProperty> transients = new ArrayList();
  
  private final ArrayList<BeanProperty> nonTransients = new ArrayList();
  
  private final TableJoin[] tableJoins;
  
  private final BeanPropertyAssocOne<?> unidirectional;
  
  public DeployBeanPropertyLists(BeanDescriptorMap owner, BeanDescriptor<?> desc, DeployBeanDescriptor<?> deploy)
  {
    this.desc = desc;
    
    DeployBeanPropertyAssocOne<?> deployUnidirectional = deploy.getUnidirectional();
    if (deployUnidirectional == null) {
      unidirectional = null;
    } else {
      unidirectional = new BeanPropertyAssocOne(owner, desc, deployUnidirectional);
    }
    
    propertyMap = new LinkedHashMap();
    
    Iterator<DeployBeanProperty> deployIt = deploy.propertiesAll();
    while (deployIt.hasNext()) {
      DeployBeanProperty deployProp = (DeployBeanProperty)deployIt.next();
      BeanProperty beanProp = createBeanProperty(owner, deployProp);
      propertyMap.put(beanProp.getName(), beanProp);
    }
    
    Iterator<BeanProperty> it = propertyMap.values().iterator();
    
    int order = 0;
    while (it.hasNext()) {
      BeanProperty prop = (BeanProperty)it.next();
      prop.setDeployOrder(order++);
      allocateToList(prop);
    }
    
    List<DeployTableJoin> deployTableJoins = deploy.getTableJoins();
    tableJoins = new TableJoin[deployTableJoins.size()];
    for (int i = 0; i < deployTableJoins.size(); i++) {
      tableJoins[i] = new TableJoin((DeployTableJoin)deployTableJoins.get(i), propertyMap);
    }
  }
  



  public BeanPropertyAssocOne<?> getUnidirectional()
  {
    return unidirectional;
  }
  


  private void allocateToList(BeanProperty prop)
  {
    if (prop.isTransient()) {
      transients.add(prop);
      return;
    }
    if (prop.isId()) {
      ids.add(prop);
      return;
    }
    nonTransients.add(prop);
    

    if ((desc.getInheritInfo() != null) && (prop.isLocal())) {
      local.add(prop);
    }
    
    if ((prop instanceof BeanPropertyAssocMany)) {
      manys.add(prop);
    }
    else if ((prop instanceof BeanPropertyAssocOne)) {
      if (prop.isEmbedded()) {
        embedded.add(prop);
      } else {
        ones.add(prop);
        BeanPropertyAssocOne<?> assocOne = (BeanPropertyAssocOne)prop;
        if (assocOne.isOneToOneExported()) {
          onesExported.add(prop);
        } else {
          onesImported.add(prop);
        }
      }
    }
    else
    {
      if (prop.isVersion()) {
        version.add(prop);
        if (derivedFirstVersionProp == null) {
          derivedFirstVersionProp = prop;
        }
      }
      if ((prop instanceof BeanPropertyCompound)) {
        baseCompound.add((BeanPropertyCompound)prop);
      } else {
        baseScalar.add(prop);
      }
    }
  }
  
  public BeanProperty getFirstVersion() {
    return derivedFirstVersionProp;
  }
  
  public BeanProperty[] getPropertiesWithValidators(boolean recurse)
  {
    ArrayList<BeanProperty> list = new ArrayList();
    Iterator<BeanProperty> it = propertyMap.values().iterator();
    while (it.hasNext()) {
      BeanProperty property = (BeanProperty)it.next();
      if (property.hasValidationRules(recurse)) {
        list.add(property);
      }
    }
    return (BeanProperty[])list.toArray(new BeanProperty[list.size()]);
  }
  
  public Validator[] getBeanValidators() {
    return new Validator[0];
  }
  
  public LinkedHashMap<String, BeanProperty> getPropertyMap() {
    return propertyMap;
  }
  
  public TableJoin[] getTableJoin() {
    return tableJoins;
  }
  



  public BeanProperty[] getBaseScalar()
  {
    return (BeanProperty[])baseScalar.toArray(new BeanProperty[baseScalar.size()]);
  }
  
  public BeanPropertyCompound[] getBaseCompound() {
    return (BeanPropertyCompound[])baseCompound.toArray(new BeanPropertyCompound[baseCompound.size()]);
  }
  
  public BeanProperty[] getId() {
    return (BeanProperty[])ids.toArray(new BeanProperty[ids.size()]);
  }
  
  public BeanProperty[] getNonTransients() {
    return (BeanProperty[])nonTransients.toArray(new BeanProperty[nonTransients.size()]);
  }
  
  public BeanProperty[] getTransients() {
    return (BeanProperty[])transients.toArray(new BeanProperty[transients.size()]);
  }
  
  public BeanProperty[] getVersion() {
    return (BeanProperty[])version.toArray(new BeanProperty[version.size()]);
  }
  
  public BeanProperty[] getLocal() {
    return (BeanProperty[])local.toArray(new BeanProperty[local.size()]);
  }
  
  public BeanPropertyAssocOne<?>[] getEmbedded() {
    return (BeanPropertyAssocOne[])embedded.toArray(new BeanPropertyAssocOne[embedded.size()]);
  }
  
  public BeanPropertyAssocOne<?>[] getOneExported() {
    return (BeanPropertyAssocOne[])onesExported.toArray(new BeanPropertyAssocOne[onesExported.size()]);
  }
  
  public BeanPropertyAssocOne<?>[] getOneImported() {
    return (BeanPropertyAssocOne[])onesImported.toArray(new BeanPropertyAssocOne[onesImported.size()]);
  }
  
  public BeanPropertyAssocOne<?>[] getOnes() {
    return (BeanPropertyAssocOne[])ones.toArray(new BeanPropertyAssocOne[ones.size()]);
  }
  
  public BeanPropertyAssocOne<?>[] getOneExportedSave() {
    return getOne(false, Mode.Save);
  }
  
  public BeanPropertyAssocOne<?>[] getOneExportedDelete() {
    return getOne(false, Mode.Delete);
  }
  
  public BeanPropertyAssocOne<?>[] getOneImportedSave() {
    return getOne(true, Mode.Save);
  }
  
  public BeanPropertyAssocOne<?>[] getOneImportedDelete() {
    return getOne(true, Mode.Delete);
  }
  
  public BeanPropertyAssocMany<?>[] getMany() {
    return (BeanPropertyAssocMany[])manys.toArray(new BeanPropertyAssocMany[manys.size()]);
  }
  
  public BeanPropertyAssocMany<?>[] getManySave() {
    return getMany(Mode.Save);
  }
  
  public BeanPropertyAssocMany<?>[] getManyDelete() {
    return getMany(Mode.Delete);
  }
  
  public BeanPropertyAssocMany<?>[] getManyToMany() {
    return getMany2Many();
  }
  


  private static enum Mode
  {
    Save,  Delete,  Validate;
    
    private Mode() {} }
  
  private BeanPropertyAssocOne<?>[] getOne(boolean imported, Mode mode) { ArrayList<BeanPropertyAssocOne<?>> list = new ArrayList();
    for (int i = 0; i < ones.size(); i++) {
      BeanPropertyAssocOne<?> prop = (BeanPropertyAssocOne)ones.get(i);
      if (imported != prop.isOneToOneExported()) {
        switch (1.$SwitchMap$com$avaje$ebeaninternal$server$deploy$meta$DeployBeanPropertyLists$Mode[mode.ordinal()]) {
        case 1: 
          if (prop.getCascadeInfo().isSave())
            list.add(prop);
          break;
        
        case 2: 
          if (prop.getCascadeInfo().isDelete())
            list.add(prop);
          break;
        
        case 3: 
          if (prop.getCascadeInfo().isValidate()) {
            list.add(prop);
          }
          
          break;
        }
        
      }
    }
    
    return (BeanPropertyAssocOne[])list.toArray(new BeanPropertyAssocOne[list.size()]);
  }
  
  private BeanPropertyAssocMany<?>[] getMany2Many() {
    ArrayList<BeanPropertyAssocMany<?>> list = new ArrayList();
    for (int i = 0; i < manys.size(); i++) {
      BeanPropertyAssocMany<?> prop = (BeanPropertyAssocMany)manys.get(i);
      if (prop.isManyToMany()) {
        list.add(prop);
      }
    }
    
    return (BeanPropertyAssocMany[])list.toArray(new BeanPropertyAssocMany[list.size()]);
  }
  
  private BeanPropertyAssocMany<?>[] getMany(Mode mode) {
    ArrayList<BeanPropertyAssocMany<?>> list = new ArrayList();
    for (int i = 0; i < manys.size(); i++) {
      BeanPropertyAssocMany<?> prop = (BeanPropertyAssocMany)manys.get(i);
      
      switch (1.$SwitchMap$com$avaje$ebeaninternal$server$deploy$meta$DeployBeanPropertyLists$Mode[mode.ordinal()]) {
      case 1: 
        if ((prop.getCascadeInfo().isSave()) || (prop.isManyToMany()) || (BeanCollection.ModifyListenMode.REMOVALS.equals(prop.getModifyListenMode())))
        {



          list.add(prop); }
        break;
      
      case 2: 
        if ((prop.getCascadeInfo().isDelete()) || (BeanCollection.ModifyListenMode.REMOVALS.equals(prop.getModifyListenMode())))
        {

          list.add(prop); }
        break;
      
      case 3: 
        if (prop.getCascadeInfo().isValidate()) {
          list.add(prop);
        }
        

        break;
      }
      
    }
    
    return (BeanPropertyAssocMany[])list.toArray(new BeanPropertyAssocMany[list.size()]);
  }
  

  private BeanProperty createBeanProperty(BeanDescriptorMap owner, DeployBeanProperty deployProp)
  {
    if ((deployProp instanceof DeployBeanPropertyAssocOne))
    {
      return new BeanPropertyAssocOne(owner, desc, (DeployBeanPropertyAssocOne)deployProp);
    }
    if ((deployProp instanceof DeployBeanPropertySimpleCollection))
    {
      return new BeanPropertySimpleCollection(owner, desc, (DeployBeanPropertySimpleCollection)deployProp);
    }
    if ((deployProp instanceof DeployBeanPropertyAssocMany))
    {
      return new BeanPropertyAssocMany(owner, desc, (DeployBeanPropertyAssocMany)deployProp);
    }
    if ((deployProp instanceof DeployBeanPropertyCompound))
    {
      return new BeanPropertyCompound(owner, desc, (DeployBeanPropertyCompound)deployProp);
    }
    
    return new BeanProperty(owner, desc, deployProp);
  }
}
