package com.avaje.ebeaninternal.server.deploy.meta;

import com.avaje.ebeaninternal.server.deploy.BeanProperty;
import com.avaje.ebeaninternal.server.deploy.BeanTable;
import javax.persistence.JoinColumn;






































public class DeployTableJoinColumn
{
  String localDbColumn;
  String foreignDbColumn;
  boolean insertable;
  boolean updateable;
  
  public DeployTableJoinColumn(String localDbColumn, String foreignDbColumn)
  {
    this(localDbColumn, foreignDbColumn, true, true);
  }
  


  public DeployTableJoinColumn(String localDbColumn, String foreignDbColumn, boolean insertable, boolean updateable)
  {
    this.localDbColumn = nullEmptyString(localDbColumn);
    this.foreignDbColumn = nullEmptyString(foreignDbColumn);
    this.insertable = insertable;
    this.updateable = updateable;
  }
  
  public DeployTableJoinColumn(boolean order, JoinColumn jc, BeanTable beanTable) {
    this(jc.referencedColumnName(), jc.name(), jc.insertable(), jc.updatable());
    setReferencedColumn(beanTable);
    if (!order) {
      reverse();
    }
  }
  
  private void setReferencedColumn(BeanTable beanTable) {
    if (localDbColumn == null) {
      BeanProperty[] idProperties = beanTable.getIdProperties();
      if (idProperties.length == 1) {
        localDbColumn = idProperties[0].getDbColumn();
      }
    }
  }
  


  public DeployTableJoinColumn reverse()
  {
    String temp = localDbColumn;
    localDbColumn = foreignDbColumn;
    foreignDbColumn = temp;
    return this;
  }
  


  private String nullEmptyString(String s)
  {
    if ("".equals(s)) {
      return null;
    }
    return s;
  }
  



  public DeployTableJoinColumn copy(boolean reverse)
  {
    if (reverse) {
      return new DeployTableJoinColumn(foreignDbColumn, localDbColumn, insertable, updateable);
    }
    
    return new DeployTableJoinColumn(localDbColumn, foreignDbColumn, insertable, updateable);
  }
  
  public String toString()
  {
    return localDbColumn + " = " + foreignDbColumn;
  }
  






  public boolean hasNullColumn()
  {
    return (localDbColumn == null) || (foreignDbColumn == null);
  }
  





  public String getNonNullColumn()
  {
    if ((localDbColumn == null) && (foreignDbColumn == null)) {
      throw new IllegalStateException("expecting only one null column?");
    }
    if ((localDbColumn != null) && (foreignDbColumn != null)) {
      throw new IllegalStateException("expecting one null column?");
    }
    if (localDbColumn != null) {
      return localDbColumn;
    }
    return foreignDbColumn;
  }
  



  public boolean isInsertable()
  {
    return insertable;
  }
  


  public boolean isUpdateable()
  {
    return updateable;
  }
  


  public String getForeignDbColumn()
  {
    return foreignDbColumn;
  }
  






  public void setForeignDbColumn(String foreignDbColumn)
  {
    this.foreignDbColumn = foreignDbColumn;
  }
  


  public String getLocalDbColumn()
  {
    return localDbColumn;
  }
  


  public void setLocalDbColumn(String localDbColumn)
  {
    this.localDbColumn = localDbColumn;
  }
}
