package com.avaje.ebeaninternal.server.el;

import com.avaje.ebean.bean.EntityBean;
import com.avaje.ebean.bean.EntityBeanIntercept;
import com.avaje.ebean.text.StringFormatter;
import com.avaje.ebean.text.StringParser;
import com.avaje.ebeaninternal.server.deploy.BeanProperty;
import com.avaje.ebeaninternal.server.lib.util.StringHelper;
import com.avaje.ebeaninternal.server.query.SplitName;
import com.avaje.ebeaninternal.server.type.ScalarType;





































public class ElPropertyChain
  implements ElPropertyValue
{
  private final String prefix;
  private final String placeHolder;
  private final String placeHolderEncrypted;
  private final String name;
  private final String expression;
  private final boolean containsMany;
  private final ElPropertyValue[] chain;
  private final boolean assocId;
  private final int last;
  private final BeanProperty lastBeanProperty;
  private final ScalarType<?> scalarType;
  private final ElPropertyValue lastElPropertyValue;
  
  public ElPropertyChain(boolean containsMany, boolean embedded, String expression, ElPropertyValue[] chain)
  {
    this.containsMany = containsMany;
    this.chain = chain;
    this.expression = expression;
    int dotPos = expression.lastIndexOf('.');
    if (dotPos > -1) {
      name = expression.substring(dotPos + 1);
      if (embedded) {
        int embPos = expression.lastIndexOf('.', dotPos - 1);
        prefix = (embPos == -1 ? null : expression.substring(0, embPos));
      }
      else {
        prefix = expression.substring(0, dotPos);
      }
    } else {
      prefix = null;
      name = expression;
    }
    
    assocId = chain[(chain.length - 1)].isAssocId();
    
    last = (chain.length - 1);
    lastBeanProperty = chain[(chain.length - 1)].getBeanProperty();
    if (lastBeanProperty != null) {
      scalarType = lastBeanProperty.getScalarType();
    }
    else {
      scalarType = null;
    }
    lastElPropertyValue = chain[(chain.length - 1)];
    placeHolder = getElPlaceHolder(prefix, lastElPropertyValue, false);
    placeHolderEncrypted = getElPlaceHolder(prefix, lastElPropertyValue, true);
  }
  
  private String getElPlaceHolder(String prefix, ElPropertyValue lastElPropertyValue, boolean encrypted) {
    if (prefix == null) {
      return lastElPropertyValue.getElPlaceholder(encrypted);
    }
    
    String el = lastElPropertyValue.getElPlaceholder(encrypted);
    
    if (!el.contains("${}"))
    {
      return StringHelper.replaceString(el, "${", "${" + prefix + ".");
    }
    return StringHelper.replaceString(el, "${}", "${" + prefix + "}");
  }
  



  public boolean isDeployOnly()
  {
    return false;
  }
  



  public boolean containsManySince(String sinceProperty)
  {
    if (sinceProperty == null) {
      return containsMany;
    }
    if (!expression.startsWith(sinceProperty)) {
      return containsMany;
    }
    
    for (int i = 1 + SplitName.count('.', sinceProperty); 
        
        i < chain.length; i++) {
      if (chain[i].getBeanProperty().containsMany()) {
        return true;
      }
    }
    
    return false;
  }
  
  public boolean containsMany() {
    return containsMany;
  }
  
  public String getElPrefix() {
    return prefix;
  }
  
  public String getName() {
    return name;
  }
  
  public String getElName() {
    return expression;
  }
  
  public String getElPlaceholder(boolean encrypted) {
    return encrypted ? placeHolderEncrypted : placeHolder;
  }
  
  public boolean isDbEncrypted() {
    return lastElPropertyValue.isDbEncrypted();
  }
  
  public boolean isLocalEncrypted() {
    return lastElPropertyValue.isLocalEncrypted();
  }
  

  public Object[] getAssocOneIdValues(Object bean)
  {
    return lastElPropertyValue.getAssocOneIdValues(bean);
  }
  
  public String getAssocOneIdExpr(String prefix, String operator) {
    return lastElPropertyValue.getAssocOneIdExpr(expression, operator);
  }
  
  public String getAssocIdInExpr(String prefix) {
    return lastElPropertyValue.getAssocIdInExpr(prefix);
  }
  
  public String getAssocIdInValueExpr(int size) {
    return lastElPropertyValue.getAssocIdInValueExpr(size);
  }
  
  public int getDeployOrder() {
    int i = lastBeanProperty.getDeployOrder();
    int max = chain.length - 1;
    for (int j = 0; j < max; j++) {
      int xtra = (max - j) * 1000 * chain[j].getDeployOrder();
      i += xtra;
    }
    return i;
  }
  
  public boolean isAssocId() {
    return assocId;
  }
  
  public boolean isAssocProperty() {
    for (int i = 0; i < chain.length; i++) {
      if (chain[i].isAssocProperty()) {
        return true;
      }
    }
    return false;
  }
  
  public String getDbColumn() {
    return lastElPropertyValue.getDbColumn();
  }
  
  public BeanProperty getBeanProperty() {
    return lastBeanProperty;
  }
  
  public boolean isDateTimeCapable()
  {
    return (scalarType != null) && (scalarType.isDateTimeCapable());
  }
  
  public int getJdbcType() {
    return scalarType == null ? 0 : scalarType.getJdbcType();
  }
  
  public Object parseDateTime(long systemTimeMillis) {
    return scalarType.parseDateTime(systemTimeMillis);
  }
  
  public StringParser getStringParser() {
    return scalarType;
  }
  
  public StringFormatter getStringFormatter() {
    return scalarType;
  }
  
  public Object elConvertType(Object value)
  {
    return lastElPropertyValue.elConvertType(value);
  }
  
  public Object elGetValue(Object bean)
  {
    for (int i = 0; i < chain.length; i++) {
      bean = chain[i].elGetValue(bean);
      if (bean == null) {
        return null;
      }
    }
    
    return bean;
  }
  
  public Object elGetReference(Object bean)
  {
    Object prevBean = bean;
    for (int i = 0; i < last; i++)
    {
      prevBean = chain[i].elGetReference(prevBean);
    }
    
    bean = chain[last].elGetValue(prevBean);
    
    return bean;
  }
  

  public void elSetLoaded(Object bean)
  {
    for (int i = 0; i < last; i++) {
      bean = chain[i].elGetValue(bean);
      if (bean == null) {
        break;
      }
    }
    if (bean != null) {
      ((EntityBean)bean)._ebean_getIntercept().setLoaded();
    }
  }
  
  public void elSetReference(Object bean)
  {
    for (int i = 0; i < last; i++) {
      bean = chain[i].elGetValue(bean);
      if (bean == null) {
        break;
      }
    }
    if (bean != null) {
      ((EntityBean)bean)._ebean_getIntercept().setReference();
    }
  }
  
  public void elSetValue(Object bean, Object value, boolean populate, boolean reference)
  {
    Object prevBean = bean;
    if (populate) {
      for (int i = 0; i < last; i++)
      {
        prevBean = chain[i].elGetReference(prevBean);
      }
    } else {
      for (int i = 0; i < last; i++)
      {
        prevBean = chain[i].elGetValue(prevBean);
        if (prevBean == null) {
          break;
        }
      }
    }
    if (prevBean != null) {
      if (lastBeanProperty != null)
      {
        lastBeanProperty.setValueIntercept(prevBean, value);
        if (reference) {
          ((EntityBean)prevBean)._ebean_getIntercept().setReference();
        }
      }
      else {
        lastElPropertyValue.elSetValue(prevBean, value, populate, reference);
      }
    }
  }
}
