package com.avaje.ebeaninternal.server.expression;

import com.avaje.ebeaninternal.server.core.BasicTypeConverter;
import org.apache.lucene.search.NumericRangeQuery;
import org.apache.lucene.search.Query;



























public class LLuceneRangeExpression
{
  private final SimpleExpression.Op op;
  private final Object value;
  private final String propertyName;
  private final int luceneType;
  String description;
  boolean minInclusive;
  boolean maxInclusive;
  
  public LLuceneRangeExpression(SimpleExpression.Op op, Object value, String propertyName, int luceneType)
  {
    this.op = op;
    this.value = value;
    this.propertyName = propertyName;
    this.luceneType = luceneType;
    
    minInclusive = ((SimpleExpression.Op.EQ.equals(op)) || (SimpleExpression.Op.GT_EQ.equals(op)));
    maxInclusive = ((SimpleExpression.Op.EQ.equals(op)) || (SimpleExpression.Op.LT_EQ.equals(op)));
    
    description = (propertyName + op.shortDesc() + value);
  }
  
  public String getDescription() {
    return description;
  }
  
  public Query buildQuery()
  {
    switch (luceneType) {
    case 1: 
      return createIntRange();
    case 2: 
      return createLongRange();
    case 3: 
      return createDoubleRange();
    case 4: 
      return createFloatRange();
    
    case 5: 
      return createLongRange();
    case 6: 
      return createLongRange();
    }
    
    throw new RuntimeException("Unhandled type " + luceneType);
  }
  

  private Query createIntRange()
  {
    Integer intVal = BasicTypeConverter.toInteger(value);
    
    Integer min = intVal;
    Integer max = intVal;
    
    if (!SimpleExpression.Op.EQ.equals(op))
    {
      if ((SimpleExpression.Op.GT.equals(op)) || (SimpleExpression.Op.GT_EQ.equals(op))) {
        max = Integer.valueOf(2147483647);
      } else if ((SimpleExpression.Op.LT.equals(op)) || (SimpleExpression.Op.LT_EQ.equals(op))) {
        min = Integer.valueOf(-2147483648);
      }
    }
    return NumericRangeQuery.newIntRange(propertyName, min, max, minInclusive, maxInclusive);
  }
  
  private Query createLongRange()
  {
    Long longVal = BasicTypeConverter.toLong(value);
    Long min = longVal;
    Long max = longVal;
    
    if (!SimpleExpression.Op.EQ.equals(op))
    {
      if ((SimpleExpression.Op.GT.equals(op)) || (SimpleExpression.Op.GT_EQ.equals(op))) {
        max = Long.valueOf(9223372036854775807L);
      } else if ((SimpleExpression.Op.LT.equals(op)) || (SimpleExpression.Op.LT_EQ.equals(op))) {
        min = Long.valueOf(-9223372036854775808L);
      }
    }
    return NumericRangeQuery.newLongRange(propertyName, min, max, minInclusive, maxInclusive);
  }
  
  private Query createFloatRange()
  {
    Float floatVal = BasicTypeConverter.toFloat(value);
    Float min = floatVal;
    Float max = floatVal;
    
    if (!SimpleExpression.Op.EQ.equals(op))
    {
      if ((SimpleExpression.Op.GT.equals(op)) || (SimpleExpression.Op.GT_EQ.equals(op))) {
        max = Float.valueOf(3.4028235E+38F);
      } else if ((SimpleExpression.Op.LT.equals(op)) || (SimpleExpression.Op.LT_EQ.equals(op))) {
        min = Float.valueOf(1.4E-45F);
      }
    }
    return NumericRangeQuery.newFloatRange(propertyName, min, max, minInclusive, maxInclusive);
  }
  
  private Query createDoubleRange()
  {
    Double doubleVal = BasicTypeConverter.toDouble(value);
    Double min = doubleVal;
    Double max = doubleVal;
    
    if (!SimpleExpression.Op.EQ.equals(op))
    {
      if ((SimpleExpression.Op.GT.equals(op)) || (SimpleExpression.Op.GT_EQ.equals(op))) {
        max = Double.valueOf(1.7976931348623157E+308D);
      } else if ((SimpleExpression.Op.LT.equals(op)) || (SimpleExpression.Op.LT_EQ.equals(op))) {
        min = Double.valueOf(4.9E-324D);
      }
    }
    return NumericRangeQuery.newDoubleRange(propertyName, min, max, minInclusive, maxInclusive);
  }
}
