package com.avaje.ebeaninternal.server.expression;

import com.avaje.ebean.event.BeanQueryRequest;
import com.avaje.ebeaninternal.api.SpiExpressionRequest;
import com.avaje.ebeaninternal.api.SpiLuceneExpr;
import com.avaje.ebeaninternal.server.lucene.LIndex;
import com.avaje.ebeaninternal.server.query.LuceneResolvableRequest;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.queryParser.QueryParser;
import org.apache.lucene.queryParser.QueryParser.Operator;



















class LuceneExpression
  extends AbstractExpression
  implements LuceneAwareExpression
{
  private static final long serialVersionUID = 8959252357123977939L;
  private final String val;
  private final boolean andOperator;
  
  LuceneExpression(FilterExprPath pathPrefix, String propertyName, String value, boolean andOperator)
  {
    super(pathPrefix, propertyName);
    val = value;
    this.andOperator = andOperator;
  }
  
  public boolean isLuceneResolvable(LuceneResolvableRequest req) {
    return true;
  }
  
  public SpiLuceneExpr createLuceneExpr(SpiExpressionRequest request)
  {
    try {
      String propertyName = getPropertyName();
      
      String desc = propertyName + " " + val;
      
      QueryParser p = request.getLuceneIndex().createQueryParser(propertyName);
      p.setDefaultOperator(QueryParser.Operator.OR);
      return new LuceneExprResponse(p.parse(val), desc);
    } catch (ParseException e) {
      throw new PersistenceLuceneParseException(e);
    }
  }
  

  public void addBindValues(SpiExpressionRequest request) {}
  
  public void addSql(SpiExpressionRequest request) {}
  
  public int queryPlanHash(BeanQueryRequest<?> request)
  {
    return queryAutoFetchHash();
  }
  


  public int queryAutoFetchHash()
  {
    int hc = LuceneExpression.class.getName().hashCode();
    hc = hc * 31 + (andOperator ? 0 : 1);
    hc = hc * 31 + propName.hashCode();
    return hc;
  }
  
  public int queryBindHash() {
    return val.hashCode();
  }
}
