package com.avaje.ebeaninternal.server.ldap;

import com.avaje.ebeaninternal.api.SpiQuery;
import com.avaje.ebeaninternal.server.deploy.BeanDescriptor;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.Attributes;
import javax.naming.directory.DirContext;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;
import javax.naming.ldap.LdapName;






















public class LdapOrmQueryExecute<T>
{
  private static final Logger logger = Logger.getLogger(LdapOrmQueryExecute.class.getName());
  
  private final SpiQuery<?> query;
  
  private final BeanDescriptor<T> beanDescriptor;
  
  private final DirContext dc;
  
  private final LdapBeanBuilder<T> beanBuilder;
  
  private final String filterExpr;
  
  private final Object[] filterValues;
  
  private final String[] selectProps;
  
  public LdapOrmQueryExecute(LdapOrmQueryRequest<T> request, boolean defaultVanillaMode, DirContext dc)
  {
    query = request.getQuery();
    beanDescriptor = request.getBeanDescriptor();
    this.dc = dc;
    
    boolean vanillaMode = query.isVanillaMode(defaultVanillaMode);
    beanBuilder = new LdapBeanBuilder(beanDescriptor, vanillaMode);
    
    LdapQueryDeployHelper deployHelper = new LdapQueryDeployHelper(request);
    selectProps = deployHelper.getSelectedProperties();
    filterExpr = deployHelper.getFilterExpr();
    filterValues = deployHelper.getFilterValues();
  }
  
  public T findId()
  {
    Object id = query.getId();
    try
    {
      LdapName dn = beanDescriptor.createLdapNameById(id);
      
      String[] findAttrs = selectProps;
      if (findAttrs == null) {
        findAttrs = beanDescriptor.getDefaultSelectDbArray();
      }
      

      String debugQuery = "Name:" + dn + " attrs:" + Arrays.toString(findAttrs);
      
      Attributes attrs = dc.getAttributes(dn, findAttrs);
      
      T bean = beanBuilder.readAttributes(attrs);
      
      query.setGeneratedSql(debugQuery);
      return bean;
    }
    catch (NamingException e) {
      throw new LdapPersistenceException(e);
    }
  }
  
  public List<T> findList()
  {
    SearchControls sc = new SearchControls();
    sc.setSearchScope(1);
    
    List<T> list = new ArrayList();
    try
    {
      LdapName dn = beanDescriptor.createLdapName(null);
      

      String debugQuery = "Name:" + dn;
      
      if (selectProps != null) {
        sc.setReturningAttributes(selectProps);
        debugQuery = debugQuery + " select:" + Arrays.toString(selectProps);
      }
      
      if (logger.isLoggable(Level.INFO)) {
        logger.info("Ldap Query  Name:" + dn + " filterExpr:" + filterExpr);
      }
      
      debugQuery = debugQuery + " filterExpr:" + filterExpr;
      NamingEnumeration<SearchResult> result;
      NamingEnumeration<SearchResult> result;
      if ((filterValues == null) || (filterValues.length == 0)) {
        result = dc.search(dn, filterExpr, sc);
      } else {
        debugQuery = debugQuery + " filterValues:" + Arrays.toString(filterValues);
        result = dc.search(dn, filterExpr, filterValues, sc);
      }
      
      query.setGeneratedSql(debugQuery);
      
      if (result != null) {
        while (result.hasMoreElements()) {
          SearchResult row = (SearchResult)result.nextElement();
          T bean = beanBuilder.readAttributes(row.getAttributes());
          list.add(bean);
        }
      }
      
      return list;
    }
    catch (NamingException e) {
      throw new LdapPersistenceException(e);
    }
  }
}
