package com.avaje.ebeaninternal.server.lib;

import java.util.Iterator;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;





































public final class BackgroundThread
{
  private static final Logger logger = Logger.getLogger(BackgroundThread.class.getName());
  
  private static class Single {
    private static BackgroundThread me = new BackgroundThread(null);
  }
  



  private Vector<BackgroundRunnable> list = new Vector();
  



  private final Object monitor = new Object();
  



  private final Thread thread;
  



  private long sleepTime = 1000L;
  



  private long count;
  



  private long exeTime;
  



  private boolean stopped;
  



  private Object threadMonitor = new Object();
  
  private BackgroundThread()
  {
    thread = new Thread(new Runner(null), "EbeanBackgroundThread");
    thread.setDaemon(true);
    thread.start();
  }
  


  public static void add(int freqInSecs, Runnable runnable)
  {
    add(new BackgroundRunnable(runnable, freqInSecs));
  }
  


  public static void add(BackgroundRunnable backgroundRunnable)
  {
    Single.me.addTask(backgroundRunnable);
  }
  


  public static void shutdown()
  {
    Single.me.stop();
  }
  


  public static Iterator<BackgroundRunnable> runnables()
  {
    synchronized (memonitor) {
      return melist.iterator();
    }
  }
  
  private void addTask(BackgroundRunnable backgroundRunnable) {
    synchronized (monitor) {
      list.add(backgroundRunnable);
    }
  }
  




  private void stop()
  {
    stopped = true;
    synchronized (threadMonitor) {
      try {
        threadMonitor.wait(10000L);
      }
      catch (InterruptedException e) {}
    }
  }
  

  private class Runner
    implements Runnable
  {
    private Runner() {}
    

    public void run()
    {
      if (ShutdownManager.isStopping()) {
        return;
      }
      
      while (!stopped) {
        try
        {
          long actualSleep = sleepTime - exeTime;
          if (actualSleep < 0L) {
            actualSleep = sleepTime;
          }
          Thread.sleep(actualSleep);
          synchronized (monitor) {
            runJobs();
          }
        }
        catch (InterruptedException e) {
          BackgroundThread.logger.log(Level.SEVERE, null, e);
        }
      }
      

      synchronized (threadMonitor) {
        threadMonitor.notifyAll();
      }
    }
    
    private void runJobs()
    {
      long startTime = System.currentTimeMillis();
      

      Iterator<BackgroundRunnable> it = list.iterator();
      while (it.hasNext()) {
        BackgroundRunnable bgr = (BackgroundRunnable)it.next();
        if (bgr.isActive())
        {
          int freqInSecs = bgr.getFreqInSecs();
          
          if (count % freqInSecs == 0L) {
            Runnable runable = bgr.getRunnable();
            if (bgr.runNow(startTime)) {
              bgr.runStart();
              if (BackgroundThread.logger.isLoggable(Level.FINER)) {
                String msg = count + " BGRunnable running [" + runable.getClass().getName() + "]";
                
                BackgroundThread.logger.finer(msg);
              }
              
              runable.run();
              bgr.runEnd();
            }
          }
        }
      }
      exeTime = (System.currentTimeMillis() - startTime);
      BackgroundThread.access$1008(BackgroundThread.this);
      
      if (count == 86400L)
      {
        count = 0L;
      }
    }
  }
  
  public String toString() {
    synchronized (monitor) {
      StringBuffer sb = new StringBuffer();
      
      Iterator<BackgroundRunnable> it = runnables();
      while (it.hasNext()) {
        BackgroundRunnable bgr = (BackgroundRunnable)it.next();
        sb.append(bgr);
      }
      
      return sb.toString();
    }
  }
}
