package com.avaje.ebeaninternal.server.lib.cron;

import com.avaje.ebean.config.GlobalProperties;
import com.avaje.ebeaninternal.server.lib.ShutdownManager;
import com.avaje.ebeaninternal.server.lib.thread.ThreadPool;
import com.avaje.ebeaninternal.server.lib.thread.ThreadPoolManager;
import java.util.Date;
import java.util.Enumeration;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;



























public final class CronManager
{
  private static final Logger logger = Logger.getLogger(CronManager.class.getName());
  
  private static class CronManagerHolder {
    private static CronManager me = new CronManager(null);
  }
  
  private boolean running = true;
  



  private ThreadPool threadPool;
  



  private Vector<CronRunnable> runList;
  



  private Thread backgroundThread;
  



  private boolean isDowntime = false;
  




  private static final long SMALL_DELAY = 10L;
  




  private CronManager()
  {
    runList = new Vector();
    threadPool = ThreadPoolManager.getThreadPool("CronManager");
    
    backgroundThread = new Thread(new Runner(null), "CronManager Daemon");
    backgroundThread.setDaemon(true);
    backgroundThread.start();
  }
  
  private void init()
  {
    CronRunnable sr = new CronRunnable("* * * * *", new HelloWorld());
    sr.setEnabled(false);
    add(sr);
    

    CronRunnable dt = new CronRunnable("25 23 * * *", new Downtime(this));
    dt.setEnabled(false);
    
    String downtimeSchedule = GlobalProperties.get("system.downtime.schedule", null);
    if (downtimeSchedule != null)
    {
      dt.setSchedule(downtimeSchedule);
      dt.setEnabled(true);
    }
    add(dt);
  }
  



  public static boolean isDowntime()
  {
    return getInstance()isDowntime;
  }
  




  protected void setDowntime(boolean isDowntime)
  {
    this.isDowntime = isDowntime;
    if (isDowntime) {
      String duration = GlobalProperties.get("system.downtime.duration", null);
      logger.warning("System downtime has started for [" + duration + "] seconds");
    } else {
      logger.warning("System downtime has finished.");
    }
  }
  


  public static void setRunning(boolean running)
  {
    merunning = running;
  }
  
  private void runScheduledJobs() {
    if (!running)
    {

      return;
    }
    




    Date nowDate = new Date((System.currentTimeMillis() + 5000L) / 60000L * 60000L);
    GregorianCalendar thisMinute = new GregorianCalendar();
    thisMinute.setTime(nowDate);
    
    Enumeration<CronRunnable> en = runList.elements();
    while (en.hasMoreElements()) {
      CronRunnable sr = (CronRunnable)en.nextElement();
      
      if (sr.isScheduledToRunNow(thisMinute)) {
        threadPool.assign(sr.getRunnable(), true);
      }
    }
  }
  




  private static CronManager getInstance()
  {
    return CronManagerHolder.me;
  }
  


  public static void add(String schedule, Runnable runnable)
  {
    CronRunnable sr = new CronRunnable(schedule, runnable);
    add(sr);
  }
  


  public static void add(CronRunnable runnable)
  {
    getInstance()runList.add(runnable);
  }
  




  public static Iterator<CronRunnable> iterator() { return getInstance()runList.iterator(); }
  
  private class Runner implements Runnable {
    private Runner() {}
    
    public void run() {
      CronManager.this.init();
      

      try
      {
        for (;;)
        {
          long nextMinute = System.currentTimeMillis() / 60000L * 60000L + 60000L;
          long now = System.currentTimeMillis();
          long nextSleepTime = nextMinute - now + 10L;
          if (nextSleepTime > 0L)
          {

            Thread.sleep(nextSleepTime);
          }
          

          long additionalDelay = nextMinute - System.currentTimeMillis();
          if (additionalDelay > 0L)
          {

            Thread.sleep(additionalDelay + 20L);
          }
          

          boolean stopping = ShutdownManager.isStopping();
          if (!stopping) {
            CronManager.this.runScheduledJobs();
          }
          





          Thread.sleep(5000L);
        }
      } catch (InterruptedException e) {
        CronManager.logger.log(Level.SEVERE, "", e);
      }
    }
  }
}
