package com.avaje.ebeaninternal.server.lib.cron;

import java.util.Calendar;
import java.util.StringTokenizer;












































public class CronSchedule
{
  private String schedule;
  private boolean[] bHours;
  private boolean[] bMinutes;
  private boolean[] bMonths;
  private boolean[] bDaysOfWeek;
  private boolean[] bDaysOfMonth;
  
  public CronSchedule(String scheduleLine)
  {
    setSchedule(scheduleLine);
  }
  
  public boolean equals(Object obj) {
    if (obj == null) {
      return false;
    }
    if ((obj instanceof CronSchedule)) {
      CronSchedule cs = (CronSchedule)obj;
      if (schedule.equals(cs.getSchedule())) {
        return true;
      }
    }
    return false;
  }
  
  public int hashCode() {
    int hc = CronSchedule.class.getName().hashCode();
    hc = hc * 31 + schedule.hashCode();
    return hc;
  }
  
  private void initBooleanArrays() {
    bHours = new boolean[24];
    bMinutes = new boolean[60];
    bMonths = new boolean[12];
    bDaysOfWeek = new boolean[7];
    bDaysOfMonth = new boolean[31];
    for (int i = 0; i < 60; i++) {
      if (i < 24)
        bHours[i] = false;
      if (i < 60)
        bMinutes[i] = false;
      if (i < 12)
        bMonths[i] = false;
      if (i < 7)
        bDaysOfWeek[i] = false;
      if (i < 31) {
        bDaysOfMonth[i] = false;
      }
    }
  }
  

  public void setSchedule(String schedule)
  {
    this.schedule = schedule;
    initBooleanArrays();
    
    StringTokenizer tokenizer = new StringTokenizer(schedule);
    
    int numTokens = tokenizer.countTokens();
    for (int i = 0; tokenizer.hasMoreElements(); i++) {
      String token = tokenizer.nextToken();
      switch (i) {
      case 0: 
        parseToken(token, bMinutes, false);
        break;
      case 1: 
        parseToken(token, bHours, false);
        break;
      case 2: 
        parseToken(token, bDaysOfMonth, true);
        break;
      case 3: 
        parseToken(token, bMonths, true);
        break;
      case 4: 
        parseToken(token, bDaysOfWeek, false);
        break;
      case 5: 
        break;
      }
      
    }
    







    if (numTokens < 5) {
      String msg = "The schedule[" + schedule + "] did not contain enough tokens (5 required) [" + numTokens + "].";
      throw new RuntimeException(msg);
    }
  }
  
  private void parseToken(String token, boolean[] arrayBool, boolean bBeginInOne)
  {
    try {
      if (token.equals("*")) {
        for (int i = 0; i < arrayBool.length; i++) {
          arrayBool[i] = true;
        }
        return;
      }
      
      int index = token.indexOf(",");
      if (index > 0) {
        StringTokenizer tokenizer = new StringTokenizer(token, ",");
        while (tokenizer.hasMoreTokens()) {
          parseToken(tokenizer.nextToken(), arrayBool, bBeginInOne);
        }
        return;
      }
      
      index = token.indexOf("-");
      if (index > 0) {
        int start = Integer.parseInt(token.substring(0, index));
        int end = Integer.parseInt(token.substring(index + 1));
        
        if (bBeginInOne) {
          start--;
          end--;
        }
        
        for (int j = start; j <= end; j++)
          arrayBool[j] = true;
        return;
      }
      
      index = token.indexOf("/");
      if (index > 0) {
        int each = Integer.parseInt(token.substring(index + 1));
        for (int j = 0; j < arrayBool.length; j += each)
          arrayBool[j] = true;
        return;
      }
      int iValue = Integer.parseInt(token);
      if (bBeginInOne) {
        iValue--;
      }
      arrayBool[iValue] = true;
      return;
    }
    catch (Exception e) {
      String msg = "The schedule[" + schedule + "] had a problem parsing a token [" + token + "].";
      throw new RuntimeException(msg, e);
    }
  }
  




  public boolean isScheduledToRunNow(Calendar thisMinute)
  {
    return (bHours[thisMinute.get(11)] != 0) && (bMinutes[thisMinute.get(12)] != 0) && (bMonths[thisMinute.get(2)] != 0) && (bDaysOfWeek[(thisMinute.get(7) - 1)] != 0) && (bDaysOfMonth[(thisMinute.get(5) - 1)] != 0);
  }
  







  public String getSchedule()
  {
    return schedule;
  }
}
