package com.avaje.ebeaninternal.server.lib.sql;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;





































class BusyConnectionBuffer
{
  private PooledConnection[] slots;
  private int growBy;
  private int size;
  private int pos = -1;
  








  protected BusyConnectionBuffer(int capacity, int growBy)
  {
    slots = new PooledConnection[capacity];
    this.growBy = growBy;
  }
  


  private void setCapacity(int newCapacity)
  {
    if (newCapacity > slots.length) {
      PooledConnection[] current = slots;
      slots = new PooledConnection[newCapacity];
      System.arraycopy(current, 0, slots, 0, current.length);
    }
  }
  
  public String toString() {
    return Arrays.toString(slots);
  }
  
  protected int getCapacity() {
    return slots.length;
  }
  
  protected int size() {
    return size;
  }
  
  protected boolean isEmpty() {
    return size == 0;
  }
  
  protected int add(PooledConnection pc) {
    if (size == slots.length)
    {
      setCapacity(slots.length + growBy);
    }
    size += 1;
    int slot = nextEmptySlot();
    pc.setSlotId(slot);
    slots[slot] = pc;
    return size;
  }
  
  protected boolean remove(PooledConnection pc) {
    size -= 1;
    int slotId = pc.getSlotId();
    if (slots[slotId] != pc) {
      return false;
    }
    slots[slotId] = null;
    return true;
  }
  







  protected List<PooledConnection> getShallowCopy()
  {
    ArrayList<PooledConnection> tmp = new ArrayList();
    for (int i = 0; i < slots.length; i++) {
      if (slots[i] != null) {
        tmp.add(slots[i]);
      }
    }
    return Collections.unmodifiableList(tmp);
  }
  




  private int nextEmptySlot()
  {
    while (++pos < slots.length) {
      if (slots[pos] == null) {
        return pos;
      }
    }
    
    pos = -1;
    while (++pos < slots.length) {
      if (slots[pos] == null) {
        return pos;
      }
    }
    

    throw new RuntimeException("No Empty Slot Found?");
  }
}
