package com.avaje.ebeaninternal.server.lib.sql;

import java.io.InputStream;
import java.io.Reader;
import java.math.BigDecimal;
import java.net.URL;
import java.sql.Array;
import java.sql.Blob;
import java.sql.Clob;
import java.sql.Date;
import java.sql.ParameterMetaData;
import java.sql.PreparedStatement;
import java.sql.Ref;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.Calendar;


































public class ExtendedPreparedStatement
  extends ExtendedStatement
  implements PreparedStatement
{
  final String sql;
  final String cacheKey;
  
  public ExtendedPreparedStatement(PooledConnection pooledConnection, PreparedStatement pstmt, String sql, String cacheKey)
  {
    super(pooledConnection, pstmt);
    this.sql = sql;
    this.cacheKey = cacheKey;
  }
  
  public PreparedStatement getDelegate() {
    return pstmt;
  }
  


  public String getCacheKey()
  {
    return cacheKey;
  }
  


  public String getSql()
  {
    return sql;
  }
  


  public void closeDestroy()
    throws SQLException
  {
    pstmt.close();
  }
  



  public void close()
    throws SQLException
  {
    pooledConnection.returnPreparedStatement(this);
  }
  
  public void addBatch()
    throws SQLException
  {
    try
    {
      pstmt.addBatch();
    }
    catch (SQLException e)
    {
      pooledConnection.addError(e);
      throw e;
    }
  }
  
  public void clearParameters()
    throws SQLException
  {
    try
    {
      pstmt.clearParameters();
    }
    catch (SQLException e)
    {
      pooledConnection.addError(e);
      throw e;
    }
  }
  
  public boolean execute()
    throws SQLException
  {
    try
    {
      return pstmt.execute();
    }
    catch (SQLException e)
    {
      pooledConnection.addError(e);
      throw e;
    }
  }
  
  public ResultSet executeQuery()
    throws SQLException
  {
    try
    {
      return pstmt.executeQuery();
    }
    catch (SQLException e)
    {
      pooledConnection.addError(e);
      throw e;
    }
  }
  
  public int executeUpdate()
    throws SQLException
  {
    try
    {
      return pstmt.executeUpdate();
    }
    catch (SQLException e)
    {
      pooledConnection.addError(e);
      throw e;
    }
  }
  
  public ResultSetMetaData getMetaData()
    throws SQLException
  {
    try
    {
      return pstmt.getMetaData();
    }
    catch (SQLException e)
    {
      pooledConnection.addError(e);
      throw e;
    }
  }
  

  public ParameterMetaData getParameterMetaData()
    throws SQLException
  {
    return pstmt.getParameterMetaData();
  }
  

  public void setArray(int i, Array x)
    throws SQLException
  {
    pstmt.setArray(i, x);
  }
  

  public void setAsciiStream(int parameterIndex, InputStream x, int length)
    throws SQLException
  {
    pstmt.setAsciiStream(parameterIndex, x, length);
  }
  

  public void setBigDecimal(int parameterIndex, BigDecimal x)
    throws SQLException
  {
    pstmt.setBigDecimal(parameterIndex, x);
  }
  

  public void setBinaryStream(int parameterIndex, InputStream x, int length)
    throws SQLException
  {
    pstmt.setBinaryStream(parameterIndex, x, length);
  }
  

  public void setBlob(int i, Blob x)
    throws SQLException
  {
    pstmt.setBlob(i, x);
  }
  

  public void setBoolean(int parameterIndex, boolean x)
    throws SQLException
  {
    pstmt.setBoolean(parameterIndex, x);
  }
  

  public void setByte(int parameterIndex, byte x)
    throws SQLException
  {
    pstmt.setByte(parameterIndex, x);
  }
  

  public void setBytes(int parameterIndex, byte[] x)
    throws SQLException
  {
    pstmt.setBytes(parameterIndex, x);
  }
  


  public void setCharacterStream(int parameterIndex, Reader reader, int length)
    throws SQLException
  {
    pstmt.setCharacterStream(parameterIndex, reader, length);
  }
  

  public void setClob(int i, Clob x)
    throws SQLException
  {
    pstmt.setClob(i, x);
  }
  

  public void setDate(int parameterIndex, Date x)
    throws SQLException
  {
    pstmt.setDate(parameterIndex, x);
  }
  

  public void setDate(int parameterIndex, Date x, Calendar cal)
    throws SQLException
  {
    pstmt.setDate(parameterIndex, x, cal);
  }
  

  public void setDouble(int parameterIndex, double x)
    throws SQLException
  {
    pstmt.setDouble(parameterIndex, x);
  }
  

  public void setFloat(int parameterIndex, float x)
    throws SQLException
  {
    pstmt.setFloat(parameterIndex, x);
  }
  

  public void setInt(int parameterIndex, int x)
    throws SQLException
  {
    pstmt.setInt(parameterIndex, x);
  }
  

  public void setLong(int parameterIndex, long x)
    throws SQLException
  {
    pstmt.setLong(parameterIndex, x);
  }
  

  public void setNull(int parameterIndex, int sqlType)
    throws SQLException
  {
    pstmt.setNull(parameterIndex, sqlType);
  }
  

  public void setNull(int paramIndex, int sqlType, String typeName)
    throws SQLException
  {
    pstmt.setNull(paramIndex, sqlType, typeName);
  }
  

  public void setObject(int parameterIndex, Object x)
    throws SQLException
  {
    pstmt.setObject(parameterIndex, x);
  }
  

  public void setObject(int parameterIndex, Object x, int targetSqlType)
    throws SQLException
  {
    pstmt.setObject(parameterIndex, x, targetSqlType);
  }
  


  public void setObject(int parameterIndex, Object x, int targetSqlType, int scale)
    throws SQLException
  {
    pstmt.setObject(parameterIndex, x, targetSqlType, scale);
  }
  

  public void setRef(int i, Ref x)
    throws SQLException
  {
    pstmt.setRef(i, x);
  }
  

  public void setShort(int parameterIndex, short x)
    throws SQLException
  {
    pstmt.setShort(parameterIndex, x);
  }
  

  public void setString(int parameterIndex, String x)
    throws SQLException
  {
    pstmt.setString(parameterIndex, x);
  }
  

  public void setTime(int parameterIndex, Time x)
    throws SQLException
  {
    pstmt.setTime(parameterIndex, x);
  }
  

  public void setTime(int parameterIndex, Time x, Calendar cal)
    throws SQLException
  {
    pstmt.setTime(parameterIndex, x, cal);
  }
  

  public void setTimestamp(int parameterIndex, Timestamp x)
    throws SQLException
  {
    pstmt.setTimestamp(parameterIndex, x);
  }
  

  public void setTimestamp(int parameterIndex, Timestamp x, Calendar cal)
    throws SQLException
  {
    pstmt.setTimestamp(parameterIndex, x, cal);
  }
  
  /**
   * @deprecated
   */
  public void setUnicodeStream(int parameterIndex, InputStream x, int length) throws SQLException
  {
    pstmt.setUnicodeStream(parameterIndex, x, length);
  }
  

  public void setURL(int parameterIndex, URL x)
    throws SQLException
  {
    pstmt.setURL(parameterIndex, x);
  }
}
