package com.avaje.ebeaninternal.server.lib.sql;

import com.avaje.ebeaninternal.jdbc.PreparedStatementDelegator;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.SQLWarning;




































public abstract class ExtendedStatement
  extends PreparedStatementDelegator
{
  protected final PooledConnection pooledConnection;
  protected final PreparedStatement pstmt;
  
  public ExtendedStatement(PooledConnection pooledConnection, PreparedStatement pstmt)
  {
    super(pstmt);
    
    this.pooledConnection = pooledConnection;
    this.pstmt = pstmt;
  }
  

  public abstract void close()
    throws SQLException;
  

  public Connection getConnection()
    throws SQLException
  {
    try
    {
      return pstmt.getConnection();
    } catch (SQLException e) {
      pooledConnection.addError(e);
      throw e;
    }
  }
  
  public void addBatch(String sql)
    throws SQLException
  {
    try
    {
      pooledConnection.setLastStatement(sql);
      pstmt.addBatch(sql);
    } catch (SQLException e) {
      pooledConnection.addError(e);
      throw e;
    }
  }
  
  public boolean execute(String sql)
    throws SQLException
  {
    try
    {
      pooledConnection.setLastStatement(sql);
      return pstmt.execute(sql);
    } catch (SQLException e) {
      pooledConnection.addError(e);
      throw e;
    }
  }
  
  public ResultSet executeQuery(String sql)
    throws SQLException
  {
    try
    {
      pooledConnection.setLastStatement(sql);
      return pstmt.executeQuery(sql);
    } catch (SQLException e) {
      pooledConnection.addError(e);
      throw e;
    }
  }
  
  public int executeUpdate(String sql)
    throws SQLException
  {
    try
    {
      pooledConnection.setLastStatement(sql);
      return pstmt.executeUpdate(sql);
    } catch (SQLException e) {
      pooledConnection.addError(e);
      throw e;
    }
  }
  

  public int[] executeBatch()
    throws SQLException
  {
    return pstmt.executeBatch();
  }
  

  public void cancel()
    throws SQLException
  {
    pstmt.cancel();
  }
  

  public void clearBatch()
    throws SQLException
  {
    pstmt.clearBatch();
  }
  

  public void clearWarnings()
    throws SQLException
  {
    pstmt.clearWarnings();
  }
  

  public int getFetchDirection()
    throws SQLException
  {
    return pstmt.getFetchDirection();
  }
  

  public int getFetchSize()
    throws SQLException
  {
    return pstmt.getFetchSize();
  }
  

  public int getMaxFieldSize()
    throws SQLException
  {
    return pstmt.getMaxFieldSize();
  }
  

  public int getMaxRows()
    throws SQLException
  {
    return pstmt.getMaxRows();
  }
  

  public boolean getMoreResults()
    throws SQLException
  {
    return pstmt.getMoreResults();
  }
  

  public int getQueryTimeout()
    throws SQLException
  {
    return pstmt.getQueryTimeout();
  }
  

  public ResultSet getResultSet()
    throws SQLException
  {
    return pstmt.getResultSet();
  }
  

  public int getResultSetConcurrency()
    throws SQLException
  {
    return pstmt.getResultSetConcurrency();
  }
  

  public int getResultSetType()
    throws SQLException
  {
    return pstmt.getResultSetType();
  }
  

  public int getUpdateCount()
    throws SQLException
  {
    return pstmt.getUpdateCount();
  }
  

  public SQLWarning getWarnings()
    throws SQLException
  {
    return pstmt.getWarnings();
  }
  

  public void setCursorName(String name)
    throws SQLException
  {
    pstmt.setCursorName(name);
  }
  

  public void setEscapeProcessing(boolean enable)
    throws SQLException
  {
    pstmt.setEscapeProcessing(enable);
  }
  

  public void setFetchDirection(int direction)
    throws SQLException
  {
    pstmt.setFetchDirection(direction);
  }
  

  public void setFetchSize(int rows)
    throws SQLException
  {
    pstmt.setFetchSize(rows);
  }
  

  public void setMaxFieldSize(int max)
    throws SQLException
  {
    pstmt.setMaxFieldSize(max);
  }
  

  public void setMaxRows(int max)
    throws SQLException
  {
    pstmt.setMaxRows(max);
  }
  

  public void setQueryTimeout(int seconds)
    throws SQLException
  {
    pstmt.setQueryTimeout(seconds);
  }
  

  public boolean getMoreResults(int i)
    throws SQLException
  {
    return pstmt.getMoreResults(i);
  }
  

  public ResultSet getGeneratedKeys()
    throws SQLException
  {
    return pstmt.getGeneratedKeys();
  }
  

  public int executeUpdate(String s, int i)
    throws SQLException
  {
    return pstmt.executeUpdate(s, i);
  }
  

  public int executeUpdate(String s, int[] i)
    throws SQLException
  {
    return pstmt.executeUpdate(s, i);
  }
  

  public int executeUpdate(String s, String[] i)
    throws SQLException
  {
    return pstmt.executeUpdate(s, i);
  }
  

  public boolean execute(String s, int i)
    throws SQLException
  {
    return pstmt.execute(s, i);
  }
  

  public boolean execute(String s, int[] i)
    throws SQLException
  {
    return pstmt.execute(s, i);
  }
  

  public boolean execute(String s, String[] i)
    throws SQLException
  {
    return pstmt.execute(s, i);
  }
  

  public int getResultSetHoldability()
    throws SQLException
  {
    return pstmt.getResultSetHoldability();
  }
}
