package com.avaje.ebeaninternal.server.lib.sql;

import java.sql.SQLException;
import java.util.LinkedHashMap;
import java.util.Map.Entry;
import java.util.logging.Level;
import java.util.logging.Logger;




















public class PstmtCache
  extends LinkedHashMap<String, ExtendedPreparedStatement>
{
  private static final Logger logger = Logger.getLogger(PstmtCache.class.getName());
  



  static final long serialVersionUID = -3096406924865550697L;
  



  final String cacheName;
  


  final int maxSize;
  


  int removeCounter;
  


  int hitCounter;
  


  int missCounter;
  


  int putCounter;
  



  public PstmtCache(String cacheName, int maxCacheSize)
  {
    super(maxCacheSize * 3, 0.75F, true);
    this.cacheName = cacheName;
    maxSize = maxCacheSize;
  }
  


  public String getDescription()
  {
    return cacheName + " size:" + size() + " max:" + maxSize + " totalHits:" + hitCounter + " hitRatio:" + getHitRatio() + " removes:" + removeCounter;
  }
  


  public int getMaxSize()
  {
    return maxSize;
  }
  



  public int getHitRatio()
  {
    if (hitCounter == 0) {
      return 0;
    }
    return hitCounter * 100 / (hitCounter + missCounter);
  }
  



  public int getHitCounter()
  {
    return hitCounter;
  }
  


  public int getMissCounter()
  {
    return missCounter;
  }
  


  public int getPutCounter()
  {
    return putCounter;
  }
  



  public ExtendedPreparedStatement get(Object key)
  {
    ExtendedPreparedStatement o = (ExtendedPreparedStatement)super.get(key);
    if (o == null) {
      missCounter += 1;
    } else {
      hitCounter += 1;
    }
    return o;
  }
  



  public ExtendedPreparedStatement remove(Object key)
  {
    ExtendedPreparedStatement o = (ExtendedPreparedStatement)super.remove(key);
    if (o == null) {
      missCounter += 1;
    } else {
      hitCounter += 1;
    }
    return o;
  }
  



  public ExtendedPreparedStatement put(String key, ExtendedPreparedStatement value)
  {
    putCounter += 1;
    return (ExtendedPreparedStatement)super.put(key, value);
  }
  





  protected boolean removeEldestEntry(Map.Entry<String, ExtendedPreparedStatement> eldest)
  {
    if (size() < maxSize) {
      return false;
    }
    
    removeCounter += 1;
    try
    {
      ExtendedPreparedStatement pstmt = (ExtendedPreparedStatement)eldest.getValue();
      pstmt.closeDestroy();
    } catch (SQLException e) {
      logger.log(Level.SEVERE, "Error closing ExtendedPreparedStatement", e);
    }
    return true;
  }
}
