package com.avaje.ebeaninternal.server.lib.util;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Set;





























public class Dnode
{
  int level = 0;
  
  String nodeName;
  
  String nodeContent;
  
  ArrayList<Dnode> children;
  
  LinkedHashMap<String, String> attrList = new LinkedHashMap();
  








  public String toXml()
  {
    StringBuilder sb = new StringBuilder();
    generate(sb);
    return sb.toString();
  }
  


  public StringBuilder generate(StringBuilder sb)
  {
    if (sb == null) {
      sb = new StringBuilder();
    }
    sb.append("<").append(nodeName);
    Iterator<String> it = attributeNames();
    while (it.hasNext()) {
      String attr = (String)it.next();
      Object attrValue = getAttribute(attr);
      sb.append(" ").append(attr).append("=\"");
      if (attrValue != null) {
        sb.append(attrValue);
      }
      sb.append("\"");
    }
    
    if ((nodeContent == null) && (!hasChildren())) {
      sb.append(" />");
    }
    else {
      sb.append(">");
      if ((children != null) && (children.size() > 0)) {
        for (int i = 0; i < children.size(); i++) {
          Dnode child = (Dnode)children.get(i);
          child.generate(sb);
        }
      }
      if (nodeContent != null) {
        sb.append(nodeContent);
      }
      sb.append("</").append(nodeName).append(">");
    }
    return sb;
  }
  


  public String getNodeName()
  {
    return nodeName;
  }
  


  public void setNodeName(String nodeName)
  {
    this.nodeName = nodeName;
  }
  


  public String getNodeContent()
  {
    return nodeContent;
  }
  


  public void setNodeContent(String nodeContent)
  {
    this.nodeContent = nodeContent;
  }
  


  public boolean hasChildren()
  {
    return getChildrenCount() > 0;
  }
  


  public int getChildrenCount()
  {
    if (children == null) {
      return 0;
    }
    return children.size();
  }
  


  public boolean remove(Dnode node)
  {
    if (children == null) {
      return false;
    }
    if (children.remove(node)) {
      return true;
    }
    Iterator<Dnode> it = children.iterator();
    while (it.hasNext()) {
      Dnode child = (Dnode)it.next();
      if (child.remove(node)) {
        return true;
      }
    }
    return false;
  }
  


  public List<Dnode> children()
  {
    if (children == null) {
      return null;
    }
    return children;
  }
  


  public void addChild(Dnode child)
  {
    if (children == null) {
      children = new ArrayList();
    }
    children.add(child);
    child.setLevel(level + 1);
  }
  


  public int getLevel()
  {
    return level;
  }
  


  public void setLevel(int level)
  {
    this.level = level;
    if (children != null) {
      for (int i = 0; i < children.size(); i++) {
        Dnode child = (Dnode)children.get(i);
        child.setLevel(level + 1);
      }
    }
  }
  



  public Dnode find(String nodeName)
  {
    return find(nodeName, null, null);
  }
  




  public Dnode find(String nodeName, String attrName, Object value)
  {
    return find(nodeName, attrName, value, -1);
  }
  







  public Dnode find(String nodeName, String attrName, Object value, int maxLevel)
  {
    ArrayList<Dnode> list = new ArrayList();
    findByNode(list, nodeName, true, attrName, value, maxLevel);
    if (list.size() >= 1) {
      return (Dnode)list.get(0);
    }
    return null;
  }
  



  public List<Dnode> findAll(String nodeName, int maxLevel)
  {
    int level = -1;
    if (maxLevel > 0) {
      level = this.level + maxLevel;
    }
    return findAll(nodeName, null, null, level);
  }
  



  public List<Dnode> findAll(String nodeName, String attrName, Object value, int maxLevel)
  {
    if ((nodeName == null) && (attrName == null)) {
      throw new RuntimeException("You can not have both nodeName and attrName null");
    }
    ArrayList<Dnode> list = new ArrayList();
    findByNode(list, nodeName, false, attrName, value, maxLevel);
    return list;
  }
  



  private void findByNode(List<Dnode> list, String node, boolean findOne, String attrName, Object value, int maxLevel)
  {
    if ((findOne) && (list.size() == 1)) {
      return;
    }
    if (((node == null) || (node.equals(nodeName))) && (
      (attrName == null) || (value.equals(getAttribute(attrName))))) {
      list.add(this);
      if (findOne) {
        return;
      }
    }
    
    if ((maxLevel <= 0) || (level < maxLevel))
    {

      if (children != null)
      {
        for (int i = 0; i < children.size(); i++) {
          Dnode child = (Dnode)children.get(i);
          child.findByNode(list, node, findOne, attrName, value, maxLevel);
        }
      }
    }
  }
  

  public Iterator<String> attributeNames()
  {
    return attrList.keySet().iterator();
  }
  


  public String getAttribute(String name)
  {
    return (String)attrList.get(name);
  }
  





  public String getStringAttr(String name, String defaultValue)
  {
    Object o = attrList.get(name);
    if (o == null) {
      return defaultValue;
    }
    return o.toString();
  }
  



  public void setAttribute(String name, String value)
  {
    attrList.put(name, value);
  }
  
  public String toString() {
    StringBuilder sb = new StringBuilder();
    sb.append("[").append(getNodeName()).append(" ").append(attrList).append("]");
    return sb.toString();
  }
}
