package com.avaje.ebeaninternal.server.loadcontext;

import com.avaje.ebean.bean.BeanLoader;
import com.avaje.ebean.bean.EntityBeanIntercept;
import com.avaje.ebean.bean.ObjectGraphNode;
import com.avaje.ebean.bean.PersistenceContext;
import com.avaje.ebeaninternal.api.LoadBeanContext;
import com.avaje.ebeaninternal.api.LoadBeanRequest;
import com.avaje.ebeaninternal.api.LoadContext;
import com.avaje.ebeaninternal.api.SpiEbeanServer;
import com.avaje.ebeaninternal.api.SpiQuery;
import com.avaje.ebeaninternal.server.core.OrmQueryRequest;
import com.avaje.ebeaninternal.server.deploy.BeanDescriptor;
import com.avaje.ebeaninternal.server.querydefn.OrmQueryProperties;
import java.util.List;



























public class DLoadBeanContext
  implements LoadBeanContext, BeanLoader
{
  protected final DLoadContext parent;
  protected final BeanDescriptor<?> desc;
  protected final String path;
  protected final String fullPath;
  private final DLoadWeakList<EntityBeanIntercept> weakList;
  private final OrmQueryProperties queryProps;
  private int batchSize;
  
  public DLoadBeanContext(DLoadContext parent, BeanDescriptor<?> desc, String path, int batchSize, OrmQueryProperties queryProps)
  {
    this.parent = parent;
    this.desc = desc;
    this.path = path;
    this.batchSize = batchSize;
    this.queryProps = queryProps;
    weakList = new DLoadWeakList();
    
    if (parent.getRelativePath() == null) {
      fullPath = path;
    } else {
      fullPath = (parent.getRelativePath() + "." + path);
    }
  }
  

  public void configureQuery(SpiQuery<?> query, String lazyLoadProperty)
  {
    query.setParentState(parent.getParentState());
    query.setParentNode(getObjectGraphNode());
    query.setLazyLoadProperty(lazyLoadProperty);
    
    if (queryProps != null) {
      queryProps.configureBeanQuery(query);
    }
    if (parent.isUseAutofetchManager()) {
      query.setAutofetch(true);
    }
  }
  
  public String getFullPath() {
    return fullPath;
  }
  
  public PersistenceContext getPersistenceContext() {
    return parent.getPersistenceContext();
  }
  
  public OrmQueryProperties getQueryProps() {
    return queryProps;
  }
  
  public ObjectGraphNode getObjectGraphNode() {
    return parent.getObjectGraphNode(path);
  }
  
  public String getPath() {
    return path;
  }
  
  public String getName() {
    return parent.getEbeanServer().getName();
  }
  
  public int getBatchSize() {
    return batchSize;
  }
  
  public void setBatchSize(int batchSize) {
    this.batchSize = batchSize;
  }
  
  public BeanDescriptor<?> getBeanDescriptor() {
    return desc;
  }
  
  public LoadContext getGraphContext() {
    return parent;
  }
  
  public void register(EntityBeanIntercept ebi) {
    int pos = weakList.add(ebi);
    ebi.setBeanLoader(pos, this);
  }
  
  public void loadBean(EntityBeanIntercept ebi)
  {
    if (desc.lazyLoadMany(ebi))
    {
      return;
    }
    
    int position = ebi.getBeanLoaderIndex();
    

    List<EntityBeanIntercept> batch = weakList.getLoadBatch(position, batchSize);
    
    LoadBeanRequest req = new LoadBeanRequest(this, batch, null, batchSize, true, ebi.getLazyLoadProperty());
    parent.getEbeanServer().loadBean(req);
  }
  
  public void loadSecondaryQuery(OrmQueryRequest<?> parentRequest, int requestedBatchSize, boolean all)
  {
    for (;;) {
      List<EntityBeanIntercept> batch = weakList.getNextBatch(requestedBatchSize);
      if (batch.size() == 0) {
        break;
      }
      
      LoadBeanRequest req = new LoadBeanRequest(this, batch, parentRequest.getTransaction(), requestedBatchSize, false, null);
      parent.getEbeanServer().loadBean(req);
      if (!all) {
        break;
      }
    }
  }
}
