package com.avaje.ebeaninternal.server.loadcontext;

import com.avaje.ebean.bean.BeanCollection;
import com.avaje.ebean.bean.BeanCollectionLoader;
import com.avaje.ebean.bean.ObjectGraphNode;
import com.avaje.ebean.bean.PersistenceContext;
import com.avaje.ebeaninternal.api.LoadManyContext;
import com.avaje.ebeaninternal.api.LoadManyRequest;
import com.avaje.ebeaninternal.api.SpiEbeanServer;
import com.avaje.ebeaninternal.api.SpiQuery;
import com.avaje.ebeaninternal.server.core.OrmQueryRequest;
import com.avaje.ebeaninternal.server.deploy.BeanDescriptor;
import com.avaje.ebeaninternal.server.deploy.BeanPropertyAssocMany;
import com.avaje.ebeaninternal.server.querydefn.OrmQueryProperties;
import java.util.List;



























public class DLoadManyContext
  implements LoadManyContext, BeanCollectionLoader
{
  protected final DLoadContext parent;
  protected final String fullPath;
  private final BeanDescriptor<?> desc;
  private final BeanPropertyAssocMany<?> property;
  private final String path;
  private final int batchSize;
  private final OrmQueryProperties queryProps;
  private final DLoadWeakList<BeanCollection<?>> weakList;
  
  public DLoadManyContext(DLoadContext parent, BeanPropertyAssocMany<?> p, String path, int batchSize, OrmQueryProperties queryProps)
  {
    this.parent = parent;
    property = p;
    desc = p.getBeanDescriptor();
    this.path = path;
    this.batchSize = batchSize;
    this.queryProps = queryProps;
    weakList = new DLoadWeakList();
    
    if (parent.getRelativePath() == null) {
      fullPath = path;
    } else {
      fullPath = (parent.getRelativePath() + "." + path);
    }
  }
  


  public void configureQuery(SpiQuery<?> query)
  {
    query.setParentState(parent.getParentState());
    query.setParentNode(getObjectGraphNode());
    
    if (queryProps != null) {
      queryProps.configureManyQuery(query);
    }
    
    if (parent.isUseAutofetchManager()) {
      query.setAutofetch(true);
    }
  }
  



  public ObjectGraphNode getObjectGraphNode()
  {
    int pos = path.lastIndexOf('.');
    if (pos == -1) {
      return parent.getObjectGraphNode(null);
    }
    String parentPath = path.substring(0, pos);
    return parent.getObjectGraphNode(parentPath);
  }
  
  public String getFullPath()
  {
    return fullPath;
  }
  
  public PersistenceContext getPersistenceContext() {
    return parent.getPersistenceContext();
  }
  
  public int getBatchSize() {
    return batchSize;
  }
  
  public BeanPropertyAssocMany<?> getBeanProperty() {
    return property;
  }
  
  public BeanDescriptor<?> getBeanDescriptor() {
    return desc;
  }
  
  public String getPath() {
    return path;
  }
  
  public String getName() {
    return parent.getEbeanServer().getName();
  }
  
  public void register(BeanCollection<?> bc) {
    int pos = weakList.add(bc);
    bc.setLoader(pos, this);
  }
  
  public void loadMany(BeanCollection<?> bc, boolean onlyIds)
  {
    int position = bc.getLoaderIndex();
    
    List<BeanCollection<?>> loadBatch = weakList.getLoadBatch(position, batchSize);
    
    LoadManyRequest req = new LoadManyRequest(this, loadBatch, null, batchSize, true, onlyIds);
    parent.getEbeanServer().loadMany(req);
  }
  
  public void loadSecondaryQuery(OrmQueryRequest<?> parentRequest, int requestedBatchSize, boolean all)
  {
    for (;;) {
      List<BeanCollection<?>> batch = weakList.getNextBatch(requestedBatchSize);
      if (batch.size() == 0) {
        break;
      }
      
      LoadManyRequest req = new LoadManyRequest(this, batch, parentRequest.getTransaction(), requestedBatchSize, false, false);
      parent.getEbeanServer().loadMany(req);
      if (!all) {
        break;
      }
    }
  }
}
