package com.avaje.ebeaninternal.server.loadcontext;

import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.List;




















public class DLoadWeakList<T>
{
  protected final ArrayList<WeakReference<T>> list = new ArrayList();
  

  protected int removedFromTop;
  


  public int add(T e)
  {
    synchronized (this) {
      int i = list.size();
      list.add(new WeakReference(e));
      return i;
    }
  }
  


  public List<T> getNextBatch(int batchSize)
  {
    synchronized (this) {
      return getBatch(0, batchSize);
    }
  }
  


  public List<T> getLoadBatch(int position, int batchSize)
  {
    synchronized (this) {
      if (batchSize < 1) {
        throw new RuntimeException("batchSize " + batchSize + " < 1 ??!!");
      }
      
      int relativePos = position - removedFromTop;
      if (relativePos - batchSize < 0) {
        relativePos = 0;
      }
      if ((relativePos > 0) && (relativePos + batchSize > list.size())) {
        relativePos = list.size() - batchSize;
        if (relativePos < 0) {
          relativePos = 0;
        }
      }
      
      return getBatch(relativePos, batchSize);
    }
  }
  
  private List<T> getBatch(int relativePos, int batchSize)
  {
    ArrayList<T> batch = new ArrayList();
    
    int count = 0;
    boolean removeFromTop = relativePos == 0;
    
    while ((count < batchSize) && 
      (!list.isEmpty()))
    {
      WeakReference<T> weakEntry;
      
      if (removeFromTop) {
        WeakReference<T> weakEntry = (WeakReference)list.remove(relativePos);
        removedFromTop += 1;
      }
      else {
        if (relativePos >= list.size()) {
          break;
        }
        weakEntry = (WeakReference)list.get(relativePos);
        list.set(relativePos, null);
        relativePos++;
      }
      T ebi = weakEntry == null ? null : weakEntry.get();
      
      if (ebi != null) {
        batch.add(ebi);
        count++;
      }
    }
    
    return batch;
  }
}
