package com.avaje.ebeaninternal.server.lucene;

import com.avaje.ebeaninternal.api.SpiEbeanServer;
import com.avaje.ebeaninternal.api.TransactionEventTable.TableIUD;
import com.avaje.ebeaninternal.server.deploy.BeanDescriptor;
import com.avaje.ebeaninternal.server.transaction.BeanDeltaList;
import com.avaje.ebeaninternal.server.transaction.BeanPersistIds;
import com.avaje.ebeaninternal.server.transaction.DeleteByIdMap;
import com.avaje.ebeaninternal.server.transaction.IndexInvalidate;
import com.avaje.ebeaninternal.server.transaction.RemoteTransactionEvent;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;






















public class IndexUpdatesBuilder
{
  private final SpiEbeanServer server;
  private final Map<String, IndexUpdates> map;
  private final RemoteTransactionEvent txnEvent;
  
  public static Collection<IndexUpdates> create(SpiEbeanServer server, RemoteTransactionEvent txnEvent)
  {
    return new IndexUpdatesBuilder(server, txnEvent).create();
  }
  
  private IndexUpdatesBuilder(SpiEbeanServer server, RemoteTransactionEvent txnEvent) {
    map = new HashMap();
    this.server = server;
    this.txnEvent = txnEvent;
  }
  
  private Collection<IndexUpdates> create()
  {
    Set<IndexInvalidate> indexInvalidations = txnEvent.getIndexInvalidations();
    if (indexInvalidations != null) {
      for (IndexInvalidate indexInvalidate : indexInvalidations) {
        LIndex index = server.getLuceneIndexManager().getIndex(indexInvalidate.getIndexName());
        BeanDescriptor<?> d = index.getBeanDescriptor();
        getEventByType(d).setInvalidate(true);
      }
    }
    
    List<TransactionEventTable.TableIUD> tableIUDList = txnEvent.getTableIUDList();
    if (tableIUDList != null) {
      for (int i = 0; i < tableIUDList.size(); i++) {
        TransactionEventTable.TableIUD tableIUD = (TransactionEventTable.TableIUD)tableIUDList.get(i);
        List<BeanDescriptor<?>> descList = server.getBeanDescriptors(tableIUD.getTable());
        if (descList != null) {
          for (int j = 0; j < descList.size(); j++) {
            BeanDescriptor<?> d = (BeanDescriptor)descList.get(j);
            getEventByType(d).addTableIUD(tableIUD);
          }
        }
      }
    }
    
    DeleteByIdMap deleteByIdMap = txnEvent.getDeleteByIdMap();
    if (deleteByIdMap != null) {
      for (BeanPersistIds delIds : deleteByIdMap.values()) {
        getEventByType(delIds.getBeanDescriptor()).setDeleteIds(delIds);
      }
    }
    
    List<BeanPersistIds> beanPersistList = txnEvent.getBeanPersistList();
    if (beanPersistList != null) {
      for (int i = 0; i < beanPersistList.size(); i++) {
        BeanPersistIds b = (BeanPersistIds)beanPersistList.get(i);
        getEventByType(b.getBeanDescriptor()).setBeanPersistIds(b);
      }
    }
    
    List<BeanDeltaList> beanDeltaLists = txnEvent.getBeanDeltaLists();
    if (beanDeltaLists != null) {
      for (int i = 0; i < beanDeltaLists.size(); i++) {
        BeanDeltaList d = (BeanDeltaList)beanDeltaLists.get(i);
        getEventByType(d.getBeanDescriptor()).setDeltaList(d);
      }
    }
    
    return map.values();
  }
  
  private IndexUpdates getEventByType(BeanDescriptor<?> d)
  {
    String beanDescKey = d.getBeanType().getName();
    IndexUpdates eventByType = (IndexUpdates)map.get(beanDescKey);
    if (eventByType == null) {
      eventByType = new IndexUpdates(d);
      map.put(beanDescKey, eventByType);
    }
    return eventByType;
  }
}
