package com.avaje.ebeaninternal.server.lucene;

import com.avaje.ebeaninternal.server.deploy.BeanProperty;
import com.avaje.ebeaninternal.server.el.ElPropertyValue;
import com.avaje.ebeaninternal.server.query.SplitName;
import com.avaje.ebeaninternal.server.type.ScalarType;
import java.util.Set;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Fieldable;
import org.apache.lucene.queryParser.QueryParser;
import org.apache.lucene.util.Version;




























public abstract class LIndexFieldBase
  implements LIndexField
{
  protected final Analyzer queryAnalyzer;
  protected final String fieldName;
  protected final String propertyName;
  protected final int luceneType;
  protected final int sortType;
  protected final ElPropertyValue property;
  protected final ScalarType<?> scalarType;
  protected final FieldFactory fieldFactory;
  protected final boolean indexed;
  protected final boolean stored;
  protected final boolean tokenized;
  
  public LIndexFieldBase(Analyzer queryAnalyzer, String fieldName, int luceneType, ElPropertyValue property, FieldFactory fieldFactory)
  {
    this.queryAnalyzer = queryAnalyzer;
    this.fieldName = fieldName;
    this.luceneType = luceneType;
    sortType = getSortType(luceneType);
    this.property = property;
    this.fieldFactory = fieldFactory;
    
    Fieldable fieldPrototype = fieldFactory.createFieldable();
    indexed = fieldPrototype.isIndexed();
    stored = fieldPrototype.isStored();
    tokenized = fieldPrototype.isTokenized();
    
    if (property == null) {
      scalarType = null;
      propertyName = null;
    } else {
      scalarType = property.getBeanProperty().getScalarType();
      propertyName = SplitName.add(property.getElPrefix(), property.getName());
    }
  }
  
  public String toString() {
    return propertyName;
  }
  
  public void addIndexRequiredPropertyNames(Set<String> requiredPropertyNames) {
    if (propertyName != null) {
      requiredPropertyNames.add(propertyName);
    }
  }
  
  public int getSortType() {
    return sortType;
  }
  
  public QueryParser createQueryParser() {
    return new QueryParser(Version.LUCENE_30, fieldName, queryAnalyzer);
  }
  
  public String getName() {
    return fieldName;
  }
  
  public boolean isIndexed() {
    return indexed;
  }
  
  public boolean isStored() {
    return stored;
  }
  
  public boolean isTokenized() {
    return tokenized;
  }
  
  public boolean isBeanProperty() {
    return property != null;
  }
  
  public int getPropertyOrder() {
    return property == null ? 0 : property.getDeployOrder();
  }
  
  public ElPropertyValue getElBeanProperty() {
    return property;
  }
  
  public void readValue(Document doc, Object bean)
  {
    Object v = readIndexValue(doc);
    if (v != null) {
      v = scalarType.luceneFromIndexValue(v);
    }
    property.elSetValue(bean, v, true, false);
  }
  
  protected Object readIndexValue(Document doc)
  {
    String s = doc.get(fieldName);
    if (s == null) {
      return null;
    }
    
    switch (luceneType) {
    case 1: 
      return Integer.valueOf(Integer.parseInt(s));
    
    case 2: 
      return Long.valueOf(Long.parseLong(s));
    
    case 5: 
      return Long.valueOf(Long.parseLong(s));
    
    case 6: 
      return Long.valueOf(Long.parseLong(s));
    
    case 3: 
      return Double.valueOf(Double.parseDouble(s));
    
    case 4: 
      return Float.valueOf(Float.parseFloat(s));
    }
    
    throw new RuntimeException("Unhandled type " + luceneType);
  }
  
  private int getSortType(int luceneType)
  {
    switch (luceneType) {
    case 1: 
      return 4;
    case 2: 
      return 6;
    case 5: 
      return 6;
    case 6: 
      return 6;
    case 3: 
      return 7;
    case 4: 
      return 5;
    case 0: 
      return 3;
    }
    
    return -1;
  }
}
