package com.avaje.ebeaninternal.server.lucene;

import com.avaje.ebeaninternal.server.el.ElPropertyValue;
import com.avaje.ebeaninternal.server.type.ScalarType;
import java.io.PrintStream;
import java.io.StringReader;
import java.util.Set;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.TokenStream;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;



















public final class LIndexFieldString
  extends LIndexFieldBase
{
  private final Analyzer indexAnalyzer;
  
  public LIndexFieldString(Analyzer queryAnalyzer, String fieldName, FieldFactory fieldFactory, ElPropertyValue property, Analyzer indexAnalyzer)
  {
    super(queryAnalyzer, fieldName, 0, property, fieldFactory);
    this.indexAnalyzer = indexAnalyzer;
  }
  
  public void addIndexResolvePropertyNames(Set<String> resolvePropertyNames) {
    if ((propertyName != null) && (isIndexed())) {
      resolvePropertyNames.add(propertyName);
    }
  }
  
  public void addIndexRestorePropertyNames(Set<String> restorePropertyNames) {
    if ((propertyName != null) && (isStored())) {
      restorePropertyNames.add(propertyName);
    }
  }
  
  public String getSortableProperty() {
    if ((isIndexed()) && (!isTokenized()))
    {

      return propertyName;
    }
    return null;
  }
  

  public void readValue(Document doc, Object bean)
  {
    Object v = doc.get(fieldName);
    if (v != null) {
      v = scalarType.luceneFromIndexValue(v);
    }
    property.elSetValue(bean, v, true, false);
  }
  
  public DocFieldWriter createDocFieldWriter()
  {
    Field f = (Field)fieldFactory.createFieldable();
    return new Writer(property, scalarType, f, indexAnalyzer);
  }
  
  private static class Writer implements DocFieldWriter
  {
    private final ElPropertyValue property;
    private final ScalarType<?> scalarType;
    private final Field field;
    private final Analyzer indexAnalyzer;
    
    Writer(ElPropertyValue property, ScalarType<?> scalarType, Field field, Analyzer indexAnalyzer) {
      this.property = property;
      this.scalarType = scalarType;
      this.field = field;
      this.indexAnalyzer = indexAnalyzer;
    }
    
    public void writeValue(Object bean, Document document)
    {
      Object value = property.elGetValue(bean);
      
      if (value != null)
      {

        System.out.println("- write " + field.name() + " " + value);
        
        String s = (String)scalarType.luceneToIndexValue(value);
        if (indexAnalyzer == null) {
          field.setValue(s);
        }
        else {
          StringReader sr = new StringReader(s);
          TokenStream tokenStream = indexAnalyzer.tokenStream(field.name(), sr);
          field.setTokenStream(tokenStream);
        }
        document.add(field);
      }
    }
  }
}
