package com.avaje.ebeaninternal.server.lucene;

import com.avaje.ebeaninternal.server.deploy.BeanProperty;
import com.avaje.ebeaninternal.server.el.ElPropertyValue;
import com.avaje.ebeaninternal.server.query.SplitName;
import com.avaje.ebeaninternal.server.type.ScalarType;
import java.io.StringReader;
import java.util.Set;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.TokenStream;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;





















public final class LIndexFieldStringConcat
  extends LIndexFieldBase
{
  private final ElPropertyValue[] properties;
  private final ScalarType<?>[] scalarTypes;
  private final Analyzer indexAnalyzer;
  
  public LIndexFieldStringConcat(Analyzer queryAnalyzer, String fieldName, FieldFactory fieldFactory, ElPropertyValue[] properties, Analyzer indexAnalyzer)
  {
    super(queryAnalyzer, fieldName, 0, null, fieldFactory);
    this.properties = properties;
    this.indexAnalyzer = indexAnalyzer;
    scalarTypes = new ScalarType[properties.length];
    for (int i = 0; i < scalarTypes.length; i++) {
      scalarTypes[i] = properties[i].getBeanProperty().getScalarType();
    }
  }
  
  public void addIndexRequiredPropertyNames(Set<String> requiredPropertyNames)
  {
    for (int i = 0; i < properties.length; i++) {
      String prefix = properties[i].getElPrefix();
      String name = properties[i].getName();
      String fullPath = SplitName.add(prefix, name);
      requiredPropertyNames.add(fullPath);
    }
  }
  


  public void addIndexResolvePropertyNames(Set<String> resolvePropertyNames) {}
  

  public void addIndexRestorePropertyNames(Set<String> restorePropertyNames) {}
  

  public String getSortableProperty()
  {
    return null;
  }
  
  public boolean isBeanProperty() {
    return false;
  }
  

  public void readValue(Document doc, Object bean) {}
  
  public DocFieldWriter createDocFieldWriter()
  {
    Field f = (Field)fieldFactory.createFieldable();
    return new Writer(f, properties, scalarTypes, indexAnalyzer, null);
  }
  
  private static class Writer implements DocFieldWriter
  {
    private final Field field;
    private final ElPropertyValue[] properties;
    private final ScalarType<?>[] scalarTypes;
    private final Analyzer indexAnalyzer;
    
    private Writer(Field field, ElPropertyValue[] properties, ScalarType<?>[] scalarTypes, Analyzer indexAnalyzer)
    {
      this.field = field;
      this.properties = properties;
      this.scalarTypes = scalarTypes;
      this.indexAnalyzer = indexAnalyzer;
    }
    
    public void writeValue(Object bean, Document document)
    {
      StringBuilder sb = new StringBuilder();
      for (int i = 0; i < properties.length; i++) {
        Object value = properties[i].elGetValue(bean);
        if (value != null) {
          String s = (String)scalarTypes[i].luceneToIndexValue(value);
          sb.append(s);
          sb.append(" ");
        }
      }
      

      String s = sb.toString();
      if (indexAnalyzer == null) {
        field.setValue(s);
      }
      else {
        StringReader sr = new StringReader(s);
        TokenStream tokenStream = indexAnalyzer.tokenStream(field.name(), sr);
        field.setTokenStream(tokenStream);
      }
      document.add(field);
    }
  }
}
