package com.avaje.ebeaninternal.server.lucene;

import com.avaje.ebean.config.lucene.IndexDefnBuilder;
import com.avaje.ebean.config.lucene.IndexFieldDefn;
import com.avaje.ebean.config.lucene.IndexFieldDefn.Sortable;
import com.avaje.ebeaninternal.server.deploy.BeanDescriptor;
import com.avaje.ebeaninternal.server.deploy.BeanProperty;
import com.avaje.ebeaninternal.server.deploy.BeanPropertyAssocMany;
import com.avaje.ebeaninternal.server.deploy.BeanPropertyAssocOne;
import com.avaje.ebeaninternal.server.el.ElPropertyValue;
import com.avaje.ebeaninternal.server.type.ScalarType;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import org.apache.lucene.document.Field.Index;
import org.apache.lucene.document.Field.Store;























public class LIndexFieldsBuilder
  implements SpiIndexDefnHelper
{
  private final BeanDescriptor<?> desc;
  private final LinkedHashMap<String, IndexFieldDefn> fieldDefnMap = new LinkedHashMap();
  
  public LIndexFieldsBuilder(BeanDescriptor<?> desc) {
    this.desc = desc;
  }
  
  public Nested assocOne(String propertyName)
  {
    BeanProperty beanProperty = desc.getBeanProperty(propertyName);
    if ((beanProperty instanceof BeanPropertyAssocOne)) {
      BeanPropertyAssocOne<?> assocOne = (BeanPropertyAssocOne)beanProperty;
      BeanDescriptor<?> targetDescriptor = assocOne.getTargetDescriptor();
      return new Nested(this, propertyName, targetDescriptor);
    }
    
    throw new IllegalArgumentException("Expecing " + propertyName + " to be an AssocOne property of " + desc.getFullName());
  }
  
  public void addAllFields()
  {
    Iterator<BeanProperty> it = desc.propertiesAll();
    while (it.hasNext()) {
      BeanProperty beanProperty = (BeanProperty)it.next();
      if (!(beanProperty instanceof BeanPropertyAssocMany))
      {
        if (!(beanProperty instanceof BeanPropertyAssocOne))
        {

          addField(beanProperty.getName()); }
      }
    }
  }
  
  public IndexFieldDefn addField(String propertyName) {
    return addField(propertyName, null);
  }
  
  public IndexFieldDefn addField(String propertyName, IndexFieldDefn.Sortable sortable) {
    return addField(propertyName, null, null, sortable);
  }
  
  public IndexFieldDefn addFieldConcat(String fieldName, String... propertyNames) {
    return addFieldConcat(fieldName, Field.Store.NO, Field.Index.ANALYZED, propertyNames);
  }
  
  public IndexFieldDefn addFieldConcat(String fieldName, Field.Store store, Field.Index index, String... propertyNames) {
    return addPrefixFieldConcat(null, fieldName, store, index, propertyNames);
  }
  
  public IndexFieldDefn addPrefixField(String prefix, String propertyName, Field.Store store, Field.Index index, IndexFieldDefn.Sortable sortable) {
    String fullPath = prefix + "." + propertyName;
    return addField(fullPath, store, index, sortable);
  }
  
  public IndexFieldDefn addPrefixFieldConcat(String prefix, String fieldName, Field.Store store, Field.Index index, String[] propertyNames)
  {
    if (prefix != null) {
      for (int i = 0; i < propertyNames.length; i++) {
        propertyNames[i] = (prefix + "." + propertyNames);
      }
    }
    
    IndexFieldDefn fieldDefn = new IndexFieldDefn(fieldName, store, index, null);
    fieldDefn.setPropertyNames(propertyNames);
    
    fieldDefnMap.put(fieldName, fieldDefn);
    return fieldDefn;
  }
  
  public IndexFieldDefn addField(IndexFieldDefn fieldDefn) {
    fieldDefnMap.put(fieldDefn.getName(), fieldDefn);
    return fieldDefn;
  }
  
  public IndexFieldDefn addField(String propertyName, Field.Store store, Field.Index index, IndexFieldDefn.Sortable sortable)
  {
    ElPropertyValue prop = desc.getElGetValue(propertyName);
    if (prop == null) {
      String msg = "Property [" + propertyName + "] not found on " + desc.getFullName();
      throw new NullPointerException(msg);
    }
    BeanProperty beanProperty = prop.getBeanProperty();
    ScalarType<?> scalarType = beanProperty.getScalarType();
    
    if (store == null) {
      store = isLob(scalarType) ? Field.Store.NO : Field.Store.YES;
    }
    
    boolean luceneStringType = (beanProperty.isId()) || (isLuceneString(scalarType.getLuceneType()));
    if (index == null) {
      if ((beanProperty.isId()) || (!luceneStringType)) {
        index = Field.Index.NOT_ANALYZED;
      } else {
        index = Field.Index.ANALYZED;
      }
    }
    
    IndexFieldDefn fieldDefn = new IndexFieldDefn(propertyName, store, index, sortable);
    fieldDefnMap.put(propertyName, fieldDefn);
    
    if ((luceneStringType) && (index.isAnalyzed()) && (IndexFieldDefn.Sortable.YES.equals(sortable))) {
      IndexFieldDefn extraFieldDefn = new IndexFieldDefn(propertyName + "_sortonly", Field.Store.NO, Field.Index.NOT_ANALYZED, IndexFieldDefn.Sortable.YES);
      extraFieldDefn.setPropertyName(propertyName);
      fieldDefnMap.put(extraFieldDefn.getName(), extraFieldDefn);
    }
    
    return fieldDefn;
  }
  
  public IndexFieldDefn getField(String fieldName) {
    return (IndexFieldDefn)fieldDefnMap.get(fieldName);
  }
  
  private boolean isLuceneString(int luceneType) {
    return 0 == luceneType;
  }
  
  private boolean isLob(ScalarType<?> scalarType) {
    int jdbcType = scalarType.getJdbcType();
    switch (jdbcType) {
    case 2005: 
      return true;
    case 2004: 
      return true;
    case -1: 
      return true;
    case -4: 
      return true;
    }
    
    return false;
  }
  

  public List<IndexFieldDefn> getFields()
  {
    ArrayList<IndexFieldDefn> fields = new ArrayList();
    fields.addAll(fieldDefnMap.values());
    return fields;
  }
  
  private static class Nested implements SpiIndexDefnHelper
  {
    private final String path;
    private final BeanDescriptor<?> targetDescriptor;
    private final SpiIndexDefnHelper parent;
    
    Nested(SpiIndexDefnHelper parent, String path, BeanDescriptor<?> targetDescriptor) {
      this.parent = parent;
      this.path = path;
      this.targetDescriptor = targetDescriptor;
    }
    
    public IndexDefnBuilder assocOne(String propertyName)
    {
      BeanProperty beanProperty = this.targetDescriptor.getBeanProperty(propertyName);
      if ((beanProperty instanceof BeanPropertyAssocOne)) {
        BeanPropertyAssocOne<?> assocOne = (BeanPropertyAssocOne)beanProperty;
        BeanDescriptor<?> targetDescriptor = assocOne.getTargetDescriptor();
        return new Nested(this, propertyName, targetDescriptor);
      }
      throw new IllegalArgumentException("Expecing " + propertyName + " to be an AssocOne property of " + this.targetDescriptor.getFullName());
    }
    
    public void addAllFields()
    {
      Iterator<BeanProperty> it = targetDescriptor.propertiesAll();
      while (it.hasNext()) {
        BeanProperty beanProperty = (BeanProperty)it.next();
        if (!(beanProperty instanceof BeanPropertyAssocMany))
        {
          if (!(beanProperty instanceof BeanPropertyAssocOne))
          {

            if (!beanProperty.isTransient())
              addField(beanProperty.getName());
          }
        }
      }
    }
    
    public IndexFieldDefn addField(IndexFieldDefn fieldDefn) {
      parent.addField(fieldDefn);
      return fieldDefn;
    }
    
    public IndexFieldDefn addField(String propertyName) {
      return addField(propertyName, null);
    }
    
    public IndexFieldDefn addField(String propertyName, IndexFieldDefn.Sortable sortable) {
      return addField(propertyName, null, null, sortable);
    }
    
    public IndexFieldDefn addField(String propertyName, Field.Store store, Field.Index index, IndexFieldDefn.Sortable sortable) {
      return parent.addPrefixField(path, propertyName, store, index, sortable);
    }
    
    public IndexFieldDefn addFieldConcat(String fieldName, String... propertyNames) {
      return addFieldConcat(fieldName, Field.Store.NO, Field.Index.ANALYZED, propertyNames);
    }
    
    public IndexFieldDefn addFieldConcat(String fieldName, Field.Store store, Field.Index index, String... propertyNames) {
      return parent.addPrefixFieldConcat(path, fieldName, store, index, propertyNames);
    }
    
    public IndexFieldDefn addPrefixFieldConcat(String prefix, String fieldName, Field.Store store, Field.Index index, String[] propertyNames)
    {
      String nestedPath = path + "." + prefix;
      return parent.addPrefixFieldConcat(nestedPath, fieldName, store, index, propertyNames);
    }
    
    public IndexFieldDefn addPrefixField(String prefix, String propertyName, Field.Store store, Field.Index index, IndexFieldDefn.Sortable sortable) {
      String nestedPath = prefix + "." + propertyName;
      return addField(nestedPath, store, index, sortable);
    }
    
    public IndexFieldDefn getField(String fieldName) {
      return parent.getField(fieldName);
    }
    
    public List<IndexFieldDefn> getFields() {
      return parent.getFields();
    }
  }
}
