package com.avaje.ebeaninternal.server.lucene;

import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.apache.lucene.index.IndexCommit;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.store.Directory;




















public class LIndexIoSearcherDefault
  implements LIndexIoSearcher
{
  private static final Logger logger = Logger.getLogger(LIndexIoSearcherDefault.class.getName());
  
  private final String name;
  
  private final IndexWriter indexWriter;
  private volatile LIndexSearch indexSearch;
  
  public LIndexIoSearcherDefault(IndexWriter indexWriter, String name)
  {
    this.name = name;
    this.indexWriter = indexWriter;
    indexSearch = refreshIndexSearch();
  }
  
  public void postCommit() {
    try {
      refreshIndexSearch();
    } catch (Exception e) {
      String msg = "Error postCommit() refreshing IndexSearcher";
      logger.log(Level.SEVERE, msg, e);
    }
  }
  
  public void refresh(boolean nearRealTime)
  {
    try {
      refreshIndexSearch();
    } catch (Exception e) {
      String msg = "Error refreshing IndexSearch";
      logger.log(Level.SEVERE, msg, e);
    }
  }
  
  public LIndexVersion getLastestVersion()
  {
    LIndexSearch s = indexSearch;
    try {
      IndexCommit c = s.getIndexReader().getIndexCommit();
      return new LIndexVersion(c.getGeneration(), c.getVersion());
    }
    catch (IOException e) {
      throw new PersistenceLuceneException(e);
    }
  }
  








  public LIndexSearch getIndexSearch()
  {
    LIndexSearch s = indexSearch;
    
    if (s.isOpenAcquire()) {
      return s;
    }
    
    return refreshIndexSearch();
  }
  

  private LIndexSearch refreshIndexSearch()
  {
    synchronized (this)
    {
      try {
        LIndexSearch currentSearch = indexSearch;
        IndexReader newReader;
        IndexReader newReader;
        if (currentSearch == null)
        {
          newReader = createIndexReader();
        }
        else {
          newReader = currentSearch.getIndexReader().reopen();
          
          if (newReader == currentSearch.getIndexReader())
          {
            return currentSearch;
          }
        }
        

        IndexSearcher searcher = new IndexSearcher(newReader);
        


        LIndexSearch newSearch = new LIndexSearch(searcher, newReader);
        
        if (currentSearch != null) {
          currentSearch.markForClose();
        }
        logger.info("Lucene Searcher refreshed " + name);
        indexSearch = newSearch;
        return newSearch;
      }
      catch (IOException e) {
        throw new PersistenceLuceneException(e);
      }
    }
  }
  
  private IndexReader createIndexReader() {
    try {
      Directory directory = indexWriter.getDirectory();
      return IndexReader.open(directory);
    }
    catch (IOException e) {
      throw new PersistenceLuceneException(e);
    }
  }
}
