package com.avaje.ebeaninternal.server.lucene;

import java.util.logging.Level;
import java.util.logging.Logger;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.search.IndexSearcher;





















public class LIndexSearch
{
  private static final Logger logger = Logger.getLogger(LIndexSearch.class.getName());
  
  private final IndexSearcher indexSearcher;
  
  private final IndexReader indexReader;
  
  private int refCount;
  
  private boolean markForClose;
  private boolean closed;
  
  public LIndexSearch(IndexSearcher indexSearcher, IndexReader indexReader)
  {
    this.indexSearcher = indexSearcher;
    this.indexReader = indexReader;
  }
  
  public IndexSearcher getIndexSearcher() {
    return indexSearcher;
  }
  
  public IndexReader getIndexReader() {
    return indexReader;
  }
  
  public boolean isOpenAcquire() {
    synchronized (this) {
      if (markForClose) {
        return false;
      }
      refCount += 1;
      return true;
    }
  }
  
  public void releaseClose() {
    synchronized (this) {
      refCount -= 1;
      closeIfMarked();
    }
  }
  
  public void markForClose() {
    synchronized (this) {
      markForClose = true;
      closeIfMarked();
    }
  }
  
  private void closeIfMarked()
  {
    if ((markForClose) && (refCount <= 0) && (!closed))
    {
      closed = true;
      try {
        indexSearcher.close();
      } catch (Exception e) {
        logger.log(Level.WARNING, "Error when closing indexSearcher", e);
      }
      try {
        indexReader.close();
      } catch (Exception e) {
        logger.log(Level.WARNING, "Error when closing indexReader", e);
      }
    }
  }
}
