package com.avaje.ebeaninternal.server.lucene;

import com.avaje.ebean.OrderBy;
import com.avaje.ebean.OrderBy.Property;
import com.avaje.ebeaninternal.server.deploy.BeanDescriptor;
import com.avaje.ebeaninternal.server.query.LuceneResolvableRequest;
import java.util.List;
import org.apache.lucene.search.Sort;
import org.apache.lucene.search.SortField;






























public class LLuceneSortResolve
{
  private final LuceneResolvableRequest req;
  private final OrderBy<?> orderBy;
  private Sort sort;
  private boolean isResolved;
  private String unsortableField;
  
  public LLuceneSortResolve(LuceneResolvableRequest req, OrderBy<?> orderBy)
  {
    this.req = req;
    this.orderBy = orderBy;
    isResolved = resolve();
  }
  
  public boolean isResolved() {
    return isResolved;
  }
  
  public Sort getSort() {
    return sort;
  }
  
  public String getUnsortableField() {
    return unsortableField;
  }
  
  private boolean resolve()
  {
    BeanDescriptor<?> beanDescriptor = req.getBeanDescriptor();
    
    if (orderBy != null)
    {
      List<OrderBy.Property> properties = orderBy.getProperties();
      
      SortField[] sortFields = new SortField[properties.size()];
      
      for (int i = 0; i < properties.size(); i++) {
        OrderBy.Property property = (OrderBy.Property)properties.get(i);
        SortField sf = createSortField(property, beanDescriptor);
        if (sf == null)
        {
          unsortableField = property.getProperty();
          return false;
        }
        sortFields[i] = sf;
      }
      
      sort = new Sort(sortFields);
    }
    return true;
  }
  
  private SortField createSortField(OrderBy.Property property, BeanDescriptor<?> beanDescriptor)
  {
    String propName = property.getProperty();
    
    LIndexField sortField = req.getSortableProperty(propName);
    if (sortField == null)
    {
      return null;
    }
    
    int sortType = sortField.getSortType();
    return sortType == -1 ? null : new SortField(sortField.getName(), sortType, !property.isAscending());
  }
}
