package com.avaje.ebeaninternal.server.lucene.cluster;

import com.avaje.ebeaninternal.api.SpiEbeanServer;
import com.avaje.ebeaninternal.server.cluster.ClusterManager;
import com.avaje.ebeaninternal.server.lucene.LIndex;
import com.avaje.ebeaninternal.server.lucene.LIndexCommitInfo;
import com.avaje.ebeaninternal.server.lucene.LIndexFileInfo;
import com.avaje.ebeaninternal.server.lucene.LuceneIndexManager;
import java.io.BufferedInputStream;
import java.io.DataInput;
import java.io.DataInputStream;
import java.io.DataOutput;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.util.logging.Level;
import java.util.logging.Logger;




















public class SLuceneClusterSocketRequest
  implements Runnable, SLuceneSocketMessageTypes
{
  private static final Logger logger = Logger.getLogger(SLuceneClusterSocketRequest.class.getName());
  
  private final LuceneIndexManager manager;
  
  private final OutputStream os;
  
  private final DataInput dataInput;
  
  private final DataOutput dataOutput;
  
  private final String serverName;
  
  private final short msgType;
  private final String idxName;
  private final long remoteIndexVersion;
  
  public SLuceneClusterSocketRequest(ClusterManager clusterManager, Socket socket)
    throws IOException
  {
    os = socket.getOutputStream();
    InputStream is = socket.getInputStream();
    dataInput = new DataInputStream(is);
    dataOutput = new DataOutputStream(os);
    
    serverName = dataInput.readUTF();
    msgType = dataInput.readShort();
    idxName = dataInput.readUTF();
    remoteIndexVersion = dataInput.readLong();
    
    SpiEbeanServer server = (SpiEbeanServer)clusterManager.getServer(serverName);
    manager = server.getLuceneIndexManager();
  }
  
  public void run()
  {
    try {
      switch (msgType) {
      case 2: 
        obtainCommit();
        break;
      
      case 3: 
        releaseCommit();
        break;
      
      case 4: 
        getFile();
        break;
      
      default: 
        throw new IOException("Invalid msgType " + msgType);
      }
    }
    catch (IOException e) {
      String msg = "Error processing msg " + msgType + " " + idxName;
      logger.log(Level.SEVERE, msg, e);
    }
    finally {
      flush();
    }
  }
  
  private void flush() {
    try {
      os.flush();
    } catch (IOException e) {
      String msg = "Error flushing Socket OuputStream";
      logger.log(Level.SEVERE, msg, e);
    }
    try {
      os.close();
    } catch (IOException e) {
      String msg = "Error closing Socket OuputStream";
      logger.log(Level.SEVERE, msg, e);
    }
  }
  




  private void releaseCommit()
    throws IOException
  {
    LIndex index = manager.getIndex(idxName);
    index.releaseIndexCommit(remoteIndexVersion);
    dataOutput.writeBoolean(true);
  }
  
  private void obtainCommit() throws IOException {
    LIndex index = manager.getIndex(idxName);
    LIndexCommitInfo commitInfo = index.obtainLastIndexCommitIfNewer(remoteIndexVersion);
    if (commitInfo == null)
    {
      dataOutput.writeBoolean(false);
    } else {
      dataOutput.writeBoolean(true);
      commitInfo.write(dataOutput);
    }
  }
  
  private void getFile() throws IOException {
    LIndex index = manager.getIndex(idxName);
    String fileName = dataInput.readUTF();
    LIndexFileInfo fileInfo = index.getFile(remoteIndexVersion, fileName);
    
    File f = fileInfo.getFile();
    if (!f.exists()) {
      dataOutput.writeBoolean(false);
    } else {
      dataOutput.writeBoolean(true);
      FileInputStream fis = new FileInputStream(f);
      try {
        byte[] buf = new byte[2048];
        BufferedInputStream bis = new BufferedInputStream(fis);
        
        int len = 0;
        while ((len = bis.read(buf)) > -1)
          dataOutput.write(buf, 0, len);
        String msg;
        String msg;
        return;
      } finally {
        try { fis.close();
        } catch (IOException e) {
          msg = "Error closing InputStream on " + f.getAbsolutePath();
          logger.log(Level.SEVERE, msg, e);
        }
      }
    }
  }
}
