package com.avaje.ebeaninternal.server.persist;

import com.avaje.ebeaninternal.api.SpiTransaction;
import com.avaje.ebeaninternal.server.core.PersistRequest;
import com.avaje.ebeaninternal.server.core.PersistRequestBean;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.persistence.PersistenceException;




































public final class BatchControl
{
  private static final Logger logger = Logger.getLogger(BatchControl.class.getName());
  



  private static final BatchDepthComparator depthComparator = new BatchDepthComparator();
  




  private final SpiTransaction transaction;
  




  private final BatchedPstmtHolder pstmtHolder = new BatchedPstmtHolder();
  



  private int batchSize;
  



  private boolean getGeneratedKeys;
  



  private boolean batchFlushOnMixed = true;
  

  private final BatchedBeanControl beanControl;
  


  public BatchControl(SpiTransaction t, int batchSize, boolean getGenKeys)
  {
    transaction = t;
    this.batchSize = batchSize;
    getGeneratedKeys = getGenKeys;
    beanControl = new BatchedBeanControl(t, this);
    transaction.setBatchControl(this);
  }
  









  public void setBatchFlushOnMixed(boolean flushBatchOnMixed)
  {
    batchFlushOnMixed = flushBatchOnMixed;
  }
  


  public int getBatchSize()
  {
    return batchSize;
  }
  





  public void setBatchSize(int batchSize)
  {
    if (batchSize > 1) {
      this.batchSize = batchSize;
    }
  }
  





  public void setGetGeneratedKeys(Boolean getGeneratedKeys)
  {
    if (getGeneratedKeys != null) {
      this.getGeneratedKeys = getGeneratedKeys.booleanValue();
    }
  }
  







  public int executeStatementOrBatch(PersistRequest request, boolean batch)
  {
    if ((!batch) || ((batchFlushOnMixed) && (!beanControl.isEmpty())))
    {
      flush();
    }
    if (!batch)
    {
      return request.executeNow();
    }
    
    if (pstmtHolder.getMaxSize() >= batchSize) {
      flush();
    }
    


    request.executeNow();
    return -1;
  }
  





  public int executeOrQueue(PersistRequestBean<?> request, boolean batch)
  {
    if ((!batch) || ((batchFlushOnMixed) && (!pstmtHolder.isEmpty())))
    {
      flush();
    }
    if (!batch) {
      return request.executeNow();
    }
    

    ArrayList<PersistRequest> persistList = beanControl.getPersistList(request);
    if (persistList == null)
    {

      if (logger.isLoggable(Level.FINE)) {
        logger.fine("Bean instance already in this batch: " + request.getBean());
      }
      return -1;
    }
    
    if (persistList.size() >= batchSize)
    {
      flush();
      


      persistList = beanControl.getPersistList(request);
    }
    
    persistList.add(request);
    
    return -1;
  }
  


  public BatchedPstmtHolder getPstmtHolder()
  {
    return pstmtHolder;
  }
  


  public boolean isEmpty()
  {
    return (beanControl.isEmpty()) && (pstmtHolder.isEmpty());
  }
  


  protected void flushPstmtHolder()
  {
    pstmtHolder.flush(getGeneratedKeys);
  }
  


  protected void executeNow(ArrayList<PersistRequest> list)
  {
    for (int i = 0; i < list.size(); i++) {
      PersistRequest request = (PersistRequest)list.get(i);
      request.executeNow();
    }
  }
  


  public void flush()
    throws PersistenceException
  {
    if (!pstmtHolder.isEmpty())
    {
      flushPstmtHolder();
    }
    if (beanControl.isEmpty())
    {
      return;
    }
    

    BatchedBeanHolder[] bsArray = beanControl.getArray();
    

    Arrays.sort(bsArray, depthComparator);
    
    if (transaction.isLogSummary()) {
      transaction.logInternal("BatchControl flush " + Arrays.toString(bsArray));
    }
    for (int i = 0; i < bsArray.length; i++) {
      BatchedBeanHolder bs = bsArray[i];
      bs.executeNow();
      
      flushPstmtHolder();
    }
  }
}
