package com.avaje.ebeaninternal.server.persist;

import com.avaje.ebeaninternal.server.core.PstmtBatch;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;







































public class BatchedPstmt
{
  private PreparedStatement pstmt;
  private final boolean isGenKeys;
  private final ArrayList<BatchPostExecute> list = new ArrayList();
  

  private final String sql;
  

  private final PstmtBatch pstmtBatch;
  

  private final boolean occCheck;
  


  public BatchedPstmt(PreparedStatement pstmt, boolean isGenKeys, String sql, PstmtBatch pstmtBatch, boolean occCheck)
  {
    this.pstmt = pstmt;
    this.isGenKeys = isGenKeys;
    this.sql = sql;
    this.pstmtBatch = pstmtBatch;
    this.occCheck = occCheck;
  }
  


  public int size()
  {
    return list.size();
  }
  


  public String getSql()
  {
    return sql;
  }
  


  public PreparedStatement getStatement()
  {
    return pstmt;
  }
  


  public void add(BatchPostExecute batchExecute)
  {
    list.add(batchExecute);
  }
  



  public void executeBatch(boolean getGeneratedKeys)
    throws SQLException
  {
    executeAndCheckRowCounts();
    if ((isGenKeys) && (getGeneratedKeys)) {
      getGeneratedKeys();
    }
    postExecute();
    close();
  }
  

  public void close()
    throws SQLException
  {
    if (pstmt != null) {
      pstmt.close();
      pstmt = null;
    }
  }
  
  private void postExecute() throws SQLException {
    for (int i = 0; i < list.size(); i++) {
      ((BatchPostExecute)list.get(i)).postExecute();
    }
  }
  
  private void executeAndCheckRowCounts() throws SQLException
  {
    if (pstmtBatch != null)
    {
      int rc = pstmtBatch.executeBatch(pstmt, list.size(), sql, occCheck);
      if (list.size() == 1) {
        ((BatchPostExecute)list.get(0)).checkRowCount(rc);
      }
      

      return;
    }
    


    int[] results = pstmt.executeBatch();
    
    if (results.length != list.size()) {
      String s = "results array error " + results.length + " " + list.size();
      throw new SQLException(s);
    }
    

    for (int i = 0; i < results.length; i++) {
      ((BatchPostExecute)list.get(i)).checkRowCount(results[i]);
    }
  }
  
  private void getGeneratedKeys() throws SQLException
  {
    int index = 0;
    ResultSet rset = pstmt.getGeneratedKeys();
    try {
      while (rset.next()) {
        Object idValue = rset.getObject(1);
        ((BatchPostExecute)list.get(index)).setGeneratedKey(idValue);
        index++;
      }
    } finally {
      if (rset != null) {
        rset.close();
      }
    }
  }
}
