package com.avaje.ebeaninternal.server.persist;

import com.avaje.ebean.config.GlobalProperties;
import com.avaje.ebean.event.BeanPersistController;
import com.avaje.ebeaninternal.api.SpiTransaction;
import com.avaje.ebeaninternal.server.core.PersistRequestBean;
import com.avaje.ebeaninternal.server.core.PersistRequestCallableSql;
import com.avaje.ebeaninternal.server.core.PersistRequestOrmUpdate;
import com.avaje.ebeaninternal.server.core.PersistRequestUpdateSql;
import com.avaje.ebeaninternal.server.core.PstmtBatch;
import com.avaje.ebeaninternal.server.deploy.BeanManager;







































public final class DefaultPersistExecute
  implements PersistExecute
{
  private final ExeCallableSql exeCallableSql;
  private final ExeUpdateSql exeUpdateSql;
  private final ExeOrmUpdate exeOrmUpdate;
  private final int defaultBatchSize;
  private final boolean defaultBatchGenKeys;
  private final boolean validate;
  
  public DefaultPersistExecute(boolean validate, Binder binder, PstmtBatch pstmtBatch)
  {
    this.validate = validate;
    exeOrmUpdate = new ExeOrmUpdate(binder, pstmtBatch);
    exeUpdateSql = new ExeUpdateSql(binder, pstmtBatch);
    exeCallableSql = new ExeCallableSql(binder, pstmtBatch);
    
    defaultBatchGenKeys = GlobalProperties.getBoolean("batch.getgeneratedkeys", true);
    defaultBatchSize = GlobalProperties.getInt("batch.size", 20);
  }
  

  public BatchControl createBatchControl(SpiTransaction t)
  {
    return new BatchControl(t, defaultBatchSize, defaultBatchGenKeys);
  }
  



  public <T> void executeInsertBean(PersistRequestBean<T> request)
  {
    BeanManager<T> mgr = request.getBeanManager();
    BeanPersister persister = mgr.getBeanPersister();
    
    BeanPersistController controller = request.getBeanController();
    if ((controller == null) || (controller.preInsert(request))) {
      if (validate) {
        request.validate();
      }
      persister.insert(request);
    }
  }
  





  public <T> void executeUpdateBean(PersistRequestBean<T> request)
  {
    BeanManager<T> mgr = request.getBeanManager();
    BeanPersister persister = mgr.getBeanPersister();
    
    BeanPersistController controller = request.getBeanController();
    if ((controller == null) || (controller.preUpdate(request))) {
      if (validate) {
        request.validate();
      }
      persister.update(request);
    }
  }
  






  public <T> void executeDeleteBean(PersistRequestBean<T> request)
  {
    BeanManager<T> mgr = request.getBeanManager();
    BeanPersister persister = mgr.getBeanPersister();
    
    BeanPersistController controller = request.getBeanController();
    if ((controller == null) || (controller.preDelete(request)))
    {
      persister.delete(request);
    }
  }
  



  public int executeOrmUpdate(PersistRequestOrmUpdate request)
  {
    return exeOrmUpdate.execute(request);
  }
  


  public int executeSqlUpdate(PersistRequestUpdateSql request)
  {
    return exeUpdateSql.execute(request);
  }
  


  public int executeSqlCallable(PersistRequestCallableSql request)
  {
    return exeCallableSql.execute(request);
  }
}
