package com.avaje.ebeaninternal.server.persist.dml;

import com.avaje.ebeaninternal.server.core.ConcurrencyMode;
import com.avaje.ebeaninternal.server.core.PersistRequestBean;
import com.avaje.ebeaninternal.server.deploy.BeanDescriptor;
import com.avaje.ebeaninternal.server.persist.dmlbind.Bindable;
import java.sql.SQLException;
import java.util.Set;






























public final class DeleteMeta
{
  private final String sqlVersion;
  private final String sqlNone;
  private final Bindable id;
  private final Bindable version;
  private final Bindable all;
  private final String tableName;
  private final boolean emptyStringAsNull;
  
  public DeleteMeta(boolean emptyStringAsNull, BeanDescriptor<?> desc, Bindable id, Bindable version, Bindable all)
  {
    this.emptyStringAsNull = emptyStringAsNull;
    tableName = desc.getBaseTable();
    this.id = id;
    this.version = version;
    this.all = all;
    
    sqlNone = genSql(ConcurrencyMode.NONE);
    sqlVersion = genSql(ConcurrencyMode.VERSION);
  }
  
  public boolean isEmptyStringAsNull() {
    return emptyStringAsNull;
  }
  


  public String getTableName()
  {
    return tableName;
  }
  



  public void bind(PersistRequestBean<?> persist, DmlHandler bind)
    throws SQLException
  {
    Object bean = persist.getBean();
    
    id.dmlBind(bind, false, bean);
    
    switch (1.$SwitchMap$com$avaje$ebeaninternal$server$core$ConcurrencyMode[persist.getConcurrencyMode().ordinal()]) {
    case 1: 
      version.dmlBind(bind, false, bean);
      break;
    
    case 2: 
      Object oldBean = persist.getOldValues();
      all.dmlBindWhere(bind, true, oldBean);
      break;
    }
    
  }
  





  public String getSql(PersistRequestBean<?> request)
    throws SQLException
  {
    switch (1.$SwitchMap$com$avaje$ebeaninternal$server$core$ConcurrencyMode[request.determineConcurrencyMode().ordinal()]) {
    case 3: 
      return sqlNone;
    
    case 1: 
      return sqlVersion;
    
    case 2: 
      return genDynamicWhere(request.getLoadedProperties(), request.getOldValues());
    }
    
    throw new RuntimeException("Invalid mode " + request.determineConcurrencyMode());
  }
  



  private String genSql(ConcurrencyMode conMode)
  {
    GenerateDmlRequest request = new GenerateDmlRequest(emptyStringAsNull);
    
    request.append("delete from ").append(tableName);
    request.append(" where ");
    
    request.setWhereIdMode();
    id.dmlAppend(request, false);
    
    if (ConcurrencyMode.VERSION.equals(conMode)) {
      if (version == null) {
        return null;
      }
      version.dmlAppend(request, false);
    }
    else if (ConcurrencyMode.ALL.equals(conMode)) {
      throw new RuntimeException("Never called for ConcurrencyMode.ALL");
    }
    
    return request.toString();
  }
  






  private String genDynamicWhere(Set<String> includedProps, Object oldBean)
    throws SQLException
  {
    GenerateDmlRequest request = new GenerateDmlRequest(emptyStringAsNull, includedProps, oldBean);
    
    request.append(sqlNone);
    
    request.setWhereMode();
    all.dmlWhere(request, true, oldBean);
    
    return request.toString();
  }
}
