package com.avaje.ebeaninternal.server.persist.dml;

import com.avaje.ebeaninternal.api.SpiTransaction;
import com.avaje.ebeaninternal.server.core.Message;
import com.avaje.ebeaninternal.server.core.PersistRequestBean;
import com.avaje.ebeaninternal.server.deploy.BeanDescriptor;
import com.avaje.ebeaninternal.server.deploy.BeanProperty;
import com.avaje.ebeaninternal.server.persist.DmlUtil;
import com.avaje.ebeaninternal.server.type.DataBind;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.persistence.OptimisticLockException;
import javax.persistence.PersistenceException;










































public class InsertHandler
  extends DmlHandler
{
  private final InsertMeta meta;
  private final boolean concatinatedKey;
  private boolean useGeneratedKeys;
  private String selectLastInsertedId;
  
  public InsertHandler(PersistRequestBean<?> persist, InsertMeta meta)
  {
    super(persist, meta.isEmptyStringToNull());
    this.meta = meta;
    concatinatedKey = meta.isConcatinatedKey();
  }
  

  public boolean isIncluded(BeanProperty prop)
  {
    return (prop.isDbInsertable()) && (super.isIncluded(prop));
  }
  


  public void bind()
    throws SQLException
  {
    BeanDescriptor<?> desc = persistRequest.getBeanDescriptor();
    Object bean = persistRequest.getBean();
    
    Object idValue = desc.getId(bean);
    
    boolean withId = !DmlUtil.isNullOrZero(idValue);
    

    if (!withId) {
      if (concatinatedKey)
      {

        withId = meta.deriveConcatenatedId(persistRequest);
      }
      else if (meta.supportsGetGeneratedKeys())
      {
        useGeneratedKeys = true;
      }
      else {
        selectLastInsertedId = meta.getSelectLastInsertedId();
      }
    }
    
    SpiTransaction t = persistRequest.getTransaction();
    boolean isBatch = t.isBatchThisRequest();
    

    sql = meta.getSql(withId);
    PreparedStatement pstmt;
    PreparedStatement pstmt;
    if (isBatch) {
      pstmt = getPstmt(t, sql, persistRequest, useGeneratedKeys);
    }
    else {
      logSql(sql);
      pstmt = getPstmt(t, sql, useGeneratedKeys);
    }
    dataBind = new DataBind(pstmt);
    
    bindLogAppend("Binding Insert [");
    bindLogAppend(desc.getBaseTable());
    bindLogAppend("]  set[");
    

    meta.bind(this, bean, withId);
    
    bindLogAppend("]");
    logBinding();
  }
  


  protected PreparedStatement getPstmt(SpiTransaction t, String sql, boolean useGeneratedKeys)
    throws SQLException
  {
    Connection conn = t.getInternalConnection();
    if (useGeneratedKeys) {
      return conn.prepareStatement(sql, 1);
    }
    
    return conn.prepareStatement(sql);
  }
  



  public void execute()
    throws SQLException, OptimisticLockException
  {
    int rc = dataBind.executeUpdate();
    if (useGeneratedKeys)
    {
      getGeneratedKeys();
    }
    else if (selectLastInsertedId != null)
    {
      fetchGeneratedKeyUsingSelect();
    }
    
    checkRowCount(rc);
    setAdditionalProperties();
  }
  


  private void getGeneratedKeys()
    throws SQLException
  {
    ResultSet rset = dataBind.getPstmt().getGeneratedKeys();
    try {
      if (rset.next()) {
        Object idValue = rset.getObject(1);
        if (idValue != null) {
          persistRequest.setGeneratedKey(idValue);
        }
      }
      else {
        throw new PersistenceException(Message.msg("persist.autoinc.norows"));
      }
      String msg;
      String msg;
      return; } finally { try { rset.close();
      } catch (SQLException ex) {
        msg = "Error closing rset for returning generatedKeys?";
        logger.log(Level.WARNING, msg, ex);
      }
    }
  }
  



  private void fetchGeneratedKeyUsingSelect()
    throws SQLException
  {
    Connection conn = transaction.getConnection();
    
    PreparedStatement stmt = null;
    ResultSet rset = null;
    try {
      stmt = conn.prepareStatement(selectLastInsertedId);
      rset = stmt.executeQuery();
      if (rset.next()) {
        Object idValue = rset.getObject(1);
        if (idValue != null) {
          persistRequest.setGeneratedKey(idValue);
        }
      } else {
        throw new PersistenceException(Message.msg("persist.autoinc.norows")); }
      String msg;
      String msg;
      String msg;
      String msg; return; } finally { try { if (rset != null) {
          rset.close();
        }
      } catch (SQLException ex) {
        msg = "Error closing rset for fetchGeneratedKeyUsingSelect?";
        logger.log(Level.WARNING, msg, ex);
      }
      try {
        if (stmt != null) {
          stmt.close();
        }
      } catch (SQLException ex) {
        msg = "Error closing stmt for fetchGeneratedKeyUsingSelect?";
        logger.log(Level.WARNING, msg, ex);
      }
    }
  }
}
