package com.avaje.ebeaninternal.server.persist.dml;

import com.avaje.ebean.config.dbplatform.DatabasePlatform;
import com.avaje.ebean.config.dbplatform.DbIdentity;
import com.avaje.ebeaninternal.server.core.PersistRequestBean;
import com.avaje.ebeaninternal.server.deploy.BeanDescriptor;
import com.avaje.ebeaninternal.server.deploy.InheritInfo;
import com.avaje.ebeaninternal.server.persist.dmlbind.Bindable;
import com.avaje.ebeaninternal.server.persist.dmlbind.BindableDiscriminator;
import com.avaje.ebeaninternal.server.persist.dmlbind.BindableId;
import java.sql.SQLException;
import java.util.Set;






































public final class InsertMeta
{
  private final String sqlNullId;
  private final String sqlWithId;
  private final BindableId id;
  private final Bindable discriminator;
  private final Bindable all;
  private final boolean supportsGetGeneratedKeys;
  private final boolean concatinatedKey;
  private final String tableName;
  private final String selectLastInsertedId;
  private final Bindable shadowFKey;
  private final String[] identityDbColumns;
  private final boolean emptyStringToNull;
  
  public InsertMeta(DatabasePlatform dbPlatform, BeanDescriptor<?> desc, Bindable shadowFKey, BindableId id, Bindable all)
  {
    emptyStringToNull = dbPlatform.isTreatEmptyStringsAsNull();
    tableName = desc.getBaseTable();
    discriminator = getDiscriminator(desc);
    this.id = id;
    this.all = all;
    this.shadowFKey = shadowFKey;
    
    sqlWithId = genSql(false, null);
    

    if (id.isConcatenated())
    {
      concatinatedKey = true;
      identityDbColumns = null;
      sqlNullId = null;
      supportsGetGeneratedKeys = false;
      selectLastInsertedId = null;
    }
    else
    {
      concatinatedKey = false;
      identityDbColumns = new String[] { id.getIdentityColumn() };
      sqlNullId = genSql(true, null);
      supportsGetGeneratedKeys = dbPlatform.getDbIdentity().isSupportsGetGeneratedKeys();
      selectLastInsertedId = desc.getSelectLastInsertedId();
    }
  }
  
  private static Bindable getDiscriminator(BeanDescriptor<?> desc) {
    InheritInfo inheritInfo = desc.getInheritInfo();
    if (inheritInfo != null) {
      return new BindableDiscriminator(inheritInfo);
    }
    return null;
  }
  



  public boolean isEmptyStringToNull()
  {
    return emptyStringToNull;
  }
  


  public boolean isConcatinatedKey()
  {
    return concatinatedKey;
  }
  
  public String[] getIdentityDbColumns() {
    return identityDbColumns;
  }
  







  public String getSelectLastInsertedId()
  {
    return selectLastInsertedId;
  }
  



  public boolean supportsGetGeneratedKeys()
  {
    return supportsGetGeneratedKeys;
  }
  


  public boolean deriveConcatenatedId(PersistRequestBean<?> persist)
  {
    return id.deriveConcatenatedId(persist);
  }
  


  public void bind(DmlHandler request, Object bean, boolean withId)
    throws SQLException
  {
    if (withId) {
      id.dmlBind(request, false, bean);
    }
    if (shadowFKey != null) {
      shadowFKey.dmlBind(request, false, bean);
    }
    if (discriminator != null) {
      discriminator.dmlBind(request, false, bean);
    }
    all.dmlBind(request, false, bean);
  }
  



  public String getSql(boolean withId)
  {
    if (withId) {
      return sqlWithId;
    }
    return sqlNullId;
  }
  

  private String genSql(boolean nullId, Set<String> loadedProps)
  {
    GenerateDmlRequest request = new GenerateDmlRequest(emptyStringToNull, loadedProps, null);
    request.setInsertSetMode();
    
    request.append("insert into ").append(tableName);
    request.append(" (");
    
    if (!nullId) {
      id.dmlInsert(request, false);
    }
    
    if (shadowFKey != null) {
      shadowFKey.dmlInsert(request, false);
    }
    
    if (discriminator != null) {
      discriminator.dmlInsert(request, false);
    }
    
    all.dmlInsert(request, false);
    
    request.append(") values (");
    request.append(request.getInsertBindBuffer());
    request.append(")");
    
    return request.toString();
  }
}
