package com.avaje.ebeaninternal.server.persist.dml;

import com.avaje.ebeaninternal.api.SpiUpdatePlan;
import com.avaje.ebeaninternal.server.core.ConcurrencyMode;
import com.avaje.ebeaninternal.server.persist.dmlbind.Bindable;
import java.sql.SQLException;
import java.util.Set;































public class UpdatePlan
  implements SpiUpdatePlan
{
  public static final UpdatePlan EMPTY_SET_CLAUSE = new UpdatePlan();
  

  private final Integer key;
  

  private final ConcurrencyMode mode;
  
  private final String sql;
  
  private final Bindable set;
  
  private final Set<String> properties;
  
  private final boolean checkIncludes;
  
  private final long timeCreated;
  
  private final boolean emptySetClause;
  
  private Long timeLastUsed;
  

  public UpdatePlan(ConcurrencyMode mode, String sql, Bindable set)
  {
    this(null, mode, sql, set, null);
  }
  




  public UpdatePlan(Integer key, ConcurrencyMode mode, String sql, Bindable set, Set<String> properties)
  {
    emptySetClause = false;
    this.key = key;
    this.mode = mode;
    this.sql = sql;
    this.set = set;
    this.properties = properties;
    checkIncludes = (properties != null);
    timeCreated = System.currentTimeMillis();
  }
  


  private UpdatePlan()
  {
    emptySetClause = true;
    key = Integer.valueOf(0);
    mode = ConcurrencyMode.NONE;
    sql = null;
    set = null;
    properties = null;
    checkIncludes = false;
    timeCreated = 0L;
  }
  
  public boolean isEmptySetClause()
  {
    return emptySetClause;
  }
  


  public void bindSet(DmlHandler bind, Object bean)
    throws SQLException
  {
    set.dmlBind(bind, checkIncludes, bean);
    

    Long touched = Long.valueOf(System.currentTimeMillis());
    timeLastUsed = touched;
  }
  


  public long getTimeCreated()
  {
    return timeCreated;
  }
  




  public Long getTimeLastUsed()
  {
    return timeLastUsed;
  }
  


  public Integer getKey()
  {
    return key;
  }
  


  public ConcurrencyMode getMode()
  {
    return mode;
  }
  


  public String getSql()
  {
    return sql;
  }
  


  public Bindable getSet()
  {
    return set;
  }
  






  public Set<String> getProperties()
  {
    return properties;
  }
}
