package com.avaje.ebeaninternal.server.persist.dmlbind;

import com.avaje.ebean.bean.EntityBean;
import com.avaje.ebean.bean.EntityBeanIntercept;
import com.avaje.ebeaninternal.server.core.PersistRequestBean;
import com.avaje.ebeaninternal.server.deploy.BeanPropertyAssocOne;
import com.avaje.ebeaninternal.server.persist.dml.GenerateDmlRequest;
import java.sql.SQLException;
import java.util.Arrays;
import java.util.List;






















public class BindableEmbedded
  implements Bindable
{
  private final Bindable[] items;
  private final BeanPropertyAssocOne<?> embProp;
  
  public BindableEmbedded(BeanPropertyAssocOne<?> embProp, List<Bindable> list)
  {
    this.embProp = embProp;
    items = ((Bindable[])list.toArray(new Bindable[list.size()]));
  }
  
  public String toString() {
    return "BindableEmbedded " + embProp + " items:" + Arrays.toString(items);
  }
  
  public void dmlInsert(GenerateDmlRequest request, boolean checkIncludes) {
    dmlAppend(request, checkIncludes);
  }
  
  public void dmlAppend(GenerateDmlRequest request, boolean checkIncludes) {
    if ((checkIncludes) && (!request.isIncluded(embProp))) {
      return;
    }
    
    for (int i = 0; i < items.length; i++) {
      items[i].dmlAppend(request, false);
    }
  }
  


  public void dmlWhere(GenerateDmlRequest request, boolean checkIncludes, Object origBean)
  {
    if ((checkIncludes) && (!request.isIncludedWhere(embProp))) {
      return;
    }
    Object embBean = embProp.getValue(origBean);
    Object oldValues = getOldValue(embBean);
    
    for (int i = 0; i < items.length; i++) {
      items[i].dmlWhere(request, false, oldValues);
    }
  }
  
  public void addChanged(PersistRequestBean<?> request, List<Bindable> list) {
    if (request.hasChanged(embProp)) {
      list.add(this);
    }
  }
  
  public void dmlBind(BindableRequest bindRequest, boolean checkIncludes, Object bean)
    throws SQLException
  {
    if ((checkIncludes) && (!bindRequest.isIncluded(embProp))) {
      return;
    }
    

    Object embBean = embProp.getValue(bean);
    
    for (int i = 0; i < items.length; i++) {
      items[i].dmlBind(bindRequest, false, embBean);
    }
  }
  
  public void dmlBindWhere(BindableRequest bindRequest, boolean checkIncludes, Object bean)
    throws SQLException
  {
    if ((checkIncludes) && (!bindRequest.isIncludedWhere(embProp))) {
      return;
    }
    

    Object embBean = embProp.getValue(bean);
    Object oldEmbBean = getOldValue(embBean);
    
    for (int i = 0; i < items.length; i++) {
      items[i].dmlBindWhere(bindRequest, false, oldEmbBean);
    }
  }
  






  private Object getOldValue(Object embBean)
  {
    Object oldValues = null;
    
    if ((embBean instanceof EntityBean))
    {
      oldValues = ((EntityBean)embBean)._ebean_getIntercept().getOldValues();
    }
    
    if (oldValues == null)
    {

      oldValues = embBean;
    }
    
    return oldValues;
  }
}
