package com.avaje.ebeaninternal.server.persist.dmlbind;

import com.avaje.ebean.config.EncryptKey;
import com.avaje.ebeaninternal.server.core.PersistRequestBean;
import com.avaje.ebeaninternal.server.deploy.BeanProperty;
import com.avaje.ebeaninternal.server.persist.dml.GenerateDmlRequest;
import java.sql.SQLException;
import java.util.List;























public class BindableEncryptedProperty
  implements Bindable
{
  private final BeanProperty prop;
  private final boolean bindEncryptDataFirst;
  
  public BindableEncryptedProperty(BeanProperty prop, boolean bindEncryptDataFirst)
  {
    this.prop = prop;
    this.bindEncryptDataFirst = bindEncryptDataFirst;
  }
  
  public String toString() {
    return prop.toString();
  }
  
  public void addChanged(PersistRequestBean<?> request, List<Bindable> list) {
    if (request.hasChanged(prop)) {
      list.add(this);
    }
  }
  
  public void dmlInsert(GenerateDmlRequest request, boolean checkIncludes)
  {
    if ((checkIncludes) && (!request.isIncluded(prop))) {
      return;
    }
    

    request.appendColumn(prop.getDbColumn(), prop.getDbBind());
  }
  
  public void dmlAppend(GenerateDmlRequest request, boolean checkIncludes)
  {
    if ((checkIncludes) && (!request.isIncluded(prop))) {
      return;
    }
    
    request.appendColumn(prop.getDbColumn(), "=", prop.getDbBind());
  }
  




  public void dmlWhere(GenerateDmlRequest request, boolean checkIncludes, Object bean)
  {
    if ((checkIncludes) && (!request.isIncluded(prop))) {
      return;
    }
    
    if ((bean == null) || (request.isDbNull(prop.getValue(bean)))) {
      request.appendColumnIsNull(prop.getDbColumn());
    }
    else
    {
      request.appendColumn("? = ", prop.getDecryptSql());
    }
  }
  



  public void dmlBind(BindableRequest request, boolean checkIncludes, Object bean)
    throws SQLException
  {
    if ((checkIncludes) && (!request.isIncluded(prop))) {
      return;
    }
    Object value = null;
    if (bean != null) {
      value = prop.getValue(bean);
    }
    

    String encryptKeyValue = prop.getEncryptKey().getStringValue();
    
    if (!bindEncryptDataFirst)
    {
      request.bindNoLog(encryptKeyValue, 12, prop.getName() + "=****");
    }
    request.bindNoLog(value, prop, prop.getName(), true);
    
    if (bindEncryptDataFirst)
    {
      request.bindNoLog(encryptKeyValue, 12, prop.getName() + "=****");
    }
  }
  





  public void dmlBindWhere(BindableRequest request, boolean checkIncludes, Object bean)
    throws SQLException
  {
    if ((checkIncludes) && (!request.isIncluded(prop))) {
      return;
    }
    Object value = null;
    if (bean != null) {
      value = prop.getValue(bean);
    }
    

    String encryptKeyValue = prop.getEncryptKey().getStringValue();
    
    request.bind(value, prop, prop.getName(), false);
    request.bindNoLog(encryptKeyValue, 12, prop.getName() + "=****");
  }
}
