package com.avaje.ebeaninternal.server.query;

import com.avaje.ebean.Query.Type;
import com.avaje.ebean.bean.BeanCollection;
import com.avaje.ebeaninternal.api.SpiQuery;
import com.avaje.ebeaninternal.api.SpiSqlQuery;
import com.avaje.ebeaninternal.server.core.OrmQueryRequest;
import com.avaje.ebeaninternal.server.core.RelationalQueryRequest;
import com.avaje.ebeaninternal.server.deploy.BeanDescriptor;
import com.avaje.ebeaninternal.server.deploy.BeanPropertyAssocMany;
import com.avaje.ebeaninternal.server.deploy.ManyType;
import com.avaje.ebeaninternal.server.util.BeanCollectionFactory;
import com.avaje.ebeaninternal.server.util.BeanCollectionParams;
import java.util.Collection;
import java.util.Map;























































public final class BeanCollectionWrapper
{
  private final boolean isMap;
  private final Query.Type queryType;
  private final String mapKey;
  private final BeanCollection<?> beanCollection;
  private final Collection<Object> collection;
  private final Map<Object, Object> map;
  private final BeanDescriptor<?> desc;
  private int rowCount;
  
  public BeanCollectionWrapper(RelationalQueryRequest request)
  {
    desc = null;
    queryType = request.getQueryType();
    mapKey = request.getQuery().getMapKey();
    isMap = Query.Type.MAP.equals(queryType);
    
    beanCollection = createBeanCollection(queryType);
    collection = getCollection(isMap);
    map = getMap(isMap);
  }
  



  public BeanCollectionWrapper(OrmQueryRequest<?> request)
  {
    desc = request.getBeanDescriptor();
    queryType = request.getQueryType();
    mapKey = request.getQuery().getMapKey();
    isMap = Query.Type.MAP.equals(queryType);
    
    beanCollection = createBeanCollection(queryType);
    collection = getCollection(isMap);
    map = getMap(isMap);
  }
  








  public BeanCollectionWrapper(BeanPropertyAssocMany<?> manyProp)
  {
    queryType = manyProp.getManyType().getQueryType();
    mapKey = manyProp.getMapKey();
    desc = manyProp.getTargetDescriptor();
    isMap = Query.Type.MAP.equals(queryType);
    
    beanCollection = createBeanCollection(queryType);
    collection = getCollection(isMap);
    map = getMap(isMap);
  }
  
  private Map<Object, Object> getMap(boolean isMap)
  {
    return isMap ? (Map)beanCollection : null;
  }
  
  private Collection<Object> getCollection(boolean isMap)
  {
    return isMap ? null : (Collection)beanCollection;
  }
  


  public BeanCollection<?> getBeanCollection()
  {
    return beanCollection;
  }
  


  private BeanCollection<?> createBeanCollection(Query.Type manyType)
  {
    BeanCollectionParams p = new BeanCollectionParams(manyType);
    return BeanCollectionFactory.create(p);
  }
  


  public boolean isMap()
  {
    return isMap;
  }
  


  public int size()
  {
    return rowCount;
  }
  


  public void add(Object bean)
  {
    add(bean, beanCollection);
  }
  








  public void add(Object bean, Object collection)
  {
    if (bean == null) {
      return;
    }
    rowCount += 1;
    if (isMap) {
      Object keyValue = null;
      if (mapKey != null)
      {
        keyValue = desc.getValue(bean, mapKey);
      }
      else {
        keyValue = desc.getId(bean);
      }
      
      Map mapColl = (Map)collection;
      mapColl.put(keyValue, bean);
    } else {
      ((Collection)collection).add(bean);
    }
  }
  


  public void addToCollection(Object bean)
  {
    collection.add(bean);
  }
  


  public void addToMap(Object bean, Object key)
  {
    map.put(key, bean);
  }
}
