package com.avaje.ebeaninternal.server.query;

import com.avaje.ebean.Query.Type;
import com.avaje.ebean.QueryIterator;
import com.avaje.ebean.QueryListener;
import com.avaje.ebean.bean.BeanCollection;
import com.avaje.ebean.bean.BeanCollectionAdd;
import com.avaje.ebean.bean.EntityBean;
import com.avaje.ebean.bean.EntityBeanIntercept;
import com.avaje.ebean.bean.NodeUsageCollector;
import com.avaje.ebean.bean.NodeUsageListener;
import com.avaje.ebean.bean.ObjectGraphNode;
import com.avaje.ebean.bean.PersistenceContext;
import com.avaje.ebeaninternal.api.LoadContext;
import com.avaje.ebeaninternal.api.SpiExpressionList;
import com.avaje.ebeaninternal.api.SpiQuery;
import com.avaje.ebeaninternal.api.SpiQuery.Mode;
import com.avaje.ebeaninternal.api.SpiTransaction;
import com.avaje.ebeaninternal.server.autofetch.AutoFetchManager;
import com.avaje.ebeaninternal.server.core.Message;
import com.avaje.ebeaninternal.server.core.OrmQueryRequest;
import com.avaje.ebeaninternal.server.core.ReferenceOptions;
import com.avaje.ebeaninternal.server.core.SpiOrmQueryRequest;
import com.avaje.ebeaninternal.server.deploy.BeanCollectionHelp;
import com.avaje.ebeaninternal.server.deploy.BeanCollectionHelpFactory;
import com.avaje.ebeaninternal.server.deploy.BeanDescriptor;
import com.avaje.ebeaninternal.server.deploy.BeanPropertyAssocMany;
import com.avaje.ebeaninternal.server.deploy.BeanPropertyAssocOne;
import com.avaje.ebeaninternal.server.deploy.DbReadContext;
import com.avaje.ebeaninternal.server.el.ElPropertyValue;
import com.avaje.ebeaninternal.server.lib.util.StringHelper;
import com.avaje.ebeaninternal.server.querydefn.OrmQueryDetail;
import com.avaje.ebeaninternal.server.querydefn.OrmQueryProperties;
import com.avaje.ebeaninternal.server.transaction.DefaultPersistenceContext;
import com.avaje.ebeaninternal.server.type.DataBind;
import com.avaje.ebeaninternal.server.type.DataReader;
import java.lang.ref.WeakReference;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.persistence.PersistenceException;

































public class CQuery<T>
  implements DbReadContext, CancelableQuery
{
  private static final Logger logger = Logger.getLogger(CQuery.class.getName());
  


  private static final int GLOBAL_ROW_LIMIT = 1000000;
  


  private int rowCount;
  


  private int loadedBeanCount;
  


  private boolean noMoreRows;
  


  private Object loadedBeanId;
  


  boolean loadedBeanChanged;
  


  private Object loadedBean;
  


  private Object prevLoadedBean;
  


  private Object loadedManyBean;
  


  private Object prevDetailCollection;
  


  private Object currentDetailCollection;
  


  private final BeanCollection<T> collection;
  


  private final BeanCollectionHelp<T> help;
  


  private final OrmQueryRequest<T> request;
  


  private final BeanDescriptor<T> desc;
  


  private final SpiQuery<T> query;
  


  private final OrmQueryDetail queryDetail;
  


  private final QueryListener<T> queryListener;
  


  private Map<String, String> currentPathMap;
  


  private String currentPrefix;
  


  private final boolean manyIncluded;
  


  private final CQueryPredicates predicates;
  


  private final SqlTree sqlTree;
  

  private final boolean rawSql;
  

  private final String sql;
  

  private final String logWhereSql;
  

  private final boolean rowNumberIncluded;
  

  private final SqlTreeNode rootNode;
  

  private final BeanPropertyAssocMany<?> manyProperty;
  

  private final ElPropertyValue manyPropertyEl;
  

  private final int backgroundFetchAfter;
  

  private final int maxRowsLimit;
  

  private boolean hasHitBackgroundFetchAfter;
  

  private final PersistenceContext persistenceContext;
  

  private DataReader dataReader;
  

  private PreparedStatement pstmt;
  

  private boolean cancelled;
  

  private String bindLog;
  

  private final CQueryPlan queryPlan;
  

  private long startNano;
  

  private final SpiQuery.Mode queryMode;
  

  private final boolean autoFetchProfiling;
  

  private final ObjectGraphNode autoFetchParentNode;
  

  private final AutoFetchManager autoFetchManager;
  

  private final WeakReference<NodeUsageListener> autoFetchManagerRef;
  

  private final HashMap<String, ReferenceOptions> referenceOptionsMap = new HashMap();
  
  private int executionTimeMicros;
  
  private final int parentState;
  
  private final SpiExpressionList<?> filterMany;
  
  private BeanCollectionAdd currentDetailAdd;
  

  public CQuery(OrmQueryRequest<T> request, CQueryPredicates predicates, CQueryPlan queryPlan)
  {
    this.request = request;
    this.queryPlan = queryPlan;
    query = request.getQuery();
    queryDetail = query.getDetail();
    queryMode = query.getMode();
    
    parentState = request.getParentState();
    
    autoFetchManager = query.getAutoFetchManager();
    autoFetchProfiling = (autoFetchManager != null);
    autoFetchParentNode = (autoFetchProfiling ? query.getParentNode() : null);
    autoFetchManagerRef = (autoFetchProfiling ? new WeakReference(autoFetchManager) : null);
    


    query.setGeneratedSql(queryPlan.getSql());
    
    sqlTree = queryPlan.getSqlTree();
    rootNode = sqlTree.getRootNode();
    
    manyProperty = sqlTree.getManyProperty();
    manyPropertyEl = sqlTree.getManyPropertyEl();
    manyIncluded = sqlTree.isManyIncluded();
    if (manyIncluded)
    {
      String manyPropertyName = sqlTree.getManyPropertyName();
      OrmQueryProperties chunk = query.getDetail().getChunk(manyPropertyName, false);
      filterMany = chunk.getFilterMany();
    } else {
      filterMany = null;
    }
    
    sql = queryPlan.getSql();
    rawSql = queryPlan.isRawSql();
    rowNumberIncluded = queryPlan.isRowNumberIncluded();
    logWhereSql = queryPlan.getLogWhereSql();
    desc = request.getBeanDescriptor();
    this.predicates = predicates;
    
    queryListener = query.getListener();
    if (queryListener == null)
    {
      persistenceContext = request.getPersistenceContext();

    }
    else
    {
      persistenceContext = new DefaultPersistenceContext();
    }
    
    maxRowsLimit = (query.getMaxRows() > 0 ? query.getMaxRows() : 1000000);
    backgroundFetchAfter = (query.getBackgroundFetchAfter() > 0 ? query.getBackgroundFetchAfter() : 2147483647);
    
    help = createHelp(request);
    collection = ((BeanCollection)(help != null ? help.createEmpty(false) : null));
  }
  
  private BeanCollectionHelp<T> createHelp(OrmQueryRequest<T> request) {
    if (request.isFindById()) {
      return null;
    }
    Query.Type manyType = request.getQuery().getType();
    if (manyType == null)
    {
      return null;
    }
    return BeanCollectionHelpFactory.create(request);
  }
  
  public int getParentState()
  {
    return parentState;
  }
  
  public void propagateState(Object e) {
    if ((parentState != 0) && 
      ((e instanceof EntityBean))) {
      ((EntityBean)e)._ebean_getIntercept().setState(parentState);
    }
  }
  
  public DataReader getDataReader()
  {
    return dataReader;
  }
  
  public SpiQuery.Mode getQueryMode() {
    return queryMode;
  }
  


  public boolean isVanillaMode()
  {
    return request.isVanillaMode();
  }
  
  public CQueryPredicates getPredicates() {
    return predicates;
  }
  
  public LoadContext getGraphContext() {
    return request.getGraphContext();
  }
  
  public SpiOrmQueryRequest<?> getQueryRequest() {
    return request;
  }
  
  public void cancel() {
    synchronized (this) {
      cancelled = true;
      if (pstmt != null) {
        try {
          pstmt.cancel();
        } catch (SQLException e) {
          String msg = "Error cancelling query";
          throw new PersistenceException(msg, e);
        }
      }
    }
  }
  
  public boolean prepareBindExecuteQuery() throws SQLException
  {
    synchronized (this) {
      if ((cancelled) || (query.isCancelled()))
      {
        cancelled = true;
        return false;
      }
      
      startNano = System.nanoTime();
      
      if (request.isLuceneQuery())
      {
        dataReader = queryPlan.createDataReader(null);
      }
      else
      {
        SpiTransaction t = request.getTransaction();
        Connection conn = t.getInternalConnection();
        pstmt = conn.prepareStatement(sql);
        
        if (query.getTimeout() > 0) {
          pstmt.setQueryTimeout(query.getTimeout());
        }
        if (query.getBufferFetchSizeHint() > 0) {
          pstmt.setFetchSize(query.getBufferFetchSizeHint());
        }
        
        DataBind dataBind = new DataBind(pstmt);
        

        queryPlan.bindEncryptedProperties(dataBind);
        
        bindLog = predicates.bind(dataBind);
        

        ResultSet rset = pstmt.executeQuery();
        dataReader = queryPlan.createDataReader(rset);
      }
      return true;
    }
  }
  





  public void close()
  {
    try
    {
      if (dataReader != null) {
        dataReader.close();
        dataReader = null;
      }
    } catch (SQLException e) {
      logger.log(Level.SEVERE, null, e);
    }
    try {
      if (pstmt != null) {
        pstmt.close();
        pstmt = null;
      }
    } catch (SQLException e) {
      logger.log(Level.SEVERE, null, e);
    }
  }
  



  public ReferenceOptions getReferenceOptionsFor(BeanPropertyAssocOne<?> beanProp)
  {
    String beanPropName = beanProp.getName();
    if (currentPrefix != null) {
      beanPropName = currentPrefix + "." + beanPropName;
    }
    ReferenceOptions opt = (ReferenceOptions)referenceOptionsMap.get(beanPropName);
    if (opt == null) {
      OrmQueryProperties chunk = queryDetail.getChunk(beanPropName, false);
      if (chunk != null)
      {
        opt = chunk.getReferenceOptions();
      }
      if (opt == null)
      {
        opt = beanProp.getTargetDescriptor().getReferenceOptions();
      }
      referenceOptionsMap.put(beanPropName, opt);
    }
    
    return opt;
  }
  


  public PersistenceContext getPersistenceContext()
  {
    return persistenceContext;
  }
  
  public void setLoadedBean(Object bean, Object id) {
    if ((id == null) || (!id.equals(loadedBeanId)))
    {




      if (manyIncluded) {
        if (rowCount > 1) {
          loadedBeanChanged = true;
        }
        prevLoadedBean = loadedBean;
        loadedBeanId = id;
      }
      loadedBean = bean;
    }
  }
  
  public void setLoadedManyBean(Object manyValue) {
    loadedManyBean = manyValue;
  }
  



  public T getLoadedBean()
  {
    if (manyIncluded) {
      if ((prevDetailCollection instanceof BeanCollection)) {
        ((BeanCollection)prevDetailCollection).setModifyListening(manyProperty.getModifyListenMode());
      }
      else if ((currentDetailCollection instanceof BeanCollection)) {
        ((BeanCollection)currentDetailCollection).setModifyListening(manyProperty.getModifyListenMode());
      }
    }
    
    if (prevLoadedBean != null) {
      return prevLoadedBean;
    }
    return loadedBean;
  }
  
  private boolean hasMoreRows() throws SQLException
  {
    synchronized (this) {
      if (cancelled) {
        return false;
      }
      return dataReader.next();
    }
  }
  






  private boolean readRow()
    throws SQLException
  {
    synchronized (this) {
      if (cancelled) {
        return false;
      }
      
      if (!dataReader.next()) {
        return false;
      }
      
      rowCount += 1;
      dataReader.resetColumnPosition();
      
      if (rowNumberIncluded)
      {
        dataReader.incrementPos(1);
      }
      
      rootNode.load(this, null, parentState);
      
      return true;
    }
  }
  
  public int getQueryExecutionTimeMicros() {
    return executionTimeMicros;
  }
  
  public boolean readBean() throws SQLException
  {
    boolean result = readBeanInternal(true);
    
    updateExecutionStatistics();
    
    return result;
  }
  
  private boolean readBeanInternal(boolean inForeground) throws SQLException
  {
    if (loadedBeanCount >= maxRowsLimit) {
      collection.setHasMoreRows(hasMoreRows());
      return false;
    }
    
    if ((inForeground) && (loadedBeanCount >= backgroundFetchAfter)) {
      hasHitBackgroundFetchAfter = true;
      collection.setFinishedFetch(false);
      return false;
    }
    
    if (!manyIncluded)
    {
      return readRow();
    }
    
    if (noMoreRows) {
      return false;
    }
    
    if (rowCount == 0) {
      if (!readRow())
      {
        return false;
      }
      createNewDetailCollection();
    }
    

    if (readIntoCurrentDetailCollection()) {
      createNewDetailCollection();
      
      return true;
    }
    

    prevDetailCollection = null;
    prevLoadedBean = null;
    noMoreRows = true;
    return true;
  }
  
  private boolean readIntoCurrentDetailCollection() throws SQLException
  {
    while (readRow()) {
      if (loadedBeanChanged) {
        loadedBeanChanged = false;
        return true;
      }
      addToCurrentDetailCollection();
    }
    
    return false;
  }
  

  private void createNewDetailCollection()
  {
    prevDetailCollection = currentDetailCollection;
    if (queryMode.equals(SpiQuery.Mode.LAZYLOAD_MANY))
    {
      currentDetailCollection = manyPropertyEl.elGetValue(loadedBean);
    }
    else {
      currentDetailCollection = manyProperty.createEmpty(request.isVanillaMode());
      manyPropertyEl.elSetValue(loadedBean, currentDetailCollection, false, false);
    }
    
    if ((filterMany != null) && (!request.isVanillaMode()))
    {
      ((BeanCollection)currentDetailCollection).setFilterMany(filterMany);
    }
    

    currentDetailAdd = manyProperty.getBeanCollectionAdd(currentDetailCollection, null);
    addToCurrentDetailCollection();
  }
  
  private void addToCurrentDetailCollection() {
    if (loadedManyBean != null) {
      currentDetailAdd.addBean(loadedManyBean);
    }
  }
  
  public BeanCollection<T> continueFetchingInBackground() throws SQLException {
    readTheRows(false);
    collection.setFinishedFetch(true);
    return collection;
  }
  
  public BeanCollection<T> readCollection() throws SQLException
  {
    readTheRows(true);
    
    updateExecutionStatistics();
    
    return collection;
  }
  
  protected void updateExecutionStatistics() {
    try {
      long exeNano = System.nanoTime() - startNano;
      executionTimeMicros = ((int)exeNano / 1000);
      
      if (autoFetchProfiling) {
        autoFetchManager.collectQueryInfo(autoFetchParentNode, loadedBeanCount, executionTimeMicros);
      }
      queryPlan.executionTime(loadedBeanCount, executionTimeMicros);
    }
    catch (Exception e) {
      logger.log(Level.SEVERE, null, e);
    }
  }
  
  public QueryIterator<T> readIterate(int bufferSize, OrmQueryRequest<T> request)
  {
    if (bufferSize > 0) {
      return new CQueryIteratorWithBuffer(this, request, bufferSize);
    }
    
    return new CQueryIteratorSimple(this, request);
  }
  
  private void readTheRows(boolean inForeground) throws SQLException
  {
    while (hasNextBean(inForeground)) {
      if (queryListener != null) {
        queryListener.process(getLoadedBean());
      }
      else
      {
        help.add(collection, getLoadedBean());
      }
    }
  }
  
  protected boolean hasNextBean(boolean inForeground)
    throws SQLException
  {
    if (!readBeanInternal(inForeground)) {
      return false;
    }
    
    loadedBeanCount += 1;
    return true;
  }
  
  public String getLoadedRowDetail()
  {
    if (!manyIncluded) {
      return String.valueOf(rowCount);
    }
    return loadedBeanCount + ":" + rowCount;
  }
  

  public void register(String path, EntityBeanIntercept ebi)
  {
    path = getPath(path);
    request.getGraphContext().register(path, ebi);
  }
  
  public void register(String path, BeanCollection<?> bc)
  {
    path = getPath(path);
    request.getGraphContext().register(path, bc);
  }
  
  public boolean useBackgroundToContinueFetch()
  {
    return hasHitBackgroundFetchAfter;
  }
  


  public String getName()
  {
    return query.getName();
  }
  


  public boolean isRawSql()
  {
    return rawSql;
  }
  


  public String getLogWhereSql()
  {
    return logWhereSql;
  }
  



  public BeanPropertyAssocMany<?> getManyProperty()
  {
    return manyProperty;
  }
  


  public String getSummary()
  {
    return sqlTree.getSummary();
  }
  



  public SqlTree getSqlTree()
  {
    return sqlTree;
  }
  
  public String getBindLog() {
    return bindLog;
  }
  
  public SpiTransaction getTransaction() {
    return request.getTransaction();
  }
  
  public String getBeanType() {
    return desc.getFullName();
  }
  


  public String getBeanName()
  {
    return desc.getName();
  }
  


  public String getGeneratedSql()
  {
    return sql;
  }
  



  public PersistenceException createPersistenceException(SQLException e)
  {
    return createPersistenceException(e, getTransaction(), bindLog, sql);
  }
  



  public static PersistenceException createPersistenceException(SQLException e, SpiTransaction t, String bindLog, String sql)
  {
    if (t.isLogSummary())
    {
      String errMsg = StringHelper.replaceStringMulti(e.getMessage(), new String[] { "\r", "\n" }, "\\n ");
      String msg = "ERROR executing query:   bindLog[" + bindLog + "] error[" + errMsg + "]";
      t.logInternal(msg);
    }
    

    t.getConnection();
    

    String m = Message.msg("fetch.sqlerror", e.getMessage(), bindLog, sql);
    return new PersistenceException(m, e);
  }
  








  public boolean isAutoFetchProfiling()
  {
    return (autoFetchProfiling) && (query.isUsageProfiling());
  }
  
  private String getPath(String propertyName)
  {
    if (currentPrefix == null)
      return propertyName;
    if (propertyName == null) {
      return currentPrefix;
    }
    
    String path = (String)currentPathMap.get(propertyName);
    if (path != null) {
      return path;
    }
    return currentPrefix + "." + propertyName;
  }
  


  public void profileBean(EntityBeanIntercept ebi, String prefix)
  {
    ObjectGraphNode node = request.getGraphContext().getObjectGraphNode(prefix);
    
    ebi.setNodeUsageCollector(new NodeUsageCollector(node, autoFetchManagerRef));
  }
  
  public void setCurrentPrefix(String currentPrefix, Map<String, String> currentPathMap) {
    this.currentPrefix = currentPrefix;
    this.currentPathMap = currentPathMap;
  }
}
