package com.avaje.ebeaninternal.server.query;

import com.avaje.ebean.BackgroundExecutor;
import com.avaje.ebean.Query.UseIndex;
import com.avaje.ebean.RawSql;
import com.avaje.ebean.RawSql.ColumnMapping;
import com.avaje.ebean.RawSql.ColumnMapping.Column;
import com.avaje.ebean.config.GlobalProperties;
import com.avaje.ebean.config.dbplatform.DatabasePlatform;
import com.avaje.ebean.config.dbplatform.SqlLimitRequest;
import com.avaje.ebean.config.dbplatform.SqlLimitResponse;
import com.avaje.ebean.config.dbplatform.SqlLimiter;
import com.avaje.ebean.text.PathProperties;
import com.avaje.ebeaninternal.api.ManyWhereJoins;
import com.avaje.ebeaninternal.api.SpiQuery;
import com.avaje.ebeaninternal.server.core.OrmQueryRequest;
import com.avaje.ebeaninternal.server.deploy.BeanDescriptor;
import com.avaje.ebeaninternal.server.deploy.BeanProperty;
import com.avaje.ebeaninternal.server.deploy.BeanPropertyAssocMany;
import com.avaje.ebeaninternal.server.deploy.BeanPropertyAssocOne;
import com.avaje.ebeaninternal.server.el.ElPropertyValue;
import com.avaje.ebeaninternal.server.lucene.LIndex;
import com.avaje.ebeaninternal.server.lucene.LuceneIndexManager;
import com.avaje.ebeaninternal.server.persist.Binder;
import com.avaje.ebeaninternal.server.querydefn.OrmQueryDetail;
import com.avaje.ebeaninternal.server.querydefn.OrmQueryLimitRequest;
import com.avaje.ebeaninternal.server.querydefn.OrmQueryProperties;
import java.util.Iterator;
import java.util.Set;
import javax.persistence.PersistenceException;




































public class CQueryBuilder
  implements Constants
{
  private final String tableAliasPlaceHolder;
  private final String columnAliasPrefix;
  private final SqlLimiter sqlLimiter;
  private final RawSqlSelectClauseBuilder sqlSelectBuilder;
  private final CQueryBuilderRawSql rawSqlHandler;
  private final Binder binder;
  private final BackgroundExecutor backgroundExecutor;
  private final boolean selectCountWithAlias;
  private final boolean luceneAvailable;
  private final Query.UseIndex defaultUseIndex;
  
  public CQueryBuilder(BackgroundExecutor backgroundExecutor, DatabasePlatform dbPlatform, Binder binder, LuceneIndexManager luceneIndexManager)
  {
    luceneAvailable = luceneIndexManager.isLuceneAvailable();
    defaultUseIndex = luceneIndexManager.getDefaultUseIndex();
    this.backgroundExecutor = backgroundExecutor;
    this.binder = binder;
    tableAliasPlaceHolder = GlobalProperties.get("ebean.tableAliasPlaceHolder", "${ta}");
    columnAliasPrefix = GlobalProperties.get("ebean.columnAliasPrefix", "c");
    sqlSelectBuilder = new RawSqlSelectClauseBuilder(dbPlatform, binder);
    
    sqlLimiter = dbPlatform.getSqlLimiter();
    rawSqlHandler = new CQueryBuilderRawSql(sqlLimiter);
    
    selectCountWithAlias = dbPlatform.isSelectCountWithAlias();
  }
  

  protected String getOrderBy(String orderBy, BeanPropertyAssocMany<?> many, BeanDescriptor<?> desc, boolean hasListener)
  {
    String manyOrderBy = null;
    
    if (many != null) {
      manyOrderBy = many.getFetchOrderBy();
      if (manyOrderBy != null) {
        manyOrderBy = prefixOrderByFields(many.getName(), manyOrderBy);
      }
    }
    if ((orderBy == null) && ((hasListener) || (manyOrderBy != null)))
    {
      StringBuffer sb = new StringBuffer();
      
      BeanProperty[] uids = desc.propertiesId();
      for (int i = 0; i < uids.length; i++) {
        if (i > 0) {
          sb.append(", ");
        }
        sb.append(uids[i].getName());
      }
      orderBy = sb.toString();
    }
    if (manyOrderBy != null)
    {
      orderBy = orderBy + " , " + manyOrderBy;
    }
    return orderBy;
  }
  


  public static String prefixOrderByFields(String name, String orderBy)
  {
    StringBuilder sb = new StringBuilder();
    for (String token : orderBy.split(",")) {
      if (sb.length() > 0) {
        sb.append(", ");
      }
      
      sb.append(name);
      sb.append(".");
      sb.append(token.trim());
    }
    
    return sb.toString();
  }
  



  public <T> CQueryFetchIds buildFetchIdsQuery(OrmQueryRequest<T> request)
  {
    SpiQuery<T> query = request.getQuery();
    
    query.setSelectId();
    
    CQueryPredicates predicates = new CQueryPredicates(binder, request);
    CQueryPlan queryPlan = request.getQueryPlan();
    if (queryPlan != null)
    {
      predicates.prepare(false);
      String sql = queryPlan.getSql();
      return new CQueryFetchIds(request, predicates, sql, backgroundExecutor);
    }
    
    String sql;
    String sql;
    if ((isLuceneSupported(request)) && (predicates.isLuceneResolvable()))
    {
      SqlTree sqlTree = createLuceneSqlTree(request, predicates);
      queryPlan = new CQueryPlanLucene(request, sqlTree);
      sql = "Lucene Index";
    }
    else
    {
      predicates.prepare(true);
      
      SqlTree sqlTree = createSqlTree(request, predicates);
      SqlLimitResponse s = buildSql(null, request, predicates, sqlTree);
      sql = s.getSql();
      

      queryPlan = new CQueryPlan(sql, sqlTree, false, s.isIncludesRowNumberColumn(), predicates.getLogWhereSql());
    }
    
    request.putQueryPlan(queryPlan);
    return new CQueryFetchIds(request, predicates, sql, backgroundExecutor);
  }
  



  public <T> CQueryRowCount buildRowCountQuery(OrmQueryRequest<T> request)
  {
    SpiQuery<T> query = request.getQuery();
    

    query.setOrder(null);
    
    boolean hasMany = !query.getManyWhereJoins().isEmpty();
    
    query.setSelectId();
    
    String sqlSelect = "select count(*)";
    if (hasMany)
    {
      query.setDistinct(true);
      sqlSelect = null;
    }
    
    CQueryPredicates predicates = new CQueryPredicates(binder, request);
    CQueryPlan queryPlan = request.getQueryPlan();
    if (queryPlan != null)
    {
      predicates.prepare(false);
      String sql = queryPlan.getSql();
      return new CQueryRowCount(request, predicates, sql);
    }
    
    predicates.prepare(true);
    
    SqlTree sqlTree = createSqlTree(request, predicates);
    SqlLimitResponse s = buildSql(sqlSelect, request, predicates, sqlTree);
    String sql = s.getSql();
    if (hasMany) {
      sql = "select count(*) from ( " + sql + ")";
      if (selectCountWithAlias) {
        sql = sql + " as c";
      }
    }
    

    queryPlan = new CQueryPlan(sql, sqlTree, false, s.isIncludesRowNumberColumn(), predicates.getLogWhereSql());
    request.putQueryPlan(queryPlan);
    
    return new CQueryRowCount(request, predicates, sql);
  }
  
  private boolean isLuceneSupported(OrmQueryRequest<?> request) {
    if (!luceneAvailable) {
      return false;
    }
    
    Query.UseIndex useIndex = request.getQuery().getUseIndex();
    if (useIndex == null)
    {
      useIndex = request.getBeanDescriptor().getUseIndex();
      if (useIndex == null)
      {
        useIndex = defaultUseIndex;
      }
    }
    
    if (Query.UseIndex.NO.equals(useIndex)) {
      return false;
    }
    
    return true;
  }
  




  public <T> CQuery<T> buildQuery(OrmQueryRequest<T> request)
  {
    if (request.isSqlSelect()) {
      return sqlSelectBuilder.build(request);
    }
    
    CQueryPredicates predicates = new CQueryPredicates(binder, request);
    
    CQueryPlan queryPlan = request.getQueryPlan();
    if (queryPlan != null)
    {

      if (queryPlan.isLucene()) {
        predicates.isLuceneResolvable();
      } else {
        predicates.prepare(false);
      }
      return new CQuery(request, predicates, queryPlan);
    }
    
    if ((isLuceneSupported(request)) && (predicates.isLuceneResolvable()))
    {
      SqlTree sqlTree = createLuceneSqlTree(request, predicates);
      queryPlan = new CQueryPlanLucene(request, sqlTree);




    }
    else
    {



      predicates.prepare(true);
      

      SqlTree sqlTree = createSqlTree(request, predicates);
      SqlLimitResponse res = buildSql(null, request, predicates, sqlTree);
      
      boolean rawSql = request.isRawSql();
      if (rawSql) {
        queryPlan = new CQueryPlanRawSql(request, res, sqlTree, predicates.getLogWhereSql());
      }
      else {
        queryPlan = new CQueryPlan(request, res, sqlTree, rawSql, predicates.getLogWhereSql(), null);
      }
    }
    


    request.putQueryPlan(queryPlan);
    
    return new CQuery(request, predicates, queryPlan);
  }
  












  private SqlTree createSqlTree(OrmQueryRequest<?> request, CQueryPredicates predicates)
  {
    if (request.isRawSql()) {
      return createRawSqlSqlTree(request, predicates);
    }
    
    return new SqlTreeBuilder(tableAliasPlaceHolder, columnAliasPrefix, request, predicates).build();
  }
  
  private SqlTree createLuceneSqlTree(OrmQueryRequest<?> request, CQueryPredicates predicates)
  {
    LIndex luceneIndex = request.getLuceneIndex();
    OrmQueryDetail ormQueryDetail = luceneIndex.getOrmQueryDetail();
    


    return new SqlTreeBuilder(request, predicates, ormQueryDetail).build();
  }
  
  private SqlTree createRawSqlSqlTree(OrmQueryRequest<?> request, CQueryPredicates predicates)
  {
    BeanDescriptor<?> descriptor = request.getBeanDescriptor();
    RawSql.ColumnMapping columnMapping = request.getQuery().getRawSql().getColumnMapping();
    
    PathProperties pathProps = new PathProperties();
    

    Iterator<RawSql.ColumnMapping.Column> it = columnMapping.getColumns();
    while (it.hasNext()) {
      RawSql.ColumnMapping.Column column = (RawSql.ColumnMapping.Column)it.next();
      String propertyName = column.getPropertyName();
      if (!"$$_IGNORE_COLUMN_$$".equals(propertyName))
      {
        ElPropertyValue el = descriptor.getElGetValue(propertyName);
        if (el == null) {
          String msg = "Property [" + propertyName + "] not found on " + descriptor.getFullName();
          throw new PersistenceException(msg);
        }
        BeanProperty beanProperty = el.getBeanProperty();
        if (beanProperty.isId())
        {
          propertyName = SplitName.parent(propertyName);
        } else if ((beanProperty instanceof BeanPropertyAssocOne)) {
          String msg = "Column [" + column.getDbColumn() + "] mapped to complex Property[" + propertyName + "]";
          msg = msg + ". It should be mapped to a simple property (proably the Id property). ";
          throw new PersistenceException(msg);
        }
        if (propertyName != null) {
          String[] pathProp = SplitName.split(propertyName);
          pathProps.addToPath(pathProp[0], pathProp[1]);
        }
      }
    }
    
    OrmQueryDetail detail = new OrmQueryDetail();
    

    Iterator<String> pathIt = pathProps.getPaths().iterator();
    while (pathIt.hasNext()) {
      String path = (String)pathIt.next();
      Set<String> props = pathProps.get(path);
      detail.getChunk(path, true).setDefaultProperties(null, props);
    }
    

    return new SqlTreeBuilder(request, predicates, detail).build();
  }
  
  private SqlLimitResponse buildSql(String selectClause, OrmQueryRequest<?> request, CQueryPredicates predicates, SqlTree select)
  {
    SpiQuery<?> query = request.getQuery();
    
    RawSql rawSql = query.getRawSql();
    if (rawSql != null) {
      return rawSqlHandler.buildSql(request, predicates, rawSql.getSql());
    }
    
    BeanPropertyAssocMany<?> manyProp = select.getManyProperty();
    
    boolean useSqlLimiter = false;
    
    StringBuilder sb = new StringBuilder(500);
    
    if (selectClause != null) {
      sb.append(selectClause);
    }
    else
    {
      useSqlLimiter = (query.hasMaxRowsOrFirstRow()) && (manyProp == null);
      
      if (!useSqlLimiter) {
        sb.append("select ");
        if (query.isDistinct()) {
          sb.append("distinct ");
        }
      }
      
      sb.append(select.getSelectSql());
    }
    
    sb.append(" ").append('\n');
    sb.append("from ");
    


    sb.append(select.getFromSql());
    
    String inheritanceWhere = select.getInheritanceWhereSql();
    
    boolean hasWhere = false;
    if (inheritanceWhere.length() > 0) {
      sb.append(" ").append('\n').append("where");
      sb.append(inheritanceWhere);
      hasWhere = true;
    }
    
    if ((request.isFindById()) || (query.getId() != null)) {
      if (hasWhere) {
        sb.append(" and ");
      } else {
        sb.append('\n').append("where ");
      }
      
      BeanDescriptor<?> desc = request.getBeanDescriptor();
      String idSql = desc.getIdBinderIdSql();
      sb.append(idSql).append(" ");
      hasWhere = true;
    }
    
    String dbWhere = predicates.getDbWhere();
    if (!isEmpty(dbWhere)) {
      if (!hasWhere) {
        hasWhere = true;
        sb.append(" ").append('\n').append("where ");
      } else {
        sb.append("and ");
      }
      sb.append(dbWhere);
    }
    
    String dbFilterMany = predicates.getDbFilterMany();
    if (!isEmpty(dbFilterMany)) {
      if (!hasWhere) {
        sb.append(" ").append('\n').append("where ");
      } else {
        sb.append("and ");
      }
      sb.append(dbFilterMany);
    }
    
    String dbOrderBy = predicates.getDbOrderBy();
    if (dbOrderBy != null) {
      sb.append(" ").append('\n');
      sb.append("order by ").append(dbOrderBy);
    }
    
    if (useSqlLimiter)
    {
      SqlLimitRequest r = new OrmQueryLimitRequest(sb.toString(), dbOrderBy, query);
      return sqlLimiter.limit(r);
    }
    

    return new SqlLimitResponse(sb.toString(), false);
  }
  

  private boolean isEmpty(String s)
  {
    return (s == null) || (s.length() == 0);
  }
}
