package com.avaje.ebeaninternal.server.query;

import com.avaje.ebean.config.EncryptKey;
import com.avaje.ebean.config.dbplatform.SqlLimitResponse;
import com.avaje.ebean.meta.MetaQueryStatistic;
import com.avaje.ebeaninternal.api.SpiQuery;
import com.avaje.ebeaninternal.server.core.OrmQueryRequest;
import com.avaje.ebeaninternal.server.deploy.BeanProperty;
import com.avaje.ebeaninternal.server.type.DataBind;
import com.avaje.ebeaninternal.server.type.DataReader;
import com.avaje.ebeaninternal.server.type.RsetDataReader;
import java.sql.ResultSet;
import java.sql.SQLException;






























public class CQueryPlan
{
  private final boolean autofetchTuned;
  private final int hash;
  private final boolean rawSql;
  private final boolean rowNumberIncluded;
  private final String sql;
  private final String logWhereSql;
  private final SqlTree sqlTree;
  private final BeanProperty[] encryptedProps;
  private CQueryStats queryStats = new CQueryStats();
  




  public CQueryPlan(OrmQueryRequest<?> request, SqlLimitResponse sqlRes, SqlTree sqlTree, boolean rawSql, String logWhereSql, String luceneQueryDescription)
  {
    hash = request.getQueryPlanHash();
    autofetchTuned = request.getQuery().isAutofetchTuned();
    if (sqlRes != null) {
      sql = sqlRes.getSql();
      rowNumberIncluded = sqlRes.isIncludesRowNumberColumn();
    } else {
      sql = luceneQueryDescription;
      rowNumberIncluded = false;
    }
    this.sqlTree = sqlTree;
    this.rawSql = rawSql;
    this.logWhereSql = logWhereSql;
    encryptedProps = sqlTree.getEncryptedProps();
  }
  




  public CQueryPlan(String sql, SqlTree sqlTree, boolean rawSql, boolean rowNumberIncluded, String logWhereSql)
  {
    hash = 0;
    autofetchTuned = false;
    this.sql = sql;
    this.sqlTree = sqlTree;
    this.rawSql = rawSql;
    this.rowNumberIncluded = rowNumberIncluded;
    this.logWhereSql = logWhereSql;
    encryptedProps = sqlTree.getEncryptedProps();
  }
  
  public boolean isLucene() {
    return false;
  }
  
  public DataReader createDataReader(ResultSet rset)
  {
    return new RsetDataReader(rset);
  }
  
  public void bindEncryptedProperties(DataBind dataBind) throws SQLException {
    if (encryptedProps != null) {
      for (int i = 0; i < encryptedProps.length; i++) {
        String key = encryptedProps[i].getEncryptKey().getStringValue();
        dataBind.setString(key);
      }
    }
  }
  
  public boolean isAutofetchTuned() {
    return autofetchTuned;
  }
  
  public int getHash() {
    return hash;
  }
  
  public String getSql() {
    return sql;
  }
  
  public SqlTree getSqlTree() {
    return sqlTree;
  }
  
  public boolean isRawSql() {
    return rawSql;
  }
  
  public boolean isRowNumberIncluded() {
    return rowNumberIncluded;
  }
  
  public String getLogWhereSql() {
    return logWhereSql;
  }
  


  public void resetStatistics()
  {
    queryStats = new CQueryStats();
  }
  



  public void executionTime(int loadedBeanCount, int timeMicros)
  {
    queryStats = queryStats.add(loadedBeanCount, timeMicros);
  }
  


  public CQueryStats getQueryStats()
  {
    return queryStats;
  }
  


  public long getLastQueryTime()
  {
    return queryStats.getLastQueryTime();
  }
  
  public MetaQueryStatistic createMetaQueryStatistic(String beanName) {
    return queryStats.createMetaQueryStatistic(beanName, this);
  }
}
