package com.avaje.ebeaninternal.server.query;

import com.avaje.ebean.RawSql;
import com.avaje.ebean.RawSql.Sql;
import com.avaje.ebeaninternal.api.BindParams;
import com.avaje.ebeaninternal.api.BindParams.OrderedList;
import com.avaje.ebeaninternal.api.SpiExpressionList;
import com.avaje.ebeaninternal.api.SpiQuery;
import com.avaje.ebeaninternal.server.core.OrmQueryRequest;
import com.avaje.ebeaninternal.server.deploy.BeanDescriptor;
import com.avaje.ebeaninternal.server.deploy.BeanPropertyAssocMany;
import com.avaje.ebeaninternal.server.deploy.DeployParser;
import com.avaje.ebeaninternal.server.persist.Binder;
import com.avaje.ebeaninternal.server.querydefn.OrmQueryDetail;
import com.avaje.ebeaninternal.server.querydefn.OrmQueryProperties;
import com.avaje.ebeaninternal.server.type.DataBind;
import com.avaje.ebeaninternal.server.util.BindParamsParser;
import com.avaje.ebeaninternal.util.DefaultExpressionRequest;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;










public class CQueryPredicates
{
  private static final Logger logger = Logger.getLogger(CQueryPredicates.class.getName());
  


  private final Binder binder;
  


  private final OrmQueryRequest<?> request;
  


  private final SpiQuery<?> query;
  


  private final Object idValue;
  


  private boolean rawSql;
  


  private final BindParams bindParams;
  


  private BindParams.OrderedList havingNamedParams;
  


  private ArrayList<Object> filterManyExprBindValues;
  


  private String filterManyExprSql;
  


  private ArrayList<Object> whereExprBindValues;
  


  private String whereExprSql;
  


  private String whereRawSql;
  


  private ArrayList<Object> havingExprBindValues;
  


  private String havingExprSql;
  


  private String havingRawSql;
  


  private String dbHaving;
  


  private String dbWhere;
  


  private String dbFilterMany;
  


  private String logicalOrderBy;
  


  private String dbOrderBy;
  


  private Set<String> predicateIncludes;
  


  public CQueryPredicates(Binder binder, OrmQueryRequest<?> request)
  {
    this.binder = binder;
    this.request = request;
    query = request.getQuery();
    bindParams = query.getBindParams();
    idValue = query.getId();
  }
  




  public boolean isLuceneResolvable()
  {
    CQueryPredicatesLuceneResolve luceneResolve = new CQueryPredicatesLuceneResolve(request);
    return luceneResolve.isLuceneResolvable();
  }
  
  public String bind(DataBind dataBind) throws SQLException
  {
    StringBuilder bindLog = new StringBuilder();
    
    if (idValue != null)
    {
      request.getBeanDescriptor().bindId(dataBind, idValue);
      bindLog.append(idValue);
    }
    
    if (bindParams != null)
    {
      binder.bind(bindParams, dataBind, bindLog);
    }
    
    if (whereExprBindValues != null)
    {
      for (int i = 0; i < whereExprBindValues.size(); i++) {
        Object bindValue = whereExprBindValues.get(i);
        binder.bindObject(dataBind, bindValue);
        if ((i > 0) || (idValue != null)) {
          bindLog.append(", ");
        }
        bindLog.append(bindValue);
      }
    }
    
    if (filterManyExprBindValues != null)
    {
      for (int i = 0; i < filterManyExprBindValues.size(); i++) {
        Object bindValue = filterManyExprBindValues.get(i);
        binder.bindObject(dataBind, bindValue);
        if ((i > 0) || (idValue != null)) {
          bindLog.append(", ");
        }
        bindLog.append(bindValue);
      }
    }
    
    if (havingNamedParams != null)
    {
      bindLog.append(" havingNamed ");
      binder.bind(havingNamedParams.list(), dataBind, bindLog);
    }
    
    if (havingExprBindValues != null)
    {
      bindLog.append(" having ");
      for (int i = 0; i < havingExprBindValues.size(); i++) {
        Object bindValue = havingExprBindValues.get(i);
        binder.bindObject(dataBind, bindValue);
        if (i > 0) {
          bindLog.append(", ");
        }
        bindLog.append(bindValue);
      }
    }
    
    return bindLog.toString();
  }
  
  private void buildBindHavingRawSql(boolean buildSql, boolean parseRaw, DeployParser deployParser) {
    if ((buildSql) || (bindParams != null))
    {
      havingRawSql = query.getAdditionalHaving();
      if (parseRaw) {
        havingRawSql = deployParser.parse(havingRawSql);
      }
      if ((havingRawSql != null) && (bindParams != null))
      {
        havingNamedParams = BindParamsParser.parseNamedParams(bindParams, havingRawSql);
        havingRawSql = havingNamedParams.getPreparedSql();
      }
    }
  }
  


  private void buildBindWhereRawSql(boolean buildSql, boolean parseRaw, DeployParser parser)
  {
    if ((buildSql) || (bindParams != null)) {
      whereRawSql = buildWhereRawSql();
      boolean hasRaw = !"".equals(whereRawSql);
      if ((hasRaw) && (parseRaw))
      {


        parser.setEncrypted(true);
        whereRawSql = parser.parse(whereRawSql);
        parser.setEncrypted(false);
      }
      
      if (bindParams != null) {
        if (hasRaw) {
          whereRawSql = BindParamsParser.parse(bindParams, whereRawSql, request.getBeanDescriptor());
        }
        else if ((query.isRawSql()) && (!buildSql))
        {


          String s = query.getRawSql().getSql().getPreWhere();
          if (bindParams.requiresNamedParamsPrepare()) {
            BindParamsParser.parse(bindParams, s);
          }
        }
      }
    }
  }
  

  private String buildWhereRawSql()
  {
    String whereRaw = query.getRawWhereClause();
    if (whereRaw == null) {
      whereRaw = "";
    }
    
    String additionalWhere = query.getAdditionalWhere();
    if (additionalWhere != null) {
      whereRaw = whereRaw + additionalWhere;
    }
    return whereRaw;
  }
  
  public void prepare(boolean buildSql)
  {
    DeployParser deployParser = request.createDeployParser();
    
    prepare(buildSql, true, deployParser);
  }
  
  public void prepareRawSql(DeployParser deployParser) {
    prepare(true, false, deployParser);
  }
  



  private void prepare(boolean buildSql, boolean parseRaw, DeployParser deployParser)
  {
    buildBindWhereRawSql(buildSql, parseRaw, deployParser);
    buildBindHavingRawSql(buildSql, parseRaw, deployParser);
    
    SpiExpressionList<?> whereExp = query.getWhereExpressions();
    if (whereExp != null) {
      DefaultExpressionRequest whereReq = new DefaultExpressionRequest(request, deployParser);
      whereExprBindValues = whereExp.buildBindValues(whereReq);
      if (buildSql) {
        whereExprSql = whereExp.buildSql(whereReq);
      }
    }
    
    BeanPropertyAssocMany<?> manyProperty = request.getManyProperty();
    if (manyProperty != null) {
      OrmQueryProperties chunk = query.getDetail().getChunk(manyProperty.getName(), false);
      SpiExpressionList<?> filterMany = chunk.getFilterMany();
      if (filterMany != null) {
        DefaultExpressionRequest filterReq = new DefaultExpressionRequest(request, deployParser);
        filterManyExprBindValues = filterMany.buildBindValues(filterReq);
        if (buildSql) {
          filterManyExprSql = filterMany.buildSql(filterReq);
        }
      }
    }
    

    SpiExpressionList<?> havingExpr = query.getHavingExpressions();
    if (havingExpr != null) {
      DefaultExpressionRequest havingReq = new DefaultExpressionRequest(request, deployParser);
      havingExprBindValues = havingExpr.buildBindValues(havingReq);
      if (buildSql) {
        havingExprSql = havingExpr.buildSql(havingReq);
      }
    }
    
    if (buildSql) {
      parsePropertiesToDbColumns(deployParser);
    }
  }
  





  private void parsePropertiesToDbColumns(DeployParser deployParser)
  {
    dbWhere = deriveWhere(deployParser);
    dbFilterMany = deriveFilterMany(deployParser);
    dbHaving = deriveHaving(deployParser);
    

    logicalOrderBy = deriveOrderByWithMany(request.getManyProperty());
    if (logicalOrderBy != null) {
      dbOrderBy = deployParser.parse(logicalOrderBy);
    }
    
    predicateIncludes = deployParser.getIncludes();
  }
  
  private String deriveFilterMany(DeployParser deployParser) {
    if (isEmpty(filterManyExprSql)) {
      return null;
    }
    return deployParser.parse(filterManyExprSql);
  }
  
  private String deriveWhere(DeployParser deployParser)
  {
    return parse(whereRawSql, whereExprSql, deployParser);
  }
  


  public void parseTableAlias(SqlTreeAlias alias)
  {
    if (dbWhere != null)
    {
      dbWhere = alias.parseWhere(dbWhere);
    }
    if (dbFilterMany != null)
    {
      dbFilterMany = alias.parse(dbFilterMany);
    }
    if (dbHaving != null) {
      dbHaving = alias.parseWhere(dbHaving);
    }
    if (dbOrderBy != null) {
      dbOrderBy = alias.parse(dbOrderBy);
    }
  }
  
  private boolean isEmpty(String s) {
    return (s == null) || (s.length() == 0);
  }
  
  private String parse(String raw, String expr, DeployParser deployParser)
  {
    StringBuilder sb = new StringBuilder();
    if (!isEmpty(raw)) {
      sb.append(raw);
    }
    if (!isEmpty(expr)) {
      if (sb.length() > 0) {
        sb.append(" and ");
      }
      sb.append(deployParser.parse(expr));
    }
    return sb.toString();
  }
  
  private String deriveHaving(DeployParser deployParser) {
    return parse(havingRawSql, havingExprSql, deployParser);
  }
  
  private String parseOrderBy()
  {
    return CQueryOrderBy.parse(request.getBeanDescriptor(), query);
  }
  




  private String deriveOrderByWithMany(BeanPropertyAssocMany<?> manyProp)
  {
    if (manyProp == null) {
      return parseOrderBy();
    }
    
    String orderBy = parseOrderBy();
    
    BeanDescriptor<?> desc = request.getBeanDescriptor();
    String orderById = desc.getDefaultOrderBy();
    
    if (orderBy == null) {
      orderBy = orderById;
    }
    

    String manyOrderBy = manyProp.getFetchOrderBy();
    if (manyOrderBy != null) {
      orderBy = orderBy + " , " + CQueryBuilder.prefixOrderByFields(manyProp.getName(), manyOrderBy);
    }
    
    if (request.isFindById())
    {
      return orderBy;
    }
    
    if (orderBy.startsWith(orderById)) {
      return orderBy;
    }
    




    int manyPos = orderBy.indexOf(manyProp.getName());
    int idPos = orderBy.indexOf(" " + orderById);
    
    if (manyPos == -1)
    {
      return orderBy;
    }
    if ((idPos <= -1) || (idPos >= manyPos))
    {


      if (idPos > manyPos)
      {
        String msg = "A Query on [" + desc + "] includes a join to a 'many' association [" + manyProp.getName();
        
        msg = msg + "] with an incorrect orderBy [" + orderBy + "]. The id property [" + orderById + "]";
        msg = msg + " must come before the many property [" + manyProp.getName() + "] in the orderBy.";
        msg = msg + " Ebean has automatically modified the orderBy clause to do this.";
        
        logger.log(Level.WARNING, msg);
      }
      

      orderBy = orderBy.substring(0, manyPos) + orderById + ", " + orderBy.substring(manyPos);
    }
    
    return orderBy;
  }
  


  public ArrayList<Object> getWhereExprBindValues()
  {
    return whereExprBindValues;
  }
  


  public String getDbHaving()
  {
    return dbHaving;
  }
  



  public String getDbWhere()
  {
    return dbWhere;
  }
  



  public String getDbFilterMany()
  {
    return dbFilterMany;
  }
  


  public String getDbOrderBy()
  {
    return dbOrderBy;
  }
  


  public Set<String> getPredicateIncludes()
  {
    return predicateIncludes;
  }
  


  public String getWhereRawSql()
  {
    return whereRawSql;
  }
  


  public String getWhereExpressionSql()
  {
    return whereExprSql;
  }
  


  public String getHavingRawSql()
  {
    return havingRawSql;
  }
  


  public String getHavingExpressionSql()
  {
    return havingExprSql;
  }
  
  public String getLogWhereSql() {
    if (rawSql) {
      return "";
    }
    if ((dbWhere == null) && (dbFilterMany == null)) {
      return "";
    }
    StringBuilder sb = new StringBuilder();
    if (dbWhere != null) {
      sb.append(dbWhere);
    }
    if (dbFilterMany != null) {
      if (sb.length() > 0) {
        sb.append(" and ");
      }
      sb.append(dbFilterMany);
    }
    String logPred = sb.toString();
    if (logPred.length() > 400) {
      logPred = logPred.substring(0, 400) + " ...";
    }
    return logPred;
  }
}
