package com.avaje.ebeaninternal.server.query;

import com.avaje.ebean.OrderBy;
import com.avaje.ebeaninternal.api.BindParams;
import com.avaje.ebeaninternal.api.SpiExpressionList;
import com.avaje.ebeaninternal.api.SpiLuceneExpr;
import com.avaje.ebeaninternal.api.SpiLuceneExpr.ExprOccur;
import com.avaje.ebeaninternal.api.SpiQuery;
import com.avaje.ebeaninternal.server.core.LuceneOrmQueryRequest;
import com.avaje.ebeaninternal.server.core.OrmQueryRequest;
import com.avaje.ebeaninternal.server.expression.PersistenceLuceneParseException;
import com.avaje.ebeaninternal.server.lucene.LIndex;
import com.avaje.ebeaninternal.server.lucene.LLuceneSortResolve;
import com.avaje.ebeaninternal.util.DefaultExpressionRequest;
import java.util.logging.Logger;
import javax.persistence.PersistenceException;
import org.apache.lucene.search.MatchAllDocsQuery;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.Sort;






















public class CQueryPredicatesLuceneResolve
{
  private static final Logger logger = Logger.getLogger(CQueryPredicatesLuceneResolve.class.getName());
  

  private final OrmQueryRequest<?> request;
  
  private final SpiQuery<?> query;
  
  private final BindParams bindParams;
  

  public CQueryPredicatesLuceneResolve(OrmQueryRequest<?> request)
  {
    this.request = request;
    query = request.getQuery();
    bindParams = query.getBindParams();
  }
  







  public boolean isLuceneResolvable()
  {
    LIndex luceneIndex = request.getLuceneIndex();
    if (luceneIndex == null)
    {
      return false;
    }
    if (bindParams != null)
    {
      return false;
    }
    if (query.getHavingExpressions() != null)
    {
      return false;
    }
    
    LuceneResolvableRequest req = new LuceneResolvableRequest(request.getBeanDescriptor(), luceneIndex);
    
    LLuceneSortResolve lucenSortResolve = new LLuceneSortResolve(req, query.getOrderBy());
    
    if (!lucenSortResolve.isResolved()) {
      logger.info("Lucene Index can't support sort/orderBy of [" + lucenSortResolve.getUnsortableField() + "]");
      return false;
    }
    
    Sort luceneSort = lucenSortResolve.getSort();
    
    OrderBy<?> orderBy = query.getOrderBy();
    String sortDesc = orderBy == null ? "" : orderBy.toStringFormat();
    
    SpiExpressionList<?> whereExp = query.getWhereExpressions();
    if (whereExp == null)
    {
      MatchAllDocsQuery q = new MatchAllDocsQuery();
      request.setLuceneOrmQueryRequest(new LuceneOrmQueryRequest(q, luceneSort, "MatchAllDocs", sortDesc));
      return true;
    }
    
    if (!whereExp.isLuceneResolvable(req))
    {
      return false;
    }
    
    try
    {
      DefaultExpressionRequest whereReq = new DefaultExpressionRequest(request, luceneIndex);
      SpiLuceneExpr luceneExpr = whereExp.createLuceneExpr(whereReq, SpiLuceneExpr.ExprOccur.MUST);
      
      Query luceneQuery = luceneExpr.mergeLuceneQuery();
      String luceneDesc = luceneExpr.getDescription();
      request.setLuceneOrmQueryRequest(new LuceneOrmQueryRequest(luceneQuery, luceneSort, luceneDesc, sortDesc));
      return true;
    }
    catch (PersistenceLuceneParseException e)
    {
      String msg = "Failed to parse the Query using Lucene";
      throw new PersistenceException(msg, e);
    }
  }
}
