package com.avaje.ebeaninternal.server.query;

import com.avaje.ebeaninternal.api.SpiQuery;
import com.avaje.ebeaninternal.api.SpiTransaction;
import com.avaje.ebeaninternal.server.core.OrmQueryRequest;
import com.avaje.ebeaninternal.server.core.SpiOrmQueryRequest;
import com.avaje.ebeaninternal.server.deploy.BeanDescriptor;
import com.avaje.ebeaninternal.server.type.DataBind;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.persistence.PersistenceException;

























public class CQueryRowCount
{
  private static final Logger logger = Logger.getLogger(CQueryRowCount.class.getName());
  


  private final OrmQueryRequest<?> request;
  


  private final BeanDescriptor<?> desc;
  


  private final SpiQuery<?> query;
  


  private final CQueryPredicates predicates;
  

  private final String sql;
  

  private ResultSet rset;
  

  private PreparedStatement pstmt;
  

  private String bindLog;
  

  private long startNano;
  

  private int executionTimeMicros;
  

  private int rowCount;
  


  public CQueryRowCount(OrmQueryRequest<?> request, CQueryPredicates predicates, String sql)
  {
    this.request = request;
    query = request.getQuery();
    this.sql = sql;
    
    query.setGeneratedSql(sql);
    
    desc = request.getBeanDescriptor();
    this.predicates = predicates;
  }
  



  public String getSummary()
  {
    StringBuilder sb = new StringBuilder();
    sb.append("FindRowCount exeMicros[").append(executionTimeMicros).append("] rows[").append(rowCount).append("] type[").append(desc.getFullName()).append("] predicates[").append(predicates.getLogWhereSql()).append("] bind[").append(bindLog).append("]");
    




    return sb.toString();
  }
  


  public String getBindLog()
  {
    return bindLog;
  }
  


  public String getGeneratedSql()
  {
    return sql;
  }
  
  public SpiOrmQueryRequest<?> getQueryRequest() {
    return request;
  }
  


  public int findRowCount()
    throws SQLException
  {
    startNano = System.nanoTime();
    try
    {
      SpiTransaction t = request.getTransaction();
      Connection conn = t.getInternalConnection();
      pstmt = conn.prepareStatement(sql);
      
      if (query.getTimeout() > 0) {
        pstmt.setQueryTimeout(query.getTimeout());
      }
      
      bindLog = predicates.bind(new DataBind(pstmt));
      
      rset = pstmt.executeQuery();
      
      if (!rset.next()) {
        throw new PersistenceException("Expecting 1 row but got none?");
      }
      
      rowCount = rset.getInt(1);
      
      long exeNano = System.nanoTime() - startNano;
      executionTimeMicros = ((int)exeNano / 1000);
      
      return rowCount;
    }
    finally {
      close();
    }
  }
  





  private void close()
  {
    try
    {
      if (rset != null) {
        rset.close();
        rset = null;
      }
    } catch (SQLException e) {
      logger.log(Level.SEVERE, null, e);
    }
    try {
      if (pstmt != null) {
        pstmt.close();
        pstmt = null;
      }
    } catch (SQLException e) {
      logger.log(Level.SEVERE, null, e);
    }
  }
}
