package com.avaje.ebeaninternal.server.query;

import com.avaje.ebeaninternal.server.deploy.BeanProperty;
import com.avaje.ebeaninternal.server.deploy.DbSqlContext;
import com.avaje.ebeaninternal.server.deploy.TableJoinColumn;
import com.avaje.ebeaninternal.server.lib.util.StringHelper;
import com.avaje.ebeaninternal.server.util.ArrayStack;
import java.util.ArrayList;
import java.util.HashSet;






public class DefaultDbSqlContext
  implements DbSqlContext
{
  private static final String NEW_LINE = "\n";
  private static final String COMMA = ", ";
  private static final String PERIOD = ".";
  private final String tableAliasPlaceHolder;
  private final String columnAliasPrefix;
  private final ArrayStack<String> tableAliasStack = new ArrayStack();
  
  private final ArrayStack<String> joinStack = new ArrayStack();
  
  private final ArrayStack<String> prefixStack = new ArrayStack();
  
  private final boolean useColumnAlias;
  
  private int columnIndex;
  
  private StringBuilder sb = new StringBuilder(140);
  

  private HashSet<String> formulaJoins;
  

  private HashSet<String> tableJoins;
  

  private SqlTreeAlias alias;
  

  private String currentPrefix;
  
  private ArrayList<BeanProperty> encryptedProps;
  

  public DefaultDbSqlContext(SqlTreeAlias alias, String tableAliasPlaceHolder)
  {
    this.tableAliasPlaceHolder = tableAliasPlaceHolder;
    columnAliasPrefix = null;
    useColumnAlias = false;
    this.alias = alias;
  }
  



  public DefaultDbSqlContext(SqlTreeAlias alias, String tableAliasPlaceHolder, String columnAliasPrefix, boolean alwaysUseColumnAlias)
  {
    this.alias = alias;
    this.tableAliasPlaceHolder = tableAliasPlaceHolder;
    this.columnAliasPrefix = columnAliasPrefix;
    useColumnAlias = alwaysUseColumnAlias;
  }
  
  public void addEncryptedProp(BeanProperty p) {
    if (encryptedProps == null) {
      encryptedProps = new ArrayList();
    }
    encryptedProps.add(p);
  }
  
  public BeanProperty[] getEncryptedProps() {
    if (encryptedProps == null) {
      return null;
    }
    
    return (BeanProperty[])encryptedProps.toArray(new BeanProperty[encryptedProps.size()]);
  }
  
  public String peekJoin() {
    return (String)joinStack.peek();
  }
  
  public void popJoin() {
    joinStack.pop();
  }
  
  public void pushJoin(String node) {
    joinStack.push(node);
  }
  
  public void addJoin(String type, String table, TableJoinColumn[] cols, String a1, String a2)
  {
    if (tableJoins == null) {
      tableJoins = new HashSet();
    }
    
    String joinKey = table + "-" + a1 + "-" + a2;
    if (tableJoins.contains(joinKey)) {
      return;
    }
    
    tableJoins.add(joinKey);
    
    sb.append("\n");
    sb.append(type);
    
    sb.append(" ").append(table).append(" ");
    sb.append(a2);
    
    sb.append(" on ");
    
    for (int i = 0; i < cols.length; i++) {
      TableJoinColumn pair = cols[i];
      if (i > 0) {
        sb.append(" and ");
      }
      
      sb.append(a2);
      sb.append(".").append(pair.getForeignDbColumn());
      sb.append(" = ");
      sb.append(a1);
      sb.append(".").append(pair.getLocalDbColumn());
    }
    
    sb.append(" ");
  }
  
  public String getTableAlias(String prefix) {
    return alias.getTableAlias(prefix);
  }
  
  public String getTableAliasManyWhere(String prefix) {
    return alias.getTableAliasManyWhere(prefix);
  }
  
  public void pushSecondaryTableAlias(String alias) {
    tableAliasStack.push(alias);
  }
  
  public String getRelativePrefix(String propName)
  {
    return currentPrefix + "." + propName;
  }
  
  public void pushTableAlias(String prefix)
  {
    prefixStack.push(currentPrefix);
    currentPrefix = prefix;
    tableAliasStack.push(getTableAlias(prefix));
  }
  
  public void popTableAlias() {
    tableAliasStack.pop();
    
    currentPrefix = ((String)prefixStack.pop());
  }
  
  public StringBuilder getBuffer() {
    return sb;
  }
  
  public DefaultDbSqlContext append(String s) {
    sb.append(s);
    return this;
  }
  
  public DefaultDbSqlContext append(char s) {
    sb.append(s);
    return this;
  }
  

  public void appendFormulaJoin(String sqlFormulaJoin, boolean forceOuterJoin)
  {
    String tableAlias = (String)tableAliasStack.peek();
    String converted = StringHelper.replaceString(sqlFormulaJoin, tableAliasPlaceHolder, tableAlias);
    
    if (formulaJoins == null) {
      formulaJoins = new HashSet();
    }
    else if (formulaJoins.contains(converted))
    {

      return;
    }
    

    formulaJoins.add(converted);
    
    sb.append("\n");
    
    if ((forceOuterJoin) && 
      ("join".equals(sqlFormulaJoin.substring(0, 4).toLowerCase())))
    {
      append(" left outer ");
    }
    

    sb.append(converted);
    sb.append(" ");
  }
  
  public void appendFormulaSelect(String sqlFormulaSelect)
  {
    String tableAlias = (String)tableAliasStack.peek();
    String converted = StringHelper.replaceString(sqlFormulaSelect, tableAliasPlaceHolder, tableAlias);
    
    sb.append(", ");
    sb.append(converted);
  }
  
  public void appendColumn(String column) {
    appendColumn((String)tableAliasStack.peek(), column);
  }
  
  public void appendColumn(String tableAlias, String column) {
    sb.append(", ");
    
    if (column.indexOf("${}") > -1)
    {

      String x = StringHelper.replaceString(column, "${}", tableAlias);
      sb.append(x);
    } else {
      sb.append(tableAlias);
      sb.append(".");
      sb.append(column);
    }
    if (useColumnAlias) {
      sb.append(" ");
      sb.append(columnAliasPrefix);
      sb.append(columnIndex);
    }
    columnIndex += 1;
  }
  
  public String peekTableAlias() {
    return (String)tableAliasStack.peek();
  }
  
  public void appendRawColumn(String rawcolumnWithTableAlias) {
    sb.append(", ");
    sb.append(rawcolumnWithTableAlias);
    
    if (useColumnAlias) {
      sb.append(" ");
      sb.append(columnAliasPrefix);
      sb.append(columnIndex);
    }
    columnIndex += 1;
  }
  
  public int length() {
    return sb.length();
  }
  
  public String getContent() {
    String s = sb.toString();
    sb = new StringBuilder();
    return s;
  }
  
  public String toString() {
    return "DefaultDbSqlContext: " + sb.toString();
  }
}
