package com.avaje.ebeaninternal.server.query;

import com.avaje.ebean.QueryIterator;
import com.avaje.ebean.bean.BeanCollection;
import com.avaje.ebean.event.BeanFinder;
import com.avaje.ebeaninternal.api.BeanIdList;
import com.avaje.ebeaninternal.api.SpiQuery;
import com.avaje.ebeaninternal.api.SpiTransaction;
import com.avaje.ebeaninternal.server.core.CopyBeanCollection;
import com.avaje.ebeaninternal.server.core.OrmQueryEngine;
import com.avaje.ebeaninternal.server.core.OrmQueryRequest;
import com.avaje.ebeaninternal.server.deploy.BeanDescriptor;
import com.avaje.ebeaninternal.server.deploy.BeanDescriptorManager;
import com.avaje.ebeaninternal.server.deploy.CopyContext;
import java.util.Collection;
import java.util.Iterator;





























public class DefaultOrmQueryEngine
  implements OrmQueryEngine
{
  private final CQueryEngine queryEngine;
  
  public DefaultOrmQueryEngine(BeanDescriptorManager descMgr, CQueryEngine queryEngine)
  {
    this.queryEngine = queryEngine;
  }
  
  public <T> int findRowCount(OrmQueryRequest<T> request)
  {
    return queryEngine.findRowCount(request);
  }
  
  public <T> BeanIdList findIds(OrmQueryRequest<T> request)
  {
    return queryEngine.findIds(request);
  }
  



  public <T> QueryIterator<T> findIterate(OrmQueryRequest<T> request)
  {
    SpiTransaction t = request.getTransaction();
    



    t.flushBatch();
    
    return queryEngine.findIterate(request);
  }
  
  public <T> BeanCollection<T> findMany(OrmQueryRequest<T> request)
  {
    SpiQuery<T> query = request.getQuery();
    if ((query.isUseQueryCache()) || (query.isLoadBeanCache()))
    {

      query.setSharedInstance();
    }
    
    BeanCollection<T> result = null;
    
    SpiTransaction t = request.getTransaction();
    



    t.flushBatch();
    
    BeanFinder<T> finder = request.getBeanFinder();
    if (finder != null)
    {
      result = finder.findMany(request);
    } else {
      result = queryEngine.findMany(request);
    }
    
    if (query.isLoadBeanCache())
    {
      BeanDescriptor<T> descriptor = request.getBeanDescriptor();
      Collection<T> c = result.getActualDetails();
      Iterator<T> it = c.iterator();
      while (it.hasNext()) {
        T bean = it.next();
        descriptor.cachePut(bean, query.isSharedInstance());
      }
    }
    
    if ((query.isSharedInstance()) && (!result.isEmpty())) {
      if (query.isUseQueryCache())
      {
        request.putToQueryCache(result);
      }
      if (Boolean.FALSE.equals(query.isReadOnly()))
      {
        CopyContext ctx = new CopyContext(request.isVanillaMode(), false);
        result = new CopyBeanCollection(result, request.getBeanDescriptor(), ctx, 5).copy();
      }
    }
    
    return result;
  }
  




  public <T> T findId(OrmQueryRequest<T> request)
  {
    T result = null;
    
    SpiTransaction t = request.getTransaction();
    
    if (t.isBatchFlushOnQuery())
    {


      t.flushBatch();
    }
    
    BeanFinder<T> finder = request.getBeanFinder();
    if (finder != null) {
      result = finder.find(request);
    } else {
      result = queryEngine.find(request);
    }
    
    if ((result != null) && (request.isUseBeanCache())) {
      BeanDescriptor<T> descriptor = request.getBeanDescriptor();
      descriptor.cachePut(result, request.isUseBeanCacheReadOnly());
    }
    
    return result;
  }
}
