package com.avaje.ebeaninternal.server.query;

import com.avaje.ebean.bean.EntityBean;
import com.avaje.ebean.bean.EntityBeanIntercept;
import com.avaje.ebeaninternal.api.SpiQuery.Mode;
import com.avaje.ebeaninternal.server.deploy.BeanProperty;
import com.avaje.ebeaninternal.server.deploy.BeanPropertyAssocMany;
import com.avaje.ebeaninternal.server.deploy.DbReadContext;
import com.avaje.ebeaninternal.server.type.DataReader;
import java.sql.SQLException;
import java.util.Set;
import javax.persistence.PersistenceException;
































public class SqlBeanLoad
{
  private final DbReadContext ctx;
  private final Object bean;
  private final Class<?> type;
  private final Object originalOldValues;
  private final boolean isLazyLoad;
  private final Set<String> excludes;
  private final boolean setOriginalOldValues;
  private final boolean rawSql;
  
  public SqlBeanLoad(DbReadContext ctx, Class<?> type, Object bean, SpiQuery.Mode queryMode)
  {
    this.ctx = ctx;
    rawSql = ctx.isRawSql();
    this.type = type;
    isLazyLoad = queryMode.equals(SpiQuery.Mode.LAZYLOAD_BEAN);
    this.bean = bean;
    
    if ((bean instanceof EntityBean)) {
      EntityBeanIntercept ebi = ((EntityBean)bean)._ebean_getIntercept();
      
      excludes = (isLazyLoad ? ebi.getLoadedProps() : null);
      if (excludes != null)
      {


        originalOldValues = ebi.getOldValues();
      } else {
        originalOldValues = null;
      }
      setOriginalOldValues = (originalOldValues != null);
    } else {
      excludes = null;
      originalOldValues = null;
      setOriginalOldValues = false;
    }
  }
  


  public boolean isLazyLoad()
  {
    return isLazyLoad;
  }
  


  public void loadIgnore(int increment)
  {
    ctx.getDataReader().incrementPos(increment);
  }
  
  public Object load(BeanProperty prop) throws SQLException
  {
    if ((!rawSql) && (prop.isTransient())) {
      return null;
    }
    
    if ((bean == null) || ((excludes != null) && (excludes.contains(prop.getName()))) || ((type != null) && (!prop.isAssignableFrom(type))))
    {







      prop.loadIgnore(ctx);
      return null;
    }
    try
    {
      Object dbVal = prop.read(ctx);
      if (isLazyLoad) {
        prop.setValue(bean, dbVal);
      } else {
        prop.setValueIntercept(bean, dbVal);
      }
      if (setOriginalOldValues)
      {
        prop.setValue(originalOldValues, dbVal);
      }
      return dbVal;
    }
    catch (Exception e) {
      String msg = "Error loading on " + prop.getFullBeanName();
      throw new PersistenceException(msg, e);
    }
  }
  
  public void loadAssocMany(BeanPropertyAssocMany<?> prop) {}
}
