package com.avaje.ebeaninternal.server.query;

import com.avaje.ebean.bean.BeanCollection;
import com.avaje.ebean.bean.EntityBean;
import com.avaje.ebean.bean.EntityBeanIntercept;
import com.avaje.ebean.bean.PersistenceContext;
import com.avaje.ebeaninternal.api.SpiQuery.Mode;
import com.avaje.ebeaninternal.server.deploy.BeanDescriptor;
import com.avaje.ebeaninternal.server.deploy.BeanProperty;
import com.avaje.ebeaninternal.server.deploy.BeanPropertyAssoc;
import com.avaje.ebeaninternal.server.deploy.BeanPropertyAssocMany;
import com.avaje.ebeaninternal.server.deploy.DbReadContext;
import com.avaje.ebeaninternal.server.deploy.DbSqlContext;
import com.avaje.ebeaninternal.server.deploy.InheritInfo;
import com.avaje.ebeaninternal.server.deploy.TableJoin;
import com.avaje.ebeaninternal.server.deploy.id.IdBinder;
import com.avaje.ebeaninternal.server.lib.util.StringHelper;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;























public class SqlTreeNodeBean
  implements SqlTreeNode
{
  private static final SqlTreeNode[] NO_CHILDREN = new SqlTreeNode[0];
  


  final BeanDescriptor<?> desc;
  


  final IdBinder idBinder;
  

  final SqlTreeNode[] children;
  

  final boolean readOnlyLeaf;
  

  final boolean partialObject;
  

  final Set<String> partialProps;
  

  final int partialHash;
  

  final BeanProperty[] properties;
  

  final String extraWhere;
  

  final BeanPropertyAssoc<?> nodeBeanProp;
  

  final TableJoin[] tableJoins;
  

  final boolean readId;
  

  final boolean disableLazyLoad;
  

  final InheritInfo inheritInfo;
  

  final String prefix;
  

  final Set<String> includedProps;
  

  final Map<String, String> pathMap;
  


  public SqlTreeNodeBean(String prefix, BeanPropertyAssoc<?> beanProp, SqlTreeProperties props, List<SqlTreeNode> myChildren, boolean withId)
  {
    this(prefix, beanProp, beanProp.getTargetDescriptor(), props, myChildren, withId);
  }
  




  public SqlTreeNodeBean(String prefix, BeanPropertyAssoc<?> beanProp, BeanDescriptor<?> desc, SqlTreeProperties props, List<SqlTreeNode> myChildren, boolean withId)
  {
    this.prefix = prefix;
    nodeBeanProp = beanProp;
    this.desc = desc;
    inheritInfo = desc.getInheritInfo();
    extraWhere = (beanProp == null ? null : beanProp.getExtraWhere());
    
    idBinder = desc.getIdBinder();
    

    readId = ((withId) && (desc.propertiesId().length > 0));
    disableLazyLoad = ((!readId) || (desc.isSqlSelectBased()));
    
    tableJoins = props.getTableJoins();
    
    partialObject = props.isPartialObject();
    partialProps = props.getIncludedProperties();
    partialHash = (partialObject ? partialProps.hashCode() : 0);
    
    readOnlyLeaf = props.isReadOnly();
    
    properties = props.getProps();
    
    if (partialObject)
    {


      includedProps = LoadedPropertiesCache.get(partialHash, partialProps, desc);
    } else {
      includedProps = null;
    }
    
    if (myChildren == null) {
      children = NO_CHILDREN;
    } else {
      children = ((SqlTreeNode[])myChildren.toArray(new SqlTreeNode[myChildren.size()]));
    }
    
    pathMap = createPathMap(prefix, desc);
  }
  
  private Map<String, String> createPathMap(String prefix, BeanDescriptor<?> desc)
  {
    BeanPropertyAssocMany<?>[] manys = desc.propertiesMany();
    
    HashMap<String, String> m = new HashMap();
    for (int i = 0; i < manys.length; i++) {
      String name = manys[i].getName();
      m.put(name, getPath(prefix, name));
    }
    
    return m;
  }
  
  private String getPath(String prefix, String propertyName) {
    if (prefix == null) {
      return propertyName;
    }
    return prefix + "." + propertyName;
  }
  

  protected void postLoad(DbReadContext cquery, Object loadedBean, Object id) {}
  
  public void buildSelectExpressionChain(List<String> selectChain)
  {
    if (readId) {
      idBinder.buildSelectExpressionChain(prefix, selectChain);
    }
    int i = 0; for (int x = properties.length; i < x; i++) {
      properties[i].buildSelectExpressionChain(prefix, selectChain);
    }
    
    for (int i = 0; i < children.length; i++)
    {

      children[i].buildSelectExpressionChain(selectChain);
    }
  }
  



  public void load(DbReadContext ctx, Object parentBean, int parentState)
    throws SQLException
  {
    Object contextBean = null;
    
    BeanDescriptor<?> localDesc;
    Class<?> localType;
    BeanDescriptor<?> localDesc;
    Object localBean;
    IdBinder localIdBinder;
    if (inheritInfo != null) {
      InheritInfo localInfo = inheritInfo.readType(ctx);
      BeanDescriptor<?> localDesc; if (localInfo == null)
      {
        IdBinder localIdBinder = idBinder;
        Object localBean = null;
        Class<?> localType = null;
        localDesc = desc;
      } else {
        Object localBean = localInfo.createBean(ctx.isVanillaMode());
        Class<?> localType = localInfo.getType();
        IdBinder localIdBinder = localInfo.getIdBinder();
        localDesc = localInfo.getBeanDescriptor();
      }
    }
    else {
      localType = null;
      localDesc = desc;
      localBean = desc.createBean(ctx.isVanillaMode());
      localIdBinder = idBinder;
    }
    
    SpiQuery.Mode queryMode = ctx.getQueryMode();
    
    PersistenceContext persistenceContext = ctx.getPersistenceContext();
    
    Object id = null;
    if (readId)
    {


      id = localIdBinder.readSet(ctx, localBean);
      if (id == null)
      {
        localBean = null;
      }
      else {
        contextBean = persistenceContext.putIfAbsent(id, localBean);
        if (contextBean == null)
        {
          contextBean = localBean;

        }
        else if (queryMode.isLoadContextBean())
        {
          localBean = contextBean;
          if ((localBean instanceof EntityBean))
          {
            ((EntityBean)localBean)._ebean_getIntercept().setIntercepting(false);
          }
        }
        else {
          localBean = null;
        }
      }
    }
    

    ctx.setCurrentPrefix(prefix, pathMap);
    
    ctx.propagateState(localBean);
    
    SqlBeanLoad sqlBeanLoad = new SqlBeanLoad(ctx, localType, localBean, queryMode);
    
    if (inheritInfo == null)
    {
      int i = 0; for (int x = properties.length; i < x; i++) {
        properties[i].load(sqlBeanLoad);
      }
      
    }
    else
    {
      int i = 0; for (int x = properties.length; i < x; i++)
      {
        BeanProperty p = localDesc.getBeanProperty(properties[i].getName());
        if (p != null) {
          p.load(sqlBeanLoad);
        } else {
          properties[i].loadIgnore(ctx);
        }
      }
    }
    
    int i = 0; for (int x = tableJoins.length; i < x; i++) {
      tableJoins[i].load(sqlBeanLoad);
    }
    
    boolean lazyLoadMany = false;
    if ((localBean == null) && (queryMode.equals(SpiQuery.Mode.LAZYLOAD_MANY)))
    {
      localBean = contextBean;
      lazyLoadMany = true;
    }
    

    for (int i = 0; i < children.length; i++)
    {

      children[i].load(ctx, localBean, parentState);
    }
    
    if (!lazyLoadMany)
    {

      if (localBean != null)
      {
        ctx.setCurrentPrefix(prefix, pathMap);
        if (!ctx.isVanillaMode())
        {

          createListProxies(localDesc, ctx, localBean);
        }
        
        localDesc.postLoad(localBean, includedProps);
        
        if ((localBean instanceof EntityBean)) {
          EntityBeanIntercept ebi = ((EntityBean)localBean)._ebean_getIntercept();
          ebi.setPersistenceContext(persistenceContext);
          ebi.setLoadedProps(includedProps);
          if (SpiQuery.Mode.LAZYLOAD_BEAN.equals(queryMode))
          {
            ebi.setLoadedLazy();
          }
          else {
            ebi.setLoaded();
          }
          
          if (partialObject) {
            ctx.register(null, ebi);
          }
          
          if (disableLazyLoad)
          {
            ebi.setDisableLazyLoad(true);
          }
          if (ctx.isAutoFetchProfiling())
          {
            ctx.profileBean(ebi, prefix);
          }
        }
      }
    }
    if ((parentBean != null) && (contextBean != null))
    {
      nodeBeanProp.setValue(parentBean, contextBean);
    }
    
    if (!readId)
    {
      postLoad(ctx, localBean, id);

    }
    else
    {

      postLoad(ctx, contextBean, id);
    }
  }
  




  private void createListProxies(BeanDescriptor<?> localDesc, DbReadContext ctx, Object localBean)
  {
    BeanPropertyAssocMany<?> fetchedMany = ctx.getManyProperty();
    

    BeanPropertyAssocMany<?>[] manys = localDesc.propertiesMany();
    for (int i = 0; i < manys.length; i++)
    {
      if ((fetchedMany == null) || (!fetchedMany.equals(manys[i])))
      {



        BeanCollection<?> ref = manys[i].createReferenceIfNull(localBean);
        if (ref != null) {
          ctx.register(manys[i].getName(), ref);
        }
      }
    }
  }
  



  public void appendSelect(DbSqlContext ctx, boolean subQuery)
  {
    ctx.pushJoin(prefix);
    ctx.pushTableAlias(prefix);
    
    if (nodeBeanProp != null) {
      ctx.append('\n').append("        ");
    }
    
    if ((!subQuery) && (inheritInfo != null)) {
      ctx.appendColumn(inheritInfo.getDiscriminatorColumn());
    }
    
    if (readId) {
      appendSelect(ctx, false, idBinder.getProperties());
    }
    appendSelect(ctx, subQuery, properties);
    appendSelectTableJoins(ctx);
    
    for (int i = 0; i < children.length; i++)
    {

      children[i].appendSelect(ctx, subQuery);
    }
    
    ctx.popTableAlias();
    ctx.popJoin();
  }
  
  private void appendSelectTableJoins(DbSqlContext ctx)
  {
    String baseAlias = ctx.getTableAlias(prefix);
    
    for (int i = 0; i < tableJoins.length; i++) {
      TableJoin join = tableJoins[i];
      
      String alias = baseAlias + i;
      
      ctx.pushSecondaryTableAlias(alias);
      join.appendSelect(ctx, false);
      ctx.popTableAlias();
    }
  }
  



  private void appendSelect(DbSqlContext ctx, boolean subQuery, BeanProperty[] props)
  {
    for (int i = 0; i < props.length; i++) {
      props[i].appendSelect(ctx, subQuery);
    }
  }
  

  public void appendWhere(DbSqlContext ctx)
  {
    if ((inheritInfo != null) && 
      (!inheritInfo.isRoot()))
    {





      if (ctx.length() > 0) {
        ctx.append(" and");
      }
      ctx.append(" ").append(ctx.getTableAlias(prefix)).append(".");
      ctx.append(inheritInfo.getWhere()).append(" ");
    }
    
    if (extraWhere != null) {
      if (ctx.length() > 0) {
        ctx.append(" and");
      }
      String ta = ctx.getTableAlias(prefix);
      String ew = StringHelper.replaceString(extraWhere, "${ta}", ta);
      ctx.append(" ").append(ew).append(" ");
    }
    
    for (int i = 0; i < children.length; i++)
    {

      children[i].appendWhere(ctx);
    }
  }
  



  public void appendFrom(DbSqlContext ctx, boolean forceOuterJoin)
  {
    ctx.pushJoin(prefix);
    ctx.pushTableAlias(prefix);
    
    forceOuterJoin = appendFromBaseTable(ctx, forceOuterJoin);
    
    for (int i = 0; i < properties.length; i++)
    {
      properties[i].appendFrom(ctx, forceOuterJoin);
    }
    
    for (int i = 0; i < children.length; i++) {
      children[i].appendFrom(ctx, forceOuterJoin);
    }
    
    ctx.popTableAlias();
    ctx.popJoin();
  }
  




  public boolean appendFromBaseTable(DbSqlContext ctx, boolean forceOuterJoin)
  {
    if ((nodeBeanProp instanceof BeanPropertyAssocMany)) {
      BeanPropertyAssocMany<?> manyProp = (BeanPropertyAssocMany)nodeBeanProp;
      if (manyProp.isManyToMany())
      {
        String alias = ctx.getTableAlias(prefix);
        String[] split = SplitName.split(prefix);
        String parentAlias = ctx.getTableAlias(split[0]);
        String alias2 = alias + "z_";
        
        TableJoin manyToManyJoin = manyProp.getIntersectionTableJoin();
        manyToManyJoin.addJoin(forceOuterJoin, parentAlias, alias2, ctx);
        
        return nodeBeanProp.addJoin(forceOuterJoin, alias2, alias, ctx);
      }
    }
    

    return nodeBeanProp.addJoin(forceOuterJoin, prefix, ctx);
  }
  



  public String toString()
  {
    return "SqlTreeNodeBean: " + desc;
  }
}
