package com.avaje.ebeaninternal.server.querydefn;

import com.avaje.ebean.EbeanServer;
import com.avaje.ebean.Expression;
import com.avaje.ebean.ExpressionFactory;
import com.avaje.ebean.ExpressionList;
import com.avaje.ebean.FetchConfig;
import com.avaje.ebean.FutureIds;
import com.avaje.ebean.FutureList;
import com.avaje.ebean.FutureRowCount;
import com.avaje.ebean.JoinConfig;
import com.avaje.ebean.OrderBy;
import com.avaje.ebean.OrderBy.Property;
import com.avaje.ebean.PagingList;
import com.avaje.ebean.Query;
import com.avaje.ebean.Query.Type;
import com.avaje.ebean.Query.UseIndex;
import com.avaje.ebean.QueryIterator;
import com.avaje.ebean.QueryListener;
import com.avaje.ebean.QueryResultVisitor;
import com.avaje.ebean.RawSql;
import com.avaje.ebean.bean.BeanCollectionTouched;
import com.avaje.ebean.bean.CallStack;
import com.avaje.ebean.bean.EntityBean;
import com.avaje.ebean.bean.ObjectGraphNode;
import com.avaje.ebean.bean.ObjectGraphOrigin;
import com.avaje.ebean.bean.PersistenceContext;
import com.avaje.ebean.event.BeanQueryRequest;
import com.avaje.ebean.meta.MetaAutoFetchStatistic;
import com.avaje.ebeaninternal.api.BindParams;
import com.avaje.ebeaninternal.api.ManyWhereJoins;
import com.avaje.ebeaninternal.api.SpiExpressionList;
import com.avaje.ebeaninternal.api.SpiQuery;
import com.avaje.ebeaninternal.api.SpiQuery.Mode;
import com.avaje.ebeaninternal.server.autofetch.AutoFetchManager;
import com.avaje.ebeaninternal.server.deploy.BeanDescriptor;
import com.avaje.ebeaninternal.server.deploy.DRawSqlSelect;
import com.avaje.ebeaninternal.server.deploy.DeployNamedQuery;
import com.avaje.ebeaninternal.server.deploy.TableJoin;
import com.avaje.ebeaninternal.server.deploy.id.IdBinder;
import com.avaje.ebeaninternal.server.query.CancelableQuery;
import com.avaje.ebeaninternal.util.DefaultExpressionList;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import javax.persistence.PersistenceException;



























public class DefaultOrmQuery<T>
  implements SpiQuery<T>
{
  private static final long serialVersionUID = 6838006264714672460L;
  private final Class<T> beanType;
  private final transient EbeanServer server;
  private transient BeanCollectionTouched beanCollectionTouched;
  private final transient ExpressionFactory expressionFactory;
  private transient ArrayList<EntityBean> contextAdditions;
  private transient QueryListener<T> queryListener;
  private transient TableJoin includeTableJoin;
  private transient AutoFetchManager autoFetchManager;
  private transient BeanDescriptor<?> beanDescriptor;
  private boolean cancelled;
  private transient CancelableQuery cancelableQuery;
  private String name;
  private Query.UseIndex useIndex;
  private Query.Type type;
  private SpiQuery.Mode mode = SpiQuery.Mode.NORMAL;
  


  private OrmQueryDetail detail;
  


  private int maxRows;
  


  private int firstRow;
  

  private String rawWhereClause;
  

  private OrderBy<T> orderBy;
  

  private String loadMode;
  

  private String loadDescription;
  

  private String generatedSql;
  

  private String query;
  

  private String additionalWhere;
  

  private String additionalHaving;
  

  private String lazyLoadProperty;
  

  private String lazyLoadManyPath;
  

  private Boolean vanillaMode;
  

  private boolean distinct;
  

  private boolean futureFetch;
  

  private boolean sharedInstance;
  

  private List<Object> partialIds;
  

  private int backgroundFetchAfter;
  

  private int timeout = -1;
  


  private String mapKey;
  


  private Object id;
  

  private BindParams bindParams;
  

  private DefaultExpressionList<T> whereExpressions;
  

  private DefaultExpressionList<T> havingExpressions;
  

  private int bufferFetchSizeHint;
  

  private boolean usageProfiling = true;
  

  private boolean loadBeanCache;
  

  private Boolean useBeanCache;
  

  private Boolean useQueryCache;
  

  private Boolean readOnly;
  

  private boolean sqlSelect;
  

  private Boolean autoFetch;
  

  private boolean autoFetchTuned;
  

  private ObjectGraphNode parentNode;
  

  private int queryPlanHash;
  

  private transient PersistenceContext persistenceContext;
  

  private ManyWhereJoins manyWhereJoins;
  
  private RawSql rawSql;
  

  public DefaultOrmQuery(Class<T> beanType, EbeanServer server, ExpressionFactory expressionFactory, String query)
  {
    this.beanType = beanType;
    this.server = server;
    this.expressionFactory = expressionFactory;
    detail = new OrmQueryDetail();
    name = "";
    if (query != null) {
      setQuery(query);
    }
  }
  



  public DefaultOrmQuery(Class<T> beanType, EbeanServer server, ExpressionFactory expressionFactory, DeployNamedQuery namedQuery)
    throws PersistenceException
  {
    this.beanType = beanType;
    this.server = server;
    this.expressionFactory = expressionFactory;
    detail = new OrmQueryDetail();
    if (namedQuery == null) {
      name = "";
    } else {
      name = namedQuery.getName();
      this.sqlSelect = namedQuery.isSqlSelect();
      if (this.sqlSelect)
      {
        DRawSqlSelect sqlSelect = namedQuery.getSqlSelect();
        additionalWhere = sqlSelect.getWhereClause();
        additionalHaving = sqlSelect.getHavingClause();
      } else if (namedQuery.isRawSql()) {
        rawSql = namedQuery.getRawSql();
      }
      else
      {
        setQuery(namedQuery.getQuery());
      }
    }
  }
  


  public void setBeanDescriptor(BeanDescriptor<?> beanDescriptor)
  {
    this.beanDescriptor = beanDescriptor;
  }
  



  public boolean selectAllForLazyLoadProperty()
  {
    if ((lazyLoadProperty != null) && 
      (!detail.containsProperty(lazyLoadProperty))) {
      detail.select("*");
      return true;
    }
    
    return false;
  }
  
  public RawSql getRawSql() {
    return rawSql;
  }
  
  public DefaultOrmQuery<T> setRawSql(RawSql rawSql) {
    this.rawSql = rawSql;
    return this;
  }
  
  public String getLazyLoadProperty() {
    return lazyLoadProperty;
  }
  
  public void setLazyLoadProperty(String lazyLoadProperty) {
    this.lazyLoadProperty = lazyLoadProperty;
  }
  
  public String getLazyLoadManyPath() {
    return lazyLoadManyPath;
  }
  
  public ExpressionFactory getExpressionFactory() {
    return expressionFactory;
  }
  
  public void setParentState(int parentState) {
    if (parentState == 3) {
      setSharedInstance();
    } else if (parentState == 2) {
      setReadOnly(true);
    }
  }
  
  public MetaAutoFetchStatistic getMetaAutoFetchStatistic() {
    if ((parentNode != null) && (server != null)) {
      ObjectGraphOrigin origin = parentNode.getOriginQueryPoint();
      return (MetaAutoFetchStatistic)server.find(MetaAutoFetchStatistic.class, origin.getKey());
    }
    return null;
  }
  


  public boolean initManyWhereJoins()
  {
    manyWhereJoins = new ManyWhereJoins();
    if (whereExpressions != null) {
      whereExpressions.containsMany(beanDescriptor, manyWhereJoins);
    }
    return !manyWhereJoins.isEmpty();
  }
  
  public ManyWhereJoins getManyWhereJoins() {
    return manyWhereJoins;
  }
  
  public List<OrmQueryProperties> removeQueryJoins() {
    List<OrmQueryProperties> queryJoins = detail.removeSecondaryQueries();
    if ((queryJoins != null) && 
      (orderBy != null))
    {

      for (int i = 0; i < queryJoins.size(); i++) {
        OrmQueryProperties joinPath = (OrmQueryProperties)queryJoins.get(i);
        


        List<OrderBy.Property> properties = orderBy.getProperties();
        Iterator<OrderBy.Property> it = properties.iterator();
        while (it.hasNext()) {
          OrderBy.Property property = (OrderBy.Property)it.next();
          if (property.getProperty().startsWith(joinPath.getPath()))
          {

            it.remove();
            joinPath.addSecJoinOrderProperty(property);
          }
        }
      }
    }
    
    return queryJoins;
  }
  
  public List<OrmQueryProperties> removeLazyJoins() {
    return detail.removeSecondaryLazyQueries();
  }
  
  public void setLazyLoadManyPath(String lazyLoadManyPath) {
    this.lazyLoadManyPath = lazyLoadManyPath;
  }
  


  public void convertManyFetchJoinsToQueryJoins(boolean allowOne, int queryBatch)
  {
    detail.convertManyFetchJoinsToQueryJoins(beanDescriptor, lazyLoadManyPath, allowOne, queryBatch);
  }
  



  public void setSelectId()
  {
    detail.clear();
    
    select(beanDescriptor.getIdBinder().getIdProperty());
  }
  







  public DefaultOrmQuery<T> copy()
  {
    DefaultOrmQuery<T> copy = new DefaultOrmQuery(beanType, server, expressionFactory, (String)null);
    name = name;
    includeTableJoin = includeTableJoin;
    autoFetchManager = autoFetchManager;
    
    query = query;
    additionalWhere = additionalWhere;
    additionalHaving = additionalHaving;
    distinct = distinct;
    backgroundFetchAfter = backgroundFetchAfter;
    timeout = timeout;
    mapKey = mapKey;
    id = id;
    vanillaMode = vanillaMode;
    loadBeanCache = loadBeanCache;
    useBeanCache = useBeanCache;
    useQueryCache = useQueryCache;
    readOnly = readOnly;
    sqlSelect = sqlSelect;
    if (detail != null) {
      detail = detail.copy();
    }
    
    firstRow = firstRow;
    maxRows = maxRows;
    rawWhereClause = rawWhereClause;
    if (orderBy != null) {
      orderBy = orderBy.copy();
    }
    if (bindParams != null) {
      bindParams = bindParams.copy();
    }
    if (whereExpressions != null) {
      whereExpressions = whereExpressions.copy(copy);
    }
    if (havingExpressions != null) {
      havingExpressions = havingExpressions.copy(copy);
    }
    usageProfiling = usageProfiling;
    autoFetch = autoFetch;
    parentNode = parentNode;
    
    return copy;
  }
  
  public Query.Type getType() {
    return type;
  }
  
  public void setType(Query.Type type) {
    this.type = type;
  }
  
  public Query.UseIndex getUseIndex() {
    return useIndex;
  }
  
  public DefaultOrmQuery<T> setUseIndex(Query.UseIndex useIndex) {
    this.useIndex = useIndex;
    return this;
  }
  
  public String getLoadDescription() {
    return loadDescription;
  }
  
  public String getLoadMode() {
    return loadMode;
  }
  
  public void setLoadDescription(String loadMode, String loadDescription) {
    this.loadMode = loadMode;
    this.loadDescription = loadDescription;
  }
  






  public PersistenceContext getPersistenceContext()
  {
    return persistenceContext;
  }
  






  public void setPersistenceContext(PersistenceContext persistenceContext)
  {
    this.persistenceContext = persistenceContext;
  }
  


  public boolean isDetailEmpty()
  {
    return detail.isEmpty();
  }
  
  public boolean isAutofetchTuned() {
    return autoFetchTuned;
  }
  
  public void setAutoFetchTuned(boolean autoFetchTuned) {
    this.autoFetchTuned = autoFetchTuned;
  }
  
  public Boolean isAutofetch() {
    return sqlSelect ? Boolean.FALSE : autoFetch;
  }
  
  public DefaultOrmQuery<T> setAutoFetch(boolean autoFetch) {
    return setAutofetch(autoFetch);
  }
  
  public DefaultOrmQuery<T> setAutofetch(boolean autoFetch) {
    this.autoFetch = Boolean.valueOf(autoFetch);
    return this;
  }
  
  public AutoFetchManager getAutoFetchManager() {
    return autoFetchManager;
  }
  
  public void setAutoFetchManager(AutoFetchManager autoFetchManager) {
    this.autoFetchManager = autoFetchManager;
  }
  


  public void deriveSharedInstance()
  {
    if ((!sharedInstance) && (
      (Boolean.TRUE.equals(useQueryCache)) || ((Boolean.TRUE.equals(readOnly)) && ((Boolean.TRUE.equals(useBeanCache)) || (Boolean.TRUE.equals(Boolean.valueOf(loadBeanCache)))))))
    {


      sharedInstance = true;
    }
  }
  
  public boolean isSharedInstance()
  {
    return sharedInstance;
  }
  
  public void setSharedInstance() {
    sharedInstance = true;
  }
  
  public SpiQuery.Mode getMode() {
    return mode;
  }
  
  public void setMode(SpiQuery.Mode mode) {
    this.mode = mode;
  }
  
  public boolean isUsageProfiling() {
    return usageProfiling;
  }
  
  public void setUsageProfiling(boolean usageProfiling) {
    this.usageProfiling = usageProfiling;
  }
  
  public void setParentNode(ObjectGraphNode parentNode) {
    this.parentNode = parentNode;
  }
  
  public ObjectGraphNode getParentNode() {
    return parentNode;
  }
  


  public ObjectGraphNode setOrigin(CallStack callStack)
  {
    ObjectGraphOrigin o = new ObjectGraphOrigin(calculateOriginQueryHash(), callStack, beanType.getName());
    parentNode = new ObjectGraphNode(o, null);
    return parentNode;
  }
  










  private int calculateOriginQueryHash()
  {
    int hc = beanType.getName().hashCode();
    hc = hc * 31 + (type == null ? 0 : type.ordinal());
    return hc;
  }
  








  private int calculateHash(BeanQueryRequest<?> request)
  {
    int hc = beanType.getName().hashCode();
    
    hc = hc * 31 + (type == null ? 0 : type.ordinal());
    hc = hc * 31 + (useIndex == null ? 0 : useIndex.hashCode());
    
    hc = hc * 31 + (rawSql == null ? 0 : rawSql.queryHash());
    
    hc = hc * 31 + (autoFetchTuned ? 31 : 0);
    hc = hc * 31 + (distinct ? 31 : 0);
    hc = hc * 31 + (query == null ? 0 : query.hashCode());
    hc = hc * 31 + detail.queryPlanHash(request);
    
    hc = hc * 31 + (firstRow == 0 ? 0 : firstRow);
    hc = hc * 31 + (maxRows == 0 ? 0 : maxRows);
    hc = hc * 31 + (orderBy == null ? 0 : orderBy.hash());
    hc = hc * 31 + (rawWhereClause == null ? 0 : rawWhereClause.hashCode());
    
    hc = hc * 31 + (additionalWhere == null ? 0 : additionalWhere.hashCode());
    hc = hc * 31 + (additionalHaving == null ? 0 : additionalHaving.hashCode());
    hc = hc * 31 + (mapKey == null ? 0 : mapKey.hashCode());
    hc = hc * 31 + (id == null ? 0 : 1);
    
    if (bindParams != null) {
      hc = hc * 31 + bindParams.getQueryPlanHash();
    }
    
    if (request == null)
    {
      hc = hc * 31 + (whereExpressions == null ? 0 : whereExpressions.queryAutoFetchHash());
      hc = hc * 31 + (havingExpressions == null ? 0 : havingExpressions.queryAutoFetchHash());
    }
    else
    {
      hc = hc * 31 + (whereExpressions == null ? 0 : whereExpressions.queryPlanHash(request));
      hc = hc * 31 + (havingExpressions == null ? 0 : havingExpressions.queryPlanHash(request));
    }
    
    return hc;
  }
  




  public int queryAutofetchHash()
  {
    return calculateHash(null);
  }
  










  public int queryPlanHash(BeanQueryRequest<?> request)
  {
    queryPlanHash = calculateHash(request);
    return queryPlanHash;
  }
  





  public int queryBindHash()
  {
    int hc = id == null ? 0 : id.hashCode();
    hc = hc * 31 + (whereExpressions == null ? 0 : whereExpressions.queryBindHash());
    hc = hc * 31 + (havingExpressions == null ? 0 : havingExpressions.queryBindHash());
    hc = hc * 31 + (bindParams == null ? 0 : bindParams.queryBindHash());
    hc = hc * 31 + (contextAdditions == null ? 0 : contextAdditions.hashCode());
    
    return hc;
  }
  








  public int queryHash()
  {
    int hc = queryPlanHash;
    hc = hc * 31 + queryBindHash();
    return hc;
  }
  


  public String getName()
  {
    return name;
  }
  
  public boolean isSqlSelect() {
    return sqlSelect;
  }
  
  public boolean isRawSql() {
    return rawSql != null;
  }
  


  public String getAdditionalWhere()
  {
    return additionalWhere;
  }
  


  public int getTimeout()
  {
    return timeout;
  }
  


  public String getAdditionalHaving()
  {
    return additionalHaving;
  }
  
  public boolean hasMaxRowsOrFirstRow() {
    return (maxRows > 0) || (firstRow > 0);
  }
  
  public boolean isVanillaMode(boolean defaultVanillaMode)
  {
    if (vanillaMode != null) {
      return vanillaMode.booleanValue();
    }
    return defaultVanillaMode;
  }
  
  public DefaultOrmQuery<T> setVanillaMode(boolean vanillaMode) {
    this.vanillaMode = Boolean.valueOf(vanillaMode);
    return this;
  }
  
  public Boolean isReadOnly() {
    return readOnly;
  }
  
  public DefaultOrmQuery<T> setReadOnly(boolean readOnly) {
    this.readOnly = Boolean.valueOf(readOnly);
    return this;
  }
  
  public Boolean isUseBeanCache() {
    return useBeanCache;
  }
  
  public boolean isUseQueryCache() {
    return Boolean.TRUE.equals(useQueryCache);
  }
  
  public DefaultOrmQuery<T> setUseCache(boolean useBeanCache) {
    this.useBeanCache = Boolean.valueOf(useBeanCache);
    return this;
  }
  
  public DefaultOrmQuery<T> setUseQueryCache(boolean useQueryCache) {
    this.useQueryCache = Boolean.valueOf(useQueryCache);
    return this;
  }
  
  public boolean isLoadBeanCache() {
    return loadBeanCache;
  }
  
  public DefaultOrmQuery<T> setLoadBeanCache(boolean loadBeanCache) {
    this.loadBeanCache = loadBeanCache;
    return this;
  }
  
  public DefaultOrmQuery<T> setTimeout(int secs) {
    timeout = secs;
    return this;
  }
  
  public DefaultOrmQuery<T> setQuery(String queryString) throws PersistenceException
  {
    query = queryString;
    
    OrmQueryDetailParser parser = new OrmQueryDetailParser(queryString);
    parser.parse();
    parser.assign(this);
    
    return this;
  }
  
  protected void setOrmQueryDetail(OrmQueryDetail detail) {
    this.detail = detail;
  }
  
  protected void setRawWhereClause(String rawWhereClause) { this.rawWhereClause = rawWhereClause; }
  
  public DefaultOrmQuery<T> setProperties(String columns)
  {
    return select(columns);
  }
  
  public void setDefaultSelectClause() {
    detail.setDefaultSelectClause(beanDescriptor);
  }
  
  public DefaultOrmQuery<T> select(String columns) {
    detail.select(columns);
    return this;
  }
  
  public DefaultOrmQuery<T> join(String property) {
    return join(property, null, null);
  }
  
  public DefaultOrmQuery<T> join(String property, JoinConfig joinConfig) {
    return join(property, null, joinConfig);
  }
  
  public DefaultOrmQuery<T> join(String property, String columns) {
    return join(property, columns, null);
  }
  
  public DefaultOrmQuery<T> join(String property, String columns, JoinConfig joinConfig) {
    FetchConfig c;
    FetchConfig c;
    if (joinConfig == null) {
      c = null;
    } else {
      c = new FetchConfig();
      c.lazy(joinConfig.getLazyBatchSize());
      if (joinConfig.isQueryAll()) {
        c.query(joinConfig.getQueryBatchSize());
      } else {
        c.queryFirst(joinConfig.getQueryBatchSize());
      }
    }
    
    detail.addFetch(property, columns, c);
    return this;
  }
  
  public DefaultOrmQuery<T> fetch(String property) {
    return fetch(property, null, null);
  }
  
  public DefaultOrmQuery<T> fetch(String property, FetchConfig joinConfig) {
    return fetch(property, null, joinConfig);
  }
  
  public DefaultOrmQuery<T> fetch(String property, String columns) {
    return fetch(property, columns, null);
  }
  
  public DefaultOrmQuery<T> fetch(String property, String columns, FetchConfig config) {
    detail.addFetch(property, columns, config);
    return this;
  }
  


  public List<Object> findIds()
  {
    return server.findIds(this, null);
  }
  


  public int findRowCount()
  {
    return server.findRowCount(this, null);
  }
  
  public void findVisit(QueryResultVisitor<T> visitor) {
    server.findVisit(this, visitor, null);
  }
  
  public QueryIterator<T> findIterate() {
    return server.findIterate(this, null);
  }
  
  public List<T> findList() {
    return server.findList(this, null);
  }
  
  public Set<T> findSet() {
    return server.findSet(this, null);
  }
  
  public Map<?, T> findMap() {
    return server.findMap(this, null);
  }
  
  public <K> Map<K, T> findMap(String keyProperty, Class<K> keyType)
  {
    setMapKey(keyProperty);
    return findMap();
  }
  
  public T findUnique() {
    return server.findUnique(this, null);
  }
  
  public FutureIds<T> findFutureIds() {
    return server.findFutureIds(this, null);
  }
  
  public FutureList<T> findFutureList() {
    return server.findFutureList(this, null);
  }
  
  public FutureRowCount<T> findFutureRowCount() {
    return server.findFutureRowCount(this, null);
  }
  
  public PagingList<T> findPagingList(int pageSize) {
    return server.findPagingList(this, null, pageSize);
  }
  




  public DefaultOrmQuery<T> setParameter(int position, Object value)
  {
    if (bindParams == null) {
      bindParams = new BindParams();
    }
    bindParams.setParameter(position, value);
    return this;
  }
  



  public DefaultOrmQuery<T> setParameter(String name, Object value)
  {
    if (bindParams == null) {
      bindParams = new BindParams();
    }
    bindParams.setParameter(name, value);
    return this;
  }
  
  public OrderBy<T> getOrderBy() {
    return orderBy;
  }
  


  public String getRawWhereClause()
  {
    return rawWhereClause;
  }
  
  public OrderBy<T> orderBy() {
    return order();
  }
  
  public OrderBy<T> order() {
    if (orderBy == null) {
      orderBy = new OrderBy(this, null);
    }
    return orderBy;
  }
  
  public DefaultOrmQuery<T> setOrderBy(String orderByClause) {
    return order(orderByClause);
  }
  
  public DefaultOrmQuery<T> orderBy(String orderByClause) {
    return order(orderByClause);
  }
  
  public DefaultOrmQuery<T> order(String orderByClause) {
    if ((orderByClause == null) || (orderByClause.trim().length() == 0)) {
      orderBy = null;
    } else {
      orderBy = new OrderBy(this, orderByClause);
    }
    return this;
  }
  
  public DefaultOrmQuery<T> setOrderBy(OrderBy<T> orderBy) {
    return setOrder(orderBy);
  }
  
  public DefaultOrmQuery<T> setOrder(OrderBy<T> orderBy) {
    this.orderBy = orderBy;
    if (orderBy != null) {
      orderBy.setQuery(this);
    }
    return this;
  }
  



  public boolean isDistinct()
  {
    return distinct;
  }
  


  public DefaultOrmQuery<T> setDistinct(boolean isDistinct)
  {
    distinct = isDistinct;
    return this;
  }
  


  public QueryListener<T> getListener()
  {
    return queryListener;
  }
  







  public DefaultOrmQuery<T> setListener(QueryListener<T> queryListener)
  {
    this.queryListener = queryListener;
    return this;
  }
  
  public Class<T> getBeanType() {
    return beanType;
  }
  
  public void setDetail(OrmQueryDetail detail) {
    this.detail = detail;
  }
  
  public boolean tuneFetchProperties(OrmQueryDetail tunedDetail) {
    return detail.tuneFetchProperties(tunedDetail);
  }
  
  public OrmQueryDetail getDetail() {
    return detail;
  }
  



  public final ArrayList<EntityBean> getContextAdditions()
  {
    return contextAdditions;
  }
  





  public void contextAdd(EntityBean bean)
  {
    if (contextAdditions == null) {
      contextAdditions = new ArrayList();
    }
    contextAdditions.add(bean);
  }
  
  public String toString() {
    return "Query [" + whereExpressions + "]";
  }
  
  public TableJoin getIncludeTableJoin() {
    return includeTableJoin;
  }
  
  public void setIncludeTableJoin(TableJoin includeTableJoin) {
    this.includeTableJoin = includeTableJoin;
  }
  
  public int getFirstRow() {
    return firstRow;
  }
  
  public DefaultOrmQuery<T> setFirstRow(int firstRow) {
    this.firstRow = firstRow;
    return this;
  }
  
  public int getMaxRows() {
    return maxRows;
  }
  
  public DefaultOrmQuery<T> setMaxRows(int maxRows) {
    this.maxRows = maxRows;
    return this;
  }
  
  public String getMapKey() {
    return mapKey;
  }
  
  public DefaultOrmQuery<T> setMapKey(String mapKey) {
    this.mapKey = mapKey;
    return this;
  }
  
  public int getBackgroundFetchAfter() {
    return backgroundFetchAfter;
  }
  
  public DefaultOrmQuery<T> setBackgroundFetchAfter(int backgroundFetchAfter) {
    this.backgroundFetchAfter = backgroundFetchAfter;
    return this;
  }
  
  public Object getId() {
    return id;
  }
  
  public DefaultOrmQuery<T> setId(Object id) {
    this.id = id;
    return this;
  }
  
  public BindParams getBindParams() {
    return bindParams;
  }
  
  public String getQuery() {
    return query;
  }
  
  public DefaultOrmQuery<T> addWhere(String addToWhereClause) {
    return where(addToWhereClause);
  }
  
  public DefaultOrmQuery<T> addWhere(Expression expression) {
    return where(expression);
  }
  
  public ExpressionList<T> addWhere() {
    return where();
  }
  
  public DefaultOrmQuery<T> where(String addToWhereClause) {
    if (additionalWhere == null) {
      additionalWhere = addToWhereClause;
    } else {
      additionalWhere = (additionalWhere + " " + addToWhereClause);
    }
    return this;
  }
  
  public DefaultOrmQuery<T> where(Expression expression) {
    if (whereExpressions == null) {
      whereExpressions = new DefaultExpressionList(this, null);
    }
    whereExpressions.add(expression);
    return this;
  }
  
  public ExpressionList<T> where() {
    if (whereExpressions == null) {
      whereExpressions = new DefaultExpressionList(this, null);
    }
    return whereExpressions;
  }
  
  public ExpressionList<T> filterMany(String prop)
  {
    OrmQueryProperties chunk = detail.getChunk(prop, true);
    return chunk.filterMany(this);
  }
  
  public void setFilterMany(String prop, ExpressionList<?> filterMany)
  {
    if (filterMany != null) {
      OrmQueryProperties chunk = detail.getChunk(prop, true);
      chunk.setFilterMany((SpiExpressionList)filterMany);
    }
  }
  
  public DefaultOrmQuery<T> addHaving(String addToHavingClause) {
    return having(addToHavingClause);
  }
  
  public DefaultOrmQuery<T> addHaving(Expression expression) {
    return having(expression);
  }
  
  public ExpressionList<T> addHaving() {
    return having();
  }
  
  public DefaultOrmQuery<T> having(String addToHavingClause) {
    if (additionalHaving == null) {
      additionalHaving = addToHavingClause;
    } else {
      additionalHaving = (additionalHaving + " " + addToHavingClause);
    }
    return this;
  }
  
  public DefaultOrmQuery<T> having(Expression expression) {
    if (havingExpressions == null) {
      havingExpressions = new DefaultExpressionList(this, null);
    }
    havingExpressions.add(expression);
    return this;
  }
  
  public ExpressionList<T> having() {
    if (havingExpressions == null) {
      havingExpressions = new DefaultExpressionList(this, null);
    }
    return havingExpressions;
  }
  
  public SpiExpressionList<T> getHavingExpressions() {
    return havingExpressions;
  }
  
  public SpiExpressionList<T> getWhereExpressions() {
    return whereExpressions;
  }
  


  public boolean createOwnTransaction()
  {
    if (futureFetch)
    {

      return false;
    }
    if ((backgroundFetchAfter > 0) || (queryListener != null))
    {

      return true;
    }
    return false;
  }
  
  public String getGeneratedSql() {
    return generatedSql;
  }
  
  public void setGeneratedSql(String generatedSql) {
    this.generatedSql = generatedSql;
  }
  
  public Query<T> setBufferFetchSizeHint(int bufferFetchSizeHint) {
    this.bufferFetchSizeHint = bufferFetchSizeHint;
    return this;
  }
  
  public int getBufferFetchSizeHint() {
    return bufferFetchSizeHint;
  }
  
  public void setBeanCollectionTouched(BeanCollectionTouched notify) {
    beanCollectionTouched = notify;
  }
  
  public BeanCollectionTouched getBeanCollectionTouched() {
    return beanCollectionTouched;
  }
  
  public List<Object> getIdList() {
    return partialIds;
  }
  
  public void setIdList(List<Object> partialIds) {
    this.partialIds = partialIds;
  }
  
  public boolean isFutureFetch() {
    return futureFetch;
  }
  
  public void setFutureFetch(boolean backgroundFetch) {
    futureFetch = backgroundFetch;
  }
  
  public void setCancelableQuery(CancelableQuery cancelableQuery) {
    synchronized (this) {
      this.cancelableQuery = cancelableQuery;
    }
  }
  
  public void cancel() {
    synchronized (this) {
      cancelled = true;
      if (cancelableQuery != null) {
        cancelableQuery.cancel();
      }
    }
  }
  
  public boolean isCancelled() {
    synchronized (this) {
      return cancelled;
    }
  }
}
