package com.avaje.ebeaninternal.server.querydefn;

import com.avaje.ebean.ExpressionFactory;
import com.avaje.ebean.FetchConfig;
import com.avaje.ebean.OrderBy;
import com.avaje.ebean.OrderBy.Property;
import com.avaje.ebean.Query;
import com.avaje.ebean.event.BeanQueryRequest;
import com.avaje.ebeaninternal.api.SpiExpressionFactory;
import com.avaje.ebeaninternal.api.SpiExpressionList;
import com.avaje.ebeaninternal.api.SpiQuery;
import com.avaje.ebeaninternal.server.core.ReferenceOptions;
import com.avaje.ebeaninternal.server.expression.FilterExprPath;
import com.avaje.ebeaninternal.server.lib.util.StringHelper;
import com.avaje.ebeaninternal.server.query.SplitName;
import com.avaje.ebeaninternal.util.FilterExpressionList;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;











public class OrmQueryProperties
  implements Serializable
{
  private static final long serialVersionUID = -8785582703966455658L;
  private String parentPath;
  private String path;
  private String properties;
  private String trimmedProperties;
  private int queryFetchBatch = -1;
  

  private boolean queryFetchAll;
  

  private int lazyFetchBatch = -1;
  



  private FetchConfig fetchConfig;
  



  private boolean cache;
  


  private boolean readOnly;
  


  private Set<String> included;
  


  private Set<String> includedBeanJoin;
  


  private Set<String> secondaryQueryJoins;
  


  private List<OrmQueryProperties> secondaryChildren;
  


  private OrderBy orderBy;
  


  private SpiExpressionList filterMany;
  



  public OrmQueryProperties(String path)
  {
    this.path = path;
    parentPath = SplitName.parent(path);
  }
  
  public OrmQueryProperties() {
    this(null);
  }
  


  public OrmQueryProperties(String path, String properties)
  {
    this(path);
    setProperties(properties);
  }
  



  public void addSecJoinOrderProperty(OrderBy.Property orderProp)
  {
    if (orderBy == null) {
      orderBy = new OrderBy();
    }
    orderBy.add(orderProp);
  }
  


  public void setFetchConfig(FetchConfig fetchConfig)
  {
    if (fetchConfig != null) {
      this.fetchConfig = fetchConfig;
      lazyFetchBatch = fetchConfig.getLazyBatchSize();
      queryFetchBatch = fetchConfig.getQueryBatchSize();
      queryFetchAll = fetchConfig.isQueryAll();
    }
  }
  
  public FetchConfig getFetchConfig() {
    return fetchConfig;
  }
  





  public void setProperties(String properties)
  {
    this.properties = properties;
    trimmedProperties = properties;
    parseProperties();
    
    if (!isAllProperties()) {
      Set<String> parsed = parseIncluded(trimmedProperties);
      if (parsed.contains("*")) {
        included = null;
      } else {
        included = parsed;
      }
    } else {
      included = null;
    }
  }
  
  private boolean isAllProperties() {
    return (trimmedProperties == null) || (trimmedProperties.length() == 0) || ("*".equals(trimmedProperties));
  }
  




  public <T> SpiExpressionList<T> filterMany(Query<T> rootQuery)
  {
    if (filterMany == null) {
      FilterExprPath exprPath = new FilterExprPath(path);
      SpiExpressionFactory queryEf = (SpiExpressionFactory)rootQuery.getExpressionFactory();
      ExpressionFactory filterEf = queryEf.createExpressionFactory(exprPath);
      filterMany = new FilterExpressionList(exprPath, filterEf, rootQuery);
      
      queryFetchAll = true;
      queryFetchBatch = 100;
      lazyFetchBatch = 100;
    }
    return filterMany;
  }
  


  public SpiExpressionList<?> getFilterManyTrimPath(int trimPath)
  {
    if (filterMany == null) {
      return null;
    }
    filterMany.trimPath(trimPath);
    return filterMany;
  }
  


  public SpiExpressionList<?> getFilterMany()
  {
    return filterMany;
  }
  


  public void setFilterMany(SpiExpressionList<?> filterMany)
  {
    this.filterMany = filterMany;
  }
  


  public void setDefaultProperties(String properties, Set<String> included)
  {
    this.properties = properties;
    trimmedProperties = properties;
    this.included = included;
  }
  


  public void setTunedProperties(OrmQueryProperties tunedProperties)
  {
    properties = properties;
    trimmedProperties = trimmedProperties;
    included = included;
  }
  




  public void configureBeanQuery(SpiQuery<?> query)
  {
    if ((trimmedProperties != null) && (trimmedProperties.length() > 0)) {
      query.select(trimmedProperties);
      if (filterMany != null) {
        query.setFilterMany(this.path, filterMany);
      }
    }
    
    if (secondaryChildren != null) {
      int trimPath = this.path.length() + 1;
      for (int i = 0; i < secondaryChildren.size(); i++) {
        OrmQueryProperties p = (OrmQueryProperties)secondaryChildren.get(i);
        String path = p.getPath();
        path = path.substring(trimPath);
        query.fetch(path, p.getProperties(), p.getFetchConfig());
        query.setFilterMany(path, p.getFilterManyTrimPath(trimPath));
      }
    }
    
    if (orderBy != null) {
      List<OrderBy.Property> orderByProps = orderBy.getProperties();
      for (int i = 0; i < orderByProps.size(); i++) {
        ((OrderBy.Property)orderByProps.get(i)).trim(this.path);
      }
      query.setOrder(orderBy);
    }
  }
  




  public void configureManyQuery(SpiQuery<?> query)
  {
    if ((trimmedProperties != null) && (trimmedProperties.length() > 0)) {
      query.fetch(query.getLazyLoadManyPath(), trimmedProperties);
    }
    if (filterMany != null) {
      query.setFilterMany(this.path, filterMany);
    }
    if (secondaryChildren != null)
    {
      int trimlen = this.path.length() - query.getLazyLoadManyPath().length();
      
      for (int i = 0; i < secondaryChildren.size(); i++) {
        OrmQueryProperties p = (OrmQueryProperties)secondaryChildren.get(i);
        String path = p.getPath();
        path = path.substring(trimlen);
        query.fetch(path, p.getProperties(), p.getFetchConfig());
        query.setFilterMany(path, p.getFilterManyTrimPath(trimlen));
      }
    }
    
    if (orderBy != null) {
      query.setOrder(orderBy);
    }
  }
  


  public OrmQueryProperties copy()
  {
    OrmQueryProperties copy = new OrmQueryProperties();
    parentPath = parentPath;
    path = path;
    properties = properties;
    cache = cache;
    readOnly = readOnly;
    queryFetchAll = queryFetchAll;
    queryFetchBatch = queryFetchBatch;
    lazyFetchBatch = lazyFetchBatch;
    filterMany = filterMany;
    if (included != null) {
      included = new HashSet(included);
    }
    if (includedBeanJoin != null) {
      includedBeanJoin = new HashSet(includedBeanJoin);
    }
    return copy;
  }
  
  public boolean hasSelectClause() {
    if ("*".equals(trimmedProperties))
    {
      return true;
    }
    
    return included != null;
  }
  
  public String toString() {
    String s = "";
    if (path != null) {
      s = s + path + " ";
    }
    if (properties != null) {
      s = s + "(" + properties + ") ";
    } else if (included != null) {
      s = s + "(" + included + ") ";
    }
    return s;
  }
  
  public boolean isChild(OrmQueryProperties possibleChild) {
    return possibleChild.getPath().startsWith(path + ".");
  }
  


  public void add(OrmQueryProperties child)
  {
    if (secondaryChildren == null) {
      secondaryChildren = new ArrayList();
    }
    secondaryChildren.add(child);
  }
  
  public int autofetchPlanHash()
  {
    int hc = path != null ? path.hashCode() : 1;
    if (properties != null) {
      hc = hc * 31 + properties.hashCode();
    } else {
      hc = hc * 31 + (included != null ? included.hashCode() : 1);
    }
    
    return hc;
  }
  




  public int queryPlanHash(BeanQueryRequest<?> request)
  {
    int hc = path != null ? path.hashCode() : 1;
    if (properties != null) {
      hc = hc * 31 + properties.hashCode();
    } else {
      hc = hc * 31 + (included != null ? included.hashCode() : 1);
    }
    hc = hc * 31 + (filterMany != null ? filterMany.queryPlanHash(request) : 1);
    
    return hc;
  }
  
  public String getProperties() {
    return properties;
  }
  
  public ReferenceOptions getReferenceOptions() {
    if ((cache) || (readOnly)) {
      return new ReferenceOptions(cache, readOnly, null);
    }
    return null;
  }
  



  public boolean hasProperties()
  {
    return (properties != null) || (included != null);
  }
  






  public boolean isIncludedBeanJoin(String propertyName)
  {
    if (includedBeanJoin == null) {
      return false;
    }
    return includedBeanJoin.contains(propertyName);
  }
  



  public void includeBeanJoin(String propertyName)
  {
    if (includedBeanJoin == null) {
      includedBeanJoin = new HashSet();
    }
    includedBeanJoin.add(propertyName);
  }
  
  public boolean allProperties() {
    return included == null;
  }
  






  public Iterator<String> getSelectProperties()
  {
    if (secondaryQueryJoins == null) {
      return included.iterator();
    }
    
    LinkedHashSet<String> temp = new LinkedHashSet(secondaryQueryJoins.size() + included.size());
    temp.addAll(included);
    temp.addAll(secondaryQueryJoins);
    return temp.iterator();
  }
  
  public void addSecondaryQueryJoin(String property) {
    if (secondaryQueryJoins == null) {
      secondaryQueryJoins = new HashSet(4);
    }
    secondaryQueryJoins.add(property);
  }
  





  public Set<String> getAllIncludedProperties()
  {
    if (included == null) {
      return null;
    }
    
    if ((includedBeanJoin == null) && (secondaryQueryJoins == null)) {
      return new LinkedHashSet(included);
    }
    
    LinkedHashSet<String> s = new LinkedHashSet(2 * (included.size() + 5));
    if (included != null) {
      s.addAll(included);
    }
    if (includedBeanJoin != null) {
      s.addAll(includedBeanJoin);
    }
    if (secondaryQueryJoins != null) {
      s.addAll(secondaryQueryJoins);
    }
    return s;
  }
  
  public boolean isIncluded(String propName)
  {
    if ((includedBeanJoin != null) && (includedBeanJoin.contains(propName))) {
      return false;
    }
    if (included == null)
    {
      return true;
    }
    return included.contains(propName);
  }
  






  public OrmQueryProperties setQueryFetchBatch(int queryFetchBatch)
  {
    this.queryFetchBatch = queryFetchBatch;
    return this;
  }
  
  public OrmQueryProperties setQueryFetchAll(boolean queryFetchAll) {
    this.queryFetchAll = queryFetchAll;
    return this;
  }
  
  public OrmQueryProperties setQueryFetch(int batch, boolean queryFetchAll) {
    queryFetchBatch = batch;
    this.queryFetchAll = queryFetchAll;
    return this;
  }
  


  public OrmQueryProperties setLazyFetchBatch(int lazyFetchBatch)
  {
    this.lazyFetchBatch = lazyFetchBatch;
    return this;
  }
  
  public boolean isFetchJoin() {
    return (!isQueryFetch()) && (!isLazyFetch());
  }
  
  public boolean isQueryFetch() {
    return queryFetchBatch > -1;
  }
  
  public int getQueryFetchBatch() {
    return queryFetchBatch;
  }
  
  public boolean isQueryFetchAll() {
    return queryFetchAll;
  }
  
  public boolean isLazyFetch() {
    return lazyFetchBatch > -1;
  }
  
  public int getLazyFetchBatch() {
    return lazyFetchBatch;
  }
  
  public boolean isReadOnly() {
    return readOnly;
  }
  
  public boolean isCache() {
    return cache;
  }
  
  public String getParentPath() {
    return parentPath;
  }
  
  public String getPath() {
    return path;
  }
  
  private void parseProperties() {
    if (trimmedProperties == null) {
      return;
    }
    int pos = trimmedProperties.indexOf("+readonly");
    if (pos > -1) {
      trimmedProperties = StringHelper.replaceString(trimmedProperties, "+readonly", "");
      readOnly = true;
    }
    pos = trimmedProperties.indexOf("+cache");
    if (pos > -1) {
      trimmedProperties = StringHelper.replaceString(trimmedProperties, "+cache", "");
      cache = true;
    }
    pos = trimmedProperties.indexOf("+query");
    if (pos > -1) {
      queryFetchBatch = parseBatchHint(pos, "+query");
    }
    pos = trimmedProperties.indexOf("+lazy");
    if (pos > -1) {
      lazyFetchBatch = parseBatchHint(pos, "+lazy");
    }
    
    trimmedProperties = trimmedProperties.trim();
    while (trimmedProperties.startsWith(",")) {
      trimmedProperties = trimmedProperties.substring(1).trim();
    }
  }
  
  private int parseBatchHint(int pos, String option)
  {
    int startPos = pos + option.length();
    
    int endPos = findEndPos(startPos, trimmedProperties);
    if (endPos == -1) {
      trimmedProperties = StringHelper.replaceString(trimmedProperties, option, "");
      return 0;
    }
    

    String batchParam = trimmedProperties.substring(startPos + 1, endPos);
    
    if (endPos + 1 >= trimmedProperties.length()) {
      trimmedProperties = trimmedProperties.substring(0, pos);
    } else {
      trimmedProperties = (trimmedProperties.substring(0, pos) + trimmedProperties.substring(endPos + 1));
    }
    return Integer.parseInt(batchParam);
  }
  

  private int findEndPos(int pos, String props)
  {
    if ((pos < props.length()) && 
      (props.charAt(pos) == '(')) {
      int endPara = props.indexOf(')', pos + 1);
      if (endPara == -1) {
        String m = "Error could not find ')' in " + props + " after position " + pos;
        throw new RuntimeException(m);
      }
      return endPara;
    }
    
    return -1;
  }
  



  private static Set<String> parseIncluded(String rawList)
  {
    String[] res = rawList.split(",");
    
    LinkedHashSet<String> set = new LinkedHashSet(res.length + 3);
    
    String temp = null;
    for (int i = 0; i < res.length; i++) {
      temp = res[i].trim();
      if (temp.length() > 0) {
        set.add(temp);
      }
    }
    
    if (set.isEmpty()) {
      return null;
    }
    
    return Collections.unmodifiableSet(set);
  }
}
