package com.avaje.ebeaninternal.server.subclass;

import com.avaje.ebean.enhance.agent.AlreadyEnhancedException;
import com.avaje.ebean.enhance.agent.ClassMeta;
import com.avaje.ebean.enhance.agent.EnhanceConstants;
import com.avaje.ebean.enhance.agent.EnhanceContext;
import com.avaje.ebean.enhance.agent.IndexFieldWeaver;
import com.avaje.ebean.enhance.agent.InterceptField;
import com.avaje.ebean.enhance.agent.MarkerField;
import com.avaje.ebean.enhance.agent.MethodEquals;
import com.avaje.ebean.enhance.agent.MethodIsEmbeddedNewOrDirty;
import com.avaje.ebean.enhance.agent.MethodNewInstance;
import com.avaje.ebean.enhance.agent.MethodPropertyChangeListener;
import com.avaje.ebean.enhance.agent.MethodSetEmbeddedLoaded;
import com.avaje.ebean.enhance.agent.NoEnhancementRequiredException;
import com.avaje.ebean.enhance.agent.VisitMethodParams;
import com.avaje.ebean.enhance.asm.AnnotationVisitor;
import com.avaje.ebean.enhance.asm.ClassAdapter;
import com.avaje.ebean.enhance.asm.ClassVisitor;
import com.avaje.ebean.enhance.asm.FieldVisitor;
import com.avaje.ebean.enhance.asm.MethodVisitor;
import java.util.logging.Logger;

public class SubClassClassAdpater
  extends ClassAdapter
  implements EnhanceConstants
{
  static final Logger logger = Logger.getLogger(SubClassClassAdpater.class.getName());
  
  final EnhanceContext enhanceContext;
  
  final ClassLoader classLoader;
  
  final ClassMeta classMeta;
  
  final String subClassSuffix;
  
  boolean firstMethod = true;
  
  public SubClassClassAdpater(String subClassSuffix, ClassVisitor cv, ClassLoader classLoader, EnhanceContext context) {
    super(cv);
    this.subClassSuffix = subClassSuffix;
    this.classLoader = classLoader;
    enhanceContext = context;
    classMeta = context.createClassMeta();
  }
  
  public boolean isLog(int level) {
    return classMeta.isLog(level);
  }
  
  public void log(String msg) {
    classMeta.log(msg);
  }
  





  public void visit(int version, int access, String name, String signature, String superName, String[] interfaces)
  {
    int n = 1 + interfaces.length;
    String[] c = new String[n];
    for (int i = 0; i < interfaces.length; i++) {
      c[i] = interfaces[i];
      if (c[i].equals("com/avaje/ebean/bean/EntityBean")) {
        throw new AlreadyEnhancedException(name);
      }
      if (c[i].equals("scala/ScalaObject")) {
        classMeta.setScalaInterface(true);
      }
      if (c[i].equals("groovy/lang/GroovyObject")) {
        classMeta.setGroovyInterface(true);
      }
    }
    

    c[(c.length - 1)] = "com/avaje/ebean/bean/EntityBean";
    
    if (!superName.equals("java/lang/Object"))
    {
      ClassMeta superMeta = enhanceContext.getSuperMeta(superName, classLoader);
      if (superMeta != null) {
        classMeta.setSuperMeta(superMeta);
        if (classMeta.isLog(2)) {
          classMeta.log("entity inheritance " + superMeta.getDescription());
        }
      }
    }
    



    superName = name;
    name = name + subClassSuffix;
    
    classMeta.setClassName(name, superName);
    
    super.visit(version, access, name, signature, superName, c);
  }
  
  public AnnotationVisitor visitAnnotation(String desc, boolean visible)
  {
    classMeta.addClassAnnotation(desc);
    return super.visitAnnotation(desc, visible);
  }
  





  public FieldVisitor visitField(int access, String name, String desc, String signature, Object value)
  {
    if ((access & 0x8) != 0)
    {
      if (isLog(2)) {
        log("Skip intercepting static field " + name);
      }
      return null;
    }
    
    if ((access & 0x80) != 0)
    {
      if (classMeta.isLog(2)) {
        classMeta.log("Skip intercepting transient field " + name);
      }
      return null;
    }
    

    if (classMeta.isLog(5)) {
      classMeta.log(" ... reading field:" + name + " desc:" + desc);
    }
    
    return classMeta.createLocalFieldVisitor(name, desc);
  }
  





  public MethodVisitor visitMethod(int access, String name, String desc, String signature, String[] exceptions)
  {
    if (firstMethod) {
      if (!classMeta.isEntityEnhancementRequired())
      {
        throw new NoEnhancementRequiredException();
      }
      

      String marker = MarkerField.addField(cv, classMeta.getClassName());
      if (isLog(4)) {
        log("... add marker field \"" + marker + "\"");
        log("... add intercept and identity fields");
      }
      

      InterceptField.addField(cv, enhanceContext.isTransientInternalFields());
      MethodEquals.addIdentityField(cv);
      firstMethod = false;
    }
    

    VisitMethodParams params = new VisitMethodParams(cv, access, name, desc, signature, exceptions);
    
    if (isDefaultConstructor(access, name, desc, signature, exceptions)) {
      SubClassConstructor.add(params, classMeta);
      return null;
    }
    
    if (isSpecialMethod(access, name, desc)) {
      return null;
    }
    




    classMeta.addExistingSuperMethod(name, desc);
    
    return null;
  }
  




  public void visitEnd()
  {
    if (!classMeta.isEntityEnhancementRequired()) {
      throw new NoEnhancementRequiredException();
    }
    
    if (!classMeta.hasDefaultConstructor()) {
      if (isLog(2)) {
        log("... adding default constructor");
      }
      SubClassConstructor.addDefault(cv, classMeta);
    }
    
    MarkerField.addGetMarker(cv, classMeta.getClassName());
    

    InterceptField.addGetterSetter(cv, classMeta.getClassName());
    

    MethodPropertyChangeListener.addMethod(cv, classMeta);
    


    GetterSetterMethods.add(cv, classMeta);
    

    IndexFieldWeaver.addMethods(cv, classMeta);
    
    MethodSetEmbeddedLoaded.addMethod(cv, classMeta);
    MethodIsEmbeddedNewOrDirty.addMethod(cv, classMeta);
    MethodNewInstance.addMethod(cv, classMeta);
    

    MethodWriteReplace.add(cv, classMeta);
    

    enhanceContext.addClassMeta(classMeta);
    
    super.visitEnd();
  }
  




  private boolean isDefaultConstructor(int access, String name, String desc, String signature, String[] exceptions)
  {
    if ((name.equals("<init>")) && (desc.equals("()V"))) {
      classMeta.setHasDefaultConstructor(true);
      return true;
    }
    
    return false;
  }
  



  private boolean isSpecialMethod(int access, String name, String desc)
  {
    if ((name.equals("hashCode")) && (desc.equals("()I"))) {
      classMeta.setHasEqualsOrHashcode(true);
      return true;
    }
    
    if ((name.equals("equals")) && (desc.equals("(Ljava/lang/Object;)Z"))) {
      classMeta.setHasEqualsOrHashcode(true);
      return true;
    }
    
    return false;
  }
}
