package com.avaje.ebeaninternal.server.text.json;

import com.avaje.ebean.bean.EntityBean;
import com.avaje.ebean.bean.EntityBeanIntercept;
import com.avaje.ebean.text.TextException;
import com.avaje.ebean.text.json.JsonElement;
import com.avaje.ebean.text.json.JsonReadBeanVisitor;
import com.avaje.ebean.text.json.JsonReadOptions;
import com.avaje.ebean.text.json.JsonValueAdapter;
import com.avaje.ebeaninternal.server.deploy.BeanDescriptor;
import com.avaje.ebeaninternal.server.util.ArrayStack;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Set;



























public class ReadJsonContext
{
  private final ReadJsonSource src;
  private final Map<String, JsonReadBeanVisitor<?>> visitorMap;
  private final JsonValueAdapter valueAdapter;
  private final PathStack pathStack;
  private final ArrayStack<ReadBeanState> beanState;
  private ReadBeanState currentState;
  private char tokenStart;
  private String tokenKey;
  
  public ReadJsonContext(ReadJsonSource src, JsonValueAdapter dfltValueAdapter, JsonReadOptions options)
  {
    this.src = src;
    beanState = new ArrayStack();
    if (options == null) {
      valueAdapter = dfltValueAdapter;
      visitorMap = null;
      pathStack = null;
    } else {
      valueAdapter = getValueAdapter(dfltValueAdapter, options.getValueAdapter());
      visitorMap = options.getVisitorMap();
      pathStack = ((visitorMap == null) || (visitorMap.isEmpty()) ? null : new PathStack());
    }
  }
  
  private JsonValueAdapter getValueAdapter(JsonValueAdapter dfltValueAdapter, JsonValueAdapter valueAdapter) {
    return valueAdapter == null ? dfltValueAdapter : valueAdapter;
  }
  
  public JsonValueAdapter getValueAdapter() {
    return valueAdapter;
  }
  
  public char getToken() {
    return tokenStart;
  }
  
  public String getTokenKey() {
    return tokenKey;
  }
  
  public boolean isTokenKey() {
    return '"' == tokenStart;
  }
  
  public boolean isTokenObjectEnd() {
    return '}' == tokenStart;
  }
  
  public boolean readObjectBegin() {
    readNextToken();
    if ('{' == tokenStart)
      return true;
    if ('n' == tokenStart)
      return false;
    if (']' == tokenStart)
    {
      return false;
    }
    throw new RuntimeException("Expected object begin at " + src.getErrorHelp());
  }
  
  public boolean readKeyNext() {
    readNextToken();
    if ('"' == tokenStart)
      return true;
    if ('}' == tokenStart) {
      return false;
    }
    throw new RuntimeException("Expected '\"' or '}' at " + src.getErrorHelp());
  }
  
  public boolean readValueNext() {
    readNextToken();
    if (',' == tokenStart)
      return true;
    if ('}' == tokenStart) {
      return false;
    }
    throw new RuntimeException("Expected ',' or '}' at " + src.getErrorHelp() + " but got " + tokenStart);
  }
  
  public boolean readArrayBegin() {
    readNextToken();
    if ('[' == tokenStart)
      return true;
    if ('n' == tokenStart) {
      return false;
    }
    throw new RuntimeException("Expected array begin at " + src.getErrorHelp());
  }
  
  public boolean readArrayNext() {
    readNextToken();
    if (',' == tokenStart) {
      return true;
    }
    if (']' == tokenStart) {
      return false;
    }
    throw new RuntimeException("Expected ',' or ']' at " + src.getErrorHelp());
  }
  
  public String readScalarValue()
  {
    ignoreWhiteSpace();
    
    char prevChar = src.nextChar("EOF reading scalarValue?");
    if ('"' == prevChar) {
      return readQuotedValue();
    }
    return readUnquotedValue(prevChar);
  }
  



  public void readNextToken()
  {
    ignoreWhiteSpace();
    
    tokenStart = src.nextChar("EOF finding next token");
    switch (tokenStart) {
    case '"': 
      internalReadKey();
      break;
    case '{': 
      break;
    case '}': 
      break;
    case '[': 
      break;
    case ']': 
      break; case ',':  break; case ':':  break; case 'n':  internalReadNull();
      break;
    
    default: 
      throw new RuntimeException("Unexpected tokenStart[" + tokenStart + "] " + src.getErrorHelp());
    }
    
  }
  

  protected String readQuotedValue()
  {
    boolean escape = false;
    StringBuilder sb = new StringBuilder();
    for (;;)
    {
      char ch = src.nextChar("EOF reading quoted value");
      if (escape)
      {
        sb.append(ch);
      }
      else {
        switch (ch)
        {
        case '\\': 
          escape = true;
          break;
        case '"': 
          return sb.toString();
        
        default: 
          sb.append(ch);
        }
      }
    }
  }
  
  protected String readUnquotedValue(char c) {
    String v = readUnquotedValueRaw(c);
    if ("null".equals(v)) {
      return null;
    }
    return v;
  }
  

  private String readUnquotedValueRaw(char c)
  {
    StringBuilder sb = new StringBuilder();
    sb.append(c);
    for (;;)
    {
      tokenStart = src.nextChar("EOF reading unquoted value");
      switch (tokenStart) {
      case ',': 
        src.back();
        return sb.toString();
      
      case '}': 
        src.back();
        return sb.toString();
      
      case ' ': 
        return sb.toString();
      
      case '\t': 
        return sb.toString();
      
      case '\r': 
        return sb.toString();
      
      case '\n': 
        return sb.toString();
      }
      
      sb.append(tokenStart);
    }
  }
  



  private void internalReadNull()
  {
    StringBuilder sb = new StringBuilder(4);
    sb.append(tokenStart);
    for (int i = 0; i < 3; i++) {
      char c = src.nextChar("EOF reading null ");
      sb.append(c);
    }
    if (!"null".equals(sb.toString())) {
      throw new TextException("Expected 'null' but got " + sb.toString() + " " + src.getErrorHelp());
    }
  }
  
  private void internalReadKey() {
    StringBuilder sb = new StringBuilder();
    for (;;) {
      char c = src.nextChar("EOF reading key");
      if ('"' == c) {
        tokenKey = sb.toString();
        break;
      }
      sb.append(c);
    }
    

    ignoreWhiteSpace();
    
    char c = src.nextChar("EOF reading ':'");
    if (':' != c) {
      throw new TextException("Expected to find colon after key at " + (src.pos() - 1) + " but found [" + c + "]" + src.getErrorHelp());
    }
  }
  
  protected void ignoreWhiteSpace() {
    src.ignoreWhiteSpace();
  }
  
  public void pushBean(Object bean, String path, BeanDescriptor<?> beanDescriptor) {
    currentState = new ReadBeanState(bean, beanDescriptor, null);
    beanState.push(currentState);
    if (pathStack != null) {
      pathStack.pushPathKey(path);
    }
  }
  
  public ReadBeanState popBeanState() {
    if (pathStack != null) {
      String path = (String)pathStack.peekWithNull();
      JsonReadBeanVisitor<?> beanVisitor = (JsonReadBeanVisitor)visitorMap.get(path);
      if (beanVisitor != null) {
        currentState.visit(beanVisitor);
      }
      pathStack.pop();
    }
    



    ReadBeanState s = currentState;
    
    beanState.pop();
    currentState = ((ReadBeanState)beanState.peekWithNull());
    return s;
  }
  
  public void setProperty(String propertyName) {
    currentState.setLoaded(propertyName);
  }
  







  public JsonElement readUnmappedJson(String key)
  {
    ReadJsonRawReader rawReader = new ReadJsonRawReader(this);
    JsonElement rawJsonValue = rawReader.readUnknownValue();
    if (visitorMap != null) {
      currentState.addUnmappedJson(key, rawJsonValue);
    }
    return rawJsonValue;
  }
  
  protected char nextChar() {
    tokenStart = src.nextChar("EOF getting nextChar for raw json");
    return tokenStart;
  }
  
  public static class ReadBeanState implements PropertyChangeListener
  {
    private final Object bean;
    private final BeanDescriptor<?> beanDescriptor;
    private final EntityBeanIntercept ebi;
    private final Set<String> loadedProps;
    private Map<String, JsonElement> unmapped;
    
    private ReadBeanState(Object bean, BeanDescriptor<?> beanDescriptor) {
      this.bean = bean;
      this.beanDescriptor = beanDescriptor;
      if ((bean instanceof EntityBean)) {
        ebi = ((EntityBean)bean)._ebean_getIntercept();
        loadedProps = new HashSet();
      } else {
        ebi = null;
        loadedProps = null;
      }
    }
    
    public String toString() { return bean.getClass().getSimpleName() + " loaded:" + loadedProps; }
    



    public void setLoaded(String propertyName)
    {
      if (ebi != null) {
        loadedProps.add(propertyName);
      }
    }
    
    private void addUnmappedJson(String key, JsonElement value) {
      if (unmapped == null) {
        unmapped = new LinkedHashMap();
      }
      unmapped.put(key, value);
    }
    


    private <T> void visit(JsonReadBeanVisitor<T> beanVisitor)
    {
      if (ebi != null) {
        ebi.addPropertyChangeListener(this);
      }
      beanVisitor.visit(bean, unmapped);
      if (ebi != null) {
        ebi.removePropertyChangeListener(this);
      }
    }
    
    public void setLoadedState() {
      if (ebi != null)
      {
        beanDescriptor.setLoadedProps(ebi, loadedProps);
      }
    }
    
    public void propertyChange(PropertyChangeEvent evt) {
      String propName = evt.getPropertyName();
      loadedProps.add(propName);
    }
    
    public Object getBean() {
      return bean;
    }
  }
}
