package com.avaje.ebeaninternal.server.text.json;

import com.avaje.ebean.text.json.JsonElement;
import com.avaje.ebean.text.json.JsonElementArray;
import com.avaje.ebean.text.json.JsonElementBoolean;
import com.avaje.ebean.text.json.JsonElementNull;
import com.avaje.ebean.text.json.JsonElementNumber;
import com.avaje.ebean.text.json.JsonElementObject;
import com.avaje.ebean.text.json.JsonElementString;





















public class ReadJsonRawReader
{
  private final ReadJsonContext ctx;
  
  public ReadJsonRawReader(ReadJsonContext ctx)
  {
    this.ctx = ctx;
  }
  
  public JsonElement readUnknownValue() {
    return readValue();
  }
  
  private JsonElement readValue()
  {
    ctx.ignoreWhiteSpace();
    
    char c = ctx.nextChar();
    
    switch (c) {
    case '{': 
      return readObject();
    
    case '[': 
      return readArray();
    
    case '"': 
      return readString();
    }
    
    return readUnquoted(c);
  }
  

  private JsonElement readArray()
  {
    JsonElementArray a = new JsonElementArray();
    for (;;)
    {
      JsonElement value = readValue();
      a.add(value);
      if (!ctx.readArrayNext()) {
        break;
      }
    }
    
    return a;
  }
  
  private JsonElement readObject()
  {
    JsonElementObject o = new JsonElementObject();
    

    while (ctx.readKeyNext())
    {


      String key = ctx.getTokenKey();
      JsonElement value = readValue();
      
      o.put(key, value);
      
      if (!ctx.readValueNext()) {
        break;
      }
    }
    

    return o;
  }
  
  private JsonElement readString() {
    String s = ctx.readQuotedValue();
    return new JsonElementString(s);
  }
  
  private JsonElement readUnquoted(char c) {
    String s = ctx.readUnquotedValue(c);
    if ("null".equals(s)) {
      return JsonElementNull.NULL;
    }
    if ("true".equals(s)) {
      return JsonElementBoolean.TRUE;
    }
    if ("false".equals(s)) {
      return JsonElementBoolean.FALSE;
    }
    
    return new JsonElementNumber(s);
  }
}
