package com.avaje.ebeaninternal.server.text.json;

import com.avaje.ebean.bean.EntityBean;
import com.avaje.ebean.bean.EntityBeanIntercept;
import com.avaje.ebean.text.PathProperties;
import com.avaje.ebean.text.json.JsonValueAdapter;
import com.avaje.ebean.text.json.JsonWriteBeanVisitor;
import com.avaje.ebean.text.json.JsonWriteOptions;
import com.avaje.ebean.text.json.JsonWriter;
import com.avaje.ebeaninternal.server.util.ArrayStack;
import java.util.Map;
import java.util.Set;
























public class WriteJsonContext
  implements JsonWriter
{
  private final WriteJsonBuffer buffer;
  private final boolean pretty;
  private final JsonValueAdapter valueAdapter;
  private final ArrayStack<Object> parentBeans = new ArrayStack();
  
  private final PathProperties pathProperties;
  
  private final Map<String, JsonWriteBeanVisitor<?>> visitorMap;
  
  private final String callback;
  
  private final PathStack pathStack;
  
  private WriteBeanState beanState;
  
  private int depthOffset;
  
  boolean assocOne;
  

  public WriteJsonContext(WriteJsonBuffer buffer, boolean pretty, JsonValueAdapter dfltValueAdapter, JsonWriteOptions options, String requestCallback)
  {
    this.buffer = buffer;
    this.pretty = pretty;
    pathStack = new PathStack();
    callback = getCallback(requestCallback, options);
    if (options == null) {
      valueAdapter = dfltValueAdapter;
      visitorMap = null;
      pathProperties = null;
    }
    else {
      valueAdapter = getValueAdapter(dfltValueAdapter, options.getValueAdapter());
      visitorMap = emptyToNull(options.getVisitorMap());
      pathProperties = emptyToNull(options.getPathProperties());
    }
    
    if (callback != null) {
      buffer.append(requestCallback).append("(");
    }
  }
  
  public void end() {
    if (callback != null) {
      buffer.append(")");
    }
  }
  
  private <MK, MV> Map<MK, MV> emptyToNull(Map<MK, MV> m) {
    if ((m == null) || (m.isEmpty())) {
      return null;
    }
    return m;
  }
  
  private PathProperties emptyToNull(PathProperties m)
  {
    if ((m == null) || (m.isEmpty())) {
      return null;
    }
    return m;
  }
  
  private String getCallback(String requestCallback, JsonWriteOptions options)
  {
    if (requestCallback != null) {
      return requestCallback;
    }
    if (options != null) {
      return options.getCallback();
    }
    return null;
  }
  
  private JsonValueAdapter getValueAdapter(JsonValueAdapter dfltValueAdapter, JsonValueAdapter valueAdapter) {
    return valueAdapter == null ? dfltValueAdapter : valueAdapter;
  }
  



  public Set<String> getIncludeProperties()
  {
    if (pathProperties != null) {
      String path = (String)pathStack.peekWithNull();
      return pathProperties.get(path);
    }
    return null;
  }
  
  public JsonWriteBeanVisitor<?> getBeanVisitor() {
    if (visitorMap != null) {
      String path = (String)pathStack.peekWithNull();
      return (JsonWriteBeanVisitor)visitorMap.get(path);
    }
    return null;
  }
  
  public String getJson() {
    return buffer.toString();
  }
  
  private void appendIndent()
  {
    buffer.append("\n");
    int depth = depthOffset + parentBeans.size();
    for (int i = 0; i < depth; i++) {
      buffer.append("    ");
    }
  }
  
  public void appendObjectBegin() {
    if ((pretty) && (!assocOne)) {
      appendIndent();
    }
    buffer.append("{");
  }
  
  public void appendObjectEnd() { buffer.append("}"); }
  
  public void appendArrayBegin()
  {
    if (pretty) {
      appendIndent();
    }
    buffer.append("[");
    depthOffset += 1;
  }
  
  public void appendArrayEnd() {
    depthOffset -= 1;
    if (pretty) {
      appendIndent();
    }
    buffer.append("]");
  }
  
  public void appendComma() {
    buffer.append(",");
  }
  
  public void addDepthOffset(int offset) {
    depthOffset += offset;
  }
  
  public void beginAssocOneIsNull(String key) {
    depthOffset += 1;
    internalAppendKeyBegin(key);
    appendNull();
    depthOffset -= 1;
  }
  
  public void beginAssocOne(String key) {
    pathStack.pushPathKey(key);
    
    internalAppendKeyBegin(key);
    assocOne = true;
  }
  
  public void endAssocOne()
  {
    pathStack.pop();
    assocOne = false;
  }
  
  public Boolean includeMany(String key) {
    if (pathProperties != null) {
      String fullPath = pathStack.peekFullPath(key);
      return Boolean.valueOf(pathProperties.hasPath(fullPath));
    }
    return null;
  }
  
  public void beginAssocMany(String key)
  {
    pathStack.pushPathKey(key);
    
    depthOffset -= 1;
    internalAppendKeyBegin(key);
    depthOffset += 1;
    buffer.append("[");
  }
  
  public void endAssocMany()
  {
    pathStack.pop();
    
    if (pretty) {
      depthOffset -= 1;
      appendIndent();
      depthOffset += 1;
    }
    buffer.append("]");
  }
  
  private void internalAppendKeyBegin(String key) {
    if (!beanState.isFirstKey()) {
      buffer.append(",");
    }
    if (pretty) {
      appendIndent();
    }
    appendKeyWithComma(key, false);
  }
  
  public void appendKey(String key) {
    appendKeyWithComma(key, true);
  }
  
  private void appendKeyWithComma(String key, boolean withComma) {
    if ((withComma) && 
      (!beanState.isFirstKey())) {
      buffer.append(",");
    }
    
    buffer.append("\"");
    if (key == null) {
      buffer.append("null");
    } else {
      buffer.append(key);
    }
    buffer.append("\":");
  }
  
  public void appendKeyValue(String key, String escapedValue) {
    appendKey(key);
    buffer.append(escapedValue);
  }
  
  public void appendNull(String key) {
    appendKey(key);
    buffer.append("null");
  }
  
  public void appendNull() {
    buffer.append("null");
  }
  
  public JsonValueAdapter getValueAdapter() {
    return valueAdapter;
  }
  
  public String toString() {
    return buffer.toString();
  }
  
  public void popParentBean() {
    parentBeans.pop();
  }
  
  public void pushParentBean(Object parentBean) {
    parentBeans.push(parentBean);
  }
  
  public void popParentBeanMany() {
    parentBeans.pop();
    depthOffset -= 1;
  }
  
  public void pushParentBeanMany(Object parentBean) {
    parentBeans.push(parentBean);
    depthOffset += 1;
  }
  
  public boolean isParentBean(Object bean) {
    if (parentBeans.isEmpty()) {
      return false;
    }
    return bean == parentBeans.peek();
  }
  
  public WriteBeanState pushBeanState(Object bean)
  {
    WriteBeanState newState = new WriteBeanState(bean);
    WriteBeanState prevState = beanState;
    beanState = newState;
    return prevState;
  }
  
  public void pushPreviousState(WriteBeanState previousState) {
    beanState = previousState;
  }
  
  public boolean isReferenceBean() {
    return beanState.isReferenceBean();
  }
  
  public boolean includedProp(String name) {
    return beanState.includedProp(name);
  }
  
  public Set<String> getLoadedProps() {
    return beanState.getLoadedProps();
  }
  
  public static class WriteBeanState
  {
    private final EntityBeanIntercept ebi;
    private final Set<String> loadedProps;
    private final boolean referenceBean;
    private boolean firstKeyOut;
    
    public WriteBeanState(Object bean)
    {
      if ((bean instanceof EntityBean)) {
        ebi = ((EntityBean)bean)._ebean_getIntercept();
        loadedProps = ebi.getLoadedProps();
        referenceBean = ebi.isReference();
      } else {
        ebi = null;
        loadedProps = null;
        referenceBean = false;
      }
    }
    
    public Set<String> getLoadedProps() {
      return loadedProps;
    }
    
    public boolean includedProp(String name) {
      if ((loadedProps == null) || (loadedProps.contains(name))) {
        return true;
      }
      return false;
    }
    
    public boolean isReferenceBean() {
      return referenceBean;
    }
    
    public boolean isFirstKey() {
      if (!firstKeyOut) {
        firstKeyOut = true;
        return true;
      }
      return false;
    }
  }
}
