package com.avaje.ebeaninternal.server.transaction;

import com.avaje.ebean.event.BeanPersistListener;
import com.avaje.ebeaninternal.api.SpiEbeanServer;
import com.avaje.ebeaninternal.server.cluster.BinaryMessage;
import com.avaje.ebeaninternal.server.cluster.BinaryMessageList;
import com.avaje.ebeaninternal.server.core.PersistRequest.Type;
import com.avaje.ebeaninternal.server.deploy.BeanDescriptor;
import com.avaje.ebeaninternal.server.deploy.id.IdBinder;
import java.io.DataInput;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;






































public class BeanPersistIds
  implements Serializable
{
  private static final long serialVersionUID = 8389469180931531409L;
  private transient BeanDescriptor<?> beanDescriptor;
  private final String descriptorId;
  private ArrayList<Serializable> insertIds;
  private ArrayList<Serializable> updateIds;
  private ArrayList<Serializable> deleteIds;
  
  public BeanPersistIds(BeanDescriptor<?> desc)
  {
    beanDescriptor = desc;
    descriptorId = desc.getDescriptorId();
  }
  
  public static BeanPersistIds readBinaryMessage(SpiEbeanServer server, DataInput dataInput) throws IOException
  {
    String descriptorId = dataInput.readUTF();
    BeanDescriptor<?> desc = server.getBeanDescriptorById(descriptorId);
    BeanPersistIds bp = new BeanPersistIds(desc);
    bp.read(dataInput);
    return bp;
  }
  
  private void read(DataInput dataInput) throws IOException
  {
    IdBinder idBinder = beanDescriptor.getIdBinder();
    
    int iudType = dataInput.readInt();
    ArrayList<Serializable> idList = readIdList(dataInput, idBinder);
    
    switch (iudType) {
    case 0: 
      insertIds = idList;
      break;
    case 1: 
      updateIds = idList;
      break;
    case 2: 
      deleteIds = idList;
      break;
    
    default: 
      throw new RuntimeException("Invalid iudType " + iudType);
    }
    
  }
  







  public void writeBinaryMessage(BinaryMessageList msgList)
    throws IOException
  {
    writeIdList(beanDescriptor, 0, insertIds, msgList);
    writeIdList(beanDescriptor, 1, updateIds, msgList);
    writeIdList(beanDescriptor, 2, deleteIds, msgList);
  }
  
  private ArrayList<Serializable> readIdList(DataInput dataInput, IdBinder idBinder)
    throws IOException
  {
    int count = dataInput.readInt();
    if (count < 1) {
      return null;
    }
    ArrayList<Serializable> idList = new ArrayList(count);
    for (int i = 0; i < count; i++) {
      Object id = idBinder.readData(dataInput);
      idList.add((Serializable)id);
    }
    return idList;
  }
  










  private void writeIdList(BeanDescriptor<?> desc, int iudType, ArrayList<Serializable> idList, BinaryMessageList msgList)
    throws IOException
  {
    IdBinder idBinder = desc.getIdBinder();
    
    int count = idList == null ? 0 : idList.size();
    if (count > 0) {
      int loop = 0;
      int i = 0;
      int eof = idList.size();
      do {
        loop++;
        int endOfLoop = Math.min(eof, loop * 100);
        
        BinaryMessage m = new BinaryMessage(endOfLoop * 4 + 20);
        
        DataOutputStream os = m.getOs();
        os.writeInt(1);
        os.writeUTF(descriptorId);
        os.writeInt(iudType);
        os.writeInt(count);
        for (; 
            i < endOfLoop; i++) {
          Serializable idValue = (Serializable)idList.get(i);
          idBinder.writeData(os, idValue);
        }
        
        os.flush();
        msgList.add(m);
      }
      while (i < eof);
    }
  }
  
  public String toString() {
    StringBuilder sb = new StringBuilder();
    if (beanDescriptor != null) {
      sb.append(beanDescriptor.getFullName());
    } else {
      sb.append("descId:").append(descriptorId);
    }
    if (insertIds != null) {
      sb.append(" insertIds:").append(insertIds);
    }
    if (updateIds != null) {
      sb.append(" updateIds:").append(updateIds);
    }
    if (deleteIds != null) {
      sb.append(" deleteIds:").append(deleteIds);
    }
    return sb.toString();
  }
  
  public void addId(PersistRequest.Type type, Serializable id) {
    switch (1.$SwitchMap$com$avaje$ebeaninternal$server$core$PersistRequest$Type[type.ordinal()]) {
    case 1: 
      addInsertId(id);
      break;
    case 2: 
      addUpdateId(id);
      break;
    case 3: 
      addDeleteId(id);
      break;
    }
    
  }
  

  private void addInsertId(Serializable id)
  {
    if (insertIds == null) {
      insertIds = new ArrayList();
    }
    insertIds.add(id);
  }
  
  private void addUpdateId(Serializable id) {
    if (updateIds == null) {
      updateIds = new ArrayList();
    }
    updateIds.add(id);
  }
  
  private void addDeleteId(Serializable id) {
    if (deleteIds == null) {
      deleteIds = new ArrayList();
    }
    deleteIds.add(id);
  }
  
  public BeanDescriptor<?> getBeanDescriptor() {
    return beanDescriptor;
  }
  



  public String getDescriptorId()
  {
    return descriptorId;
  }
  
  public List<Serializable> getInsertIds() {
    return insertIds;
  }
  
  public List<Serializable> getUpdateIds() {
    return updateIds;
  }
  
  public List<Serializable> getDeleteIds() {
    return deleteIds;
  }
  
  public void setBeanDescriptor(BeanDescriptor<?> beanDescriptor) {
    this.beanDescriptor = beanDescriptor;
  }
  




  public void notifyCacheAndListener()
  {
    BeanPersistListener<?> listener = beanDescriptor.getPersistListener();
    

    beanDescriptor.queryCacheClear();
    
    if ((insertIds != null) && 
      (listener != null))
    {
      for (int i = 0; i < insertIds.size(); i++) {
        listener.remoteInsert(insertIds.get(i));
      }
    }
    
    if (updateIds != null) {
      for (int i = 0; i < updateIds.size(); i++) {
        Serializable id = (Serializable)updateIds.get(i);
        

        beanDescriptor.cacheRemove(id);
        if (listener != null)
        {
          listener.remoteInsert(id);
        }
      }
    }
    if (deleteIds != null) {
      for (int i = 0; i < deleteIds.size(); i++) {
        Serializable id = (Serializable)deleteIds.get(i);
        

        beanDescriptor.cacheRemove(id);
        if (listener != null)
        {
          listener.remoteInsert(id);
        }
      }
    }
  }
}
