package com.avaje.ebeaninternal.server.transaction;

import com.avaje.ebean.bean.PersistenceContext;
import com.avaje.ebeaninternal.api.Monitor;
import com.avaje.ebeaninternal.server.subclass.SubClassUtil;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map.Entry;
import java.util.Set;







































public final class DefaultPersistenceContext
  implements PersistenceContext
{
  private final HashMap<String, ClassContext> typeCache = new HashMap();
  
  private final Monitor monitor = new Monitor();
  








  public void put(Object id, Object bean)
  {
    synchronized (monitor) {
      getClassContext(bean.getClass()).put(id, bean);
    }
  }
  
  public Object putIfAbsent(Object id, Object bean) {
    synchronized (monitor) {
      return getClassContext(bean.getClass()).putIfAbsent(id, bean);
    }
  }
  




  public Object get(Class<?> beanType, Object id)
  {
    synchronized (monitor) {
      return getClassContext(beanType).get(id);
    }
  }
  


  public int size(Class<?> beanType)
  {
    synchronized (monitor) {
      ClassContext classMap = (ClassContext)typeCache.get(beanType.getName());
      return classMap == null ? 0 : classMap.size();
    }
  }
  


  public void clear()
  {
    synchronized (monitor) {
      typeCache.clear();
    }
  }
  
  public void clear(Class<?> beanType) {
    synchronized (monitor) {
      ClassContext classMap = (ClassContext)typeCache.get(beanType.getName());
      if (classMap != null) {
        classMap.clear();
      }
    }
  }
  
  public void clear(Class<?> beanType, Object id) {
    synchronized (monitor) {
      ClassContext classMap = (ClassContext)typeCache.get(beanType.getName());
      if ((classMap != null) && (id != null))
      {
        classMap.remove(id);
      }
    }
  }
  
  public String toString() {
    synchronized (monitor) {
      StringBuilder sb = new StringBuilder();
      Iterator<Map.Entry<String, ClassContext>> it = typeCache.entrySet().iterator();
      while (it.hasNext()) {
        Map.Entry<String, ClassContext> entry = (Map.Entry)it.next();
        if (((ClassContext)entry.getValue()).size() > 0) {
          sb.append((String)entry.getKey() + ":" + ((ClassContext)entry.getValue()).size() + "; ");
        }
      }
      return sb.toString();
    }
  }
  

  private ClassContext getClassContext(Class<?> beanType)
  {
    String clsName = SubClassUtil.getSuperClassName(beanType.getName());
    
    ClassContext classMap = (ClassContext)typeCache.get(clsName);
    if (classMap == null) {
      classMap = new ClassContext(null);
      typeCache.put(clsName, classMap);
    }
    return classMap;
  }
  
  private static class ClassContext
  {
    private final WeakValueMap<Object, Object> map = new WeakValueMap();
    
    private Object get(Object id) {
      return map.get(id);
    }
    
    private Object putIfAbsent(Object id, Object bean)
    {
      return map.putIfAbsent(id, bean);
    }
    
    private void put(Object id, Object b) {
      map.put(id, b);
    }
    
    private int size() {
      return map.size();
    }
    
    private void clear() {
      map.clear();
    }
    
    private Object remove(Object id) {
      return map.remove(id);
    }
  }
}
