package com.avaje.ebeaninternal.server.transaction;

import com.avaje.ebeaninternal.api.SpiTransaction;
import com.avaje.ebeaninternal.api.TransactionEvent;
import com.avaje.ebeaninternal.api.TransactionEventBeans;
import com.avaje.ebeaninternal.api.TransactionEventTable;
import com.avaje.ebeaninternal.api.TransactionEventTable.TableIUD;
import com.avaje.ebeaninternal.server.cluster.ClusterManager;
import com.avaje.ebeaninternal.server.core.PersistRequestBean;
import com.avaje.ebeaninternal.server.deploy.BeanDescriptorManager;
import com.avaje.ebeaninternal.server.lucene.LuceneIndexManager;
import java.util.List;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;



























public final class PostCommitProcessing
{
  private static final Logger logger = Logger.getLogger(PostCommitProcessing.class.getName());
  

  private final ClusterManager clusterManager;
  

  private final LuceneIndexManager luceneIndexManager;
  

  private final SpiTransaction transaction;
  
  private final TransactionEvent event;
  
  private final String serverName;
  
  private final TransactionManager manager;
  
  private final List<PersistRequestBean<?>> persistBeanRequests;
  
  private final BeanPersistIdMap beanPersistIdMap;
  
  private final BeanDeltaMap beanDeltaMap;
  
  private final RemoteTransactionEvent remoteTransactionEvent;
  
  private final DeleteByIdMap deleteByIdMap;
  

  public PostCommitProcessing(ClusterManager clusterManager, LuceneIndexManager luceneIndexManager, TransactionManager manager, SpiTransaction transaction, TransactionEvent event)
  {
    this.clusterManager = clusterManager;
    this.luceneIndexManager = luceneIndexManager;
    this.manager = manager;
    serverName = manager.getServerName();
    this.transaction = transaction;
    this.event = event;
    deleteByIdMap = event.getDeleteByIdMap();
    persistBeanRequests = createPersistBeanRequests();
    
    beanPersistIdMap = createBeanPersistIdMap();
    beanDeltaMap = new BeanDeltaMap(event.getBeanDeltas());
    
    remoteTransactionEvent = createRemoteTransactionEvent();
  }
  
  public void notifyLocalCacheIndex()
  {
    if (luceneIndexManager.isLuceneAvailable()) {
      luceneIndexManager.processEvent(remoteTransactionEvent, transaction);
    }
    

    processTableEvents(event.getEventTables());
    

    event.notifyCache();
  }
  


  private void processTableEvents(TransactionEventTable tableEvents)
  {
    BeanDescriptorManager dm;
    
    if ((tableEvents != null) && (!tableEvents.isEmpty()))
    {
      dm = manager.getBeanDescriptorManager();
      for (TransactionEventTable.TableIUD tableIUD : tableEvents.values()) {
        dm.cacheNotify(tableIUD);
      }
    }
  }
  
  public void notifyCluster() {
    if ((remoteTransactionEvent != null) && (!remoteTransactionEvent.isEmpty()))
    {
      if ((manager.getClusterDebugLevel() > 0) || (logger.isLoggable(Level.FINE))) {
        logger.info("Cluster Send: " + remoteTransactionEvent.toString());
      }
      
      clusterManager.broadcast(remoteTransactionEvent);
    }
  }
  
  public Runnable notifyPersistListeners() {
    new Runnable() {
      public void run() {
        PostCommitProcessing.this.localPersistListenersNotify();
      }
    };
  }
  
  private void localPersistListenersNotify() {
    if (persistBeanRequests != null) {
      for (int i = 0; i < persistBeanRequests.size(); i++) {
        ((PersistRequestBean)persistBeanRequests.get(i)).notifyLocalPersistListener();
      }
    }
  }
  
  private List<PersistRequestBean<?>> createPersistBeanRequests() {
    TransactionEventBeans eventBeans = event.getEventBeans();
    if (eventBeans != null) {
      return eventBeans.getRequests();
    }
    return null;
  }
  
  private BeanPersistIdMap createBeanPersistIdMap()
  {
    if (persistBeanRequests == null) {
      return null;
    }
    
    BeanPersistIdMap m = new BeanPersistIdMap();
    for (int i = 0; i < persistBeanRequests.size(); i++) {
      ((PersistRequestBean)persistBeanRequests.get(i)).addToPersistMap(m);
    }
    return m;
  }
  
  private RemoteTransactionEvent createRemoteTransactionEvent()
  {
    if ((!clusterManager.isClustering()) && (!luceneIndexManager.isLuceneAvailable())) {
      return null;
    }
    
    RemoteTransactionEvent remoteTransactionEvent = new RemoteTransactionEvent(serverName);
    
    if (beanDeltaMap != null) {
      for (BeanDeltaList deltaList : beanDeltaMap.deltaLists()) {
        remoteTransactionEvent.addBeanDeltaList(deltaList);
      }
    }
    
    if (beanPersistIdMap != null) {
      for (BeanPersistIds beanPersist : beanPersistIdMap.values()) {
        remoteTransactionEvent.addBeanPersistIds(beanPersist);
      }
    }
    
    if (deleteByIdMap != null) {
      remoteTransactionEvent.setDeleteByIdMap(deleteByIdMap);
    }
    
    TransactionEventTable eventTables = event.getEventTables();
    if ((eventTables != null) && (!eventTables.isEmpty())) {
      for (TransactionEventTable.TableIUD tableIUD : eventTables.values()) {
        remoteTransactionEvent.addTableIUD(tableIUD);
      }
    }
    
    Set<IndexInvalidate> indexInvalidations = event.getIndexInvalidations();
    if (indexInvalidations != null) {
      for (IndexInvalidate indexInvalidate : indexInvalidations) {
        remoteTransactionEvent.addIndexInvalidate(indexInvalidate);
      }
    }
    
    return remoteTransactionEvent;
  }
}
