package com.avaje.ebeaninternal.server.transaction;

import com.avaje.ebeaninternal.api.SpiEbeanServer;
import com.avaje.ebeaninternal.api.TransactionEventTable.TableIUD;
import com.avaje.ebeaninternal.server.cluster.BinaryMessageList;
import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;





















public class RemoteTransactionEvent
  implements Serializable, Runnable
{
  private static final long serialVersionUID = 757920022500956949L;
  private List<BeanPersistIds> beanPersistList = new ArrayList();
  
  private List<TransactionEventTable.TableIUD> tableList;
  
  private List<BeanDeltaList> beanDeltaLists;
  
  private BeanDeltaMap beanDeltaMap;
  
  private List<IndexEvent> indexEventList;
  
  private Set<IndexInvalidate> indexInvalidations;
  
  private DeleteByIdMap deleteByIdMap;
  
  private String serverName;
  private transient SpiEbeanServer server;
  
  public RemoteTransactionEvent(String serverName)
  {
    this.serverName = serverName;
  }
  
  public RemoteTransactionEvent(SpiEbeanServer server) {
    this.server = server;
  }
  
  public void run() {
    server.remoteTransactionEvent(this);
  }
  
  public String toString() {
    StringBuilder sb = new StringBuilder();
    if (beanDeltaMap != null) {
      sb.append(beanDeltaMap);
    }
    sb.append(beanPersistList);
    if (tableList != null) {
      sb.append(tableList);
    }
    return sb.toString();
  }
  
  public void writeBinaryMessage(BinaryMessageList msgList) throws IOException
  {
    if (indexInvalidations != null) {
      for (IndexInvalidate indexInvalidate : indexInvalidations) {
        indexInvalidate.writeBinaryMessage(msgList);
      }
    }
    
    if (tableList != null) {
      for (int i = 0; i < tableList.size(); i++) {
        ((TransactionEventTable.TableIUD)tableList.get(i)).writeBinaryMessage(msgList);
      }
    }
    
    if (deleteByIdMap != null) {
      for (BeanPersistIds deleteIds : deleteByIdMap.values()) {
        deleteIds.writeBinaryMessage(msgList);
      }
    }
    
    if (beanPersistList != null) {
      for (int i = 0; i < beanPersistList.size(); i++) {
        ((BeanPersistIds)beanPersistList.get(i)).writeBinaryMessage(msgList);
      }
    }
    
    if (beanDeltaLists != null) {
      for (int i = 0; i < beanDeltaLists.size(); i++) {
        ((BeanDeltaList)beanDeltaLists.get(i)).writeBinaryMessage(msgList);
      }
    }
    
    if (indexEventList != null) {
      for (int i = 0; i < indexEventList.size(); i++) {
        ((IndexEvent)indexEventList.get(i)).writeBinaryMessage(msgList);
      }
    }
  }
  
  public boolean isEmpty() {
    return (beanPersistList.isEmpty()) && ((tableList == null) || (tableList.isEmpty()));
  }
  
  public void addBeanPersistIds(BeanPersistIds beanPersist) {
    beanPersistList.add(beanPersist);
  }
  
  public void addIndexInvalidate(IndexInvalidate indexInvalidate) {
    if (indexInvalidations == null) {
      indexInvalidations = new HashSet();
    }
    indexInvalidations.add(indexInvalidate);
  }
  
  public void addTableIUD(TransactionEventTable.TableIUD tableIud) {
    if (tableList == null) {
      tableList = new ArrayList(4);
    }
    tableList.add(tableIud);
  }
  
  public void addBeanDeltaList(BeanDeltaList deltaList) {
    if (beanDeltaLists == null) {
      beanDeltaLists = new ArrayList();
    }
    beanDeltaLists.add(deltaList);
  }
  
  public void addBeanDelta(BeanDelta beanDelta) {
    if (beanDeltaMap == null) {
      beanDeltaMap = new BeanDeltaMap();
    }
    beanDeltaMap.addBeanDelta(beanDelta);
  }
  
  public void addIndexEvent(IndexEvent indexEvent) {
    if (indexEventList == null) {
      indexEventList = new ArrayList(2);
    }
    indexEventList.add(indexEvent);
  }
  
  public String getServerName() {
    return serverName;
  }
  
  public SpiEbeanServer getServer() {
    return server;
  }
  
  public void setServer(SpiEbeanServer server) {
    this.server = server;
  }
  
  public DeleteByIdMap getDeleteByIdMap() {
    return deleteByIdMap;
  }
  
  public void setDeleteByIdMap(DeleteByIdMap deleteByIdMap) {
    this.deleteByIdMap = deleteByIdMap;
  }
  
  public Set<IndexInvalidate> getIndexInvalidations() {
    return indexInvalidations;
  }
  
  public List<IndexEvent> getIndexEventList() {
    return indexEventList;
  }
  
  public List<TransactionEventTable.TableIUD> getTableIUDList() {
    return tableList;
  }
  
  public List<BeanPersistIds> getBeanPersistList() {
    return beanPersistList;
  }
  
  public List<BeanDeltaList> getBeanDeltaLists() {
    if (beanDeltaMap != null) {
      beanDeltaLists.addAll(beanDeltaMap.deltaLists());
    }
    return beanDeltaLists;
  }
}
