package com.avaje.ebeaninternal.server.transaction;

import java.lang.ref.Reference;
import java.lang.ref.ReferenceQueue;
import java.lang.ref.WeakReference;
import java.util.HashMap;
import java.util.Map;





























public class WeakValueMap<K, V>
{
  protected final ReferenceQueue<V> refQueue = new ReferenceQueue();
  

  private final Map<K, WeakReferenceWithKey<K, V>> backing;
  


  private static class WeakReferenceWithKey<K, V>
    extends WeakReference<V>
  {
    private final K key;
    

    public WeakReferenceWithKey(K key, V referent, ReferenceQueue<? super V> q)
    {
      super(q);
      this.key = key;
    }
    
    public K getKey() {
      return key;
    }
  }
  
  public WeakValueMap() {
    backing = new HashMap();
  }
  
  private WeakReferenceWithKey<K, V> createReference(K key, V value) {
    return new WeakReferenceWithKey(key, value, refQueue);
  }
  

  private void expunge()
  {
    Reference ref;
    
    while ((ref = refQueue.poll()) != null) {
      backing.remove(((WeakReferenceWithKey)ref).getKey());
    }
  }
  



  public Object putIfAbsent(K key, V value)
  {
    expunge();
    
    Reference<V> ref = (Reference)backing.get(key);
    if (ref != null) {
      V existingValue = ref.get();
      if (existingValue != null)
      {
        return existingValue;
      }
    }
    

    backing.put(key, createReference(key, value));
    return null;
  }
  
  public void put(K key, V value) {
    expunge();
    
    backing.put(key, createReference(key, value));
  }
  
  public V get(K key) {
    expunge();
    
    Reference<V> v = (Reference)backing.get(key);
    return v == null ? null : v.get();
  }
  
  public int size() {
    expunge();
    
    return backing.size();
  }
  
  public boolean isEmpty() {
    expunge();
    
    return backing.isEmpty();
  }
  
  public boolean containsKey(Object key) {
    expunge();
    
    return backing.containsKey(key);
  }
  
  public V remove(K key) {
    expunge();
    
    Reference<V> v = (Reference)backing.remove(key);
    return v == null ? null : v.get();
  }
  
  public void clear() {
    expunge();
    backing.clear();
    expunge();
  }
  
  public String toString() {
    expunge();
    
    return backing.toString();
  }
}
