package com.avaje.ebeaninternal.server.type;

import com.avaje.ebean.config.ScalarTypeConverter;
import com.avaje.ebean.text.json.JsonValueAdapter;
import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.sql.SQLException;


































public class ScalarTypeWrapper<B, S>
  implements ScalarType<B>
{
  private final ScalarType<S> scalarType;
  private final ScalarTypeConverter<B, S> converter;
  private final Class<B> wrapperType;
  private final B nullValue;
  
  public ScalarTypeWrapper(Class<B> wrapperType, ScalarType<S> scalarType, ScalarTypeConverter<B, S> converter)
  {
    this.scalarType = scalarType;
    this.converter = converter;
    nullValue = converter.getNullValue();
    this.wrapperType = wrapperType;
  }
  
  public String toString() {
    return "ScalarTypeWrapper " + wrapperType + " to " + scalarType.getType();
  }
  
  public Object readData(DataInput dataInput) throws IOException
  {
    Object v = scalarType.readData(dataInput);
    return converter.wrapValue(v);
  }
  
  public void writeData(DataOutput dataOutput, Object v) throws IOException
  {
    S sv = converter.unwrapValue(v);
    scalarType.writeData(dataOutput, sv);
  }
  
  public void bind(DataBind b, B value) throws SQLException {
    if (value == null) {
      scalarType.bind(b, null);
    } else {
      S sv = converter.unwrapValue(value);
      scalarType.bind(b, sv);
    }
  }
  
  public int getJdbcType() {
    return scalarType.getJdbcType();
  }
  
  public int getLength() {
    return scalarType.getLength();
  }
  
  public Class<B> getType() {
    return wrapperType;
  }
  
  public boolean isDateTimeCapable() {
    return scalarType.isDateTimeCapable();
  }
  
  public boolean isJdbcNative() {
    return false;
  }
  
  public String format(Object v)
  {
    return formatValue(v);
  }
  
  public String formatValue(B v) {
    S sv = converter.unwrapValue(v);
    return scalarType.formatValue(sv);
  }
  
  public B parse(String value) {
    S sv = scalarType.parse(value);
    if (sv == null) {
      return nullValue;
    }
    return converter.wrapValue(sv);
  }
  
  public B parseDateTime(long systemTimeMillis) {
    S sv = scalarType.parseDateTime(systemTimeMillis);
    if (sv == null) {
      return nullValue;
    }
    return converter.wrapValue(sv);
  }
  
  public void loadIgnore(DataReader dataReader) {
    dataReader.incrementPos(1);
  }
  
  public B read(DataReader dataReader) throws SQLException
  {
    S sv = scalarType.read(dataReader);
    if (sv == null) {
      return nullValue;
    }
    return converter.wrapValue(sv);
  }
  
  public B toBeanType(Object value)
  {
    if (value == null) {
      return nullValue;
    }
    if (getType().isAssignableFrom(value.getClass())) {
      return value;
    }
    if ((value instanceof String)) {
      return parse((String)value);
    }
    S sv = scalarType.toBeanType(value);
    return converter.wrapValue(sv);
  }
  

  public Object toJdbcType(Object value)
  {
    Object sv = converter.unwrapValue(value);
    if (sv == null) {
      return nullValue;
    }
    return scalarType.toJdbcType(sv);
  }
  
  public void accumulateScalarTypes(String propName, CtCompoundTypeScalarList list) {
    list.addScalarType(propName, this);
  }
  
  public ScalarType<?> getScalarType() {
    return this;
  }
  
  public String jsonToString(B value, JsonValueAdapter ctx)
  {
    S sv = converter.unwrapValue(value);
    return scalarType.jsonToString(sv, ctx);
  }
  
  public B jsonFromString(String value, JsonValueAdapter ctx) {
    S s = scalarType.jsonFromString(value, ctx);
    return converter.wrapValue(s);
  }
  
  public Object luceneFromIndexValue(Object value)
  {
    S s = scalarType.luceneFromIndexValue(value);
    return converter.wrapValue(s);
  }
  
  public Object luceneToIndexValue(Object value)
  {
    S sv = converter.unwrapValue(value);
    return scalarType.luceneToIndexValue(sv);
  }
  
  public int getLuceneType() {
    return scalarType.getLuceneType();
  }
}
