package com.google.common.base;

import com.google.common.annotations.GwtCompatible;

























@GwtCompatible
public enum CaseFormat
{
  LOWER_HYPHEN(CharMatcher.is('-'), "-"), 
  



  LOWER_UNDERSCORE(CharMatcher.is('_'), "_"), 
  



  LOWER_CAMEL(CharMatcher.inRange('A', 'Z'), ""), 
  



  UPPER_CAMEL(CharMatcher.inRange('A', 'Z'), ""), 
  



  UPPER_UNDERSCORE(CharMatcher.is('_'), "_");
  
  private final CharMatcher wordBoundary;
  private final String wordSeparator;
  
  private CaseFormat(CharMatcher wordBoundary, String wordSeparator) {
    this.wordBoundary = wordBoundary;
    this.wordSeparator = wordSeparator;
  }
  




  public String to(CaseFormat format, String s)
  {
    if (format == null) {
      throw new NullPointerException();
    }
    if (s == null) {
      throw new NullPointerException();
    }
    
    if (format == this) {
      return s;
    }
    

    switch (1.$SwitchMap$com$google$common$base$CaseFormat[ordinal()]) {
    case 3: 
      switch (1.$SwitchMap$com$google$common$base$CaseFormat[format.ordinal()]) {
      case 1: 
        return s.replace('-', '_');
      case 2: 
        return Ascii.toUpperCase(s.replace('-', '_'));
      }
      break;
    case 1: 
      switch (1.$SwitchMap$com$google$common$base$CaseFormat[format.ordinal()]) {
      case 3: 
        return s.replace('_', '-');
      case 2: 
        return Ascii.toUpperCase(s);
      }
      break;
    case 2: 
      switch (1.$SwitchMap$com$google$common$base$CaseFormat[format.ordinal()]) {
      case 3: 
        return Ascii.toLowerCase(s.replace('_', '-'));
      case 1: 
        return Ascii.toLowerCase(s);
      }
      
      break;
    }
    
    StringBuilder out = null;
    int i = 0;
    int j = -1;
    while ((j = wordBoundary.indexIn(s, ++j)) != -1) {
      if (i == 0)
      {
        out = new StringBuilder(s.length() + 4 * wordSeparator.length());
        out.append(format.normalizeFirstWord(s.substring(i, j)));
      } else {
        out.append(format.normalizeWord(s.substring(i, j)));
      }
      out.append(wordSeparator);
      i = j + wordSeparator.length();
    }
    if (i == 0) {
      return format.normalizeFirstWord(s);
    }
    out.append(format.normalizeWord(s.substring(i)));
    return out.toString();
  }
  
  private String normalizeFirstWord(String word) {
    switch (1.$SwitchMap$com$google$common$base$CaseFormat[ordinal()]) {
    case 4: 
      return Ascii.toLowerCase(word);
    }
    return normalizeWord(word);
  }
  
  private String normalizeWord(String word)
  {
    switch (1.$SwitchMap$com$google$common$base$CaseFormat[ordinal()]) {
    case 3: 
      return Ascii.toLowerCase(word);
    case 1: 
      return Ascii.toLowerCase(word);
    case 4: 
      return firstCharOnlyToUpper(word);
    case 5: 
      return firstCharOnlyToUpper(word);
    case 2: 
      return Ascii.toUpperCase(word);
    }
    throw new RuntimeException("unknown case: " + this);
  }
  
  private static String firstCharOnlyToUpper(String word) {
    int length = word.length();
    if (length == 0) {
      return word;
    }
    return length + Ascii.toUpperCase(word.charAt(0)) + Ascii.toLowerCase(word.substring(1));
  }
}
