package com.google.common.base;

import com.google.common.annotations.GwtCompatible;
import java.util.Arrays;
import javax.annotation.Nullable;








































@GwtCompatible
public final class Objects
{
  public static boolean equal(@Nullable Object a, @Nullable Object b)
  {
    return (a == b) || ((a != null) && (a.equals(b)));
  }
  














  public static int hashCode(@Nullable Object... objects)
  {
    return Arrays.hashCode(objects);
  }
  































  public static ToStringHelper toStringHelper(Object self)
  {
    return new ToStringHelper(simpleName(self.getClass()), null);
  }
  









  public static ToStringHelper toStringHelper(Class<?> clazz)
  {
    return new ToStringHelper(simpleName(clazz), null);
  }
  







  public static ToStringHelper toStringHelper(String className)
  {
    return new ToStringHelper(className, null);
  }
  



  private static String simpleName(Class<?> clazz)
  {
    String name = clazz.getName();
    


    name = name.replaceAll("\\$[0-9]+", "\\$");
    

    int start = name.lastIndexOf('$');
    


    if (start == -1) {
      start = name.lastIndexOf('.');
    }
    return name.substring(start + 1);
  }
  















  public static <T> T firstNonNull(@Nullable T first, @Nullable T second)
  {
    return first != null ? first : Preconditions.checkNotNull(second);
  }
  



  public static final class ToStringHelper
  {
    private final StringBuilder builder;
    

    private boolean needsSeparator = false;
    


    private ToStringHelper(String className)
    {
      Preconditions.checkNotNull(className);
      builder = new StringBuilder(32).append(className).append('{');
    }
    




    public ToStringHelper add(String name, @Nullable Object value)
    {
      Preconditions.checkNotNull(name);
      maybeAppendSeparator().append(name).append('=').append(value);
      return this;
    }
    





    public ToStringHelper addValue(@Nullable Object value)
    {
      maybeAppendSeparator().append(value);
      return this;
    }
    


    public String toString()
    {
      try
      {
        return '}';
      }
      finally
      {
        builder.setLength(builder.length() - 1);
      }
    }
    
    private StringBuilder maybeAppendSeparator() {
      if (needsSeparator) {
        return builder.append(", ");
      }
      needsSeparator = true;
      return builder;
    }
  }
}
