package com.google.common.base.internal;

import java.lang.ref.PhantomReference;
import java.lang.ref.Reference;
import java.lang.ref.ReferenceQueue;
import java.lang.ref.WeakReference;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.logging.Level;
import java.util.logging.Logger;





































public class Finalizer
  extends Thread
{
  private static final Logger logger = Logger.getLogger(Finalizer.class.getName());
  




  private static final String FINALIZABLE_REFERENCE = "com.google.common.base.FinalizableReference";
  




  private final WeakReference<Class<?>> finalizableReferenceClassReference;
  




  private final PhantomReference<Object> frqReference;
  




  public static ReferenceQueue<Object> startFinalizer(Class<?> finalizableReferenceClass, Object frq)
  {
    if (!finalizableReferenceClass.getName().equals("com.google.common.base.FinalizableReference")) {
      throw new IllegalArgumentException("Expected com.google.common.base.FinalizableReference.");
    }
    

    Finalizer finalizer = new Finalizer(finalizableReferenceClass, frq);
    finalizer.start();
    return queue;
  }
  


  private final ReferenceQueue<Object> queue = new ReferenceQueue();
  
  private static final Field inheritableThreadLocals = getInheritableThreadLocalsField();
  

  private Finalizer(Class<?> finalizableReferenceClass, Object frq)
  {
    super(Finalizer.class.getName());
    
    finalizableReferenceClassReference = new WeakReference(finalizableReferenceClass);
    


    frqReference = new PhantomReference(frq, queue);
    
    setDaemon(true);
    try
    {
      if (inheritableThreadLocals != null) {
        inheritableThreadLocals.set(this, null);
      }
    } catch (Throwable t) {
      logger.log(Level.INFO, "Failed to clear thread local values inherited by reference finalizer thread.", t);
    }
  }
  




  public void run()
  {
    try
    {
      try
      {
        for (;;)
        {
          cleanUp(queue.remove());
        }
      }
      catch (InterruptedException e) {}
    }
    catch (ShutDown shutDown) {}
  }
  
  private void cleanUp(Reference<?> reference) throws Finalizer.ShutDown
  {
    Method finalizeReferentMethod = getFinalizeReferentMethod();
    


    do
    {
      reference.clear();
      
      if (reference == frqReference)
      {



        throw new ShutDown(null);
      }
      try
      {
        finalizeReferentMethod.invoke(reference, new Object[0]);
      } catch (Throwable t) {
        logger.log(Level.SEVERE, "Error cleaning up after reference.", t);


      }
      

    }
    while ((reference = queue.poll()) != null);
  }
  

  private Method getFinalizeReferentMethod()
    throws Finalizer.ShutDown
  {
    Class<?> finalizableReferenceClass = (Class)finalizableReferenceClassReference.get();
    
    if (finalizableReferenceClass == null)
    {







      throw new ShutDown(null);
    }
    try {
      return finalizableReferenceClass.getMethod("finalizeReferent", new Class[0]);
    } catch (NoSuchMethodException e) {
      throw new AssertionError(e);
    }
  }
  
  public static Field getInheritableThreadLocalsField() {
    try {
      Field inheritableThreadLocals = Thread.class.getDeclaredField("inheritableThreadLocals");
      
      inheritableThreadLocals.setAccessible(true);
      return inheritableThreadLocals;
    } catch (Throwable t) {
      logger.log(Level.INFO, "Couldn't access Thread.inheritableThreadLocals. Reference finalizer threads will inherit thread local values.");
    }
    
    return null;
  }
  
  private static class ShutDown
    extends Exception
  {}
}
