package com.google.common.cache;

import com.google.common.base.Preconditions;
import com.google.common.base.Supplier;
import java.io.Serializable;
import java.util.concurrent.ConcurrentMap;
import java.util.concurrent.ExecutionException;






















class ComputingCache<K, V>
  extends AbstractCache<K, V>
  implements Serializable
{
  final CustomConcurrentHashMap<K, V> map;
  private static final long serialVersionUID = 1L;
  
  ComputingCache(CacheBuilder<? super K, ? super V> builder, Supplier<? extends AbstractCache.StatsCounter> statsCounterSupplier, CacheLoader<? super K, V> loader)
  {
    map = new CustomConcurrentHashMap(builder, statsCounterSupplier, loader);
  }
  

  public V get(K key)
    throws ExecutionException
  {
    return map.getOrCompute(key);
  }
  
  public void invalidate(Object key)
  {
    Preconditions.checkNotNull(key);
    map.remove(key);
  }
  
  public void invalidateAll()
  {
    map.clear();
  }
  
  public long size()
  {
    return map.longSize();
  }
  
  public ConcurrentMap<K, V> asMap()
  {
    return map;
  }
  
  public CacheStats stats()
  {
    AbstractCache.SimpleStatsCounter aggregator = new AbstractCache.SimpleStatsCounter();
    for (CustomConcurrentHashMap.Segment<K, V> segment : map.segments) {
      aggregator.incrementBy(statsCounter);
    }
    return aggregator.snapshot();
  }
  
  public void cleanUp()
  {
    map.cleanUp();
  }
  



  Object writeReplace()
  {
    return map.cacheSerializationProxy();
  }
}
