package com.google.common.collect;

import com.google.common.annotations.GwtCompatible;
import com.google.common.base.Optional;
import com.google.common.base.Preconditions;
import java.util.NoSuchElementException;
import javax.annotation.Nullable;



















@GwtCompatible
final class BstInOrderPath<N extends BstNode<?, N>>
  extends BstPath<N, BstInOrderPath<N>>
{
  private final BstSide sideExtension;
  private transient Optional<BstInOrderPath<N>> prevInOrder;
  private transient Optional<BstInOrderPath<N>> nextInOrder;
  
  public static <N extends BstNode<?, N>> BstPathFactory<N, BstInOrderPath<N>> inOrderFactory()
  {
    new BstPathFactory()
    {
      public BstInOrderPath<N> extension(BstInOrderPath<N> path, BstSide side) {
        return BstInOrderPath.extension(path, side);
      }
      
      public BstInOrderPath<N> initialPath(N root)
      {
        return new BstInOrderPath(root, null, null, null);
      }
    };
  }
  
  private static <N extends BstNode<?, N>> BstInOrderPath<N> extension(BstInOrderPath<N> path, BstSide side)
  {
    Preconditions.checkNotNull(path);
    N tip = path.getTip();
    return new BstInOrderPath(tip.getChild(side), side, path);
  }
  




  private BstInOrderPath(N tip, @Nullable BstSide sideExtension, @Nullable BstInOrderPath<N> tail)
  {
    super(tip, tail);
    this.sideExtension = sideExtension;
    if (!$assertionsDisabled) if ((sideExtension == null ? 1 : 0) != (tail == null ? 1 : 0)) throw new AssertionError();
  }
  
  private Optional<BstInOrderPath<N>> computeNextInOrder(BstSide side) {
    if (getTip().hasChild(side)) {
      BstInOrderPath<N> path = extension(this, side);
      BstSide otherSide = side.other();
      while (path.getTip().hasChild(otherSide)) {
        path = extension(path, otherSide);
      }
      return Optional.of(path);
    }
    BstInOrderPath<N> current = this;
    while (sideExtension == side) {
      current = (BstInOrderPath)current.getPrefix();
    }
    current = (BstInOrderPath)current.prefixOrNull();
    return Optional.fromNullable(current);
  }
  
  private Optional<BstInOrderPath<N>> nextInOrder(BstSide side)
  {
    Optional<BstInOrderPath<N>> result;
    switch (2.$SwitchMap$com$google$common$collect$BstSide[side.ordinal()]) {
    case 1: 
      result = prevInOrder;
      return result == null ? (this.prevInOrder = computeNextInOrder(side)) : result;
    case 2: 
      result = nextInOrder;
      return result == null ? (this.nextInOrder = computeNextInOrder(side)) : result;
    }
    throw new AssertionError();
  }
  



  public boolean hasNext(BstSide side)
  {
    return nextInOrder(side).isPresent();
  }
  




  public BstInOrderPath<N> next(BstSide side)
  {
    if (!hasNext(side)) {
      throw new NoSuchElementException();
    }
    return (BstInOrderPath)nextInOrder(side).get();
  }
  



  public BstSide getSideOfExtension()
  {
    return sideExtension;
  }
}
