package com.google.common.collect;

import com.google.common.annotations.GwtCompatible;
import com.google.common.base.Preconditions;
import javax.annotation.Nullable;
































@GwtCompatible
final class BstMutationResult<K, N extends BstNode<K, N>>
{
  private final K targetKey;
  @Nullable
  private N originalRoot;
  @Nullable
  private N changedRoot;
  private final BstModificationResult<N> modificationResult;
  
  public static <K, N extends BstNode<K, N>> BstMutationResult<K, N> mutationResult(K targetKey, @Nullable N originalRoot, @Nullable N changedRoot, BstModificationResult<N> modificationResult)
  {
    return new BstMutationResult(targetKey, originalRoot, changedRoot, modificationResult);
  }
  










  private BstMutationResult(K targetKey, @Nullable N originalRoot, @Nullable N changedRoot, BstModificationResult<N> modificationResult)
  {
    this.targetKey = Preconditions.checkNotNull(targetKey);
    this.originalRoot = originalRoot;
    this.changedRoot = changedRoot;
    this.modificationResult = ((BstModificationResult)Preconditions.checkNotNull(modificationResult));
  }
  


  public K getTargetKey()
  {
    return targetKey;
  }
  


  @Nullable
  public N getOriginalRoot()
  {
    return originalRoot;
  }
  


  @Nullable
  public N getChangedRoot()
  {
    return changedRoot;
  }
  




  @Nullable
  public N getOriginalTarget()
  {
    return modificationResult.getOriginalTarget();
  }
  




  @Nullable
  public N getChangedTarget()
  {
    return modificationResult.getChangedTarget();
  }
  
  BstModificationResult.ModificationType modificationType() {
    return modificationResult.getType();
  }
  





  public BstMutationResult<K, N> lift(N liftOriginalRoot, BstSide side, BstNodeFactory<N> nodeFactory, BstBalancePolicy<N> balancePolicy)
  {
    if (!$assertionsDisabled) if (((liftOriginalRoot != null ? 1 : 0) & (side != null ? 1 : 0) & (nodeFactory != null ? 1 : 0) & (balancePolicy != null ? 1 : 0)) == 0) throw new AssertionError();
    switch (1.$SwitchMap$com$google$common$collect$BstModificationResult$ModificationType[modificationType().ordinal()]) {
    case 1: 
      originalRoot = (this.changedRoot = liftOriginalRoot);
      return this;
    case 2: 
    case 3: 
      originalRoot = liftOriginalRoot;
      N resultLeft = liftOriginalRoot.childOrNull(BstSide.LEFT);
      N resultRight = liftOriginalRoot.childOrNull(BstSide.RIGHT);
      switch (side) {
      case LEFT: 
        resultLeft = changedRoot;
        break;
      case RIGHT: 
        resultRight = changedRoot;
        break;
      default: 
        throw new AssertionError();
      }
      if (modificationType() == BstModificationResult.ModificationType.REBUILDING_CHANGE) {
        changedRoot = nodeFactory.createNode(liftOriginalRoot, resultLeft, resultRight);
      } else {
        changedRoot = balancePolicy.balance(nodeFactory, liftOriginalRoot, resultLeft, resultRight);
      }
      
      return this;
    }
    throw new AssertionError();
  }
}
