package com.google.common.collect;

import com.google.common.annotations.GwtCompatible;
import com.google.common.base.Preconditions;
import java.util.Comparator;
import javax.annotation.Nullable;














































@GwtCompatible
class BstNode<K, N extends BstNode<K, N>>
{
  private final K key;
  @Nullable
  private final N left;
  @Nullable
  private final N right;
  
  BstNode(K key, @Nullable N left, @Nullable N right)
  {
    this.key = Preconditions.checkNotNull(key);
    this.left = left;
    this.right = right;
  }
  


  public final K getKey()
  {
    return key;
  }
  


  @Nullable
  public final N childOrNull(BstSide side)
  {
    switch (1.$SwitchMap$com$google$common$collect$BstSide[side.ordinal()]) {
    case 1: 
      return left;
    case 2: 
      return right;
    }
    throw new AssertionError();
  }
  



  public final boolean hasChild(BstSide side)
  {
    return childOrNull(side) != null;
  }
  




  public final N getChild(BstSide side)
  {
    N child = childOrNull(side);
    Preconditions.checkState(child != null);
    return child;
  }
  



  protected final boolean orderingInvariantHolds(Comparator<? super K> comparator)
  {
    Preconditions.checkNotNull(comparator);
    boolean result = true;
    if (hasChild(BstSide.LEFT)) {
      result &= comparator.compare(getChild(BstSide.LEFT).getKey(), key) < 0;
    }
    if (hasChild(BstSide.RIGHT)) {
      result &= comparator.compare(getChild(BstSide.RIGHT).getKey(), key) > 0;
    }
    return result;
  }
}
