package com.google.common.collect;

import javax.annotation.Nullable;

















final class DescendingImmutableSortedMultiset<E>
  extends ImmutableSortedMultiset<E>
{
  private final transient ImmutableSortedMultiset<E> forward;
  
  DescendingImmutableSortedMultiset(ImmutableSortedMultiset<E> forward)
  {
    super(forward.reverseComparator());
    this.forward = forward;
  }
  
  public int count(@Nullable Object element)
  {
    return forward.count(element);
  }
  
  public Multiset.Entry<E> firstEntry()
  {
    return forward.lastEntry();
  }
  
  public Multiset.Entry<E> lastEntry()
  {
    return forward.firstEntry();
  }
  
  public int size()
  {
    return forward.size();
  }
  
  ImmutableSortedSet<E> createElementSet()
  {
    return forward.createDescendingElementSet();
  }
  
  ImmutableSortedSet<E> createDescendingElementSet()
  {
    return forward.elementSet();
  }
  
  UnmodifiableIterator<Multiset.Entry<E>> descendingEntryIterator()
  {
    return forward.entryIterator();
  }
  
  public ImmutableSortedMultiset<E> descendingMultiset()
  {
    return forward;
  }
  
  public ImmutableSortedMultiset<E> headMultiset(E upperBound, BoundType boundType)
  {
    return forward.tailMultiset(upperBound, boundType).descendingMultiset();
  }
  
  public ImmutableSortedMultiset<E> tailMultiset(E lowerBound, BoundType boundType)
  {
    return forward.headMultiset(lowerBound, boundType).descendingMultiset();
  }
  
  UnmodifiableIterator<Multiset.Entry<E>> entryIterator()
  {
    return forward.descendingEntryIterator();
  }
  
  int distinctElements()
  {
    return forward.distinctElements();
  }
  
  boolean isPartialView()
  {
    return forward.isPartialView();
  }
}
