package com.google.common.collect;

import com.google.common.annotations.Beta;
import com.google.common.annotations.GwtCompatible;
import java.io.Serializable;
import java.math.BigInteger;



























@GwtCompatible
@Beta
public final class DiscreteDomains
{
  public static DiscreteDomain<Integer> integers()
  {
    return IntegerDomain.INSTANCE;
  }
  
  private static final class IntegerDomain extends DiscreteDomain<Integer> implements Serializable
  {
    private static final IntegerDomain INSTANCE = new IntegerDomain();
    private static final long serialVersionUID = 0L;
    
    public Integer next(Integer value) { int i = value.intValue();
      return i == 2147483647 ? null : Integer.valueOf(i + 1);
    }
    
    public Integer previous(Integer value) {
      int i = value.intValue();
      return i == -2147483648 ? null : Integer.valueOf(i - 1);
    }
    
    public long distance(Integer start, Integer end) {
      return end.intValue() - start.intValue();
    }
    
    public Integer minValue() {
      return Integer.valueOf(-2147483648);
    }
    
    public Integer maxValue() {
      return Integer.valueOf(2147483647);
    }
    
    private Object readResolve() {
      return INSTANCE;
    }
  }
  




  public static DiscreteDomain<Long> longs()
  {
    return LongDomain.INSTANCE;
  }
  
  private static final class LongDomain extends DiscreteDomain<Long> implements Serializable
  {
    private static final LongDomain INSTANCE = new LongDomain();
    private static final long serialVersionUID = 0L;
    
    public Long next(Long value) { long l = value.longValue();
      return l == 9223372036854775807L ? null : Long.valueOf(l + 1L);
    }
    
    public Long previous(Long value) {
      long l = value.longValue();
      return l == -9223372036854775808L ? null : Long.valueOf(l - 1L);
    }
    
    public long distance(Long start, Long end) {
      long result = end.longValue() - start.longValue();
      if ((end.longValue() > start.longValue()) && (result < 0L)) {
        return 9223372036854775807L;
      }
      if ((end.longValue() < start.longValue()) && (result > 0L)) {
        return -9223372036854775808L;
      }
      return result;
    }
    
    public Long minValue() {
      return Long.valueOf(-9223372036854775808L);
    }
    
    public Long maxValue() {
      return Long.valueOf(9223372036854775807L);
    }
    
    private Object readResolve() {
      return INSTANCE;
    }
  }
  





  static DiscreteDomain<BigInteger> bigIntegers()
  {
    return BigIntegerDomain.INSTANCE;
  }
  
  private static final class BigIntegerDomain extends DiscreteDomain<BigInteger> implements Serializable
  {
    private static final BigIntegerDomain INSTANCE = new BigIntegerDomain();
    
    private static final BigInteger MIN_LONG = BigInteger.valueOf(-9223372036854775808L);
    
    private static final BigInteger MAX_LONG = BigInteger.valueOf(9223372036854775807L);
    private static final long serialVersionUID = 0L;
    
    public BigInteger next(BigInteger value) {
      return value.add(BigInteger.ONE);
    }
    
    public BigInteger previous(BigInteger value) {
      return value.subtract(BigInteger.ONE);
    }
    
    public long distance(BigInteger start, BigInteger end) {
      return start.subtract(end).max(MIN_LONG).min(MAX_LONG).longValue();
    }
    
    private Object readResolve() {
      return INSTANCE;
    }
  }
}
