package com.google.common.collect;

import com.google.common.annotations.Beta;
import com.google.common.annotations.GwtCompatible;
import com.google.common.base.Objects;
import java.util.Collection;
import java.util.Iterator;
import javax.annotation.Nullable;










































@GwtCompatible
public abstract class ForwardingCollection<E>
  extends ForwardingObject
  implements Collection<E>
{
  protected abstract Collection<E> delegate();
  
  public Iterator<E> iterator()
  {
    return delegate().iterator();
  }
  
  public int size()
  {
    return delegate().size();
  }
  
  public boolean removeAll(Collection<?> collection)
  {
    return delegate().removeAll(collection);
  }
  
  public boolean isEmpty()
  {
    return delegate().isEmpty();
  }
  
  public boolean contains(Object object)
  {
    return delegate().contains(object);
  }
  
  public boolean add(E element)
  {
    return delegate().add(element);
  }
  
  public boolean remove(Object object)
  {
    return delegate().remove(object);
  }
  
  public boolean containsAll(Collection<?> collection)
  {
    return delegate().containsAll(collection);
  }
  
  public boolean addAll(Collection<? extends E> collection)
  {
    return delegate().addAll(collection);
  }
  
  public boolean retainAll(Collection<?> collection)
  {
    return delegate().retainAll(collection);
  }
  
  public void clear()
  {
    delegate().clear();
  }
  
  public Object[] toArray()
  {
    return delegate().toArray();
  }
  
  public <T> T[] toArray(T[] array)
  {
    return delegate().toArray(array);
  }
  





  @Beta
  protected boolean standardContains(@Nullable Object object)
  {
    return Iterators.contains(iterator(), object);
  }
  





  @Beta
  protected boolean standardContainsAll(Collection<?> collection)
  {
    for (Object o : collection) {
      if (!contains(o)) {
        return false;
      }
    }
    return true;
  }
  





  @Beta
  protected boolean standardAddAll(Collection<? extends E> collection)
  {
    return Iterators.addAll(this, collection.iterator());
  }
  






  @Beta
  protected boolean standardRemove(@Nullable Object object)
  {
    Iterator<E> iterator = iterator();
    while (iterator.hasNext()) {
      if (Objects.equal(iterator.next(), object)) {
        iterator.remove();
        return true;
      }
    }
    return false;
  }
  






  @Beta
  protected boolean standardRemoveAll(Collection<?> collection)
  {
    return Iterators.removeAll(iterator(), collection);
  }
  






  @Beta
  protected boolean standardRetainAll(Collection<?> collection)
  {
    return Iterators.retainAll(iterator(), collection);
  }
  






  @Beta
  protected void standardClear()
  {
    Iterator<E> iterator = iterator();
    while (iterator.hasNext()) {
      iterator.next();
      iterator.remove();
    }
  }
  






  @Beta
  protected boolean standardIsEmpty()
  {
    return !iterator().hasNext();
  }
  





  @Beta
  protected String standardToString()
  {
    return Collections2.toStringImpl(this);
  }
  





  @Beta
  protected Object[] standardToArray()
  {
    Object[] newArray = new Object[size()];
    return toArray(newArray);
  }
  





  @Beta
  protected <T> T[] standardToArray(T[] array)
  {
    return ObjectArrays.toArrayImpl(this, array);
  }
}
