package com.google.common.collect;

import com.google.common.base.Preconditions;
import java.util.Comparator;
import javax.annotation.Nullable;





















final class ImmutableSortedAsList<E>
  extends ImmutableList<E>
  implements SortedIterable<E>
{
  private final transient ImmutableSortedSet<E> backingSet;
  private final transient ImmutableList<E> backingList;
  
  ImmutableSortedAsList(ImmutableSortedSet<E> backingSet, ImmutableList<E> backingList)
  {
    this.backingSet = backingSet;
    this.backingList = backingList;
  }
  
  public Comparator<? super E> comparator() {
    return backingSet.comparator();
  }
  


  public boolean contains(@Nullable Object target)
  {
    return backingSet.indexOf(target) >= 0;
  }
  
  public int indexOf(@Nullable Object target) {
    return backingSet.indexOf(target);
  }
  
  public int lastIndexOf(@Nullable Object target) {
    return backingSet.indexOf(target);
  }
  

  public ImmutableList<E> subList(int fromIndex, int toIndex)
  {
    Preconditions.checkPositionIndexes(fromIndex, toIndex, size());
    return fromIndex == toIndex ? ImmutableList.of() : new RegularImmutableSortedSet(backingList.subList(fromIndex, toIndex), backingSet.comparator()).asList();
  }
  



  Object writeReplace()
  {
    return new ImmutableAsList.SerializedForm(backingSet);
  }
  
  public UnmodifiableIterator<E> iterator() {
    return backingList.iterator();
  }
  
  public E get(int index) {
    return backingList.get(index);
  }
  
  public UnmodifiableListIterator<E> listIterator() {
    return backingList.listIterator();
  }
  
  public UnmodifiableListIterator<E> listIterator(int index) {
    return backingList.listIterator(index);
  }
  
  public int size() {
    return backingList.size();
  }
  
  public boolean equals(@Nullable Object obj) {
    return backingList.equals(obj);
  }
  
  public int hashCode() {
    return backingList.hashCode();
  }
  
  boolean isPartialView() {
    return backingList.isPartialView();
  }
}
