package com.google.common.collect;

import com.google.common.annotations.GwtCompatible;
import com.google.common.base.Preconditions;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.NoSuchElementException;
import java.util.Set;
import java.util.SortedMap;
import javax.annotation.Nullable;

















































@GwtCompatible(serializable=true, emulated=true)
public class ImmutableSortedMap<K, V>
  extends ImmutableSortedMapFauxverideShim<K, V>
  implements SortedMap<K, V>
{
  private static final Comparator<Comparable> NATURAL_ORDER = ;
  


  private static final ImmutableSortedMap<Comparable, Object> NATURAL_EMPTY_MAP = new ImmutableSortedMap(ImmutableList.of(), NATURAL_ORDER);
  
  final transient ImmutableList<Map.Entry<K, V>> entries;
  private final transient Comparator<? super K> comparator;
  private transient ImmutableSet<Map.Entry<K, V>> entrySet;
  private transient ImmutableSortedSet<K> keySet;
  private transient ImmutableCollection<V> values;
  private static final long serialVersionUID = 0L;
  
  public static <K, V> ImmutableSortedMap<K, V> of()
  {
    return NATURAL_EMPTY_MAP;
  }
  

  private static <K, V> ImmutableSortedMap<K, V> emptyMap(Comparator<? super K> comparator)
  {
    if (NATURAL_ORDER.equals(comparator)) {
      return NATURAL_EMPTY_MAP;
    }
    return new ImmutableSortedMap(ImmutableList.of(), comparator);
  }
  





  public static <K extends Comparable<? super K>, V> ImmutableSortedMap<K, V> of(K k1, V v1)
  {
    return new ImmutableSortedMap(ImmutableList.of(entryOf(k1, v1)), Ordering.natural());
  }
  








  public static <K extends Comparable<? super K>, V> ImmutableSortedMap<K, V> of(K k1, V v1, K k2, V v2)
  {
    return new Builder(Ordering.natural()).put(k1, v1).put(k2, v2).build();
  }
  








  public static <K extends Comparable<? super K>, V> ImmutableSortedMap<K, V> of(K k1, V v1, K k2, V v2, K k3, V v3)
  {
    return new Builder(Ordering.natural()).put(k1, v1).put(k2, v2).put(k3, v3).build();
  }
  








  public static <K extends Comparable<? super K>, V> ImmutableSortedMap<K, V> of(K k1, V v1, K k2, V v2, K k3, V v3, K k4, V v4)
  {
    return new Builder(Ordering.natural()).put(k1, v1).put(k2, v2).put(k3, v3).put(k4, v4).build();
  }
  








  public static <K extends Comparable<? super K>, V> ImmutableSortedMap<K, V> of(K k1, V v1, K k2, V v2, K k3, V v3, K k4, V v4, K k5, V v5)
  {
    return new Builder(Ordering.natural()).put(k1, v1).put(k2, v2).put(k3, v3).put(k4, v4).put(k5, v5).build();
  }
  





















  public static <K, V> ImmutableSortedMap<K, V> copyOf(Map<? extends K, ? extends V> map)
  {
    Ordering<K> naturalOrder = Ordering.natural();
    return copyOfInternal(map, naturalOrder);
  }
  












  public static <K, V> ImmutableSortedMap<K, V> copyOf(Map<? extends K, ? extends V> map, Comparator<? super K> comparator)
  {
    return copyOfInternal(map, (Comparator)Preconditions.checkNotNull(comparator));
  }
  











  public static <K, V> ImmutableSortedMap<K, V> copyOfSorted(SortedMap<K, ? extends V> map)
  {
    Comparator<? super K> comparator = map.comparator();
    if (comparator == null)
    {

      comparator = NATURAL_ORDER;
    }
    return copyOfInternal(map, comparator);
  }
  
  private static <K, V> ImmutableSortedMap<K, V> copyOfInternal(Map<? extends K, ? extends V> map, Comparator<? super K> comparator)
  {
    boolean sameComparator = false;
    if ((map instanceof SortedMap)) {
      SortedMap<?, ?> sortedMap = (SortedMap)map;
      Comparator<?> comparator2 = sortedMap.comparator();
      sameComparator = comparator2 == null ? false : comparator == NATURAL_ORDER ? true : comparator.equals(comparator2);
    }
    


    if ((sameComparator) && ((map instanceof ImmutableSortedMap)))
    {


      ImmutableSortedMap<K, V> kvMap = (ImmutableSortedMap)map;
      if (!kvMap.isPartialView()) {
        return kvMap;
      }
    }
    




    Map.Entry<K, V>[] entries = (Map.Entry[])map.entrySet().toArray(new Map.Entry[0]);
    
    for (int i = 0; i < entries.length; i++) {
      Map.Entry<K, V> entry = entries[i];
      entries[i] = entryOf(entry.getKey(), entry.getValue());
    }
    
    List<Map.Entry<K, V>> list = Arrays.asList(entries);
    
    if (!sameComparator) {
      sortEntries(list, comparator);
      validateEntries(list, comparator);
    }
    
    return new ImmutableSortedMap(ImmutableList.copyOf(list), comparator);
  }
  
  private static <K, V> void sortEntries(List<Map.Entry<K, V>> entries, Comparator<? super K> comparator)
  {
    Comparator<Map.Entry<K, V>> entryComparator = new Comparator()
    {
      public int compare(Map.Entry<K, V> entry1, Map.Entry<K, V> entry2) {
        return val$comparator.compare(entry1.getKey(), entry2.getKey());
      }
      
    };
    Collections.sort(entries, entryComparator);
  }
  
  private static <K, V> void validateEntries(List<Map.Entry<K, V>> entries, Comparator<? super K> comparator)
  {
    for (int i = 1; i < entries.size(); i++) {
      if (comparator.compare(((Map.Entry)entries.get(i - 1)).getKey(), ((Map.Entry)entries.get(i)).getKey()) == 0)
      {
        throw new IllegalArgumentException("Duplicate keys in mappings " + entries.get(i - 1) + " and " + entries.get(i));
      }
    }
  }
  











  public static <K extends Comparable<K>, V> Builder<K, V> naturalOrder()
  {
    return new Builder(Ordering.natural());
  }
  







  public static <K, V> Builder<K, V> orderedBy(Comparator<K> comparator)
  {
    return new Builder(comparator);
  }
  








  public static <K extends Comparable<K>, V> Builder<K, V> reverseOrder()
  {
    return new Builder(Ordering.natural().reverse());
  }
  











  public static class Builder<K, V>
    extends ImmutableMap.Builder<K, V>
  {
    private final Comparator<? super K> comparator;
    










    public Builder(Comparator<? super K> comparator)
    {
      this.comparator = ((Comparator)Preconditions.checkNotNull(comparator));
    }
    




    public Builder<K, V> put(K key, V value)
    {
      entries.add(ImmutableMap.entryOf(key, value));
      return this;
    }
    






    public Builder<K, V> putAll(Map<? extends K, ? extends V> map)
    {
      for (Map.Entry<? extends K, ? extends V> entry : map.entrySet()) {
        put(entry.getKey(), entry.getValue());
      }
      return this;
    }
    





    public ImmutableSortedMap<K, V> build()
    {
      ImmutableSortedMap.sortEntries(entries, comparator);
      ImmutableSortedMap.validateEntries(entries, comparator);
      return new ImmutableSortedMap(ImmutableList.copyOf(entries), comparator);
    }
  }
  




  ImmutableSortedMap(ImmutableList<Map.Entry<K, V>> entries, Comparator<? super K> comparator)
  {
    this.entries = entries;
    this.comparator = comparator;
  }
  
  public int size()
  {
    return entries.size();
  }
  



  Comparator<Object> unsafeComparator()
  {
    return comparator;
  }
  
  public V get(@Nullable Object key) {
    if (key == null) {
      return null;
    }
    int i;
    try {
      i = index(key, SortedLists.KeyPresentBehavior.ANY_PRESENT, SortedLists.KeyAbsentBehavior.INVERTED_INSERTION_INDEX);
    } catch (ClassCastException e) {
      return null;
    }
    return i >= 0 ? ((Map.Entry)entries.get(i)).getValue() : null;
  }
  
  public boolean containsValue(@Nullable Object value) {
    if (value == null) {
      return false;
    }
    return Iterators.contains(valueIterator(), value);
  }
  
  boolean isPartialView() {
    return entries.isPartialView();
  }
  





  public ImmutableSet<Map.Entry<K, V>> entrySet()
  {
    ImmutableSet<Map.Entry<K, V>> es = entrySet;
    return es == null ? (this.entrySet = createEntrySet()) : es;
  }
  
  private ImmutableSet<Map.Entry<K, V>> createEntrySet() {
    return isEmpty() ? ImmutableSet.of() : new EntrySet(this);
  }
  
  private static class EntrySet<K, V> extends ImmutableSet<Map.Entry<K, V>>
  {
    final transient ImmutableSortedMap<K, V> map;
    
    EntrySet(ImmutableSortedMap<K, V> map)
    {
      this.map = map;
    }
    
    boolean isPartialView() {
      return map.isPartialView();
    }
    
    public int size()
    {
      return map.size();
    }
    
    public UnmodifiableIterator<Map.Entry<K, V>> iterator() {
      return map.entries.iterator();
    }
    
    public boolean contains(Object target) {
      if ((target instanceof Map.Entry)) {
        Map.Entry<?, ?> entry = (Map.Entry)target;
        V mappedValue = map.get(entry.getKey());
        return (mappedValue != null) && (mappedValue.equals(entry.getValue()));
      }
      return false;
    }
    

    Object writeReplace() { return new ImmutableSortedMap.EntrySetSerializedForm(map); }
  }
  
  private static class EntrySetSerializedForm<K, V> implements Serializable {
    final ImmutableSortedMap<K, V> map;
    private static final long serialVersionUID = 0L;
    
    EntrySetSerializedForm(ImmutableSortedMap<K, V> map) { this.map = map; }
    
    Object readResolve() {
      return map.entrySet();
    }
  }
  





  public ImmutableSortedSet<K> keySet()
  {
    ImmutableSortedSet<K> ks = keySet;
    return ks == null ? (this.keySet = createKeySet()) : ks;
  }
  
  private ImmutableSortedSet<K> createKeySet()
  {
    if (isEmpty()) {
      return ImmutableSortedSet.emptySet(comparator);
    }
    
    new RegularImmutableSortedSet(new TransformedImmutableList(entries)
    {

      K transform(Map.Entry<K, V> entry) {
        return entry.getKey(); } }, comparator);
  }
  







  public ImmutableCollection<V> values()
  {
    ImmutableCollection<V> v = values;
    return v == null ? (this.values = new Values(this)) : v;
  }
  
  UnmodifiableIterator<V> valueIterator() {
    final UnmodifiableIterator<Map.Entry<K, V>> entryIterator = entries.iterator();
    new UnmodifiableIterator()
    {
      public boolean hasNext() {
        return entryIterator.hasNext();
      }
      
      public V next() {
        return ((Map.Entry)entryIterator.next()).getValue();
      }
    };
  }
  
  private static class Values<V> extends ImmutableCollection<V>
  {
    private final ImmutableSortedMap<?, V> map;
    
    Values(ImmutableSortedMap<?, V> map) {
      this.map = map;
    }
    
    public int size()
    {
      return map.size();
    }
    
    public UnmodifiableIterator<V> iterator() {
      return map.valueIterator();
    }
    
    public boolean contains(Object target) {
      return map.containsValue(target);
    }
    
    boolean isPartialView() {
      return true;
    }
    

    Object writeReplace() { return new ImmutableSortedMap.ValuesSerializedForm(map); }
  }
  
  private static class ValuesSerializedForm<V> implements Serializable {
    final ImmutableSortedMap<?, V> map;
    private static final long serialVersionUID = 0L;
    
    ValuesSerializedForm(ImmutableSortedMap<?, V> map) { this.map = map; }
    
    Object readResolve() {
      return map.values();
    }
  }
  







  public Comparator<? super K> comparator()
  {
    return comparator;
  }
  
  public K firstKey()
  {
    if (isEmpty()) {
      throw new NoSuchElementException();
    }
    return ((Map.Entry)entries.get(0)).getKey();
  }
  
  public K lastKey()
  {
    if (isEmpty()) {
      throw new NoSuchElementException();
    }
    return ((Map.Entry)entries.get(size() - 1)).getKey();
  }
  












  public ImmutableSortedMap<K, V> headMap(K toKey) { return headMap(toKey, false); }
  
  ImmutableSortedMap<K, V> headMap(K toKey, boolean inclusive) {
    int index;
    int index;
    if (inclusive) {
      index = index(toKey, SortedLists.KeyPresentBehavior.ANY_PRESENT, SortedLists.KeyAbsentBehavior.NEXT_LOWER) + 1;
    } else {
      index = index(toKey, SortedLists.KeyPresentBehavior.ANY_PRESENT, SortedLists.KeyAbsentBehavior.NEXT_HIGHER);
    }
    return createSubmap(0, index);
  }
  













  public ImmutableSortedMap<K, V> subMap(K fromKey, K toKey)
  {
    return subMap(fromKey, true, toKey, false);
  }
  
  ImmutableSortedMap<K, V> subMap(K fromKey, boolean fromInclusive, K toKey, boolean toInclusive)
  {
    Preconditions.checkNotNull(fromKey);
    Preconditions.checkNotNull(toKey);
    Preconditions.checkArgument(comparator.compare(fromKey, toKey) <= 0);
    return tailMap(fromKey, fromInclusive).headMap(toKey, toInclusive);
  }
  












  public ImmutableSortedMap<K, V> tailMap(K fromKey) { return tailMap(fromKey, true); }
  
  ImmutableSortedMap<K, V> tailMap(K fromKey, boolean inclusive) {
    int index;
    int index;
    if (inclusive) {
      index = index(fromKey, SortedLists.KeyPresentBehavior.ANY_PRESENT, SortedLists.KeyAbsentBehavior.NEXT_HIGHER);
    } else {
      index = index(fromKey, SortedLists.KeyPresentBehavior.ANY_PRESENT, SortedLists.KeyAbsentBehavior.NEXT_LOWER) + 1;
    }
    return createSubmap(index, size());
  }
  
  private ImmutableList<K> keyList() {
    new TransformedImmutableList(entries)
    {
      K transform(Map.Entry<K, V> entry) {
        return entry.getKey();
      }
    };
  }
  
  private int index(Object key, SortedLists.KeyPresentBehavior presentBehavior, SortedLists.KeyAbsentBehavior absentBehavior)
  {
    return SortedLists.binarySearch(keyList(), Preconditions.checkNotNull(key), unsafeComparator(), presentBehavior, absentBehavior);
  }
  

  private ImmutableSortedMap<K, V> createSubmap(int newFromIndex, int newToIndex)
  {
    if (newFromIndex < newToIndex) {
      return new ImmutableSortedMap(entries.subList(newFromIndex, newToIndex), comparator);
    }
    
    return emptyMap(comparator);
  }
  

  private static class SerializedForm
    extends ImmutableMap.SerializedForm
  {
    private final Comparator<Object> comparator;
    
    private static final long serialVersionUID = 0L;
    

    SerializedForm(ImmutableSortedMap<?, ?> sortedMap)
    {
      super();
      comparator = sortedMap.comparator();
    }
    
    Object readResolve() { ImmutableSortedMap.Builder<Object, Object> builder = new ImmutableSortedMap.Builder(comparator);
      return createMap(builder);
    }
  }
  
  Object writeReplace()
  {
    return new SerializedForm(this);
  }
}
