package com.google.common.collect;

import com.google.common.annotations.GwtIncompatible;
import com.google.common.base.Preconditions;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import java.util.Set;




























































@GwtIncompatible("hasn't been tested yet")
abstract class ImmutableSortedMultiset<E>
  extends ImmutableSortedMultisetFauxverideShim<E>
  implements SortedMultiset<E>
{
  private static final Comparator<Comparable> NATURAL_ORDER = ;
  
  private static final ImmutableSortedMultiset<Comparable> NATURAL_EMPTY_MULTISET = new EmptyImmutableSortedMultiset(NATURAL_ORDER);
  private final transient Comparator<? super E> comparator;
  private transient Comparator<? super E> reverseComparator;
  private transient ImmutableSortedSet<E> elementSet;
  transient ImmutableSortedMultiset<E> descendingMultiset;
  
  public static <E> ImmutableSortedMultiset<E> of()
  {
    return NATURAL_EMPTY_MULTISET;
  }
  


  public static <E extends Comparable<? super E>> ImmutableSortedMultiset<E> of(E element)
  {
    return RegularImmutableSortedMultiset.createFromSorted(NATURAL_ORDER, ImmutableList.of(Multisets.immutableEntry(Preconditions.checkNotNull(element), 1)));
  }
  







  public static <E extends Comparable<? super E>> ImmutableSortedMultiset<E> of(E e1, E e2)
  {
    return copyOf(Ordering.natural(), Arrays.asList(new Comparable[] { e1, e2 }));
  }
  






  public static <E extends Comparable<? super E>> ImmutableSortedMultiset<E> of(E e1, E e2, E e3)
  {
    return copyOf(Ordering.natural(), Arrays.asList(new Comparable[] { e1, e2, e3 }));
  }
  







  public static <E extends Comparable<? super E>> ImmutableSortedMultiset<E> of(E e1, E e2, E e3, E e4)
  {
    return copyOf(Ordering.natural(), Arrays.asList(new Comparable[] { e1, e2, e3, e4 }));
  }
  







  public static <E extends Comparable<? super E>> ImmutableSortedMultiset<E> of(E e1, E e2, E e3, E e4, E e5)
  {
    return copyOf(Ordering.natural(), Arrays.asList(new Comparable[] { e1, e2, e3, e4, e5 }));
  }
  













  public static <E extends Comparable<? super E>> ImmutableSortedMultiset<E> of(E e1, E e2, E e3, E e4, E e5, E e6, E... remaining)
  {
    int size = remaining.length + 6;
    List<E> all = new ArrayList(size);
    Collections.addAll(all, new Comparable[] { e1, e2, e3, e4, e5, e6 });
    Collections.addAll(all, remaining);
    return copyOf(Ordering.natural(), all);
  }
  





  public static <E extends Comparable<? super E>> ImmutableSortedMultiset<E> copyOf(E[] elements)
  {
    return copyOf(Ordering.natural(), Arrays.asList(elements));
  }
  
























  public static <E> ImmutableSortedMultiset<E> copyOf(Iterable<? extends E> elements)
  {
    Ordering<E> naturalOrder = Ordering.natural();
    return copyOf(naturalOrder, elements);
  }
  












  public static <E> ImmutableSortedMultiset<E> copyOf(Iterator<? extends E> elements)
  {
    Ordering<E> naturalOrder = Ordering.natural();
    return copyOfInternal(naturalOrder, elements);
  }
  






  public static <E> ImmutableSortedMultiset<E> copyOf(Comparator<? super E> comparator, Iterator<? extends E> elements)
  {
    Preconditions.checkNotNull(comparator);
    return copyOfInternal(comparator, elements);
  }
  










  public static <E> ImmutableSortedMultiset<E> copyOf(Comparator<? super E> comparator, Iterable<? extends E> elements)
  {
    Preconditions.checkNotNull(comparator);
    return copyOfInternal(comparator, elements);
  }
  














  public static <E> ImmutableSortedMultiset<E> copyOfSorted(SortedMultiset<E> sortedMultiset)
  {
    Comparator<? super E> comparator = sortedMultiset.comparator();
    if (comparator == null) {
      comparator = NATURAL_ORDER;
    }
    return copyOfInternal(comparator, sortedMultiset);
  }
  

  private static <E> ImmutableSortedMultiset<E> copyOfInternal(Comparator<? super E> comparator, Iterable<? extends E> iterable)
  {
    if ((SortedIterables.hasSameComparator(comparator, iterable)) && ((iterable instanceof ImmutableSortedMultiset)))
    {
      ImmutableSortedMultiset<E> multiset = (ImmutableSortedMultiset)iterable;
      if (!multiset.isPartialView()) {
        return (ImmutableSortedMultiset)iterable;
      }
    }
    ImmutableList<Multiset.Entry<E>> entries = ImmutableList.copyOf(SortedIterables.sortedCounts(comparator, iterable));
    
    if (entries.isEmpty()) {
      return emptyMultiset(comparator);
    }
    verifyEntries(entries);
    return RegularImmutableSortedMultiset.createFromSorted(comparator, entries);
  }
  

  private static <E> ImmutableSortedMultiset<E> copyOfInternal(Comparator<? super E> comparator, Iterator<? extends E> iterator)
  {
    ImmutableList<Multiset.Entry<E>> entries = ImmutableList.copyOf(SortedIterables.sortedCounts(comparator, iterator));
    
    if (entries.isEmpty()) {
      return emptyMultiset(comparator);
    }
    verifyEntries(entries);
    return RegularImmutableSortedMultiset.createFromSorted(comparator, entries);
  }
  
  private static <E> void verifyEntries(Collection<Multiset.Entry<E>> entries) {
    for (Multiset.Entry<E> entry : entries) {
      Preconditions.checkNotNull(entry.getElement());
    }
  }
  
  static <E> ImmutableSortedMultiset<E> emptyMultiset(Comparator<? super E> comparator)
  {
    if (NATURAL_ORDER.equals(comparator)) {
      return NATURAL_EMPTY_MULTISET;
    }
    return new EmptyImmutableSortedMultiset(comparator);
  }
  

  ImmutableSortedMultiset(Comparator<? super E> comparator)
  {
    this.comparator = ((Comparator)Preconditions.checkNotNull(comparator));
  }
  
  public Comparator<? super E> comparator()
  {
    return comparator;
  }
  



  Comparator<Object> unsafeComparator()
  {
    return comparator;
  }
  

  Comparator<? super E> reverseComparator()
  {
    Comparator<? super E> result = reverseComparator;
    if (result == null) {
      return this.reverseComparator = Ordering.from(comparator).reverse();
    }
    return result;
  }
  


  public ImmutableSortedSet<E> elementSet()
  {
    ImmutableSortedSet<E> result = elementSet;
    if (result == null) {
      return this.elementSet = createElementSet();
    }
    return result;
  }
  

  abstract ImmutableSortedSet<E> createElementSet();
  

  abstract ImmutableSortedSet<E> createDescendingElementSet();
  
  public ImmutableSortedMultiset<E> descendingMultiset()
  {
    ImmutableSortedMultiset<E> result = descendingMultiset;
    if (result == null) {
      return this.descendingMultiset = new DescendingImmutableSortedMultiset(this);
    }
    return result;
  }
  



  abstract UnmodifiableIterator<Multiset.Entry<E>> descendingEntryIterator();
  


  public final Multiset.Entry<E> pollFirstEntry()
  {
    throw new UnsupportedOperationException();
  }
  





  public Multiset.Entry<E> pollLastEntry()
  {
    throw new UnsupportedOperationException();
  }
  

  public abstract ImmutableSortedMultiset<E> headMultiset(E paramE, BoundType paramBoundType);
  

  public ImmutableSortedMultiset<E> subMultiset(E lowerBound, BoundType lowerBoundType, E upperBound, BoundType upperBoundType)
  {
    return tailMultiset(lowerBound, lowerBoundType).headMultiset(upperBound, upperBoundType);
  }
  




  public abstract ImmutableSortedMultiset<E> tailMultiset(E paramE, BoundType paramBoundType);
  




  public static <E> Builder<E> orderedBy(Comparator<E> comparator)
  {
    return new Builder(comparator);
  }
  







  public static <E extends Comparable<E>> Builder<E> reverseOrder()
  {
    return new Builder(Ordering.natural().reverse());
  }
  









  public static <E extends Comparable<E>> Builder<E> naturalOrder()
  {
    return new Builder(Ordering.natural());
  }
  









  public static class Builder<E>
    extends ImmutableMultiset.Builder<E>
  {
    private final Comparator<? super E> comparator;
    









    public Builder(Comparator<? super E> comparator)
    {
      super();
      this.comparator = ((Comparator)Preconditions.checkNotNull(comparator));
    }
    







    public Builder<E> add(E element)
    {
      super.add(element);
      return this;
    }
    











    public Builder<E> addCopies(E element, int occurrences)
    {
      super.addCopies(element, occurrences);
      return this;
    }
    










    public Builder<E> setCount(E element, int count)
    {
      super.setCount(element, count);
      return this;
    }
    







    public Builder<E> add(E... elements)
    {
      super.add(elements);
      return this;
    }
    







    public Builder<E> addAll(Iterable<? extends E> elements)
    {
      super.addAll(elements);
      return this;
    }
    







    public Builder<E> addAll(Iterator<? extends E> elements)
    {
      super.addAll(elements);
      return this;
    }
    




    public ImmutableSortedMultiset<E> build()
    {
      return ImmutableSortedMultiset.copyOf(comparator, contents);
    }
  }
  
  private static final class SerializedForm implements Serializable {
    Comparator comparator;
    Object[] elements;
    int[] counts;
    
    SerializedForm(SortedMultiset<?> multiset) {
      comparator = multiset.comparator();
      int n = multiset.entrySet().size();
      elements = new Object[n];
      counts = new int[n];
      int i = 0;
      for (Multiset.Entry<?> entry : multiset.entrySet()) {
        elements[i] = entry.getElement();
        counts[i] = entry.getCount();
        i++;
      }
    }
    
    Object readResolve()
    {
      int n = elements.length;
      ImmutableSortedMultiset.Builder<Object> builder = ImmutableSortedMultiset.orderedBy(comparator);
      for (int i = 0; i < n; i++) {
        builder.addCopies(elements[i], counts[i]);
      }
      return builder.build();
    }
  }
  
  Object writeReplace()
  {
    return new SerializedForm(this);
  }
}
