package com.google.common.collect;

import com.google.common.annotations.Beta;
import com.google.common.annotations.GwtCompatible;
import com.google.common.annotations.GwtIncompatible;
import com.google.common.base.Function;
import com.google.common.base.Joiner;
import com.google.common.base.Joiner.MapJoiner;
import com.google.common.base.Preconditions;
import com.google.common.base.Supplier;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.AbstractCollection;
import java.util.AbstractSet;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.NoSuchElementException;
import java.util.Set;
import java.util.SortedSet;
import javax.annotation.Nullable;










































































@GwtCompatible(emulated=true)
public final class Multimaps
{
  public static <K, V> Multimap<K, V> newMultimap(Map<K, Collection<V>> map, Supplier<? extends Collection<V>> factory) { return new CustomMultimap(map, factory); }
  
  private static class CustomMultimap<K, V> extends AbstractMultimap<K, V> {
    transient Supplier<? extends Collection<V>> factory;
    @GwtIncompatible("java serialization not supported")
    private static final long serialVersionUID = 0L;
    
    CustomMultimap(Map<K, Collection<V>> map, Supplier<? extends Collection<V>> factory) {
      super();
      this.factory = ((Supplier)Preconditions.checkNotNull(factory));
    }
    
    protected Collection<V> createCollection() {
      return (Collection)factory.get();
    }
    


    @GwtIncompatible("java.io.ObjectOutputStream")
    private void writeObject(ObjectOutputStream stream)
      throws IOException
    {
      stream.defaultWriteObject();
      stream.writeObject(factory);
      stream.writeObject(backingMap());
    }
    
    @GwtIncompatible("java.io.ObjectInputStream")
    private void readObject(ObjectInputStream stream)
      throws IOException, ClassNotFoundException
    {
      stream.defaultReadObject();
      factory = ((Supplier)stream.readObject());
      Map<K, Collection<V>> map = (Map)stream.readObject();
      setMap(map);
    }
  }
  








































  public static <K, V> ListMultimap<K, V> newListMultimap(Map<K, Collection<V>> map, Supplier<? extends List<V>> factory)
  {
    return new CustomListMultimap(map, factory);
  }
  
  private static class CustomListMultimap<K, V> extends AbstractListMultimap<K, V> {
    transient Supplier<? extends List<V>> factory;
    @GwtIncompatible("java serialization not supported")
    private static final long serialVersionUID = 0L;
    
    CustomListMultimap(Map<K, Collection<V>> map, Supplier<? extends List<V>> factory) {
      super();
      this.factory = ((Supplier)Preconditions.checkNotNull(factory));
    }
    
    protected List<V> createCollection() {
      return (List)factory.get();
    }
    
    @GwtIncompatible("java.io.ObjectOutputStream")
    private void writeObject(ObjectOutputStream stream) throws IOException
    {
      stream.defaultWriteObject();
      stream.writeObject(factory);
      stream.writeObject(backingMap());
    }
    
    @GwtIncompatible("java.io.ObjectInputStream")
    private void readObject(ObjectInputStream stream)
      throws IOException, ClassNotFoundException
    {
      stream.defaultReadObject();
      factory = ((Supplier)stream.readObject());
      Map<K, Collection<V>> map = (Map)stream.readObject();
      setMap(map);
    }
  }
  








































  public static <K, V> SetMultimap<K, V> newSetMultimap(Map<K, Collection<V>> map, Supplier<? extends Set<V>> factory)
  {
    return new CustomSetMultimap(map, factory);
  }
  
  private static class CustomSetMultimap<K, V> extends AbstractSetMultimap<K, V> {
    transient Supplier<? extends Set<V>> factory;
    @GwtIncompatible("not needed in emulated source")
    private static final long serialVersionUID = 0L;
    
    CustomSetMultimap(Map<K, Collection<V>> map, Supplier<? extends Set<V>> factory) {
      super();
      this.factory = ((Supplier)Preconditions.checkNotNull(factory));
    }
    
    protected Set<V> createCollection() {
      return (Set)factory.get();
    }
    
    @GwtIncompatible("java.io.ObjectOutputStream")
    private void writeObject(ObjectOutputStream stream) throws IOException
    {
      stream.defaultWriteObject();
      stream.writeObject(factory);
      stream.writeObject(backingMap());
    }
    
    @GwtIncompatible("java.io.ObjectInputStream")
    private void readObject(ObjectInputStream stream)
      throws IOException, ClassNotFoundException
    {
      stream.defaultReadObject();
      factory = ((Supplier)stream.readObject());
      Map<K, Collection<V>> map = (Map)stream.readObject();
      setMap(map);
    }
  }
  








































  public static <K, V> SortedSetMultimap<K, V> newSortedSetMultimap(Map<K, Collection<V>> map, Supplier<? extends SortedSet<V>> factory)
  {
    return new CustomSortedSetMultimap(map, factory);
  }
  
  private static class CustomSortedSetMultimap<K, V> extends AbstractSortedSetMultimap<K, V> {
    transient Supplier<? extends SortedSet<V>> factory;
    transient Comparator<? super V> valueComparator;
    @GwtIncompatible("not needed in emulated source")
    private static final long serialVersionUID = 0L;
    
    CustomSortedSetMultimap(Map<K, Collection<V>> map, Supplier<? extends SortedSet<V>> factory) {
      super();
      this.factory = ((Supplier)Preconditions.checkNotNull(factory));
      valueComparator = ((SortedSet)factory.get()).comparator();
    }
    
    protected SortedSet<V> createCollection() {
      return (SortedSet)factory.get();
    }
    
    public Comparator<? super V> valueComparator() {
      return valueComparator;
    }
    
    @GwtIncompatible("java.io.ObjectOutputStream")
    private void writeObject(ObjectOutputStream stream) throws IOException
    {
      stream.defaultWriteObject();
      stream.writeObject(factory);
      stream.writeObject(backingMap());
    }
    
    @GwtIncompatible("java.io.ObjectInputStream")
    private void readObject(ObjectInputStream stream)
      throws IOException, ClassNotFoundException
    {
      stream.defaultReadObject();
      factory = ((Supplier)stream.readObject());
      valueComparator = ((SortedSet)factory.get()).comparator();
      Map<K, Collection<V>> map = (Map)stream.readObject();
      setMap(map);
    }
  }
  











  public static <K, V, M extends Multimap<K, V>> M invertFrom(Multimap<? extends V, ? extends K> source, M dest)
  {
    Preconditions.checkNotNull(dest);
    for (Map.Entry<? extends V, ? extends K> entry : source.entries()) {
      dest.put(entry.getValue(), entry.getKey());
    }
    return dest;
  }
  

































  public static <K, V> Multimap<K, V> synchronizedMultimap(Multimap<K, V> multimap)
  {
    return Synchronized.multimap(multimap, null);
  }
  

















  public static <K, V> Multimap<K, V> unmodifiableMultimap(Multimap<K, V> delegate)
  {
    if (((delegate instanceof UnmodifiableMultimap)) || ((delegate instanceof ImmutableMultimap)))
    {
      return delegate;
    }
    return new UnmodifiableMultimap(delegate);
  }
  





  @Deprecated
  public static <K, V> Multimap<K, V> unmodifiableMultimap(ImmutableMultimap<K, V> delegate)
  {
    return (Multimap)Preconditions.checkNotNull(delegate);
  }
  
  private static class UnmodifiableMultimap<K, V> extends ForwardingMultimap<K, V> implements Serializable {
    final Multimap<K, V> delegate;
    transient Collection<Map.Entry<K, V>> entries;
    transient Multiset<K> keys;
    transient Set<K> keySet;
    transient Collection<V> values;
    transient Map<K, Collection<V>> map;
    private static final long serialVersionUID = 0L;
    
    UnmodifiableMultimap(Multimap<K, V> delegate) {
      this.delegate = ((Multimap)Preconditions.checkNotNull(delegate));
    }
    
    protected Multimap<K, V> delegate() {
      return delegate;
    }
    
    public void clear() {
      throw new UnsupportedOperationException();
    }
    
    public Map<K, Collection<V>> asMap() {
      Map<K, Collection<V>> result = map;
      if (result == null) {
        final Map<K, Collection<V>> unmodifiableMap = Collections.unmodifiableMap(delegate.asMap());
        
        map = (result = new ForwardingMap() { Set<Map.Entry<K, Collection<V>>> entrySet;
          
          protected Map<K, Collection<V>> delegate() { return unmodifiableMap; }
          

          Collection<Collection<V>> asMapValues;
          public Set<Map.Entry<K, Collection<V>>> entrySet()
          {
            Set<Map.Entry<K, Collection<V>>> result = entrySet;
            return result == null ? (this.entrySet = Multimaps.unmodifiableAsMapEntries(unmodifiableMap.entrySet())) : result;
          }
          


          public Collection<V> get(Object key)
          {
            Collection<V> collection = (Collection)unmodifiableMap.get(key);
            return collection == null ? null : Multimaps.unmodifiableValueCollection(collection);
          }
          


          public Collection<Collection<V>> values()
          {
            Collection<Collection<V>> result = asMapValues;
            return result == null ? (this.asMapValues = new Multimaps.UnmodifiableAsMapValues(unmodifiableMap.values())) : result;
          }
          


          public boolean containsValue(Object o)
          {
            return values().contains(o);
          }
        });
      }
      return result;
    }
    
    public Collection<Map.Entry<K, V>> entries() {
      Collection<Map.Entry<K, V>> result = entries;
      if (result == null) {
        entries = (result = Multimaps.unmodifiableEntries(delegate.entries()));
      }
      return result;
    }
    
    public Collection<V> get(K key) {
      return Multimaps.unmodifiableValueCollection(delegate.get(key));
    }
    
    public Multiset<K> keys() {
      Multiset<K> result = keys;
      if (result == null) {
        keys = (result = Multisets.unmodifiableMultiset(delegate.keys()));
      }
      return result;
    }
    
    public Set<K> keySet() {
      Set<K> result = keySet;
      if (result == null) {
        keySet = (result = Collections.unmodifiableSet(delegate.keySet()));
      }
      return result;
    }
    
    public boolean put(K key, V value) {
      throw new UnsupportedOperationException();
    }
    
    public boolean putAll(K key, Iterable<? extends V> values) {
      throw new UnsupportedOperationException();
    }
    
    public boolean putAll(Multimap<? extends K, ? extends V> multimap)
    {
      throw new UnsupportedOperationException();
    }
    
    public boolean remove(Object key, Object value) {
      throw new UnsupportedOperationException();
    }
    
    public Collection<V> removeAll(Object key) {
      throw new UnsupportedOperationException();
    }
    
    public Collection<V> replaceValues(K key, Iterable<? extends V> values)
    {
      throw new UnsupportedOperationException();
    }
    
    public Collection<V> values() {
      Collection<V> result = values;
      if (result == null) {
        values = (result = Collections.unmodifiableCollection(delegate.values()));
      }
      return result;
    }
  }
  
  private static class UnmodifiableAsMapValues<V> extends ForwardingCollection<Collection<V>>
  {
    final Collection<Collection<V>> delegate;
    
    UnmodifiableAsMapValues(Collection<Collection<V>> delegate)
    {
      this.delegate = Collections.unmodifiableCollection(delegate);
    }
    
    protected Collection<Collection<V>> delegate() { return delegate; }
    
    public Iterator<Collection<V>> iterator() {
      final Iterator<Collection<V>> iterator = delegate.iterator();
      new Iterator()
      {
        public boolean hasNext() {
          return iterator.hasNext();
        }
        
        public Collection<V> next() {
          return Multimaps.unmodifiableValueCollection((Collection)iterator.next());
        }
        
        public void remove() {
          throw new UnsupportedOperationException();
        }
      };
    }
    
    public Object[] toArray() { return standardToArray(); }
    
    public <T> T[] toArray(T[] array) {
      return standardToArray(array);
    }
    
    public boolean contains(Object o) { return standardContains(o); }
    
    public boolean containsAll(Collection<?> c) {
      return standardContainsAll(c);
    }
  }
  
  private static class UnmodifiableListMultimap<K, V> extends Multimaps.UnmodifiableMultimap<K, V> implements ListMultimap<K, V> {
    private static final long serialVersionUID = 0L;
    
    UnmodifiableListMultimap(ListMultimap<K, V> delegate) { super(); }
    
    public ListMultimap<K, V> delegate() {
      return (ListMultimap)super.delegate();
    }
    
    public List<V> get(K key) { return Collections.unmodifiableList(delegate().get(key)); }
    
    public List<V> removeAll(Object key) {
      throw new UnsupportedOperationException();
    }
    
    public List<V> replaceValues(K key, Iterable<? extends V> values) {
      throw new UnsupportedOperationException();
    }
  }
  
  private static class UnmodifiableSetMultimap<K, V> extends Multimaps.UnmodifiableMultimap<K, V> implements SetMultimap<K, V> {
    private static final long serialVersionUID = 0L;
    
    UnmodifiableSetMultimap(SetMultimap<K, V> delegate) {
      super();
    }
    
    public SetMultimap<K, V> delegate() { return (SetMultimap)super.delegate(); }
    



    public Set<V> get(K key)
    {
      return Collections.unmodifiableSet(delegate().get(key));
    }
    
    public Set<Map.Entry<K, V>> entries() { return Maps.unmodifiableEntrySet(delegate().entries()); }
    
    public Set<V> removeAll(Object key) {
      throw new UnsupportedOperationException();
    }
    
    public Set<V> replaceValues(K key, Iterable<? extends V> values) {
      throw new UnsupportedOperationException();
    }
  }
  
  private static class UnmodifiableSortedSetMultimap<K, V> extends Multimaps.UnmodifiableSetMultimap<K, V> implements SortedSetMultimap<K, V> {
    private static final long serialVersionUID = 0L;
    
    UnmodifiableSortedSetMultimap(SortedSetMultimap<K, V> delegate) {
      super();
    }
    
    public SortedSetMultimap<K, V> delegate() { return (SortedSetMultimap)super.delegate(); }
    
    public SortedSet<V> get(K key) {
      return Collections.unmodifiableSortedSet(delegate().get(key));
    }
    
    public SortedSet<V> removeAll(Object key) { throw new UnsupportedOperationException(); }
    
    public SortedSet<V> replaceValues(K key, Iterable<? extends V> values)
    {
      throw new UnsupportedOperationException();
    }
    
    public Comparator<? super V> valueComparator() {
      return delegate().valueComparator();
    }
  }
  













  public static <K, V> SetMultimap<K, V> synchronizedSetMultimap(SetMultimap<K, V> multimap)
  {
    return Synchronized.setMultimap(multimap, null);
  }
  


















  public static <K, V> SetMultimap<K, V> unmodifiableSetMultimap(SetMultimap<K, V> delegate)
  {
    if (((delegate instanceof UnmodifiableSetMultimap)) || ((delegate instanceof ImmutableSetMultimap)))
    {
      return delegate;
    }
    return new UnmodifiableSetMultimap(delegate);
  }
  





  @Deprecated
  public static <K, V> SetMultimap<K, V> unmodifiableSetMultimap(ImmutableSetMultimap<K, V> delegate)
  {
    return (SetMultimap)Preconditions.checkNotNull(delegate);
  }
  












  public static <K, V> SortedSetMultimap<K, V> synchronizedSortedSetMultimap(SortedSetMultimap<K, V> multimap)
  {
    return Synchronized.sortedSetMultimap(multimap, null);
  }
  


















  public static <K, V> SortedSetMultimap<K, V> unmodifiableSortedSetMultimap(SortedSetMultimap<K, V> delegate)
  {
    if ((delegate instanceof UnmodifiableSortedSetMultimap)) {
      return delegate;
    }
    return new UnmodifiableSortedSetMultimap(delegate);
  }
  









  public static <K, V> ListMultimap<K, V> synchronizedListMultimap(ListMultimap<K, V> multimap)
  {
    return Synchronized.listMultimap(multimap, null);
  }
  


















  public static <K, V> ListMultimap<K, V> unmodifiableListMultimap(ListMultimap<K, V> delegate)
  {
    if (((delegate instanceof UnmodifiableListMultimap)) || ((delegate instanceof ImmutableListMultimap)))
    {
      return delegate;
    }
    return new UnmodifiableListMultimap(delegate);
  }
  





  @Deprecated
  public static <K, V> ListMultimap<K, V> unmodifiableListMultimap(ImmutableListMultimap<K, V> delegate)
  {
    return (ListMultimap)Preconditions.checkNotNull(delegate);
  }
  








  private static <V> Collection<V> unmodifiableValueCollection(Collection<V> collection)
  {
    if ((collection instanceof SortedSet))
      return Collections.unmodifiableSortedSet((SortedSet)collection);
    if ((collection instanceof Set))
      return Collections.unmodifiableSet((Set)collection);
    if ((collection instanceof List)) {
      return Collections.unmodifiableList((List)collection);
    }
    return Collections.unmodifiableCollection(collection);
  }
  











  private static <K, V> Map.Entry<K, Collection<V>> unmodifiableAsMapEntry(Map.Entry<K, Collection<V>> entry)
  {
    Preconditions.checkNotNull(entry);
    new AbstractMapEntry() {
      public K getKey() {
        return val$entry.getKey();
      }
      
      public Collection<V> getValue() {
        return Multimaps.unmodifiableValueCollection((Collection)val$entry.getValue());
      }
    };
  }
  









  private static <K, V> Collection<Map.Entry<K, V>> unmodifiableEntries(Collection<Map.Entry<K, V>> entries)
  {
    if ((entries instanceof Set)) {
      return Maps.unmodifiableEntrySet((Set)entries);
    }
    return new Maps.UnmodifiableEntries(Collections.unmodifiableCollection(entries));
  }
  











  private static <K, V> Set<Map.Entry<K, Collection<V>>> unmodifiableAsMapEntries(Set<Map.Entry<K, Collection<V>>> asMapEntries)
  {
    return new UnmodifiableAsMapEntries(Collections.unmodifiableSet(asMapEntries));
  }
  
  static class UnmodifiableAsMapEntries<K, V> extends ForwardingSet<Map.Entry<K, Collection<V>>>
  {
    private final Set<Map.Entry<K, Collection<V>>> delegate;
    
    UnmodifiableAsMapEntries(Set<Map.Entry<K, Collection<V>>> delegate)
    {
      this.delegate = delegate;
    }
    
    protected Set<Map.Entry<K, Collection<V>>> delegate() {
      return delegate;
    }
    
    public Iterator<Map.Entry<K, Collection<V>>> iterator() {
      final Iterator<Map.Entry<K, Collection<V>>> iterator = delegate.iterator();
      new ForwardingIterator() {
        protected Iterator<Map.Entry<K, Collection<V>>> delegate() {
          return iterator;
        }
        
        public Map.Entry<K, Collection<V>> next() { return Multimaps.unmodifiableAsMapEntry((Map.Entry)iterator.next()); }
      };
    }
    
    public Object[] toArray()
    {
      return standardToArray();
    }
    
    public <T> T[] toArray(T[] array) {
      return standardToArray(array);
    }
    
    public boolean contains(Object o) {
      return Maps.containsEntryImpl(delegate(), o);
    }
    
    public boolean containsAll(Collection<?> c) {
      return standardContainsAll(c);
    }
    
    public boolean equals(@Nullable Object object) {
      return standardEquals(object);
    }
  }
  
















  public static <K, V> SetMultimap<K, V> forMap(Map<K, V> map)
  {
    return new MapMultimap(map);
  }
  
  private static class MapMultimap<K, V> implements SetMultimap<K, V>, Serializable
  {
    final Map<K, V> map;
    transient Map<K, Collection<V>> asMap;
    
    MapMultimap(Map<K, V> map)
    {
      this.map = ((Map)Preconditions.checkNotNull(map));
    }
    
    public int size()
    {
      return map.size();
    }
    
    public boolean isEmpty()
    {
      return map.isEmpty();
    }
    
    public boolean containsKey(Object key)
    {
      return map.containsKey(key);
    }
    
    public boolean containsValue(Object value)
    {
      return map.containsValue(value);
    }
    
    public boolean containsEntry(Object key, Object value)
    {
      return map.entrySet().contains(Maps.immutableEntry(key, value));
    }
    
    public Set<V> get(final K key)
    {
      new AbstractSet() {
        public Iterator<V> iterator() {
          new Iterator()
          {
            int i;
            
            public boolean hasNext() {
              return (i == 0) && (map.containsKey(val$key));
            }
            
            public V next()
            {
              if (!hasNext()) {
                throw new NoSuchElementException();
              }
              i += 1;
              return map.get(val$key);
            }
            
            public void remove()
            {
              Preconditions.checkState(i == 1);
              i = -1;
              map.remove(val$key);
            }
          };
        }
        
        public int size() {
          return map.containsKey(key) ? 1 : 0;
        }
      };
    }
    
    public boolean put(K key, V value)
    {
      throw new UnsupportedOperationException();
    }
    
    public boolean putAll(K key, Iterable<? extends V> values)
    {
      throw new UnsupportedOperationException();
    }
    
    public boolean putAll(Multimap<? extends K, ? extends V> multimap)
    {
      throw new UnsupportedOperationException();
    }
    
    public Set<V> replaceValues(K key, Iterable<? extends V> values)
    {
      throw new UnsupportedOperationException();
    }
    
    public boolean remove(Object key, Object value)
    {
      return map.entrySet().remove(Maps.immutableEntry(key, value));
    }
    
    public Set<V> removeAll(Object key)
    {
      Set<V> values = new HashSet(2);
      if (!map.containsKey(key)) {
        return values;
      }
      values.add(map.remove(key));
      return values;
    }
    
    public void clear()
    {
      map.clear();
    }
    
    public Set<K> keySet()
    {
      return map.keySet();
    }
    
    public Multiset<K> keys()
    {
      return Multisets.forSet(map.keySet());
    }
    
    public Collection<V> values()
    {
      return map.values();
    }
    
    public Set<Map.Entry<K, V>> entries()
    {
      return map.entrySet();
    }
    
    public Map<K, Collection<V>> asMap()
    {
      Map<K, Collection<V>> result = asMap;
      if (result == null) {
        asMap = (result = new AsMap());
      }
      return result;
    }
    
    public boolean equals(@Nullable Object object) {
      if (object == this) {
        return true;
      }
      if ((object instanceof Multimap)) {
        Multimap<?, ?> that = (Multimap)object;
        return (size() == that.size()) && (asMap().equals(that.asMap()));
      }
      return false;
    }
    
    public int hashCode() {
      return map.hashCode();
    }
    
    private static final Joiner.MapJoiner JOINER = Joiner.on("], ").withKeyValueSeparator("=[").useForNull("null");
    private static final long serialVersionUID = 7845222491160860175L;
    
    public String toString() {
      if (map.isEmpty()) {
        return "{}";
      }
      StringBuilder builder = Collections2.newStringBuilderForCollection(map.size()).append('{');
      
      JOINER.appendTo(builder, map);
      return "]}";
    }
    
    class AsMapEntries extends AbstractSet<Map.Entry<K, Collection<V>>> {
      AsMapEntries() {}
      
      public int size() { return map.size(); }
      
      public Iterator<Map.Entry<K, Collection<V>>> iterator()
      {
        new Iterator() {
          final Iterator<K> keys = map.keySet().iterator();
          
          public boolean hasNext()
          {
            return keys.hasNext();
          }
          
          public Map.Entry<K, Collection<V>> next() {
            final K key = keys.next();
            new AbstractMapEntry() {
              public K getKey() {
                return key;
              }
              
              public Collection<V> getValue() { return get(key); }
            };
          }
          
          public void remove()
          {
            keys.remove();
          }
        };
      }
      
      public boolean contains(Object o) {
        if (!(o instanceof Map.Entry)) {
          return false;
        }
        Map.Entry<?, ?> entry = (Map.Entry)o;
        if (!(entry.getValue() instanceof Set)) {
          return false;
        }
        Set<?> set = (Set)entry.getValue();
        return (set.size() == 1) && (containsEntry(entry.getKey(), set.iterator().next()));
      }
      
      public boolean remove(Object o)
      {
        if (!(o instanceof Map.Entry)) {
          return false;
        }
        Map.Entry<?, ?> entry = (Map.Entry)o;
        if (!(entry.getValue() instanceof Set)) {
          return false;
        }
        Set<?> set = (Set)entry.getValue();
        return (set.size() == 1) && (map.entrySet().remove(Maps.immutableEntry(entry.getKey(), set.iterator().next())));
      }
    }
    
    class AsMap extends Maps.ImprovedAbstractMap<K, Collection<V>>
    {
      AsMap() {}
      
      protected Set<Map.Entry<K, Collection<V>>> createEntrySet() {
        return new Multimaps.MapMultimap.AsMapEntries(Multimaps.MapMultimap.this);
      }
      

      public boolean containsKey(Object key)
      {
        return map.containsKey(key);
      }
      
      public Collection<V> get(Object key)
      {
        Collection<V> collection = get(key);
        return collection.isEmpty() ? null : collection;
      }
      
      public Collection<V> remove(Object key) {
        Collection<V> collection = removeAll(key);
        return collection.isEmpty() ? null : collection;
      }
    }
  }
  













































  @Beta
  public static <K, V1, V2> Multimap<K, V2> transformValues(Multimap<K, V1> fromMultimap, Function<? super V1, V2> function)
  {
    Preconditions.checkNotNull(function);
    Maps.EntryTransformer<K, V1, V2> transformer = new Maps.EntryTransformer()
    {
      public V2 transformEntry(K key, V1 value)
      {
        return val$function.apply(value);
      }
    };
    return transformEntries(fromMultimap, transformer);
  }
  


























































  @Beta
  public static <K, V1, V2> Multimap<K, V2> transformEntries(Multimap<K, V1> fromMap, Maps.EntryTransformer<? super K, ? super V1, V2> transformer) { return new TransformedEntriesMultimap(fromMap, transformer); }
  
  private static class TransformedEntriesMultimap<K, V1, V2> implements Multimap<K, V2> {
    final Multimap<K, V1> fromMultimap;
    final Maps.EntryTransformer<? super K, ? super V1, V2> transformer;
    private transient Map<K, Collection<V2>> asMap;
    private transient Collection<Map.Entry<K, V2>> entries;
    private transient Collection<V2> values;
    
    TransformedEntriesMultimap(Multimap<K, V1> fromMultimap, Maps.EntryTransformer<? super K, ? super V1, V2> transformer) {
      this.fromMultimap = ((Multimap)Preconditions.checkNotNull(fromMultimap));
      this.transformer = ((Maps.EntryTransformer)Preconditions.checkNotNull(transformer));
    }
    
    Collection<V2> transform(final K key, Collection<V1> values) {
      Collections2.transform(values, new Function() {
        public V2 apply(V1 value) {
          return transformer.transformEntry(key, value);
        }
      });
    }
    

    public Map<K, Collection<V2>> asMap()
    {
      if (asMap == null) {
        Map<K, Collection<V2>> aM = Maps.transformEntries(fromMultimap.asMap(), new Maps.EntryTransformer()
        {

          public Collection<V2> transformEntry(K key, Collection<V1> value)
          {
            return transform(key, value);
          }
        });
        asMap = aM;
        return aM;
      }
      return asMap;
    }
    
    public void clear() {
      fromMultimap.clear();
    }
    
    public boolean containsEntry(Object key, Object value)
    {
      Collection<V2> values = get(key);
      return values.contains(value);
    }
    
    public boolean containsKey(Object key) {
      return fromMultimap.containsKey(key);
    }
    
    public boolean containsValue(Object value) {
      return values().contains(value);
    }
    

    public Collection<Map.Entry<K, V2>> entries()
    {
      if (entries == null) {
        Collection<Map.Entry<K, V2>> es = new TransformedEntries(transformer);
        entries = es;
        return es;
      }
      return entries;
    }
    
    private class TransformedEntries
      extends Collections2.TransformedCollection<Map.Entry<K, V1>, Map.Entry<K, V2>>
    {
      TransformedEntries()
      {
        super(new Function()
        {
          public Map.Entry<K, V2> apply(final Map.Entry<K, V1> entry) {
            new AbstractMapEntry()
            {
              public K getKey() {
                return entry.getKey();
              }
              
              public V2 getValue() {
                return val$transformer.transformEntry(entry.getKey(), entry.getValue());
              }
            };
          }
        });
      }
      
      public boolean contains(Object o)
      {
        if ((o instanceof Map.Entry)) {
          Map.Entry<?, ?> entry = (Map.Entry)o;
          return containsEntry(entry.getKey(), entry.getValue());
        }
        return false;
      }
      
      public boolean remove(Object o)
      {
        if ((o instanceof Map.Entry)) {
          Map.Entry<?, ?> entry = (Map.Entry)o;
          Collection<V2> values = get(entry.getKey());
          return values.remove(entry.getValue());
        }
        return false;
      }
    }
    
    public Collection<V2> get(K key)
    {
      return transform(key, fromMultimap.get(key));
    }
    
    public boolean isEmpty() {
      return fromMultimap.isEmpty();
    }
    
    public Set<K> keySet() {
      return fromMultimap.keySet();
    }
    
    public Multiset<K> keys() {
      return fromMultimap.keys();
    }
    
    public boolean put(K key, V2 value) {
      throw new UnsupportedOperationException();
    }
    
    public boolean putAll(K key, Iterable<? extends V2> values) {
      throw new UnsupportedOperationException();
    }
    
    public boolean putAll(Multimap<? extends K, ? extends V2> multimap)
    {
      throw new UnsupportedOperationException();
    }
    
    public boolean remove(Object key, Object value)
    {
      return get(key).remove(value);
    }
    
    public Collection<V2> removeAll(Object key)
    {
      return transform(key, fromMultimap.removeAll(key));
    }
    
    public Collection<V2> replaceValues(K key, Iterable<? extends V2> values)
    {
      throw new UnsupportedOperationException();
    }
    
    public int size() {
      return fromMultimap.size();
    }
    

    public Collection<V2> values()
    {
      if (values == null) {
        Collection<V2> vs = Collections2.transform(fromMultimap.entries(), new Function()
        {
          public V2 apply(Map.Entry<K, V1> entry)
          {
            return transformer.transformEntry(entry.getKey(), entry.getValue());
          }
          
        });
        values = vs;
        return vs;
      }
      return values;
    }
    
    public boolean equals(Object obj) {
      if ((obj instanceof Multimap)) {
        Multimap<?, ?> other = (Multimap)obj;
        return asMap().equals(other.asMap());
      }
      return false;
    }
    
    public int hashCode() {
      return asMap().hashCode();
    }
    
    public String toString() {
      return asMap().toString();
    }
  }
  










































  @Beta
  public static <K, V1, V2> ListMultimap<K, V2> transformValues(ListMultimap<K, V1> fromMultimap, Function<? super V1, V2> function)
  {
    Preconditions.checkNotNull(function);
    Maps.EntryTransformer<K, V1, V2> transformer = new Maps.EntryTransformer()
    {
      public V2 transformEntry(K key, V1 value)
      {
        return val$function.apply(value);
      }
    };
    return transformEntries(fromMultimap, transformer);
  }
  





















































  @Beta
  public static <K, V1, V2> ListMultimap<K, V2> transformEntries(ListMultimap<K, V1> fromMap, Maps.EntryTransformer<? super K, ? super V1, V2> transformer)
  {
    return new TransformedEntriesListMultimap(fromMap, transformer);
  }
  
  private static final class TransformedEntriesListMultimap<K, V1, V2>
    extends Multimaps.TransformedEntriesMultimap<K, V1, V2>
    implements ListMultimap<K, V2>
  {
    TransformedEntriesListMultimap(ListMultimap<K, V1> fromMultimap, Maps.EntryTransformer<? super K, ? super V1, V2> transformer)
    {
      super(transformer);
    }
    
    List<V2> transform(final K key, Collection<V1> values) {
      Lists.transform((List)values, new Function() {
        public V2 apply(V1 value) {
          return transformer.transformEntry(key, value);
        }
      });
    }
    
    public List<V2> get(K key) {
      return transform(key, fromMultimap.get(key));
    }
    
    public List<V2> removeAll(Object key)
    {
      return transform(key, fromMultimap.removeAll(key));
    }
    
    public List<V2> replaceValues(K key, Iterable<? extends V2> values)
    {
      throw new UnsupportedOperationException();
    }
  }
  










































  public static <K, V> ImmutableListMultimap<K, V> index(Iterable<V> values, Function<? super V, K> keyFunction)
  {
    return index(values.iterator(), keyFunction);
  }
  










  @Deprecated
  @Beta
  public static <K, V, I,  extends Iterable<V>,  extends Iterator<V>> ImmutableListMultimap<K, V> index(I values, Function<? super V, K> keyFunction)
  {
    Iterable<V> valuesIterable = (Iterable)Preconditions.checkNotNull(values);
    return index(valuesIterable, keyFunction);
  }
  











































  public static <K, V> ImmutableListMultimap<K, V> index(Iterator<V> values, Function<? super V, K> keyFunction)
  {
    Preconditions.checkNotNull(keyFunction);
    ImmutableListMultimap.Builder<K, V> builder = ImmutableListMultimap.builder();
    
    while (values.hasNext()) {
      V value = values.next();
      Preconditions.checkNotNull(value, values);
      builder.put(keyFunction.apply(value), value);
    }
    return builder.build();
  }
  
  static abstract class Keys<K, V> extends AbstractMultiset<K> {
    private Set<Multiset.Entry<K>> entrySet;
    
    abstract Multimap<K, V> multimap();
    
    public Set<Multiset.Entry<K>> entrySet() {
      return entrySet == null ? (this.entrySet = createEntrySet()) : entrySet;
    }
    
    Iterator<Multiset.Entry<K>> entryIterator() {
      final Iterator<Map.Entry<K, Collection<V>>> backingIterator = multimap().asMap().entrySet().iterator();
      
      new Iterator() {
        public boolean hasNext() {
          return backingIterator.hasNext();
        }
        
        public Multiset.Entry<K> next() {
          final Map.Entry<K, Collection<V>> backingEntry = (Map.Entry)backingIterator.next();
          
          new Multisets.AbstractEntry() {
            public K getElement() {
              return backingEntry.getKey();
            }
            
            public int getCount() {
              return ((Collection)backingEntry.getValue()).size();
            }
          };
        }
        
        public void remove() {
          backingIterator.remove();
        }
      };
    }
    
    int distinctElements() {
      return multimap().asMap().size();
    }
    

    Set<Multiset.Entry<K>> createEntrySet() { return new KeysEntrySet(); }
    
    class KeysEntrySet extends Multisets.EntrySet<K> {
      KeysEntrySet() {}
      
      Multiset<K> multiset() { return Multimaps.Keys.this; }
      
      public Iterator<Multiset.Entry<K>> iterator()
      {
        return entryIterator();
      }
      
      public int size() {
        return distinctElements();
      }
      
      public boolean isEmpty() {
        return multimap().isEmpty();
      }
      
      public boolean contains(@Nullable Object o) {
        if ((o instanceof Multiset.Entry)) {
          Multiset.Entry<?> entry = (Multiset.Entry)o;
          Collection<V> collection = (Collection)multimap().asMap().get(entry.getElement());
          return (collection != null) && (collection.size() == entry.getCount());
        }
        return false;
      }
      
      public boolean remove(@Nullable Object o) {
        if ((o instanceof Multiset.Entry)) {
          Multiset.Entry<?> entry = (Multiset.Entry)o;
          Collection<V> collection = (Collection)multimap().asMap().get(entry.getElement());
          if ((collection != null) && (collection.size() == entry.getCount())) {
            collection.clear();
            return true;
          }
        }
        return false;
      }
    }
    
    public boolean contains(@Nullable Object element) {
      return multimap().containsKey(element);
    }
    
    public Iterator<K> iterator() {
      Iterators.transform(multimap().entries().iterator(), new Function()
      {
        public K apply(Map.Entry<K, V> entry) {
          return entry.getKey();
        }
      });
    }
    
    public int count(@Nullable Object element) {
      try {
        if (multimap().containsKey(element)) {
          Collection<V> values = (Collection)multimap().asMap().get(element);
          return values == null ? 0 : values.size();
        }
        return 0;
      } catch (ClassCastException e) {
        return 0;
      } catch (NullPointerException e) {}
      return 0;
    }
    
    public int remove(@Nullable Object element, int occurrences)
    {
      Preconditions.checkArgument(occurrences >= 0);
      if (occurrences == 0) {
        return count(element);
      }
      Collection<V> values;
      try
      {
        values = (Collection)multimap().asMap().get(element);
      } catch (ClassCastException e) {
        return 0;
      } catch (NullPointerException e) {
        return 0;
      }
      
      if (values == null) {
        return 0;
      }
      
      int oldCount = values.size();
      if (occurrences >= oldCount) {
        values.clear();
      } else {
        Iterator<V> iterator = values.iterator();
        for (int i = 0; i < occurrences; i++) {
          iterator.next();
          iterator.remove();
        }
      }
      return oldCount;
    }
    
    public void clear() {
      multimap().clear();
    }
    
    public Set<K> elementSet() {
      return multimap().keySet();
    }
  }
  
  static abstract class Values<K, V> extends AbstractCollection<V> {
    abstract Multimap<K, V> multimap();
    
    public Iterator<V> iterator() {
      final Iterator<Map.Entry<K, V>> backingIterator = multimap().entries().iterator();
      
      new Iterator() {
        public boolean hasNext() {
          return backingIterator.hasNext();
        }
        
        public V next() {
          return ((Map.Entry)backingIterator.next()).getValue();
        }
        
        public void remove() {
          backingIterator.remove();
        }
      };
    }
    
    public int size() {
      return multimap().size();
    }
    
    public boolean contains(@Nullable Object o) {
      return multimap().containsValue(o);
    }
    
    public void clear() {
      multimap().clear();
    }
  }
  

  static abstract class Entries<K, V>
    extends AbstractCollection<Map.Entry<K, V>>
  {
    abstract Multimap<K, V> multimap();
    
    public int size()
    {
      return multimap().size();
    }
    
    public boolean contains(@Nullable Object o) {
      if ((o instanceof Map.Entry)) {
        Map.Entry<?, ?> entry = (Map.Entry)o;
        return multimap().containsEntry(entry.getKey(), entry.getValue());
      }
      return false;
    }
    
    public boolean remove(@Nullable Object o) {
      if ((o instanceof Map.Entry)) {
        Map.Entry<?, ?> entry = (Map.Entry)o;
        return multimap().remove(entry.getKey(), entry.getValue());
      }
      return false;
    }
    
    public void clear() {
      multimap().clear();
    }
  }
  
  static abstract class EntrySet<K, V>
    extends Multimaps.Entries<K, V>
    implements Set<Map.Entry<K, V>>
  {
    public int hashCode()
    {
      return Sets.hashCodeImpl(this);
    }
    
    public boolean equals(@Nullable Object obj) {
      return Sets.equalsImpl(this, obj);
    }
  }
  

  static abstract class AsMap<K, V>
    extends Maps.ImprovedAbstractMap<K, Collection<V>>
  {
    abstract Multimap<K, V> multimap();
    
    public abstract int size();
    
    abstract Iterator<Map.Entry<K, Collection<V>>> entryIterator();
    
    protected Set<Map.Entry<K, Collection<V>>> createEntrySet()
    {
      return new EntrySet();
    }
    

    void removeValuesForKey(Object key) { multimap().removeAll(key); }
    
    class EntrySet extends Maps.EntrySet<K, Collection<V>> {
      EntrySet() {}
      
      Map<K, Collection<V>> map() { return Multimaps.AsMap.this; }
      
      public Iterator<Map.Entry<K, Collection<V>>> iterator()
      {
        return entryIterator();
      }
      
      public boolean remove(Object o) {
        if (!contains(o)) {
          return false;
        }
        Map.Entry<?, ?> entry = (Map.Entry)o;
        removeValuesForKey(entry.getKey());
        return true;
      }
    }
    
    public Collection<V> get(Object key)
    {
      return containsKey(key) ? multimap().get(key) : null;
    }
    
    public Collection<V> remove(Object key) {
      return containsKey(key) ? multimap().removeAll(key) : null;
    }
    
    public Set<K> keySet() {
      return multimap().keySet();
    }
    
    public boolean isEmpty() {
      return multimap().isEmpty();
    }
    
    public boolean containsKey(Object key) {
      return multimap().containsKey(key);
    }
    
    public void clear() {
      multimap().clear();
    }
  }
}
