package com.google.common.collect;

import com.google.common.annotations.GwtCompatible;
import java.util.Map.Entry;



















@GwtCompatible(serializable=true, emulated=true)
class RegularImmutableBiMap<K, V>
  extends ImmutableBiMap<K, V>
{
  final transient ImmutableMap<K, V> delegate;
  final transient ImmutableBiMap<V, K> inverse;
  
  RegularImmutableBiMap(ImmutableMap<K, V> delegate)
  {
    this.delegate = delegate;
    
    ImmutableMap.Builder<V, K> builder = ImmutableMap.builder();
    for (Map.Entry<K, V> entry : delegate.entrySet()) {
      builder.put(entry.getValue(), entry.getKey());
    }
    ImmutableMap<V, K> backwardMap = builder.build();
    inverse = new RegularImmutableBiMap(backwardMap, this);
  }
  
  RegularImmutableBiMap(ImmutableMap<K, V> delegate, ImmutableBiMap<V, K> inverse)
  {
    this.delegate = delegate;
    this.inverse = inverse;
  }
  
  ImmutableMap<K, V> delegate() {
    return delegate;
  }
  
  public ImmutableBiMap<V, K> inverse() {
    return inverse;
  }
  
  boolean isPartialView() {
    return (delegate.isPartialView()) || (inverse.delegate().isPartialView());
  }
}
