package com.google.common.collect;

import com.google.common.annotations.GwtCompatible;
import java.util.Iterator;
import java.util.Map.Entry;
import javax.annotation.Nullable;
























@GwtCompatible(serializable=true)
class RegularImmutableMultiset<E>
  extends ImmutableMultiset<E>
{
  private final transient ImmutableMap<E, Integer> map;
  private final transient int size;
  
  RegularImmutableMultiset(ImmutableMap<E, Integer> map, int size)
  {
    this.map = map;
    this.size = size;
  }
  
  boolean isPartialView()
  {
    return map.isPartialView();
  }
  
  public int count(@Nullable Object element)
  {
    Integer value = (Integer)map.get(element);
    return value == null ? 0 : value.intValue();
  }
  
  public int size()
  {
    return size;
  }
  
  public boolean contains(@Nullable Object element)
  {
    return map.containsKey(element);
  }
  
  public ImmutableSet<E> elementSet()
  {
    return map.keySet();
  }
  
  UnmodifiableIterator<Multiset.Entry<E>> entryIterator()
  {
    final Iterator<Map.Entry<E, Integer>> mapIterator = map.entrySet().iterator();
    
    new UnmodifiableIterator()
    {
      public boolean hasNext() {
        return mapIterator.hasNext();
      }
      
      public Multiset.Entry<E> next()
      {
        Map.Entry<E, Integer> mapEntry = (Map.Entry)mapIterator.next();
        return Multisets.immutableEntry(mapEntry.getKey(), ((Integer)mapEntry.getValue()).intValue());
      }
    };
  }
  
  public int hashCode()
  {
    return map.hashCode();
  }
  
  int distinctElements()
  {
    return map.size();
  }
}
