package com.google.common.io;

import com.google.common.annotations.Beta;
import com.google.common.base.Preconditions;
import java.io.FilterInputStream;
import java.io.IOException;
import java.io.InputStream;























@Beta
public final class LimitInputStream
  extends FilterInputStream
{
  private long left;
  private long mark = -1L;
  





  public LimitInputStream(InputStream in, long limit)
  {
    super(in);
    Preconditions.checkNotNull(in);
    Preconditions.checkArgument(limit >= 0L, "limit must be non-negative");
    left = limit;
  }
  
  public int available() throws IOException {
    return (int)Math.min(in.available(), left);
  }
  
  public void mark(int readlimit) {
    in.mark(readlimit);
    mark = left;
  }
  
  public int read() throws IOException
  {
    if (left == 0L) {
      return -1;
    }
    
    int result = in.read();
    if (result != -1) {
      left -= 1L;
    }
    return result;
  }
  
  public int read(byte[] b, int off, int len) throws IOException {
    if (left == 0L) {
      return -1;
    }
    
    len = (int)Math.min(len, left);
    int result = in.read(b, off, len);
    if (result != -1) {
      left -= result;
    }
    return result;
  }
  
  public void reset() throws IOException {
    if (!in.markSupported()) {
      throw new IOException("Mark not supported");
    }
    if (mark == -1L) {
      throw new IOException("Mark not set");
    }
    
    in.reset();
    left = mark;
  }
  
  public long skip(long n) throws IOException {
    n = Math.min(n, left);
    long skipped = in.skip(n);
    left -= skipped;
    return skipped;
  }
}
