package com.google.common.util.concurrent;

import com.google.common.annotations.Beta;
import com.google.common.base.Throwables;
import java.util.concurrent.Executor;



























@Beta
public abstract class AbstractExecutionThreadService
  implements Service
{
  private final Service delegate = new AbstractService() {
    protected final void doStart() {
      executor().execute(new Runnable()
      {
        public void run() {
          try {
            startUp();
            notifyStarted();
            
            if (isRunning()) {
              try {
                AbstractExecutionThreadService.this.run();
              } catch (Throwable t) {
                try {
                  shutDown();
                } catch (Exception ignored) {}
                throw t;
              }
            }
            
            shutDown();
            notifyStopped();
          } catch (Throwable t) {
            notifyFailed(t);
            throw Throwables.propagate(t);
          }
        }
      });
    }
    
    protected void doStop() {
      triggerShutdown();
    }
  };
  






  protected void startUp()
    throws Exception
  {}
  





  protected abstract void run()
    throws Exception;
  





  protected void shutDown()
    throws Exception
  {}
  





  protected void triggerShutdown() {}
  





  protected Executor executor()
  {
    new Executor()
    {
      public void execute(Runnable command) {
        new Thread(command, getServiceName()).start();
      }
    };
  }
  
  public String toString() {
    return getServiceName() + " [" + state() + "]";
  }
  

  public final ListenableFuture<Service.State> start()
  {
    return delegate.start();
  }
  
  public final Service.State startAndWait() {
    return delegate.startAndWait();
  }
  
  public final boolean isRunning() {
    return delegate.isRunning();
  }
  
  public final Service.State state() {
    return delegate.state();
  }
  
  public final ListenableFuture<Service.State> stop() {
    return delegate.stop();
  }
  
  public final Service.State stopAndWait() {
    return delegate.stopAndWait();
  }
  







  protected String getServiceName()
  {
    return getClass().getSimpleName();
  }
}
