package com.google.common.util.concurrent;

import com.google.common.annotations.Beta;
import com.google.common.base.Throwables;
import java.util.concurrent.Executor;




























@Beta
public abstract class AbstractIdleService
  implements Service
{
  private final Service delegate = new AbstractService() {
    protected final void doStart() {
      executor(Service.State.STARTING).execute(new Runnable() {
        public void run() {
          try {
            startUp();
            notifyStarted();
          } catch (Throwable t) {
            notifyFailed(t);
            throw Throwables.propagate(t);
          }
        }
      });
    }
    
    protected final void doStop() {
      executor(Service.State.STOPPING).execute(new Runnable() {
        public void run() {
          try {
            shutDown();
            notifyStopped();
          } catch (Throwable t) {
            notifyFailed(t);
            throw Throwables.propagate(t);
          }
        }
      });
    }
  };
  




  protected abstract void startUp()
    throws Exception;
  




  protected abstract void shutDown()
    throws Exception;
  



  protected Executor executor(final Service.State state)
  {
    new Executor()
    {
      public void execute(Runnable command) {
        new Thread(command, AbstractIdleService.this.getServiceName() + " " + state).start();
      }
    };
  }
  
  public String toString() {
    return getServiceName() + " [" + state() + "]";
  }
  

  public final ListenableFuture<Service.State> start()
  {
    return delegate.start();
  }
  
  public final Service.State startAndWait() {
    return delegate.startAndWait();
  }
  
  public final boolean isRunning() {
    return delegate.isRunning();
  }
  
  public final Service.State state() {
    return delegate.state();
  }
  
  public final ListenableFuture<Service.State> stop() {
    return delegate.stop();
  }
  
  public final Service.State stopAndWait() {
    return delegate.stopAndWait();
  }
  
  private String getServiceName() {
    return getClass().getSimpleName();
  }
}
