package com.google.common.util.concurrent;

import com.google.common.annotations.Beta;
import com.google.common.base.Function;
import com.google.common.base.Preconditions;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.Lists;
import com.google.common.collect.Ordering;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.UndeclaredThrowableException;
import java.util.Arrays;
import java.util.List;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.CancellationException;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Executor;
import java.util.concurrent.Future;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;
import java.util.concurrent.atomic.AtomicInteger;
import javax.annotation.Nullable;

















































@Beta
public final class Futures
{
  @Deprecated
  public static <V> UninterruptibleFuture<V> makeUninterruptible(Future<V> future)
  {
    Preconditions.checkNotNull(future);
    if ((future instanceof UninterruptibleFuture)) {
      return (UninterruptibleFuture)future;
    }
    new UninterruptibleFuture()
    {
      public boolean cancel(boolean mayInterruptIfRunning) {
        return val$future.cancel(mayInterruptIfRunning);
      }
      
      public boolean isCancelled() {
        return val$future.isCancelled();
      }
      
      public boolean isDone() {
        return val$future.isDone();
      }
      
      public V get(long timeout, TimeUnit unit)
        throws TimeoutException, ExecutionException
      {
        return Uninterruptibles.getUninterruptibly(val$future, timeout, unit);
      }
      
      public V get() throws ExecutionException
      {
        return Uninterruptibles.getUninterruptibly(val$future);
      }
    };
  }
  






















  @Deprecated
  public static <V> ListenableFuture<V> makeListenable(Future<V> future)
  {
    return JdkFutureAdapters.listenInPoolThread(future);
  }
  






















  @Deprecated
  public static <V, X extends Exception> CheckedFuture<V, X> makeChecked(Future<V> future, Function<Exception, X> mapper)
  {
    return new MappingCheckedFuture(makeListenable(future), mapper);
  }
  












  public static <V, X extends Exception> CheckedFuture<V, X> makeChecked(ListenableFuture<V> future, Function<Exception, X> mapper)
  {
    return new MappingCheckedFuture((ListenableFuture)Preconditions.checkNotNull(future), mapper);
  }
  





  public static <V> ListenableFuture<V> immediateFuture(@Nullable V value)
  {
    SettableFuture<V> future = SettableFuture.create();
    future.set(value);
    return future;
  }
  








  public static <V, X extends Exception> CheckedFuture<V, X> immediateCheckedFuture(@Nullable V value)
  {
    SettableFuture<V> future = SettableFuture.create();
    future.set(value);
    makeChecked(future, new Function()
    {
      public X apply(Exception e) {
        throw new AssertionError("impossible");
      }
    });
  }
  











  public static <V> ListenableFuture<V> immediateFailedFuture(Throwable throwable)
  {
    Preconditions.checkNotNull(throwable);
    SettableFuture<V> future = SettableFuture.create();
    future.setException(throwable);
    return future;
  }
  












  public static <V, X extends Exception> CheckedFuture<V, X> immediateFailedCheckedFuture(X exception)
  {
    Preconditions.checkNotNull(exception);
    makeChecked(immediateFailedFuture(exception), new Function()
    {
      public X apply(Exception e)
      {
        return val$exception;
      }
    });
  }
  




















































  public static <I, O> ListenableFuture<O> chain(ListenableFuture<I> input, Function<? super I, ? extends ListenableFuture<? extends O>> function)
  {
    return chain(input, function, MoreExecutors.sameThreadExecutor());
  }
  















































  public static <I, O> ListenableFuture<O> chain(ListenableFuture<I> input, Function<? super I, ? extends ListenableFuture<? extends O>> function, Executor exec)
  {
    ChainingListenableFuture<I, O> chain = new ChainingListenableFuture(function, input, null);
    
    input.addListener(chain, exec);
    return chain;
  }
  















































  public static <I, O> ListenableFuture<O> transform(ListenableFuture<I> future, Function<? super I, ? extends O> function)
  {
    return transform(future, function, MoreExecutors.sameThreadExecutor());
  }
  
















































  public static <I, O> ListenableFuture<O> transform(ListenableFuture<I> future, Function<? super I, ? extends O> function, Executor exec)
  {
    Preconditions.checkNotNull(function);
    Function<I, ListenableFuture<O>> wrapperFunction = new Function()
    {
      public ListenableFuture<O> apply(I input) {
        O output = val$function.apply(input);
        return Futures.immediateFuture(output);
      }
    };
    return chain(future, wrapperFunction, exec);
  }
  























  @Beta
  public static <I, O> Future<O> lazyTransform(Future<I> future, final Function<? super I, ? extends O> function)
  {
    Preconditions.checkNotNull(future);
    Preconditions.checkNotNull(function);
    new Future()
    {
      public boolean cancel(boolean mayInterruptIfRunning)
      {
        return val$future.cancel(mayInterruptIfRunning);
      }
      
      public boolean isCancelled()
      {
        return val$future.isCancelled();
      }
      
      public boolean isDone()
      {
        return val$future.isDone();
      }
      
      public O get() throws InterruptedException, ExecutionException
      {
        return applyTransformation(val$future.get());
      }
      
      public O get(long timeout, TimeUnit unit)
        throws InterruptedException, ExecutionException, TimeoutException
      {
        return applyTransformation(val$future.get(timeout, unit));
      }
      
      private O applyTransformation(I input) throws ExecutionException {
        try {
          return function.apply(input);
        } catch (Throwable t) {
          throw new ExecutionException(t);
        }
      }
    };
  }
  






































  @Deprecated
  public static <I, O> Future<O> transform(Future<I> future, final Function<? super I, ? extends O> function)
  {
    if ((future instanceof ListenableFuture)) {
      return transform((ListenableFuture)future, function);
    }
    Preconditions.checkNotNull(future);
    Preconditions.checkNotNull(function);
    new Future()
    {





















      private final Object lock = new Object();
      private boolean set = false;
      private O value = null;
      private ExecutionException exception = null;
      
      public O get() throws InterruptedException, ExecutionException
      {
        return apply(val$future.get());
      }
      
      public O get(long timeout, TimeUnit unit)
        throws InterruptedException, ExecutionException, TimeoutException
      {
        return apply(val$future.get(timeout, unit));
      }
      
      private O apply(I raw) throws ExecutionException {
        synchronized (lock) {
          if (!set) {
            try {
              value = function.apply(raw);
            } catch (RuntimeException e) {
              exception = new ExecutionException(e);
            } catch (Error e) {
              exception = new ExecutionException(e);
            }
            set = true;
          }
          
          if (exception != null) {
            throw exception;
          }
          return value;
        }
      }
      
      public boolean cancel(boolean mayInterruptIfRunning)
      {
        return val$future.cancel(mayInterruptIfRunning);
      }
      
      public boolean isCancelled()
      {
        return val$future.isCancelled();
      }
      
      public boolean isDone()
      {
        return val$future.isDone();
      }
    };
  }
  



  private static class ChainingListenableFuture<I, O>
    extends AbstractFuture<O>
    implements Runnable
  {
    private Function<? super I, ? extends ListenableFuture<? extends O>> function;
    


    private ListenableFuture<? extends I> inputFuture;
    

    private volatile ListenableFuture<? extends O> outputFuture;
    

    private final BlockingQueue<Boolean> mayInterruptIfRunningChannel = new LinkedBlockingQueue(1);
    
    private final CountDownLatch outputCreated = new CountDownLatch(1);
    

    private ChainingListenableFuture(Function<? super I, ? extends ListenableFuture<? extends O>> function, ListenableFuture<? extends I> inputFuture)
    {
      this.function = ((Function)Preconditions.checkNotNull(function));
      this.inputFuture = ((ListenableFuture)Preconditions.checkNotNull(inputFuture));
    }
    




    public O get()
      throws InterruptedException, ExecutionException
    {
      if (!isDone())
      {



        ListenableFuture<? extends I> inputFuture = this.inputFuture;
        if (inputFuture != null) {
          inputFuture.get();
        }
        



        outputCreated.await();
        





        ListenableFuture<? extends O> outputFuture = this.outputFuture;
        if (outputFuture != null) {
          outputFuture.get();
        }
      }
      return super.get();
    }
    





    public O get(long timeout, TimeUnit unit)
      throws TimeoutException, ExecutionException, InterruptedException
    {
      if (!isDone())
      {

        if (unit != TimeUnit.NANOSECONDS) {
          timeout = TimeUnit.NANOSECONDS.convert(timeout, unit);
          unit = TimeUnit.NANOSECONDS;
        }
        




        ListenableFuture<? extends I> inputFuture = this.inputFuture;
        if (inputFuture != null) {
          long start = System.nanoTime();
          inputFuture.get(timeout, unit);
          timeout -= Math.max(0L, System.nanoTime() - start);
        }
        



        long start = System.nanoTime();
        if (!outputCreated.await(timeout, unit)) {
          throw new TimeoutException();
        }
        timeout -= Math.max(0L, System.nanoTime() - start);
        





        ListenableFuture<? extends O> outputFuture = this.outputFuture;
        if (outputFuture != null) {
          outputFuture.get(timeout, unit);
        }
      }
      return super.get(timeout, unit);
    }
    




    public boolean cancel(boolean mayInterruptIfRunning)
    {
      if (super.cancel(mayInterruptIfRunning))
      {

        Uninterruptibles.putUninterruptibly(mayInterruptIfRunningChannel, Boolean.valueOf(mayInterruptIfRunning));
        cancel(inputFuture, mayInterruptIfRunning);
        cancel(outputFuture, mayInterruptIfRunning);
        return true;
      }
      return false;
    }
    
    private void cancel(@Nullable Future<?> future, boolean mayInterruptIfRunning)
    {
      if (future != null) {
        future.cancel(mayInterruptIfRunning);
      }
    }
    
    public void run()
    {
      try {
        I sourceResult;
        try {
          sourceResult = Uninterruptibles.getUninterruptibly(inputFuture);

        }
        catch (CancellationException e)
        {
          cancel(false); return;
        }
        catch (ExecutionException e)
        {
          setException(e.getCause()); return;
        }
        

        final ListenableFuture<? extends O> outputFuture = this.outputFuture = (ListenableFuture)function.apply(sourceResult);
        
        if (isCancelled())
        {




          outputFuture.cancel(((Boolean)Uninterruptibles.takeUninterruptibly(mayInterruptIfRunningChannel)).booleanValue());
          
          this.outputFuture = null;
        }
        else {
          outputFuture.addListener(new Runnable()
          {

            public void run()
            {
              try
              {
                set(Uninterruptibles.getUninterruptibly(outputFuture));

              }
              catch (CancellationException e)
              {
                cancel(false);
              }
              catch (ExecutionException e)
              {
                setException(e.getCause());
              }
              finally {
                outputFuture = null; } } }, MoreExecutors.sameThreadExecutor());
        }
        
      }
      catch (UndeclaredThrowableException e)
      {
        setException(e.getCause());
      }
      catch (RuntimeException e)
      {
        setException(e);
      }
      catch (Error e) {
        setException(e);
      }
      finally {
        function = null;
        inputFuture = null;
        
        outputCreated.countDown();
      }
    }
  }
  
















  @Beta
  public static <V> ListenableFuture<List<V>> allAsList(ListenableFuture<? extends V>... futures)
  {
    return new ListFuture(ImmutableList.copyOf(futures), true, MoreExecutors.sameThreadExecutor());
  }
  

















  @Beta
  public static <V> ListenableFuture<List<V>> allAsList(Iterable<? extends ListenableFuture<? extends V>> futures)
  {
    return new ListFuture(ImmutableList.copyOf(futures), true, MoreExecutors.sameThreadExecutor());
  }
  














  @Beta
  public static <V> ListenableFuture<List<V>> successfulAsList(ListenableFuture<? extends V>... futures)
  {
    return new ListFuture(ImmutableList.copyOf(futures), false, MoreExecutors.sameThreadExecutor());
  }
  














  @Beta
  public static <V> ListenableFuture<List<V>> successfulAsList(Iterable<? extends ListenableFuture<? extends V>> futures)
  {
    return new ListFuture(ImmutableList.copyOf(futures), false, MoreExecutors.sameThreadExecutor());
  }
  











































  public static <V> void addCallback(ListenableFuture<V> future, FutureCallback<? super V> callback)
  {
    addCallback(future, callback, MoreExecutors.sameThreadExecutor());
  }
  















































  public static <V> void addCallback(ListenableFuture<V> future, final FutureCallback<? super V> callback, Executor executor)
  {
    Preconditions.checkNotNull(callback);
    Runnable callbackListener = new Runnable()
    {
      public void run()
      {
        try
        {
          V value = Uninterruptibles.getUninterruptibly(val$future);
          callback.onSuccess(value);
        } catch (ExecutionException e) {
          callback.onFailure(e.getCause());
        } catch (RuntimeException e) {
          callback.onFailure(e);
        } catch (Error e) {
          callback.onFailure(e);
        }
      }
    };
    future.addListener(callbackListener, executor);
  }
  














































  @Beta
  public static <V, X extends Exception> V get(Future<V> future, Class<X> exceptionClass)
    throws Exception
  {
    Preconditions.checkNotNull(future);
    Preconditions.checkArgument(!RuntimeException.class.isAssignableFrom(exceptionClass), "Futures.get exception type (%s) must not be a RuntimeException", new Object[] { exceptionClass });
    
    try
    {
      return future.get();
    } catch (InterruptedException e) {
      Thread.currentThread().interrupt();
      throw newWithCause(exceptionClass, e);
    } catch (ExecutionException e) {
      wrapAndThrowExceptionOrError(e.getCause(), exceptionClass);
      throw new AssertionError();
    }
  }
  
















































  @Beta
  public static <V, X extends Exception> V get(Future<V> future, long timeout, TimeUnit unit, Class<X> exceptionClass)
    throws Exception
  {
    Preconditions.checkNotNull(future);
    Preconditions.checkNotNull(unit);
    Preconditions.checkArgument(!RuntimeException.class.isAssignableFrom(exceptionClass), "Futures.get exception type (%s) must not be a RuntimeException", new Object[] { exceptionClass });
    
    try
    {
      return future.get(timeout, unit);
    } catch (InterruptedException e) {
      Thread.currentThread().interrupt();
      throw newWithCause(exceptionClass, e);
    } catch (TimeoutException e) {
      throw newWithCause(exceptionClass, e);
    } catch (ExecutionException e) {
      wrapAndThrowExceptionOrError(e.getCause(), exceptionClass);
      throw new AssertionError();
    }
  }
  
  private static <X extends Exception> void wrapAndThrowExceptionOrError(Throwable cause, Class<X> exceptionClass) throws Exception
  {
    if ((cause instanceof Error)) {
      throw new ExecutionError((Error)cause);
    }
    if ((cause instanceof RuntimeException)) {
      throw new UncheckedExecutionException(cause);
    }
    throw newWithCause(exceptionClass, cause);
  }
  




































  @Beta
  public static <V> V getUnchecked(Future<V> future)
  {
    Preconditions.checkNotNull(future);
    try {
      return Uninterruptibles.getUninterruptibly(future);
    } catch (ExecutionException e) {
      wrapAndThrowUnchecked(e.getCause());
      throw new AssertionError();
    }
  }
  
  private static void wrapAndThrowUnchecked(Throwable cause) {
    if ((cause instanceof Error)) {
      throw new ExecutionError((Error)cause);
    }
    




    throw new UncheckedExecutionException(cause);
  }
  



















  private static <X extends Exception> X newWithCause(Class<X> exceptionClass, Throwable cause)
  {
    List<Constructor<X>> constructors = Arrays.asList(exceptionClass.getConstructors());
    
    for (Constructor<X> constructor : preferringStrings(constructors)) {
      X instance = (Exception)newFromConstructor(constructor, cause);
      if (instance != null) {
        if (instance.getCause() == null) {
          instance.initCause(cause);
        }
        return instance;
      }
    }
    throw new IllegalArgumentException("No appropriate constructor for exception of type " + exceptionClass + " in response to chained exception", cause);
  }
  


  private static <X extends Exception> List<Constructor<X>> preferringStrings(List<Constructor<X>> constructors)
  {
    return WITH_STRING_PARAM_FIRST.sortedCopy(constructors);
  }
  
  private static final Ordering<Constructor<?>> WITH_STRING_PARAM_FIRST = Ordering.natural().onResultOf(new Function()
  {
    public Boolean apply(Constructor<?> input) {
      return Boolean.valueOf(Arrays.asList(input.getParameterTypes()).contains(String.class));
    }
  }).reverse();
  




  @Nullable
  private static <X> X newFromConstructor(Constructor<X> constructor, Throwable cause)
  {
    Class<?>[] paramTypes = constructor.getParameterTypes();
    Object[] params = new Object[paramTypes.length];
    for (int i = 0; i < paramTypes.length; i++) {
      Class<?> paramType = paramTypes[i];
      if (paramType.equals(String.class)) {
        params[i] = cause.toString();
      } else if (paramType.equals(Throwable.class)) {
        params[i] = cause;
      } else {
        return null;
      }
    }
    try {
      return constructor.newInstance(params);
    } catch (IllegalArgumentException e) {
      return null;
    } catch (InstantiationException e) {
      return null;
    } catch (IllegalAccessException e) {
      return null;
    } catch (InvocationTargetException e) {}
    return null;
  }
  



  private static class ListFuture<V>
    extends AbstractFuture<List<V>>
  {
    ImmutableList<? extends ListenableFuture<? extends V>> futures;
    


    final boolean allMustSucceed;
    


    final AtomicInteger remaining;
    


    List<V> values;
    



    ListFuture(ImmutableList<? extends ListenableFuture<? extends V>> futures, boolean allMustSucceed, Executor listenerExecutor)
    {
      this.futures = futures;
      values = Lists.newArrayListWithCapacity(futures.size());
      this.allMustSucceed = allMustSucceed;
      remaining = new AtomicInteger(futures.size());
      
      init(listenerExecutor);
    }
    
    private void init(Executor listenerExecutor)
    {
      addListener(new Runnable()
      {

        public void run()
        {
          values = null;
          

          futures = null; } }, MoreExecutors.sameThreadExecutor());
      





      if (futures.isEmpty()) {
        set(Lists.newArrayList(values));
        return;
      }
      

      for (int i = 0; i < futures.size(); i++) {
        values.add(null);
      }
      







      ImmutableList<? extends ListenableFuture<? extends V>> localFutures = futures;
      for (int i = 0; i < localFutures.size(); i++) {
        final ListenableFuture<? extends V> listenable = (ListenableFuture)localFutures.get(i);
        final int index = i;
        listenable.addListener(new Runnable()
        {

          public void run() { Futures.ListFuture.this.setOneValue(index, listenable); } }, listenerExecutor);
      }
    }
    




    private void setOneValue(int index, Future<? extends V> future)
    {
      List<V> localValues = values;
      if ((isDone()) || (localValues == null))
      {


        Preconditions.checkState(allMustSucceed, "Future was done before all dependencies completed");
        
        return;
      }
      try
      {
        Preconditions.checkState(future.isDone(), "Tried to set value from future which is not done");
        
        localValues.set(index, Uninterruptibles.getUninterruptibly(future));
      } catch (CancellationException e) { int newRemaining;
        if (allMustSucceed)
        {



          cancel(false); }
      } catch (ExecutionException e) {
        int newRemaining;
        if (allMustSucceed)
        {

          setException(e.getCause()); }
      } catch (RuntimeException e) {
        int newRemaining;
        if (allMustSucceed) {
          setException(e);
        }
      } catch (Error e) {
        int newRemaining;
        setException(e);
      } finally { int newRemaining;
        int newRemaining = remaining.decrementAndGet();
        Preconditions.checkState(newRemaining >= 0, "Less than 0 remaining futures");
        if (newRemaining == 0) {
          localValues = values;
          if (localValues != null) {
            set(Lists.newArrayList(localValues));
          } else {
            Preconditions.checkState(isDone());
          }
        }
      }
    }
    
    public List<V> get() throws InterruptedException, ExecutionException
    {
      callAllGets();
      


      return (List)super.get();
    }
    



    private void callAllGets()
      throws InterruptedException
    {
      List<? extends ListenableFuture<? extends V>> oldFutures = futures;
      if ((oldFutures != null) && (!isDone())) {
        for (ListenableFuture<? extends V> future : oldFutures)
        {


          while (!future.isDone()) {
            try {
              future.get();
            } catch (Error e) {
              throw e;
            } catch (InterruptedException e) {
              throw e;
            }
            catch (Throwable e) {
              if (allMustSucceed) {
                return;
              }
            }
          }
        }
      }
    }
  }
  



  private static class MappingCheckedFuture<V, X extends Exception>
    extends AbstractCheckedFuture<V, X>
  {
    final Function<Exception, X> mapper;
    



    MappingCheckedFuture(ListenableFuture<V> delegate, Function<Exception, X> mapper)
    {
      super();
      
      this.mapper = ((Function)Preconditions.checkNotNull(mapper));
    }
    
    protected X mapException(Exception e)
    {
      return (Exception)mapper.apply(e);
    }
  }
}
