package com.mysql.jdbc;

import com.mysql.jdbc.log.Log;
import java.io.EOFException;
import java.io.IOException;
import java.io.InputStream;
import java.sql.SQLException;
import java.util.zip.DataFormatException;
import java.util.zip.Inflater;











































class CompressedInputStream
  extends InputStream
{
  private byte[] buffer;
  private Connection connection;
  private InputStream in;
  private Inflater inflater;
  private byte[] packetHeaderBuffer = new byte[7];
  

  private int pos = 0;
  







  public CompressedInputStream(Connection conn, InputStream streamFromServer)
  {
    connection = conn;
    in = streamFromServer;
    inflater = new Inflater();
  }
  

  public int available()
    throws IOException
  {
    if (buffer == null) {
      return in.available();
    }
    
    return buffer.length - pos + in.available();
  }
  

  public void close()
    throws IOException
  {
    in.close();
    buffer = null;
    inflater = null;
  }
  





  private void getNextPacketFromServer()
    throws IOException
  {
    byte[] uncompressedData = null;
    
    int lengthRead = readFully(packetHeaderBuffer, 0, 7);
    
    if (lengthRead < 7) {
      throw new IOException("Unexpected end of input stream");
    }
    
    int compressedPacketLength = (packetHeaderBuffer[0] & 0xFF) + ((packetHeaderBuffer[1] & 0xFF) << 8) + ((packetHeaderBuffer[2] & 0xFF) << 16);
    


    int uncompressedLength = (packetHeaderBuffer[4] & 0xFF) + ((packetHeaderBuffer[5] & 0xFF) << 8) + ((packetHeaderBuffer[6] & 0xFF) << 16);
    


    if (connection.getTraceProtocol()) {
      try {
        connection.getLog().logTrace("Reading compressed packet of length " + compressedPacketLength + " uncompressed to " + uncompressedLength);

      }
      catch (SQLException sqlEx)
      {
        throw new IOException(sqlEx.toString());
      }
    }
    

    if (uncompressedLength > 0) {
      uncompressedData = new byte[uncompressedLength];
      
      byte[] compressedBuffer = new byte[compressedPacketLength];
      
      readFully(compressedBuffer, 0, compressedPacketLength);
      try
      {
        inflater.reset();
      } catch (NullPointerException npe) {
        inflater = new Inflater();
      }
      
      inflater.setInput(compressedBuffer);
      try
      {
        inflater.inflate(uncompressedData);
      } catch (DataFormatException dfe) {
        throw new IOException("Error while uncompressing packet from server.");
      }
      

      inflater.end();
    } else {
      if (connection.getTraceProtocol()) {
        try {
          connection.getLog().logTrace("Packet didn't meet compression threshold, not uncompressing...");

        }
        catch (SQLException sqlEx)
        {
          throw new IOException(sqlEx.toString());
        }
      }
      





      uncompressedData = new byte[compressedPacketLength];
      readFully(uncompressedData, 0, compressedPacketLength);
    }
    
    if (connection.getTraceProtocol()) {
      try {
        connection.getLog().logTrace("Uncompressed packet: \n" + StringUtils.dumpAsHex(uncompressedData, compressedPacketLength));

      }
      catch (SQLException sqlEx)
      {
        throw new IOException(sqlEx.toString());
      }
    }
    

    if ((buffer != null) && (pos < buffer.length)) {
      if (connection.getTraceProtocol()) {
        try {
          connection.getLog().logTrace("Combining remaining packet with new: ");
        }
        catch (SQLException sqlEx) {
          throw new IOException(sqlEx.toString());
        }
      }
      

      int remaining = buffer.length - pos;
      byte[] newBuffer = new byte[remaining + uncompressedData.length];
      
      int newIndex = 0;
      
      for (int i = pos; i < buffer.length; i++) {
        newBuffer[(newIndex++)] = buffer[i];
      }
      System.arraycopy(uncompressedData, 0, newBuffer, newIndex, uncompressedData.length);
      

      uncompressedData = newBuffer;
    }
    
    pos = 0;
    buffer = uncompressedData;
  }
  










  private void getNextPacketIfRequired(int numBytes)
    throws IOException
  {
    if ((buffer == null) || (pos + numBytes > buffer.length))
    {
      getNextPacketFromServer();
    }
  }
  
  public int read()
    throws IOException
  {
    try
    {
      getNextPacketIfRequired(1);
    } catch (IOException ioEx) {
      return -1;
    }
    
    return buffer[(pos++)] & 0xFF;
  }
  

  public int read(byte[] b)
    throws IOException
  {
    return read(b, 0, b.length);
  }
  

  public int read(byte[] b, int off, int len)
    throws IOException
  {
    if (b == null)
      throw new NullPointerException();
    if ((off < 0) || (off > b.length) || (len < 0) || (off + len > b.length) || (off + len < 0))
    {
      throw new IndexOutOfBoundsException();
    }
    
    if (len <= 0) {
      return 0;
    }
    try
    {
      getNextPacketIfRequired(len);
    } catch (IOException ioEx) {
      return -1;
    }
    
    System.arraycopy(buffer, pos, b, off, len);
    pos += len;
    
    return len;
  }
  
  private final int readFully(byte[] b, int off, int len) throws IOException {
    if (len < 0) {
      throw new IndexOutOfBoundsException();
    }
    
    int n = 0;
    
    while (n < len) {
      int count = in.read(b, off + n, len - n);
      
      if (count < 0) {
        throw new EOFException();
      }
      
      n += count;
    }
    
    return n;
  }
  

  public long skip(long n)
    throws IOException
  {
    long count = 0L;
    
    for (long i = 0L; i < n; i += 1L) {
      int bytesRead = read();
      
      if (bytesRead == -1) {
        break;
      }
      
      count += 1L;
    }
    
    return count;
  }
}
