package com.mysql.jdbc;

import java.sql.SQLException;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;





















public class ConnectionGroup
{
  private String groupName;
  private long connections = 0L;
  private long activeConnections = 0L;
  private HashMap<Long, LoadBalancingConnectionProxy> connectionProxies = new HashMap();
  private Set<String> hostList = new HashSet();
  private boolean isInitialized = false;
  private long closedProxyTotalPhysicalConnections = 0L;
  private long closedProxyTotalTransactions = 0L;
  private int activeHosts = 0;
  private Set<String> closedHosts = new HashSet();
  
  ConnectionGroup(String groupName) {
    this.groupName = groupName;
  }
  
  public long registerConnectionProxy(LoadBalancingConnectionProxy proxy, List<String> localHostList)
  {
    long currentConnectionId;
    synchronized (this) {
      if (!isInitialized) {
        hostList.addAll(localHostList);
        isInitialized = true;
        activeHosts = localHostList.size();
      }
      currentConnectionId = ++connections;
      connectionProxies.put(new Long(currentConnectionId), proxy);
    }
    activeConnections += 1L;
    
    return currentConnectionId;
  }
  



  public String getGroupName()
  {
    return groupName;
  }
  


  public Collection<String> getInitialHosts()
  {
    return hostList;
  }
  


  public int getActiveHostCount()
  {
    return activeHosts;
  }
  
  public Collection<String> getClosedHosts()
  {
    return closedHosts;
  }
  



  public long getTotalLogicalConnectionCount()
  {
    return connections;
  }
  


  public long getActiveLogicalConnectionCount()
  {
    return activeConnections;
  }
  

  public long getActivePhysicalConnectionCount()
  {
    long connections = 0L;
    Map<Long, LoadBalancingConnectionProxy> proxyMap = new HashMap();
    synchronized (connectionProxies) {
      proxyMap.putAll(connectionProxies);
    }
    Iterator<Map.Entry<Long, LoadBalancingConnectionProxy>> i = proxyMap.entrySet().iterator();
    while (i.hasNext()) {
      LoadBalancingConnectionProxy proxy = (LoadBalancingConnectionProxy)((Map.Entry)i.next()).getValue();
      connections += proxy.getActivePhysicalConnectionCount();
    }
    
    return connections;
  }
  


  public long getTotalPhysicalConnectionCount()
  {
    long allConnections = closedProxyTotalPhysicalConnections;
    Map<Long, LoadBalancingConnectionProxy> proxyMap = new HashMap();
    synchronized (connectionProxies) {
      proxyMap.putAll(connectionProxies);
    }
    Iterator<Map.Entry<Long, LoadBalancingConnectionProxy>> i = proxyMap.entrySet().iterator();
    while (i.hasNext()) {
      LoadBalancingConnectionProxy proxy = (LoadBalancingConnectionProxy)((Map.Entry)i.next()).getValue();
      allConnections += proxy.getTotalPhysicalConnectionCount();
    }
    
    return allConnections;
  }
  



  public long getTotalTransactionCount()
  {
    long transactions = closedProxyTotalTransactions;
    Map<Long, LoadBalancingConnectionProxy> proxyMap = new HashMap();
    synchronized (connectionProxies) {
      proxyMap.putAll(connectionProxies);
    }
    Iterator<Map.Entry<Long, LoadBalancingConnectionProxy>> i = proxyMap.entrySet().iterator();
    while (i.hasNext()) {
      LoadBalancingConnectionProxy proxy = (LoadBalancingConnectionProxy)((Map.Entry)i.next()).getValue();
      transactions += proxy.getTransactionCount();
    }
    
    return transactions;
  }
  
  public void closeConnectionProxy(LoadBalancingConnectionProxy proxy)
  {
    activeConnections -= 1L;
    connectionProxies.remove(new Long(proxy.getConnectionGroupProxyID()));
    closedProxyTotalPhysicalConnections += proxy.getTotalPhysicalConnectionCount();
    closedProxyTotalTransactions += proxy.getTransactionCount();
  }
  
  public void removeHost(String host) throws SQLException
  {
    removeHost(host, false);
  }
  
  public void removeHost(String host, boolean killExistingConnections) throws SQLException {
    removeHost(host, killExistingConnections, true);
  }
  

  public synchronized void removeHost(String host, boolean killExistingConnections, boolean waitForGracefulFailover)
    throws SQLException
  {
    if (activeHosts == 1) {
      throw SQLError.createSQLException("Cannot remove host, only one configured host active.", null);
    }
    
    if (hostList.remove(host)) {
      activeHosts -= 1;
    } else {
      throw SQLError.createSQLException("Host is not configured: " + host, null);
    }
    
    if (killExistingConnections)
    {
      Map<Long, LoadBalancingConnectionProxy> proxyMap = new HashMap();
      synchronized (connectionProxies) {
        proxyMap.putAll(connectionProxies);
      }
      
      Iterator<Map.Entry<Long, LoadBalancingConnectionProxy>> i = proxyMap.entrySet().iterator();
      while (i.hasNext()) {
        LoadBalancingConnectionProxy proxy = (LoadBalancingConnectionProxy)((Map.Entry)i.next()).getValue();
        if (waitForGracefulFailover) {
          proxy.removeHostWhenNotInUse(host);
        } else {
          proxy.removeHost(host);
        }
      }
    }
    closedHosts.add(host);
  }
  
  public void addHost(String host)
  {
    addHost(host, false);
  }
  




  public void addHost(String host, boolean forExisting)
  {
    synchronized (this) {
      if (hostList.add(host)) {
        activeHosts += 1;
      }
    }
    
    if (!forExisting) {
      return;
    }
    


    Map<Long, LoadBalancingConnectionProxy> proxyMap = new HashMap();
    synchronized (connectionProxies) {
      proxyMap.putAll(connectionProxies);
    }
    
    Object i = proxyMap.entrySet().iterator();
    while (((Iterator)i).hasNext()) {
      LoadBalancingConnectionProxy proxy = (LoadBalancingConnectionProxy)((Map.Entry)((Iterator)i).next()).getValue();
      proxy.addHost(host);
    }
  }
}
