package com.mysql.jdbc;

import com.mysql.jdbc.log.Log;
import com.mysql.jdbc.log.StandardLogger;
import java.io.Serializable;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.Field;
import java.sql.DriverPropertyInfo;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;
import java.util.TreeMap;
import javax.naming.RefAddr;
import javax.naming.Reference;
import javax.naming.StringRefAddr;







































public class ConnectionPropertiesImpl
  implements Serializable, ConnectionProperties
{
  private static final long serialVersionUID = 4257801713007640580L;
  
  class BooleanConnectionProperty
    extends ConnectionPropertiesImpl.ConnectionProperty
    implements Serializable
  {
    private static final long serialVersionUID = 2540132501709159404L;
    
    BooleanConnectionProperty(String propertyNameToSet, boolean defaultValueToSet, String descriptionToSet, String sinceVersionToSet, String category, int orderInCategory)
    {
      super(propertyNameToSet, Boolean.valueOf(defaultValueToSet), null, 0, 0, descriptionToSet, sinceVersionToSet, category, orderInCategory);
    }
    




    String[] getAllowableValues()
    {
      return new String[] { "true", "false", "yes", "no" };
    }
    
    boolean getValueAsBoolean() {
      return ((Boolean)valueAsObject).booleanValue();
    }
    


    boolean hasValueConstraints()
    {
      return true;
    }
    

    void initializeFrom(String extractedValue)
      throws SQLException
    {
      if (extractedValue != null) {
        validateStringValues(extractedValue);
        
        valueAsObject = Boolean.valueOf((extractedValue.equalsIgnoreCase("TRUE")) || (extractedValue.equalsIgnoreCase("YES")));
      }
      else
      {
        valueAsObject = defaultValue;
      }
    }
    


    boolean isRangeBased()
    {
      return false;
    }
    
    void setValue(boolean valueFlag) {
      valueAsObject = Boolean.valueOf(valueFlag);
    }
  }
  

  abstract class ConnectionProperty
    implements Serializable
  {
    String[] allowableValues;
    
    String categoryName;
    
    Object defaultValue;
    
    int lowerBound;
    
    int order;
    
    String propertyName;
    
    String sinceVersion;
    
    int upperBound;
    
    Object valueAsObject;
    
    boolean required;
    
    String description;
    
    public ConnectionProperty() {}
    
    ConnectionProperty(String propertyNameToSet, Object defaultValueToSet, String[] allowableValuesToSet, int lowerBoundToSet, int upperBoundToSet, String descriptionToSet, String sinceVersionToSet, String category, int orderInCategory)
    {
      description = descriptionToSet;
      propertyName = propertyNameToSet;
      defaultValue = defaultValueToSet;
      valueAsObject = defaultValueToSet;
      allowableValues = allowableValuesToSet;
      lowerBound = lowerBoundToSet;
      upperBound = upperBoundToSet;
      required = false;
      sinceVersion = sinceVersionToSet;
      categoryName = category;
      order = orderInCategory;
    }
    
    String[] getAllowableValues() {
      return allowableValues;
    }
    


    String getCategoryName()
    {
      return categoryName;
    }
    
    Object getDefaultValue() {
      return defaultValue;
    }
    
    int getLowerBound() {
      return lowerBound;
    }
    


    int getOrder()
    {
      return order;
    }
    
    String getPropertyName() {
      return propertyName;
    }
    
    int getUpperBound() {
      return upperBound;
    }
    
    Object getValueAsObject() {
      return valueAsObject;
    }
    
    abstract boolean hasValueConstraints();
    
    void initializeFrom(Properties extractFrom) throws SQLException {
      String extractedValue = extractFrom.getProperty(getPropertyName());
      extractFrom.remove(getPropertyName());
      initializeFrom(extractedValue);
    }
    
    void initializeFrom(Reference ref) throws SQLException {
      RefAddr refAddr = ref.get(getPropertyName());
      
      if (refAddr != null) {
        String refContentAsString = (String)refAddr.getContent();
        
        initializeFrom(refContentAsString);
      }
    }
    

    abstract void initializeFrom(String paramString)
      throws SQLException;
    

    abstract boolean isRangeBased();
    
    void setCategoryName(String categoryName)
    {
      this.categoryName = categoryName;
    }
    



    void setOrder(int order)
    {
      this.order = order;
    }
    
    void setValueAsObject(Object obj) {
      valueAsObject = obj;
    }
    
    void storeTo(Reference ref) {
      if (getValueAsObject() != null) {
        ref.add(new StringRefAddr(getPropertyName(), getValueAsObject().toString()));
      }
    }
    
    DriverPropertyInfo getAsDriverPropertyInfo()
    {
      DriverPropertyInfo dpi = new DriverPropertyInfo(propertyName, null);
      choices = getAllowableValues();
      value = (valueAsObject != null ? valueAsObject.toString() : null);
      required = required;
      description = description;
      
      return dpi;
    }
    
    void validateStringValues(String valueToValidate) throws SQLException
    {
      String[] validateAgainst = getAllowableValues();
      
      if (valueToValidate == null) {
        return;
      }
      
      if ((validateAgainst == null) || (validateAgainst.length == 0)) {
        return;
      }
      
      for (int i = 0; i < validateAgainst.length; i++) {
        if ((validateAgainst[i] != null) && (validateAgainst[i].equalsIgnoreCase(valueToValidate)))
        {
          return;
        }
      }
      
      StringBuffer errorMessageBuf = new StringBuffer();
      
      errorMessageBuf.append("The connection property '");
      errorMessageBuf.append(getPropertyName());
      errorMessageBuf.append("' only accepts values of the form: ");
      
      if (validateAgainst.length != 0) {
        errorMessageBuf.append("'");
        errorMessageBuf.append(validateAgainst[0]);
        errorMessageBuf.append("'");
        
        for (int i = 1; i < validateAgainst.length - 1; i++) {
          errorMessageBuf.append(", ");
          errorMessageBuf.append("'");
          errorMessageBuf.append(validateAgainst[i]);
          errorMessageBuf.append("'");
        }
        
        errorMessageBuf.append(" or '");
        errorMessageBuf.append(validateAgainst[(validateAgainst.length - 1)]);
        
        errorMessageBuf.append("'");
      }
      
      errorMessageBuf.append(". The value '");
      errorMessageBuf.append(valueToValidate);
      errorMessageBuf.append("' is not in this set.");
      
      throw SQLError.createSQLException(errorMessageBuf.toString(), "S1009", getExceptionInterceptor());
    }
  }
  

  class IntegerConnectionProperty
    extends ConnectionPropertiesImpl.ConnectionProperty
    implements Serializable
  {
    private static final long serialVersionUID = -3004305481796850832L;
    

    public IntegerConnectionProperty(String propertyNameToSet, Object defaultValueToSet, String[] allowableValuesToSet, int lowerBoundToSet, int upperBoundToSet, String descriptionToSet, String sinceVersionToSet, String category, int orderInCategory)
    {
      super(propertyNameToSet, defaultValueToSet, allowableValuesToSet, lowerBoundToSet, upperBoundToSet, descriptionToSet, sinceVersionToSet, category, orderInCategory);
    }
    


    int multiplier = 1;
    


    IntegerConnectionProperty(String propertyNameToSet, int defaultValueToSet, int lowerBoundToSet, int upperBoundToSet, String descriptionToSet, String sinceVersionToSet, String category, int orderInCategory)
    {
      super(propertyNameToSet, new Integer(defaultValueToSet), null, lowerBoundToSet, upperBoundToSet, descriptionToSet, sinceVersionToSet, category, orderInCategory);
    }
    













    IntegerConnectionProperty(String propertyNameToSet, int defaultValueToSet, String descriptionToSet, String sinceVersionToSet, String category, int orderInCategory)
    {
      this(propertyNameToSet, defaultValueToSet, 0, 0, descriptionToSet, sinceVersionToSet, category, orderInCategory);
    }
    



    String[] getAllowableValues()
    {
      return null;
    }
    


    int getLowerBound()
    {
      return lowerBound;
    }
    


    int getUpperBound()
    {
      return upperBound;
    }
    
    int getValueAsInt() {
      return ((Integer)valueAsObject).intValue();
    }
    


    boolean hasValueConstraints()
    {
      return false;
    }
    

    void initializeFrom(String extractedValue)
      throws SQLException
    {
      if (extractedValue != null) {
        try
        {
          int intValue = Double.valueOf(extractedValue).intValue();
          









          valueAsObject = new Integer(intValue * multiplier);
        } catch (NumberFormatException nfe) {
          throw SQLError.createSQLException("The connection property '" + getPropertyName() + "' only accepts integer values. The value '" + extractedValue + "' can not be converted to an integer.", "S1009", getExceptionInterceptor());

        }
        

      }
      else
      {
        valueAsObject = defaultValue;
      }
    }
    


    boolean isRangeBased()
    {
      return getUpperBound() != getLowerBound();
    }
    
    void setValue(int valueFlag) {
      valueAsObject = new Integer(valueFlag);
    }
  }
  

  public class LongConnectionProperty
    extends ConnectionPropertiesImpl.IntegerConnectionProperty
  {
    private static final long serialVersionUID = 6068572984340480895L;
    
    LongConnectionProperty(String propertyNameToSet, long defaultValueToSet, long lowerBoundToSet, long upperBoundToSet, String descriptionToSet, String sinceVersionToSet, String category, int orderInCategory)
    {
      super(propertyNameToSet, new Long(defaultValueToSet), null, (int)lowerBoundToSet, (int)upperBoundToSet, descriptionToSet, sinceVersionToSet, category, orderInCategory);
    }
    




    LongConnectionProperty(String propertyNameToSet, long defaultValueToSet, String descriptionToSet, String sinceVersionToSet, String category, int orderInCategory)
    {
      this(propertyNameToSet, defaultValueToSet, 0L, 0L, descriptionToSet, sinceVersionToSet, category, orderInCategory);
    }
    


    void setValue(long value)
    {
      valueAsObject = new Long(value);
    }
    
    long getValueAsLong() {
      return ((Long)valueAsObject).longValue();
    }
    
    void initializeFrom(String extractedValue) throws SQLException {
      if (extractedValue != null) {
        try
        {
          long longValue = Double.valueOf(extractedValue).longValue();
          
          valueAsObject = new Long(longValue);
        } catch (NumberFormatException nfe) {
          throw SQLError.createSQLException("The connection property '" + getPropertyName() + "' only accepts long integer values. The value '" + extractedValue + "' can not be converted to a long integer.", "S1009", getExceptionInterceptor());

        }
        

      }
      else
      {
        valueAsObject = defaultValue;
      }
    }
  }
  

  class MemorySizeConnectionProperty
    extends ConnectionPropertiesImpl.IntegerConnectionProperty
    implements Serializable
  {
    private static final long serialVersionUID = 7351065128998572656L;
    private String valueAsString;
    
    MemorySizeConnectionProperty(String propertyNameToSet, int defaultValueToSet, int lowerBoundToSet, int upperBoundToSet, String descriptionToSet, String sinceVersionToSet, String category, int orderInCategory)
    {
      super(propertyNameToSet, defaultValueToSet, lowerBoundToSet, upperBoundToSet, descriptionToSet, sinceVersionToSet, category, orderInCategory);
    }
    
    void initializeFrom(String extractedValue)
      throws SQLException
    {
      valueAsString = extractedValue;
      
      if (extractedValue != null) {
        if ((extractedValue.endsWith("k")) || (extractedValue.endsWith("K")) || (extractedValue.endsWith("kb")) || (extractedValue.endsWith("Kb")) || (extractedValue.endsWith("kB")))
        {



          multiplier = 1024;
          int indexOfK = StringUtils.indexOfIgnoreCase(extractedValue, "k");
          
          extractedValue = extractedValue.substring(0, indexOfK);
        } else if ((extractedValue.endsWith("m")) || (extractedValue.endsWith("M")) || (extractedValue.endsWith("G")) || (extractedValue.endsWith("mb")) || (extractedValue.endsWith("Mb")) || (extractedValue.endsWith("mB")))
        {




          multiplier = 1048576;
          int indexOfM = StringUtils.indexOfIgnoreCase(extractedValue, "m");
          
          extractedValue = extractedValue.substring(0, indexOfM);
        } else if ((extractedValue.endsWith("g")) || (extractedValue.endsWith("G")) || (extractedValue.endsWith("gb")) || (extractedValue.endsWith("Gb")) || (extractedValue.endsWith("gB")))
        {



          multiplier = 1073741824;
          int indexOfG = StringUtils.indexOfIgnoreCase(extractedValue, "g");
          
          extractedValue = extractedValue.substring(0, indexOfG);
        }
      }
      
      super.initializeFrom(extractedValue);
    }
    
    void setValue(String value) throws SQLException {
      initializeFrom(value);
    }
    
    String getValueAsString() {
      return valueAsString;
    }
  }
  
  class StringConnectionProperty
    extends ConnectionPropertiesImpl.ConnectionProperty implements Serializable
  {
    private static final long serialVersionUID = 5432127962785948272L;
    
    StringConnectionProperty(String propertyNameToSet, String defaultValueToSet, String descriptionToSet, String sinceVersionToSet, String category, int orderInCategory)
    {
      this(propertyNameToSet, defaultValueToSet, null, descriptionToSet, sinceVersionToSet, category, orderInCategory);
    }
    













    StringConnectionProperty(String propertyNameToSet, String defaultValueToSet, String[] allowableValuesToSet, String descriptionToSet, String sinceVersionToSet, String category, int orderInCategory)
    {
      super(propertyNameToSet, defaultValueToSet, allowableValuesToSet, 0, 0, descriptionToSet, sinceVersionToSet, category, orderInCategory);
    }
    

    String getValueAsString()
    {
      return (String)valueAsObject;
    }
    


    boolean hasValueConstraints()
    {
      return (allowableValues != null) && (allowableValues.length > 0);
    }
    


    void initializeFrom(String extractedValue)
      throws SQLException
    {
      if (extractedValue != null) {
        validateStringValues(extractedValue);
        
        valueAsObject = extractedValue;
      } else {
        valueAsObject = defaultValue;
      }
    }
    


    boolean isRangeBased()
    {
      return false;
    }
    
    void setValue(String valueFlag) {
      valueAsObject = valueFlag;
    }
  }
  
  private static final String CONNECTION_AND_AUTH_CATEGORY = Messages.getString("ConnectionProperties.categoryConnectionAuthentication");
  
  private static final String NETWORK_CATEGORY = Messages.getString("ConnectionProperties.categoryNetworking");
  
  private static final String DEBUGING_PROFILING_CATEGORY = Messages.getString("ConnectionProperties.categoryDebuggingProfiling");
  
  private static final String HA_CATEGORY = Messages.getString("ConnectionProperties.categorryHA");
  
  private static final String MISC_CATEGORY = Messages.getString("ConnectionProperties.categoryMisc");
  
  private static final String PERFORMANCE_CATEGORY = Messages.getString("ConnectionProperties.categoryPerformance");
  
  private static final String SECURITY_CATEGORY = Messages.getString("ConnectionProperties.categorySecurity");
  
  private static final String[] PROPERTY_CATEGORIES = { CONNECTION_AND_AUTH_CATEGORY, NETWORK_CATEGORY, HA_CATEGORY, SECURITY_CATEGORY, PERFORMANCE_CATEGORY, DEBUGING_PROFILING_CATEGORY, MISC_CATEGORY };
  



  private static final ArrayList PROPERTY_LIST = new ArrayList();
  



  private static final String STANDARD_LOGGER_NAME = StandardLogger.class.getName();
  
  protected static final String ZERO_DATETIME_BEHAVIOR_CONVERT_TO_NULL = "convertToNull";
  protected static final String ZERO_DATETIME_BEHAVIOR_EXCEPTION = "exception";
  protected static final String ZERO_DATETIME_BEHAVIOR_ROUND = "round";
  
  static
  {
    try
    {
      Field[] declaredFields = ConnectionPropertiesImpl.class.getDeclaredFields();
      

      for (int i = 0; i < declaredFields.length; i++) {
        if (ConnectionProperty.class.isAssignableFrom(declaredFields[i].getType()))
        {
          PROPERTY_LIST.add(declaredFields[i]);
        }
      }
    } catch (Exception ex) {
      RuntimeException rtEx = new RuntimeException();
      rtEx.initCause(ex);
      
      throw rtEx;
    }
  }
  
  public ExceptionInterceptor getExceptionInterceptor() {
    return null;
  }
  













  protected static DriverPropertyInfo[] exposeAsDriverPropertyInfo(Properties info, int slotsToReserve)
    throws SQLException
  {
    new ConnectionPropertiesImpl() {}.exposeAsDriverPropertyInfoInternal(info, slotsToReserve);
  }
  

  private BooleanConnectionProperty allowLoadLocalInfile = new BooleanConnectionProperty("allowLoadLocalInfile", true, Messages.getString("ConnectionProperties.loadDataLocal"), "3.0.3", SECURITY_CATEGORY, 2147483647);
  




  private BooleanConnectionProperty allowMultiQueries = new BooleanConnectionProperty("allowMultiQueries", false, Messages.getString("ConnectionProperties.allowMultiQueries"), "3.1.1", SECURITY_CATEGORY, 1);
  




  private BooleanConnectionProperty allowNanAndInf = new BooleanConnectionProperty("allowNanAndInf", false, Messages.getString("ConnectionProperties.allowNANandINF"), "3.1.5", MISC_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty allowUrlInLocalInfile = new BooleanConnectionProperty("allowUrlInLocalInfile", false, Messages.getString("ConnectionProperties.allowUrlInLoadLocal"), "3.1.4", SECURITY_CATEGORY, 2147483647);
  




  private BooleanConnectionProperty alwaysSendSetIsolation = new BooleanConnectionProperty("alwaysSendSetIsolation", true, Messages.getString("ConnectionProperties.alwaysSendSetIsolation"), "3.1.7", PERFORMANCE_CATEGORY, 2147483647);
  




  private BooleanConnectionProperty autoClosePStmtStreams = new BooleanConnectionProperty("autoClosePStmtStreams", false, Messages.getString("ConnectionProperties.autoClosePstmtStreams"), "3.1.12", MISC_CATEGORY, -2147483648);
  






  private BooleanConnectionProperty autoDeserialize = new BooleanConnectionProperty("autoDeserialize", false, Messages.getString("ConnectionProperties.autoDeserialize"), "3.1.5", MISC_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty autoGenerateTestcaseScript = new BooleanConnectionProperty("autoGenerateTestcaseScript", false, Messages.getString("ConnectionProperties.autoGenerateTestcaseScript"), "3.1.9", DEBUGING_PROFILING_CATEGORY, -2147483648);
  



  private boolean autoGenerateTestcaseScriptAsBoolean = false;
  
  private BooleanConnectionProperty autoReconnect = new BooleanConnectionProperty("autoReconnect", false, Messages.getString("ConnectionProperties.autoReconnect"), "1.1", HA_CATEGORY, 0);
  




  private BooleanConnectionProperty autoReconnectForPools = new BooleanConnectionProperty("autoReconnectForPools", false, Messages.getString("ConnectionProperties.autoReconnectForPools"), "3.1.3", HA_CATEGORY, 1);
  




  private boolean autoReconnectForPoolsAsBoolean = false;
  
  private MemorySizeConnectionProperty blobSendChunkSize = new MemorySizeConnectionProperty("blobSendChunkSize", 1048576, 1, 2147483647, Messages.getString("ConnectionProperties.blobSendChunkSize"), "3.1.9", PERFORMANCE_CATEGORY, -2147483648);
  






  private BooleanConnectionProperty autoSlowLog = new BooleanConnectionProperty("autoSlowLog", true, Messages.getString("ConnectionProperties.autoSlowLog"), "5.1.4", DEBUGING_PROFILING_CATEGORY, -2147483648);
  



  private BooleanConnectionProperty blobsAreStrings = new BooleanConnectionProperty("blobsAreStrings", false, "Should the driver always treat BLOBs as Strings - specifically to work around dubious metadata returned by the server for GROUP BY clauses?", "5.0.8", MISC_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty functionsNeverReturnBlobs = new BooleanConnectionProperty("functionsNeverReturnBlobs", false, "Should the driver always treat data from functions returning BLOBs as Strings - specifically to work around dubious metadata returned by the server for GROUP BY clauses?", "5.0.8", MISC_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty cacheCallableStatements = new BooleanConnectionProperty("cacheCallableStmts", false, Messages.getString("ConnectionProperties.cacheCallableStatements"), "3.1.2", PERFORMANCE_CATEGORY, -2147483648);
  



  private BooleanConnectionProperty cachePreparedStatements = new BooleanConnectionProperty("cachePrepStmts", false, Messages.getString("ConnectionProperties.cachePrepStmts"), "3.0.10", PERFORMANCE_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty cacheResultSetMetadata = new BooleanConnectionProperty("cacheResultSetMetadata", false, Messages.getString("ConnectionProperties.cacheRSMetadata"), "3.1.1", PERFORMANCE_CATEGORY, -2147483648);
  


  private boolean cacheResultSetMetaDataAsBoolean;
  


  private BooleanConnectionProperty cacheServerConfiguration = new BooleanConnectionProperty("cacheServerConfiguration", false, Messages.getString("ConnectionProperties.cacheServerConfiguration"), "3.1.5", PERFORMANCE_CATEGORY, -2147483648);
  




  private IntegerConnectionProperty callableStatementCacheSize = new IntegerConnectionProperty("callableStmtCacheSize", 100, 0, 2147483647, Messages.getString("ConnectionProperties.callableStmtCacheSize"), "3.1.2", PERFORMANCE_CATEGORY, 5);
  






  private BooleanConnectionProperty capitalizeTypeNames = new BooleanConnectionProperty("capitalizeTypeNames", true, Messages.getString("ConnectionProperties.capitalizeTypeNames"), "2.0.7", MISC_CATEGORY, -2147483648);
  




  private StringConnectionProperty characterEncoding = new StringConnectionProperty("characterEncoding", null, Messages.getString("ConnectionProperties.characterEncoding"), "1.1g", MISC_CATEGORY, 5);
  




  private String characterEncodingAsString = null;
  
  private StringConnectionProperty characterSetResults = new StringConnectionProperty("characterSetResults", null, Messages.getString("ConnectionProperties.characterSetResults"), "3.0.13", MISC_CATEGORY, 6);
  



  private StringConnectionProperty clientInfoProvider = new StringConnectionProperty("clientInfoProvider", "com.mysql.jdbc.JDBC4CommentClientInfoProvider", Messages.getString("ConnectionProperties.clientInfoProvider"), "5.1.0", DEBUGING_PROFILING_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty clobberStreamingResults = new BooleanConnectionProperty("clobberStreamingResults", false, Messages.getString("ConnectionProperties.clobberStreamingResults"), "3.0.9", MISC_CATEGORY, -2147483648);
  




  private StringConnectionProperty clobCharacterEncoding = new StringConnectionProperty("clobCharacterEncoding", null, Messages.getString("ConnectionProperties.clobCharacterEncoding"), "5.0.0", MISC_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty compensateOnDuplicateKeyUpdateCounts = new BooleanConnectionProperty("compensateOnDuplicateKeyUpdateCounts", false, Messages.getString("ConnectionProperties.compensateOnDuplicateKeyUpdateCounts"), "5.1.7", MISC_CATEGORY, -2147483648);
  



  private StringConnectionProperty connectionCollation = new StringConnectionProperty("connectionCollation", null, Messages.getString("ConnectionProperties.connectionCollation"), "3.0.13", MISC_CATEGORY, 7);
  




  private StringConnectionProperty connectionLifecycleInterceptors = new StringConnectionProperty("connectionLifecycleInterceptors", null, Messages.getString("ConnectionProperties.connectionLifecycleInterceptors"), "5.1.4", CONNECTION_AND_AUTH_CATEGORY, 2147483647);
  




  private IntegerConnectionProperty connectTimeout = new IntegerConnectionProperty("connectTimeout", 0, 0, 2147483647, Messages.getString("ConnectionProperties.connectTimeout"), "3.0.1", CONNECTION_AND_AUTH_CATEGORY, 9);
  



  private BooleanConnectionProperty continueBatchOnError = new BooleanConnectionProperty("continueBatchOnError", true, Messages.getString("ConnectionProperties.continueBatchOnError"), "3.0.3", MISC_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty createDatabaseIfNotExist = new BooleanConnectionProperty("createDatabaseIfNotExist", false, Messages.getString("ConnectionProperties.createDatabaseIfNotExist"), "3.1.9", MISC_CATEGORY, -2147483648);
  




  private IntegerConnectionProperty defaultFetchSize = new IntegerConnectionProperty("defaultFetchSize", 0, Messages.getString("ConnectionProperties.defaultFetchSize"), "3.1.9", PERFORMANCE_CATEGORY, -2147483648);
  
  private BooleanConnectionProperty detectServerPreparedStmts = new BooleanConnectionProperty("useServerPrepStmts", false, Messages.getString("ConnectionProperties.useServerPrepStmts"), "3.1.0", MISC_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty dontTrackOpenResources = new BooleanConnectionProperty("dontTrackOpenResources", false, Messages.getString("ConnectionProperties.dontTrackOpenResources"), "3.1.7", PERFORMANCE_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty dumpQueriesOnException = new BooleanConnectionProperty("dumpQueriesOnException", false, Messages.getString("ConnectionProperties.dumpQueriesOnException"), "3.1.3", DEBUGING_PROFILING_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty dynamicCalendars = new BooleanConnectionProperty("dynamicCalendars", false, Messages.getString("ConnectionProperties.dynamicCalendars"), "3.1.5", PERFORMANCE_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty elideSetAutoCommits = new BooleanConnectionProperty("elideSetAutoCommits", false, Messages.getString("ConnectionProperties.eliseSetAutoCommit"), "3.1.3", PERFORMANCE_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty emptyStringsConvertToZero = new BooleanConnectionProperty("emptyStringsConvertToZero", true, Messages.getString("ConnectionProperties.emptyStringsConvertToZero"), "3.1.8", MISC_CATEGORY, -2147483648);
  



  private BooleanConnectionProperty emulateLocators = new BooleanConnectionProperty("emulateLocators", false, Messages.getString("ConnectionProperties.emulateLocators"), "3.1.0", MISC_CATEGORY, -2147483648);
  


  private BooleanConnectionProperty emulateUnsupportedPstmts = new BooleanConnectionProperty("emulateUnsupportedPstmts", true, Messages.getString("ConnectionProperties.emulateUnsupportedPstmts"), "3.1.7", MISC_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty enablePacketDebug = new BooleanConnectionProperty("enablePacketDebug", false, Messages.getString("ConnectionProperties.enablePacketDebug"), "3.1.3", DEBUGING_PROFILING_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty enableQueryTimeouts = new BooleanConnectionProperty("enableQueryTimeouts", true, Messages.getString("ConnectionProperties.enableQueryTimeouts"), "5.0.6", PERFORMANCE_CATEGORY, -2147483648);
  





  private BooleanConnectionProperty explainSlowQueries = new BooleanConnectionProperty("explainSlowQueries", false, Messages.getString("ConnectionProperties.explainSlowQueries"), "3.1.2", DEBUGING_PROFILING_CATEGORY, -2147483648);
  




  private StringConnectionProperty exceptionInterceptors = new StringConnectionProperty("exceptionInterceptors", null, Messages.getString("ConnectionProperties.exceptionInterceptors"), "5.1.8", MISC_CATEGORY, -2147483648);
  





  private BooleanConnectionProperty failOverReadOnly = new BooleanConnectionProperty("failOverReadOnly", true, Messages.getString("ConnectionProperties.failoverReadOnly"), "3.0.12", HA_CATEGORY, 2);
  




  private BooleanConnectionProperty gatherPerformanceMetrics = new BooleanConnectionProperty("gatherPerfMetrics", false, Messages.getString("ConnectionProperties.gatherPerfMetrics"), "3.1.2", DEBUGING_PROFILING_CATEGORY, 1);
  




  private BooleanConnectionProperty generateSimpleParameterMetadata = new BooleanConnectionProperty("generateSimpleParameterMetadata", false, Messages.getString("ConnectionProperties.generateSimpleParameterMetadata"), "5.0.5", MISC_CATEGORY, -2147483648);
  

  private boolean highAvailabilityAsBoolean = false;
  
  private BooleanConnectionProperty holdResultsOpenOverStatementClose = new BooleanConnectionProperty("holdResultsOpenOverStatementClose", false, Messages.getString("ConnectionProperties.holdRSOpenOverStmtClose"), "3.1.7", PERFORMANCE_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty includeInnodbStatusInDeadlockExceptions = new BooleanConnectionProperty("includeInnodbStatusInDeadlockExceptions", false, "Include the output of \"SHOW ENGINE INNODB STATUS\" in exception messages when deadlock exceptions are detected?", "5.0.7", DEBUGING_PROFILING_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty ignoreNonTxTables = new BooleanConnectionProperty("ignoreNonTxTables", false, Messages.getString("ConnectionProperties.ignoreNonTxTables"), "3.0.9", MISC_CATEGORY, -2147483648);
  




  private IntegerConnectionProperty initialTimeout = new IntegerConnectionProperty("initialTimeout", 2, 1, 2147483647, Messages.getString("ConnectionProperties.initialTimeout"), "1.1", HA_CATEGORY, 5);
  



  private BooleanConnectionProperty isInteractiveClient = new BooleanConnectionProperty("interactiveClient", false, Messages.getString("ConnectionProperties.interactiveClient"), "3.1.0", CONNECTION_AND_AUTH_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty jdbcCompliantTruncation = new BooleanConnectionProperty("jdbcCompliantTruncation", true, Messages.getString("ConnectionProperties.jdbcCompliantTruncation"), "3.1.2", MISC_CATEGORY, -2147483648);
  




  private boolean jdbcCompliantTruncationForReads = jdbcCompliantTruncation.getValueAsBoolean();
  

  protected MemorySizeConnectionProperty largeRowSizeThreshold = new MemorySizeConnectionProperty("largeRowSizeThreshold", 2048, 0, 2147483647, Messages.getString("ConnectionProperties.largeRowSizeThreshold"), "5.1.1", PERFORMANCE_CATEGORY, -2147483648);
  



  private StringConnectionProperty loadBalanceStrategy = new StringConnectionProperty("loadBalanceStrategy", "random", null, Messages.getString("ConnectionProperties.loadBalanceStrategy"), "5.0.6", PERFORMANCE_CATEGORY, -2147483648);
  





  private IntegerConnectionProperty loadBalanceBlacklistTimeout = new IntegerConnectionProperty("loadBalanceBlacklistTimeout", 0, 0, 2147483647, Messages.getString("ConnectionProperties.loadBalanceBlacklistTimeout"), "5.1.0", MISC_CATEGORY, -2147483648);
  




  private IntegerConnectionProperty loadBalancePingTimeout = new IntegerConnectionProperty("loadBalancePingTimeout", 0, 0, 2147483647, Messages.getString("ConnectionProperties.loadBalancePingTimeout"), "5.1.13", MISC_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty loadBalanceValidateConnectionOnSwapServer = new BooleanConnectionProperty("loadBalanceValidateConnectionOnSwapServer", false, Messages.getString("ConnectionProperties.loadBalanceValidateConnectionOnSwapServer"), "5.1.13", MISC_CATEGORY, -2147483648);
  




  private StringConnectionProperty loadBalanceConnectionGroup = new StringConnectionProperty("loadBalanceConnectionGroup", null, Messages.getString("ConnectionProperties.loadBalanceConnectionGroup"), "5.1.13", MISC_CATEGORY, -2147483648);
  




  private StringConnectionProperty loadBalanceExceptionChecker = new StringConnectionProperty("loadBalanceExceptionChecker", "com.mysql.jdbc.StandardLoadBalanceExceptionChecker", null, Messages.getString("ConnectionProperties.loadBalanceExceptionChecker"), "5.1.13", MISC_CATEGORY, -2147483648);
  





  private StringConnectionProperty loadBalanceSQLStateFailover = new StringConnectionProperty("loadBalanceSQLStateFailover", null, Messages.getString("ConnectionProperties.loadBalanceSQLStateFailover"), "5.1.13", MISC_CATEGORY, -2147483648);
  




  private StringConnectionProperty loadBalanceSQLExceptionSubclassFailover = new StringConnectionProperty("loadBalanceSQLExceptionSubclassFailover", null, Messages.getString("ConnectionProperties.loadBalanceSQLExceptionSubclassFailover"), "5.1.13", MISC_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty loadBalanceEnableJMX = new BooleanConnectionProperty("loadBalanceEnableJMX", false, Messages.getString("ConnectionProperties.loadBalanceEnableJMX"), "5.1.13", MISC_CATEGORY, 2147483647);
  




  private StringConnectionProperty loadBalanceAutoCommitStatementRegex = new StringConnectionProperty("loadBalanceAutoCommitStatementRegex", null, Messages.getString("ConnectionProperties.loadBalanceAutoCommitStatementRegex"), "5.1.15", MISC_CATEGORY, -2147483648);
  




  private IntegerConnectionProperty loadBalanceAutoCommitStatementThreshold = new IntegerConnectionProperty("loadBalanceAutoCommitStatementThreshold", 0, 0, 2147483647, Messages.getString("ConnectionProperties.loadBalanceAutoCommitStatementThreshold"), "5.1.15", MISC_CATEGORY, -2147483648);
  





  private StringConnectionProperty localSocketAddress = new StringConnectionProperty("localSocketAddress", null, Messages.getString("ConnectionProperties.localSocketAddress"), "5.0.5", CONNECTION_AND_AUTH_CATEGORY, -2147483648);
  


  private MemorySizeConnectionProperty locatorFetchBufferSize = new MemorySizeConnectionProperty("locatorFetchBufferSize", 1048576, 0, 2147483647, Messages.getString("ConnectionProperties.locatorFetchBufferSize"), "3.2.1", PERFORMANCE_CATEGORY, -2147483648);
  






  private StringConnectionProperty loggerClassName = new StringConnectionProperty("logger", STANDARD_LOGGER_NAME, Messages.getString("ConnectionProperties.logger", new Object[] { Log.class.getName(), STANDARD_LOGGER_NAME }), "3.1.1", DEBUGING_PROFILING_CATEGORY, 0);
  




  private BooleanConnectionProperty logSlowQueries = new BooleanConnectionProperty("logSlowQueries", false, Messages.getString("ConnectionProperties.logSlowQueries"), "3.1.2", DEBUGING_PROFILING_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty logXaCommands = new BooleanConnectionProperty("logXaCommands", false, Messages.getString("ConnectionProperties.logXaCommands"), "5.0.5", DEBUGING_PROFILING_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty maintainTimeStats = new BooleanConnectionProperty("maintainTimeStats", true, Messages.getString("ConnectionProperties.maintainTimeStats"), "3.1.9", PERFORMANCE_CATEGORY, 2147483647);
  




  private boolean maintainTimeStatsAsBoolean = true;
  
  private IntegerConnectionProperty maxQuerySizeToLog = new IntegerConnectionProperty("maxQuerySizeToLog", 2048, 0, 2147483647, Messages.getString("ConnectionProperties.maxQuerySizeToLog"), "3.1.3", DEBUGING_PROFILING_CATEGORY, 4);
  






  private IntegerConnectionProperty maxReconnects = new IntegerConnectionProperty("maxReconnects", 3, 1, 2147483647, Messages.getString("ConnectionProperties.maxReconnects"), "1.1", HA_CATEGORY, 4);
  






  private IntegerConnectionProperty retriesAllDown = new IntegerConnectionProperty("retriesAllDown", 120, 0, 2147483647, Messages.getString("ConnectionProperties.retriesAllDown"), "5.1.6", HA_CATEGORY, 4);
  






  private IntegerConnectionProperty maxRows = new IntegerConnectionProperty("maxRows", -1, -1, 2147483647, Messages.getString("ConnectionProperties.maxRows"), Messages.getString("ConnectionProperties.allVersions"), MISC_CATEGORY, -2147483648);
  



  private int maxRowsAsInt = -1;
  
  private IntegerConnectionProperty metadataCacheSize = new IntegerConnectionProperty("metadataCacheSize", 50, 1, 2147483647, Messages.getString("ConnectionProperties.metadataCacheSize"), "3.1.1", PERFORMANCE_CATEGORY, 5);
  






  private IntegerConnectionProperty netTimeoutForStreamingResults = new IntegerConnectionProperty("netTimeoutForStreamingResults", 600, 0, 2147483647, Messages.getString("ConnectionProperties.netTimeoutForStreamingResults"), "5.1.0", MISC_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty noAccessToProcedureBodies = new BooleanConnectionProperty("noAccessToProcedureBodies", false, "When determining procedure parameter types for CallableStatements, and the connected user  can't access procedure bodies through \"SHOW CREATE PROCEDURE\" or select on mysql.proc  should the driver instead create basic metadata (all parameters reported as IN VARCHARs, but allowing registerOutParameter() to be called on them anyway) instead  of throwing an exception?", "5.0.3", MISC_CATEGORY, -2147483648);
  








  private BooleanConnectionProperty noDatetimeStringSync = new BooleanConnectionProperty("noDatetimeStringSync", false, Messages.getString("ConnectionProperties.noDatetimeStringSync"), "3.1.7", MISC_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty noTimezoneConversionForTimeType = new BooleanConnectionProperty("noTimezoneConversionForTimeType", false, Messages.getString("ConnectionProperties.noTzConversionForTimeType"), "5.0.0", MISC_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty nullCatalogMeansCurrent = new BooleanConnectionProperty("nullCatalogMeansCurrent", true, Messages.getString("ConnectionProperties.nullCatalogMeansCurrent"), "3.1.8", MISC_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty nullNamePatternMatchesAll = new BooleanConnectionProperty("nullNamePatternMatchesAll", true, Messages.getString("ConnectionProperties.nullNamePatternMatchesAll"), "3.1.8", MISC_CATEGORY, -2147483648);
  




  private IntegerConnectionProperty packetDebugBufferSize = new IntegerConnectionProperty("packetDebugBufferSize", 20, 0, 2147483647, Messages.getString("ConnectionProperties.packetDebugBufferSize"), "3.1.3", DEBUGING_PROFILING_CATEGORY, 7);
  






  private BooleanConnectionProperty padCharsWithSpace = new BooleanConnectionProperty("padCharsWithSpace", false, Messages.getString("ConnectionProperties.padCharsWithSpace"), "5.0.6", MISC_CATEGORY, -2147483648);
  






  private BooleanConnectionProperty paranoid = new BooleanConnectionProperty("paranoid", false, Messages.getString("ConnectionProperties.paranoid"), "3.0.1", SECURITY_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty pedantic = new BooleanConnectionProperty("pedantic", false, Messages.getString("ConnectionProperties.pedantic"), "3.0.0", MISC_CATEGORY, -2147483648);
  


  private BooleanConnectionProperty pinGlobalTxToPhysicalConnection = new BooleanConnectionProperty("pinGlobalTxToPhysicalConnection", false, Messages.getString("ConnectionProperties.pinGlobalTxToPhysicalConnection"), "5.0.1", MISC_CATEGORY, -2147483648);
  


  private BooleanConnectionProperty populateInsertRowWithDefaultValues = new BooleanConnectionProperty("populateInsertRowWithDefaultValues", false, Messages.getString("ConnectionProperties.populateInsertRowWithDefaultValues"), "5.0.5", MISC_CATEGORY, -2147483648);
  



  private IntegerConnectionProperty preparedStatementCacheSize = new IntegerConnectionProperty("prepStmtCacheSize", 25, 0, 2147483647, Messages.getString("ConnectionProperties.prepStmtCacheSize"), "3.0.10", PERFORMANCE_CATEGORY, 10);
  



  private IntegerConnectionProperty preparedStatementCacheSqlLimit = new IntegerConnectionProperty("prepStmtCacheSqlLimit", 256, 1, 2147483647, Messages.getString("ConnectionProperties.prepStmtCacheSqlLimit"), "3.0.10", PERFORMANCE_CATEGORY, 11);
  






  private BooleanConnectionProperty processEscapeCodesForPrepStmts = new BooleanConnectionProperty("processEscapeCodesForPrepStmts", true, Messages.getString("ConnectionProperties.processEscapeCodesForPrepStmts"), "3.1.12", MISC_CATEGORY, -2147483648);
  





  private StringConnectionProperty profilerEventHandler = new StringConnectionProperty("profilerEventHandler", "com.mysql.jdbc.profiler.LoggingProfilerEventHandler", Messages.getString("ConnectionProperties.profilerEventHandler"), "5.1.6", DEBUGING_PROFILING_CATEGORY, -2147483648);
  




  private StringConnectionProperty profileSql = new StringConnectionProperty("profileSql", null, Messages.getString("ConnectionProperties.profileSqlDeprecated"), "2.0.14", DEBUGING_PROFILING_CATEGORY, 3);
  




  private BooleanConnectionProperty profileSQL = new BooleanConnectionProperty("profileSQL", false, Messages.getString("ConnectionProperties.profileSQL"), "3.1.0", DEBUGING_PROFILING_CATEGORY, 1);
  




  private boolean profileSQLAsBoolean = false;
  
  private StringConnectionProperty propertiesTransform = new StringConnectionProperty("propertiesTransform", null, Messages.getString("ConnectionProperties.connectionPropertiesTransform"), "3.1.4", CONNECTION_AND_AUTH_CATEGORY, -2147483648);
  




  private IntegerConnectionProperty queriesBeforeRetryMaster = new IntegerConnectionProperty("queriesBeforeRetryMaster", 50, 1, 2147483647, Messages.getString("ConnectionProperties.queriesBeforeRetryMaster"), "3.0.2", HA_CATEGORY, 7);
  






  private BooleanConnectionProperty queryTimeoutKillsConnection = new BooleanConnectionProperty("queryTimeoutKillsConnection", false, Messages.getString("ConnectionProperties.queryTimeoutKillsConnection"), "5.1.9", MISC_CATEGORY, -2147483648);
  


  private BooleanConnectionProperty reconnectAtTxEnd = new BooleanConnectionProperty("reconnectAtTxEnd", false, Messages.getString("ConnectionProperties.reconnectAtTxEnd"), "3.0.10", HA_CATEGORY, 4);
  



  private boolean reconnectTxAtEndAsBoolean = false;
  
  private BooleanConnectionProperty relaxAutoCommit = new BooleanConnectionProperty("relaxAutoCommit", false, Messages.getString("ConnectionProperties.relaxAutoCommit"), "2.0.13", MISC_CATEGORY, -2147483648);
  




  private IntegerConnectionProperty reportMetricsIntervalMillis = new IntegerConnectionProperty("reportMetricsIntervalMillis", 30000, 0, 2147483647, Messages.getString("ConnectionProperties.reportMetricsIntervalMillis"), "3.1.2", DEBUGING_PROFILING_CATEGORY, 3);
  






  private BooleanConnectionProperty requireSSL = new BooleanConnectionProperty("requireSSL", false, Messages.getString("ConnectionProperties.requireSSL"), "3.1.0", SECURITY_CATEGORY, 3);
  



  private StringConnectionProperty resourceId = new StringConnectionProperty("resourceId", null, Messages.getString("ConnectionProperties.resourceId"), "5.0.1", HA_CATEGORY, -2147483648);
  





  private IntegerConnectionProperty resultSetSizeThreshold = new IntegerConnectionProperty("resultSetSizeThreshold", 100, Messages.getString("ConnectionProperties.resultSetSizeThreshold"), "5.0.5", DEBUGING_PROFILING_CATEGORY, -2147483648);
  

  private BooleanConnectionProperty retainStatementAfterResultSetClose = new BooleanConnectionProperty("retainStatementAfterResultSetClose", false, Messages.getString("ConnectionProperties.retainStatementAfterResultSetClose"), "3.1.11", MISC_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty rewriteBatchedStatements = new BooleanConnectionProperty("rewriteBatchedStatements", false, Messages.getString("ConnectionProperties.rewriteBatchedStatements"), "3.1.13", PERFORMANCE_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty rollbackOnPooledClose = new BooleanConnectionProperty("rollbackOnPooledClose", true, Messages.getString("ConnectionProperties.rollbackOnPooledClose"), "3.0.15", MISC_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty roundRobinLoadBalance = new BooleanConnectionProperty("roundRobinLoadBalance", false, Messages.getString("ConnectionProperties.roundRobinLoadBalance"), "3.1.2", HA_CATEGORY, 5);
  




  private BooleanConnectionProperty runningCTS13 = new BooleanConnectionProperty("runningCTS13", false, Messages.getString("ConnectionProperties.runningCTS13"), "3.1.7", MISC_CATEGORY, -2147483648);
  




  private IntegerConnectionProperty secondsBeforeRetryMaster = new IntegerConnectionProperty("secondsBeforeRetryMaster", 30, 1, 2147483647, Messages.getString("ConnectionProperties.secondsBeforeRetryMaster"), "3.0.2", HA_CATEGORY, 8);
  






  private IntegerConnectionProperty selfDestructOnPingSecondsLifetime = new IntegerConnectionProperty("selfDestructOnPingSecondsLifetime", 0, 0, 2147483647, Messages.getString("ConnectionProperties.selfDestructOnPingSecondsLifetime"), "5.1.6", HA_CATEGORY, 2147483647);
  






  private IntegerConnectionProperty selfDestructOnPingMaxOperations = new IntegerConnectionProperty("selfDestructOnPingMaxOperations", 0, 0, 2147483647, Messages.getString("ConnectionProperties.selfDestructOnPingMaxOperations"), "5.1.6", HA_CATEGORY, 2147483647);
  






  private StringConnectionProperty serverTimezone = new StringConnectionProperty("serverTimezone", null, Messages.getString("ConnectionProperties.serverTimezone"), "3.0.2", MISC_CATEGORY, -2147483648);
  




  private StringConnectionProperty sessionVariables = new StringConnectionProperty("sessionVariables", null, Messages.getString("ConnectionProperties.sessionVariables"), "3.1.8", MISC_CATEGORY, 2147483647);
  



  private IntegerConnectionProperty slowQueryThresholdMillis = new IntegerConnectionProperty("slowQueryThresholdMillis", 2000, 0, 2147483647, Messages.getString("ConnectionProperties.slowQueryThresholdMillis"), "3.1.2", DEBUGING_PROFILING_CATEGORY, 9);
  






  private LongConnectionProperty slowQueryThresholdNanos = new LongConnectionProperty("slowQueryThresholdNanos", 0L, Messages.getString("ConnectionProperties.slowQueryThresholdNanos"), "5.0.7", DEBUGING_PROFILING_CATEGORY, 10);
  






  private StringConnectionProperty socketFactoryClassName = new StringConnectionProperty("socketFactory", StandardSocketFactory.class.getName(), Messages.getString("ConnectionProperties.socketFactory"), "3.0.3", CONNECTION_AND_AUTH_CATEGORY, 4);
  




  private IntegerConnectionProperty socketTimeout = new IntegerConnectionProperty("socketTimeout", 0, 0, 2147483647, Messages.getString("ConnectionProperties.socketTimeout"), "3.0.1", CONNECTION_AND_AUTH_CATEGORY, 10);
  






  private StringConnectionProperty statementInterceptors = new StringConnectionProperty("statementInterceptors", null, Messages.getString("ConnectionProperties.statementInterceptors"), "5.1.1", MISC_CATEGORY, -2147483648);
  

  private BooleanConnectionProperty strictFloatingPoint = new BooleanConnectionProperty("strictFloatingPoint", false, Messages.getString("ConnectionProperties.strictFloatingPoint"), "3.0.0", MISC_CATEGORY, -2147483648);
  



  private BooleanConnectionProperty strictUpdates = new BooleanConnectionProperty("strictUpdates", true, Messages.getString("ConnectionProperties.strictUpdates"), "3.0.4", MISC_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty overrideSupportsIntegrityEnhancementFacility = new BooleanConnectionProperty("overrideSupportsIntegrityEnhancementFacility", false, Messages.getString("ConnectionProperties.overrideSupportsIEF"), "3.1.12", MISC_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty tcpNoDelay = new BooleanConnectionProperty("tcpNoDelay", Boolean.valueOf("true").booleanValue(), Messages.getString("ConnectionProperties.tcpNoDelay"), "5.0.7", NETWORK_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty tcpKeepAlive = new BooleanConnectionProperty("tcpKeepAlive", Boolean.valueOf("true").booleanValue(), Messages.getString("ConnectionProperties.tcpKeepAlive"), "5.0.7", NETWORK_CATEGORY, -2147483648);
  




  private IntegerConnectionProperty tcpRcvBuf = new IntegerConnectionProperty("tcpRcvBuf", Integer.parseInt("0"), 0, 2147483647, Messages.getString("ConnectionProperties.tcpSoRcvBuf"), "5.0.7", NETWORK_CATEGORY, -2147483648);
  





  private IntegerConnectionProperty tcpSndBuf = new IntegerConnectionProperty("tcpSndBuf", Integer.parseInt("0"), 0, 2147483647, Messages.getString("ConnectionProperties.tcpSoSndBuf"), "5.0.7", NETWORK_CATEGORY, -2147483648);
  





  private IntegerConnectionProperty tcpTrafficClass = new IntegerConnectionProperty("tcpTrafficClass", Integer.parseInt("0"), 0, 255, Messages.getString("ConnectionProperties.tcpTrafficClass"), "5.0.7", NETWORK_CATEGORY, -2147483648);
  





  private BooleanConnectionProperty tinyInt1isBit = new BooleanConnectionProperty("tinyInt1isBit", true, Messages.getString("ConnectionProperties.tinyInt1isBit"), "3.0.16", MISC_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty traceProtocol = new BooleanConnectionProperty("traceProtocol", false, Messages.getString("ConnectionProperties.traceProtocol"), "3.1.2", DEBUGING_PROFILING_CATEGORY, -2147483648);
  



  private BooleanConnectionProperty treatUtilDateAsTimestamp = new BooleanConnectionProperty("treatUtilDateAsTimestamp", true, Messages.getString("ConnectionProperties.treatUtilDateAsTimestamp"), "5.0.5", MISC_CATEGORY, -2147483648);
  



  private BooleanConnectionProperty transformedBitIsBoolean = new BooleanConnectionProperty("transformedBitIsBoolean", false, Messages.getString("ConnectionProperties.transformedBitIsBoolean"), "3.1.9", MISC_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty useBlobToStoreUTF8OutsideBMP = new BooleanConnectionProperty("useBlobToStoreUTF8OutsideBMP", false, Messages.getString("ConnectionProperties.useBlobToStoreUTF8OutsideBMP"), "5.1.3", MISC_CATEGORY, 128);
  




  private StringConnectionProperty utf8OutsideBmpExcludedColumnNamePattern = new StringConnectionProperty("utf8OutsideBmpExcludedColumnNamePattern", null, Messages.getString("ConnectionProperties.utf8OutsideBmpExcludedColumnNamePattern"), "5.1.3", MISC_CATEGORY, 129);
  




  private StringConnectionProperty utf8OutsideBmpIncludedColumnNamePattern = new StringConnectionProperty("utf8OutsideBmpIncludedColumnNamePattern", null, Messages.getString("ConnectionProperties.utf8OutsideBmpIncludedColumnNamePattern"), "5.1.3", MISC_CATEGORY, 129);
  




  private BooleanConnectionProperty useCompression = new BooleanConnectionProperty("useCompression", false, Messages.getString("ConnectionProperties.useCompression"), "3.0.17", CONNECTION_AND_AUTH_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty useColumnNamesInFindColumn = new BooleanConnectionProperty("useColumnNamesInFindColumn", false, Messages.getString("ConnectionProperties.useColumnNamesInFindColumn"), "5.1.7", MISC_CATEGORY, 2147483647);
  




  private StringConnectionProperty useConfigs = new StringConnectionProperty("useConfigs", null, Messages.getString("ConnectionProperties.useConfigs"), "3.1.5", CONNECTION_AND_AUTH_CATEGORY, 2147483647);
  




  private BooleanConnectionProperty useCursorFetch = new BooleanConnectionProperty("useCursorFetch", false, Messages.getString("ConnectionProperties.useCursorFetch"), "5.0.0", PERFORMANCE_CATEGORY, 2147483647);
  




  private BooleanConnectionProperty useDynamicCharsetInfo = new BooleanConnectionProperty("useDynamicCharsetInfo", true, Messages.getString("ConnectionProperties.useDynamicCharsetInfo"), "5.0.6", PERFORMANCE_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty useDirectRowUnpack = new BooleanConnectionProperty("useDirectRowUnpack", true, "Use newer result set row unpacking code that skips a copy from network buffers  to a MySQL packet instance and instead reads directly into the result set row data buffers.", "5.1.1", PERFORMANCE_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty useFastIntParsing = new BooleanConnectionProperty("useFastIntParsing", true, Messages.getString("ConnectionProperties.useFastIntParsing"), "3.1.4", PERFORMANCE_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty useFastDateParsing = new BooleanConnectionProperty("useFastDateParsing", true, Messages.getString("ConnectionProperties.useFastDateParsing"), "5.0.5", PERFORMANCE_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty useHostsInPrivileges = new BooleanConnectionProperty("useHostsInPrivileges", true, Messages.getString("ConnectionProperties.useHostsInPrivileges"), "3.0.2", MISC_CATEGORY, -2147483648);
  



  private BooleanConnectionProperty useInformationSchema = new BooleanConnectionProperty("useInformationSchema", false, Messages.getString("ConnectionProperties.useInformationSchema"), "5.0.0", MISC_CATEGORY, -2147483648);
  



  private BooleanConnectionProperty useJDBCCompliantTimezoneShift = new BooleanConnectionProperty("useJDBCCompliantTimezoneShift", false, Messages.getString("ConnectionProperties.useJDBCCompliantTimezoneShift"), "5.0.0", MISC_CATEGORY, -2147483648);
  





  private BooleanConnectionProperty useLocalSessionState = new BooleanConnectionProperty("useLocalSessionState", false, Messages.getString("ConnectionProperties.useLocalSessionState"), "3.1.7", PERFORMANCE_CATEGORY, 5);
  




  private BooleanConnectionProperty useLocalTransactionState = new BooleanConnectionProperty("useLocalTransactionState", false, Messages.getString("ConnectionProperties.useLocalTransactionState"), "5.1.7", PERFORMANCE_CATEGORY, 6);
  




  private BooleanConnectionProperty useLegacyDatetimeCode = new BooleanConnectionProperty("useLegacyDatetimeCode", true, Messages.getString("ConnectionProperties.useLegacyDatetimeCode"), "5.1.6", MISC_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty useNanosForElapsedTime = new BooleanConnectionProperty("useNanosForElapsedTime", false, Messages.getString("ConnectionProperties.useNanosForElapsedTime"), "5.0.7", DEBUGING_PROFILING_CATEGORY, -2147483648);
  





  private BooleanConnectionProperty useOldAliasMetadataBehavior = new BooleanConnectionProperty("useOldAliasMetadataBehavior", false, Messages.getString("ConnectionProperties.useOldAliasMetadataBehavior"), "5.0.4", MISC_CATEGORY, -2147483648);
  






  private BooleanConnectionProperty useOldUTF8Behavior = new BooleanConnectionProperty("useOldUTF8Behavior", false, Messages.getString("ConnectionProperties.useOldUtf8Behavior"), "3.1.6", MISC_CATEGORY, -2147483648);
  




  private boolean useOldUTF8BehaviorAsBoolean = false;
  
  private BooleanConnectionProperty useOnlyServerErrorMessages = new BooleanConnectionProperty("useOnlyServerErrorMessages", true, Messages.getString("ConnectionProperties.useOnlyServerErrorMessages"), "3.0.15", MISC_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty useReadAheadInput = new BooleanConnectionProperty("useReadAheadInput", true, Messages.getString("ConnectionProperties.useReadAheadInput"), "3.1.5", PERFORMANCE_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty useSqlStateCodes = new BooleanConnectionProperty("useSqlStateCodes", true, Messages.getString("ConnectionProperties.useSqlStateCodes"), "3.1.3", MISC_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty useSSL = new BooleanConnectionProperty("useSSL", false, Messages.getString("ConnectionProperties.useSSL"), "3.0.2", SECURITY_CATEGORY, 2);
  




  private BooleanConnectionProperty useSSPSCompatibleTimezoneShift = new BooleanConnectionProperty("useSSPSCompatibleTimezoneShift", false, Messages.getString("ConnectionProperties.useSSPSCompatibleTimezoneShift"), "5.0.5", MISC_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty useStreamLengthsInPrepStmts = new BooleanConnectionProperty("useStreamLengthsInPrepStmts", true, Messages.getString("ConnectionProperties.useStreamLengthsInPrepStmts"), "3.0.2", MISC_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty useTimezone = new BooleanConnectionProperty("useTimezone", false, Messages.getString("ConnectionProperties.useTimezone"), "3.0.2", MISC_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty useUltraDevWorkAround = new BooleanConnectionProperty("ultraDevHack", false, Messages.getString("ConnectionProperties.ultraDevHack"), "2.0.3", MISC_CATEGORY, -2147483648);
  




  private BooleanConnectionProperty useUnbufferedInput = new BooleanConnectionProperty("useUnbufferedInput", true, Messages.getString("ConnectionProperties.useUnbufferedInput"), "3.0.11", MISC_CATEGORY, -2147483648);
  



  private BooleanConnectionProperty useUnicode = new BooleanConnectionProperty("useUnicode", true, Messages.getString("ConnectionProperties.useUnicode"), "1.1g", MISC_CATEGORY, 0);
  





  private boolean useUnicodeAsBoolean = true;
  
  private BooleanConnectionProperty useUsageAdvisor = new BooleanConnectionProperty("useUsageAdvisor", false, Messages.getString("ConnectionProperties.useUsageAdvisor"), "3.1.1", DEBUGING_PROFILING_CATEGORY, 10);
  




  private boolean useUsageAdvisorAsBoolean = false;
  
  private BooleanConnectionProperty yearIsDateType = new BooleanConnectionProperty("yearIsDateType", true, Messages.getString("ConnectionProperties.yearIsDateType"), "3.1.9", MISC_CATEGORY, -2147483648);
  




  private StringConnectionProperty zeroDateTimeBehavior = new StringConnectionProperty("zeroDateTimeBehavior", "exception", new String[] { "exception", "round", "convertToNull" }, Messages.getString("ConnectionProperties.zeroDateTimeBehavior", new Object[] { "exception", "round", "convertToNull" }), "3.1.4", MISC_CATEGORY, -2147483648);
  








  private BooleanConnectionProperty useJvmCharsetConverters = new BooleanConnectionProperty("useJvmCharsetConverters", false, Messages.getString("ConnectionProperties.useJvmCharsetConverters"), "5.0.1", PERFORMANCE_CATEGORY, -2147483648);
  

  private BooleanConnectionProperty useGmtMillisForDatetimes = new BooleanConnectionProperty("useGmtMillisForDatetimes", false, Messages.getString("ConnectionProperties.useGmtMillisForDatetimes"), "3.1.12", MISC_CATEGORY, -2147483648);
  
  private BooleanConnectionProperty dumpMetadataOnColumnNotFound = new BooleanConnectionProperty("dumpMetadataOnColumnNotFound", false, Messages.getString("ConnectionProperties.dumpMetadataOnColumnNotFound"), "3.1.13", DEBUGING_PROFILING_CATEGORY, -2147483648);
  


  private StringConnectionProperty clientCertificateKeyStoreUrl = new StringConnectionProperty("clientCertificateKeyStoreUrl", null, Messages.getString("ConnectionProperties.clientCertificateKeyStoreUrl"), "5.1.0", SECURITY_CATEGORY, 5);
  



  private StringConnectionProperty trustCertificateKeyStoreUrl = new StringConnectionProperty("trustCertificateKeyStoreUrl", null, Messages.getString("ConnectionProperties.trustCertificateKeyStoreUrl"), "5.1.0", SECURITY_CATEGORY, 8);
  



  private StringConnectionProperty clientCertificateKeyStoreType = new StringConnectionProperty("clientCertificateKeyStoreType", "JKS", Messages.getString("ConnectionProperties.clientCertificateKeyStoreType"), "5.1.0", SECURITY_CATEGORY, 6);
  



  private StringConnectionProperty clientCertificateKeyStorePassword = new StringConnectionProperty("clientCertificateKeyStorePassword", null, Messages.getString("ConnectionProperties.clientCertificateKeyStorePassword"), "5.1.0", SECURITY_CATEGORY, 7);
  



  private StringConnectionProperty trustCertificateKeyStoreType = new StringConnectionProperty("trustCertificateKeyStoreType", "JKS", Messages.getString("ConnectionProperties.trustCertificateKeyStoreType"), "5.1.0", SECURITY_CATEGORY, 9);
  



  private StringConnectionProperty trustCertificateKeyStorePassword = new StringConnectionProperty("trustCertificateKeyStorePassword", null, Messages.getString("ConnectionProperties.trustCertificateKeyStorePassword"), "5.1.0", SECURITY_CATEGORY, 10);
  



  private BooleanConnectionProperty verifyServerCertificate = new BooleanConnectionProperty("verifyServerCertificate", true, Messages.getString("ConnectionProperties.verifyServerCertificate"), "5.1.6", SECURITY_CATEGORY, 4);
  




  private BooleanConnectionProperty useAffectedRows = new BooleanConnectionProperty("useAffectedRows", false, Messages.getString("ConnectionProperties.useAffectedRows"), "5.1.7", MISC_CATEGORY, -2147483648);
  



  private StringConnectionProperty passwordCharacterEncoding = new StringConnectionProperty("passwordCharacterEncoding", null, Messages.getString("ConnectionProperties.passwordCharacterEncoding"), "5.1.7", SECURITY_CATEGORY, -2147483648);
  



  private IntegerConnectionProperty maxAllowedPacket = new IntegerConnectionProperty("maxAllowedPacket", -1, Messages.getString("ConnectionProperties.maxAllowedPacket"), "5.1.8", NETWORK_CATEGORY, -2147483648);
  

  protected DriverPropertyInfo[] exposeAsDriverPropertyInfoInternal(Properties info, int slotsToReserve)
    throws SQLException
  {
    initializeProperties(info);
    
    int numProperties = PROPERTY_LIST.size();
    
    int listSize = numProperties + slotsToReserve;
    
    DriverPropertyInfo[] driverProperties = new DriverPropertyInfo[listSize];
    
    for (int i = slotsToReserve; i < listSize; i++) {
      Field propertyField = (Field)PROPERTY_LIST.get(i - slotsToReserve);
      
      try
      {
        ConnectionProperty propToExpose = (ConnectionProperty)propertyField.get(this);
        

        if (info != null) {
          propToExpose.initializeFrom(info);
        }
        

        driverProperties[i] = propToExpose.getAsDriverPropertyInfo();
      } catch (IllegalAccessException iae) {
        throw SQLError.createSQLException(Messages.getString("ConnectionProperties.InternalPropertiesFailure"), "S1000", getExceptionInterceptor());
      }
    }
    

    return driverProperties;
  }
  
  protected Properties exposeAsProperties(Properties info) throws SQLException
  {
    if (info == null) {
      info = new Properties();
    }
    
    int numPropertiesToSet = PROPERTY_LIST.size();
    
    for (int i = 0; i < numPropertiesToSet; i++) {
      Field propertyField = (Field)PROPERTY_LIST.get(i);
      
      try
      {
        ConnectionProperty propToGet = (ConnectionProperty)propertyField.get(this);
        

        Object propValue = propToGet.getValueAsObject();
        
        if (propValue != null) {
          info.setProperty(propToGet.getPropertyName(), propValue.toString());
        }
      }
      catch (IllegalAccessException iae) {
        throw SQLError.createSQLException("Internal properties failure", "S1000", getExceptionInterceptor());
      }
    }
    

    return info;
  }
  

  public String exposeAsXml()
    throws SQLException
  {
    StringBuffer xmlBuf = new StringBuffer();
    xmlBuf.append("<ConnectionProperties>");
    
    int numPropertiesToSet = PROPERTY_LIST.size();
    
    int numCategories = PROPERTY_CATEGORIES.length;
    
    Map propertyListByCategory = new HashMap();
    
    for (int i = 0; i < numCategories; i++) {
      propertyListByCategory.put(PROPERTY_CATEGORIES[i], new Map[] { new TreeMap(), new TreeMap() });
    }
    







    StringConnectionProperty userProp = new StringConnectionProperty("user", null, Messages.getString("ConnectionProperties.Username"), Messages.getString("ConnectionProperties.allVersions"), CONNECTION_AND_AUTH_CATEGORY, -2147483647);
    


    StringConnectionProperty passwordProp = new StringConnectionProperty("password", null, Messages.getString("ConnectionProperties.Password"), Messages.getString("ConnectionProperties.allVersions"), CONNECTION_AND_AUTH_CATEGORY, -2147483646);
    



    Map[] connectionSortMaps = (Map[])propertyListByCategory.get(CONNECTION_AND_AUTH_CATEGORY);
    
    TreeMap userMap = new TreeMap();
    userMap.put(userProp.getPropertyName(), userProp);
    
    connectionSortMaps[0].put(new Integer(userProp.getOrder()), userMap);
    
    TreeMap passwordMap = new TreeMap();
    passwordMap.put(passwordProp.getPropertyName(), passwordProp);
    
    connectionSortMaps[0].put(new Integer(passwordProp.getOrder()), passwordMap);
    
    try
    {
      for (int i = 0; i < numPropertiesToSet; i++) {
        Field propertyField = (Field)PROPERTY_LIST.get(i);
        
        ConnectionProperty propToGet = (ConnectionProperty)propertyField.get(this);
        
        Map[] sortMaps = (Map[])propertyListByCategory.get(propToGet.getCategoryName());
        
        int orderInCategory = propToGet.getOrder();
        
        if (orderInCategory == -2147483648) {
          sortMaps[1].put(propToGet.getPropertyName(), propToGet);
        } else {
          Integer order = new Integer(orderInCategory);
          
          Map orderMap = (Map)sortMaps[0].get(order);
          
          if (orderMap == null) {
            orderMap = new TreeMap();
            sortMaps[0].put(order, orderMap);
          }
          
          orderMap.put(propToGet.getPropertyName(), propToGet);
        }
      }
      
      for (int j = 0; j < numCategories; j++) {
        Map[] sortMaps = (Map[])propertyListByCategory.get(PROPERTY_CATEGORIES[j]);
        
        Iterator orderedIter = sortMaps[0].values().iterator();
        Iterator alphaIter = sortMaps[1].values().iterator();
        
        xmlBuf.append("\n <PropertyCategory name=\"");
        xmlBuf.append(PROPERTY_CATEGORIES[j]);
        xmlBuf.append("\">");
        
        while (orderedIter.hasNext()) {
          Iterator orderedAlphaIter = ((Map)orderedIter.next()).values().iterator();
          
          while (orderedAlphaIter.hasNext()) {
            ConnectionProperty propToGet = (ConnectionProperty)orderedAlphaIter.next();
            

            xmlBuf.append("\n  <Property name=\"");
            xmlBuf.append(propToGet.getPropertyName());
            xmlBuf.append("\" required=\"");
            xmlBuf.append(required ? "Yes" : "No");
            
            xmlBuf.append("\" default=\"");
            
            if (propToGet.getDefaultValue() != null) {
              xmlBuf.append(propToGet.getDefaultValue());
            }
            
            xmlBuf.append("\" sortOrder=\"");
            xmlBuf.append(propToGet.getOrder());
            xmlBuf.append("\" since=\"");
            xmlBuf.append(sinceVersion);
            xmlBuf.append("\">\n");
            xmlBuf.append("    ");
            xmlBuf.append(description);
            xmlBuf.append("\n  </Property>");
          }
        }
        
        while (alphaIter.hasNext()) {
          ConnectionProperty propToGet = (ConnectionProperty)alphaIter.next();
          

          xmlBuf.append("\n  <Property name=\"");
          xmlBuf.append(propToGet.getPropertyName());
          xmlBuf.append("\" required=\"");
          xmlBuf.append(required ? "Yes" : "No");
          
          xmlBuf.append("\" default=\"");
          
          if (propToGet.getDefaultValue() != null) {
            xmlBuf.append(propToGet.getDefaultValue());
          }
          
          xmlBuf.append("\" sortOrder=\"alpha\" since=\"");
          xmlBuf.append(sinceVersion);
          xmlBuf.append("\">\n");
          xmlBuf.append("    ");
          xmlBuf.append(description);
          xmlBuf.append("\n  </Property>");
        }
        
        xmlBuf.append("\n </PropertyCategory>");
      }
    } catch (IllegalAccessException iae) {
      throw SQLError.createSQLException("Internal properties failure", "S1000", getExceptionInterceptor());
    }
    

    xmlBuf.append("\n</ConnectionProperties>");
    
    return xmlBuf.toString();
  }
  


  public boolean getAllowLoadLocalInfile()
  {
    return allowLoadLocalInfile.getValueAsBoolean();
  }
  


  public boolean getAllowMultiQueries()
  {
    return allowMultiQueries.getValueAsBoolean();
  }
  


  public boolean getAllowNanAndInf()
  {
    return allowNanAndInf.getValueAsBoolean();
  }
  


  public boolean getAllowUrlInLocalInfile()
  {
    return allowUrlInLocalInfile.getValueAsBoolean();
  }
  


  public boolean getAlwaysSendSetIsolation()
  {
    return alwaysSendSetIsolation.getValueAsBoolean();
  }
  


  public boolean getAutoDeserialize()
  {
    return autoDeserialize.getValueAsBoolean();
  }
  


  public boolean getAutoGenerateTestcaseScript()
  {
    return autoGenerateTestcaseScriptAsBoolean;
  }
  


  public boolean getAutoReconnectForPools()
  {
    return autoReconnectForPoolsAsBoolean;
  }
  


  public int getBlobSendChunkSize()
  {
    return blobSendChunkSize.getValueAsInt();
  }
  


  public boolean getCacheCallableStatements()
  {
    return cacheCallableStatements.getValueAsBoolean();
  }
  


  public boolean getCachePreparedStatements()
  {
    return ((Boolean)cachePreparedStatements.getValueAsObject()).booleanValue();
  }
  



  public boolean getCacheResultSetMetadata()
  {
    return cacheResultSetMetaDataAsBoolean;
  }
  


  public boolean getCacheServerConfiguration()
  {
    return cacheServerConfiguration.getValueAsBoolean();
  }
  


  public int getCallableStatementCacheSize()
  {
    return callableStatementCacheSize.getValueAsInt();
  }
  


  public boolean getCapitalizeTypeNames()
  {
    return capitalizeTypeNames.getValueAsBoolean();
  }
  


  public String getCharacterSetResults()
  {
    return characterSetResults.getValueAsString();
  }
  


  public boolean getClobberStreamingResults()
  {
    return clobberStreamingResults.getValueAsBoolean();
  }
  


  public String getClobCharacterEncoding()
  {
    return clobCharacterEncoding.getValueAsString();
  }
  


  public String getConnectionCollation()
  {
    return connectionCollation.getValueAsString();
  }
  


  public int getConnectTimeout()
  {
    return connectTimeout.getValueAsInt();
  }
  


  public boolean getContinueBatchOnError()
  {
    return continueBatchOnError.getValueAsBoolean();
  }
  


  public boolean getCreateDatabaseIfNotExist()
  {
    return createDatabaseIfNotExist.getValueAsBoolean();
  }
  


  public int getDefaultFetchSize()
  {
    return defaultFetchSize.getValueAsInt();
  }
  


  public boolean getDontTrackOpenResources()
  {
    return dontTrackOpenResources.getValueAsBoolean();
  }
  


  public boolean getDumpQueriesOnException()
  {
    return dumpQueriesOnException.getValueAsBoolean();
  }
  


  public boolean getDynamicCalendars()
  {
    return dynamicCalendars.getValueAsBoolean();
  }
  


  public boolean getElideSetAutoCommits()
  {
    return elideSetAutoCommits.getValueAsBoolean();
  }
  


  public boolean getEmptyStringsConvertToZero()
  {
    return emptyStringsConvertToZero.getValueAsBoolean();
  }
  


  public boolean getEmulateLocators()
  {
    return emulateLocators.getValueAsBoolean();
  }
  


  public boolean getEmulateUnsupportedPstmts()
  {
    return emulateUnsupportedPstmts.getValueAsBoolean();
  }
  


  public boolean getEnablePacketDebug()
  {
    return enablePacketDebug.getValueAsBoolean();
  }
  


  public String getEncoding()
  {
    return characterEncodingAsString;
  }
  


  public boolean getExplainSlowQueries()
  {
    return explainSlowQueries.getValueAsBoolean();
  }
  


  public boolean getFailOverReadOnly()
  {
    return failOverReadOnly.getValueAsBoolean();
  }
  


  public boolean getGatherPerformanceMetrics()
  {
    return gatherPerformanceMetrics.getValueAsBoolean();
  }
  




  protected boolean getHighAvailability()
  {
    return highAvailabilityAsBoolean;
  }
  


  public boolean getHoldResultsOpenOverStatementClose()
  {
    return holdResultsOpenOverStatementClose.getValueAsBoolean();
  }
  


  public boolean getIgnoreNonTxTables()
  {
    return ignoreNonTxTables.getValueAsBoolean();
  }
  


  public int getInitialTimeout()
  {
    return initialTimeout.getValueAsInt();
  }
  


  public boolean getInteractiveClient()
  {
    return isInteractiveClient.getValueAsBoolean();
  }
  


  public boolean getIsInteractiveClient()
  {
    return isInteractiveClient.getValueAsBoolean();
  }
  


  public boolean getJdbcCompliantTruncation()
  {
    return jdbcCompliantTruncation.getValueAsBoolean();
  }
  


  public int getLocatorFetchBufferSize()
  {
    return locatorFetchBufferSize.getValueAsInt();
  }
  


  public String getLogger()
  {
    return loggerClassName.getValueAsString();
  }
  


  public String getLoggerClassName()
  {
    return loggerClassName.getValueAsString();
  }
  


  public boolean getLogSlowQueries()
  {
    return logSlowQueries.getValueAsBoolean();
  }
  


  public boolean getMaintainTimeStats()
  {
    return maintainTimeStatsAsBoolean;
  }
  


  public int getMaxQuerySizeToLog()
  {
    return maxQuerySizeToLog.getValueAsInt();
  }
  


  public int getMaxReconnects()
  {
    return maxReconnects.getValueAsInt();
  }
  


  public int getMaxRows()
  {
    return maxRowsAsInt;
  }
  


  public int getMetadataCacheSize()
  {
    return metadataCacheSize.getValueAsInt();
  }
  


  public boolean getNoDatetimeStringSync()
  {
    return noDatetimeStringSync.getValueAsBoolean();
  }
  


  public boolean getNullCatalogMeansCurrent()
  {
    return nullCatalogMeansCurrent.getValueAsBoolean();
  }
  


  public boolean getNullNamePatternMatchesAll()
  {
    return nullNamePatternMatchesAll.getValueAsBoolean();
  }
  


  public int getPacketDebugBufferSize()
  {
    return packetDebugBufferSize.getValueAsInt();
  }
  


  public boolean getParanoid()
  {
    return paranoid.getValueAsBoolean();
  }
  


  public boolean getPedantic()
  {
    return pedantic.getValueAsBoolean();
  }
  


  public int getPreparedStatementCacheSize()
  {
    return ((Integer)preparedStatementCacheSize.getValueAsObject()).intValue();
  }
  



  public int getPreparedStatementCacheSqlLimit()
  {
    return ((Integer)preparedStatementCacheSqlLimit.getValueAsObject()).intValue();
  }
  



  public boolean getProfileSql()
  {
    return profileSQLAsBoolean;
  }
  


  public boolean getProfileSQL()
  {
    return profileSQL.getValueAsBoolean();
  }
  


  public String getPropertiesTransform()
  {
    return propertiesTransform.getValueAsString();
  }
  


  public int getQueriesBeforeRetryMaster()
  {
    return queriesBeforeRetryMaster.getValueAsInt();
  }
  


  public boolean getReconnectAtTxEnd()
  {
    return reconnectTxAtEndAsBoolean;
  }
  


  public boolean getRelaxAutoCommit()
  {
    return relaxAutoCommit.getValueAsBoolean();
  }
  


  public int getReportMetricsIntervalMillis()
  {
    return reportMetricsIntervalMillis.getValueAsInt();
  }
  


  public boolean getRequireSSL()
  {
    return requireSSL.getValueAsBoolean();
  }
  
  public boolean getRetainStatementAfterResultSetClose() {
    return retainStatementAfterResultSetClose.getValueAsBoolean();
  }
  


  public boolean getRollbackOnPooledClose()
  {
    return rollbackOnPooledClose.getValueAsBoolean();
  }
  


  public boolean getRoundRobinLoadBalance()
  {
    return roundRobinLoadBalance.getValueAsBoolean();
  }
  


  public boolean getRunningCTS13()
  {
    return runningCTS13.getValueAsBoolean();
  }
  


  public int getSecondsBeforeRetryMaster()
  {
    return secondsBeforeRetryMaster.getValueAsInt();
  }
  


  public String getServerTimezone()
  {
    return serverTimezone.getValueAsString();
  }
  


  public String getSessionVariables()
  {
    return sessionVariables.getValueAsString();
  }
  


  public int getSlowQueryThresholdMillis()
  {
    return slowQueryThresholdMillis.getValueAsInt();
  }
  


  public String getSocketFactoryClassName()
  {
    return socketFactoryClassName.getValueAsString();
  }
  


  public int getSocketTimeout()
  {
    return socketTimeout.getValueAsInt();
  }
  


  public boolean getStrictFloatingPoint()
  {
    return strictFloatingPoint.getValueAsBoolean();
  }
  


  public boolean getStrictUpdates()
  {
    return strictUpdates.getValueAsBoolean();
  }
  


  public boolean getTinyInt1isBit()
  {
    return tinyInt1isBit.getValueAsBoolean();
  }
  


  public boolean getTraceProtocol()
  {
    return traceProtocol.getValueAsBoolean();
  }
  


  public boolean getTransformedBitIsBoolean()
  {
    return transformedBitIsBoolean.getValueAsBoolean();
  }
  


  public boolean getUseCompression()
  {
    return useCompression.getValueAsBoolean();
  }
  


  public boolean getUseFastIntParsing()
  {
    return useFastIntParsing.getValueAsBoolean();
  }
  


  public boolean getUseHostsInPrivileges()
  {
    return useHostsInPrivileges.getValueAsBoolean();
  }
  


  public boolean getUseInformationSchema()
  {
    return useInformationSchema.getValueAsBoolean();
  }
  


  public boolean getUseLocalSessionState()
  {
    return useLocalSessionState.getValueAsBoolean();
  }
  


  public boolean getUseOldUTF8Behavior()
  {
    return useOldUTF8BehaviorAsBoolean;
  }
  


  public boolean getUseOnlyServerErrorMessages()
  {
    return useOnlyServerErrorMessages.getValueAsBoolean();
  }
  


  public boolean getUseReadAheadInput()
  {
    return useReadAheadInput.getValueAsBoolean();
  }
  


  public boolean getUseServerPreparedStmts()
  {
    return detectServerPreparedStmts.getValueAsBoolean();
  }
  


  public boolean getUseSqlStateCodes()
  {
    return useSqlStateCodes.getValueAsBoolean();
  }
  


  public boolean getUseSSL()
  {
    return useSSL.getValueAsBoolean();
  }
  


  public boolean getUseStreamLengthsInPrepStmts()
  {
    return useStreamLengthsInPrepStmts.getValueAsBoolean();
  }
  


  public boolean getUseTimezone()
  {
    return useTimezone.getValueAsBoolean();
  }
  


  public boolean getUseUltraDevWorkAround()
  {
    return useUltraDevWorkAround.getValueAsBoolean();
  }
  


  public boolean getUseUnbufferedInput()
  {
    return useUnbufferedInput.getValueAsBoolean();
  }
  


  public boolean getUseUnicode()
  {
    return useUnicodeAsBoolean;
  }
  


  public boolean getUseUsageAdvisor()
  {
    return useUsageAdvisorAsBoolean;
  }
  


  public boolean getYearIsDateType()
  {
    return yearIsDateType.getValueAsBoolean();
  }
  


  public String getZeroDateTimeBehavior()
  {
    return zeroDateTimeBehavior.getValueAsString();
  }
  








  protected void initializeFromRef(Reference ref)
    throws SQLException
  {
    int numPropertiesToSet = PROPERTY_LIST.size();
    
    for (int i = 0; i < numPropertiesToSet; i++) {
      Field propertyField = (Field)PROPERTY_LIST.get(i);
      
      try
      {
        ConnectionProperty propToSet = (ConnectionProperty)propertyField.get(this);
        

        if (ref != null) {
          propToSet.initializeFrom(ref);
        }
      } catch (IllegalAccessException iae) {
        throw SQLError.createSQLException("Internal properties failure", "S1000", getExceptionInterceptor());
      }
    }
    

    postInitialization();
  }
  







  protected void initializeProperties(Properties info)
    throws SQLException
  {
    if (info != null)
    {
      String profileSqlLc = info.getProperty("profileSql");
      
      if (profileSqlLc != null) {
        info.put("profileSQL", profileSqlLc);
      }
      
      Properties infoCopy = (Properties)info.clone();
      
      infoCopy.remove("HOST");
      infoCopy.remove("user");
      infoCopy.remove("password");
      infoCopy.remove("DBNAME");
      infoCopy.remove("PORT");
      infoCopy.remove("profileSql");
      
      int numPropertiesToSet = PROPERTY_LIST.size();
      
      for (int i = 0; i < numPropertiesToSet; i++) {
        Field propertyField = (Field)PROPERTY_LIST.get(i);
        
        try
        {
          ConnectionProperty propToSet = (ConnectionProperty)propertyField.get(this);
          

          propToSet.initializeFrom(infoCopy);
        } catch (IllegalAccessException iae) {
          throw SQLError.createSQLException(Messages.getString("ConnectionProperties.unableToInitDriverProperties") + iae.toString(), "S1000", getExceptionInterceptor());
        }
      }
      



      postInitialization();
    }
  }
  
  protected void postInitialization()
    throws SQLException
  {
    if (profileSql.getValueAsObject() != null) {
      profileSQL.initializeFrom(profileSql.getValueAsObject().toString());
    }
    

    reconnectTxAtEndAsBoolean = ((Boolean)reconnectAtTxEnd.getValueAsObject()).booleanValue();
    


    if (getMaxRows() == 0)
    {

      maxRows.setValueAsObject(Constants.integerValueOf(-1));
    }
    



    String testEncoding = getEncoding();
    
    if (testEncoding != null)
    {
      try
      {
        String testString = "abc";
        testString.getBytes(testEncoding);
      } catch (UnsupportedEncodingException UE) {
        throw SQLError.createSQLException(Messages.getString("ConnectionProperties.unsupportedCharacterEncoding", new Object[] { testEncoding }), "0S100", getExceptionInterceptor());
      }
    }
    





    if (((Boolean)cacheResultSetMetadata.getValueAsObject()).booleanValue()) {
      try
      {
        Class.forName("java.util.LinkedHashMap");
      } catch (ClassNotFoundException cnfe) {
        cacheResultSetMetadata.setValue(false);
      }
    }
    
    cacheResultSetMetaDataAsBoolean = cacheResultSetMetadata.getValueAsBoolean();
    
    useUnicodeAsBoolean = useUnicode.getValueAsBoolean();
    characterEncodingAsString = ((String)characterEncoding.getValueAsObject());
    
    highAvailabilityAsBoolean = autoReconnect.getValueAsBoolean();
    autoReconnectForPoolsAsBoolean = autoReconnectForPools.getValueAsBoolean();
    
    maxRowsAsInt = ((Integer)maxRows.getValueAsObject()).intValue();
    
    profileSQLAsBoolean = profileSQL.getValueAsBoolean();
    useUsageAdvisorAsBoolean = useUsageAdvisor.getValueAsBoolean();
    
    useOldUTF8BehaviorAsBoolean = useOldUTF8Behavior.getValueAsBoolean();
    
    autoGenerateTestcaseScriptAsBoolean = autoGenerateTestcaseScript.getValueAsBoolean();
    
    maintainTimeStatsAsBoolean = maintainTimeStats.getValueAsBoolean();
    
    jdbcCompliantTruncationForReads = getJdbcCompliantTruncation();
    
    if (getUseCursorFetch())
    {

      setDetectServerPreparedStmts(true);
    }
  }
  


  public void setAllowLoadLocalInfile(boolean property)
  {
    allowLoadLocalInfile.setValue(property);
  }
  


  public void setAllowMultiQueries(boolean property)
  {
    allowMultiQueries.setValue(property);
  }
  


  public void setAllowNanAndInf(boolean flag)
  {
    allowNanAndInf.setValue(flag);
  }
  


  public void setAllowUrlInLocalInfile(boolean flag)
  {
    allowUrlInLocalInfile.setValue(flag);
  }
  


  public void setAlwaysSendSetIsolation(boolean flag)
  {
    alwaysSendSetIsolation.setValue(flag);
  }
  


  public void setAutoDeserialize(boolean flag)
  {
    autoDeserialize.setValue(flag);
  }
  


  public void setAutoGenerateTestcaseScript(boolean flag)
  {
    autoGenerateTestcaseScript.setValue(flag);
    autoGenerateTestcaseScriptAsBoolean = autoGenerateTestcaseScript.getValueAsBoolean();
  }
  



  public void setAutoReconnect(boolean flag)
  {
    autoReconnect.setValue(flag);
  }
  


  public void setAutoReconnectForConnectionPools(boolean property)
  {
    autoReconnectForPools.setValue(property);
    autoReconnectForPoolsAsBoolean = autoReconnectForPools.getValueAsBoolean();
  }
  



  public void setAutoReconnectForPools(boolean flag)
  {
    autoReconnectForPools.setValue(flag);
  }
  

  public void setBlobSendChunkSize(String value)
    throws SQLException
  {
    blobSendChunkSize.setValue(value);
  }
  


  public void setCacheCallableStatements(boolean flag)
  {
    cacheCallableStatements.setValue(flag);
  }
  


  public void setCachePreparedStatements(boolean flag)
  {
    cachePreparedStatements.setValue(flag);
  }
  


  public void setCacheResultSetMetadata(boolean property)
  {
    cacheResultSetMetadata.setValue(property);
    cacheResultSetMetaDataAsBoolean = cacheResultSetMetadata.getValueAsBoolean();
  }
  



  public void setCacheServerConfiguration(boolean flag)
  {
    cacheServerConfiguration.setValue(flag);
  }
  


  public void setCallableStatementCacheSize(int size)
  {
    callableStatementCacheSize.setValue(size);
  }
  


  public void setCapitalizeDBMDTypes(boolean property)
  {
    capitalizeTypeNames.setValue(property);
  }
  


  public void setCapitalizeTypeNames(boolean flag)
  {
    capitalizeTypeNames.setValue(flag);
  }
  


  public void setCharacterEncoding(String encoding)
  {
    characterEncoding.setValue(encoding);
  }
  


  public void setCharacterSetResults(String characterSet)
  {
    characterSetResults.setValue(characterSet);
  }
  


  public void setClobberStreamingResults(boolean flag)
  {
    clobberStreamingResults.setValue(flag);
  }
  


  public void setClobCharacterEncoding(String encoding)
  {
    clobCharacterEncoding.setValue(encoding);
  }
  


  public void setConnectionCollation(String collation)
  {
    connectionCollation.setValue(collation);
  }
  


  public void setConnectTimeout(int timeoutMs)
  {
    connectTimeout.setValue(timeoutMs);
  }
  


  public void setContinueBatchOnError(boolean property)
  {
    continueBatchOnError.setValue(property);
  }
  


  public void setCreateDatabaseIfNotExist(boolean flag)
  {
    createDatabaseIfNotExist.setValue(flag);
  }
  


  public void setDefaultFetchSize(int n)
  {
    defaultFetchSize.setValue(n);
  }
  


  public void setDetectServerPreparedStmts(boolean property)
  {
    detectServerPreparedStmts.setValue(property);
  }
  


  public void setDontTrackOpenResources(boolean flag)
  {
    dontTrackOpenResources.setValue(flag);
  }
  


  public void setDumpQueriesOnException(boolean flag)
  {
    dumpQueriesOnException.setValue(flag);
  }
  


  public void setDynamicCalendars(boolean flag)
  {
    dynamicCalendars.setValue(flag);
  }
  


  public void setElideSetAutoCommits(boolean flag)
  {
    elideSetAutoCommits.setValue(flag);
  }
  


  public void setEmptyStringsConvertToZero(boolean flag)
  {
    emptyStringsConvertToZero.setValue(flag);
  }
  


  public void setEmulateLocators(boolean property)
  {
    emulateLocators.setValue(property);
  }
  


  public void setEmulateUnsupportedPstmts(boolean flag)
  {
    emulateUnsupportedPstmts.setValue(flag);
  }
  


  public void setEnablePacketDebug(boolean flag)
  {
    enablePacketDebug.setValue(flag);
  }
  


  public void setEncoding(String property)
  {
    characterEncoding.setValue(property);
    characterEncodingAsString = characterEncoding.getValueAsString();
  }
  



  public void setExplainSlowQueries(boolean flag)
  {
    explainSlowQueries.setValue(flag);
  }
  


  public void setFailOverReadOnly(boolean flag)
  {
    failOverReadOnly.setValue(flag);
  }
  


  public void setGatherPerformanceMetrics(boolean flag)
  {
    gatherPerformanceMetrics.setValue(flag);
  }
  




  protected void setHighAvailability(boolean property)
  {
    autoReconnect.setValue(property);
    highAvailabilityAsBoolean = autoReconnect.getValueAsBoolean();
  }
  


  public void setHoldResultsOpenOverStatementClose(boolean flag)
  {
    holdResultsOpenOverStatementClose.setValue(flag);
  }
  


  public void setIgnoreNonTxTables(boolean property)
  {
    ignoreNonTxTables.setValue(property);
  }
  


  public void setInitialTimeout(int property)
  {
    initialTimeout.setValue(property);
  }
  


  public void setIsInteractiveClient(boolean property)
  {
    isInteractiveClient.setValue(property);
  }
  


  public void setJdbcCompliantTruncation(boolean flag)
  {
    jdbcCompliantTruncation.setValue(flag);
  }
  

  public void setLocatorFetchBufferSize(String value)
    throws SQLException
  {
    locatorFetchBufferSize.setValue(value);
  }
  


  public void setLogger(String property)
  {
    loggerClassName.setValueAsObject(property);
  }
  


  public void setLoggerClassName(String className)
  {
    loggerClassName.setValue(className);
  }
  


  public void setLogSlowQueries(boolean flag)
  {
    logSlowQueries.setValue(flag);
  }
  


  public void setMaintainTimeStats(boolean flag)
  {
    maintainTimeStats.setValue(flag);
    maintainTimeStatsAsBoolean = maintainTimeStats.getValueAsBoolean();
  }
  



  public void setMaxQuerySizeToLog(int sizeInBytes)
  {
    maxQuerySizeToLog.setValue(sizeInBytes);
  }
  


  public void setMaxReconnects(int property)
  {
    maxReconnects.setValue(property);
  }
  


  public void setMaxRows(int property)
  {
    maxRows.setValue(property);
    maxRowsAsInt = maxRows.getValueAsInt();
  }
  


  public void setMetadataCacheSize(int value)
  {
    metadataCacheSize.setValue(value);
  }
  


  public void setNoDatetimeStringSync(boolean flag)
  {
    noDatetimeStringSync.setValue(flag);
  }
  


  public void setNullCatalogMeansCurrent(boolean value)
  {
    nullCatalogMeansCurrent.setValue(value);
  }
  


  public void setNullNamePatternMatchesAll(boolean value)
  {
    nullNamePatternMatchesAll.setValue(value);
  }
  


  public void setPacketDebugBufferSize(int size)
  {
    packetDebugBufferSize.setValue(size);
  }
  


  public void setParanoid(boolean property)
  {
    paranoid.setValue(property);
  }
  


  public void setPedantic(boolean property)
  {
    pedantic.setValue(property);
  }
  


  public void setPreparedStatementCacheSize(int cacheSize)
  {
    preparedStatementCacheSize.setValue(cacheSize);
  }
  


  public void setPreparedStatementCacheSqlLimit(int cacheSqlLimit)
  {
    preparedStatementCacheSqlLimit.setValue(cacheSqlLimit);
  }
  


  public void setProfileSql(boolean property)
  {
    profileSQL.setValue(property);
    profileSQLAsBoolean = profileSQL.getValueAsBoolean();
  }
  


  public void setProfileSQL(boolean flag)
  {
    profileSQL.setValue(flag);
  }
  


  public void setPropertiesTransform(String value)
  {
    propertiesTransform.setValue(value);
  }
  


  public void setQueriesBeforeRetryMaster(int property)
  {
    queriesBeforeRetryMaster.setValue(property);
  }
  


  public void setReconnectAtTxEnd(boolean property)
  {
    reconnectAtTxEnd.setValue(property);
    reconnectTxAtEndAsBoolean = reconnectAtTxEnd.getValueAsBoolean();
  }
  



  public void setRelaxAutoCommit(boolean property)
  {
    relaxAutoCommit.setValue(property);
  }
  


  public void setReportMetricsIntervalMillis(int millis)
  {
    reportMetricsIntervalMillis.setValue(millis);
  }
  


  public void setRequireSSL(boolean property)
  {
    requireSSL.setValue(property);
  }
  


  public void setRetainStatementAfterResultSetClose(boolean flag)
  {
    retainStatementAfterResultSetClose.setValue(flag);
  }
  


  public void setRollbackOnPooledClose(boolean flag)
  {
    rollbackOnPooledClose.setValue(flag);
  }
  


  public void setRoundRobinLoadBalance(boolean flag)
  {
    roundRobinLoadBalance.setValue(flag);
  }
  


  public void setRunningCTS13(boolean flag)
  {
    runningCTS13.setValue(flag);
  }
  


  public void setSecondsBeforeRetryMaster(int property)
  {
    secondsBeforeRetryMaster.setValue(property);
  }
  


  public void setServerTimezone(String property)
  {
    serverTimezone.setValue(property);
  }
  


  public void setSessionVariables(String variables)
  {
    sessionVariables.setValue(variables);
  }
  


  public void setSlowQueryThresholdMillis(int millis)
  {
    slowQueryThresholdMillis.setValue(millis);
  }
  


  public void setSocketFactoryClassName(String property)
  {
    socketFactoryClassName.setValue(property);
  }
  


  public void setSocketTimeout(int property)
  {
    socketTimeout.setValue(property);
  }
  


  public void setStrictFloatingPoint(boolean property)
  {
    strictFloatingPoint.setValue(property);
  }
  


  public void setStrictUpdates(boolean property)
  {
    strictUpdates.setValue(property);
  }
  


  public void setTinyInt1isBit(boolean flag)
  {
    tinyInt1isBit.setValue(flag);
  }
  


  public void setTraceProtocol(boolean flag)
  {
    traceProtocol.setValue(flag);
  }
  


  public void setTransformedBitIsBoolean(boolean flag)
  {
    transformedBitIsBoolean.setValue(flag);
  }
  


  public void setUseCompression(boolean property)
  {
    useCompression.setValue(property);
  }
  


  public void setUseFastIntParsing(boolean flag)
  {
    useFastIntParsing.setValue(flag);
  }
  


  public void setUseHostsInPrivileges(boolean property)
  {
    useHostsInPrivileges.setValue(property);
  }
  


  public void setUseInformationSchema(boolean flag)
  {
    useInformationSchema.setValue(flag);
  }
  


  public void setUseLocalSessionState(boolean flag)
  {
    useLocalSessionState.setValue(flag);
  }
  


  public void setUseOldUTF8Behavior(boolean flag)
  {
    useOldUTF8Behavior.setValue(flag);
    useOldUTF8BehaviorAsBoolean = useOldUTF8Behavior.getValueAsBoolean();
  }
  



  public void setUseOnlyServerErrorMessages(boolean flag)
  {
    useOnlyServerErrorMessages.setValue(flag);
  }
  


  public void setUseReadAheadInput(boolean flag)
  {
    useReadAheadInput.setValue(flag);
  }
  


  public void setUseServerPreparedStmts(boolean flag)
  {
    detectServerPreparedStmts.setValue(flag);
  }
  


  public void setUseSqlStateCodes(boolean flag)
  {
    useSqlStateCodes.setValue(flag);
  }
  


  public void setUseSSL(boolean property)
  {
    useSSL.setValue(property);
  }
  


  public void setUseStreamLengthsInPrepStmts(boolean property)
  {
    useStreamLengthsInPrepStmts.setValue(property);
  }
  


  public void setUseTimezone(boolean property)
  {
    useTimezone.setValue(property);
  }
  


  public void setUseUltraDevWorkAround(boolean property)
  {
    useUltraDevWorkAround.setValue(property);
  }
  


  public void setUseUnbufferedInput(boolean flag)
  {
    useUnbufferedInput.setValue(flag);
  }
  


  public void setUseUnicode(boolean flag)
  {
    useUnicode.setValue(flag);
    useUnicodeAsBoolean = useUnicode.getValueAsBoolean();
  }
  


  public void setUseUsageAdvisor(boolean useUsageAdvisorFlag)
  {
    useUsageAdvisor.setValue(useUsageAdvisorFlag);
    useUsageAdvisorAsBoolean = useUsageAdvisor.getValueAsBoolean();
  }
  



  public void setYearIsDateType(boolean flag)
  {
    yearIsDateType.setValue(flag);
  }
  


  public void setZeroDateTimeBehavior(String behavior)
  {
    zeroDateTimeBehavior.setValue(behavior);
  }
  
  protected void storeToRef(Reference ref) throws SQLException {
    int numPropertiesToSet = PROPERTY_LIST.size();
    
    for (int i = 0; i < numPropertiesToSet; i++) {
      Field propertyField = (Field)PROPERTY_LIST.get(i);
      
      try
      {
        ConnectionProperty propToStore = (ConnectionProperty)propertyField.get(this);
        

        if (ref != null) {
          propToStore.storeTo(ref);
        }
      } catch (IllegalAccessException iae) {
        throw SQLError.createSQLException(Messages.getString("ConnectionProperties.errorNotExpected"), getExceptionInterceptor());
      }
    }
  }
  


  public boolean useUnbufferedInput()
  {
    return useUnbufferedInput.getValueAsBoolean();
  }
  


  public boolean getUseCursorFetch()
  {
    return useCursorFetch.getValueAsBoolean();
  }
  


  public void setUseCursorFetch(boolean flag)
  {
    useCursorFetch.setValue(flag);
  }
  


  public boolean getOverrideSupportsIntegrityEnhancementFacility()
  {
    return overrideSupportsIntegrityEnhancementFacility.getValueAsBoolean();
  }
  


  public void setOverrideSupportsIntegrityEnhancementFacility(boolean flag)
  {
    overrideSupportsIntegrityEnhancementFacility.setValue(flag);
  }
  


  public boolean getNoTimezoneConversionForTimeType()
  {
    return noTimezoneConversionForTimeType.getValueAsBoolean();
  }
  


  public void setNoTimezoneConversionForTimeType(boolean flag)
  {
    noTimezoneConversionForTimeType.setValue(flag);
  }
  


  public boolean getUseJDBCCompliantTimezoneShift()
  {
    return useJDBCCompliantTimezoneShift.getValueAsBoolean();
  }
  


  public void setUseJDBCCompliantTimezoneShift(boolean flag)
  {
    useJDBCCompliantTimezoneShift.setValue(flag);
  }
  


  public boolean getAutoClosePStmtStreams()
  {
    return autoClosePStmtStreams.getValueAsBoolean();
  }
  


  public void setAutoClosePStmtStreams(boolean flag)
  {
    autoClosePStmtStreams.setValue(flag);
  }
  


  public boolean getProcessEscapeCodesForPrepStmts()
  {
    return processEscapeCodesForPrepStmts.getValueAsBoolean();
  }
  


  public void setProcessEscapeCodesForPrepStmts(boolean flag)
  {
    processEscapeCodesForPrepStmts.setValue(flag);
  }
  


  public boolean getUseGmtMillisForDatetimes()
  {
    return useGmtMillisForDatetimes.getValueAsBoolean();
  }
  


  public void setUseGmtMillisForDatetimes(boolean flag)
  {
    useGmtMillisForDatetimes.setValue(flag);
  }
  


  public boolean getDumpMetadataOnColumnNotFound()
  {
    return dumpMetadataOnColumnNotFound.getValueAsBoolean();
  }
  


  public void setDumpMetadataOnColumnNotFound(boolean flag)
  {
    dumpMetadataOnColumnNotFound.setValue(flag);
  }
  


  public String getResourceId()
  {
    return resourceId.getValueAsString();
  }
  


  public void setResourceId(String resourceId)
  {
    this.resourceId.setValue(resourceId);
  }
  


  public boolean getRewriteBatchedStatements()
  {
    return rewriteBatchedStatements.getValueAsBoolean();
  }
  


  public void setRewriteBatchedStatements(boolean flag)
  {
    rewriteBatchedStatements.setValue(flag);
  }
  


  public boolean getJdbcCompliantTruncationForReads()
  {
    return jdbcCompliantTruncationForReads;
  }
  



  public void setJdbcCompliantTruncationForReads(boolean jdbcCompliantTruncationForReads)
  {
    this.jdbcCompliantTruncationForReads = jdbcCompliantTruncationForReads;
  }
  


  public boolean getUseJvmCharsetConverters()
  {
    return useJvmCharsetConverters.getValueAsBoolean();
  }
  


  public void setUseJvmCharsetConverters(boolean flag)
  {
    useJvmCharsetConverters.setValue(flag);
  }
  


  public boolean getPinGlobalTxToPhysicalConnection()
  {
    return pinGlobalTxToPhysicalConnection.getValueAsBoolean();
  }
  


  public void setPinGlobalTxToPhysicalConnection(boolean flag)
  {
    pinGlobalTxToPhysicalConnection.setValue(flag);
  }
  







  public void setGatherPerfMetrics(boolean flag)
  {
    setGatherPerformanceMetrics(flag);
  }
  


  public boolean getGatherPerfMetrics()
  {
    return getGatherPerformanceMetrics();
  }
  


  public void setUltraDevHack(boolean flag)
  {
    setUseUltraDevWorkAround(flag);
  }
  


  public boolean getUltraDevHack()
  {
    return getUseUltraDevWorkAround();
  }
  


  public void setInteractiveClient(boolean property)
  {
    setIsInteractiveClient(property);
  }
  


  public void setSocketFactory(String name)
  {
    setSocketFactoryClassName(name);
  }
  


  public String getSocketFactory()
  {
    return getSocketFactoryClassName();
  }
  


  public void setUseServerPrepStmts(boolean flag)
  {
    setUseServerPreparedStmts(flag);
  }
  


  public boolean getUseServerPrepStmts()
  {
    return getUseServerPreparedStmts();
  }
  


  public void setCacheCallableStmts(boolean flag)
  {
    setCacheCallableStatements(flag);
  }
  


  public boolean getCacheCallableStmts()
  {
    return getCacheCallableStatements();
  }
  


  public void setCachePrepStmts(boolean flag)
  {
    setCachePreparedStatements(flag);
  }
  


  public boolean getCachePrepStmts()
  {
    return getCachePreparedStatements();
  }
  


  public void setCallableStmtCacheSize(int cacheSize)
  {
    setCallableStatementCacheSize(cacheSize);
  }
  


  public int getCallableStmtCacheSize()
  {
    return getCallableStatementCacheSize();
  }
  


  public void setPrepStmtCacheSize(int cacheSize)
  {
    setPreparedStatementCacheSize(cacheSize);
  }
  


  public int getPrepStmtCacheSize()
  {
    return getPreparedStatementCacheSize();
  }
  


  public void setPrepStmtCacheSqlLimit(int sqlLimit)
  {
    setPreparedStatementCacheSqlLimit(sqlLimit);
  }
  


  public int getPrepStmtCacheSqlLimit()
  {
    return getPreparedStatementCacheSqlLimit();
  }
  


  public boolean getNoAccessToProcedureBodies()
  {
    return noAccessToProcedureBodies.getValueAsBoolean();
  }
  


  public void setNoAccessToProcedureBodies(boolean flag)
  {
    noAccessToProcedureBodies.setValue(flag);
  }
  


  public boolean getUseOldAliasMetadataBehavior()
  {
    return useOldAliasMetadataBehavior.getValueAsBoolean();
  }
  


  public void setUseOldAliasMetadataBehavior(boolean flag)
  {
    useOldAliasMetadataBehavior.setValue(flag);
  }
  


  public String getClientCertificateKeyStorePassword()
  {
    return clientCertificateKeyStorePassword.getValueAsString();
  }
  



  public void setClientCertificateKeyStorePassword(String value)
  {
    clientCertificateKeyStorePassword.setValue(value);
  }
  


  public String getClientCertificateKeyStoreType()
  {
    return clientCertificateKeyStoreType.getValueAsString();
  }
  



  public void setClientCertificateKeyStoreType(String value)
  {
    clientCertificateKeyStoreType.setValue(value);
  }
  


  public String getClientCertificateKeyStoreUrl()
  {
    return clientCertificateKeyStoreUrl.getValueAsString();
  }
  



  public void setClientCertificateKeyStoreUrl(String value)
  {
    clientCertificateKeyStoreUrl.setValue(value);
  }
  


  public String getTrustCertificateKeyStorePassword()
  {
    return trustCertificateKeyStorePassword.getValueAsString();
  }
  



  public void setTrustCertificateKeyStorePassword(String value)
  {
    trustCertificateKeyStorePassword.setValue(value);
  }
  


  public String getTrustCertificateKeyStoreType()
  {
    return trustCertificateKeyStoreType.getValueAsString();
  }
  



  public void setTrustCertificateKeyStoreType(String value)
  {
    trustCertificateKeyStoreType.setValue(value);
  }
  


  public String getTrustCertificateKeyStoreUrl()
  {
    return trustCertificateKeyStoreUrl.getValueAsString();
  }
  



  public void setTrustCertificateKeyStoreUrl(String value)
  {
    trustCertificateKeyStoreUrl.setValue(value);
  }
  


  public boolean getUseSSPSCompatibleTimezoneShift()
  {
    return useSSPSCompatibleTimezoneShift.getValueAsBoolean();
  }
  


  public void setUseSSPSCompatibleTimezoneShift(boolean flag)
  {
    useSSPSCompatibleTimezoneShift.setValue(flag);
  }
  


  public boolean getTreatUtilDateAsTimestamp()
  {
    return treatUtilDateAsTimestamp.getValueAsBoolean();
  }
  


  public void setTreatUtilDateAsTimestamp(boolean flag)
  {
    treatUtilDateAsTimestamp.setValue(flag);
  }
  


  public boolean getUseFastDateParsing()
  {
    return useFastDateParsing.getValueAsBoolean();
  }
  


  public void setUseFastDateParsing(boolean flag)
  {
    useFastDateParsing.setValue(flag);
  }
  


  public String getLocalSocketAddress()
  {
    return localSocketAddress.getValueAsString();
  }
  


  public void setLocalSocketAddress(String address)
  {
    localSocketAddress.setValue(address);
  }
  


  public void setUseConfigs(String configs)
  {
    useConfigs.setValue(configs);
  }
  


  public String getUseConfigs()
  {
    return useConfigs.getValueAsString();
  }
  



  public boolean getGenerateSimpleParameterMetadata()
  {
    return generateSimpleParameterMetadata.getValueAsBoolean();
  }
  


  public void setGenerateSimpleParameterMetadata(boolean flag)
  {
    generateSimpleParameterMetadata.setValue(flag);
  }
  


  public boolean getLogXaCommands()
  {
    return logXaCommands.getValueAsBoolean();
  }
  


  public void setLogXaCommands(boolean flag)
  {
    logXaCommands.setValue(flag);
  }
  


  public int getResultSetSizeThreshold()
  {
    return resultSetSizeThreshold.getValueAsInt();
  }
  


  public void setResultSetSizeThreshold(int threshold)
  {
    resultSetSizeThreshold.setValue(threshold);
  }
  


  public int getNetTimeoutForStreamingResults()
  {
    return netTimeoutForStreamingResults.getValueAsInt();
  }
  


  public void setNetTimeoutForStreamingResults(int value)
  {
    netTimeoutForStreamingResults.setValue(value);
  }
  


  public boolean getEnableQueryTimeouts()
  {
    return enableQueryTimeouts.getValueAsBoolean();
  }
  


  public void setEnableQueryTimeouts(boolean flag)
  {
    enableQueryTimeouts.setValue(flag);
  }
  


  public boolean getPadCharsWithSpace()
  {
    return padCharsWithSpace.getValueAsBoolean();
  }
  


  public void setPadCharsWithSpace(boolean flag)
  {
    padCharsWithSpace.setValue(flag);
  }
  


  public boolean getUseDynamicCharsetInfo()
  {
    return useDynamicCharsetInfo.getValueAsBoolean();
  }
  


  public void setUseDynamicCharsetInfo(boolean flag)
  {
    useDynamicCharsetInfo.setValue(flag);
  }
  


  public String getClientInfoProvider()
  {
    return clientInfoProvider.getValueAsString();
  }
  


  public void setClientInfoProvider(String classname)
  {
    clientInfoProvider.setValue(classname);
  }
  
  public boolean getPopulateInsertRowWithDefaultValues() {
    return populateInsertRowWithDefaultValues.getValueAsBoolean();
  }
  
  public void setPopulateInsertRowWithDefaultValues(boolean flag) {
    populateInsertRowWithDefaultValues.setValue(flag);
  }
  
  public String getLoadBalanceStrategy() {
    return loadBalanceStrategy.getValueAsString();
  }
  
  public void setLoadBalanceStrategy(String strategy) {
    loadBalanceStrategy.setValue(strategy);
  }
  
  public boolean getTcpNoDelay() {
    return tcpNoDelay.getValueAsBoolean();
  }
  
  public void setTcpNoDelay(boolean flag) {
    tcpNoDelay.setValue(flag);
  }
  
  public boolean getTcpKeepAlive() {
    return tcpKeepAlive.getValueAsBoolean();
  }
  
  public void setTcpKeepAlive(boolean flag) {
    tcpKeepAlive.setValue(flag);
  }
  
  public int getTcpRcvBuf() {
    return tcpRcvBuf.getValueAsInt();
  }
  
  public void setTcpRcvBuf(int bufSize) {
    tcpRcvBuf.setValue(bufSize);
  }
  
  public int getTcpSndBuf() {
    return tcpSndBuf.getValueAsInt();
  }
  
  public void setTcpSndBuf(int bufSize) {
    tcpSndBuf.setValue(bufSize);
  }
  
  public int getTcpTrafficClass() {
    return tcpTrafficClass.getValueAsInt();
  }
  
  public void setTcpTrafficClass(int classFlags) {
    tcpTrafficClass.setValue(classFlags);
  }
  
  public boolean getUseNanosForElapsedTime() {
    return useNanosForElapsedTime.getValueAsBoolean();
  }
  
  public void setUseNanosForElapsedTime(boolean flag) {
    useNanosForElapsedTime.setValue(flag);
  }
  
  public long getSlowQueryThresholdNanos() {
    return slowQueryThresholdNanos.getValueAsLong();
  }
  
  public void setSlowQueryThresholdNanos(long nanos) {
    slowQueryThresholdNanos.setValue(nanos);
  }
  
  public String getStatementInterceptors() {
    return statementInterceptors.getValueAsString();
  }
  
  public void setStatementInterceptors(String value) {
    statementInterceptors.setValue(value);
  }
  
  public boolean getUseDirectRowUnpack() {
    return useDirectRowUnpack.getValueAsBoolean();
  }
  
  public void setUseDirectRowUnpack(boolean flag) {
    useDirectRowUnpack.setValue(flag);
  }
  
  public String getLargeRowSizeThreshold() {
    return largeRowSizeThreshold.getValueAsString();
  }
  
  public void setLargeRowSizeThreshold(String value) {
    try {
      largeRowSizeThreshold.setValue(value);
    } catch (SQLException sqlEx) {
      RuntimeException ex = new RuntimeException(sqlEx.getMessage());
      ex.initCause(sqlEx);
      
      throw ex;
    }
  }
  
  public boolean getUseBlobToStoreUTF8OutsideBMP() {
    return useBlobToStoreUTF8OutsideBMP.getValueAsBoolean();
  }
  
  public void setUseBlobToStoreUTF8OutsideBMP(boolean flag) {
    useBlobToStoreUTF8OutsideBMP.setValue(flag);
  }
  
  public String getUtf8OutsideBmpExcludedColumnNamePattern() {
    return utf8OutsideBmpExcludedColumnNamePattern.getValueAsString();
  }
  
  public void setUtf8OutsideBmpExcludedColumnNamePattern(String regexPattern) {
    utf8OutsideBmpExcludedColumnNamePattern.setValue(regexPattern);
  }
  
  public String getUtf8OutsideBmpIncludedColumnNamePattern() {
    return utf8OutsideBmpIncludedColumnNamePattern.getValueAsString();
  }
  
  public void setUtf8OutsideBmpIncludedColumnNamePattern(String regexPattern) {
    utf8OutsideBmpIncludedColumnNamePattern.setValue(regexPattern);
  }
  
  public boolean getIncludeInnodbStatusInDeadlockExceptions() {
    return includeInnodbStatusInDeadlockExceptions.getValueAsBoolean();
  }
  
  public void setIncludeInnodbStatusInDeadlockExceptions(boolean flag) {
    includeInnodbStatusInDeadlockExceptions.setValue(flag);
  }
  
  public boolean getBlobsAreStrings() {
    return blobsAreStrings.getValueAsBoolean();
  }
  
  public void setBlobsAreStrings(boolean flag) {
    blobsAreStrings.setValue(flag);
  }
  
  public boolean getFunctionsNeverReturnBlobs() {
    return functionsNeverReturnBlobs.getValueAsBoolean();
  }
  
  public void setFunctionsNeverReturnBlobs(boolean flag) {
    functionsNeverReturnBlobs.setValue(flag);
  }
  
  public boolean getAutoSlowLog() {
    return autoSlowLog.getValueAsBoolean();
  }
  
  public void setAutoSlowLog(boolean flag) {
    autoSlowLog.setValue(flag);
  }
  
  public String getConnectionLifecycleInterceptors() {
    return connectionLifecycleInterceptors.getValueAsString();
  }
  
  public void setConnectionLifecycleInterceptors(String interceptors) {
    connectionLifecycleInterceptors.setValue(interceptors);
  }
  
  public String getProfilerEventHandler() {
    return profilerEventHandler.getValueAsString();
  }
  
  public void setProfilerEventHandler(String handler) {
    profilerEventHandler.setValue(handler);
  }
  
  public boolean getVerifyServerCertificate() {
    return verifyServerCertificate.getValueAsBoolean();
  }
  
  public void setVerifyServerCertificate(boolean flag) {
    verifyServerCertificate.setValue(flag);
  }
  
  public boolean getUseLegacyDatetimeCode() {
    return useLegacyDatetimeCode.getValueAsBoolean();
  }
  
  public void setUseLegacyDatetimeCode(boolean flag) {
    useLegacyDatetimeCode.setValue(flag);
  }
  
  public int getSelfDestructOnPingSecondsLifetime() {
    return selfDestructOnPingSecondsLifetime.getValueAsInt();
  }
  
  public void setSelfDestructOnPingSecondsLifetime(int seconds) {
    selfDestructOnPingSecondsLifetime.setValue(seconds);
  }
  
  public int getSelfDestructOnPingMaxOperations() {
    return selfDestructOnPingMaxOperations.getValueAsInt();
  }
  
  public void setSelfDestructOnPingMaxOperations(int maxOperations) {
    selfDestructOnPingMaxOperations.setValue(maxOperations);
  }
  
  public boolean getUseColumnNamesInFindColumn() {
    return useColumnNamesInFindColumn.getValueAsBoolean();
  }
  
  public void setUseColumnNamesInFindColumn(boolean flag) {
    useColumnNamesInFindColumn.setValue(flag);
  }
  
  public boolean getUseLocalTransactionState() {
    return useLocalTransactionState.getValueAsBoolean();
  }
  
  public void setUseLocalTransactionState(boolean flag) {
    useLocalTransactionState.setValue(flag);
  }
  
  public boolean getCompensateOnDuplicateKeyUpdateCounts() {
    return compensateOnDuplicateKeyUpdateCounts.getValueAsBoolean();
  }
  
  public void setCompensateOnDuplicateKeyUpdateCounts(boolean flag) {
    compensateOnDuplicateKeyUpdateCounts.setValue(flag);
  }
  
  public int getLoadBalanceBlacklistTimeout() {
    return loadBalanceBlacklistTimeout.getValueAsInt();
  }
  
  public void setLoadBalanceBlacklistTimeout(int loadBalanceBlacklistTimeout) {
    this.loadBalanceBlacklistTimeout.setValue(loadBalanceBlacklistTimeout);
  }
  
  public int getLoadBalancePingTimeout() {
    return loadBalancePingTimeout.getValueAsInt();
  }
  
  public void setLoadBalancePingTimeout(int loadBalancePingTimeout) {
    this.loadBalancePingTimeout.setValue(loadBalancePingTimeout);
  }
  
  public void setRetriesAllDown(int retriesAllDown) {
    this.retriesAllDown.setValue(retriesAllDown);
  }
  
  public int getRetriesAllDown() {
    return retriesAllDown.getValueAsInt();
  }
  
  public void setUseAffectedRows(boolean flag) {
    useAffectedRows.setValue(flag);
  }
  
  public boolean getUseAffectedRows() {
    return useAffectedRows.getValueAsBoolean();
  }
  
  public void setPasswordCharacterEncoding(String characterSet) {
    passwordCharacterEncoding.setValue(characterSet);
  }
  
  public String getPasswordCharacterEncoding() {
    return passwordCharacterEncoding.getValueAsString();
  }
  
  public void setExceptionInterceptors(String exceptionInterceptors) {
    this.exceptionInterceptors.setValue(exceptionInterceptors);
  }
  
  public String getExceptionInterceptors() {
    return exceptionInterceptors.getValueAsString();
  }
  
  public void setMaxAllowedPacket(int max) {
    maxAllowedPacket.setValue(max);
  }
  
  public int getMaxAllowedPacket() {
    return maxAllowedPacket.getValueAsInt();
  }
  
  public boolean getQueryTimeoutKillsConnection() {
    return queryTimeoutKillsConnection.getValueAsBoolean();
  }
  
  public void setQueryTimeoutKillsConnection(boolean queryTimeoutKillsConnection) {
    this.queryTimeoutKillsConnection.setValue(queryTimeoutKillsConnection);
  }
  
  public boolean getLoadBalanceValidateConnectionOnSwapServer() {
    return loadBalanceValidateConnectionOnSwapServer.getValueAsBoolean();
  }
  
  public void setLoadBalanceValidateConnectionOnSwapServer(boolean loadBalanceValidateConnectionOnSwapServer)
  {
    this.loadBalanceValidateConnectionOnSwapServer.setValue(loadBalanceValidateConnectionOnSwapServer);
  }
  
  public String getLoadBalanceConnectionGroup()
  {
    return loadBalanceConnectionGroup.getValueAsString();
  }
  
  public void setLoadBalanceConnectionGroup(String loadBalanceConnectionGroup) {
    this.loadBalanceConnectionGroup.setValue(loadBalanceConnectionGroup);
  }
  
  public String getLoadBalanceExceptionChecker() {
    return loadBalanceExceptionChecker.getValueAsString();
  }
  
  public void setLoadBalanceExceptionChecker(String loadBalanceExceptionChecker) {
    this.loadBalanceExceptionChecker.setValue(loadBalanceExceptionChecker);
  }
  
  public String getLoadBalanceSQLStateFailover() {
    return loadBalanceSQLStateFailover.getValueAsString();
  }
  
  public void setLoadBalanceSQLStateFailover(String loadBalanceSQLStateFailover) {
    this.loadBalanceSQLStateFailover.setValue(loadBalanceSQLStateFailover);
  }
  
  public String getLoadBalanceSQLExceptionSubclassFailover() {
    return loadBalanceSQLExceptionSubclassFailover.getValueAsString();
  }
  
  public void setLoadBalanceSQLExceptionSubclassFailover(String loadBalanceSQLExceptionSubclassFailover) {
    this.loadBalanceSQLExceptionSubclassFailover.setValue(loadBalanceSQLExceptionSubclassFailover);
  }
  
  public boolean getLoadBalanceEnableJMX() {
    return loadBalanceEnableJMX.getValueAsBoolean();
  }
  
  public void setLoadBalanceEnableJMX(boolean loadBalanceEnableJMX) {
    this.loadBalanceEnableJMX.setValue(loadBalanceEnableJMX);
  }
  
  public void setLoadBalanceAutoCommitStatementThreshold(int loadBalanceAutoCommitStatementThreshold) {
    this.loadBalanceAutoCommitStatementThreshold.setValue(loadBalanceAutoCommitStatementThreshold);
  }
  
  public int getLoadBalanceAutoCommitStatementThreshold() {
    return loadBalanceAutoCommitStatementThreshold.getValueAsInt();
  }
  
  public void setLoadBalanceAutoCommitStatementRegex(String loadBalanceAutoCommitStatementRegex) {
    this.loadBalanceAutoCommitStatementRegex.setValue(loadBalanceAutoCommitStatementRegex);
  }
  
  public String getLoadBalanceAutoCommitStatementRegex() {
    return loadBalanceAutoCommitStatementRegex.getValueAsString();
  }
}
