package com.mysql.jdbc;

import com.mysql.jdbc.log.Log;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.sql.SQLException;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Properties;
import java.util.Set;










public class FailoverConnectionProxy
  extends LoadBalancingConnectionProxy
{
  boolean failedOver;
  boolean hasTriedMaster;
  private long masterFailTimeMillis;
  boolean preferSlaveDuringFailover;
  private String primaryHostPortSpec;
  private long queriesBeforeRetryMaster;
  long queriesIssuedFailedOver;
  private int secondsBeforeRetryMaster;
  
  class FailoverInvocationHandler
    extends LoadBalancingConnectionProxy.ConnectionErrorFiringInvocationHandler
  {
    public FailoverInvocationHandler(Object toInvokeOn)
    {
      super(toInvokeOn);
    }
    
    public Object invoke(Object proxy, Method method, Object[] args) throws Throwable
    {
      String methodName = method.getName();
      
      if ((failedOver) && (methodName.indexOf("execute") != -1)) {
        queriesIssuedFailedOver += 1L;
      }
      
      return super.invoke(proxy, method, args);
    }
  }
  








  FailoverConnectionProxy(List<String> hosts, Properties props)
    throws SQLException
  {
    super(hosts, props);
    ConnectionPropertiesImpl connectionProps = new ConnectionPropertiesImpl();
    connectionProps.initializeProperties(props);
    
    queriesBeforeRetryMaster = connectionProps.getQueriesBeforeRetryMaster();
    secondsBeforeRetryMaster = connectionProps.getSecondsBeforeRetryMaster();
    preferSlaveDuringFailover = false;
  }
  
  protected LoadBalancingConnectionProxy.ConnectionErrorFiringInvocationHandler createConnectionProxy(Object toProxy)
  {
    return new FailoverInvocationHandler(toProxy);
  }
  


  void dealWithInvocationException(InvocationTargetException e)
    throws SQLException, Throwable, InvocationTargetException
  {
    Throwable t = e.getTargetException();
    
    if (t != null) {
      if (failedOver) {
        createPrimaryConnection();
        
        if (currentConn != null) {
          throw t;
        }
      }
      
      failOver();
      
      throw t;
    }
    
    throw e;
  }
  
  public Object invoke(Object proxy, Method method, Object[] args) throws Throwable
  {
    String methodName = method.getName();
    
    if ("setPreferSlaveDuringFailover".equals(methodName)) {
      preferSlaveDuringFailover = ((Boolean)args[0]).booleanValue();
    } else if ("clearHasTriedMaster".equals(methodName)) {
      hasTriedMaster = false;
    } else { if ("hasTriedMaster".equals(methodName))
        return Boolean.valueOf(hasTriedMaster);
      if ("isMasterConnection".equals(methodName))
        return Boolean.valueOf(!failedOver);
      if ("isSlaveConnection".equals(methodName))
        return Boolean.valueOf(failedOver);
      if ("setReadOnly".equals(methodName)) {
        if (failedOver) {
          return null;
        }
      } else if (("setAutoCommit".equals(methodName)) && (failedOver) && (shouldFallBack()) && (Boolean.TRUE.equals(args[0])) && (failedOver))
      {
        createPrimaryConnection();
        
        return super.invoke(proxy, method, args, failedOver);
      }
    }
    return super.invoke(proxy, method, args, failedOver);
  }
  
  private void createPrimaryConnection() throws SQLException {
    try {
      currentConn = createConnectionForHost(primaryHostPortSpec);
      failedOver = false;
      hasTriedMaster = true;
      

      queriesIssuedFailedOver = 0L;
    } catch (SQLException sqlEx) {
      failedOver = true;
      
      if (currentConn != null) {
        currentConn.getLog().logWarn("Connection to primary host failed", sqlEx);
      }
    }
  }
  
  synchronized void invalidateCurrentConnection() throws SQLException {
    if (!failedOver) {
      failedOver = true;
      queriesIssuedFailedOver = 0L;
      masterFailTimeMillis = System.currentTimeMillis();
    }
    super.invalidateCurrentConnection();
  }
  
  protected synchronized void pickNewConnection() throws SQLException {
    if (primaryHostPortSpec == null) {
      primaryHostPortSpec = ((String)hostList.remove(0));
    }
    
    if ((currentConn == null) || ((failedOver) && (shouldFallBack()))) {
      createPrimaryConnection();
      
      if (currentConn != null) {
        return;
      }
    }
    
    failOver();
  }
  
  private void failOver() throws SQLException {
    if (failedOver) {
      Iterator<Map.Entry<String, ConnectionImpl>> iter = liveConnections.entrySet().iterator();
      
      while (iter.hasNext()) {
        Map.Entry<String, ConnectionImpl> entry = (Map.Entry)iter.next();
        ((ConnectionImpl)entry.getValue()).close();
      }
      
      liveConnections.clear();
    }
    
    super.pickNewConnection();
    
    if (currentConn.getFailOverReadOnly()) {
      currentConn.setReadOnly(true);
    } else {
      currentConn.setReadOnly(false);
    }
    
    failedOver = true;
  }
  






  private boolean shouldFallBack()
  {
    long secondsSinceFailedOver = (System.currentTimeMillis() - masterFailTimeMillis) / 1000L;
    
    if (secondsSinceFailedOver >= secondsBeforeRetryMaster)
    {
      masterFailTimeMillis = System.currentTimeMillis();
      
      return true; }
    if ((queriesBeforeRetryMaster != 0L) && (queriesIssuedFailedOver >= queriesBeforeRetryMaster)) {
      return true;
    }
    
    return false;
  }
}
