package com.mysql.jdbc;

import java.sql.SQLException;
import java.util.Properties;





























public class LoadBalancedAutoCommitInterceptor
  implements StatementInterceptorV2
{
  private int matchingAfterStatementCount = 0;
  private int matchingAfterStatementThreshold = 0;
  private String matchingAfterStatementRegex;
  private ConnectionImpl conn;
  private LoadBalancingConnectionProxy proxy = null;
  




  public void destroy() {}
  



  public boolean executeTopLevelOnly()
  {
    return false;
  }
  
  public void init(Connection conn, Properties props) throws SQLException {
    this.conn = ((ConnectionImpl)conn);
    
    String autoCommitSwapThresholdAsString = props.getProperty("loadBalanceAutoCommitStatementThreshold", "0");
    try
    {
      matchingAfterStatementThreshold = Integer.parseInt(autoCommitSwapThresholdAsString);
    }
    catch (NumberFormatException nfe) {}
    
    String autoCommitSwapRegex = props.getProperty("loadBalanceAutoCommitStatementRegex", "");
    if ("".equals(autoCommitSwapRegex)) {
      return;
    }
    matchingAfterStatementRegex = autoCommitSwapRegex;
  }
  








  public ResultSetInternalMethods postProcess(String sql, Statement interceptedStatement, ResultSetInternalMethods originalResultSet, Connection connection, int warningCount, boolean noIndexUsed, boolean noGoodIndexUsed, SQLException statementException)
    throws SQLException
  {
    if (!conn.getAutoCommit()) {
      matchingAfterStatementCount = 0;
    }
    else
    {
      if ((proxy == null) && (conn.isProxySet())) {
        MySQLConnection lcl_proxy = conn.getLoadBalanceSafeProxy();
        while ((lcl_proxy != null) && (!(lcl_proxy instanceof LoadBalancedMySQLConnection))) {
          lcl_proxy = lcl_proxy.getLoadBalanceSafeProxy();
        }
        if (lcl_proxy != null) {
          proxy = ((LoadBalancedMySQLConnection)lcl_proxy).getProxy();
        }
      }
      

      if (proxy != null)
      {
        if ((matchingAfterStatementRegex == null) || (sql.matches(matchingAfterStatementRegex)))
        {
          matchingAfterStatementCount += 1;
        }
      }
      
      if (matchingAfterStatementCount >= matchingAfterStatementThreshold) {
        matchingAfterStatementCount = 0;
        try {
          if (proxy != null) {
            proxy.pickNewConnection();
          }
        }
        catch (SQLException e) {}
      }
    }
    




    return originalResultSet;
  }
  


  public ResultSetInternalMethods preProcess(String sql, Statement interceptedStatement, Connection connection)
    throws SQLException
  {
    return null;
  }
}
