package com.mysql.jdbc;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;












































public class LoadBalancingConnectionProxy
  implements InvocationHandler, PingTarget
{
  private static Method getLocalTimeMethod;
  private long totalPhysicalConnections = 0L;
  private long activePhysicalConnections = 0L;
  private String hostToRemove = null;
  private long lastUsed = 0L;
  private long transactionCount = 0L;
  private ConnectionGroup connectionGroup = null;
  private String closedReason = null;
  
  public static final String BLACKLIST_TIMEOUT_PROPERTY_KEY = "loadBalanceBlacklistTimeout";
  
  protected MySQLConnection currentConn;
  
  protected List<String> hostList;
  
  protected Map<String, ConnectionImpl> liveConnections;
  
  private Map<ConnectionImpl, String> connectionsToHostsMap;
  
  private long[] responseTimes;
  
  private Map<String, Integer> hostsToListIndexMap;
  
  protected class ConnectionErrorFiringInvocationHandler
    implements InvocationHandler
  {
    Object invokeOn = null;
    
    public ConnectionErrorFiringInvocationHandler(Object toInvokeOn) {
      invokeOn = toInvokeOn;
    }
    
    public Object invoke(Object proxy, Method method, Object[] args) throws Throwable
    {
      Object result = null;
      try
      {
        result = method.invoke(invokeOn, args);
        
        if (result != null) {
          result = proxyIfInterfaceIsJdbc(result, result.getClass());
        }
      } catch (InvocationTargetException e) {
        dealWithInvocationException(e);
      }
      
      return result;
    }
  }
  












  private boolean inTransaction = false;
  
  private long transactionStartTime = 0L;
  
  private Properties localProps;
  
  private boolean isClosed = false;
  
  private BalanceStrategy balancer;
  
  private int retriesAllDown;
  
  private static Map<String, Long> globalBlacklist;
  
  private int globalBlacklistTimeout = 0;
  
  private long connectionGroupProxyID = 0L;
  
  private LoadBalanceExceptionChecker exceptionChecker;
  
  private Map<Class, Boolean> jdbcInterfacesForProxyCache = new HashMap();
  
  private MySQLConnection thisAsConnection = null;
  
  private int autoCommitSwapThreshold = 0;
  private static Constructor JDBC_4_LB_CONNECTION_CTOR;
  
  static
  {
    try
    {
      getLocalTimeMethod = System.class.getMethod("nanoTime", new Class[0]);
    }
    catch (SecurityException e) {}catch (NoSuchMethodException e) {}
    

























































    globalBlacklist = new HashMap();
    















    if (Util.isJdbc4()) {
      try {
        JDBC_4_LB_CONNECTION_CTOR = Class.forName("com.mysql.jdbc.JDBC4LoadBalancedMySQLConnection").getConstructor(new Class[] { LoadBalancingConnectionProxy.class });
      }
      catch (SecurityException e)
      {
        throw new RuntimeException(e);
      } catch (NoSuchMethodException e) {
        throw new RuntimeException(e);
      } catch (ClassNotFoundException e) {
        throw new RuntimeException(e);
      }
    }
  }
  














  LoadBalancingConnectionProxy(List<String> hosts, Properties props)
    throws SQLException
  {
    String group = props.getProperty("loadBalanceConnectionGroup", null);
    
    boolean enableJMX = false;
    String enableJMXAsString = props.getProperty("loadBalanceEnableJMX", "false");
    try
    {
      enableJMX = Boolean.parseBoolean(enableJMXAsString);
    } catch (Exception e) {
      throw SQLError.createSQLException(Messages.getString("LoadBalancingConnectionProxy.badValueForLoadBalanceEnableJMX", new Object[] { enableJMXAsString }), "S1009", null);
    }
    



    if (group != null) {
      connectionGroup = ConnectionGroupManager.getConnectionGroupInstance(group);
      if (enableJMX) {
        ConnectionGroupManager.registerJmx();
      }
      connectionGroupProxyID = connectionGroup.registerConnectionProxy(this, hosts);
      hosts = new ArrayList(connectionGroup.getInitialHosts());
    }
    
    hostList = hosts;
    
    int numHosts = hostList.size();
    
    liveConnections = new HashMap(numHosts);
    connectionsToHostsMap = new HashMap(numHosts);
    responseTimes = new long[numHosts];
    hostsToListIndexMap = new HashMap(numHosts);
    
    localProps = ((Properties)props.clone());
    localProps.remove("HOST");
    localProps.remove("PORT");
    
    for (int i = 0; i < numHosts; i++) {
      hostsToListIndexMap.put(hostList.get(i), new Integer(i));
      localProps.remove("HOST." + (i + 1));
      
      localProps.remove("PORT." + (i + 1));
    }
    

    localProps.remove("NUM_HOSTS");
    localProps.setProperty("useLocalSessionState", "true");
    
    String strategy = localProps.getProperty("loadBalanceStrategy", "random");
    

    String lbExceptionChecker = localProps.getProperty("loadBalanceExceptionChecker", "com.mysql.jdbc.StandardLoadBalanceExceptionChecker");
    


    String retriesAllDownAsString = localProps.getProperty("retriesAllDown", "120");
    
    try
    {
      retriesAllDown = Integer.parseInt(retriesAllDownAsString);
    } catch (NumberFormatException nfe) {
      throw SQLError.createSQLException(Messages.getString("LoadBalancingConnectionProxy.badValueForRetriesAllDown", new Object[] { retriesAllDownAsString }), "S1009", null);
    }
    


    String blacklistTimeoutAsString = localProps.getProperty("loadBalanceBlacklistTimeout", "0");
    
    try
    {
      globalBlacklistTimeout = Integer.parseInt(blacklistTimeoutAsString);
    }
    catch (NumberFormatException nfe) {
      throw SQLError.createSQLException(Messages.getString("LoadBalancingConnectionProxy.badValueForLoadBalanceBlacklistTimeout", new Object[] { retriesAllDownAsString }), "S1009", null);
    }
    






    if ("random".equals(strategy)) {
      balancer = ((BalanceStrategy)Util.loadExtensions(null, props, "com.mysql.jdbc.RandomBalanceStrategy", "InvalidLoadBalanceStrategy", null).get(0));

    }
    else if ("bestResponseTime".equals(strategy)) {
      balancer = ((BalanceStrategy)Util.loadExtensions(null, props, "com.mysql.jdbc.BestResponseTimeBalanceStrategy", "InvalidLoadBalanceStrategy", null).get(0));
    }
    else
    {
      balancer = ((BalanceStrategy)Util.loadExtensions(null, props, strategy, "InvalidLoadBalanceStrategy", null).get(0));
    }
    

    String autoCommitSwapThresholdAsString = props.getProperty("loadBalanceAutoCommitStatementThreshold", "0");
    try
    {
      autoCommitSwapThreshold = Integer.parseInt(autoCommitSwapThresholdAsString);
    } catch (NumberFormatException nfe) {
      throw SQLError.createSQLException(Messages.getString("LoadBalancingConnectionProxy.badValueForLoadBalanceAutoCommitStatementThreshold", new Object[] { autoCommitSwapThresholdAsString }), "S1009", null);
    }
    



    String autoCommitSwapRegex = props.getProperty("loadBalanceAutoCommitStatementRegex", "");
    if (!"".equals(autoCommitSwapRegex)) {
      try {
        "".matches(autoCommitSwapRegex);
      } catch (Exception e) {
        throw SQLError.createSQLException(Messages.getString("LoadBalancingConnectionProxy.badValueForLoadBalanceAutoCommitStatementRegex", new Object[] { autoCommitSwapRegex }), "S1009", null);
      }
    }
    



    if (autoCommitSwapThreshold > 0) {
      String statementInterceptors = localProps.getProperty("statementInterceptors");
      if (statementInterceptors == null) {
        localProps.setProperty("statementInterceptors", "com.mysql.jdbc.LoadBalancedAutoCommitInterceptor");
      } else if (statementInterceptors.length() > 0) {
        localProps.setProperty("statementInterceptors", statementInterceptors + ",com.mysql.jdbc.LoadBalancedAutoCommitInterceptor");
      }
      props.setProperty("statementInterceptors", localProps.getProperty("statementInterceptors"));
    }
    
    balancer.init(null, props);
    

    exceptionChecker = ((LoadBalanceExceptionChecker)Util.loadExtensions(null, props, lbExceptionChecker, "InvalidLoadBalanceExceptionChecker", null).get(0));
    
    exceptionChecker.init(null, props);
    
    if ((Util.isJdbc4()) || (JDBC_4_LB_CONNECTION_CTOR != null)) {
      thisAsConnection = ((MySQLConnection)Util.handleNewInstance(JDBC_4_LB_CONNECTION_CTOR, new Object[] { this }, null));
    }
    else {
      thisAsConnection = new LoadBalancedMySQLConnection(this);
    }
    pickNewConnection();
  }
  









  public synchronized ConnectionImpl createConnectionForHost(String hostPortSpec)
    throws SQLException
  {
    Properties connProps = (Properties)localProps.clone();
    
    String[] hostPortPair = NonRegisteringDriver.parseHostPortPair(hostPortSpec);
    
    String hostName = hostPortPair[0];
    String portNumber = hostPortPair[1];
    String dbName = connProps.getProperty("DBNAME");
    

    if (hostName == null) {
      throw new SQLException("Could not find a hostname to start a connection to");
    }
    
    if (portNumber == null) {
      portNumber = "3306";
    }
    
    connProps.setProperty("HOST", hostName);
    connProps.setProperty("PORT", portNumber);
    
    connProps.setProperty("HOST.1", hostName);
    
    connProps.setProperty("PORT.1", portNumber);
    
    connProps.setProperty("NUM_HOSTS", "1");
    connProps.setProperty("roundRobinLoadBalance", "false");
    





    ConnectionImpl conn = (ConnectionImpl)ConnectionImpl.getInstance(hostName, Integer.parseInt(portNumber), connProps, dbName, "jdbc:mysql://" + hostName + ":" + portNumber + "/");
    




    liveConnections.put(hostPortSpec, conn);
    connectionsToHostsMap.put(conn, hostPortSpec);
    

    activePhysicalConnections += 1L;
    totalPhysicalConnections += 1L;
    
    conn.setProxy(thisAsConnection);
    
    return conn;
  }
  





  void dealWithInvocationException(InvocationTargetException e)
    throws SQLException, Throwable, InvocationTargetException
  {
    Throwable t = e.getTargetException();
    
    if (t != null) {
      if (((t instanceof SQLException)) && (shouldExceptionTriggerFailover((SQLException)t))) {
        invalidateCurrentConnection();
        pickNewConnection();
      }
      
      throw t;
    }
    
    throw e;
  }
  


  synchronized void invalidateCurrentConnection()
    throws SQLException
  {
    try
    {
      if (!currentConn.isClosed()) {
        currentConn.close();
      }
    }
    finally
    {
      if (isGlobalBlacklistEnabled()) {
        addToGlobalBlacklist((String)connectionsToHostsMap.get(currentConn));
      }
      


      liveConnections.remove(connectionsToHostsMap.get(currentConn));
      
      Object mappedHost = connectionsToHostsMap.remove(currentConn);
      
      if ((mappedHost != null) && (hostsToListIndexMap.containsKey(mappedHost)))
      {
        int hostIndex = ((Integer)hostsToListIndexMap.get(mappedHost)).intValue();
        

        synchronized (responseTimes) {
          responseTimes[hostIndex] = 0L;
        }
      }
    }
  }
  
  private void closeAllConnections() {
    synchronized (this)
    {
      Iterator<ConnectionImpl> allConnections = liveConnections.values().iterator();
      
      while (allConnections.hasNext()) {
        try {
          activePhysicalConnections -= 1L;
          ((ConnectionImpl)allConnections.next()).close();
        }
        catch (SQLException e) {}
      }
      
      if (!isClosed) {
        balancer.destroy();
        if (connectionGroup != null) {
          connectionGroup.closeConnectionProxy(this);
        }
      }
      
      liveConnections.clear();
      connectionsToHostsMap.clear();
    }
  }
  
  public Object invoke(Object proxy, Method method, Object[] args)
    throws Throwable
  {
    return invoke(proxy, method, args, true);
  }
  





  public Object invoke(Object proxy, Method method, Object[] args, boolean swapAtTransactionBoundary)
    throws Throwable
  {
    String methodName = method.getName();
    
    if ("getLoadBalanceSafeProxy".equals(methodName)) {
      return currentConn;
    }
    
    if (("equals".equals(methodName)) && (args.length == 1)) {
      if ((args[0] instanceof Proxy)) {
        return Boolean.valueOf(((Proxy)args[0]).equals(this));
      }
      return Boolean.valueOf(equals(args[0]));
    }
    
    if ("hashCode".equals(methodName)) {
      return new Integer(hashCode());
    }
    
    if ("close".equals(methodName)) {
      closeAllConnections();
      isClosed = true;
      closedReason = "Connection explicitly closed.";
      return null;
    }
    
    if ("isClosed".equals(methodName)) {
      return Boolean.valueOf(isClosed);
    }
    
    if (isClosed) {
      String reason = "No operations allowed after connection closed.";
      if (closedReason != null) {
        reason = reason + "  " + closedReason;
      }
      throw SQLError.createSQLException(reason, "08003", null);
    }
    






    if (!inTransaction) {
      inTransaction = true;
      transactionStartTime = getLocalTimeBestResolution();
      transactionCount += 1L;
    }
    
    Object result = null;
    try
    {
      lastUsed = System.currentTimeMillis();
      result = method.invoke(thisAsConnection, args);
      
      if (result != null) {
        if ((result instanceof Statement)) {
          ((Statement)result).setPingTarget(this);
        }
        
        result = proxyIfInterfaceIsJdbc(result, result.getClass());
      }
    } catch (InvocationTargetException e) {
      dealWithInvocationException(e);
    } finally {
      if ((swapAtTransactionBoundary) && (("commit".equals(methodName)) || ("rollback".equals(methodName)))) {
        inTransaction = false;
        

        String host = (String)connectionsToHostsMap.get(currentConn);
        


        if (host != null) {
          synchronized (responseTimes) {
            int hostIndex = ((Integer)hostsToListIndexMap.get(host)).intValue();
            

            if (hostIndex < responseTimes.length) {
              responseTimes[hostIndex] = (getLocalTimeBestResolution() - transactionStartTime);
            }
          }
        }
        
        pickNewConnection();
      }
    }
    
    return result;
  }
  




  protected synchronized void pickNewConnection()
    throws SQLException
  {
    if (currentConn == null) {
      currentConn = balancer.pickConnection(this, Collections.unmodifiableList(hostList), Collections.unmodifiableMap(liveConnections), (long[])responseTimes.clone(), retriesAllDown);
      


      return;
    }
    
    if (currentConn.isClosed()) {
      invalidateCurrentConnection();
    }
    
    int pingTimeout = currentConn.getLoadBalancePingTimeout();
    boolean pingBeforeReturn = currentConn.getLoadBalanceValidateConnectionOnSwapServer();
    
    int hostsTried = 0; for (int hostsToTry = hostList.size(); hostsTried <= hostsToTry; hostsTried++) {
      try {
        ConnectionImpl newConn = balancer.pickConnection(this, Collections.unmodifiableList(hostList), Collections.unmodifiableMap(liveConnections), (long[])responseTimes.clone(), retriesAllDown);
        




        if (currentConn != null) {
          if (pingBeforeReturn) {
            if (pingTimeout == 0) {
              newConn.ping();
            } else {
              newConn.pingInternal(true, pingTimeout);
            }
          }
          
          syncSessionState(currentConn, newConn);
        }
        
        currentConn = newConn;
        return;
      }
      catch (SQLException e) {
        if (shouldExceptionTriggerFailover(e))
        {

          invalidateCurrentConnection();
        }
      }
    }
    

    isClosed = true;
    closedReason = "Connection closed after inability to pick valid new connection during fail-over.";
  }
  










  Object proxyIfInterfaceIsJdbc(Object toProxy, Class clazz)
  {
    if (isInterfaceJdbc(clazz))
    {
      Class[] interfacesToProxy = getAllInterfacesToProxy(clazz);
      
      return Proxy.newProxyInstance(toProxy.getClass().getClassLoader(), interfacesToProxy, createConnectionProxy(toProxy));
    }
    


    return toProxy;
  }
  
  private Map<Class, Class[]> allInterfacesToProxy = new HashMap();
  
  private Class[] getAllInterfacesToProxy(Class clazz) {
    Class[] interfacesToProxy = (Class[])allInterfacesToProxy.get(clazz);
    
    if (interfacesToProxy != null) {
      return interfacesToProxy;
    }
    
    List<Class> interfaces = new LinkedList();
    
    Class superClass = clazz;
    
    while (!superClass.equals(Object.class)) {
      Class[] declared = superClass.getInterfaces();
      
      for (int i = 0; i < declared.length; i++) {
        interfaces.add(declared[i]);
      }
      
      superClass = superClass.getSuperclass();
    }
    
    interfacesToProxy = new Class[interfaces.size()];
    interfaces.toArray(interfacesToProxy);
    
    allInterfacesToProxy.put(clazz, interfacesToProxy);
    
    return interfacesToProxy;
  }
  
  private boolean isInterfaceJdbc(Class clazz)
  {
    if (jdbcInterfacesForProxyCache.containsKey(clazz)) {
      return ((Boolean)jdbcInterfacesForProxyCache.get(clazz)).booleanValue();
    }
    
    Class[] interfaces = clazz.getInterfaces();
    
    for (int i = 0; i < interfaces.length; i++) {
      String packageName = interfaces[i].getPackage().getName();
      
      if (("java.sql".equals(packageName)) || ("javax.sql".equals(packageName)) || ("com.mysql.jdbc".equals(packageName)))
      {

        jdbcInterfacesForProxyCache.put(clazz, new Boolean(true));
        return true;
      }
      
      if (isInterfaceJdbc(interfaces[i])) {
        jdbcInterfacesForProxyCache.put(clazz, new Boolean(true));
        return true;
      }
    }
    
    jdbcInterfacesForProxyCache.put(clazz, new Boolean(false));
    return false;
  }
  

  protected ConnectionErrorFiringInvocationHandler createConnectionProxy(Object toProxy)
  {
    return new ConnectionErrorFiringInvocationHandler(toProxy);
  }
  



  private static long getLocalTimeBestResolution()
  {
    if (getLocalTimeMethod != null) {
      try {
        return ((Long)getLocalTimeMethod.invoke(null, (Object[])null)).longValue();
      }
      catch (IllegalArgumentException e) {}catch (IllegalAccessException e) {}catch (InvocationTargetException e) {}
    }
    






    return System.currentTimeMillis();
  }
  
  public synchronized void doPing() throws SQLException {
    SQLException se = null;
    boolean foundHost = false;
    int pingTimeout = currentConn.getLoadBalancePingTimeout();
    Iterator<String> i; synchronized (this) {
      for (i = hostList.iterator(); i.hasNext();) {
        String host = (String)i.next();
        ConnectionImpl conn = (ConnectionImpl)liveConnections.get(host);
        if (conn != null)
        {
          try
          {
            if (pingTimeout == 0) {
              conn.ping();
            } else {
              conn.pingInternal(true, pingTimeout);
            }
            foundHost = true;
          } catch (SQLException e) {
            activePhysicalConnections -= 1L;
            

            if (host.equals(connectionsToHostsMap.get(currentConn)))
            {


              closeAllConnections();
              isClosed = true;
              closedReason = "Connection closed because ping of current connection failed.";
              throw e;
            }
            


            if (e.getMessage().equals(Messages.getString("Connection.exceededConnectionLifetime")))
            {




              if (se == null) {
                se = e;
              }
            }
            else {
              se = e;
              if (isGlobalBlacklistEnabled()) {
                addToGlobalBlacklist(host);
              }
            }
            
            liveConnections.remove(connectionsToHostsMap.get(conn));
          }
        }
      }
    }
    
    if (!foundHost) {
      closeAllConnections();
      isClosed = true;
      closedReason = "Connection closed due to inability to ping any active connections.";
      
      if (se != null) {
        throw se;
      }
      

      ((ConnectionImpl)currentConn).throwConnectionClosedException();
    }
  }
  
  public void addToGlobalBlacklist(String host, long timeout)
  {
    if (isGlobalBlacklistEnabled()) {
      synchronized (globalBlacklist) {
        globalBlacklist.put(host, new Long(timeout));
      }
    }
  }
  
  public void addToGlobalBlacklist(String host) {
    addToGlobalBlacklist(host, System.currentTimeMillis() + globalBlacklistTimeout);
  }
  

  public boolean isGlobalBlacklistEnabled()
  {
    return globalBlacklistTimeout > 0;
  }
  
  public Map<String, Long> getGlobalBlacklist() {
    if (!isGlobalBlacklistEnabled()) {
      String localHostToRemove = hostToRemove;
      if (hostToRemove != null) {
        HashMap<String, Long> fakedBlacklist = new HashMap();
        fakedBlacklist.put(localHostToRemove, new Long(System.currentTimeMillis() + 5000L));
        return fakedBlacklist;
      }
      return new HashMap(1);
    }
    

    Map<String, Long> blacklistClone = new HashMap(globalBlacklist.size());
    

    synchronized (globalBlacklist) {
      blacklistClone.putAll(globalBlacklist);
    }
    Set<String> keys = blacklistClone.keySet();
    

    keys.retainAll(hostList);
    if (keys.size() == hostList.size())
    {




      return new HashMap(1);
    }
    

    for (Object i = keys.iterator(); ((Iterator)i).hasNext();) {
      String host = (String)((Iterator)i).next();
      

      Long timeout = (Long)globalBlacklist.get(host);
      if ((timeout != null) && (timeout.longValue() < System.currentTimeMillis()))
      {

        synchronized (globalBlacklist) {
          globalBlacklist.remove(host);
        }
        ((Iterator)i).remove();
      }
    }
    

    return blacklistClone;
  }
  
  public boolean shouldExceptionTriggerFailover(SQLException ex) {
    return exceptionChecker.shouldExceptionTriggerFailover(ex);
  }
  
  public void removeHostWhenNotInUse(String host) throws SQLException
  {
    int timeBetweenChecks = 1000;
    long timeBeforeHardFail = 15000L;
    addToGlobalBlacklist(host, timeBeforeHardFail + 1000L);
    long cur = System.currentTimeMillis();
    while (System.currentTimeMillis() - timeBeforeHardFail < cur) {
      synchronized (this) {
        hostToRemove = host;
        if (!host.equals(currentConn.getHost())) {
          removeHost(host);
          return;
        }
      }
      try {
        Thread.sleep(timeBetweenChecks);
      }
      catch (InterruptedException e) {}
    }
    

    removeHost(host);
  }
  
  public void removeHost(String host) throws SQLException {
    synchronized (this) {
      if ((connectionGroup != null) && 
        (connectionGroup.getInitialHosts().size() == 1) && (connectionGroup.getInitialHosts().contains(host))) {
        throw SQLError.createSQLException("Cannot remove only configured host.", null);
      }
      
      hostToRemove = host;
      if (host.equals(currentConn.getHost())) {
        closeAllConnections();
      } else {
        connectionsToHostsMap.remove(liveConnections.remove(host));
        Integer idx = (Integer)hostsToListIndexMap.remove(host);
        long[] newResponseTimes = new long[responseTimes.length - 1];
        int newIdx = 0;
        for (Iterator<String> i = hostList.iterator(); i.hasNext(); newIdx++) {
          String copyHost = (String)i.next();
          if ((idx != null) && (idx.intValue() < responseTimes.length)) {
            newResponseTimes[newIdx] = responseTimes[idx.intValue()];
            hostsToListIndexMap.put(copyHost, new Integer(newIdx));
          }
        }
        responseTimes = newResponseTimes;
      }
    }
  }
  
  public synchronized boolean addHost(String host)
  {
    synchronized (this) {
      if (hostsToListIndexMap.containsKey(host)) {
        return false;
      }
      long[] newResponseTimes = new long[responseTimes.length + 1];
      for (int i = 0; i < responseTimes.length; i++) {
        newResponseTimes[i] = responseTimes[i];
      }
      responseTimes = newResponseTimes;
      hostList.add(host);
      hostsToListIndexMap.put(host, new Integer(responseTimes.length - 1));
    }
    return true;
  }
  

  public long getLastUsed()
  {
    return lastUsed;
  }
  
  public boolean inTransaction() {
    return inTransaction;
  }
  
  public long getTransactionCount() {
    return transactionCount;
  }
  
  public long getActivePhysicalConnectionCount() {
    return activePhysicalConnections;
  }
  
  public long getTotalPhysicalConnectionCount() { return totalPhysicalConnections; }
  
  public long getConnectionGroupProxyID()
  {
    return connectionGroupProxyID;
  }
  
  public String getCurrentActiveHost() {
    MySQLConnection c = currentConn;
    if (c != null) {
      Object o = connectionsToHostsMap.get(c);
      if (o != null) {
        return o.toString();
      }
    }
    return null;
  }
  
  public long getCurrentTransactionDuration() {
    long st = 0L;
    
    if ((inTransaction) && ((st = transactionStartTime) > 0L)) {
      return getLocalTimeBestResolution() - transactionStartTime;
    }
    return 0L;
  }
  
  protected void syncSessionState(Connection initial, Connection target) throws SQLException
  {
    if ((initial == null) || (target == null)) {
      return;
    }
    target.setAutoCommit(initial.getAutoCommit());
    target.setCatalog(initial.getCatalog());
    target.setTransactionIsolation(initial.getTransactionIsolation());
    target.setReadOnly(initial.isReadOnly());
  }
}
