package com.mysql.jdbc;

import com.mysql.jdbc.log.Log;
import java.sql.CallableStatement;
import java.sql.DatabaseMetaData;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.Savepoint;
import java.util.Map;
import java.util.Properties;
import java.util.TimeZone;































public class ReplicationConnection
  implements Connection, PingTarget
{
  protected Connection currentConnection;
  protected Connection masterConnection;
  protected Connection slavesConnection;
  
  protected ReplicationConnection() {}
  
  public ReplicationConnection(Properties masterProperties, Properties slaveProperties)
    throws SQLException
  {
    NonRegisteringDriver driver = new NonRegisteringDriver();
    
    StringBuffer masterUrl = new StringBuffer("jdbc:mysql://");
    StringBuffer slaveUrl = new StringBuffer("jdbc:mysql:loadbalance://");
    
    String masterHost = masterProperties.getProperty("HOST");
    

    if (masterHost != null) {
      masterUrl.append(masterHost);
    }
    
    int numHosts = Integer.parseInt(slaveProperties.getProperty("NUM_HOSTS"));
    

    for (int i = 1; i <= numHosts; i++) {
      String slaveHost = slaveProperties.getProperty("HOST." + i);
      

      if (slaveHost != null) {
        if (i > 1) {
          slaveUrl.append(',');
        }
        slaveUrl.append(slaveHost);
      }
    }
    
    String masterDb = masterProperties.getProperty("DBNAME");
    

    masterUrl.append("/");
    
    if (masterDb != null) {
      masterUrl.append(masterDb);
    }
    
    String slaveDb = slaveProperties.getProperty("DBNAME");
    

    slaveUrl.append("/");
    
    if (slaveDb != null) {
      slaveUrl.append(slaveDb);
    }
    
    slaveProperties.setProperty("roundRobinLoadBalance", "true");
    
    masterConnection = ((Connection)driver.connect(masterUrl.toString(), masterProperties));
    
    slavesConnection = ((Connection)driver.connect(slaveUrl.toString(), slaveProperties));
    
    slavesConnection.setReadOnly(true);
    
    currentConnection = masterConnection;
  }
  



  public synchronized void clearWarnings()
    throws SQLException
  {
    currentConnection.clearWarnings();
  }
  



  public synchronized void close()
    throws SQLException
  {
    masterConnection.close();
    slavesConnection.close();
  }
  



  public synchronized void commit()
    throws SQLException
  {
    currentConnection.commit();
  }
  



  public java.sql.Statement createStatement()
    throws SQLException
  {
    java.sql.Statement stmt = currentConnection.createStatement();
    ((Statement)stmt).setPingTarget(this);
    
    return stmt;
  }
  




  public synchronized java.sql.Statement createStatement(int resultSetType, int resultSetConcurrency)
    throws SQLException
  {
    java.sql.Statement stmt = currentConnection.createStatement(resultSetType, resultSetConcurrency);
    

    ((Statement)stmt).setPingTarget(this);
    
    return stmt;
  }
  





  public synchronized java.sql.Statement createStatement(int resultSetType, int resultSetConcurrency, int resultSetHoldability)
    throws SQLException
  {
    java.sql.Statement stmt = currentConnection.createStatement(resultSetType, resultSetConcurrency, resultSetHoldability);
    

    ((Statement)stmt).setPingTarget(this);
    
    return stmt;
  }
  



  public synchronized boolean getAutoCommit()
    throws SQLException
  {
    return currentConnection.getAutoCommit();
  }
  



  public synchronized String getCatalog()
    throws SQLException
  {
    return currentConnection.getCatalog();
  }
  
  public synchronized Connection getCurrentConnection() {
    return currentConnection;
  }
  



  public synchronized int getHoldability()
    throws SQLException
  {
    return currentConnection.getHoldability();
  }
  
  public synchronized Connection getMasterConnection() {
    return masterConnection;
  }
  



  public synchronized DatabaseMetaData getMetaData()
    throws SQLException
  {
    return currentConnection.getMetaData();
  }
  
  public synchronized Connection getSlavesConnection() {
    return slavesConnection;
  }
  



  public synchronized int getTransactionIsolation()
    throws SQLException
  {
    return currentConnection.getTransactionIsolation();
  }
  



  public synchronized Map getTypeMap()
    throws SQLException
  {
    return currentConnection.getTypeMap();
  }
  



  public synchronized SQLWarning getWarnings()
    throws SQLException
  {
    return currentConnection.getWarnings();
  }
  



  public synchronized boolean isClosed()
    throws SQLException
  {
    return currentConnection.isClosed();
  }
  



  public synchronized boolean isReadOnly()
    throws SQLException
  {
    return currentConnection == slavesConnection;
  }
  



  public synchronized String nativeSQL(String sql)
    throws SQLException
  {
    return currentConnection.nativeSQL(sql);
  }
  



  public CallableStatement prepareCall(String sql)
    throws SQLException
  {
    return currentConnection.prepareCall(sql);
  }
  




  public synchronized CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency)
    throws SQLException
  {
    return currentConnection.prepareCall(sql, resultSetType, resultSetConcurrency);
  }
  






  public synchronized CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency, int resultSetHoldability)
    throws SQLException
  {
    return currentConnection.prepareCall(sql, resultSetType, resultSetConcurrency, resultSetHoldability);
  }
  




  public PreparedStatement prepareStatement(String sql)
    throws SQLException
  {
    PreparedStatement pstmt = currentConnection.prepareStatement(sql);
    
    ((Statement)pstmt).setPingTarget(this);
    
    return pstmt;
  }
  




  public synchronized PreparedStatement prepareStatement(String sql, int autoGeneratedKeys)
    throws SQLException
  {
    PreparedStatement pstmt = currentConnection.prepareStatement(sql, autoGeneratedKeys);
    
    ((Statement)pstmt).setPingTarget(this);
    
    return pstmt;
  }
  




  public synchronized PreparedStatement prepareStatement(String sql, int resultSetType, int resultSetConcurrency)
    throws SQLException
  {
    PreparedStatement pstmt = currentConnection.prepareStatement(sql, resultSetType, resultSetConcurrency);
    

    ((Statement)pstmt).setPingTarget(this);
    
    return pstmt;
  }
  






  public synchronized PreparedStatement prepareStatement(String sql, int resultSetType, int resultSetConcurrency, int resultSetHoldability)
    throws SQLException
  {
    PreparedStatement pstmt = currentConnection.prepareStatement(sql, resultSetType, resultSetConcurrency, resultSetHoldability);
    

    ((Statement)pstmt).setPingTarget(this);
    
    return pstmt;
  }
  




  public synchronized PreparedStatement prepareStatement(String sql, int[] columnIndexes)
    throws SQLException
  {
    PreparedStatement pstmt = currentConnection.prepareStatement(sql, columnIndexes);
    
    ((Statement)pstmt).setPingTarget(this);
    
    return pstmt;
  }
  





  public synchronized PreparedStatement prepareStatement(String sql, String[] columnNames)
    throws SQLException
  {
    PreparedStatement pstmt = currentConnection.prepareStatement(sql, columnNames);
    
    ((Statement)pstmt).setPingTarget(this);
    
    return pstmt;
  }
  




  public synchronized void releaseSavepoint(Savepoint savepoint)
    throws SQLException
  {
    currentConnection.releaseSavepoint(savepoint);
  }
  



  public synchronized void rollback()
    throws SQLException
  {
    currentConnection.rollback();
  }
  



  public synchronized void rollback(Savepoint savepoint)
    throws SQLException
  {
    currentConnection.rollback(savepoint);
  }
  




  public synchronized void setAutoCommit(boolean autoCommit)
    throws SQLException
  {
    currentConnection.setAutoCommit(autoCommit);
  }
  



  public synchronized void setCatalog(String catalog)
    throws SQLException
  {
    currentConnection.setCatalog(catalog);
  }
  




  public synchronized void setHoldability(int holdability)
    throws SQLException
  {
    currentConnection.setHoldability(holdability);
  }
  



  public synchronized void setReadOnly(boolean readOnly)
    throws SQLException
  {
    if (readOnly) {
      if (currentConnection != slavesConnection) {
        switchToSlavesConnection();
      }
    }
    else if (currentConnection != masterConnection) {
      switchToMasterConnection();
    }
  }
  




  public synchronized Savepoint setSavepoint()
    throws SQLException
  {
    return currentConnection.setSavepoint();
  }
  



  public synchronized Savepoint setSavepoint(String name)
    throws SQLException
  {
    return currentConnection.setSavepoint(name);
  }
  




  public synchronized void setTransactionIsolation(int level)
    throws SQLException
  {
    currentConnection.setTransactionIsolation(level);
  }
  





  public synchronized void setTypeMap(Map arg0)
    throws SQLException
  {
    currentConnection.setTypeMap(arg0);
  }
  
  private synchronized void switchToMasterConnection() throws SQLException {
    swapConnections(masterConnection, slavesConnection);
  }
  
  private synchronized void switchToSlavesConnection() throws SQLException {
    swapConnections(slavesConnection, masterConnection);
  }
  









  private synchronized void swapConnections(Connection switchToConnection, Connection switchFromConnection)
    throws SQLException
  {
    String switchFromCatalog = switchFromConnection.getCatalog();
    String switchToCatalog = switchToConnection.getCatalog();
    
    if ((switchToCatalog != null) && (!switchToCatalog.equals(switchFromCatalog))) {
      switchToConnection.setCatalog(switchFromCatalog);
    } else if (switchFromCatalog != null) {
      switchToConnection.setCatalog(switchFromCatalog);
    }
    
    boolean switchToAutoCommit = switchToConnection.getAutoCommit();
    boolean switchFromConnectionAutoCommit = switchFromConnection.getAutoCommit();
    
    if (switchFromConnectionAutoCommit != switchToAutoCommit) {
      switchToConnection.setAutoCommit(switchFromConnectionAutoCommit);
    }
    
    int switchToIsolation = switchToConnection.getTransactionIsolation();
    

    int switchFromIsolation = switchFromConnection.getTransactionIsolation();
    
    if (switchFromIsolation != switchToIsolation) {
      switchToConnection.setTransactionIsolation(switchFromIsolation);
    }
    

    currentConnection = switchToConnection;
  }
  
  public synchronized void doPing() throws SQLException {
    if (masterConnection != null) {
      masterConnection.ping();
    }
    
    if (slavesConnection != null) {
      slavesConnection.ping();
    }
  }
  
  public synchronized void changeUser(String userName, String newPassword) throws SQLException
  {
    masterConnection.changeUser(userName, newPassword);
    slavesConnection.changeUser(userName, newPassword);
  }
  
  public synchronized void clearHasTriedMaster() {
    masterConnection.clearHasTriedMaster();
    slavesConnection.clearHasTriedMaster();
  }
  
  public synchronized PreparedStatement clientPrepareStatement(String sql)
    throws SQLException
  {
    PreparedStatement pstmt = currentConnection.clientPrepareStatement(sql);
    ((Statement)pstmt).setPingTarget(this);
    
    return pstmt;
  }
  
  public synchronized PreparedStatement clientPrepareStatement(String sql, int autoGenKeyIndex) throws SQLException
  {
    PreparedStatement pstmt = currentConnection.clientPrepareStatement(sql, autoGenKeyIndex);
    ((Statement)pstmt).setPingTarget(this);
    
    return pstmt;
  }
  
  public synchronized PreparedStatement clientPrepareStatement(String sql, int resultSetType, int resultSetConcurrency) throws SQLException
  {
    PreparedStatement pstmt = currentConnection.clientPrepareStatement(sql, resultSetType, resultSetConcurrency);
    ((Statement)pstmt).setPingTarget(this);
    
    return pstmt;
  }
  
  public synchronized PreparedStatement clientPrepareStatement(String sql, int[] autoGenKeyIndexes) throws SQLException
  {
    PreparedStatement pstmt = currentConnection.clientPrepareStatement(sql, autoGenKeyIndexes);
    ((Statement)pstmt).setPingTarget(this);
    
    return pstmt;
  }
  
  public synchronized PreparedStatement clientPrepareStatement(String sql, int resultSetType, int resultSetConcurrency, int resultSetHoldability)
    throws SQLException
  {
    PreparedStatement pstmt = currentConnection.clientPrepareStatement(sql, resultSetType, resultSetConcurrency, resultSetHoldability);
    ((Statement)pstmt).setPingTarget(this);
    
    return pstmt;
  }
  
  public synchronized PreparedStatement clientPrepareStatement(String sql, String[] autoGenKeyColNames) throws SQLException
  {
    PreparedStatement pstmt = currentConnection.clientPrepareStatement(sql, autoGenKeyColNames);
    ((Statement)pstmt).setPingTarget(this);
    
    return pstmt;
  }
  
  public synchronized int getActiveStatementCount() {
    return currentConnection.getActiveStatementCount();
  }
  
  public synchronized long getIdleFor() {
    return currentConnection.getIdleFor();
  }
  
  public synchronized Log getLog() throws SQLException {
    return currentConnection.getLog();
  }
  
  public synchronized String getServerCharacterEncoding() {
    return currentConnection.getServerCharacterEncoding();
  }
  
  public synchronized TimeZone getServerTimezoneTZ() {
    return currentConnection.getServerTimezoneTZ();
  }
  
  public synchronized String getStatementComment() {
    return currentConnection.getStatementComment();
  }
  
  public synchronized boolean hasTriedMaster() {
    return currentConnection.hasTriedMaster();
  }
  
  public synchronized void initializeExtension(Extension ex) throws SQLException {
    currentConnection.initializeExtension(ex);
  }
  
  public synchronized boolean isAbonormallyLongQuery(long millisOrNanos) {
    return currentConnection.isAbonormallyLongQuery(millisOrNanos);
  }
  
  public synchronized boolean isInGlobalTx() {
    return currentConnection.isInGlobalTx();
  }
  
  public synchronized boolean isMasterConnection() {
    return currentConnection.isMasterConnection();
  }
  
  public synchronized boolean isNoBackslashEscapesSet() {
    return currentConnection.isNoBackslashEscapesSet();
  }
  
  public synchronized boolean lowerCaseTableNames() {
    return currentConnection.lowerCaseTableNames();
  }
  
  public synchronized boolean parserKnowsUnicode() {
    return currentConnection.parserKnowsUnicode();
  }
  
  public synchronized void ping() throws SQLException {
    masterConnection.ping();
    slavesConnection.ping();
  }
  
  public synchronized void reportQueryTime(long millisOrNanos) {
    currentConnection.reportQueryTime(millisOrNanos);
  }
  
  public synchronized void resetServerState() throws SQLException {
    currentConnection.resetServerState();
  }
  
  public synchronized PreparedStatement serverPrepareStatement(String sql) throws SQLException
  {
    PreparedStatement pstmt = currentConnection.serverPrepareStatement(sql);
    ((Statement)pstmt).setPingTarget(this);
    
    return pstmt;
  }
  
  public synchronized PreparedStatement serverPrepareStatement(String sql, int autoGenKeyIndex) throws SQLException
  {
    PreparedStatement pstmt = currentConnection.serverPrepareStatement(sql, autoGenKeyIndex);
    ((Statement)pstmt).setPingTarget(this);
    
    return pstmt;
  }
  
  public synchronized PreparedStatement serverPrepareStatement(String sql, int resultSetType, int resultSetConcurrency) throws SQLException
  {
    PreparedStatement pstmt = currentConnection.serverPrepareStatement(sql, resultSetType, resultSetConcurrency);
    ((Statement)pstmt).setPingTarget(this);
    
    return pstmt;
  }
  
  public synchronized PreparedStatement serverPrepareStatement(String sql, int resultSetType, int resultSetConcurrency, int resultSetHoldability)
    throws SQLException
  {
    PreparedStatement pstmt = currentConnection.serverPrepareStatement(sql, resultSetType, resultSetConcurrency, resultSetHoldability);
    ((Statement)pstmt).setPingTarget(this);
    
    return pstmt;
  }
  
  public synchronized PreparedStatement serverPrepareStatement(String sql, int[] autoGenKeyIndexes) throws SQLException
  {
    PreparedStatement pstmt = currentConnection.serverPrepareStatement(sql, autoGenKeyIndexes);
    ((Statement)pstmt).setPingTarget(this);
    
    return pstmt;
  }
  
  public synchronized PreparedStatement serverPrepareStatement(String sql, String[] autoGenKeyColNames) throws SQLException
  {
    PreparedStatement pstmt = currentConnection.serverPrepareStatement(sql, autoGenKeyColNames);
    ((Statement)pstmt).setPingTarget(this);
    
    return pstmt;
  }
  
  public synchronized void setFailedOver(boolean flag) {
    currentConnection.setFailedOver(flag);
  }
  
  public synchronized void setPreferSlaveDuringFailover(boolean flag) {
    currentConnection.setPreferSlaveDuringFailover(flag);
  }
  
  public synchronized void setStatementComment(String comment) {
    masterConnection.setStatementComment(comment);
    slavesConnection.setStatementComment(comment);
  }
  
  public synchronized void shutdownServer() throws SQLException {
    currentConnection.shutdownServer();
  }
  
  public synchronized boolean supportsIsolationLevel() {
    return currentConnection.supportsIsolationLevel();
  }
  
  public synchronized boolean supportsQuotedIdentifiers() {
    return currentConnection.supportsQuotedIdentifiers();
  }
  
  public synchronized boolean supportsTransactions() {
    return currentConnection.supportsTransactions();
  }
  
  public synchronized boolean versionMeetsMinimum(int major, int minor, int subminor) throws SQLException
  {
    return currentConnection.versionMeetsMinimum(major, minor, subminor);
  }
  
  public synchronized String exposeAsXml() throws SQLException {
    return currentConnection.exposeAsXml();
  }
  
  public synchronized boolean getAllowLoadLocalInfile() {
    return currentConnection.getAllowLoadLocalInfile();
  }
  
  public synchronized boolean getAllowMultiQueries() {
    return currentConnection.getAllowMultiQueries();
  }
  
  public synchronized boolean getAllowNanAndInf() {
    return currentConnection.getAllowNanAndInf();
  }
  
  public synchronized boolean getAllowUrlInLocalInfile() {
    return currentConnection.getAllowUrlInLocalInfile();
  }
  
  public synchronized boolean getAlwaysSendSetIsolation() {
    return currentConnection.getAlwaysSendSetIsolation();
  }
  
  public synchronized boolean getAutoClosePStmtStreams() {
    return currentConnection.getAutoClosePStmtStreams();
  }
  
  public synchronized boolean getAutoDeserialize() {
    return currentConnection.getAutoDeserialize();
  }
  
  public synchronized boolean getAutoGenerateTestcaseScript() {
    return currentConnection.getAutoGenerateTestcaseScript();
  }
  
  public synchronized boolean getAutoReconnectForPools() {
    return currentConnection.getAutoReconnectForPools();
  }
  
  public synchronized boolean getAutoSlowLog() {
    return currentConnection.getAutoSlowLog();
  }
  
  public synchronized int getBlobSendChunkSize() {
    return currentConnection.getBlobSendChunkSize();
  }
  
  public synchronized boolean getBlobsAreStrings() {
    return currentConnection.getBlobsAreStrings();
  }
  
  public synchronized boolean getCacheCallableStatements() {
    return currentConnection.getCacheCallableStatements();
  }
  
  public synchronized boolean getCacheCallableStmts() {
    return currentConnection.getCacheCallableStmts();
  }
  
  public synchronized boolean getCachePrepStmts() {
    return currentConnection.getCachePrepStmts();
  }
  
  public synchronized boolean getCachePreparedStatements() {
    return currentConnection.getCachePreparedStatements();
  }
  
  public synchronized boolean getCacheResultSetMetadata() {
    return currentConnection.getCacheResultSetMetadata();
  }
  
  public synchronized boolean getCacheServerConfiguration() {
    return currentConnection.getCacheServerConfiguration();
  }
  
  public synchronized int getCallableStatementCacheSize() {
    return currentConnection.getCallableStatementCacheSize();
  }
  
  public synchronized int getCallableStmtCacheSize() {
    return currentConnection.getCallableStmtCacheSize();
  }
  
  public synchronized boolean getCapitalizeTypeNames() {
    return currentConnection.getCapitalizeTypeNames();
  }
  
  public synchronized String getCharacterSetResults() {
    return currentConnection.getCharacterSetResults();
  }
  
  public synchronized String getClientCertificateKeyStorePassword() {
    return currentConnection.getClientCertificateKeyStorePassword();
  }
  
  public synchronized String getClientCertificateKeyStoreType() {
    return currentConnection.getClientCertificateKeyStoreType();
  }
  
  public synchronized String getClientCertificateKeyStoreUrl() {
    return currentConnection.getClientCertificateKeyStoreUrl();
  }
  
  public synchronized String getClientInfoProvider() {
    return currentConnection.getClientInfoProvider();
  }
  
  public synchronized String getClobCharacterEncoding() {
    return currentConnection.getClobCharacterEncoding();
  }
  
  public synchronized boolean getClobberStreamingResults() {
    return currentConnection.getClobberStreamingResults();
  }
  
  public synchronized int getConnectTimeout() {
    return currentConnection.getConnectTimeout();
  }
  
  public synchronized String getConnectionCollation() {
    return currentConnection.getConnectionCollation();
  }
  
  public synchronized String getConnectionLifecycleInterceptors() {
    return currentConnection.getConnectionLifecycleInterceptors();
  }
  
  public synchronized boolean getContinueBatchOnError() {
    return currentConnection.getContinueBatchOnError();
  }
  
  public synchronized boolean getCreateDatabaseIfNotExist() {
    return currentConnection.getCreateDatabaseIfNotExist();
  }
  
  public synchronized int getDefaultFetchSize() {
    return currentConnection.getDefaultFetchSize();
  }
  
  public synchronized boolean getDontTrackOpenResources() {
    return currentConnection.getDontTrackOpenResources();
  }
  
  public synchronized boolean getDumpMetadataOnColumnNotFound() {
    return currentConnection.getDumpMetadataOnColumnNotFound();
  }
  
  public synchronized boolean getDumpQueriesOnException() {
    return currentConnection.getDumpQueriesOnException();
  }
  
  public synchronized boolean getDynamicCalendars() {
    return currentConnection.getDynamicCalendars();
  }
  
  public synchronized boolean getElideSetAutoCommits() {
    return currentConnection.getElideSetAutoCommits();
  }
  
  public synchronized boolean getEmptyStringsConvertToZero() {
    return currentConnection.getEmptyStringsConvertToZero();
  }
  
  public synchronized boolean getEmulateLocators() {
    return currentConnection.getEmulateLocators();
  }
  
  public synchronized boolean getEmulateUnsupportedPstmts() {
    return currentConnection.getEmulateUnsupportedPstmts();
  }
  
  public synchronized boolean getEnablePacketDebug() {
    return currentConnection.getEnablePacketDebug();
  }
  
  public synchronized boolean getEnableQueryTimeouts() {
    return currentConnection.getEnableQueryTimeouts();
  }
  
  public synchronized String getEncoding() {
    return currentConnection.getEncoding();
  }
  
  public synchronized boolean getExplainSlowQueries() {
    return currentConnection.getExplainSlowQueries();
  }
  
  public synchronized boolean getFailOverReadOnly() {
    return currentConnection.getFailOverReadOnly();
  }
  
  public synchronized boolean getFunctionsNeverReturnBlobs() {
    return currentConnection.getFunctionsNeverReturnBlobs();
  }
  
  public synchronized boolean getGatherPerfMetrics() {
    return currentConnection.getGatherPerfMetrics();
  }
  
  public synchronized boolean getGatherPerformanceMetrics() {
    return currentConnection.getGatherPerformanceMetrics();
  }
  
  public synchronized boolean getGenerateSimpleParameterMetadata() {
    return currentConnection.getGenerateSimpleParameterMetadata();
  }
  
  public synchronized boolean getHoldResultsOpenOverStatementClose() {
    return currentConnection.getHoldResultsOpenOverStatementClose();
  }
  
  public synchronized boolean getIgnoreNonTxTables() {
    return currentConnection.getIgnoreNonTxTables();
  }
  
  public synchronized boolean getIncludeInnodbStatusInDeadlockExceptions() {
    return currentConnection.getIncludeInnodbStatusInDeadlockExceptions();
  }
  
  public synchronized int getInitialTimeout() {
    return currentConnection.getInitialTimeout();
  }
  
  public synchronized boolean getInteractiveClient() {
    return currentConnection.getInteractiveClient();
  }
  
  public synchronized boolean getIsInteractiveClient() {
    return currentConnection.getIsInteractiveClient();
  }
  
  public synchronized boolean getJdbcCompliantTruncation() {
    return currentConnection.getJdbcCompliantTruncation();
  }
  
  public synchronized boolean getJdbcCompliantTruncationForReads() {
    return currentConnection.getJdbcCompliantTruncationForReads();
  }
  
  public synchronized String getLargeRowSizeThreshold() {
    return currentConnection.getLargeRowSizeThreshold();
  }
  
  public synchronized String getLoadBalanceStrategy() {
    return currentConnection.getLoadBalanceStrategy();
  }
  
  public synchronized String getLocalSocketAddress() {
    return currentConnection.getLocalSocketAddress();
  }
  
  public synchronized int getLocatorFetchBufferSize() {
    return currentConnection.getLocatorFetchBufferSize();
  }
  
  public synchronized boolean getLogSlowQueries() {
    return currentConnection.getLogSlowQueries();
  }
  
  public synchronized boolean getLogXaCommands() {
    return currentConnection.getLogXaCommands();
  }
  
  public synchronized String getLogger() {
    return currentConnection.getLogger();
  }
  
  public synchronized String getLoggerClassName() {
    return currentConnection.getLoggerClassName();
  }
  
  public synchronized boolean getMaintainTimeStats() {
    return currentConnection.getMaintainTimeStats();
  }
  
  public synchronized int getMaxQuerySizeToLog() {
    return currentConnection.getMaxQuerySizeToLog();
  }
  
  public synchronized int getMaxReconnects() {
    return currentConnection.getMaxReconnects();
  }
  
  public synchronized int getMaxRows() {
    return currentConnection.getMaxRows();
  }
  
  public synchronized int getMetadataCacheSize() {
    return currentConnection.getMetadataCacheSize();
  }
  
  public synchronized int getNetTimeoutForStreamingResults() {
    return currentConnection.getNetTimeoutForStreamingResults();
  }
  
  public synchronized boolean getNoAccessToProcedureBodies() {
    return currentConnection.getNoAccessToProcedureBodies();
  }
  
  public synchronized boolean getNoDatetimeStringSync() {
    return currentConnection.getNoDatetimeStringSync();
  }
  
  public synchronized boolean getNoTimezoneConversionForTimeType() {
    return currentConnection.getNoTimezoneConversionForTimeType();
  }
  
  public synchronized boolean getNullCatalogMeansCurrent() {
    return currentConnection.getNullCatalogMeansCurrent();
  }
  
  public synchronized boolean getNullNamePatternMatchesAll() {
    return currentConnection.getNullNamePatternMatchesAll();
  }
  
  public synchronized boolean getOverrideSupportsIntegrityEnhancementFacility() {
    return currentConnection.getOverrideSupportsIntegrityEnhancementFacility();
  }
  
  public synchronized int getPacketDebugBufferSize() {
    return currentConnection.getPacketDebugBufferSize();
  }
  
  public synchronized boolean getPadCharsWithSpace() {
    return currentConnection.getPadCharsWithSpace();
  }
  
  public synchronized boolean getParanoid() {
    return currentConnection.getParanoid();
  }
  
  public synchronized boolean getPedantic() {
    return currentConnection.getPedantic();
  }
  
  public synchronized boolean getPinGlobalTxToPhysicalConnection() {
    return currentConnection.getPinGlobalTxToPhysicalConnection();
  }
  
  public synchronized boolean getPopulateInsertRowWithDefaultValues() {
    return currentConnection.getPopulateInsertRowWithDefaultValues();
  }
  
  public synchronized int getPrepStmtCacheSize() {
    return currentConnection.getPrepStmtCacheSize();
  }
  
  public synchronized int getPrepStmtCacheSqlLimit() {
    return currentConnection.getPrepStmtCacheSqlLimit();
  }
  
  public synchronized int getPreparedStatementCacheSize() {
    return currentConnection.getPreparedStatementCacheSize();
  }
  
  public synchronized int getPreparedStatementCacheSqlLimit() {
    return currentConnection.getPreparedStatementCacheSqlLimit();
  }
  
  public synchronized boolean getProcessEscapeCodesForPrepStmts() {
    return currentConnection.getProcessEscapeCodesForPrepStmts();
  }
  
  public synchronized boolean getProfileSQL() {
    return currentConnection.getProfileSQL();
  }
  
  public synchronized boolean getProfileSql() {
    return currentConnection.getProfileSql();
  }
  
  public synchronized String getProfilerEventHandler() {
    return currentConnection.getProfilerEventHandler();
  }
  
  public synchronized String getPropertiesTransform() {
    return currentConnection.getPropertiesTransform();
  }
  
  public synchronized int getQueriesBeforeRetryMaster() {
    return currentConnection.getQueriesBeforeRetryMaster();
  }
  
  public synchronized boolean getReconnectAtTxEnd() {
    return currentConnection.getReconnectAtTxEnd();
  }
  
  public synchronized boolean getRelaxAutoCommit() {
    return currentConnection.getRelaxAutoCommit();
  }
  
  public synchronized int getReportMetricsIntervalMillis() {
    return currentConnection.getReportMetricsIntervalMillis();
  }
  
  public synchronized boolean getRequireSSL() {
    return currentConnection.getRequireSSL();
  }
  
  public synchronized String getResourceId() {
    return currentConnection.getResourceId();
  }
  
  public synchronized int getResultSetSizeThreshold() {
    return currentConnection.getResultSetSizeThreshold();
  }
  
  public synchronized boolean getRewriteBatchedStatements() {
    return currentConnection.getRewriteBatchedStatements();
  }
  
  public synchronized boolean getRollbackOnPooledClose() {
    return currentConnection.getRollbackOnPooledClose();
  }
  
  public synchronized boolean getRoundRobinLoadBalance() {
    return currentConnection.getRoundRobinLoadBalance();
  }
  
  public synchronized boolean getRunningCTS13() {
    return currentConnection.getRunningCTS13();
  }
  
  public synchronized int getSecondsBeforeRetryMaster() {
    return currentConnection.getSecondsBeforeRetryMaster();
  }
  
  public synchronized int getSelfDestructOnPingMaxOperations() {
    return currentConnection.getSelfDestructOnPingMaxOperations();
  }
  
  public synchronized int getSelfDestructOnPingSecondsLifetime() {
    return currentConnection.getSelfDestructOnPingSecondsLifetime();
  }
  
  public synchronized String getServerTimezone() {
    return currentConnection.getServerTimezone();
  }
  
  public synchronized String getSessionVariables() {
    return currentConnection.getSessionVariables();
  }
  
  public synchronized int getSlowQueryThresholdMillis() {
    return currentConnection.getSlowQueryThresholdMillis();
  }
  
  public synchronized long getSlowQueryThresholdNanos() {
    return currentConnection.getSlowQueryThresholdNanos();
  }
  
  public synchronized String getSocketFactory() {
    return currentConnection.getSocketFactory();
  }
  
  public synchronized String getSocketFactoryClassName() {
    return currentConnection.getSocketFactoryClassName();
  }
  
  public synchronized int getSocketTimeout() {
    return currentConnection.getSocketTimeout();
  }
  
  public synchronized String getStatementInterceptors() {
    return currentConnection.getStatementInterceptors();
  }
  
  public synchronized boolean getStrictFloatingPoint() {
    return currentConnection.getStrictFloatingPoint();
  }
  
  public synchronized boolean getStrictUpdates() {
    return currentConnection.getStrictUpdates();
  }
  
  public synchronized boolean getTcpKeepAlive() {
    return currentConnection.getTcpKeepAlive();
  }
  
  public synchronized boolean getTcpNoDelay() {
    return currentConnection.getTcpNoDelay();
  }
  
  public synchronized int getTcpRcvBuf() {
    return currentConnection.getTcpRcvBuf();
  }
  
  public synchronized int getTcpSndBuf() {
    return currentConnection.getTcpSndBuf();
  }
  
  public synchronized int getTcpTrafficClass() {
    return currentConnection.getTcpTrafficClass();
  }
  
  public synchronized boolean getTinyInt1isBit() {
    return currentConnection.getTinyInt1isBit();
  }
  
  public synchronized boolean getTraceProtocol() {
    return currentConnection.getTraceProtocol();
  }
  
  public synchronized boolean getTransformedBitIsBoolean() {
    return currentConnection.getTransformedBitIsBoolean();
  }
  
  public synchronized boolean getTreatUtilDateAsTimestamp() {
    return currentConnection.getTreatUtilDateAsTimestamp();
  }
  
  public synchronized String getTrustCertificateKeyStorePassword() {
    return currentConnection.getTrustCertificateKeyStorePassword();
  }
  
  public synchronized String getTrustCertificateKeyStoreType() {
    return currentConnection.getTrustCertificateKeyStoreType();
  }
  
  public synchronized String getTrustCertificateKeyStoreUrl() {
    return currentConnection.getTrustCertificateKeyStoreUrl();
  }
  
  public synchronized boolean getUltraDevHack() {
    return currentConnection.getUltraDevHack();
  }
  
  public synchronized boolean getUseBlobToStoreUTF8OutsideBMP() {
    return currentConnection.getUseBlobToStoreUTF8OutsideBMP();
  }
  
  public synchronized boolean getUseCompression() {
    return currentConnection.getUseCompression();
  }
  
  public synchronized String getUseConfigs() {
    return currentConnection.getUseConfigs();
  }
  
  public synchronized boolean getUseCursorFetch() {
    return currentConnection.getUseCursorFetch();
  }
  
  public synchronized boolean getUseDirectRowUnpack() {
    return currentConnection.getUseDirectRowUnpack();
  }
  
  public synchronized boolean getUseDynamicCharsetInfo() {
    return currentConnection.getUseDynamicCharsetInfo();
  }
  
  public synchronized boolean getUseFastDateParsing() {
    return currentConnection.getUseFastDateParsing();
  }
  
  public synchronized boolean getUseFastIntParsing() {
    return currentConnection.getUseFastIntParsing();
  }
  
  public synchronized boolean getUseGmtMillisForDatetimes() {
    return currentConnection.getUseGmtMillisForDatetimes();
  }
  
  public synchronized boolean getUseHostsInPrivileges() {
    return currentConnection.getUseHostsInPrivileges();
  }
  
  public synchronized boolean getUseInformationSchema() {
    return currentConnection.getUseInformationSchema();
  }
  
  public synchronized boolean getUseJDBCCompliantTimezoneShift() {
    return currentConnection.getUseJDBCCompliantTimezoneShift();
  }
  
  public synchronized boolean getUseJvmCharsetConverters() {
    return currentConnection.getUseJvmCharsetConverters();
  }
  
  public synchronized boolean getUseLegacyDatetimeCode() {
    return currentConnection.getUseLegacyDatetimeCode();
  }
  
  public synchronized boolean getUseLocalSessionState() {
    return currentConnection.getUseLocalSessionState();
  }
  
  public synchronized boolean getUseNanosForElapsedTime() {
    return currentConnection.getUseNanosForElapsedTime();
  }
  
  public synchronized boolean getUseOldAliasMetadataBehavior() {
    return currentConnection.getUseOldAliasMetadataBehavior();
  }
  
  public synchronized boolean getUseOldUTF8Behavior() {
    return currentConnection.getUseOldUTF8Behavior();
  }
  
  public synchronized boolean getUseOnlyServerErrorMessages() {
    return currentConnection.getUseOnlyServerErrorMessages();
  }
  
  public synchronized boolean getUseReadAheadInput() {
    return currentConnection.getUseReadAheadInput();
  }
  
  public synchronized boolean getUseSSL() {
    return currentConnection.getUseSSL();
  }
  
  public synchronized boolean getUseSSPSCompatibleTimezoneShift() {
    return currentConnection.getUseSSPSCompatibleTimezoneShift();
  }
  
  public synchronized boolean getUseServerPrepStmts() {
    return currentConnection.getUseServerPrepStmts();
  }
  
  public synchronized boolean getUseServerPreparedStmts() {
    return currentConnection.getUseServerPreparedStmts();
  }
  
  public synchronized boolean getUseSqlStateCodes() {
    return currentConnection.getUseSqlStateCodes();
  }
  
  public synchronized boolean getUseStreamLengthsInPrepStmts() {
    return currentConnection.getUseStreamLengthsInPrepStmts();
  }
  
  public synchronized boolean getUseTimezone() {
    return currentConnection.getUseTimezone();
  }
  
  public synchronized boolean getUseUltraDevWorkAround() {
    return currentConnection.getUseUltraDevWorkAround();
  }
  
  public synchronized boolean getUseUnbufferedInput() {
    return currentConnection.getUseUnbufferedInput();
  }
  
  public synchronized boolean getUseUnicode() {
    return currentConnection.getUseUnicode();
  }
  
  public synchronized boolean getUseUsageAdvisor() {
    return currentConnection.getUseUsageAdvisor();
  }
  
  public synchronized String getUtf8OutsideBmpExcludedColumnNamePattern() {
    return currentConnection.getUtf8OutsideBmpExcludedColumnNamePattern();
  }
  
  public synchronized String getUtf8OutsideBmpIncludedColumnNamePattern() {
    return currentConnection.getUtf8OutsideBmpIncludedColumnNamePattern();
  }
  
  public synchronized boolean getVerifyServerCertificate() {
    return currentConnection.getVerifyServerCertificate();
  }
  
  public synchronized boolean getYearIsDateType() {
    return currentConnection.getYearIsDateType();
  }
  
  public synchronized String getZeroDateTimeBehavior() {
    return currentConnection.getZeroDateTimeBehavior();
  }
  



  public synchronized void setAllowLoadLocalInfile(boolean property) {}
  



  public synchronized void setAllowMultiQueries(boolean property) {}
  



  public synchronized void setAllowNanAndInf(boolean flag) {}
  



  public synchronized void setAllowUrlInLocalInfile(boolean flag) {}
  



  public synchronized void setAlwaysSendSetIsolation(boolean flag) {}
  



  public synchronized void setAutoClosePStmtStreams(boolean flag) {}
  



  public synchronized void setAutoDeserialize(boolean flag) {}
  



  public synchronized void setAutoGenerateTestcaseScript(boolean flag) {}
  



  public synchronized void setAutoReconnect(boolean flag) {}
  



  public synchronized void setAutoReconnectForConnectionPools(boolean property) {}
  



  public synchronized void setAutoReconnectForPools(boolean flag) {}
  



  public synchronized void setAutoSlowLog(boolean flag) {}
  



  public synchronized void setBlobSendChunkSize(String value)
    throws SQLException
  {}
  



  public synchronized void setBlobsAreStrings(boolean flag) {}
  



  public synchronized void setCacheCallableStatements(boolean flag) {}
  



  public synchronized void setCacheCallableStmts(boolean flag) {}
  



  public synchronized void setCachePrepStmts(boolean flag) {}
  



  public synchronized void setCachePreparedStatements(boolean flag) {}
  



  public synchronized void setCacheResultSetMetadata(boolean property) {}
  



  public synchronized void setCacheServerConfiguration(boolean flag) {}
  



  public synchronized void setCallableStatementCacheSize(int size) {}
  



  public synchronized void setCallableStmtCacheSize(int cacheSize) {}
  



  public synchronized void setCapitalizeDBMDTypes(boolean property) {}
  



  public synchronized void setCapitalizeTypeNames(boolean flag) {}
  



  public synchronized void setCharacterEncoding(String encoding) {}
  



  public synchronized void setCharacterSetResults(String characterSet) {}
  



  public synchronized void setClientCertificateKeyStorePassword(String value) {}
  



  public synchronized void setClientCertificateKeyStoreType(String value) {}
  



  public synchronized void setClientCertificateKeyStoreUrl(String value) {}
  



  public synchronized void setClientInfoProvider(String classname) {}
  



  public synchronized void setClobCharacterEncoding(String encoding) {}
  



  public synchronized void setClobberStreamingResults(boolean flag) {}
  



  public synchronized void setConnectTimeout(int timeoutMs) {}
  



  public synchronized void setConnectionCollation(String collation) {}
  



  public synchronized void setConnectionLifecycleInterceptors(String interceptors) {}
  



  public synchronized void setContinueBatchOnError(boolean property) {}
  



  public synchronized void setCreateDatabaseIfNotExist(boolean flag) {}
  



  public synchronized void setDefaultFetchSize(int n) {}
  



  public synchronized void setDetectServerPreparedStmts(boolean property) {}
  



  public synchronized void setDontTrackOpenResources(boolean flag) {}
  



  public synchronized void setDumpMetadataOnColumnNotFound(boolean flag) {}
  



  public synchronized void setDumpQueriesOnException(boolean flag) {}
  



  public synchronized void setDynamicCalendars(boolean flag) {}
  



  public synchronized void setElideSetAutoCommits(boolean flag) {}
  



  public synchronized void setEmptyStringsConvertToZero(boolean flag) {}
  



  public synchronized void setEmulateLocators(boolean property) {}
  



  public synchronized void setEmulateUnsupportedPstmts(boolean flag) {}
  



  public synchronized void setEnablePacketDebug(boolean flag) {}
  



  public synchronized void setEnableQueryTimeouts(boolean flag) {}
  



  public synchronized void setEncoding(String property) {}
  



  public synchronized void setExplainSlowQueries(boolean flag) {}
  



  public synchronized void setFailOverReadOnly(boolean flag) {}
  



  public synchronized void setFunctionsNeverReturnBlobs(boolean flag) {}
  



  public synchronized void setGatherPerfMetrics(boolean flag) {}
  



  public synchronized void setGatherPerformanceMetrics(boolean flag) {}
  



  public synchronized void setGenerateSimpleParameterMetadata(boolean flag) {}
  



  public synchronized void setHoldResultsOpenOverStatementClose(boolean flag) {}
  



  public synchronized void setIgnoreNonTxTables(boolean property) {}
  



  public synchronized void setIncludeInnodbStatusInDeadlockExceptions(boolean flag) {}
  



  public synchronized void setInitialTimeout(int property) {}
  



  public synchronized void setInteractiveClient(boolean property) {}
  



  public synchronized void setIsInteractiveClient(boolean property) {}
  



  public synchronized void setJdbcCompliantTruncation(boolean flag) {}
  



  public synchronized void setJdbcCompliantTruncationForReads(boolean jdbcCompliantTruncationForReads) {}
  



  public synchronized void setLargeRowSizeThreshold(String value) {}
  



  public synchronized void setLoadBalanceStrategy(String strategy) {}
  



  public synchronized void setLocalSocketAddress(String address) {}
  



  public synchronized void setLocatorFetchBufferSize(String value)
    throws SQLException
  {}
  



  public synchronized void setLogSlowQueries(boolean flag) {}
  



  public synchronized void setLogXaCommands(boolean flag) {}
  



  public synchronized void setLogger(String property) {}
  



  public synchronized void setLoggerClassName(String className) {}
  



  public synchronized void setMaintainTimeStats(boolean flag) {}
  



  public synchronized void setMaxQuerySizeToLog(int sizeInBytes) {}
  



  public synchronized void setMaxReconnects(int property) {}
  



  public synchronized void setMaxRows(int property) {}
  



  public synchronized void setMetadataCacheSize(int value) {}
  



  public synchronized void setNetTimeoutForStreamingResults(int value) {}
  



  public synchronized void setNoAccessToProcedureBodies(boolean flag) {}
  



  public synchronized void setNoDatetimeStringSync(boolean flag) {}
  



  public synchronized void setNoTimezoneConversionForTimeType(boolean flag) {}
  



  public synchronized void setNullCatalogMeansCurrent(boolean value) {}
  



  public synchronized void setNullNamePatternMatchesAll(boolean value) {}
  



  public synchronized void setOverrideSupportsIntegrityEnhancementFacility(boolean flag) {}
  



  public synchronized void setPacketDebugBufferSize(int size) {}
  



  public synchronized void setPadCharsWithSpace(boolean flag) {}
  



  public synchronized void setParanoid(boolean property) {}
  



  public synchronized void setPedantic(boolean property) {}
  



  public synchronized void setPinGlobalTxToPhysicalConnection(boolean flag) {}
  



  public synchronized void setPopulateInsertRowWithDefaultValues(boolean flag) {}
  



  public synchronized void setPrepStmtCacheSize(int cacheSize) {}
  



  public synchronized void setPrepStmtCacheSqlLimit(int sqlLimit) {}
  



  public synchronized void setPreparedStatementCacheSize(int cacheSize) {}
  



  public synchronized void setPreparedStatementCacheSqlLimit(int cacheSqlLimit) {}
  



  public synchronized void setProcessEscapeCodesForPrepStmts(boolean flag) {}
  



  public synchronized void setProfileSQL(boolean flag) {}
  



  public synchronized void setProfileSql(boolean property) {}
  



  public synchronized void setProfilerEventHandler(String handler) {}
  



  public synchronized void setPropertiesTransform(String value) {}
  



  public synchronized void setQueriesBeforeRetryMaster(int property) {}
  



  public synchronized void setReconnectAtTxEnd(boolean property) {}
  



  public synchronized void setRelaxAutoCommit(boolean property) {}
  



  public synchronized void setReportMetricsIntervalMillis(int millis) {}
  



  public synchronized void setRequireSSL(boolean property) {}
  



  public synchronized void setResourceId(String resourceId) {}
  



  public synchronized void setResultSetSizeThreshold(int threshold) {}
  



  public synchronized void setRetainStatementAfterResultSetClose(boolean flag) {}
  



  public synchronized void setRewriteBatchedStatements(boolean flag) {}
  



  public synchronized void setRollbackOnPooledClose(boolean flag) {}
  



  public synchronized void setRoundRobinLoadBalance(boolean flag) {}
  



  public synchronized void setRunningCTS13(boolean flag) {}
  



  public synchronized void setSecondsBeforeRetryMaster(int property) {}
  



  public synchronized void setSelfDestructOnPingMaxOperations(int maxOperations) {}
  



  public synchronized void setSelfDestructOnPingSecondsLifetime(int seconds) {}
  



  public synchronized void setServerTimezone(String property) {}
  



  public synchronized void setSessionVariables(String variables) {}
  



  public synchronized void setSlowQueryThresholdMillis(int millis) {}
  



  public synchronized void setSlowQueryThresholdNanos(long nanos) {}
  



  public synchronized void setSocketFactory(String name) {}
  



  public synchronized void setSocketFactoryClassName(String property) {}
  



  public synchronized void setSocketTimeout(int property) {}
  



  public synchronized void setStatementInterceptors(String value) {}
  



  public synchronized void setStrictFloatingPoint(boolean property) {}
  



  public synchronized void setStrictUpdates(boolean property) {}
  



  public synchronized void setTcpKeepAlive(boolean flag) {}
  



  public synchronized void setTcpNoDelay(boolean flag) {}
  



  public synchronized void setTcpRcvBuf(int bufSize) {}
  



  public synchronized void setTcpSndBuf(int bufSize) {}
  



  public synchronized void setTcpTrafficClass(int classFlags) {}
  



  public synchronized void setTinyInt1isBit(boolean flag) {}
  



  public synchronized void setTraceProtocol(boolean flag) {}
  



  public synchronized void setTransformedBitIsBoolean(boolean flag) {}
  



  public synchronized void setTreatUtilDateAsTimestamp(boolean flag) {}
  



  public synchronized void setTrustCertificateKeyStorePassword(String value) {}
  



  public synchronized void setTrustCertificateKeyStoreType(String value) {}
  



  public synchronized void setTrustCertificateKeyStoreUrl(String value) {}
  



  public synchronized void setUltraDevHack(boolean flag) {}
  



  public synchronized void setUseBlobToStoreUTF8OutsideBMP(boolean flag) {}
  



  public synchronized void setUseCompression(boolean property) {}
  



  public synchronized void setUseConfigs(String configs) {}
  



  public synchronized void setUseCursorFetch(boolean flag) {}
  



  public synchronized void setUseDirectRowUnpack(boolean flag) {}
  



  public synchronized void setUseDynamicCharsetInfo(boolean flag) {}
  



  public synchronized void setUseFastDateParsing(boolean flag) {}
  



  public synchronized void setUseFastIntParsing(boolean flag) {}
  



  public synchronized void setUseGmtMillisForDatetimes(boolean flag) {}
  



  public synchronized void setUseHostsInPrivileges(boolean property) {}
  



  public synchronized void setUseInformationSchema(boolean flag) {}
  



  public synchronized void setUseJDBCCompliantTimezoneShift(boolean flag) {}
  



  public synchronized void setUseJvmCharsetConverters(boolean flag) {}
  



  public synchronized void setUseLegacyDatetimeCode(boolean flag) {}
  



  public synchronized void setUseLocalSessionState(boolean flag) {}
  



  public synchronized void setUseNanosForElapsedTime(boolean flag) {}
  



  public synchronized void setUseOldAliasMetadataBehavior(boolean flag) {}
  



  public synchronized void setUseOldUTF8Behavior(boolean flag) {}
  



  public synchronized void setUseOnlyServerErrorMessages(boolean flag) {}
  



  public synchronized void setUseReadAheadInput(boolean flag) {}
  



  public synchronized void setUseSSL(boolean property) {}
  



  public synchronized void setUseSSPSCompatibleTimezoneShift(boolean flag) {}
  



  public synchronized void setUseServerPrepStmts(boolean flag) {}
  



  public synchronized void setUseServerPreparedStmts(boolean flag) {}
  



  public synchronized void setUseSqlStateCodes(boolean flag) {}
  



  public synchronized void setUseStreamLengthsInPrepStmts(boolean property) {}
  



  public synchronized void setUseTimezone(boolean property) {}
  



  public synchronized void setUseUltraDevWorkAround(boolean property) {}
  



  public synchronized void setUseUnbufferedInput(boolean flag) {}
  



  public synchronized void setUseUnicode(boolean flag) {}
  


  public synchronized void setUseUsageAdvisor(boolean useUsageAdvisorFlag) {}
  


  public synchronized void setUtf8OutsideBmpExcludedColumnNamePattern(String regexPattern) {}
  


  public synchronized void setUtf8OutsideBmpIncludedColumnNamePattern(String regexPattern) {}
  


  public synchronized void setVerifyServerCertificate(boolean flag) {}
  


  public synchronized void setYearIsDateType(boolean flag) {}
  


  public synchronized void setZeroDateTimeBehavior(String behavior) {}
  


  public synchronized boolean useUnbufferedInput()
  {
    return currentConnection.useUnbufferedInput();
  }
  
  public synchronized boolean isSameResource(Connection c) {
    return currentConnection.isSameResource(c);
  }
  
  public void setInGlobalTx(boolean flag) {
    currentConnection.setInGlobalTx(flag);
  }
  
  public boolean getUseColumnNamesInFindColumn() {
    return currentConnection.getUseColumnNamesInFindColumn();
  }
  

  public void setUseColumnNamesInFindColumn(boolean flag) {}
  
  public boolean getUseLocalTransactionState()
  {
    return currentConnection.getUseLocalTransactionState();
  }
  

  public void setUseLocalTransactionState(boolean flag) {}
  

  public boolean getCompensateOnDuplicateKeyUpdateCounts()
  {
    return currentConnection.getCompensateOnDuplicateKeyUpdateCounts();
  }
  

  public void setCompensateOnDuplicateKeyUpdateCounts(boolean flag) {}
  

  public boolean getUseAffectedRows()
  {
    return currentConnection.getUseAffectedRows();
  }
  

  public void setUseAffectedRows(boolean flag) {}
  

  public String getPasswordCharacterEncoding()
  {
    return currentConnection.getPasswordCharacterEncoding();
  }
  
  public void setPasswordCharacterEncoding(String characterSet) {
    currentConnection.setPasswordCharacterEncoding(characterSet);
  }
  
  public int getAutoIncrementIncrement() {
    return currentConnection.getAutoIncrementIncrement();
  }
  
  public int getLoadBalanceBlacklistTimeout() {
    return currentConnection.getLoadBalanceBlacklistTimeout();
  }
  
  public void setLoadBalanceBlacklistTimeout(int loadBalanceBlacklistTimeout) {
    currentConnection.setLoadBalanceBlacklistTimeout(loadBalanceBlacklistTimeout);
  }
  
  public int getLoadBalancePingTimeout() {
    return currentConnection.getLoadBalancePingTimeout();
  }
  
  public void setLoadBalancePingTimeout(int loadBalancePingTimeout) {
    currentConnection.setLoadBalancePingTimeout(loadBalancePingTimeout);
  }
  
  public boolean getLoadBalanceValidateConnectionOnSwapServer() {
    return currentConnection.getLoadBalanceValidateConnectionOnSwapServer();
  }
  
  public void setLoadBalanceValidateConnectionOnSwapServer(boolean loadBalanceValidateConnectionOnSwapServer) {
    currentConnection.setLoadBalanceValidateConnectionOnSwapServer(loadBalanceValidateConnectionOnSwapServer);
  }
  
  public int getRetriesAllDown() {
    return currentConnection.getRetriesAllDown();
  }
  
  public void setRetriesAllDown(int retriesAllDown) {
    currentConnection.setRetriesAllDown(retriesAllDown);
  }
  
  public ExceptionInterceptor getExceptionInterceptor() {
    return currentConnection.getExceptionInterceptor();
  }
  
  public String getExceptionInterceptors() {
    return currentConnection.getExceptionInterceptors();
  }
  
  public void setExceptionInterceptors(String exceptionInterceptors) {
    currentConnection.setExceptionInterceptors(exceptionInterceptors);
  }
  
  public boolean getQueryTimeoutKillsConnection() {
    return currentConnection.getQueryTimeoutKillsConnection();
  }
  
  public void setQueryTimeoutKillsConnection(boolean queryTimeoutKillsConnection)
  {
    currentConnection.setQueryTimeoutKillsConnection(queryTimeoutKillsConnection);
  }
  
  public boolean hasSameProperties(Connection c) {
    return (masterConnection.hasSameProperties(c)) && (slavesConnection.hasSameProperties(c));
  }
  
  public Properties getProperties()
  {
    Properties props = new Properties();
    props.putAll(masterConnection.getProperties());
    props.putAll(slavesConnection.getProperties());
    
    return props;
  }
  
  public String getHost() {
    return currentConnection.getHost();
  }
  
  public void setProxy(MySQLConnection proxy) {
    currentConnection.setProxy(proxy);
  }
  
  public boolean getRetainStatementAfterResultSetClose() {
    return currentConnection.getRetainStatementAfterResultSetClose();
  }
  
  public int getMaxAllowedPacket() {
    return currentConnection.getMaxAllowedPacket();
  }
  
  public String getLoadBalanceConnectionGroup() {
    return currentConnection.getLoadBalanceConnectionGroup();
  }
  
  public boolean getLoadBalanceEnableJMX() {
    return currentConnection.getLoadBalanceEnableJMX();
  }
  
  public String getLoadBalanceExceptionChecker() {
    return currentConnection.getLoadBalanceExceptionChecker();
  }
  
  public String getLoadBalanceSQLExceptionSubclassFailover()
  {
    return currentConnection.getLoadBalanceSQLExceptionSubclassFailover();
  }
  
  public String getLoadBalanceSQLStateFailover()
  {
    return currentConnection.getLoadBalanceSQLStateFailover();
  }
  
  public void setLoadBalanceConnectionGroup(String loadBalanceConnectionGroup)
  {
    currentConnection.setLoadBalanceConnectionGroup(loadBalanceConnectionGroup);
  }
  

  public void setLoadBalanceEnableJMX(boolean loadBalanceEnableJMX)
  {
    currentConnection.setLoadBalanceEnableJMX(loadBalanceEnableJMX);
  }
  


  public void setLoadBalanceExceptionChecker(String loadBalanceExceptionChecker)
  {
    currentConnection.setLoadBalanceExceptionChecker(loadBalanceExceptionChecker);
  }
  


  public void setLoadBalanceSQLExceptionSubclassFailover(String loadBalanceSQLExceptionSubclassFailover)
  {
    currentConnection.setLoadBalanceSQLExceptionSubclassFailover(loadBalanceSQLExceptionSubclassFailover);
  }
  


  public void setLoadBalanceSQLStateFailover(String loadBalanceSQLStateFailover)
  {
    currentConnection.setLoadBalanceSQLStateFailover(loadBalanceSQLStateFailover);
  }
  

  public String getLoadBalanceAutoCommitStatementRegex()
  {
    return currentConnection.getLoadBalanceAutoCommitStatementRegex();
  }
  
  public int getLoadBalanceAutoCommitStatementThreshold() {
    return currentConnection.getLoadBalanceAutoCommitStatementThreshold();
  }
  
  public void setLoadBalanceAutoCommitStatementRegex(String loadBalanceAutoCommitStatementRegex)
  {
    currentConnection.setLoadBalanceAutoCommitStatementRegex(loadBalanceAutoCommitStatementRegex);
  }
  

  public void setLoadBalanceAutoCommitStatementThreshold(int loadBalanceAutoCommitStatementThreshold)
  {
    currentConnection.setLoadBalanceAutoCommitStatementThreshold(loadBalanceAutoCommitStatementThreshold);
  }
}
