package com.mysql.jdbc;

import com.mysql.jdbc.exceptions.MySQLStatementCancelledException;
import com.mysql.jdbc.exceptions.MySQLTimeoutException;
import com.mysql.jdbc.profiler.ProfilerEvent;
import com.mysql.jdbc.profiler.ProfilerEventHandler;
import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.math.BigDecimal;
import java.net.URL;
import java.sql.Array;
import java.sql.BatchUpdateException;
import java.sql.Blob;
import java.sql.Clob;
import java.sql.ParameterMetaData;
import java.sql.Ref;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.TimeZone;
import java.util.Timer;

























public class ServerPreparedStatement
  extends PreparedStatement
{
  private static final Constructor JDBC_4_SPS_CTOR;
  protected static final int BLOB_STREAM_READ_BUF_SIZE = 8192;
  private static final byte MAX_DATE_REP_LENGTH = 5;
  private static final byte MAX_DATETIME_REP_LENGTH = 12;
  private static final byte MAX_TIME_REP_LENGTH = 13;
  
  static
  {
    if (Util.isJdbc4()) {
      try {
        JDBC_4_SPS_CTOR = Class.forName("com.mysql.jdbc.JDBC4ServerPreparedStatement").getConstructor(new Class[] { MySQLConnection.class, String.class, String.class, Integer.TYPE, Integer.TYPE });

      }
      catch (SecurityException e)
      {
        throw new RuntimeException(e);
      } catch (NoSuchMethodException e) {
        throw new RuntimeException(e);
      } catch (ClassNotFoundException e) {
        throw new RuntimeException(e);
      }
    } else {
      JDBC_4_SPS_CTOR = null;
    }
  }
  
  static class BatchedBindValues
  {
    ServerPreparedStatement.BindValue[] batchedParameterValues;
    
    BatchedBindValues(ServerPreparedStatement.BindValue[] paramVals)
    {
      int numParams = paramVals.length;
      
      batchedParameterValues = new ServerPreparedStatement.BindValue[numParams];
      
      for (int i = 0; i < numParams; i++) {
        batchedParameterValues[i] = new ServerPreparedStatement.BindValue(paramVals[i]);
      }
    }
  }
  
  public static class BindValue
  {
    long boundBeforeExecutionNum = 0L;
    
    public long bindLength;
    
    int bufferType;
    
    byte byteBinding;
    
    double doubleBinding;
    
    float floatBinding;
    
    int intBinding;
    
    public boolean isLongData;
    
    public boolean isNull;
    
    boolean isSet = false;
    
    long longBinding;
    
    short shortBinding;
    
    public Object value;
    
    BindValue() {}
    
    BindValue(BindValue copyMe)
    {
      value = value;
      isSet = isSet;
      isLongData = isLongData;
      isNull = isNull;
      bufferType = bufferType;
      bindLength = bindLength;
      byteBinding = byteBinding;
      shortBinding = shortBinding;
      intBinding = intBinding;
      longBinding = longBinding;
      floatBinding = floatBinding;
      doubleBinding = doubleBinding;
    }
    
    void reset() {
      isSet = false;
      value = null;
      isLongData = false;
      
      byteBinding = 0;
      shortBinding = 0;
      intBinding = 0;
      longBinding = 0L;
      floatBinding = 0.0F;
      doubleBinding = 0.0D;
    }
    
    public String toString() {
      return toString(false);
    }
    
    public String toString(boolean quoteIfNeeded) {
      if (isLongData) {
        return "' STREAM DATA '";
      }
      
      switch (bufferType) {
      case 1: 
        return String.valueOf(byteBinding);
      case 2: 
        return String.valueOf(shortBinding);
      case 3: 
        return String.valueOf(intBinding);
      case 8: 
        return String.valueOf(longBinding);
      case 4: 
        return String.valueOf(floatBinding);
      case 5: 
        return String.valueOf(doubleBinding);
      case 7: 
      case 10: 
      case 11: 
      case 12: 
      case 15: 
      case 253: 
      case 254: 
        if (quoteIfNeeded) {
          return "'" + String.valueOf(value) + "'";
        }
        return String.valueOf(value);
      }
      
      if ((value instanceof byte[])) {
        return "byte data";
      }
      
      if (quoteIfNeeded) {
        return "'" + String.valueOf(value) + "'";
      }
      return String.valueOf(value);
    }
    


    long getBoundLength()
    {
      if (isNull) {
        return 0L;
      }
      
      if (isLongData) {
        return bindLength;
      }
      
      switch (bufferType)
      {
      case 1: 
        return 1L;
      case 2: 
        return 2L;
      case 3: 
        return 4L;
      case 8: 
        return 8L;
      case 4: 
        return 4L;
      case 5: 
        return 8L;
      case 11: 
        return 9L;
      case 10: 
        return 7L;
      case 7: 
      case 12: 
        return 11L;
      case 0: 
      case 15: 
      case 246: 
      case 253: 
      case 254: 
        if ((value instanceof byte[])) {
          return ((byte[])value).length;
        }
        return ((String)value).length();
      }
      
      return 0L;
    }
  }
  
















  private boolean hasOnDuplicateKeyUpdate = false;
  
  private void storeTime(Buffer intoBuf, Time tm) throws SQLException
  {
    intoBuf.ensureCapacity(9);
    intoBuf.writeByte((byte)8);
    intoBuf.writeByte((byte)0);
    intoBuf.writeLong(0L);
    
    Calendar sessionCalendar = getCalendarInstanceForSessionOrNew();
    
    synchronized (sessionCalendar) {
      java.util.Date oldTime = sessionCalendar.getTime();
      try {
        sessionCalendar.setTime(tm);
        intoBuf.writeByte((byte)sessionCalendar.get(11));
        intoBuf.writeByte((byte)sessionCalendar.get(12));
        intoBuf.writeByte((byte)sessionCalendar.get(13));
      }
      finally
      {
        sessionCalendar.setTime(oldTime);
      }
    }
  }
  





  private boolean detectedLongParameterSwitch = false;
  



  private int fieldCount;
  


  private boolean invalid = false;
  

  private SQLException invalidationException;
  

  private boolean isSelectQuery;
  

  private Buffer outByteBuffer;
  

  private BindValue[] parameterBindings;
  

  private Field[] parameterFields;
  

  private Field[] resultFields;
  
  private boolean sendTypesToServer = false;
  

  private long serverStatementId;
  

  private int stringTypeCode = 254;
  



  private boolean serverNeedsResetBeforeEachExecution;
  




  protected static ServerPreparedStatement getInstance(MySQLConnection conn, String sql, String catalog, int resultSetType, int resultSetConcurrency)
    throws SQLException
  {
    if (!Util.isJdbc4()) {
      return new ServerPreparedStatement(conn, sql, catalog, resultSetType, resultSetConcurrency);
    }
    
    try
    {
      return (ServerPreparedStatement)JDBC_4_SPS_CTOR.newInstance(new Object[] { conn, sql, catalog, Constants.integerValueOf(resultSetType), Constants.integerValueOf(resultSetConcurrency) });
    }
    catch (IllegalArgumentException e)
    {
      throw new SQLException(e.toString(), "S1000");
    } catch (InstantiationException e) {
      throw new SQLException(e.toString(), "S1000");
    } catch (IllegalAccessException e) {
      throw new SQLException(e.toString(), "S1000");
    } catch (InvocationTargetException e) {
      Throwable target = e.getTargetException();
      
      if ((target instanceof SQLException)) {
        throw ((SQLException)target);
      }
      
      throw new SQLException(target.toString(), "S1000");
    }
  }
  













  protected ServerPreparedStatement(MySQLConnection conn, String sql, String catalog, int resultSetType, int resultSetConcurrency)
    throws SQLException
  {
    super(conn, catalog);
    
    checkNullOrEmptyQuery(sql);
    
    hasOnDuplicateKeyUpdate = containsOnDuplicateKeyInString(sql);
    
    int startOfStatement = findStartOfStatement(sql);
    
    firstCharOfStmt = StringUtils.firstAlphaCharUc(sql, startOfStatement);
    
    isSelectQuery = ('S' == firstCharOfStmt);
    
    if (connection.versionMeetsMinimum(5, 0, 0)) {
      serverNeedsResetBeforeEachExecution = (!connection.versionMeetsMinimum(5, 0, 3));
    }
    else {
      serverNeedsResetBeforeEachExecution = (!connection.versionMeetsMinimum(4, 1, 10));
    }
    

    useAutoSlowLog = connection.getAutoSlowLog();
    useTrueBoolean = connection.versionMeetsMinimum(3, 21, 23);
    hasLimitClause = (StringUtils.indexOfIgnoreCase(sql, "LIMIT") != -1);
    
    String statementComment = connection.getStatementComment();
    
    originalSql = ("/* " + statementComment + " */ " + sql);
    

    if (connection.versionMeetsMinimum(4, 1, 2)) {
      stringTypeCode = 253;
    } else {
      stringTypeCode = 254;
    }
    try
    {
      serverPrepare(sql);
    } catch (SQLException sqlEx) {
      realClose(false, true);
      
      throw sqlEx;
    } catch (Exception ex) {
      realClose(false, true);
      
      SQLException sqlEx = SQLError.createSQLException(ex.toString(), "S1000", getExceptionInterceptor());
      
      sqlEx.initCause(ex);
      
      throw sqlEx;
    }
    
    setResultSetType(resultSetType);
    setResultSetConcurrency(resultSetConcurrency);
    
    parameterTypes = new int[parameterCount];
  }
  






  public synchronized void addBatch()
    throws SQLException
  {
    checkClosed();
    
    if (batchedArgs == null) {
      batchedArgs = new ArrayList();
    }
    
    batchedArgs.add(new BatchedBindValues(parameterBindings));
  }
  
  protected String asSql(boolean quoteStreamsAndUnknowns) throws SQLException
  {
    if (isClosed) {
      return "statement has been closed, no further internal information available";
    }
    
    PreparedStatement pStmtForSub = null;
    try
    {
      pStmtForSub = PreparedStatement.getInstance(connection, originalSql, currentCatalog);
      

      int numParameters = parameterCount;
      int ourNumParameters = parameterCount;
      
      for (int i = 0; (i < numParameters) && (i < ourNumParameters); i++) {
        if (parameterBindings[i] != null) {
          if (parameterBindings[i].isNull) {
            pStmtForSub.setNull(i + 1, 0);
          } else {
            BindValue bindValue = parameterBindings[i];
            



            switch (bufferType)
            {
            case 1: 
              pStmtForSub.setByte(i + 1, byteBinding);
              break;
            case 2: 
              pStmtForSub.setShort(i + 1, shortBinding);
              break;
            case 3: 
              pStmtForSub.setInt(i + 1, intBinding);
              break;
            case 8: 
              pStmtForSub.setLong(i + 1, longBinding);
              break;
            case 4: 
              pStmtForSub.setFloat(i + 1, floatBinding);
              break;
            case 5: 
              pStmtForSub.setDouble(i + 1, doubleBinding);
              
              break;
            case 6: case 7: default: 
              pStmtForSub.setObject(i + 1, parameterBindings[i].value);
            }
            
          }
        }
      }
      

      return pStmtForSub.asSql(quoteStreamsAndUnknowns);
    } finally {
      if (pStmtForSub != null) {
        try {
          pStmtForSub.close();
        }
        catch (SQLException sqlEx) {}
      }
    }
  }
  




  protected void checkClosed()
    throws SQLException
  {
    if (invalid) {
      throw invalidationException;
    }
    
    super.checkClosed();
  }
  

  public void clearParameters()
    throws SQLException
  {
    checkClosed();
    clearParametersInternal(true);
  }
  
  private void clearParametersInternal(boolean clearServerParameters) throws SQLException
  {
    boolean hadLongData = false;
    
    if (parameterBindings != null) {
      for (int i = 0; i < parameterCount; i++) {
        if ((parameterBindings[i] != null) && (parameterBindings[i].isLongData))
        {
          hadLongData = true;
        }
        
        parameterBindings[i].reset();
      }
    }
    
    if ((clearServerParameters) && (hadLongData)) {
      serverResetStatement();
      
      detectedLongParameterSwitch = false;
    }
  }
  
  protected boolean isCached = false;
  
  private boolean useAutoSlowLog;
  
  private Calendar serverTzCalendar;
  private Calendar defaultTzCalendar;
  
  protected void setClosed(boolean flag)
  {
    isClosed = flag;
  }
  

  public synchronized void close()
    throws SQLException
  {
    if ((isCached) && (!isClosed)) {
      clearParameters();
      
      isClosed = true;
      
      connection.recachePreparedStatement(this);
      return;
    }
    
    realClose(true, true);
  }
  
  private void dumpCloseForTestcase() {
    StringBuffer buf = new StringBuffer();
    connection.generateConnectionCommentBlock(buf);
    buf.append("DEALLOCATE PREPARE debug_stmt_");
    buf.append(statementId);
    buf.append(";\n");
    
    connection.dumpTestcaseQuery(buf.toString());
  }
  
  private void dumpExecuteForTestcase() throws SQLException {
    StringBuffer buf = new StringBuffer();
    
    for (int i = 0; i < parameterCount; i++) {
      connection.generateConnectionCommentBlock(buf);
      
      buf.append("SET @debug_stmt_param");
      buf.append(statementId);
      buf.append("_");
      buf.append(i);
      buf.append("=");
      
      if (parameterBindings[i].isNull) {
        buf.append("NULL");
      } else {
        buf.append(parameterBindings[i].toString(true));
      }
      
      buf.append(";\n");
    }
    
    connection.generateConnectionCommentBlock(buf);
    
    buf.append("EXECUTE debug_stmt_");
    buf.append(statementId);
    
    if (parameterCount > 0) {
      buf.append(" USING ");
      for (int i = 0; i < parameterCount; i++) {
        if (i > 0) {
          buf.append(", ");
        }
        
        buf.append("@debug_stmt_param");
        buf.append(statementId);
        buf.append("_");
        buf.append(i);
      }
    }
    

    buf.append(";\n");
    
    connection.dumpTestcaseQuery(buf.toString());
  }
  
  private void dumpPrepareForTestcase() throws SQLException
  {
    StringBuffer buf = new StringBuffer(originalSql.length() + 64);
    
    connection.generateConnectionCommentBlock(buf);
    
    buf.append("PREPARE debug_stmt_");
    buf.append(statementId);
    buf.append(" FROM \"");
    buf.append(originalSql);
    buf.append("\";\n");
    
    connection.dumpTestcaseQuery(buf.toString());
  }
  
  protected int[] executeBatchSerially(int batchTimeout) throws SQLException {
    MySQLConnection locallyScopedConn = connection;
    
    if (locallyScopedConn == null) {
      checkClosed();
    }
    
    if (locallyScopedConn.isReadOnly()) {
      throw SQLError.createSQLException(Messages.getString("ServerPreparedStatement.2") + Messages.getString("ServerPreparedStatement.3"), "S1009", getExceptionInterceptor());
    }
    



    checkClosed();
    BindValue[] oldBindValues;
    synchronized (locallyScopedConn.getMutex()) {
      clearWarnings();
      


      oldBindValues = parameterBindings;
    }
    try {
      int[] updateCounts = null;
      
      if (batchedArgs != null) {
        nbrCommands = batchedArgs.size();
        updateCounts = new int[nbrCommands];
        
        if (retrieveGeneratedKeys) {
          batchedGeneratedKeys = new ArrayList(nbrCommands);
        }
        
        for (int i = 0; i < nbrCommands; i++) {
          updateCounts[i] = -3;
        }
        
        SQLException sqlEx = null;
        
        int commandIndex = 0;
        
        BindValue[] previousBindValuesForBatch = null;
        
        StatementImpl.CancelTask timeoutTask = null;
        try
        {
          if ((locallyScopedConn.getEnableQueryTimeouts()) && (batchTimeout != 0) && (locallyScopedConn.versionMeetsMinimum(5, 0, 0)))
          {

            timeoutTask = new StatementImpl.CancelTask(this, this);
            locallyScopedConn.getCancelTimer().schedule(timeoutTask, batchTimeout);
          }
          

          for (commandIndex = 0; commandIndex < nbrCommands; commandIndex++) {
            Object arg = batchedArgs.get(commandIndex);
            
            if ((arg instanceof String)) {
              updateCounts[commandIndex] = executeUpdate((String)arg);
            } else {
              parameterBindings = batchedParameterValues;
              



              try
              {
                if (previousBindValuesForBatch != null) {
                  for (int j = 0; j < parameterBindings.length; j++) {
                    if (parameterBindings[j].bufferType != bufferType) {
                      sendTypesToServer = true;
                      
                      break;
                    }
                  }
                }
                try
                {
                  updateCounts[commandIndex] = executeUpdate(false, true);
                } finally {
                  previousBindValuesForBatch = parameterBindings;
                }
                
                if (retrieveGeneratedKeys) {
                  ResultSet rs = null;
                  








                  try
                  {
                    rs = getGeneratedKeysInternal();
                    
                    while (rs.next()) {
                      batchedGeneratedKeys.add(new ByteArrayRow(new byte[][] { rs.getBytes(1) }, getExceptionInterceptor()));
                    }
                  }
                  finally
                  {
                    if (rs != null) {
                      rs.close();
                    }
                  }
                }
              } catch (SQLException ex) {
                updateCounts[commandIndex] = -3;
                
                if ((continueBatchOnError) && (!(ex instanceof MySQLTimeoutException)) && (!(ex instanceof MySQLStatementCancelledException)) && (!hasDeadlockOrTimeoutRolledBackTx(ex)))
                {


                  sqlEx = ex;
                } else {
                  int[] newUpdateCounts = new int[commandIndex];
                  System.arraycopy(updateCounts, 0, newUpdateCounts, 0, commandIndex);
                  

                  throw new BatchUpdateException(ex.getMessage(), ex.getSQLState(), ex.getErrorCode(), newUpdateCounts);
                }
              }
            }
          }
        }
        finally
        {
          if (timeoutTask != null) {
            timeoutTask.cancel();
            
            locallyScopedConn.getCancelTimer().purge();
          }
          
          resetCancelledState();
        }
        
        if (sqlEx != null) {
          throw new BatchUpdateException(sqlEx.getMessage(), sqlEx.getSQLState(), sqlEx.getErrorCode(), updateCounts);
        }
      }
      


      int nbrCommands = updateCounts != null ? updateCounts : new int[0];jsr 16;return nbrCommands;
    } finally {
      jsr 6; } localObject8 = returnAddress;parameterBindings = oldBindValues;
    sendTypesToServer = true;
    
    clearBatch();ret;
    
    localObject9 = finally;throw localObject9;
  }
  






  protected ResultSetInternalMethods executeInternal(int maxRowsToRetrieve, Buffer sendPacket, boolean createStreamingResultSet, boolean queryIsSelectOnly, Field[] metadataFromCache, boolean isBatch)
    throws SQLException
  {
    numberOfExecutions += 1;
    
    try
    {
      return serverExecute(maxRowsToRetrieve, createStreamingResultSet, metadataFromCache);
    }
    catch (SQLException sqlEx)
    {
      if (connection.getEnablePacketDebug()) {
        connection.getIO().dumpPacketRingBuffer();
      }
      
      if (connection.getDumpQueriesOnException()) {
        String extractedSql = toString();
        StringBuffer messageBuf = new StringBuffer(extractedSql.length() + 32);
        
        messageBuf.append("\n\nQuery being executed when exception was thrown:\n");
        
        messageBuf.append(extractedSql);
        messageBuf.append("\n\n");
        
        sqlEx = ConnectionImpl.appendMessageToException(sqlEx, messageBuf.toString(), getExceptionInterceptor());
      }
      

      throw sqlEx;
    } catch (Exception ex) {
      if (connection.getEnablePacketDebug()) {
        connection.getIO().dumpPacketRingBuffer();
      }
      
      SQLException sqlEx = SQLError.createSQLException(ex.toString(), "S1000", getExceptionInterceptor());
      

      if (connection.getDumpQueriesOnException()) {
        String extractedSql = toString();
        StringBuffer messageBuf = new StringBuffer(extractedSql.length() + 32);
        
        messageBuf.append("\n\nQuery being executed when exception was thrown:\n");
        
        messageBuf.append(extractedSql);
        messageBuf.append("\n\n");
        
        sqlEx = ConnectionImpl.appendMessageToException(sqlEx, messageBuf.toString(), getExceptionInterceptor());
      }
      

      sqlEx.initCause(ex);
      
      throw sqlEx;
    }
  }
  

  protected Buffer fillSendPacket()
    throws SQLException
  {
    return null;
  }
  




  protected Buffer fillSendPacket(byte[][] batchedParameterStrings, InputStream[] batchedParameterStreams, boolean[] batchedIsStream, int[] batchedStreamLengths)
    throws SQLException
  {
    return null;
  }
  








  protected BindValue getBinding(int parameterIndex, boolean forLongData)
    throws SQLException
  {
    checkClosed();
    
    if (parameterBindings.length == 0) {
      throw SQLError.createSQLException(Messages.getString("ServerPreparedStatement.8"), "S1009", getExceptionInterceptor());
    }
    


    parameterIndex--;
    
    if ((parameterIndex < 0) || (parameterIndex >= parameterBindings.length))
    {
      throw SQLError.createSQLException(Messages.getString("ServerPreparedStatement.9") + (parameterIndex + 1) + Messages.getString("ServerPreparedStatement.10") + parameterBindings.length, "S1009", getExceptionInterceptor());
    }
    





    if (parameterBindings[parameterIndex] == null) {
      parameterBindings[parameterIndex] = new BindValue();
    }
    else if ((parameterBindings[parameterIndex].isLongData) && (!forLongData))
    {
      detectedLongParameterSwitch = true;
    }
    

    parameterBindings[parameterIndex].isSet = true;
    parameterBindings[parameterIndex].boundBeforeExecutionNum = numberOfExecutions;
    
    return parameterBindings[parameterIndex];
  }
  

  byte[] getBytes(int parameterIndex)
    throws SQLException
  {
    BindValue bindValue = getBinding(parameterIndex, false);
    
    if (isNull)
      return null;
    if (isLongData) {
      throw SQLError.notImplemented();
    }
    if (outByteBuffer == null) {
      outByteBuffer = new Buffer(connection.getNetBufferLength());
    }
    

    outByteBuffer.clear();
    
    int originalPosition = outByteBuffer.getPosition();
    
    storeBinding(outByteBuffer, bindValue, connection.getIO());
    
    int newPosition = outByteBuffer.getPosition();
    
    int length = newPosition - originalPosition;
    
    byte[] valueAsBytes = new byte[length];
    
    System.arraycopy(outByteBuffer.getByteBuffer(), originalPosition, valueAsBytes, 0, length);
    

    return valueAsBytes;
  }
  


  public java.sql.ResultSetMetaData getMetaData()
    throws SQLException
  {
    checkClosed();
    
    if (resultFields == null) {
      return null;
    }
    
    return new ResultSetMetaData(resultFields, connection.getUseOldAliasMetadataBehavior(), getExceptionInterceptor());
  }
  


  public ParameterMetaData getParameterMetaData()
    throws SQLException
  {
    checkClosed();
    
    if (parameterMetaData == null) {
      parameterMetaData = new MysqlParameterMetadata(parameterFields, parameterCount, getExceptionInterceptor());
    }
    

    return parameterMetaData;
  }
  


  boolean isNull(int paramIndex)
  {
    throw new IllegalArgumentException(Messages.getString("ServerPreparedStatement.7"));
  }
  









  protected void realClose(boolean calledExplicitly, boolean closeOpenResults)
    throws SQLException
  {
    if (isClosed) {
      return;
    }
    
    if (connection != null) {
      if (connection.getAutoGenerateTestcaseScript()) {
        dumpCloseForTestcase();
      }
      











      SQLException exceptionDuringClose = null;
      
      if ((calledExplicitly) && (!connection.isClosed())) {
        synchronized (connection.getMutex())
        {
          try {
            MysqlIO mysql = connection.getIO();
            
            Buffer packet = mysql.getSharedSendPacket();
            
            packet.writeByte((byte)25);
            packet.writeLong(serverStatementId);
            
            mysql.sendCommand(25, null, packet, true, null, 0);
          }
          catch (SQLException sqlEx) {
            exceptionDuringClose = sqlEx;
          }
        }
      }
      
      super.realClose(calledExplicitly, closeOpenResults);
      
      clearParametersInternal(false);
      parameterBindings = null;
      
      parameterFields = null;
      resultFields = null;
      
      if (exceptionDuringClose != null) {
        throw exceptionDuringClose;
      }
    }
  }
  





  protected void rePrepare()
    throws SQLException
  {
    invalidationException = null;
    try
    {
      serverPrepare(originalSql);
    }
    catch (SQLException sqlEx) {
      invalidationException = sqlEx;
    } catch (Exception ex) {
      invalidationException = SQLError.createSQLException(ex.toString(), "S1000", getExceptionInterceptor());
      
      invalidationException.initCause(ex);
    }
    
    if (invalidationException != null) {
      invalid = true;
      
      parameterBindings = null;
      
      parameterFields = null;
      resultFields = null;
      
      if (results != null) {
        try {
          results.close();
        }
        catch (Exception ex) {}
      }
      

      if (connection != null) {
        if (maxRowsChanged) {
          connection.unsetMaxRows(this);
        }
        
        if (!connection.getDontTrackOpenResources()) {
          connection.unregisterStatement(this);
        }
      }
    }
  }
  

































  private ResultSetInternalMethods serverExecute(int maxRowsToRetrieve, boolean createStreamingResultSet, Field[] metadataFromCache)
    throws SQLException
  {
    synchronized (connection.getMutex()) {
      MysqlIO mysql = connection.getIO();
      
      if (mysql.shouldIntercept()) {
        ResultSetInternalMethods interceptedResults = mysql.invokeStatementInterceptorsPre(originalSql, this, true);
        

        if (interceptedResults != null) {
          return interceptedResults;
        }
      }
      
      if (detectedLongParameterSwitch)
      {
        boolean firstFound = false;
        long boundTimeToCheck = 0L;
        
        for (int i = 0; i < parameterCount - 1; i++) {
          if (parameterBindings[i].isLongData) {
            if ((firstFound) && (boundTimeToCheck != parameterBindings[i].boundBeforeExecutionNum))
            {
              throw SQLError.createSQLException(Messages.getString("ServerPreparedStatement.11") + Messages.getString("ServerPreparedStatement.12"), "S1C00", getExceptionInterceptor());
            }
            


            firstFound = true;
            boundTimeToCheck = parameterBindings[i].boundBeforeExecutionNum;
          }
        }
        




        serverResetStatement();
      }
      


      for (int i = 0; i < parameterCount; i++) {
        if (!parameterBindings[i].isSet) {
          throw SQLError.createSQLException(Messages.getString("ServerPreparedStatement.13") + (i + 1) + Messages.getString("ServerPreparedStatement.14"), "S1009", getExceptionInterceptor());
        }
      }
      






      for (int i = 0; i < parameterCount; i++) {
        if (parameterBindings[i].isLongData) {
          serverLongData(i, parameterBindings[i]);
        }
      }
      
      if (connection.getAutoGenerateTestcaseScript()) {
        dumpExecuteForTestcase();
      }
      




      Buffer packet = mysql.getSharedSendPacket();
      
      packet.clear();
      packet.writeByte((byte)23);
      packet.writeLong(serverStatementId);
      
      boolean usingCursor = false;
      
      if (connection.versionMeetsMinimum(4, 1, 2))
      {





        if ((resultFields != null) && (connection.isCursorFetchEnabled()) && (getResultSetType() == 1003) && (getResultSetConcurrency() == 1007) && (getFetchSize() > 0))
        {



          packet.writeByte((byte)1);
          usingCursor = true;
        } else {
          packet.writeByte((byte)0);
        }
        
        packet.writeLong(1L);
      }
      


      int nullCount = (parameterCount + 7) / 8;
      



      int nullBitsPosition = packet.getPosition();
      
      for (int i = 0; i < nullCount; i++) {
        packet.writeByte((byte)0);
      }
      
      byte[] nullBitsBuffer = new byte[nullCount];
      

      packet.writeByte((byte)(sendTypesToServer ? 1 : 0));
      
      if (sendTypesToServer)
      {



        for (int i = 0; i < parameterCount; i++) {
          packet.writeInt(parameterBindings[i].bufferType);
        }
      }
      



      for (int i = 0; i < parameterCount; i++) {
        if (!parameterBindings[i].isLongData) {
          if (!parameterBindings[i].isNull) {
            storeBinding(packet, parameterBindings[i], mysql);
          } else {
            int tmp601_600 = (i / 8); byte[] tmp601_594 = nullBitsBuffer;tmp601_594[tmp601_600] = ((byte)(tmp601_594[tmp601_600] | 1 << (i & 0x7)));
          }
        }
      }
      




      int endPosition = packet.getPosition();
      packet.setPosition(nullBitsPosition);
      packet.writeBytesNoNull(nullBitsBuffer);
      packet.setPosition(endPosition);
      
      long begin = 0L;
      
      boolean logSlowQueries = connection.getLogSlowQueries();
      boolean gatherPerformanceMetrics = connection.getGatherPerformanceMetrics();
      

      if ((profileSQL) || (logSlowQueries) || (gatherPerformanceMetrics)) {
        begin = mysql.getCurrentTimeNanosOrMillis();
      }
      
      resetCancelledState();
      
      StatementImpl.CancelTask timeoutTask = null;
      try
      {
        if ((connection.getEnableQueryTimeouts()) && (timeoutInMillis != 0) && (connection.versionMeetsMinimum(5, 0, 0)))
        {

          timeoutTask = new StatementImpl.CancelTask(this, this);
          connection.getCancelTimer().schedule(timeoutTask, timeoutInMillis);
        }
        

        Buffer resultPacket = mysql.sendCommand(23, null, packet, false, null, 0);
        

        long queryEndTime = 0L;
        
        if ((logSlowQueries) || (gatherPerformanceMetrics) || (profileSQL)) {
          queryEndTime = mysql.getCurrentTimeNanosOrMillis();
        }
        
        if (timeoutTask != null) {
          timeoutTask.cancel();
          
          connection.getCancelTimer().purge();
          
          if (caughtWhileCancelling != null) {
            throw caughtWhileCancelling;
          }
          
          timeoutTask = null;
        }
        
        synchronized (cancelTimeoutMutex) {
          if (wasCancelled) {
            SQLException cause = null;
            
            if (wasCancelledByTimeout) {
              cause = new MySQLTimeoutException();
            } else {
              cause = new MySQLStatementCancelledException();
            }
            
            resetCancelledState();
            
            throw cause;
          }
        }
        
        boolean queryWasSlow = false;
        
        if ((logSlowQueries) || (gatherPerformanceMetrics)) {
          long elapsedTime = queryEndTime - begin;
          
          if (logSlowQueries) {
            if (useAutoSlowLog) {
              queryWasSlow = elapsedTime > connection.getSlowQueryThresholdMillis();
            } else {
              queryWasSlow = connection.isAbonormallyLongQuery(elapsedTime);
              
              connection.reportQueryTime(elapsedTime);
            }
          }
          
          if (queryWasSlow)
          {
            StringBuffer mesgBuf = new StringBuffer(48 + originalSql.length());
            
            mesgBuf.append(Messages.getString("ServerPreparedStatement.15"));
            
            mesgBuf.append(mysql.getSlowQueryThreshold());
            mesgBuf.append(Messages.getString("ServerPreparedStatement.15a"));
            
            mesgBuf.append(elapsedTime);
            mesgBuf.append(Messages.getString("ServerPreparedStatement.16"));
            

            mesgBuf.append("as prepared: ");
            mesgBuf.append(originalSql);
            mesgBuf.append("\n\n with parameters bound:\n\n");
            mesgBuf.append(asSql(true));
            
            eventSink.consumeEvent(new ProfilerEvent((byte)6, "", currentCatalog, connection.getId(), getId(), 0, System.currentTimeMillis(), elapsedTime, mysql.getQueryTimingUnits(), null, new Throwable(), mesgBuf.toString()));
          }
          







          if (gatherPerformanceMetrics) {
            connection.registerQueryExecutionTime(elapsedTime);
          }
        }
        
        connection.incrementNumberOfPreparedExecutes();
        
        if (profileSQL) {
          eventSink = ProfilerEventHandlerFactory.getInstance(connection);
          

          eventSink.consumeEvent(new ProfilerEvent((byte)4, "", currentCatalog, connectionId, statementId, -1, System.currentTimeMillis(), (int)(mysql.getCurrentTimeNanosOrMillis() - begin), mysql.getQueryTimingUnits(), null, new Throwable(), truncateQueryToLog(asSql(true))));
        }
        







        ResultSetInternalMethods rs = mysql.readAllResults(this, maxRowsToRetrieve, resultSetType, resultSetConcurrency, createStreamingResultSet, currentCatalog, resultPacket, true, fieldCount, metadataFromCache);
        




        if (mysql.shouldIntercept()) {
          ResultSetInternalMethods interceptedResults = mysql.invokeStatementInterceptorsPost(originalSql, this, rs, true, null);
          

          if (interceptedResults != null) {
            rs = interceptedResults;
          }
        }
        
        if (profileSQL) {
          long fetchEndTime = mysql.getCurrentTimeNanosOrMillis();
          
          eventSink.consumeEvent(new ProfilerEvent((byte)5, "", currentCatalog, connection.getId(), getId(), 0, System.currentTimeMillis(), fetchEndTime - queryEndTime, mysql.getQueryTimingUnits(), null, new Throwable(), null));
        }
        






        if ((queryWasSlow) && (connection.getExplainSlowQueries())) {
          queryAsString = asSql(true);
          
          mysql.explainSlowQuery(((String)queryAsString).getBytes(), (String)queryAsString);
        }
        

        if ((!createStreamingResultSet) && (serverNeedsResetBeforeEachExecution))
        {
          serverResetStatement();
        }
        

        sendTypesToServer = false;
        results = rs;
        
        if (mysql.hadWarnings()) {
          mysql.scanForAndThrowDataTruncation();
        }
        
        Object queryAsString = rs;jsr 45;return queryAsString;
      } catch (SQLException sqlEx) {
        if (mysql.shouldIntercept()) {
          mysql.invokeStatementInterceptorsPost(originalSql, this, null, true, sqlEx);
        }
        
        throw sqlEx;
      } finally {
        jsr 6; } localObject3 = returnAddress; if (timeoutTask != null) {
        timeoutTask.cancel();
        connection.getCancelTimer().purge(); } ret;
    }
  }
  




























  private void serverLongData(int parameterIndex, BindValue longData)
    throws SQLException
  {
    synchronized (connection.getMutex()) {
      MysqlIO mysql = connection.getIO();
      
      Buffer packet = mysql.getSharedSendPacket();
      
      Object value = value;
      
      if ((value instanceof byte[])) {
        packet.clear();
        packet.writeByte((byte)24);
        packet.writeLong(serverStatementId);
        packet.writeInt(parameterIndex);
        
        packet.writeBytesNoNull((byte[])value);
        
        mysql.sendCommand(24, null, packet, true, null, 0);
      }
      else if ((value instanceof InputStream)) {
        storeStream(mysql, parameterIndex, packet, (InputStream)value);
      } else if ((value instanceof Blob)) {
        storeStream(mysql, parameterIndex, packet, ((Blob)value).getBinaryStream());
      }
      else if ((value instanceof Reader)) {
        storeReader(mysql, parameterIndex, packet, (Reader)value);
      } else {
        throw SQLError.createSQLException(Messages.getString("ServerPreparedStatement.18") + value.getClass().getName() + "'", "S1009", getExceptionInterceptor());
      }
    }
  }
  

  private void serverPrepare(String sql)
    throws SQLException
  {
    synchronized (connection.getMutex()) {
      MysqlIO mysql = connection.getIO();
      
      if (connection.getAutoGenerateTestcaseScript()) {
        dumpPrepareForTestcase();
      }
      try
      {
        long begin = 0L;
        
        if (StringUtils.startsWithIgnoreCaseAndWs(sql, "LOAD DATA")) {
          isLoadDataQuery = true;
        } else {
          isLoadDataQuery = false;
        }
        
        if (connection.getProfileSql()) {
          begin = System.currentTimeMillis();
        }
        
        String characterEncoding = null;
        String connectionEncoding = connection.getEncoding();
        
        if ((!isLoadDataQuery) && (connection.getUseUnicode()) && (connectionEncoding != null))
        {
          characterEncoding = connectionEncoding;
        }
        
        Buffer prepareResultPacket = mysql.sendCommand(22, sql, null, false, characterEncoding, 0);
        


        if (connection.versionMeetsMinimum(4, 1, 1))
        {



          prepareResultPacket.setPosition(1);
        }
        else
        {
          prepareResultPacket.setPosition(0);
        }
        
        serverStatementId = prepareResultPacket.readLong();
        fieldCount = prepareResultPacket.readInt();
        parameterCount = prepareResultPacket.readInt();
        parameterBindings = new BindValue[parameterCount];
        
        for (int i = 0; i < parameterCount; i++) {
          parameterBindings[i] = new BindValue();
        }
        
        connection.incrementNumberOfPrepares();
        
        if (profileSQL) {
          eventSink.consumeEvent(new ProfilerEvent((byte)2, "", currentCatalog, connectionId, statementId, -1, System.currentTimeMillis(), mysql.getCurrentTimeNanosOrMillis() - begin, mysql.getQueryTimingUnits(), null, new Throwable(), truncateQueryToLog(sql)));
        }
        







        if ((parameterCount > 0) && 
          (connection.versionMeetsMinimum(4, 1, 2)) && (!mysql.isVersion(5, 0, 0)))
        {
          parameterFields = new Field[parameterCount];
          
          Buffer metaDataPacket = mysql.readPacket();
          
          int i = 0;
          

          while ((!metaDataPacket.isLastDataPacket()) && (i < parameterCount)) {
            parameterFields[(i++)] = mysql.unpackField(metaDataPacket, false);
            
            metaDataPacket = mysql.readPacket();
          }
        }
        

        if (fieldCount > 0) {
          resultFields = new Field[fieldCount];
          
          Buffer fieldPacket = mysql.readPacket();
          
          int i = 0;
          


          while ((!fieldPacket.isLastDataPacket()) && (i < fieldCount)) {
            resultFields[(i++)] = mysql.unpackField(fieldPacket, false);
            
            fieldPacket = mysql.readPacket();
          }
        }
      } catch (SQLException sqlEx) {
        if (connection.getDumpQueriesOnException()) {
          StringBuffer messageBuf = new StringBuffer(originalSql.length() + 32);
          
          messageBuf.append("\n\nQuery being prepared when exception was thrown:\n\n");
          
          messageBuf.append(originalSql);
          
          sqlEx = ConnectionImpl.appendMessageToException(sqlEx, messageBuf.toString(), getExceptionInterceptor());
        }
        

        throw sqlEx;

      }
      finally
      {
        connection.getIO().clearInputStream();
      }
    }
  }
  
  private String truncateQueryToLog(String sql) {
    String query = null;
    
    if (sql.length() > connection.getMaxQuerySizeToLog()) {
      StringBuffer queryBuf = new StringBuffer(connection.getMaxQuerySizeToLog() + 12);
      
      queryBuf.append(sql.substring(0, connection.getMaxQuerySizeToLog()));
      queryBuf.append(Messages.getString("MysqlIO.25"));
      
      query = queryBuf.toString();
    } else {
      query = sql;
    }
    
    return query;
  }
  
  private void serverResetStatement() throws SQLException {
    synchronized (connection.getMutex())
    {
      MysqlIO mysql = connection.getIO();
      
      Buffer packet = mysql.getSharedSendPacket();
      
      packet.clear();
      packet.writeByte((byte)26);
      packet.writeLong(serverStatementId);
      try
      {
        mysql.sendCommand(26, null, packet, !connection.versionMeetsMinimum(4, 1, 2), null, 0);
      }
      catch (SQLException sqlEx) {
        throw sqlEx;
      } catch (Exception ex) {
        SQLException sqlEx = SQLError.createSQLException(ex.toString(), "S1000", getExceptionInterceptor());
        
        sqlEx.initCause(ex);
        
        throw sqlEx;
      } finally {
        mysql.clearInputStream();
      }
    }
  }
  

  public void setArray(int i, Array x)
    throws SQLException
  {
    throw SQLError.notImplemented();
  }
  



  public void setAsciiStream(int parameterIndex, InputStream x, int length)
    throws SQLException
  {
    checkClosed();
    
    if (x == null) {
      setNull(parameterIndex, -2);
    } else {
      BindValue binding = getBinding(parameterIndex, true);
      setType(binding, 252);
      
      value = x;
      isNull = false;
      isLongData = true;
      
      if (connection.getUseStreamLengthsInPrepStmts()) {
        bindLength = length;
      } else {
        bindLength = -1L;
      }
    }
  }
  


  public void setBigDecimal(int parameterIndex, BigDecimal x)
    throws SQLException
  {
    checkClosed();
    
    if (x == null) {
      setNull(parameterIndex, 3);
    }
    else {
      BindValue binding = getBinding(parameterIndex, false);
      
      if (connection.versionMeetsMinimum(5, 0, 3)) {
        setType(binding, 246);
      } else {
        setType(binding, stringTypeCode);
      }
      
      value = StringUtils.fixDecimalExponent(StringUtils.consistentToString(x));
      
      isNull = false;
      isLongData = false;
    }
  }
  



  public void setBinaryStream(int parameterIndex, InputStream x, int length)
    throws SQLException
  {
    checkClosed();
    
    if (x == null) {
      setNull(parameterIndex, -2);
    } else {
      BindValue binding = getBinding(parameterIndex, true);
      setType(binding, 252);
      
      value = x;
      isNull = false;
      isLongData = true;
      
      if (connection.getUseStreamLengthsInPrepStmts()) {
        bindLength = length;
      } else {
        bindLength = -1L;
      }
    }
  }
  

  public void setBlob(int parameterIndex, Blob x)
    throws SQLException
  {
    checkClosed();
    
    if (x == null) {
      setNull(parameterIndex, -2);
    } else {
      BindValue binding = getBinding(parameterIndex, true);
      setType(binding, 252);
      
      value = x;
      isNull = false;
      isLongData = true;
      
      if (connection.getUseStreamLengthsInPrepStmts()) {
        bindLength = x.length();
      } else {
        bindLength = -1L;
      }
    }
  }
  

  public void setBoolean(int parameterIndex, boolean x)
    throws SQLException
  {
    setByte(parameterIndex, (byte)(x ? 1 : 0));
  }
  

  public void setByte(int parameterIndex, byte x)
    throws SQLException
  {
    checkClosed();
    
    BindValue binding = getBinding(parameterIndex, false);
    setType(binding, 1);
    
    value = null;
    byteBinding = x;
    isNull = false;
    isLongData = false;
  }
  

  public void setBytes(int parameterIndex, byte[] x)
    throws SQLException
  {
    checkClosed();
    
    if (x == null) {
      setNull(parameterIndex, -2);
    } else {
      BindValue binding = getBinding(parameterIndex, false);
      setType(binding, 253);
      
      value = x;
      isNull = false;
      isLongData = false;
    }
  }
  



  public void setCharacterStream(int parameterIndex, Reader reader, int length)
    throws SQLException
  {
    checkClosed();
    
    if (reader == null) {
      setNull(parameterIndex, -2);
    } else {
      BindValue binding = getBinding(parameterIndex, true);
      setType(binding, 252);
      
      value = reader;
      isNull = false;
      isLongData = true;
      
      if (connection.getUseStreamLengthsInPrepStmts()) {
        bindLength = length;
      } else {
        bindLength = -1L;
      }
    }
  }
  

  public void setClob(int parameterIndex, Clob x)
    throws SQLException
  {
    checkClosed();
    
    if (x == null) {
      setNull(parameterIndex, -2);
    } else {
      BindValue binding = getBinding(parameterIndex, true);
      setType(binding, 252);
      
      value = x.getCharacterStream();
      isNull = false;
      isLongData = true;
      
      if (connection.getUseStreamLengthsInPrepStmts()) {
        bindLength = x.length();
      } else {
        bindLength = -1L;
      }
    }
  }
  










  public void setDate(int parameterIndex, java.sql.Date x)
    throws SQLException
  {
    setDate(parameterIndex, x, null);
  }
  













  public void setDate(int parameterIndex, java.sql.Date x, Calendar cal)
    throws SQLException
  {
    if (x == null) {
      setNull(parameterIndex, 91);
    } else {
      BindValue binding = getBinding(parameterIndex, false);
      setType(binding, 10);
      
      value = x;
      isNull = false;
      isLongData = false;
    }
  }
  

  public void setDouble(int parameterIndex, double x)
    throws SQLException
  {
    checkClosed();
    
    if ((!connection.getAllowNanAndInf()) && ((x == (1.0D / 0.0D)) || (x == (-1.0D / 0.0D)) || (Double.isNaN(x))))
    {

      throw SQLError.createSQLException("'" + x + "' is not a valid numeric or approximate numeric value", "S1009", getExceptionInterceptor());
    }
    



    BindValue binding = getBinding(parameterIndex, false);
    setType(binding, 5);
    
    value = null;
    doubleBinding = x;
    isNull = false;
    isLongData = false;
  }
  

  public void setFloat(int parameterIndex, float x)
    throws SQLException
  {
    checkClosed();
    
    BindValue binding = getBinding(parameterIndex, false);
    setType(binding, 4);
    
    value = null;
    floatBinding = x;
    isNull = false;
    isLongData = false;
  }
  

  public void setInt(int parameterIndex, int x)
    throws SQLException
  {
    checkClosed();
    
    BindValue binding = getBinding(parameterIndex, false);
    setType(binding, 3);
    
    value = null;
    intBinding = x;
    isNull = false;
    isLongData = false;
  }
  

  public void setLong(int parameterIndex, long x)
    throws SQLException
  {
    checkClosed();
    
    BindValue binding = getBinding(parameterIndex, false);
    setType(binding, 8);
    
    value = null;
    longBinding = x;
    isNull = false;
    isLongData = false;
  }
  

  public void setNull(int parameterIndex, int sqlType)
    throws SQLException
  {
    checkClosed();
    
    BindValue binding = getBinding(parameterIndex, false);
    




    if (bufferType == 0) {
      setType(binding, 6);
    }
    
    value = null;
    isNull = true;
    isLongData = false;
  }
  


  public void setNull(int parameterIndex, int sqlType, String typeName)
    throws SQLException
  {
    checkClosed();
    
    BindValue binding = getBinding(parameterIndex, false);
    




    if (bufferType == 0) {
      setType(binding, 6);
    }
    
    value = null;
    isNull = true;
    isLongData = false;
  }
  

  public void setRef(int i, Ref x)
    throws SQLException
  {
    throw SQLError.notImplemented();
  }
  

  public void setShort(int parameterIndex, short x)
    throws SQLException
  {
    checkClosed();
    
    BindValue binding = getBinding(parameterIndex, false);
    setType(binding, 2);
    
    value = null;
    shortBinding = x;
    isNull = false;
    isLongData = false;
  }
  

  public void setString(int parameterIndex, String x)
    throws SQLException
  {
    checkClosed();
    
    if (x == null) {
      setNull(parameterIndex, 1);
    } else {
      BindValue binding = getBinding(parameterIndex, false);
      
      setType(binding, stringTypeCode);
      
      value = x;
      isNull = false;
      isLongData = false;
    }
  }
  










  public void setTime(int parameterIndex, Time x)
    throws SQLException
  {
    setTimeInternal(parameterIndex, x, null, connection.getDefaultTimeZone(), false);
  }
  














  public void setTime(int parameterIndex, Time x, Calendar cal)
    throws SQLException
  {
    setTimeInternal(parameterIndex, x, cal, cal.getTimeZone(), true);
  }
  















  public void setTimeInternal(int parameterIndex, Time x, Calendar targetCalendar, TimeZone tz, boolean rollForward)
    throws SQLException
  {
    if (x == null) {
      setNull(parameterIndex, 92);
    } else {
      BindValue binding = getBinding(parameterIndex, false);
      setType(binding, 11);
      
      if (!useLegacyDatetimeCode) {
        value = x;
      } else {
        Calendar sessionCalendar = getCalendarInstanceForSessionOrNew();
        
        synchronized (sessionCalendar) {
          value = TimeUtil.changeTimezone(connection, sessionCalendar, targetCalendar, x, tz, connection.getServerTimezoneTZ(), rollForward);
        }
      }
      





      isNull = false;
      isLongData = false;
    }
  }
  











  public void setTimestamp(int parameterIndex, Timestamp x)
    throws SQLException
  {
    setTimestampInternal(parameterIndex, x, null, connection.getDefaultTimeZone(), false);
  }
  













  public void setTimestamp(int parameterIndex, Timestamp x, Calendar cal)
    throws SQLException
  {
    setTimestampInternal(parameterIndex, x, cal, cal.getTimeZone(), true);
  }
  

  protected void setTimestampInternal(int parameterIndex, Timestamp x, Calendar targetCalendar, TimeZone tz, boolean rollForward)
    throws SQLException
  {
    if (x == null) {
      setNull(parameterIndex, 93);
    } else {
      BindValue binding = getBinding(parameterIndex, false);
      setType(binding, 12);
      
      if (!useLegacyDatetimeCode) {
        value = x;
      } else {
        Calendar sessionCalendar = connection.getUseJDBCCompliantTimezoneShift() ? connection.getUtcCalendar() : getCalendarInstanceForSessionOrNew();
        


        synchronized (sessionCalendar) {
          value = TimeUtil.changeTimezone(connection, sessionCalendar, targetCalendar, x, tz, connection.getServerTimezoneTZ(), rollForward);
        }
        





        isNull = false;
        isLongData = false;
      }
    }
  }
  
  protected void setType(BindValue oldValue, int bufferType) {
    if (bufferType != bufferType) {
      sendTypesToServer = true;
    }
    
    bufferType = bufferType;
  }
  















  /**
   * @deprecated
   */
  public void setUnicodeStream(int parameterIndex, InputStream x, int length)
    throws SQLException
  {
    checkClosed();
    
    throw SQLError.notImplemented();
  }
  

  public void setURL(int parameterIndex, URL x)
    throws SQLException
  {
    checkClosed();
    
    setString(parameterIndex, x.toString());
  }
  









  private void storeBinding(Buffer packet, BindValue bindValue, MysqlIO mysql)
    throws SQLException
  {
    try
    {
      Object value = value;
      



      switch (bufferType)
      {
      case 1: 
        packet.writeByte(byteBinding);
        return;
      case 2: 
        packet.ensureCapacity(2);
        packet.writeInt(shortBinding);
        return;
      case 3: 
        packet.ensureCapacity(4);
        packet.writeLong(intBinding);
        return;
      case 8: 
        packet.ensureCapacity(8);
        packet.writeLongLong(longBinding);
        return;
      case 4: 
        packet.ensureCapacity(4);
        packet.writeFloat(floatBinding);
        return;
      case 5: 
        packet.ensureCapacity(8);
        packet.writeDouble(doubleBinding);
        return;
      case 11: 
        storeTime(packet, (Time)value);
        return;
      case 7: 
      case 10: 
      case 12: 
        storeDateTime(packet, (java.util.Date)value, mysql, bufferType);
        return;
      case 0: 
      case 15: 
      case 246: 
      case 253: 
      case 254: 
        if ((value instanceof byte[])) {
          packet.writeLenBytes((byte[])value);
        } else if (!isLoadDataQuery) {
          packet.writeLenString((String)value, charEncoding, connection.getServerCharacterEncoding(), charConverter, connection.parserKnowsUnicode(), connection);

        }
        else
        {

          packet.writeLenBytes(((String)value).getBytes());
        }
        
        return;
      }
    }
    catch (UnsupportedEncodingException uEE)
    {
      throw SQLError.createSQLException(Messages.getString("ServerPreparedStatement.22") + connection.getEncoding() + "'", "S1000", getExceptionInterceptor());
    }
  }
  



  private void storeDateTime412AndOlder(Buffer intoBuf, java.util.Date dt, int bufferType)
    throws SQLException
  {
    Calendar sessionCalendar = null;
    
    if (!useLegacyDatetimeCode) {
      if (bufferType == 10) {
        sessionCalendar = getDefaultTzCalendar();
      } else {
        sessionCalendar = getServerTzCalendar();
      }
    } else {
      sessionCalendar = ((dt instanceof Timestamp)) && (connection.getUseJDBCCompliantTimezoneShift()) ? connection.getUtcCalendar() : getCalendarInstanceForSessionOrNew();
    }
    


    synchronized (sessionCalendar) {
      java.util.Date oldTime = sessionCalendar.getTime();
      try
      {
        intoBuf.ensureCapacity(8);
        intoBuf.writeByte((byte)7);
        
        sessionCalendar.setTime(dt);
        
        int year = sessionCalendar.get(1);
        int month = sessionCalendar.get(2) + 1;
        int date = sessionCalendar.get(5);
        
        intoBuf.writeInt(year);
        intoBuf.writeByte((byte)month);
        intoBuf.writeByte((byte)date);
        
        if ((dt instanceof java.sql.Date)) {
          intoBuf.writeByte((byte)0);
          intoBuf.writeByte((byte)0);
          intoBuf.writeByte((byte)0);
        } else {
          intoBuf.writeByte((byte)sessionCalendar.get(11));
          
          intoBuf.writeByte((byte)sessionCalendar.get(12));
          
          intoBuf.writeByte((byte)sessionCalendar.get(13));
        }
      }
      finally {
        sessionCalendar.setTime(oldTime);
      }
    }
  }
  
  private void storeDateTime(Buffer intoBuf, java.util.Date dt, MysqlIO mysql, int bufferType) throws SQLException
  {
    if (connection.versionMeetsMinimum(4, 1, 3)) {
      storeDateTime413AndNewer(intoBuf, dt, bufferType);
    } else {
      storeDateTime412AndOlder(intoBuf, dt, bufferType);
    }
  }
  
  private void storeDateTime413AndNewer(Buffer intoBuf, java.util.Date dt, int bufferType) throws SQLException
  {
    Calendar sessionCalendar = null;
    
    if (!useLegacyDatetimeCode) {
      if (bufferType == 10) {
        sessionCalendar = getDefaultTzCalendar();
      } else {
        sessionCalendar = getServerTzCalendar();
      }
    } else {
      sessionCalendar = ((dt instanceof Timestamp)) && (connection.getUseJDBCCompliantTimezoneShift()) ? connection.getUtcCalendar() : getCalendarInstanceForSessionOrNew();
    }
    


    synchronized (sessionCalendar) {
      java.util.Date oldTime = sessionCalendar.getTime();
      try
      {
        sessionCalendar.setTime(dt);
        
        if ((dt instanceof java.sql.Date)) {
          sessionCalendar.set(11, 0);
          sessionCalendar.set(12, 0);
          sessionCalendar.set(13, 0);
        }
        
        byte length = 7;
        
        if ((dt instanceof Timestamp)) {
          length = 11;
        }
        
        intoBuf.ensureCapacity(length);
        
        intoBuf.writeByte(length);
        
        int year = sessionCalendar.get(1);
        int month = sessionCalendar.get(2) + 1;
        int date = sessionCalendar.get(5);
        
        intoBuf.writeInt(year);
        intoBuf.writeByte((byte)month);
        intoBuf.writeByte((byte)date);
        
        if ((dt instanceof java.sql.Date)) {
          intoBuf.writeByte((byte)0);
          intoBuf.writeByte((byte)0);
          intoBuf.writeByte((byte)0);
        } else {
          intoBuf.writeByte((byte)sessionCalendar.get(11));
          
          intoBuf.writeByte((byte)sessionCalendar.get(12));
          
          intoBuf.writeByte((byte)sessionCalendar.get(13));
        }
        

        if (length == 11)
        {
          intoBuf.writeLong(((Timestamp)dt).getNanos() / 1000);
        }
      }
      finally {
        sessionCalendar.setTime(oldTime);
      }
    }
  }
  
  private Calendar getServerTzCalendar() {
    synchronized (this) {
      if (serverTzCalendar == null) {
        serverTzCalendar = new GregorianCalendar(connection.getServerTimezoneTZ());
      }
      
      return serverTzCalendar;
    }
  }
  
  private Calendar getDefaultTzCalendar() {
    synchronized (this) {
      if (defaultTzCalendar == null) {
        defaultTzCalendar = new GregorianCalendar(TimeZone.getDefault());
      }
      
      return defaultTzCalendar;
    }
  }
  


  private void storeReader(MysqlIO mysql, int parameterIndex, Buffer packet, Reader inStream)
    throws SQLException
  {
    String forcedEncoding = connection.getClobCharacterEncoding();
    
    String clobEncoding = forcedEncoding == null ? connection.getEncoding() : forcedEncoding;
    

    int maxBytesChar = 2;
    
    if (clobEncoding != null) {
      if (!clobEncoding.equals("UTF-16")) {
        maxBytesChar = connection.getMaxBytesPerChar(clobEncoding);
        
        if (maxBytesChar == 1) {
          maxBytesChar = 2;
        }
      } else {
        maxBytesChar = 4;
      }
    }
    
    char[] buf = new char[8192 / maxBytesChar];
    
    int numRead = 0;
    
    int bytesInPacket = 0;
    int totalBytesRead = 0;
    int bytesReadAtLastSend = 0;
    int packetIsFullAt = connection.getBlobSendChunkSize();
    

    try
    {
      packet.clear();
      packet.writeByte((byte)24);
      packet.writeLong(serverStatementId);
      packet.writeInt(parameterIndex);
      
      boolean readAny = false;
      
      while ((numRead = inStream.read(buf)) != -1) {
        readAny = true;
        
        byte[] valueAsBytes = StringUtils.getBytes(buf, null, clobEncoding, connection.getServerCharacterEncoding(), 0, numRead, connection.parserKnowsUnicode(), getExceptionInterceptor());
        



        packet.writeBytesNoNull(valueAsBytes, 0, valueAsBytes.length);
        
        bytesInPacket += valueAsBytes.length;
        totalBytesRead += valueAsBytes.length;
        
        if (bytesInPacket >= packetIsFullAt) {
          bytesReadAtLastSend = totalBytesRead;
          
          mysql.sendCommand(24, null, packet, true, null, 0);
          

          bytesInPacket = 0;
          packet.clear();
          packet.writeByte((byte)24);
          packet.writeLong(serverStatementId);
          packet.writeInt(parameterIndex);
        }
      }
      
      if (totalBytesRead != bytesReadAtLastSend) {
        mysql.sendCommand(24, null, packet, true, null, 0);
      }
      

      if (!readAny) {
        mysql.sendCommand(24, null, packet, true, null, 0);
      }
    }
    catch (IOException ioEx) {
      SQLException sqlEx = SQLError.createSQLException(Messages.getString("ServerPreparedStatement.24") + ioEx.toString(), "S1000", getExceptionInterceptor());
      

      sqlEx.initCause(ioEx);
      
      throw sqlEx;
    } finally {
      if ((connection.getAutoClosePStmtStreams()) && 
        (inStream != null)) {
        try {
          inStream.close();
        }
        catch (IOException ioEx) {}
      }
    }
  }
  

  private void storeStream(MysqlIO mysql, int parameterIndex, Buffer packet, InputStream inStream)
    throws SQLException
  {
    byte[] buf = new byte[8192];
    
    int numRead = 0;
    try
    {
      int bytesInPacket = 0;
      int totalBytesRead = 0;
      int bytesReadAtLastSend = 0;
      int packetIsFullAt = connection.getBlobSendChunkSize();
      
      packet.clear();
      packet.writeByte((byte)24);
      packet.writeLong(serverStatementId);
      packet.writeInt(parameterIndex);
      
      boolean readAny = false;
      
      while ((numRead = inStream.read(buf)) != -1)
      {
        readAny = true;
        
        packet.writeBytesNoNull(buf, 0, numRead);
        bytesInPacket += numRead;
        totalBytesRead += numRead;
        
        if (bytesInPacket >= packetIsFullAt) {
          bytesReadAtLastSend = totalBytesRead;
          
          mysql.sendCommand(24, null, packet, true, null, 0);
          

          bytesInPacket = 0;
          packet.clear();
          packet.writeByte((byte)24);
          packet.writeLong(serverStatementId);
          packet.writeInt(parameterIndex);
        }
      }
      
      if (totalBytesRead != bytesReadAtLastSend) {
        mysql.sendCommand(24, null, packet, true, null, 0);
      }
      

      if (!readAny) {
        mysql.sendCommand(24, null, packet, true, null, 0);
      }
    }
    catch (IOException ioEx) {
      SQLException sqlEx = SQLError.createSQLException(Messages.getString("ServerPreparedStatement.25") + ioEx.toString(), "S1000", getExceptionInterceptor());
      

      sqlEx.initCause(ioEx);
      
      throw sqlEx;
    } finally {
      if ((connection.getAutoClosePStmtStreams()) && 
        (inStream != null)) {
        try {
          inStream.close();
        }
        catch (IOException ioEx) {}
      }
    }
  }
  




  public String toString()
  {
    StringBuffer toStringBuf = new StringBuffer();
    
    toStringBuf.append("com.mysql.jdbc.ServerPreparedStatement[");
    toStringBuf.append(serverStatementId);
    toStringBuf.append("] - ");
    try
    {
      toStringBuf.append(asSql());
    } catch (SQLException sqlEx) {
      toStringBuf.append(Messages.getString("ServerPreparedStatement.6"));
      toStringBuf.append(sqlEx);
    }
    
    return toStringBuf.toString();
  }
  
  protected long getServerStatementId() {
    return serverStatementId;
  }
  
  private boolean hasCheckedRewrite = false;
  private boolean canRewrite = false;
  
  public synchronized boolean canRewriteAsMultiValueInsertAtSqlLevel() throws SQLException {
    if (!hasCheckedRewrite) {
      hasCheckedRewrite = true;
      canRewrite = canRewrite(originalSql, isOnDuplicateKeyUpdate(), getLocationOfOnDuplicateKeyUpdate(), 0);
      
      parseInfo = new PreparedStatement.ParseInfo(this, originalSql, connection, connection.getMetaData(), charEncoding, charConverter);
    }
    
    return canRewrite;
  }
  
  public synchronized boolean canRewriteAsMultivalueInsertStatement() throws SQLException
  {
    if (!canRewriteAsMultiValueInsertAtSqlLevel()) {
      return false;
    }
    
    BindValue[] currentBindValues = null;
    BindValue[] previousBindValues = null;
    
    int nbrCommands = batchedArgs.size();
    


    for (int commandIndex = 0; commandIndex < nbrCommands; commandIndex++) {
      Object arg = batchedArgs.get(commandIndex);
      
      if (!(arg instanceof String))
      {
        currentBindValues = batchedParameterValues;
        




        if (previousBindValues != null) {
          for (int j = 0; j < parameterBindings.length; j++) {
            if (bufferType != bufferType) {
              return false;
            }
          }
        }
      }
    }
    


    return true;
  }
  
  private int locationOfOnDuplicateKeyUpdate = -2;
  
  protected synchronized int getLocationOfOnDuplicateKeyUpdate() {
    if (locationOfOnDuplicateKeyUpdate == -2) {
      locationOfOnDuplicateKeyUpdate = getOnDuplicateKeyLocation(originalSql);
    }
    
    return locationOfOnDuplicateKeyUpdate;
  }
  
  protected synchronized boolean isOnDuplicateKeyUpdate() {
    return getLocationOfOnDuplicateKeyUpdate() != -1;
  }
  






  protected long[] computeMaxParameterSetSizeAndBatchSize(int numBatchedArgs)
  {
    long sizeOfEntireBatch = 10L;
    long maxSizeOfParameterSet = 0L;
    
    for (int i = 0; i < numBatchedArgs; i++) {
      BindValue[] paramArg = batchedArgs.get(i)).batchedParameterValues;
      
      long sizeOfParameterSet = 0L;
      
      sizeOfParameterSet += (parameterCount + 7) / 8;
      
      sizeOfParameterSet += parameterCount * 2;
      
      for (int j = 0; j < parameterBindings.length; j++) {
        if (!isNull)
        {
          long size = paramArg[j].getBoundLength();
          
          if (isLongData) {
            if (size != -1L) {
              sizeOfParameterSet += size;
            }
          } else {
            sizeOfParameterSet += size;
          }
        }
      }
      
      sizeOfEntireBatch += sizeOfParameterSet;
      
      if (sizeOfParameterSet > maxSizeOfParameterSet) {
        maxSizeOfParameterSet = sizeOfParameterSet;
      }
    }
    
    return new long[] { maxSizeOfParameterSet, sizeOfEntireBatch };
  }
  
  protected int setOneBatchedParameterSet(java.sql.PreparedStatement batchedStatement, int batchedParamIndex, Object paramSet)
    throws SQLException
  {
    BindValue[] paramArg = batchedParameterValues;
    
    for (int j = 0; j < paramArg.length; j++) {
      if (isNull) {
        batchedStatement.setNull(batchedParamIndex++, 0);
      }
      else if (isLongData) {
        Object value = value;
        
        if ((value instanceof InputStream)) {
          batchedStatement.setBinaryStream(batchedParamIndex++, (InputStream)value, (int)bindLength);
        }
        else
        {
          batchedStatement.setCharacterStream(batchedParamIndex++, (Reader)value, (int)bindLength);
        }
        
      }
      else
      {
        switch (bufferType)
        {
        case 1: 
          batchedStatement.setByte(batchedParamIndex++, byteBinding);
          
          break;
        case 2: 
          batchedStatement.setShort(batchedParamIndex++, shortBinding);
          
          break;
        case 3: 
          batchedStatement.setInt(batchedParamIndex++, intBinding);
          
          break;
        case 8: 
          batchedStatement.setLong(batchedParamIndex++, longBinding);
          
          break;
        case 4: 
          batchedStatement.setFloat(batchedParamIndex++, floatBinding);
          
          break;
        case 5: 
          batchedStatement.setDouble(batchedParamIndex++, doubleBinding);
          
          break;
        case 11: 
          batchedStatement.setTime(batchedParamIndex++, (Time)value);
          
          break;
        case 10: 
          batchedStatement.setDate(batchedParamIndex++, (java.sql.Date)value);
          
          break;
        case 7: 
        case 12: 
          batchedStatement.setTimestamp(batchedParamIndex++, (Timestamp)value);
          
          break;
        case 0: 
        case 15: 
        case 246: 
        case 253: 
        case 254: 
          Object value = value;
          
          if ((value instanceof byte[])) {
            batchedStatement.setBytes(batchedParamIndex, (byte[])value);
          }
          else {
            batchedStatement.setString(batchedParamIndex, (String)value);
          }
          



          if ((batchedStatement instanceof ServerPreparedStatement)) {
            BindValue asBound = ((ServerPreparedStatement)batchedStatement).getBinding(batchedParamIndex, false);
            


            bufferType = bufferType;
          }
          
          batchedParamIndex++;
          
          break;
        default: 
          throw new IllegalArgumentException("Unknown type when re-binding parameter into batched statement for parameter index " + batchedParamIndex);
        }
        
      }
    }
    


    return batchedParamIndex;
  }
  
  protected boolean containsOnDuplicateKeyUpdateInSQL() {
    return hasOnDuplicateKeyUpdate;
  }
  
  protected PreparedStatement prepareBatchedInsertSQL(MySQLConnection localConn, int numBatches) throws SQLException {
    try {
      PreparedStatement pstmt = new ServerPreparedStatement(localConn, parseInfo.getSqlForBatch(numBatches), currentCatalog, resultSetConcurrency, resultSetType);
      pstmt.setRetrieveGeneratedKeys(retrieveGeneratedKeys);
      
      return pstmt;
    } catch (UnsupportedEncodingException e) {
      SQLException sqlEx = SQLError.createSQLException("Unable to prepare batch statement", "S1000", getExceptionInterceptor());
      sqlEx.initCause(e);
      
      throw sqlEx;
    }
  }
}
