package com.mysql.jdbc;

import com.mysql.jdbc.exceptions.MySQLStatementCancelledException;
import com.mysql.jdbc.exceptions.MySQLTimeoutException;
import com.mysql.jdbc.profiler.ProfilerEvent;
import com.mysql.jdbc.profiler.ProfilerEventHandler;
import java.io.InputStream;
import java.math.BigInteger;
import java.sql.BatchUpdateException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.Timer;
import java.util.TimerTask;











































public class StatementImpl
  implements Statement
{
  protected static final String PING_MARKER = "/* ping */";
  
  class CancelTask
    extends TimerTask
  {
    long connectionId = 0L;
    SQLException caughtWhileCancelling = null;
    StatementImpl toCancel;
    
    CancelTask(StatementImpl cancellee) throws SQLException {
      connectionId = connection.getIO().getThreadId();
      toCancel = cancellee;
    }
    
    public void run()
    {
      Thread cancelThread = new Thread()
      {
        public void run() {
          if (connection.getQueryTimeoutKillsConnection()) {
            try {
              toCancel.wasCancelled = true;
              toCancel.wasCancelledByTimeout = true;
              connection.realClose(false, false, true, new MySQLStatementCancelledException(Messages.getString("Statement.ConnectionKilledDueToTimeout")));

            }
            catch (NullPointerException npe) {}catch (SQLException sqlEx)
            {
              caughtWhileCancelling = sqlEx;
            }
          } else {
            Connection cancelConn = null;
            java.sql.Statement cancelStmt = null;
            try
            {
              synchronized (cancelTimeoutMutex) {
                cancelConn = connection.duplicate();
                cancelStmt = cancelConn.createStatement();
                cancelStmt.execute("KILL QUERY " + connectionId);
                toCancel.wasCancelled = true;
                toCancel.wasCancelledByTimeout = true;
              }
            } catch (SQLException sqlEx) {
              caughtWhileCancelling = sqlEx;



            }
            catch (NullPointerException npe) {}finally
            {


              if (cancelStmt != null) {
                try {
                  cancelStmt.close();
                } catch (SQLException sqlEx) {
                  throw new RuntimeException(sqlEx.toString());
                }
              }
              
              if (cancelConn != null) {
                try {
                  cancelConn.close();
                } catch (SQLException sqlEx) {
                  throw new RuntimeException(sqlEx.toString());
                }
              }
              
              toCancel = null;
            }
            
          }
        }
      };
      cancelThread.start();
    }
  }
  



  protected Object cancelTimeoutMutex = new Object();
  

  protected static int statementCounter = 1;
  
  public static final byte USES_VARIABLES_FALSE = 0;
  
  public static final byte USES_VARIABLES_TRUE = 1;
  
  public static final byte USES_VARIABLES_UNKNOWN = -1;
  
  protected boolean wasCancelled = false;
  protected boolean wasCancelledByTimeout = false;
  

  protected List batchedArgs;
  

  protected SingleByteCharsetConverter charConverter = null;
  

  protected String charEncoding = null;
  

  protected MySQLConnection connection = null;
  
  protected long connectionId = 0L;
  

  protected String currentCatalog = null;
  

  protected boolean doEscapeProcessing = true;
  

  protected ProfilerEventHandler eventSink = null;
  

  private int fetchSize = 0;
  

  protected boolean isClosed = false;
  

  protected long lastInsertId = -1L;
  

  protected int maxFieldSize = MysqlIO.getMaxBuf();
  




  protected int maxRows = -1;
  

  protected boolean maxRowsChanged = false;
  

  protected Set openResults = new HashSet();
  

  protected boolean pedantic = false;
  



  protected Throwable pointOfOrigin;
  


  protected boolean profileSQL = false;
  

  protected ResultSetInternalMethods results = null;
  

  protected int resultSetConcurrency = 0;
  

  protected int resultSetType = 0;
  

  protected int statementId;
  

  protected int timeoutInMillis = 0;
  

  protected long updateCount = -1L;
  

  protected boolean useUsageAdvisor = false;
  

  protected SQLWarning warningChain = null;
  




  protected boolean holdResultsOpenOverClose = false;
  
  protected ArrayList batchedGeneratedKeys = null;
  
  protected boolean retrieveGeneratedKeys = false;
  
  protected boolean continueBatchOnError = false;
  
  protected PingTarget pingTarget = null;
  

  protected boolean useLegacyDatetimeCode;
  
  private ExceptionInterceptor exceptionInterceptor;
  
  protected boolean lastQueryIsOnDupKeyUpdate = false;
  









  public StatementImpl(MySQLConnection c, String catalog)
    throws SQLException
  {
    if ((c == null) || (c.isClosed())) {
      throw SQLError.createSQLException(Messages.getString("Statement.0"), "08003", null);
    }
    


    connection = c;
    connectionId = connection.getId();
    exceptionInterceptor = connection.getExceptionInterceptor();
    

    currentCatalog = catalog;
    pedantic = connection.getPedantic();
    continueBatchOnError = connection.getContinueBatchOnError();
    useLegacyDatetimeCode = connection.getUseLegacyDatetimeCode();
    
    if (!connection.getDontTrackOpenResources()) {
      connection.registerStatement(this);
    }
    




    if (connection != null) {
      maxFieldSize = connection.getMaxAllowedPacket();
      
      int defaultFetchSize = connection.getDefaultFetchSize();
      
      if (defaultFetchSize != 0) {
        setFetchSize(defaultFetchSize);
      }
    }
    
    if (connection.getUseUnicode()) {
      charEncoding = connection.getEncoding();
      
      charConverter = connection.getCharsetConverter(charEncoding);
    }
    
    boolean profiling = (connection.getProfileSql()) || (connection.getUseUsageAdvisor()) || (connection.getLogSlowQueries());
    

    if ((connection.getAutoGenerateTestcaseScript()) || (profiling)) {
      statementId = (statementCounter++);
    }
    
    if (profiling) {
      pointOfOrigin = new Throwable();
      profileSQL = connection.getProfileSql();
      useUsageAdvisor = connection.getUseUsageAdvisor();
      eventSink = ProfilerEventHandlerFactory.getInstance(connection);
    }
    
    int maxRowsConn = connection.getMaxRows();
    
    if (maxRowsConn != -1) {
      setMaxRows(maxRowsConn);
    }
    
    holdResultsOpenOverClose = connection.getHoldResultsOpenOverStatementClose();
  }
  







  public synchronized void addBatch(String sql)
    throws SQLException
  {
    if (batchedArgs == null) {
      batchedArgs = new ArrayList();
    }
    
    if (sql != null) {
      batchedArgs.add(sql);
    }
  }
  



  public void cancel()
    throws SQLException
  {
    if ((!isClosed) && (connection != null) && (connection.versionMeetsMinimum(5, 0, 0)))
    {

      Connection cancelConn = null;
      java.sql.Statement cancelStmt = null;
      try
      {
        cancelConn = connection.duplicate();
        cancelStmt = cancelConn.createStatement();
        cancelStmt.execute("KILL QUERY " + connection.getIO().getThreadId());
        
        wasCancelled = true;
      } finally {
        if (cancelStmt != null) {
          cancelStmt.close();
        }
        
        if (cancelConn != null) {
          cancelConn.close();
        }
      }
    }
  }
  







  protected void checkClosed()
    throws SQLException
  {
    if (isClosed) {
      throw SQLError.createSQLException(Messages.getString("Statement.49"), "08003", getExceptionInterceptor());
    }
  }
  













  protected void checkForDml(String sql, char firstStatementChar)
    throws SQLException
  {
    if ((firstStatementChar == 'I') || (firstStatementChar == 'U') || (firstStatementChar == 'D') || (firstStatementChar == 'A') || (firstStatementChar == 'C'))
    {

      String noCommentSql = StringUtils.stripComments(sql, "'\"", "'\"", true, false, true, true);
      

      if ((StringUtils.startsWithIgnoreCaseAndWs(noCommentSql, "INSERT")) || (StringUtils.startsWithIgnoreCaseAndWs(noCommentSql, "UPDATE")) || (StringUtils.startsWithIgnoreCaseAndWs(noCommentSql, "DELETE")) || (StringUtils.startsWithIgnoreCaseAndWs(noCommentSql, "DROP")) || (StringUtils.startsWithIgnoreCaseAndWs(noCommentSql, "CREATE")) || (StringUtils.startsWithIgnoreCaseAndWs(noCommentSql, "ALTER")))
      {




        throw SQLError.createSQLException(Messages.getString("Statement.57"), "S1009", getExceptionInterceptor());
      }
    }
  }
  









  protected void checkNullOrEmptyQuery(String sql)
    throws SQLException
  {
    if (sql == null) {
      throw SQLError.createSQLException(Messages.getString("Statement.59"), "S1009", getExceptionInterceptor());
    }
    


    if (sql.length() == 0) {
      throw SQLError.createSQLException(Messages.getString("Statement.61"), "S1009", getExceptionInterceptor());
    }
  }
  








  public synchronized void clearBatch()
    throws SQLException
  {
    if (batchedArgs != null) {
      batchedArgs.clear();
    }
  }
  





  public void clearWarnings()
    throws SQLException
  {
    warningChain = null;
  }
  













  public synchronized void close()
    throws SQLException
  {
    realClose(true, true);
  }
  


  protected synchronized void closeAllOpenResults()
  {
    if (openResults != null) {
      for (Iterator iter = openResults.iterator(); iter.hasNext();) {
        ResultSetInternalMethods element = (ResultSetInternalMethods)iter.next();
        try
        {
          element.realClose(false);
        } catch (SQLException sqlEx) {
          AssertionFailedException.shouldNotHappen(sqlEx);
        }
      }
      
      openResults.clear();
    }
  }
  
  public synchronized void removeOpenResultSet(ResultSet rs) {
    if (openResults != null) {
      openResults.remove(rs);
    }
  }
  
  public synchronized int getOpenResultSetCount() {
    if (openResults != null) {
      return openResults.size();
    }
    
    return 0;
  }
  



  private ResultSetInternalMethods createResultSetUsingServerFetch(String sql)
    throws SQLException
  {
    java.sql.PreparedStatement pStmt = connection.prepareStatement(sql, resultSetType, resultSetConcurrency);
    

    pStmt.setFetchSize(fetchSize);
    
    if (maxRows > -1) {
      pStmt.setMaxRows(maxRows);
    }
    
    pStmt.execute();
    




    ResultSetInternalMethods rs = ((StatementImpl)pStmt).getResultSetInternal();
    

    rs.setStatementUsedForFetchingRows((PreparedStatement)pStmt);
    

    results = rs;
    
    return rs;
  }
  






  protected boolean createStreamingResultSet()
  {
    return (resultSetType == 1003) && (resultSetConcurrency == 1007) && (fetchSize == -2147483648);
  }
  

  private int originalResultSetType = 0;
  private int originalFetchSize = 0;
  

  public void enableStreamingResults()
    throws SQLException
  {
    originalResultSetType = resultSetType;
    originalFetchSize = fetchSize;
    
    setFetchSize(-2147483648);
    setResultSetType(1003);
  }
  
  public void disableStreamingResults() throws SQLException {
    if ((fetchSize == -2147483648) && (resultSetType == 1003))
    {
      setFetchSize(originalFetchSize);
      setResultSetType(originalResultSetType);
    }
  }
  












  public boolean execute(String sql)
    throws SQLException
  {
    return execute(sql, false);
  }
  
  private boolean execute(String sql, boolean returnGeneratedKeys) throws SQLException {
    checkClosed();
    
    MySQLConnection locallyScopedConn = connection;
    
    synchronized (locallyScopedConn.getMutex()) {
      retrieveGeneratedKeys = returnGeneratedKeys;
      lastQueryIsOnDupKeyUpdate = false;
      if (returnGeneratedKeys) {
        lastQueryIsOnDupKeyUpdate = containsOnDuplicateKeyInString(sql);
      }
      resetCancelledState();
      
      checkNullOrEmptyQuery(sql);
      
      checkClosed();
      
      char firstNonWsChar = StringUtils.firstAlphaCharUc(sql, findStartOfStatement(sql));
      
      boolean isSelect = true;
      
      if (firstNonWsChar != 'S') {
        isSelect = false;
        
        if (locallyScopedConn.isReadOnly()) {
          throw SQLError.createSQLException(Messages.getString("Statement.27") + Messages.getString("Statement.28"), "S1009", getExceptionInterceptor());
        }
      }
      



      boolean doStreaming = createStreamingResultSet();
      








      if ((doStreaming) && (locallyScopedConn.getNetTimeoutForStreamingResults() > 0))
      {
        executeSimpleNonQuery(locallyScopedConn, "SET net_write_timeout=" + locallyScopedConn.getNetTimeoutForStreamingResults());
      }
      

      if (doEscapeProcessing) {
        Object escapedSqlResult = EscapeProcessor.escapeSQL(sql, locallyScopedConn.serverSupportsConvertFn(), locallyScopedConn);
        

        if ((escapedSqlResult instanceof String)) {
          sql = (String)escapedSqlResult;
        } else {
          sql = escapedSql;
        }
      }
      
      if ((results != null) && 
        (!locallyScopedConn.getHoldResultsOpenOverStatementClose())) {
        results.realClose(false);
      }
      

      if ((sql.charAt(0) == '/') && 
        (sql.startsWith("/* ping */"))) {
        doPingInstead();
        
        return true;
      }
      

      CachedResultSetMetaData cachedMetaData = null;
      
      ResultSetInternalMethods rs = null;
      







      batchedGeneratedKeys = null;
      
      if (useServerFetch()) {
        rs = createResultSetUsingServerFetch(sql);
      } else {
        CancelTask timeoutTask = null;
        
        String oldCatalog = null;
        try
        {
          if ((locallyScopedConn.getEnableQueryTimeouts()) && (timeoutInMillis != 0) && (locallyScopedConn.versionMeetsMinimum(5, 0, 0)))
          {

            timeoutTask = new CancelTask(this);
            locallyScopedConn.getCancelTimer().schedule(timeoutTask, timeoutInMillis);
          }
          



          if (!locallyScopedConn.getCatalog().equals(currentCatalog))
          {
            oldCatalog = locallyScopedConn.getCatalog();
            locallyScopedConn.setCatalog(currentCatalog);
          }
          




          Field[] cachedFields = null;
          
          if (locallyScopedConn.getCacheResultSetMetadata()) {
            cachedMetaData = locallyScopedConn.getCachedMetaData(sql);
            
            if (cachedMetaData != null) {
              cachedFields = fields;
            }
          }
          



          if (locallyScopedConn.useMaxRows()) {
            int rowLimit = -1;
            
            if (isSelect) {
              if (StringUtils.indexOfIgnoreCase(sql, "LIMIT") != -1) {
                rowLimit = maxRows;
              }
              else if (maxRows <= 0) {
                executeSimpleNonQuery(locallyScopedConn, "SET OPTION SQL_SELECT_LIMIT=DEFAULT");
              }
              else {
                executeSimpleNonQuery(locallyScopedConn, "SET OPTION SQL_SELECT_LIMIT=" + maxRows);
              }
              

            }
            else {
              executeSimpleNonQuery(locallyScopedConn, "SET OPTION SQL_SELECT_LIMIT=DEFAULT");
            }
            


            rs = locallyScopedConn.execSQL(this, sql, rowLimit, null, resultSetType, resultSetConcurrency, doStreaming, currentCatalog, cachedFields);

          }
          else
          {
            rs = locallyScopedConn.execSQL(this, sql, -1, null, resultSetType, resultSetConcurrency, doStreaming, currentCatalog, cachedFields);
          }
          



          if (timeoutTask != null) {
            if (caughtWhileCancelling != null) {
              throw caughtWhileCancelling;
            }
            
            timeoutTask.cancel();
            timeoutTask = null;
          }
          
          synchronized (cancelTimeoutMutex) {
            if (wasCancelled) {
              SQLException cause = null;
              
              if (wasCancelledByTimeout) {
                cause = new MySQLTimeoutException();
              } else {
                cause = new MySQLStatementCancelledException();
              }
              
              resetCancelledState();
              
              throw cause;
            }
          }
        } finally {
          if (timeoutTask != null) {
            timeoutTask.cancel();
            locallyScopedConn.getCancelTimer().purge();
          }
          
          if (oldCatalog != null) {
            locallyScopedConn.setCatalog(oldCatalog);
          }
        }
      }
      
      if (rs != null) {
        lastInsertId = rs.getUpdateID();
        
        results = rs;
        
        rs.setFirstCharOfQuery(firstNonWsChar);
        
        if (rs.reallyResult()) {
          if (cachedMetaData != null) {
            locallyScopedConn.initializeResultsMetadataFromCache(sql, cachedMetaData, results);

          }
          else if (connection.getCacheResultSetMetadata()) {
            locallyScopedConn.initializeResultsMetadataFromCache(sql, null, results);
          }
        }
      }
      


      return (rs != null) && (rs.reallyResult());
    }
  }
  
  protected synchronized void resetCancelledState() {
    if (cancelTimeoutMutex == null) {
      return;
    }
    
    synchronized (cancelTimeoutMutex) {
      wasCancelled = false;
      wasCancelledByTimeout = false;
    }
  }
  




  public boolean execute(String sql, int returnGeneratedKeys)
    throws SQLException
  {
    if (returnGeneratedKeys == 1) {
      checkClosed();
      
      MySQLConnection locallyScopedConn = connection;
      
      synchronized (locallyScopedConn.getMutex())
      {


        boolean readInfoMsgState = connection.isReadInfoMsgEnabled();
        
        locallyScopedConn.setReadInfoMsgEnabled(true);
        try
        {
          boolean bool1 = execute(sql, true);jsr 17;return bool1;
        } finally {
          jsr 6; } localObject2 = returnAddress;locallyScopedConn.setReadInfoMsgEnabled(readInfoMsgState);ret;
      }
    }
    

    return execute(sql);
  }
  


  public boolean execute(String sql, int[] generatedKeyIndices)
    throws SQLException
  {
    if ((generatedKeyIndices != null) && (generatedKeyIndices.length > 0)) {
      checkClosed();
      
      MySQLConnection locallyScopedConn = connection;
      
      synchronized (locallyScopedConn.getMutex()) {
        retrieveGeneratedKeys = true;
        



        boolean readInfoMsgState = locallyScopedConn.isReadInfoMsgEnabled();
        
        locallyScopedConn.setReadInfoMsgEnabled(true);
        try
        {
          boolean bool1 = execute(sql, true);jsr 17;return bool1;
        } finally {
          jsr 6; } localObject2 = returnAddress;locallyScopedConn.setReadInfoMsgEnabled(readInfoMsgState);ret;
      }
    }
    

    return execute(sql);
  }
  


  public boolean execute(String sql, String[] generatedKeyNames)
    throws SQLException
  {
    if ((generatedKeyNames != null) && (generatedKeyNames.length > 0)) {
      checkClosed();
      
      MySQLConnection locallyScopedConn = connection;
      
      synchronized (locallyScopedConn.getMutex()) {
        retrieveGeneratedKeys = true;
        


        boolean readInfoMsgState = connection.isReadInfoMsgEnabled();
        
        locallyScopedConn.setReadInfoMsgEnabled(true);
        try
        {
          boolean bool1 = execute(sql, true);jsr 17;return bool1;
        } finally {
          jsr 6; } localObject2 = returnAddress;locallyScopedConn.setReadInfoMsgEnabled(readInfoMsgState);ret;
      }
    }
    

    return execute(sql);
  }
  












  public synchronized int[] executeBatch()
    throws SQLException
  {
    checkClosed();
    
    MySQLConnection locallyScopedConn = connection;
    
    if (locallyScopedConn.isReadOnly()) {
      throw SQLError.createSQLException(Messages.getString("Statement.34") + Messages.getString("Statement.35"), "S1009", getExceptionInterceptor());
    }
    



    if ((results != null) && 
      (!locallyScopedConn.getHoldResultsOpenOverStatementClose())) {
      results.realClose(false);
    }
    

    synchronized (locallyScopedConn.getMutex()) {
      if ((batchedArgs == null) || (batchedArgs.size() == 0)) {
        return new int[0];
      }
      

      int individualStatementTimeout = timeoutInMillis;
      timeoutInMillis = 0;
      
      CancelTask timeoutTask = null;
      try
      {
        resetCancelledState();
        
        retrieveGeneratedKeys = true;
        
        int[] updateCounts = null;
        

        if (batchedArgs != null) {
          nbrCommands = batchedArgs.size();
          
          batchedGeneratedKeys = new ArrayList(batchedArgs.size());
          
          boolean multiQueriesEnabled = locallyScopedConn.getAllowMultiQueries();
          
          if ((locallyScopedConn.versionMeetsMinimum(4, 1, 1)) && ((multiQueriesEnabled) || ((locallyScopedConn.getRewriteBatchedStatements()) && (nbrCommands > 4))))
          {


            int[] arrayOfInt1 = executeBatchUsingMultiQueries(multiQueriesEnabled, nbrCommands, individualStatementTimeout);jsr 383;return arrayOfInt1;
          }
          
          if ((locallyScopedConn.getEnableQueryTimeouts()) && (individualStatementTimeout != 0) && (locallyScopedConn.versionMeetsMinimum(5, 0, 0)))
          {

            timeoutTask = new CancelTask(this);
            locallyScopedConn.getCancelTimer().schedule(timeoutTask, individualStatementTimeout);
          }
          

          updateCounts = new int[nbrCommands];
          
          for (int i = 0; i < nbrCommands; i++) {
            updateCounts[i] = -3;
          }
          
          SQLException sqlEx = null;
          
          int commandIndex = 0;
          
          for (commandIndex = 0; commandIndex < nbrCommands; commandIndex++) {
            try {
              String sql = (String)batchedArgs.get(commandIndex);
              updateCounts[commandIndex] = executeUpdate(sql, true, true);
              
              getBatchedGeneratedKeys(containsOnDuplicateKeyInString(sql) ? 1 : 0);
            } catch (SQLException ex) {
              updateCounts[commandIndex] = -3;
              
              if ((continueBatchOnError) && (!(ex instanceof MySQLTimeoutException)) && (!(ex instanceof MySQLStatementCancelledException)) && (!hasDeadlockOrTimeoutRolledBackTx(ex)))
              {


                sqlEx = ex;
              } else {
                int[] newUpdateCounts = new int[commandIndex];
                
                if (hasDeadlockOrTimeoutRolledBackTx(ex)) {
                  for (int i = 0; i < newUpdateCounts.length; i++) {
                    newUpdateCounts[i] = -3;
                  }
                } else {
                  System.arraycopy(updateCounts, 0, newUpdateCounts, 0, commandIndex);
                }
                

                throw new BatchUpdateException(ex.getMessage(), ex.getSQLState(), ex.getErrorCode(), newUpdateCounts);
              }
            }
          }
          


          if (sqlEx != null) {
            throw new BatchUpdateException(sqlEx.getMessage(), sqlEx.getSQLState(), sqlEx.getErrorCode(), updateCounts);
          }
        }
        


        if (timeoutTask != null) {
          if (caughtWhileCancelling != null) {
            throw caughtWhileCancelling;
          }
          
          timeoutTask.cancel();
          
          locallyScopedConn.getCancelTimer().purge();
          timeoutTask = null;
        }
        
        int nbrCommands = updateCounts != null ? updateCounts : new int[0];jsr 16;return nbrCommands;
      }
      finally {
        jsr 6; } localObject2 = returnAddress; if (timeoutTask != null) {
        timeoutTask.cancel();
        
        locallyScopedConn.getCancelTimer().purge();
      }
      
      resetCancelledState();
      
      timeoutInMillis = individualStatementTimeout;
      
      clearBatch();ret;
    }
  }
  
  protected final boolean hasDeadlockOrTimeoutRolledBackTx(SQLException ex)
  {
    int vendorCode = ex.getErrorCode();
    
    switch (vendorCode) {
    case 1206: 
    case 1213: 
      return true;
    case 1205: 
      try {
        return !connection.versionMeetsMinimum(5, 0, 13);
      }
      catch (SQLException sqlEx) {
        return false;
      }
    }
    return false;
  }
  









  private int[] executeBatchUsingMultiQueries(boolean multiQueriesEnabled, int nbrCommands, int individualStatementTimeout)
    throws SQLException
  {
    MySQLConnection locallyScopedConn = connection;
    
    if (!multiQueriesEnabled) {
      locallyScopedConn.getIO().enableMultiQueries();
    }
    
    java.sql.Statement batchStmt = null;
    
    CancelTask timeoutTask = null;
    try
    {
      int[] updateCounts = new int[nbrCommands];
      
      for (int i = 0; i < nbrCommands; i++) {
        updateCounts[i] = -3;
      }
      
      int commandIndex = 0;
      
      StringBuffer queryBuf = new StringBuffer();
      
      batchStmt = locallyScopedConn.createStatement();
      
      if ((locallyScopedConn.getEnableQueryTimeouts()) && (individualStatementTimeout != 0) && (locallyScopedConn.versionMeetsMinimum(5, 0, 0)))
      {

        timeoutTask = new CancelTask((StatementImpl)batchStmt);
        locallyScopedConn.getCancelTimer().schedule(timeoutTask, individualStatementTimeout);
      }
      

      int counter = 0;
      
      int numberOfBytesPerChar = 1;
      
      String connectionEncoding = locallyScopedConn.getEncoding();
      
      if (StringUtils.startsWithIgnoreCase(connectionEncoding, "utf")) {
        numberOfBytesPerChar = 3;
      } else if (CharsetMapping.isMultibyteCharset(connectionEncoding)) {
        numberOfBytesPerChar = 2;
      }
      
      int escapeAdjust = 1;
      
      batchStmt.setEscapeProcessing(doEscapeProcessing);
      
      if (doEscapeProcessing)
      {
        escapeAdjust = 2;
      }
      


      SQLException sqlEx = null;
      
      int argumentSetsInBatchSoFar = 0;
      
      for (commandIndex = 0; commandIndex < nbrCommands; commandIndex++) {
        String nextQuery = (String)batchedArgs.get(commandIndex);
        
        if (((queryBuf.length() + nextQuery.length()) * numberOfBytesPerChar + 1 + 4) * escapeAdjust + 32 > connection.getMaxAllowedPacket())
        {

          try
          {
            batchStmt.execute(queryBuf.toString(), 1);
          } catch (SQLException ex) {
            sqlEx = handleExceptionForBatch(commandIndex, argumentSetsInBatchSoFar, updateCounts, ex);
          }
          

          counter = processMultiCountsAndKeys((StatementImpl)batchStmt, counter, updateCounts);
          

          queryBuf = new StringBuffer();
          argumentSetsInBatchSoFar = 0;
        }
        
        queryBuf.append(nextQuery);
        queryBuf.append(";");
        argumentSetsInBatchSoFar++;
      }
      
      if (queryBuf.length() > 0) {
        try {
          batchStmt.execute(queryBuf.toString(), 1);
        } catch (SQLException ex) {
          sqlEx = handleExceptionForBatch(commandIndex - 1, argumentSetsInBatchSoFar, updateCounts, ex);
        }
        

        counter = processMultiCountsAndKeys((StatementImpl)batchStmt, counter, updateCounts);
      }
      

      if (timeoutTask != null) {
        if (caughtWhileCancelling != null) {
          throw caughtWhileCancelling;
        }
        
        timeoutTask.cancel();
        
        locallyScopedConn.getCancelTimer().purge();
        
        timeoutTask = null;
      }
      
      if (sqlEx != null) {
        throw new BatchUpdateException(sqlEx.getMessage(), sqlEx.getSQLState(), sqlEx.getErrorCode(), updateCounts);
      }
      


      return updateCounts != null ? updateCounts : new int[0];
    } finally {
      if (timeoutTask != null) {
        timeoutTask.cancel();
        
        locallyScopedConn.getCancelTimer().purge();
      }
      
      resetCancelledState();
    }
    





    ret;
  }
  



  protected int processMultiCountsAndKeys(StatementImpl batchedStatement, int updateCountCounter, int[] updateCounts)
    throws SQLException
  {
    updateCounts[(updateCountCounter++)] = batchedStatement.getUpdateCount();
    
    boolean doGenKeys = batchedGeneratedKeys != null;
    
    byte[][] row = (byte[][])null;
    
    if (doGenKeys) {
      long generatedKey = batchedStatement.getLastInsertID();
      
      row = new byte[1][];
      row[0] = Long.toString(generatedKey).getBytes();
      batchedGeneratedKeys.add(new ByteArrayRow(row, getExceptionInterceptor()));
    }
    

    while ((batchedStatement.getMoreResults()) || (batchedStatement.getUpdateCount() != -1)) {
      updateCounts[(updateCountCounter++)] = batchedStatement.getUpdateCount();
      
      if (doGenKeys) {
        long generatedKey = batchedStatement.getLastInsertID();
        
        row = new byte[1][];
        row[0] = Long.toString(generatedKey).getBytes();
        batchedGeneratedKeys.add(new ByteArrayRow(row, getExceptionInterceptor()));
      }
    }
    
    return updateCountCounter;
  }
  


  protected SQLException handleExceptionForBatch(int endOfBatchIndex, int numValuesPerBatch, int[] updateCounts, SQLException ex)
    throws BatchUpdateException
  {
    for (int j = endOfBatchIndex; j > endOfBatchIndex - numValuesPerBatch; j--) {
      updateCounts[j] = -3;
    }
    SQLException sqlEx;
    if ((continueBatchOnError) && (!(ex instanceof MySQLTimeoutException)) && (!(ex instanceof MySQLStatementCancelledException)) && (!hasDeadlockOrTimeoutRolledBackTx(ex)))
    {


      sqlEx = ex;
    } else {
      int[] newUpdateCounts = new int[endOfBatchIndex];
      System.arraycopy(updateCounts, 0, newUpdateCounts, 0, endOfBatchIndex);
      

      throw new BatchUpdateException(ex.getMessage(), ex.getSQLState(), ex.getErrorCode(), newUpdateCounts);
    }
    
    SQLException sqlEx;
    
    return sqlEx;
  }
  










  public ResultSet executeQuery(String sql)
    throws SQLException
  {
    checkClosed();
    
    MySQLConnection locallyScopedConn = connection;
    
    synchronized (locallyScopedConn.getMutex()) {
      retrieveGeneratedKeys = false;
      
      resetCancelledState();
      
      checkNullOrEmptyQuery(sql);
      
      boolean doStreaming = createStreamingResultSet();
      








      if ((doStreaming) && (connection.getNetTimeoutForStreamingResults() > 0))
      {
        executeSimpleNonQuery(locallyScopedConn, "SET net_write_timeout=" + connection.getNetTimeoutForStreamingResults());
      }
      

      if (doEscapeProcessing) {
        Object escapedSqlResult = EscapeProcessor.escapeSQL(sql, locallyScopedConn.serverSupportsConvertFn(), connection);
        

        if ((escapedSqlResult instanceof String)) {
          sql = (String)escapedSqlResult;
        } else {
          sql = escapedSql;
        }
      }
      
      char firstStatementChar = StringUtils.firstNonWsCharUc(sql, findStartOfStatement(sql));
      

      if ((sql.charAt(0) == '/') && 
        (sql.startsWith("/* ping */"))) {
        doPingInstead();
        
        return results;
      }
      

      checkForDml(sql, firstStatementChar);
      
      if ((results != null) && 
        (!locallyScopedConn.getHoldResultsOpenOverStatementClose())) {
        results.realClose(false);
      }
      

      CachedResultSetMetaData cachedMetaData = null;
      







      if (useServerFetch()) {
        results = createResultSetUsingServerFetch(sql);
        
        return results;
      }
      
      CancelTask timeoutTask = null;
      
      String oldCatalog = null;
      try
      {
        if ((locallyScopedConn.getEnableQueryTimeouts()) && (timeoutInMillis != 0) && (locallyScopedConn.versionMeetsMinimum(5, 0, 0)))
        {

          timeoutTask = new CancelTask(this);
          locallyScopedConn.getCancelTimer().schedule(timeoutTask, timeoutInMillis);
        }
        

        if (!locallyScopedConn.getCatalog().equals(currentCatalog)) {
          oldCatalog = locallyScopedConn.getCatalog();
          locallyScopedConn.setCatalog(currentCatalog);
        }
        




        Field[] cachedFields = null;
        
        if (locallyScopedConn.getCacheResultSetMetadata()) {
          cachedMetaData = locallyScopedConn.getCachedMetaData(sql);
          
          if (cachedMetaData != null) {
            cachedFields = fields;
          }
        }
        
        if (locallyScopedConn.useMaxRows())
        {



          if (StringUtils.indexOfIgnoreCase(sql, "LIMIT") != -1) {
            results = locallyScopedConn.execSQL(this, sql, maxRows, null, resultSetType, resultSetConcurrency, doStreaming, currentCatalog, cachedFields);

          }
          else
          {

            if (maxRows <= 0) {
              executeSimpleNonQuery(locallyScopedConn, "SET OPTION SQL_SELECT_LIMIT=DEFAULT");
            }
            else {
              executeSimpleNonQuery(locallyScopedConn, "SET OPTION SQL_SELECT_LIMIT=" + maxRows);
            }
            

            results = locallyScopedConn.execSQL(this, sql, -1, null, resultSetType, resultSetConcurrency, doStreaming, currentCatalog, cachedFields);
            




            if (oldCatalog != null) {
              locallyScopedConn.setCatalog(oldCatalog);
            }
          }
        } else {
          results = locallyScopedConn.execSQL(this, sql, -1, null, resultSetType, resultSetConcurrency, doStreaming, currentCatalog, cachedFields);
        }
        



        if (timeoutTask != null) {
          if (caughtWhileCancelling != null) {
            throw caughtWhileCancelling;
          }
          
          timeoutTask.cancel();
          
          locallyScopedConn.getCancelTimer().purge();
          
          timeoutTask = null;
        }
        
        synchronized (cancelTimeoutMutex) {
          if (wasCancelled) {
            SQLException cause = null;
            
            if (wasCancelledByTimeout) {
              cause = new MySQLTimeoutException();
            } else {
              cause = new MySQLStatementCancelledException();
            }
            
            resetCancelledState();
            
            throw cause;
          }
        }
      } finally {
        if (timeoutTask != null) {
          timeoutTask.cancel();
          
          locallyScopedConn.getCancelTimer().purge();
        }
        
        if (oldCatalog != null) {
          locallyScopedConn.setCatalog(oldCatalog);
        }
      }
      
      lastInsertId = results.getUpdateID();
      
      if (cachedMetaData != null) {
        locallyScopedConn.initializeResultsMetadataFromCache(sql, cachedMetaData, results);

      }
      else if (connection.getCacheResultSetMetadata()) {
        locallyScopedConn.initializeResultsMetadataFromCache(sql, null, results);
      }
      


      return results;
    }
  }
  
  protected void doPingInstead() throws SQLException {
    if (pingTarget != null) {
      pingTarget.doPing();
    } else {
      connection.ping();
    }
    
    ResultSetInternalMethods fakeSelectOneResultSet = generatePingResultSet();
    results = fakeSelectOneResultSet;
  }
  
  protected ResultSetInternalMethods generatePingResultSet() throws SQLException {
    Field[] fields = { new Field(null, "1", -5, 1) };
    ArrayList rows = new ArrayList();
    byte[] colVal = { 49 };
    
    rows.add(new ByteArrayRow(new byte[][] { colVal }, getExceptionInterceptor()));
    
    return (ResultSetInternalMethods)DatabaseMetaData.buildResultSet(fields, rows, connection);
  }
  
  protected void executeSimpleNonQuery(MySQLConnection c, String nonQuery)
    throws SQLException
  {
    c.execSQL(this, nonQuery, -1, null, 1003, 1007, false, currentCatalog, null, false).close();
  }
  
















  public int executeUpdate(String sql)
    throws SQLException
  {
    return executeUpdate(sql, false, false);
  }
  
  protected int executeUpdate(String sql, boolean isBatch, boolean returnGeneratedKeys) throws SQLException
  {
    checkClosed();
    
    MySQLConnection locallyScopedConn = connection;
    
    char firstStatementChar = StringUtils.firstAlphaCharUc(sql, findStartOfStatement(sql));
    

    ResultSetInternalMethods rs = null;
    
    synchronized (locallyScopedConn.getMutex()) {
      retrieveGeneratedKeys = returnGeneratedKeys;
      
      resetCancelledState();
      
      checkNullOrEmptyQuery(sql);
      
      if (doEscapeProcessing) {
        Object escapedSqlResult = EscapeProcessor.escapeSQL(sql, connection.serverSupportsConvertFn(), connection);
        

        if ((escapedSqlResult instanceof String)) {
          sql = (String)escapedSqlResult;
        } else {
          sql = escapedSql;
        }
      }
      
      if (locallyScopedConn.isReadOnly()) {
        throw SQLError.createSQLException(Messages.getString("Statement.42") + Messages.getString("Statement.43"), "S1009", getExceptionInterceptor());
      }
      



      if (StringUtils.startsWithIgnoreCaseAndWs(sql, "select")) {
        throw SQLError.createSQLException(Messages.getString("Statement.46"), "01S03", getExceptionInterceptor());
      }
      


      if ((results != null) && 
        (!locallyScopedConn.getHoldResultsOpenOverStatementClose())) {
        results.realClose(false);
      }
      





      CancelTask timeoutTask = null;
      
      String oldCatalog = null;
      try
      {
        if ((locallyScopedConn.getEnableQueryTimeouts()) && (timeoutInMillis != 0) && (locallyScopedConn.versionMeetsMinimum(5, 0, 0)))
        {

          timeoutTask = new CancelTask(this);
          locallyScopedConn.getCancelTimer().schedule(timeoutTask, timeoutInMillis);
        }
        

        if (!locallyScopedConn.getCatalog().equals(currentCatalog)) {
          oldCatalog = locallyScopedConn.getCatalog();
          locallyScopedConn.setCatalog(currentCatalog);
        }
        



        if (locallyScopedConn.useMaxRows()) {
          executeSimpleNonQuery(locallyScopedConn, "SET OPTION SQL_SELECT_LIMIT=DEFAULT");
        }
        

        rs = locallyScopedConn.execSQL(this, sql, -1, null, 1003, 1007, false, currentCatalog, null, isBatch);
        





        if (timeoutTask != null) {
          if (caughtWhileCancelling != null) {
            throw caughtWhileCancelling;
          }
          
          timeoutTask.cancel();
          
          locallyScopedConn.getCancelTimer().purge();
          
          timeoutTask = null;
        }
        
        synchronized (cancelTimeoutMutex) {
          if (wasCancelled) {
            SQLException cause = null;
            
            if (wasCancelledByTimeout) {
              cause = new MySQLTimeoutException();
            } else {
              cause = new MySQLStatementCancelledException();
            }
            
            resetCancelledState();
            
            throw cause;
          }
        }
      } finally {
        if (timeoutTask != null) {
          timeoutTask.cancel();
          
          locallyScopedConn.getCancelTimer().purge();
        }
        
        if (oldCatalog != null) {
          locallyScopedConn.setCatalog(oldCatalog);
        }
      }
    }
    
    results = rs;
    
    rs.setFirstCharOfQuery(firstStatementChar);
    
    updateCount = rs.getUpdateCount();
    
    int truncatedUpdateCount = 0;
    
    if (updateCount > 2147483647L) {
      truncatedUpdateCount = 2147483647;
    } else {
      truncatedUpdateCount = (int)updateCount;
    }
    
    lastInsertId = rs.getUpdateID();
    
    return truncatedUpdateCount;
  }
  



  public int executeUpdate(String sql, int returnGeneratedKeys)
    throws SQLException
  {
    if (returnGeneratedKeys == 1) {
      checkClosed();
      
      MySQLConnection locallyScopedConn = connection;
      
      synchronized (locallyScopedConn.getMutex())
      {


        boolean readInfoMsgState = locallyScopedConn.isReadInfoMsgEnabled();
        
        locallyScopedConn.setReadInfoMsgEnabled(true);
        try
        {
          int i = executeUpdate(sql, false, true);jsr 17;return i;
        } finally {
          jsr 6; } localObject2 = returnAddress;locallyScopedConn.setReadInfoMsgEnabled(readInfoMsgState);ret;
      }
    }
    

    return executeUpdate(sql);
  }
  


  public int executeUpdate(String sql, int[] generatedKeyIndices)
    throws SQLException
  {
    if ((generatedKeyIndices != null) && (generatedKeyIndices.length > 0)) {
      checkClosed();
      
      MySQLConnection locallyScopedConn = connection;
      
      synchronized (locallyScopedConn.getMutex())
      {


        boolean readInfoMsgState = locallyScopedConn.isReadInfoMsgEnabled();
        
        locallyScopedConn.setReadInfoMsgEnabled(true);
        try
        {
          int i = executeUpdate(sql, false, true);jsr 17;return i;
        } finally {
          jsr 6; } localObject2 = returnAddress;locallyScopedConn.setReadInfoMsgEnabled(readInfoMsgState);ret;
      }
    }
    

    return executeUpdate(sql);
  }
  


  public int executeUpdate(String sql, String[] generatedKeyNames)
    throws SQLException
  {
    if ((generatedKeyNames != null) && (generatedKeyNames.length > 0)) {
      checkClosed();
      
      MySQLConnection locallyScopedConn = connection;
      
      synchronized (locallyScopedConn.getMutex())
      {


        boolean readInfoMsgState = connection.isReadInfoMsgEnabled();
        
        locallyScopedConn.setReadInfoMsgEnabled(true);
        try
        {
          int i = executeUpdate(sql, false, true);jsr 17;return i;
        } finally {
          jsr 6; } localObject2 = returnAddress;locallyScopedConn.setReadInfoMsgEnabled(readInfoMsgState);ret;
      }
    }
    

    return executeUpdate(sql);
  }
  



  protected Calendar getCalendarInstanceForSessionOrNew()
  {
    if (connection != null) {
      return connection.getCalendarInstanceForSessionOrNew();
    }
    
    return new GregorianCalendar();
  }
  







  public java.sql.Connection getConnection()
    throws SQLException
  {
    return connection;
  }
  






  public int getFetchDirection()
    throws SQLException
  {
    return 1000;
  }
  






  public int getFetchSize()
    throws SQLException
  {
    return fetchSize;
  }
  







  public synchronized ResultSet getGeneratedKeys()
    throws SQLException
  {
    if (!retrieveGeneratedKeys) {
      throw SQLError.createSQLException(Messages.getString("Statement.GeneratedKeysNotRequested"), "S1009", getExceptionInterceptor());
    }
    
    if (batchedGeneratedKeys == null) {
      if (lastQueryIsOnDupKeyUpdate) {
        return getGeneratedKeysInternal(1);
      }
      return getGeneratedKeysInternal();
    }
    
    Field[] fields = new Field[1];
    fields[0] = new Field("", "GENERATED_KEY", -5, 17);
    fields[0].setConnection(connection);
    
    return ResultSetImpl.getInstance(currentCatalog, fields, new RowDataStatic(batchedGeneratedKeys), connection, this, false);
  }
  






  protected ResultSet getGeneratedKeysInternal()
    throws SQLException
  {
    int numKeys = getUpdateCount();
    return getGeneratedKeysInternal(numKeys);
  }
  
  protected synchronized ResultSet getGeneratedKeysInternal(int numKeys) throws SQLException
  {
    Field[] fields = new Field[1];
    fields[0] = new Field("", "GENERATED_KEY", -5, 17);
    fields[0].setConnection(connection);
    fields[0].setUseOldNameMetadata(true);
    
    ArrayList rowSet = new ArrayList();
    
    long beginAt = getLastInsertID();
    
    if (beginAt < 0L) {
      fields[0].setUnsigned();
    }
    
    if (results != null) {
      String serverInfo = results.getServerInfo();
      




      if ((numKeys > 0) && (results.getFirstCharOfQuery() == 'R') && (serverInfo != null) && (serverInfo.length() > 0))
      {
        numKeys = getRecordCountFromInfo(serverInfo);
      }
      
      if ((beginAt != 0L) && (numKeys > 0)) {
        for (int i = 0; i < numKeys; i++) {
          byte[][] row = new byte[1][];
          if (beginAt > 0L) {
            row[0] = Long.toString(beginAt).getBytes();
          } else {
            byte[] asBytes = new byte[8];
            asBytes[7] = ((byte)(int)(beginAt & 0xFF));
            asBytes[6] = ((byte)(int)(beginAt >>> 8));
            asBytes[5] = ((byte)(int)(beginAt >>> 16));
            asBytes[4] = ((byte)(int)(beginAt >>> 24));
            asBytes[3] = ((byte)(int)(beginAt >>> 32));
            asBytes[2] = ((byte)(int)(beginAt >>> 40));
            asBytes[1] = ((byte)(int)(beginAt >>> 48));
            asBytes[0] = ((byte)(int)(beginAt >>> 56));
            
            BigInteger val = new BigInteger(1, asBytes);
            
            row[0] = val.toString().getBytes();
          }
          rowSet.add(new ByteArrayRow(row, getExceptionInterceptor()));
          beginAt += connection.getAutoIncrementIncrement();
        }
      }
    }
    
    ResultSetImpl gkRs = ResultSetImpl.getInstance(currentCatalog, fields, new RowDataStatic(rowSet), connection, this, false);
    

    openResults.add(gkRs);
    
    return gkRs;
  }
  




  protected int getId()
  {
    return statementId;
  }
  












  public long getLastInsertID()
  {
    return lastInsertId;
  }
  











  public long getLongUpdateCount()
  {
    if (results == null) {
      return -1L;
    }
    
    if (results.reallyResult()) {
      return -1L;
    }
    
    return updateCount;
  }
  









  public int getMaxFieldSize()
    throws SQLException
  {
    return maxFieldSize;
  }
  








  public int getMaxRows()
    throws SQLException
  {
    if (maxRows <= 0) {
      return 0;
    }
    
    return maxRows;
  }
  







  public boolean getMoreResults()
    throws SQLException
  {
    return getMoreResults(1);
  }
  


  public synchronized boolean getMoreResults(int current)
    throws SQLException
  {
    if (results == null) {
      return false;
    }
    
    boolean streamingMode = createStreamingResultSet();
    
    while ((streamingMode) && 
      (results.reallyResult()) && 
      (results.next())) {}
    



    ResultSetInternalMethods nextResultSet = results.getNextResultSet();
    
    switch (current)
    {
    case 1: 
      if (results != null) {
        if (!streamingMode) {
          results.close();
        }
        
        results.clearNextResult();
      }
      


      break;
    case 3: 
      if (results != null) {
        if (!streamingMode) {
          results.close();
        }
        
        results.clearNextResult();
      }
      
      closeAllOpenResults();
      
      break;
    
    case 2: 
      if (!connection.getDontTrackOpenResources()) {
        openResults.add(results);
      }
      
      results.clearNextResult();
      
      break;
    
    default: 
      throw SQLError.createSQLException(Messages.getString("Statement.19"), "S1009", getExceptionInterceptor());
    }
    
    

    results = nextResultSet;
    
    if (results == null) {
      updateCount = -1L;
      lastInsertId = -1L;
    } else if (results.reallyResult()) {
      updateCount = -1L;
      lastInsertId = -1L;
    } else {
      updateCount = results.getUpdateCount();
      lastInsertId = results.getUpdateID();
    }
    
    return (results != null) && (results.reallyResult());
  }
  









  public int getQueryTimeout()
    throws SQLException
  {
    return timeoutInMillis / 1000;
  }
  







  private int getRecordCountFromInfo(String serverInfo)
  {
    StringBuffer recordsBuf = new StringBuffer();
    int recordsCount = 0;
    int duplicatesCount = 0;
    
    char c = '\000';
    
    int length = serverInfo.length();
    for (int i = 0; 
        
        i < length; i++) {
      c = serverInfo.charAt(i);
      
      if (Character.isDigit(c)) {
        break;
      }
    }
    
    recordsBuf.append(c);
    i++;
    for (; 
        i < length; i++) {
      c = serverInfo.charAt(i);
      
      if (!Character.isDigit(c)) {
        break;
      }
      
      recordsBuf.append(c);
    }
    
    recordsCount = Integer.parseInt(recordsBuf.toString());
    
    StringBuffer duplicatesBuf = new StringBuffer();
    for (; 
        i < length; i++) {
      c = serverInfo.charAt(i);
      
      if (Character.isDigit(c)) {
        break;
      }
    }
    
    duplicatesBuf.append(c);
    i++;
    for (; 
        i < length; i++) {
      c = serverInfo.charAt(i);
      
      if (!Character.isDigit(c)) {
        break;
      }
      
      duplicatesBuf.append(c);
    }
    
    duplicatesCount = Integer.parseInt(duplicatesBuf.toString());
    
    return recordsCount - duplicatesCount;
  }
  







  public ResultSet getResultSet()
    throws SQLException
  {
    return (results != null) && (results.reallyResult()) ? results : null;
  }
  







  public int getResultSetConcurrency()
    throws SQLException
  {
    return resultSetConcurrency;
  }
  

  public int getResultSetHoldability()
    throws SQLException
  {
    return 1;
  }
  
  protected ResultSetInternalMethods getResultSetInternal() {
    return results;
  }
  






  public int getResultSetType()
    throws SQLException
  {
    return resultSetType;
  }
  








  public int getUpdateCount()
    throws SQLException
  {
    if (results == null) {
      return -1;
    }
    
    if (results.reallyResult()) {
      return -1;
    }
    
    int truncatedUpdateCount = 0;
    
    if (results.getUpdateCount() > 2147483647L) {
      truncatedUpdateCount = 2147483647;
    } else {
      truncatedUpdateCount = (int)results.getUpdateCount();
    }
    
    return truncatedUpdateCount;
  }
  



















  public SQLWarning getWarnings()
    throws SQLException
  {
    checkClosed();
    
    if ((connection != null) && (!connection.isClosed()) && (connection.versionMeetsMinimum(4, 1, 0)))
    {
      SQLWarning pendingWarningsFromServer = SQLError.convertShowWarningsToSQLWarnings(connection);
      

      if (warningChain != null) {
        warningChain.setNextWarning(pendingWarningsFromServer);
      } else {
        warningChain = pendingWarningsFromServer;
      }
      
      return warningChain;
    }
    
    return warningChain;
  }
  








  protected synchronized void realClose(boolean calledExplicitly, boolean closeOpenResults)
    throws SQLException
  {
    if (isClosed) {
      return;
    }
    
    if ((useUsageAdvisor) && 
      (!calledExplicitly)) {
      String message = Messages.getString("Statement.63") + Messages.getString("Statement.64");
      

      eventSink.consumeEvent(new ProfilerEvent((byte)0, "", currentCatalog, connectionId, getId(), -1, System.currentTimeMillis(), 0L, Constants.MILLIS_I18N, null, pointOfOrigin, message));
    }
    







    if (closeOpenResults) {
      closeOpenResults = !holdResultsOpenOverClose;
    }
    
    if (closeOpenResults) {
      if (results != null) {
        try
        {
          results.close();
        }
        catch (Exception ex) {}
      }
      

      closeAllOpenResults();
    }
    
    if (connection != null) {
      if (maxRowsChanged) {
        connection.unsetMaxRows(this);
      }
      
      if (!connection.getDontTrackOpenResources()) {
        connection.unregisterStatement(this);
      }
    }
    
    isClosed = true;
    
    results = null;
    connection = null;
    warningChain = null;
    openResults = null;
    batchedGeneratedKeys = null;
    localInfileInputStream = null;
    pingTarget = null;
  }
  













  public void setCursorName(String name)
    throws SQLException
  {}
  













  public void setEscapeProcessing(boolean enable)
    throws SQLException
  {
    doEscapeProcessing = enable;
  }
  











  public void setFetchDirection(int direction)
    throws SQLException
  {
    switch (direction)
    {
    case 1000: 
    case 1001: 
    case 1002: 
      break;
    default: 
      throw SQLError.createSQLException(Messages.getString("Statement.5"), "S1009", getExceptionInterceptor());
    }
    
  }
  













  public void setFetchSize(int rows)
    throws SQLException
  {
    if (((rows < 0) && (rows != -2147483648)) || ((maxRows != 0) && (maxRows != -1) && (rows > getMaxRows())))
    {

      throw SQLError.createSQLException(Messages.getString("Statement.7"), "S1009", getExceptionInterceptor());
    }
    


    fetchSize = rows;
  }
  
  public void setHoldResultsOpenOverClose(boolean holdResultsOpenOverClose) {
    this.holdResultsOpenOverClose = holdResultsOpenOverClose;
  }
  







  public void setMaxFieldSize(int max)
    throws SQLException
  {
    if (max < 0) {
      throw SQLError.createSQLException(Messages.getString("Statement.11"), "S1009", getExceptionInterceptor());
    }
    


    int maxBuf = connection != null ? connection.getMaxAllowedPacket() : MysqlIO.getMaxBuf();
    

    if (max > maxBuf) {
      throw SQLError.createSQLException(Messages.getString("Statement.13", new Object[] { Constants.longValueOf(maxBuf) }), "S1009", getExceptionInterceptor());
    }
    



    maxFieldSize = max;
  }
  









  public void setMaxRows(int max)
    throws SQLException
  {
    if ((max > 50000000) || (max < 0)) {
      throw SQLError.createSQLException(Messages.getString("Statement.15") + max + " > " + 50000000 + ".", "S1009", getExceptionInterceptor());
    }
    




    if (max == 0) {
      max = -1;
    }
    
    maxRows = max;
    maxRowsChanged = true;
    
    if (maxRows == -1) {
      connection.unsetMaxRows(this);
      maxRowsChanged = false;


    }
    else
    {

      connection.maxRowsChanged(this);
    }
  }
  







  public void setQueryTimeout(int seconds)
    throws SQLException
  {
    if (seconds < 0) {
      throw SQLError.createSQLException(Messages.getString("Statement.21"), "S1009", getExceptionInterceptor());
    }
    


    timeoutInMillis = (seconds * 1000);
  }
  





  void setResultSetConcurrency(int concurrencyFlag)
  {
    resultSetConcurrency = concurrencyFlag;
  }
  





  void setResultSetType(int typeFlag)
  {
    resultSetType = typeFlag;
  }
  
  protected void getBatchedGeneratedKeys(java.sql.Statement batchedStatement) throws SQLException {
    if (retrieveGeneratedKeys) {
      ResultSet rs = null;
      try
      {
        rs = batchedStatement.getGeneratedKeys();
        
        while (rs.next()) {
          batchedGeneratedKeys.add(new ByteArrayRow(new byte[][] { rs.getBytes(1) }, getExceptionInterceptor()));
        }
      }
      finally {
        if (rs != null) {
          rs.close();
        }
      }
    }
  }
  
  protected void getBatchedGeneratedKeys(int maxKeys) throws SQLException {
    if (retrieveGeneratedKeys) {
      ResultSet rs = null;
      try
      {
        if (maxKeys == 0) {
          rs = getGeneratedKeysInternal();
        } else {
          rs = getGeneratedKeysInternal(maxKeys);
        }
        while (rs.next()) {
          batchedGeneratedKeys.add(new ByteArrayRow(new byte[][] { rs.getBytes(1) }, getExceptionInterceptor()));
        }
      }
      finally {
        if (rs != null) {
          rs.close();
        }
      }
    }
  }
  


  private boolean useServerFetch()
    throws SQLException
  {
    return (connection.isCursorFetchEnabled()) && (fetchSize > 0) && (resultSetConcurrency == 1007) && (resultSetType == 1003);
  }
  
  public synchronized boolean isClosed()
    throws SQLException
  {
    return isClosed;
  }
  
  private boolean isPoolable = true;
  private InputStream localInfileInputStream;
  
  public boolean isPoolable() throws SQLException { return isPoolable; }
  
  public void setPoolable(boolean poolable) throws SQLException
  {
    isPoolable = poolable;
  }
  













  public boolean isWrapperFor(Class iface)
    throws SQLException
  {
    checkClosed();
    


    return iface.isInstance(this);
  }
  













  public Object unwrap(Class iface)
    throws SQLException
  {
    try
    {
      return Util.cast(iface, this);
    } catch (ClassCastException cce) {
      throw SQLError.createSQLException("Unable to unwrap to " + iface.toString(), "S1009", getExceptionInterceptor());
    }
  }
  
  protected int findStartOfStatement(String sql)
  {
    int statementStartPos = 0;
    
    if (StringUtils.startsWithIgnoreCaseAndWs(sql, "/*")) {
      statementStartPos = sql.indexOf("*/");
      
      if (statementStartPos == -1) {
        statementStartPos = 0;
      } else {
        statementStartPos += 2;
      }
    } else if ((StringUtils.startsWithIgnoreCaseAndWs(sql, "--")) || (StringUtils.startsWithIgnoreCaseAndWs(sql, "#")))
    {
      statementStartPos = sql.indexOf('\n');
      
      if (statementStartPos == -1) {
        statementStartPos = sql.indexOf('\r');
        
        if (statementStartPos == -1) {
          statementStartPos = 0;
        }
      }
    }
    
    return statementStartPos;
  }
  

  public synchronized InputStream getLocalInfileInputStream()
  {
    return localInfileInputStream;
  }
  
  public synchronized void setLocalInfileInputStream(InputStream stream) {
    localInfileInputStream = stream;
  }
  
  public synchronized void setPingTarget(PingTarget pingTarget) {
    this.pingTarget = pingTarget;
  }
  
  public ExceptionInterceptor getExceptionInterceptor() {
    return exceptionInterceptor;
  }
  
  protected boolean containsOnDuplicateKeyInString(String sql) {
    return getOnDuplicateKeyLocation(sql) != -1;
  }
  
  protected int getOnDuplicateKeyLocation(String sql) {
    return StringUtils.indexOfIgnoreCaseRespectMarker(0, sql, " ON DUPLICATE KEY UPDATE ", "\"'`", "\"'`", !connection.isNoBackslashEscapesSet());
  }
}
