package net.minecraft.server.v1_7_R1;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Random;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.bukkit.Server;
import org.bukkit.craftbukkit.v1_7_R1.CraftServer;
import org.bukkit.craftbukkit.v1_7_R1.CraftWorld;
import org.bukkit.craftbukkit.v1_7_R1.chunkio.ChunkIOExecutor;
import org.bukkit.craftbukkit.v1_7_R1.util.LongHash;
import org.bukkit.craftbukkit.v1_7_R1.util.LongHashSet;
import org.bukkit.craftbukkit.v1_7_R1.util.LongObjectHashMap;
import org.bukkit.event.world.ChunkLoadEvent;
import org.bukkit.event.world.ChunkPopulateEvent;
import org.bukkit.event.world.ChunkUnloadEvent;
import org.bukkit.generator.BlockPopulator;
import org.bukkit.plugin.PluginManager;


public class ChunkProviderServer
  implements IChunkProvider
{
  private static final Logger b = ;
  
  public LongHashSet unloadQueue = new LongHashSet();
  public Chunk emptyChunk;
  public IChunkProvider chunkProvider;
  private IChunkLoader f;
  public boolean forceChunkLoad = false;
  public LongObjectHashMap<Chunk> chunks = new LongObjectHashMap();
  public WorldServer world;
  
  public ChunkProviderServer(WorldServer worldserver, IChunkLoader ichunkloader, IChunkProvider ichunkprovider)
  {
    emptyChunk = new EmptyChunk(worldserver, 0, 0);
    world = worldserver;
    f = ichunkloader;
    chunkProvider = ichunkprovider;
  }
  
  public boolean isChunkLoaded(int i, int j) {
    return chunks.containsKey(LongHash.toLong(i, j));
  }
  
  public void queueUnload(int i, int j) {
    if (world.worldProvider.e()) {
      ChunkCoordinates chunkcoordinates = world.getSpawn();
      int k = i * 16 + 8 - x;
      int l = j * 16 + 8 - z;
      short short1 = 128;
      

      if ((k < -short1) || (k > short1) || (l < -short1) || (l > short1) || (!world.keepSpawnInMemory)) {
        unloadQueue.add(i, j);
        
        Chunk c = (Chunk)chunks.get(LongHash.toLong(i, j));
        if (c != null) {
          mustSave = true;
        }
      }
    }
    else
    {
      unloadQueue.add(i, j);
      
      Chunk c = (Chunk)chunks.get(LongHash.toLong(i, j));
      if (c != null) {
        mustSave = true;
      }
    }
  }
  
  public void a()
  {
    Iterator iterator = chunks.values().iterator();
    
    while (iterator.hasNext()) {
      Chunk chunk = (Chunk)iterator.next();
      
      queueUnload(locX, locZ);
    }
  }
  
  public Chunk getChunkAt(int i, int j)
  {
    return getChunkAt(i, j, null);
  }
  
  public Chunk getChunkAt(int i, int j, Runnable runnable) {
    unloadQueue.remove(i, j);
    Chunk chunk = (Chunk)chunks.get(LongHash.toLong(i, j));
    ChunkRegionLoader loader = null;
    
    if ((f instanceof ChunkRegionLoader)) {
      loader = (ChunkRegionLoader)f;
    }
    

    if ((chunk == null) && (loader != null) && (loader.chunkExists(world, i, j))) {
      if (runnable != null) {
        ChunkIOExecutor.queueChunkLoad(world, loader, this, i, j, runnable);
        return null;
      }
      chunk = ChunkIOExecutor.syncChunkLoad(world, loader, this, i, j);
    }
    else if (chunk == null) {
      chunk = originalGetChunkAt(i, j);
    }
    

    if (runnable != null) {
      runnable.run();
    }
    
    return chunk;
  }
  
  public Chunk originalGetChunkAt(int i, int j) {
    unloadQueue.remove(i, j);
    Chunk chunk = (Chunk)chunks.get(LongHash.toLong(i, j));
    boolean newChunk = false;
    
    if (chunk == null) {
      chunk = loadChunk(i, j);
      if (chunk == null) {
        if (chunkProvider == null) {
          chunk = emptyChunk;
        } else {
          try {
            chunk = chunkProvider.getOrCreateChunk(i, j);
          } catch (Throwable throwable) {
            CrashReport crashreport = CrashReport.a(throwable, "Exception generating new chunk");
            CrashReportSystemDetails crashreportsystemdetails = crashreport.a("Chunk to be generated");
            
            crashreportsystemdetails.a("Location", String.format("%d,%d", new Object[] { Integer.valueOf(i), Integer.valueOf(j) }));
            crashreportsystemdetails.a("Position hash", Long.valueOf(LongHash.toLong(i, j)));
            crashreportsystemdetails.a("Generator", chunkProvider.getName());
            throw new ReportedException(crashreport);
          }
        }
        newChunk = true;
      }
      
      chunks.put(LongHash.toLong(i, j), chunk);
      chunk.addEntities();
      

      Server server = world.getServer();
      if (server != null)
      {




        server.getPluginManager().callEvent(new ChunkLoadEvent(bukkitChunk, newChunk));
      }
      
      chunk.a(this, this, i, j);
    }
    
    return chunk;
  }
  
  public Chunk getOrCreateChunk(int i, int j)
  {
    Chunk chunk = (Chunk)chunks.get(LongHash.toLong(i, j));
    
    chunk = chunk == null ? getChunkAt(i, j) : (!world.isLoading) && (!forceChunkLoad) ? emptyChunk : chunk;
    if (chunk == emptyChunk) return chunk;
    if ((i != locX) || (j != locZ)) {
      b.error("Chunk (" + locX + ", " + locZ + ") stored at  (" + i + ", " + j + ") in world '" + world.getWorld().getName() + "'");
      b.error(chunk.getClass().getName());
      Throwable ex = new Throwable();
      ex.fillInStackTrace();
      ex.printStackTrace();
    }
    return chunk;
  }
  
  public Chunk loadChunk(int i, int j)
  {
    if (f == null) {
      return null;
    }
    try {
      Chunk chunk = f.a(world, i, j);
      
      if (chunk != null) {
        p = world.getTime();
        if (chunkProvider != null) {
          chunkProvider.recreateStructures(i, j);
        }
      }
      
      return chunk;
    } catch (Exception exception) {
      b.error("Couldn't load chunk", exception); }
    return null;
  }
  

  public void saveChunkNOP(Chunk chunk)
  {
    if (f != null) {
      try {
        f.b(world, chunk);
      } catch (Exception exception) {
        b.error("Couldn't save entities", exception);
      }
    }
  }
  
  public void saveChunk(Chunk chunk) {
    if (f != null) {
      try {
        p = world.getTime();
        f.a(world, chunk);
      }
      catch (Exception ioexception) {
        b.error("Couldn't save chunk", ioexception);
      }
    }
  }
  



  public void getChunkAt(IChunkProvider ichunkprovider, int i, int j)
  {
    Chunk chunk = getOrCreateChunk(i, j);
    
    if (!done) {
      chunk.p();
      if (chunkProvider != null) {
        chunkProvider.getChunkAt(ichunkprovider, i, j);
        

        BlockSand.instaFall = true;
        Random random = new Random();
        random.setSeed(this.world.getSeed());
        long xRand = random.nextLong() / 2L * 2L + 1L;
        long zRand = random.nextLong() / 2L * 2L + 1L;
        random.setSeed(i * xRand + j * zRand ^ this.world.getSeed());
        
        org.bukkit.World world = this.world.getWorld();
        if (world != null) {
          worldpopulating = true;
          try {
            for (BlockPopulator populator : world.getPopulators()) {
              populator.populate(world, random, bukkitChunk);
            }
          } finally {
            worldpopulating = false;
          }
        }
        BlockSand.instaFall = false;
        this.world.getServer().getPluginManager().callEvent(new ChunkPopulateEvent(bukkitChunk));
        

        chunk.e();
      }
    }
  }
  
  public boolean saveChunks(boolean flag, IProgressUpdate iprogressupdate) {
    int i = 0;
    
    Iterator iterator = chunks.values().iterator();
    
    while (iterator.hasNext()) {
      Chunk chunk = (Chunk)iterator.next();
      

      if (flag) {
        saveChunkNOP(chunk);
      }
      
      if (chunk.a(flag)) {
        saveChunk(chunk);
        n = false;
        i++;
        if ((i == 24) && (!flag)) {
          return false;
        }
      }
    }
    
    return true;
  }
  
  public void b() {
    if (f != null) {
      f.b();
    }
  }
  
  public boolean unloadChunks() {
    if (!world.savingDisabled)
    {
      Server server = world.getServer();
      for (int i = 0; (i < 100) && (!unloadQueue.isEmpty()); i++) {
        long chunkcoordinates = unloadQueue.popFirst();
        Chunk chunk = (Chunk)chunks.get(chunkcoordinates);
        if (chunk != null)
        {
          ChunkUnloadEvent event = new ChunkUnloadEvent(bukkitChunk);
          server.getPluginManager().callEvent(event);
          if (!event.isCancelled()) {
            chunk.removeEntities();
            saveChunk(chunk);
            saveChunkNOP(chunk);
            

            chunks.remove(chunkcoordinates);
          }
        }
      }
      
      if (f != null) {
        f.a();
      }
    }
    
    return chunkProvider.unloadChunks();
  }
  
  public boolean canSave() {
    return !world.savingDisabled;
  }
  
  public String getName()
  {
    return "ServerChunkCache: " + chunks.values().size() + " Drop: " + unloadQueue.size();
  }
  
  public List getMobsFor(EnumCreatureType enumcreaturetype, int i, int j, int k) {
    return chunkProvider.getMobsFor(enumcreaturetype, i, j, k);
  }
  
  public ChunkPosition findNearestMapFeature(World world, String s, int i, int j, int k) {
    return chunkProvider.findNearestMapFeature(world, s, i, j, k);
  }
  
  public int getLoadedChunks()
  {
    return chunks.values().size();
  }
  
  public void recreateStructures(int i, int j) {}
}
