package net.minecraft.server.v1_7_R1;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

public class ChunkRegionLoader implements IChunkLoader, IAsyncChunkSaver
{
  private static final Logger a = ;
  private List b = new ArrayList();
  private Set c = new HashSet();
  private Object d = new Object();
  private final File e;
  
  public ChunkRegionLoader(File file1) {
    e = file1;
  }
  
  public boolean chunkExists(World world, int i, int j)
  {
    ChunkCoordIntPair chunkcoordintpair = new ChunkCoordIntPair(i, j);
    
    synchronized (d) {
      if (c.contains(chunkcoordintpair)) {
        for (int k = 0; k < b.size(); k++) {
          if (b.get(k)).a.equals(chunkcoordintpair)) {
            return true;
          }
        }
      }
    }
    
    return RegionFileCache.a(e, i, j).chunkExists(i & 0x1F, j & 0x1F);
  }
  

  public Chunk a(World world, int i, int j)
  {
    Object[] data = loadChunk(world, i, j);
    if (data != null) {
      Chunk chunk = (Chunk)data[0];
      NBTTagCompound nbttagcompound = (NBTTagCompound)data[1];
      loadEntities(chunk, nbttagcompound.getCompound("Level"), world);
      return chunk;
    }
    
    return null;
  }
  
  public Object[] loadChunk(World world, int i, int j)
  {
    NBTTagCompound nbttagcompound = null;
    ChunkCoordIntPair chunkcoordintpair = new ChunkCoordIntPair(i, j);
    Object object = d;
    
    synchronized (d) {
      if (c.contains(chunkcoordintpair)) {
        for (int k = 0; k < b.size(); k++) {
          if (b.get(k)).a.equals(chunkcoordintpair)) {
            nbttagcompound = b.get(k)).b;
            break;
          }
        }
      }
    }
    
    if (nbttagcompound == null) {
      DataInputStream datainputstream = RegionFileCache.c(e, i, j);
      
      if (datainputstream == null) {
        return null;
      }
      
      nbttagcompound = NBTCompressedStreamTools.a(datainputstream);
    }
    
    return a(world, i, j, nbttagcompound);
  }
  
  protected Object[] a(World world, int i, int j, NBTTagCompound nbttagcompound) {
    if (!nbttagcompound.hasKeyOfType("Level", 10)) {
      a.error("Chunk file at " + i + "," + j + " is missing level data, skipping");
      return null; }
    if (!nbttagcompound.getCompound("Level").hasKeyOfType("Sections", 9)) {
      a.error("Chunk file at " + i + "," + j + " is missing block data, skipping");
      return null;
    }
    Chunk chunk = a(world, nbttagcompound.getCompound("Level"));
    
    if (!chunk.a(i, j)) {
      a.error("Chunk file at " + i + "," + j + " is in the wrong location; relocating. (Expected " + i + ", " + j + ", got " + locX + ", " + locZ + ")");
      nbttagcompound.getCompound("Level").setInt("xPos", i);
      nbttagcompound.getCompound("Level").setInt("zPos", j);
      

      NBTTagList tileEntities = nbttagcompound.getCompound("Level").getList("TileEntities", 10);
      if (tileEntities != null) {
        for (int te = 0; te < tileEntities.size(); te++) {
          NBTTagCompound tileEntity = tileEntities.get(te);
          int x = tileEntity.getInt("x") - locX * 16;
          int z = tileEntity.getInt("z") - locZ * 16;
          tileEntity.setInt("x", i * 16 + x);
          tileEntity.setInt("z", j * 16 + z);
        }
      }
      
      chunk = a(world, nbttagcompound.getCompound("Level"));
    }
    

    Object[] data = new Object[2];
    data[0] = chunk;
    data[1] = nbttagcompound;
    return data;
  }
  

  public void a(World world, Chunk chunk)
  {
    try
    {
      world.F();
    } catch (ExceptionWorldConflict ex) {
      ex.printStackTrace();
    }
    
    try
    {
      NBTTagCompound nbttagcompound = new NBTTagCompound();
      NBTTagCompound nbttagcompound1 = new NBTTagCompound();
      
      nbttagcompound.set("Level", nbttagcompound1);
      a(chunk, world, nbttagcompound1);
      a(chunk.l(), nbttagcompound);
    } catch (Exception exception) {
      exception.printStackTrace();
    }
  }
  
  protected void a(ChunkCoordIntPair chunkcoordintpair, NBTTagCompound nbttagcompound) {
    Object object = d;
    
    synchronized (d) {
      if (c.contains(chunkcoordintpair)) {
        for (int i = 0; i < b.size(); i++) {
          if (b.get(i)).a.equals(chunkcoordintpair)) {
            b.set(i, new PendingChunkToSave(chunkcoordintpair, nbttagcompound));
            return;
          }
        }
      }
      
      b.add(new PendingChunkToSave(chunkcoordintpair, nbttagcompound));
      c.add(chunkcoordintpair);
      FileIOThread.a.a(this);
    }
  }
  
  public boolean c() {
    PendingChunkToSave pendingchunktosave = null;
    Object object = d;
    
    synchronized (d) {
      if (b.isEmpty()) {
        return false;
      }
      
      pendingchunktosave = (PendingChunkToSave)b.remove(0);
      c.remove(a);
    }
    
    if (pendingchunktosave != null) {
      try {
        a(pendingchunktosave);
      } catch (Exception exception) {
        exception.printStackTrace();
      }
    }
    
    return true;
  }
  
  public void a(PendingChunkToSave pendingchunktosave) throws IOException {
    DataOutputStream dataoutputstream = RegionFileCache.d(e, a.x, a.z);
    
    NBTCompressedStreamTools.a(b, dataoutputstream);
    dataoutputstream.close();
  }
  
  public void b(World world, Chunk chunk) {}
  
  public void a() {}
  
  public void b() {
    while (c()) {}
  }
  

  private void a(Chunk chunk, World world, NBTTagCompound nbttagcompound)
  {
    nbttagcompound.setByte("V", (byte)1);
    nbttagcompound.setInt("xPos", locX);
    nbttagcompound.setInt("zPos", locZ);
    nbttagcompound.setLong("LastUpdate", world.getTime());
    nbttagcompound.setIntArray("HeightMap", heightMap);
    nbttagcompound.setBoolean("TerrainPopulated", done);
    nbttagcompound.setBoolean("LightPopulated", lit);
    nbttagcompound.setLong("InhabitedTime", s);
    ChunkSection[] achunksection = chunk.i();
    NBTTagList nbttaglist = new NBTTagList();
    boolean flag = !worldProvider.g;
    ChunkSection[] achunksection1 = achunksection;
    int i = achunksection.length;
    


    for (int j = 0; j < i; j++) {
      ChunkSection chunksection = achunksection1[j];
      
      if (chunksection != null) {
        NBTTagCompound nbttagcompound1 = new NBTTagCompound();
        nbttagcompound1.setByte("Y", (byte)(chunksection.getYPosition() >> 4 & 0xFF));
        nbttagcompound1.setByteArray("Blocks", chunksection.getIdArray());
        if (chunksection.getExtendedIdArray() != null) {
          nbttagcompound1.setByteArray("Add", getExtendedIdArray()a);
        }
        
        nbttagcompound1.setByteArray("Data", getDataArray()a);
        nbttagcompound1.setByteArray("BlockLight", getEmittedLightArray()a);
        if (flag) {
          nbttagcompound1.setByteArray("SkyLight", getSkyLightArray()a);
        } else {
          nbttagcompound1.setByteArray("SkyLight", new byte[getEmittedLightArray()a.length]);
        }
        
        nbttaglist.add(nbttagcompound1);
      }
    }
    
    nbttagcompound.set("Sections", nbttaglist);
    nbttagcompound.setByteArray("Biomes", chunk.m());
    o = false;
    NBTTagList nbttaglist1 = new NBTTagList();
    


    for (i = 0; i < entitySlices.length; i++) {
      Iterator iterator = entitySlices[i].iterator();
      
      while (iterator.hasNext()) {
        Entity entity = (Entity)iterator.next();
        
        NBTTagCompound nbttagcompound1 = new NBTTagCompound();
        if (entity.d(nbttagcompound1)) {
          o = true;
          nbttaglist1.add(nbttagcompound1);
        }
      }
    }
    
    nbttagcompound.set("Entities", nbttaglist1);
    NBTTagList nbttaglist2 = new NBTTagList();
    
    Iterator iterator = tileEntities.values().iterator();
    
    while (iterator.hasNext()) {
      TileEntity tileentity = (TileEntity)iterator.next();
      
      NBTTagCompound nbttagcompound1 = new NBTTagCompound();
      tileentity.b(nbttagcompound1);
      nbttaglist2.add(nbttagcompound1);
    }
    
    nbttagcompound.set("TileEntities", nbttaglist2);
    List list = world.a(chunk, false);
    
    if (list != null) {
      long k = world.getTime();
      NBTTagList nbttaglist3 = new NBTTagList();
      Iterator iterator1 = list.iterator();
      
      while (iterator1.hasNext()) {
        NextTickListEntry nextticklistentry = (NextTickListEntry)iterator1.next();
        NBTTagCompound nbttagcompound2 = new NBTTagCompound();
        
        nbttagcompound2.setInt("i", Block.b(nextticklistentry.a()));
        nbttagcompound2.setInt("x", a);
        nbttagcompound2.setInt("y", b);
        nbttagcompound2.setInt("z", c);
        nbttagcompound2.setInt("t", (int)(d - k));
        nbttagcompound2.setInt("p", e);
        nbttaglist3.add(nbttagcompound2);
      }
      
      nbttagcompound.set("TileTicks", nbttaglist3);
    }
  }
  
  private Chunk a(World world, NBTTagCompound nbttagcompound) {
    int i = nbttagcompound.getInt("xPos");
    int j = nbttagcompound.getInt("zPos");
    Chunk chunk = new Chunk(world, i, j);
    
    heightMap = nbttagcompound.getIntArray("HeightMap");
    done = nbttagcompound.getBoolean("TerrainPopulated");
    lit = nbttagcompound.getBoolean("LightPopulated");
    s = nbttagcompound.getLong("InhabitedTime");
    NBTTagList nbttaglist = nbttagcompound.getList("Sections", 10);
    byte b0 = 16;
    ChunkSection[] achunksection = new ChunkSection[b0];
    boolean flag = !worldProvider.g;
    
    for (int k = 0; k < nbttaglist.size(); k++) {
      NBTTagCompound nbttagcompound1 = nbttaglist.get(k);
      byte b1 = nbttagcompound1.getByte("Y");
      ChunkSection chunksection = new ChunkSection(b1 << 4, flag);
      
      chunksection.setIdArray(nbttagcompound1.getByteArray("Blocks"));
      if (nbttagcompound1.hasKeyOfType("Add", 7)) {
        chunksection.setExtendedIdArray(new NibbleArray(nbttagcompound1.getByteArray("Add"), 4));
      }
      
      chunksection.setDataArray(new NibbleArray(nbttagcompound1.getByteArray("Data"), 4));
      chunksection.setEmittedLightArray(new NibbleArray(nbttagcompound1.getByteArray("BlockLight"), 4));
      if (flag) {
        chunksection.setSkyLightArray(new NibbleArray(nbttagcompound1.getByteArray("SkyLight"), 4));
      }
      
      chunksection.recalcBlockCounts();
      achunksection[b1] = chunksection;
    }
    
    chunk.a(achunksection);
    if (nbttagcompound.hasKeyOfType("Biomes", 7)) {
      chunk.a(nbttagcompound.getByteArray("Biomes"));
    }
    

    return chunk;
  }
  
  public void loadEntities(Chunk chunk, NBTTagCompound nbttagcompound, World world)
  {
    NBTTagList nbttaglist1 = nbttagcompound.getList("Entities", 10);
    
    if (nbttaglist1 != null) {
      for (int l = 0; l < nbttaglist1.size(); l++) {
        NBTTagCompound nbttagcompound2 = nbttaglist1.get(l);
        Entity entity = EntityTypes.a(nbttagcompound2, world);
        
        o = true;
        if (entity != null) {
          chunk.a(entity);
          Entity entity1 = entity;
          
          for (NBTTagCompound nbttagcompound3 = nbttagcompound2; nbttagcompound3.hasKeyOfType("Riding", 10); nbttagcompound3 = nbttagcompound3.getCompound("Riding")) {
            Entity entity2 = EntityTypes.a(nbttagcompound3.getCompound("Riding"), world);
            
            if (entity2 != null) {
              chunk.a(entity2);
              entity1.mount(entity2);
            }
            
            entity1 = entity2;
          }
        }
      }
    }
    
    NBTTagList nbttaglist2 = nbttagcompound.getList("TileEntities", 10);
    
    if (nbttaglist2 != null) {
      for (int i1 = 0; i1 < nbttaglist2.size(); i1++) {
        NBTTagCompound nbttagcompound4 = nbttaglist2.get(i1);
        TileEntity tileentity = TileEntity.c(nbttagcompound4);
        
        if (tileentity != null) {
          chunk.a(tileentity);
        }
      }
    }
    
    if (nbttagcompound.hasKeyOfType("TileTicks", 9)) {
      NBTTagList nbttaglist3 = nbttagcompound.getList("TileTicks", 10);
      
      if (nbttaglist3 != null) {
        for (int j1 = 0; j1 < nbttaglist3.size(); j1++) {
          NBTTagCompound nbttagcompound5 = nbttaglist3.get(j1);
          
          world.b(nbttagcompound5.getInt("x"), nbttagcompound5.getInt("y"), nbttagcompound5.getInt("z"), Block.e(nbttagcompound5.getInt("i")), nbttagcompound5.getInt("t"), nbttagcompound5.getInt("p"));
        }
      }
    }
  }
}
