package net.minecraft.server.v1_7_R1;

public class ChunkSection
{
  private int yPos;
  private int nonEmptyBlockCount;
  private int tickingBlockCount;
  private byte[] blockIds;
  private NibbleArray extBlockIds;
  private NibbleArray blockData;
  private NibbleArray emittedLight;
  private NibbleArray skyLight;
  
  public ChunkSection(int i, boolean flag) {
    yPos = i;
    blockIds = new byte[4096];
    blockData = new NibbleArray(blockIds.length, 4);
    emittedLight = new NibbleArray(blockIds.length, 4);
    if (flag) {
      skyLight = new NibbleArray(blockIds.length, 4);
    }
  }
  
  public ChunkSection(int y, boolean flag, byte[] blkIds, byte[] extBlkIds)
  {
    yPos = y;
    blockIds = blkIds;
    if (extBlkIds != null) {
      extBlockIds = new NibbleArray(extBlkIds, 4);
    }
    blockData = new NibbleArray(blockIds.length, 4);
    emittedLight = new NibbleArray(blockIds.length, 4);
    if (flag) {
      skyLight = new NibbleArray(blockIds.length, 4);
    }
    recalcBlockCounts();
  }
  
  public Block getTypeId(int i, int j, int k)
  {
    int l = blockIds[(j << 8 | k << 4 | i)] & 0xFF;
    
    if (extBlockIds != null) {
      l |= extBlockIds.a(i, j, k) << 8;
    }
    
    return Block.e(l);
  }
  
  public void setTypeId(int i, int j, int k, Block block) {
    int l = blockIds[(j << 8 | k << 4 | i)] & 0xFF;
    
    if (extBlockIds != null) {
      l |= extBlockIds.a(i, j, k) << 8;
    }
    
    Block block1 = Block.e(l);
    
    if (block1 != Blocks.AIR) {
      nonEmptyBlockCount -= 1;
      if (block1.isTicking()) {
        tickingBlockCount -= 1;
      }
    }
    
    if (block != Blocks.AIR) {
      nonEmptyBlockCount += 1;
      if (block.isTicking()) {
        tickingBlockCount += 1;
      }
    }
    
    int i1 = Block.b(block);
    
    blockIds[(j << 8 | k << 4 | i)] = ((byte)(i1 & 0xFF));
    if (i1 > 255) {
      if (extBlockIds == null) {
        extBlockIds = new NibbleArray(blockIds.length, 4);
      }
      
      extBlockIds.a(i, j, k, (i1 & 0xF00) >> 8);
    } else if (extBlockIds != null) {
      extBlockIds.a(i, j, k, 0);
    }
  }
  
  public int getData(int i, int j, int k) {
    return blockData.a(i, j, k);
  }
  
  public void setData(int i, int j, int k, int l) {
    blockData.a(i, j, k, l);
  }
  
  public boolean isEmpty() {
    return nonEmptyBlockCount == 0;
  }
  
  public boolean shouldTick() {
    return tickingBlockCount > 0;
  }
  
  public int getYPosition() {
    return yPos;
  }
  
  public void setSkyLight(int i, int j, int k, int l) {
    skyLight.a(i, j, k, l);
  }
  
  public int getSkyLight(int i, int j, int k) {
    return skyLight.a(i, j, k);
  }
  
  public void setEmittedLight(int i, int j, int k, int l) {
    emittedLight.a(i, j, k, l);
  }
  
  public int getEmittedLight(int i, int j, int k) {
    return emittedLight.a(i, j, k);
  }
  
  public void recalcBlockCounts()
  {
    byte[] blkIds = blockIds;
    int cntNonEmpty = 0;
    int cntTicking = 0;
    byte[] ext; int off; int off2; if (extBlockIds == null) {
      for (int off = 0; off < blkIds.length; off++) {
        int l = blkIds[off] & 0xFF;
        if (l > 0) {
          if (Block.e(l) == null) {
            blkIds[off] = 0;
          } else {
            cntNonEmpty++;
            if (Block.e(l).isTicking()) {
              cntTicking++;
            }
          }
        }
      }
    } else {
      ext = extBlockIds.a;
      off = 0; for (off2 = 0; off < blkIds.length;) {
        byte extid = ext[off2];
        int l = blkIds[off] & 0xFF | (extid & 0xF) << 8;
        if (l > 0) {
          if (Block.e(l) == null) {
            blkIds[off] = 0; int 
              tmp153_151 = off2; byte[] tmp153_149 = ext;tmp153_149[tmp153_151] = ((byte)(tmp153_149[tmp153_151] & 0xF0));
          } else {
            cntNonEmpty++;
            if (Block.e(l).isTicking()) {
              cntTicking++;
            }
          }
        }
        off++;
        l = blkIds[off] & 0xFF | (extid & 0xF0) << 4;
        if (l > 0) {
          if (Block.e(l) == null) {
            blkIds[off] = 0; int 
              tmp225_223 = off2; byte[] tmp225_221 = ext;tmp225_221[tmp225_223] = ((byte)(tmp225_221[tmp225_223] & 0xF));
          } else {
            cntNonEmpty++;
            if (Block.e(l).isTicking()) {
              cntTicking++;
            }
          }
        }
        off++;
        off2++;
      }
    }
    nonEmptyBlockCount = cntNonEmpty;
    tickingBlockCount = cntTicking;
  }
  
  public void old_recalcBlockCounts()
  {
    nonEmptyBlockCount = 0;
    tickingBlockCount = 0;
    
    for (int i = 0; i < 16; i++) {
      for (int j = 0; j < 16; j++) {
        for (int k = 0; k < 16; k++) {
          Block block = getTypeId(i, j, k);
          
          if (block != Blocks.AIR) {
            nonEmptyBlockCount += 1;
            if (block.isTicking()) {
              tickingBlockCount += 1;
            }
          }
        }
      }
    }
  }
  
  public byte[] getIdArray() {
    return blockIds;
  }
  
  public NibbleArray getExtendedIdArray() {
    return extBlockIds;
  }
  
  public NibbleArray getDataArray() {
    return blockData;
  }
  
  public NibbleArray getEmittedLightArray() {
    return emittedLight;
  }
  
  public NibbleArray getSkyLightArray() {
    return skyLight;
  }
  
  public void setIdArray(byte[] abyte) {
    blockIds = validateByteArray(abyte);
  }
  
  public void setExtendedIdArray(NibbleArray nibblearray)
  {
    boolean empty = true;
    for (int i = 0; i < a.length; i++) {
      if (a[i] != 0) {
        empty = false;
        break;
      }
    }
    
    if (empty) {
      return;
    }
    

    extBlockIds = validateNibbleArray(nibblearray);
  }
  
  public void setDataArray(NibbleArray nibblearray) {
    blockData = validateNibbleArray(nibblearray);
  }
  
  public void setEmittedLightArray(NibbleArray nibblearray) {
    emittedLight = validateNibbleArray(nibblearray);
  }
  
  public void setSkyLightArray(NibbleArray nibblearray) {
    skyLight = validateNibbleArray(nibblearray);
  }
  
  private NibbleArray validateNibbleArray(NibbleArray nibbleArray)
  {
    if ((nibbleArray != null) && (a.length < 2048)) {
      byte[] newArray = new byte[2048];
      System.arraycopy(a, 0, newArray, 0, a.length > 2048 ? 2048 : a.length);
      nibbleArray = new NibbleArray(newArray, 4);
    }
    
    return nibbleArray;
  }
  
  private byte[] validateByteArray(byte[] byteArray) {
    if ((byteArray != null) && (byteArray.length < 4096)) {
      byte[] newArray = new byte[4096];
      System.arraycopy(byteArray, 0, newArray, 0, byteArray.length);
      byteArray = newArray;
    }
    
    return byteArray;
  }
}
