package net.minecraft.server.v1_7_R1;

import java.net.SocketAddress;
import java.util.Queue;
import javax.crypto.SecretKey;
import net.minecraft.util.com.google.common.collect.Queues;
import net.minecraft.util.com.google.common.util.concurrent.ThreadFactoryBuilder;
import net.minecraft.util.io.netty.channel.Channel;
import net.minecraft.util.io.netty.channel.ChannelConfig;
import net.minecraft.util.io.netty.channel.ChannelFuture;
import net.minecraft.util.io.netty.channel.ChannelFutureListener;
import net.minecraft.util.io.netty.channel.ChannelHandlerContext;
import net.minecraft.util.io.netty.channel.ChannelPipeline;
import net.minecraft.util.io.netty.channel.EventLoop;
import net.minecraft.util.io.netty.channel.SimpleChannelInboundHandler;
import net.minecraft.util.io.netty.channel.nio.NioEventLoopGroup;
import net.minecraft.util.io.netty.util.Attribute;
import net.minecraft.util.io.netty.util.AttributeKey;
import net.minecraft.util.io.netty.util.concurrent.GenericFutureListener;
import org.apache.logging.log4j.Logger;
import org.apache.logging.log4j.Marker;
import org.apache.logging.log4j.MarkerManager;

public class NetworkManager extends SimpleChannelInboundHandler
{
  private static final Logger g = ;
  public static final Marker a = MarkerManager.getMarker("NETWORK");
  public static final Marker b = MarkerManager.getMarker("NETWORK_PACKETS", a);
  public static final AttributeKey c = new AttributeKey("protocol");
  public static final AttributeKey d = new AttributeKey("receivable_packets");
  public static final AttributeKey e = new AttributeKey("sendable_packets");
  public static final NioEventLoopGroup f = new NioEventLoopGroup(0, new ThreadFactoryBuilder().setNameFormat("Netty Client IO #%d").setDaemon(true).build());
  private final boolean h;
  private final Queue i = Queues.newConcurrentLinkedQueue();
  private final Queue j = Queues.newConcurrentLinkedQueue();
  private Channel k;
  private SocketAddress l;
  private PacketListener m;
  private EnumProtocol n;
  private IChatBaseComponent o;
  
  public NetworkManager(boolean flag) {
    h = flag;
  }
  
  public void channelActive(ChannelHandlerContext channelhandlercontext) throws Exception {
    super.channelActive(channelhandlercontext);
    k = channelhandlercontext.channel();
    l = k.remoteAddress();
    a(EnumProtocol.HANDSHAKING);
  }
  
  public void a(EnumProtocol enumprotocol) {
    n = ((EnumProtocol)k.attr(c).getAndSet(enumprotocol));
    k.attr(d).set(enumprotocol.a(h));
    k.attr(e).set(enumprotocol.b(h));
    k.config().setAutoRead(true);
    g.debug("Enabled auto read");
  }
  
  public void channelInactive(ChannelHandlerContext channelhandlercontext) {
    a(new ChatMessage("disconnect.endOfStream", new Object[0]));
  }
  
  public void exceptionCaught(ChannelHandlerContext channelhandlercontext, Throwable throwable) {
    a(new ChatMessage("disconnect.genericReason", new Object[] { "Internal Exception: " + throwable }));
  }
  
  protected void a(ChannelHandlerContext channelhandlercontext, Packet packet) {
    if (k.isOpen()) {
      if (packet.a()) {
        packet.handle(m);
      } else {
        i.add(packet);
      }
    }
  }
  
  public void a(PacketListener packetlistener) {
    net.minecraft.util.org.apache.commons.lang3.Validate.notNull(packetlistener, "packetListener", new Object[0]);
    g.debug("Set listener of {} to {}", new Object[] { this, packetlistener });
    m = packetlistener;
  }
  
  public void handle(Packet packet, GenericFutureListener... agenericfuturelistener) {
    if ((k != null) && (k.isOpen())) {
      h();
      b(packet, agenericfuturelistener);
    } else {
      j.add(new QueuedPacket(packet, agenericfuturelistener));
    }
  }
  
  private void b(Packet packet, GenericFutureListener[] agenericfuturelistener) {
    EnumProtocol enumprotocol = EnumProtocol.a(packet);
    EnumProtocol enumprotocol1 = (EnumProtocol)k.attr(c).get();
    
    if (enumprotocol1 != enumprotocol) {
      g.debug("Disabled auto read");
      k.config().setAutoRead(false);
    }
    
    if (k.eventLoop().inEventLoop()) {
      if (enumprotocol != enumprotocol1) {
        a(enumprotocol);
      }
      
      k.writeAndFlush(packet).addListeners(agenericfuturelistener).addListener(ChannelFutureListener.FIRE_EXCEPTION_ON_FAILURE);
    } else {
      k.eventLoop().execute(new QueuedProtocolSwitch(this, enumprotocol, enumprotocol1, packet, agenericfuturelistener));
    }
  }
  
  private void h() {
    if ((k != null) && (k.isOpen())) {
      while (!j.isEmpty()) {
        QueuedPacket queuedpacket = (QueuedPacket)j.poll();
        
        b(QueuedPacket.a(queuedpacket), QueuedPacket.b(queuedpacket));
      }
    }
  }
  
  public void a() {
    h();
    EnumProtocol enumprotocol = (EnumProtocol)k.attr(c).get();
    
    if (n != enumprotocol) {
      if (n != null) {
        m.a(n, enumprotocol);
      }
      
      n = enumprotocol;
    }
    
    if (m != null) {
      for (int i = 1000; (!this.i.isEmpty()) && (i >= 0); i--) {
        Packet packet = (Packet)this.i.poll();
        

        if ((d()) && (k.config().isAutoRead()))
        {



          packet.handle(m);
        }
      }
      m.a();
    }
    
    k.flush();
  }
  
  public SocketAddress getSocketAddress() {
    return l;
  }
  
  public void a(IChatBaseComponent ichatbasecomponent) {
    if (k.isOpen()) {
      k.close();
      o = ichatbasecomponent;
    }
  }
  
  public boolean c() {
    return ((k instanceof net.minecraft.util.io.netty.channel.local.LocalChannel)) || ((k instanceof net.minecraft.util.io.netty.channel.local.LocalServerChannel));
  }
  
  public void a(SecretKey secretkey) {
    k.pipeline().addBefore("splitter", "decrypt", new PacketDecrypter(MinecraftEncryption.a(2, secretkey)));
    k.pipeline().addBefore("prepender", "encrypt", new PacketEncrypter(MinecraftEncryption.a(1, secretkey)));
  }
  
  public boolean d() {
    return (k != null) && (k.isOpen());
  }
  
  public PacketListener getPacketListener() {
    return m;
  }
  
  public IChatBaseComponent f() {
    return o;
  }
  
  public void g() {
    k.config().setAutoRead(false);
  }
  
  protected void channelRead0(ChannelHandlerContext channelhandlercontext, Object object) {
    a(channelhandlercontext, (Packet)object);
  }
  
  static Channel a(NetworkManager networkmanager) {
    return k;
  }
}
