package net.minecraft.server.v1_7_R1;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Queue;
import java.util.concurrent.ConcurrentLinkedQueue;

public class PlayerChunkMap
{
  private final WorldServer world;
  private final List managedPlayers = new ArrayList();
  private final LongHashMap c = new LongHashMap();
  private final Queue d = new ConcurrentLinkedQueue();
  private final Queue e = new ConcurrentLinkedQueue();
  private final int f;
  private long g;
  private final int[][] h = { { 1, 0 }, { 0, 1 }, { -1, 0 }, { 0, -1 } };
  private boolean wasNotEmpty;
  
  public PlayerChunkMap(WorldServer worldserver, int i) {
    if (i > 15)
      throw new IllegalArgumentException("Too big view radius!");
    if (i < 3) {
      throw new IllegalArgumentException("Too small view radius!");
    }
    f = i;
    world = worldserver;
  }
  
  public WorldServer a()
  {
    return world;
  }
  
  public void flush() {
    long i = world.getTime();
    


    if (i - g > 8000L) {
      g = i;
      

      Iterator iterator = e.iterator();
      while (iterator.hasNext()) {
        PlayerChunk playerchunk = (PlayerChunk)iterator.next();
        playerchunk.b();
        playerchunk.a();
      }
    } else {
      Iterator iterator = d.iterator();
      
      while (iterator.hasNext()) {
        PlayerChunk playerchunk = (PlayerChunk)iterator.next();
        playerchunk.b();
        iterator.remove();
      }
    }
    


    if (managedPlayers.isEmpty()) {
      if (!wasNotEmpty) return;
      WorldProvider worldprovider = world.worldProvider;
      
      if (!worldprovider.e()) {
        world.chunkProviderServer.a();
      }
      
      wasNotEmpty = false;
    } else {
      wasNotEmpty = true;
    }
  }
  
  private PlayerChunk a(int i, int j, boolean flag)
  {
    long k = i + 2147483647L | j + 2147483647L << 32;
    PlayerChunk playerchunk = (PlayerChunk)c.getEntry(k);
    
    if ((playerchunk == null) && (flag)) {
      playerchunk = new PlayerChunk(this, i, j);
      c.put(k, playerchunk);
      e.add(playerchunk);
    }
    
    return playerchunk;
  }
  
  public final boolean isChunkInUse(int x, int z) {
    PlayerChunk pi = a(x, z, false);
    if (pi != null) {
      return PlayerChunk.b(pi).size() > 0;
    }
    return false;
  }
  
  public void flagDirty(int i, int j, int k)
  {
    int l = i >> 4;
    int i1 = k >> 4;
    PlayerChunk playerchunk = a(l, i1, false);
    
    if (playerchunk != null) {
      playerchunk.a(i & 0xF, j, k & 0xF);
    }
  }
  
  public void addPlayer(EntityPlayer entityplayer) {
    int i = (int)locX >> 4;
    int j = (int)locZ >> 4;
    
    d = locX;
    e = locZ;
    

    List<ChunkCoordIntPair> chunkList = new LinkedList();
    for (int k = i - f; k <= i + f; k++) {
      for (int l = j - f; l <= j + f; l++) {
        chunkList.add(new ChunkCoordIntPair(k, l));
      }
    }
    
    Collections.sort(chunkList, new ChunkCoordComparator(entityplayer));
    for (ChunkCoordIntPair pair : chunkList) {
      a(x, z, true).a(entityplayer);
    }
    

    managedPlayers.add(entityplayer);
    b(entityplayer);
  }
  
  public void b(EntityPlayer entityplayer) {
    ArrayList arraylist = new ArrayList(chunkCoordIntPairQueue);
    int i = 0;
    int j = f;
    int k = (int)locX >> 4;
    int l = (int)locZ >> 4;
    int i1 = 0;
    int j1 = 0;
    ChunkCoordIntPair chunkcoordintpair = PlayerChunk.a(a(k, l, true));
    
    chunkCoordIntPairQueue.clear();
    if (arraylist.contains(chunkcoordintpair)) {
      chunkCoordIntPairQueue.add(chunkcoordintpair);
    }
    


    for (int k1 = 1; k1 <= j * 2; k1++) {
      for (int l1 = 0; l1 < 2; l1++) {
        int[] aint = h[(i++ % 4)];
        
        for (int i2 = 0; i2 < k1; i2++) {
          i1 += aint[0];
          j1 += aint[1];
          chunkcoordintpair = PlayerChunk.a(a(k + i1, l + j1, true));
          if (arraylist.contains(chunkcoordintpair)) {
            chunkCoordIntPairQueue.add(chunkcoordintpair);
          }
        }
      }
    }
    
    i %= 4;
    
    for (k1 = 0; k1 < j * 2; k1++) {
      i1 += h[i][0];
      j1 += h[i][1];
      chunkcoordintpair = PlayerChunk.a(a(k + i1, l + j1, true));
      if (arraylist.contains(chunkcoordintpair)) {
        chunkCoordIntPairQueue.add(chunkcoordintpair);
      }
    }
  }
  
  public void removePlayer(EntityPlayer entityplayer) {
    int i = (int)d >> 4;
    int j = (int)e >> 4;
    
    for (int k = i - f; k <= i + f; k++) {
      for (int l = j - f; l <= j + f; l++) {
        PlayerChunk playerchunk = a(k, l, false);
        
        if (playerchunk != null) {
          playerchunk.b(entityplayer);
        }
      }
    }
    
    managedPlayers.remove(entityplayer);
  }
  
  private boolean a(int i, int j, int k, int l, int i1) {
    int j1 = i - k;
    int k1 = j - l;
    
    return (k1 >= -i1) && (k1 <= i1);
  }
  
  public void movePlayer(EntityPlayer entityplayer) {
    int i = (int)locX >> 4;
    int j = (int)locZ >> 4;
    double d0 = d - locX;
    double d1 = e - locZ;
    double d2 = d0 * d0 + d1 * d1;
    
    if (d2 >= 64.0D) {
      int k = (int)d >> 4;
      int l = (int)e >> 4;
      int i1 = f;
      int j1 = i - k;
      int k1 = j - l;
      List<ChunkCoordIntPair> chunksToLoad = new LinkedList();
      
      if ((j1 != 0) || (k1 != 0)) {
        for (int l1 = i - i1; l1 <= i + i1; l1++) {
          for (int i2 = j - i1; i2 <= j + i1; i2++) {
            if (!a(l1, i2, k, l, i1)) {
              chunksToLoad.add(new ChunkCoordIntPair(l1, i2));
            }
            
            if (!a(l1 - j1, i2 - k1, i, j, i1)) {
              PlayerChunk playerchunk = a(l1 - j1, i2 - k1, false);
              
              if (playerchunk != null) {
                playerchunk.b(entityplayer);
              }
            }
          }
        }
        
        b(entityplayer);
        d = locX;
        e = locZ;
        

        Collections.sort(chunksToLoad, new ChunkCoordComparator(entityplayer));
        for (ChunkCoordIntPair pair : chunksToLoad) {
          a(x, z, true).a(entityplayer);
        }
        
        if ((i1 > 1) || (i1 < -1) || (j1 > 1) || (j1 < -1)) {
          Collections.sort(chunkCoordIntPairQueue, new ChunkCoordComparator(entityplayer));
        }
      }
    }
  }
  
  public boolean a(EntityPlayer entityplayer, int i, int j)
  {
    PlayerChunk playerchunk = a(i, j, false);
    
    return playerchunk != null;
  }
  
  public static int getFurthestViewableBlock(int i) {
    return i * 16 - 16;
  }
  
  static WorldServer a(PlayerChunkMap playerchunkmap) {
    return world;
  }
  
  static LongHashMap b(PlayerChunkMap playerchunkmap) {
    return c;
  }
  
  static Queue c(PlayerChunkMap playermanager) {
    return e;
  }
  
  static Queue d(PlayerChunkMap playermanager) {
    return d;
  }
  
  private static class ChunkCoordComparator implements Comparator<ChunkCoordIntPair>
  {
    private int x;
    private int z;
    
    public ChunkCoordComparator(EntityPlayer entityplayer) {
      x = ((int)locX >> 4);
      z = ((int)locZ >> 4);
    }
    
    public int compare(ChunkCoordIntPair a, ChunkCoordIntPair b) {
      if (a.equals(b)) {
        return 0;
      }
      

      int ax = x - x;
      int az = z - z;
      int bx = x - x;
      int bz = z - z;
      
      int result = (ax - bx) * (ax + bx) + (az - bz) * (az + bz);
      if (result != 0) {
        return result;
      }
      
      if (ax < 0) {
        if (bx < 0) {
          return bz - az;
        }
        return -1;
      }
      
      if (bx < 0) {
        return 1;
      }
      return az - bz;
    }
  }
}
