package net.minecraft.server.v1_7_R1;

import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.PortUnreachableException;
import java.net.SocketAddress;
import java.net.SocketException;
import java.net.SocketTimeoutException;
import java.net.UnknownHostException;
import java.util.Date;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

public class RemoteStatusListener extends RemoteConnectionThread
{
  private long clearedTime;
  private int bindPort;
  private int serverPort;
  private int maxPlayers;
  private String localAddress;
  private String worldName;
  private DatagramSocket socket;
  private byte[] o = new byte[1460];
  
  private DatagramPacket p;
  private Map q;
  private String hostname;
  private String motd;
  private Map challenges;
  private long u;
  private RemoteStatusReply cachedReply;
  private long cacheTime;
  
  public RemoteStatusListener(IMinecraftServer paramIMinecraftServer)
  {
    super(paramIMinecraftServer, "Query Listener");
    
    bindPort = paramIMinecraftServer.a("query.port", 0);
    motd = paramIMinecraftServer.x();
    serverPort = paramIMinecraftServer.y();
    localAddress = paramIMinecraftServer.z();
    maxPlayers = paramIMinecraftServer.C();
    worldName = paramIMinecraftServer.M();
    

    cacheTime = 0L;
    
    hostname = "0.0.0.0";
    

    if ((0 == motd.length()) || (hostname.equals(motd)))
    {
      motd = "0.0.0.0";
      try {
        InetAddress localInetAddress = InetAddress.getLocalHost();
        hostname = localInetAddress.getHostAddress();
      } catch (UnknownHostException localUnknownHostException) {
        warning("Unable to determine local host IP, please set server-ip in '" + paramIMinecraftServer.b() + "' : " + localUnknownHostException.getMessage());
      }
    } else {
      hostname = motd;
    }
    

    if (0 == bindPort)
    {
      bindPort = serverPort;
      info("Setting default query port to " + bindPort);
      paramIMinecraftServer.a("query.port", Integer.valueOf(bindPort));
      paramIMinecraftServer.a("debug", Boolean.valueOf(false));
      paramIMinecraftServer.a();
    }
    
    q = new java.util.HashMap();
    cachedReply = new RemoteStatusReply(1460);
    challenges = new java.util.HashMap();
    u = new Date().getTime();
  }
  
  private void send(byte[] paramArrayOfByte, DatagramPacket paramDatagramPacket) {
    socket.send(new DatagramPacket(paramArrayOfByte, paramArrayOfByte.length, paramDatagramPacket.getSocketAddress()));
  }
  
  private boolean parsePacket(DatagramPacket paramDatagramPacket) {
    byte[] arrayOfByte = paramDatagramPacket.getData();
    int i = paramDatagramPacket.getLength();
    SocketAddress localSocketAddress = paramDatagramPacket.getSocketAddress();
    debug("Packet len " + i + " [" + localSocketAddress + "]");
    if ((3 > i) || (-2 != arrayOfByte[0]) || (-3 != arrayOfByte[1]))
    {
      debug("Invalid packet [" + localSocketAddress + "]");
      return false;
    }
    

    debug("Packet '" + StatusChallengeUtils.a(arrayOfByte[2]) + "' [" + localSocketAddress + "]");
    switch (arrayOfByte[2])
    {
    case 9: 
      createChallenge(paramDatagramPacket);
      debug("Challenge [" + localSocketAddress + "]");
      return true;
    

    case 0: 
      if (!hasChallenged(paramDatagramPacket).booleanValue()) {
        debug("Invalid challenge [" + localSocketAddress + "]");
        return false;
      }
      
      if (15 == i)
      {
        send(getFullReply(paramDatagramPacket), paramDatagramPacket);
        debug("Rules [" + localSocketAddress + "]");
      }
      else {
        RemoteStatusReply localRemoteStatusReply = new RemoteStatusReply(1460);
        localRemoteStatusReply.write(0);
        localRemoteStatusReply.write(getIdentityToken(paramDatagramPacket.getSocketAddress()));
        localRemoteStatusReply.write(localAddress);
        localRemoteStatusReply.write("SMP");
        localRemoteStatusReply.write(worldName);
        localRemoteStatusReply.write(Integer.toString(d()));
        localRemoteStatusReply.write(Integer.toString(maxPlayers));
        localRemoteStatusReply.write((short)serverPort);
        localRemoteStatusReply.write(hostname);
        
        send(localRemoteStatusReply.getBytes(), paramDatagramPacket);
        debug("Status [" + localSocketAddress + "]");
      }
      break;
    }
    return true;
  }
  
  private byte[] getFullReply(DatagramPacket paramDatagramPacket) {
    long l = MinecraftServer.ap();
    Object localObject2; if (l < cacheTime + 5000L)
    {
      localObject1 = cachedReply.getBytes();
      localObject2 = getIdentityToken(paramDatagramPacket.getSocketAddress());
      localObject1[1] = localObject2[0];
      localObject1[2] = localObject2[1];
      localObject1[3] = localObject2[2];
      localObject1[4] = localObject2[3];
      
      return localObject1;
    }
    
    cacheTime = l;
    
    cachedReply.reset();
    cachedReply.write(0);
    cachedReply.write(getIdentityToken(paramDatagramPacket.getSocketAddress()));
    cachedReply.write("splitnum");
    cachedReply.write(128);
    cachedReply.write(0);
    

    cachedReply.write("hostname");
    cachedReply.write(localAddress);
    cachedReply.write("gametype");
    cachedReply.write("SMP");
    cachedReply.write("game_id");
    cachedReply.write("MINECRAFT");
    cachedReply.write("version");
    cachedReply.write(server.getVersion());
    cachedReply.write("plugins");
    cachedReply.write(server.getPlugins());
    cachedReply.write("map");
    cachedReply.write(worldName);
    cachedReply.write("numplayers");
    cachedReply.write("" + d());
    cachedReply.write("maxplayers");
    cachedReply.write("" + maxPlayers);
    cachedReply.write("hostport");
    cachedReply.write("" + serverPort);
    cachedReply.write("hostip");
    cachedReply.write(hostname);
    cachedReply.write(0);
    cachedReply.write(1);
    


    cachedReply.write("player_");
    cachedReply.write(0);
    
    Object localObject1 = server.getPlayers();
    for (String str : localObject1) {
      cachedReply.write(str);
    }
    cachedReply.write(0);
    
    return cachedReply.getBytes();
  }
  
  private byte[] getIdentityToken(SocketAddress paramSocketAddress) {
    return ((RemoteStatusChallenge)challenges.get(paramSocketAddress)).getIdentityToken();
  }
  
  private Boolean hasChallenged(DatagramPacket paramDatagramPacket) {
    SocketAddress localSocketAddress = paramDatagramPacket.getSocketAddress();
    if (!challenges.containsKey(localSocketAddress))
    {
      return Boolean.valueOf(false);
    }
    
    byte[] arrayOfByte = paramDatagramPacket.getData();
    if (((RemoteStatusChallenge)challenges.get(localSocketAddress)).getToken() != StatusChallengeUtils.c(arrayOfByte, 7, paramDatagramPacket.getLength()))
    {
      return Boolean.valueOf(false);
    }
    

    return Boolean.valueOf(true);
  }
  
  private void createChallenge(DatagramPacket paramDatagramPacket) {
    RemoteStatusChallenge localRemoteStatusChallenge = new RemoteStatusChallenge(this, paramDatagramPacket);
    challenges.put(paramDatagramPacket.getSocketAddress(), localRemoteStatusChallenge);
    
    send(localRemoteStatusChallenge.getChallengeResponse(), paramDatagramPacket);
  }
  
  private void cleanChallenges() {
    if (!running) {
      return;
    }
    
    long l = MinecraftServer.ap();
    if (l < clearedTime + 30000L) {
      return;
    }
    clearedTime = l;
    
    Iterator localIterator = challenges.entrySet().iterator();
    while (localIterator.hasNext()) {
      Map.Entry localEntry = (Map.Entry)localIterator.next();
      if (((RemoteStatusChallenge)localEntry.getValue()).isExpired(l).booleanValue()) {
        localIterator.remove();
      }
    }
  }
  
  public void run()
  {
    info("Query running on " + motd + ":" + bindPort);
    clearedTime = MinecraftServer.ap();
    p = new DatagramPacket(o, o.length);
    try
    {
      while (running) {
        try {
          socket.receive(p);
          

          cleanChallenges();
          

          parsePacket(p);
        }
        catch (SocketTimeoutException localSocketTimeoutException) {
          cleanChallenges();

        }
        catch (PortUnreachableException localPortUnreachableException) {}catch (java.io.IOException localIOException)
        {
          a(localIOException);
        }
      }
    } finally {
      e();
    }
  }
  
  public void a()
  {
    if (running) {
      return;
    }
    
    if ((0 >= bindPort) || (65535 < bindPort)) {
      warning("Invalid query port " + bindPort + " found in '" + server.b() + "' (queries disabled)");
      return;
    }
    
    if (g()) {
      super.a();
    }
  }
  
  private void a(Exception paramException) {
    if (!running) {
      return;
    }
    

    warning("Unexpected exception, buggy JRE? (" + paramException.toString() + ")");
    

    if (!g()) {
      error("Failed to recover from buggy JRE, shutting down!");
      running = false;
    }
  }
  
  private boolean g() {
    try {
      socket = new DatagramSocket(bindPort, InetAddress.getByName(motd));
      a(socket);
      socket.setSoTimeout(500);
      return true;
    } catch (SocketException localSocketException) {
      warning("Unable to initialise query system on " + motd + ":" + bindPort + " (Socket): " + localSocketException.getMessage());
    } catch (UnknownHostException localUnknownHostException) {
      warning("Unable to initialise query system on " + motd + ":" + bindPort + " (Unknown Host): " + localUnknownHostException.getMessage());
    } catch (Exception localException) {
      warning("Unable to initialise query system on " + motd + ":" + bindPort + " (E): " + localException.getMessage());
    }
    
    return false;
  }
}
