package net.minecraft.server.v1_7_R1;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.UUID;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.bukkit.craftbukkit.v1_7_R1.entity.CraftPlayer;







public class WorldNBTStorage
  implements IDataManager, IPlayerFileData
{
  private static final Logger a = ;
  private final File baseDir;
  private final File playerDir;
  private final File dataDir;
  private final long sessionId = MinecraftServer.ap();
  private final String f;
  private UUID uuid = null;
  
  public WorldNBTStorage(File file1, String s, boolean flag) {
    baseDir = new File(file1, s);
    baseDir.mkdirs();
    playerDir = new File(baseDir, "players");
    dataDir = new File(baseDir, "data");
    dataDir.mkdirs();
    f = s;
    if (flag) {
      playerDir.mkdirs();
    }
    
    h();
  }
  
  private void h() {
    try {
      File file1 = new File(baseDir, "session.lock");
      DataOutputStream dataoutputstream = new DataOutputStream(new FileOutputStream(file1));
      try
      {
        dataoutputstream.writeLong(sessionId);
      } finally {
        dataoutputstream.close();
      }
    } catch (IOException ioexception) {
      ioexception.printStackTrace();
      throw new RuntimeException("Failed to check session lock, aborting");
    }
  }
  
  public File getDirectory() {
    return baseDir;
  }
  
  public void checkSession() throws ExceptionWorldConflict {
    try {
      File file1 = new File(baseDir, "session.lock");
      DataInputStream datainputstream = new DataInputStream(new FileInputStream(file1));
      try
      {
        if (datainputstream.readLong() != sessionId) {
          throw new ExceptionWorldConflict("The save is being accessed from another location, aborting");
        }
      } finally {
        datainputstream.close();
      }
    } catch (IOException ioexception) {
      throw new ExceptionWorldConflict("Failed to check session lock, aborting");
    }
  }
  
  public IChunkLoader createChunkLoader(WorldProvider worldprovider) {
    throw new RuntimeException("Old Chunk Storage is no longer supported.");
  }
  
  public WorldData getWorldData() {
    File file1 = new File(baseDir, "level.dat");
    
    NBTTagCompound nbttagcompound;
    NBTTagCompound nbttagcompound1;
    if (file1.exists()) {
      try {
        nbttagcompound = NBTCompressedStreamTools.a(new FileInputStream(file1));
        nbttagcompound1 = nbttagcompound.getCompound("Data");
        return new WorldData(nbttagcompound1);
      } catch (Exception exception) {
        exception.printStackTrace();
      }
    }
    
    file1 = new File(baseDir, "level.dat_old");
    if (file1.exists()) {
      try {
        nbttagcompound = NBTCompressedStreamTools.a(new FileInputStream(file1));
        nbttagcompound1 = nbttagcompound.getCompound("Data");
        return new WorldData(nbttagcompound1);
      } catch (Exception exception1) {
        exception1.printStackTrace();
      }
    }
    
    return null;
  }
  
  public void saveWorldData(WorldData worlddata, NBTTagCompound nbttagcompound) {
    NBTTagCompound nbttagcompound1 = worlddata.a(nbttagcompound);
    NBTTagCompound nbttagcompound2 = new NBTTagCompound();
    
    nbttagcompound2.set("Data", nbttagcompound1);
    try
    {
      File file1 = new File(baseDir, "level.dat_new");
      File file2 = new File(baseDir, "level.dat_old");
      File file3 = new File(baseDir, "level.dat");
      
      NBTCompressedStreamTools.a(nbttagcompound2, new FileOutputStream(file1));
      if (file2.exists()) {
        file2.delete();
      }
      
      file3.renameTo(file2);
      if (file3.exists()) {
        file3.delete();
      }
      
      file1.renameTo(file3);
      if (file1.exists()) {
        file1.delete();
      }
    } catch (Exception exception) {
      exception.printStackTrace();
    }
  }
  
  public void saveWorldData(WorldData worlddata) {
    NBTTagCompound nbttagcompound = worlddata.a();
    NBTTagCompound nbttagcompound1 = new NBTTagCompound();
    
    nbttagcompound1.set("Data", nbttagcompound);
    try
    {
      File file1 = new File(baseDir, "level.dat_new");
      File file2 = new File(baseDir, "level.dat_old");
      File file3 = new File(baseDir, "level.dat");
      
      NBTCompressedStreamTools.a(nbttagcompound1, new FileOutputStream(file1));
      if (file2.exists()) {
        file2.delete();
      }
      
      file3.renameTo(file2);
      if (file3.exists()) {
        file3.delete();
      }
      
      file1.renameTo(file3);
      if (file1.exists()) {
        file1.delete();
      }
    } catch (Exception exception) {
      exception.printStackTrace();
    }
  }
  
  public void save(EntityHuman entityhuman) {
    try {
      NBTTagCompound nbttagcompound = new NBTTagCompound();
      
      entityhuman.e(nbttagcompound);
      File file1 = new File(playerDir, entityhuman.getName() + ".dat.tmp");
      File file2 = new File(playerDir, entityhuman.getName() + ".dat");
      
      NBTCompressedStreamTools.a(nbttagcompound, new FileOutputStream(file1));
      if (file2.exists()) {
        file2.delete();
      }
      
      file1.renameTo(file2);
    } catch (Exception exception) {
      a.warn("Failed to save player data for " + entityhuman.getName());
    }
  }
  
  public NBTTagCompound load(EntityHuman entityhuman) {
    NBTTagCompound nbttagcompound = getPlayerData(entityhuman.getName());
    
    if (nbttagcompound != null)
    {
      if ((entityhuman instanceof EntityPlayer)) {
        CraftPlayer player = (CraftPlayer)bukkitEntity;
        player.setFirstPlayed(new File(playerDir, entityhuman.getName() + ".dat").lastModified());
      }
      
      entityhuman.f(nbttagcompound);
    }
    
    return nbttagcompound;
  }
  
  public NBTTagCompound getPlayerData(String s) {
    try {
      File file1 = new File(playerDir, s + ".dat");
      
      if (file1.exists()) {
        return NBTCompressedStreamTools.a(new FileInputStream(file1));
      }
    } catch (Exception exception) {
      a.warn("Failed to load player data for " + s);
    }
    
    return null;
  }
  
  public IPlayerFileData getPlayerFileData() {
    return this;
  }
  
  public String[] getSeenPlayers() {
    String[] astring = playerDir.list();
    
    for (int i = 0; i < astring.length; i++) {
      if (astring[i].endsWith(".dat")) {
        astring[i] = astring[i].substring(0, astring[i].length() - 4);
      }
    }
    
    return astring;
  }
  
  public void a() {}
  
  public File getDataFile(String s) {
    return new File(dataDir, s + ".dat");
  }
  
  public String g() {
    return f;
  }
  
  public UUID getUUID()
  {
    if (uuid != null) return uuid;
    File file1 = new File(baseDir, "uid.dat");
    if (file1.exists()) {
      DataInputStream dis = null;
      try {
        dis = new DataInputStream(new FileInputStream(file1));
        return this.uuid = new UUID(dis.readLong(), dis.readLong());
      } catch (IOException ex) {
        a.warn("Failed to read " + file1 + ", generating new random UUID", ex);
      } finally {
        if (dis != null) {
          try {
            dis.close();
          }
          catch (IOException ex) {}
        }
      }
    }
    
    uuid = UUID.randomUUID();
    DataOutputStream dos = null;
    try {
      dos = new DataOutputStream(new FileOutputStream(file1));
      dos.writeLong(uuid.getMostSignificantBits());
      dos.writeLong(uuid.getLeastSignificantBits());
      










      return uuid;
    }
    catch (IOException ex)
    {
      a.warn("Failed to write " + file1, ex);
    } finally {
      if (dos != null) {
        try {
          dos.close();
        }
        catch (IOException ex) {}
      }
    }
  }
  

  public File getPlayerDir()
  {
    return playerDir;
  }
}
