package net.minecraft.server.v1_7_R1;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.Set;
import java.util.TreeSet;
import org.bukkit.Location;
import org.bukkit.WeatherType;
import org.bukkit.World.Environment;
import org.bukkit.block.BlockState;
import org.bukkit.craftbukkit.v1_7_R1.CraftServer;
import org.bukkit.craftbukkit.v1_7_R1.CraftWorld;
import org.bukkit.craftbukkit.v1_7_R1.generator.InternalChunkGenerator;
import org.bukkit.craftbukkit.v1_7_R1.util.LongHash;
import org.bukkit.event.block.BlockFormEvent;
import org.bukkit.event.weather.LightningStrikeEvent;
import org.bukkit.event.weather.ThunderChangeEvent;
import org.bukkit.event.weather.WeatherChangeEvent;
import org.bukkit.generator.ChunkGenerator;
import org.bukkit.plugin.PluginManager;

public class WorldServer extends World implements org.bukkit.BlockChangeDelegate
{
  private static final org.apache.logging.log4j.Logger a = ;
  private final MinecraftServer server;
  public EntityTracker tracker;
  private final PlayerChunkMap manager;
  private Set M;
  private TreeSet N;
  public ChunkProviderServer chunkProviderServer;
  public boolean savingDisabled;
  private boolean O;
  private int emptyTime;
  private final PortalTravelAgent Q;
  private final SpawnerCreature R = new SpawnerCreature();
  private NoteDataList[] S = { new NoteDataList((EmptyClass2)null), new NoteDataList((EmptyClass2)null) };
  private int T;
  private static final StructurePieceTreasure[] U = { new StructurePieceTreasure(Items.STICK, 0, 1, 3, 10), new StructurePieceTreasure(Item.getItemOf(Blocks.WOOD), 0, 1, 3, 10), new StructurePieceTreasure(Item.getItemOf(Blocks.LOG), 0, 1, 3, 10), new StructurePieceTreasure(Items.STONE_AXE, 0, 1, 1, 3), new StructurePieceTreasure(Items.WOOD_AXE, 0, 1, 1, 5), new StructurePieceTreasure(Items.STONE_PICKAXE, 0, 1, 1, 3), new StructurePieceTreasure(Items.WOOD_PICKAXE, 0, 1, 1, 5), new StructurePieceTreasure(Items.APPLE, 0, 2, 3, 5), new StructurePieceTreasure(Items.BREAD, 0, 2, 3, 3), new StructurePieceTreasure(Item.getItemOf(Blocks.LOG2), 0, 1, 3, 10) };
  private List V = new ArrayList();
  
  private IntHashMap entitiesById;
  
  public final int dimension;
  
  public WorldServer(MinecraftServer minecraftserver, IDataManager idatamanager, String s, int i, WorldSettings worldsettings, MethodProfiler methodprofiler, World.Environment env, ChunkGenerator gen)
  {
    super(idatamanager, s, worldsettings, WorldProvider.byDimension(env.getId()), methodprofiler, gen, env);
    dimension = i;
    pvpMode = minecraftserver.getPvP();
    
    server = minecraftserver;
    tracker = new EntityTracker(this);
    manager = new PlayerChunkMap(this, minecraftserver.getPlayerList().o());
    if (entitiesById == null) {
      entitiesById = new IntHashMap();
    }
    
    if (M == null) {
      M = new HashSet();
    }
    
    if (N == null) {
      N = new TreeSet();
    }
    
    Q = new org.bukkit.craftbukkit.v1_7_R1.CraftTravelAgent(this);
    scoreboard = new ScoreboardServer(minecraftserver);
    PersistentScoreboard persistentscoreboard = (PersistentScoreboard)worldMaps.get(PersistentScoreboard.class, "scoreboard");
    
    if (persistentscoreboard == null) {
      persistentscoreboard = new PersistentScoreboard();
      worldMaps.a("scoreboard", persistentscoreboard);
    }
    
    persistentscoreboard.a(scoreboard);
    ((ScoreboardServer)scoreboard).a(persistentscoreboard);
  }
  

  public TileEntity getTileEntity(int i, int j, int k)
  {
    TileEntity result = super.getTileEntity(i, j, k);
    Block type = getType(i, j, k);
    
    if (type == Blocks.CHEST) {
      if (!(result instanceof TileEntityChest)) {
        result = fixTileEntity(i, j, k, type, result);
      }
    } else if (type == Blocks.FURNACE) {
      if (!(result instanceof TileEntityFurnace)) {
        result = fixTileEntity(i, j, k, type, result);
      }
    } else if (type == Blocks.DROPPER) {
      if (!(result instanceof TileEntityDropper)) {
        result = fixTileEntity(i, j, k, type, result);
      }
    } else if (type == Blocks.DISPENSER) {
      if (!(result instanceof TileEntityDispenser)) {
        result = fixTileEntity(i, j, k, type, result);
      }
    } else if (type == Blocks.JUKEBOX) {
      if (!(result instanceof TileEntityRecordPlayer)) {
        result = fixTileEntity(i, j, k, type, result);
      }
    } else if (type == Blocks.NOTE_BLOCK) {
      if (!(result instanceof TileEntityNote)) {
        result = fixTileEntity(i, j, k, type, result);
      }
    } else if (type == Blocks.MOB_SPAWNER) {
      if (!(result instanceof TileEntityMobSpawner)) {
        result = fixTileEntity(i, j, k, type, result);
      }
    } else if ((type == Blocks.SIGN_POST) || (type == Blocks.WALL_SIGN)) {
      if (!(result instanceof TileEntitySign)) {
        result = fixTileEntity(i, j, k, type, result);
      }
    } else if (type == Blocks.ENDER_CHEST) {
      if (!(result instanceof TileEntityEnderChest)) {
        result = fixTileEntity(i, j, k, type, result);
      }
    } else if (type == Blocks.BREWING_STAND) {
      if (!(result instanceof TileEntityBrewingStand)) {
        result = fixTileEntity(i, j, k, type, result);
      }
    } else if (type == Blocks.BEACON) {
      if (!(result instanceof TileEntityBeacon)) {
        result = fixTileEntity(i, j, k, type, result);
      }
    } else if ((type == Blocks.HOPPER) && 
      (!(result instanceof TileEntityHopper))) {
      result = fixTileEntity(i, j, k, type, result);
    }
    

    return result;
  }
  
  private TileEntity fixTileEntity(int x, int y, int z, Block type, TileEntity found) {
    getServer().getLogger().severe("Block at " + x + "," + y + "," + z + " is " + org.bukkit.Material.getMaterial(Block.b(type)).toString() + " but has " + found + ". " + "Bukkit will attempt to fix this, but there may be additional damage that we cannot recover.");
    

    if ((type instanceof IContainer)) {
      TileEntity replacement = ((IContainer)type).a(this, getData(x, y, z));
      world = this;
      setTileEntity(x, y, z, replacement);
      return replacement;
    }
    getServer().getLogger().severe("Don't know how to fix for this type... Can't do anything! :(");
    return found;
  }
  
  private boolean canSpawn(int x, int z)
  {
    if (generator != null) {
      return generator.canSpawn(getWorld(), x, z);
    }
    return worldProvider.canSpawn(x, z);
  }
  

  public void doTick()
  {
    super.doTick();
    if ((getWorldData().isHardcore()) && (difficulty != EnumDifficulty.HARD)) {
      difficulty = EnumDifficulty.HARD;
    }
    
    worldProvider.e.b();
    if (everyoneDeeplySleeping()) {
      if (getGameRules().getBoolean("doDaylightCycle")) {
        long i = worldData.getDayTime() + 24000L;
        
        worldData.setDayTime(i - i % 24000L);
      }
      
      d();
    }
    
    methodProfiler.a("mobSpawner");
    
    long time = worldData.getTime();
    if ((getGameRules().getBoolean("doMobSpawning")) && ((allowMonsters) || (allowAnimals)) && ((this instanceof WorldServer)) && (players.size() > 0)) {
      R.spawnEntities(this, (allowMonsters) && (ticksPerMonsterSpawns != 0L) && (time % ticksPerMonsterSpawns == 0L), (allowAnimals) && (ticksPerAnimalSpawns != 0L) && (time % ticksPerAnimalSpawns == 0L), worldData.getTime() % 400L == 0L);
    }
    

    methodProfiler.c("chunkSource");
    chunkProvider.unloadChunks();
    int j = a(1.0F);
    
    if (j != this.j) {
      this.j = j;
    }
    
    worldData.setTime(worldData.getTime() + 1L);
    if (getGameRules().getBoolean("doDaylightCycle")) {
      worldData.setDayTime(worldData.getDayTime() + 1L);
    }
    
    methodProfiler.c("tickPending");
    a(false);
    methodProfiler.c("tickBlocks");
    g();
    methodProfiler.c("chunkMap");
    manager.flush();
    methodProfiler.c("village");
    villages.tick();
    siegeManager.a();
    methodProfiler.c("portalForcer");
    Q.a(getTime());
    methodProfiler.b();
    Z();
    
    getWorld().processChunkGC();
  }
  
  public BiomeMeta a(EnumCreatureType enumcreaturetype, int i, int j, int k) {
    List list = K().getMobsFor(enumcreaturetype, i, j, k);
    
    return (list != null) && (!list.isEmpty()) ? (BiomeMeta)WeightedRandom.a(random, list) : null;
  }
  
  public void everyoneSleeping() {
    O = (!players.isEmpty());
    Iterator iterator = players.iterator();
    
    while (iterator.hasNext()) {
      EntityHuman entityhuman = (EntityHuman)iterator.next();
      
      if ((!entityhuman.isSleeping()) && (!fauxSleeping)) {
        O = false;
        break;
      }
    }
  }
  
  protected void d() {
    O = false;
    Iterator iterator = players.iterator();
    
    while (iterator.hasNext()) {
      EntityHuman entityhuman = (EntityHuman)iterator.next();
      
      if (entityhuman.isSleeping()) {
        entityhuman.a(false, false, true);
      }
    }
    
    Y();
  }
  
  private void Y()
  {
    WeatherChangeEvent weather = new WeatherChangeEvent(getWorld(), false);
    getServer().getPluginManager().callEvent(weather);
    
    ThunderChangeEvent thunder = new ThunderChangeEvent(getWorld(), false);
    getServer().getPluginManager().callEvent(thunder);
    if (!weather.isCancelled()) {
      worldData.setWeatherDuration(0);
      worldData.setStorm(false);
    }
    if (!thunder.isCancelled()) {
      worldData.setThunderDuration(0);
      worldData.setThundering(false);
    }
  }
  
  public boolean everyoneDeeplySleeping()
  {
    if ((O) && (!isStatic)) {
      Iterator iterator = players.iterator();
      

      boolean foundActualSleepers = false;
      
      EntityHuman entityhuman;
      do
      {
        if (!iterator.hasNext()) {
          return foundActualSleepers;
        }
        
        entityhuman = (EntityHuman)iterator.next();
        
        if (entityhuman.isDeeplySleeping()) {
          foundActualSleepers = true;
        }
      } while ((entityhuman.isDeeplySleeping()) || (fauxSleeping));
      

      return false;
    }
    return false;
  }
  
  protected void g()
  {
    super.g();
    int i = 0;
    int j = 0;
    


    for (long chunkCoord : chunkTickList.popAll())
    {
      int chunkX = LongHash.msw(chunkCoord);
      int chunkZ = LongHash.lsw(chunkCoord);
      int k = chunkX * 16;
      int l = chunkZ * 16;
      
      methodProfiler.a("getChunk");
      Chunk chunk = getChunkAt(chunkX, chunkZ);
      

      a(k, l, chunk);
      methodProfiler.c("tickChunk");
      chunk.b(false);
      methodProfiler.c("thunder");
      




      if ((random.nextInt(100000) == 0) && (P()) && (O())) {
        this.k = (this.k * 3 + 1013904223);
        int i1 = this.k >> 2;
        int j1 = k + (i1 & 0xF);
        int k1 = l + (i1 >> 8 & 0xF);
        int l1 = h(j1, k1);
        if (isRainingAt(j1, l1, k1)) {
          strikeLightning(new EntityLightning(this, j1, l1, k1));
        }
      }
      
      methodProfiler.c("iceandsnow");
      if (random.nextInt(16) == 0) {
        this.k = (this.k * 3 + 1013904223);
        int i1 = this.k >> 2;
        int j1 = i1 & 0xF;
        int k1 = i1 >> 8 & 0xF;
        int l1 = h(j1 + k, k1 + l);
        if (s(j1 + k, l1 - 1, k1 + l))
        {
          BlockState blockState = getWorld().getBlockAt(j1 + k, l1 - 1, k1 + l).getState();
          blockState.setTypeId(Block.b(Blocks.ICE));
          
          BlockFormEvent iceBlockForm = new BlockFormEvent(blockState.getBlock(), blockState);
          getServer().getPluginManager().callEvent(iceBlockForm);
          if (!iceBlockForm.isCancelled()) {
            blockState.update(true);
          }
        }
        

        if ((P()) && (e(j1 + k, l1, k1 + l, true)))
        {
          BlockState blockState = getWorld().getBlockAt(j1 + k, l1, k1 + l).getState();
          blockState.setTypeId(Block.b(Blocks.SNOW));
          
          BlockFormEvent snow = new BlockFormEvent(blockState.getBlock(), blockState);
          getServer().getPluginManager().callEvent(snow);
          if (!snow.isCancelled()) {
            blockState.update(true);
          }
        }
        

        if (P()) {
          BiomeBase biomebase = getBiome(j1 + k, k1 + l);
          
          if (biomebase.e()) {
            getType(j1 + k, l1 - 1, k1 + l).l(this, j1 + k, l1 - 1, k1 + l);
          }
        }
      }
      
      methodProfiler.c("tickBlocks");
      ChunkSection[] achunksection = chunk.i();
      
      int j1 = achunksection.length;
      
      for (int k1 = 0; k1 < j1; k1++) {
        ChunkSection chunksection = achunksection[k1];
        
        if ((chunksection != null) && (chunksection.shouldTick())) {
          for (int i2 = 0; i2 < 3; i2++) {
            this.k = (this.k * 3 + 1013904223);
            int j2 = this.k >> 2;
            int k2 = j2 & 0xF;
            int l2 = j2 >> 8 & 0xF;
            int i3 = j2 >> 16 & 0xF;
            
            j++;
            Block block = chunksection.getTypeId(k2, i3, l2);
            
            if (block.isTicking()) {
              i++;
              block.a(this, k2 + k, i3 + chunksection.getYPosition(), l2 + l, random);
            }
          }
        }
      }
      
      methodProfiler.b();
    }
  }
  
  public boolean a(int i, int j, int k, Block block) {
    NextTickListEntry nextticklistentry = new NextTickListEntry(i, j, k, block);
    
    return V.contains(nextticklistentry);
  }
  
  public void a(int i, int j, int k, Block block, int l) {
    a(i, j, k, block, l, 0);
  }
  
  public void a(int i, int j, int k, Block block, int l, int i1) {
    NextTickListEntry nextticklistentry = new NextTickListEntry(i, j, k, block);
    byte b0 = 0;
    
    if ((d) && (block.getMaterial() != Material.AIR)) {
      if (block.L()) {
        b0 = 8;
        if (b(a - b0, b - b0, c - b0, a + b0, b + b0, c + b0)) {
          Block block1 = getType(a, b, c);
          
          if ((block1.getMaterial() != Material.AIR) && (block1 == nextticklistentry.a())) {
            block1.a(this, a, b, c, random);
          }
        }
        
        return;
      }
      
      l = 1;
    }
    
    if (b(i - b0, j - b0, k - b0, i + b0, j + b0, k + b0)) {
      if (block.getMaterial() != Material.AIR) {
        nextticklistentry.a(l + worldData.getTime());
        nextticklistentry.a(i1);
      }
      
      if (!M.contains(nextticklistentry)) {
        M.add(nextticklistentry);
        N.add(nextticklistentry);
      }
    }
  }
  
  public void b(int i, int j, int k, Block block, int l, int i1) {
    NextTickListEntry nextticklistentry = new NextTickListEntry(i, j, k, block);
    
    nextticklistentry.a(i1);
    if (block.getMaterial() != Material.AIR) {
      nextticklistentry.a(l + worldData.getTime());
    }
    
    if (!M.contains(nextticklistentry)) {
      M.add(nextticklistentry);
      N.add(nextticklistentry);
    }
  }
  




  public void tickEntities()
  {
    i();
    

    super.tickEntities();
  }
  
  public void i() {
    emptyTime = 0;
  }
  
  public boolean a(boolean flag) {
    int i = N.size();
    
    if (i != M.size()) {
      throw new IllegalStateException("TickNextTick list out of synch");
    }
    if (i > 1000)
    {
      if (i > 20000) {
        i /= 20;
      } else {
        i = 1000;
      }
    }
    

    methodProfiler.a("cleaning");
    


    for (int j = 0; j < i; j++) {
      NextTickListEntry nextticklistentry = (NextTickListEntry)N.first();
      if ((!flag) && (d > worldData.getTime())) {
        break;
      }
      
      N.remove(nextticklistentry);
      M.remove(nextticklistentry);
      V.add(nextticklistentry);
    }
    
    methodProfiler.b();
    methodProfiler.a("ticking");
    Iterator iterator = V.iterator();
    
    while (iterator.hasNext()) {
      NextTickListEntry nextticklistentry = (NextTickListEntry)iterator.next();
      iterator.remove();
      byte b0 = 0;
      
      if (b(a - b0, b - b0, c - b0, a + b0, b + b0, c + b0)) {
        Block block = getType(a, b, c);
        
        if ((block.getMaterial() != Material.AIR) && (Block.a(block, nextticklistentry.a()))) {
          try {
            block.a(this, a, b, c, random);
          } catch (Throwable throwable) {
            CrashReport crashreport = CrashReport.a(throwable, "Exception while ticking a block");
            CrashReportSystemDetails crashreportsystemdetails = crashreport.a("Block being ticked");
            
            int k;
            try
            {
              k = getData(a, b, c);
            } catch (Throwable throwable1) {
              k = -1;
            }
            
            CrashReportSystemDetails.a(crashreportsystemdetails, a, b, c, block, k);
            throw new ReportedException(crashreport);
          }
        }
      } else {
        a(a, b, c, nextticklistentry.a(), 0);
      }
    }
    
    methodProfiler.b();
    V.clear();
    return !N.isEmpty();
  }
  
  public List a(Chunk chunk, boolean flag)
  {
    ArrayList arraylist = null;
    ChunkCoordIntPair chunkcoordintpair = chunk.l();
    int i = (x << 4) - 2;
    int j = i + 16 + 2;
    int k = (z << 4) - 2;
    int l = k + 16 + 2;
    
    for (int i1 = 0; i1 < 2; i1++) {
      Iterator iterator;
      Iterator iterator;
      if (i1 == 0) {
        iterator = N.iterator();
      } else {
        iterator = V.iterator();
        if (!V.isEmpty()) {
          a.debug("toBeTicked = " + V.size());
        }
      }
      
      while (iterator.hasNext()) {
        NextTickListEntry nextticklistentry = (NextTickListEntry)iterator.next();
        
        if ((a >= i) && (a < j) && (c >= k) && (c < l)) {
          if (flag) {
            M.remove(nextticklistentry);
            iterator.remove();
          }
          
          if (arraylist == null) {
            arraylist = new ArrayList();
          }
          
          arraylist.add(nextticklistentry);
        }
      }
    }
    
    return arraylist;
  }
  













  protected IChunkProvider j()
  {
    IChunkLoader ichunkloader = dataManager.createChunkLoader(worldProvider);
    
    InternalChunkGenerator gen;
    
    InternalChunkGenerator gen;
    if (generator != null) {
      gen = new org.bukkit.craftbukkit.v1_7_R1.generator.CustomChunkGenerator(this, getSeed(), generator); } else { InternalChunkGenerator gen;
      if ((worldProvider instanceof WorldProviderHell)) {
        gen = new org.bukkit.craftbukkit.v1_7_R1.generator.NetherChunkGenerator(this, getSeed()); } else { InternalChunkGenerator gen;
        if ((worldProvider instanceof WorldProviderTheEnd)) {
          gen = new org.bukkit.craftbukkit.v1_7_R1.generator.SkyLandsChunkGenerator(this, getSeed());
        } else
          gen = new org.bukkit.craftbukkit.v1_7_R1.generator.NormalChunkGenerator(this, getSeed());
      }
    }
    chunkProviderServer = new ChunkProviderServer(this, ichunkloader, gen);
    

    return chunkProviderServer;
  }
  
  public List getTileEntities(int i, int j, int k, int l, int i1, int j1) {
    ArrayList arraylist = new ArrayList();
    

    for (int chunkX = i >> 4; chunkX <= l - 1 >> 4; chunkX++) {
      for (int chunkZ = k >> 4; chunkZ <= j1 - 1 >> 4; chunkZ++) {
        Chunk chunk = getChunkAt(chunkX, chunkZ);
        if (chunk != null)
        {


          for (Object te : tileEntities.values()) {
            TileEntity tileentity = (TileEntity)te;
            if ((x >= i) && (y >= j) && (z >= k) && (x < l) && (y < i1) && (z < j1)) {
              arraylist.add(tileentity);
            }
          }
        }
      }
    }
    
    return arraylist;
  }
  
  public boolean a(EntityHuman entityhuman, int i, int j, int k) {
    return !server.a(this, i, j, k, entityhuman);
  }
  
  protected void a(WorldSettings worldsettings) {
    if (entitiesById == null) {
      entitiesById = new IntHashMap();
    }
    
    if (M == null) {
      M = new HashSet();
    }
    
    if (N == null) {
      N = new TreeSet();
    }
    
    b(worldsettings);
    super.a(worldsettings);
  }
  
  protected void b(WorldSettings worldsettings) {
    if (!worldProvider.e()) {
      worldData.setSpawn(0, worldProvider.getSeaLevel(), 0);
    } else {
      isLoading = true;
      WorldChunkManager worldchunkmanager = worldProvider.e;
      List list = worldchunkmanager.a();
      Random random = new Random(getSeed());
      ChunkPosition chunkposition = worldchunkmanager.a(0, 0, 256, list, random);
      int i = 0;
      int j = worldProvider.getSeaLevel();
      int k = 0;
      

      if (generator != null) {
        Random rand = new Random(getSeed());
        Location spawn = generator.getFixedSpawnLocation(getWorld(), rand);
        
        if (spawn != null) {
          if (spawn.getWorld() != getWorld()) {
            throw new IllegalStateException("Cannot set spawn point for " + worldData.getName() + " to be in another world (" + spawn.getWorld().getName() + ")");
          }
          worldData.setSpawn(spawn.getBlockX(), spawn.getBlockY(), spawn.getBlockZ());
          isLoading = false;
          return;
        }
      }
      


      if (chunkposition != null) {
        i = x;
        k = z;
      } else {
        a.warn("Unable to find spawn biome");
      }
      
      int l = 0;
      
      while (!canSpawn(i, k)) {
        i += random.nextInt(64) - random.nextInt(64);
        k += random.nextInt(64) - random.nextInt(64);
        l++;
        if (l == 1000) {
          break;
        }
      }
      
      worldData.setSpawn(i, j, k);
      isLoading = false;
      if (worldsettings.c()) {
        k();
      }
    }
  }
  
  protected void k() {
    WorldGenBonusChest worldgenbonuschest = new WorldGenBonusChest(U, 10);
    
    for (int i = 0; i < 10; i++) {
      int j = worldData.c() + random.nextInt(6) - random.nextInt(6);
      int k = worldData.e() + random.nextInt(6) - random.nextInt(6);
      int l = i(j, k) + 1;
      
      if (worldgenbonuschest.a(this, random, j, l, k)) {
        break;
      }
    }
  }
  
  public ChunkCoordinates getDimensionSpawn() {
    return worldProvider.h();
  }
  
  public void save(boolean flag, IProgressUpdate iprogressupdate) throws ExceptionWorldConflict {
    if (chunkProvider.canSave()) {
      if (iprogressupdate != null) {
        iprogressupdate.a("Saving level");
      }
      
      a();
      if (iprogressupdate != null) {
        iprogressupdate.c("Saving chunks");
      }
      
      chunkProvider.saveChunks(flag, iprogressupdate);
    }
  }
  
  public void flushSave() {
    if (chunkProvider.canSave()) {
      chunkProvider.b();
    }
  }
  
  protected void a() throws ExceptionWorldConflict {
    F();
    dataManager.saveWorldData(worldData, server.getPlayerList().q());
    
    if (!(this instanceof SecondaryWorldServer)) {
      worldMaps.a();
    }
  }
  
  protected void a(Entity entity)
  {
    super.a(entity);
    entitiesById.a(entity.getId(), entity);
    Entity[] aentity = entity.at();
    
    if (aentity != null) {
      for (int i = 0; i < aentity.length; i++) {
        entitiesById.a(aentity[i].getId(), aentity[i]);
      }
    }
  }
  
  protected void b(Entity entity) {
    super.b(entity);
    entitiesById.d(entity.getId());
    Entity[] aentity = entity.at();
    
    if (aentity != null) {
      for (int i = 0; i < aentity.length; i++) {
        entitiesById.d(aentity[i].getId());
      }
    }
  }
  
  public Entity getEntity(int i) {
    return (Entity)entitiesById.get(i);
  }
  
  public boolean strikeLightning(Entity entity)
  {
    LightningStrikeEvent lightning = new LightningStrikeEvent(getWorld(), (org.bukkit.entity.LightningStrike)entity.getBukkitEntity());
    getServer().getPluginManager().callEvent(lightning);
    
    if (lightning.isCancelled()) {
      return false;
    }
    
    if (super.strikeLightning(entity)) {
      server.getPlayerList().sendPacketNearby(locX, locY, locZ, 512.0D, dimension, new PacketPlayOutSpawnEntityWeather(entity));
      
      return true;
    }
    return false;
  }
  
  public void broadcastEntityEffect(Entity entity, byte b0)
  {
    getTracker().sendPacketToEntity(entity, new PacketPlayOutEntityStatus(entity, b0));
  }
  
  public Explosion createExplosion(Entity entity, double d0, double d1, double d2, float f, boolean flag, boolean flag1)
  {
    Explosion explosion = super.createExplosion(entity, d0, d1, d2, f, flag, flag1);
    
    if (wasCanceled) {
      return explosion;
    }
    








    if (!flag1) {
      blocks.clear();
    }
    
    Iterator iterator = players.iterator();
    
    while (iterator.hasNext()) {
      EntityHuman entityhuman = (EntityHuman)iterator.next();
      
      if (entityhuman.e(d0, d1, d2) < 4096.0D) {
        playerConnection.sendPacket(new PacketPlayOutExplosion(d0, d1, d2, f, blocks, (Vec3D)explosion.b().get(entityhuman)));
      }
    }
    
    return explosion;
  }
  
  public void playNote(int i, int j, int k, Block block, int l, int i1) {
    NoteBlockData noteblockdata = new NoteBlockData(i, j, k, block, l, i1);
    Iterator iterator = S[T].iterator();
    
    NoteBlockData noteblockdata1;
    do
    {
      if (!iterator.hasNext()) {
        S[T].add(noteblockdata);
        return;
      }
      
      noteblockdata1 = (NoteBlockData)iterator.next();
    } while (!noteblockdata1.equals(noteblockdata));
  }
  
  private void Z()
  {
    while (!S[T].isEmpty()) {
      int i = T;
      
      T ^= 0x1;
      Iterator iterator = S[i].iterator();
      
      while (iterator.hasNext()) {
        NoteBlockData noteblockdata = (NoteBlockData)iterator.next();
        
        if (a(noteblockdata))
        {
          server.getPlayerList().sendPacketNearby(noteblockdata.a(), noteblockdata.b(), noteblockdata.c(), 64.0D, dimension, new PacketPlayOutBlockAction(noteblockdata.a(), noteblockdata.b(), noteblockdata.c(), noteblockdata.f(), noteblockdata.d(), noteblockdata.e()));
        }
      }
      
      S[i].clear();
    }
  }
  
  private boolean a(NoteBlockData noteblockdata) {
    Block block = getType(noteblockdata.a(), noteblockdata.b(), noteblockdata.c());
    
    return block == noteblockdata.f() ? block.a(this, noteblockdata.a(), noteblockdata.b(), noteblockdata.c(), noteblockdata.d(), noteblockdata.e()) : false;
  }
  
  public void saveLevel() {
    dataManager.a();
  }
  
  protected void o() {
    boolean flag = P();
    
    super.o();
    



















    if (flag != P())
    {
      for (int i = 0; i < players.size(); i++) {
        if (players.get(i)).world == this) {
          ((EntityPlayer)players.get(i)).setPlayerWeather(!flag ? WeatherType.DOWNFALL : WeatherType.CLEAR, false);
        }
      }
    }
  }
  
  public MinecraftServer getMinecraftServer()
  {
    return server;
  }
  
  public EntityTracker getTracker() {
    return tracker;
  }
  
  public PlayerChunkMap getPlayerChunkMap() {
    return manager;
  }
  
  public PortalTravelAgent t() {
    return Q;
  }
  
  public void a(String s, double d0, double d1, double d2, int i, double d3, double d4, double d5, double d6) {
    PacketPlayOutWorldParticles packetplayoutworldparticles = new PacketPlayOutWorldParticles(s, (float)d0, (float)d1, (float)d2, (float)d3, (float)d4, (float)d5, (float)d6, i);
    
    for (int j = 0; j < players.size(); j++) {
      EntityPlayer entityplayer = (EntityPlayer)players.get(j);
      ChunkCoordinates chunkcoordinates = entityplayer.getChunkCoordinates();
      double d7 = d0 - x;
      double d8 = d1 - y;
      double d9 = d2 - z;
      double d10 = d7 * d7 + d8 * d8 + d9 * d9;
      
      if (d10 <= 256.0D) {
        playerConnection.sendPacket(packetplayoutworldparticles);
      }
    }
  }
  
  public boolean setRawTypeId(int x, int y, int z, int typeId)
  {
    return setTypeAndData(x, y, z, Block.e(typeId), 0, 4);
  }
  
  public boolean setRawTypeIdAndData(int x, int y, int z, int typeId, int data) {
    return setTypeAndData(x, y, z, Block.e(typeId), data, 4);
  }
  
  public boolean setTypeId(int x, int y, int z, int typeId) {
    return setTypeAndData(x, y, z, Block.e(typeId), 0, 3);
  }
  
  public boolean setTypeIdAndData(int x, int y, int z, int typeId, int data) {
    return setTypeAndData(x, y, z, Block.e(typeId), data, 3);
  }
  
  public int getTypeId(int x, int y, int z) {
    return Block.b(getType(x, y, z));
  }
}
