package net.minecraft.util.com.google.common.cache;

import java.util.Map;
import java.util.concurrent.ExecutionException;
import net.minecraft.util.com.google.common.annotations.Beta;
import net.minecraft.util.com.google.common.collect.ImmutableMap;
import net.minecraft.util.com.google.common.collect.Maps;
import net.minecraft.util.com.google.common.util.concurrent.UncheckedExecutionException;



































@Beta
public abstract class AbstractLoadingCache<K, V>
  extends AbstractCache<K, V>
  implements LoadingCache<K, V>
{
  public V getUnchecked(K key)
  {
    try
    {
      return get(key);
    } catch (ExecutionException e) {
      throw new UncheckedExecutionException(e.getCause());
    }
  }
  
  public ImmutableMap<K, V> getAll(Iterable<? extends K> keys) throws ExecutionException
  {
    Map<K, V> result = Maps.newLinkedHashMap();
    for (K key : keys) {
      if (!result.containsKey(key)) {
        result.put(key, get(key));
      }
    }
    return ImmutableMap.copyOf(result);
  }
  
  public final V apply(K key)
  {
    return getUnchecked(key);
  }
  
  public void refresh(K key)
  {
    throw new UnsupportedOperationException();
  }
}
