package net.minecraft.util.com.google.common.cache;

import java.io.Serializable;
import java.util.Map;
import net.minecraft.util.com.google.common.annotations.Beta;
import net.minecraft.util.com.google.common.annotations.GwtCompatible;
import net.minecraft.util.com.google.common.annotations.GwtIncompatible;
import net.minecraft.util.com.google.common.base.Function;
import net.minecraft.util.com.google.common.base.Preconditions;
import net.minecraft.util.com.google.common.base.Supplier;
import net.minecraft.util.com.google.common.util.concurrent.Futures;
import net.minecraft.util.com.google.common.util.concurrent.ListenableFuture;





































































@GwtCompatible(emulated=true)
public abstract class CacheLoader<K, V>
{
  public abstract V load(K paramK)
    throws Exception;
  
  @GwtIncompatible("Futures")
  public ListenableFuture<V> reload(K key, V oldValue)
    throws Exception
  {
    Preconditions.checkNotNull(key);
    Preconditions.checkNotNull(oldValue);
    return Futures.immediateFuture(load(key));
  }
  






















  public Map<K, V> loadAll(Iterable<? extends K> keys)
    throws Exception
  {
    throw new UnsupportedLoadingOperationException();
  }
  







  @Beta
  public static <K, V> CacheLoader<K, V> from(Function<K, V> function)
  {
    return new FunctionToCacheLoader(function);
  }
  
  private static final class FunctionToCacheLoader<K, V> extends CacheLoader<K, V> implements Serializable {
    private final Function<K, V> computingFunction;
    private static final long serialVersionUID = 0L;
    
    public FunctionToCacheLoader(Function<K, V> computingFunction) {
      this.computingFunction = ((Function)Preconditions.checkNotNull(computingFunction));
    }
    
    public V load(K key)
    {
      return computingFunction.apply(Preconditions.checkNotNull(key));
    }
  }
  










  @Beta
  public static <V> CacheLoader<Object, V> from(Supplier<V> supplier)
  {
    return new SupplierToCacheLoader(supplier);
  }
  
  private static final class SupplierToCacheLoader<V> extends CacheLoader<Object, V> implements Serializable {
    private final Supplier<V> computingSupplier;
    private static final long serialVersionUID = 0L;
    
    public SupplierToCacheLoader(Supplier<V> computingSupplier) {
      this.computingSupplier = ((Supplier)Preconditions.checkNotNull(computingSupplier));
    }
    
    public V load(Object key)
    {
      Preconditions.checkNotNull(key);
      return computingSupplier.get();
    }
  }
  

  static final class UnsupportedLoadingOperationException
    extends UnsupportedOperationException
  {}
  

  public static final class InvalidCacheLoadException
    extends RuntimeException
  {
    public InvalidCacheLoadException(String message)
    {
      super();
    }
  }
}
