package net.minecraft.util.com.google.common.collect;

import java.util.ArrayDeque;
import java.util.Collection;
import java.util.Queue;
import net.minecraft.util.com.google.common.annotations.Beta;
import net.minecraft.util.com.google.common.annotations.GwtIncompatible;
import net.minecraft.util.com.google.common.base.Preconditions;































@Beta
@GwtIncompatible("java.util.ArrayDeque")
public final class EvictingQueue<E>
  extends ForwardingQueue<E>
{
  private final Queue<E> delegate;
  private final int maxSize;
  
  private EvictingQueue(int maxSize)
  {
    Preconditions.checkArgument(maxSize >= 0, "maxSize (%s) must >= 0", new Object[] { Integer.valueOf(maxSize) });
    delegate = new ArrayDeque(maxSize);
    this.maxSize = maxSize;
  }
  





  public static <E> EvictingQueue<E> create(int maxSize)
  {
    return new EvictingQueue(maxSize);
  }
  
  protected Queue<E> delegate() {
    return delegate;
  }
  





  public boolean offer(E e)
  {
    return add(e);
  }
  





  public boolean add(E e)
  {
    Preconditions.checkNotNull(e);
    if (maxSize == 0) {
      return true;
    }
    if (size() == maxSize) {
      delegate.remove();
    }
    delegate.add(e);
    return true;
  }
  
  public boolean addAll(Collection<? extends E> collection) {
    return standardAddAll(collection);
  }
  
  public boolean contains(Object object)
  {
    return delegate().contains(Preconditions.checkNotNull(object));
  }
  
  public boolean remove(Object object)
  {
    return delegate().remove(Preconditions.checkNotNull(object));
  }
}
