package net.minecraft.util.com.google.common.collect;

import java.util.Comparator;
import java.util.Iterator;
import java.util.NoSuchElementException;
import java.util.SortedSet;
import javax.annotation.Nullable;
import net.minecraft.util.com.google.common.annotations.Beta;
import net.minecraft.util.com.google.common.annotations.GwtCompatible;
















































@GwtCompatible
public abstract class ForwardingSortedSet<E>
  extends ForwardingSet<E>
  implements SortedSet<E>
{
  protected abstract SortedSet<E> delegate();
  
  public Comparator<? super E> comparator()
  {
    return delegate().comparator();
  }
  
  public E first()
  {
    return delegate().first();
  }
  
  public SortedSet<E> headSet(E toElement)
  {
    return delegate().headSet(toElement);
  }
  
  public E last()
  {
    return delegate().last();
  }
  
  public SortedSet<E> subSet(E fromElement, E toElement)
  {
    return delegate().subSet(fromElement, toElement);
  }
  
  public SortedSet<E> tailSet(E fromElement)
  {
    return delegate().tailSet(fromElement);
  }
  

  private int unsafeCompare(Object o1, Object o2)
  {
    Comparator<? super E> comparator = comparator();
    return comparator == null ? ((Comparable)o1).compareTo(o2) : comparator.compare(o1, o2);
  }
  








  @Beta
  protected boolean standardContains(@Nullable Object object)
  {
    try
    {
      SortedSet<Object> self = this;
      Object ceiling = self.tailSet(object).first();
      return unsafeCompare(ceiling, object) == 0;
    } catch (ClassCastException e) {
      return false;
    } catch (NoSuchElementException e) {
      return false;
    } catch (NullPointerException e) {}
    return false;
  }
  







  @Beta
  protected boolean standardRemove(@Nullable Object object)
  {
    try
    {
      SortedSet<Object> self = this;
      Iterator<Object> iterator = self.tailSet(object).iterator();
      if (iterator.hasNext()) {
        Object ceiling = iterator.next();
        if (unsafeCompare(ceiling, object) == 0) {
          iterator.remove();
          return true;
        }
      }
    } catch (ClassCastException e) {
      return false;
    } catch (NullPointerException e) {
      return false;
    }
    return false;
  }
  






  @Beta
  protected SortedSet<E> standardSubSet(E fromElement, E toElement)
  {
    return tailSet(fromElement).headSet(toElement);
  }
}
