package net.minecraft.util.com.google.common.collect;

import java.io.Serializable;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import net.minecraft.util.com.google.common.annotations.Beta;
import net.minecraft.util.com.google.common.annotations.GwtIncompatible;
import net.minecraft.util.com.google.common.base.Preconditions;


































































@Beta
@GwtIncompatible("hasn't been tested yet")
public abstract class ImmutableSortedMultiset<E>
  extends ImmutableSortedMultisetFauxverideShim<E>
  implements SortedMultiset<E>
{
  private static final Comparator<Comparable> NATURAL_ORDER = ;
  
  private static final ImmutableSortedMultiset<Comparable> NATURAL_EMPTY_MULTISET = new EmptyImmutableSortedMultiset(NATURAL_ORDER);
  

  transient ImmutableSortedMultiset<E> descendingMultiset;
  

  public static <E> ImmutableSortedMultiset<E> of()
  {
    return NATURAL_EMPTY_MULTISET;
  }
  


  public static <E extends Comparable<? super E>> ImmutableSortedMultiset<E> of(E element)
  {
    RegularImmutableSortedSet<E> elementSet = (RegularImmutableSortedSet)ImmutableSortedSet.of(element);
    
    int[] counts = { 1 };
    long[] cumulativeCounts = { 0L, 1L };
    return new RegularImmutableSortedMultiset(elementSet, counts, cumulativeCounts, 0, 1);
  }
  






  public static <E extends Comparable<? super E>> ImmutableSortedMultiset<E> of(E e1, E e2)
  {
    return copyOf(Ordering.natural(), Arrays.asList(new Comparable[] { e1, e2 }));
  }
  






  public static <E extends Comparable<? super E>> ImmutableSortedMultiset<E> of(E e1, E e2, E e3)
  {
    return copyOf(Ordering.natural(), Arrays.asList(new Comparable[] { e1, e2, e3 }));
  }
  







  public static <E extends Comparable<? super E>> ImmutableSortedMultiset<E> of(E e1, E e2, E e3, E e4)
  {
    return copyOf(Ordering.natural(), Arrays.asList(new Comparable[] { e1, e2, e3, e4 }));
  }
  







  public static <E extends Comparable<? super E>> ImmutableSortedMultiset<E> of(E e1, E e2, E e3, E e4, E e5)
  {
    return copyOf(Ordering.natural(), Arrays.asList(new Comparable[] { e1, e2, e3, e4, e5 }));
  }
  







  public static <E extends Comparable<? super E>> ImmutableSortedMultiset<E> of(E e1, E e2, E e3, E e4, E e5, E e6, E... remaining)
  {
    int size = remaining.length + 6;
    List<E> all = Lists.newArrayListWithCapacity(size);
    Collections.addAll(all, new Comparable[] { e1, e2, e3, e4, e5, e6 });
    Collections.addAll(all, remaining);
    return copyOf(Ordering.natural(), all);
  }
  





  public static <E extends Comparable<? super E>> ImmutableSortedMultiset<E> copyOf(E[] elements)
  {
    return copyOf(Ordering.natural(), Arrays.asList(elements));
  }
  
























  public static <E> ImmutableSortedMultiset<E> copyOf(Iterable<? extends E> elements)
  {
    Ordering<E> naturalOrder = Ordering.natural();
    return copyOf(naturalOrder, elements);
  }
  












  public static <E> ImmutableSortedMultiset<E> copyOf(Iterator<? extends E> elements)
  {
    Ordering<E> naturalOrder = Ordering.natural();
    return copyOf(naturalOrder, elements);
  }
  






  public static <E> ImmutableSortedMultiset<E> copyOf(Comparator<? super E> comparator, Iterator<? extends E> elements)
  {
    Preconditions.checkNotNull(comparator);
    return new Builder(comparator).addAll(elements).build();
  }
  










  public static <E> ImmutableSortedMultiset<E> copyOf(Comparator<? super E> comparator, Iterable<? extends E> elements)
  {
    if ((elements instanceof ImmutableSortedMultiset))
    {
      ImmutableSortedMultiset<E> multiset = (ImmutableSortedMultiset)elements;
      if (comparator.equals(multiset.comparator())) {
        if (multiset.isPartialView()) {
          return copyOfSortedEntries(comparator, multiset.entrySet().asList());
        }
        return multiset;
      }
    }
    
    elements = Lists.newArrayList(elements);
    TreeMultiset<E> sortedCopy = TreeMultiset.create((Comparator)Preconditions.checkNotNull(comparator));
    Iterables.addAll(sortedCopy, elements);
    return copyOfSortedEntries(comparator, sortedCopy.entrySet());
  }
  













  public static <E> ImmutableSortedMultiset<E> copyOfSorted(SortedMultiset<E> sortedMultiset)
  {
    return copyOfSortedEntries(sortedMultiset.comparator(), Lists.newArrayList(sortedMultiset.entrySet()));
  }
  

  private static <E> ImmutableSortedMultiset<E> copyOfSortedEntries(Comparator<? super E> comparator, Collection<Multiset.Entry<E>> entries)
  {
    if (entries.isEmpty()) {
      return emptyMultiset(comparator);
    }
    ImmutableList.Builder<E> elementsBuilder = new ImmutableList.Builder(entries.size());
    int[] counts = new int[entries.size()];
    long[] cumulativeCounts = new long[entries.size() + 1];
    int i = 0;
    for (Multiset.Entry<E> entry : entries) {
      elementsBuilder.add(entry.getElement());
      counts[i] = entry.getCount();
      cumulativeCounts[(i + 1)] = (cumulativeCounts[i] + counts[i]);
      i++;
    }
    return new RegularImmutableSortedMultiset(new RegularImmutableSortedSet(elementsBuilder.build(), comparator), counts, cumulativeCounts, 0, entries.size());
  }
  


  static <E> ImmutableSortedMultiset<E> emptyMultiset(Comparator<? super E> comparator)
  {
    if (NATURAL_ORDER.equals(comparator)) {
      return NATURAL_EMPTY_MULTISET;
    }
    return new EmptyImmutableSortedMultiset(comparator);
  }
  


  public final Comparator<? super E> comparator()
  {
    return elementSet().comparator();
  }
  


  public abstract ImmutableSortedSet<E> elementSet();
  

  public ImmutableSortedMultiset<E> descendingMultiset()
  {
    ImmutableSortedMultiset<E> result = descendingMultiset;
    if (result == null) {
      return this.descendingMultiset = new DescendingImmutableSortedMultiset(this);
    }
    return result;
  }
  








  @Deprecated
  public final Multiset.Entry<E> pollFirstEntry()
  {
    throw new UnsupportedOperationException();
  }
  








  @Deprecated
  public final Multiset.Entry<E> pollLastEntry()
  {
    throw new UnsupportedOperationException();
  }
  

  public abstract ImmutableSortedMultiset<E> headMultiset(E paramE, BoundType paramBoundType);
  

  public ImmutableSortedMultiset<E> subMultiset(E lowerBound, BoundType lowerBoundType, E upperBound, BoundType upperBoundType)
  {
    Preconditions.checkArgument(comparator().compare(lowerBound, upperBound) <= 0, "Expected lowerBound <= upperBound but %s > %s", new Object[] { lowerBound, upperBound });
    
    return tailMultiset(lowerBound, lowerBoundType).headMultiset(upperBound, upperBoundType);
  }
  




  public abstract ImmutableSortedMultiset<E> tailMultiset(E paramE, BoundType paramBoundType);
  




  public static <E> Builder<E> orderedBy(Comparator<E> comparator)
  {
    return new Builder(comparator);
  }
  







  public static <E extends Comparable<E>> Builder<E> reverseOrder()
  {
    return new Builder(Ordering.natural().reverse());
  }
  









  public static <E extends Comparable<E>> Builder<E> naturalOrder()
  {
    return new Builder(Ordering.natural());
  }
  










  public static class Builder<E>
    extends ImmutableMultiset.Builder<E>
  {
    private final Comparator<? super E> comparator;
    










    public Builder(Comparator<? super E> comparator)
    {
      super();
      this.comparator = ((Comparator)Preconditions.checkNotNull(comparator));
    }
    







    public Builder<E> add(E element)
    {
      super.add(element);
      return this;
    }
    











    public Builder<E> addCopies(E element, int occurrences)
    {
      super.addCopies(element, occurrences);
      return this;
    }
    










    public Builder<E> setCount(E element, int count)
    {
      super.setCount(element, count);
      return this;
    }
    







    public Builder<E> add(E... elements)
    {
      super.add(elements);
      return this;
    }
    







    public Builder<E> addAll(Iterable<? extends E> elements)
    {
      super.addAll(elements);
      return this;
    }
    







    public Builder<E> addAll(Iterator<? extends E> elements)
    {
      super.addAll(elements);
      return this;
    }
    




    public ImmutableSortedMultiset<E> build()
    {
      return ImmutableSortedMultiset.copyOfSorted((SortedMultiset)contents);
    }
  }
  
  private static final class SerializedForm implements Serializable {
    Comparator comparator;
    Object[] elements;
    int[] counts;
    
    SerializedForm(SortedMultiset<?> multiset) {
      comparator = multiset.comparator();
      int n = multiset.entrySet().size();
      elements = new Object[n];
      counts = new int[n];
      int i = 0;
      for (Multiset.Entry<?> entry : multiset.entrySet()) {
        elements[i] = entry.getElement();
        counts[i] = entry.getCount();
        i++;
      }
    }
    
    Object readResolve()
    {
      int n = elements.length;
      ImmutableSortedMultiset.Builder<Object> builder = ImmutableSortedMultiset.orderedBy(comparator);
      for (int i = 0; i < n; i++) {
        builder.addCopies(elements[i], counts[i]);
      }
      return builder.build();
    }
  }
  
  Object writeReplace()
  {
    return new SerializedForm(this);
  }
}
