package net.minecraft.util.com.google.common.collect;

import net.minecraft.util.com.google.common.annotations.GwtCompatible;
import net.minecraft.util.com.google.common.annotations.VisibleForTesting;






















@GwtCompatible(serializable=true, emulated=true)
final class RegularImmutableSet<E>
  extends ImmutableSet<E>
{
  private final Object[] elements;
  @VisibleForTesting
  final transient Object[] table;
  private final transient int mask;
  private final transient int hashCode;
  
  RegularImmutableSet(Object[] elements, int hashCode, Object[] table, int mask)
  {
    this.elements = elements;
    this.table = table;
    this.mask = mask;
    this.hashCode = hashCode;
  }
  
  public boolean contains(Object target) {
    if (target == null) {
      return false;
    }
    for (int i = Hashing.smear(target.hashCode());; i++) {
      Object candidate = table[(i & mask)];
      if (candidate == null) {
        return false;
      }
      if (candidate.equals(target)) {
        return true;
      }
    }
  }
  
  public int size()
  {
    return elements.length;
  }
  

  public UnmodifiableIterator<E> iterator()
  {
    return Iterators.forArray(elements);
  }
  
  int copyIntoArray(Object[] dst, int offset)
  {
    System.arraycopy(elements, 0, dst, offset, elements.length);
    return offset + elements.length;
  }
  
  ImmutableList<E> createAsList()
  {
    return new RegularImmutableAsList(this, elements);
  }
  
  boolean isPartialView()
  {
    return false;
  }
  
  public int hashCode() {
    return hashCode;
  }
  
  boolean isHashCodeFast() {
    return true;
  }
}
