package net.minecraft.util.com.google.common.escape;

import net.minecraft.util.com.google.common.annotations.Beta;
import net.minecraft.util.com.google.common.annotations.GwtCompatible;
import net.minecraft.util.com.google.common.base.Preconditions;












































@Beta
@GwtCompatible
public abstract class CharEscaper
  extends Escaper
{
  private static final int DEST_PAD = 32;
  
  public String escape(String string)
  {
    Preconditions.checkNotNull(string);
    
    int length = string.length();
    for (int index = 0; index < length; index++) {
      if (escape(string.charAt(index)) != null) {
        return escapeSlow(string, index);
      }
    }
    return string;
  }
  










  protected final String escapeSlow(String s, int index)
  {
    int slen = s.length();
    

    char[] dest = Platform.charBufferFromThreadLocal();
    int destSize = dest.length;
    int destIndex = 0;
    int lastEscape = 0;
    for (; 
        

        index < slen; index++)
    {

      char[] r = escape(s.charAt(index));
      

      if (r != null)
      {
        int rlen = r.length;
        int charsSkipped = index - lastEscape;
        


        int sizeNeeded = destIndex + charsSkipped + rlen;
        if (destSize < sizeNeeded) {
          destSize = sizeNeeded + (slen - index) + 32;
          dest = growBuffer(dest, destIndex, destSize);
        }
        

        if (charsSkipped > 0) {
          s.getChars(lastEscape, index, dest, destIndex);
          destIndex += charsSkipped;
        }
        

        if (rlen > 0) {
          System.arraycopy(r, 0, dest, destIndex, rlen);
          destIndex += rlen;
        }
        lastEscape = index + 1;
      }
    }
    
    int charsLeft = slen - lastEscape;
    if (charsLeft > 0) {
      int sizeNeeded = destIndex + charsLeft;
      if (destSize < sizeNeeded)
      {

        dest = growBuffer(dest, destIndex, sizeNeeded);
      }
      s.getChars(lastEscape, slen, dest, destIndex);
      destIndex = sizeNeeded;
    }
    return new String(dest, 0, destIndex);
  }
  









  protected abstract char[] escape(char paramChar);
  









  private static char[] growBuffer(char[] dest, int index, int size)
  {
    char[] copy = new char[size];
    if (index > 0) {
      System.arraycopy(dest, 0, copy, 0, index);
    }
    return copy;
  }
}
