package net.minecraft.util.com.google.common.eventbus;

import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Set;
import javax.annotation.Nullable;
import net.minecraft.util.com.google.common.base.Objects;
import net.minecraft.util.com.google.common.base.Throwables;
import net.minecraft.util.com.google.common.cache.CacheBuilder;
import net.minecraft.util.com.google.common.cache.CacheLoader;
import net.minecraft.util.com.google.common.cache.LoadingCache;
import net.minecraft.util.com.google.common.collect.HashMultimap;
import net.minecraft.util.com.google.common.collect.ImmutableList;
import net.minecraft.util.com.google.common.collect.Maps;
import net.minecraft.util.com.google.common.collect.Multimap;
import net.minecraft.util.com.google.common.reflect.TypeToken;
import net.minecraft.util.com.google.common.reflect.TypeToken.TypeSet;
import net.minecraft.util.com.google.common.util.concurrent.UncheckedExecutionException;





























class AnnotatedHandlerFinder
  implements HandlerFindingStrategy
{
  private static final LoadingCache<Class<?>, ImmutableList<Method>> handlerMethodsCache = CacheBuilder.newBuilder().weakKeys().build(new CacheLoader()
  {

    public ImmutableList<Method> load(Class<?> concreteClass)
      throws Exception
    {
      return AnnotatedHandlerFinder.getAnnotatedMethodsInternal(concreteClass);
    }
  });
  













  public Multimap<Class<?>, EventHandler> findAllHandlers(Object listener)
  {
    Multimap<Class<?>, EventHandler> methodsInListener = HashMultimap.create();
    Class<?> clazz = listener.getClass();
    for (Method method : getAnnotatedMethods(clazz)) {
      Class<?>[] parameterTypes = method.getParameterTypes();
      Class<?> eventType = parameterTypes[0];
      EventHandler handler = makeHandler(listener, method);
      methodsInListener.put(eventType, handler);
    }
    return methodsInListener;
  }
  
  private static ImmutableList<Method> getAnnotatedMethods(Class<?> clazz) {
    try {
      return (ImmutableList)handlerMethodsCache.getUnchecked(clazz);
    } catch (UncheckedExecutionException e) {
      throw Throwables.propagate(e.getCause());
    }
  }
  
  private static final class MethodIdentifier {
    private final String name;
    private final List<Class<?>> parameterTypes;
    
    MethodIdentifier(Method method) {
      name = method.getName();
      parameterTypes = Arrays.asList(method.getParameterTypes());
    }
    
    public int hashCode()
    {
      return Objects.hashCode(new Object[] { name, parameterTypes });
    }
    
    public boolean equals(@Nullable Object o)
    {
      if ((o instanceof MethodIdentifier)) {
        MethodIdentifier ident = (MethodIdentifier)o;
        return (name.equals(name)) && (parameterTypes.equals(parameterTypes));
      }
      return false;
    }
  }
  
  private static ImmutableList<Method> getAnnotatedMethodsInternal(Class<?> clazz) {
    Set<? extends Class<?>> supers = TypeToken.of(clazz).getTypes().rawTypes();
    Map<MethodIdentifier, Method> identifiers = Maps.newHashMap();
    for (Class<?> superClazz : supers) {
      for (Method superClazzMethod : superClazz.getMethods()) {
        if (superClazzMethod.isAnnotationPresent(Subscribe.class)) {
          Class<?>[] parameterTypes = superClazzMethod.getParameterTypes();
          if (parameterTypes.length != 1) {
            throw new IllegalArgumentException("Method " + superClazzMethod + " has @Subscribe annotation, but requires " + parameterTypes.length + " arguments.  Event handler methods must require a single argument.");
          }
          


          MethodIdentifier ident = new MethodIdentifier(superClazzMethod);
          if (!identifiers.containsKey(ident)) {
            identifiers.put(ident, superClazzMethod);
          }
        }
      }
    }
    return ImmutableList.copyOf(identifiers.values());
  }
  



  private static EventHandler makeHandler(Object listener, Method method)
  {
    EventHandler wrapper;
    


    EventHandler wrapper;
    


    if (methodIsDeclaredThreadSafe(method)) {
      wrapper = new EventHandler(listener, method);
    } else {
      wrapper = new SynchronizedEventHandler(listener, method);
    }
    return wrapper;
  }
  







  private static boolean methodIsDeclaredThreadSafe(Method method)
  {
    return method.getAnnotation(AllowConcurrentEvents.class) != null;
  }
}
