package net.minecraft.util.com.google.common.primitives;

import java.io.Serializable;
import java.util.AbstractList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.RandomAccess;
import net.minecraft.util.com.google.common.annotations.GwtCompatible;
import net.minecraft.util.com.google.common.annotations.GwtIncompatible;
import net.minecraft.util.com.google.common.base.Preconditions;



















































@GwtCompatible(emulated=true)
public final class Shorts
{
  public static final int BYTES = 2;
  public static final short MAX_POWER_OF_TWO = 16384;
  
  public static int hashCode(short value)
  {
    return value;
  }
  








  public static short checkedCast(long value)
  {
    short result = (short)(int)value;
    Preconditions.checkArgument(result == value, "Out of range: %s", new Object[] { Long.valueOf(value) });
    return result;
  }
  







  public static short saturatedCast(long value)
  {
    if (value > 32767L) {
      return 32767;
    }
    if (value < -32768L) {
      return -32768;
    }
    return (short)(int)value;
  }
  








  public static int compare(short a, short b)
  {
    return a - b;
  }
  








  public static boolean contains(short[] array, short target)
  {
    for (short value : array) {
      if (value == target) {
        return true;
      }
    }
    return false;
  }
  








  public static int indexOf(short[] array, short target)
  {
    return indexOf(array, target, 0, array.length);
  }
  

  private static int indexOf(short[] array, short target, int start, int end)
  {
    for (int i = start; i < end; i++) {
      if (array[i] == target) {
        return i;
      }
    }
    return -1;
  }
  










  public static int indexOf(short[] array, short[] target)
  {
    Preconditions.checkNotNull(array, "array");
    Preconditions.checkNotNull(target, "target");
    if (target.length == 0) {
      return 0;
    }
    
    label64:
    for (int i = 0; i < array.length - target.length + 1; i++) {
      for (int j = 0; j < target.length; j++) {
        if (array[(i + j)] != target[j]) {
          break label64;
        }
      }
      return i;
    }
    return -1;
  }
  








  public static int lastIndexOf(short[] array, short target)
  {
    return lastIndexOf(array, target, 0, array.length);
  }
  

  private static int lastIndexOf(short[] array, short target, int start, int end)
  {
    for (int i = end - 1; i >= start; i--) {
      if (array[i] == target) {
        return i;
      }
    }
    return -1;
  }
  







  public static short min(short... array)
  {
    Preconditions.checkArgument(array.length > 0);
    short min = array[0];
    for (int i = 1; i < array.length; i++) {
      if (array[i] < min) {
        min = array[i];
      }
    }
    return min;
  }
  







  public static short max(short... array)
  {
    Preconditions.checkArgument(array.length > 0);
    short max = array[0];
    for (int i = 1; i < array.length; i++) {
      if (array[i] > max) {
        max = array[i];
      }
    }
    return max;
  }
  








  public static short[] concat(short[]... arrays)
  {
    int length = 0;
    for (short[] array : arrays) {
      length += array.length;
    }
    short[] result = new short[length];
    int pos = 0;
    for (short[] array : arrays) {
      System.arraycopy(array, 0, result, pos, array.length);
      pos += array.length;
    }
    return result;
  }
  











  @GwtIncompatible("doesn't work")
  public static byte[] toByteArray(short value)
  {
    return new byte[] { (byte)(value >> 8), (byte)value };
  }
  













  @GwtIncompatible("doesn't work")
  public static short fromByteArray(byte[] bytes)
  {
    Preconditions.checkArgument(bytes.length >= 2, "array too small: %s < %s", new Object[] { Integer.valueOf(bytes.length), Integer.valueOf(2) });
    
    return fromBytes(bytes[0], bytes[1]);
  }
  






  @GwtIncompatible("doesn't work")
  public static short fromBytes(byte b1, byte b2)
  {
    return (short)(b1 << 8 | b2 & 0xFF);
  }
  
















  public static short[] ensureCapacity(short[] array, int minLength, int padding)
  {
    Preconditions.checkArgument(minLength >= 0, "Invalid minLength: %s", new Object[] { Integer.valueOf(minLength) });
    Preconditions.checkArgument(padding >= 0, "Invalid padding: %s", new Object[] { Integer.valueOf(padding) });
    return array.length < minLength ? copyOf(array, minLength + padding) : array;
  }
  


  private static short[] copyOf(short[] original, int length)
  {
    short[] copy = new short[length];
    System.arraycopy(original, 0, copy, 0, Math.min(original.length, length));
    return copy;
  }
  








  public static String join(String separator, short... array)
  {
    Preconditions.checkNotNull(separator);
    if (array.length == 0) {
      return "";
    }
    

    StringBuilder builder = new StringBuilder(array.length * 6);
    builder.append(array[0]);
    for (int i = 1; i < array.length; i++) {
      builder.append(separator).append(array[i]);
    }
    return builder.toString();
  }
  















  public static Comparator<short[]> lexicographicalComparator()
  {
    return LexicographicalComparator.INSTANCE;
  }
  
  private static enum LexicographicalComparator implements Comparator<short[]> {
    INSTANCE;
    
    private LexicographicalComparator() {}
    
    public int compare(short[] left, short[] right) { int minLength = Math.min(left.length, right.length);
      for (int i = 0; i < minLength; i++) {
        int result = Shorts.compare(left[i], right[i]);
        if (result != 0) {
          return result;
        }
      }
      return left.length - right.length;
    }
  }
  














  public static short[] toArray(Collection<? extends Number> collection)
  {
    if ((collection instanceof ShortArrayAsList)) {
      return ((ShortArrayAsList)collection).toShortArray();
    }
    
    Object[] boxedArray = collection.toArray();
    int len = boxedArray.length;
    short[] array = new short[len];
    for (int i = 0; i < len; i++)
    {
      array[i] = ((Number)Preconditions.checkNotNull(boxedArray[i])).shortValue();
    }
    return array;
  }
  













  public static List<Short> asList(short... backingArray)
  {
    if (backingArray.length == 0) {
      return Collections.emptyList();
    }
    return new ShortArrayAsList(backingArray);
  }
  
  @GwtCompatible
  private static class ShortArrayAsList extends AbstractList<Short> implements RandomAccess, Serializable {
    final short[] array;
    final int start;
    final int end;
    private static final long serialVersionUID = 0L;
    
    ShortArrayAsList(short[] array) {
      this(array, 0, array.length);
    }
    
    ShortArrayAsList(short[] array, int start, int end) {
      this.array = array;
      this.start = start;
      this.end = end;
    }
    
    public int size() {
      return end - start;
    }
    
    public boolean isEmpty() {
      return false;
    }
    
    public Short get(int index) {
      Preconditions.checkElementIndex(index, size());
      return Short.valueOf(array[(start + index)]);
    }
    
    public boolean contains(Object target)
    {
      return ((target instanceof Short)) && (Shorts.indexOf(array, ((Short)target).shortValue(), start, end) != -1);
    }
    

    public int indexOf(Object target)
    {
      if ((target instanceof Short)) {
        int i = Shorts.indexOf(array, ((Short)target).shortValue(), start, end);
        if (i >= 0) {
          return i - start;
        }
      }
      return -1;
    }
    
    public int lastIndexOf(Object target)
    {
      if ((target instanceof Short)) {
        int i = Shorts.lastIndexOf(array, ((Short)target).shortValue(), start, end);
        if (i >= 0) {
          return i - start;
        }
      }
      return -1;
    }
    
    public Short set(int index, Short element) {
      Preconditions.checkElementIndex(index, size());
      short oldValue = array[(start + index)];
      
      array[(start + index)] = ((Short)Preconditions.checkNotNull(element)).shortValue();
      return Short.valueOf(oldValue);
    }
    
    public List<Short> subList(int fromIndex, int toIndex) {
      int size = size();
      Preconditions.checkPositionIndexes(fromIndex, toIndex, size);
      if (fromIndex == toIndex) {
        return Collections.emptyList();
      }
      return new ShortArrayAsList(array, start + fromIndex, start + toIndex);
    }
    
    public boolean equals(Object object) {
      if (object == this) {
        return true;
      }
      if ((object instanceof ShortArrayAsList)) {
        ShortArrayAsList that = (ShortArrayAsList)object;
        int size = size();
        if (that.size() != size) {
          return false;
        }
        for (int i = 0; i < size; i++) {
          if (array[(start + i)] != array[(start + i)]) {
            return false;
          }
        }
        return true;
      }
      return super.equals(object);
    }
    
    public int hashCode() {
      int result = 1;
      for (int i = start; i < end; i++) {
        result = 31 * result + Shorts.hashCode(array[i]);
      }
      return result;
    }
    
    public String toString() {
      StringBuilder builder = new StringBuilder(size() * 6);
      builder.append('[').append(array[start]);
      for (int i = start + 1; i < end; i++) {
        builder.append(", ").append(array[i]);
      }
      return ']';
    }
    
    short[] toShortArray()
    {
      int size = size();
      short[] result = new short[size];
      System.arraycopy(array, start, result, 0, size);
      return result;
    }
  }
}
