package net.minecraft.util.com.mojang.authlib;

import java.util.HashMap;
import java.util.Map;
import net.minecraft.util.org.apache.commons.lang3.StringUtils;
import net.minecraft.util.org.apache.commons.lang3.Validate;

public abstract class BaseUserAuthentication implements UserAuthentication
{
  protected static final String STORAGE_KEY_PROFILE_NAME = "displayName";
  protected static final String STORAGE_KEY_PROFILE_ID = "uuid";
  protected static final String STORAGE_KEY_USER_NAME = "username";
  protected static final String STORAGE_KEY_USER_ID = "userid";
  private final AuthenticationService authenticationService;
  private String userid;
  private String username;
  private String password;
  private GameProfile selectedProfile;
  
  protected BaseUserAuthentication(AuthenticationService authenticationService)
  {
    Validate.notNull(authenticationService);
    this.authenticationService = authenticationService;
  }
  
  public boolean canLogIn()
  {
    return (!canPlayOnline()) && (StringUtils.isNotBlank(getUsername())) && (StringUtils.isNotBlank(getPassword()));
  }
  
  public void logOut()
  {
    password = null;
    userid = null;
    setSelectedProfile(null);
  }
  
  public boolean isLoggedIn()
  {
    return getSelectedProfile() != null;
  }
  
  public void setUsername(String username)
  {
    if ((isLoggedIn()) && (canPlayOnline())) {
      throw new IllegalStateException("Cannot change username whilst logged in & online");
    }
    
    this.username = username;
  }
  
  public void setPassword(String password)
  {
    if ((isLoggedIn()) && (canPlayOnline()) && (StringUtils.isNotBlank(password))) {
      throw new IllegalStateException("Cannot set password whilst logged in & online");
    }
    
    this.password = password;
  }
  
  protected String getUsername() {
    return username;
  }
  
  protected String getPassword() {
    return password;
  }
  
  public void loadFromStorage(Map<String, String> credentials)
  {
    logOut();
    
    setUsername((String)credentials.get("username"));
    
    if (credentials.containsKey("userid")) {
      userid = ((String)credentials.get("userid"));
    } else {
      userid = username;
    }
    
    if ((credentials.containsKey("displayName")) && (credentials.containsKey("uuid"))) {
      setSelectedProfile(new GameProfile((String)credentials.get("uuid"), (String)credentials.get("displayName")));
    }
  }
  
  public Map<String, String> saveForStorage()
  {
    Map<String, String> result = new HashMap();
    
    if (getUsername() != null) {
      result.put("username", getUsername());
    }
    if (getUserID() != null) {
      result.put("userid", getUserID());
    } else if (getUsername() != null) {
      result.put("username", getUsername());
    }
    
    if (getSelectedProfile() != null) {
      result.put("displayName", getSelectedProfile().getName());
      result.put("uuid", getSelectedProfile().getId());
    }
    
    return result;
  }
  
  protected void setSelectedProfile(GameProfile selectedProfile) {
    this.selectedProfile = selectedProfile;
  }
  
  public GameProfile getSelectedProfile()
  {
    return selectedProfile;
  }
  
  public String toString()
  {
    StringBuilder result = new StringBuilder();
    
    result.append(getClass().getSimpleName());
    result.append("{");
    
    if (isLoggedIn()) {
      result.append("Logged in as ");
      result.append(getUsername());
      
      if (getSelectedProfile() != null) {
        result.append(" / ");
        result.append(getSelectedProfile());
        result.append(" - ");
        
        if (canPlayOnline()) {
          result.append("Online");
        } else {
          result.append("Offline");
        }
      }
    } else {
      result.append("Not logged in");
    }
    
    result.append("}");
    
    return result.toString();
  }
  
  public AuthenticationService getAuthenticationService() {
    return authenticationService;
  }
  
  public String getUserID()
  {
    return userid;
  }
  
  protected void setUserid(String userid) {
    this.userid = userid;
  }
}
