package net.minecraft.util.com.mojang.authlib.legacy;

import java.io.IOException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;
import net.minecraft.util.com.mojang.authlib.GameProfile;
import net.minecraft.util.com.mojang.authlib.HttpAuthenticationService;
import net.minecraft.util.com.mojang.authlib.exceptions.AuthenticationException;
import net.minecraft.util.com.mojang.authlib.exceptions.AuthenticationUnavailableException;
import net.minecraft.util.com.mojang.authlib.minecraft.HttpMinecraftSessionService;

public class LegacyMinecraftSessionService
  extends HttpMinecraftSessionService
{
  private static final String BASE_URL = "http://session.minecraft.net/game/";
  private static final URL JOIN_URL = HttpAuthenticationService.constantURL("http://session.minecraft.net/game/joinserver.jsp");
  private static final URL CHECK_URL = HttpAuthenticationService.constantURL("http://session.minecraft.net/game/checkserver.jsp");
  
  protected LegacyMinecraftSessionService(LegacyAuthenticationService authenticationService) {
    super(authenticationService);
  }
  
  public void joinServer(GameProfile profile, String authenticationToken, String serverId) throws AuthenticationException
  {
    Map<String, Object> arguments = new HashMap();
    
    arguments.put("user", profile.getName());
    arguments.put("sessionId", authenticationToken);
    arguments.put("serverId", serverId);
    
    URL url = HttpAuthenticationService.concatenateURL(JOIN_URL, HttpAuthenticationService.buildQuery(arguments));
    try
    {
      String response = getAuthenticationService().performGetRequest(url);
      
      if (!response.equals("OK")) {
        throw new AuthenticationException(response);
      }
    } catch (IOException e) {
      throw new AuthenticationUnavailableException(e);
    }
  }
  
  public GameProfile hasJoinedServer(GameProfile user, String serverId) throws AuthenticationUnavailableException
  {
    Map<String, Object> arguments = new HashMap();
    
    arguments.put("user", user.getName());
    arguments.put("serverId", serverId);
    
    URL url = HttpAuthenticationService.concatenateURL(CHECK_URL, HttpAuthenticationService.buildQuery(arguments));
    try
    {
      String response = getAuthenticationService().performGetRequest(url);
      
      return response.equals("YES") ? user : null;
    } catch (IOException e) {
      throw new AuthenticationUnavailableException(e);
    }
  }
  
  public LegacyAuthenticationService getAuthenticationService()
  {
    return (LegacyAuthenticationService)super.getAuthenticationService();
  }
}
