package net.minecraft.util.com.mojang.authlib.yggdrasil;

import java.io.IOException;
import java.net.Proxy;
import java.net.URL;
import net.minecraft.util.com.google.gson.Gson;
import net.minecraft.util.com.google.gson.JsonParseException;
import net.minecraft.util.com.mojang.authlib.Agent;
import net.minecraft.util.com.mojang.authlib.HttpAuthenticationService;
import net.minecraft.util.com.mojang.authlib.UserAuthentication;
import net.minecraft.util.com.mojang.authlib.exceptions.AuthenticationException;
import net.minecraft.util.com.mojang.authlib.exceptions.AuthenticationUnavailableException;
import net.minecraft.util.com.mojang.authlib.exceptions.InvalidCredentialsException;
import net.minecraft.util.com.mojang.authlib.exceptions.UserMigratedException;
import net.minecraft.util.com.mojang.authlib.minecraft.MinecraftSessionService;
import net.minecraft.util.com.mojang.authlib.yggdrasil.response.Response;
import net.minecraft.util.org.apache.commons.lang3.StringUtils;

public class YggdrasilAuthenticationService extends HttpAuthenticationService
{
  private final String clientToken;
  private final Gson gson = new Gson();
  
  public YggdrasilAuthenticationService(Proxy proxy, String clientToken) {
    super(proxy);
    this.clientToken = clientToken;
  }
  
  public UserAuthentication createUserAuthentication(Agent agent)
  {
    return new YggdrasilUserAuthentication(this, agent);
  }
  
  public MinecraftSessionService createMinecraftSessionService()
  {
    return new YggdrasilMinecraftSessionService(this);
  }
  
  protected <T extends Response> T makeRequest(URL url, Object input, Class<T> classOfT) throws AuthenticationException {
    try {
      String jsonResult = input == null ? performGetRequest(url) : performPostRequest(url, gson.toJson(input), "application/json");
      T result = (Response)gson.fromJson(jsonResult, classOfT);
      
      if (result == null) { return null;
      }
      if (StringUtils.isNotBlank(result.getError())) {
        if ("UserMigratedException".equals(result.getCause()))
          throw new UserMigratedException(result.getErrorMessage());
        if (result.getError().equals("ForbiddenOperationException")) {
          throw new InvalidCredentialsException(result.getErrorMessage());
        }
        throw new AuthenticationException(result.getErrorMessage());
      }
      

      return result;
    } catch (IOException e) {
      throw new AuthenticationUnavailableException("Cannot contact authentication server", e);
    } catch (IllegalStateException e) {
      throw new AuthenticationUnavailableException("Cannot contact authentication server", e);
    } catch (JsonParseException e) {
      throw new AuthenticationUnavailableException("Cannot contact authentication server", e);
    }
  }
  
  public String getClientToken() {
    return clientToken;
  }
}
