package net.minecraft.util.com.mojang.authlib.yggdrasil;

import java.net.URL;
import java.util.HashMap;
import java.util.Map;
import net.minecraft.util.com.mojang.authlib.GameProfile;
import net.minecraft.util.com.mojang.authlib.HttpAuthenticationService;
import net.minecraft.util.com.mojang.authlib.exceptions.AuthenticationException;
import net.minecraft.util.com.mojang.authlib.exceptions.AuthenticationUnavailableException;
import net.minecraft.util.com.mojang.authlib.minecraft.HttpMinecraftSessionService;
import net.minecraft.util.com.mojang.authlib.yggdrasil.request.JoinMinecraftServerRequest;
import net.minecraft.util.com.mojang.authlib.yggdrasil.response.HasJoinedMinecraftServerResponse;
import net.minecraft.util.com.mojang.authlib.yggdrasil.response.Response;

public class YggdrasilMinecraftSessionService extends HttpMinecraftSessionService
{
  private static final String BASE_URL = "https://sessionserver.mojang.com/session/minecraft/";
  private static final URL JOIN_URL = HttpAuthenticationService.constantURL("https://sessionserver.mojang.com/session/minecraft/join");
  private static final URL CHECK_URL = HttpAuthenticationService.constantURL("https://sessionserver.mojang.com/session/minecraft/hasJoined");
  
  protected YggdrasilMinecraftSessionService(YggdrasilAuthenticationService authenticationService) {
    super(authenticationService);
  }
  
  public void joinServer(GameProfile profile, String authenticationToken, String serverId) throws AuthenticationException
  {
    JoinMinecraftServerRequest request = new JoinMinecraftServerRequest();
    accessToken = authenticationToken;
    selectedProfile = profile.getId();
    serverId = serverId;
    
    getAuthenticationService().makeRequest(JOIN_URL, request, Response.class);
  }
  
  public GameProfile hasJoinedServer(GameProfile user, String serverId) throws AuthenticationUnavailableException
  {
    Map<String, Object> arguments = new HashMap();
    
    arguments.put("username", user.getName());
    arguments.put("serverId", serverId);
    
    URL url = HttpAuthenticationService.concatenateURL(CHECK_URL, HttpAuthenticationService.buildQuery(arguments));
    try
    {
      HasJoinedMinecraftServerResponse response = (HasJoinedMinecraftServerResponse)getAuthenticationService().makeRequest(url, null, HasJoinedMinecraftServerResponse.class);
      return (response != null) && (response.getId() != null) ? new GameProfile(response.getId(), user.getName()) : null;
    } catch (AuthenticationUnavailableException e) {
      throw e;
    } catch (AuthenticationException e) {}
    return null;
  }
  

  public YggdrasilAuthenticationService getAuthenticationService()
  {
    return (YggdrasilAuthenticationService)super.getAuthenticationService();
  }
}
