package net.minecraft.util.gnu.trove.impl.unmodifiable;

import java.io.Serializable;
import java.util.Collection;
import java.util.Collections;
import java.util.Map;
import net.minecraft.util.gnu.trove.TCollections;
import net.minecraft.util.gnu.trove.function.TObjectFunction;
import net.minecraft.util.gnu.trove.iterator.TIntObjectIterator;
import net.minecraft.util.gnu.trove.map.TIntObjectMap;
import net.minecraft.util.gnu.trove.procedure.TIntObjectProcedure;
import net.minecraft.util.gnu.trove.procedure.TIntProcedure;
import net.minecraft.util.gnu.trove.procedure.TObjectProcedure;
import net.minecraft.util.gnu.trove.set.TIntSet;






























public class TUnmodifiableIntObjectMap<V>
  implements TIntObjectMap<V>, Serializable
{
  private static final long serialVersionUID = -1034234728574286014L;
  private final TIntObjectMap<V> m;
  
  public TUnmodifiableIntObjectMap(TIntObjectMap<V> m)
  {
    if (m == null)
      throw new NullPointerException();
    this.m = m;
  }
  
  public int size() { return m.size(); }
  public boolean isEmpty() { return m.isEmpty(); }
  public boolean containsKey(int key) { return m.containsKey(key); }
  public boolean containsValue(Object val) { return m.containsValue(val); }
  public V get(int key) { return m.get(key); }
  
  public V put(int key, V value) { throw new UnsupportedOperationException(); }
  public V remove(int key) { throw new UnsupportedOperationException(); }
  public void putAll(TIntObjectMap<? extends V> m) { throw new UnsupportedOperationException(); }
  public void putAll(Map<? extends Integer, ? extends V> map) { throw new UnsupportedOperationException(); }
  public void clear() { throw new UnsupportedOperationException(); }
  
  private transient TIntSet keySet = null;
  private transient Collection<V> values = null;
  
  public TIntSet keySet() {
    if (keySet == null)
      keySet = TCollections.unmodifiableSet(m.keySet());
    return keySet; }
  
  public int[] keys() { return m.keys(); }
  public int[] keys(int[] array) { return m.keys(array); }
  
  public Collection<V> valueCollection() {
    if (values == null)
      values = Collections.unmodifiableCollection(m.valueCollection());
    return values; }
  
  public Object[] values() { return m.values(); }
  public V[] values(V[] array) { return m.values(array); }
  
  public boolean equals(Object o) { return (o == this) || (m.equals(o)); }
  public int hashCode() { return m.hashCode(); }
  public String toString() { return m.toString(); }
  public int getNoEntryKey() { return m.getNoEntryKey(); }
  
  public boolean forEachKey(TIntProcedure procedure) {
    return m.forEachKey(procedure);
  }
  
  public boolean forEachValue(TObjectProcedure<? super V> procedure) { return m.forEachValue(procedure); }
  
  public boolean forEachEntry(TIntObjectProcedure<? super V> procedure) {
    return m.forEachEntry(procedure);
  }
  
  public TIntObjectIterator<V> iterator() {
    new TIntObjectIterator() {
      TIntObjectIterator<V> iter = m.iterator();
      
      public int key() { return iter.key(); }
      public V value() { return iter.value(); }
      public void advance() { iter.advance(); }
      public boolean hasNext() { return iter.hasNext(); }
      public V setValue(V val) { throw new UnsupportedOperationException(); }
      public void remove() { throw new UnsupportedOperationException(); }
    };
  }
  
  public V putIfAbsent(int key, V value) { throw new UnsupportedOperationException(); }
  public void transformValues(TObjectFunction<V, V> function) { throw new UnsupportedOperationException(); }
  public boolean retainEntries(TIntObjectProcedure<? super V> procedure) { throw new UnsupportedOperationException(); }
}
