package net.minecraft.util.gnu.trove.impl.unmodifiable;

import java.io.Serializable;
import java.util.Collections;
import java.util.Map;
import java.util.Set;
import net.minecraft.util.gnu.trove.TByteCollection;
import net.minecraft.util.gnu.trove.TCollections;
import net.minecraft.util.gnu.trove.function.TByteFunction;
import net.minecraft.util.gnu.trove.iterator.TObjectByteIterator;
import net.minecraft.util.gnu.trove.map.TObjectByteMap;
import net.minecraft.util.gnu.trove.procedure.TByteProcedure;
import net.minecraft.util.gnu.trove.procedure.TObjectByteProcedure;
import net.minecraft.util.gnu.trove.procedure.TObjectProcedure;






























public class TUnmodifiableObjectByteMap<K>
  implements TObjectByteMap<K>, Serializable
{
  private static final long serialVersionUID = -1034234728574286014L;
  private final TObjectByteMap<K> m;
  
  public TUnmodifiableObjectByteMap(TObjectByteMap<K> m)
  {
    if (m == null)
      throw new NullPointerException();
    this.m = m;
  }
  
  public int size() { return m.size(); }
  public boolean isEmpty() { return m.isEmpty(); }
  public boolean containsKey(Object key) { return m.containsKey(key); }
  public boolean containsValue(byte val) { return m.containsValue(val); }
  public byte get(Object key) { return m.get(key); }
  
  public byte put(K key, byte value) { throw new UnsupportedOperationException(); }
  public byte remove(Object key) { throw new UnsupportedOperationException(); }
  public void putAll(TObjectByteMap<? extends K> m) { throw new UnsupportedOperationException(); }
  public void putAll(Map<? extends K, ? extends Byte> map) { throw new UnsupportedOperationException(); }
  public void clear() { throw new UnsupportedOperationException(); }
  
  private transient Set<K> keySet = null;
  private transient TByteCollection values = null;
  
  public Set<K> keySet() {
    if (keySet == null)
      keySet = Collections.unmodifiableSet(m.keySet());
    return keySet; }
  
  public Object[] keys() { return m.keys(); }
  public K[] keys(K[] array) { return m.keys(array); }
  
  public TByteCollection valueCollection() {
    if (values == null)
      values = TCollections.unmodifiableCollection(m.valueCollection());
    return values; }
  
  public byte[] values() { return m.values(); }
  public byte[] values(byte[] array) { return m.values(array); }
  
  public boolean equals(Object o) { return (o == this) || (m.equals(o)); }
  public int hashCode() { return m.hashCode(); }
  public String toString() { return m.toString(); }
  public byte getNoEntryValue() { return m.getNoEntryValue(); }
  
  public boolean forEachKey(TObjectProcedure<? super K> procedure) {
    return m.forEachKey(procedure);
  }
  
  public boolean forEachValue(TByteProcedure procedure) { return m.forEachValue(procedure); }
  
  public boolean forEachEntry(TObjectByteProcedure<? super K> procedure) {
    return m.forEachEntry(procedure);
  }
  
  public TObjectByteIterator<K> iterator() {
    new TObjectByteIterator() {
      TObjectByteIterator<K> iter = m.iterator();
      
      public K key() { return iter.key(); }
      public byte value() { return iter.value(); }
      public void advance() { iter.advance(); }
      public boolean hasNext() { return iter.hasNext(); }
      public byte setValue(byte val) { throw new UnsupportedOperationException(); }
      public void remove() { throw new UnsupportedOperationException(); }
    };
  }
  
  public byte putIfAbsent(K key, byte value) { throw new UnsupportedOperationException(); }
  public void transformValues(TByteFunction function) { throw new UnsupportedOperationException(); }
  public boolean retainEntries(TObjectByteProcedure<? super K> procedure) { throw new UnsupportedOperationException(); }
  public boolean increment(K key) { throw new UnsupportedOperationException(); }
  public boolean adjustValue(K key, byte amount) { throw new UnsupportedOperationException(); }
  public byte adjustOrPutValue(K key, byte adjust_amount, byte put_amount) { throw new UnsupportedOperationException(); }
}
