package net.minecraft.util.io.netty.buffer;

import java.util.concurrent.atomic.AtomicIntegerFieldUpdater;
import net.minecraft.util.io.netty.util.IllegalReferenceCountException;
import net.minecraft.util.io.netty.util.internal.PlatformDependent;




















public abstract class AbstractReferenceCountedByteBuf
  extends AbstractByteBuf
{
  private static final AtomicIntegerFieldUpdater<AbstractReferenceCountedByteBuf> refCntUpdater = AtomicIntegerFieldUpdater.newUpdater(AbstractReferenceCountedByteBuf.class, "refCnt");
  
  private static final long REFCNT_FIELD_OFFSET;
  
  static
  {
    long refCntFieldOffset = -1L;
    try {
      if (PlatformDependent.hasUnsafe()) {
        refCntFieldOffset = PlatformDependent.objectFieldOffset(AbstractReferenceCountedByteBuf.class.getDeclaredField("refCnt"));
      }
    }
    catch (Throwable t) {}
    


    REFCNT_FIELD_OFFSET = refCntFieldOffset;
  }
  
  private volatile int refCnt = 1;
  
  protected AbstractReferenceCountedByteBuf(int maxCapacity)
  {
    super(maxCapacity);
  }
  
  public final int refCnt()
  {
    if (REFCNT_FIELD_OFFSET >= 0L)
    {
      return PlatformDependent.getInt(this, REFCNT_FIELD_OFFSET);
    }
    return refCnt;
  }
  



  protected final void setRefCnt(int refCnt)
  {
    this.refCnt = refCnt;
  }
  
  public ByteBuf retain()
  {
    for (;;) {
      int refCnt = this.refCnt;
      if (refCnt == 0) {
        throw new IllegalReferenceCountException(0, 1);
      }
      if (refCnt == 2147483647) {
        throw new IllegalReferenceCountException(2147483647, 1);
      }
      if (refCntUpdater.compareAndSet(this, refCnt, refCnt + 1)) {
        break;
      }
    }
    return this;
  }
  
  public ByteBuf retain(int increment)
  {
    if (increment <= 0) {
      throw new IllegalArgumentException("increment: " + increment + " (expected: > 0)");
    }
    for (;;)
    {
      int refCnt = this.refCnt;
      if (refCnt == 0) {
        throw new IllegalReferenceCountException(0, increment);
      }
      if (refCnt > 2147483647 - increment) {
        throw new IllegalReferenceCountException(refCnt, increment);
      }
      if (refCntUpdater.compareAndSet(this, refCnt, refCnt + increment)) {
        break;
      }
    }
    return this;
  }
  
  public final boolean release()
  {
    for (;;) {
      int refCnt = this.refCnt;
      if (refCnt == 0) {
        throw new IllegalReferenceCountException(0, -1);
      }
      
      if (refCntUpdater.compareAndSet(this, refCnt, refCnt - 1)) {
        if (refCnt == 1) {
          deallocate();
          return true;
        }
        return false;
      }
    }
  }
  
  public final boolean release(int decrement)
  {
    if (decrement <= 0) {
      throw new IllegalArgumentException("decrement: " + decrement + " (expected: > 0)");
    }
    for (;;)
    {
      int refCnt = this.refCnt;
      if (refCnt < decrement) {
        throw new IllegalReferenceCountException(refCnt, -decrement);
      }
      
      if (refCntUpdater.compareAndSet(this, refCnt, refCnt - decrement)) {
        if (refCnt == decrement) {
          deallocate();
          return true;
        }
        return false;
      }
    }
  }
  
  protected abstract void deallocate();
}
