package net.minecraft.util.io.netty.buffer;

import java.io.DataOutput;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.OutputStream;



























public class ByteBufOutputStream
  extends OutputStream
  implements DataOutput
{
  private final ByteBuf buffer;
  private final int startIndex;
  private final DataOutputStream utf8out = new DataOutputStream(this);
  


  public ByteBufOutputStream(ByteBuf buffer)
  {
    if (buffer == null) {
      throw new NullPointerException("buffer");
    }
    this.buffer = buffer;
    startIndex = buffer.writerIndex();
  }
  


  public int writtenBytes()
  {
    return buffer.writerIndex() - startIndex;
  }
  
  public void write(byte[] b, int off, int len) throws IOException
  {
    if (len == 0) {
      return;
    }
    
    buffer.writeBytes(b, off, len);
  }
  
  public void write(byte[] b) throws IOException
  {
    buffer.writeBytes(b);
  }
  
  public void write(int b) throws IOException
  {
    buffer.writeByte((byte)b);
  }
  
  public void writeBoolean(boolean v) throws IOException
  {
    write(v ? 1 : 0);
  }
  
  public void writeByte(int v) throws IOException
  {
    write(v);
  }
  
  public void writeBytes(String s) throws IOException
  {
    int len = s.length();
    for (int i = 0; i < len; i++) {
      write((byte)s.charAt(i));
    }
  }
  
  public void writeChar(int v) throws IOException
  {
    writeShort((short)v);
  }
  
  public void writeChars(String s) throws IOException
  {
    int len = s.length();
    for (int i = 0; i < len; i++) {
      writeChar(s.charAt(i));
    }
  }
  
  public void writeDouble(double v) throws IOException
  {
    writeLong(Double.doubleToLongBits(v));
  }
  
  public void writeFloat(float v) throws IOException
  {
    writeInt(Float.floatToIntBits(v));
  }
  
  public void writeInt(int v) throws IOException
  {
    buffer.writeInt(v);
  }
  
  public void writeLong(long v) throws IOException
  {
    buffer.writeLong(v);
  }
  
  public void writeShort(int v) throws IOException
  {
    buffer.writeShort((short)v);
  }
  
  public void writeUTF(String s) throws IOException
  {
    utf8out.writeUTF(s);
  }
  


  public ByteBuf buffer()
  {
    return buffer;
  }
}
