package net.minecraft.util.io.netty.buffer;

import java.nio.Buffer;
import java.nio.ByteBuffer;
import net.minecraft.util.io.netty.util.internal.PlatformDependent;
import net.minecraft.util.io.netty.util.internal.StringUtil;






















abstract class PoolArena<T>
{
  final PooledByteBufAllocator parent;
  private final int pageSize;
  private final int maxOrder;
  private final int pageShifts;
  private final int chunkSize;
  private final int subpageOverflowMask;
  private final PoolSubpage<T>[] tinySubpagePools;
  private final PoolSubpage<T>[] smallSubpagePools;
  private final PoolChunkList<T> q050;
  private final PoolChunkList<T> q025;
  private final PoolChunkList<T> q000;
  private final PoolChunkList<T> qInit;
  private final PoolChunkList<T> q075;
  private final PoolChunkList<T> q100;
  
  protected PoolArena(PooledByteBufAllocator parent, int pageSize, int maxOrder, int pageShifts, int chunkSize)
  {
    this.parent = parent;
    this.pageSize = pageSize;
    this.maxOrder = maxOrder;
    this.pageShifts = pageShifts;
    this.chunkSize = chunkSize;
    subpageOverflowMask = (pageSize - 1 ^ 0xFFFFFFFF);
    
    tinySubpagePools = newSubpagePoolArray(32);
    for (int i = 0; i < tinySubpagePools.length; i++) {
      tinySubpagePools[i] = newSubpagePoolHead(pageSize);
    }
    
    smallSubpagePools = newSubpagePoolArray(pageShifts - 9);
    for (int i = 0; i < smallSubpagePools.length; i++) {
      smallSubpagePools[i] = newSubpagePoolHead(pageSize);
    }
    
    q100 = new PoolChunkList(this, null, 100, 2147483647);
    q075 = new PoolChunkList(this, q100, 75, 100);
    q050 = new PoolChunkList(this, q075, 50, 100);
    q025 = new PoolChunkList(this, q050, 25, 75);
    q000 = new PoolChunkList(this, q025, 1, 50);
    qInit = new PoolChunkList(this, q000, -2147483648, 25);
    
    q100.prevList = q075;
    q075.prevList = q050;
    q050.prevList = q025;
    q025.prevList = q000;
    q000.prevList = null;
    qInit.prevList = qInit;
  }
  
  private PoolSubpage<T> newSubpagePoolHead(int pageSize) {
    PoolSubpage<T> head = new PoolSubpage(pageSize);
    prev = head;
    next = head;
    return head;
  }
  
  private PoolSubpage<T>[] newSubpagePoolArray(int size)
  {
    return new PoolSubpage[size];
  }
  
  PooledByteBuf<T> allocate(PoolThreadCache cache, int reqCapacity, int maxCapacity) {
    PooledByteBuf<T> buf = newByteBuf(maxCapacity);
    allocate(cache, buf, reqCapacity);
    return buf;
  }
  
  private void allocate(PoolThreadCache cache, PooledByteBuf<T> buf, int reqCapacity) {
    int normCapacity = normalizeCapacity(reqCapacity);
    if ((normCapacity & subpageOverflowMask) == 0) { PoolSubpage<T>[] table;
      int tableIdx;
      PoolSubpage<T>[] table;
      if ((normCapacity & 0xFFFFFE00) == 0) {
        int tableIdx = normCapacity >>> 4;
        table = tinySubpagePools;
      } else {
        tableIdx = 0;
        int i = normCapacity >>> 10;
        while (i != 0) {
          i >>>= 1;
          tableIdx++;
        }
        table = smallSubpagePools;
      }
      
      synchronized (this) {
        PoolSubpage<T> head = table[tableIdx];
        PoolSubpage<T> s = next;
        if (s != head) {
          assert ((doNotDestroy) && (elemSize == normCapacity));
          long handle = s.allocate();
          assert (handle >= 0L);
          chunk.initBufWithSubpage(buf, handle, reqCapacity);
          return;
        }
      }
    } else if (normCapacity > chunkSize) {
      allocateHuge(buf, reqCapacity);
      return;
    }
    
    allocateNormal(buf, reqCapacity, normCapacity);
  }
  
  private synchronized void allocateNormal(PooledByteBuf<T> buf, int reqCapacity, int normCapacity) {
    if ((q050.allocate(buf, reqCapacity, normCapacity)) || (q025.allocate(buf, reqCapacity, normCapacity)) || (q000.allocate(buf, reqCapacity, normCapacity)) || (qInit.allocate(buf, reqCapacity, normCapacity)) || (q075.allocate(buf, reqCapacity, normCapacity)) || (q100.allocate(buf, reqCapacity, normCapacity)))
    {

      return;
    }
    

    PoolChunk<T> c = newChunk(pageSize, maxOrder, pageShifts, chunkSize);
    long handle = c.allocate(normCapacity);
    assert (handle > 0L);
    c.initBuf(buf, handle, reqCapacity);
    qInit.add(c);
  }
  
  private void allocateHuge(PooledByteBuf<T> buf, int reqCapacity) {
    buf.initUnpooled(newUnpooledChunk(reqCapacity), reqCapacity);
  }
  
  synchronized void free(PoolChunk<T> chunk, long handle) {
    if (unpooled) {
      destroyChunk(chunk);
    } else
      parent.free(chunk, handle);
  }
  
  PoolSubpage<T> findSubpagePoolHead(int elemSize) {
    PoolSubpage<T>[] table;
    int tableIdx;
    PoolSubpage<T>[] table;
    if ((elemSize & 0xFFFFFE00) == 0) {
      int tableIdx = elemSize >>> 4;
      table = tinySubpagePools;
    } else {
      tableIdx = 0;
      elemSize >>>= 10;
      while (elemSize != 0) {
        elemSize >>>= 1;
        tableIdx++;
      }
      table = smallSubpagePools;
    }
    
    return table[tableIdx];
  }
  
  private int normalizeCapacity(int reqCapacity) {
    if (reqCapacity < 0) {
      throw new IllegalArgumentException("capacity: " + reqCapacity + " (expected: 0+)");
    }
    if (reqCapacity >= chunkSize) {
      return reqCapacity;
    }
    
    if ((reqCapacity & 0xFFFFFE00) != 0)
    {

      int normalizedCapacity = reqCapacity;
      normalizedCapacity |= normalizedCapacity >>> 1;
      normalizedCapacity |= normalizedCapacity >>> 2;
      normalizedCapacity |= normalizedCapacity >>> 4;
      normalizedCapacity |= normalizedCapacity >>> 8;
      normalizedCapacity |= normalizedCapacity >>> 16;
      normalizedCapacity++;
      
      if (normalizedCapacity < 0) {
        normalizedCapacity >>>= 1;
      }
      
      return normalizedCapacity;
    }
    

    if ((reqCapacity & 0xF) == 0) {
      return reqCapacity;
    }
    
    return (reqCapacity & 0xFFFFFFF0) + 16;
  }
  
  void reallocate(PooledByteBuf<T> buf, int newCapacity, boolean freeOldMemory) {
    if ((newCapacity < 0) || (newCapacity > buf.maxCapacity())) {
      throw new IllegalArgumentException("newCapacity: " + newCapacity);
    }
    
    int oldCapacity = length;
    if (oldCapacity == newCapacity) {
      return;
    }
    
    PoolChunk<T> oldChunk = chunk;
    long oldHandle = handle;
    T oldMemory = memory;
    int oldOffset = offset;
    
    int readerIndex = buf.readerIndex();
    int writerIndex = buf.writerIndex();
    
    allocate((PoolThreadCache)parent.threadCache.get(), buf, newCapacity);
    if (newCapacity > oldCapacity) {
      memoryCopy(oldMemory, oldOffset + readerIndex, memory, offset + readerIndex, writerIndex - readerIndex);

    }
    else if (newCapacity < oldCapacity) {
      if (readerIndex < newCapacity) {
        if (writerIndex > newCapacity) {
          writerIndex = newCapacity;
        }
        memoryCopy(oldMemory, oldOffset + readerIndex, memory, offset + readerIndex, writerIndex - readerIndex);
      }
      else
      {
        readerIndex = writerIndex = newCapacity;
      }
    }
    
    buf.setIndex(readerIndex, writerIndex);
    
    if (freeOldMemory)
      free(oldChunk, oldHandle); }
  
  protected abstract PoolChunk<T> newChunk(int paramInt1, int paramInt2, int paramInt3, int paramInt4);
  
  protected abstract PoolChunk<T> newUnpooledChunk(int paramInt);
  
  protected abstract PooledByteBuf<T> newByteBuf(int paramInt);
  
  protected abstract void memoryCopy(T paramT1, int paramInt1, T paramT2, int paramInt2, int paramInt3);
  
  protected abstract void destroyChunk(PoolChunk<T> paramPoolChunk);
  public synchronized String toString() { StringBuilder buf = new StringBuilder();
    buf.append("Chunk(s) at 0~25%:");
    buf.append(StringUtil.NEWLINE);
    buf.append(qInit);
    buf.append(StringUtil.NEWLINE);
    buf.append("Chunk(s) at 0~50%:");
    buf.append(StringUtil.NEWLINE);
    buf.append(q000);
    buf.append(StringUtil.NEWLINE);
    buf.append("Chunk(s) at 25~75%:");
    buf.append(StringUtil.NEWLINE);
    buf.append(q025);
    buf.append(StringUtil.NEWLINE);
    buf.append("Chunk(s) at 50~100%:");
    buf.append(StringUtil.NEWLINE);
    buf.append(q050);
    buf.append(StringUtil.NEWLINE);
    buf.append("Chunk(s) at 75~100%:");
    buf.append(StringUtil.NEWLINE);
    buf.append(q075);
    buf.append(StringUtil.NEWLINE);
    buf.append("Chunk(s) at 100%:");
    buf.append(StringUtil.NEWLINE);
    buf.append(q100);
    buf.append(StringUtil.NEWLINE);
    buf.append("tiny subpages:");
    for (int i = 1; i < tinySubpagePools.length; i++) {
      PoolSubpage<T> head = tinySubpagePools[i];
      if (next != head)
      {


        buf.append(StringUtil.NEWLINE);
        buf.append(i);
        buf.append(": ");
        PoolSubpage<T> s = next;
        for (;;) {
          buf.append(s);
          s = next;
          if (s == head)
            break;
        }
      }
    }
    buf.append(StringUtil.NEWLINE);
    buf.append("small subpages:");
    for (int i = 1; i < smallSubpagePools.length; i++) {
      PoolSubpage<T> head = smallSubpagePools[i];
      if (next != head)
      {


        buf.append(StringUtil.NEWLINE);
        buf.append(i);
        buf.append(": ");
        PoolSubpage<T> s = next;
        for (;;) {
          buf.append(s);
          s = next;
          if (s == head)
            break;
        }
      }
    }
    buf.append(StringUtil.NEWLINE);
    
    return buf.toString();
  }
  
  static final class HeapArena extends PoolArena<byte[]>
  {
    HeapArena(PooledByteBufAllocator parent, int pageSize, int maxOrder, int pageShifts, int chunkSize) {
      super(pageSize, maxOrder, pageShifts, chunkSize);
    }
    
    protected PoolChunk<byte[]> newChunk(int pageSize, int maxOrder, int pageShifts, int chunkSize)
    {
      return new PoolChunk(this, new byte[chunkSize], pageSize, maxOrder, pageShifts, chunkSize);
    }
    
    protected PoolChunk<byte[]> newUnpooledChunk(int capacity)
    {
      return new PoolChunk(this, new byte[capacity], capacity);
    }
    


    protected void destroyChunk(PoolChunk<byte[]> chunk) {}
    

    protected PooledByteBuf<byte[]> newByteBuf(int maxCapacity)
    {
      return PooledHeapByteBuf.newInstance(maxCapacity);
    }
    
    protected void memoryCopy(byte[] src, int srcOffset, byte[] dst, int dstOffset, int length)
    {
      if (length == 0) {
        return;
      }
      
      System.arraycopy(src, srcOffset, dst, dstOffset, length);
    }
  }
  
  static final class DirectArena extends PoolArena<ByteBuffer>
  {
    private static final boolean HAS_UNSAFE = ;
    
    DirectArena(PooledByteBufAllocator parent, int pageSize, int maxOrder, int pageShifts, int chunkSize) {
      super(pageSize, maxOrder, pageShifts, chunkSize);
    }
    
    protected PoolChunk<ByteBuffer> newChunk(int pageSize, int maxOrder, int pageShifts, int chunkSize)
    {
      return new PoolChunk(this, ByteBuffer.allocateDirect(chunkSize), pageSize, maxOrder, pageShifts, chunkSize);
    }
    

    protected PoolChunk<ByteBuffer> newUnpooledChunk(int capacity)
    {
      return new PoolChunk(this, ByteBuffer.allocateDirect(capacity), capacity);
    }
    
    protected void destroyChunk(PoolChunk<ByteBuffer> chunk)
    {
      PlatformDependent.freeDirectBuffer((ByteBuffer)memory);
    }
    
    protected PooledByteBuf<ByteBuffer> newByteBuf(int maxCapacity)
    {
      if (HAS_UNSAFE) {
        return PooledUnsafeDirectByteBuf.newInstance(maxCapacity);
      }
      return PooledDirectByteBuf.newInstance(maxCapacity);
    }
    

    protected void memoryCopy(ByteBuffer src, int srcOffset, ByteBuffer dst, int dstOffset, int length)
    {
      if (length == 0) {
        return;
      }
      
      if (HAS_UNSAFE) {
        PlatformDependent.copyMemory(PlatformDependent.directBufferAddress(src) + srcOffset, PlatformDependent.directBufferAddress(dst) + dstOffset, length);

      }
      else
      {
        src = src.duplicate();
        dst = dst.duplicate();
        src.position(srcOffset).limit(srcOffset + length);
        dst.position(dstOffset);
        dst.put(src);
      }
    }
  }
}
