package net.minecraft.util.io.netty.buffer;



final class PoolChunk<T>
{
  private static final int ST_UNUSED = 0;
  

  private static final int ST_BRANCH = 1;
  

  private static final int ST_ALLOCATED = 2;
  
  private static final int ST_ALLOCATED_SUBPAGE = 3;
  
  private static final long multiplier = 25214903917L;
  
  private static final long addend = 11L;
  
  private static final long mask = 281474976710655L;
  
  final PoolArena<T> arena;
  
  final T memory;
  
  final boolean unpooled;
  
  private final int[] memoryMap;
  
  private final PoolSubpage<T>[] subpages;
  
  private final int subpageOverflowMask;
  
  private final int pageSize;
  
  private final int pageShifts;
  
  private final int chunkSize;
  
  private final int maxSubpageAllocs;
  
  private long random = (System.nanoTime() ^ 0xDEECE66D) & 0xFFFFFFFF;
  
  private int freeBytes;
  
  PoolChunkList<T> parent;
  
  PoolChunk<T> prev;
  
  PoolChunk<T> next;
  
  PoolChunk(PoolArena<T> arena, T memory, int pageSize, int maxOrder, int pageShifts, int chunkSize)
  {
    unpooled = false;
    this.arena = arena;
    this.memory = memory;
    this.pageSize = pageSize;
    this.pageShifts = pageShifts;
    this.chunkSize = chunkSize;
    subpageOverflowMask = (pageSize - 1 ^ 0xFFFFFFFF);
    freeBytes = chunkSize;
    
    int chunkSizeInPages = chunkSize >>> pageShifts;
    maxSubpageAllocs = (1 << maxOrder);
    

    memoryMap = new int[maxSubpageAllocs << 1];
    int memoryMapIndex = 1;
    for (int i = 0; i <= maxOrder; i++) {
      int runSizeInPages = chunkSizeInPages >>> i;
      for (int j = 0; j < chunkSizeInPages; j += runSizeInPages)
      {
        memoryMap[(memoryMapIndex++)] = (j << 17 | runSizeInPages << 2 | 0x0);
      }
    }
    
    subpages = newSubpageArray(maxSubpageAllocs);
  }
  
  PoolChunk(PoolArena<T> arena, T memory, int size)
  {
    unpooled = true;
    this.arena = arena;
    this.memory = memory;
    memoryMap = null;
    subpages = null;
    subpageOverflowMask = 0;
    pageSize = 0;
    pageShifts = 0;
    chunkSize = size;
    maxSubpageAllocs = 0;
  }
  
  private PoolSubpage<T>[] newSubpageArray(int size)
  {
    return new PoolSubpage[size];
  }
  
  int usage() {
    if (freeBytes == 0) {
      return 100;
    }
    
    int freePercentage = (int)(freeBytes * 100L / chunkSize);
    if (freePercentage == 0) {
      return 99;
    }
    return 100 - freePercentage;
  }
  
  long allocate(int normCapacity) {
    int firstVal = memoryMap[1];
    if ((normCapacity & subpageOverflowMask) != 0) {
      return allocateRun(normCapacity, 1, firstVal);
    }
    return allocateSubpage(normCapacity, 1, firstVal);
  }
  
  private long allocateRun(int normCapacity, int curIdx, int val)
  {
    for (;;) {
      if ((val & 0x2) != 0) {
        return -1L;
      }
      
      if ((val & 0x1) == 0) break;
      int nextIdx = curIdx << 1 ^ nextRandom();
      long res = allocateRun(normCapacity, nextIdx, memoryMap[nextIdx]);
      if (res > 0L) {
        return res;
      }
      
      curIdx = nextIdx ^ 0x1;
      val = memoryMap[curIdx];
    }
    


    return allocateRunSimple(normCapacity, curIdx, val);
  }
  
  private long allocateRunSimple(int normCapacity, int curIdx, int val)
  {
    int runLength = runLength(val);
    if (normCapacity > runLength) {
      return -1L;
    }
    for (;;)
    {
      if (normCapacity == runLength)
      {


        memoryMap[curIdx] = (val & 0xFFFFFFFC | 0x2);
        freeBytes -= runLength;
        return curIdx;
      }
      
      int nextIdx = curIdx << 1 ^ nextRandom();
      int unusedIdx = nextIdx ^ 0x1;
      
      memoryMap[curIdx] = (val & 0xFFFFFFFC | 0x1);
      
      memoryMap[unusedIdx] = (memoryMap[unusedIdx] & 0xFFFFFFFC | 0x0);
      
      runLength >>>= 1;
      curIdx = nextIdx;
      val = memoryMap[curIdx];
    }
  }
  
  private long allocateSubpage(int normCapacity, int curIdx, int val) {
    int state = val & 0x3;
    if (state == 1) {
      int nextIdx = curIdx << 1 ^ nextRandom();
      long res = branchSubpage(normCapacity, nextIdx);
      if (res > 0L) {
        return res;
      }
      
      return branchSubpage(normCapacity, nextIdx ^ 0x1);
    }
    
    if (state == 0) {
      return allocateSubpageSimple(normCapacity, curIdx, val);
    }
    
    if (state == 3) {
      PoolSubpage<T> subpage = subpages[subpageIdx(curIdx)];
      int elemSize = elemSize;
      if (normCapacity != elemSize) {
        return -1L;
      }
      
      return subpage.allocate();
    }
    
    return -1L;
  }
  
  private long allocateSubpageSimple(int normCapacity, int curIdx, int val) {
    int runLength = runLength(val);
    for (;;) {
      if (runLength == pageSize) {
        memoryMap[curIdx] = (val & 0xFFFFFFFC | 0x3);
        freeBytes -= runLength;
        
        int subpageIdx = subpageIdx(curIdx);
        PoolSubpage<T> subpage = subpages[subpageIdx];
        if (subpage == null) {
          subpage = new PoolSubpage(this, curIdx, runOffset(val), pageSize, normCapacity);
          subpages[subpageIdx] = subpage;
        } else {
          subpage.init(normCapacity);
        }
        return subpage.allocate();
      }
      
      int nextIdx = curIdx << 1 ^ nextRandom();
      int unusedIdx = nextIdx ^ 0x1;
      
      memoryMap[curIdx] = (val & 0xFFFFFFFC | 0x1);
      
      memoryMap[unusedIdx] = (memoryMap[unusedIdx] & 0xFFFFFFFC | 0x0);
      
      runLength >>>= 1;
      curIdx = nextIdx;
      val = memoryMap[curIdx];
    }
  }
  
  private long branchSubpage(int normCapacity, int nextIdx) {
    int nextVal = memoryMap[nextIdx];
    if ((nextVal & 0x3) != 2) {
      return allocateSubpage(normCapacity, nextIdx, nextVal);
    }
    return -1L;
  }
  
  void free(long handle) {
    int memoryMapIdx = (int)handle;
    int bitmapIdx = (int)(handle >>> 32);
    
    int val = memoryMap[memoryMapIdx];
    int state = val & 0x3;
    if (state == 3) {
      assert (bitmapIdx != 0);
      PoolSubpage<T> subpage = subpages[subpageIdx(memoryMapIdx)];
      assert ((subpage != null) && (doNotDestroy));
      if (subpage.free(bitmapIdx & 0x3FFFFFFF)) {
        return;
      }
    } else {
      assert (state == 2) : ("state: " + state);
      assert (bitmapIdx == 0);
    }
    
    freeBytes += runLength(val);
    
    for (;;)
    {
      memoryMap[memoryMapIdx] = (val & 0xFFFFFFFC | 0x0);
      if (memoryMapIdx == 1) {
        assert (freeBytes == chunkSize);
        return;
      }
      
      if ((memoryMap[siblingIdx(memoryMapIdx)] & 0x3) != 0) {
        break;
      }
      
      memoryMapIdx = parentIdx(memoryMapIdx);
      val = memoryMap[memoryMapIdx];
    }
  }
  
  void initBuf(PooledByteBuf<T> buf, long handle, int reqCapacity) {
    int memoryMapIdx = (int)handle;
    int bitmapIdx = (int)(handle >>> 32);
    if (bitmapIdx == 0) {
      int val = memoryMap[memoryMapIdx];
      assert ((val & 0x3) == 2) : String.valueOf(val & 0x3);
      buf.init(this, handle, runOffset(val), reqCapacity, runLength(val));
    } else {
      initBufWithSubpage(buf, handle, bitmapIdx, reqCapacity);
    }
  }
  
  void initBufWithSubpage(PooledByteBuf<T> buf, long handle, int reqCapacity) {
    initBufWithSubpage(buf, handle, (int)(handle >>> 32), reqCapacity);
  }
  
  private void initBufWithSubpage(PooledByteBuf<T> buf, long handle, int bitmapIdx, int reqCapacity) {
    assert (bitmapIdx != 0);
    
    int memoryMapIdx = (int)handle;
    int val = memoryMap[memoryMapIdx];
    assert ((val & 0x3) == 3);
    
    PoolSubpage<T> subpage = subpages[subpageIdx(memoryMapIdx)];
    assert (doNotDestroy);
    assert (reqCapacity <= elemSize);
    
    buf.init(this, handle, runOffset(val) + (bitmapIdx & 0x3FFFFFFF) * elemSize, reqCapacity, elemSize);
  }
  

  private static int parentIdx(int memoryMapIdx)
  {
    return memoryMapIdx >>> 1;
  }
  
  private static int siblingIdx(int memoryMapIdx) {
    return memoryMapIdx ^ 0x1;
  }
  
  private int runLength(int val) {
    return (val >>> 2 & 0x7FFF) << pageShifts;
  }
  
  private int runOffset(int val) {
    return val >>> 17 << pageShifts;
  }
  
  private int subpageIdx(int memoryMapIdx) {
    return memoryMapIdx - maxSubpageAllocs;
  }
  
  private int nextRandom() {
    random = (random * 25214903917L + 11L & 0xFFFFFFFF);
    return (int)(random >>> 47) & 0x1;
  }
  
  public String toString() {
    StringBuilder buf = new StringBuilder();
    buf.append("Chunk(");
    buf.append(Integer.toHexString(System.identityHashCode(this)));
    buf.append(": ");
    buf.append(usage());
    buf.append("%, ");
    buf.append(chunkSize - freeBytes);
    buf.append('/');
    buf.append(chunkSize);
    buf.append(')');
    return buf.toString();
  }
}
