package net.minecraft.util.io.netty.channel;

import java.net.SocketAddress;
import net.minecraft.util.io.netty.util.ReferenceCountUtil;
import net.minecraft.util.io.netty.util.ReferenceCounted;
import net.minecraft.util.io.netty.util.internal.StringUtil;


























public class DefaultAddressedEnvelope<M, A extends SocketAddress>
  implements AddressedEnvelope<M, A>
{
  private final M message;
  private final A sender;
  private final A recipient;
  
  public DefaultAddressedEnvelope(M message, A recipient, A sender)
  {
    if (message == null) {
      throw new NullPointerException("message");
    }
    
    this.message = message;
    this.sender = sender;
    this.recipient = recipient;
  }
  



  public DefaultAddressedEnvelope(M message, A recipient)
  {
    this(message, recipient, null);
  }
  
  public M content()
  {
    return message;
  }
  
  public A sender()
  {
    return sender;
  }
  
  public A recipient()
  {
    return recipient;
  }
  
  public int refCnt()
  {
    if ((message instanceof ReferenceCounted)) {
      return ((ReferenceCounted)message).refCnt();
    }
    return 1;
  }
  

  public AddressedEnvelope<M, A> retain()
  {
    ReferenceCountUtil.retain(message);
    return this;
  }
  
  public AddressedEnvelope<M, A> retain(int increment)
  {
    ReferenceCountUtil.retain(message, increment);
    return this;
  }
  
  public boolean release()
  {
    return ReferenceCountUtil.release(message);
  }
  
  public boolean release(int decrement)
  {
    return ReferenceCountUtil.release(message, decrement);
  }
  
  public String toString()
  {
    if (sender != null) {
      return StringUtil.simpleClassName(this) + '(' + sender + " => " + recipient + ", " + message + ')';
    }
    
    return StringUtil.simpleClassName(this) + "(=> " + recipient + ", " + message + ')';
  }
}
