package net.minecraft.util.io.netty.channel;

import java.net.SocketAddress;
import java.util.Map;
import net.minecraft.util.io.netty.buffer.ByteBufAllocator;
import net.minecraft.util.io.netty.util.DefaultAttributeMap;
import net.minecraft.util.io.netty.util.Recycler;
import net.minecraft.util.io.netty.util.Recycler.Handle;
import net.minecraft.util.io.netty.util.concurrent.EventExecutor;
import net.minecraft.util.io.netty.util.concurrent.EventExecutorGroup;
import net.minecraft.util.io.netty.util.internal.StringUtil;
import net.minecraft.util.io.netty.util.internal.logging.InternalLogger;




















final class DefaultChannelHandlerContext
  extends DefaultAttributeMap
  implements ChannelHandlerContext
{
  volatile DefaultChannelHandlerContext next;
  volatile DefaultChannelHandlerContext prev;
  private final boolean inbound;
  private final boolean outbound;
  private final AbstractChannel channel;
  private final DefaultChannelPipeline pipeline;
  private final String name;
  private final ChannelHandler handler;
  private boolean removed;
  final EventExecutor executor;
  private ChannelFuture succeededFuture;
  private Runnable invokeChannelReadCompleteTask;
  private Runnable invokeReadTask;
  private Runnable invokeFlushTask;
  private Runnable invokeChannelWritableStateChangedTask;
  
  DefaultChannelHandlerContext(DefaultChannelPipeline pipeline, EventExecutorGroup group, String name, ChannelHandler handler)
  {
    if (name == null) {
      throw new NullPointerException("name");
    }
    if (handler == null) {
      throw new NullPointerException("handler");
    }
    
    channel = channel;
    this.pipeline = pipeline;
    this.name = name;
    this.handler = handler;
    
    if (group != null)
    {

      EventExecutor childExecutor = (EventExecutor)childExecutors.get(group);
      if (childExecutor == null) {
        childExecutor = group.next();
        childExecutors.put(group, childExecutor);
      }
      executor = childExecutor;
    } else {
      executor = null;
    }
    
    inbound = (handler instanceof ChannelInboundHandler);
    outbound = (handler instanceof ChannelOutboundHandler);
  }
  
  void teardown()
  {
    EventExecutor executor = executor();
    if (executor.inEventLoop()) {
      teardown0();
    } else {
      executor.execute(new Runnable()
      {
        public void run() {
          DefaultChannelHandlerContext.this.teardown0();
        }
      });
    }
  }
  
  private void teardown0() {
    DefaultChannelHandlerContext prev = this.prev;
    if (prev != null) {
      synchronized (pipeline) {
        pipeline.remove0(this);
      }
      prev.teardown();
    }
  }
  
  public Channel channel()
  {
    return channel;
  }
  
  public ChannelPipeline pipeline()
  {
    return pipeline;
  }
  
  public ByteBufAllocator alloc()
  {
    return channel().config().getAllocator();
  }
  
  public EventExecutor executor()
  {
    if (executor == null) {
      return channel().eventLoop();
    }
    return executor;
  }
  

  public ChannelHandler handler()
  {
    return handler;
  }
  
  public String name()
  {
    return name;
  }
  
  public ChannelHandlerContext fireChannelRegistered()
  {
    final DefaultChannelHandlerContext next = findContextInbound();
    EventExecutor executor = next.executor();
    if (executor.inEventLoop()) {
      next.invokeChannelRegistered();
    } else {
      executor.execute(new Runnable()
      {
        public void run() {
          next.invokeChannelRegistered();
        }
      });
    }
    return this;
  }
  
  private void invokeChannelRegistered() {
    try {
      ((ChannelInboundHandler)handler).channelRegistered(this);
    } catch (Throwable t) {
      notifyHandlerException(t);
    }
  }
  
  public ChannelHandlerContext fireChannelUnregistered()
  {
    final DefaultChannelHandlerContext next = findContextInbound();
    EventExecutor executor = next.executor();
    if (executor.inEventLoop()) {
      next.invokeChannelUnregistered();
    } else {
      executor.execute(new Runnable()
      {
        public void run() {
          next.invokeChannelUnregistered();
        }
      });
    }
    return this;
  }
  
  private void invokeChannelUnregistered() {
    try {
      ((ChannelInboundHandler)handler).channelUnregistered(this);
    } catch (Throwable t) {
      notifyHandlerException(t);
    }
  }
  
  public ChannelHandlerContext fireChannelActive()
  {
    final DefaultChannelHandlerContext next = findContextInbound();
    EventExecutor executor = next.executor();
    if (executor.inEventLoop()) {
      next.invokeChannelActive();
    } else {
      executor.execute(new Runnable()
      {
        public void run() {
          next.invokeChannelActive();
        }
      });
    }
    return this;
  }
  
  private void invokeChannelActive() {
    try {
      ((ChannelInboundHandler)handler).channelActive(this);
    } catch (Throwable t) {
      notifyHandlerException(t);
    }
  }
  
  public ChannelHandlerContext fireChannelInactive()
  {
    final DefaultChannelHandlerContext next = findContextInbound();
    EventExecutor executor = next.executor();
    if (executor.inEventLoop()) {
      next.invokeChannelInactive();
    } else {
      executor.execute(new Runnable()
      {
        public void run() {
          next.invokeChannelInactive();
        }
      });
    }
    return this;
  }
  
  private void invokeChannelInactive() {
    try {
      ((ChannelInboundHandler)handler).channelInactive(this);
    } catch (Throwable t) {
      notifyHandlerException(t);
    }
  }
  
  public ChannelHandlerContext fireExceptionCaught(final Throwable cause)
  {
    if (cause == null) {
      throw new NullPointerException("cause");
    }
    
    final DefaultChannelHandlerContext next = this.next;
    
    EventExecutor executor = next.executor();
    if (executor.inEventLoop()) {
      next.invokeExceptionCaught(cause);
    } else {
      try {
        executor.execute(new Runnable()
        {
          public void run() {
            next.invokeExceptionCaught(cause);
          }
        });
      } catch (Throwable t) {
        if (DefaultChannelPipeline.logger.isWarnEnabled()) {
          DefaultChannelPipeline.logger.warn("Failed to submit an exceptionCaught() event.", t);
          DefaultChannelPipeline.logger.warn("The exceptionCaught() event that was failed to submit was:", cause);
        }
      }
    }
    
    return this;
  }
  
  private void invokeExceptionCaught(Throwable cause) {
    try {
      handler.exceptionCaught(this, cause);
    } catch (Throwable t) {
      if (DefaultChannelPipeline.logger.isWarnEnabled()) {
        DefaultChannelPipeline.logger.warn("An exception was thrown by a user handler's exceptionCaught() method while handling the following exception:", cause);
      }
    }
  }
  


  public ChannelHandlerContext fireUserEventTriggered(final Object event)
  {
    if (event == null) {
      throw new NullPointerException("event");
    }
    
    final DefaultChannelHandlerContext next = findContextInbound();
    EventExecutor executor = next.executor();
    if (executor.inEventLoop()) {
      next.invokeUserEventTriggered(event);
    } else {
      executor.execute(new Runnable()
      {
        public void run() {
          next.invokeUserEventTriggered(event);
        }
      });
    }
    return this;
  }
  
  private void invokeUserEventTriggered(Object event) {
    try {
      ((ChannelInboundHandler)handler).userEventTriggered(this, event);
    } catch (Throwable t) {
      notifyHandlerException(t);
    }
  }
  
  public ChannelHandlerContext fireChannelRead(final Object msg)
  {
    if (msg == null) {
      throw new NullPointerException("msg");
    }
    
    final DefaultChannelHandlerContext next = findContextInbound();
    EventExecutor executor = next.executor();
    if (executor.inEventLoop()) {
      next.invokeChannelRead(msg);
    } else {
      executor.execute(new Runnable()
      {
        public void run() {
          next.invokeChannelRead(msg);
        }
      });
    }
    return this;
  }
  
  private void invokeChannelRead(Object msg) {
    try {
      ((ChannelInboundHandler)handler).channelRead(this, msg);
    } catch (Throwable t) {
      notifyHandlerException(t);
    }
  }
  
  public ChannelHandlerContext fireChannelReadComplete()
  {
    final DefaultChannelHandlerContext next = findContextInbound();
    EventExecutor executor = next.executor();
    if (executor.inEventLoop()) {
      next.invokeChannelReadComplete();
    } else {
      Runnable task = invokeChannelReadCompleteTask;
      if (task == null) {
        invokeChannelReadCompleteTask = (task = new Runnable()
        {
          public void run() {
            next.invokeChannelReadComplete();
          }
        });
      }
      executor.execute(task);
    }
    return this;
  }
  
  private void invokeChannelReadComplete() {
    try {
      ((ChannelInboundHandler)handler).channelReadComplete(this);
    } catch (Throwable t) {
      notifyHandlerException(t);
    }
  }
  
  public ChannelHandlerContext fireChannelWritabilityChanged()
  {
    final DefaultChannelHandlerContext next = findContextInbound();
    EventExecutor executor = next.executor();
    if (executor.inEventLoop()) {
      next.invokeChannelWritabilityChanged();
    } else {
      Runnable task = invokeChannelWritableStateChangedTask;
      if (task == null) {
        invokeChannelWritableStateChangedTask = (task = new Runnable()
        {
          public void run() {
            next.invokeChannelWritabilityChanged();
          }
        });
      }
      executor.execute(task);
    }
    return this;
  }
  
  private void invokeChannelWritabilityChanged() {
    try {
      ((ChannelInboundHandler)handler).channelWritabilityChanged(this);
    } catch (Throwable t) {
      notifyHandlerException(t);
    }
  }
  
  public ChannelFuture bind(SocketAddress localAddress)
  {
    return bind(localAddress, newPromise());
  }
  
  public ChannelFuture connect(SocketAddress remoteAddress)
  {
    return connect(remoteAddress, newPromise());
  }
  
  public ChannelFuture connect(SocketAddress remoteAddress, SocketAddress localAddress)
  {
    return connect(remoteAddress, localAddress, newPromise());
  }
  
  public ChannelFuture disconnect()
  {
    return disconnect(newPromise());
  }
  
  public ChannelFuture close()
  {
    return close(newPromise());
  }
  
  public ChannelFuture deregister()
  {
    return deregister(newPromise());
  }
  
  public ChannelFuture bind(final SocketAddress localAddress, final ChannelPromise promise)
  {
    if (localAddress == null) {
      throw new NullPointerException("localAddress");
    }
    validatePromise(promise, false);
    
    final DefaultChannelHandlerContext next = findContextOutbound();
    EventExecutor executor = next.executor();
    if (executor.inEventLoop()) {
      next.invokeBind(localAddress, promise);
    } else {
      executor.execute(new Runnable()
      {
        public void run() {
          next.invokeBind(localAddress, promise);
        }
      });
    }
    
    return promise;
  }
  
  private void invokeBind(SocketAddress localAddress, ChannelPromise promise) {
    try {
      ((ChannelOutboundHandler)handler).bind(this, localAddress, promise);
    } catch (Throwable t) {
      notifyOutboundHandlerException(t, promise);
    }
  }
  
  public ChannelFuture connect(SocketAddress remoteAddress, ChannelPromise promise)
  {
    return connect(remoteAddress, null, promise);
  }
  


  public ChannelFuture connect(final SocketAddress remoteAddress, final SocketAddress localAddress, final ChannelPromise promise)
  {
    if (remoteAddress == null) {
      throw new NullPointerException("remoteAddress");
    }
    validatePromise(promise, false);
    
    final DefaultChannelHandlerContext next = findContextOutbound();
    EventExecutor executor = next.executor();
    if (executor.inEventLoop()) {
      next.invokeConnect(remoteAddress, localAddress, promise);
    } else {
      executor.execute(new Runnable()
      {
        public void run() {
          next.invokeConnect(remoteAddress, localAddress, promise);
        }
      });
    }
    
    return promise;
  }
  
  private void invokeConnect(SocketAddress remoteAddress, SocketAddress localAddress, ChannelPromise promise) {
    try {
      ((ChannelOutboundHandler)handler).connect(this, remoteAddress, localAddress, promise);
    } catch (Throwable t) {
      notifyOutboundHandlerException(t, promise);
    }
  }
  
  public ChannelFuture disconnect(final ChannelPromise promise)
  {
    validatePromise(promise, false);
    
    final DefaultChannelHandlerContext next = findContextOutbound();
    EventExecutor executor = next.executor();
    if (executor.inEventLoop())
    {

      if (!channel().metadata().hasDisconnect()) {
        next.invokeClose(promise);
      } else {
        next.invokeDisconnect(promise);
      }
    } else {
      executor.execute(new Runnable()
      {
        public void run() {
          if (!channel().metadata().hasDisconnect()) {
            next.invokeClose(promise);
          } else {
            next.invokeDisconnect(promise);
          }
        }
      });
    }
    
    return promise;
  }
  
  private void invokeDisconnect(ChannelPromise promise) {
    try {
      ((ChannelOutboundHandler)handler).disconnect(this, promise);
    } catch (Throwable t) {
      notifyOutboundHandlerException(t, promise);
    }
  }
  
  public ChannelFuture close(final ChannelPromise promise)
  {
    validatePromise(promise, false);
    
    final DefaultChannelHandlerContext next = findContextOutbound();
    EventExecutor executor = next.executor();
    if (executor.inEventLoop()) {
      next.invokeClose(promise);
    } else {
      executor.execute(new Runnable()
      {
        public void run() {
          next.invokeClose(promise);
        }
      });
    }
    
    return promise;
  }
  
  private void invokeClose(ChannelPromise promise) {
    try {
      ((ChannelOutboundHandler)handler).close(this, promise);
    } catch (Throwable t) {
      notifyOutboundHandlerException(t, promise);
    }
  }
  
  public ChannelFuture deregister(final ChannelPromise promise)
  {
    validatePromise(promise, false);
    
    final DefaultChannelHandlerContext next = findContextOutbound();
    EventExecutor executor = next.executor();
    if (executor.inEventLoop()) {
      next.invokeDeregister(promise);
    } else {
      executor.execute(new Runnable()
      {
        public void run() {
          next.invokeDeregister(promise);
        }
      });
    }
    
    return promise;
  }
  
  private void invokeDeregister(ChannelPromise promise) {
    try {
      ((ChannelOutboundHandler)handler).deregister(this, promise);
    } catch (Throwable t) {
      notifyOutboundHandlerException(t, promise);
    }
  }
  
  public ChannelHandlerContext read()
  {
    final DefaultChannelHandlerContext next = findContextOutbound();
    EventExecutor executor = next.executor();
    if (executor.inEventLoop()) {
      next.invokeRead();
    } else {
      Runnable task = invokeReadTask;
      if (task == null) {
        invokeReadTask = (task = new Runnable()
        {
          public void run() {
            next.invokeRead();
          }
        });
      }
      executor.execute(task);
    }
    
    return this;
  }
  
  private void invokeRead() {
    try {
      ((ChannelOutboundHandler)handler).read(this);
    } catch (Throwable t) {
      notifyHandlerException(t);
    }
  }
  
  public ChannelFuture write(Object msg)
  {
    return write(msg, newPromise());
  }
  
  public ChannelFuture write(Object msg, ChannelPromise promise)
  {
    if (msg == null) {
      throw new NullPointerException("msg");
    }
    
    validatePromise(promise, true);
    
    write(msg, false, promise);
    
    return promise;
  }
  
  private void invokeWrite(Object msg, ChannelPromise promise) {
    try {
      ((ChannelOutboundHandler)handler).write(this, msg, promise);
    } catch (Throwable t) {
      notifyOutboundHandlerException(t, promise);
    }
  }
  
  public ChannelHandlerContext flush()
  {
    final DefaultChannelHandlerContext next = findContextOutbound();
    EventExecutor executor = next.executor();
    if (executor.inEventLoop()) {
      next.invokeFlush();
    } else {
      Runnable task = invokeFlushTask;
      if (task == null) {
        invokeFlushTask = (task = new Runnable()
        {
          public void run() {
            next.invokeFlush();
          }
        });
      }
      executor.execute(task);
    }
    
    return this;
  }
  
  private void invokeFlush() {
    try {
      ((ChannelOutboundHandler)handler).flush(this);
    } catch (Throwable t) {
      notifyHandlerException(t);
    }
  }
  
  public ChannelFuture writeAndFlush(Object msg, ChannelPromise promise)
  {
    if (msg == null) {
      throw new NullPointerException("msg");
    }
    
    validatePromise(promise, true);
    
    write(msg, true, promise);
    
    return promise;
  }
  
  private void write(Object msg, boolean flush, ChannelPromise promise)
  {
    DefaultChannelHandlerContext next = findContextOutbound();
    EventExecutor executor = next.executor();
    if (executor.inEventLoop()) {
      next.invokeWrite(msg, promise);
      if (flush) {
        next.invokeFlush();
      }
    } else {
      int size = channel.estimatorHandle().size(msg);
      if (size > 0) {
        ChannelOutboundBuffer buffer = channel.unsafe().outboundBuffer();
        
        if (buffer != null) {
          buffer.incrementPendingOutboundBytes(size);
        }
      }
      executor.execute(WriteTask.newInstance(next, msg, size, flush, promise));
    }
  }
  
  public ChannelFuture writeAndFlush(Object msg)
  {
    return writeAndFlush(msg, newPromise());
  }
  

  private static void notifyOutboundHandlerException(Throwable cause, ChannelPromise promise)
  {
    if ((promise instanceof VoidChannelPromise)) {
      return;
    }
    
    if ((!promise.tryFailure(cause)) && 
      (DefaultChannelPipeline.logger.isWarnEnabled())) {
      DefaultChannelPipeline.logger.warn("Failed to fail the promise because it's done already: {}", promise, cause);
    }
  }
  
  private void notifyHandlerException(Throwable cause)
  {
    if (inExceptionCaught(cause)) {
      if (DefaultChannelPipeline.logger.isWarnEnabled()) {
        DefaultChannelPipeline.logger.warn("An exception was thrown by a user handler while handling an exceptionCaught event", cause);
      }
      

      return;
    }
    
    invokeExceptionCaught(cause);
  }
  
  private static boolean inExceptionCaught(Throwable cause) {
    do {
      StackTraceElement[] trace = cause.getStackTrace();
      if (trace != null) {
        for (StackTraceElement t : trace) {
          if (t == null) {
            break;
          }
          if ("exceptionCaught".equals(t.getMethodName())) {
            return true;
          }
        }
      }
      
      cause = cause.getCause();
    } while (cause != null);
    
    return false;
  }
  
  public ChannelPromise newPromise()
  {
    return new DefaultChannelPromise(channel(), executor());
  }
  
  public ChannelProgressivePromise newProgressivePromise()
  {
    return new DefaultChannelProgressivePromise(channel(), executor());
  }
  
  public ChannelFuture newSucceededFuture()
  {
    ChannelFuture succeededFuture = this.succeededFuture;
    if (succeededFuture == null) {
      this.succeededFuture = (succeededFuture = new SucceededChannelFuture(channel(), executor()));
    }
    return succeededFuture;
  }
  
  public ChannelFuture newFailedFuture(Throwable cause)
  {
    return new FailedChannelFuture(channel(), executor(), cause);
  }
  
  private void validatePromise(ChannelPromise promise, boolean allowVoidPromise) {
    if (promise == null) {
      throw new NullPointerException("promise");
    }
    
    if (promise.isDone()) {
      throw new IllegalArgumentException("promise already done: " + promise);
    }
    
    if (promise.channel() != channel()) {
      throw new IllegalArgumentException(String.format("promise.channel does not match: %s (expected: %s)", new Object[] { promise.channel(), channel() }));
    }
    

    if (promise.getClass() == DefaultChannelPromise.class) {
      return;
    }
    
    if ((!allowVoidPromise) && ((promise instanceof VoidChannelPromise))) {
      throw new IllegalArgumentException(StringUtil.simpleClassName(VoidChannelPromise.class) + " not allowed for this operation");
    }
    

    if ((promise instanceof AbstractChannel.CloseFuture)) {
      throw new IllegalArgumentException(StringUtil.simpleClassName(AbstractChannel.CloseFuture.class) + " not allowed in a pipeline");
    }
  }
  
  private DefaultChannelHandlerContext findContextInbound()
  {
    DefaultChannelHandlerContext ctx = this;
    do {
      ctx = next;
    } while (!inbound);
    return ctx;
  }
  
  private DefaultChannelHandlerContext findContextOutbound() {
    DefaultChannelHandlerContext ctx = this;
    do {
      ctx = prev;
    } while (!outbound);
    return ctx;
  }
  
  public ChannelPromise voidPromise()
  {
    return channel.voidPromise();
  }
  
  void setRemoved() {
    removed = true;
  }
  
  public boolean isRemoved()
  {
    return removed;
  }
  
  static final class WriteTask implements Runnable
  {
    private DefaultChannelHandlerContext ctx;
    private Object msg;
    private ChannelPromise promise;
    private int size;
    private boolean flush;
    private static final Recycler<WriteTask> RECYCLER = new Recycler()
    {
      protected DefaultChannelHandlerContext.WriteTask newObject(Recycler.Handle handle) {
        return new DefaultChannelHandlerContext.WriteTask(handle, null);
      }
    };
    private final Recycler.Handle handle;
    
    private static WriteTask newInstance(DefaultChannelHandlerContext ctx, Object msg, int size, boolean flush, ChannelPromise promise) {
      WriteTask task = (WriteTask)RECYCLER.get();
      ctx = ctx;
      msg = msg;
      promise = promise;
      size = size;
      flush = flush;
      return task;
    }
    

    private WriteTask(Recycler.Handle handle)
    {
      this.handle = handle;
    }
    
    public void run()
    {
      try {
        if (size > 0) {
          ChannelOutboundBuffer buffer = ctx.channel.unsafe().outboundBuffer();
          
          if (buffer != null) {
            buffer.decrementPendingOutboundBytes(size);
          }
        }
        ctx.invokeWrite(msg, promise);
        if (flush) {
          ctx.invokeFlush();
        }
      }
      finally {
        ctx = null;
        msg = null;
        promise = null;
        
        RECYCLER.recycle(this, handle);
      }
    }
  }
}
