package net.minecraft.util.io.netty.channel;

import java.util.concurrent.ThreadFactory;
import net.minecraft.util.io.netty.util.concurrent.DefaultThreadFactory;
import net.minecraft.util.io.netty.util.concurrent.MultithreadEventExecutorGroup;
import net.minecraft.util.io.netty.util.internal.SystemPropertyUtil;
import net.minecraft.util.io.netty.util.internal.logging.InternalLogger;
import net.minecraft.util.io.netty.util.internal.logging.InternalLoggerFactory;



















public abstract class MultithreadEventLoopGroup
  extends MultithreadEventExecutorGroup
  implements EventLoopGroup
{
  private static final InternalLogger logger = InternalLoggerFactory.getInstance(MultithreadEventLoopGroup.class);
  



  private static final int DEFAULT_EVENT_LOOP_THREADS = Math.max(1, SystemPropertyUtil.getInt("io.netty.eventLoopThreads", Runtime.getRuntime().availableProcessors() * 2));
  
  static {
    if (logger.isDebugEnabled()) {
      logger.debug("-Dio.netty.eventLoopThreads: {}", Integer.valueOf(DEFAULT_EVENT_LOOP_THREADS));
    }
  }
  


  protected MultithreadEventLoopGroup(int nThreads, ThreadFactory threadFactory, Object... args)
  {
    super(nThreads == 0 ? DEFAULT_EVENT_LOOP_THREADS : nThreads, threadFactory, args);
  }
  
  protected ThreadFactory newDefaultThreadFactory()
  {
    return new DefaultThreadFactory(getClass(), 10);
  }
  
  public EventLoop next()
  {
    return (EventLoop)super.next();
  }
  
  public ChannelFuture register(Channel channel)
  {
    return next().register(channel);
  }
  
  public ChannelFuture register(Channel channel, ChannelPromise promise)
  {
    return next().register(channel, promise);
  }
}
