package net.minecraft.util.io.netty.channel.nio;

import java.io.IOException;
import java.nio.channels.SelectableChannel;
import java.nio.channels.SelectionKey;
import net.minecraft.util.io.netty.buffer.ByteBuf;
import net.minecraft.util.io.netty.buffer.ByteBufAllocator;
import net.minecraft.util.io.netty.channel.Channel;
import net.minecraft.util.io.netty.channel.ChannelConfig;
import net.minecraft.util.io.netty.channel.ChannelOption;
import net.minecraft.util.io.netty.channel.ChannelOutboundBuffer;
import net.minecraft.util.io.netty.channel.ChannelPipeline;
import net.minecraft.util.io.netty.channel.FileRegion;
import net.minecraft.util.io.netty.channel.RecvByteBufAllocator;
import net.minecraft.util.io.netty.channel.RecvByteBufAllocator.Handle;
import net.minecraft.util.io.netty.channel.socket.ChannelInputShutdownEvent;
import net.minecraft.util.io.netty.util.internal.StringUtil;























public abstract class AbstractNioByteChannel
  extends AbstractNioChannel
{
  protected AbstractNioByteChannel(Channel parent, SelectableChannel ch)
  {
    super(parent, ch, 1);
  }
  


  protected AbstractNioChannel.AbstractNioUnsafe newUnsafe() { return new NioByteUnsafe(null); }
  
  private final class NioByteUnsafe extends AbstractNioChannel.AbstractNioUnsafe {
    private NioByteUnsafe() { super(); }
    
    private RecvByteBufAllocator.Handle allocHandle;
    public void read()
    {
      assert (eventLoop().inEventLoop());
      SelectionKey key = selectionKey();
      ChannelConfig config = config();
      if (!config.isAutoRead()) {
        int interestOps = key.interestOps();
        if ((interestOps & readInterestOp) != 0)
        {
          key.interestOps(interestOps & (readInterestOp ^ 0xFFFFFFFF));
        }
      }
      
      ChannelPipeline pipeline = pipeline();
      
      RecvByteBufAllocator.Handle allocHandle = this.allocHandle;
      if (allocHandle == null) {
        this.allocHandle = (allocHandle = config.getRecvByteBufAllocator().newHandle());
      }
      
      ByteBufAllocator allocator = config.getAllocator();
      int maxMessagesPerRead = config.getMaxMessagesPerRead();
      
      boolean closed = false;
      Throwable exception = null;
      ByteBuf byteBuf = null;
      int messages = 0;
      try {
        for (;;) {
          byteBuf = allocHandle.allocate(allocator);
          int localReadAmount = doReadBytes(byteBuf);
          if (localReadAmount == 0) {
            byteBuf.release();
            byteBuf = null;

          }
          else if (localReadAmount < 0) {
            closed = true;
            byteBuf.release();
            byteBuf = null;
          }
          else
          {
            pipeline.fireChannelRead(byteBuf);
            allocHandle.record(localReadAmount);
            byteBuf = null;
            messages++; if (messages == maxMessagesPerRead)
              break;
          }
        }
      } catch (Throwable t) {
        exception = t;
      } finally {
        if (byteBuf != null) {
          if (byteBuf.isReadable()) {
            pipeline.fireChannelRead(byteBuf);
          } else {
            byteBuf.release();
          }
        }
        
        pipeline.fireChannelReadComplete();
        
        if (exception != null) {
          if ((exception instanceof IOException)) {
            closed = true;
          }
          
          pipeline().fireExceptionCaught(exception);
        }
        
        if (closed) {
          setInputShutdown();
          if (isOpen()) {
            if (Boolean.TRUE.equals(config().getOption(ChannelOption.ALLOW_HALF_CLOSURE))) {
              key.interestOps(key.interestOps() & (readInterestOp ^ 0xFFFFFFFF));
              pipeline.fireUserEventTriggered(ChannelInputShutdownEvent.INSTANCE);
            } else {
              close(voidPromise());
            }
          }
        }
      }
    }
  }
  
  protected void doWrite(ChannelOutboundBuffer in) throws Exception
  {
    int writeSpinCount = -1;
    for (;;)
    {
      Object msg = in.current();
      if (msg == null)
      {
        clearOpWrite();
        break;
      }
      
      if ((msg instanceof ByteBuf)) {
        ByteBuf buf = (ByteBuf)msg;
        int readableBytes = buf.readableBytes();
        if (readableBytes == 0) {
          in.remove();
        }
        else {
          if (!buf.isDirect()) {
            ByteBufAllocator alloc = alloc();
            if (alloc.isDirectBufferPooled())
            {


              buf = alloc.directBuffer(readableBytes).writeBytes(buf);
              in.current(buf);
            }
          }
          boolean done = false;
          long flushedAmount = 0L;
          if (writeSpinCount == -1) {
            writeSpinCount = config().getWriteSpinCount();
          }
          for (int i = writeSpinCount - 1; i >= 0; i--) {
            int localFlushedAmount = doWriteBytes(buf);
            if (localFlushedAmount == 0) {
              break;
            }
            
            flushedAmount += localFlushedAmount;
            if (!buf.isReadable()) {
              done = true;
              break;
            }
          }
          
          in.progress(flushedAmount);
          
          if (done) {
            in.remove();
          }
          else {
            setOpWrite();
            break;
          }
        } } else if ((msg instanceof FileRegion)) {
        FileRegion region = (FileRegion)msg;
        boolean done = false;
        long flushedAmount = 0L;
        if (writeSpinCount == -1) {
          writeSpinCount = config().getWriteSpinCount();
        }
        for (int i = writeSpinCount - 1; i >= 0; i--) {
          long localFlushedAmount = doWriteFileRegion(region);
          if (localFlushedAmount == 0L) {
            break;
          }
          
          flushedAmount += localFlushedAmount;
          if (region.transfered() >= region.count()) {
            done = true;
            break;
          }
        }
        
        in.progress(flushedAmount);
        
        if (done) {
          in.remove();
        }
        else {
          setOpWrite();
          break;
        }
      } else {
        throw new UnsupportedOperationException("unsupported message type: " + StringUtil.simpleClassName(msg));
      }
    }
  }
  



  protected abstract long doWriteFileRegion(FileRegion paramFileRegion)
    throws Exception;
  



  protected abstract int doReadBytes(ByteBuf paramByteBuf)
    throws Exception;
  


  protected abstract int doWriteBytes(ByteBuf paramByteBuf)
    throws Exception;
  


  protected final void setOpWrite()
  {
    SelectionKey key = selectionKey();
    int interestOps = key.interestOps();
    if ((interestOps & 0x4) == 0) {
      key.interestOps(interestOps | 0x4);
    }
  }
  
  protected final void clearOpWrite() {
    SelectionKey key = selectionKey();
    int interestOps = key.interestOps();
    if ((interestOps & 0x4) != 0) {
      key.interestOps(interestOps & 0xFFFFFFFB);
    }
  }
}
