package net.minecraft.util.io.netty.channel.oio;

import java.io.IOException;
import net.minecraft.util.io.netty.buffer.ByteBuf;
import net.minecraft.util.io.netty.buffer.ByteBufAllocator;
import net.minecraft.util.io.netty.channel.Channel;
import net.minecraft.util.io.netty.channel.Channel.Unsafe;
import net.minecraft.util.io.netty.channel.ChannelConfig;
import net.minecraft.util.io.netty.channel.ChannelMetadata;
import net.minecraft.util.io.netty.channel.ChannelOption;
import net.minecraft.util.io.netty.channel.ChannelOutboundBuffer;
import net.minecraft.util.io.netty.channel.ChannelPipeline;
import net.minecraft.util.io.netty.channel.FileRegion;
import net.minecraft.util.io.netty.channel.socket.ChannelInputShutdownEvent;
import net.minecraft.util.io.netty.util.internal.StringUtil;
















public abstract class AbstractOioByteChannel
  extends AbstractOioChannel
{
  private volatile boolean inputShutdown;
  private static final ChannelMetadata METADATA = new ChannelMetadata(false);
  


  protected AbstractOioByteChannel(Channel parent)
  {
    super(parent);
  }
  
  protected boolean isInputShutdown() {
    return inputShutdown;
  }
  
  public ChannelMetadata metadata()
  {
    return METADATA;
  }
  



  protected boolean checkInputShutdown()
  {
    if (inputShutdown) {
      try {
        Thread.sleep(1000L);
      }
      catch (InterruptedException e) {}
      
      return true;
    }
    return false;
  }
  
  protected void doRead()
  {
    if (checkInputShutdown()) {
      return;
    }
    
    ChannelPipeline pipeline = pipeline();
    

    ByteBuf byteBuf = alloc().buffer();
    boolean closed = false;
    boolean read = false;
    Throwable exception = null;
    try {
      for (;;) {
        int localReadAmount = doReadBytes(byteBuf);
        if (localReadAmount > 0) {
          read = true;
        } else if (localReadAmount < 0) {
          closed = true;
        }
        
        int available = available();
        if (available <= 0) {
          break;
        }
        
        if (!byteBuf.isWritable()) {
          int capacity = byteBuf.capacity();
          int maxCapacity = byteBuf.maxCapacity();
          if (capacity == maxCapacity) {
            if (read) {
              read = false;
              pipeline.fireChannelRead(byteBuf);
              byteBuf = alloc().buffer();
            }
          } else {
            int writerIndex = byteBuf.writerIndex();
            if (writerIndex + available > maxCapacity) {
              byteBuf.capacity(maxCapacity);
            } else {
              byteBuf.ensureWritable(available);
            }
          }
        }
        if (!config().isAutoRead()) {
          break;
        }
      }
    }
    catch (Throwable t)
    {
      exception = t;
    } finally {
      if (read) {
        pipeline.fireChannelRead(byteBuf);
      }
      else {
        byteBuf.release();
      }
      
      pipeline.fireChannelReadComplete();
      if (exception != null) {
        if ((exception instanceof IOException)) {
          closed = true;
          pipeline().fireExceptionCaught(exception);
        } else {
          pipeline.fireExceptionCaught(exception);
          unsafe().close(voidPromise());
        }
      }
      
      if (closed) {
        inputShutdown = true;
        if (isOpen()) {
          if (Boolean.TRUE.equals(config().getOption(ChannelOption.ALLOW_HALF_CLOSURE))) {
            pipeline.fireUserEventTriggered(ChannelInputShutdownEvent.INSTANCE);
          } else {
            unsafe().close(unsafe().voidPromise());
          }
        }
      }
    }
  }
  
  protected void doWrite(ChannelOutboundBuffer in) throws Exception
  {
    for (;;) {
      Object msg = in.current();
      
      if ((msg instanceof ByteBuf)) {
        ByteBuf buf = (ByteBuf)msg;
        while (buf.isReadable()) {
          doWriteBytes(buf);
        }
        in.remove();
      } else if ((msg instanceof FileRegion)) {
        doWriteFileRegion((FileRegion)msg);
        in.remove();
      } else {
        in.remove(new UnsupportedOperationException("unsupported message type: " + StringUtil.simpleClassName(msg)));
      }
    }
  }
  
  protected abstract int available();
  
  protected abstract int doReadBytes(ByteBuf paramByteBuf)
    throws Exception;
  
  protected abstract void doWriteBytes(ByteBuf paramByteBuf)
    throws Exception;
  
  protected abstract void doWriteFileRegion(FileRegion paramFileRegion)
    throws Exception;
}
