package net.minecraft.util.io.netty.channel.oio;

import java.net.ConnectException;
import java.net.SocketAddress;
import net.minecraft.util.io.netty.channel.AbstractChannel;
import net.minecraft.util.io.netty.channel.AbstractChannel.AbstractUnsafe;
import net.minecraft.util.io.netty.channel.Channel;
import net.minecraft.util.io.netty.channel.ChannelPipeline;
import net.minecraft.util.io.netty.channel.ChannelPromise;
import net.minecraft.util.io.netty.channel.EventLoop;
import net.minecraft.util.io.netty.channel.ThreadPerChannelEventLoop;



















public abstract class AbstractOioChannel
  extends AbstractChannel
{
  protected static final int SO_TIMEOUT = 1000;
  private boolean readInProgress;
  private final Runnable readTask = new Runnable()
  {
    public void run() {
      readInProgress = false;
      doRead();
    }
  };
  


  protected AbstractOioChannel(Channel parent)
  {
    super(parent);
  }
  


  protected AbstractChannel.AbstractUnsafe newUnsafe() { return new DefaultOioUnsafe(null); }
  
  private final class DefaultOioUnsafe extends AbstractChannel.AbstractUnsafe {
    private DefaultOioUnsafe() { super(); }
    

    public void connect(SocketAddress remoteAddress, SocketAddress localAddress, ChannelPromise promise)
    {
      if (!ensureOpen(promise)) {
        return;
      }
      
      if (!promise.setUncancellable()) {
        close(voidPromise());
        return;
      }
      try
      {
        boolean wasActive = isActive();
        doConnect(remoteAddress, localAddress);
        promise.setSuccess();
        if ((!wasActive) && (isActive())) {
          pipeline().fireChannelActive();
        }
      } catch (Throwable t) {
        if ((t instanceof ConnectException)) {
          Throwable newT = new ConnectException(t.getMessage() + ": " + remoteAddress);
          newT.setStackTrace(t.getStackTrace());
          t = newT;
        }
        closeIfClosed();
        promise.setFailure(t);
      }
    }
  }
  
  protected boolean isCompatible(EventLoop loop)
  {
    return loop instanceof ThreadPerChannelEventLoop;
  }
  

  protected abstract void doConnect(SocketAddress paramSocketAddress1, SocketAddress paramSocketAddress2)
    throws Exception;
  

  protected void doBeginRead()
    throws Exception
  {
    if (readInProgress) {
      return;
    }
    
    readInProgress = true;
    eventLoop().execute(readTask);
  }
  
  protected abstract void doRead();
}
