package net.minecraft.util.io.netty.channel.sctp.nio;

import com.sun.nio.sctp.Association;
import com.sun.nio.sctp.MessageInfo;
import com.sun.nio.sctp.NotificationHandler;
import java.io.IOException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.SocketAddress;
import java.nio.ByteBuffer;
import java.nio.channels.SelectionKey;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;
import net.minecraft.util.io.netty.buffer.ByteBuf;
import net.minecraft.util.io.netty.buffer.ByteBufAllocator;
import net.minecraft.util.io.netty.channel.Channel;
import net.minecraft.util.io.netty.channel.ChannelException;
import net.minecraft.util.io.netty.channel.ChannelFuture;
import net.minecraft.util.io.netty.channel.ChannelMetadata;
import net.minecraft.util.io.netty.channel.ChannelOutboundBuffer;
import net.minecraft.util.io.netty.channel.ChannelPromise;
import net.minecraft.util.io.netty.channel.RecvByteBufAllocator;
import net.minecraft.util.io.netty.channel.RecvByteBufAllocator.Handle;
import net.minecraft.util.io.netty.channel.nio.AbstractNioMessageChannel;
import net.minecraft.util.io.netty.channel.nio.NioEventLoop;
import net.minecraft.util.io.netty.channel.sctp.DefaultSctpChannelConfig;
import net.minecraft.util.io.netty.channel.sctp.SctpChannelConfig;
import net.minecraft.util.io.netty.channel.sctp.SctpMessage;
import net.minecraft.util.io.netty.channel.sctp.SctpNotificationHandler;
import net.minecraft.util.io.netty.channel.sctp.SctpServerChannel;
import net.minecraft.util.io.netty.util.internal.PlatformDependent;
import net.minecraft.util.io.netty.util.internal.logging.InternalLogger;
import net.minecraft.util.io.netty.util.internal.logging.InternalLoggerFactory;




















public class NioSctpChannel
  extends AbstractNioMessageChannel
  implements net.minecraft.util.io.netty.channel.sctp.SctpChannel
{
  private static final ChannelMetadata METADATA = new ChannelMetadata(false);
  
  private static final InternalLogger logger = InternalLoggerFactory.getInstance(NioSctpChannel.class);
  
  private final SctpChannelConfig config;
  private final NotificationHandler<?> notificationHandler;
  private RecvByteBufAllocator.Handle allocHandle;
  
  private static com.sun.nio.sctp.SctpChannel newSctpChannel()
  {
    try
    {
      return com.sun.nio.sctp.SctpChannel.open();
    } catch (IOException e) {
      throw new ChannelException("Failed to open a sctp channel.", e);
    }
  }
  


  public NioSctpChannel()
  {
    this(newSctpChannel());
  }
  


  public NioSctpChannel(com.sun.nio.sctp.SctpChannel sctpChannel)
  {
    this(null, sctpChannel);
  }
  






  public NioSctpChannel(Channel parent, com.sun.nio.sctp.SctpChannel sctpChannel)
  {
    super(parent, sctpChannel, 1);
    try {
      sctpChannel.configureBlocking(false);
      config = new DefaultSctpChannelConfig(this, sctpChannel);
      notificationHandler = new SctpNotificationHandler(this);
    } catch (IOException e) {
      try {
        sctpChannel.close();
      } catch (IOException e2) {
        if (logger.isWarnEnabled()) {
          logger.warn("Failed to close a partially initialized sctp channel.", e2);
        }
      }
      

      throw new ChannelException("Failed to enter non-blocking mode.", e);
    }
  }
  
  public InetSocketAddress localAddress()
  {
    return (InetSocketAddress)super.localAddress();
  }
  
  public InetSocketAddress remoteAddress()
  {
    return (InetSocketAddress)super.remoteAddress();
  }
  
  public SctpServerChannel parent()
  {
    return (SctpServerChannel)super.parent();
  }
  
  public ChannelMetadata metadata()
  {
    return METADATA;
  }
  
  public Association association()
  {
    try {
      return javaChannel().association();
    } catch (IOException e) {}
    return null;
  }
  
  public Set<InetSocketAddress> allLocalAddresses()
  {
    try
    {
      Set<SocketAddress> allLocalAddresses = javaChannel().getAllLocalAddresses();
      Set<InetSocketAddress> addresses = new LinkedHashSet(allLocalAddresses.size());
      for (SocketAddress socketAddress : allLocalAddresses) {
        addresses.add((InetSocketAddress)socketAddress);
      }
      return addresses;
    } catch (Throwable t) {}
    return Collections.emptySet();
  }
  

  public SctpChannelConfig config()
  {
    return config;
  }
  
  public Set<InetSocketAddress> allRemoteAddresses()
  {
    try {
      Set<SocketAddress> allLocalAddresses = javaChannel().getRemoteAddresses();
      Set<InetSocketAddress> addresses = new HashSet(allLocalAddresses.size());
      for (SocketAddress socketAddress : allLocalAddresses) {
        addresses.add((InetSocketAddress)socketAddress);
      }
      return addresses;
    } catch (Throwable t) {}
    return Collections.emptySet();
  }
  

  protected com.sun.nio.sctp.SctpChannel javaChannel()
  {
    return (com.sun.nio.sctp.SctpChannel)super.javaChannel();
  }
  
  public boolean isActive()
  {
    com.sun.nio.sctp.SctpChannel ch = javaChannel();
    return (ch.isOpen()) && (association() != null);
  }
  
  protected SocketAddress localAddress0()
  {
    try {
      Iterator<SocketAddress> i = javaChannel().getAllLocalAddresses().iterator();
      if (i.hasNext()) {
        return (SocketAddress)i.next();
      }
    }
    catch (IOException e) {}
    
    return null;
  }
  
  protected SocketAddress remoteAddress0()
  {
    try {
      Iterator<SocketAddress> i = javaChannel().getRemoteAddresses().iterator();
      if (i.hasNext()) {
        return (SocketAddress)i.next();
      }
    }
    catch (IOException e) {}
    
    return null;
  }
  
  protected void doBind(SocketAddress localAddress) throws Exception
  {
    javaChannel().bind(localAddress);
  }
  
  protected boolean doConnect(SocketAddress remoteAddress, SocketAddress localAddress) throws Exception
  {
    if (localAddress != null) {
      javaChannel().bind(localAddress);
    }
    
    boolean success = false;
    try {
      boolean connected = javaChannel().connect(remoteAddress);
      if (!connected) {
        selectionKey().interestOps(8);
      }
      success = true;
      return connected;
    } finally {
      if (!success) {
        doClose();
      }
    }
  }
  
  protected void doFinishConnect() throws Exception
  {
    if (!javaChannel().finishConnect()) {
      throw new Error();
    }
  }
  
  protected void doDisconnect() throws Exception
  {
    doClose();
  }
  
  protected void doClose() throws Exception
  {
    javaChannel().close();
  }
  
  protected int doReadMessages(List<Object> buf) throws Exception
  {
    com.sun.nio.sctp.SctpChannel ch = javaChannel();
    
    RecvByteBufAllocator.Handle allocHandle = this.allocHandle;
    if (allocHandle == null) {
      this.allocHandle = (allocHandle = config().getRecvByteBufAllocator().newHandle());
    }
    ByteBuf buffer = allocHandle.allocate(config().getAllocator());
    boolean free = true;
    try {
      ByteBuffer data = buffer.nioBuffer(buffer.writerIndex(), buffer.writableBytes());
      messageInfo = ch.receive(data, null, notificationHandler);
      int i; if (messageInfo == null) { int bytesRead;
        return 0;
      }
      
      data.flip();
      buf.add(new SctpMessage(messageInfo, buffer.writerIndex(buffer.writerIndex() + data.remaining())));
      free = false;
      int bytesRead; return 1;
    } catch (Throwable cause) { MessageInfo messageInfo;
      PlatformDependent.throwException(cause);
      int bytesRead; return -1;
    } finally {
      int bytesRead = buffer.readableBytes();
      allocHandle.record(bytesRead);
      if (free) {
        buffer.release();
      }
    }
  }
  
  protected boolean doWriteMessage(Object msg, ChannelOutboundBuffer in) throws Exception
  {
    SctpMessage packet = (SctpMessage)msg;
    ByteBuf data = packet.content();
    int dataLen = data.readableBytes();
    if (dataLen == 0) {
      return true;
    }
    
    ByteBufAllocator alloc = alloc();
    boolean needsCopy = data.nioBufferCount() != 1;
    if ((!needsCopy) && 
      (!data.isDirect()) && (alloc.isDirectBufferPooled())) {
      needsCopy = true;
    }
    ByteBuffer nioData;
    ByteBuffer nioData;
    if (!needsCopy) {
      nioData = data.nioBuffer();
    } else {
      data = alloc.directBuffer(dataLen).writeBytes(data);
      nioData = data.nioBuffer();
    }
    
    MessageInfo mi = MessageInfo.createOutgoing(association(), null, packet.streamIdentifier());
    mi.payloadProtocolID(packet.protocolIdentifier());
    mi.streamNumber(packet.streamIdentifier());
    
    int writtenBytes = javaChannel().send(nioData, mi);
    
    boolean done = writtenBytes > 0;
    if (needsCopy) {
      if (!done) {
        in.current(new SctpMessage(mi, data));
      } else {
        in.current(data);
      }
    }
    return done;
  }
  
  public ChannelFuture bindAddress(InetAddress localAddress)
  {
    return bindAddress(localAddress, newPromise());
  }
  
  public ChannelFuture bindAddress(final InetAddress localAddress, final ChannelPromise promise)
  {
    if (eventLoop().inEventLoop()) {
      try {
        javaChannel().bindAddress(localAddress);
        promise.setSuccess();
      } catch (Throwable t) {
        promise.setFailure(t);
      }
    } else {
      eventLoop().execute(new Runnable()
      {
        public void run() {
          bindAddress(localAddress, promise);
        }
      });
    }
    return promise;
  }
  
  public ChannelFuture unbindAddress(InetAddress localAddress)
  {
    return unbindAddress(localAddress, newPromise());
  }
  
  public ChannelFuture unbindAddress(final InetAddress localAddress, final ChannelPromise promise)
  {
    if (eventLoop().inEventLoop()) {
      try {
        javaChannel().unbindAddress(localAddress);
        promise.setSuccess();
      } catch (Throwable t) {
        promise.setFailure(t);
      }
    } else {
      eventLoop().execute(new Runnable()
      {
        public void run() {
          unbindAddress(localAddress, promise);
        }
      });
    }
    return promise;
  }
}
