package net.minecraft.util.io.netty.channel.socket.nio;

import java.io.IOException;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.NetworkInterface;
import java.net.SocketAddress;
import java.net.SocketException;
import java.nio.ByteBuffer;
import java.nio.channels.MembershipKey;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import net.minecraft.util.io.netty.buffer.ByteBuf;
import net.minecraft.util.io.netty.buffer.ByteBufAllocator;
import net.minecraft.util.io.netty.buffer.ByteBufHolder;
import net.minecraft.util.io.netty.channel.AddressedEnvelope;
import net.minecraft.util.io.netty.channel.ChannelException;
import net.minecraft.util.io.netty.channel.ChannelFuture;
import net.minecraft.util.io.netty.channel.ChannelMetadata;
import net.minecraft.util.io.netty.channel.ChannelOption;
import net.minecraft.util.io.netty.channel.ChannelOutboundBuffer;
import net.minecraft.util.io.netty.channel.ChannelPromise;
import net.minecraft.util.io.netty.channel.DefaultAddressedEnvelope;
import net.minecraft.util.io.netty.channel.RecvByteBufAllocator;
import net.minecraft.util.io.netty.channel.RecvByteBufAllocator.Handle;
import net.minecraft.util.io.netty.channel.nio.AbstractNioMessageChannel;
import net.minecraft.util.io.netty.channel.socket.DatagramChannelConfig;
import net.minecraft.util.io.netty.channel.socket.DatagramPacket;
import net.minecraft.util.io.netty.channel.socket.InternetProtocolFamily;
import net.minecraft.util.io.netty.util.internal.PlatformDependent;
import net.minecraft.util.io.netty.util.internal.StringUtil;
























public final class NioDatagramChannel
  extends AbstractNioMessageChannel
  implements net.minecraft.util.io.netty.channel.socket.DatagramChannel
{
  private static final ChannelMetadata METADATA = new ChannelMetadata(true);
  
  private final DatagramChannelConfig config;
  private final Map<InetAddress, List<MembershipKey>> memberships = new HashMap();
  private RecvByteBufAllocator.Handle allocHandle;
  
  private static java.nio.channels.DatagramChannel newSocket()
  {
    try
    {
      return java.nio.channels.DatagramChannel.open();
    } catch (IOException e) {
      throw new ChannelException("Failed to open a socket.", e);
    }
  }
  
  private static java.nio.channels.DatagramChannel newSocket(InternetProtocolFamily ipFamily) {
    if (ipFamily == null) {
      return newSocket();
    }
    
    if (PlatformDependent.javaVersion() < 7) {
      throw new UnsupportedOperationException();
    }
    try
    {
      return java.nio.channels.DatagramChannel.open(ProtocolFamilyConverter.convert(ipFamily));
    } catch (IOException e) {
      throw new ChannelException("Failed to open a socket.", e);
    }
  }
  


  public NioDatagramChannel()
  {
    this(newSocket());
  }
  



  public NioDatagramChannel(InternetProtocolFamily ipFamily)
  {
    this(newSocket(ipFamily));
  }
  


  public NioDatagramChannel(java.nio.channels.DatagramChannel socket)
  {
    super(null, socket, 1);
    config = new NioDatagramChannelConfig(this, socket);
  }
  
  public ChannelMetadata metadata()
  {
    return METADATA;
  }
  
  public DatagramChannelConfig config()
  {
    return config;
  }
  
  public boolean isActive()
  {
    java.nio.channels.DatagramChannel ch = javaChannel();
    return (ch.isOpen()) && (((((Boolean)config.getOption(ChannelOption.DATAGRAM_CHANNEL_ACTIVE_ON_REGISTRATION)).booleanValue()) && (isRegistered())) || (ch.socket().isBound()));
  }
  


  public boolean isConnected()
  {
    return javaChannel().isConnected();
  }
  
  protected java.nio.channels.DatagramChannel javaChannel()
  {
    return (java.nio.channels.DatagramChannel)super.javaChannel();
  }
  
  protected SocketAddress localAddress0()
  {
    return javaChannel().socket().getLocalSocketAddress();
  }
  
  protected SocketAddress remoteAddress0()
  {
    return javaChannel().socket().getRemoteSocketAddress();
  }
  
  protected void doBind(SocketAddress localAddress) throws Exception
  {
    javaChannel().socket().bind(localAddress);
  }
  
  protected boolean doConnect(SocketAddress remoteAddress, SocketAddress localAddress)
    throws Exception
  {
    if (localAddress != null) {
      javaChannel().socket().bind(localAddress);
    }
    
    boolean success = false;
    try {
      javaChannel().connect(remoteAddress);
      success = true;
      return true;
    } finally {
      if (!success) {
        doClose();
      }
    }
  }
  
  protected void doFinishConnect() throws Exception
  {
    throw new Error();
  }
  
  protected void doDisconnect() throws Exception
  {
    javaChannel().disconnect();
  }
  
  protected void doClose() throws Exception
  {
    javaChannel().close();
  }
  
  protected int doReadMessages(List<Object> buf) throws Exception
  {
    java.nio.channels.DatagramChannel ch = javaChannel();
    DatagramChannelConfig config = config();
    RecvByteBufAllocator.Handle allocHandle = this.allocHandle;
    if (allocHandle == null) {
      this.allocHandle = (allocHandle = config.getRecvByteBufAllocator().newHandle());
    }
    ByteBuf data = allocHandle.allocate(config.getAllocator());
    boolean free = true;
    try {
      ByteBuffer nioData = data.nioBuffer(data.writerIndex(), data.writableBytes());
      
      remoteAddress = (InetSocketAddress)ch.receive(nioData);
      if (remoteAddress == null) {
        return 0;
      }
      
      int readBytes = nioData.position();
      data.writerIndex(data.writerIndex() + readBytes);
      allocHandle.record(readBytes);
      
      buf.add(new DatagramPacket(data, localAddress(), remoteAddress));
      free = false;
      return 1;
    } catch (Throwable cause) { InetSocketAddress remoteAddress;
      PlatformDependent.throwException(cause);
      return -1;
    } finally {
      if (free) {
        data.release();
      }
    }
  }
  
  protected boolean doWriteMessage(Object msg, ChannelOutboundBuffer in) throws Exception
  {
    Object m;
    Object m;
    SocketAddress remoteAddress;
    if ((msg instanceof AddressedEnvelope))
    {
      AddressedEnvelope<Object, SocketAddress> envelope = (AddressedEnvelope)msg;
      SocketAddress remoteAddress = envelope.recipient();
      m = envelope.content();
    } else {
      m = msg;
      remoteAddress = null;
    }
    ByteBuf data;
    if ((m instanceof ByteBufHolder)) {
      data = ((ByteBufHolder)m).content(); } else { ByteBuf data;
      if ((m instanceof ByteBuf)) {
        data = (ByteBuf)m;
      } else
        throw new UnsupportedOperationException("unsupported message type: " + StringUtil.simpleClassName(msg));
    }
    ByteBuf data;
    int dataLen = data.readableBytes();
    if (dataLen == 0) {
      return true;
    }
    
    ByteBufAllocator alloc = alloc();
    boolean needsCopy = data.nioBufferCount() != 1;
    if ((!needsCopy) && 
      (!data.isDirect()) && (alloc.isDirectBufferPooled())) {
      needsCopy = true;
    }
    ByteBuffer nioData;
    ByteBuffer nioData;
    if (!needsCopy) {
      nioData = data.nioBuffer();
    } else {
      data = alloc.directBuffer(dataLen).writeBytes(data);
      nioData = data.nioBuffer();
    }
    int writtenBytes;
    int writtenBytes;
    if (remoteAddress != null) {
      writtenBytes = javaChannel().send(nioData, remoteAddress);
    } else {
      writtenBytes = javaChannel().write(nioData);
    }
    
    boolean done = writtenBytes > 0;
    if (needsCopy)
    {

      if (remoteAddress == null)
      {
        in.current(data);
      }
      else if (!done)
      {
        in.current(new DefaultAddressedEnvelope(data, remoteAddress));
      }
      else {
        in.current(data);
      }
    }
    
    return done;
  }
  
  public InetSocketAddress localAddress()
  {
    return (InetSocketAddress)super.localAddress();
  }
  
  public InetSocketAddress remoteAddress()
  {
    return (InetSocketAddress)super.remoteAddress();
  }
  
  public ChannelFuture joinGroup(InetAddress multicastAddress)
  {
    return joinGroup(multicastAddress, newPromise());
  }
  
  public ChannelFuture joinGroup(InetAddress multicastAddress, ChannelPromise promise)
  {
    try {
      return joinGroup(multicastAddress, NetworkInterface.getByInetAddress(localAddress().getAddress()), null, promise);

    }
    catch (SocketException e)
    {
      promise.setFailure(e);
    }
    return promise;
  }
  

  public ChannelFuture joinGroup(InetSocketAddress multicastAddress, NetworkInterface networkInterface)
  {
    return joinGroup(multicastAddress, networkInterface, newPromise());
  }
  


  public ChannelFuture joinGroup(InetSocketAddress multicastAddress, NetworkInterface networkInterface, ChannelPromise promise)
  {
    return joinGroup(multicastAddress.getAddress(), networkInterface, null, promise);
  }
  

  public ChannelFuture joinGroup(InetAddress multicastAddress, NetworkInterface networkInterface, InetAddress source)
  {
    return joinGroup(multicastAddress, networkInterface, source, newPromise());
  }
  


  public ChannelFuture joinGroup(InetAddress multicastAddress, NetworkInterface networkInterface, InetAddress source, ChannelPromise promise)
  {
    if (PlatformDependent.javaVersion() >= 7) {
      if (multicastAddress == null) {
        throw new NullPointerException("multicastAddress");
      }
      
      if (networkInterface == null) {
        throw new NullPointerException("networkInterface");
      }
      try {
        MembershipKey key;
        MembershipKey key;
        if (source == null) {
          key = javaChannel().join(multicastAddress, networkInterface);
        } else {
          key = javaChannel().join(multicastAddress, networkInterface, source);
        }
        
        synchronized (this) {
          List<MembershipKey> keys = (List)memberships.get(multicastAddress);
          if (keys == null) {
            keys = new ArrayList();
            memberships.put(multicastAddress, keys);
          }
          keys.add(key);
        }
        
        promise.setSuccess();
      } catch (Throwable e) {
        promise.setFailure(e);
      }
    } else {
      throw new UnsupportedOperationException();
    }
    return promise;
  }
  
  public ChannelFuture leaveGroup(InetAddress multicastAddress)
  {
    return leaveGroup(multicastAddress, newPromise());
  }
  
  public ChannelFuture leaveGroup(InetAddress multicastAddress, ChannelPromise promise)
  {
    try {
      return leaveGroup(multicastAddress, NetworkInterface.getByInetAddress(localAddress().getAddress()), null, promise);
    }
    catch (SocketException e) {
      promise.setFailure(e);
    }
    return promise;
  }
  

  public ChannelFuture leaveGroup(InetSocketAddress multicastAddress, NetworkInterface networkInterface)
  {
    return leaveGroup(multicastAddress, networkInterface, newPromise());
  }
  


  public ChannelFuture leaveGroup(InetSocketAddress multicastAddress, NetworkInterface networkInterface, ChannelPromise promise)
  {
    return leaveGroup(multicastAddress.getAddress(), networkInterface, null, promise);
  }
  

  public ChannelFuture leaveGroup(InetAddress multicastAddress, NetworkInterface networkInterface, InetAddress source)
  {
    return leaveGroup(multicastAddress, networkInterface, source, newPromise());
  }
  


  public ChannelFuture leaveGroup(InetAddress multicastAddress, NetworkInterface networkInterface, InetAddress source, ChannelPromise promise)
  {
    if (PlatformDependent.javaVersion() < 7) {
      throw new UnsupportedOperationException();
    }
    if (multicastAddress == null) {
      throw new NullPointerException("multicastAddress");
    }
    if (networkInterface == null) {
      throw new NullPointerException("networkInterface");
    }
    
    synchronized (this) {
      if (memberships != null) {
        List<MembershipKey> keys = (List)memberships.get(multicastAddress);
        if (keys != null) {
          Iterator<MembershipKey> keyIt = keys.iterator();
          
          while (keyIt.hasNext()) {
            MembershipKey key = (MembershipKey)keyIt.next();
            if ((networkInterface.equals(key.networkInterface())) && (
              ((source == null) && (key.sourceAddress() == null)) || ((source != null) && (source.equals(key.sourceAddress())))))
            {
              key.drop();
              keyIt.remove();
            }
          }
          
          if (keys.isEmpty()) {
            memberships.remove(multicastAddress);
          }
        }
      }
    }
    
    promise.setSuccess();
    return promise;
  }
  





  public ChannelFuture block(InetAddress multicastAddress, NetworkInterface networkInterface, InetAddress sourceToBlock)
  {
    return block(multicastAddress, networkInterface, sourceToBlock, newPromise());
  }
  





  public ChannelFuture block(InetAddress multicastAddress, NetworkInterface networkInterface, InetAddress sourceToBlock, ChannelPromise promise)
  {
    if (PlatformDependent.javaVersion() < 7) {
      throw new UnsupportedOperationException();
    }
    if (multicastAddress == null) {
      throw new NullPointerException("multicastAddress");
    }
    if (sourceToBlock == null) {
      throw new NullPointerException("sourceToBlock");
    }
    
    if (networkInterface == null) {
      throw new NullPointerException("networkInterface");
    }
    synchronized (this) {
      if (memberships != null) {
        List<MembershipKey> keys = (List)memberships.get(multicastAddress);
        for (MembershipKey key : keys) {
          if (networkInterface.equals(key.networkInterface())) {
            try {
              key.block(sourceToBlock);
            } catch (IOException e) {
              promise.setFailure(e);
            }
          }
        }
      }
    }
    promise.setSuccess();
    return promise;
  }
  





  public ChannelFuture block(InetAddress multicastAddress, InetAddress sourceToBlock)
  {
    return block(multicastAddress, sourceToBlock, newPromise());
  }
  




  public ChannelFuture block(InetAddress multicastAddress, InetAddress sourceToBlock, ChannelPromise promise)
  {
    try
    {
      return block(multicastAddress, NetworkInterface.getByInetAddress(localAddress().getAddress()), sourceToBlock, promise);

    }
    catch (SocketException e)
    {
      promise.setFailure(e);
    }
    return promise;
  }
}
