package net.minecraft.util.io.netty.channel.socket.nio;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.net.SocketAddress;
import java.nio.ByteBuffer;
import java.nio.channels.SelectionKey;
import net.minecraft.util.io.netty.buffer.ByteBuf;
import net.minecraft.util.io.netty.channel.Channel;
import net.minecraft.util.io.netty.channel.ChannelException;
import net.minecraft.util.io.netty.channel.ChannelFuture;
import net.minecraft.util.io.netty.channel.ChannelMetadata;
import net.minecraft.util.io.netty.channel.ChannelOutboundBuffer;
import net.minecraft.util.io.netty.channel.ChannelPromise;
import net.minecraft.util.io.netty.channel.EventLoop;
import net.minecraft.util.io.netty.channel.FileRegion;
import net.minecraft.util.io.netty.channel.nio.AbstractNioByteChannel;
import net.minecraft.util.io.netty.channel.socket.DefaultSocketChannelConfig;
import net.minecraft.util.io.netty.channel.socket.ServerSocketChannel;
import net.minecraft.util.io.netty.channel.socket.SocketChannelConfig;


















public class NioSocketChannel
  extends AbstractNioByteChannel
  implements net.minecraft.util.io.netty.channel.socket.SocketChannel
{
  private static final ChannelMetadata METADATA = new ChannelMetadata(false);
  private final SocketChannelConfig config;
  
  private static java.nio.channels.SocketChannel newSocket() {
    try { return java.nio.channels.SocketChannel.open();
    } catch (IOException e) {
      throw new ChannelException("Failed to open a socket.", e);
    }
  }
  




  public NioSocketChannel()
  {
    this(newSocket());
  }
  


  public NioSocketChannel(java.nio.channels.SocketChannel socket)
  {
    this(null, socket);
  }
  





  public NioSocketChannel(Channel parent, java.nio.channels.SocketChannel socket)
  {
    super(parent, socket);
    config = new DefaultSocketChannelConfig(this, socket.socket());
  }
  
  public ServerSocketChannel parent()
  {
    return (ServerSocketChannel)super.parent();
  }
  
  public ChannelMetadata metadata()
  {
    return METADATA;
  }
  
  public SocketChannelConfig config()
  {
    return config;
  }
  
  protected java.nio.channels.SocketChannel javaChannel()
  {
    return (java.nio.channels.SocketChannel)super.javaChannel();
  }
  
  public boolean isActive()
  {
    java.nio.channels.SocketChannel ch = javaChannel();
    return (ch.isOpen()) && (ch.isConnected());
  }
  
  public boolean isInputShutdown()
  {
    return super.isInputShutdown();
  }
  
  public InetSocketAddress localAddress()
  {
    return (InetSocketAddress)super.localAddress();
  }
  
  public InetSocketAddress remoteAddress()
  {
    return (InetSocketAddress)super.remoteAddress();
  }
  
  public boolean isOutputShutdown()
  {
    return (javaChannel().socket().isOutputShutdown()) || (!isActive());
  }
  
  public ChannelFuture shutdownOutput()
  {
    return shutdownOutput(newPromise());
  }
  
  public ChannelFuture shutdownOutput(final ChannelPromise promise)
  {
    EventLoop loop = eventLoop();
    if (loop.inEventLoop()) {
      try {
        javaChannel().socket().shutdownOutput();
        promise.setSuccess();
      } catch (Throwable t) {
        promise.setFailure(t);
      }
    } else {
      loop.execute(new Runnable()
      {
        public void run() {
          shutdownOutput(promise);
        }
      });
    }
    return promise;
  }
  
  protected SocketAddress localAddress0()
  {
    return javaChannel().socket().getLocalSocketAddress();
  }
  
  protected SocketAddress remoteAddress0()
  {
    return javaChannel().socket().getRemoteSocketAddress();
  }
  
  protected void doBind(SocketAddress localAddress) throws Exception
  {
    javaChannel().socket().bind(localAddress);
  }
  
  protected boolean doConnect(SocketAddress remoteAddress, SocketAddress localAddress) throws Exception
  {
    if (localAddress != null) {
      javaChannel().socket().bind(localAddress);
    }
    
    boolean success = false;
    try {
      boolean connected = javaChannel().connect(remoteAddress);
      if (!connected) {
        selectionKey().interestOps(8);
      }
      success = true;
      return connected;
    } finally {
      if (!success) {
        doClose();
      }
    }
  }
  
  protected void doFinishConnect() throws Exception
  {
    if (!javaChannel().finishConnect()) {
      throw new Error();
    }
  }
  
  protected void doDisconnect() throws Exception
  {
    doClose();
  }
  
  protected void doClose() throws Exception
  {
    javaChannel().close();
  }
  
  protected int doReadBytes(ByteBuf byteBuf) throws Exception
  {
    return byteBuf.writeBytes(javaChannel(), byteBuf.writableBytes());
  }
  
  protected int doWriteBytes(ByteBuf buf) throws Exception
  {
    int expectedWrittenBytes = buf.readableBytes();
    int writtenBytes = buf.readBytes(javaChannel(), expectedWrittenBytes);
    return writtenBytes;
  }
  
  protected long doWriteFileRegion(FileRegion region) throws Exception
  {
    long position = region.transfered();
    long writtenBytes = region.transferTo(javaChannel(), position);
    return writtenBytes;
  }
  
  protected void doWrite(ChannelOutboundBuffer in) throws Exception
  {
    for (;;)
    {
      int msgCount = in.size();
      if (msgCount <= 1) {
        super.doWrite(in);
        return;
      }
      

      ByteBuffer[] nioBuffers = in.nioBuffers();
      if (nioBuffers == null) {
        super.doWrite(in);
        return;
      }
      
      int nioBufferCnt = in.nioBufferCount();
      long expectedWrittenBytes = in.nioBufferSize();
      
      java.nio.channels.SocketChannel ch = javaChannel();
      long writtenBytes = 0L;
      boolean done = false;
      for (int i = config().getWriteSpinCount() - 1; i >= 0; i--) {
        long localWrittenBytes = ch.write(nioBuffers, 0, nioBufferCnt);
        if (localWrittenBytes == 0L) {
          break;
        }
        expectedWrittenBytes -= localWrittenBytes;
        writtenBytes += localWrittenBytes;
        if (expectedWrittenBytes == 0L) {
          done = true;
          break;
        }
      }
      
      if (done)
      {
        for (int i = msgCount; i > 0; i--) {
          in.remove();
        }
        

        if (in.isEmpty()) {
          clearOpWrite();
          break;
        }
        
      }
      else
      {
        for (int i = msgCount; i > 0; i--) {
          ByteBuf buf = (ByteBuf)in.current();
          int readerIndex = buf.readerIndex();
          int readableBytes = buf.writerIndex() - readerIndex;
          
          if (readableBytes < writtenBytes) {
            in.progress(readableBytes);
            in.remove();
            writtenBytes -= readableBytes;
          } else { if (readableBytes > writtenBytes) {
              buf.readerIndex(readerIndex + (int)writtenBytes);
              in.progress(writtenBytes);
              break;
            }
            in.progress(readableBytes);
            in.remove();
            break;
          }
        }
        
        setOpWrite();
        break;
      }
    }
  }
}
