package net.minecraft.util.io.netty.handler.codec;

import java.util.List;
import net.minecraft.util.io.netty.buffer.ByteBuf;
import net.minecraft.util.io.netty.channel.ChannelDuplexHandler;
import net.minecraft.util.io.netty.channel.ChannelHandler.Sharable;
import net.minecraft.util.io.netty.channel.ChannelHandlerContext;
import net.minecraft.util.io.netty.channel.ChannelPromise;
import net.minecraft.util.io.netty.util.internal.TypeParameterMatcher;
























public abstract class ByteToMessageCodec<I>
  extends ChannelDuplexHandler
{
  private final TypeParameterMatcher outboundMsgMatcher;
  private final MessageToByteEncoder<I> encoder;
  private final ByteToMessageDecoder decoder = new ByteToMessageDecoder()
  {
    public void decode(ChannelHandlerContext ctx, ByteBuf in, List<Object> out) throws Exception {
      ByteToMessageCodec.this.decode(ctx, in, out);
    }
    
    protected void decodeLast(ChannelHandlerContext ctx, ByteBuf in, List<Object> out) throws Exception
    {
      ByteToMessageCodec.this.decodeLast(ctx, in, out);
    }
  };
  


  protected ByteToMessageCodec()
  {
    this(true);
  }
  


  protected ByteToMessageCodec(Class<? extends I> outboundMessageType)
  {
    this(outboundMessageType, true);
  }
  






  protected ByteToMessageCodec(boolean preferDirect)
  {
    outboundMsgMatcher = TypeParameterMatcher.find(this, ByteToMessageCodec.class, "I");
    encoder = new Encoder(preferDirect);
  }
  







  protected ByteToMessageCodec(Class<? extends I> outboundMessageType, boolean preferDirect)
  {
    checkForSharableAnnotation();
    outboundMsgMatcher = TypeParameterMatcher.get(outboundMessageType);
    encoder = new Encoder(preferDirect);
  }
  
  private void checkForSharableAnnotation() {
    if (getClass().isAnnotationPresent(ChannelHandler.Sharable.class)) {
      throw new IllegalStateException("@Sharable annotation is not allowed");
    }
  }
  



  public boolean acceptOutboundMessage(Object msg)
    throws Exception
  {
    return outboundMsgMatcher.match(msg);
  }
  
  public void channelRead(ChannelHandlerContext ctx, Object msg) throws Exception
  {
    decoder.channelRead(ctx, msg);
  }
  
  public void write(ChannelHandlerContext ctx, Object msg, ChannelPromise promise) throws Exception
  {
    encoder.write(ctx, msg, promise);
  }
  


  protected abstract void encode(ChannelHandlerContext paramChannelHandlerContext, I paramI, ByteBuf paramByteBuf)
    throws Exception;
  


  protected abstract void decode(ChannelHandlerContext paramChannelHandlerContext, ByteBuf paramByteBuf, List<Object> paramList)
    throws Exception;
  

  protected void decodeLast(ChannelHandlerContext ctx, ByteBuf in, List<Object> out)
    throws Exception
  {
    decode(ctx, in, out);
  }
  
  private final class Encoder extends MessageToByteEncoder<I> {
    Encoder(boolean preferDirect) {
      super();
    }
    
    public boolean acceptOutboundMessage(Object msg) throws Exception
    {
      return ByteToMessageCodec.this.acceptOutboundMessage(msg);
    }
    
    protected void encode(ChannelHandlerContext ctx, I msg, ByteBuf out) throws Exception
    {
      ByteToMessageCodec.this.encode(ctx, msg, out);
    }
  }
}
