package net.minecraft.util.io.netty.handler.codec;

import java.util.List;
import net.minecraft.util.io.netty.buffer.ByteBuf;
import net.minecraft.util.io.netty.buffer.ByteBufAllocator;
import net.minecraft.util.io.netty.buffer.Unpooled;
import net.minecraft.util.io.netty.channel.Channel;
import net.minecraft.util.io.netty.channel.ChannelConfig;
import net.minecraft.util.io.netty.channel.ChannelHandler.Sharable;
import net.minecraft.util.io.netty.channel.ChannelHandlerContext;
import net.minecraft.util.io.netty.channel.ChannelInboundHandlerAdapter;
import net.minecraft.util.io.netty.util.internal.RecyclableArrayList;
import net.minecraft.util.io.netty.util.internal.StringUtil;































public abstract class ByteToMessageDecoder
  extends ChannelInboundHandlerAdapter
{
  ByteBuf cumulation;
  private boolean singleDecode;
  private boolean decodeWasNull;
  
  protected ByteToMessageDecoder()
  {
    if (getClass().isAnnotationPresent(ChannelHandler.Sharable.class)) {
      throw new IllegalStateException("@Sharable annotation is not allowed");
    }
  }
  





  public void setSingleDecode(boolean singleDecode)
  {
    this.singleDecode = singleDecode;
  }
  





  public boolean isSingleDecode()
  {
    return singleDecode;
  }
  





  protected int actualReadableBytes()
  {
    return internalBuffer().readableBytes();
  }
  




  protected ByteBuf internalBuffer()
  {
    if (cumulation != null) {
      return cumulation;
    }
    return Unpooled.EMPTY_BUFFER;
  }
  
  public final void handlerRemoved(ChannelHandlerContext ctx)
    throws Exception
  {
    ByteBuf buf = internalBuffer();
    int readable = buf.readableBytes();
    if (buf.isReadable()) {
      ByteBuf bytes = buf.readBytes(readable);
      buf.release();
      ctx.fireChannelRead(bytes);
    }
    cumulation = null;
    ctx.fireChannelReadComplete();
    handlerRemoved0(ctx);
  }
  

  protected void handlerRemoved0(ChannelHandlerContext ctx)
    throws Exception
  {}
  
  public void channelRead(ChannelHandlerContext ctx, Object msg)
    throws Exception
  {
    RecyclableArrayList out = RecyclableArrayList.newInstance();
    try {
      if ((msg instanceof ByteBuf)) {
        ByteBuf data = (ByteBuf)msg;
        if (cumulation == null) {
          cumulation = data;
          try {
            callDecode(ctx, cumulation, out);
          } finally {
            if ((cumulation != null) && (!cumulation.isReadable())) {
              cumulation.release();
              cumulation = null;
            }
          }
        } else {
          try {
            if (cumulation.writerIndex() > cumulation.maxCapacity() - data.readableBytes()) {
              ByteBuf oldCumulation = cumulation;
              cumulation = ctx.alloc().buffer(oldCumulation.readableBytes() + data.readableBytes());
              cumulation.writeBytes(oldCumulation);
              oldCumulation.release();
            }
            cumulation.writeBytes(data);
            callDecode(ctx, cumulation, out);
          } finally {
            if (cumulation != null) {
              if (!cumulation.isReadable()) {
                cumulation.release();
                cumulation = null;
              } else {
                cumulation.discardSomeReadBytes();
              }
            }
            data.release();
          }
        }
      } else {
        out.add(msg);
      } } catch (DecoderException e) { int size;
      int i;
      throw e;
    } catch (Throwable t) {
      throw new DecoderException(t);
    } finally {
      int size = out.size();
      if (size == 0) {
        decodeWasNull = true;
      } else {
        for (int i = 0; i < size; i++) {
          ctx.fireChannelRead(out.get(i));
        }
      }
      out.recycle();
    }
  }
  
  public void channelReadComplete(ChannelHandlerContext ctx) throws Exception
  {
    if (decodeWasNull) {
      decodeWasNull = false;
      if (!ctx.channel().config().isAutoRead()) {
        ctx.read();
      }
    }
    ctx.fireChannelReadComplete();
  }
  
  public void channelInactive(ChannelHandlerContext ctx) throws Exception
  {
    RecyclableArrayList out = RecyclableArrayList.newInstance();
    try {
      if (cumulation != null) {
        callDecode(ctx, cumulation, out);
        decodeLast(ctx, cumulation, out);
      } else {
        decodeLast(ctx, Unpooled.EMPTY_BUFFER, out);
      } } catch (DecoderException e) { int size;
      int i;
      throw e;
    } catch (Exception e) {
      throw new DecoderException(e);
    } finally {
      if (cumulation != null) {
        cumulation.release();
        cumulation = null;
      }
      int size = out.size();
      for (int i = 0; i < size; i++) {
        ctx.fireChannelRead(out.get(i));
      }
      ctx.fireChannelInactive();
      out.recycle();
    }
  }
  






  protected void callDecode(ChannelHandlerContext ctx, ByteBuf in, List<Object> out)
  {
    try
    {
      while (in.isReadable()) {
        int outSize = out.size();
        int oldInputLength = in.readableBytes();
        decode(ctx, in, out);
        




        if (ctx.isRemoved()) {
          break;
        }
        
        if (outSize == out.size()) {
          if (oldInputLength == in.readableBytes()) {
            break;
          }
          
        }
        else
        {
          if (oldInputLength == in.readableBytes()) {
            throw new DecoderException(StringUtil.simpleClassName(getClass()) + ".decode() did not read anything but decoded a message.");
          }
          


          if (isSingleDecode())
            break;
        }
      }
    } catch (DecoderException e) {
      throw e;
    } catch (Throwable cause) {
      throw new DecoderException(cause);
    }
  }
  








  protected abstract void decode(ChannelHandlerContext paramChannelHandlerContext, ByteBuf paramByteBuf, List<Object> paramList)
    throws Exception;
  







  protected void decodeLast(ChannelHandlerContext ctx, ByteBuf in, List<Object> out)
    throws Exception
  {
    decode(ctx, in, out);
  }
}
