package net.minecraft.util.io.netty.handler.codec;

import net.minecraft.util.io.netty.buffer.ByteBuf;
import net.minecraft.util.io.netty.buffer.ByteBufAllocator;
import net.minecraft.util.io.netty.buffer.Unpooled;
import net.minecraft.util.io.netty.channel.ChannelHandlerContext;
import net.minecraft.util.io.netty.channel.ChannelOutboundHandlerAdapter;
import net.minecraft.util.io.netty.channel.ChannelPromise;
import net.minecraft.util.io.netty.util.ReferenceCountUtil;
import net.minecraft.util.io.netty.util.internal.TypeParameterMatcher;




































public abstract class MessageToByteEncoder<I>
  extends ChannelOutboundHandlerAdapter
{
  private final TypeParameterMatcher matcher;
  private final boolean preferDirect;
  
  protected MessageToByteEncoder()
  {
    this(true);
  }
  


  protected MessageToByteEncoder(Class<? extends I> outboundMessageType)
  {
    this(outboundMessageType, true);
  }
  






  protected MessageToByteEncoder(boolean preferDirect)
  {
    matcher = TypeParameterMatcher.find(this, MessageToByteEncoder.class, "I");
    this.preferDirect = preferDirect;
  }
  







  protected MessageToByteEncoder(Class<? extends I> outboundMessageType, boolean preferDirect)
  {
    matcher = TypeParameterMatcher.get(outboundMessageType);
    this.preferDirect = preferDirect;
  }
  


  public boolean acceptOutboundMessage(Object msg)
    throws Exception
  {
    return matcher.match(msg);
  }
  
  public void write(ChannelHandlerContext ctx, Object msg, ChannelPromise promise) throws Exception
  {
    ByteBuf buf = null;
    try {
      if (acceptOutboundMessage(msg))
      {
        I cast = msg;
        if (preferDirect) {
          buf = ctx.alloc().ioBuffer();
        } else {
          buf = ctx.alloc().heapBuffer();
        }
        try {
          encode(ctx, cast, buf);
        } finally {
          ReferenceCountUtil.release(cast);
        }
        
        if (buf.isReadable()) {
          ctx.write(buf, promise);
        } else {
          buf.release();
          ctx.write(Unpooled.EMPTY_BUFFER, promise);
        }
        buf = null;
      } else {
        ctx.write(msg, promise);
      }
    } catch (EncoderException e) {
      throw e;
    } catch (Throwable e) {
      throw new EncoderException(e);
    } finally {
      if (buf != null) {
        buf.release();
      }
    }
  }
  
  protected abstract void encode(ChannelHandlerContext paramChannelHandlerContext, I paramI, ByteBuf paramByteBuf)
    throws Exception;
}
