package net.minecraft.util.io.netty.handler.codec;

import java.util.List;
import net.minecraft.util.io.netty.channel.ChannelHandlerContext;
import net.minecraft.util.io.netty.channel.ChannelOutboundHandlerAdapter;
import net.minecraft.util.io.netty.channel.ChannelPromise;
import net.minecraft.util.io.netty.util.ReferenceCountUtil;
import net.minecraft.util.io.netty.util.internal.RecyclableArrayList;
import net.minecraft.util.io.netty.util.internal.StringUtil;
import net.minecraft.util.io.netty.util.internal.TypeParameterMatcher;










































public abstract class MessageToMessageEncoder<I>
  extends ChannelOutboundHandlerAdapter
{
  private final TypeParameterMatcher matcher;
  
  protected MessageToMessageEncoder()
  {
    matcher = TypeParameterMatcher.find(this, MessageToMessageEncoder.class, "I");
  }
  




  protected MessageToMessageEncoder(Class<? extends I> outboundMessageType)
  {
    matcher = TypeParameterMatcher.get(outboundMessageType);
  }
  


  public boolean acceptOutboundMessage(Object msg)
    throws Exception
  {
    return matcher.match(msg);
  }
  
  public void write(ChannelHandlerContext ctx, Object msg, ChannelPromise promise) throws Exception
  {
    RecyclableArrayList out = null;
    try {
      if (acceptOutboundMessage(msg)) {
        out = RecyclableArrayList.newInstance();
        
        I cast = msg;
        try {
          encode(ctx, cast, out);
        } finally {
          ReferenceCountUtil.release(cast);
        }
        
        if (out.isEmpty()) {
          out.recycle();
          out = null;
          
          throw new EncoderException(StringUtil.simpleClassName(this) + " must produce at least one message.");
        }
      }
      else {
        ctx.write(msg, promise);
      } } catch (EncoderException e) { int sizeMinusOne;
      int i;
      throw e;
    } catch (Throwable t) {
      throw new EncoderException(t);
    } finally {
      if (out != null) {
        int sizeMinusOne = out.size() - 1;
        if (sizeMinusOne >= 0) {
          for (int i = 0; i < sizeMinusOne; i++) {
            ctx.write(out.get(i));
          }
          ctx.write(out.get(sizeMinusOne), promise);
        }
        out.recycle();
      }
    }
  }
  
  protected abstract void encode(ChannelHandlerContext paramChannelHandlerContext, I paramI, List<Object> paramList)
    throws Exception;
}
