package net.minecraft.util.io.netty.handler.codec.http;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;
import net.minecraft.util.io.netty.util.internal.StringUtil;

































public final class CookieDecoder
{
  private static final char COMMA = ',';
  
  public static Set<Cookie> decode(String header)
  {
    List<String> names = new ArrayList(8);
    List<String> values = new ArrayList(8);
    extractKeyValuePairs(header, names, values);
    
    if (names.isEmpty()) {
      return Collections.emptySet();
    }
    

    int version = 0;
    
    int i;
    int i;
    if (((String)names.get(0)).equalsIgnoreCase("Version")) {
      try {
        version = Integer.parseInt((String)values.get(0));
      }
      catch (NumberFormatException e) {}
      
      i = 1;
    } else {
      i = 0;
    }
    
    if (names.size() <= i)
    {
      return Collections.emptySet();
    }
    
    Set<Cookie> cookies = new TreeSet();
    for (; i < names.size(); i++) {
      String name = (String)names.get(i);
      String value = (String)values.get(i);
      if (value == null) {
        value = "";
      }
      
      Cookie c = new DefaultCookie(name, value);
      
      boolean discard = false;
      boolean secure = false;
      boolean httpOnly = false;
      String comment = null;
      String commentURL = null;
      String domain = null;
      String path = null;
      long maxAge = -9223372036854775808L;
      List<Integer> ports = new ArrayList(2);
      
      for (int j = i + 1; j < names.size(); i++) {
        name = (String)names.get(j);
        value = (String)values.get(j);
        
        if ("Discard".equalsIgnoreCase(name)) {
          discard = true;
        } else if ("Secure".equalsIgnoreCase(name)) {
          secure = true;
        } else if ("HTTPOnly".equalsIgnoreCase(name)) {
          httpOnly = true;
        } else if ("Comment".equalsIgnoreCase(name)) {
          comment = value;
        } else if ("CommentURL".equalsIgnoreCase(name)) {
          commentURL = value;
        } else if ("Domain".equalsIgnoreCase(name)) {
          domain = value;
        } else if ("Path".equalsIgnoreCase(name)) {
          path = value;
        } else if ("Expires".equalsIgnoreCase(name)) {
          try {
            long maxAgeMillis = HttpHeaderDateFormat.get().parse(value).getTime() - System.currentTimeMillis();
            


            maxAge = maxAgeMillis / 1000L + (maxAgeMillis % 1000L != 0L ? 1 : 0);
          }
          catch (ParseException e) {}
        }
        else if ("Max-Age".equalsIgnoreCase(name)) {
          maxAge = Integer.parseInt(value);
        } else if ("Version".equalsIgnoreCase(name)) {
          version = Integer.parseInt(value);
        } else { if (!"Port".equalsIgnoreCase(name)) break;
          String[] portList = StringUtil.split(value, ',');
          for (String s1 : portList) {
            try {
              ports.add(Integer.valueOf(s1));
            }
            catch (NumberFormatException e) {}
          }
        }
        j++;
      }
      











































      c.setVersion(version);
      c.setMaxAge(maxAge);
      c.setPath(path);
      c.setDomain(domain);
      c.setSecure(secure);
      c.setHttpOnly(httpOnly);
      if (version > 0) {
        c.setComment(comment);
      }
      if (version > 1) {
        c.setCommentUrl(commentURL);
        c.setPorts(ports);
        c.setDiscard(discard);
      }
      
      cookies.add(c);
    }
    
    return cookies;
  }
  

  private static void extractKeyValuePairs(String header, List<String> names, List<String> values)
  {
    int headerLen = header.length();
    int i = 0;
    


    while (i != headerLen)
    {

      switch (header.charAt(i)) {
      case '\t': case '\n': case '\013': case '\f': 
      case '\r': case ' ': case ',': case ';': 
        i++;
        break;
      


      default: 
        for (;;)
        {
          if (i == headerLen) {
            return;
          }
          if (header.charAt(i) != '$') break;
          i++;
        }
        
        String value;
        
        String name;
        
        String value;
        
        if (i == headerLen) {
          String name = null;
          value = null;
        } else {
          int newNameStart = i;
          do { String value;
            switch (header.charAt(i))
            {
            case ';': 
              name = header.substring(newNameStart, i);
              value = null;
              break;
            
            case '=': 
              name = header.substring(newNameStart, i);
              i++;
              if (i == headerLen)
              {
                value = "";
              }
              else
              {
                int newValueStart = i;
                char c = header.charAt(i);
                if ((c == '"') || (c == '\''))
                {
                  StringBuilder newValueBuf = new StringBuilder(header.length() - i);
                  char q = c;
                  boolean hadBackslash = false;
                  i++;
                  for (;;) {
                    if (i == headerLen) {
                      String value = newValueBuf.toString();
                      break;
                    }
                    if (hadBackslash) {
                      hadBackslash = false;
                      c = header.charAt(i++);
                      switch (c) {
                      case '"': case '\'': 
                      case '\\': 
                        newValueBuf.setCharAt(newValueBuf.length() - 1, c);
                        break;
                      
                      default: 
                        newValueBuf.append(c); break;
                      }
                    } else {
                      c = header.charAt(i++);
                      if (c == q) {
                        String value = newValueBuf.toString();
                        break;
                      }
                      newValueBuf.append(c);
                      if (c == '\\') {
                        hadBackslash = true;
                      }
                    }
                  }
                }
                
                int semiPos = header.indexOf(';', i);
                if (semiPos > 0) {
                  String value = header.substring(newValueStart, semiPos);
                  i = semiPos;
                } else {
                  value = header.substring(newValueStart);
                  i = headerLen;
                }
              }
              break;
            default: 
              i++;
            }
            
          } while (i != headerLen);
          
          name = header.substring(newNameStart);
          value = null;
        }
        



        names.add(name);
        values.add(value);
      }
    }
  }
}
