package net.minecraft.util.io.netty.handler.codec.http;

import java.util.List;
import net.minecraft.util.io.netty.buffer.ByteBuf;
import net.minecraft.util.io.netty.buffer.CompositeByteBuf;
import net.minecraft.util.io.netty.buffer.Unpooled;
import net.minecraft.util.io.netty.channel.ChannelHandlerContext;
import net.minecraft.util.io.netty.handler.codec.DecoderResult;
import net.minecraft.util.io.netty.handler.codec.MessageToMessageDecoder;
import net.minecraft.util.io.netty.handler.codec.TooLongFrameException;
import net.minecraft.util.io.netty.util.CharsetUtil;
import net.minecraft.util.io.netty.util.ReferenceCountUtil;



































public class HttpObjectAggregator
  extends MessageToMessageDecoder<HttpObject>
{
  public static final int DEFAULT_MAX_COMPOSITEBUFFER_COMPONENTS = 1024;
  private static final ByteBuf CONTINUE = Unpooled.unreleasableBuffer(Unpooled.copiedBuffer("HTTP/1.1 100 Continue\r\n\r\n", CharsetUtil.US_ASCII));
  
  private final int maxContentLength;
  
  private FullHttpMessage currentMessage;
  
  private boolean tooLongFrameFound;
  private int maxCumulationBufferComponents = 1024;
  



  private ChannelHandlerContext ctx;
  



  public HttpObjectAggregator(int maxContentLength)
  {
    if (maxContentLength <= 0) {
      throw new IllegalArgumentException("maxContentLength must be a positive integer: " + maxContentLength);
    }
    

    this.maxContentLength = maxContentLength;
  }
  





  public final int getMaxCumulationBufferComponents()
  {
    return maxCumulationBufferComponents;
  }
  






  public final void setMaxCumulationBufferComponents(int maxCumulationBufferComponents)
  {
    if (maxCumulationBufferComponents < 2) {
      throw new IllegalArgumentException("maxCumulationBufferComponents: " + maxCumulationBufferComponents + " (expected: >= 2)");
    }
    


    if (ctx == null) {
      this.maxCumulationBufferComponents = maxCumulationBufferComponents;
    } else {
      throw new IllegalStateException("decoder properties cannot be changed once the decoder is added to a pipeline.");
    }
  }
  
  protected void decode(ChannelHandlerContext ctx, HttpObject msg, List<Object> out)
    throws Exception
  {
    FullHttpMessage currentMessage = this.currentMessage;
    
    if ((msg instanceof HttpMessage)) {
      tooLongFrameFound = false;
      assert (currentMessage == null);
      
      HttpMessage m = (HttpMessage)msg;
      





      if (HttpHeaders.is100ContinueExpected(m)) {
        ctx.writeAndFlush(CONTINUE.duplicate());
      }
      
      if (!m.getDecoderResult().isSuccess()) {
        HttpHeaders.removeTransferEncodingChunked(m);
        this.currentMessage = null;
        out.add(ReferenceCountUtil.retain(m));
        return;
      }
      if ((msg instanceof HttpRequest)) {
        HttpRequest header = (HttpRequest)msg;
        this.currentMessage = (currentMessage = new DefaultFullHttpRequest(header.getProtocolVersion(), header.getMethod(), header.getUri(), Unpooled.compositeBuffer(maxCumulationBufferComponents)));
      }
      else if ((msg instanceof HttpResponse)) {
        HttpResponse header = (HttpResponse)msg;
        this.currentMessage = (currentMessage = new DefaultFullHttpResponse(header.getProtocolVersion(), header.getStatus(), Unpooled.compositeBuffer(maxCumulationBufferComponents)));
      }
      else
      {
        throw new Error();
      }
      
      currentMessage.headers().set(m.headers());
      

      HttpHeaders.removeTransferEncodingChunked(currentMessage);
    } else if ((msg instanceof HttpContent)) {
      if (tooLongFrameFound) {
        if ((msg instanceof LastHttpContent)) {
          this.currentMessage = null;
        }
        
        return;
      }
      assert (currentMessage != null);
      

      HttpContent chunk = (HttpContent)msg;
      CompositeByteBuf content = (CompositeByteBuf)currentMessage.content();
      
      if (content.readableBytes() > maxContentLength - chunk.content().readableBytes()) {
        tooLongFrameFound = true;
        

        currentMessage.release();
        this.currentMessage = null;
        
        throw new TooLongFrameException("HTTP content length exceeded " + maxContentLength + " bytes.");
      }
      



      if (chunk.content().isReadable()) {
        chunk.retain();
        content.addComponent(chunk.content());
        content.writerIndex(content.writerIndex() + chunk.content().readableBytes());
      }
      boolean last;
      boolean last;
      if (!chunk.getDecoderResult().isSuccess()) {
        currentMessage.setDecoderResult(DecoderResult.failure(chunk.getDecoderResult().cause()));
        
        last = true;
      } else {
        last = chunk instanceof LastHttpContent;
      }
      
      if (last) {
        this.currentMessage = null;
        

        if ((chunk instanceof LastHttpContent)) {
          LastHttpContent trailer = (LastHttpContent)chunk;
          currentMessage.headers().add(trailer.trailingHeaders());
        }
        

        currentMessage.headers().set("Content-Length", String.valueOf(content.readableBytes()));
        



        out.add(currentMessage);
      }
    } else {
      throw new Error();
    }
  }
  
  public void channelInactive(ChannelHandlerContext ctx) throws Exception
  {
    super.channelInactive(ctx);
    

    if (currentMessage != null) {
      currentMessage.release();
      currentMessage = null;
    }
  }
  
  public void handlerAdded(ChannelHandlerContext ctx) throws Exception
  {
    this.ctx = ctx;
  }
  
  public void handlerRemoved(ChannelHandlerContext ctx) throws Exception
  {
    super.handlerRemoved(ctx);
    

    if (currentMessage != null) {
      currentMessage.release();
      currentMessage = null;
    }
  }
}
