package net.minecraft.util.io.netty.handler.codec.http;

import java.util.List;
import net.minecraft.util.io.netty.buffer.ByteBuf;
import net.minecraft.util.io.netty.buffer.ByteBufUtil;
import net.minecraft.util.io.netty.buffer.Unpooled;
import net.minecraft.util.io.netty.channel.ChannelHandlerContext;
import net.minecraft.util.io.netty.handler.codec.DecoderResult;
import net.minecraft.util.io.netty.handler.codec.ReplayingDecoder;
import net.minecraft.util.io.netty.handler.codec.TooLongFrameException;

























































































public abstract class HttpObjectDecoder
  extends ReplayingDecoder<State>
{
  private static final ThreadLocal<StringBuilder> BUILDERS = new ThreadLocal()
  {
    protected StringBuilder initialValue() {
      return new StringBuilder(512);
    }
    
    public StringBuilder get()
    {
      StringBuilder builder = (StringBuilder)super.get();
      builder.setLength(0);
      return builder;
    }
  };
  
  private final int maxInitialLineLength;
  
  private final int maxHeaderSize;
  
  private final int maxChunkSize;
  
  private final boolean chunkedSupported;
  
  private ByteBuf content;
  private HttpMessage message;
  private long chunkSize;
  private int headerSize;
  private int contentRead;
  
  static enum State
  {
    SKIP_CONTROL_CHARS, 
    READ_INITIAL, 
    READ_HEADER, 
    READ_VARIABLE_LENGTH_CONTENT, 
    READ_VARIABLE_LENGTH_CONTENT_AS_CHUNKS, 
    READ_FIXED_LENGTH_CONTENT, 
    READ_FIXED_LENGTH_CONTENT_AS_CHUNKS, 
    READ_CHUNK_SIZE, 
    READ_CHUNKED_CONTENT, 
    READ_CHUNKED_CONTENT_AS_CHUNKS, 
    READ_CHUNK_DELIMITER, 
    READ_CHUNK_FOOTER, 
    BAD_MESSAGE;
    

    private State() {}
  }
  

  protected HttpObjectDecoder()
  {
    this(4096, 8192, 8192, true);
  }
  




  protected HttpObjectDecoder(int maxInitialLineLength, int maxHeaderSize, int maxChunkSize, boolean chunkedSupported)
  {
    super(State.SKIP_CONTROL_CHARS);
    
    if (maxInitialLineLength <= 0) {
      throw new IllegalArgumentException("maxInitialLineLength must be a positive integer: " + maxInitialLineLength);
    }
    

    if (maxHeaderSize <= 0) {
      throw new IllegalArgumentException("maxHeaderSize must be a positive integer: " + maxHeaderSize);
    }
    

    if (maxChunkSize < 0) {
      throw new IllegalArgumentException("maxChunkSize must be a positive integer: " + maxChunkSize);
    }
    

    this.maxInitialLineLength = maxInitialLineLength;
    this.maxHeaderSize = maxHeaderSize;
    this.maxChunkSize = maxChunkSize;
    this.chunkedSupported = chunkedSupported;
  }
  
  protected void decode(ChannelHandlerContext ctx, ByteBuf buffer, List<Object> out) throws Exception
  {
    switch (2.$SwitchMap$io$netty$handler$codec$http$HttpObjectDecoder$State[((State)state()).ordinal()]) {
    case 3: 
      try {
        skipControlCharacters(buffer);
        checkpoint(State.READ_INITIAL);
      } finally {
        checkpoint();
      }
    case 4: 
      try {
        String[] initialLine = splitInitialLine(readLine(buffer, maxInitialLineLength));
        if (initialLine.length < 3)
        {
          checkpoint(State.SKIP_CONTROL_CHARS);
          return;
        }
        
        message = createMessage(initialLine);
        checkpoint(State.READ_HEADER);
      }
      catch (Exception e) {
        out.add(invalidMessage(e));
        return;
      }
    case 5:  try {
        State nextState = readHeaders(buffer);
        checkpoint(nextState);
        if (nextState == State.READ_CHUNK_SIZE) {
          if (!chunkedSupported) {
            throw new IllegalArgumentException("Chunked messages not supported");
          }
          
          out.add(message);
          return;
        }
        if (nextState == State.SKIP_CONTROL_CHARS)
        {
          reset(out);
          return;
        }
        long contentLength = HttpHeaders.getContentLength(message, -1L);
        if ((contentLength == 0L) || ((contentLength == -1L) && (isDecodingRequest()))) {
          this.content = Unpooled.EMPTY_BUFFER;
          reset(out);
          return;
        }
        
        switch (2.$SwitchMap$io$netty$handler$codec$http$HttpObjectDecoder$State[nextState.ordinal()]) {
        case 1: 
          if ((contentLength > maxChunkSize) || (HttpHeaders.is100ContinueExpected(message)))
          {
            checkpoint(State.READ_FIXED_LENGTH_CONTENT_AS_CHUNKS);
            

            this.chunkSize = HttpHeaders.getContentLength(message, -1L);
            out.add(message); return;
          }
          
          break;
        case 2: 
          if ((buffer.readableBytes() > maxChunkSize) || (HttpHeaders.is100ContinueExpected(message)))
          {
            checkpoint(State.READ_VARIABLE_LENGTH_CONTENT_AS_CHUNKS);
            out.add(message); return;
          }
          
          break;
        default: 
          throw new IllegalStateException("Unexpected state: " + nextState);
        }
        
        return;
      } catch (Exception e) {
        out.add(invalidMessage(e));
        return;
      }
    case 2: 
      int toRead = actualReadableBytes();
      if (toRead > maxChunkSize) {
        toRead = maxChunkSize;
      }
      out.add(message);
      out.add(new DefaultHttpContent(ByteBufUtil.readBytes(ctx.alloc(), buffer, toRead)));
      return;
    

    case 6: 
      int toRead = actualReadableBytes();
      if (toRead > maxChunkSize) {
        toRead = maxChunkSize;
      }
      ByteBuf content = ByteBufUtil.readBytes(ctx.alloc(), buffer, toRead);
      if (!buffer.isReadable()) {
        reset();
        out.add(new DefaultLastHttpContent(content));
        return;
      }
      out.add(new DefaultHttpContent(content));
      return;
    
    case 1: 
      readFixedLengthContent(ctx, buffer, out);
      return;
    
    case 7: 
      long chunkSize = this.chunkSize;
      int readLimit = actualReadableBytes();
      






      if (readLimit == 0) {
        return;
      }
      
      int toRead = readLimit;
      if (toRead > maxChunkSize) {
        toRead = maxChunkSize;
      }
      if (toRead > chunkSize) {
        toRead = (int)chunkSize;
      }
      ByteBuf content = ByteBufUtil.readBytes(ctx.alloc(), buffer, toRead);
      if (chunkSize > toRead) {
        chunkSize -= toRead;
      } else {
        chunkSize = 0L;
      }
      this.chunkSize = chunkSize;
      
      if (chunkSize == 0L)
      {
        reset();
        out.add(new DefaultLastHttpContent(content));
        return;
      }
      out.add(new DefaultHttpContent(content));
      return;
    


    case 8: 
      try
      {
        StringBuilder line = readLine(buffer, maxInitialLineLength);
        int chunkSize = getChunkSize(line.toString());
        this.chunkSize = chunkSize;
        if (chunkSize == 0) {
          checkpoint(State.READ_CHUNK_FOOTER);
          return; }
        if (chunkSize > maxChunkSize)
        {
          checkpoint(State.READ_CHUNKED_CONTENT_AS_CHUNKS);
        } else {
          checkpoint(State.READ_CHUNKED_CONTENT);
        }
      } catch (Exception e) {
        out.add(invalidChunk(e));
        return;
      }
    case 9: 
      assert (this.chunkSize <= 2147483647L);
      Object chunk = new DefaultHttpContent(ByteBufUtil.readBytes(ctx.alloc(), buffer, (int)this.chunkSize));
      checkpoint(State.READ_CHUNK_DELIMITER);
      out.add(chunk);
      return;
    
    case 10: 
      assert (this.chunkSize <= 2147483647L);
      int chunkSize = (int)this.chunkSize;
      int readLimit = actualReadableBytes();
      






      if (readLimit == 0) {
        return;
      }
      
      int toRead = chunkSize;
      if (toRead > maxChunkSize) {
        toRead = maxChunkSize;
      }
      if (toRead > readLimit) {
        toRead = readLimit;
      }
      HttpContent chunk = new DefaultHttpContent(ByteBufUtil.readBytes(ctx.alloc(), buffer, toRead));
      if (chunkSize > toRead) {
        chunkSize -= toRead;
      } else {
        chunkSize = 0;
      }
      this.chunkSize = chunkSize;
      
      if (chunkSize == 0)
      {
        checkpoint(State.READ_CHUNK_DELIMITER);
      }
      
      out.add(chunk); return;
    
    case 11: 
      for (;;)
      {
        byte next = buffer.readByte();
        if (next == 13) {
          if (buffer.readByte() == 10) {
            checkpoint(State.READ_CHUNK_SIZE);
          }
        } else {
          if (next == 10) {
            checkpoint(State.READ_CHUNK_SIZE);
            return;
          }
          checkpoint();
        }
      }
    case 12: 
      try {
        LastHttpContent trailer = readTrailingHeaders(buffer);
        if (maxChunkSize == 0)
        {
          reset(out);
          return;
        }
        reset();
        
        out.add(trailer);
        return;
      }
      catch (Exception e) {
        out.add(invalidChunk(e));
        return;
      }
    
    case 13: 
      buffer.skipBytes(actualReadableBytes());
      return;
    }
    
    throw new Error("Shouldn't reach here.");
  }
  

  protected void decodeLast(ChannelHandlerContext ctx, ByteBuf in, List<Object> out)
    throws Exception
  {
    decode(ctx, in, out);
    

    if (this.message != null)
    {
      HttpMessage message = this.message;
      int actualContentLength;
      int actualContentLength; if (content != null) {
        actualContentLength = content.readableBytes();
      } else {
        actualContentLength = 0;
      }
      
      boolean prematureClosure;
      boolean prematureClosure;
      if (isDecodingRequest())
      {
        prematureClosure = true;

      }
      else
      {
        long expectedContentLength = HttpHeaders.getContentLength(message, -1L);
        prematureClosure = (expectedContentLength >= 0L) && (actualContentLength != expectedContentLength);
      }
      
      if (!prematureClosure) {
        if (actualContentLength == 0) {
          out.add(LastHttpContent.EMPTY_LAST_CONTENT);
        } else {
          out.add(new DefaultLastHttpContent(content));
        }
      }
    }
  }
  
  protected boolean isContentAlwaysEmpty(HttpMessage msg) {
    if ((msg instanceof HttpResponse)) {
      HttpResponse res = (HttpResponse)msg;
      int code = res.getStatus().code();
      





      if ((code >= 100) && (code < 200)) {
        if ((code == 101) && (!res.headers().contains("Sec-WebSocket-Accept")))
        {
          return false;
        }
        return true;
      }
      
      switch (code) {
      case 204: case 205: case 304: 
        return true;
      }
    }
    return false;
  }
  
  private void reset() {
    reset(null);
  }
  
  private void reset(List<Object> out) {
    if (out != null) {
      HttpMessage message = this.message;
      ByteBuf content = this.content;
      LastHttpContent httpContent;
      LastHttpContent httpContent;
      if ((content == null) || (!content.isReadable())) {
        httpContent = LastHttpContent.EMPTY_LAST_CONTENT;
      } else {
        httpContent = new DefaultLastHttpContent(content);
      }
      
      out.add(message);
      out.add(httpContent);
    }
    
    this.content = null;
    this.message = null;
    
    checkpoint(State.SKIP_CONTROL_CHARS);
  }
  
  private HttpMessage invalidMessage(Exception cause) {
    checkpoint(State.BAD_MESSAGE);
    if (message != null) {
      message.setDecoderResult(DecoderResult.failure(cause));
    } else {
      message = createInvalidMessage();
      message.setDecoderResult(DecoderResult.failure(cause));
    }
    return message;
  }
  
  private HttpContent invalidChunk(Exception cause) {
    checkpoint(State.BAD_MESSAGE);
    HttpContent chunk = new DefaultHttpContent(Unpooled.EMPTY_BUFFER);
    chunk.setDecoderResult(DecoderResult.failure(cause));
    return chunk;
  }
  
  private static void skipControlCharacters(ByteBuf buffer) {
    for (;;) {
      char c = (char)buffer.readUnsignedByte();
      if ((!Character.isISOControl(c)) && (!Character.isWhitespace(c)))
      {
        buffer.readerIndex(buffer.readerIndex() - 1);
        break;
      }
    }
  }
  
  private void readFixedLengthContent(ChannelHandlerContext ctx, ByteBuf buffer, List<Object> out)
  {
    long length = HttpHeaders.getContentLength(message, -1L);
    assert (length <= 2147483647L);
    int toRead = (int)length - contentRead;
    if (toRead > actualReadableBytes()) {
      toRead = actualReadableBytes();
    }
    contentRead += toRead;
    if (length < contentRead) {
      out.add(message);
      out.add(new DefaultHttpContent(ByteBufUtil.readBytes(ctx.alloc(), buffer, toRead)));
      return;
    }
    if (content == null) {
      content = ByteBufUtil.readBytes(ctx.alloc(), buffer, (int)length);
    } else {
      content.writeBytes(buffer, (int)length);
    }
    reset(out);
  }
  
  private State readHeaders(ByteBuf buffer) {
    headerSize = 0;
    HttpMessage message = this.message;
    HttpHeaders headers = message.headers();
    
    StringBuilder line = readHeader(buffer);
    String name = null;
    String value = null;
    if (line.length() > 0) {
      headers.clear();
      do {
        char firstChar = line.charAt(0);
        if ((name != null) && ((firstChar == ' ') || (firstChar == '\t'))) {
          value = value + ' ' + line.toString().trim();
        } else {
          if (name != null) {
            headers.add(name, value);
          }
          String[] header = splitHeader(line);
          name = header[0];
          value = header[1];
        }
        
        line = readHeader(buffer);
      } while (line.length() > 0);
      

      if (name != null) {
        headers.add(name, value);
      }
    }
    
    State nextState;
    State nextState;
    if (isContentAlwaysEmpty(message)) {
      HttpHeaders.removeTransferEncodingChunked(message);
      nextState = State.SKIP_CONTROL_CHARS; } else { State nextState;
      if (HttpHeaders.isTransferEncodingChunked(message)) {
        nextState = State.READ_CHUNK_SIZE; } else { State nextState;
        if (HttpHeaders.getContentLength(message, -1L) >= 0L) {
          nextState = State.READ_FIXED_LENGTH_CONTENT;
        } else
          nextState = State.READ_VARIABLE_LENGTH_CONTENT;
      } }
    return nextState;
  }
  
  private LastHttpContent readTrailingHeaders(ByteBuf buffer) {
    headerSize = 0;
    StringBuilder line = readHeader(buffer);
    String lastHeader = null;
    if (line.length() > 0) {
      LastHttpContent trailer = new DefaultLastHttpContent(Unpooled.EMPTY_BUFFER);
      do {
        char firstChar = line.charAt(0);
        if ((lastHeader != null) && ((firstChar == ' ') || (firstChar == '\t'))) {
          List<String> current = trailer.trailingHeaders().getAll(lastHeader);
          if (!current.isEmpty()) {
            int lastPos = current.size() - 1;
            String newString = (String)current.get(lastPos) + line.toString().trim();
            current.set(lastPos, newString);
          }
        }
        else
        {
          String[] header = splitHeader(line);
          String name = header[0];
          if ((!name.equalsIgnoreCase("Content-Length")) && (!name.equalsIgnoreCase("Transfer-Encoding")) && (!name.equalsIgnoreCase("Trailer")))
          {

            trailer.trailingHeaders().add(name, header[1]);
          }
          lastHeader = name;
        }
        
        line = readHeader(buffer);
      } while (line.length() > 0);
      
      return trailer;
    }
    
    return LastHttpContent.EMPTY_LAST_CONTENT;
  }
  
  private StringBuilder readHeader(ByteBuf buffer) {
    StringBuilder sb = (StringBuilder)BUILDERS.get();
    int headerSize = this.headerSize;
    
    for (;;)
    {
      char nextByte = (char)buffer.readByte();
      headerSize++;
      
      switch (nextByte) {
      case '\r': 
        nextByte = (char)buffer.readByte();
        headerSize++;
        if (nextByte != '\n') break;
        break;
      case '\n': 
        break;
      }
      
      


      if (headerSize >= maxHeaderSize)
      {



        throw new TooLongFrameException("HTTP header is larger than " + maxHeaderSize + " bytes.");
      }
      


      sb.append(nextByte);
    }
    
    this.headerSize = headerSize;
    return sb; }
  
  protected abstract boolean isDecodingRequest();
  
  protected abstract HttpMessage createMessage(String[] paramArrayOfString) throws Exception;
  
  protected abstract HttpMessage createInvalidMessage();
  
  private static int getChunkSize(String hex) { hex = hex.trim();
    for (int i = 0; i < hex.length(); i++) {
      char c = hex.charAt(i);
      if ((c == ';') || (Character.isWhitespace(c)) || (Character.isISOControl(c))) {
        hex = hex.substring(0, i);
        break;
      }
    }
    
    return Integer.parseInt(hex, 16);
  }
  
  private static StringBuilder readLine(ByteBuf buffer, int maxLineLength) {
    StringBuilder sb = (StringBuilder)BUILDERS.get();
    int lineLength = 0;
    for (;;) {
      byte nextByte = buffer.readByte();
      if (nextByte == 13) {
        nextByte = buffer.readByte();
        if (nextByte == 10)
          return sb;
      } else {
        if (nextByte == 10) {
          return sb;
        }
        if (lineLength >= maxLineLength)
        {



          throw new TooLongFrameException("An HTTP line is larger than " + maxLineLength + " bytes.");
        }
        

        lineLength++;
        sb.append((char)nextByte);
      }
    }
  }
  






  private static String[] splitInitialLine(StringBuilder sb)
  {
    int aStart = findNonWhitespace(sb, 0);
    int aEnd = findWhitespace(sb, aStart);
    
    int bStart = findNonWhitespace(sb, aEnd);
    int bEnd = findWhitespace(sb, bStart);
    
    int cStart = findNonWhitespace(sb, bEnd);
    int cEnd = findEndOfString(sb);
    
    return new String[] { sb.substring(aStart, aEnd), sb.substring(bStart, bEnd), cStart < cEnd ? sb.substring(cStart, cEnd) : "" };
  }
  


  private static String[] splitHeader(StringBuilder sb)
  {
    int length = sb.length();
    





    int nameStart = findNonWhitespace(sb, 0);
    for (int nameEnd = nameStart; nameEnd < length; nameEnd++) {
      char ch = sb.charAt(nameEnd);
      if ((ch == ':') || (Character.isWhitespace(ch))) {
        break;
      }
    }
    
    for (int colonEnd = nameEnd; colonEnd < length; colonEnd++) {
      if (sb.charAt(colonEnd) == ':') {
        colonEnd++;
        break;
      }
    }
    
    int valueStart = findNonWhitespace(sb, colonEnd);
    if (valueStart == length) {
      return new String[] { sb.substring(nameStart, nameEnd), "" };
    }
    



    int valueEnd = findEndOfString(sb);
    return new String[] { sb.substring(nameStart, nameEnd), sb.substring(valueStart, valueEnd) };
  }
  



  private static int findNonWhitespace(CharSequence sb, int offset)
  {
    for (int result = offset; result < sb.length(); result++) {
      if (!Character.isWhitespace(sb.charAt(result))) {
        break;
      }
    }
    return result;
  }
  
  private static int findWhitespace(CharSequence sb, int offset)
  {
    for (int result = offset; result < sb.length(); result++) {
      if (Character.isWhitespace(sb.charAt(result))) {
        break;
      }
    }
    return result;
  }
  
  private static int findEndOfString(CharSequence sb)
  {
    for (int result = sb.length(); result > 0; result--) {
      if (!Character.isWhitespace(sb.charAt(result - 1))) {
        break;
      }
    }
    return result;
  }
}
