package net.minecraft.util.io.netty.handler.codec.http;

import java.util.List;
import java.util.Map.Entry;
import net.minecraft.util.io.netty.buffer.ByteBuf;
import net.minecraft.util.io.netty.buffer.ByteBufAllocator;
import net.minecraft.util.io.netty.buffer.Unpooled;
import net.minecraft.util.io.netty.channel.ChannelHandlerContext;
import net.minecraft.util.io.netty.channel.FileRegion;
import net.minecraft.util.io.netty.handler.codec.MessageToMessageEncoder;
import net.minecraft.util.io.netty.util.CharsetUtil;





























public abstract class HttpObjectEncoder<H extends HttpMessage>
  extends MessageToMessageEncoder<Object>
{
  private static final byte[] CRLF = { 13, 10 };
  private static final byte[] ZERO_CRLF = { 48, 13, 10 };
  private static final byte[] ZERO_CRLF_CRLF = { 48, 13, 10, 13, 10 };
  private static final byte[] HEADER_SEPARATOR = { 58, 32 };
  private static final ByteBuf CRLF_BUF = Unpooled.unreleasableBuffer(Unpooled.directBuffer(CRLF.length).writeBytes(CRLF));
  private static final ByteBuf ZERO_CRLF_CRLF_BUF = Unpooled.unreleasableBuffer(Unpooled.directBuffer(ZERO_CRLF_CRLF.length).writeBytes(ZERO_CRLF_CRLF));
  
  private static final int ST_INIT = 0;
  
  private static final int ST_CONTENT_NON_CHUNK = 1;
  
  private static final int ST_CONTENT_CHUNK = 2;
  private int state = 0;
  
  protected void encode(ChannelHandlerContext ctx, Object msg, List<Object> out)
    throws Exception
  {
    if ((msg instanceof HttpMessage)) {
      if (state != 0) {
        throw new IllegalStateException("unexpected message type: " + msg.getClass().getSimpleName());
      }
      

      H m = (HttpMessage)msg;
      
      ByteBuf buf = ctx.alloc().buffer();
      
      encodeInitialLine(buf, m);
      encodeHeaders(buf, m.headers());
      buf.writeBytes(CRLF);
      out.add(buf);
      state = (HttpHeaders.isTransferEncodingChunked(m) ? 2 : 1);
    }
    if (((msg instanceof HttpContent)) || ((msg instanceof ByteBuf)) || ((msg instanceof FileRegion))) {
      if (state == 0) {
        throw new IllegalStateException("unexpected message type: " + msg.getClass().getSimpleName());
      }
      
      int contentLength = contentLength(msg);
      if (state == 1) {
        if (contentLength > 0) {
          out.add(encodeAndRetain(msg));
        }
        else
        {
          out.add(Unpooled.EMPTY_BUFFER);
        }
        
        if ((msg instanceof LastHttpContent)) {
          state = 0;
        }
      } else if (state == 2) {
        encodeChunkedContent(ctx, msg, contentLength, out);
      } else {
        throw new Error();
      }
    }
  }
  
  private void encodeChunkedContent(ChannelHandlerContext ctx, Object msg, int contentLength, List<Object> out) {
    if (contentLength > 0) {
      byte[] length = Integer.toHexString(contentLength).getBytes(CharsetUtil.US_ASCII);
      ByteBuf buf = ctx.alloc().buffer(length.length + 2);
      buf.writeBytes(length);
      buf.writeBytes(CRLF);
      out.add(buf);
      out.add(encodeAndRetain(msg));
      out.add(CRLF_BUF.duplicate());
    }
    
    if ((msg instanceof LastHttpContent)) {
      HttpHeaders headers = ((LastHttpContent)msg).trailingHeaders();
      if (headers.isEmpty()) {
        out.add(ZERO_CRLF_CRLF_BUF.duplicate());
      } else {
        ByteBuf buf = ctx.alloc().buffer();
        buf.writeBytes(ZERO_CRLF);
        encodeHeaders(buf, headers);
        buf.writeBytes(CRLF);
        out.add(buf);
      }
      
      state = 0;
    }
    else if (contentLength == 0)
    {

      out.add(Unpooled.EMPTY_BUFFER);
    }
  }
  
  public boolean acceptOutboundMessage(Object msg)
    throws Exception
  {
    return ((msg instanceof HttpObject)) || ((msg instanceof ByteBuf)) || ((msg instanceof FileRegion));
  }
  
  private static Object encodeAndRetain(Object msg) {
    if ((msg instanceof ByteBuf)) {
      return ((ByteBuf)msg).retain();
    }
    if ((msg instanceof HttpContent)) {
      return ((HttpContent)msg).content().retain();
    }
    if ((msg instanceof FileRegion)) {
      return ((FileRegion)msg).retain();
    }
    throw new IllegalStateException("unexpected message type: " + msg.getClass().getSimpleName());
  }
  
  private static int contentLength(Object msg) {
    if ((msg instanceof HttpContent)) {
      return ((HttpContent)msg).content().readableBytes();
    }
    if ((msg instanceof ByteBuf)) {
      return ((ByteBuf)msg).readableBytes();
    }
    if ((msg instanceof FileRegion)) {
      return (int)((FileRegion)msg).count();
    }
    throw new IllegalStateException("unexpected message type: " + msg.getClass().getSimpleName());
  }
  
  private static void encodeHeaders(ByteBuf buf, HttpHeaders headers) {
    for (Map.Entry<String, String> h : headers) {
      encodeHeader(buf, (String)h.getKey(), (String)h.getValue());
    }
  }
  
  private static void encodeHeader(ByteBuf buf, String header, String value) {
    encodeAscii(header, buf);
    buf.writeBytes(HEADER_SEPARATOR);
    encodeAscii(value, buf);
    buf.writeBytes(CRLF);
  }
  
  protected static void encodeAscii(String s, ByteBuf buf) {
    for (int i = 0; i < s.length(); i++) {
      buf.writeByte(s.charAt(i));
    }
  }
  
  protected abstract void encodeInitialLine(ByteBuf paramByteBuf, H paramH)
    throws Exception;
}
