package net.minecraft.util.io.netty.handler.codec.http.websocketx;

import java.net.URI;
import net.minecraft.util.io.netty.channel.Channel;
import net.minecraft.util.io.netty.channel.ChannelFuture;
import net.minecraft.util.io.netty.channel.ChannelFutureListener;
import net.minecraft.util.io.netty.channel.ChannelHandlerContext;
import net.minecraft.util.io.netty.channel.ChannelPipeline;
import net.minecraft.util.io.netty.channel.ChannelPromise;
import net.minecraft.util.io.netty.handler.codec.http.FullHttpRequest;
import net.minecraft.util.io.netty.handler.codec.http.FullHttpResponse;
import net.minecraft.util.io.netty.handler.codec.http.HttpClientCodec;
import net.minecraft.util.io.netty.handler.codec.http.HttpContentDecompressor;
import net.minecraft.util.io.netty.handler.codec.http.HttpHeaders;
import net.minecraft.util.io.netty.handler.codec.http.HttpRequestEncoder;
import net.minecraft.util.io.netty.handler.codec.http.HttpResponseDecoder;









































public abstract class WebSocketClientHandshaker
{
  private final URI uri;
  private final WebSocketVersion version;
  private boolean handshakeComplete;
  private final String expectedSubprotocol;
  private String actualSubprotocol;
  protected final HttpHeaders customHeaders;
  private final int maxFramePayloadLength;
  
  protected WebSocketClientHandshaker(URI uri, WebSocketVersion version, String subprotocol, HttpHeaders customHeaders, int maxFramePayloadLength)
  {
    this.uri = uri;
    this.version = version;
    expectedSubprotocol = subprotocol;
    this.customHeaders = customHeaders;
    this.maxFramePayloadLength = maxFramePayloadLength;
  }
  


  public URI uri()
  {
    return uri;
  }
  


  public WebSocketVersion version()
  {
    return version;
  }
  


  public int maxFramePayloadLength()
  {
    return maxFramePayloadLength;
  }
  


  public boolean isHandshakeComplete()
  {
    return handshakeComplete;
  }
  
  private void setHandshakeComplete() {
    handshakeComplete = true;
  }
  


  public String expectedSubprotocol()
  {
    return expectedSubprotocol;
  }
  



  public String actualSubprotocol()
  {
    return actualSubprotocol;
  }
  
  private void setActualSubprotocol(String actualSubprotocol) {
    this.actualSubprotocol = actualSubprotocol;
  }
  





  public ChannelFuture handshake(Channel channel)
  {
    if (channel == null) {
      throw new NullPointerException("channel");
    }
    return handshake(channel, channel.newPromise());
  }
  







  public final ChannelFuture handshake(Channel channel, final ChannelPromise promise)
  {
    FullHttpRequest request = newHandshakeRequest();
    
    HttpResponseDecoder decoder = (HttpResponseDecoder)channel.pipeline().get(HttpResponseDecoder.class);
    if (decoder == null) {
      HttpClientCodec codec = (HttpClientCodec)channel.pipeline().get(HttpClientCodec.class);
      if (codec == null) {
        promise.setFailure(new IllegalStateException("ChannelPipeline does not contain a HttpResponseDecoder or HttpClientCodec"));
        
        return promise;
      }
      codec.setSingleDecode(true);
    } else {
      decoder.setSingleDecode(true);
    }
    channel.writeAndFlush(request).addListener(new ChannelFutureListener()
    {
      public void operationComplete(ChannelFuture future) {
        if (future.isSuccess()) {
          ChannelPipeline p = future.channel().pipeline();
          ChannelHandlerContext ctx = p.context(HttpRequestEncoder.class);
          if (ctx == null) {
            ctx = p.context(HttpClientCodec.class);
          }
          if (ctx == null) {
            promise.setFailure(new IllegalStateException("ChannelPipeline does not contain a HttpRequestEncoder or HttpClientCodec"));
            
            return;
          }
          p.addAfter(ctx.name(), "ws-encoder", newWebSocketEncoder());
          
          promise.setSuccess();
        } else {
          promise.setFailure(future.cause());
        }
      }
    });
    return promise;
  }
  





  protected abstract FullHttpRequest newHandshakeRequest();
  





  public final void finishHandshake(Channel channel, FullHttpResponse response)
  {
    verify(response);
    setActualSubprotocol(response.headers().get("Sec-WebSocket-Protocol"));
    setHandshakeComplete();
    
    ChannelPipeline p = channel.pipeline();
    
    HttpContentDecompressor decompressor = (HttpContentDecompressor)p.get(HttpContentDecompressor.class);
    if (decompressor != null) {
      p.remove(decompressor);
    }
    
    ChannelHandlerContext ctx = p.context(HttpResponseDecoder.class);
    if (ctx == null) {
      ctx = p.context(HttpClientCodec.class);
      if (ctx == null) {
        throw new IllegalStateException("ChannelPipeline does not contain a HttpRequestEncoder or HttpClientCodec");
      }
      
      p.replace(ctx.name(), "ws-decoder", newWebsocketDecoder());
    } else {
      if (p.get(HttpRequestEncoder.class) != null) {
        p.remove(HttpRequestEncoder.class);
      }
      p.replace(ctx.name(), "ws-decoder", newWebsocketDecoder());
    }
  }
  





  protected abstract void verify(FullHttpResponse paramFullHttpResponse);
  




  protected abstract WebSocketFrameDecoder newWebsocketDecoder();
  




  protected abstract WebSocketFrameEncoder newWebSocketEncoder();
  




  public ChannelFuture close(Channel channel, CloseWebSocketFrame frame)
  {
    if (channel == null) {
      throw new NullPointerException("channel");
    }
    return close(channel, frame, channel.newPromise());
  }
  









  public ChannelFuture close(Channel channel, CloseWebSocketFrame frame, ChannelPromise promise)
  {
    if (channel == null) {
      throw new NullPointerException("channel");
    }
    return channel.writeAndFlush(frame, promise);
  }
}
