package net.minecraft.util.io.netty.handler.codec.http.websocketx;

import net.minecraft.util.io.netty.channel.ChannelFuture;
import net.minecraft.util.io.netty.channel.ChannelFutureListener;
import net.minecraft.util.io.netty.channel.ChannelHandlerContext;
import net.minecraft.util.io.netty.channel.ChannelInboundHandlerAdapter;
import net.minecraft.util.io.netty.channel.ChannelPipeline;
import net.minecraft.util.io.netty.handler.codec.http.FullHttpResponse;












class WebSocketClientProtocolHandshakeHandler
  extends ChannelInboundHandlerAdapter
{
  private final WebSocketClientHandshaker handshaker;
  
  public WebSocketClientProtocolHandshakeHandler(WebSocketClientHandshaker handshaker)
  {
    this.handshaker = handshaker;
  }
  
  public void channelActive(final ChannelHandlerContext ctx) throws Exception
  {
    super.channelActive(ctx);
    handshaker.handshake(ctx.channel()).addListener(new ChannelFutureListener()
    {
      public void operationComplete(ChannelFuture future) throws Exception {
        if (!future.isSuccess()) {
          ctx.fireExceptionCaught(future.cause());
        } else {
          ctx.fireUserEventTriggered(WebSocketClientProtocolHandler.ClientHandshakeStateEvent.HANDSHAKE_ISSUED);
        }
      }
    });
  }
  
  public void channelRead(ChannelHandlerContext ctx, Object msg)
    throws Exception
  {
    if (!(msg instanceof FullHttpResponse)) {
      ctx.fireChannelRead(msg);
      return;
    }
    
    if (!handshaker.isHandshakeComplete()) {
      handshaker.finishHandshake(ctx.channel(), (FullHttpResponse)msg);
      ctx.fireUserEventTriggered(WebSocketClientProtocolHandler.ClientHandshakeStateEvent.HANDSHAKE_COMPLETE);
      
      ctx.pipeline().remove(this);
      return;
    }
    throw new IllegalStateException("WebSocketClientHandshaker should have been non finished yet");
  }
}
