package net.minecraft.util.io.netty.handler.codec.http.websocketx;

import java.util.List;
import net.minecraft.util.io.netty.buffer.ByteBuf;
import net.minecraft.util.io.netty.buffer.ByteBufAllocator;
import net.minecraft.util.io.netty.buffer.CompositeByteBuf;
import net.minecraft.util.io.netty.channel.ChannelHandlerContext;
import net.minecraft.util.io.netty.handler.codec.MessageToMessageDecoder;
import net.minecraft.util.io.netty.handler.codec.TooLongFrameException;

























public class WebSocketFrameAggregator
  extends MessageToMessageDecoder<WebSocketFrame>
{
  private final int maxFrameSize;
  private WebSocketFrame currentFrame;
  private boolean tooLongFrameFound;
  
  public WebSocketFrameAggregator(int maxFrameSize)
  {
    if (maxFrameSize < 1) {
      throw new IllegalArgumentException("maxFrameSize must be > 0");
    }
    this.maxFrameSize = maxFrameSize;
  }
  
  protected void decode(ChannelHandlerContext ctx, WebSocketFrame msg, List<Object> out) throws Exception
  {
    if (this.currentFrame == null) {
      tooLongFrameFound = false;
      if (msg.isFinalFragment()) {
        out.add(msg.retain());
        return;
      }
      ByteBuf buf = ctx.alloc().compositeBuffer().addComponent(msg.content().retain());
      buf.writerIndex(buf.writerIndex() + msg.content().readableBytes());
      
      if ((msg instanceof TextWebSocketFrame)) {
        this.currentFrame = new TextWebSocketFrame(true, msg.rsv(), buf);
      } else if ((msg instanceof BinaryWebSocketFrame)) {
        this.currentFrame = new BinaryWebSocketFrame(true, msg.rsv(), buf);
      } else {
        throw new IllegalStateException("WebSocket frame was not of type TextWebSocketFrame or BinaryWebSocketFrame");
      }
      
      return;
    }
    if ((msg instanceof ContinuationWebSocketFrame)) {
      if (tooLongFrameFound) {
        if (msg.isFinalFragment()) {
          this.currentFrame = null;
        }
        return;
      }
      CompositeByteBuf content = (CompositeByteBuf)this.currentFrame.content();
      if (content.readableBytes() > maxFrameSize - msg.content().readableBytes()) {
        tooLongFrameFound = true;
        throw new TooLongFrameException("WebSocketFrame length exceeded " + content + " bytes.");
      }
      

      content.addComponent(msg.content().retain());
      content.writerIndex(content.writerIndex() + msg.content().readableBytes());
      
      if (msg.isFinalFragment()) {
        WebSocketFrame currentFrame = this.currentFrame;
        this.currentFrame = null;
        out.add(currentFrame);
        return;
      }
      return;
    }
    


    out.add(msg.retain());
  }
  
  public void channelInactive(ChannelHandlerContext ctx) throws Exception
  {
    super.channelInactive(ctx);
    

    if (currentFrame != null) {
      currentFrame.release();
      currentFrame = null;
    }
  }
  
  public void handlerRemoved(ChannelHandlerContext ctx) throws Exception
  {
    super.handlerRemoved(ctx);
    

    if (currentFrame != null) {
      currentFrame.release();
      currentFrame = null;
    }
  }
}
