package net.minecraft.util.io.netty.handler.codec.http.websocketx;

import java.util.regex.Matcher;
import java.util.regex.Pattern;
import net.minecraft.util.io.netty.buffer.ByteBuf;
import net.minecraft.util.io.netty.buffer.Unpooled;
import net.minecraft.util.io.netty.channel.Channel;
import net.minecraft.util.io.netty.channel.ChannelFuture;
import net.minecraft.util.io.netty.channel.ChannelPromise;
import net.minecraft.util.io.netty.handler.codec.http.DefaultFullHttpResponse;
import net.minecraft.util.io.netty.handler.codec.http.FullHttpRequest;
import net.minecraft.util.io.netty.handler.codec.http.FullHttpResponse;
import net.minecraft.util.io.netty.handler.codec.http.HttpHeaders;
import net.minecraft.util.io.netty.handler.codec.http.HttpResponseStatus;
import net.minecraft.util.io.netty.handler.codec.http.HttpVersion;






























public class WebSocketServerHandshaker00
  extends WebSocketServerHandshaker
{
  private static final Pattern BEGINNING_DIGIT = Pattern.compile("[^0-9]");
  private static final Pattern BEGINNING_SPACE = Pattern.compile("[^ ]");
  











  public WebSocketServerHandshaker00(String webSocketURL, String subprotocols, int maxFramePayloadLength)
  {
    super(WebSocketVersion.V00, webSocketURL, subprotocols, maxFramePayloadLength);
  }
  










































  protected FullHttpResponse newHandshakeResponse(FullHttpRequest req, HttpHeaders headers)
  {
    if ((!"Upgrade".equalsIgnoreCase(req.headers().get("Connection"))) || (!"WebSocket".equalsIgnoreCase(req.headers().get("Upgrade"))))
    {
      throw new WebSocketHandshakeException("not a WebSocket handshake request: missing upgrade");
    }
    

    boolean isHixie76 = (req.headers().contains("Sec-WebSocket-Key1")) && (req.headers().contains("Sec-WebSocket-Key2"));
    

    FullHttpResponse res = new DefaultFullHttpResponse(HttpVersion.HTTP_1_1, new HttpResponseStatus(101, isHixie76 ? "WebSocket Protocol Handshake" : "Web Socket Protocol Handshake"));
    
    if (headers != null) {
      res.headers().add(headers);
    }
    
    res.headers().add("Upgrade", "WebSocket");
    res.headers().add("Connection", "Upgrade");
    

    if (isHixie76)
    {
      res.headers().add("Sec-WebSocket-Origin", req.headers().get("Origin"));
      res.headers().add("Sec-WebSocket-Location", uri());
      String subprotocols = req.headers().get("Sec-WebSocket-Protocol");
      if (subprotocols != null) {
        String selectedSubprotocol = selectSubprotocol(subprotocols);
        if (selectedSubprotocol == null) {
          throw new WebSocketHandshakeException("Requested subprotocol(s) not supported: " + subprotocols);
        }
        res.headers().add("Sec-WebSocket-Protocol", selectedSubprotocol);
      }
      


      String key1 = req.headers().get("Sec-WebSocket-Key1");
      String key2 = req.headers().get("Sec-WebSocket-Key2");
      int a = (int)(Long.parseLong(BEGINNING_DIGIT.matcher(key1).replaceAll("")) / BEGINNING_SPACE.matcher(key1).replaceAll("").length());
      
      int b = (int)(Long.parseLong(BEGINNING_DIGIT.matcher(key2).replaceAll("")) / BEGINNING_SPACE.matcher(key2).replaceAll("").length());
      
      long c = req.content().readLong();
      ByteBuf input = Unpooled.buffer(16);
      input.writeInt(a);
      input.writeInt(b);
      input.writeLong(c);
      res.content().writeBytes(WebSocketUtil.md5(input.array()));
    }
    else {
      res.headers().add("WebSocket-Origin", req.headers().get("Origin"));
      res.headers().add("WebSocket-Location", uri());
      String protocol = req.headers().get("WebSocket-Protocol");
      if (protocol != null) {
        res.headers().add("WebSocket-Protocol", selectSubprotocol(protocol));
      }
    }
    return res;
  }
  








  public ChannelFuture close(Channel channel, CloseWebSocketFrame frame, ChannelPromise promise)
  {
    return channel.writeAndFlush(frame, promise);
  }
  
  protected WebSocketFrameDecoder newWebsocketDecoder()
  {
    return new WebSocket00FrameDecoder(maxFramePayloadLength());
  }
  
  protected WebSocketFrameEncoder newWebSocketEncoder()
  {
    return new WebSocket00FrameEncoder();
  }
}
