package net.minecraft.util.io.netty.handler.codec.http.websocketx;

import java.util.List;
import net.minecraft.util.io.netty.buffer.Unpooled;
import net.minecraft.util.io.netty.channel.Channel;
import net.minecraft.util.io.netty.channel.ChannelFuture;
import net.minecraft.util.io.netty.channel.ChannelFutureListener;
import net.minecraft.util.io.netty.channel.ChannelHandler;
import net.minecraft.util.io.netty.channel.ChannelHandlerContext;
import net.minecraft.util.io.netty.channel.ChannelInboundHandlerAdapter;
import net.minecraft.util.io.netty.channel.ChannelPipeline;
import net.minecraft.util.io.netty.handler.codec.http.DefaultFullHttpResponse;
import net.minecraft.util.io.netty.handler.codec.http.FullHttpRequest;
import net.minecraft.util.io.netty.handler.codec.http.FullHttpResponse;
import net.minecraft.util.io.netty.handler.codec.http.HttpResponseStatus;
import net.minecraft.util.io.netty.handler.codec.http.HttpVersion;
import net.minecraft.util.io.netty.util.Attribute;
import net.minecraft.util.io.netty.util.AttributeKey;




































public class WebSocketServerProtocolHandler
  extends WebSocketProtocolHandler
{
  public static enum ServerHandshakeStateEvent
  {
    HANDSHAKE_COMPLETE;
    
    private ServerHandshakeStateEvent() {} }
  private static final AttributeKey<WebSocketServerHandshaker> HANDSHAKER_ATTR_KEY = new AttributeKey(WebSocketServerHandshaker.class.getName());
  
  private final String websocketPath;
  private final String subprotocols;
  private final boolean allowExtensions;
  
  public WebSocketServerProtocolHandler(String websocketPath)
  {
    this(websocketPath, null, false);
  }
  
  public WebSocketServerProtocolHandler(String websocketPath, String subprotocols) {
    this(websocketPath, subprotocols, false);
  }
  
  public WebSocketServerProtocolHandler(String websocketPath, String subprotocols, boolean allowExtensions) {
    this.websocketPath = websocketPath;
    this.subprotocols = subprotocols;
    this.allowExtensions = allowExtensions;
  }
  
  public void handlerAdded(ChannelHandlerContext ctx)
  {
    ChannelPipeline cp = ctx.pipeline();
    if (cp.get(WebSocketServerProtocolHandshakeHandler.class) == null)
    {
      ctx.pipeline().addBefore(ctx.name(), WebSocketServerProtocolHandshakeHandler.class.getName(), new WebSocketServerProtocolHandshakeHandler(websocketPath, subprotocols, allowExtensions));
    }
  }
  
  protected void decode(ChannelHandlerContext ctx, WebSocketFrame frame, List<Object> out)
    throws Exception
  {
    if ((frame instanceof CloseWebSocketFrame)) {
      WebSocketServerHandshaker handshaker = getHandshaker(ctx);
      frame.retain();
      handshaker.close(ctx.channel(), (CloseWebSocketFrame)frame);
      return;
    }
    super.decode(ctx, frame, out);
  }
  
  public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception
  {
    if ((cause instanceof WebSocketHandshakeException)) {
      FullHttpResponse response = new DefaultFullHttpResponse(HttpVersion.HTTP_1_1, HttpResponseStatus.BAD_REQUEST, Unpooled.wrappedBuffer(cause.getMessage().getBytes()));
      
      ctx.channel().writeAndFlush(response).addListener(ChannelFutureListener.CLOSE);
    } else {
      ctx.close();
    }
  }
  
  static WebSocketServerHandshaker getHandshaker(ChannelHandlerContext ctx) {
    return (WebSocketServerHandshaker)ctx.attr(HANDSHAKER_ATTR_KEY).get();
  }
  
  static void setHandshaker(ChannelHandlerContext ctx, WebSocketServerHandshaker handshaker) {
    ctx.attr(HANDSHAKER_ATTR_KEY).set(handshaker);
  }
  
  static ChannelHandler forbiddenHttpRequestResponder() {
    new ChannelInboundHandlerAdapter()
    {
      public void channelRead(ChannelHandlerContext ctx, Object msg) throws Exception {
        if ((msg instanceof FullHttpRequest)) {
          FullHttpResponse response = new DefaultFullHttpResponse(HttpVersion.HTTP_1_1, HttpResponseStatus.FORBIDDEN);
          
          ctx.channel().writeAndFlush(response);
        } else {
          ctx.fireChannelRead(msg);
        }
      }
    };
  }
}
