package net.minecraft.util.io.netty.handler.codec.http.websocketx;

import net.minecraft.util.io.netty.channel.Channel;
import net.minecraft.util.io.netty.channel.ChannelFuture;
import net.minecraft.util.io.netty.channel.ChannelFutureListener;
import net.minecraft.util.io.netty.channel.ChannelHandlerContext;
import net.minecraft.util.io.netty.channel.ChannelInboundHandlerAdapter;
import net.minecraft.util.io.netty.channel.ChannelPipeline;
import net.minecraft.util.io.netty.handler.codec.http.DefaultFullHttpResponse;
import net.minecraft.util.io.netty.handler.codec.http.FullHttpRequest;
import net.minecraft.util.io.netty.handler.codec.http.HttpHeaders;
import net.minecraft.util.io.netty.handler.codec.http.HttpMethod;
import net.minecraft.util.io.netty.handler.codec.http.HttpRequest;
import net.minecraft.util.io.netty.handler.codec.http.HttpResponse;
import net.minecraft.util.io.netty.handler.codec.http.HttpResponseStatus;
import net.minecraft.util.io.netty.handler.codec.http.HttpVersion;
import net.minecraft.util.io.netty.handler.ssl.SslHandler;




















class WebSocketServerProtocolHandshakeHandler
  extends ChannelInboundHandlerAdapter
{
  private final String websocketPath;
  private final String subprotocols;
  private final boolean allowExtensions;
  
  public WebSocketServerProtocolHandshakeHandler(String websocketPath, String subprotocols, boolean allowExtensions)
  {
    this.websocketPath = websocketPath;
    this.subprotocols = subprotocols;
    this.allowExtensions = allowExtensions;
  }
  
  public void channelRead(final ChannelHandlerContext ctx, Object msg) throws Exception
  {
    FullHttpRequest req = (FullHttpRequest)msg;
    if (req.getMethod() != HttpMethod.GET) {
      sendHttpResponse(ctx, req, new DefaultFullHttpResponse(HttpVersion.HTTP_1_1, HttpResponseStatus.FORBIDDEN));
      return;
    }
    
    WebSocketServerHandshakerFactory wsFactory = new WebSocketServerHandshakerFactory(getWebSocketLocation(ctx.pipeline(), req, websocketPath), subprotocols, allowExtensions);
    
    WebSocketServerHandshaker handshaker = wsFactory.newHandshaker(req);
    if (handshaker == null) {
      WebSocketServerHandshakerFactory.sendUnsupportedWebSocketVersionResponse(ctx.channel());
    } else {
      ChannelFuture handshakeFuture = handshaker.handshake(ctx.channel(), req);
      handshakeFuture.addListener(new ChannelFutureListener()
      {
        public void operationComplete(ChannelFuture future) throws Exception {
          if (!future.isSuccess()) {
            ctx.fireExceptionCaught(future.cause());
          } else {
            ctx.fireUserEventTriggered(WebSocketServerProtocolHandler.ServerHandshakeStateEvent.HANDSHAKE_COMPLETE);
          }
          
        }
      });
      WebSocketServerProtocolHandler.setHandshaker(ctx, handshaker);
      ctx.pipeline().replace(this, "WS403Responder", WebSocketServerProtocolHandler.forbiddenHttpRequestResponder());
    }
  }
  
  private static void sendHttpResponse(ChannelHandlerContext ctx, HttpRequest req, HttpResponse res)
  {
    ChannelFuture f = ctx.channel().writeAndFlush(res);
    if ((!HttpHeaders.isKeepAlive(req)) || (res.getStatus().code() != 200)) {
      f.addListener(ChannelFutureListener.CLOSE);
    }
  }
  
  private static String getWebSocketLocation(ChannelPipeline cp, HttpRequest req, String path) {
    String protocol = "ws";
    if (cp.get(SslHandler.class) != null)
    {
      protocol = "wss";
    }
    return protocol + "://" + req.headers().get("Host") + path;
  }
}
