package net.minecraft.util.io.netty.handler.codec.marshalling;

import java.util.List;
import net.minecraft.util.io.netty.buffer.ByteBuf;
import net.minecraft.util.io.netty.channel.ChannelHandlerContext;
import net.minecraft.util.io.netty.handler.codec.ReplayingDecoder;
import net.minecraft.util.io.netty.handler.codec.TooLongFrameException;
import org.jboss.marshalling.ByteInput;
import org.jboss.marshalling.Unmarshaller;



































public class CompatibleMarshallingDecoder
  extends ReplayingDecoder<Void>
{
  protected final UnmarshallerProvider provider;
  protected final int maxObjectSize;
  private boolean discardingTooLongFrame;
  
  public CompatibleMarshallingDecoder(UnmarshallerProvider provider, int maxObjectSize)
  {
    this.provider = provider;
    this.maxObjectSize = maxObjectSize;
  }
  
  protected void decode(ChannelHandlerContext ctx, ByteBuf buffer, List<Object> out) throws Exception
  {
    if (discardingTooLongFrame) {
      buffer.skipBytes(actualReadableBytes());
      checkpoint();
      return;
    }
    
    Unmarshaller unmarshaller = provider.getUnmarshaller(ctx);
    ByteInput input = new ChannelBufferByteInput(buffer);
    if (maxObjectSize != 2147483647) {
      input = new LimitingByteInput(input, maxObjectSize);
    }
    try {
      unmarshaller.start(input);
      Object obj = unmarshaller.readObject();
      unmarshaller.finish();
      out.add(obj);
    } catch (LimitingByteInput.TooBigObjectException e) {
      discardingTooLongFrame = true;
      throw new TooLongFrameException();
    }
    finally
    {
      unmarshaller.close();
    }
  }
  
  protected void decodeLast(ChannelHandlerContext ctx, ByteBuf buffer, List<Object> out) throws Exception
  {
    switch (buffer.readableBytes()) {
    case 0: 
      return;
    
    case 1: 
      if (buffer.getByte(buffer.readerIndex()) == 121) {
        buffer.skipBytes(1); return;
      }
      break;
    }
    
    decode(ctx, buffer, out);
  }
  
  public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception
  {
    if ((cause instanceof TooLongFrameException)) {
      ctx.close();
    } else {
      super.exceptionCaught(ctx, cause);
    }
  }
}
