package net.minecraft.util.io.netty.handler.codec.serialization;

import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.io.Serializable;
import net.minecraft.util.io.netty.buffer.ByteBuf;
import net.minecraft.util.io.netty.buffer.ByteBufOutputStream;
import net.minecraft.util.io.netty.channel.ChannelHandlerContext;
import net.minecraft.util.io.netty.handler.codec.MessageToByteEncoder;
import net.minecraft.util.io.netty.util.Attribute;
import net.minecraft.util.io.netty.util.AttributeKey;
























public class CompatibleObjectEncoder
  extends MessageToByteEncoder<Serializable>
{
  private static final AttributeKey<ObjectOutputStream> OOS = new AttributeKey(CompatibleObjectEncoder.class.getName() + ".oos");
  

  private final int resetInterval;
  
  private int writtenObjects;
  

  public CompatibleObjectEncoder()
  {
    this(16);
  }
  








  public CompatibleObjectEncoder(int resetInterval)
  {
    if (resetInterval < 0) {
      throw new IllegalArgumentException("resetInterval: " + resetInterval);
    }
    
    this.resetInterval = resetInterval;
  }
  



  protected ObjectOutputStream newObjectOutputStream(OutputStream out)
    throws Exception
  {
    return new ObjectOutputStream(out);
  }
  
  protected void encode(ChannelHandlerContext ctx, Serializable msg, ByteBuf out) throws Exception
  {
    Attribute<ObjectOutputStream> oosAttr = ctx.attr(OOS);
    ObjectOutputStream oos = (ObjectOutputStream)oosAttr.get();
    if (oos == null) {
      oos = newObjectOutputStream(new ByteBufOutputStream(out));
      ObjectOutputStream newOos = (ObjectOutputStream)oosAttr.setIfAbsent(oos);
      if (newOos != null) {
        oos = newOos;
      }
    }
    
    synchronized (oos) {
      if (resetInterval != 0)
      {
        writtenObjects += 1;
        if (writtenObjects % resetInterval == 0) {
          oos.reset();
        }
      }
      
      oos.writeObject(msg);
      oos.flush();
    }
  }
}
