package net.minecraft.util.io.netty.handler.codec.spdy;

import net.minecraft.util.io.netty.buffer.ByteBuf;
import net.minecraft.util.io.netty.buffer.Unpooled;
import net.minecraft.util.io.netty.util.IllegalReferenceCountException;
import net.minecraft.util.io.netty.util.internal.StringUtil;





















public class DefaultSpdyDataFrame
  extends DefaultSpdyStreamFrame
  implements SpdyDataFrame
{
  private final ByteBuf data;
  
  public DefaultSpdyDataFrame(int streamId)
  {
    this(streamId, Unpooled.buffer(0));
  }
  





  public DefaultSpdyDataFrame(int streamId, ByteBuf data)
  {
    super(streamId);
    if (data == null) {
      throw new NullPointerException("data");
    }
    this.data = validate(data);
  }
  
  private static ByteBuf validate(ByteBuf data) {
    if (data.readableBytes() > 16777215) {
      throw new IllegalArgumentException("data payload cannot exceed 16777215 bytes");
    }
    
    return data;
  }
  
  public SpdyDataFrame setStreamId(int streamId)
  {
    super.setStreamId(streamId);
    return this;
  }
  
  public SpdyDataFrame setLast(boolean last)
  {
    super.setLast(last);
    return this;
  }
  
  public ByteBuf content()
  {
    if (data.refCnt() <= 0) {
      throw new IllegalReferenceCountException(data.refCnt());
    }
    return data;
  }
  
  public SpdyDataFrame copy()
  {
    SpdyDataFrame frame = new DefaultSpdyDataFrame(getStreamId(), content().copy());
    frame.setLast(isLast());
    return frame;
  }
  
  public SpdyDataFrame duplicate()
  {
    SpdyDataFrame frame = new DefaultSpdyDataFrame(getStreamId(), content().duplicate());
    frame.setLast(isLast());
    return frame;
  }
  
  public int refCnt()
  {
    return data.refCnt();
  }
  
  public SpdyDataFrame retain()
  {
    data.retain();
    return this;
  }
  
  public SpdyDataFrame retain(int increment)
  {
    data.retain(increment);
    return this;
  }
  
  public boolean release()
  {
    return data.release();
  }
  
  public boolean release(int decrement)
  {
    return data.release(decrement);
  }
  
  public String toString()
  {
    StringBuilder buf = new StringBuilder();
    buf.append(getClass().getSimpleName());
    buf.append("(last: ");
    buf.append(isLast());
    buf.append(')');
    buf.append(StringUtil.NEWLINE);
    buf.append("--> Stream-ID = ");
    buf.append(getStreamId());
    buf.append(StringUtil.NEWLINE);
    buf.append("--> Size = ");
    if (refCnt() == 0) {
      buf.append("(freed)");
    } else {
      buf.append(content().readableBytes());
    }
    return buf.toString();
  }
}
