package net.minecraft.util.io.netty.handler.codec.spdy;

import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.TreeMap;
import net.minecraft.util.io.netty.util.internal.StringUtil;














public class DefaultSpdySettingsFrame
  implements SpdySettingsFrame
{
  private boolean clear;
  private final Map<Integer, Setting> settingsMap;
  
  public DefaultSpdySettingsFrame()
  {
    settingsMap = new TreeMap();
  }
  
  public Set<Integer> getIds() {
    return settingsMap.keySet();
  }
  
  public boolean isSet(int id)
  {
    Integer key = Integer.valueOf(id);
    return settingsMap.containsKey(key);
  }
  
  public int getValue(int id)
  {
    Integer key = Integer.valueOf(id);
    if (settingsMap.containsKey(key)) {
      return ((Setting)settingsMap.get(key)).getValue();
    }
    return -1;
  }
  

  public SpdySettingsFrame setValue(int id, int value)
  {
    return setValue(id, value, false, false);
  }
  
  public SpdySettingsFrame setValue(int id, int value, boolean persistValue, boolean persisted)
  {
    if ((id <= 0) || (id > 16777215)) {
      throw new IllegalArgumentException("Setting ID is not valid: " + id);
    }
    Integer key = Integer.valueOf(id);
    if (settingsMap.containsKey(key)) {
      Setting setting = (Setting)settingsMap.get(key);
      setting.setValue(value);
      setting.setPersist(persistValue);
      setting.setPersisted(persisted);
    } else {
      settingsMap.put(key, new Setting(value, persistValue, persisted));
    }
    return this;
  }
  
  public SpdySettingsFrame removeValue(int id)
  {
    Integer key = Integer.valueOf(id);
    if (settingsMap.containsKey(key)) {
      settingsMap.remove(key);
    }
    return this;
  }
  
  public boolean isPersistValue(int id)
  {
    Integer key = Integer.valueOf(id);
    if (settingsMap.containsKey(key)) {
      return ((Setting)settingsMap.get(key)).isPersist();
    }
    return false;
  }
  

  public SpdySettingsFrame setPersistValue(int id, boolean persistValue)
  {
    Integer key = Integer.valueOf(id);
    if (settingsMap.containsKey(key)) {
      ((Setting)settingsMap.get(key)).setPersist(persistValue);
    }
    return this;
  }
  
  public boolean isPersisted(int id)
  {
    Integer key = Integer.valueOf(id);
    if (settingsMap.containsKey(key)) {
      return ((Setting)settingsMap.get(key)).isPersisted();
    }
    return false;
  }
  

  public SpdySettingsFrame setPersisted(int id, boolean persisted)
  {
    Integer key = Integer.valueOf(id);
    if (settingsMap.containsKey(key)) {
      ((Setting)settingsMap.get(key)).setPersisted(persisted);
    }
    return this;
  }
  
  public boolean clearPreviouslyPersistedSettings()
  {
    return clear;
  }
  
  public SpdySettingsFrame setClearPreviouslyPersistedSettings(boolean clear)
  {
    this.clear = clear;
    return this;
  }
  
  private Set<Map.Entry<Integer, Setting>> getSettings() {
    return settingsMap.entrySet();
  }
  
  private void appendSettings(StringBuilder buf) {
    for (Map.Entry<Integer, Setting> e : getSettings()) {
      Setting setting = (Setting)e.getValue();
      buf.append("--> ");
      buf.append(((Integer)e.getKey()).toString());
      buf.append(':');
      buf.append(setting.getValue());
      buf.append(" (persist value: ");
      buf.append(setting.isPersist());
      buf.append("; persisted: ");
      buf.append(setting.isPersisted());
      buf.append(')');
      buf.append(StringUtil.NEWLINE);
    }
  }
  
  public String toString()
  {
    StringBuilder buf = new StringBuilder();
    buf.append(getClass().getSimpleName());
    buf.append(StringUtil.NEWLINE);
    appendSettings(buf);
    buf.setLength(buf.length() - StringUtil.NEWLINE.length());
    return buf.toString();
  }
  
  private static final class Setting
  {
    private int value;
    private boolean persist;
    private boolean persisted;
    
    Setting(int value, boolean persist, boolean persisted) {
      this.value = value;
      this.persist = persist;
      this.persisted = persisted;
    }
    
    int getValue() {
      return value;
    }
    
    void setValue(int value) {
      this.value = value;
    }
    
    boolean isPersist() {
      return persist;
    }
    
    void setPersist(boolean persist) {
      this.persist = persist;
    }
    
    boolean isPersisted() {
      return persisted;
    }
    
    void setPersisted(boolean persisted) {
      this.persisted = persisted;
    }
  }
}
