package net.minecraft.util.io.netty.handler.codec.spdy;

import net.minecraft.util.io.netty.buffer.ByteBuf;


















public class SpdyHeaderBlockRawDecoder
  extends SpdyHeaderBlockDecoder
{
  private final int version;
  private final int maxHeaderSize;
  private final int lengthFieldSize;
  private int headerSize;
  private int numHeaders;
  
  public SpdyHeaderBlockRawDecoder(int version, int maxHeaderSize)
  {
    if ((version < 2) || (version > 3)) {
      throw new IllegalArgumentException("unsupported version: " + version);
    }
    

    this.version = version;
    this.maxHeaderSize = maxHeaderSize;
    lengthFieldSize = (version < 3 ? 2 : 4);
    reset();
  }
  
  private int readLengthField(ByteBuf buffer) {
    int length;
    if (version < 3) {
      int length = SpdyCodecUtil.getUnsignedShort(buffer, buffer.readerIndex());
      buffer.skipBytes(2);
    } else {
      length = SpdyCodecUtil.getSignedInt(buffer, buffer.readerIndex());
      buffer.skipBytes(4);
    }
    return length;
  }
  
  void decode(ByteBuf encoded, SpdyHeadersFrame frame) throws Exception
  {
    if (encoded == null) {
      throw new NullPointerException("encoded");
    }
    if (frame == null) {
      throw new NullPointerException("frame");
    }
    
    if (numHeaders == -1)
    {
      if (encoded.readableBytes() < lengthFieldSize) {
        return;
      }
      numHeaders = readLengthField(encoded);
      if (numHeaders < 0) {
        frame.setInvalid();
        return;
      }
    }
    
    while (numHeaders > 0) {
      int headerSize = this.headerSize;
      encoded.markReaderIndex();
      

      if (encoded.readableBytes() < lengthFieldSize) {
        encoded.resetReaderIndex();
        return;
      }
      int nameLength = readLengthField(encoded);
      

      if (nameLength <= 0) {
        frame.setInvalid();
        return;
      }
      headerSize += nameLength;
      if (headerSize > maxHeaderSize) {
        frame.setTruncated();
        return;
      }
      

      if (encoded.readableBytes() < nameLength) {
        encoded.resetReaderIndex();
        return;
      }
      byte[] nameBytes = new byte[nameLength];
      encoded.readBytes(nameBytes);
      String name = new String(nameBytes, "UTF-8");
      

      if (frame.headers().contains(name)) {
        frame.setInvalid();
        return;
      }
      

      if (encoded.readableBytes() < lengthFieldSize) {
        encoded.resetReaderIndex();
        return;
      }
      int valueLength = readLengthField(encoded);
      

      if (valueLength < 0) {
        frame.setInvalid();
        return;
      }
      

      if (valueLength == 0) {
        if (version < 3) {
          frame.setInvalid();
          return;
        }
        frame.headers().add(name, "");
        numHeaders -= 1;
        this.headerSize = headerSize;

      }
      else
      {
        headerSize += valueLength;
        if (headerSize > maxHeaderSize) {
          frame.setTruncated();
          return;
        }
        

        if (encoded.readableBytes() < valueLength) {
          encoded.resetReaderIndex();
          return;
        }
        byte[] valueBytes = new byte[valueLength];
        encoded.readBytes(valueBytes);
        

        int index = 0;
        int offset = 0;
        while (index < valueLength) {
          while ((index < valueBytes.length) && (valueBytes[index] != 0)) {
            index++;
          }
          if ((index < valueBytes.length) && (valueBytes[(index + 1)] == 0))
          {

            frame.setInvalid();
            return;
          }
          String value = new String(valueBytes, offset, index - offset, "UTF-8");
          try
          {
            frame.headers().add(name, value);
          }
          catch (IllegalArgumentException e) {
            frame.setInvalid();
            return;
          }
          index++;
          offset = index;
        }
        numHeaders -= 1;
        this.headerSize = headerSize;
      }
    }
  }
  
  void reset()
  {
    headerSize = 0;
    numHeaders = -1;
  }
  
  void end() {}
}
