package net.minecraft.util.io.netty.handler.codec.spdy;

import java.util.Set;
import net.minecraft.util.io.netty.buffer.ByteBuf;
import net.minecraft.util.io.netty.buffer.Unpooled;
import net.minecraft.util.io.netty.channel.ChannelHandlerContext;

















public class SpdyHeaderBlockRawEncoder
  extends SpdyHeaderBlockEncoder
{
  private final int version;
  
  public SpdyHeaderBlockRawEncoder(int version)
  {
    if ((version < 2) || (version > 3)) {
      throw new IllegalArgumentException("unknown version: " + version);
    }
    
    this.version = version;
  }
  
  private void setLengthField(ByteBuf buffer, int writerIndex, int length) {
    if (version < 3) {
      buffer.setShort(writerIndex, length);
    } else {
      buffer.setInt(writerIndex, length);
    }
  }
  
  private void writeLengthField(ByteBuf buffer, int length) {
    if (version < 3) {
      buffer.writeShort(length);
    } else {
      buffer.writeInt(length);
    }
  }
  
  public ByteBuf encode(ChannelHandlerContext ctx, SpdyHeadersFrame frame) throws Exception
  {
    Set<String> names = frame.headers().names();
    int numHeaders = names.size();
    if (numHeaders == 0) {
      return Unpooled.EMPTY_BUFFER;
    }
    if (numHeaders > 65535) {
      throw new IllegalArgumentException("header block contains too many headers");
    }
    
    ByteBuf headerBlock = Unpooled.buffer();
    writeLengthField(headerBlock, numHeaders);
    for (String name : names) {
      byte[] nameBytes = name.getBytes("UTF-8");
      writeLengthField(headerBlock, nameBytes.length);
      headerBlock.writeBytes(nameBytes);
      int savedIndex = headerBlock.writerIndex();
      int valueLength = 0;
      writeLengthField(headerBlock, valueLength);
      for (String value : frame.headers().getAll(name)) {
        byte[] valueBytes = value.getBytes("UTF-8");
        if (valueBytes.length > 0) {
          headerBlock.writeBytes(valueBytes);
          headerBlock.writeByte(0);
          valueLength += valueBytes.length + 1;
        }
      }
      if (valueLength == 0) {
        if (version < 3) {
          throw new IllegalArgumentException("header value cannot be empty: " + name);
        }
      }
      else {
        valueLength--;
      }
      if (valueLength > 65535) {
        throw new IllegalArgumentException("header exceeds allowable length: " + name);
      }
      
      if (valueLength > 0) {
        setLengthField(headerBlock, savedIndex, valueLength);
        headerBlock.writerIndex(headerBlock.writerIndex() - 1);
      }
    }
    return headerBlock;
  }
  
  void end() {}
}
