package net.minecraft.util.io.netty.handler.codec.spdy;

import java.util.zip.Deflater;
import net.minecraft.util.io.netty.buffer.ByteBuf;
import net.minecraft.util.io.netty.buffer.ByteBufAllocator;
import net.minecraft.util.io.netty.buffer.Unpooled;
import net.minecraft.util.io.netty.channel.ChannelHandlerContext;

















class SpdyHeaderBlockZlibEncoder
  extends SpdyHeaderBlockRawEncoder
{
  private final byte[] out = new byte[8192];
  private final Deflater compressor;
  private boolean finished;
  
  public SpdyHeaderBlockZlibEncoder(int version, int compressionLevel)
  {
    super(version);
    if ((compressionLevel < 0) || (compressionLevel > 9)) {
      throw new IllegalArgumentException("compressionLevel: " + compressionLevel + " (expected: 0-9)");
    }
    
    compressor = new Deflater(compressionLevel);
    if (version < 3) {
      compressor.setDictionary(SpdyCodecUtil.SPDY2_DICT);
    } else {
      compressor.setDictionary(SpdyCodecUtil.SPDY_DICT);
    }
  }
  
  private void setInput(ByteBuf decompressed) {
    byte[] in = new byte[decompressed.readableBytes()];
    decompressed.readBytes(in);
    compressor.setInput(in);
  }
  
  private void encode(ByteBuf compressed) {
    int numBytes = out.length;
    while (numBytes == out.length) {
      numBytes = compressor.deflate(out, 0, out.length, 2);
      compressed.writeBytes(out, 0, numBytes);
    }
  }
  
  public ByteBuf encode(ChannelHandlerContext ctx, SpdyHeadersFrame frame) throws Exception
  {
    if (frame == null) {
      throw new IllegalArgumentException("frame");
    }
    
    if (finished) {
      return Unpooled.EMPTY_BUFFER;
    }
    
    ByteBuf decompressed = super.encode(ctx, frame);
    if (decompressed.readableBytes() == 0) {
      return Unpooled.EMPTY_BUFFER;
    }
    
    ByteBuf compressed = ctx.alloc().buffer();
    setInput(decompressed);
    encode(compressed);
    return compressed;
  }
  
  public void end()
  {
    if (finished) {
      return;
    }
    finished = true;
    compressor.end();
    super.end();
  }
}
