package net.minecraft.util.io.netty.handler.codec.spdy;

import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map.Entry;
import java.util.Set;
import net.minecraft.util.io.netty.handler.codec.http.HttpMethod;
import net.minecraft.util.io.netty.handler.codec.http.HttpResponseStatus;
import net.minecraft.util.io.netty.handler.codec.http.HttpVersion;




















public abstract class SpdyHeaders
  implements Iterable<Map.Entry<String, String>>
{
  public static final SpdyHeaders EMPTY_HEADERS = new SpdyHeaders()
  {
    public List<String> getAll(String name)
    {
      return Collections.emptyList();
    }
    
    public List<Map.Entry<String, String>> entries()
    {
      return Collections.emptyList();
    }
    
    public boolean contains(String name)
    {
      return false;
    }
    
    public boolean isEmpty()
    {
      return true;
    }
    
    public Set<String> names()
    {
      return Collections.emptySet();
    }
    
    public SpdyHeaders add(String name, Object value)
    {
      throw new UnsupportedOperationException("read only");
    }
    
    public SpdyHeaders add(String name, Iterable<?> values)
    {
      throw new UnsupportedOperationException("read only");
    }
    
    public SpdyHeaders set(String name, Object value)
    {
      throw new UnsupportedOperationException("read only");
    }
    
    public SpdyHeaders set(String name, Iterable<?> values)
    {
      throw new UnsupportedOperationException("read only");
    }
    
    public SpdyHeaders remove(String name)
    {
      throw new UnsupportedOperationException("read only");
    }
    
    public SpdyHeaders clear()
    {
      throw new UnsupportedOperationException("read only");
    }
    
    public Iterator<Map.Entry<String, String>> iterator()
    {
      return entries().iterator();
    }
    
    public String get(String name)
    {
      return null;
    }
  };
  


































































  public static String getHeader(SpdyHeadersFrame frame, String name)
  {
    return frame.headers().get(name);
  }
  







  public static String getHeader(SpdyHeadersFrame frame, String name, String defaultValue)
  {
    String value = frame.headers().get(name);
    if (value == null) {
      return defaultValue;
    }
    return value;
  }
  



  public static void setHeader(SpdyHeadersFrame frame, String name, Object value)
  {
    frame.headers().set(name, value);
  }
  



  public static void setHeader(SpdyHeadersFrame frame, String name, Iterable<?> values)
  {
    frame.headers().set(name, values);
  }
  


  public static void addHeader(SpdyHeadersFrame frame, String name, Object value)
  {
    frame.headers().add(name, value);
  }
  


  public static void removeHost(SpdyHeadersFrame frame)
  {
    frame.headers().remove(":host");
  }
  


  public static String getHost(SpdyHeadersFrame frame)
  {
    return frame.headers().get(":host");
  }
  


  public static void setHost(SpdyHeadersFrame frame, String host)
  {
    frame.headers().set(":host", host);
  }
  


  public static void removeMethod(int spdyVersion, SpdyHeadersFrame frame)
  {
    if (spdyVersion < 3) {
      frame.headers().remove("method");
    } else {
      frame.headers().remove(":method");
    }
  }
  

  public static HttpMethod getMethod(int spdyVersion, SpdyHeadersFrame frame)
  {
    try
    {
      if (spdyVersion < 3) {
        return HttpMethod.valueOf(frame.headers().get("method"));
      }
      return HttpMethod.valueOf(frame.headers().get(":method"));
    }
    catch (Exception e) {}
    return null;
  }
  



  public static void setMethod(int spdyVersion, SpdyHeadersFrame frame, HttpMethod method)
  {
    if (spdyVersion < 3) {
      frame.headers().set("method", method.name());
    } else {
      frame.headers().set(":method", method.name());
    }
  }
  


  public static void removeScheme(int spdyVersion, SpdyHeadersFrame frame)
  {
    if (spdyVersion < 2) {
      frame.headers().remove("scheme");
    } else {
      frame.headers().remove(":scheme");
    }
  }
  


  public static String getScheme(int spdyVersion, SpdyHeadersFrame frame)
  {
    if (spdyVersion < 3) {
      return frame.headers().get("scheme");
    }
    return frame.headers().get(":scheme");
  }
  



  public static void setScheme(int spdyVersion, SpdyHeadersFrame frame, String scheme)
  {
    if (spdyVersion < 3) {
      frame.headers().set("scheme", scheme);
    } else {
      frame.headers().set(":scheme", scheme);
    }
  }
  


  public static void removeStatus(int spdyVersion, SpdyHeadersFrame frame)
  {
    if (spdyVersion < 3) {
      frame.headers().remove("status");
    } else {
      frame.headers().remove(":status");
    }
  }
  
  public static HttpResponseStatus getStatus(int spdyVersion, SpdyHeadersFrame frame)
  {
    try
    {
      String status;
      String status;
      if (spdyVersion < 3) {
        status = frame.headers().get("status");
      } else {
        status = frame.headers().get(":status");
      }
      int space = status.indexOf(' ');
      if (space == -1) {
        return HttpResponseStatus.valueOf(Integer.parseInt(status));
      }
      int code = Integer.parseInt(status.substring(0, space));
      String reasonPhrase = status.substring(space + 1);
      HttpResponseStatus responseStatus = HttpResponseStatus.valueOf(code);
      if (responseStatus.reasonPhrase().equals(reasonPhrase)) {
        return responseStatus;
      }
      return new HttpResponseStatus(code, reasonPhrase);
    }
    catch (Exception e) {}
    
    return null;
  }
  



  public static void setStatus(int spdyVersion, SpdyHeadersFrame frame, HttpResponseStatus status)
  {
    if (spdyVersion < 3) {
      frame.headers().set("status", status.toString());
    } else {
      frame.headers().set(":status", status.toString());
    }
  }
  


  public static void removeUrl(int spdyVersion, SpdyHeadersFrame frame)
  {
    if (spdyVersion < 3) {
      frame.headers().remove("url");
    } else {
      frame.headers().remove(":path");
    }
  }
  


  public static String getUrl(int spdyVersion, SpdyHeadersFrame frame)
  {
    if (spdyVersion < 3) {
      return frame.headers().get("url");
    }
    return frame.headers().get(":path");
  }
  



  public static void setUrl(int spdyVersion, SpdyHeadersFrame frame, String path)
  {
    if (spdyVersion < 3) {
      frame.headers().set("url", path);
    } else {
      frame.headers().set(":path", path);
    }
  }
  


  public static void removeVersion(int spdyVersion, SpdyHeadersFrame frame)
  {
    if (spdyVersion < 3) {
      frame.headers().remove("version");
    } else {
      frame.headers().remove(":version");
    }
  }
  

  public static HttpVersion getVersion(int spdyVersion, SpdyHeadersFrame frame)
  {
    try
    {
      if (spdyVersion < 3) {
        return HttpVersion.valueOf(frame.headers().get("version"));
      }
      return HttpVersion.valueOf(frame.headers().get(":version"));
    }
    catch (Exception e) {}
    return null;
  }
  



  public static void setVersion(int spdyVersion, SpdyHeadersFrame frame, HttpVersion httpVersion)
  {
    if (spdyVersion < 3) {
      frame.headers().set("version", httpVersion.text());
    } else {
      frame.headers().set(":version", httpVersion.text());
    }
  }
  
  public Iterator<Map.Entry<String, String>> iterator() {
    return entries().iterator();
  }
  
  public abstract String get(String paramString);
  
  public abstract List<String> getAll(String paramString);
  
  public abstract List<Map.Entry<String, String>> entries();
  
  public abstract boolean contains(String paramString);
  
  public abstract Set<String> names();
  
  public abstract SpdyHeaders add(String paramString, Object paramObject);
  
  public abstract SpdyHeaders add(String paramString, Iterable<?> paramIterable);
  
  public abstract SpdyHeaders set(String paramString, Object paramObject);
  
  public abstract SpdyHeaders set(String paramString, Iterable<?> paramIterable);
  
  public abstract SpdyHeaders remove(String paramString);
  
  public abstract SpdyHeaders clear();
  
  public abstract boolean isEmpty();
  
  public static final class Spdy2HttpNames
  {
    public static final String METHOD = "method";
    public static final String SCHEME = "scheme";
    public static final String STATUS = "status";
    public static final String URL = "url";
    public static final String VERSION = "version";
  }
  
  public static final class HttpNames
  {
    public static final String HOST = ":host";
    public static final String METHOD = ":method";
    public static final String PATH = ":path";
    public static final String SCHEME = ":scheme";
    public static final String STATUS = ":status";
    public static final String VERSION = ":version";
  }
}
