package net.minecraft.util.io.netty.handler.codec.spdy;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import net.minecraft.util.io.netty.buffer.ByteBuf;
import net.minecraft.util.io.netty.channel.ChannelHandlerContext;
import net.minecraft.util.io.netty.handler.codec.MessageToMessageDecoder;
import net.minecraft.util.io.netty.handler.codec.TooLongFrameException;
import net.minecraft.util.io.netty.handler.codec.http.DefaultFullHttpRequest;
import net.minecraft.util.io.netty.handler.codec.http.DefaultFullHttpResponse;
import net.minecraft.util.io.netty.handler.codec.http.FullHttpMessage;
import net.minecraft.util.io.netty.handler.codec.http.FullHttpRequest;
import net.minecraft.util.io.netty.handler.codec.http.FullHttpResponse;
import net.minecraft.util.io.netty.handler.codec.http.HttpHeaders;
import net.minecraft.util.io.netty.handler.codec.http.HttpMethod;
import net.minecraft.util.io.netty.handler.codec.http.HttpResponseStatus;
import net.minecraft.util.io.netty.handler.codec.http.HttpVersion;


























public class SpdyHttpDecoder
  extends MessageToMessageDecoder<SpdyFrame>
{
  private final int spdyVersion;
  private final int maxContentLength;
  private final Map<Integer, FullHttpMessage> messageMap;
  
  public SpdyHttpDecoder(int version, int maxContentLength)
  {
    this(version, maxContentLength, new HashMap());
  }
  








  protected SpdyHttpDecoder(int version, int maxContentLength, Map<Integer, FullHttpMessage> messageMap)
  {
    if ((version < 2) || (version > 3)) {
      throw new IllegalArgumentException("unsupported version: " + version);
    }
    
    if (maxContentLength <= 0) {
      throw new IllegalArgumentException("maxContentLength must be a positive integer: " + maxContentLength);
    }
    
    spdyVersion = version;
    this.maxContentLength = maxContentLength;
    this.messageMap = messageMap;
  }
  
  protected FullHttpMessage putMessage(int streamId, FullHttpMessage message) {
    return (FullHttpMessage)messageMap.put(Integer.valueOf(streamId), message);
  }
  
  protected FullHttpMessage getMessage(int streamId) {
    return (FullHttpMessage)messageMap.get(Integer.valueOf(streamId));
  }
  
  protected FullHttpMessage removeMessage(int streamId) {
    return (FullHttpMessage)messageMap.remove(Integer.valueOf(streamId));
  }
  
  protected void decode(ChannelHandlerContext ctx, SpdyFrame msg, List<Object> out)
    throws Exception
  {
    if ((msg instanceof SpdySynStreamFrame))
    {

      SpdySynStreamFrame spdySynStreamFrame = (SpdySynStreamFrame)msg;
      int streamId = spdySynStreamFrame.getStreamId();
      
      if (SpdyCodecUtil.isServerId(streamId))
      {
        int associatedToStreamId = spdySynStreamFrame.getAssociatedToStreamId();
        


        if (associatedToStreamId == 0) {
          SpdyRstStreamFrame spdyRstStreamFrame = new DefaultSpdyRstStreamFrame(streamId, SpdyStreamStatus.INVALID_STREAM);
          
          ctx.writeAndFlush(spdyRstStreamFrame);
          return;
        }
        
        String URL = SpdyHeaders.getUrl(spdyVersion, spdySynStreamFrame);
        


        if (URL == null) {
          SpdyRstStreamFrame spdyRstStreamFrame = new DefaultSpdyRstStreamFrame(streamId, SpdyStreamStatus.PROTOCOL_ERROR);
          
          ctx.writeAndFlush(spdyRstStreamFrame);
          return;
        }
        


        if (spdySynStreamFrame.isTruncated()) {
          SpdyRstStreamFrame spdyRstStreamFrame = new DefaultSpdyRstStreamFrame(streamId, SpdyStreamStatus.INTERNAL_ERROR);
          
          ctx.writeAndFlush(spdyRstStreamFrame);
          return;
        }
        try
        {
          FullHttpResponse httpResponseWithEntity = createHttpResponse(spdyVersion, spdySynStreamFrame);
          


          SpdyHttpHeaders.setStreamId(httpResponseWithEntity, streamId);
          SpdyHttpHeaders.setAssociatedToStreamId(httpResponseWithEntity, associatedToStreamId);
          SpdyHttpHeaders.setPriority(httpResponseWithEntity, spdySynStreamFrame.getPriority());
          SpdyHttpHeaders.setUrl(httpResponseWithEntity, URL);
          
          if (spdySynStreamFrame.isLast()) {
            HttpHeaders.setContentLength(httpResponseWithEntity, 0L);
            out.add(httpResponseWithEntity);
          }
          else {
            putMessage(streamId, httpResponseWithEntity);
          }
        } catch (Exception e) {
          SpdyRstStreamFrame spdyRstStreamFrame = new DefaultSpdyRstStreamFrame(streamId, SpdyStreamStatus.PROTOCOL_ERROR);
          
          ctx.writeAndFlush(spdyRstStreamFrame);
        }
        

      }
      else
      {
        if (spdySynStreamFrame.isTruncated()) {
          SpdySynReplyFrame spdySynReplyFrame = new DefaultSpdySynReplyFrame(streamId);
          spdySynReplyFrame.setLast(true);
          SpdyHeaders.setStatus(spdyVersion, spdySynReplyFrame, HttpResponseStatus.REQUEST_HEADER_FIELDS_TOO_LARGE);
          

          SpdyHeaders.setVersion(spdyVersion, spdySynReplyFrame, HttpVersion.HTTP_1_0);
          ctx.writeAndFlush(spdySynReplyFrame);
          return;
        }
        try
        {
          FullHttpRequest httpRequestWithEntity = createHttpRequest(spdyVersion, spdySynStreamFrame);
          

          SpdyHttpHeaders.setStreamId(httpRequestWithEntity, streamId);
          
          if (spdySynStreamFrame.isLast()) {
            out.add(httpRequestWithEntity);
          }
          else {
            putMessage(streamId, httpRequestWithEntity);
          }
          
        }
        catch (Exception e)
        {
          SpdySynReplyFrame spdySynReplyFrame = new DefaultSpdySynReplyFrame(streamId);
          spdySynReplyFrame.setLast(true);
          SpdyHeaders.setStatus(spdyVersion, spdySynReplyFrame, HttpResponseStatus.BAD_REQUEST);
          SpdyHeaders.setVersion(spdyVersion, spdySynReplyFrame, HttpVersion.HTTP_1_0);
          ctx.writeAndFlush(spdySynReplyFrame);
        }
      }
    }
    else if ((msg instanceof SpdySynReplyFrame))
    {
      SpdySynReplyFrame spdySynReplyFrame = (SpdySynReplyFrame)msg;
      int streamId = spdySynReplyFrame.getStreamId();
      


      if (spdySynReplyFrame.isTruncated()) {
        SpdyRstStreamFrame spdyRstStreamFrame = new DefaultSpdyRstStreamFrame(streamId, SpdyStreamStatus.INTERNAL_ERROR);
        
        ctx.writeAndFlush(spdyRstStreamFrame);
        return;
      }
      try
      {
        FullHttpResponse httpResponseWithEntity = createHttpResponse(spdyVersion, spdySynReplyFrame);
        

        SpdyHttpHeaders.setStreamId(httpResponseWithEntity, streamId);
        
        if (spdySynReplyFrame.isLast()) {
          HttpHeaders.setContentLength(httpResponseWithEntity, 0L);
          out.add(httpResponseWithEntity);
        }
        else {
          putMessage(streamId, httpResponseWithEntity);
        }
      }
      catch (Exception e)
      {
        SpdyRstStreamFrame spdyRstStreamFrame = new DefaultSpdyRstStreamFrame(streamId, SpdyStreamStatus.PROTOCOL_ERROR);
        
        ctx.writeAndFlush(spdyRstStreamFrame);
      }
    }
    else if ((msg instanceof SpdyHeadersFrame))
    {
      SpdyHeadersFrame spdyHeadersFrame = (SpdyHeadersFrame)msg;
      int streamId = spdyHeadersFrame.getStreamId();
      FullHttpMessage fullHttpMessage = getMessage(streamId);
      

      if (fullHttpMessage == null) {
        return;
      }
      

      if (!spdyHeadersFrame.isTruncated()) {
        for (Map.Entry<String, String> e : spdyHeadersFrame.headers().entries()) {
          fullHttpMessage.headers().add((String)e.getKey(), e.getValue());
        }
      }
      
      if (spdyHeadersFrame.isLast()) {
        HttpHeaders.setContentLength(fullHttpMessage, fullHttpMessage.content().readableBytes());
        removeMessage(streamId);
        out.add(fullHttpMessage);
      }
    }
    else if ((msg instanceof SpdyDataFrame))
    {
      SpdyDataFrame spdyDataFrame = (SpdyDataFrame)msg;
      int streamId = spdyDataFrame.getStreamId();
      FullHttpMessage fullHttpMessage = getMessage(streamId);
      

      if (fullHttpMessage == null) {
        return;
      }
      
      ByteBuf content = fullHttpMessage.content();
      if (content.readableBytes() > maxContentLength - spdyDataFrame.content().readableBytes()) {
        removeMessage(streamId);
        throw new TooLongFrameException("HTTP content length exceeded " + maxContentLength + " bytes.");
      }
      

      ByteBuf spdyDataFrameData = spdyDataFrame.content();
      int spdyDataFrameDataLen = spdyDataFrameData.readableBytes();
      content.writeBytes(spdyDataFrameData, spdyDataFrameData.readerIndex(), spdyDataFrameDataLen);
      
      if (spdyDataFrame.isLast()) {
        HttpHeaders.setContentLength(fullHttpMessage, content.readableBytes());
        removeMessage(streamId);
        out.add(fullHttpMessage);
      }
    }
    else if ((msg instanceof SpdyRstStreamFrame))
    {
      SpdyRstStreamFrame spdyRstStreamFrame = (SpdyRstStreamFrame)msg;
      int streamId = spdyRstStreamFrame.getStreamId();
      removeMessage(streamId);
    }
  }
  
  private static FullHttpRequest createHttpRequest(int spdyVersion, SpdyHeadersFrame requestFrame)
    throws Exception
  {
    HttpMethod method = SpdyHeaders.getMethod(spdyVersion, requestFrame);
    String url = SpdyHeaders.getUrl(spdyVersion, requestFrame);
    HttpVersion httpVersion = SpdyHeaders.getVersion(spdyVersion, requestFrame);
    SpdyHeaders.removeMethod(spdyVersion, requestFrame);
    SpdyHeaders.removeUrl(spdyVersion, requestFrame);
    SpdyHeaders.removeVersion(spdyVersion, requestFrame);
    
    FullHttpRequest req = new DefaultFullHttpRequest(httpVersion, method, url);
    

    SpdyHeaders.removeScheme(spdyVersion, requestFrame);
    
    if (spdyVersion >= 3)
    {
      String host = SpdyHeaders.getHost(requestFrame);
      SpdyHeaders.removeHost(requestFrame);
      HttpHeaders.setHost(req, host);
    }
    
    for (Map.Entry<String, String> e : requestFrame.headers().entries()) {
      req.headers().add((String)e.getKey(), e.getValue());
    }
    

    HttpHeaders.setKeepAlive(req, true);
    

    req.headers().remove("Transfer-Encoding");
    
    return req;
  }
  
  private static FullHttpResponse createHttpResponse(int spdyVersion, SpdyHeadersFrame responseFrame)
    throws Exception
  {
    HttpResponseStatus status = SpdyHeaders.getStatus(spdyVersion, responseFrame);
    HttpVersion version = SpdyHeaders.getVersion(spdyVersion, responseFrame);
    SpdyHeaders.removeStatus(spdyVersion, responseFrame);
    SpdyHeaders.removeVersion(spdyVersion, responseFrame);
    
    FullHttpResponse res = new DefaultFullHttpResponse(version, status);
    for (Map.Entry<String, String> e : responseFrame.headers().entries()) {
      res.headers().add((String)e.getKey(), e.getValue());
    }
    

    HttpHeaders.setKeepAlive(res, true);
    

    res.headers().remove("Transfer-Encoding");
    res.headers().remove("Trailer");
    
    return res;
  }
}
