package net.minecraft.util.io.netty.handler.codec.spdy;

import java.util.List;
import java.util.Map.Entry;
import net.minecraft.util.io.netty.buffer.ByteBuf;
import net.minecraft.util.io.netty.channel.ChannelHandlerContext;
import net.minecraft.util.io.netty.handler.codec.MessageToMessageEncoder;
import net.minecraft.util.io.netty.handler.codec.UnsupportedMessageTypeException;
import net.minecraft.util.io.netty.handler.codec.http.FullHttpMessage;
import net.minecraft.util.io.netty.handler.codec.http.FullHttpRequest;
import net.minecraft.util.io.netty.handler.codec.http.HttpContent;
import net.minecraft.util.io.netty.handler.codec.http.HttpHeaders;
import net.minecraft.util.io.netty.handler.codec.http.HttpMessage;
import net.minecraft.util.io.netty.handler.codec.http.HttpObject;
import net.minecraft.util.io.netty.handler.codec.http.HttpRequest;
import net.minecraft.util.io.netty.handler.codec.http.HttpResponse;
import net.minecraft.util.io.netty.handler.codec.http.LastHttpContent;












































































































public class SpdyHttpEncoder
  extends MessageToMessageEncoder<HttpObject>
{
  private final int spdyVersion;
  private int currentStreamId;
  
  public SpdyHttpEncoder(int version)
  {
    if ((version < 2) || (version > 3)) {
      throw new IllegalArgumentException("unsupported version: " + version);
    }
    
    spdyVersion = version;
  }
  
  protected void encode(ChannelHandlerContext ctx, HttpObject msg, List<Object> out)
    throws Exception
  {
    boolean valid = false;
    boolean last = false;
    
    if ((msg instanceof HttpRequest))
    {
      HttpRequest httpRequest = (HttpRequest)msg;
      SpdySynStreamFrame spdySynStreamFrame = createSynStreamFrame(httpRequest);
      out.add(spdySynStreamFrame);
      
      last = spdySynStreamFrame.isLast();
      valid = true;
    }
    if ((msg instanceof HttpResponse))
    {
      HttpResponse httpResponse = (HttpResponse)msg;
      if (httpResponse.headers().contains("X-SPDY-Associated-To-Stream-ID")) {
        SpdySynStreamFrame spdySynStreamFrame = createSynStreamFrame(httpResponse);
        last = spdySynStreamFrame.isLast();
        out.add(spdySynStreamFrame);
      } else {
        SpdySynReplyFrame spdySynReplyFrame = createSynReplyFrame(httpResponse);
        last = spdySynReplyFrame.isLast();
        out.add(spdySynReplyFrame);
      }
      
      valid = true;
    }
    if (((msg instanceof HttpContent)) && (!last))
    {
      HttpContent chunk = (HttpContent)msg;
      
      chunk.content().retain();
      SpdyDataFrame spdyDataFrame = new DefaultSpdyDataFrame(currentStreamId, chunk.content());
      spdyDataFrame.setLast(chunk instanceof LastHttpContent);
      if ((chunk instanceof LastHttpContent)) {
        LastHttpContent trailer = (LastHttpContent)chunk;
        List<Map.Entry<String, String>> trailers = trailer.trailingHeaders().entries();
        if (trailers.isEmpty()) {
          out.add(spdyDataFrame);
        }
        else {
          SpdyHeadersFrame spdyHeadersFrame = new DefaultSpdyHeadersFrame(currentStreamId);
          for (Map.Entry<String, String> entry : trailers) {
            spdyHeadersFrame.headers().add((String)entry.getKey(), entry.getValue());
          }
          

          out.add(spdyHeadersFrame);
          out.add(spdyDataFrame);
        }
      } else {
        out.add(spdyDataFrame);
      }
      
      valid = true;
    }
    
    if (!valid) {
      throw new UnsupportedMessageTypeException(msg, new Class[0]);
    }
  }
  
  private SpdySynStreamFrame createSynStreamFrame(HttpMessage httpMessage)
    throws Exception
  {
    int streamID = SpdyHttpHeaders.getStreamId(httpMessage);
    int associatedToStreamId = SpdyHttpHeaders.getAssociatedToStreamId(httpMessage);
    byte priority = SpdyHttpHeaders.getPriority(httpMessage);
    String URL = SpdyHttpHeaders.getUrl(httpMessage);
    String scheme = SpdyHttpHeaders.getScheme(httpMessage);
    SpdyHttpHeaders.removeStreamId(httpMessage);
    SpdyHttpHeaders.removeAssociatedToStreamId(httpMessage);
    SpdyHttpHeaders.removePriority(httpMessage);
    SpdyHttpHeaders.removeUrl(httpMessage);
    SpdyHttpHeaders.removeScheme(httpMessage);
    


    httpMessage.headers().remove("Connection");
    httpMessage.headers().remove("Keep-Alive");
    httpMessage.headers().remove("Proxy-Connection");
    httpMessage.headers().remove("Transfer-Encoding");
    
    SpdySynStreamFrame spdySynStreamFrame = new DefaultSpdySynStreamFrame(streamID, associatedToStreamId, priority);
    


    if ((httpMessage instanceof FullHttpRequest)) {
      HttpRequest httpRequest = (HttpRequest)httpMessage;
      SpdyHeaders.setMethod(spdyVersion, spdySynStreamFrame, httpRequest.getMethod());
      SpdyHeaders.setUrl(spdyVersion, spdySynStreamFrame, httpRequest.getUri());
      SpdyHeaders.setVersion(spdyVersion, spdySynStreamFrame, httpMessage.getProtocolVersion());
    }
    if ((httpMessage instanceof HttpResponse)) {
      HttpResponse httpResponse = (HttpResponse)httpMessage;
      SpdyHeaders.setStatus(spdyVersion, spdySynStreamFrame, httpResponse.getStatus());
      SpdyHeaders.setUrl(spdyVersion, spdySynStreamFrame, URL);
      SpdyHeaders.setVersion(spdyVersion, spdySynStreamFrame, httpMessage.getProtocolVersion());
      spdySynStreamFrame.setUnidirectional(true);
    }
    

    if (spdyVersion >= 3) {
      String host = HttpHeaders.getHost(httpMessage);
      httpMessage.headers().remove("Host");
      SpdyHeaders.setHost(spdySynStreamFrame, host);
    }
    

    if (scheme == null) {
      scheme = "https";
    }
    SpdyHeaders.setScheme(spdyVersion, spdySynStreamFrame, scheme);
    

    for (Map.Entry<String, String> entry : httpMessage.headers()) {
      spdySynStreamFrame.headers().add((String)entry.getKey(), entry.getValue());
    }
    currentStreamId = spdySynStreamFrame.getStreamId();
    spdySynStreamFrame.setLast(isLast(httpMessage));
    
    return spdySynStreamFrame;
  }
  
  private SpdySynReplyFrame createSynReplyFrame(HttpResponse httpResponse)
    throws Exception
  {
    int streamID = SpdyHttpHeaders.getStreamId(httpResponse);
    SpdyHttpHeaders.removeStreamId(httpResponse);
    


    httpResponse.headers().remove("Connection");
    httpResponse.headers().remove("Keep-Alive");
    httpResponse.headers().remove("Proxy-Connection");
    httpResponse.headers().remove("Transfer-Encoding");
    
    SpdySynReplyFrame spdySynReplyFrame = new DefaultSpdySynReplyFrame(streamID);
    

    SpdyHeaders.setStatus(spdyVersion, spdySynReplyFrame, httpResponse.getStatus());
    SpdyHeaders.setVersion(spdyVersion, spdySynReplyFrame, httpResponse.getProtocolVersion());
    

    for (Map.Entry<String, String> entry : httpResponse.headers()) {
      spdySynReplyFrame.headers().add((String)entry.getKey(), entry.getValue());
    }
    
    currentStreamId = streamID;
    spdySynReplyFrame.setLast(isLast(httpResponse));
    
    return spdySynReplyFrame;
  }
  





  private static boolean isLast(HttpMessage httpMessage)
  {
    if ((httpMessage instanceof FullHttpMessage)) {
      FullHttpMessage fullMessage = (FullHttpMessage)httpMessage;
      if ((fullMessage.trailingHeaders().isEmpty()) && (!fullMessage.content().isReadable())) {
        return true;
      }
    }
    
    return false;
  }
}
