package net.minecraft.util.io.netty.handler.codec.spdy;

import java.util.LinkedList;
import java.util.List;
import java.util.Queue;
import net.minecraft.util.io.netty.channel.ChannelHandlerContext;
import net.minecraft.util.io.netty.handler.codec.MessageToMessageCodec;
import net.minecraft.util.io.netty.handler.codec.http.HttpHeaders;
import net.minecraft.util.io.netty.handler.codec.http.HttpMessage;
import net.minecraft.util.io.netty.util.ReferenceCountUtil;




















public class SpdyHttpResponseStreamIdHandler
  extends MessageToMessageCodec<Object, HttpMessage>
{
  private static final Integer NO_ID = Integer.valueOf(-1);
  private final Queue<Integer> ids = new LinkedList();
  
  public boolean acceptInboundMessage(Object msg) throws Exception
  {
    return ((msg instanceof HttpMessage)) || ((msg instanceof SpdyRstStreamFrame));
  }
  
  protected void encode(ChannelHandlerContext ctx, HttpMessage msg, List<Object> out) throws Exception
  {
    Integer id = (Integer)ids.poll();
    if ((id != null) && (id.intValue() != NO_ID.intValue()) && (!msg.headers().contains("X-SPDY-Stream-ID"))) {
      SpdyHttpHeaders.setStreamId(msg, id.intValue());
    }
    
    out.add(ReferenceCountUtil.retain(msg));
  }
  
  protected void decode(ChannelHandlerContext ctx, Object msg, List<Object> out) throws Exception
  {
    if ((msg instanceof HttpMessage)) {
      boolean contains = ((HttpMessage)msg).headers().contains("X-SPDY-Stream-ID");
      if (!contains) {
        ids.add(NO_ID);
      } else {
        ids.add(Integer.valueOf(SpdyHttpHeaders.getStreamId((HttpMessage)msg)));
      }
    } else if ((msg instanceof SpdyRstStreamFrame)) {
      ids.remove(Integer.valueOf(((SpdyRstStreamFrame)msg).getStreamId()));
    }
    
    out.add(ReferenceCountUtil.retain(msg));
  }
}
