package net.minecraft.util.io.netty.handler.codec.spdy;

import java.util.List;
import javax.net.ssl.SSLEngine;
import net.minecraft.util.io.netty.buffer.ByteBuf;
import net.minecraft.util.io.netty.channel.ChannelHandlerContext;
import net.minecraft.util.io.netty.channel.ChannelInboundHandler;
import net.minecraft.util.io.netty.channel.ChannelPipeline;
import net.minecraft.util.io.netty.handler.codec.ByteToMessageDecoder;
import net.minecraft.util.io.netty.handler.codec.http.HttpObjectAggregator;
import net.minecraft.util.io.netty.handler.codec.http.HttpRequestDecoder;
import net.minecraft.util.io.netty.handler.codec.http.HttpResponseEncoder;
import net.minecraft.util.io.netty.handler.ssl.SslHandler;




















public abstract class SpdyOrHttpChooser
  extends ByteToMessageDecoder
{
  private final int maxSpdyContentLength;
  private final int maxHttpContentLength;
  
  public static enum SelectedProtocol
  {
    SPDY_2, 
    SPDY_3, 
    HTTP_1_1, 
    HTTP_1_0, 
    UNKNOWN;
    
    private SelectedProtocol() {}
  }
  
  protected SpdyOrHttpChooser(int maxSpdyContentLength, int maxHttpContentLength)
  {
    this.maxSpdyContentLength = maxSpdyContentLength;
    this.maxHttpContentLength = maxHttpContentLength;
  }
  


  protected abstract SelectedProtocol getProtocol(SSLEngine paramSSLEngine);
  


  protected void decode(ChannelHandlerContext ctx, ByteBuf in, List<Object> out)
    throws Exception
  {
    if (initPipeline(ctx))
    {

      ctx.pipeline().remove(this);
    }
  }
  
  private boolean initPipeline(ChannelHandlerContext ctx)
  {
    SslHandler handler = (SslHandler)ctx.pipeline().get(SslHandler.class);
    if (handler == null)
    {
      throw new IllegalStateException("SslHandler is needed for SPDY");
    }
    
    SelectedProtocol protocol = getProtocol(handler.engine());
    switch (1.$SwitchMap$io$netty$handler$codec$spdy$SpdyOrHttpChooser$SelectedProtocol[protocol.ordinal()])
    {
    case 1: 
      return false;
    case 2: 
      addSpdyHandlers(ctx, 2);
      break;
    case 3: 
      addSpdyHandlers(ctx, 3);
      break;
    case 4: 
    case 5: 
      addHttpHandlers(ctx);
      break;
    default: 
      throw new IllegalStateException("Unknown SelectedProtocol");
    }
    return true;
  }
  


  protected void addSpdyHandlers(ChannelHandlerContext ctx, int version)
  {
    ChannelPipeline pipeline = ctx.pipeline();
    pipeline.addLast("spdyDecoder", new SpdyFrameDecoder(version));
    pipeline.addLast("spdyEncoder", new SpdyFrameEncoder(version));
    pipeline.addLast("spdySessionHandler", new SpdySessionHandler(version, true));
    pipeline.addLast("spdyHttpEncoder", new SpdyHttpEncoder(version));
    pipeline.addLast("spdyHttpDecoder", new SpdyHttpDecoder(version, maxSpdyContentLength));
    pipeline.addLast("spdyStreamIdHandler", new SpdyHttpResponseStreamIdHandler());
    pipeline.addLast("httpRquestHandler", createHttpRequestHandlerForSpdy());
  }
  


  protected void addHttpHandlers(ChannelHandlerContext ctx)
  {
    ChannelPipeline pipeline = ctx.pipeline();
    pipeline.addLast("httpRquestDecoder", new HttpRequestDecoder());
    pipeline.addLast("httpResponseEncoder", new HttpResponseEncoder());
    pipeline.addLast("httpChunkAggregator", new HttpObjectAggregator(maxHttpContentLength));
    pipeline.addLast("httpRquestHandler", createHttpRequestHandlerForHttp());
  }
  






  protected abstract ChannelInboundHandler createHttpRequestHandlerForHttp();
  






  protected ChannelInboundHandler createHttpRequestHandlerForSpdy()
  {
    return createHttpRequestHandlerForHttp();
  }
}
