package net.minecraft.util.io.netty.handler.stream;

import java.nio.ByteBuffer;
import java.nio.channels.ReadableByteChannel;
import net.minecraft.util.io.netty.buffer.ByteBuf;
import net.minecraft.util.io.netty.buffer.ByteBufAllocator;
import net.minecraft.util.io.netty.channel.ChannelHandlerContext;



























public class ChunkedNioStream
  implements ChunkedInput<ByteBuf>
{
  private final ReadableByteChannel in;
  private final int chunkSize;
  private long offset;
  private final ByteBuffer byteBuffer;
  
  public ChunkedNioStream(ReadableByteChannel in)
  {
    this(in, 8192);
  }
  





  public ChunkedNioStream(ReadableByteChannel in, int chunkSize)
  {
    if (in == null) {
      throw new NullPointerException("in");
    }
    if (chunkSize <= 0) {
      throw new IllegalArgumentException("chunkSize: " + chunkSize + " (expected: a positive integer)");
    }
    
    this.in = in;
    offset = 0L;
    this.chunkSize = chunkSize;
    byteBuffer = ByteBuffer.allocate(chunkSize);
  }
  


  public long transferredBytes()
  {
    return offset;
  }
  
  public boolean isEndOfInput() throws Exception
  {
    if (byteBuffer.position() > 0)
    {
      return false;
    }
    if (in.isOpen())
    {
      int b = in.read(byteBuffer);
      if (b < 0) {
        return true;
      }
      offset += b;
      return false;
    }
    
    return true;
  }
  
  public void close() throws Exception
  {
    in.close();
  }
  
  public ByteBuf readChunk(ChannelHandlerContext ctx) throws Exception
  {
    if (isEndOfInput()) {
      return null;
    }
    
    int readBytes = byteBuffer.position();
    for (;;) {
      int localReadBytes = in.read(byteBuffer);
      if (localReadBytes < 0) {
        break;
      }
      readBytes += localReadBytes;
      offset += localReadBytes;
      if (readBytes == chunkSize) {
        break;
      }
    }
    byteBuffer.flip();
    boolean release = true;
    ByteBuf buffer = ctx.alloc().buffer(byteBuffer.remaining());
    try {
      buffer.writeBytes(byteBuffer);
      byteBuffer.clear();
      release = false;
      return buffer;
    } finally {
      if (release) {
        buffer.release();
      }
    }
  }
}
