package net.minecraft.util.io.netty.handler.timeout;

import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import net.minecraft.util.io.netty.channel.Channel;
import net.minecraft.util.io.netty.channel.ChannelHandlerContext;
import net.minecraft.util.io.netty.channel.ChannelInboundHandlerAdapter;
import net.minecraft.util.io.netty.util.concurrent.EventExecutor;






























































public class ReadTimeoutHandler
  extends ChannelInboundHandlerAdapter
{
  private final long timeoutMillis;
  private volatile ScheduledFuture<?> timeout;
  private volatile long lastReadTime;
  private volatile int state;
  private boolean closed;
  
  public ReadTimeoutHandler(int timeoutSeconds)
  {
    this(timeoutSeconds, TimeUnit.SECONDS);
  }
  







  public ReadTimeoutHandler(long timeout, TimeUnit unit)
  {
    if (unit == null) {
      throw new NullPointerException("unit");
    }
    
    if (timeout <= 0L) {
      timeoutMillis = 0L;
    } else {
      timeoutMillis = Math.max(unit.toMillis(timeout), 1L);
    }
  }
  
  public void handlerAdded(ChannelHandlerContext ctx) throws Exception
  {
    if ((ctx.channel().isActive()) && (ctx.channel().isRegistered()))
    {

      initialize(ctx);
    }
  }
  


  public void handlerRemoved(ChannelHandlerContext ctx)
    throws Exception
  {
    destroy();
  }
  
  public void channelRegistered(ChannelHandlerContext ctx)
    throws Exception
  {
    if (ctx.channel().isActive()) {
      initialize(ctx);
    }
    super.channelRegistered(ctx);
  }
  


  public void channelActive(ChannelHandlerContext ctx)
    throws Exception
  {
    initialize(ctx);
    super.channelActive(ctx);
  }
  
  public void channelInactive(ChannelHandlerContext ctx) throws Exception
  {
    destroy();
    super.channelInactive(ctx);
  }
  
  public void channelRead(ChannelHandlerContext ctx, Object msg) throws Exception
  {
    lastReadTime = System.currentTimeMillis();
    ctx.fireChannelRead(msg);
  }
  

  private void initialize(ChannelHandlerContext ctx)
  {
    switch (state) {
    case 1: 
    case 2: 
      return;
    }
    
    state = 1;
    
    lastReadTime = System.currentTimeMillis();
    if (timeoutMillis > 0L) {
      timeout = ctx.executor().schedule(new ReadTimeoutTask(ctx), timeoutMillis, TimeUnit.MILLISECONDS);
    }
  }
  

  private void destroy()
  {
    state = 2;
    
    if (timeout != null) {
      timeout.cancel(false);
      timeout = null;
    }
  }
  

  protected void readTimedOut(ChannelHandlerContext ctx)
    throws Exception
  {
    if (!closed) {
      ctx.fireExceptionCaught(ReadTimeoutException.INSTANCE);
      ctx.close();
      closed = true;
    }
  }
  
  private final class ReadTimeoutTask implements Runnable
  {
    private final ChannelHandlerContext ctx;
    
    ReadTimeoutTask(ChannelHandlerContext ctx) {
      this.ctx = ctx;
    }
    
    public void run()
    {
      if (!ctx.channel().isOpen()) {
        return;
      }
      
      long currentTime = System.currentTimeMillis();
      long nextDelay = timeoutMillis - (currentTime - lastReadTime);
      if (nextDelay <= 0L)
      {
        timeout = ctx.executor().schedule(this, timeoutMillis, TimeUnit.MILLISECONDS);
        try {
          readTimedOut(ctx);
        } catch (Throwable t) {
          ctx.fireExceptionCaught(t);
        }
      }
      else {
        timeout = ctx.executor().schedule(this, nextDelay, TimeUnit.MILLISECONDS);
      }
    }
  }
}
