package net.minecraft.util.io.netty.handler.timeout;

import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import net.minecraft.util.io.netty.channel.ChannelFuture;
import net.minecraft.util.io.netty.channel.ChannelFutureListener;
import net.minecraft.util.io.netty.channel.ChannelHandlerContext;
import net.minecraft.util.io.netty.channel.ChannelOutboundHandlerAdapter;
import net.minecraft.util.io.netty.channel.ChannelPromise;
import net.minecraft.util.io.netty.util.concurrent.EventExecutor;





























































public class WriteTimeoutHandler
  extends ChannelOutboundHandlerAdapter
{
  private final long timeoutMillis;
  private boolean closed;
  
  public WriteTimeoutHandler(int timeoutSeconds)
  {
    this(timeoutSeconds, TimeUnit.SECONDS);
  }
  







  public WriteTimeoutHandler(long timeout, TimeUnit unit)
  {
    if (unit == null) {
      throw new NullPointerException("unit");
    }
    
    if (timeout <= 0L) {
      timeoutMillis = 0L;
    } else {
      timeoutMillis = Math.max(unit.toMillis(timeout), 1L);
    }
  }
  
  public void write(ChannelHandlerContext ctx, Object msg, ChannelPromise promise) throws Exception
  {
    scheduleTimeout(ctx, promise);
    ctx.write(msg, promise);
  }
  
  private void scheduleTimeout(final ChannelHandlerContext ctx, final ChannelPromise future) {
    if (timeoutMillis > 0L)
    {
      final ScheduledFuture<?> sf = ctx.executor().schedule(new Runnable()
      {
        public void run() {
          if (future.tryFailure(WriteTimeoutException.INSTANCE))
            try
            {
              writeTimedOut(ctx);
            } catch (Throwable t) {
              ctx.fireExceptionCaught(t); } } }, timeoutMillis, TimeUnit.MILLISECONDS);
      





      future.addListener(new ChannelFutureListener()
      {
        public void operationComplete(ChannelFuture future) throws Exception {
          sf.cancel(false);
        }
      });
    }
  }
  

  protected void writeTimedOut(ChannelHandlerContext ctx)
    throws Exception
  {
    if (!closed) {
      ctx.fireExceptionCaught(WriteTimeoutException.INSTANCE);
      ctx.close();
      closed = true;
    }
  }
}
