package net.minecraft.util.io.netty.handler.traffic;

import java.util.concurrent.TimeUnit;
import net.minecraft.util.io.netty.buffer.ByteBuf;
import net.minecraft.util.io.netty.buffer.ByteBufHolder;
import net.minecraft.util.io.netty.channel.ChannelDuplexHandler;
import net.minecraft.util.io.netty.channel.ChannelHandlerContext;
import net.minecraft.util.io.netty.channel.ChannelPromise;
import net.minecraft.util.io.netty.util.Attribute;
import net.minecraft.util.io.netty.util.AttributeKey;
import net.minecraft.util.io.netty.util.concurrent.EventExecutor;






















































public abstract class AbstractTrafficShapingHandler
  extends ChannelDuplexHandler
{
  public static final long DEFAULT_CHECK_INTERVAL = 1000L;
  private static final long MINIMAL_WAIT = 10L;
  protected TrafficCounter trafficCounter;
  private long writeLimit;
  private long readLimit;
  protected long checkInterval = 1000L;
  
  private static final AttributeKey<Boolean> READ_SUSPENDED = new AttributeKey("readSuspended");
  private static final AttributeKey<Runnable> REOPEN_TASK = new AttributeKey("reopenTask");
  



  void setTrafficCounter(TrafficCounter newTrafficCounter)
  {
    trafficCounter = newTrafficCounter;
  }
  









  protected AbstractTrafficShapingHandler(long writeLimit, long readLimit, long checkInterval)
  {
    this.writeLimit = writeLimit;
    this.readLimit = readLimit;
    this.checkInterval = checkInterval;
  }
  







  protected AbstractTrafficShapingHandler(long writeLimit, long readLimit)
  {
    this(writeLimit, readLimit, 1000L);
  }
  


  protected AbstractTrafficShapingHandler()
  {
    this(0L, 0L, 1000L);
  }
  






  protected AbstractTrafficShapingHandler(long checkInterval)
  {
    this(0L, 0L, checkInterval);
  }
  







  public void configure(long newWriteLimit, long newReadLimit, long newCheckInterval)
  {
    configure(newWriteLimit, newReadLimit);
    configure(newCheckInterval);
  }
  





  public void configure(long newWriteLimit, long newReadLimit)
  {
    writeLimit = newWriteLimit;
    readLimit = newReadLimit;
    if (trafficCounter != null) {
      trafficCounter.resetAccounting(System.currentTimeMillis() + 1L);
    }
  }
  




  public void configure(long newCheckInterval)
  {
    checkInterval = newCheckInterval;
    if (trafficCounter != null) {
      trafficCounter.configure(checkInterval);
    }
  }
  



  protected void doAccounting(TrafficCounter counter) {}
  



  private static final class ReopenReadTimerTask
    implements Runnable
  {
    final ChannelHandlerContext ctx;
    



    ReopenReadTimerTask(ChannelHandlerContext ctx)
    {
      this.ctx = ctx;
    }
    
    public void run()
    {
      ctx.attr(AbstractTrafficShapingHandler.READ_SUSPENDED).set(Boolean.valueOf(false));
      ctx.read();
    }
  }
  


  private static long getTimeToWait(long limit, long bytes, long lastTime, long curtime)
  {
    long interval = curtime - lastTime;
    if (interval <= 0L)
    {
      return 0L;
    }
    return (bytes * 1000L / limit - interval) / 10L * 10L;
  }
  
  public void channelRead(final ChannelHandlerContext ctx, final Object msg) throws Exception
  {
    long size = calculateSize(msg);
    long curtime = System.currentTimeMillis();
    
    if (trafficCounter != null) {
      trafficCounter.bytesRecvFlowControl(size);
      if (readLimit == 0L)
      {
        ctx.fireChannelRead(msg);
        
        return;
      }
      

      long wait = getTimeToWait(readLimit, trafficCounter.currentReadBytes(), trafficCounter.lastTime(), curtime);
      

      if (wait >= 10L)
      {

        if (!isSuspended(ctx)) {
          ctx.attr(READ_SUSPENDED).set(Boolean.valueOf(true));
          


          Attribute<Runnable> attr = ctx.attr(REOPEN_TASK);
          Runnable reopenTask = (Runnable)attr.get();
          if (reopenTask == null) {
            reopenTask = new ReopenReadTimerTask(ctx);
            attr.set(reopenTask);
          }
          ctx.executor().schedule(reopenTask, wait, TimeUnit.MILLISECONDS);

        }
        else
        {
          Runnable bufferUpdateTask = new Runnable()
          {
            public void run() {
              ctx.fireChannelRead(msg);
            }
          };
          ctx.executor().schedule(bufferUpdateTask, wait, TimeUnit.MILLISECONDS);
          return;
        }
      }
    }
    ctx.fireChannelRead(msg);
  }
  
  public void read(ChannelHandlerContext ctx)
  {
    if (!isSuspended(ctx)) {
      ctx.read();
    }
  }
  
  private static boolean isSuspended(ChannelHandlerContext ctx) {
    Boolean suspended = (Boolean)ctx.attr(READ_SUSPENDED).get();
    if ((suspended == null) || (Boolean.FALSE.equals(suspended))) {
      return false;
    }
    return true;
  }
  
  public void write(final ChannelHandlerContext ctx, final Object msg, final ChannelPromise promise)
    throws Exception
  {
    long curtime = System.currentTimeMillis();
    long size = calculateSize(msg);
    
    if ((size > -1L) && (trafficCounter != null)) {
      trafficCounter.bytesWriteFlowControl(size);
      if (writeLimit == 0L) {
        ctx.write(msg, promise);
        return;
      }
      

      long wait = getTimeToWait(writeLimit, trafficCounter.currentWrittenBytes(), trafficCounter.lastTime(), curtime);
      

      if (wait >= 10L) {
        ctx.executor().schedule(new Runnable()
        {

          public void run() { ctx.write(msg, promise); } }, wait, TimeUnit.MILLISECONDS);
        

        return;
      }
    }
    ctx.write(msg, promise);
  }
  




  public TrafficCounter trafficCounter()
  {
    return trafficCounter;
  }
  
  public String toString()
  {
    return "TrafficShaping with Write Limit: " + writeLimit + " Read Limit: " + readLimit + " and Counter: " + (trafficCounter != null ? trafficCounter.toString() : "none");
  }
  








  protected long calculateSize(Object msg)
  {
    if ((msg instanceof ByteBuf)) {
      return ((ByteBuf)msg).readableBytes();
    }
    if ((msg instanceof ByteBufHolder)) {
      return ((ByteBufHolder)msg).content().readableBytes();
    }
    return -1L;
  }
}
