package net.minecraft.util.io.netty.util;

import java.util.IdentityHashMap;
import java.util.Map;
import java.util.concurrent.atomic.AtomicReference;
import java.util.concurrent.atomic.AtomicReferenceFieldUpdater;




















public class DefaultAttributeMap
  implements AttributeMap
{
  private static final AtomicReferenceFieldUpdater<DefaultAttributeMap, Map> updater = AtomicReferenceFieldUpdater.newUpdater(DefaultAttributeMap.class, Map.class, "map");
  

  private volatile Map<AttributeKey<?>, Attribute<?>> map;
  


  public <T> Attribute<T> attr(AttributeKey<T> key)
  {
    Map<AttributeKey<?>, Attribute<?>> map = this.map;
    if (map == null)
    {
      map = new IdentityHashMap(2);
      if (!updater.compareAndSet(this, null, map)) {
        map = this.map;
      }
    }
    
    synchronized (map)
    {
      Attribute<T> attr = (Attribute)map.get(key);
      if (attr == null) {
        attr = new DefaultAttribute(map, key);
        map.put(key, attr);
      }
      return attr;
    }
  }
  
  private static final class DefaultAttribute<T> extends AtomicReference<T> implements Attribute<T>
  {
    private static final long serialVersionUID = -2661411462200283011L;
    private final Map<AttributeKey<?>, Attribute<?>> map;
    private final AttributeKey<T> key;
    
    DefaultAttribute(Map<AttributeKey<?>, Attribute<?>> map, AttributeKey<T> key)
    {
      this.map = map;
      this.key = key;
    }
    
    public AttributeKey<T> key()
    {
      return key;
    }
    
    public T setIfAbsent(T value)
    {
      while (!compareAndSet(null, value)) {
        T old = get();
        if (old != null) {
          return old;
        }
      }
      return null;
    }
    
    public T getAndRemove()
    {
      T oldValue = getAndSet(null);
      remove0();
      return oldValue;
    }
    
    public void remove()
    {
      set(null);
      remove0();
    }
    
    private void remove0() {
      synchronized (map) {
        map.remove(key);
      }
    }
  }
}
