package net.minecraft.util.io.netty.util;

import java.util.IdentityHashMap;
import java.util.Map;




















public abstract class Recycler<T>
{
  private final ThreadLocal<Stack<T>> threadLocal;
  
  public Recycler() { threadLocal = new ThreadLocal()
    {
      protected Recycler.Stack<T> initialValue() {
        return new Recycler.Stack(Recycler.this, Thread.currentThread());
      }
    }; }
  
  public final T get() {
    Stack<T> stack = (Stack)threadLocal.get();
    T o = stack.pop();
    if (o == null) {
      o = newObject(stack);
    }
    return o;
  }
  
  public final boolean recycle(T o, Handle handle)
  {
    Stack<T> stack = (Stack)handle;
    if (parent != this) {
      return false;
    }
    
    if (Thread.currentThread() != thread) {
      return false;
    }
    
    stack.push(o);
    return true;
  }
  

  protected abstract T newObject(Handle paramHandle);
  

  static final class Stack<T>
    implements Recycler.Handle
  {
    private static final int INITIAL_CAPACITY = 256;
    final Recycler<T> parent;
    final Thread thread;
    private T[] elements;
    private int size;
    private final Map<T, Boolean> map = new IdentityHashMap(256);
    
    Stack(Recycler<T> parent, Thread thread)
    {
      this.parent = parent;
      this.thread = thread;
      elements = newArray(256);
    }
    
    T pop() {
      int size = this.size;
      if (size == 0) {
        return null;
      }
      size--;
      T ret = elements[size];
      elements[size] = null;
      map.remove(ret);
      this.size = size;
      return ret;
    }
    
    void push(T o) {
      if (map.put(o, Boolean.TRUE) != null) {
        throw new IllegalStateException("recycled already");
      }
      
      int size = this.size;
      if (size == elements.length) {
        T[] newElements = newArray(size << 1);
        System.arraycopy(elements, 0, newElements, 0, size);
        elements = newElements;
      }
      
      elements[size] = o;
      this.size = (size + 1);
    }
    
    private static <T> T[] newArray(int length)
    {
      return (Object[])new Object[length];
    }
  }
  
  public static abstract interface Handle {}
}
