package net.minecraft.util.io.netty.util.internal;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.nio.Buffer;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import net.minecraft.util.io.netty.util.internal.logging.InternalLogger;
import net.minecraft.util.io.netty.util.internal.logging.InternalLoggerFactory;
import sun.misc.Cleaner;
import sun.misc.Unsafe;




















final class PlatformDependent0
{
  private static final InternalLogger logger = InternalLoggerFactory.getInstance(PlatformDependent0.class);
  private static final Unsafe UNSAFE;
  private static final boolean BIG_ENDIAN = ByteOrder.nativeOrder() == ByteOrder.BIG_ENDIAN;
  

  private static final long CLEANER_FIELD_OFFSET;
  
  private static final long ADDRESS_FIELD_OFFSET;
  
  private static final Field CLEANER_FIELD;
  
  private static final boolean UNALIGNED;
  

  static
  {
    ByteBuffer direct = ByteBuffer.allocateDirect(1);
    Field cleanerField;
    try {
      cleanerField = direct.getClass().getDeclaredField("cleaner");
      cleanerField.setAccessible(true);
      Cleaner cleaner = (Cleaner)cleanerField.get(direct);
      cleaner.clean();
    } catch (Throwable t) {
      cleanerField = null;
    }
    CLEANER_FIELD = cleanerField;
    logger.debug("java.nio.ByteBuffer.cleaner: {}", cleanerField != null ? "available" : "unavailable");
    Field addressField;
    try
    {
      addressField = Buffer.class.getDeclaredField("address");
      addressField.setAccessible(true);
      if (addressField.getLong(ByteBuffer.allocate(1)) != 0L) {
        addressField = null;
      } else {
        direct = ByteBuffer.allocateDirect(1);
        if (addressField.getLong(direct) == 0L) {
          addressField = null;
        }
        Cleaner cleaner = (Cleaner)cleanerField.get(direct);
        cleaner.clean();
      }
    } catch (Throwable t) {
      addressField = null;
    }
    logger.debug("java.nio.Buffer.address: {}", addressField != null ? "available" : "unavailable");
    
    Unsafe unsafe;
    if ((addressField != null) && (cleanerField != null)) {
      try {
        Field unsafeField = Unsafe.class.getDeclaredField("theUnsafe");
        unsafeField.setAccessible(true);
        unsafe = (Unsafe)unsafeField.get(null);
        logger.debug("sun.misc.Unsafe.theUnsafe: {}", unsafe != null ? "available" : "unavailable");
        


        try
        {
          unsafe.getClass().getDeclaredMethod("copyMemory", new Class[] { Object.class, Long.TYPE, Object.class, Long.TYPE, Long.TYPE });
          


          logger.debug("sun.misc.Unsafe.copyMemory: available");
        } catch (NoSuchMethodError t) {
          logger.debug("sun.misc.Unsafe.copyMemory: unavailable");
          throw t;
        } catch (NoSuchMethodException e) {
          logger.debug("sun.misc.Unsafe.copyMemory: unavailable");
          throw e;
        }
      } catch (Throwable cause) {
        Unsafe unsafe = null;
      }
      
    }
    else {
      unsafe = null;
    }
    UNSAFE = unsafe;
    
    if (unsafe == null) {
      CLEANER_FIELD_OFFSET = -1L;
      ADDRESS_FIELD_OFFSET = -1L;
      UNALIGNED = false;
    } else {
      ADDRESS_FIELD_OFFSET = objectFieldOffset(addressField);
      CLEANER_FIELD_OFFSET = objectFieldOffset(cleanerField);
      boolean unaligned;
      try
      {
        Class<?> bitsClass = Class.forName("java.nio.Bits", false, ClassLoader.getSystemClassLoader());
        Method unalignedMethod = bitsClass.getDeclaredMethod("unaligned", new Class[0]);
        unalignedMethod.setAccessible(true);
        unaligned = Boolean.TRUE.equals(unalignedMethod.invoke(null, new Object[0]));
      }
      catch (Throwable t) {
        String arch = SystemPropertyUtil.get("os.arch", "");
        
        unaligned = arch.matches("^(i[3-6]86|x86(_64)?|x64|amd64)$");
      }
      
      UNALIGNED = unaligned;
      logger.debug("java.nio.Bits.unaligned: {}", Boolean.valueOf(UNALIGNED));
    }
  }
  
  static boolean hasUnsafe() {
    return UNSAFE != null;
  }
  
  static void throwException(Throwable t) {
    UNSAFE.throwException(t);
  }
  
  static void freeDirectBufferUnsafe(ByteBuffer buffer)
  {
    try {
      Cleaner cleaner = (Cleaner)getObject(buffer, CLEANER_FIELD_OFFSET);
      if (cleaner == null) {
        throw new IllegalArgumentException("attempted to deallocate the buffer which was allocated via JNIEnv->NewDirectByteBuffer()");
      }
      
      cleaner.clean();
    }
    catch (Throwable t) {}
  }
  
  static void freeDirectBuffer(ByteBuffer buffer)
  {
    if (CLEANER_FIELD == null) {
      return;
    }
    try {
      Cleaner cleaner = (Cleaner)CLEANER_FIELD.get(buffer);
      if (cleaner == null) {
        throw new IllegalArgumentException("attempted to deallocate the buffer which was allocated via JNIEnv->NewDirectByteBuffer()");
      }
      
      cleaner.clean();
    }
    catch (Throwable t) {}
  }
  
  static long directBufferAddress(ByteBuffer buffer)
  {
    return getLong(buffer, ADDRESS_FIELD_OFFSET);
  }
  
  static long arrayBaseOffset() {
    return UNSAFE.arrayBaseOffset([B.class);
  }
  
  static Object getObject(Object object, long fieldOffset) {
    return UNSAFE.getObject(object, fieldOffset);
  }
  
  static int getInt(Object object, long fieldOffset) {
    return UNSAFE.getInt(object, fieldOffset);
  }
  
  private static long getLong(Object object, long fieldOffset) {
    return UNSAFE.getLong(object, fieldOffset);
  }
  
  static long objectFieldOffset(Field field) {
    return UNSAFE.objectFieldOffset(field);
  }
  
  static byte getByte(long address) {
    return UNSAFE.getByte(address);
  }
  
  static short getShort(long address) {
    if (UNALIGNED)
      return UNSAFE.getShort(address);
    if (BIG_ENDIAN) {
      return (short)(getByte(address) << 8 | getByte(address + 1L) & 0xFF);
    }
    return (short)(getByte(address + 1L) << 8 | getByte(address) & 0xFF);
  }
  
  static int getInt(long address)
  {
    if (UNALIGNED)
      return UNSAFE.getInt(address);
    if (BIG_ENDIAN) {
      return getByte(address) << 24 | (getByte(address + 1L) & 0xFF) << 16 | (getByte(address + 2L) & 0xFF) << 8 | getByte(address + 3L) & 0xFF;
    }
    


    return getByte(address + 3L) << 24 | (getByte(address + 2L) & 0xFF) << 16 | (getByte(address + 1L) & 0xFF) << 8 | getByte(address) & 0xFF;
  }
  



  static long getLong(long address)
  {
    if (UNALIGNED)
      return UNSAFE.getLong(address);
    if (BIG_ENDIAN) {
      return getByte(address) << 56 | (getByte(address + 1L) & 0xFF) << 48 | (getByte(address + 2L) & 0xFF) << 40 | (getByte(address + 3L) & 0xFF) << 32 | (getByte(address + 4L) & 0xFF) << 24 | (getByte(address + 5L) & 0xFF) << 16 | (getByte(address + 6L) & 0xFF) << 8 | getByte(address + 7L) & 0xFF;
    }
    






    return getByte(address + 7L) << 56 | (getByte(address + 6L) & 0xFF) << 48 | (getByte(address + 5L) & 0xFF) << 40 | (getByte(address + 4L) & 0xFF) << 32 | (getByte(address + 3L) & 0xFF) << 24 | (getByte(address + 2L) & 0xFF) << 16 | (getByte(address + 1L) & 0xFF) << 8 | getByte(address) & 0xFF;
  }
  







  static void putByte(long address, byte value)
  {
    UNSAFE.putByte(address, value);
  }
  
  static void putShort(long address, short value) {
    if (UNALIGNED) {
      UNSAFE.putShort(address, value);
    } else if (BIG_ENDIAN) {
      putByte(address, (byte)(value >>> 8));
      putByte(address + 1L, (byte)value);
    } else {
      putByte(address + 1L, (byte)(value >>> 8));
      putByte(address, (byte)value);
    }
  }
  
  static void putInt(long address, int value) {
    if (UNALIGNED) {
      UNSAFE.putInt(address, value);
    } else if (BIG_ENDIAN) {
      putByte(address, (byte)(value >>> 24));
      putByte(address + 1L, (byte)(value >>> 16));
      putByte(address + 2L, (byte)(value >>> 8));
      putByte(address + 3L, (byte)value);
    } else {
      putByte(address + 3L, (byte)(value >>> 24));
      putByte(address + 2L, (byte)(value >>> 16));
      putByte(address + 1L, (byte)(value >>> 8));
      putByte(address, (byte)value);
    }
  }
  
  static void putLong(long address, long value) {
    if (UNALIGNED) {
      UNSAFE.putLong(address, value);
    } else if (BIG_ENDIAN) {
      putByte(address, (byte)(int)(value >>> 56));
      putByte(address + 1L, (byte)(int)(value >>> 48));
      putByte(address + 2L, (byte)(int)(value >>> 40));
      putByte(address + 3L, (byte)(int)(value >>> 32));
      putByte(address + 4L, (byte)(int)(value >>> 24));
      putByte(address + 5L, (byte)(int)(value >>> 16));
      putByte(address + 6L, (byte)(int)(value >>> 8));
      putByte(address + 7L, (byte)(int)value);
    } else {
      putByte(address + 7L, (byte)(int)(value >>> 56));
      putByte(address + 6L, (byte)(int)(value >>> 48));
      putByte(address + 5L, (byte)(int)(value >>> 40));
      putByte(address + 4L, (byte)(int)(value >>> 32));
      putByte(address + 3L, (byte)(int)(value >>> 24));
      putByte(address + 2L, (byte)(int)(value >>> 16));
      putByte(address + 1L, (byte)(int)(value >>> 8));
      putByte(address, (byte)(int)value);
    }
  }
  
  static void copyMemory(long srcAddr, long dstAddr, long length) {
    UNSAFE.copyMemory(srcAddr, dstAddr, length);
  }
  
  static void copyMemory(Object src, long srcOffset, Object dst, long dstOffset, long length) {
    UNSAFE.copyMemory(src, srcOffset, dst, dstOffset, length);
  }
}
