package net.minecraft.util.io.netty.util.internal;

import java.security.SecureRandom;
import java.util.Random;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicLong;
import net.minecraft.util.io.netty.util.internal.logging.InternalLogger;
import net.minecraft.util.io.netty.util.internal.logging.InternalLoggerFactory;
















































public class ThreadLocalRandom
  extends Random
{
  private static final InternalLogger logger = InternalLoggerFactory.getInstance(ThreadLocalRandom.class);
  
  private static final AtomicLong seedUniquifier = new AtomicLong();
  private static volatile long initialSeedUniquifier;
  private static final long multiplier = 25214903917L;
  private static final long addend = 11L;
  
  public static void setInitialSeedUniquifier(long initialSeedUniquifier) { initialSeedUniquifier = initialSeedUniquifier; }
  

  public static synchronized long getInitialSeedUniquifier()
  {
    long initialSeedUniquifier = initialSeedUniquifier;
    if (initialSeedUniquifier == 0L)
    {
      initialSeedUniquifier = initialSeedUniquifier = SystemPropertyUtil.getLong("io.netty.initialSeedUniquifier", 0L);
    }
    


    if (initialSeedUniquifier == 0L)
    {

      final BlockingQueue<Long> queue = new LinkedBlockingQueue();
      Thread generatorThread = new Thread("initialSeedUniquifierGenerator")
      {
        public void run() {
          SecureRandom random = new SecureRandom();
          queue.add(Long.valueOf(random.nextLong()));
        }
      };
      generatorThread.start();
      

      long timeoutSeconds = 3L;
      long deadLine = System.nanoTime() + TimeUnit.SECONDS.toNanos(3L);
      for (;;) {
        long waitTime = deadLine - System.nanoTime();
        if (waitTime <= 0L) {
          logger.warn("Failed to get the secure random number from SecureRandom within {} seconds. Not enough entrophy?", Long.valueOf(3L));
          

          break;
        }
        try
        {
          Long result = (Long)queue.poll(waitTime, TimeUnit.NANOSECONDS);
          if (result != null) {
            initialSeedUniquifier = result.longValue();
            break;
          }
        }
        catch (InterruptedException ignore) {}
      }
      


      initialSeedUniquifier ^= 0x33BAE119;
      initialSeedUniquifier ^= Long.reverse(System.nanoTime());
      
      initialSeedUniquifier = initialSeedUniquifier;
    }
    
    return initialSeedUniquifier;
  }
  
  private static long newSeed() {
    for (;;) {
      long current = seedUniquifier.get();
      long actualCurrent = current != 0L ? current : getInitialSeedUniquifier();
      

      long next = actualCurrent * 181783497276652981L;
      
      if (seedUniquifier.compareAndSet(current, next)) {
        if ((current == 0L) && (logger.isDebugEnabled())) {
          logger.debug(String.format("-Dio.netty.initialSeedUniquifier: 0x%016x", new Object[] { Long.valueOf(actualCurrent) }));
        }
        return next ^ System.nanoTime();
      }
    }
  }
  


  private static final long mask = 281474976710655L;
  

  private long rnd;
  
  boolean initialized;
  
  private long pad0;
  
  private long pad1;
  
  private long pad2;
  
  private long pad3;
  
  private long pad4;
  
  private long pad5;
  
  private long pad6;
  
  private long pad7;
  
  ThreadLocalRandom()
  {
    super(newSeed());
    initialized = true;
  }
  



  private static final ThreadLocal<ThreadLocalRandom> localRandom = new ThreadLocal()
  {
    protected ThreadLocalRandom initialValue() {
      return new ThreadLocalRandom();
    }
  };
  

  private static final long serialVersionUID = -5851777807851030925L;
  

  public static ThreadLocalRandom current()
  {
    return (ThreadLocalRandom)localRandom.get();
  }
  





  public void setSeed(long seed)
  {
    if (initialized) {
      throw new UnsupportedOperationException();
    }
    rnd = ((seed ^ 0xDEECE66D) & 0xFFFFFFFF);
  }
  
  protected int next(int bits) {
    rnd = (rnd * 25214903917L + 11L & 0xFFFFFFFF);
    return (int)(rnd >>> 48 - bits);
  }
  









  public int nextInt(int least, int bound)
  {
    if (least >= bound) {
      throw new IllegalArgumentException();
    }
    return nextInt(bound - least) + least;
  }
  








  public long nextLong(long n)
  {
    if (n <= 0L) {
      throw new IllegalArgumentException("n must be positive");
    }
    





    long offset = 0L;
    while (n >= 2147483647L) {
      int bits = next(2);
      long half = n >>> 1;
      long nextn = (bits & 0x2) == 0 ? half : n - half;
      if ((bits & 0x1) == 0) {
        offset += n - nextn;
      }
      n = nextn;
    }
    return offset + nextInt((int)n);
  }
  









  public long nextLong(long least, long bound)
  {
    if (least >= bound) {
      throw new IllegalArgumentException();
    }
    return nextLong(bound - least) + least;
  }
  








  public double nextDouble(double n)
  {
    if (n <= 0.0D) {
      throw new IllegalArgumentException("n must be positive");
    }
    return nextDouble() * n;
  }
  









  public double nextDouble(double least, double bound)
  {
    if (least >= bound) {
      throw new IllegalArgumentException();
    }
    return nextDouble() * (bound - least) + least;
  }
}
