package net.minecraft.util.org.apache.commons.lang3.reflect;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Modifier;
import net.minecraft.util.org.apache.commons.lang3.ArrayUtils;
import net.minecraft.util.org.apache.commons.lang3.ClassUtils;




































































public class ConstructorUtils
{
  public static <T> T invokeConstructor(Class<T> cls, Object... args)
    throws NoSuchMethodException, IllegalAccessException, InvocationTargetException, InstantiationException
  {
    if (args == null) {
      args = ArrayUtils.EMPTY_OBJECT_ARRAY;
    }
    Class<?>[] parameterTypes = new Class[args.length];
    for (int i = 0; i < args.length; i++) {
      parameterTypes[i] = args[i].getClass();
    }
    return invokeConstructor(cls, args, parameterTypes);
  }
  



















  public static <T> T invokeConstructor(Class<T> cls, Object[] args, Class<?>[] parameterTypes)
    throws NoSuchMethodException, IllegalAccessException, InvocationTargetException, InstantiationException
  {
    if (parameterTypes == null) {
      parameterTypes = ArrayUtils.EMPTY_CLASS_ARRAY;
    }
    if (args == null) {
      args = ArrayUtils.EMPTY_OBJECT_ARRAY;
    }
    Constructor<T> ctor = getMatchingAccessibleConstructor(cls, parameterTypes);
    if (ctor == null) {
      throw new NoSuchMethodException("No such accessible constructor on object: " + cls.getName());
    }
    
    return ctor.newInstance(args);
  }
  


















  public static <T> T invokeExactConstructor(Class<T> cls, Object... args)
    throws NoSuchMethodException, IllegalAccessException, InvocationTargetException, InstantiationException
  {
    if (args == null) {
      args = ArrayUtils.EMPTY_OBJECT_ARRAY;
    }
    int arguments = args.length;
    Class<?>[] parameterTypes = new Class[arguments];
    for (int i = 0; i < arguments; i++) {
      parameterTypes[i] = args[i].getClass();
    }
    return invokeExactConstructor(cls, args, parameterTypes);
  }
  



















  public static <T> T invokeExactConstructor(Class<T> cls, Object[] args, Class<?>[] parameterTypes)
    throws NoSuchMethodException, IllegalAccessException, InvocationTargetException, InstantiationException
  {
    if (args == null) {
      args = ArrayUtils.EMPTY_OBJECT_ARRAY;
    }
    if (parameterTypes == null) {
      parameterTypes = ArrayUtils.EMPTY_CLASS_ARRAY;
    }
    Constructor<T> ctor = getAccessibleConstructor(cls, parameterTypes);
    if (ctor == null) {
      throw new NoSuchMethodException("No such accessible constructor on object: " + cls.getName());
    }
    
    return ctor.newInstance(args);
  }
  













  public static <T> Constructor<T> getAccessibleConstructor(Class<T> cls, Class<?>... parameterTypes)
  {
    try
    {
      return getAccessibleConstructor(cls.getConstructor(parameterTypes));
    } catch (NoSuchMethodException e) {}
    return null;
  }
  










  public static <T> Constructor<T> getAccessibleConstructor(Constructor<T> ctor)
  {
    return (MemberUtils.isAccessible(ctor)) && (Modifier.isPublic(ctor.getDeclaringClass().getModifiers())) ? ctor : null;
  }
  



















  public static <T> Constructor<T> getMatchingAccessibleConstructor(Class<T> cls, Class<?>... parameterTypes)
  {
    try
    {
      Constructor<T> ctor = cls.getConstructor(parameterTypes);
      MemberUtils.setAccessibleWorkaround(ctor);
      return ctor;
    }
    catch (NoSuchMethodException e) {
      Constructor<T> result = null;
      



      Constructor<?>[] ctors = cls.getConstructors();
      

      for (Constructor<?> ctor : ctors)
      {
        if (ClassUtils.isAssignable(parameterTypes, ctor.getParameterTypes(), true))
        {
          ctor = getAccessibleConstructor(ctor);
          if (ctor != null) {
            MemberUtils.setAccessibleWorkaround(ctor);
            if ((result == null) || (MemberUtils.compareParameterTypes(ctor.getParameterTypes(), result.getParameterTypes(), parameterTypes) < 0))
            {



              Constructor<T> constructor = ctor;
              result = constructor;
            }
          }
        }
      }
      return result;
    }
  }
}
