package net.minecraft.util.org.apache.commons.lang3.time;










public class StopWatch
{
  private static final long NANO_2_MILLIS = 1000000L;
  








  private static final int STATE_UNSTARTED = 0;
  








  private static final int STATE_RUNNING = 1;
  







  private static final int STATE_STOPPED = 2;
  







  private static final int STATE_SUSPENDED = 3;
  







  private static final int STATE_UNSPLIT = 10;
  







  private static final int STATE_SPLIT = 11;
  







  private int runningState = 0;
  



  private int splitState = 10;
  








  private long startTime;
  








  private long startTimeMillis;
  







  private long stopTime;
  








  public void start()
  {
    if (runningState == 2) {
      throw new IllegalStateException("Stopwatch must be reset before being restarted. ");
    }
    if (runningState != 0) {
      throw new IllegalStateException("Stopwatch already started. ");
    }
    startTime = System.nanoTime();
    startTimeMillis = System.currentTimeMillis();
    runningState = 1;
  }
  











  public void stop()
  {
    if ((runningState != 1) && (runningState != 3)) {
      throw new IllegalStateException("Stopwatch is not running. ");
    }
    if (runningState == 1) {
      stopTime = System.nanoTime();
    }
    runningState = 2;
  }
  








  public void reset()
  {
    runningState = 0;
    splitState = 10;
  }
  












  public void split()
  {
    if (runningState != 1) {
      throw new IllegalStateException("Stopwatch is not running. ");
    }
    stopTime = System.nanoTime();
    splitState = 11;
  }
  












  public void unsplit()
  {
    if (splitState != 11) {
      throw new IllegalStateException("Stopwatch has not been split. ");
    }
    splitState = 10;
  }
  












  public void suspend()
  {
    if (runningState != 1) {
      throw new IllegalStateException("Stopwatch must be running to suspend. ");
    }
    stopTime = System.nanoTime();
    runningState = 3;
  }
  












  public void resume()
  {
    if (runningState != 3) {
      throw new IllegalStateException("Stopwatch must be suspended to resume. ");
    }
    startTime += System.nanoTime() - stopTime;
    runningState = 1;
  }
  











  public long getTime()
  {
    return getNanoTime() / 1000000L;
  }
  











  public long getNanoTime()
  {
    if ((runningState == 2) || (runningState == 3))
      return stopTime - startTime;
    if (runningState == 0)
      return 0L;
    if (runningState == 1) {
      return System.nanoTime() - startTime;
    }
    throw new RuntimeException("Illegal running state has occured. ");
  }
  














  public long getSplitTime()
  {
    return getSplitNanoTime() / 1000000L;
  }
  













  public long getSplitNanoTime()
  {
    if (splitState != 11) {
      throw new IllegalStateException("Stopwatch must be split to get the split time. ");
    }
    return stopTime - startTime;
  }
  







  public long getStartTime()
  {
    if (runningState == 0) {
      throw new IllegalStateException("Stopwatch has not been started");
    }
    
    return startTimeMillis;
  }
  











  public String toString()
  {
    return DurationFormatUtils.formatDurationHMS(getTime());
  }
  











  public String toSplitString()
  {
    return DurationFormatUtils.formatDurationHMS(getSplitTime());
  }
}
