package org.apache.commons.lang;

import java.io.Serializable;











































public final class CharRange
  implements Serializable
{
  private static final long serialVersionUID = 8270183163158333422L;
  private final char start;
  private final char end;
  private final boolean negated;
  private transient String iToString;
  
  public CharRange(char ch)
  {
    this(ch, ch, false);
  }
  








  public CharRange(char ch, boolean negated)
  {
    this(ch, ch, negated);
  }
  





  public CharRange(char start, char end)
  {
    this(start, end, false);
  }
  














  public CharRange(char start, char end, boolean negated)
  {
    if (start > end) {
      char temp = start;
      start = end;
      end = temp;
    }
    
    this.start = start;
    this.end = end;
    this.negated = negated;
  }
  






  public char getStart()
  {
    return start;
  }
  




  public char getEnd()
  {
    return end;
  }
  







  public boolean isNegated()
  {
    return negated;
  }
  







  public boolean contains(char ch)
  {
    return ((ch >= start) && (ch <= end)) != negated;
  }
  







  public boolean contains(CharRange range)
  {
    if (range == null) {
      throw new IllegalArgumentException("The Range must not be null");
    }
    if (negated) {
      if (negated) {
        return (start >= start) && (end <= end);
      }
      return (end < start) || (start > end);
    }
    
    if (negated) {
      return (start == 0) && (end == 65535);
    }
    return (start <= start) && (end >= end);
  }
  










  public boolean equals(Object obj)
  {
    if (obj == this) {
      return true;
    }
    if (!(obj instanceof CharRange)) {
      return false;
    }
    CharRange other = (CharRange)obj;
    return (start == start) && (end == end) && (negated == negated);
  }
  




  public int hashCode()
  {
    return 'S' + start + '\007' * end + (negated ? 1 : 0);
  }
  




  public String toString()
  {
    if (iToString == null) {
      StringBuffer buf = new StringBuffer(4);
      if (isNegated()) {
        buf.append('^');
      }
      buf.append(start);
      if (start != end) {
        buf.append('-');
        buf.append(end);
      }
      iToString = buf.toString();
    }
    return iToString;
  }
}
