package org.apache.commons.lang;









class IntHashMap
{
  private transient Entry[] table;
  







  private transient int count;
  






  private int threshold;
  






  private float loadFactor;
  







  private static class Entry
  {
    int hash;
    






    int key;
    






    Object value;
    






    Entry next;
    







    protected Entry(int hash, int key, Object value, Entry next)
    {
      this.hash = hash;
      this.key = key;
      this.value = value;
      this.next = next;
    }
  }
  



  public IntHashMap()
  {
    this(20, 0.75F);
  }
  







  public IntHashMap(int initialCapacity)
  {
    this(initialCapacity, 0.75F);
  }
  









  public IntHashMap(int initialCapacity, float loadFactor)
  {
    if (initialCapacity < 0) {
      throw new IllegalArgumentException("Illegal Capacity: " + initialCapacity);
    }
    if (loadFactor <= 0.0F) {
      throw new IllegalArgumentException("Illegal Load: " + loadFactor);
    }
    if (initialCapacity == 0) {
      initialCapacity = 1;
    }
    
    this.loadFactor = loadFactor;
    table = new Entry[initialCapacity];
    threshold = ((int)(initialCapacity * loadFactor));
  }
  




  public int size()
  {
    return count;
  }
  





  public boolean isEmpty()
  {
    return count == 0;
  }
  

















  public boolean contains(Object value)
  {
    if (value == null) {
      throw new NullPointerException();
    }
    
    Entry[] tab = table;
    Entry e; for (int i = tab.length; i-- > 0; 
        e != null) { e = tab[i]; continue;
      if (value.equals(value)) {
        return true;
      }
      e = next;
    }
    



    return false;
  }
  











  public boolean containsValue(Object value)
  {
    return contains(value);
  }
  








  public boolean containsKey(int key)
  {
    Entry[] tab = table;
    int hash = key;
    int index = (hash & 0x7FFFFFFF) % tab.length;
    for (Entry e = tab[index]; e != null; e = next) {
      if (hash == hash) {
        return true;
      }
    }
    return false;
  }
  








  public Object get(int key)
  {
    Entry[] tab = table;
    int hash = key;
    int index = (hash & 0x7FFFFFFF) % tab.length;
    for (Entry e = tab[index]; e != null; e = next) {
      if (hash == hash) {
        return value;
      }
    }
    return null;
  }
  








  protected void rehash()
  {
    int oldCapacity = table.length;
    Entry[] oldMap = table;
    
    int newCapacity = oldCapacity * 2 + 1;
    Entry[] newMap = new Entry[newCapacity];
    
    threshold = ((int)(newCapacity * loadFactor));
    table = newMap;
    Entry old;
    for (int i = oldCapacity; i-- > 0; 
        old != null) { old = oldMap[i]; continue;
      Entry e = old;
      old = next;
      
      int index = (hash & 0x7FFFFFFF) % newCapacity;
      next = newMap[index];
      newMap[index] = e;
    }
  }
  
















  public Object put(int key, Object value)
  {
    Entry[] tab = table;
    int hash = key;
    int index = (hash & 0x7FFFFFFF) % tab.length;
    for (Entry e = tab[index]; e != null; e = next) {
      if (hash == hash) {
        Object old = value;
        value = value;
        return old;
      }
    }
    
    if (count >= threshold)
    {
      rehash();
      
      tab = table;
      index = (hash & 0x7FFFFFFF) % tab.length;
    }
    

    Entry e = new Entry(hash, key, value, tab[index]);
    tab[index] = e;
    count += 1;
    return null;
  }
  










  public Object remove(int key)
  {
    Entry[] tab = table;
    int hash = key;
    int index = (hash & 0x7FFFFFFF) % tab.length;
    Entry e = tab[index]; for (Entry prev = null; e != null; e = next) {
      if (hash == hash) {
        if (prev != null) {
          next = next;
        } else {
          tab[index] = next;
        }
        count -= 1;
        Object oldValue = value;
        value = null;
        return oldValue;
      }
      prev = e;
    }
    










    return null;
  }
  


  public synchronized void clear()
  {
    Entry[] tab = table;
    int index = tab.length;
    do { tab[index] = null;index--;
    } while (index >= 0);
    

    count = 0;
  }
}
