package org.apache.logging.log4j.core;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import org.apache.logging.log4j.Level;
import org.apache.logging.log4j.Marker;
import org.apache.logging.log4j.core.config.Configuration;
import org.apache.logging.log4j.core.config.ConfigurationMonitor;
import org.apache.logging.log4j.core.config.LoggerConfig;
import org.apache.logging.log4j.core.filter.CompositeFilter;
import org.apache.logging.log4j.message.Message;
import org.apache.logging.log4j.message.MessageFactory;
import org.apache.logging.log4j.message.SimpleMessage;
import org.apache.logging.log4j.spi.AbstractLogger;































public class Logger
  extends AbstractLogger
{
  protected volatile PrivateConfig config;
  private final LoggerContext context;
  
  protected Logger(LoggerContext context, String name, MessageFactory messageFactory)
  {
    super(name, messageFactory);
    this.context = context;
    config = new PrivateConfig(context.getConfiguration(), this);
  }
  




  public Logger getParent()
  {
    LoggerConfig lc = config.loggerConfig.getName().equals(getName()) ? config.loggerConfig.getParent() : config.loggerConfig;
    
    if (lc == null) {
      return null;
    }
    if (context.hasLogger(lc.getName())) {
      return context.getLogger(lc.getName(), getMessageFactory());
    }
    return new Logger(context, lc.getName(), getMessageFactory());
  }
  



  public LoggerContext getContext()
  {
    return context;
  }
  



  public synchronized void setLevel(Level level)
  {
    if (level != null) {
      config = new PrivateConfig(config, level);
    }
  }
  



  public Level getLevel()
  {
    return config.level;
  }
  
  public void log(Marker marker, String fqcn, Level level, Message data, Throwable t)
  {
    if (data == null) {
      data = new SimpleMessage("");
    }
    config.config.getConfigurationMonitor().checkConfiguration();
    config.loggerConfig.log(getName(), marker, fqcn, level, data, t);
  }
  
  public boolean isEnabled(Level level, Marker marker, String msg)
  {
    return config.filter(level, marker, msg);
  }
  
  public boolean isEnabled(Level level, Marker marker, String msg, Throwable t)
  {
    return config.filter(level, marker, msg, t);
  }
  
  public boolean isEnabled(Level level, Marker marker, String msg, Object... p1)
  {
    return config.filter(level, marker, msg, p1);
  }
  
  public boolean isEnabled(Level level, Marker marker, Object msg, Throwable t)
  {
    return config.filter(level, marker, msg, t);
  }
  
  public boolean isEnabled(Level level, Marker marker, Message msg, Throwable t)
  {
    return config.filter(level, marker, msg, t);
  }
  



  public void addAppender(Appender appender)
  {
    config.config.addLoggerAppender(this, appender);
  }
  



  public void removeAppender(Appender appender)
  {
    config.loggerConfig.removeAppender(appender.getName());
  }
  



  public Map<String, Appender> getAppenders()
  {
    return config.loggerConfig.getAppenders();
  }
  



  public Iterator<Filter> getFilters()
  {
    Filter filter = config.loggerConfig.getFilter();
    if (filter == null)
      return new ArrayList().iterator();
    if ((filter instanceof CompositeFilter)) {
      return ((CompositeFilter)filter).iterator();
    }
    List<Filter> filters = new ArrayList();
    filters.add(filter);
    return filters.iterator();
  }
  




  public int filterCount()
  {
    Filter filter = config.loggerConfig.getFilter();
    if (filter == null)
      return 0;
    if ((filter instanceof CompositeFilter)) {
      return ((CompositeFilter)filter).size();
    }
    return 1;
  }
  



  public void addFilter(Filter filter)
  {
    config.config.addLoggerFilter(this, filter);
  }
  




  public boolean isAdditive()
  {
    return config.loggerConfig.isAdditive();
  }
  




  public void setAdditive(boolean additive)
  {
    config.config.setLoggerAdditive(this, additive);
  }
  











  void updateConfiguration(Configuration config)
  {
    this.config = new PrivateConfig(config, this);
  }
  

  protected class PrivateConfig
  {
    public final LoggerConfig loggerConfig;
    
    public final Configuration config;
    private final Level level;
    private final int intLevel;
    private final Logger logger;
    
    public PrivateConfig(Configuration config, Logger logger)
    {
      this.config = config;
      loggerConfig = config.getLoggerConfig(getName());
      level = loggerConfig.getLevel();
      intLevel = level.intLevel();
      this.logger = logger;
    }
    
    public PrivateConfig(PrivateConfig pc, Level level) {
      config = config;
      loggerConfig = loggerConfig;
      this.level = level;
      intLevel = this.level.intLevel();
      logger = logger;
    }
    
    public PrivateConfig(PrivateConfig pc, LoggerConfig lc) {
      config = config;
      loggerConfig = lc;
      level = lc.getLevel();
      intLevel = level.intLevel();
      logger = logger;
    }
    
    public void logEvent(LogEvent event)
    {
      config.getConfigurationMonitor().checkConfiguration();
      loggerConfig.log(event);
    }
    
    boolean filter(Level level, Marker marker, String msg) {
      config.getConfigurationMonitor().checkConfiguration();
      Filter filter = config.getFilter();
      if (filter != null) {
        Filter.Result r = filter.filter(logger, level, marker, msg, new Object[0]);
        if (r != Filter.Result.NEUTRAL) {
          return r == Filter.Result.ACCEPT;
        }
      }
      
      return intLevel >= level.intLevel();
    }
    
    boolean filter(Level level, Marker marker, String msg, Throwable t) {
      config.getConfigurationMonitor().checkConfiguration();
      Filter filter = config.getFilter();
      if (filter != null) {
        Filter.Result r = filter.filter(logger, level, marker, msg, t);
        if (r != Filter.Result.NEUTRAL) {
          return r == Filter.Result.ACCEPT;
        }
      }
      
      return intLevel >= level.intLevel();
    }
    
    boolean filter(Level level, Marker marker, String msg, Object... p1) {
      config.getConfigurationMonitor().checkConfiguration();
      Filter filter = config.getFilter();
      if (filter != null) {
        Filter.Result r = filter.filter(logger, level, marker, msg, p1);
        if (r != Filter.Result.NEUTRAL) {
          return r == Filter.Result.ACCEPT;
        }
      }
      
      return intLevel >= level.intLevel();
    }
    
    boolean filter(Level level, Marker marker, Object msg, Throwable t) {
      config.getConfigurationMonitor().checkConfiguration();
      Filter filter = config.getFilter();
      if (filter != null) {
        Filter.Result r = filter.filter(logger, level, marker, msg, t);
        if (r != Filter.Result.NEUTRAL) {
          return r == Filter.Result.ACCEPT;
        }
      }
      
      return intLevel >= level.intLevel();
    }
    
    boolean filter(Level level, Marker marker, Message msg, Throwable t) {
      config.getConfigurationMonitor().checkConfiguration();
      Filter filter = config.getFilter();
      if (filter != null) {
        Filter.Result r = filter.filter(logger, level, marker, msg, t);
        if (r != Filter.Result.NEUTRAL) {
          return r == Filter.Result.ACCEPT;
        }
      }
      
      return intLevel >= level.intLevel();
    }
  }
  




  public String toString()
  {
    String nameLevel = "" + getName() + ":" + getLevel();
    if (context == null) {
      return nameLevel;
    }
    String contextName = context.getName();
    return nameLevel + " in " + contextName;
  }
}
