package org.apache.logging.log4j.core.appender;

import java.io.Serializable;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReadWriteLock;
import java.util.concurrent.locks.ReentrantReadWriteLock;
import org.apache.logging.log4j.Logger;
import org.apache.logging.log4j.core.Filter;
import org.apache.logging.log4j.core.Layout;
import org.apache.logging.log4j.core.LogEvent;






























public abstract class AbstractOutputStreamAppender
  extends AbstractAppender
{
  protected final boolean immediateFlush;
  private volatile OutputStreamManager manager;
  private final ReadWriteLock rwLock = new ReentrantReadWriteLock();
  private final Lock readLock = rwLock.readLock();
  private final Lock writeLock = rwLock.writeLock();
  









  protected AbstractOutputStreamAppender(String name, Layout<? extends Serializable> layout, Filter filter, boolean ignoreExceptions, boolean immediateFlush, OutputStreamManager manager)
  {
    super(name, filter, layout, ignoreExceptions);
    this.manager = manager;
    this.immediateFlush = immediateFlush;
  }
  
  protected OutputStreamManager getManager() {
    return manager;
  }
  
  protected void replaceManager(OutputStreamManager newManager)
  {
    writeLock.lock();
    try {
      OutputStreamManager old = manager;
      manager = newManager;
      old.release();
    } finally {
      writeLock.unlock();
    }
  }
  

  public void start()
  {
    if (getLayout() == null) {
      LOGGER.error("No layout set for the appender named [" + getName() + "].");
    }
    if (manager == null) {
      LOGGER.error("No OutputStreamManager set for the appender named [" + getName() + "].");
    }
    super.start();
  }
  
  public void stop()
  {
    super.stop();
    manager.release();
  }
  







  public void append(LogEvent event)
  {
    readLock.lock();
    try {
      byte[] bytes = getLayout().toByteArray(event);
      if (bytes.length > 0) {
        manager.write(bytes);
        if ((immediateFlush) || (event.isEndOfBatch())) {
          manager.flush();
        }
      }
    } catch (AppenderLoggingException ex) {
      error("Unable to write to stream " + manager.getName() + " for appender " + getName());
      throw ex;
    } finally {
      readLock.unlock();
    }
  }
}
