package org.apache.logging.log4j.core.appender.db.jdbc;

import org.apache.logging.log4j.Logger;
import org.apache.logging.log4j.core.config.Configuration;
import org.apache.logging.log4j.core.config.plugins.Plugin;
import org.apache.logging.log4j.core.config.plugins.PluginAttribute;
import org.apache.logging.log4j.core.config.plugins.PluginConfiguration;
import org.apache.logging.log4j.core.config.plugins.PluginFactory;
import org.apache.logging.log4j.core.helpers.Booleans;
import org.apache.logging.log4j.core.helpers.Strings;
import org.apache.logging.log4j.core.layout.PatternLayout;
import org.apache.logging.log4j.status.StatusLogger;



















@Plugin(name="Column", category="Core", printObject=true)
public final class ColumnConfig
{
  private static final Logger LOGGER = ;
  
  private final String columnName;
  private final PatternLayout layout;
  private final String literalValue;
  private final boolean eventTimestamp;
  private final boolean unicode;
  private final boolean clob;
  
  private ColumnConfig(String columnName, PatternLayout layout, String literalValue, boolean eventDate, boolean unicode, boolean clob)
  {
    this.columnName = columnName;
    this.layout = layout;
    this.literalValue = literalValue;
    eventTimestamp = eventDate;
    this.unicode = unicode;
    this.clob = clob;
  }
  
  public String getColumnName() {
    return columnName;
  }
  
  public PatternLayout getLayout() {
    return layout;
  }
  
  public String getLiteralValue() {
    return literalValue;
  }
  
  public boolean isEventTimestamp() {
    return eventTimestamp;
  }
  
  public boolean isUnicode() {
    return unicode;
  }
  
  public boolean isClob() {
    return clob;
  }
  
  public String toString()
  {
    return "{ name=" + columnName + ", layout=" + layout + ", literal=" + literalValue + ", timestamp=" + eventTimestamp + " }";
  }
  























  @PluginFactory
  public static ColumnConfig createColumnConfig(@PluginConfiguration Configuration config, @PluginAttribute("name") String name, @PluginAttribute("pattern") String pattern, @PluginAttribute("literal") String literalValue, @PluginAttribute("isEventTimestamp") String eventTimestamp, @PluginAttribute("isUnicode") String unicode, @PluginAttribute("isClob") String clob)
  {
    if (Strings.isEmpty(name)) {
      LOGGER.error("The column config is not valid because it does not contain a column name.");
      return null;
    }
    
    boolean isPattern = Strings.isNotEmpty(pattern);
    boolean isLiteralValue = Strings.isNotEmpty(literalValue);
    boolean isEventTimestamp = Boolean.parseBoolean(eventTimestamp);
    boolean isUnicode = Booleans.parseBoolean(unicode, true);
    boolean isClob = Boolean.parseBoolean(clob);
    
    if (((isPattern) && (isLiteralValue)) || ((isPattern) && (isEventTimestamp)) || ((isLiteralValue) && (isEventTimestamp))) {
      LOGGER.error("The pattern, literal, and isEventTimestamp attributes are mutually exclusive.");
      return null;
    }
    
    if (isEventTimestamp) {
      return new ColumnConfig(name, null, null, true, false, false);
    }
    if (isLiteralValue) {
      return new ColumnConfig(name, null, literalValue, false, false, false);
    }
    if (isPattern) {
      return new ColumnConfig(name, PatternLayout.createLayout(pattern, config, null, null, "false"), null, false, isUnicode, isClob);
    }
    



    LOGGER.error("To configure a column you must specify a pattern or literal or set isEventDate to true.");
    return null;
  }
}
