package org.apache.logging.log4j.core.appender.db.jpa;

import java.lang.reflect.Constructor;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;
import org.apache.logging.log4j.core.LogEvent;
import org.apache.logging.log4j.core.appender.AppenderLoggingException;
import org.apache.logging.log4j.core.appender.ManagerFactory;
import org.apache.logging.log4j.core.appender.db.AbstractDatabaseManager;
import org.apache.logging.log4j.core.appender.db.AbstractDatabaseManager.AbstractFactoryData;


















public final class JPADatabaseManager
  extends AbstractDatabaseManager
{
  private static final JPADatabaseManagerFactory FACTORY = new JPADatabaseManagerFactory(null);
  
  private final String entityClassName;
  
  private final Constructor<? extends AbstractLogEventWrapperEntity> entityConstructor;
  
  private final String persistenceUnitName;
  
  private EntityManagerFactory entityManagerFactory;
  
  private JPADatabaseManager(String name, int bufferSize, Class<? extends AbstractLogEventWrapperEntity> entityClass, Constructor<? extends AbstractLogEventWrapperEntity> entityConstructor, String persistenceUnitName)
  {
    super(name, bufferSize);
    entityClassName = entityClass.getName();
    this.entityConstructor = entityConstructor;
    this.persistenceUnitName = persistenceUnitName;
  }
  
  protected void connectInternal()
  {
    entityManagerFactory = Persistence.createEntityManagerFactory(persistenceUnitName);
  }
  
  protected void disconnectInternal()
  {
    if ((entityManagerFactory != null) && (entityManagerFactory.isOpen())) {
      entityManagerFactory.close();
    }
  }
  
  protected void writeInternal(LogEvent event)
  {
    if ((!isConnected()) || (entityManagerFactory == null)) {
      throw new AppenderLoggingException("Cannot write logging event; JPA manager not connected to the database.");
    }
    
    AbstractLogEventWrapperEntity entity;
    try
    {
      entity = (AbstractLogEventWrapperEntity)entityConstructor.newInstance(new Object[] { event });
    } catch (Exception e) {
      throw new AppenderLoggingException("Failed to instantiate entity class [" + entityClassName + "].", e);
    }
    
    EntityManager entityManager = null;
    EntityTransaction transaction = null;
    try {
      entityManager = entityManagerFactory.createEntityManager();
      transaction = entityManager.getTransaction();
      transaction.begin();
      entityManager.persist(entity);
      transaction.commit();

    }
    catch (Exception e)
    {
      throw new AppenderLoggingException("Failed to insert record for log event in JDBC manager: " + e.getMessage(), e);
    }
    finally {
      if ((entityManager != null) && (entityManager.isOpen())) {
        entityManager.close();
      }
    }
  }
  
















  public static JPADatabaseManager getJPADatabaseManager(String name, int bufferSize, Class<? extends AbstractLogEventWrapperEntity> entityClass, Constructor<? extends AbstractLogEventWrapperEntity> entityConstructor, String persistenceUnitName)
  {
    return (JPADatabaseManager)AbstractDatabaseManager.getManager(name, new FactoryData(bufferSize, entityClass, entityConstructor, persistenceUnitName), FACTORY);
  }
  

  private static final class FactoryData
    extends AbstractDatabaseManager.AbstractFactoryData
  {
    private final Class<? extends AbstractLogEventWrapperEntity> entityClass;
    
    private final Constructor<? extends AbstractLogEventWrapperEntity> entityConstructor;
    
    private final String persistenceUnitName;
    

    protected FactoryData(int bufferSize, Class<? extends AbstractLogEventWrapperEntity> entityClass, Constructor<? extends AbstractLogEventWrapperEntity> entityConstructor, String persistenceUnitName)
    {
      super();
      
      this.entityClass = entityClass;
      this.entityConstructor = entityConstructor;
      this.persistenceUnitName = persistenceUnitName;
    }
  }
  

  private static final class JPADatabaseManagerFactory
    implements ManagerFactory<JPADatabaseManager, JPADatabaseManager.FactoryData>
  {
    public JPADatabaseManager createManager(String name, JPADatabaseManager.FactoryData data)
    {
      return new JPADatabaseManager(name, data.getBufferSize(), JPADatabaseManager.FactoryData.access$100(data), JPADatabaseManager.FactoryData.access$200(data), JPADatabaseManager.FactoryData.access$300(data), null);
    }
  }
}
