package org.apache.logging.log4j.core.async;

import com.lmax.disruptor.EventFactory;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import org.apache.logging.log4j.Level;
import org.apache.logging.log4j.Marker;
import org.apache.logging.log4j.ThreadContext.ContextStack;
import org.apache.logging.log4j.core.LogEvent;
import org.apache.logging.log4j.core.config.Property;
import org.apache.logging.log4j.core.lookup.StrSubstitutor;
import org.apache.logging.log4j.message.Message;
import org.apache.logging.log4j.message.SimpleMessage;






















public class RingBufferLogEvent
  implements LogEvent
{
  private static final long serialVersionUID = 8462119088943934758L;
  
  private static class Factory
    implements EventFactory<RingBufferLogEvent>
  {
    public RingBufferLogEvent newInstance()
    {
      return new RingBufferLogEvent();
    }
  }
  

  public static final Factory FACTORY = new Factory(null);
  
  private AsyncLogger asyncLogger;
  
  private String loggerName;
  
  private Marker marker;
  
  private String fqcn;
  
  private Level level;
  private Message message;
  private Throwable thrown;
  private Map<String, String> contextMap;
  private ThreadContext.ContextStack contextStack;
  private String threadName;
  private StackTraceElement location;
  private long currentTimeMillis;
  private boolean endOfBatch;
  private boolean includeLocation;
  
  public void setValues(AsyncLogger asyncLogger, String loggerName, Marker marker, String fqcn, Level level, Message data, Throwable t, Map<String, String> map, ThreadContext.ContextStack contextStack, String threadName, StackTraceElement location, long currentTimeMillis)
  {
    this.asyncLogger = asyncLogger;
    this.loggerName = loggerName;
    this.marker = marker;
    this.fqcn = fqcn;
    this.level = level;
    message = data;
    thrown = t;
    contextMap = map;
    this.contextStack = contextStack;
    this.threadName = threadName;
    this.location = location;
    this.currentTimeMillis = currentTimeMillis;
  }
  






  public void execute(boolean endOfBatch)
  {
    this.endOfBatch = endOfBatch;
    asyncLogger.actualAsyncLog(this);
  }
  







  public boolean isEndOfBatch()
  {
    return endOfBatch;
  }
  
  public void setEndOfBatch(boolean endOfBatch)
  {
    this.endOfBatch = endOfBatch;
  }
  
  public boolean isIncludeLocation()
  {
    return includeLocation;
  }
  
  public void setIncludeLocation(boolean includeLocation)
  {
    this.includeLocation = includeLocation;
  }
  
  public String getLoggerName()
  {
    return loggerName;
  }
  
  public Marker getMarker()
  {
    return marker;
  }
  
  public String getFQCN()
  {
    return fqcn;
  }
  
  public Level getLevel()
  {
    return level;
  }
  
  public Message getMessage()
  {
    if (message == null) {
      message = new SimpleMessage("");
    }
    return message;
  }
  
  public Throwable getThrown()
  {
    return thrown;
  }
  
  public Map<String, String> getContextMap()
  {
    return contextMap;
  }
  
  public ThreadContext.ContextStack getContextStack()
  {
    return contextStack;
  }
  
  public String getThreadName()
  {
    return threadName;
  }
  
  public StackTraceElement getSource()
  {
    return location;
  }
  
  public long getMillis()
  {
    return currentTimeMillis;
  }
  









  public void mergePropertiesIntoContextMap(Map<Property, Boolean> properties, StrSubstitutor strSubstitutor)
  {
    if (properties == null) {
      return;
    }
    
    Map<String, String> map = contextMap == null ? new HashMap() : new HashMap(contextMap);
    

    for (Map.Entry<Property, Boolean> entry : properties.entrySet()) {
      Property prop = (Property)entry.getKey();
      if (!map.containsKey(prop.getName()))
      {

        String value = ((Boolean)entry.getValue()).booleanValue() ? strSubstitutor.replace(prop.getValue()) : prop.getValue();
        
        map.put(prop.getName(), value);
      } }
    contextMap = map;
  }
  



  public void clear()
  {
    setValues(null, null, null, null, null, null, null, null, null, null, null, 0L);
  }
}
