package org.apache.logging.log4j.core.config;

import org.apache.logging.log4j.Level;
import org.apache.logging.log4j.core.Appender;
import org.apache.logging.log4j.core.ErrorHandler;
import org.apache.logging.log4j.core.Filter;
import org.apache.logging.log4j.core.Filter.Result;
import org.apache.logging.log4j.core.LogEvent;
import org.apache.logging.log4j.core.appender.AppenderLoggingException;
import org.apache.logging.log4j.core.filter.AbstractFilterable;
import org.apache.logging.log4j.core.filter.Filterable;

















public class AppenderControl
  extends AbstractFilterable
{
  private final ThreadLocal<AppenderControl> recursive = new ThreadLocal();
  

  private final Appender appender;
  

  private final Level level;
  

  private final int intLevel;
  


  public AppenderControl(Appender appender, Level level, Filter filter)
  {
    super(filter);
    this.appender = appender;
    this.level = level;
    intLevel = (level == null ? Level.ALL.intLevel() : level.intLevel());
    startFilter();
  }
  



  public Appender getAppender()
  {
    return appender;
  }
  



  public void callAppender(LogEvent event)
  {
    if (getFilter() != null) {
      Filter.Result r = getFilter().filter(event);
      if (r == Filter.Result.DENY) {
        return;
      }
    }
    if ((level != null) && 
      (intLevel < event.getLevel().intLevel())) {
      return;
    }
    
    if (recursive.get() != null) {
      appender.getHandler().error("Recursive call to appender " + appender.getName());
      return;
    }
    try {
      recursive.set(this);
      
      if (!appender.isStarted()) {
        appender.getHandler().error("Attempted to append to non-started appender " + appender.getName());
        
        if (!appender.ignoreExceptions()) {
          throw new AppenderLoggingException("Attempted to append to non-started appender " + appender.getName());
        }
      }
      

      if (((appender instanceof Filterable)) && (((Filterable)appender).isFiltered(event))) {
        return;
      }
      try
      {
        appender.append(event);
      } catch (RuntimeException ex) {
        appender.getHandler().error("An exception occurred processing Appender " + appender.getName(), ex);
        if (!appender.ignoreExceptions()) {
          throw ex;
        }
      } catch (Exception ex) {
        appender.getHandler().error("An exception occurred processing Appender " + appender.getName(), ex);
        if (!appender.ignoreExceptions()) {
          throw new AppenderLoggingException(ex);
        }
      }
    } finally {
      recursive.set(null);
    }
  }
}
