package org.apache.logging.log4j.core.config;

import java.io.File;
import java.util.List;




























public class FileConfigurationMonitor
  implements ConfigurationMonitor
{
  private static final int MASK = 15;
  private static final int MIN_INTERVAL = 5;
  private static final int MILLIS_PER_SECOND = 1000;
  private final File file;
  private long lastModified;
  private final List<ConfigurationListener> listeners;
  private final int interval;
  private long nextCheck;
  private volatile int counter = 0;
  




  private final Reconfigurable reconfigurable;
  




  public FileConfigurationMonitor(Reconfigurable reconfigurable, File file, List<ConfigurationListener> listeners, int interval)
  {
    this.reconfigurable = reconfigurable;
    this.file = file;
    lastModified = file.lastModified();
    this.listeners = listeners;
    this.interval = ((interval < 5 ? 5 : interval) * 1000);
    nextCheck = (System.currentTimeMillis() + interval);
  }
  



  public void checkConfiguration()
  {
    if ((++counter & 0xF) == 0) {
      synchronized (this) {
        long current = System.currentTimeMillis();
        if (current >= nextCheck) {
          nextCheck = (current + interval);
          if (file.lastModified() > lastModified) {
            lastModified = file.lastModified();
            for (ConfigurationListener listener : listeners) {
              listener.onChange(reconfigurable);
            }
          }
        }
      }
    }
  }
}
