package org.apache.logging.log4j.core.filter;

import java.util.Iterator;
import org.apache.logging.log4j.Logger;
import org.apache.logging.log4j.core.Filter;
import org.apache.logging.log4j.core.Filter.Result;
import org.apache.logging.log4j.core.LifeCycle;
import org.apache.logging.log4j.core.LogEvent;
import org.apache.logging.log4j.status.StatusLogger;



















public abstract class AbstractFilterable
  implements Filterable
{
  protected static final Logger LOGGER = ;
  private volatile Filter filter;
  
  protected AbstractFilterable(Filter filter)
  {
    this.filter = filter;
  }
  



  protected AbstractFilterable() {}
  


  public Filter getFilter()
  {
    return filter;
  }
  




  public synchronized void addFilter(Filter filter)
  {
    if (this.filter == null) {
      this.filter = filter;
    } else if ((filter instanceof CompositeFilter)) {
      this.filter = ((CompositeFilter)this.filter).addFilter(filter);
    } else {
      Filter[] filters = { this.filter, filter };
      this.filter = CompositeFilter.createFilters(filters);
    }
  }
  




  public synchronized void removeFilter(Filter filter)
  {
    if (this.filter == filter) {
      this.filter = null;
    } else if ((filter instanceof CompositeFilter)) {
      CompositeFilter composite = (CompositeFilter)filter;
      composite = composite.removeFilter(filter);
      if (composite.size() > 1) {
        this.filter = composite;
      } else if (composite.size() == 1) {
        Iterator<Filter> iter = composite.iterator();
        this.filter = ((Filter)iter.next());
      } else {
        this.filter = null;
      }
    }
  }
  




  public boolean hasFilter()
  {
    return filter != null;
  }
  


  public void startFilter()
  {
    if ((filter != null) && ((filter instanceof LifeCycle))) {
      ((LifeCycle)filter).start();
    }
  }
  


  public void stopFilter()
  {
    if ((filter != null) && ((filter instanceof LifeCycle))) {
      ((LifeCycle)filter).stop();
    }
  }
  





  public boolean isFiltered(LogEvent event)
  {
    return (filter != null) && (filter.filter(event) == Filter.Result.DENY);
  }
}
