package org.apache.logging.log4j.core.helpers;

import java.lang.reflect.Array;





















public class CyclicBuffer<T>
{
  private final T[] ring;
  private int first = 0;
  private int last = 0;
  private int numElems = 0;
  

  private final Class<T> clazz;
  


  public CyclicBuffer(Class<T> clazz, int size)
    throws IllegalArgumentException
  {
    if (size < 1) {
      throw new IllegalArgumentException("The maxSize argument (" + size + ") is not a positive integer.");
    }
    ring = makeArray(clazz, size);
    this.clazz = clazz;
  }
  
  private T[] makeArray(Class<T> clazz, int size)
  {
    return (Object[])Array.newInstance(clazz, size);
  }
  



  public synchronized void add(T item)
  {
    ring[last] = item;
    if (++last == ring.length) {
      last = 0;
    }
    
    if (numElems < ring.length) {
      numElems += 1;
    } else if (++first == ring.length) {
      first = 0;
    }
  }
  



  public synchronized T[] removeAll()
  {
    T[] array = makeArray(clazz, numElems);
    int index = 0;
    while (numElems > 0) {
      numElems -= 1;
      array[(index++)] = ring[first];
      ring[first] = null;
      if (++first == ring.length) {
        first = 0;
      }
    }
    return array;
  }
  



  public boolean isEmpty()
  {
    return 0 == numElems;
  }
}
