package org.apache.logging.log4j.core.impl;

import java.io.InvalidObjectException;
import java.io.ObjectInputStream;
import java.io.Serializable;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.apache.logging.log4j.Level;
import org.apache.logging.log4j.Marker;
import org.apache.logging.log4j.ThreadContext;
import org.apache.logging.log4j.ThreadContext.ContextStack;
import org.apache.logging.log4j.core.LogEvent;
import org.apache.logging.log4j.core.config.Property;
import org.apache.logging.log4j.message.LoggerNameAwareMessage;
import org.apache.logging.log4j.message.Message;
import org.apache.logging.log4j.message.TimestampMessage;



















public class Log4jLogEvent
  implements LogEvent
{
  private static final long serialVersionUID = -1351367343806656055L;
  private static final String NOT_AVAIL = "?";
  private final String fqcnOfLogger;
  private final Marker marker;
  private final Level level;
  private final String name;
  private final Message message;
  private final long timestamp;
  private final ThrowableProxy throwable;
  private final Map<String, String> mdc;
  private final ThreadContext.ContextStack ndc;
  private String threadName = null;
  private StackTraceElement location;
  private boolean includeLocation;
  private boolean endOfBatch = false;
  


  public Log4jLogEvent(long timestamp)
  {
    this("", null, "", null, null, (ThrowableProxy)null, null, null, null, null, timestamp);
  }
  









  public Log4jLogEvent(String loggerName, Marker marker, String fqcn, Level level, Message message, Throwable t)
  {
    this(loggerName, marker, fqcn, level, message, null, t);
  }
  










  public Log4jLogEvent(String loggerName, Marker marker, String fqcn, Level level, Message message, List<Property> properties, Throwable t)
  {
    this(loggerName, marker, fqcn, level, message, t, createMap(properties), ThreadContext.getDepth() == 0 ? null : ThreadContext.cloneStack(), null, null, System.currentTimeMillis());
  }
  



















  public Log4jLogEvent(String loggerName, Marker marker, String fqcn, Level level, Message message, Throwable t, Map<String, String> mdc, ThreadContext.ContextStack ndc, String threadName, StackTraceElement location, long timestamp)
  {
    this(loggerName, marker, fqcn, level, message, t == null ? null : new ThrowableProxy(t), mdc, ndc, threadName, location, timestamp);
  }
  


















  public static Log4jLogEvent createEvent(String loggerName, Marker marker, String fqcn, Level level, Message message, ThrowableProxy t, Map<String, String> mdc, ThreadContext.ContextStack ndc, String threadName, StackTraceElement location, long timestamp)
  {
    return new Log4jLogEvent(loggerName, marker, fqcn, level, message, t, mdc, ndc, threadName, location, timestamp);
  }
  
















  private Log4jLogEvent(String loggerName, Marker marker, String fqcn, Level level, Message message, ThrowableProxy t, Map<String, String> mdc, ThreadContext.ContextStack ndc, String threadName, StackTraceElement location, long timestamp)
  {
    name = loggerName;
    this.marker = marker;
    fqcnOfLogger = fqcn;
    this.level = level;
    this.message = message;
    throwable = t;
    this.mdc = mdc;
    this.ndc = ndc;
    this.timestamp = ((message instanceof TimestampMessage) ? ((TimestampMessage)message).getTimestamp() : timestamp);
    this.threadName = threadName;
    this.location = location;
    if ((message != null) && ((message instanceof LoggerNameAwareMessage))) {
      ((LoggerNameAwareMessage)message).setLoggerName(name);
    }
  }
  
  private static Map<String, String> createMap(List<Property> properties) {
    Map<String, String> contextMap = ThreadContext.getImmutableContext();
    if ((contextMap == null) && ((properties == null) || (properties.size() == 0))) {
      return null;
    }
    if ((properties == null) || (properties.size() == 0)) {
      return contextMap;
    }
    Map<String, String> map = new HashMap(contextMap);
    
    for (Property prop : properties) {
      if (!map.containsKey(prop.getName())) {
        map.put(prop.getName(), prop.getValue());
      }
    }
    return Collections.unmodifiableMap(map);
  }
  




  public Level getLevel()
  {
    return level;
  }
  




  public String getLoggerName()
  {
    return name;
  }
  




  public Message getMessage()
  {
    return message;
  }
  




  public String getThreadName()
  {
    if (threadName == null) {
      threadName = Thread.currentThread().getName();
    }
    return threadName;
  }
  




  public long getMillis()
  {
    return timestamp;
  }
  




  public Throwable getThrown()
  {
    return throwable == null ? null : throwable.getThrowable();
  }
  



  public ThrowableProxy getThrownProxy()
  {
    return throwable;
  }
  





  public Marker getMarker()
  {
    return marker;
  }
  




  public String getFQCN()
  {
    return fqcnOfLogger;
  }
  




  public Map<String, String> getContextMap()
  {
    return mdc == null ? ThreadContext.EMPTY_MAP : mdc;
  }
  




  public ThreadContext.ContextStack getContextStack()
  {
    return ndc == null ? ThreadContext.EMPTY_STACK : ndc;
  }
  





  public StackTraceElement getSource()
  {
    if (location != null) {
      return location;
    }
    if ((fqcnOfLogger == null) || (!includeLocation)) {
      return null;
    }
    location = calcLocation(fqcnOfLogger);
    return location;
  }
  
  public static StackTraceElement calcLocation(String fqcnOfLogger) {
    if (fqcnOfLogger == null) {
      return null;
    }
    StackTraceElement[] stackTrace = Thread.currentThread().getStackTrace();
    boolean next = false;
    for (StackTraceElement element : stackTrace) {
      String className = element.getClassName();
      if (next) {
        if (!fqcnOfLogger.equals(className))
        {

          return element;
        }
      }
      else if (fqcnOfLogger.equals(className))
        next = true; else {
        if ("?".equals(className))
          break;
      }
    }
    return null;
  }
  
  public boolean isIncludeLocation()
  {
    return includeLocation;
  }
  
  public void setIncludeLocation(boolean includeLocation)
  {
    this.includeLocation = includeLocation;
  }
  
  public boolean isEndOfBatch()
  {
    return endOfBatch;
  }
  
  public void setEndOfBatch(boolean endOfBatch)
  {
    this.endOfBatch = endOfBatch;
  }
  



  protected Object writeReplace()
  {
    return new LogEventProxy(this, includeLocation);
  }
  
  public static Serializable serialize(Log4jLogEvent event, boolean includeLocation)
  {
    return new LogEventProxy(event, includeLocation);
  }
  
  public static Log4jLogEvent deserialize(Serializable event) {
    if (event == null) {
      throw new NullPointerException("Event cannot be null");
    }
    if ((event instanceof LogEventProxy)) {
      LogEventProxy proxy = (LogEventProxy)event;
      Log4jLogEvent result = new Log4jLogEvent(name, marker, fqcnOfLogger, level, message, throwable, mdc, ndc, threadName, location, timestamp);
      


      result.setEndOfBatch(isEndOfBatch);
      result.setIncludeLocation(isLocationRequired);
      return result;
    }
    throw new IllegalArgumentException("Event is not a serialized LogEvent: " + event.toString());
  }
  
  private void readObject(ObjectInputStream stream) throws InvalidObjectException {
    throw new InvalidObjectException("Proxy required");
  }
  
  public String toString()
  {
    StringBuilder sb = new StringBuilder();
    String n = name.isEmpty() ? "root" : name;
    sb.append("Logger=").append(n);
    sb.append(" Level=").append(level.name());
    sb.append(" Message=").append(message.getFormattedMessage());
    return sb.toString();
  }
  

  private static class LogEventProxy
    implements Serializable
  {
    private static final long serialVersionUID = -7139032940312647146L;
    private final String fqcnOfLogger;
    private final Marker marker;
    private final Level level;
    private final String name;
    private final Message message;
    private final long timestamp;
    private final ThrowableProxy throwable;
    private final Map<String, String> mdc;
    private final ThreadContext.ContextStack ndc;
    private final String threadName;
    private final StackTraceElement location;
    private final boolean isLocationRequired;
    private final boolean isEndOfBatch;
    
    public LogEventProxy(Log4jLogEvent event, boolean includeLocation)
    {
      fqcnOfLogger = fqcnOfLogger;
      marker = marker;
      level = level;
      name = name;
      message = message;
      timestamp = timestamp;
      throwable = throwable;
      mdc = mdc;
      ndc = ndc;
      location = (includeLocation ? event.getSource() : null);
      threadName = event.getThreadName();
      isLocationRequired = includeLocation;
      isEndOfBatch = endOfBatch;
    }
    



    protected Object readResolve()
    {
      Log4jLogEvent result = new Log4jLogEvent(name, marker, fqcnOfLogger, level, message, throwable, mdc, ndc, threadName, location, timestamp, null);
      

      result.setEndOfBatch(isEndOfBatch);
      result.setIncludeLocation(isLocationRequired);
      return result;
    }
  }
}
