package org.apache.logging.log4j.core.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;
import org.apache.logging.log4j.core.helpers.Constants;

























public final class ThrowableFormatOptions
{
  private static final int DEFAULT_LINES = 2147483647;
  protected static final ThrowableFormatOptions DEFAULT = new ThrowableFormatOptions();
  


  private static final String FULL = "full";
  


  private static final String NONE = "none";
  


  private static final String SHORT = "short";
  


  private final int lines;
  

  private final String separator;
  

  private final List<String> packages;
  

  public static final String CLASS_NAME = "short.className";
  

  public static final String METHOD_NAME = "short.methodName";
  

  public static final String LINE_NUMBER = "short.lineNumber";
  

  public static final String FILE_NAME = "short.fileName";
  

  public static final String MESSAGE = "short.message";
  

  public static final String LOCALIZED_MESSAGE = "short.localizedMessage";
  


  protected ThrowableFormatOptions(int lines, String separator, List<String> packages)
  {
    this.lines = lines;
    this.separator = (separator == null ? Constants.LINE_SEP : separator);
    this.packages = packages;
  }
  



  protected ThrowableFormatOptions(List<String> packages)
  {
    this(2147483647, null, packages);
  }
  


  protected ThrowableFormatOptions()
  {
    this(2147483647, null, null);
  }
  



  public int getLines()
  {
    return lines;
  }
  



  public String getSeparator()
  {
    return separator;
  }
  



  public List<String> getPackages()
  {
    return packages;
  }
  



  public boolean allLines()
  {
    return lines == 2147483647;
  }
  



  public boolean anyLines()
  {
    return lines > 0;
  }
  




  public int minLines(int maxLines)
  {
    return lines > maxLines ? maxLines : lines;
  }
  



  public boolean hasPackages()
  {
    return (packages != null) && (!packages.isEmpty());
  }
  



  public String toString()
  {
    StringBuilder s = new StringBuilder();
    s.append("{").append(anyLines() ? String.valueOf(lines) : lines == 2 ? "short" : allLines() ? "full" : "none").append("}");
    s.append("{separator(").append(separator).append(")}");
    if (hasPackages()) {
      s.append("{filters(");
      for (String p : packages) {
        s.append(p).append(",");
      }
      s.deleteCharAt(s.length() - 1);
      s.append(")}");
    }
    return s.toString();
  }
  



  public static ThrowableFormatOptions newInstance(String[] options)
  {
    if ((options == null) || (options.length == 0)) {
      return DEFAULT;
    }
    





    if ((options.length == 1) && (options[0] != null) && (options[0].length() > 0)) {
      String[] opts = options[0].split(",", 2);
      String first = opts[0].trim();
      Scanner scanner = new Scanner(first);
      if ((opts.length > 1) && ((first.equalsIgnoreCase("full")) || (first.equalsIgnoreCase("short")) || (first.equalsIgnoreCase("none")) || (scanner.hasNextInt()))) {
        options = new String[] { first, opts[1].trim() };
      }
      scanner.close();
    }
    
    int lines = DEFAULTlines;
    String separator = DEFAULTseparator;
    List<String> packages = DEFAULTpackages;
    for (String rawOption : options) {
      if (rawOption != null) {
        String option = rawOption.trim();
        if (!option.isEmpty())
        {
          if ((option.startsWith("separator(")) && (option.endsWith(")"))) {
            separator = option.substring("separator(".length(), option.length() - 1);
          } else if ((option.startsWith("filters(")) && (option.endsWith(")"))) {
            String filterStr = option.substring("filters(".length(), option.length() - 1);
            if (filterStr.length() > 0) {
              String[] array = filterStr.split(",");
              if (array.length > 0) {
                packages = new ArrayList(array.length);
                for (String token : array) {
                  token = token.trim();
                  if (token.length() > 0) {
                    packages.add(token);
                  }
                }
              }
            }
          } else if (option.equalsIgnoreCase("none")) {
            lines = 0;
          } else if ((option.equalsIgnoreCase("short")) || (option.equalsIgnoreCase("short.className")) || (option.equalsIgnoreCase("short.methodName")) || (option.equalsIgnoreCase("short.lineNumber")) || (option.equalsIgnoreCase("short.fileName")) || (option.equalsIgnoreCase("short.message")) || (option.equalsIgnoreCase("short.localizedMessage")))
          {


            lines = 2;
          } else if (!option.equalsIgnoreCase("full")) {
            lines = Integer.parseInt(option);
          } }
      }
    }
    return new ThrowableFormatOptions(lines, separator, packages);
  }
}
