package org.apache.logging.log4j.core.jmx;

import java.util.List;
import javax.management.ObjectName;
import org.apache.logging.log4j.Level;
import org.apache.logging.log4j.core.config.AppenderRef;
import org.apache.logging.log4j.core.config.LoggerConfig;
import org.apache.logging.log4j.core.helpers.Assert;



























public class LoggerConfigAdmin
  implements LoggerConfigAdminMBean
{
  private final String contextName;
  private final LoggerConfig loggerConfig;
  private final ObjectName objectName;
  
  public LoggerConfigAdmin(String contextName, LoggerConfig loggerConfig)
  {
    this.contextName = ((String)Assert.isNotNull(contextName, "contextName"));
    this.loggerConfig = ((LoggerConfig)Assert.isNotNull(loggerConfig, "loggerConfig"));
    try {
      String ctxName = Server.escape(this.contextName);
      String configName = Server.escape(loggerConfig.getName());
      String name = String.format("org.apache.logging.log4j2:type=LoggerContext,ctx=%s,sub=LoggerConfig,name=%s", new Object[] { ctxName, configName });
      objectName = new ObjectName(name);
    } catch (Exception e) {
      throw new IllegalStateException(e);
    }
  }
  





  public ObjectName getObjectName()
  {
    return objectName;
  }
  
  public String getName()
  {
    return loggerConfig.getName();
  }
  
  public String getLevel()
  {
    return loggerConfig.getLevel().name();
  }
  
  public void setLevel(String level)
  {
    loggerConfig.setLevel(Level.valueOf(level));
  }
  
  public boolean isAdditive()
  {
    return loggerConfig.isAdditive();
  }
  
  public void setAdditive(boolean additive)
  {
    loggerConfig.setAdditive(additive);
  }
  
  public boolean isIncludeLocation()
  {
    return loggerConfig.isIncludeLocation();
  }
  
  public String getFilter()
  {
    return String.valueOf(loggerConfig.getFilter());
  }
  
  public String[] getAppenderRefs()
  {
    List<AppenderRef> refs = loggerConfig.getAppenderRefs();
    String[] result = new String[refs.size()];
    for (int i = 0; i < result.length; i++) {
      result[i] = ((AppenderRef)refs.get(i)).getRef();
    }
    return result;
  }
}
