package org.apache.logging.log4j.core.net;

import java.io.IOException;
import java.io.OutputStream;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketException;
import java.net.UnknownHostException;
import org.apache.logging.log4j.Logger;
import org.apache.logging.log4j.core.appender.AppenderLoggingException;
import org.apache.logging.log4j.status.StatusLogger;























public class DatagramOutputStream
  extends OutputStream
{
  protected static final Logger LOGGER = ;
  
  private static final int SHIFT_1 = 8;
  
  private static final int SHIFT_2 = 16;
  
  private static final int SHIFT_3 = 24;
  
  private DatagramSocket ds;
  
  private final InetAddress address;
  
  private final int port;
  
  private byte[] data;
  
  private final byte[] header;
  private final byte[] footer;
  
  public DatagramOutputStream(String host, int port, byte[] header, byte[] footer)
  {
    this.port = port;
    this.header = header;
    this.footer = footer;
    try {
      address = InetAddress.getByName(host);
    } catch (UnknownHostException ex) {
      String msg = "Could not find host " + host;
      LOGGER.error(msg, ex);
      throw new AppenderLoggingException(msg, ex);
    }
    try
    {
      ds = new DatagramSocket();
    } catch (SocketException ex) {
      String msg = "Could not instantiate DatagramSocket to " + host;
      LOGGER.error(msg, ex);
      throw new AppenderLoggingException(msg, ex);
    }
  }
  
  public synchronized void write(byte[] bytes, int offset, int length) throws IOException
  {
    copy(bytes, offset, length);
  }
  
  public synchronized void write(int i) throws IOException
  {
    copy(new byte[] { (byte)(i >>> 24), (byte)(i >>> 16), (byte)(i >>> 8), (byte)i }, 0, 4);
  }
  
  public synchronized void write(byte[] bytes) throws IOException
  {
    copy(bytes, 0, bytes.length);
  }
  
  public synchronized void flush() throws IOException
  {
    try {
      if ((data != null) && (ds != null) && (address != null)) {
        if (footer != null) {
          copy(footer, 0, footer.length);
        }
        DatagramPacket packet = new DatagramPacket(data, data.length, address, port);
        ds.send(packet);
      }
    } finally {
      data = null;
      if (header != null) {
        copy(header, 0, header.length);
      }
    }
  }
  
  public synchronized void close() throws IOException
  {
    if (ds != null) {
      if (data != null) {
        flush();
      }
      ds.close();
      ds = null;
    }
  }
  
  private void copy(byte[] bytes, int offset, int length) {
    int index = data == null ? 0 : data.length;
    byte[] copy = new byte[length + index];
    if (data != null) {
      System.arraycopy(data, 0, copy, 0, data.length);
    }
    System.arraycopy(bytes, offset, copy, index, length);
    data = copy;
  }
}
