package org.apache.logging.log4j.core.net;

import java.nio.charset.Charset;
import javax.mail.Message.RecipientType;
import javax.mail.MessagingException;
import javax.mail.Session;
import javax.mail.internet.AddressException;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;
import org.apache.logging.log4j.core.helpers.Charsets;


















public class MimeMessageBuilder
{
  private final MimeMessage message;
  
  public MimeMessageBuilder(Session session)
  {
    message = new MimeMessage(session);
  }
  
  public MimeMessageBuilder setFrom(String from) throws MessagingException {
    InternetAddress address = parseAddress(from);
    
    if (null != address) {
      message.setFrom(address);
    } else {
      try {
        message.setFrom();
      } catch (Exception ex) {
        message.setFrom((InternetAddress)null);
      }
    }
    return this;
  }
  
  public MimeMessageBuilder setReplyTo(String replyTo) throws MessagingException {
    InternetAddress[] addresses = parseAddresses(replyTo);
    
    if (null != addresses) {
      message.setReplyTo(addresses);
    }
    return this;
  }
  
  public MimeMessageBuilder setRecipients(Message.RecipientType recipientType, String recipients) throws MessagingException
  {
    InternetAddress[] addresses = parseAddresses(recipients);
    
    if (null != addresses) {
      message.setRecipients(recipientType, addresses);
    }
    return this;
  }
  
  public MimeMessageBuilder setSubject(String subject) throws MessagingException {
    if (subject != null) {
      message.setSubject(subject, Charsets.UTF_8.name());
    }
    return this;
  }
  
  public MimeMessage getMimeMessage() {
    return message;
  }
  
  private static InternetAddress parseAddress(String address) throws AddressException {
    return address == null ? null : new InternetAddress(address);
  }
  
  private static InternetAddress[] parseAddresses(String addresses) throws AddressException {
    return addresses == null ? null : InternetAddress.parse(addresses, true);
  }
}
