package org.apache.logging.log4j.core.pattern;























public final class FormattingInfo
{
  private static final char[] SPACES = { ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ' };
  



  private static final FormattingInfo DEFAULT = new FormattingInfo(false, 0, 2147483647);
  





  private final int minLength;
  




  private final int maxLength;
  




  private final boolean leftAlign;
  





  public FormattingInfo(boolean leftAlign, int minLength, int maxLength)
  {
    this.leftAlign = leftAlign;
    this.minLength = minLength;
    this.maxLength = maxLength;
  }
  




  public static FormattingInfo getDefault()
  {
    return DEFAULT;
  }
  




  public boolean isLeftAligned()
  {
    return leftAlign;
  }
  




  public int getMinLength()
  {
    return minLength;
  }
  




  public int getMaxLength()
  {
    return maxLength;
  }
  







  public void format(int fieldStart, StringBuilder buffer)
  {
    int rawLength = buffer.length() - fieldStart;
    
    if (rawLength > maxLength) {
      buffer.delete(fieldStart, buffer.length() - maxLength);
    } else if (rawLength < minLength) {
      if (leftAlign) {
        int fieldEnd = buffer.length();
        buffer.setLength(fieldStart + minLength);
        
        for (int i = fieldEnd; i < buffer.length(); i++) {
          buffer.setCharAt(i, ' ');
        }
      } else {
        int padLength = minLength - rawLength;
        for (; 
            padLength > SPACES.length; padLength -= SPACES.length) {
          buffer.insert(fieldStart, SPACES);
        }
        
        buffer.insert(fieldStart, SPACES, 0, padLength);
      }
    }
  }
  





  public String toString()
  {
    StringBuilder sb = new StringBuilder();
    sb.append(super.toString());
    sb.append("[leftAlign=");
    sb.append(leftAlign);
    sb.append(", maxLength=");
    sb.append(maxLength);
    sb.append(", minLength=");
    sb.append(minLength);
    sb.append("]");
    return sb.toString();
  }
}
