package org.apache.logging.log4j.core.pattern;

import java.util.List;
import org.apache.logging.log4j.Logger;
import org.apache.logging.log4j.core.LogEvent;
import org.apache.logging.log4j.core.config.Configuration;
import org.apache.logging.log4j.core.config.plugins.Plugin;
import org.apache.logging.log4j.core.layout.PatternLayout;
























@Plugin(name="style", category="Converter")
@ConverterKeys({"style"})
public final class StyleConverter
  extends LogEventPatternConverter
{
  private final List<PatternFormatter> patternFormatters;
  private final String style;
  
  private StyleConverter(List<PatternFormatter> patternFormatters, String style)
  {
    super("style", "style");
    this.patternFormatters = patternFormatters;
    this.style = style;
  }
  







  public static StyleConverter newInstance(Configuration config, String[] options)
  {
    if (options.length < 1) {
      LOGGER.error("Incorrect number of options on style. Expected at least 1, received " + options.length);
      return null;
    }
    if (options[0] == null) {
      LOGGER.error("No pattern supplied on style");
      return null;
    }
    if (options[1] == null) {
      LOGGER.error("No style attributes provided");
      return null;
    }
    
    PatternParser parser = PatternLayout.createPatternParser(config);
    List<PatternFormatter> formatters = parser.parse(options[0]);
    String style = AnsiEscape.createSequence(options[1].split("\\s*,\\s*"));
    return new StyleConverter(formatters, style);
  }
  




  public void format(LogEvent event, StringBuilder toAppendTo)
  {
    StringBuilder buf = new StringBuilder();
    for (PatternFormatter formatter : patternFormatters) {
      formatter.format(event, buf);
    }
    
    if (buf.length() > 0) {
      toAppendTo.append(style).append(buf.toString()).append(AnsiEscape.getDefaultStyle());
    }
  }
  
  public boolean handlesThrowable()
  {
    for (PatternFormatter formatter : patternFormatters) {
      if (formatter.handlesThrowable()) {
        return true;
      }
    }
    return false;
  }
  





  public String toString()
  {
    StringBuilder sb = new StringBuilder();
    sb.append(super.toString());
    sb.append("[style=");
    sb.append(style);
    sb.append(", patternFormatters=");
    sb.append(patternFormatters);
    sb.append("]");
    return sb.toString();
  }
}
