package org.apache.logging.log4j.core.web;

import java.io.IOException;
import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;



























public class Log4jServletFilter
  implements Filter
{
  static final String ALREADY_FILTERED_ATTRIBUTE = Log4jServletFilter.class.getName() + ".FILTERED";
  private ServletContext servletContext;
  private Log4jWebInitializer initializer;
  
  public void init(FilterConfig filterConfig)
    throws ServletException
  {
    servletContext = filterConfig.getServletContext();
    servletContext.log("Log4jServletFilter initialized.");
    
    initializer = Log4jWebInitializerImpl.getLog4jWebInitializer(servletContext);
    initializer.clearLoggerContext();
  }
  
  public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain)
    throws IOException, ServletException
  {
    if (request.getAttribute(ALREADY_FILTERED_ATTRIBUTE) != null) {
      chain.doFilter(request, response);
    } else {
      request.setAttribute(ALREADY_FILTERED_ATTRIBUTE, Boolean.valueOf(true));
      try
      {
        initializer.setLoggerContext();
        
        chain.doFilter(request, response);
      } finally {
        initializer.clearLoggerContext();
      }
    }
  }
  
  public void destroy()
  {
    if ((servletContext == null) || (initializer == null)) {
      throw new IllegalStateException("Filter destroyed before it was initialized.");
    }
    servletContext.log("Log4jServletFilter destroyed.");
    
    initializer.setLoggerContext();
  }
}
