package org.apache.logging.log4j.message;



class BasicThreadInformation
  implements ThreadInformation
{
  private static final int HASH_SHIFT = 32;
  

  private static final int HASH_MULTIPLIER = 31;
  

  private final long id;
  

  private final String name;
  

  private final String longName;
  

  private final Thread.State state;
  

  private final int priority;
  

  private final boolean isAlive;
  

  private final boolean isDaemon;
  
  private final String threadGroupName;
  

  public BasicThreadInformation(Thread thread)
  {
    id = thread.getId();
    name = thread.getName();
    longName = thread.toString();
    state = thread.getState();
    priority = thread.getPriority();
    isAlive = thread.isAlive();
    isDaemon = thread.isDaemon();
    ThreadGroup group = thread.getThreadGroup();
    threadGroupName = (group == null ? null : group.getName());
  }
  
  public boolean equals(Object o)
  {
    if (this == o) {
      return true;
    }
    if ((o == null) || (getClass() != o.getClass())) {
      return false;
    }
    
    BasicThreadInformation that = (BasicThreadInformation)o;
    
    if (id != id) {
      return false;
    }
    if (name != null ? !name.equals(name) : name != null) {
      return false;
    }
    
    return true;
  }
  
  public int hashCode()
  {
    int result = (int)(id ^ id >>> 32);
    result = 31 * result + (name != null ? name.hashCode() : 0);
    return result;
  }
  




  public void printThreadInfo(StringBuilder sb)
  {
    sb.append("\"").append(name).append("\" ");
    if (isDaemon) {
      sb.append("daemon ");
    }
    sb.append("prio=").append(priority).append(" tid=").append(id).append(" ");
    if (threadGroupName != null) {
      sb.append("group=\"").append(threadGroupName).append("\"");
    }
    sb.append("\n");
    sb.append("\tThread state: ").append(state.name()).append("\n");
  }
  





  public void printStack(StringBuilder sb, StackTraceElement[] trace)
  {
    for (StackTraceElement element : trace) {
      sb.append("\tat ").append(element).append("\n");
    }
  }
}
