package org.apache.logging.log4j.message;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.text.Format;
import java.text.MessageFormat;
import java.util.Arrays;
import java.util.regex.Matcher;
import java.util.regex.Pattern;



















public class FormattedMessage
  implements Message
{
  private static final long serialVersionUID = -665975803997290697L;
  private static final int HASHVAL = 31;
  private static final String FORMAT_SPECIFIER = "%(\\d+\\$)?([-#+ 0,(\\<]*)?(\\d+)?(\\.\\d+)?([tT])?([a-zA-Z%])";
  private static final Pattern MSG_PATTERN = Pattern.compile("%(\\d+\\$)?([-#+ 0,(\\<]*)?(\\d+)?(\\.\\d+)?([tT])?([a-zA-Z%])");
  private String messagePattern;
  private transient Object[] argArray;
  private String[] stringArgs;
  private transient String formattedMessage;
  private final Throwable throwable;
  private Message message;
  
  public FormattedMessage(String messagePattern, Object[] arguments, Throwable throwable)
  {
    this.messagePattern = messagePattern;
    argArray = arguments;
    this.throwable = throwable;
  }
  
  public FormattedMessage(String messagePattern, Object[] arguments) {
    this.messagePattern = messagePattern;
    argArray = arguments;
    throwable = null;
  }
  




  public FormattedMessage(String messagePattern, Object arg)
  {
    this.messagePattern = messagePattern;
    argArray = new Object[] { arg };
    throwable = null;
  }
  





  public FormattedMessage(String messagePattern, Object arg1, Object arg2)
  {
    this(messagePattern, new Object[] { arg1, arg2 });
  }
  





  public String getFormattedMessage()
  {
    if (formattedMessage == null) {
      if (message == null) {
        message = getMessage(messagePattern, argArray, throwable);
      }
      formattedMessage = message.getFormattedMessage();
    }
    return formattedMessage;
  }
  




  public String getFormat()
  {
    return messagePattern;
  }
  




  public Object[] getParameters()
  {
    if (argArray != null) {
      return argArray;
    }
    return stringArgs;
  }
  
  protected Message getMessage(String msgPattern, Object[] args, Throwable throwable) {
    try {
      MessageFormat format = new MessageFormat(msgPattern);
      Format[] formats = format.getFormats();
      if ((formats != null) && (formats.length > 0)) {
        return new MessageFormatMessage(msgPattern, args);
      }
    }
    catch (Exception ex) {}
    try
    {
      if (MSG_PATTERN.matcher(msgPattern).find()) {
        return new StringFormattedMessage(msgPattern, args);
      }
    }
    catch (Exception ex) {}
    
    return new ParameterizedMessage(msgPattern, args, throwable);
  }
  
  public boolean equals(Object o)
  {
    if (this == o) {
      return true;
    }
    if ((o == null) || (getClass() != o.getClass())) {
      return false;
    }
    
    FormattedMessage that = (FormattedMessage)o;
    
    if (messagePattern != null ? !messagePattern.equals(messagePattern) : messagePattern != null) {
      return false;
    }
    if (!Arrays.equals(stringArgs, stringArgs)) {
      return false;
    }
    
    return true;
  }
  
  public int hashCode()
  {
    int result = messagePattern != null ? messagePattern.hashCode() : 0;
    result = 31 * result + (stringArgs != null ? Arrays.hashCode(stringArgs) : 0);
    return result;
  }
  

  public String toString()
  {
    return "FormattedMessage[messagePattern=" + messagePattern + ", args=" + Arrays.toString(argArray) + "]";
  }
  
  private void writeObject(ObjectOutputStream out) throws IOException
  {
    out.defaultWriteObject();
    getFormattedMessage();
    out.writeUTF(formattedMessage);
    out.writeUTF(messagePattern);
    out.writeInt(argArray.length);
    stringArgs = new String[argArray.length];
    int i = 0;
    for (Object obj : argArray) {
      stringArgs[i] = obj.toString();
      i++;
    }
  }
  
  private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
    in.defaultReadObject();
    formattedMessage = in.readUTF();
    messagePattern = in.readUTF();
    int length = in.readInt();
    stringArgs = new String[length];
    for (int i = 0; i < length; i++) {
      stringArgs[i] = in.readUTF();
    }
  }
  





  public Throwable getThrowable()
  {
    if (throwable != null) {
      return throwable;
    }
    if (message == null) {
      message = getMessage(messagePattern, argArray, throwable);
    }
    return message.getThrowable();
  }
}
