package org.apache.logging.log4j.message;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import org.apache.logging.log4j.status.StatusLogger;





























public class LocalizedMessage
  implements Message, LoggerNameAwareMessage
{
  private static final long serialVersionUID = 3893703791567290742L;
  private String bundleId;
  private transient ResourceBundle bundle;
  private Locale locale;
  private transient StatusLogger logger = StatusLogger.getLogger();
  
  private String loggerName;
  
  private String messagePattern;
  
  private String[] stringArgs;
  
  private transient Object[] argArray;
  
  private String formattedMessage;
  
  private transient Throwable throwable;
  
  public LocalizedMessage(String messagePattern, Object[] arguments)
  {
    this((ResourceBundle)null, (Locale)null, messagePattern, arguments);
  }
  
  public LocalizedMessage(String bundleId, String key, Object[] arguments) {
    this(bundleId, (Locale)null, key, arguments);
  }
  
  public LocalizedMessage(ResourceBundle bundle, String key, Object[] arguments) {
    this(bundle, (Locale)null, key, arguments);
  }
  
  public LocalizedMessage(String bundleId, Locale locale, String key, Object[] arguments) {
    messagePattern = key;
    argArray = arguments;
    throwable = null;
    setup(bundleId, null, locale);
  }
  
  public LocalizedMessage(ResourceBundle bundle, Locale locale, String key, Object[] arguments)
  {
    messagePattern = key;
    argArray = arguments;
    throwable = null;
    setup(null, bundle, locale);
  }
  
  public LocalizedMessage(Locale locale, String key, Object[] arguments) {
    this((ResourceBundle)null, locale, key, arguments);
  }
  
  public LocalizedMessage(String messagePattern, Object arg) {
    this((ResourceBundle)null, (Locale)null, messagePattern, new Object[] { arg });
  }
  
  public LocalizedMessage(String bundleId, String key, Object arg) {
    this(bundleId, (Locale)null, key, new Object[] { arg });
  }
  
  public LocalizedMessage(ResourceBundle bundle, String key, Object arg) {
    this(bundle, (Locale)null, key, new Object[] { arg });
  }
  
  public LocalizedMessage(String bundleId, Locale locale, String key, Object arg) {
    this(bundleId, locale, key, new Object[] { arg });
  }
  
  public LocalizedMessage(ResourceBundle bundle, Locale locale, String key, Object arg) {
    this(bundle, locale, key, new Object[] { arg });
  }
  
  public LocalizedMessage(Locale locale, String key, Object arg) {
    this((ResourceBundle)null, locale, key, new Object[] { arg });
  }
  
  public LocalizedMessage(String messagePattern, Object arg1, Object arg2) {
    this((ResourceBundle)null, (Locale)null, messagePattern, new Object[] { arg1, arg2 });
  }
  
  public LocalizedMessage(String bundleId, String key, Object arg1, Object arg2) {
    this(bundleId, (Locale)null, key, new Object[] { arg1, arg2 });
  }
  
  public LocalizedMessage(ResourceBundle bundle, String key, Object arg1, Object arg2) {
    this(bundle, (Locale)null, key, new Object[] { arg1, arg2 });
  }
  
  public LocalizedMessage(String bundleId, Locale locale, String key, Object arg1, Object arg2)
  {
    this(bundleId, locale, key, new Object[] { arg1, arg2 });
  }
  
  public LocalizedMessage(ResourceBundle bundle, Locale locale, String key, Object arg1, Object arg2)
  {
    this(bundle, locale, key, new Object[] { arg1, arg2 });
  }
  
  public LocalizedMessage(Locale locale, String key, Object arg1, Object arg2) {
    this((ResourceBundle)null, locale, key, new Object[] { arg1, arg2 });
  }
  




  public void setLoggerName(String name)
  {
    loggerName = name;
  }
  




  public String getLoggerName()
  {
    return loggerName;
  }
  
  private void setup(String bundleId, ResourceBundle bundle, Locale locale) {
    this.bundleId = bundleId;
    this.bundle = bundle;
    this.locale = locale;
  }
  




  public String getFormattedMessage()
  {
    if (formattedMessage != null) {
      return formattedMessage;
    }
    ResourceBundle bundle = this.bundle;
    if (bundle == null) {
      if (bundleId != null) {
        bundle = getBundle(bundleId, locale, false);
      } else {
        bundle = getBundle(loggerName, locale, true);
      }
    }
    String messagePattern = getFormat();
    String msgPattern = (bundle == null) || (!bundle.containsKey(messagePattern)) ? messagePattern : bundle.getString(messagePattern);
    
    Object[] array = argArray == null ? stringArgs : argArray;
    FormattedMessage msg = new FormattedMessage(msgPattern, array);
    formattedMessage = msg.getFormattedMessage();
    throwable = msg.getThrowable();
    return formattedMessage;
  }
  
  public String getFormat()
  {
    return messagePattern;
  }
  
  public Object[] getParameters()
  {
    if (argArray != null) {
      return argArray;
    }
    return stringArgs;
  }
  
  public Throwable getThrowable()
  {
    return throwable;
  }
  







  protected ResourceBundle getBundle(String key, Locale locale, boolean loop)
  {
    ResourceBundle rb = null;
    
    if (key == null) {
      return null;
    }
    try {
      if (locale != null) {
        rb = ResourceBundle.getBundle(key, locale);
      } else {
        rb = ResourceBundle.getBundle(key);
      }
    } catch (MissingResourceException ex) {
      if (!loop) {
        logger.debug("Unable to locate ResourceBundle " + key);
        return null;
      }
    }
    
    String substr = key;
    int i;
    while ((rb == null) && ((i = substr.lastIndexOf('.')) > 0)) {
      substr = substr.substring(0, i);
      try {
        if (locale != null) {
          rb = ResourceBundle.getBundle(substr, locale);
        } else {
          rb = ResourceBundle.getBundle(substr);
        }
      } catch (MissingResourceException ex) {
        logger.debug("Unable to locate ResourceBundle " + substr);
      }
    }
    return rb;
  }
  
  private void writeObject(ObjectOutputStream out) throws IOException {
    out.defaultWriteObject();
    getFormattedMessage();
    out.writeUTF(formattedMessage);
    out.writeUTF(messagePattern);
    out.writeUTF(bundleId);
    out.writeInt(argArray.length);
    stringArgs = new String[argArray.length];
    int i = 0;
    for (Object obj : argArray) {
      stringArgs[i] = obj.toString();
      i++;
    }
  }
  
  private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
    in.defaultReadObject();
    formattedMessage = in.readUTF();
    messagePattern = in.readUTF();
    bundleId = in.readUTF();
    int length = in.readInt();
    stringArgs = new String[length];
    for (int i = 0; i < length; i++) {
      stringArgs[i] = in.readUTF();
    }
    logger = StatusLogger.getLogger();
    bundle = null;
    argArray = null;
  }
}
