package org.apache.logging.log4j.message;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Arrays;
import java.util.IllegalFormatException;
import org.apache.logging.log4j.Logger;
import org.apache.logging.log4j.status.StatusLogger;




















public class StringFormattedMessage
  implements Message
{
  private static final Logger LOGGER = ;
  
  private static final long serialVersionUID = -665975803997290697L;
  
  private static final int HASHVAL = 31;
  private String messagePattern;
  private transient Object[] argArray;
  private String[] stringArgs;
  private transient String formattedMessage;
  private transient Throwable throwable;
  
  public StringFormattedMessage(String messagePattern, Object... arguments)
  {
    this.messagePattern = messagePattern;
    argArray = arguments;
    if ((arguments != null) && (arguments.length > 0) && ((arguments[(arguments.length - 1)] instanceof Throwable))) {
      throwable = ((Throwable)arguments[(arguments.length - 1)]);
    }
  }
  




  public String getFormattedMessage()
  {
    if (formattedMessage == null) {
      formattedMessage = formatMessage(messagePattern, argArray);
    }
    return formattedMessage;
  }
  




  public String getFormat()
  {
    return messagePattern;
  }
  




  public Object[] getParameters()
  {
    if (argArray != null) {
      return argArray;
    }
    return stringArgs;
  }
  
  protected String formatMessage(String msgPattern, Object... args) {
    try {
      return String.format(msgPattern, args);
    } catch (IllegalFormatException ife) {
      LOGGER.error("Unable to format msg: " + msgPattern, ife); }
    return msgPattern;
  }
  

  public boolean equals(Object o)
  {
    if (this == o) {
      return true;
    }
    if ((o == null) || (getClass() != o.getClass())) {
      return false;
    }
    
    StringFormattedMessage that = (StringFormattedMessage)o;
    
    if (messagePattern != null ? !messagePattern.equals(messagePattern) : messagePattern != null) {
      return false;
    }
    
    return Arrays.equals(stringArgs, stringArgs);
  }
  
  public int hashCode()
  {
    int result = messagePattern != null ? messagePattern.hashCode() : 0;
    result = 31 * result + (stringArgs != null ? Arrays.hashCode(stringArgs) : 0);
    return result;
  }
  

  public String toString()
  {
    return "StringFormatMessage[messagePattern=" + messagePattern + ", args=" + Arrays.toString(argArray) + "]";
  }
  
  private void writeObject(ObjectOutputStream out) throws IOException
  {
    out.defaultWriteObject();
    getFormattedMessage();
    out.writeUTF(formattedMessage);
    out.writeUTF(messagePattern);
    out.writeInt(argArray.length);
    stringArgs = new String[argArray.length];
    int i = 0;
    for (Object obj : argArray) {
      stringArgs[i] = obj.toString();
      i++;
    }
  }
  
  private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
    in.defaultReadObject();
    formattedMessage = in.readUTF();
    messagePattern = in.readUTF();
    int length = in.readInt();
    stringArgs = new String[length];
    for (int i = 0; i < length; i++) {
      stringArgs[i] = in.readUTF();
    }
  }
  





  public Throwable getThrowable()
  {
    return throwable;
  }
}
