package org.apache.logging.log4j.simple;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.Properties;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;
import org.apache.logging.log4j.Level;
import org.apache.logging.log4j.Logger;
import org.apache.logging.log4j.message.MessageFactory;
import org.apache.logging.log4j.spi.AbstractLogger;
import org.apache.logging.log4j.spi.LoggerContext;
import org.apache.logging.log4j.util.PropertiesUtil;

























public class SimpleLoggerContext
  implements LoggerContext
{
  protected static final String DEFAULT_DATE_TIME_FORMAT = "yyyy/MM/dd HH:mm:ss:SSS zzz";
  protected static final String SYSTEM_PREFIX = "org.apache.logging.log4j.simplelog.";
  private final Properties simpleLogProps = new Properties();
  

  private final PropertiesUtil props;
  

  private final boolean showLogName;
  

  private final boolean showShortName;
  

  private final boolean showDateTime;
  
  private final boolean showContextMap;
  
  private final String dateTimeFormat;
  
  private final Level defaultLevel;
  
  private final PrintStream stream;
  
  private final ConcurrentMap<String, Logger> loggers = new ConcurrentHashMap();
  
  public SimpleLoggerContext() {
    props = new PropertiesUtil("log4j2.simplelog.properties");
    
    showContextMap = props.getBooleanProperty("org.apache.logging.log4j.simplelog.showContextMap", false);
    showLogName = props.getBooleanProperty("org.apache.logging.log4j.simplelog.showlogname", false);
    showShortName = props.getBooleanProperty("org.apache.logging.log4j.simplelog.showShortLogname", true);
    showDateTime = props.getBooleanProperty("org.apache.logging.log4j.simplelog.showdatetime", false);
    String lvl = props.getStringProperty("org.apache.logging.log4j.simplelog.level");
    defaultLevel = Level.toLevel(lvl, Level.ERROR);
    
    dateTimeFormat = (showDateTime ? props.getStringProperty("org.apache.logging.log4j.simplelog.dateTimeFormat", "yyyy/MM/dd HH:mm:ss:SSS zzz") : null);
    

    String fileName = props.getStringProperty("org.apache.logging.log4j.simplelog.logFile", "system.err");
    PrintStream ps;
    PrintStream ps; if ("system.err".equalsIgnoreCase(fileName)) {
      ps = System.err; } else { PrintStream ps;
      if ("system.out".equalsIgnoreCase(fileName)) {
        ps = System.out;
      } else
        try {
          FileOutputStream os = new FileOutputStream(fileName);
          ps = new PrintStream(os);
        } catch (FileNotFoundException fnfe) {
          ps = System.err;
        }
    }
    stream = ps;
  }
  
  public Logger getLogger(String name)
  {
    return getLogger(name, null);
  }
  
  public Logger getLogger(String name, MessageFactory messageFactory)
  {
    if (loggers.containsKey(name)) {
      Logger logger = (Logger)loggers.get(name);
      AbstractLogger.checkMessageFactory(logger, messageFactory);
      return logger;
    }
    
    loggers.putIfAbsent(name, new SimpleLogger(name, defaultLevel, showLogName, showShortName, showDateTime, showContextMap, dateTimeFormat, messageFactory, props, stream));
    
    return (Logger)loggers.get(name);
  }
  
  public boolean hasLogger(String name)
  {
    return false;
  }
  
  public Object getExternalContext()
  {
    return null;
  }
}
