package org.bukkit;

import org.bukkit.block.Block;
import org.bukkit.util.NumberConversions;
import org.bukkit.util.Vector;









public class Location
  implements Cloneable
{
  private World world;
  private double x;
  private double y;
  private double z;
  private float pitch;
  private float yaw;
  
  public Location(World world, double x, double y, double z)
  {
    this(world, x, y, z, 0.0F, 0.0F);
  }
  









  public Location(World world, double x, double y, double z, float yaw, float pitch)
  {
    this.world = world;
    this.x = x;
    this.y = y;
    this.z = z;
    this.pitch = pitch;
    this.yaw = yaw;
  }
  




  public void setWorld(World world)
  {
    this.world = world;
  }
  




  public World getWorld()
  {
    return world;
  }
  




  public Chunk getChunk()
  {
    return world.getChunkAt(this);
  }
  




  public Block getBlock()
  {
    return world.getBlockAt(this);
  }
  




  public void setX(double x)
  {
    this.x = x;
  }
  




  public double getX()
  {
    return x;
  }
  





  public int getBlockX()
  {
    return locToBlock(x);
  }
  




  public void setY(double y)
  {
    this.y = y;
  }
  




  public double getY()
  {
    return y;
  }
  





  public int getBlockY()
  {
    return locToBlock(y);
  }
  




  public void setZ(double z)
  {
    this.z = z;
  }
  




  public double getZ()
  {
    return z;
  }
  





  public int getBlockZ()
  {
    return locToBlock(z);
  }
  













  public void setYaw(float yaw)
  {
    this.yaw = yaw;
  }
  













  public float getYaw()
  {
    return yaw;
  }
  











  public void setPitch(float pitch)
  {
    this.pitch = pitch;
  }
  











  public float getPitch()
  {
    return pitch;
  }
  






  public Vector getDirection()
  {
    Vector vector = new Vector();
    
    double rotX = getYaw();
    double rotY = getPitch();
    
    vector.setY(-Math.sin(Math.toRadians(rotY)));
    
    double xz = Math.cos(Math.toRadians(rotY));
    
    vector.setX(-xz * Math.sin(Math.toRadians(rotX)));
    vector.setZ(xz * Math.cos(Math.toRadians(rotX)));
    
    return vector;
  }
  











  public Location setDirection(Vector vector)
  {
    double _2PI = 6.283185307179586D;
    double x = vector.getX();
    double z = vector.getZ();
    
    if ((x == 0.0D) && (z == 0.0D)) {
      pitch = (vector.getY() > 0.0D ? -90.0F : 90.0F);
      return this;
    }
    
    double theta = Math.atan2(-x, z);
    yaw = ((float)Math.toDegrees((theta + 6.283185307179586D) % 6.283185307179586D));
    
    double x2 = NumberConversions.square(x);
    double z2 = NumberConversions.square(z);
    double xz = Math.sqrt(x2 + z2);
    pitch = ((float)Math.toDegrees(Math.atan(-vector.getY() / xz)));
    
    return this;
  }
  







  public Location add(Location vec)
  {
    if ((vec == null) || (vec.getWorld() != getWorld())) {
      throw new IllegalArgumentException("Cannot add Locations of differing worlds");
    }
    
    x += x;
    y += y;
    z += z;
    return this;
  }
  






  public Location add(Vector vec)
  {
    x += vec.getX();
    y += vec.getY();
    z += vec.getZ();
    return this;
  }
  








  public Location add(double x, double y, double z)
  {
    this.x += x;
    this.y += y;
    this.z += z;
    return this;
  }
  







  public Location subtract(Location vec)
  {
    if ((vec == null) || (vec.getWorld() != getWorld())) {
      throw new IllegalArgumentException("Cannot add Locations of differing worlds");
    }
    
    x -= x;
    y -= y;
    z -= z;
    return this;
  }
  






  public Location subtract(Vector vec)
  {
    x -= vec.getX();
    y -= vec.getY();
    z -= vec.getZ();
    return this;
  }
  









  public Location subtract(double x, double y, double z)
  {
    this.x -= x;
    this.y -= y;
    this.z -= z;
    return this;
  }
  










  public double length()
  {
    return Math.sqrt(NumberConversions.square(x) + NumberConversions.square(y) + NumberConversions.square(z));
  }
  






  public double lengthSquared()
  {
    return NumberConversions.square(x) + NumberConversions.square(y) + NumberConversions.square(z);
  }
  











  public double distance(Location o)
  {
    return Math.sqrt(distanceSquared(o));
  }
  







  public double distanceSquared(Location o)
  {
    if (o == null)
      throw new IllegalArgumentException("Cannot measure distance to a null location");
    if ((o.getWorld() == null) || (getWorld() == null))
      throw new IllegalArgumentException("Cannot measure distance to a null world");
    if (o.getWorld() != getWorld()) {
      throw new IllegalArgumentException("Cannot measure distance between " + getWorld().getName() + " and " + o.getWorld().getName());
    }
    
    return NumberConversions.square(x - x) + NumberConversions.square(y - y) + NumberConversions.square(z - z);
  }
  







  public Location multiply(double m)
  {
    x *= m;
    y *= m;
    z *= m;
    return this;
  }
  





  public Location zero()
  {
    x = 0.0D;
    y = 0.0D;
    z = 0.0D;
    return this;
  }
  
  public boolean equals(Object obj)
  {
    if (obj == null) {
      return false;
    }
    if (getClass() != obj.getClass()) {
      return false;
    }
    Location other = (Location)obj;
    
    if ((world != world) && ((world == null) || (!world.equals(world)))) {
      return false;
    }
    if (Double.doubleToLongBits(x) != Double.doubleToLongBits(x)) {
      return false;
    }
    if (Double.doubleToLongBits(y) != Double.doubleToLongBits(y)) {
      return false;
    }
    if (Double.doubleToLongBits(z) != Double.doubleToLongBits(z)) {
      return false;
    }
    if (Float.floatToIntBits(pitch) != Float.floatToIntBits(pitch)) {
      return false;
    }
    if (Float.floatToIntBits(yaw) != Float.floatToIntBits(yaw)) {
      return false;
    }
    return true;
  }
  
  public int hashCode()
  {
    int hash = 3;
    
    hash = 19 * hash + (world != null ? world.hashCode() : 0);
    hash = 19 * hash + (int)(Double.doubleToLongBits(x) ^ Double.doubleToLongBits(x) >>> 32);
    hash = 19 * hash + (int)(Double.doubleToLongBits(y) ^ Double.doubleToLongBits(y) >>> 32);
    hash = 19 * hash + (int)(Double.doubleToLongBits(z) ^ Double.doubleToLongBits(z) >>> 32);
    hash = 19 * hash + Float.floatToIntBits(pitch);
    hash = 19 * hash + Float.floatToIntBits(yaw);
    return hash;
  }
  
  public String toString()
  {
    return "Location{world=" + world + ",x=" + x + ",y=" + y + ",z=" + z + ",pitch=" + pitch + ",yaw=" + yaw + '}';
  }
  





  public Vector toVector()
  {
    return new Vector(x, y, z);
  }
  
  public Location clone()
  {
    try {
      return (Location)super.clone();
    } catch (CloneNotSupportedException e) {
      throw new Error(e);
    }
  }
  






  public static int locToBlock(double loc)
  {
    return NumberConversions.floor(loc);
  }
}
