package org.bukkit.command.defaults;

import com.google.common.collect.ImmutableList;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import org.apache.commons.lang.Validate;
import org.bukkit.Bukkit;
import org.bukkit.OfflinePlayer;
import org.bukkit.command.CommandSender;
import org.bukkit.util.StringUtil;

public class BanListCommand extends VanillaCommand
{
  private static final List<String> BANLIST_TYPES = ImmutableList.of("ips", "players");
  
  public BanListCommand() {
    super("banlist");
    description = "View all players banned from this server";
    usageMessage = "/banlist [ips|players]";
    setPermission("bukkit.command.ban.list");
  }
  
  public boolean execute(CommandSender sender, String currentAlias, String[] args)
  {
    if (!testPermission(sender)) { return true;
    }
    
    StringBuilder message = new StringBuilder();
    OfflinePlayer[] banlist = (OfflinePlayer[])Bukkit.getServer().getBannedPlayers().toArray(new OfflinePlayer[0]);
    
    for (int x = 0; x < banlist.length; x++) {
      if (x != 0) {
        if (x == banlist.length - 1) {
          message.append(" and ");
        } else {
          message.append(", ");
        }
      }
      message.append(banlist[x].getName());
    }
    
    sender.sendMessage("There are " + banlist.length + " total banned players:");
    sender.sendMessage(message.toString());
    return true;
  }
  
  public List<String> tabComplete(CommandSender sender, String alias, String[] args)
  {
    Validate.notNull(sender, "Sender cannot be null");
    Validate.notNull(args, "Arguments cannot be null");
    Validate.notNull(alias, "Alias cannot be null");
    
    if (args.length == 1) {
      return (List)StringUtil.copyPartialMatches(args[0], BANLIST_TYPES, new ArrayList(BANLIST_TYPES.size()));
    }
    return ImmutableList.of();
  }
}
