package org.bukkit.command.defaults;

import com.google.common.collect.ImmutableList;
import java.util.List;
import org.apache.commons.lang.Validate;
import org.bukkit.Bukkit;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;

public class ListCommand extends VanillaCommand
{
  public ListCommand()
  {
    super("list");
    description = "Lists all online players";
    usageMessage = "/list";
    setPermission("bukkit.command.list");
  }
  
  public boolean execute(CommandSender sender, String currentAlias, String[] args)
  {
    if (!testPermission(sender)) { return true;
    }
    StringBuilder online = new StringBuilder();
    
    Player[] players = Bukkit.getOnlinePlayers();
    
    for (Player player : players)
    {
      if ((!(sender instanceof Player)) || (((Player)sender).canSee(player)))
      {

        if (online.length() > 0) {
          online.append(", ");
        }
        
        online.append(player.getDisplayName());
      }
    }
    sender.sendMessage("There are " + players.length + "/" + Bukkit.getMaxPlayers() + " players online:\n" + online.toString());
    
    return true;
  }
  
  public List<String> tabComplete(CommandSender sender, String alias, String[] args) throws IllegalArgumentException
  {
    Validate.notNull(sender, "Sender cannot be null");
    Validate.notNull(args, "Arguments cannot be null");
    Validate.notNull(alias, "Alias cannot be null");
    
    return ImmutableList.of();
  }
}
