package org.bukkit.command.defaults;

import com.google.common.collect.ImmutableList;
import java.io.File;
import java.io.IOException;
import java.io.PrintStream;
import java.util.List;
import org.apache.commons.lang.Validate;
import org.bukkit.ChatColor;
import org.bukkit.Server;
import org.bukkit.command.CommandSender;
import org.bukkit.event.Event;
import org.bukkit.event.HandlerList;
import org.bukkit.plugin.Plugin;
import org.bukkit.plugin.PluginDescriptionFile;
import org.bukkit.plugin.PluginManager;
import org.bukkit.plugin.RegisteredListener;
import org.bukkit.plugin.TimedRegisteredListener;
import org.bukkit.util.StringUtil;

public class TimingsCommand extends BukkitCommand
{
  private static final List<String> TIMINGS_SUBCOMMANDS = ImmutableList.of("merged", "reset", "separate");
  
  public TimingsCommand(String name) {
    super(name);
    description = "Records timings for all plugin events";
    usageMessage = "/timings <reset|merged|separate>";
    setPermission("bukkit.command.timings");
  }
  
  public boolean execute(CommandSender sender, String currentAlias, String[] args)
  {
    if (!testPermission(sender)) return true;
    if (args.length != 1) {
      sender.sendMessage(ChatColor.RED + "Usage: " + usageMessage);
      return false;
    }
    if (!sender.getServer().getPluginManager().useTimings()) {
      sender.sendMessage("Please enable timings by setting \"settings.plugin-profiling\" to true in bukkit.yml");
      return true;
    }
    
    boolean separate = "separate".equals(args[0]);
    if ("reset".equals(args[0])) {
      for (HandlerList handlerList : HandlerList.getHandlerLists()) {
        for (RegisteredListener listener : handlerList.getRegisteredListeners()) {
          if ((listener instanceof TimedRegisteredListener)) {
            ((TimedRegisteredListener)listener).reset();
          }
        }
      }
      sender.sendMessage("Timings reset");
    } else if (("merged".equals(args[0])) || (separate))
    {
      int index = 0;
      int pluginIdx = 0;
      File timingFolder = new File("timings");
      timingFolder.mkdirs();
      File timings = new File(timingFolder, "timings.txt");
      File names = null;
      while (timings.exists()) timings = new File(timingFolder, "timings" + ++index + ".txt");
      PrintStream fileTimings = null;
      PrintStream fileNames = null;
      try {
        fileTimings = new PrintStream(timings);
        if (separate) {
          names = new File(timingFolder, "names" + index + ".txt");
          fileNames = new PrintStream(names);
        }
        for (Plugin plugin : org.bukkit.Bukkit.getPluginManager().getPlugins()) {
          pluginIdx++;
          long totalTime = 0L;
          if (separate) {
            fileNames.println(pluginIdx + " " + plugin.getDescription().getFullName());
            fileTimings.println("Plugin " + pluginIdx);
          } else {
            fileTimings.println(plugin.getDescription().getFullName()); }
          for (RegisteredListener listener : HandlerList.getRegisteredListeners(plugin)) {
            if ((listener instanceof TimedRegisteredListener)) {
              TimedRegisteredListener trl = (TimedRegisteredListener)listener;
              long time = trl.getTotalTime();
              int count = trl.getCount();
              if (count != 0) {
                long avg = time / count;
                totalTime += time;
                Class<? extends Event> eventClass = trl.getEventClass();
                if ((count > 0) && (eventClass != null))
                  fileTimings.println("    " + eventClass.getSimpleName() + (trl.hasMultiple() ? " (and sub-classes)" : "") + " Time: " + time + " Count: " + count + " Avg: " + avg);
              }
            }
          }
          fileTimings.println("    Total time " + totalTime + " (" + totalTime / 1000000000L + "s)");
        }
        sender.sendMessage("Timings written to " + timings.getPath());
        if (separate) sender.sendMessage("Names written to " + names.getPath());
      }
      catch (IOException e) {}finally {
        if (fileTimings != null) {
          fileTimings.close();
        }
        if (fileNames != null) {
          fileNames.close();
        }
      }
    }
    return true;
  }
  
  public List<String> tabComplete(CommandSender sender, String alias, String[] args)
  {
    Validate.notNull(sender, "Sender cannot be null");
    Validate.notNull(args, "Arguments cannot be null");
    Validate.notNull(alias, "Alias cannot be null");
    
    if (args.length == 1) {
      return (List)StringUtil.copyPartialMatches(args[0], TIMINGS_SUBCOMMANDS, new java.util.ArrayList(TIMINGS_SUBCOMMANDS.size()));
    }
    return ImmutableList.of();
  }
}
