package org.bukkit.command.defaults;

import com.google.common.collect.ImmutableList;
import java.util.List;
import org.apache.commons.lang.Validate;
import org.bukkit.Bukkit;
import org.bukkit.ChatColor;
import org.bukkit.World;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;

public class ToggleDownfallCommand extends VanillaCommand
{
  public ToggleDownfallCommand()
  {
    super("toggledownfall");
    description = "Toggles rain on/off on a given world";
    usageMessage = "/toggledownfall";
    setPermission("bukkit.command.toggledownfall");
  }
  
  public boolean execute(CommandSender sender, String currentAlias, String[] args)
  {
    if (!testPermission(sender)) { return true;
    }
    World world = null;
    
    if (args.length == 1) {
      world = Bukkit.getWorld(args[0]);
      
      if (world == null) {
        sender.sendMessage(ChatColor.RED + "No world exists with the name '" + args[0] + "'");
        return true;
      }
    } else if ((sender instanceof Player)) {
      world = ((Player)sender).getWorld();
    } else {
      world = (World)Bukkit.getWorlds().get(0);
    }
    
    Command.broadcastCommandMessage(sender, "Toggling downfall " + (world.hasStorm() ? "off" : "on") + " for world '" + world.getName() + "'");
    world.setStorm(!world.hasStorm());
    
    return true;
  }
  
  public List<String> tabComplete(CommandSender sender, String alias, String[] args) throws IllegalArgumentException
  {
    Validate.notNull(sender, "Sender cannot be null");
    Validate.notNull(args, "Arguments cannot be null");
    Validate.notNull(alias, "Alias cannot be null");
    
    return ImmutableList.of();
  }
}
