package org.bukkit.craftbukkit.libs.com.google.gson.stream;

import java.io.Closeable;
import java.io.EOFException;
import java.io.IOException;
import java.io.Reader;
import org.bukkit.craftbukkit.libs.com.google.gson.internal.JsonReaderInternalAccess;
import org.bukkit.craftbukkit.libs.com.google.gson.internal.bind.JsonTreeReader;





















































































































































































public class JsonReader
  implements Closeable
{
  private static final char[] NON_EXECUTE_PREFIX = ")]}'\n".toCharArray();
  
  private static final String TRUE = "true";
  
  private static final String FALSE = "false";
  private final StringPool stringPool = new StringPool();
  

  private final Reader in;
  

  private boolean lenient = false;
  






  private final char[] buffer = new char[1024];
  private int pos = 0;
  private int limit = 0;
  



  private int bufferStartLine = 1;
  private int bufferStartColumn = 1;
  



  private JsonScope[] stack = new JsonScope[32];
  private int stackSize = 0;
  
  public JsonReader(Reader in) { push(JsonScope.EMPTY_DOCUMENT);
    



















    skipping = false;
    




    if (in == null) {
      throw new NullPointerException("in == null");
    }
    this.in = in;
  }
  




  private JsonToken token;
  


  private String name;
  


  private String value;
  


  private int valuePos;
  


  private int valueLength;
  


  private boolean skipping;
  


  public final void setLenient(boolean lenient)
  {
    this.lenient = lenient;
  }
  


  public final boolean isLenient()
  {
    return lenient;
  }
  


  public void beginArray()
    throws IOException
  {
    expect(JsonToken.BEGIN_ARRAY);
  }
  


  public void endArray()
    throws IOException
  {
    expect(JsonToken.END_ARRAY);
  }
  


  public void beginObject()
    throws IOException
  {
    expect(JsonToken.BEGIN_OBJECT);
  }
  


  public void endObject()
    throws IOException
  {
    expect(JsonToken.END_OBJECT);
  }
  

  private void expect(JsonToken expected)
    throws IOException
  {
    peek();
    if (token != expected) {
      throw new IllegalStateException("Expected " + expected + " but was " + peek() + " at line " + getLineNumber() + " column " + getColumnNumber());
    }
    
    advance();
  }
  

  public boolean hasNext()
    throws IOException
  {
    peek();
    return (token != JsonToken.END_OBJECT) && (token != JsonToken.END_ARRAY);
  }
  

  public JsonToken peek()
    throws IOException
  {
    if (token != null) {
      return token;
    }
    
    switch (2.$SwitchMap$com$google$gson$stream$JsonScope[stack[(stackSize - 1)].ordinal()]) {
    case 1: 
      if (lenient) {
        consumeNonExecutePrefix();
      }
      stack[(stackSize - 1)] = JsonScope.NONEMPTY_DOCUMENT;
      JsonToken firstToken = nextValue();
      if ((!lenient) && (token != JsonToken.BEGIN_ARRAY) && (token != JsonToken.BEGIN_OBJECT)) {
        throw new IOException("Expected JSON document to start with '[' or '{' but was " + token + " at line " + getLineNumber() + " column " + getColumnNumber());
      }
      
      return firstToken;
    case 2: 
      return nextInArray(true);
    case 3: 
      return nextInArray(false);
    case 4: 
      return nextInObject(true);
    case 5: 
      return objectValue();
    case 6: 
      return nextInObject(false);
    case 7: 
      int c = nextNonWhitespace(false);
      if (c == -1) {
        return JsonToken.END_DOCUMENT;
      }
      pos -= 1;
      if (!lenient) {
        throw syntaxError("Expected EOF");
      }
      return nextValue();
    case 8: 
      throw new IllegalStateException("JsonReader is closed");
    }
    throw new AssertionError();
  }
  



  private void consumeNonExecutePrefix()
    throws IOException
  {
    nextNonWhitespace(true);
    pos -= 1;
    
    if ((pos + NON_EXECUTE_PREFIX.length > limit) && (!fillBuffer(NON_EXECUTE_PREFIX.length))) {
      return;
    }
    
    for (int i = 0; i < NON_EXECUTE_PREFIX.length; i++) {
      if (buffer[(pos + i)] != NON_EXECUTE_PREFIX[i]) {
        return;
      }
    }
    

    pos += NON_EXECUTE_PREFIX.length;
  }
  

  private JsonToken advance()
    throws IOException
  {
    peek();
    
    JsonToken result = token;
    token = null;
    value = null;
    name = null;
    return result;
  }
  





  public String nextName()
    throws IOException
  {
    peek();
    if (token != JsonToken.NAME) {
      throw new IllegalStateException("Expected a name but was " + peek() + " at line " + getLineNumber() + " column " + getColumnNumber());
    }
    
    String result = name;
    advance();
    return result;
  }
  






  public String nextString()
    throws IOException
  {
    peek();
    if ((token != JsonToken.STRING) && (token != JsonToken.NUMBER)) {
      throw new IllegalStateException("Expected a string but was " + peek() + " at line " + getLineNumber() + " column " + getColumnNumber());
    }
    

    String result = value;
    advance();
    return result;
  }
  





  public boolean nextBoolean()
    throws IOException
  {
    peek();
    if (token != JsonToken.BOOLEAN) {
      throw new IllegalStateException("Expected a boolean but was " + token + " at line " + getLineNumber() + " column " + getColumnNumber());
    }
    

    boolean result = value == "true";
    advance();
    return result;
  }
  





  public void nextNull()
    throws IOException
  {
    peek();
    if (token != JsonToken.NULL) {
      throw new IllegalStateException("Expected null but was " + token + " at line " + getLineNumber() + " column " + getColumnNumber());
    }
    

    advance();
  }
  







  public double nextDouble()
    throws IOException
  {
    peek();
    if ((token != JsonToken.STRING) && (token != JsonToken.NUMBER)) {
      throw new IllegalStateException("Expected a double but was " + token + " at line " + getLineNumber() + " column " + getColumnNumber());
    }
    

    double result = Double.parseDouble(value);
    
    if ((result >= 1.0D) && (value.startsWith("0"))) {
      throw new MalformedJsonException("JSON forbids octal prefixes: " + value + " at line " + getLineNumber() + " column " + getColumnNumber());
    }
    
    if ((!lenient) && ((Double.isNaN(result)) || (Double.isInfinite(result)))) {
      throw new MalformedJsonException("JSON forbids NaN and infinities: " + value + " at line " + getLineNumber() + " column " + getColumnNumber());
    }
    

    advance();
    return result;
  }
  








  public long nextLong()
    throws IOException
  {
    peek();
    if ((token != JsonToken.STRING) && (token != JsonToken.NUMBER)) {
      throw new IllegalStateException("Expected a long but was " + token + " at line " + getLineNumber() + " column " + getColumnNumber());
    }
    
    long result;
    try
    {
      result = Long.parseLong(value);
    } catch (NumberFormatException ignored) {
      double asDouble = Double.parseDouble(value);
      result = asDouble;
      if (result != asDouble) {
        throw new NumberFormatException("Expected a long but was " + value + " at line " + getLineNumber() + " column " + getColumnNumber());
      }
    }
    

    if ((result >= 1L) && (value.startsWith("0"))) {
      throw new MalformedJsonException("JSON forbids octal prefixes: " + value + " at line " + getLineNumber() + " column " + getColumnNumber());
    }
    

    advance();
    return result;
  }
  








  public int nextInt()
    throws IOException
  {
    peek();
    if ((token != JsonToken.STRING) && (token != JsonToken.NUMBER)) {
      throw new IllegalStateException("Expected an int but was " + token + " at line " + getLineNumber() + " column " + getColumnNumber());
    }
    
    int result;
    try
    {
      result = Integer.parseInt(value);
    } catch (NumberFormatException ignored) {
      double asDouble = Double.parseDouble(value);
      result = (int)asDouble;
      if (result != asDouble) {
        throw new NumberFormatException("Expected an int but was " + value + " at line " + getLineNumber() + " column " + getColumnNumber());
      }
    }
    

    if ((result >= 1L) && (value.startsWith("0"))) {
      throw new MalformedJsonException("JSON forbids octal prefixes: " + value + " at line " + getLineNumber() + " column " + getColumnNumber());
    }
    

    advance();
    return result;
  }
  

  public void close()
    throws IOException
  {
    value = null;
    token = null;
    stack[0] = JsonScope.CLOSED;
    stackSize = 1;
    in.close();
  }
  



  public void skipValue()
    throws IOException
  {
    skipping = true;
    try {
      int count = 0;
      do {
        JsonToken token = advance();
        if ((token == JsonToken.BEGIN_ARRAY) || (token == JsonToken.BEGIN_OBJECT)) {
          count++;
        } else if ((token == JsonToken.END_ARRAY) || (token == JsonToken.END_OBJECT)) {
          count--;
        }
      } while (count != 0);
    } finally {
      skipping = false;
    }
  }
  
  private void push(JsonScope newTop) {
    if (stackSize == stack.length) {
      JsonScope[] newStack = new JsonScope[stackSize * 2];
      System.arraycopy(stack, 0, newStack, 0, stackSize);
      stack = newStack;
    }
    stack[(stackSize++)] = newTop;
  }
  
  private JsonToken nextInArray(boolean firstElement) throws IOException
  {
    if (firstElement) {
      stack[(stackSize - 1)] = JsonScope.NONEMPTY_ARRAY;
    }
    else {
      switch (nextNonWhitespace(true)) {
      case 93: 
        stackSize -= 1;
        return this.token = JsonToken.END_ARRAY;
      case 59: 
        checkLenient();
      case 44: 
        break;
      default: 
        throw syntaxError("Unterminated array");
      }
      
    }
    switch (nextNonWhitespace(true)) {
    case 93: 
      if (firstElement) {
        stackSize -= 1;
        return this.token = JsonToken.END_ARRAY;
      }
    

    case 44: 
    case 59: 
      checkLenient();
      pos -= 1;
      value = "null";
      return this.token = JsonToken.NULL;
    }
    pos -= 1;
    return nextValue();
  }
  





  private JsonToken nextInObject(boolean firstElement)
    throws IOException
  {
    if (firstElement)
    {
      switch (nextNonWhitespace(true)) {
      case 125: 
        stackSize -= 1;
        return this.token = JsonToken.END_OBJECT;
      }
      pos -= 1;
    }
    else {
      switch (nextNonWhitespace(true)) {
      case 125: 
        stackSize -= 1;
        return this.token = JsonToken.END_OBJECT;
      case 44: 
      case 59: 
        break;
      default: 
        throw syntaxError("Unterminated object");
      }
      
    }
    
    int quote = nextNonWhitespace(true);
    switch (quote) {
    case 39: 
      checkLenient();
    case 34: 
      name = nextString((char)quote);
      break;
    default: 
      checkLenient();
      pos -= 1;
      name = nextLiteral(false);
      if (name.length() == 0) {
        throw syntaxError("Expected name");
      }
      break;
    }
    stack[(stackSize - 1)] = JsonScope.DANGLING_NAME;
    return this.token = JsonToken.NAME;
  }
  


  private JsonToken objectValue()
    throws IOException
  {
    switch (nextNonWhitespace(true)) {
    case 58: 
      break;
    case 61: 
      checkLenient();
      if (((pos < limit) || (fillBuffer(1))) && (buffer[pos] == '>')) {
        pos += 1;
      }
      break;
    default: 
      throw syntaxError("Expected ':'");
    }
    
    stack[(stackSize - 1)] = JsonScope.NONEMPTY_OBJECT;
    return nextValue();
  }
  
  private JsonToken nextValue() throws IOException
  {
    int c = nextNonWhitespace(true);
    switch (c) {
    case 123: 
      push(JsonScope.EMPTY_OBJECT);
      return this.token = JsonToken.BEGIN_OBJECT;
    
    case 91: 
      push(JsonScope.EMPTY_ARRAY);
      return this.token = JsonToken.BEGIN_ARRAY;
    
    case 39: 
      checkLenient();
    case 34: 
      value = nextString((char)c);
      return this.token = JsonToken.STRING;
    }
    
    pos -= 1;
    return readLiteral();
  }
  




  private boolean fillBuffer(int minimum)
    throws IOException
  {
    char[] buffer = this.buffer;
    


    int line = bufferStartLine;
    int column = bufferStartColumn;
    int i = 0; for (int p = pos; i < p; i++) {
      if (buffer[i] == '\n') {
        line++;
        column = 1;
      } else {
        column++;
      }
    }
    bufferStartLine = line;
    bufferStartColumn = column;
    
    if (limit != pos) {
      limit -= pos;
      System.arraycopy(buffer, pos, buffer, 0, limit);
    } else {
      limit = 0;
    }
    
    pos = 0;
    int total;
    while ((total = in.read(buffer, limit, buffer.length - limit)) != -1) {
      limit += total;
      

      if ((bufferStartLine == 1) && (bufferStartColumn == 1) && (limit > 0) && (buffer[0] == 65279)) {
        pos += 1;
        bufferStartColumn -= 1;
      }
      
      if (limit >= minimum) {
        return true;
      }
    }
    return false;
  }
  
  private int getLineNumber() {
    int result = bufferStartLine;
    for (int i = 0; i < pos; i++) {
      if (buffer[i] == '\n') {
        result++;
      }
    }
    return result;
  }
  
  private int getColumnNumber() {
    int result = bufferStartColumn;
    for (int i = 0; i < pos; i++) {
      if (buffer[i] == '\n') {
        result = 1;
      } else {
        result++;
      }
    }
    return result;
  }
  






  private int nextNonWhitespace(boolean throwOnEof)
    throws IOException
  {
    char[] buffer = this.buffer;
    int p = pos;
    int l = limit;
    int c;
    for (;;) { if (p == l) {
        pos = p;
        if (!fillBuffer(1)) {
          break;
        }
        p = pos;
        l = limit;
      }
      
      c = buffer[(p++)];
      switch (c)
      {
      case 9: 
      case 10: 
      case 13: 
      case 32: 
        break;
      case 47: 
        pos = p;
        if ((p == l) && (!fillBuffer(1))) {
          return c;
        }
        
        checkLenient();
        char peek = buffer[pos];
        switch (peek)
        {
        case '*': 
          pos += 1;
          if (!skipTo("*/")) {
            throw syntaxError("Unterminated comment");
          }
          p = pos + 2;
          l = limit;
          break;
        

        case '/': 
          pos += 1;
          skipToEndOfLine();
          p = pos;
          l = limit;
          break;
        
        default: 
          return c;
        }
        break;
      case 35: 
        pos = p;
        




        checkLenient();
        skipToEndOfLine();
        p = pos;
        l = limit;
      }
      
    }
    pos = p;
    return c;
    

    if (throwOnEof) {
      throw new EOFException("End of input at line " + getLineNumber() + " column " + getColumnNumber());
    }
    
    return -1;
  }
  
  private void checkLenient() throws IOException
  {
    if (!lenient) {
      throw syntaxError("Use JsonReader.setLenient(true) to accept malformed JSON");
    }
  }
  



  private void skipToEndOfLine()
    throws IOException
  {
    while ((pos < limit) || (fillBuffer(1))) {
      char c = buffer[(pos++)];
      if ((c == '\r') || (c == '\n')) {
        break;
      }
    }
  }
  
  private boolean skipTo(String toFind) throws IOException {
    label67:
    for (; (pos + toFind.length() <= limit) || (fillBuffer(toFind.length())); pos += 1) {
      for (int c = 0; c < toFind.length(); c++) {
        if (buffer[(pos + c)] != toFind.charAt(c)) {
          break label67;
        }
      }
      return true;
    }
    return false;
  }
  









  private String nextString(char quote)
    throws IOException
  {
    char[] buffer = this.buffer;
    StringBuilder builder = null;
    for (;;) {
      int p = pos;
      int l = limit;
      
      int start = p;
      while (p < l) {
        int c = buffer[(p++)];
        
        if (c == quote) {
          pos = p;
          if (skipping)
            return "skipped!";
          if (builder == null) {
            return stringPool.get(buffer, start, p - start - 1);
          }
          builder.append(buffer, start, p - start - 1);
          return builder.toString();
        }
        
        if (c == 92) {
          pos = p;
          if (builder == null) {
            builder = new StringBuilder();
          }
          builder.append(buffer, start, p - start - 1);
          builder.append(readEscapeCharacter());
          p = pos;
          l = limit;
          start = p;
        }
      }
      
      if (builder == null) {
        builder = new StringBuilder();
      }
      builder.append(buffer, start, p - start);
      pos = p;
      if (!fillBuffer(1)) {
        throw syntaxError("Unterminated string");
      }
    }
  }
  







  private String nextLiteral(boolean assignOffsetsOnly)
    throws IOException
  {
    StringBuilder builder = null;
    valuePos = -1;
    valueLength = 0;
    int i = 0;
    
    for (;;)
    {
      if (pos + i < limit) {
        switch (buffer[(pos + i)]) {
        case '#': 
        case '/': 
        case ';': 
        case '=': 
        case '\\': 
          checkLenient();
        case '\t': 
        case '\n': 
        case '\f': 
        case '\r': 
        case ' ': 
        case ',': 
        case ':': 
        case '[': 
        case ']': 
        case '{': 
        case '}': 
          break;
        default: 
          i++; break;
        























        }
        
      }
      else if (i < buffer.length) {
        if (!fillBuffer(i + 1))
        {

          buffer[limit] = '\000';
        }
        
      }
      else
      {
        if (builder == null) {
          builder = new StringBuilder();
        }
        builder.append(buffer, pos, i);
        valueLength += i;
        pos += i;
        i = 0;
        if (!fillBuffer(1))
          break;
      }
    }
    String result;
    String result;
    if ((assignOffsetsOnly) && (builder == null)) {
      valuePos = pos;
      result = null; } else { String result;
      if (skipping) {
        result = "skipped!"; } else { String result;
        if (builder == null) {
          result = stringPool.get(buffer, pos, i);
        } else {
          builder.append(buffer, pos, i);
          result = builder.toString();
        } } }
    valueLength += i;
    pos += i;
    return result;
  }
  
  public String toString() {
    return getClass().getSimpleName() + " near " + getSnippet();
  }
  







  private char readEscapeCharacter()
    throws IOException
  {
    if ((pos == limit) && (!fillBuffer(1))) {
      throw syntaxError("Unterminated escape sequence");
    }
    
    char escaped = buffer[(pos++)];
    switch (escaped) {
    case 'u': 
      if ((pos + 4 > limit) && (!fillBuffer(4))) {
        throw syntaxError("Unterminated escape sequence");
      }
      
      char result = '\000';
      int i = pos; for (int end = i + 4; i < end; i++) {
        char c = buffer[i];
        result = (char)(result << '\004');
        if ((c >= '0') && (c <= '9')) {
          result = (char)(result + (c - '0'));
        } else if ((c >= 'a') && (c <= 'f')) {
          result = (char)(result + (c - 'a' + 10));
        } else if ((c >= 'A') && (c <= 'F')) {
          result = (char)(result + (c - 'A' + 10));
        } else {
          throw new NumberFormatException("\\u" + stringPool.get(buffer, pos, 4));
        }
      }
      pos += 4;
      return result;
    
    case 't': 
      return '\t';
    
    case 'b': 
      return '\b';
    
    case 'n': 
      return '\n';
    
    case 'r': 
      return '\r';
    
    case 'f': 
      return '\f';
    }
    
    


    return escaped;
  }
  


  private JsonToken readLiteral()
    throws IOException
  {
    value = nextLiteral(true);
    if (valueLength == 0) {
      throw syntaxError("Expected literal value");
    }
    token = decodeLiteral();
    if (token == JsonToken.STRING) {
      checkLenient();
    }
    return token;
  }
  

  private JsonToken decodeLiteral()
    throws IOException
  {
    if (valuePos == -1)
    {
      return JsonToken.STRING; }
    if ((valueLength == 4) && (('n' == buffer[valuePos]) || ('N' == buffer[valuePos])) && (('u' == buffer[(valuePos + 1)]) || ('U' == buffer[(valuePos + 1)])) && (('l' == buffer[(valuePos + 2)]) || ('L' == buffer[(valuePos + 2)])) && (('l' == buffer[(valuePos + 3)]) || ('L' == buffer[(valuePos + 3)])))
    {



      value = "null";
      return JsonToken.NULL; }
    if ((valueLength == 4) && (('t' == buffer[valuePos]) || ('T' == buffer[valuePos])) && (('r' == buffer[(valuePos + 1)]) || ('R' == buffer[(valuePos + 1)])) && (('u' == buffer[(valuePos + 2)]) || ('U' == buffer[(valuePos + 2)])) && (('e' == buffer[(valuePos + 3)]) || ('E' == buffer[(valuePos + 3)])))
    {



      value = "true";
      return JsonToken.BOOLEAN; }
    if ((valueLength == 5) && (('f' == buffer[valuePos]) || ('F' == buffer[valuePos])) && (('a' == buffer[(valuePos + 1)]) || ('A' == buffer[(valuePos + 1)])) && (('l' == buffer[(valuePos + 2)]) || ('L' == buffer[(valuePos + 2)])) && (('s' == buffer[(valuePos + 3)]) || ('S' == buffer[(valuePos + 3)])) && (('e' == buffer[(valuePos + 4)]) || ('E' == buffer[(valuePos + 4)])))
    {




      value = "false";
      return JsonToken.BOOLEAN;
    }
    value = stringPool.get(buffer, valuePos, valueLength);
    return decodeNumber(buffer, valuePos, valueLength);
  }
  






  private JsonToken decodeNumber(char[] chars, int offset, int length)
  {
    int i = offset;
    int c = chars[i];
    
    if (c == 45) {
      c = chars[(++i)];
    }
    
    if (c == 48) {
      c = chars[(++i)];
    } else { if ((c >= 49) && (c <= 57))
        c = chars[(++i)];
      while ((c >= 48) && (c <= 57)) {
        c = chars[(++i)]; continue;
        

        return JsonToken.STRING;
      }
    }
    if (c == 46) {
      c = chars[(++i)];
      while ((c >= 48) && (c <= 57)) {
        c = chars[(++i)];
      }
    }
    
    if ((c == 101) || (c == 69)) {
      c = chars[(++i)];
      if ((c == 43) || (c == 45)) {
        c = chars[(++i)];
      }
      if ((c >= 48) && (c <= 57))
        c = chars[(++i)];
      while ((c >= 48) && (c <= 57)) {
        c = chars[(++i)]; continue;
        

        return JsonToken.STRING;
      }
    }
    
    if (i == offset + length) {
      return JsonToken.NUMBER;
    }
    return JsonToken.STRING;
  }
  



  private IOException syntaxError(String message)
    throws IOException
  {
    throw new MalformedJsonException(message + " at line " + getLineNumber() + " column " + getColumnNumber());
  }
  
  private CharSequence getSnippet()
  {
    StringBuilder snippet = new StringBuilder();
    int beforePos = Math.min(pos, 20);
    snippet.append(buffer, pos - beforePos, beforePos);
    int afterPos = Math.min(limit - pos, 20);
    snippet.append(buffer, pos, afterPos);
    return snippet;
  }
  
  static {
    JsonReaderInternalAccess.INSTANCE = new JsonReaderInternalAccess() {
      public void promoteNameToValue(JsonReader reader) throws IOException {
        if ((reader instanceof JsonTreeReader)) {
          ((JsonTreeReader)reader).promoteNameToValue();
          return;
        }
        reader.peek();
        if (token != JsonToken.NAME) {
          throw new IllegalStateException("Expected a name but was " + reader.peek() + " " + " at line " + reader.getLineNumber() + " column " + reader.getColumnNumber());
        }
        
        value = name;
        name = null;
        token = JsonToken.STRING;
      }
    };
  }
}
