package org.bukkit.craftbukkit.libs.com.google.gson.stream;

import java.io.Closeable;
import java.io.IOException;
import java.io.Writer;
import java.util.ArrayList;
import java.util.List;




















































































































public class JsonWriter
  implements Closeable
{
  private final Writer out;
  private final List<JsonScope> stack = new ArrayList();
  
  public JsonWriter(Writer out) { stack.add(JsonScope.EMPTY_DOCUMENT);
    










    separator = ":";
    






    serializeNulls = true;
    






    if (out == null) {
      throw new NullPointerException("out == null");
    }
    this.out = out;
  }
  

  private String indent;
  
  private String separator;
  
  private boolean lenient;
  
  public final void setIndent(String indent)
  {
    if (indent.length() == 0) {
      this.indent = null;
      separator = ":";
    } else {
      this.indent = indent;
      separator = ": ";
    }
  }
  


  private boolean htmlSafe;
  

  private String deferredName;
  

  private boolean serializeNulls;
  

  public final void setLenient(boolean lenient)
  {
    this.lenient = lenient;
  }
  


  public boolean isLenient()
  {
    return lenient;
  }
  






  public final void setHtmlSafe(boolean htmlSafe)
  {
    this.htmlSafe = htmlSafe;
  }
  



  public final boolean isHtmlSafe()
  {
    return htmlSafe;
  }
  



  public final void setSerializeNulls(boolean serializeNulls)
  {
    this.serializeNulls = serializeNulls;
  }
  



  public final boolean getSerializeNulls()
  {
    return serializeNulls;
  }
  




  public JsonWriter beginArray()
    throws IOException
  {
    writeDeferredName();
    return open(JsonScope.EMPTY_ARRAY, "[");
  }
  



  public JsonWriter endArray()
    throws IOException
  {
    return close(JsonScope.EMPTY_ARRAY, JsonScope.NONEMPTY_ARRAY, "]");
  }
  




  public JsonWriter beginObject()
    throws IOException
  {
    writeDeferredName();
    return open(JsonScope.EMPTY_OBJECT, "{");
  }
  



  public JsonWriter endObject()
    throws IOException
  {
    return close(JsonScope.EMPTY_OBJECT, JsonScope.NONEMPTY_OBJECT, "}");
  }
  


  private JsonWriter open(JsonScope empty, String openBracket)
    throws IOException
  {
    beforeValue(true);
    stack.add(empty);
    out.write(openBracket);
    return this;
  }
  



  private JsonWriter close(JsonScope empty, JsonScope nonempty, String closeBracket)
    throws IOException
  {
    JsonScope context = peek();
    if ((context != nonempty) && (context != empty)) {
      throw new IllegalStateException("Nesting problem: " + stack);
    }
    if (deferredName != null) {
      throw new IllegalStateException("Dangling name: " + deferredName);
    }
    
    stack.remove(stack.size() - 1);
    if (context == nonempty) {
      newline();
    }
    out.write(closeBracket);
    return this;
  }
  


  private JsonScope peek()
  {
    return (JsonScope)stack.get(stack.size() - 1);
  }
  


  private void replaceTop(JsonScope topOfStack)
  {
    stack.set(stack.size() - 1, topOfStack);
  }
  




  public JsonWriter name(String name)
    throws IOException
  {
    if (name == null) {
      throw new NullPointerException("name == null");
    }
    if (deferredName != null) {
      throw new IllegalStateException();
    }
    deferredName = name;
    return this;
  }
  
  private void writeDeferredName() throws IOException {
    if (deferredName != null) {
      beforeName();
      string(deferredName);
      deferredName = null;
    }
  }
  




  public JsonWriter value(String value)
    throws IOException
  {
    if (value == null) {
      return nullValue();
    }
    writeDeferredName();
    beforeValue(false);
    string(value);
    return this;
  }
  



  public JsonWriter nullValue()
    throws IOException
  {
    if (deferredName != null) {
      if (serializeNulls) {
        writeDeferredName();
      } else {
        deferredName = null;
        return this;
      }
    }
    beforeValue(false);
    out.write("null");
    return this;
  }
  



  public JsonWriter value(boolean value)
    throws IOException
  {
    writeDeferredName();
    beforeValue(false);
    out.write(value ? "true" : "false");
    return this;
  }
  





  public JsonWriter value(double value)
    throws IOException
  {
    if ((Double.isNaN(value)) || (Double.isInfinite(value))) {
      throw new IllegalArgumentException("Numeric values must be finite, but was " + value);
    }
    writeDeferredName();
    beforeValue(false);
    out.append(Double.toString(value));
    return this;
  }
  



  public JsonWriter value(long value)
    throws IOException
  {
    writeDeferredName();
    beforeValue(false);
    out.write(Long.toString(value));
    return this;
  }
  





  public JsonWriter value(Number value)
    throws IOException
  {
    if (value == null) {
      return nullValue();
    }
    
    writeDeferredName();
    String string = value.toString();
    if ((!lenient) && ((string.equals("-Infinity")) || (string.equals("Infinity")) || (string.equals("NaN"))))
    {
      throw new IllegalArgumentException("Numeric values must be finite, but was " + value);
    }
    beforeValue(false);
    out.append(string);
    return this;
  }
  


  public void flush()
    throws IOException
  {
    out.flush();
  }
  



  public void close()
    throws IOException
  {
    out.close();
    
    if (peek() != JsonScope.NONEMPTY_DOCUMENT) {
      throw new IOException("Incomplete document");
    }
  }
  
  private void string(String value) throws IOException {
    out.write("\"");
    int i = 0; for (int length = value.length(); i < length; i++) {
      char c = value.charAt(i);
      










      switch (c) {
      case '"': 
      case '\\': 
        out.write(92);
        out.write(c);
        break;
      
      case '\t': 
        out.write("\\t");
        break;
      
      case '\b': 
        out.write("\\b");
        break;
      
      case '\n': 
        out.write("\\n");
        break;
      
      case '\r': 
        out.write("\\r");
        break;
      
      case '\f': 
        out.write("\\f");
        break;
      
      case '&': 
      case '\'': 
      case '<': 
      case '=': 
      case '>': 
        if (htmlSafe) {
          out.write(String.format("\\u%04x", new Object[] { Integer.valueOf(c) }));
        } else {
          out.write(c);
        }
        break;
      
      case ' ': 
      case ' ': 
        out.write(String.format("\\u%04x", new Object[] { Integer.valueOf(c) }));
        break;
      
      default: 
        if (c <= '\037') {
          out.write(String.format("\\u%04x", new Object[] { Integer.valueOf(c) }));
        } else {
          out.write(c);
        }
        break;
      }
    }
    out.write("\"");
  }
  
  private void newline() throws IOException {
    if (indent == null) {
      return;
    }
    
    out.write("\n");
    for (int i = 1; i < stack.size(); i++) {
      out.write(indent);
    }
  }
  


  private void beforeName()
    throws IOException
  {
    JsonScope context = peek();
    if (context == JsonScope.NONEMPTY_OBJECT) {
      out.write(44);
    } else if (context != JsonScope.EMPTY_OBJECT) {
      throw new IllegalStateException("Nesting problem: " + stack);
    }
    newline();
    replaceTop(JsonScope.DANGLING_NAME);
  }
  






  private void beforeValue(boolean root)
    throws IOException
  {
    switch (1.$SwitchMap$com$google$gson$stream$JsonScope[peek().ordinal()]) {
    case 1: 
      if ((!lenient) && (!root)) {
        throw new IllegalStateException("JSON must start with an array or an object.");
      }
      
      replaceTop(JsonScope.NONEMPTY_DOCUMENT);
      break;
    
    case 2: 
      replaceTop(JsonScope.NONEMPTY_ARRAY);
      newline();
      break;
    
    case 3: 
      out.append(',');
      newline();
      break;
    
    case 4: 
      out.append(separator);
      replaceTop(JsonScope.NONEMPTY_OBJECT);
      break;
    
    case 5: 
      throw new IllegalStateException("JSON must have only one top-level value.");
    

    default: 
      throw new IllegalStateException("Nesting problem: " + stack);
    }
  }
}
