package org.bukkit.craftbukkit.libs.jline;

import java.text.MessageFormat;
import java.util.HashMap;
import java.util.Map;
import org.bukkit.craftbukkit.libs.jline.internal.Configuration;
import org.bukkit.craftbukkit.libs.jline.internal.Log;





















public class TerminalFactory
{
  public static final String JLINE_TERMINAL = "org.bukkit.craftbukkit.libs.jline.terminal";
  public static final String AUTO = "auto";
  public static final String UNIX = "unix";
  public static final String WIN = "win";
  public static final String WINDOWS = "windows";
  public static final String NONE = "none";
  public static final String OFF = "off";
  public static final String FALSE = "false";
  private static final InheritableThreadLocal<Terminal> holder;
  private static final Map<Flavor, Class<? extends Terminal>> FLAVORS;
  
  public static synchronized Terminal create()
  {
    if (Log.TRACE)
    {
      Log.trace(new Object[] { new Throwable("CREATE MARKER") });
    }
    
    String type = Configuration.getString("org.bukkit.craftbukkit.libs.jline.terminal", "auto");
    
    Log.debug(new Object[] { "Creating terminal; type=", type });
    Terminal t;
    try
    {
      String tmp = type.toLowerCase();
      Terminal t;
      if (tmp.equals("unix")) {
        t = getFlavor(Flavor.UNIX);
      } else { Terminal t;
        if ((tmp.equals("win") | tmp.equals("windows"))) {
          t = getFlavor(Flavor.WINDOWS);
        } else { Terminal t;
          if ((tmp.equals("none")) || (tmp.equals("off")) || (tmp.equals("false"))) {
            t = new UnsupportedTerminal();
          } else {
            Terminal t;
            if (tmp.equals("auto")) {
              String os = Configuration.getOsName();
              Flavor flavor = Flavor.UNIX;
              if (os.contains("windows")) {
                flavor = Flavor.WINDOWS;
              }
              t = getFlavor(flavor);
            }
            else {
              try {
                t = (Terminal)Thread.currentThread().getContextClassLoader().loadClass(type).newInstance();
              }
              catch (Exception e) {
                throw new IllegalArgumentException(MessageFormat.format("Invalid terminal type: {0}", new Object[] { type }), e);
              }
            }
          }
        }
      }
    } catch (Exception e) { Log.error(new Object[] { "Failed to construct terminal; falling back to unsupported", e });
      t = new UnsupportedTerminal();
    }
    
    Log.debug(new Object[] { "Created Terminal: ", t });
    try
    {
      t.init();
    }
    catch (Exception e) {
      Log.error(new Object[] { "Terminal initialization failed; falling back to unsupported", e });
      return new UnsupportedTerminal();
    }
    
    return t;
  }
  
  public static synchronized void reset() {
    holder.remove();
  }
  
  public static synchronized void resetIf(Terminal t) {
    if (holder.get() == t) {
      reset();
    }
  }
  
  public static enum Type
  {
    AUTO, 
    WINDOWS, 
    UNIX, 
    NONE;
    
    private Type() {} }
  
  public static synchronized void configure(String type) { assert (type != null);
    System.setProperty("org.bukkit.craftbukkit.libs.jline.terminal", type);
  }
  
  public static synchronized void configure(Type type) {
    assert (type != null);
    configure(type.name().toLowerCase());
  }
  




  public static enum Flavor
  {
    WINDOWS, 
    UNIX;
    
    private Flavor() {}
  }
  
  static
  {
    holder = new InheritableThreadLocal();
    


































































































    FLAVORS = new HashMap();
    

    registerFlavor(Flavor.WINDOWS, AnsiWindowsTerminal.class);
    registerFlavor(Flavor.UNIX, UnixTerminal.class);
  }
  
  public static synchronized Terminal get() {
    Terminal t = (Terminal)holder.get();
    if (t == null) {
      t = create();
      holder.set(t);
    }
    return t;
  }
  
  public static Terminal getFlavor(Flavor flavor) throws Exception {
    Class<? extends Terminal> type = (Class)FLAVORS.get(flavor);
    if (type != null) {
      return (Terminal)type.newInstance();
    }
    
    throw new InternalError();
  }
  
  public static void registerFlavor(Flavor flavor, Class<? extends Terminal> type) {
    FLAVORS.put(flavor, type);
  }
}
