package org.bukkit.craftbukkit.libs.jline;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import org.bukkit.craftbukkit.libs.jline.internal.Configuration;
import org.bukkit.craftbukkit.libs.jline.internal.Log;






















public abstract class TerminalSupport
  implements Terminal
{
  public static final String JLINE_SHUTDOWNHOOK = "org.bukkit.craftbukkit.libs.jline.shutdownhook";
  public static final int DEFAULT_WIDTH = 80;
  public static final int DEFAULT_HEIGHT = 24;
  private Thread shutdownHook;
  private boolean shutdownHookEnabled;
  private boolean supported;
  private boolean echoEnabled;
  private boolean ansiSupported;
  private Configuration configuration;
  
  protected TerminalSupport(boolean supported)
  {
    this.supported = supported;
    shutdownHookEnabled = Configuration.getBoolean("org.bukkit.craftbukkit.libs.jline.shutdownhook", false);
  }
  
  public void init() throws Exception {
    installShutdownHook(new RestoreHook());
  }
  
  public void restore() throws Exception {
    TerminalFactory.resetIf(this);
    removeShutdownHook();
  }
  
  public void reset() throws Exception {
    restore();
    init();
  }
  

  protected void installShutdownHook(Thread hook)
  {
    if (!shutdownHookEnabled) {
      Log.debug(new Object[] { "Not install shutdown hook " + hook + " because they are disabled." });
      return;
    }
    
    assert (hook != null);
    
    if (shutdownHook != null) {
      throw new IllegalStateException("Shutdown hook already installed");
    }
    try
    {
      Runtime.getRuntime().addShutdownHook(hook);
      shutdownHook = hook;
    }
    catch (AbstractMethodError e)
    {
      Log.trace(new Object[] { "Failed to register shutdown hook: ", e });
    }
  }
  
  protected void removeShutdownHook() {
    if (!shutdownHookEnabled) {
      return;
    }
    if (shutdownHook != null) {
      try {
        Runtime.getRuntime().removeShutdownHook(shutdownHook);
      }
      catch (AbstractMethodError e)
      {
        Log.trace(new Object[] { "Failed to remove shutdown hook: ", e });
      }
      catch (IllegalStateException e) {}
      

      shutdownHook = null;
    }
  }
  
  public final boolean isSupported() {
    return supported;
  }
  
  public synchronized boolean isAnsiSupported() {
    return ansiSupported;
  }
  
  protected synchronized void setAnsiSupported(boolean supported) {
    ansiSupported = supported;
    Log.debug(new Object[] { "Ansi supported: ", Boolean.valueOf(supported) });
  }
  



  public OutputStream wrapOutIfNeeded(OutputStream out)
  {
    return out;
  }
  


  public boolean hasWeirdWrap()
  {
    return true;
  }
  
  public int getWidth() {
    return 80;
  }
  
  public int getHeight() {
    return 24;
  }
  
  public synchronized boolean isEchoEnabled() {
    return echoEnabled;
  }
  
  public synchronized void setEchoEnabled(boolean enabled) {
    echoEnabled = enabled;
    Log.debug(new Object[] { "Echo enabled: ", Boolean.valueOf(enabled) });
  }
  
  public InputStream wrapInIfNeeded(InputStream in) throws IOException {
    return in;
  }
  
  protected class RestoreHook
    extends Thread
  {
    protected RestoreHook() {}
    
    public void start()
    {
      try
      {
        restore();
      }
      catch (Exception e) {
        Log.trace(new Object[] { "Failed to restore: ", e });
      }
    }
  }
}
