package org.bukkit.craftbukkit.libs.jline;

import java.io.FileDescriptor;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import org.bukkit.craftbukkit.libs.jline.internal.Configuration;
import org.bukkit.craftbukkit.libs.jline.internal.Log;
import org.fusesource.jansi.internal.WindowsSupport;












































public class WindowsTerminal
  extends TerminalSupport
{
  public static final String JLINE_WINDOWS_TERMINAL_DIRECT_CONSOLE = "org.bukkit.craftbukkit.libs.jline.WindowsTerminal.directConsole";
  public static final String ANSI = WindowsTerminal.class.getName() + ".ansi";
  private boolean directConsole;
  private int originalMode;
  
  public WindowsTerminal()
    throws Exception
  {
    super(true);
  }
  
  public void init() throws Exception
  {
    super.init();
    
    setAnsiSupported(Configuration.getBoolean(ANSI, true));
    




    setDirectConsole(Configuration.getBoolean("org.bukkit.craftbukkit.libs.jline.WindowsTerminal.directConsole", true));
    
    originalMode = getConsoleMode();
    setConsoleMode(originalMode & (ENABLE_ECHO_INPUTcode ^ 0xFFFFFFFF));
    setEchoEnabled(false);
  }
  





  public void restore()
    throws Exception
  {
    setConsoleMode(originalMode);
    super.restore();
  }
  
  public int getWidth()
  {
    int w = getWindowsTerminalWidth();
    return w < 1 ? 80 : w;
  }
  
  public int getHeight()
  {
    int h = getWindowsTerminalHeight();
    return h < 1 ? 24 : h;
  }
  

  public void setEchoEnabled(boolean enabled)
  {
    if (enabled) {
      setConsoleMode(getConsoleMode() | ENABLE_ECHO_INPUTcode | ENABLE_LINE_INPUTcode | ENABLE_PROCESSED_INPUTcode | ENABLE_WINDOW_INPUTcode);


    }
    else
    {

      setConsoleMode(getConsoleMode() & ((ENABLE_LINE_INPUTcode | ENABLE_ECHO_INPUTcode | ENABLE_PROCESSED_INPUTcode | ENABLE_WINDOW_INPUTcode) ^ 0xFFFFFFFF));
    }
    



    super.setEchoEnabled(enabled);
  }
  


  public void setDirectConsole(boolean flag)
  {
    directConsole = flag;
    Log.debug(new Object[] { "Direct console: ", Boolean.valueOf(flag) });
  }
  


  public Boolean getDirectConsole()
  {
    return Boolean.valueOf(directConsole);
  }
  
  public InputStream wrapInIfNeeded(InputStream in)
    throws IOException
  {
    if ((directConsole) && (isSystemIn(in))) {
      new InputStream()
      {
        public int read() throws IOException {
          return WindowsTerminal.this.readByte();
        }
      };
    }
    return super.wrapInIfNeeded(in);
  }
  
  protected boolean isSystemIn(InputStream in) throws IOException
  {
    assert (in != null);
    
    if (in == System.in) {
      return true;
    }
    if (((in instanceof FileInputStream)) && (((FileInputStream)in).getFD() == FileDescriptor.in)) {
      return true;
    }
    
    return false;
  }
  


  private int getConsoleMode()
  {
    return WindowsSupport.getConsoleMode();
  }
  
  private void setConsoleMode(int mode) {
    WindowsSupport.setConsoleMode(mode);
  }
  
  private int readByte() {
    return WindowsSupport.readByte();
  }
  
  private int getWindowsTerminalWidth() {
    return WindowsSupport.getWindowsTerminalWidth();
  }
  
  private int getWindowsTerminalHeight() {
    return WindowsSupport.getWindowsTerminalHeight();
  }
  










  public static enum ConsoleMode
  {
    ENABLE_LINE_INPUT(2), 
    





    ENABLE_ECHO_INPUT(4), 
    








    ENABLE_PROCESSED_INPUT(1), 
    







    ENABLE_WINDOW_INPUT(8), 
    






    ENABLE_MOUSE_INPUT(16), 
    







    ENABLE_PROCESSED_OUTPUT(1), 
    





    ENABLE_WRAP_AT_EOL_OUTPUT(2);
    
    public final int code;
    
    private ConsoleMode(int code) {
      this.code = code;
    }
  }
}
