package org.bukkit.craftbukkit.libs.jline.console;

import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileDescriptor;
import java.io.FileInputStream;
import java.io.FilterInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Reader;
import java.io.Writer;
import java.net.URL;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.ResourceBundle;
import org.bukkit.craftbukkit.libs.jline.Terminal;
import org.bukkit.craftbukkit.libs.jline.TerminalFactory;
import org.bukkit.craftbukkit.libs.jline.console.completer.CandidateListCompletionHandler;
import org.bukkit.craftbukkit.libs.jline.console.completer.Completer;
import org.bukkit.craftbukkit.libs.jline.console.completer.CompletionHandler;
import org.bukkit.craftbukkit.libs.jline.console.history.History;
import org.bukkit.craftbukkit.libs.jline.console.history.History.Entry;
import org.bukkit.craftbukkit.libs.jline.console.history.MemoryHistory;
import org.bukkit.craftbukkit.libs.jline.internal.Configuration;
import org.bukkit.craftbukkit.libs.jline.internal.InputStreamReader;
import org.bukkit.craftbukkit.libs.jline.internal.Log;
import org.fusesource.jansi.AnsiOutputStream;



























public class ConsoleReader
{
  public static final String JLINE_NOBELL = "org.bukkit.craftbukkit.libs.jline.nobell";
  public static final char BACKSPACE = '\b';
  public static final char RESET_LINE = '\r';
  public static final char KEYBOARD_BELL = '\007';
  public static final char NULL_MASK = '\000';
  public static final int TAB_WIDTH = 4;
  private static final ResourceBundle resources = ResourceBundle.getBundle(CandidateListCompletionHandler.class.getName());
  
  private final Terminal terminal;
  
  private InputStream in;
  
  private final Writer out;
  
  private final CursorBuffer buf = new CursorBuffer();
  
  private String prompt;
  
  private boolean expandEvents = true;
  
  private Character mask;
  
  private Character echoCharacter;
  
  private StringBuffer searchTerm = null;
  
  private String previousSearchTerm = "";
  
  private int searchIndex = -1;
  
  private Reader reader;
  
  private String encoding;
  
  private boolean recording;
  
  private String macro = "";
  
  private String appName;
  
  private URL inputrcUrl;
  
  private ConsoleKeys consoleKeys;
  
  private boolean skipLF = false;
  public static final String JLINE_COMPLETION_THRESHOLD = "org.bukkit.craftbukkit.libs.jline.completion.threshold";
  
  public ConsoleReader() throws IOException { this(null, new FileInputStream(FileDescriptor.in), System.out, null); }
  

  public ConsoleReader(InputStream in, OutputStream out)
    throws IOException
  {
    this(null, in, out, null);
  }
  
  public ConsoleReader(InputStream in, OutputStream out, Terminal term)
    throws IOException
  {
    this(null, in, out, term);
  }
  
  public ConsoleReader(String appName, InputStream in, OutputStream out, Terminal term)
    throws IOException
  {
    this.appName = (appName != null ? appName : "JLine");
    encoding = (encoding != null ? encoding : Configuration.getEncoding());
    terminal = (term != null ? term : TerminalFactory.get());
    this.out = new OutputStreamWriter(terminal.wrapOutIfNeeded(out), encoding);
    setInput(in);
    
    inputrcUrl = Configuration.getUrlFrom(Configuration.getString("org.bukkit.craftbukkit.libs.jline.inputrc", Configuration.getUrlFrom(new File(Configuration.getUserHome(), ".inputrc")).toExternalForm()));
    
    consoleKeys = new ConsoleKeys(appName, inputrcUrl);
  }
  
  public KeyMap getKeys() {
    return consoleKeys.getKeys();
  }
  
  void setInput(InputStream in) throws IOException {
    InputStream wrapped = terminal.wrapInIfNeeded(in);
    
    this.in = new FilterInputStream(wrapped)
    {
      public int read(byte[] b, int off, int len) throws IOException {
        if (b == null)
          throw new NullPointerException();
        if ((off < 0) || (len < 0) || (len > b.length - off))
          throw new IndexOutOfBoundsException();
        if (len == 0) {
          return 0;
        }
        
        int c = read();
        if (c == -1) {
          return -1;
        }
        b[off] = ((byte)c);
        return 1;
      }
    };
    reader = new InputStreamReader(this.in, encoding);
  }
  
  public InputStream getInput() {
    return in;
  }
  
  public Writer getOutput() {
    return out;
  }
  
  public Terminal getTerminal() {
    return terminal;
  }
  
  public CursorBuffer getCursorBuffer() {
    return buf;
  }
  
  public void setExpandEvents(boolean expand) {
    expandEvents = expand;
  }
  
  public boolean getExpandEvents() {
    return expandEvents;
  }
  
  public void setPrompt(String prompt) {
    this.prompt = prompt;
  }
  
  public String getPrompt() {
    return prompt;
  }
  






















  public void setEchoCharacter(Character c)
  {
    echoCharacter = c;
  }
  


  public Character getEchoCharacter()
  {
    return echoCharacter;
  }
  



  protected final boolean resetLine()
    throws IOException
  {
    if (buf.cursor == 0) {
      return false;
    }
    
    backspaceAll();
    
    return true;
  }
  
  int getCursorPosition()
  {
    String prompt = getPrompt();
    return (prompt == null ? 0 : stripAnsi(lastLine(prompt)).length()) + buf.cursor;
  }
  




  private String lastLine(String str)
  {
    if (str == null) return "";
    int last = str.lastIndexOf("\n");
    
    if (last >= 0) {
      return str.substring(last + 1, str.length());
    }
    
    return str;
  }
  
  private String stripAnsi(String str) {
    if (str == null) return "";
    try {
      ByteArrayOutputStream baos = new ByteArrayOutputStream();
      AnsiOutputStream aos = new AnsiOutputStream(baos);
      aos.write(str.getBytes());
      aos.flush();
      return baos.toString();
    } catch (IOException e) {}
    return str;
  }
  


  public final boolean setCursorPosition(int position)
    throws IOException
  {
    return moveCursor(position - buf.cursor) != 0;
  }
  





  private void setBuffer(String buffer)
    throws IOException
  {
    if (buffer.equals(buf.buffer.toString())) {
      return;
    }
    

    int sameIndex = 0;
    
    int i = 0;int l1 = buffer.length();int l2 = buf.buffer.length();
    for (; (i < l1) && (i < l2); i++) {
      if (buffer.charAt(i) != buf.buffer.charAt(i)) break;
      sameIndex++;
    }
    




    int diff = buf.cursor - sameIndex;
    if (diff < 0) {
      moveToEnd();
      diff = buf.buffer.length() - sameIndex;
    }
    
    backspace(diff);
    killLine();
    buf.buffer.setLength(sameIndex);
    putString(buffer.substring(sameIndex));
  }
  
  private void setBuffer(CharSequence buffer) throws IOException {
    setBuffer(String.valueOf(buffer));
  }
  

  public final void drawLine()
    throws IOException
  {
    String prompt = getPrompt();
    if (prompt != null) {
      print(prompt);
    }
    
    print(buf.buffer.toString());
    
    if (buf.length() != buf.cursor) {
      back(buf.length() - buf.cursor - 1);
    }
    
    drawBuffer();
  }
  

  public final void redrawLine()
    throws IOException
  {
    print(13);
    
    drawLine();
  }
  



  final String finishBuffer()
    throws IOException
  {
    String str = buf.buffer.toString();
    String historyLine = str;
    
    if (expandEvents) {
      str = expandEvents(str);
      historyLine = str.replaceAll("\\!", "\\\\!");
    }
    



    if (str.length() > 0) {
      if ((mask == null) && (isHistoryEnabled())) {
        history.add(historyLine);
      }
      else {
        mask = null;
      }
    }
    
    history.moveToEnd();
    
    buf.buffer.setLength(0);
    buf.cursor = 0;
    
    return str;
  }
  





  protected String expandEvents(String str)
    throws IOException
  {
    StringBuilder sb = new StringBuilder();
    boolean escaped = false;
    for (int i = 0; i < str.length(); i++) {
      char c = str.charAt(i);
      if (escaped) {
        sb.append(c);
        escaped = false;
      }
      else if (c == '\\') {
        escaped = true;
      }
      else {
        escaped = false;
        
        switch (c) {
        case '!': 
          if (i + 1 < str.length()) {
            c = str.charAt(++i);
            boolean neg = false;
            String rep = null;
            int i1;
            int idx; switch (c) {
            case '!': 
              if (history.size() == 0) {
                throw new IllegalArgumentException("!!: event not found");
              }
              rep = history.get(history.index() - 1).toString();
              break;
            case '#': 
              sb.append(sb.toString());
              break;
            case '?': 
              i1 = str.indexOf('?', i + 1);
              if (i1 < 0) {
                i1 = str.length();
              }
              String sc = str.substring(i + 1, i1);
              i = i1;
              idx = searchBackwards(sc);
              if (idx < 0) {
                throw new IllegalArgumentException("!?" + sc + ": event not found");
              }
              rep = history.get(idx).toString();
              
              break;
            case '\t': 
            case ' ': 
              sb.append('!');
              sb.append(c);
              break;
            case '-': 
              neg = true;
              i++;
            
            case '0': 
            case '1': 
            case '2': 
            case '3': 
            case '4': 
            case '5': 
            case '6': 
            case '7': 
            case '8': 
            case '9': 
              i1 = i;
              for (; i < str.length(); i++) {
                c = str.charAt(i);
                if ((c < '0') || (c > '9')) {
                  break;
                }
              }
              idx = 0;
              try {
                idx = Integer.parseInt(str.substring(i1, i));
              } catch (NumberFormatException e) {
                throw new IllegalArgumentException((neg ? "!-" : "!") + str.substring(i1, i) + ": event not found");
              }
              if (neg) {
                if (idx < history.size()) {
                  rep = history.get(history.index() - idx).toString();
                } else {
                  throw new IllegalArgumentException((neg ? "!-" : "!") + str.substring(i1, i) + ": event not found");
                }
              }
              else if ((idx >= history.index() - history.size()) && (idx < history.index())) {
                rep = history.get(idx).toString();
              } else {
                throw new IllegalArgumentException((neg ? "!-" : "!") + str.substring(i1, i) + ": event not found");
              }
              break;
            case '\n': case '\013': case '\f': case '\r': case '\016': case '\017': case '\020': case '\021': case '\022': case '\023': case '\024': case '\025': case '\026': case '\027': case '\030': case '\031': case '\032': case '\033': case '\034': case '\035': 
            case '\036': case '\037': case '"': case '$': case '%': case '&': case '\'': case '(': case ')': case '*': case '+': case ',': case '.': case '/': case ':': case ';': case '<': case '=': case '>': default: 
              String ss = str.substring(i);
              i = str.length();
              idx = searchBackwards(ss, history.index(), true);
              if (idx < 0) {
                throw new IllegalArgumentException("!" + ss + ": event not found");
              }
              rep = history.get(idx).toString();
            }
            
            
            if (rep != null) {
              sb.append(rep);
            }
          } else {
            sb.append(c);
          }
          break;
        case '^': 
          if (i == 0) {
            int i1 = str.indexOf('^', i + 1);
            int i2 = str.indexOf('^', i1 + 1);
            if (i2 < 0) {
              i2 = str.length();
            }
            if ((i1 > 0) && (i2 > 0)) {
              String s1 = str.substring(i + 1, i1);
              String s2 = str.substring(i1 + 1, i2);
              String s = history.get(history.index() - 1).toString().replace(s1, s2);
              sb.append(s);
              i = i2 + 1;
              continue;
            }
          }
          sb.append(c);
          break;
        default: 
          sb.append(c);
        }
      }
    }
    if (escaped) {
      sb.append('\\');
    }
    String result = sb.toString();
    if (!str.equals(result)) {
      print(result);
      println();
      flush();
    }
    return result;
  }
  


  public final void putString(CharSequence str)
    throws IOException
  {
    buf.write(str);
    if (mask == null)
    {
      print(str);
    } else if (mask.charValue() != 0)
    {

      print(mask.charValue(), str.length());
    }
    drawBuffer();
  }
  





  private void drawBuffer(int clear)
    throws IOException
  {
    if ((buf.cursor != buf.length()) || (clear != 0))
    {
      char[] chars = buf.buffer.substring(buf.cursor).toCharArray();
      if (mask != null) {
        Arrays.fill(chars, mask.charValue());
      }
      if (terminal.hasWeirdWrap())
      {
        int width = terminal.getWidth();
        int pos = getCursorPosition();
        for (int i = 0; i < chars.length; i++) {
          print(chars[i]);
          if ((pos + i + 1) % width == 0) {
            print(32);
            print(13);
          }
        }
      } else {
        print(chars);
      }
      clearAhead(clear, chars.length);
      if (terminal.isAnsiSupported()) {
        if (chars.length > 0) {
          back(chars.length);
        }
      } else {
        back(chars.length);
      }
    }
    if (terminal.hasWeirdWrap()) {
      int width = terminal.getWidth();
      


      if ((getCursorPosition() > 0) && (getCursorPosition() % width == 0) && (buf.cursor == buf.length()) && (clear == 0))
      {


        print(32);
        print(13);
      }
    }
  }
  


  private void drawBuffer()
    throws IOException
  {
    drawBuffer(0);
  }
  






  private void clearAhead(int num, int delta)
    throws IOException
  {
    if (num == 0) {
      return;
    }
    
    if (terminal.isAnsiSupported()) {
      int width = terminal.getWidth();
      int screenCursorCol = getCursorPosition() + delta;
      
      printAnsiSequence("K");
      
      int curCol = screenCursorCol % width;
      int endCol = (screenCursorCol + num - 1) % width;
      int lines = num / width;
      if (endCol < curCol) lines++;
      for (int i = 0; i < lines; i++) {
        printAnsiSequence("B");
        printAnsiSequence("2K");
      }
      for (int i = 0; i < lines; i++) {
        printAnsiSequence("A");
      }
      return;
    }
    

    print(' ', num);
    





    back(num);
  }
  



  protected void back(int num)
    throws IOException
  {
    if (num == 0) return;
    if (terminal.isAnsiSupported()) {
      int width = getTerminal().getWidth();
      int cursor = getCursorPosition();
      int realCursor = cursor + num;
      int realCol = realCursor % width;
      int newCol = cursor % width;
      int moveup = num / width;
      int delta = realCol - newCol;
      if (delta < 0) moveup++;
      if (moveup > 0) {
        printAnsiSequence(moveup + "A");
      }
      printAnsiSequence(1 + newCol + "G");
      return;
    }
    print('\b', num);
  }
  



  public void flush()
    throws IOException
  {
    out.flush();
  }
  
  private int backspaceAll() throws IOException {
    return backspace(2147483647);
  }
  



  private int backspace(int num)
    throws IOException
  {
    if (buf.cursor == 0) {
      return 0;
    }
    
    int count = 0;
    
    int termwidth = getTerminal().getWidth();
    int lines = getCursorPosition() / termwidth;
    count = moveCursor(-1 * num) * -1;
    buf.buffer.delete(buf.cursor, buf.cursor + count);
    if ((getCursorPosition() / termwidth != lines) && 
      (terminal.isAnsiSupported()))
    {
      printAnsiSequence("K");
    }
    

















    drawBuffer(count);
    
    return count;
  }
  



  public boolean backspace()
    throws IOException
  {
    return backspace(1) == 1;
  }
  
  protected boolean moveToEnd() throws IOException {
    return moveCursor(buf.length() - buf.cursor) > 0;
  }
  

  private boolean deleteCurrentCharacter()
    throws IOException
  {
    if ((buf.length() == 0) || (buf.cursor == buf.length())) {
      return false;
    }
    
    buf.buffer.deleteCharAt(buf.cursor);
    drawBuffer(1);
    return true;
  }
  
  private boolean previousWord() throws IOException {
    while ((isDelimiter(buf.current())) && (moveCursor(-1) != 0)) {}
    


    while ((!isDelimiter(buf.current())) && (moveCursor(-1) != 0)) {}
    


    return true;
  }
  
  private boolean nextWord() throws IOException {
    while ((isDelimiter(buf.nextChar())) && (moveCursor(1) != 0)) {}
    


    while ((!isDelimiter(buf.nextChar())) && (moveCursor(1) != 0)) {}
    


    return true;
  }
  
  private boolean deletePreviousWord() throws IOException {
    while ((isDelimiter(buf.current())) && (backspace())) {}
    


    while ((!isDelimiter(buf.current())) && (backspace())) {}
    


    return true;
  }
  
  private boolean deleteNextWord() throws IOException {
    while ((isDelimiter(buf.nextChar())) && (delete())) {}
    


    while ((!isDelimiter(buf.nextChar())) && (delete())) {}
    


    return true;
  }
  
  private boolean capitalizeWord() throws IOException {
    boolean first = true;
    int i = 1;
    char c;
    while ((buf.cursor + i - 1 < buf.length()) && (!isDelimiter(c = buf.buffer.charAt(buf.cursor + i - 1)))) {
      buf.buffer.setCharAt(buf.cursor + i - 1, first ? Character.toUpperCase(c) : Character.toLowerCase(c));
      first = false;
      i++;
    }
    drawBuffer();
    moveCursor(i - 1);
    return true;
  }
  
  private boolean upCaseWord() throws IOException {
    int i = 1;
    char c;
    while ((buf.cursor + i - 1 < buf.length()) && (!isDelimiter(c = buf.buffer.charAt(buf.cursor + i - 1)))) {
      buf.buffer.setCharAt(buf.cursor + i - 1, Character.toUpperCase(c));
      i++;
    }
    drawBuffer();
    moveCursor(i - 1);
    return true;
  }
  
  private boolean downCaseWord() throws IOException {
    int i = 1;
    char c;
    while ((buf.cursor + i - 1 < buf.length()) && (!isDelimiter(c = buf.buffer.charAt(buf.cursor + i - 1)))) {
      buf.buffer.setCharAt(buf.cursor + i - 1, Character.toLowerCase(c));
      i++;
    }
    drawBuffer();
    moveCursor(i - 1);
    return true;
  }
  




  public int moveCursor(int num)
    throws IOException
  {
    int where = num;
    
    if ((buf.cursor == 0) && (where <= 0)) {
      return 0;
    }
    
    if ((buf.cursor == buf.buffer.length()) && (where >= 0)) {
      return 0;
    }
    
    if (buf.cursor + where < 0) {
      where = -buf.cursor;
    }
    else if (buf.cursor + where > buf.buffer.length()) {
      where = buf.buffer.length() - buf.cursor;
    }
    
    moveInternal(where);
    
    return where;
  }
  





  private void moveInternal(int where)
    throws IOException
  {
    buf.cursor += where;
    
    if (terminal.isAnsiSupported()) {
      if (where < 0) {
        back(Math.abs(where));
      } else {
        int width = getTerminal().getWidth();
        int cursor = getCursorPosition();
        int oldLine = (cursor - where) / width;
        int newLine = cursor / width;
        if (newLine > oldLine) {
          if (terminal.hasWeirdWrap())
          {



            if (getCurrentAnsiRow() == terminal.getHeight()) {
              printAnsiSequence(newLine - oldLine + "S");
            }
          }
          printAnsiSequence(newLine - oldLine + "B");
        }
        printAnsiSequence(1 + cursor % width + "G");
      }
      
      return;
    }
    


    if (where < 0) {
      int len = 0;
      for (int i = buf.cursor; i < buf.cursor - where; i++) {
        if (buf.buffer.charAt(i) == '\t') {
          len += 4;
        }
        else {
          len++;
        }
      }
      
      char[] chars = new char[len];
      Arrays.fill(chars, '\b');
      out.write(chars);
      
      return;
    }
    if (buf.cursor == 0)
      return;
    char c;
    if (mask != null) {
      c = mask.charValue();
    }
    else {
      print(buf.buffer.substring(buf.cursor - where, buf.cursor).toCharArray()); return;
    }
    
    char c;
    
    if (mask.charValue() == 0) {
      return;
    }
    
    print(c, Math.abs(where));
  }
  

  public final boolean replace(int num, String replacement)
  {
    buf.buffer.replace(buf.cursor - num, buf.cursor, replacement);
    try {
      moveCursor(-num);
      drawBuffer(Math.max(0, num - replacement.length()));
      moveCursor(replacement.length());
    }
    catch (IOException e) {
      e.printStackTrace();
      return false;
    }
    return true;
  }
  



  public final int readCharacter()
    throws IOException
  {
    int c = reader.read();
    if (c >= 0) {
      Log.trace(new Object[] { "Keystroke: ", Integer.valueOf(c) });
      
      clearEcho(c);
    }
    return c;
  }
  


  private int clearEcho(int c)
    throws IOException
  {
    if (!terminal.isEchoEnabled()) {
      return 0;
    }
    

    int num = countEchoCharacters(c);
    back(num);
    drawBuffer(num);
    
    return num;
  }
  

  private int countEchoCharacters(int c)
  {
    if (c == 9) {
      int tabStop = 8;
      int position = getCursorPosition();
      
      return tabStop - position % tabStop;
    }
    
    return getPrintableCharacters(c).length();
  }
  





  private StringBuilder getPrintableCharacters(int ch)
  {
    StringBuilder sbuff = new StringBuilder();
    
    if (ch >= 32) {
      if (ch < 127) {
        sbuff.append(ch);
      }
      else if (ch == 127) {
        sbuff.append('^');
        sbuff.append('?');
      }
      else {
        sbuff.append('M');
        sbuff.append('-');
        
        if (ch >= 160) {
          if (ch < 255) {
            sbuff.append((char)(ch - 128));
          }
          else {
            sbuff.append('^');
            sbuff.append('?');
          }
        }
        else {
          sbuff.append('^');
          sbuff.append((char)(ch - 128 + 64));
        }
      }
    }
    else {
      sbuff.append('^');
      sbuff.append((char)(ch + 64));
    }
    
    return sbuff;
  }
  

  public final int readCharacter(char... allowed)
    throws IOException
  {
    Arrays.sort(allowed);
    char c;
    while (Arrays.binarySearch(allowed, c = (char)readCharacter()) < 0) {}
    


    return c;
  }
  











  public String readLine()
    throws IOException
  {
    return readLine((String)null);
  }
  


  public String readLine(Character mask)
    throws IOException
  {
    return readLine(null, mask);
  }
  
  public String readLine(String prompt) throws IOException {
    return readLine(prompt, null);
  }
  
  /* Error */
  public String readLine(String prompt, Character mask)
    throws IOException
  {
    // Byte code:
    //   0: aload_0
    //   1: aload_2
    //   2: putfield 433	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:mask	Ljava/lang/Character;
    //   5: aload_1
    //   6: ifnull +11 -> 17
    //   9: aload_0
    //   10: aload_1
    //   11: invokevirtual 722	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:setPrompt	(Ljava/lang/String;)V
    //   14: goto +8 -> 22
    //   17: aload_0
    //   18: invokevirtual 296	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:getPrompt	()Ljava/lang/String;
    //   21: astore_1
    //   22: aload_0
    //   23: getfield 186	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:terminal	Lorg/bukkit/craftbukkit/libs/jline/Terminal;
    //   26: invokeinterface 725 1 0
    //   31: ifne +9 -> 40
    //   34: aload_0
    //   35: aload_1
    //   36: aload_2
    //   37: invokespecial 729	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:beforeReadLine	(Ljava/lang/String;Ljava/lang/Character;)V
    //   40: aload_1
    //   41: ifnull +25 -> 66
    //   44: aload_1
    //   45: invokevirtual 308	java/lang/String:length	()I
    //   48: ifle +18 -> 66
    //   51: aload_0
    //   52: getfield 199	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:out	Ljava/io/Writer;
    //   55: aload_1
    //   56: invokevirtual 731	java/io/Writer:write	(Ljava/lang/String;)V
    //   59: aload_0
    //   60: getfield 199	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:out	Ljava/io/Writer;
    //   63: invokevirtual 592	java/io/Writer:flush	()V
    //   66: aload_0
    //   67: getfield 186	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:terminal	Lorg/bukkit/craftbukkit/libs/jline/Terminal;
    //   70: invokeinterface 725 1 0
    //   75: ifne +26 -> 101
    //   78: aload_0
    //   79: invokespecial 734	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:readLineSimple	()Ljava/lang/String;
    //   82: astore_3
    //   83: aload_0
    //   84: getfield 186	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:terminal	Lorg/bukkit/craftbukkit/libs/jline/Terminal;
    //   87: invokeinterface 725 1 0
    //   92: ifne +7 -> 99
    //   95: aload_0
    //   96: invokespecial 737	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:afterReadLine	()V
    //   99: aload_3
    //   100: areturn
    //   101: aload_0
    //   102: getfield 278	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:prompt	Ljava/lang/String;
    //   105: astore_3
    //   106: iconst_1
    //   107: istore 4
    //   109: iconst_2
    //   110: istore 5
    //   112: iconst_1
    //   113: istore 6
    //   115: iconst_1
    //   116: istore 7
    //   118: new 362	java/lang/StringBuilder
    //   121: dup
    //   122: invokespecial 445	java/lang/StringBuilder:<init>	()V
    //   125: astore 8
    //   127: new 739	java/util/ArrayList
    //   130: dup
    //   131: invokespecial 740	java/util/ArrayList:<init>	()V
    //   134: astore 9
    //   136: aload 9
    //   138: invokeinterface 745 1 0
    //   143: ifeq +10 -> 153
    //   146: aload_0
    //   147: invokevirtual 708	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:readCharacter	()I
    //   150: goto +25 -> 175
    //   153: aload 9
    //   155: aload 9
    //   157: invokeinterface 746 1 0
    //   162: iconst_1
    //   163: isub
    //   164: invokeinterface 750 2 0
    //   169: checkcast 515	java/lang/Character
    //   172: invokevirtual 519	java/lang/Character:charValue	()C
    //   175: istore 10
    //   177: iload 10
    //   179: iconst_m1
    //   180: if_icmpne +25 -> 205
    //   183: aconst_null
    //   184: astore 11
    //   186: aload_0
    //   187: getfield 186	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:terminal	Lorg/bukkit/craftbukkit/libs/jline/Terminal;
    //   190: invokeinterface 725 1 0
    //   195: ifne +7 -> 202
    //   198: aload_0
    //   199: invokespecial 737	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:afterReadLine	()V
    //   202: aload 11
    //   204: areturn
    //   205: aload 8
    //   207: iload 10
    //   209: i2c
    //   210: invokevirtual 449	java/lang/StringBuilder:append	(C)Ljava/lang/StringBuilder;
    //   213: pop
    //   214: aload_0
    //   215: getfield 752	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:recording	Z
    //   218: ifeq +30 -> 248
    //   221: new 362	java/lang/StringBuilder
    //   224: dup
    //   225: invokespecial 445	java/lang/StringBuilder:<init>	()V
    //   228: aload_0
    //   229: dup_x1
    //   230: getfield 130	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:macro	Ljava/lang/String;
    //   233: invokevirtual 468	java/lang/StringBuilder:append	(Ljava/lang/String;)Ljava/lang/StringBuilder;
    //   236: iload 10
    //   238: i2c
    //   239: invokevirtual 449	java/lang/StringBuilder:append	(C)Ljava/lang/StringBuilder;
    //   242: invokevirtual 363	java/lang/StringBuilder:toString	()Ljava/lang/String;
    //   245: putfield 130	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:macro	Ljava/lang/String;
    //   248: aload_0
    //   249: invokevirtual 753	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:getKeys	()Lorg/bukkit/craftbukkit/libs/jline/console/KeyMap;
    //   252: aload 8
    //   254: invokevirtual 759	org/bukkit/craftbukkit/libs/jline/console/KeyMap:getBound	(Ljava/lang/CharSequence;)Ljava/lang/Object;
    //   257: astore 11
    //   259: aload 11
    //   261: getstatic 765	org/bukkit/craftbukkit/libs/jline/console/Operation:DO_LOWERCASE_VERSION	Lorg/bukkit/craftbukkit/libs/jline/console/Operation;
    //   264: if_acmpne +38 -> 302
    //   267: aload 8
    //   269: aload 8
    //   271: invokevirtual 368	java/lang/StringBuilder:length	()I
    //   274: iconst_1
    //   275: isub
    //   276: invokevirtual 386	java/lang/StringBuilder:setLength	(I)V
    //   279: aload 8
    //   281: iload 10
    //   283: i2c
    //   284: invokestatic 629	java/lang/Character:toLowerCase	(C)C
    //   287: invokevirtual 449	java/lang/StringBuilder:append	(C)Ljava/lang/StringBuilder;
    //   290: pop
    //   291: aload_0
    //   292: invokevirtual 753	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:getKeys	()Lorg/bukkit/craftbukkit/libs/jline/console/KeyMap;
    //   295: aload 8
    //   297: invokevirtual 759	org/bukkit/craftbukkit/libs/jline/console/KeyMap:getBound	(Ljava/lang/CharSequence;)Ljava/lang/Object;
    //   300: astore 11
    //   302: aload 11
    //   304: instanceof 755
    //   307: ifeq +6 -> 313
    //   310: goto -174 -> 136
    //   313: aload 11
    //   315: ifnonnull +91 -> 406
    //   318: aload 8
    //   320: invokevirtual 368	java/lang/StringBuilder:length	()I
    //   323: ifle +83 -> 406
    //   326: aload 8
    //   328: aload 8
    //   330: invokevirtual 368	java/lang/StringBuilder:length	()I
    //   333: iconst_1
    //   334: isub
    //   335: invokevirtual 373	java/lang/StringBuilder:charAt	(I)C
    //   338: istore 10
    //   340: aload 8
    //   342: aload 8
    //   344: invokevirtual 368	java/lang/StringBuilder:length	()I
    //   347: iconst_1
    //   348: isub
    //   349: invokevirtual 386	java/lang/StringBuilder:setLength	(I)V
    //   352: aload_0
    //   353: invokevirtual 753	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:getKeys	()Lorg/bukkit/craftbukkit/libs/jline/console/KeyMap;
    //   356: aload 8
    //   358: invokevirtual 759	org/bukkit/craftbukkit/libs/jline/console/KeyMap:getBound	(Ljava/lang/CharSequence;)Ljava/lang/Object;
    //   361: astore 12
    //   363: aload 12
    //   365: instanceof 755
    //   368: ifeq +35 -> 403
    //   371: aload 12
    //   373: checkcast 755	org/bukkit/craftbukkit/libs/jline/console/KeyMap
    //   376: invokevirtual 769	org/bukkit/craftbukkit/libs/jline/console/KeyMap:getAnotherKey	()Ljava/lang/Object;
    //   379: astore 11
    //   381: aload 11
    //   383: ifnonnull +6 -> 389
    //   386: goto -73 -> 313
    //   389: aload 9
    //   391: iload 10
    //   393: i2c
    //   394: invokestatic 772	java/lang/Character:valueOf	(C)Ljava/lang/Character;
    //   397: invokeinterface 774 2 0
    //   402: pop
    //   403: goto -90 -> 313
    //   406: aload 11
    //   408: ifnonnull +6 -> 414
    //   411: goto -275 -> 136
    //   414: iconst_2
    //   415: anewarray 4	java/lang/Object
    //   418: dup
    //   419: iconst_0
    //   420: ldc_w 776
    //   423: aastore
    //   424: dup
    //   425: iconst_1
    //   426: aload 11
    //   428: aastore
    //   429: invokestatic 687	org/bukkit/craftbukkit/libs/jline/internal/Log:trace	([Ljava/lang/Object;)V
    //   432: aload 11
    //   434: instanceof 305
    //   437: ifeq +64 -> 501
    //   440: aload 11
    //   442: checkcast 305	java/lang/String
    //   445: astore 12
    //   447: iconst_0
    //   448: istore 13
    //   450: iload 13
    //   452: aload 12
    //   454: invokevirtual 308	java/lang/String:length	()I
    //   457: if_icmpge +35 -> 492
    //   460: aload 9
    //   462: aload 12
    //   464: aload 12
    //   466: invokevirtual 308	java/lang/String:length	()I
    //   469: iconst_1
    //   470: isub
    //   471: iload 13
    //   473: isub
    //   474: invokevirtual 372	java/lang/String:charAt	(I)C
    //   477: invokestatic 772	java/lang/Character:valueOf	(C)Ljava/lang/Character;
    //   480: invokeinterface 774 2 0
    //   485: pop
    //   486: iinc 13 1
    //   489: goto -39 -> 450
    //   492: aload 8
    //   494: iconst_0
    //   495: invokevirtual 386	java/lang/StringBuilder:setLength	(I)V
    //   498: goto -362 -> 136
    //   501: aload 11
    //   503: instanceof 778
    //   506: ifeq +23 -> 529
    //   509: aload 11
    //   511: checkcast 778	java/awt/event/ActionListener
    //   514: aconst_null
    //   515: invokeinterface 782 2 0
    //   520: aload 8
    //   522: iconst_0
    //   523: invokevirtual 386	java/lang/StringBuilder:setLength	(I)V
    //   526: goto -390 -> 136
    //   529: iload 6
    //   531: iconst_2
    //   532: if_icmpne +330 -> 862
    //   535: iconst_m1
    //   536: istore 12
    //   538: getstatic 786	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader$3:$SwitchMap$jline$console$Operation	[I
    //   541: aload 11
    //   543: checkcast 761	org/bukkit/craftbukkit/libs/jline/console/Operation
    //   546: invokevirtual 789	org/bukkit/craftbukkit/libs/jline/console/Operation:ordinal	()I
    //   549: iaload
    //   550: tableswitch	default:+179 -> 729, 1:+30->580, 2:+36->586, 3:+106->656, 4:+151->701
    //   581: istore 6
    //   583: goto +195 -> 778
    //   586: aload_0
    //   587: getfield 122	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchTerm	Ljava/lang/StringBuffer;
    //   590: invokevirtual 792	java/lang/StringBuffer:length	()I
    //   593: ifne +15 -> 608
    //   596: aload_0
    //   597: getfield 122	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchTerm	Ljava/lang/StringBuffer;
    //   600: aload_0
    //   601: getfield 126	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:previousSearchTerm	Ljava/lang/String;
    //   604: invokevirtual 795	java/lang/StringBuffer:append	(Ljava/lang/String;)Ljava/lang/StringBuffer;
    //   607: pop
    //   608: aload_0
    //   609: getfield 128	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchIndex	I
    //   612: iconst_m1
    //   613: if_icmpne +21 -> 634
    //   616: aload_0
    //   617: aload_0
    //   618: aload_0
    //   619: getfield 122	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchTerm	Ljava/lang/StringBuffer;
    //   622: invokevirtual 796	java/lang/StringBuffer:toString	()Ljava/lang/String;
    //   625: invokevirtual 475	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchBackwards	(Ljava/lang/String;)I
    //   628: putfield 128	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchIndex	I
    //   631: goto +147 -> 778
    //   634: aload_0
    //   635: aload_0
    //   636: aload_0
    //   637: getfield 122	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchTerm	Ljava/lang/StringBuffer;
    //   640: invokevirtual 796	java/lang/StringBuffer:toString	()Ljava/lang/String;
    //   643: aload_0
    //   644: getfield 128	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchIndex	I
    //   647: invokevirtual 799	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchBackwards	(Ljava/lang/String;I)I
    //   650: putfield 128	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchIndex	I
    //   653: goto +125 -> 778
    //   656: aload_0
    //   657: getfield 122	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchTerm	Ljava/lang/StringBuffer;
    //   660: invokevirtual 792	java/lang/StringBuffer:length	()I
    //   663: ifle +115 -> 778
    //   666: aload_0
    //   667: getfield 122	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchTerm	Ljava/lang/StringBuffer;
    //   670: aload_0
    //   671: getfield 122	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchTerm	Ljava/lang/StringBuffer;
    //   674: invokevirtual 792	java/lang/StringBuffer:length	()I
    //   677: iconst_1
    //   678: isub
    //   679: invokevirtual 802	java/lang/StringBuffer:deleteCharAt	(I)Ljava/lang/StringBuffer;
    //   682: pop
    //   683: aload_0
    //   684: aload_0
    //   685: aload_0
    //   686: getfield 122	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchTerm	Ljava/lang/StringBuffer;
    //   689: invokevirtual 796	java/lang/StringBuffer:toString	()Ljava/lang/String;
    //   692: invokevirtual 475	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchBackwards	(Ljava/lang/String;)I
    //   695: putfield 128	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchIndex	I
    //   698: goto +80 -> 778
    //   701: aload_0
    //   702: getfield 122	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchTerm	Ljava/lang/StringBuffer;
    //   705: iload 10
    //   707: invokevirtual 805	java/lang/StringBuffer:appendCodePoint	(I)Ljava/lang/StringBuffer;
    //   710: pop
    //   711: aload_0
    //   712: aload_0
    //   713: aload_0
    //   714: getfield 122	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchTerm	Ljava/lang/StringBuffer;
    //   717: invokevirtual 796	java/lang/StringBuffer:toString	()Ljava/lang/String;
    //   720: invokevirtual 475	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchBackwards	(Ljava/lang/String;)I
    //   723: putfield 128	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchIndex	I
    //   726: goto +52 -> 778
    //   729: aload_0
    //   730: getfield 128	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchIndex	I
    //   733: iconst_m1
    //   734: if_icmpeq +41 -> 775
    //   737: aload_0
    //   738: getfield 159	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:history	Lorg/bukkit/craftbukkit/libs/jline/console/history/History;
    //   741: aload_0
    //   742: getfield 128	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchIndex	I
    //   745: invokeinterface 808 2 0
    //   750: pop
    //   751: aload_0
    //   752: getfield 159	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:history	Lorg/bukkit/craftbukkit/libs/jline/console/history/History;
    //   755: invokeinterface 811 1 0
    //   760: invokevirtual 465	java/lang/Object:toString	()Ljava/lang/String;
    //   763: aload_0
    //   764: getfield 122	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchTerm	Ljava/lang/StringBuffer;
    //   767: invokevirtual 796	java/lang/StringBuffer:toString	()Ljava/lang/String;
    //   770: invokevirtual 813	java/lang/String:indexOf	(Ljava/lang/String;)I
    //   773: istore 12
    //   775: iconst_1
    //   776: istore 6
    //   778: iload 6
    //   780: iconst_2
    //   781: if_icmpne +74 -> 855
    //   784: aload_0
    //   785: getfield 122	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchTerm	Ljava/lang/StringBuffer;
    //   788: invokevirtual 792	java/lang/StringBuffer:length	()I
    //   791: ifne +19 -> 810
    //   794: aload_0
    //   795: ldc 124
    //   797: ldc 124
    //   799: invokevirtual 817	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:printSearchStatus	(Ljava/lang/String;Ljava/lang/String;)V
    //   802: aload_0
    //   803: iconst_m1
    //   804: putfield 128	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchIndex	I
    //   807: goto +55 -> 862
    //   810: aload_0
    //   811: getfield 128	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchIndex	I
    //   814: iconst_m1
    //   815: if_icmpne +10 -> 825
    //   818: aload_0
    //   819: invokevirtual 820	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:beep	()V
    //   822: goto +40 -> 862
    //   825: aload_0
    //   826: aload_0
    //   827: getfield 122	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchTerm	Ljava/lang/StringBuffer;
    //   830: invokevirtual 796	java/lang/StringBuffer:toString	()Ljava/lang/String;
    //   833: aload_0
    //   834: getfield 159	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:history	Lorg/bukkit/craftbukkit/libs/jline/console/history/History;
    //   837: aload_0
    //   838: getfield 128	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchIndex	I
    //   841: invokeinterface 464 2 0
    //   846: invokevirtual 465	java/lang/Object:toString	()Ljava/lang/String;
    //   849: invokevirtual 817	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:printSearchStatus	(Ljava/lang/String;Ljava/lang/String;)V
    //   852: goto +10 -> 862
    //   855: aload_0
    //   856: aload_3
    //   857: iload 12
    //   859: invokevirtual 824	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:restoreLine	(Ljava/lang/String;I)V
    //   862: iload 6
    //   864: iconst_1
    //   865: if_icmpne +938 -> 1803
    //   868: aload 11
    //   870: instanceof 761
    //   873: ifeq +930 -> 1803
    //   876: getstatic 786	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader$3:$SwitchMap$jline$console$Operation	[I
    //   879: aload 11
    //   881: checkcast 761	org/bukkit/craftbukkit/libs/jline/console/Operation
    //   884: invokevirtual 789	org/bukkit/craftbukkit/libs/jline/console/Operation:ordinal	()I
    //   887: iaload
    //   888: tableswitch	default:+912 -> 1800, 2:+534->1422, 3:+345->1233, 4:+257->1145, 5:+160->1048, 6:+169->1057, 7:+179->1067, 8:+189->1077, 9:+198->1086, 10:+223->1111, 11:+232->1120, 12:+269->1157, 13:+307->1195, 14:+316->1204, 15:+325->1213, 16:+335->1223, 17:+354->1242, 18:+398->1286, 19:+407->1295, 20:+425->1313, 21:+443->1331, 22:+452->1340, 23:+452->1340, 24:+461->1349, 25:+470->1358, 26:+502->1390, 27:+669->1557, 28:+678->1566, 29:+687->1575, 30:+696->1584, 31:+705->1593, 32:+718->1606, 33:+739->1627, 34:+747->1635, 35:+780->1668, 36:+840->1728, 37:+876->1764
    //   1049: invokevirtual 827	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:complete	()Z
    //   1052: istore 7
    //   1054: goto +749 -> 1803
    //   1057: aload_0
    //   1058: invokevirtual 830	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:printCompletionCandidates	()V
    //   1061: iconst_1
    //   1062: istore 7
    //   1064: goto +739 -> 1803
    //   1067: aload_0
    //   1068: iconst_0
    //   1069: invokevirtual 832	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:setCursorPosition	(I)Z
    //   1072: istore 7
    //   1074: goto +729 -> 1803
    //   1077: aload_0
    //   1078: invokevirtual 382	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:killLine	()Z
    //   1081: istore 7
    //   1083: goto +720 -> 1803
    //   1086: aload_0
    //   1087: iconst_0
    //   1088: invokevirtual 832	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:setCursorPosition	(I)Z
    //   1091: ifeq +14 -> 1105
    //   1094: aload_0
    //   1095: invokevirtual 382	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:killLine	()Z
    //   1098: ifeq +7 -> 1105
    //   1101: iconst_1
    //   1102: goto +4 -> 1106
    //   1105: iconst_0
    //   1106: istore 7
    //   1108: goto +695 -> 1803
    //   1111: aload_0
    //   1112: invokevirtual 835	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:clearScreen	()Z
    //   1115: istore 7
    //   1117: goto +686 -> 1803
    //   1120: aload_0
    //   1121: getfield 118	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:buf	Lorg/bukkit/craftbukkit/libs/jline/console/CursorBuffer;
    //   1124: aload_0
    //   1125: getfield 118	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:buf	Lorg/bukkit/craftbukkit/libs/jline/console/CursorBuffer;
    //   1128: invokevirtual 838	org/bukkit/craftbukkit/libs/jline/console/CursorBuffer:isOverTyping	()Z
    //   1131: ifne +7 -> 1138
    //   1134: iconst_1
    //   1135: goto +4 -> 1139
    //   1138: iconst_0
    //   1139: invokevirtual 841	org/bukkit/craftbukkit/libs/jline/console/CursorBuffer:setOverTyping	(Z)V
    //   1142: goto +661 -> 1803
    //   1145: aload_0
    //   1146: aload 8
    //   1148: invokevirtual 393	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:putString	(Ljava/lang/CharSequence;)V
    //   1151: iconst_1
    //   1152: istore 7
    //   1154: goto +649 -> 1803
    //   1157: aload_0
    //   1158: invokevirtual 376	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:moveToEnd	()Z
    //   1161: pop
    //   1162: aload_0
    //   1163: invokevirtual 496	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:println	()V
    //   1166: aload_0
    //   1167: invokevirtual 497	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:flush	()V
    //   1170: aload_0
    //   1171: invokevirtual 843	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:finishBuffer	()Ljava/lang/String;
    //   1174: astore 12
    //   1176: aload_0
    //   1177: getfield 186	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:terminal	Lorg/bukkit/craftbukkit/libs/jline/Terminal;
    //   1180: invokeinterface 725 1 0
    //   1185: ifne +7 -> 1192
    //   1188: aload_0
    //   1189: invokespecial 737	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:afterReadLine	()V
    //   1192: aload 12
    //   1194: areturn
    //   1195: aload_0
    //   1196: invokespecial 845	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:previousWord	()Z
    //   1199: istore 7
    //   1201: goto +602 -> 1803
    //   1204: aload_0
    //   1205: invokespecial 847	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:nextWord	()Z
    //   1208: istore 7
    //   1210: goto +593 -> 1803
    //   1213: aload_0
    //   1214: iconst_0
    //   1215: invokespecial 851	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:moveHistory	(Z)Z
    //   1218: istore 7
    //   1220: goto +583 -> 1803
    //   1223: aload_0
    //   1224: iconst_1
    //   1225: invokespecial 851	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:moveHistory	(Z)Z
    //   1228: istore 7
    //   1230: goto +573 -> 1803
    //   1233: aload_0
    //   1234: invokevirtual 618	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:backspace	()Z
    //   1237: istore 7
    //   1239: goto +564 -> 1803
    //   1242: aload_0
    //   1243: getfield 118	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:buf	Lorg/bukkit/craftbukkit/libs/jline/console/CursorBuffer;
    //   1246: getfield 360	org/bukkit/craftbukkit/libs/jline/console/CursorBuffer:buffer	Ljava/lang/StringBuilder;
    //   1249: invokevirtual 368	java/lang/StringBuilder:length	()I
    //   1252: ifne +25 -> 1277
    //   1255: aconst_null
    //   1256: astore 12
    //   1258: aload_0
    //   1259: getfield 186	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:terminal	Lorg/bukkit/craftbukkit/libs/jline/Terminal;
    //   1262: invokeinterface 725 1 0
    //   1267: ifne +7 -> 1274
    //   1270: aload_0
    //   1271: invokespecial 737	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:afterReadLine	()V
    //   1274: aload 12
    //   1276: areturn
    //   1277: aload_0
    //   1278: invokespecial 853	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:deleteCurrentCharacter	()Z
    //   1281: istore 7
    //   1283: goto +520 -> 1803
    //   1286: aload_0
    //   1287: invokespecial 853	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:deleteCurrentCharacter	()Z
    //   1290: istore 7
    //   1292: goto +511 -> 1803
    //   1295: aload_0
    //   1296: iconst_m1
    //   1297: invokevirtual 354	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:moveCursor	(I)I
    //   1300: ifeq +7 -> 1307
    //   1303: iconst_1
    //   1304: goto +4 -> 1308
    //   1307: iconst_0
    //   1308: istore 7
    //   1310: goto +493 -> 1803
    //   1313: aload_0
    //   1314: iconst_1
    //   1315: invokevirtual 354	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:moveCursor	(I)I
    //   1318: ifeq +7 -> 1325
    //   1321: iconst_1
    //   1322: goto +4 -> 1326
    //   1325: iconst_0
    //   1326: istore 7
    //   1328: goto +475 -> 1803
    //   1331: aload_0
    //   1332: invokevirtual 855	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:resetLine	()Z
    //   1335: istore 7
    //   1337: goto +466 -> 1803
    //   1340: aload_0
    //   1341: invokespecial 857	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:deletePreviousWord	()Z
    //   1344: istore 7
    //   1346: goto +457 -> 1803
    //   1349: aload_0
    //   1350: invokespecial 859	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:deleteNextWord	()Z
    //   1353: istore 7
    //   1355: goto +448 -> 1803
    //   1358: aload_0
    //   1359: getfield 159	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:history	Lorg/bukkit/craftbukkit/libs/jline/console/history/History;
    //   1362: invokeinterface 862 1 0
    //   1367: istore 7
    //   1369: iload 7
    //   1371: ifeq +432 -> 1803
    //   1374: aload_0
    //   1375: aload_0
    //   1376: getfield 159	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:history	Lorg/bukkit/craftbukkit/libs/jline/console/history/History;
    //   1379: invokeinterface 811 1 0
    //   1384: invokespecial 864	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:setBuffer	(Ljava/lang/CharSequence;)V
    //   1387: goto +416 -> 1803
    //   1390: aload_0
    //   1391: getfield 159	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:history	Lorg/bukkit/craftbukkit/libs/jline/console/history/History;
    //   1394: invokeinterface 867 1 0
    //   1399: istore 7
    //   1401: iload 7
    //   1403: ifeq +400 -> 1803
    //   1406: aload_0
    //   1407: aload_0
    //   1408: getfield 159	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:history	Lorg/bukkit/craftbukkit/libs/jline/console/history/History;
    //   1411: invokeinterface 811 1 0
    //   1416: invokespecial 864	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:setBuffer	(Ljava/lang/CharSequence;)V
    //   1419: goto +384 -> 1803
    //   1422: aload_0
    //   1423: getfield 122	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchTerm	Ljava/lang/StringBuffer;
    //   1426: ifnull +14 -> 1440
    //   1429: aload_0
    //   1430: aload_0
    //   1431: getfield 122	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchTerm	Ljava/lang/StringBuffer;
    //   1434: invokevirtual 796	java/lang/StringBuffer:toString	()Ljava/lang/String;
    //   1437: putfield 126	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:previousSearchTerm	Ljava/lang/String;
    //   1440: aload_0
    //   1441: new 791	java/lang/StringBuffer
    //   1444: dup
    //   1445: aload_0
    //   1446: getfield 118	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:buf	Lorg/bukkit/craftbukkit/libs/jline/console/CursorBuffer;
    //   1449: getfield 360	org/bukkit/craftbukkit/libs/jline/console/CursorBuffer:buffer	Ljava/lang/StringBuilder;
    //   1452: invokespecial 869	java/lang/StringBuffer:<init>	(Ljava/lang/CharSequence;)V
    //   1455: putfield 122	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchTerm	Ljava/lang/StringBuffer;
    //   1458: iconst_2
    //   1459: istore 6
    //   1461: aload_0
    //   1462: getfield 122	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchTerm	Ljava/lang/StringBuffer;
    //   1465: invokevirtual 792	java/lang/StringBuffer:length	()I
    //   1468: ifle +73 -> 1541
    //   1471: aload_0
    //   1472: aload_0
    //   1473: aload_0
    //   1474: getfield 122	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchTerm	Ljava/lang/StringBuffer;
    //   1477: invokevirtual 796	java/lang/StringBuffer:toString	()Ljava/lang/String;
    //   1480: invokevirtual 475	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchBackwards	(Ljava/lang/String;)I
    //   1483: putfield 128	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchIndex	I
    //   1486: aload_0
    //   1487: getfield 128	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchIndex	I
    //   1490: iconst_m1
    //   1491: if_icmpne +7 -> 1498
    //   1494: aload_0
    //   1495: invokevirtual 820	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:beep	()V
    //   1498: aload_0
    //   1499: aload_0
    //   1500: getfield 122	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchTerm	Ljava/lang/StringBuffer;
    //   1503: invokevirtual 796	java/lang/StringBuffer:toString	()Ljava/lang/String;
    //   1506: aload_0
    //   1507: getfield 128	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchIndex	I
    //   1510: iconst_m1
    //   1511: if_icmple +22 -> 1533
    //   1514: aload_0
    //   1515: getfield 159	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:history	Lorg/bukkit/craftbukkit/libs/jline/console/history/History;
    //   1518: aload_0
    //   1519: getfield 128	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchIndex	I
    //   1522: invokeinterface 464 2 0
    //   1527: invokevirtual 465	java/lang/Object:toString	()Ljava/lang/String;
    //   1530: goto +5 -> 1535
    //   1533: ldc 124
    //   1535: invokevirtual 817	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:printSearchStatus	(Ljava/lang/String;Ljava/lang/String;)V
    //   1538: goto +265 -> 1803
    //   1541: aload_0
    //   1542: iconst_m1
    //   1543: putfield 128	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:searchIndex	I
    //   1546: aload_0
    //   1547: ldc 124
    //   1549: ldc 124
    //   1551: invokevirtual 817	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:printSearchStatus	(Ljava/lang/String;Ljava/lang/String;)V
    //   1554: goto +249 -> 1803
    //   1557: aload_0
    //   1558: invokespecial 871	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:capitalizeWord	()Z
    //   1561: istore 7
    //   1563: goto +240 -> 1803
    //   1566: aload_0
    //   1567: invokespecial 873	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:upCaseWord	()Z
    //   1570: istore 7
    //   1572: goto +231 -> 1803
    //   1575: aload_0
    //   1576: invokespecial 875	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:downCaseWord	()Z
    //   1579: istore 7
    //   1581: goto +222 -> 1803
    //   1584: aload_0
    //   1585: invokevirtual 376	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:moveToEnd	()Z
    //   1588: istore 7
    //   1590: goto +213 -> 1803
    //   1593: aload_0
    //   1594: ldc_w 877
    //   1597: invokevirtual 393	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:putString	(Ljava/lang/CharSequence;)V
    //   1600: iconst_1
    //   1601: istore 7
    //   1603: goto +200 -> 1803
    //   1606: aload_0
    //   1607: getfield 241	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:consoleKeys	Lorg/bukkit/craftbukkit/libs/jline/console/ConsoleKeys;
    //   1610: aload_0
    //   1611: getfield 170	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:appName	Ljava/lang/String;
    //   1614: aload_0
    //   1615: getfield 234	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:inputrcUrl	Ljava/net/URL;
    //   1618: invokevirtual 880	org/bukkit/craftbukkit/libs/jline/console/ConsoleKeys:loadKeys	(Ljava/lang/String;Ljava/net/URL;)V
    //   1621: iconst_1
    //   1622: istore 7
    //   1624: goto +179 -> 1803
    //   1627: aload_0
    //   1628: iconst_1
    //   1629: putfield 752	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:recording	Z
    //   1632: goto +171 -> 1803
    //   1635: aload_0
    //   1636: iconst_0
    //   1637: putfield 752	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:recording	Z
    //   1640: aload_0
    //   1641: aload_0
    //   1642: getfield 130	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:macro	Ljava/lang/String;
    //   1645: iconst_0
    //   1646: aload_0
    //   1647: getfield 130	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:macro	Ljava/lang/String;
    //   1650: invokevirtual 308	java/lang/String:length	()I
    //   1653: aload 8
    //   1655: invokevirtual 368	java/lang/StringBuilder:length	()I
    //   1658: isub
    //   1659: invokevirtual 318	java/lang/String:substring	(II)Ljava/lang/String;
    //   1662: putfield 130	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:macro	Ljava/lang/String;
    //   1665: goto +138 -> 1803
    //   1668: iconst_0
    //   1669: istore 12
    //   1671: iload 12
    //   1673: aload_0
    //   1674: getfield 130	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:macro	Ljava/lang/String;
    //   1677: invokevirtual 308	java/lang/String:length	()I
    //   1680: if_icmpge +39 -> 1719
    //   1683: aload 9
    //   1685: aload_0
    //   1686: getfield 130	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:macro	Ljava/lang/String;
    //   1689: aload_0
    //   1690: getfield 130	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:macro	Ljava/lang/String;
    //   1693: invokevirtual 308	java/lang/String:length	()I
    //   1696: iconst_1
    //   1697: isub
    //   1698: iload 12
    //   1700: isub
    //   1701: invokevirtual 372	java/lang/String:charAt	(I)C
    //   1704: invokestatic 772	java/lang/Character:valueOf	(C)Ljava/lang/Character;
    //   1707: invokeinterface 774 2 0
    //   1712: pop
    //   1713: iinc 12 1
    //   1716: goto -45 -> 1671
    //   1719: aload 8
    //   1721: iconst_0
    //   1722: invokevirtual 386	java/lang/StringBuilder:setLength	(I)V
    //   1725: goto +78 -> 1803
    //   1728: aload_0
    //   1729: getfield 241	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:consoleKeys	Lorg/bukkit/craftbukkit/libs/jline/console/ConsoleKeys;
    //   1732: iconst_1
    //   1733: invokevirtual 883	org/bukkit/craftbukkit/libs/jline/console/ConsoleKeys:setViEditMode	(Z)V
    //   1736: aload_0
    //   1737: getfield 241	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:consoleKeys	Lorg/bukkit/craftbukkit/libs/jline/console/ConsoleKeys;
    //   1740: aload_0
    //   1741: getfield 241	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:consoleKeys	Lorg/bukkit/craftbukkit/libs/jline/console/ConsoleKeys;
    //   1744: invokevirtual 887	org/bukkit/craftbukkit/libs/jline/console/ConsoleKeys:getKeyMaps	()Ljava/util/Map;
    //   1747: ldc_w 889
    //   1750: invokeinterface 894 2 0
    //   1755: checkcast 755	org/bukkit/craftbukkit/libs/jline/console/KeyMap
    //   1758: invokevirtual 898	org/bukkit/craftbukkit/libs/jline/console/ConsoleKeys:setKeys	(Lorg/bukkit/craftbukkit/libs/jline/console/KeyMap;)V
    //   1761: goto +42 -> 1803
    //   1764: aload_0
    //   1765: getfield 241	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:consoleKeys	Lorg/bukkit/craftbukkit/libs/jline/console/ConsoleKeys;
    //   1768: iconst_0
    //   1769: invokevirtual 883	org/bukkit/craftbukkit/libs/jline/console/ConsoleKeys:setViEditMode	(Z)V
    //   1772: aload_0
    //   1773: getfield 241	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:consoleKeys	Lorg/bukkit/craftbukkit/libs/jline/console/ConsoleKeys;
    //   1776: aload_0
    //   1777: getfield 241	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:consoleKeys	Lorg/bukkit/craftbukkit/libs/jline/console/ConsoleKeys;
    //   1780: invokevirtual 887	org/bukkit/craftbukkit/libs/jline/console/ConsoleKeys:getKeyMaps	()Ljava/util/Map;
    //   1783: ldc_w 900
    //   1786: invokeinterface 894 2 0
    //   1791: checkcast 755	org/bukkit/craftbukkit/libs/jline/console/KeyMap
    //   1794: invokevirtual 898	org/bukkit/craftbukkit/libs/jline/console/ConsoleKeys:setKeys	(Lorg/bukkit/craftbukkit/libs/jline/console/KeyMap;)V
    //   1797: goto +6 -> 1803
    //   1800: iconst_0
    //   1801: istore 12
    //   1803: iload 7
    //   1805: ifne +7 -> 1812
    //   1808: aload_0
    //   1809: invokevirtual 820	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:beep	()V
    //   1812: aload 8
    //   1814: iconst_0
    //   1815: invokevirtual 386	java/lang/StringBuilder:setLength	(I)V
    //   1818: aload_0
    //   1819: invokevirtual 497	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:flush	()V
    //   1822: goto -1686 -> 136
    //   1825: astore 14
    //   1827: aload_0
    //   1828: getfield 186	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:terminal	Lorg/bukkit/craftbukkit/libs/jline/Terminal;
    //   1831: invokeinterface 725 1 0
    //   1836: ifne +7 -> 1843
    //   1839: aload_0
    //   1840: invokespecial 737	org/bukkit/craftbukkit/libs/jline/console/ConsoleReader:afterReadLine	()V
    //   1843: aload 14
    //   1845: athrow
    // Line number table:
    //   Java source line #1133	-> byte code offset #0
    //   Java source line #1134	-> byte code offset #5
    //   Java source line #1135	-> byte code offset #9
    //   Java source line #1138	-> byte code offset #17
    //   Java source line #1142	-> byte code offset #22
    //   Java source line #1143	-> byte code offset #34
    //   Java source line #1146	-> byte code offset #40
    //   Java source line #1147	-> byte code offset #51
    //   Java source line #1148	-> byte code offset #59
    //   Java source line #1152	-> byte code offset #66
    //   Java source line #1153	-> byte code offset #78
    //   Java source line #1479	-> byte code offset #83
    //   Java source line #1480	-> byte code offset #95
    //   Java source line #1156	-> byte code offset #101
    //   Java source line #1158	-> byte code offset #106
    //   Java source line #1159	-> byte code offset #109
    //   Java source line #1160	-> byte code offset #112
    //   Java source line #1162	-> byte code offset #115
    //   Java source line #1164	-> byte code offset #118
    //   Java source line #1165	-> byte code offset #127
    //   Java source line #1167	-> byte code offset #136
    //   Java source line #1168	-> byte code offset #177
    //   Java source line #1169	-> byte code offset #183
    //   Java source line #1479	-> byte code offset #186
    //   Java source line #1480	-> byte code offset #198
    //   Java source line #1171	-> byte code offset #205
    //   Java source line #1172	-> byte code offset #214
    //   Java source line #1173	-> byte code offset #221
    //   Java source line #1176	-> byte code offset #248
    //   Java source line #1177	-> byte code offset #259
    //   Java source line #1178	-> byte code offset #267
    //   Java source line #1179	-> byte code offset #279
    //   Java source line #1180	-> byte code offset #291
    //   Java source line #1182	-> byte code offset #302
    //   Java source line #1183	-> byte code offset #310
    //   Java source line #1185	-> byte code offset #313
    //   Java source line #1186	-> byte code offset #326
    //   Java source line #1187	-> byte code offset #340
    //   Java source line #1188	-> byte code offset #352
    //   Java source line #1189	-> byte code offset #363
    //   Java source line #1190	-> byte code offset #371
    //   Java source line #1191	-> byte code offset #381
    //   Java source line #1192	-> byte code offset #386
    //   Java source line #1194	-> byte code offset #389
    //   Java source line #1197	-> byte code offset #403
    //   Java source line #1198	-> byte code offset #406
    //   Java source line #1199	-> byte code offset #411
    //   Java source line #1201	-> byte code offset #414
    //   Java source line #1205	-> byte code offset #432
    //   Java source line #1206	-> byte code offset #440
    //   Java source line #1207	-> byte code offset #447
    //   Java source line #1208	-> byte code offset #460
    //   Java source line #1207	-> byte code offset #486
    //   Java source line #1210	-> byte code offset #492
    //   Java source line #1211	-> byte code offset #498
    //   Java source line #1215	-> byte code offset #501
    //   Java source line #1216	-> byte code offset #509
    //   Java source line #1217	-> byte code offset #520
    //   Java source line #1218	-> byte code offset #526
    //   Java source line #1226	-> byte code offset #529
    //   Java source line #1227	-> byte code offset #535
    //   Java source line #1228	-> byte code offset #538
    //   Java source line #1230	-> byte code offset #580
    //   Java source line #1231	-> byte code offset #583
    //   Java source line #1234	-> byte code offset #586
    //   Java source line #1235	-> byte code offset #596
    //   Java source line #1238	-> byte code offset #608
    //   Java source line #1239	-> byte code offset #616
    //   Java source line #1241	-> byte code offset #634
    //   Java source line #1243	-> byte code offset #653
    //   Java source line #1246	-> byte code offset #656
    //   Java source line #1247	-> byte code offset #666
    //   Java source line #1248	-> byte code offset #683
    //   Java source line #1253	-> byte code offset #701
    //   Java source line #1254	-> byte code offset #711
    //   Java source line #1255	-> byte code offset #726
    //   Java source line #1259	-> byte code offset #729
    //   Java source line #1260	-> byte code offset #737
    //   Java source line #1262	-> byte code offset #751
    //   Java source line #1264	-> byte code offset #775
    //   Java source line #1269	-> byte code offset #778
    //   Java source line #1270	-> byte code offset #784
    //   Java source line #1271	-> byte code offset #794
    //   Java source line #1272	-> byte code offset #802
    //   Java source line #1274	-> byte code offset #810
    //   Java source line #1275	-> byte code offset #818
    //   Java source line #1277	-> byte code offset #825
    //   Java source line #1283	-> byte code offset #855
    //   Java source line #1286	-> byte code offset #862
    //   Java source line #1287	-> byte code offset #868
    //   Java source line #1288	-> byte code offset #876
    //   Java source line #1290	-> byte code offset #1048
    //   Java source line #1291	-> byte code offset #1054
    //   Java source line #1294	-> byte code offset #1057
    //   Java source line #1295	-> byte code offset #1061
    //   Java source line #1296	-> byte code offset #1064
    //   Java source line #1299	-> byte code offset #1067
    //   Java source line #1300	-> byte code offset #1074
    //   Java source line #1303	-> byte code offset #1077
    //   Java source line #1304	-> byte code offset #1083
    //   Java source line #1307	-> byte code offset #1086
    //   Java source line #1308	-> byte code offset #1108
    //   Java source line #1311	-> byte code offset #1111
    //   Java source line #1312	-> byte code offset #1117
    //   Java source line #1315	-> byte code offset #1120
    //   Java source line #1316	-> byte code offset #1142
    //   Java source line #1319	-> byte code offset #1145
    //   Java source line #1320	-> byte code offset #1151
    //   Java source line #1321	-> byte code offset #1154
    //   Java source line #1324	-> byte code offset #1157
    //   Java source line #1325	-> byte code offset #1162
    //   Java source line #1326	-> byte code offset #1166
    //   Java source line #1327	-> byte code offset #1170
    //   Java source line #1479	-> byte code offset #1176
    //   Java source line #1480	-> byte code offset #1188
    //   Java source line #1330	-> byte code offset #1195
    //   Java source line #1331	-> byte code offset #1201
    //   Java source line #1334	-> byte code offset #1204
    //   Java source line #1335	-> byte code offset #1210
    //   Java source line #1338	-> byte code offset #1213
    //   Java source line #1339	-> byte code offset #1220
    //   Java source line #1342	-> byte code offset #1223
    //   Java source line #1343	-> byte code offset #1230
    //   Java source line #1346	-> byte code offset #1233
    //   Java source line #1347	-> byte code offset #1239
    //   Java source line #1350	-> byte code offset #1242
    //   Java source line #1351	-> byte code offset #1255
    //   Java source line #1479	-> byte code offset #1258
    //   Java source line #1480	-> byte code offset #1270
    //   Java source line #1353	-> byte code offset #1277
    //   Java source line #1354	-> byte code offset #1283
    //   Java source line #1357	-> byte code offset #1286
    //   Java source line #1358	-> byte code offset #1292
    //   Java source line #1361	-> byte code offset #1295
    //   Java source line #1362	-> byte code offset #1310
    //   Java source line #1365	-> byte code offset #1313
    //   Java source line #1366	-> byte code offset #1328
    //   Java source line #1369	-> byte code offset #1331
    //   Java source line #1370	-> byte code offset #1337
    //   Java source line #1375	-> byte code offset #1340
    //   Java source line #1376	-> byte code offset #1346
    //   Java source line #1378	-> byte code offset #1349
    //   Java source line #1379	-> byte code offset #1355
    //   Java source line #1381	-> byte code offset #1358
    //   Java source line #1382	-> byte code offset #1369
    //   Java source line #1383	-> byte code offset #1374
    //   Java source line #1388	-> byte code offset #1390
    //   Java source line #1389	-> byte code offset #1401
    //   Java source line #1390	-> byte code offset #1406
    //   Java source line #1395	-> byte code offset #1422
    //   Java source line #1396	-> byte code offset #1429
    //   Java source line #1398	-> byte code offset #1440
    //   Java source line #1399	-> byte code offset #1458
    //   Java source line #1400	-> byte code offset #1461
    //   Java source line #1401	-> byte code offset #1471
    //   Java source line #1402	-> byte code offset #1486
    //   Java source line #1403	-> byte code offset #1494
    //   Java source line #1405	-> byte code offset #1498
    //   Java source line #1408	-> byte code offset #1541
    //   Java source line #1409	-> byte code offset #1546
    //   Java source line #1411	-> byte code offset #1554
    //   Java source line #1414	-> byte code offset #1557
    //   Java source line #1415	-> byte code offset #1563
    //   Java source line #1418	-> byte code offset #1566
    //   Java source line #1419	-> byte code offset #1572
    //   Java source line #1422	-> byte code offset #1575
    //   Java source line #1423	-> byte code offset #1581
    //   Java source line #1426	-> byte code offset #1584
    //   Java source line #1427	-> byte code offset #1590
    //   Java source line #1430	-> byte code offset #1593
    //   Java source line #1431	-> byte code offset #1600
    //   Java source line #1432	-> byte code offset #1603
    //   Java source line #1435	-> byte code offset #1606
    //   Java source line #1436	-> byte code offset #1621
    //   Java source line #1437	-> byte code offset #1624
    //   Java source line #1440	-> byte code offset #1627
    //   Java source line #1441	-> byte code offset #1632
    //   Java source line #1444	-> byte code offset #1635
    //   Java source line #1445	-> byte code offset #1640
    //   Java source line #1446	-> byte code offset #1665
    //   Java source line #1449	-> byte code offset #1668
    //   Java source line #1450	-> byte code offset #1683
    //   Java source line #1449	-> byte code offset #1713
    //   Java source line #1452	-> byte code offset #1719
    //   Java source line #1453	-> byte code offset #1725
    //   Java source line #1456	-> byte code offset #1728
    //   Java source line #1457	-> byte code offset #1736
    //   Java source line #1458	-> byte code offset #1761
    //   Java source line #1461	-> byte code offset #1764
    //   Java source line #1462	-> byte code offset #1772
    //   Java source line #1463	-> byte code offset #1797
    //   Java source line #1466	-> byte code offset #1800
    //   Java source line #1471	-> byte code offset #1803
    //   Java source line #1472	-> byte code offset #1808
    //   Java source line #1474	-> byte code offset #1812
    //   Java source line #1475	-> byte code offset #1818
    //   Java source line #1476	-> byte code offset #1822
    //   Java source line #1479	-> byte code offset #1825
    //   Java source line #1480	-> byte code offset #1839
    // Local variable table:
    //   start	length	slot	name	signature
    //   0	1846	0	this	ConsoleReader
    //   0	1846	1	prompt	String
    //   0	1846	2	mask	Character
    //   82	18	3	str1	String
    //   105	752	3	originalPrompt	String
    //   107	3	4	NORMAL	int
    //   110	3	5	SEARCH	int
    //   113	1347	6	state	int
    //   116	1688	7	success	boolean
    //   125	1688	8	sb	StringBuilder
    //   134	1550	9	pushBackChar	List<Character>
    //   175	531	10	c	int
    //   184	19	11	str2	String
    //   257	623	11	o	Object
    //   361	11	12	o2	Object
    //   445	20	12	macro	String
    //   536	739	12	cursorDest	int
    //   1669	45	12	i	int
    //   1801	3	12	i	int
    //   448	39	13	i	int
    //   1825	19	14	localObject1	Object
    // Exception table:
    //   from	to	target	type
    //   22	83	1825	finally
    //   101	186	1825	finally
    //   205	1176	1825	finally
    //   1195	1258	1825	finally
    //   1277	1827	1825	finally
  }
  
  private String readLineSimple()
    throws IOException
  {
    StringBuilder buff = new StringBuilder();
    
    if (skipLF) {
      skipLF = false;
      
      int i = readCharacter();
      
      if ((i == -1) || (i == 13))
        return buff.toString();
      if (i != 10)
      {

        buff.append((char)i);
      }
    }
    for (;;)
    {
      int i = readCharacter();
      
      if ((i == -1) || (i == 10))
        return buff.toString();
      if (i == 13) {
        skipLF = true;
        return buff.toString();
      }
      buff.append((char)i);
    }
  }
  





  private final List<Completer> completers = new LinkedList();
  
  private CompletionHandler completionHandler = new CandidateListCompletionHandler();
  





  public boolean addCompleter(Completer completer)
  {
    return completers.add(completer);
  }
  





  public boolean removeCompleter(Completer completer)
  {
    return completers.remove(completer);
  }
  


  public Collection<Completer> getCompleters()
  {
    return Collections.unmodifiableList(completers);
  }
  
  public void setCompletionHandler(CompletionHandler handler) {
    assert (handler != null);
    completionHandler = handler;
  }
  
  public CompletionHandler getCompletionHandler() {
    return completionHandler;
  }
  




  protected boolean complete()
    throws IOException
  {
    if (completers.size() == 0) {
      return false;
    }
    
    List<CharSequence> candidates = new LinkedList();
    String bufstr = buf.buffer.toString();
    int cursor = buf.cursor;
    
    int position = -1;
    
    for (Completer comp : completers) {
      if ((position = comp.complete(bufstr, cursor, candidates)) != -1) {
        break;
      }
    }
    
    return (candidates.size() != 0) && (getCompletionHandler().complete(this, candidates, position));
  }
  
  protected void printCompletionCandidates() throws IOException
  {
    if (completers.size() == 0) {
      return;
    }
    
    List<CharSequence> candidates = new LinkedList();
    String bufstr = buf.buffer.toString();
    int cursor = buf.cursor;
    
    for (Completer comp : completers) {
      if (comp.complete(bufstr, cursor, candidates) != -1) {
        break;
      }
    }
    CandidateListCompletionHandler.printCandidates(this, candidates);
    drawLine();
  }
  




  private int autoprintThreshold = Integer.getInteger("org.bukkit.craftbukkit.libs.jline.completion.threshold", 100).intValue();
  
  private boolean paginationEnabled;
  
  public void setAutoprintThreshold(int threshold)
  {
    autoprintThreshold = threshold;
  }
  


  public int getAutoprintThreshold()
  {
    return autoprintThreshold;
  }
  




  public void setPaginationEnabled(boolean enabled)
  {
    paginationEnabled = enabled;
  }
  


  public boolean isPaginationEnabled()
  {
    return paginationEnabled;
  }
  




  private History history = new MemoryHistory();
  
  public void setHistory(History history) {
    this.history = history;
  }
  
  public History getHistory() {
    return history;
  }
  
  private boolean historyEnabled = true;
  


  public void setHistoryEnabled(boolean enabled)
  {
    historyEnabled = enabled;
  }
  


  public boolean isHistoryEnabled()
  {
    return historyEnabled;
  }
  

  private boolean moveHistory(boolean next)
    throws IOException
  {
    if ((next) && (!history.next())) {
      return false;
    }
    if ((!next) && (!history.previous())) {
      return false;
    }
    
    setBuffer(history.current());
    
    return true;
  }
  




  public static final String CR = System.getProperty("line.separator");
  

  private void print(int c)
    throws IOException
  {
    if (c == 9) {
      char[] chars = new char[4];
      Arrays.fill(chars, ' ');
      out.write(chars);
      return;
    }
    
    out.write(c);
  }
  

  private void print(char... buff)
    throws IOException
  {
    int len = 0;
    for (char c : buff) {
      if (c == '\t') {
        len += 4;
      }
      else {
        len++;
      }
    }
    char[] chars;
    char[] chars;
    if (len == buff.length) {
      chars = buff;
    }
    else {
      chars = new char[len];
      int pos = 0;
      for (char c : buff) {
        if (c == '\t') {
          Arrays.fill(chars, pos, pos + 4, ' ');
          pos += 4;
        }
        else {
          chars[pos] = c;
          pos++;
        }
      }
    }
    
    out.write(chars);
  }
  
  private void print(char c, int num) throws IOException {
    if (num == 1) {
      print(c);
    }
    else {
      char[] chars = new char[num];
      Arrays.fill(chars, c);
      print(chars);
    }
  }
  

  public final void print(CharSequence s)
    throws IOException
  {
    assert (s != null);
    print(s.toString().toCharArray());
  }
  
  public final void println(CharSequence s) throws IOException {
    assert (s != null);
    print(s.toString().toCharArray());
    println();
  }
  

  public final void println()
    throws IOException
  {
    print(CR);
  }
  








  public final boolean delete()
    throws IOException
  {
    return delete(1) == 1;
  }
  













  private int delete(int num)
    throws IOException
  {
    buf.buffer.delete(buf.cursor, buf.cursor + 1);
    drawBuffer(1);
    
    return 1;
  }
  



  public boolean killLine()
    throws IOException
  {
    int cp = buf.cursor;
    int len = buf.buffer.length();
    
    if (cp >= len) {
      return false;
    }
    
    int num = buf.buffer.length() - cp;
    clearAhead(num, 0);
    
    for (int i = 0; i < num; i++) {
      buf.buffer.deleteCharAt(len - i - 1);
    }
    
    return true;
  }
  

  public boolean clearScreen()
    throws IOException
  {
    if (!terminal.isAnsiSupported()) {
      return false;
    }
    

    printAnsiSequence("2J");
    

    printAnsiSequence("1;1H");
    
    redrawLine();
    
    return true;
  }
  

  public void beep()
    throws IOException
  {
    if (!Configuration.getBoolean("org.bukkit.craftbukkit.libs.jline.nobell", true)) {
      print(7);
      
      flush();
    }
  }
  

  public boolean paste()
    throws IOException
  {
    Clipboard clipboard;
    
    try
    {
      clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
    }
    catch (Exception e) {
      return false;
    }
    
    if (clipboard == null) {
      return false;
    }
    
    Transferable transferable = clipboard.getContents(null);
    
    if (transferable == null) {
      return false;
    }
    try
    {
      Object content = transferable.getTransferData(DataFlavor.plainTextFlavor);
      





      if (content == null) {
        try {
          content = new DataFlavor().getReaderForText(transferable);
        }
        catch (Exception e) {}
      }
      


      if (content == null) {
        return false;
      }
      
      String value;
      
      if ((content instanceof Reader))
      {

        String value = "";
        

        BufferedReader read = new BufferedReader((Reader)content);
        String line; while ((line = read.readLine()) != null) {
          if (value.length() > 0) {
            value = value + "\n";
          }
          
          value = value + line;
        }
      }
      else {
        value = content.toString();
      }
      
      if (value == null) {
        return true;
      }
      
      putString(value);
      
      return true;
    }
    catch (UnsupportedFlavorException e) {
      Log.error(new Object[] { "Paste failed: ", e });
    }
    return false;
  }
  





  private final Map<Character, ActionListener> triggeredActions = new HashMap();
  

  private Thread maskThread;
  


  public void addTriggeredAction(char c, ActionListener listener)
  {
    triggeredActions.put(Character.valueOf(c), listener);
  }
  





  public void printColumns(Collection<? extends CharSequence> items)
    throws IOException
  {
    if ((items == null) || (items.isEmpty())) {
      return;
    }
    
    int width = getTerminal().getWidth();
    int height = getTerminal().getHeight();
    
    int maxWidth = 0;
    for (CharSequence item : items) {
      maxWidth = Math.max(maxWidth, item.length());
    }
    maxWidth += 3;
    Log.debug(new Object[] { "Max width: ", Integer.valueOf(maxWidth) });
    int showLines;
    int showLines;
    if (isPaginationEnabled()) {
      showLines = height - 1;
    }
    else {
      showLines = 2147483647;
    }
    
    StringBuilder buff = new StringBuilder();
    for (CharSequence item : items) {
      if (buff.length() + maxWidth > width) {
        println(buff);
        buff.setLength(0);
        
        showLines--; if (showLines == 0)
        {
          print(resources.getString("DISPLAY_MORE"));
          flush();
          int c = readCharacter();
          if ((c == 13) || (c == 10))
          {
            showLines = 1;
          }
          else if (c != 113)
          {
            showLines = height - 1;
          }
          
          back(resources.getString("DISPLAY_MORE").length());
          if (c == 113) {
            break;
          }
        }
      }
      


      buff.append(item.toString());
      for (int i = 0; i < maxWidth - item.length(); i++) {
        buff.append(' ');
      }
    }
    
    if (buff.length() > 0) {
      println(buff);
    }
  }
  





  private void beforeReadLine(String prompt, Character mask)
  {
    if ((mask != null) && (maskThread == null)) {
      final String fullPrompt = "\r" + prompt + "                 " + "                 " + "                 " + "\r" + prompt;
      




      maskThread = new Thread()
      {
        public void run() {
          while (!interrupted()) {
            try {
              Writer out = getOutput();
              out.write(fullPrompt);
              out.flush();
              sleep(3L);


            }
            catch (IOException e) {}catch (InterruptedException e) {}

          }
          
        }
        

      };
      maskThread.setPriority(10);
      maskThread.setDaemon(true);
      maskThread.start();
    }
  }
  
  private void afterReadLine() {
    if ((maskThread != null) && (maskThread.isAlive())) {
      maskThread.interrupt();
    }
    
    maskThread = null;
  }
  










  public void resetPromptLine(String prompt, String buffer, int cursorDest)
    throws IOException
  {
    moveToEnd();
    

    buf.buffer.append(this.prompt);
    buf.cursor += this.prompt.length();
    this.prompt = "";
    backspaceAll();
    
    this.prompt = prompt;
    redrawLine();
    setBuffer(buffer);
    

    if (cursorDest < 0) cursorDest = buffer.length();
    setCursorPosition(cursorDest);
    
    flush();
  }
  
  public void printSearchStatus(String searchTerm, String match) throws IOException {
    String prompt = "(reverse-i-search)`" + searchTerm + "': ";
    String buffer = match;
    int cursorDest = match.indexOf(searchTerm);
    resetPromptLine(prompt, buffer, cursorDest);
  }
  
  public void restoreLine(String originalPrompt, int cursorDest) throws IOException
  {
    String prompt = lastLine(originalPrompt);
    String buffer = buf.buffer.toString();
    resetPromptLine(prompt, buffer, cursorDest);
  }
  









  public int searchBackwards(String searchTerm, int startIndex)
  {
    return searchBackwards(searchTerm, startIndex, false);
  }
  





  public int searchBackwards(String searchTerm)
  {
    return searchBackwards(searchTerm, history.index());
  }
  
  public int searchBackwards(String searchTerm, int startIndex, boolean startsWith)
  {
    ListIterator<History.Entry> it = history.entries(startIndex);
    while (it.hasPrevious()) {
      History.Entry e = (History.Entry)it.previous();
      if (startsWith) {
        if (e.value().toString().startsWith(searchTerm)) {
          return e.index();
        }
      }
      else if (e.value().toString().contains(searchTerm)) {
        return e.index();
      }
    }
    
    return -1;
  }
  










  private boolean isDelimiter(char c)
  {
    return !Character.isLetterOrDigit(c);
  }
  
  private void printAnsiSequence(String sequence) throws IOException {
    print(27);
    print(91);
    print(sequence);
    flush();
  }
  

  private int getCurrentPosition()
  {
    if ((terminal.isAnsiSupported()) && (!(in instanceof ByteArrayInputStream))) {
      try {
        printAnsiSequence("6n");
        flush();
        StringBuffer b = new StringBuffer(8);
        
        int r;
        while (((r = in.read()) > -1) && (r != 82)) {
          if ((r != 27) && (r != 91)) {
            b.append((char)r);
          }
        }
        String[] pos = b.toString().split(";");
        return Integer.parseInt(pos[1]);
      }
      catch (Exception x) {}
    }
    

    return -1;
  }
  



  private int getCurrentAnsiRow()
  {
    if ((terminal.isAnsiSupported()) && (!(in instanceof ByteArrayInputStream))) {
      try {
        printAnsiSequence("6n");
        flush();
        StringBuffer b = new StringBuffer(8);
        
        int r;
        while (((r = in.read()) > -1) && (r != 82)) {
          if ((r != 27) && (r != 91)) {
            b.append((char)r);
          }
        }
        String[] pos = b.toString().split(";");
        return Integer.parseInt(pos[0]);
      }
      catch (Exception x) {}
    }
    

    return -1;
  }
}
