package org.bukkit.craftbukkit.libs.jline.console.completer;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;


























public class AggregateCompleter
  implements Completer
{
  private final List<Completer> completers = new ArrayList();
  

  public AggregateCompleter() {}
  
  public AggregateCompleter(Collection<Completer> completers)
  {
    assert (completers != null);
    this.completers.addAll(completers);
  }
  
  public AggregateCompleter(Completer... completers) {
    this(Arrays.asList(completers));
  }
  
  public Collection<Completer> getCompleters() {
    return completers;
  }
  
  public int complete(String buffer, int cursor, List<CharSequence> candidates)
  {
    assert (candidates != null);
    
    List<Completion> completions = new ArrayList(completers.size());
    

    int max = -1;
    for (Completer completer : completers) {
      Completion completion = new Completion(candidates);
      completion.complete(completer, buffer, cursor);
      

      max = Math.max(max, cursor);
      
      completions.add(completion);
    }
    

    for (Completion completion : completions) {
      if (cursor == max) {
        candidates.addAll(candidates);
      }
    }
    
    return max;
  }
  
  public String toString()
  {
    return getClass().getSimpleName() + "{" + "completers=" + completers + '}';
  }
  

  private class Completion
  {
    public final List<CharSequence> candidates;
    
    public int cursor;
    
    public Completion()
    {
      assert (candidates != null);
      this.candidates = new LinkedList(candidates);
    }
    
    public void complete(Completer completer, String buffer, int cursor) {
      assert (completer != null);
      
      this.cursor = completer.complete(buffer, cursor, candidates);
    }
  }
}
