package org.bukkit.craftbukkit.libs.jline.console.history;

import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.Flushable;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.io.Reader;
import org.bukkit.craftbukkit.libs.jline.internal.Log;
















public class FileHistory
  extends MemoryHistory
  implements PersistentHistory, Flushable
{
  private final File file;
  
  public FileHistory(File file)
    throws IOException
  {
    assert (file != null);
    this.file = file;
    load(file);
  }
  
  public File getFile() {
    return file;
  }
  
  public void load(File file) throws IOException {
    assert (file != null);
    if (file.exists()) {
      Log.trace(new Object[] { "Loading history from: ", file });
      load(new FileReader(file));
    }
  }
  
  public void load(InputStream input) throws IOException {
    assert (input != null);
    load(new InputStreamReader(input));
  }
  
  public void load(Reader reader) throws IOException {
    assert (reader != null);
    BufferedReader input = new BufferedReader(reader);
    
    String item;
    while ((item = input.readLine()) != null) {
      internalAdd(item);
    }
  }
  
  public void flush() throws IOException {
    Log.trace(new Object[] { "Flushing history" });
    
    if (!file.exists()) {
      File dir = file.getParentFile();
      if ((!dir.exists()) && (!dir.mkdirs())) {
        Log.warn(new Object[] { "Failed to create directory: ", dir });
      }
      if (!file.createNewFile()) {
        Log.warn(new Object[] { "Failed to create file: ", file });
      }
    }
    
    PrintStream out = new PrintStream(new BufferedOutputStream(new FileOutputStream(file)));
    try {
      for (History.Entry entry : this) {
        out.println(entry.value());
      }
    }
    finally {
      out.close();
    }
  }
  
  public void purge() throws IOException {
    Log.trace(new Object[] { "Purging history" });
    
    clear();
    
    if (!file.delete()) {
      Log.warn(new Object[] { "Failed to delete history file: ", file });
    }
  }
}
