package org.bukkit.craftbukkit.libs.jline.console.history;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.ListIterator;
import java.util.NoSuchElementException;







public class MemoryHistory
  implements History
{
  public static final int DEFAULT_MAX_SIZE = 500;
  private final LinkedList<CharSequence> items;
  private int maxSize;
  private boolean ignoreDuplicates;
  private boolean autoTrim;
  private int offset;
  private int index;
  
  public MemoryHistory()
  {
    items = new LinkedList();
    
    maxSize = 500;
    
    ignoreDuplicates = true;
    
    autoTrim = false;
    










    offset = 0;
    
    index = 0;
  }
  
  public void setMaxSize(int maxSize) { this.maxSize = maxSize;
    maybeResize();
  }
  
  public int getMaxSize() {
    return maxSize;
  }
  
  public boolean isIgnoreDuplicates() {
    return ignoreDuplicates;
  }
  
  public void setIgnoreDuplicates(boolean flag) {
    ignoreDuplicates = flag;
  }
  
  public boolean isAutoTrim() {
    return autoTrim;
  }
  
  public void setAutoTrim(boolean flag) {
    autoTrim = flag;
  }
  
  public int size() {
    return items.size();
  }
  
  public boolean isEmpty() {
    return items.isEmpty();
  }
  
  public int index() {
    return offset + index;
  }
  
  public void clear() {
    items.clear();
    offset = 0;
    index = 0;
  }
  
  public CharSequence get(int index) {
    return (CharSequence)items.get(index - offset);
  }
  
  public void add(CharSequence item) {
    assert (item != null);
    
    if (isAutoTrim()) {
      item = String.valueOf(item).trim();
    }
    
    if ((isIgnoreDuplicates()) && 
      (!items.isEmpty()) && (item.equals(items.getLast()))) {
      return;
    }
    

    internalAdd(item);
  }
  
  protected void internalAdd(CharSequence item) {
    items.add(item);
    
    maybeResize();
  }
  
  public void replace(CharSequence item) {
    items.removeLast();
    add(item);
  }
  
  private void maybeResize() {
    while (size() > getMaxSize()) {
      items.removeFirst();
      offset += 1;
    }
    
    index = size();
  }
  
  public ListIterator<History.Entry> entries(int index) {
    return new EntriesIterator(index - offset, null);
  }
  
  public ListIterator<History.Entry> entries() {
    return entries(offset);
  }
  
  public Iterator<History.Entry> iterator() {
    return entries();
  }
  
  private static class EntryImpl
    implements History.Entry
  {
    private final int index;
    private final CharSequence value;
    
    public EntryImpl(int index, CharSequence value)
    {
      this.index = index;
      this.value = value;
    }
    
    public int index() {
      return index;
    }
    
    public CharSequence value() {
      return value;
    }
    
    public String toString()
    {
      return String.format("%d: %s", new Object[] { Integer.valueOf(index), value });
    }
  }
  
  private class EntriesIterator implements ListIterator<History.Entry>
  {
    private final ListIterator<CharSequence> source;
    
    private EntriesIterator(int index)
    {
      source = items.listIterator(index);
    }
    
    public History.Entry next() {
      if (!source.hasNext()) {
        throw new NoSuchElementException();
      }
      return new MemoryHistory.EntryImpl(offset + source.nextIndex(), (CharSequence)source.next());
    }
    
    public History.Entry previous() {
      if (!source.hasPrevious()) {
        throw new NoSuchElementException();
      }
      return new MemoryHistory.EntryImpl(offset + source.previousIndex(), (CharSequence)source.previous());
    }
    
    public int nextIndex() {
      return offset + source.nextIndex();
    }
    
    public int previousIndex() {
      return offset + source.previousIndex();
    }
    
    public boolean hasNext() {
      return source.hasNext();
    }
    
    public boolean hasPrevious() {
      return source.hasPrevious();
    }
    
    public void remove() {
      throw new UnsupportedOperationException();
    }
    
    public void set(History.Entry entry) {
      throw new UnsupportedOperationException();
    }
    
    public void add(History.Entry entry) {
      throw new UnsupportedOperationException();
    }
  }
  










  public boolean moveToLast()
  {
    int lastEntry = size() - 1;
    if ((lastEntry >= 0) && (lastEntry != index)) {
      index = (size() - 1);
      return true;
    }
    
    return false;
  }
  




  public boolean moveTo(int index)
  {
    index -= offset;
    if ((index >= 0) && (index < size())) {
      this.index = index;
      return true;
    }
    return false;
  }
  





  public boolean moveToFirst()
  {
    if ((size() > 0) && (index != 0)) {
      index = 0;
      return true;
    }
    
    return false;
  }
  



  public void moveToEnd()
  {
    index = size();
  }
  


  public CharSequence current()
  {
    if (index >= size()) {
      return "";
    }
    
    return (CharSequence)items.get(index);
  }
  




  public boolean previous()
  {
    if (index <= 0) {
      return false;
    }
    
    index -= 1;
    
    return true;
  }
  




  public boolean next()
  {
    if (index >= size()) {
      return false;
    }
    
    index += 1;
    
    return true;
  }
}
