package org.bukkit.craftbukkit.libs.jline.internal;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.nio.charset.Charset;
import java.util.Properties;

























public class Configuration
{
  public static final String JLINE_INPUTRC = "org.bukkit.craftbukkit.libs.jline.inputrc";
  public static final String JLINE_RC = ".jline.rc";
  public static final String INPUT_RC = ".inputrc";
  private static Configuration configuration;
  private final Properties props;
  private final URL jlinercUrl;
  
  public static Configuration getConfig()
  {
    return getConfig((URL)null);
  }
  
  public static Configuration getConfig(String fileOrUrl) {
    return getConfig(getUrlFrom(fileOrUrl));
  }
  
  public static Configuration getConfig(URL url) {
    if (url == null) {
      url = getUrlFrom(new File(getUserHome(), ".jline.rc"));
    }
    if ((configuration == null) || (!url.equals(configurationjlinercUrl))) {
      configuration = new Configuration(url);
    }
    return configuration;
  }
  





  public Configuration()
  {
    this(getUrlFrom(new File(getUserHome(), ".jline.rc")));
  }
  
  public Configuration(File inputRc) {
    this(getUrlFrom(inputRc));
  }
  
  public Configuration(String fileOrUrl) {
    this(getUrlFrom(fileOrUrl));
  }
  
  public Configuration(URL jlinercUrl) {
    this.jlinercUrl = jlinercUrl;
    props = loadProps();
  }
  
  protected Properties loadProps()
  {
    props = new Properties();
    try {
      InputStream input = jlinercUrl.openStream();
      try {
        props.load(new BufferedInputStream(input));
      }
      finally {
        try {
          input.close();
        }
        catch (IOException e) {}
      }
      




      File file;
      




      return props;
    }
    catch (IOException e)
    {
      if (jlinercUrl.getProtocol().equals("file")) {
        file = new File(jlinercUrl.getPath());
        if (file.exists()) {
          Log.warn(new Object[] { "Unable to read user configuration: ", jlinercUrl, e });
        }
      } else {
        Log.warn(new Object[] { "Unable to read user configuration: ", jlinercUrl, e });
      }
    }
  }
  
  public static URL getUrlFrom(String fileOrUrl)
  {
    if (fileOrUrl == null) {
      return null;
    }
    try {
      return new URL(fileOrUrl);
    } catch (MalformedURLException e) {}
    return getUrlFrom(new File(fileOrUrl));
  }
  
  public static URL getUrlFrom(File inputRc)
  {
    try {
      return inputRc != null ? inputRc.toURI().toURL() : null;
    } catch (MalformedURLException e) {
      throw new IllegalStateException(e);
    }
  }
  
  public String string(String name, String defaultValue) {
    assert (name != null);
    



    String value = System.getProperty(name);
    
    if (value == null)
    {
      value = props.getProperty(name);
      
      if (value == null)
      {
        value = defaultValue;
      }
    }
    
    return value;
  }
  
  public String string(String name) {
    return string(name, null);
  }
  
  public boolean bool(String name, boolean defaultValue) {
    String value = string(name, null);
    if (value == null) {
      return defaultValue;
    }
    return (value.length() == 0) || (value.equalsIgnoreCase("1")) || (value.equalsIgnoreCase("on")) || (value.equalsIgnoreCase("true"));
  }
  
  public boolean bool(String name)
  {
    return bool(name, false);
  }
  
  public static String getString(String name, String defaultValue) {
    return getConfig().string(name, defaultValue);
  }
  
  public static String getString(String name) {
    return getString(name, null);
  }
  
  public static boolean getBoolean(String name, boolean defaultValue) {
    return getConfig().bool(name, defaultValue);
  }
  
  public static boolean getBoolean(String name) {
    return getBoolean(name, false);
  }
  



  public static File getUserHome()
  {
    return new File(System.getProperty("user.home"));
  }
  
  public static String getOsName() {
    return System.getProperty("os.name").toLowerCase();
  }
  
  public static String getFileEncoding() {
    return System.getProperty("file.encoding");
  }
  
  public static String getEncoding()
  {
    String ctype = System.getenv("LC_CTYPE");
    if ((ctype != null) && (ctype.indexOf('.') > 0)) {
      return ctype.substring(ctype.indexOf('.') + 1);
    }
    return System.getProperty("input.encoding", Charset.defaultCharset().name());
  }
}
