package org.bukkit.craftbukkit.libs.jline.internal;

import java.io.ByteArrayOutputStream;
import java.io.Closeable;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.text.MessageFormat;
import java.util.regex.Matcher;
import java.util.regex.Pattern;






















public final class TerminalLineSettings
{
  public static final String JLINE_STTY = "org.bukkit.craftbukkit.libs.jline.stty";
  public static final String DEFAULT_STTY = "stty";
  public static final String JLINE_SH = "org.bukkit.craftbukkit.libs.jline.sh";
  public static final String DEFAULT_SH = "sh";
  private String sttyCommand;
  private String shCommand;
  private String config;
  private long configLastFetched;
  
  public TerminalLineSettings()
    throws IOException, InterruptedException
  {
    sttyCommand = Configuration.getString("org.bukkit.craftbukkit.libs.jline.stty", "stty");
    shCommand = Configuration.getString("org.bukkit.craftbukkit.libs.jline.sh", "sh");
    config = get("-a");
    configLastFetched = System.currentTimeMillis();
    
    Log.debug(new Object[] { "Config: ", config });
    

    if (config.length() == 0) {
      throw new IOException(MessageFormat.format("Unrecognized stty code: {0}", new Object[] { config }));
    }
  }
  
  public String getConfig() {
    return config;
  }
  
  public void restore() throws IOException, InterruptedException {
    set("sane");
  }
  
  public String get(String args) throws IOException, InterruptedException {
    return stty(args);
  }
  
  public void set(String args) throws IOException, InterruptedException {
    stty(args);
  }
  







  public int getProperty(String name)
  {
    assert (name != null);
    
    long currentTime = System.currentTimeMillis();
    
    try
    {
      if ((config == null) || (currentTime - configLastFetched > 1000L)) {
        config = get("-a");
      }
    } catch (Exception e) {
      Log.debug(new Object[] { "Failed to query stty ", name, "\n", e });
    }
    

    if (currentTime - configLastFetched > 1000L) {
      configLastFetched = currentTime;
    }
    
    return getProperty(name, config);
  }
  










  protected static int getProperty(String name, String stty)
  {
    Pattern pattern = Pattern.compile(name + "\\s+=\\s+([^;]*)[;\\n\\r]");
    Matcher matcher = pattern.matcher(stty);
    if (!matcher.find())
    {
      pattern = Pattern.compile(name + "\\s+([^;]*)[;\\n\\r]");
      matcher = pattern.matcher(stty);
      if (!matcher.find())
      {
        pattern = Pattern.compile("(\\S*)\\s+" + name);
        matcher = pattern.matcher(stty);
        if (!matcher.find()) {
          return -1;
        }
      }
    }
    return parseControlChar(matcher.group(1));
  }
  
  private static int parseControlChar(String str)
  {
    if ("<undef>".equals(str)) {
      return -1;
    }
    
    if (str.charAt(0) == '0') {
      return Integer.parseInt(str, 8);
    }
    
    if ((str.charAt(0) >= '1') && (str.charAt(0) <= '9')) {
      return Integer.parseInt(str, 10);
    }
    
    if (str.charAt(0) == '^') {
      if (str.charAt(1) == '?') {
        return 127;
      }
      return str.charAt(1) - '@';
    }
    if ((str.charAt(0) == 'M') && (str.charAt(1) == '-')) {
      if (str.charAt(2) == '^') {
        if (str.charAt(3) == '?') {
          return 255;
        }
        return str.charAt(3) - '@' + 128;
      }
      
      return str.charAt(2) + '';
    }
    
    return str.charAt(0);
  }
  
  private String stty(String args) throws IOException, InterruptedException
  {
    assert (args != null);
    return exec(String.format("%s %s < /dev/tty", new Object[] { sttyCommand, args }));
  }
  
  private String exec(String cmd) throws IOException, InterruptedException {
    assert (cmd != null);
    return exec(new String[] { shCommand, "-c", cmd });
  }
  
  private String exec(String... cmd) throws IOException, InterruptedException {
    assert (cmd != null);
    
    ByteArrayOutputStream bout = new ByteArrayOutputStream();
    
    Log.trace(new Object[] { "Running: ", cmd });
    
    Process p = Runtime.getRuntime().exec(cmd);
    
    InputStream in = null;
    InputStream err = null;
    OutputStream out = null;
    try
    {
      in = p.getInputStream();
      int c; while ((c = in.read()) != -1) {
        bout.write(c);
      }
      err = p.getErrorStream();
      while ((c = err.read()) != -1) {
        bout.write(c);
      }
      out = p.getOutputStream();
      p.waitFor();
    }
    finally {
      close(new Closeable[] { in, out, err });
    }
    
    String result = bout.toString();
    
    Log.trace(new Object[] { "Result: ", result });
    
    return result;
  }
  
  private static void close(Closeable... closeables) {
    for (Closeable c : closeables) {
      try {
        c.close();
      }
      catch (Exception e) {}
    }
  }
}
