package org.bukkit.craftbukkit.libs.joptsimple;

import java.util.NoSuchElementException;

































class OptionSpecTokenizer
{
  private static final char POSIXLY_CORRECT_MARKER = '+';
  private String specification;
  private int index;
  
  OptionSpecTokenizer(String specification)
  {
    if (specification == null) {
      throw new NullPointerException("null option specification");
    }
    this.specification = specification;
  }
  
  boolean hasMore() {
    return index < specification.length();
  }
  
  AbstractOptionSpec<?> next() {
    if (!hasMore()) {
      throw new NoSuchElementException();
    }
    
    String optionCandidate = String.valueOf(specification.charAt(index));
    index += 1;
    

    if ("W".equals(optionCandidate)) {
      AbstractOptionSpec<?> spec = handleReservedForExtensionsToken();
      
      if (spec != null) {
        return spec;
      }
    }
    ParserRules.ensureLegalOption(optionCandidate);
    AbstractOptionSpec<?> spec;
    AbstractOptionSpec<?> spec; if (hasMore()) {
      spec = specification.charAt(index) == ':' ? handleArgumentAcceptingOption(optionCandidate) : new NoArgumentOptionSpec(optionCandidate);
    }
    else
    {
      spec = new NoArgumentOptionSpec(optionCandidate);
    }
    return spec;
  }
  
  void configure(OptionParser parser) {
    adjustForPosixlyCorrect(parser);
    
    while (hasMore())
      parser.recognize(next());
  }
  
  private void adjustForPosixlyCorrect(OptionParser parser) {
    if ('+' == specification.charAt(0)) {
      parser.posixlyCorrect(true);
      specification = specification.substring(1);
    }
  }
  
  private AbstractOptionSpec<?> handleReservedForExtensionsToken() {
    if (!hasMore()) {
      return new NoArgumentOptionSpec("W");
    }
    if (specification.charAt(index) == ';') {
      index += 1;
      return new AlternativeLongOptionSpec();
    }
    
    return null;
  }
  
  private AbstractOptionSpec<?> handleArgumentAcceptingOption(String candidate) {
    index += 1;
    
    if ((hasMore()) && (specification.charAt(index) == ':')) {
      index += 1;
      return new OptionalArgumentOptionSpec(candidate);
    }
    
    return new RequiredArgumentOptionSpec(candidate);
  }
}
