package org.bukkit.craftbukkit.libs.org.ibex.nestedvm.util;

import java.io.BufferedInputStream;
import java.io.DataInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintStream;










public class ELF
{
  private static final int ELF_MAGIC = 2135247942;
  public static final int ELFCLASSNONE = 0;
  public static final int ELFCLASS32 = 1;
  public static final int ELFCLASS64 = 2;
  public static final int ELFDATANONE = 0;
  public static final int ELFDATA2LSB = 1;
  public static final int ELFDATA2MSB = 2;
  public static final int SHT_SYMTAB = 2;
  public static final int SHT_STRTAB = 3;
  public static final int SHT_NOBITS = 8;
  public static final int SHF_WRITE = 1;
  public static final int SHF_ALLOC = 2;
  public static final int SHF_EXECINSTR = 4;
  public static final int PF_X = 1;
  public static final int PF_W = 2;
  public static final int PF_R = 4;
  public static final int PT_LOAD = 1;
  public static final short ET_EXEC = 2;
  public static final short EM_MIPS = 8;
  private Seekable data;
  public ELFIdent ident;
  public ELFHeader header;
  public PHeader[] pheaders;
  public SHeader[] sheaders;
  private byte[] stringTable;
  private boolean sectionReaderActive;
  private Symtab _symtab;
  
  private void readFully(byte[] paramArrayOfByte)
    throws IOException
  {
    int i = paramArrayOfByte.length;
    int j = 0;
    while (i > 0) {
      int k = data.read(paramArrayOfByte, j, i);
      if (k == -1) throw new IOException("EOF");
      j += k;
      i -= k;
    }
  }
  
  private int readIntBE() throws IOException {
    byte[] arrayOfByte = new byte[4];
    readFully(arrayOfByte);
    return (arrayOfByte[0] & 0xFF) << 24 | (arrayOfByte[1] & 0xFF) << 16 | (arrayOfByte[2] & 0xFF) << 8 | (arrayOfByte[3] & 0xFF) << 0;
  }
  
  private int readInt() throws IOException { int i = readIntBE();
    if ((ident != null) && (ident.data == 1))
      i = i << 24 & 0xFF000000 | i << 8 & 0xFF0000 | i >>> 8 & 0xFF00 | i >> 24 & 0xFF;
    return i;
  }
  
  private short readShortBE() throws IOException {
    byte[] arrayOfByte = new byte[2];
    readFully(arrayOfByte);
    return (short)((arrayOfByte[0] & 0xFF) << 8 | (arrayOfByte[1] & 0xFF) << 0);
  }
  
  private short readShort() throws IOException { short s = readShortBE();
    if ((ident != null) && (ident.data == 1))
      s = (short)((s << 8 & 0xFF00 | s >> 8 & 0xFF) & 0xFFFF);
    return s;
  }
  
  private byte readByte() throws IOException {
    byte[] arrayOfByte = new byte[1];
    readFully(arrayOfByte);
    return arrayOfByte[0];
  }
  
  public class ELFIdent {
    public byte klass;
    public byte data;
    public byte osabi;
    public byte abiversion;
    
    ELFIdent() throws IOException {
      if (ELF.this.readIntBE() != 2135247942) { throw new ELF.ELFException(ELF.this, "Bad Magic");
      }
      klass = ELF.this.readByte();
      if (klass != 1) { throw new ELF.ELFException(ELF.this, "org.bukkit.craftbukkit.libs.org.ibex.nestedvm.util.ELF does not suport 64-bit binaries");
      }
      data = ELF.this.readByte();
      if ((data != 1) && (data != 2)) { throw new ELF.ELFException(ELF.this, "Unknown byte order");
      }
      ELF.this.readByte();
      osabi = ELF.this.readByte();
      abiversion = ELF.this.readByte();
      for (int i = 0; i < 7; i++) ELF.this.readByte();
    }
  }
  
  public class ELFHeader {
    public short type;
    public short machine;
    public int version;
    public int entry;
    public int phoff;
    public int shoff;
    public int flags;
    public short ehsize;
    public short phentsize;
    public short phnum;
    public short shentsize;
    public short shnum;
    public short shstrndx;
    
    ELFHeader() throws IOException {
      type = ELF.this.readShort();
      machine = ELF.this.readShort();
      version = ELF.this.readInt();
      if (version != 1) throw new ELF.ELFException(ELF.this, "version != 1");
      entry = ELF.this.readInt();
      phoff = ELF.this.readInt();
      shoff = ELF.this.readInt();
      flags = ELF.this.readInt();
      ehsize = ELF.this.readShort();
      phentsize = ELF.this.readShort();
      phnum = ELF.this.readShort();
      shentsize = ELF.this.readShort();
      shnum = ELF.this.readShort();
      shstrndx = ELF.this.readShort();
    }
  }
  
  public class PHeader {
    public int type;
    public int offset;
    public int vaddr;
    public int paddr;
    public int filesz;
    public int memsz;
    public int flags;
    public int align;
    
    PHeader() throws IOException {
      type = ELF.this.readInt();
      offset = ELF.this.readInt();
      vaddr = ELF.this.readInt();
      paddr = ELF.this.readInt();
      filesz = ELF.this.readInt();
      memsz = ELF.this.readInt();
      flags = ELF.this.readInt();
      align = ELF.this.readInt();
      if (filesz > memsz) throw new ELF.ELFException(ELF.this, "ELF inconsistency: filesz > memsz (" + ELF.toHex(filesz) + " > " + ELF.toHex(memsz) + ")");
    }
    
    public boolean writable() { return (flags & 0x2) != 0; }
    
    public InputStream getInputStream() throws IOException {
      return new BufferedInputStream(new ELF.SectionInputStream(ELF.this, offset, offset + filesz));
    }
  }
  
  public class SHeader
  {
    int nameidx;
    public String name;
    public int type;
    public int flags;
    public int addr;
    public int offset;
    public int size;
    public int link;
    public int info;
    public int addralign;
    public int entsize;
    
    SHeader() throws IOException {
      nameidx = ELF.this.readInt();
      type = ELF.this.readInt();
      flags = ELF.this.readInt();
      addr = ELF.this.readInt();
      offset = ELF.this.readInt();
      size = ELF.this.readInt();
      link = ELF.this.readInt();
      info = ELF.this.readInt();
      addralign = ELF.this.readInt();
      entsize = ELF.this.readInt();
    }
    
    public InputStream getInputStream() throws IOException {
      return new BufferedInputStream(new ELF.SectionInputStream(ELF.this, offset, type == 8 ? 0 : offset + size));
    }
    

    public boolean isText() { return name.equals(".text"); }
    public boolean isData() { return (name.equals(".data")) || (name.equals(".sdata")) || (name.equals(".rodata")) || (name.equals(".ctors")) || (name.equals(".dtors")); }
    public boolean isBSS() { return (name.equals(".bss")) || (name.equals(".sbss")); }
  }
  
  public ELF(String paramString) throws IOException, ELF.ELFException { this(new Seekable.File(paramString, false)); }
  
  public ELF(Seekable paramSeekable) throws IOException, ELF.ELFException { data = paramSeekable;
    ident = new ELFIdent();
    header = new ELFHeader();
    pheaders = new PHeader[header.phnum];
    for (int i = 0; i < header.phnum; i++) {
      paramSeekable.seek(header.phoff + i * header.phentsize);
      pheaders[i] = new PHeader();
    }
    sheaders = new SHeader[header.shnum];
    for (i = 0; i < header.shnum; i++) {
      paramSeekable.seek(header.shoff + i * header.shentsize);
      sheaders[i] = new SHeader();
    }
    if ((header.shstrndx < 0) || (header.shstrndx >= header.shnum)) throw new ELFException("Bad shstrndx");
    paramSeekable.seek(sheaders[header.shstrndx].offset);
    stringTable = new byte[sheaders[header.shstrndx].size];
    readFully(stringTable);
    
    for (i = 0; i < header.shnum; i++) {
      SHeader localSHeader = sheaders[i];
      name = getString(nameidx);
    }
  }
  
  private String getString(int paramInt) { return getString(paramInt, stringTable); }
  
  private String getString(int paramInt, byte[] paramArrayOfByte) { StringBuffer localStringBuffer = new StringBuffer();
    if ((paramInt < 0) || (paramInt >= paramArrayOfByte.length)) return "<invalid strtab entry>";
    while ((paramInt >= 0) && (paramInt < paramArrayOfByte.length) && (paramArrayOfByte[paramInt] != 0)) localStringBuffer.append((char)paramArrayOfByte[(paramInt++)]);
    return localStringBuffer.toString();
  }
  
  public SHeader sectionWithName(String paramString) {
    for (int i = 0; i < sheaders.length; i++)
      if (sheaders[i].name.equals(paramString))
        return sheaders[i];
    return null;
  }
  
  public class ELFException extends IOException { ELFException(String paramString) { super(); }
  }
  
  private class SectionInputStream extends InputStream { private int pos;
    private int maxpos;
    
    SectionInputStream(int paramInt1, int paramInt2) throws IOException { if (sectionReaderActive)
        throw new IOException("Section reader already active");
      sectionReaderActive = true;
      pos = paramInt1;
      data.seek(pos);
      maxpos = paramInt2;
    }
    
    private int bytesLeft() { return maxpos - pos; }
    
    public int read() throws IOException { byte[] arrayOfByte = new byte[1];
      return read(arrayOfByte, 0, 1) == -1 ? -1 : arrayOfByte[0] & 0xFF;
    }
    
    public int read(byte[] paramArrayOfByte, int paramInt1, int paramInt2) throws IOException { int i = data.read(paramArrayOfByte, paramInt1, Math.min(paramInt2, bytesLeft())); if (i > 0) pos += i; return i; }
    
    public void close() { sectionReaderActive = false; }
  }
  
  public Symtab getSymtab() throws IOException
  {
    if (_symtab != null) { return _symtab;
    }
    if (sectionReaderActive) { throw new ELFException("Can't read the symtab while a section reader is active");
    }
    SHeader localSHeader1 = sectionWithName(".symtab");
    if ((localSHeader1 == null) || (type != 2)) { return null;
    }
    SHeader localSHeader2 = sectionWithName(".strtab");
    if ((localSHeader2 == null) || (type != 3)) { return null;
    }
    byte[] arrayOfByte = new byte[size];
    DataInputStream localDataInputStream = new DataInputStream(localSHeader2.getInputStream());
    localDataInputStream.readFully(arrayOfByte);
    localDataInputStream.close();
    
    return this._symtab = new Symtab(offset, size, arrayOfByte);
  }
  
  public class Symtab {
    public ELF.Symbol[] symbols;
    
    Symtab(int paramInt1, int paramInt2, byte[] paramArrayOfByte) throws IOException {
      data.seek(paramInt1);
      int i = paramInt2 / 16;
      symbols = new ELF.Symbol[i];
      for (int j = 0; j < i; j++) symbols[j] = new ELF.Symbol(ELF.this, paramArrayOfByte);
    }
    
    public ELF.Symbol getSymbol(String paramString) {
      ELF.Symbol localSymbol = null;
      for (int i = 0; i < symbols.length; i++) {
        if (symbols[i].name.equals(paramString)) {
          if (localSymbol == null) {
            localSymbol = symbols[i];
          } else
            System.err.println("WARNING: Multiple symbol matches for " + paramString);
        }
      }
      return localSymbol;
    }
    
    public ELF.Symbol getGlobalSymbol(String paramString) {
      for (int i = 0; i < symbols.length; i++)
        if ((symbols[i].binding == 1) && (symbols[i].name.equals(paramString)))
          return symbols[i];
      return null;
    }
  }
  
  public class Symbol
  {
    public String name;
    public int addr;
    public int size;
    public byte info;
    public byte type;
    public byte binding;
    public byte other;
    public short shndx;
    public ELF.SHeader sheader;
    public static final int STT_FUNC = 2;
    public static final int STB_GLOBAL = 1;
    
    Symbol(byte[] paramArrayOfByte) throws IOException {
      name = ELF.this.getString(ELF.access$300(ELF.this), paramArrayOfByte);
      addr = ELF.this.readInt();
      size = ELF.this.readInt();
      info = ELF.this.readByte();
      type = ((byte)(info & 0xF));
      binding = ((byte)(info >> 4));
      other = ELF.this.readByte();
      shndx = ELF.this.readShort();
    }
  }
  
  private static String toHex(int paramInt) { return "0x" + Long.toString(paramInt & 0xFFFFFFFF, 16); }
}
