package org.bukkit.craftbukkit.v1_7_R1;

import java.lang.ref.WeakReference;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Set;
import net.minecraft.server.v1_7_R1.BiomeBase;
import net.minecraft.server.v1_7_R1.ChunkPosition;
import net.minecraft.server.v1_7_R1.ChunkSection;
import net.minecraft.server.v1_7_R1.EmptyChunk;
import net.minecraft.server.v1_7_R1.NibbleArray;
import net.minecraft.server.v1_7_R1.WorldChunkManager;
import net.minecraft.server.v1_7_R1.WorldServer;
import org.bukkit.ChunkSnapshot;
import org.bukkit.block.Block;
import org.bukkit.block.BlockState;
import org.bukkit.craftbukkit.v1_7_R1.block.CraftBlock;

public class CraftChunk implements org.bukkit.Chunk
{
  private WeakReference<net.minecraft.server.v1_7_R1.Chunk> weakChunk;
  private final WorldServer worldServer;
  private final int x;
  private final int z;
  private static final byte[] emptyData = new byte[2048];
  private static final short[] emptyBlockIDs = new short[4096];
  private static final byte[] emptySkyLight = new byte[2048];
  
  public CraftChunk(net.minecraft.server.v1_7_R1.Chunk chunk) {
    if (!(chunk instanceof EmptyChunk)) {
      weakChunk = new WeakReference(chunk);
    }
    
    worldServer = ((WorldServer)getHandle()world);
    x = getHandle()locX;
    z = getHandle()locZ;
  }
  
  public org.bukkit.World getWorld() {
    return worldServer.getWorld();
  }
  
  public CraftWorld getCraftWorld() {
    return (CraftWorld)getWorld();
  }
  
  public net.minecraft.server.v1_7_R1.Chunk getHandle() {
    net.minecraft.server.v1_7_R1.Chunk c = (net.minecraft.server.v1_7_R1.Chunk)weakChunk.get();
    
    if (c == null) {
      c = worldServer.getChunkAt(x, z);
      
      if (!(c instanceof EmptyChunk)) {
        weakChunk = new WeakReference(c);
      }
    }
    
    return c;
  }
  
  void breakLink() {
    weakChunk.clear();
  }
  
  public int getX() {
    return x;
  }
  
  public int getZ() {
    return z;
  }
  
  public String toString()
  {
    return "CraftChunk{x=" + getX() + "z=" + getZ() + '}';
  }
  
  public Block getBlock(int x, int y, int z) {
    return new CraftBlock(this, getX() << 4 | x & 0xF, y & 0xFF, getZ() << 4 | z & 0xF);
  }
  
  public org.bukkit.entity.Entity[] getEntities() {
    int count = 0;int index = 0;
    net.minecraft.server.v1_7_R1.Chunk chunk = getHandle();
    
    for (int i = 0; i < 16; i++) {
      count += entitySlices[i].size();
    }
    
    org.bukkit.entity.Entity[] entities = new org.bukkit.entity.Entity[count];
    
    for (int i = 0; i < 16; i++) {
      for (Object obj : entitySlices[i].toArray()) {
        if ((obj instanceof net.minecraft.server.v1_7_R1.Entity))
        {


          entities[(index++)] = ((net.minecraft.server.v1_7_R1.Entity)obj).getBukkitEntity();
        }
      }
    }
    return entities;
  }
  
  public BlockState[] getTileEntities() {
    int index = 0;
    net.minecraft.server.v1_7_R1.Chunk chunk = getHandle();
    BlockState[] entities = new BlockState[tileEntities.size()];
    
    for (Object obj : tileEntities.keySet().toArray())
      if ((obj instanceof ChunkPosition))
      {


        ChunkPosition position = (ChunkPosition)obj;
        entities[(index++)] = worldServer.getWorld().getBlockAt(x + (locX << 4), y, z + (locZ << 4)).getState();
      }
    return entities;
  }
  
  public boolean isLoaded() {
    return getWorld().isChunkLoaded(this);
  }
  
  public boolean load() {
    return getWorld().loadChunk(getX(), getZ(), true);
  }
  
  public boolean load(boolean generate) {
    return getWorld().loadChunk(getX(), getZ(), generate);
  }
  
  public boolean unload() {
    return getWorld().unloadChunk(getX(), getZ());
  }
  
  public boolean unload(boolean save) {
    return getWorld().unloadChunk(getX(), getZ(), save);
  }
  
  public boolean unload(boolean save, boolean safe) {
    return getWorld().unloadChunk(getX(), getZ(), save, safe);
  }
  
  public ChunkSnapshot getChunkSnapshot() {
    return getChunkSnapshot(true, false, false);
  }
  
  public ChunkSnapshot getChunkSnapshot(boolean includeMaxBlockY, boolean includeBiome, boolean includeBiomeTempRain) {
    net.minecraft.server.v1_7_R1.Chunk chunk = getHandle();
    
    ChunkSection[] cs = chunk.i();
    short[][] sectionBlockIDs = new short[cs.length][];
    byte[][] sectionBlockData = new byte[cs.length][];
    byte[][] sectionSkyLights = new byte[cs.length][];
    byte[][] sectionEmitLights = new byte[cs.length][];
    boolean[] sectionEmpty = new boolean[cs.length];
    
    for (int i = 0; i < cs.length; i++) {
      if (cs[i] == null) {
        sectionBlockIDs[i] = emptyBlockIDs;
        sectionBlockData[i] = emptyData;
        sectionSkyLights[i] = emptySkyLight;
        sectionEmitLights[i] = emptyData;
        sectionEmpty[i] = true;
      } else {
        short[] blockids = new short[4096];
        byte[] baseids = cs[i].getIdArray();
        

        for (int j = 0; j < 4096; j++) {
          blockids[j] = ((short)(baseids[j] & 0xFF));
        }
        
        if (cs[i].getExtendedIdArray() != null) {
          byte[] extids = getExtendedIdArray()a;
          
          for (int j = 0; j < 2048; j++) {
            short b = (short)(extids[j] & 0xFF);
            
            if (b != 0)
            {


              int tmp222_221 = (j << 1); short[] tmp222_216 = blockids;tmp222_216[tmp222_221] = ((short)(tmp222_216[tmp222_221] | (b & 0xF) << 8)); int 
                tmp243_242 = ((j << 1) + 1); short[] tmp243_235 = blockids;tmp243_235[tmp243_242] = ((short)(tmp243_235[tmp243_242] | (b & 0xF0) << 4));
            }
          }
        }
        sectionBlockIDs[i] = blockids;
        

        sectionBlockData[i] = new byte[2048];
        System.arraycopy(getDataArray()a, 0, sectionBlockData[i], 0, 2048);
        if (cs[i].getSkyLightArray() == null) {
          sectionSkyLights[i] = emptyData;
        } else {
          sectionSkyLights[i] = new byte[2048];
          System.arraycopy(getSkyLightArray()a, 0, sectionSkyLights[i], 0, 2048);
        }
        sectionEmitLights[i] = new byte[2048];
        System.arraycopy(getEmittedLightArray()a, 0, sectionEmitLights[i], 0, 2048);
      }
    }
    
    int[] hmap = null;
    
    if (includeMaxBlockY) {
      hmap = new int[256];
      System.arraycopy(heightMap, 0, hmap, 0, 256);
    }
    
    BiomeBase[] biome = null;
    double[] biomeTemp = null;
    double[] biomeRain = null;
    
    if ((includeBiome) || (includeBiomeTempRain)) {
      WorldChunkManager wcm = world.getWorldChunkManager();
      
      if (includeBiome) {
        biome = new BiomeBase[256];
        for (int i = 0; i < 256; i++) {
          biome[i] = chunk.a(i & 0xF, i >> 4, wcm);
        }
      }
      
      if (includeBiomeTempRain) {
        biomeTemp = new double[256];
        biomeRain = new double[256];
        float[] dat = getTemperatures(wcm, getX() << 4, getZ() << 4);
        
        for (int i = 0; i < 256; i++) {
          biomeTemp[i] = dat[i];
        }
        
        dat = wcm.getWetness(null, getX() << 4, getZ() << 4, 16, 16);
        
        for (int i = 0; i < 256; i++) {
          biomeRain[i] = dat[i];
        }
      }
    }
    
    org.bukkit.World world = getWorld();
    return new CraftChunkSnapshot(getX(), getZ(), world.getName(), world.getFullTime(), sectionBlockIDs, sectionBlockData, sectionSkyLights, sectionEmitLights, sectionEmpty, hmap, biome, biomeTemp, biomeRain);
  }
  
  public static ChunkSnapshot getEmptyChunkSnapshot(int x, int z, CraftWorld world, boolean includeBiome, boolean includeBiomeTempRain) {
    BiomeBase[] biome = null;
    double[] biomeTemp = null;
    double[] biomeRain = null;
    
    if ((includeBiome) || (includeBiomeTempRain)) {
      WorldChunkManager wcm = world.getHandle().getWorldChunkManager();
      
      if (includeBiome) {
        biome = new BiomeBase[256];
        for (int i = 0; i < 256; i++) {
          biome[i] = world.getHandle().getBiome((x << 4) + (i & 0xF), (z << 4) + (i >> 4));
        }
      }
      
      if (includeBiomeTempRain) {
        biomeTemp = new double[256];
        biomeRain = new double[256];
        float[] dat = getTemperatures(wcm, x << 4, z << 4);
        
        for (int i = 0; i < 256; i++) {
          biomeTemp[i] = dat[i];
        }
        
        dat = wcm.getWetness(null, x << 4, z << 4, 16, 16);
        
        for (int i = 0; i < 256; i++) {
          biomeRain[i] = dat[i];
        }
      }
    }
    

    int hSection = world.getMaxHeight() >> 4;
    short[][] blockIDs = new short[hSection][];
    byte[][] skyLight = new byte[hSection][];
    byte[][] emitLight = new byte[hSection][];
    byte[][] blockData = new byte[hSection][];
    boolean[] empty = new boolean[hSection];
    
    for (int i = 0; i < hSection; i++) {
      blockIDs[i] = emptyBlockIDs;
      skyLight[i] = emptySkyLight;
      emitLight[i] = emptyData;
      blockData[i] = emptyData;
      empty[i] = true;
    }
    
    return new CraftChunkSnapshot(x, z, world.getName(), world.getFullTime(), blockIDs, blockData, skyLight, emitLight, empty, new int[256], biome, biomeTemp, biomeRain);
  }
  
  private static float[] getTemperatures(WorldChunkManager chunkmanager, int chunkX, int chunkZ) {
    BiomeBase[] biomes = chunkmanager.getBiomes(null, chunkX, chunkZ, 16, 16);
    float[] temps = new float[biomes.length];
    
    for (int i = 0; i < biomes.length; i++) {
      float temp = temperature;
      
      if (temp > 1.0F) {
        temp = 1.0F;
      }
      
      temps[i] = temp;
    }
    
    return temps;
  }
  
  static {
    Arrays.fill(emptySkyLight, (byte)-1);
  }
}
