package org.bukkit.craftbukkit.v1_7_R1;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Random;
import java.util.Set;
import net.minecraft.server.v1_7_R1.BiomeBase;
import net.minecraft.server.v1_7_R1.ChunkCoordinates;
import net.minecraft.server.v1_7_R1.ChunkProviderServer;
import net.minecraft.server.v1_7_R1.EmptyChunk;
import net.minecraft.server.v1_7_R1.EntityArrow;
import net.minecraft.server.v1_7_R1.EntityFallingBlock;
import net.minecraft.server.v1_7_R1.EntityLightning;
import net.minecraft.server.v1_7_R1.EntityPlayer;
import net.minecraft.server.v1_7_R1.ExceptionWorldConflict;
import net.minecraft.server.v1_7_R1.GameRules;
import net.minecraft.server.v1_7_R1.WorldData;
import net.minecraft.server.v1_7_R1.WorldServer;
import org.apache.commons.lang.Validate;
import org.bukkit.Difficulty;
import org.bukkit.Effect;
import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.TreeType;
import org.bukkit.World.Environment;
import org.bukkit.block.Biome;
import org.bukkit.block.BlockFace;
import org.bukkit.craftbukkit.v1_7_R1.entity.CraftPlayer;
import org.bukkit.craftbukkit.v1_7_R1.metadata.BlockMetadataStore;
import org.bukkit.craftbukkit.v1_7_R1.metadata.WorldMetadataStore;
import org.bukkit.craftbukkit.v1_7_R1.util.LongHash;
import org.bukkit.craftbukkit.v1_7_R1.util.LongHashSet;
import org.bukkit.craftbukkit.v1_7_R1.util.LongObjectHashMap;
import org.bukkit.entity.Arrow;
import org.bukkit.entity.CreatureType;
import org.bukkit.entity.EntityType;
import org.bukkit.entity.FallingBlock;
import org.bukkit.entity.LightningStrike;
import org.bukkit.entity.LivingEntity;
import org.bukkit.entity.Player;
import org.bukkit.event.entity.CreatureSpawnEvent.SpawnReason;
import org.bukkit.event.weather.ThunderChangeEvent;
import org.bukkit.event.weather.WeatherChangeEvent;
import org.bukkit.generator.ChunkGenerator;
import org.bukkit.inventory.ItemStack;
import org.bukkit.metadata.MetadataValue;
import org.bukkit.plugin.Plugin;
import org.bukkit.plugin.PluginManager;
import org.bukkit.util.Vector;

public class CraftWorld implements org.bukkit.World
{
  public static final int CUSTOM_DIMENSION_OFFSET = 10;
  private final WorldServer world;
  private World.Environment environment;
  private final CraftServer server = (CraftServer)org.bukkit.Bukkit.getServer();
  private final ChunkGenerator generator;
  private final List<org.bukkit.generator.BlockPopulator> populators = new ArrayList();
  private final BlockMetadataStore blockMetadata = new BlockMetadataStore(this);
  private int monsterSpawn = -1;
  private int animalSpawn = -1;
  private int waterAnimalSpawn = -1;
  private int ambientSpawn = -1;
  private int chunkLoadCount = 0;
  
  private int chunkGCTickCount;
  private static final Random rand = new Random();
  
  public CraftWorld(WorldServer world, ChunkGenerator gen, World.Environment env) {
    this.world = world;
    generator = gen;
    
    environment = env;
    
    if (server.chunkGCPeriod > 0) {
      chunkGCTickCount = rand.nextInt(server.chunkGCPeriod);
    }
  }
  
  public org.bukkit.block.Block getBlockAt(int x, int y, int z) {
    return getChunkAt(x >> 4, z >> 4).getBlock(x & 0xF, y & 0xFF, z & 0xF);
  }
  
  public int getBlockTypeIdAt(int x, int y, int z) {
    return world.getTypeId(x, y, z);
  }
  
  public int getHighestBlockYAt(int x, int z) {
    if (!isChunkLoaded(x >> 4, z >> 4)) {
      loadChunk(x >> 4, z >> 4);
    }
    
    return world.getHighestBlockYAt(x, z);
  }
  
  public Location getSpawnLocation() {
    ChunkCoordinates spawn = world.getSpawn();
    return new Location(this, x, y, z);
  }
  
  public boolean setSpawnLocation(int x, int y, int z) {
    try {
      Location previousLocation = getSpawnLocation();
      world.worldData.setSpawn(x, y, z);
      

      org.bukkit.event.world.SpawnChangeEvent event = new org.bukkit.event.world.SpawnChangeEvent(this, previousLocation);
      server.getPluginManager().callEvent(event);
      
      return true;
    } catch (Exception e) {}
    return false;
  }
  
  public org.bukkit.Chunk getChunkAt(int x, int z)
  {
    return world.chunkProviderServer.getChunkAt(x, z).bukkitChunk;
  }
  
  public org.bukkit.Chunk getChunkAt(org.bukkit.block.Block block) {
    return getChunkAt(block.getX() >> 4, block.getZ() >> 4);
  }
  
  public boolean isChunkLoaded(int x, int z) {
    return world.chunkProviderServer.isChunkLoaded(x, z);
  }
  
  public org.bukkit.Chunk[] getLoadedChunks() {
    Object[] chunks = world.chunkProviderServer.chunks.values().toArray();
    org.bukkit.Chunk[] craftChunks = new CraftChunk[chunks.length];
    
    for (int i = 0; i < chunks.length; i++) {
      net.minecraft.server.v1_7_R1.Chunk chunk = (net.minecraft.server.v1_7_R1.Chunk)chunks[i];
      craftChunks[i] = bukkitChunk;
    }
    
    return craftChunks;
  }
  
  public void loadChunk(int x, int z) {
    loadChunk(x, z, true);
  }
  
  public boolean unloadChunk(org.bukkit.Chunk chunk) {
    return unloadChunk(chunk.getX(), chunk.getZ());
  }
  
  public boolean unloadChunk(int x, int z) {
    return unloadChunk(x, z, true);
  }
  
  public boolean unloadChunk(int x, int z, boolean save) {
    return unloadChunk(x, z, save, false);
  }
  
  public boolean unloadChunkRequest(int x, int z) {
    return unloadChunkRequest(x, z, true);
  }
  
  public boolean unloadChunkRequest(int x, int z, boolean safe) {
    if ((safe) && (isChunkInUse(x, z))) {
      return false;
    }
    
    world.chunkProviderServer.queueUnload(x, z);
    
    return true;
  }
  
  public boolean unloadChunk(int x, int z, boolean save, boolean safe) {
    if ((safe) && (isChunkInUse(x, z))) {
      return false;
    }
    
    net.minecraft.server.v1_7_R1.Chunk chunk = world.chunkProviderServer.getOrCreateChunk(x, z);
    if (mustSave) {
      save = true;
    }
    
    chunk.removeEntities();
    
    if ((save) && (!(chunk instanceof EmptyChunk))) {
      world.chunkProviderServer.saveChunk(chunk);
      world.chunkProviderServer.saveChunkNOP(chunk);
    }
    
    world.chunkProviderServer.unloadQueue.remove(x, z);
    world.chunkProviderServer.chunks.remove(LongHash.toLong(x, z));
    
    return true;
  }
  
  public boolean regenerateChunk(int x, int z) {
    unloadChunk(x, z, false, false);
    
    world.chunkProviderServer.unloadQueue.remove(x, z);
    
    net.minecraft.server.v1_7_R1.Chunk chunk = null;
    
    if (world.chunkProviderServer.chunkProvider == null) {
      chunk = world.chunkProviderServer.emptyChunk;
    } else {
      chunk = world.chunkProviderServer.chunkProvider.getOrCreateChunk(x, z);
    }
    
    chunkLoadPostProcess(chunk, x, z);
    
    refreshChunk(x, z);
    
    return chunk != null;
  }
  
  public boolean refreshChunk(int x, int z) {
    if (!isChunkLoaded(x, z)) {
      return false;
    }
    
    int px = x << 4;
    int pz = z << 4;
    



    int height = getMaxHeight() / 16;
    for (int idx = 0; idx < 64; idx++) {
      world.notify(px + idx / height, idx % height * 16, pz);
    }
    world.notify(px + 15, height * 16 - 1, pz + 15);
    
    return true;
  }
  
  public boolean isChunkInUse(int x, int z) {
    return world.getPlayerChunkMap().isChunkInUse(x, z);
  }
  
  public boolean loadChunk(int x, int z, boolean generate) {
    chunkLoadCount += 1;
    if (generate)
    {
      return world.chunkProviderServer.getChunkAt(x, z) != null;
    }
    
    world.chunkProviderServer.unloadQueue.remove(x, z);
    net.minecraft.server.v1_7_R1.Chunk chunk = (net.minecraft.server.v1_7_R1.Chunk)world.chunkProviderServer.chunks.get(LongHash.toLong(x, z));
    
    if (chunk == null) {
      chunk = world.chunkProviderServer.loadChunk(x, z);
      
      chunkLoadPostProcess(chunk, x, z);
    }
    return chunk != null;
  }
  
  private void chunkLoadPostProcess(net.minecraft.server.v1_7_R1.Chunk chunk, int x, int z) {
    if (chunk != null) {
      world.chunkProviderServer.chunks.put(LongHash.toLong(x, z), chunk);
      
      chunk.addEntities();
      
      if ((!done) && (world.chunkProviderServer.isChunkLoaded(x + 1, z + 1)) && (world.chunkProviderServer.isChunkLoaded(x, z + 1)) && (world.chunkProviderServer.isChunkLoaded(x + 1, z))) {
        world.chunkProviderServer.getChunkAt(world.chunkProviderServer, x, z);
      }
      
      if ((world.chunkProviderServer.isChunkLoaded(x - 1, z)) && (!world.chunkProviderServer.getOrCreateChunk(x - 1, z).done) && (world.chunkProviderServer.isChunkLoaded(x - 1, z + 1)) && (world.chunkProviderServer.isChunkLoaded(x, z + 1)) && (world.chunkProviderServer.isChunkLoaded(x - 1, z))) {
        world.chunkProviderServer.getChunkAt(world.chunkProviderServer, x - 1, z);
      }
      
      if ((world.chunkProviderServer.isChunkLoaded(x, z - 1)) && (!world.chunkProviderServer.getOrCreateChunk(x, z - 1).done) && (world.chunkProviderServer.isChunkLoaded(x + 1, z - 1)) && (world.chunkProviderServer.isChunkLoaded(x, z - 1)) && (world.chunkProviderServer.isChunkLoaded(x + 1, z))) {
        world.chunkProviderServer.getChunkAt(world.chunkProviderServer, x, z - 1);
      }
      
      if ((world.chunkProviderServer.isChunkLoaded(x - 1, z - 1)) && (!world.chunkProviderServer.getOrCreateChunk(x - 1, z - 1).done) && (world.chunkProviderServer.isChunkLoaded(x - 1, z - 1)) && (world.chunkProviderServer.isChunkLoaded(x, z - 1)) && (world.chunkProviderServer.isChunkLoaded(x - 1, z))) {
        world.chunkProviderServer.getChunkAt(world.chunkProviderServer, x - 1, z - 1);
      }
    }
  }
  
  public boolean isChunkLoaded(org.bukkit.Chunk chunk) {
    return isChunkLoaded(chunk.getX(), chunk.getZ());
  }
  
  public void loadChunk(org.bukkit.Chunk chunk) {
    loadChunk(chunk.getX(), chunk.getZ());
    getChunkAtgetX(), getZ()getHandle()bukkitChunk = chunk;
  }
  
  public WorldServer getHandle() {
    return world;
  }
  
  public org.bukkit.entity.Item dropItem(Location loc, ItemStack item) {
    Validate.notNull(item, "Cannot drop a Null item.");
    Validate.isTrue(item.getTypeId() != 0, "Cannot drop AIR.");
    net.minecraft.server.v1_7_R1.EntityItem entity = new net.minecraft.server.v1_7_R1.EntityItem(world, loc.getX(), loc.getY(), loc.getZ(), org.bukkit.craftbukkit.v1_7_R1.inventory.CraftItemStack.asNMSCopy(item));
    pickupDelay = 10;
    world.addEntity(entity);
    

    return new org.bukkit.craftbukkit.v1_7_R1.entity.CraftItem(world.getServer(), entity);
  }
  
  public org.bukkit.entity.Item dropItemNaturally(Location loc, ItemStack item) {
    double xs = world.random.nextFloat() * 0.7F + 0.1500000059604645D;
    double ys = world.random.nextFloat() * 0.7F + 0.1500000059604645D;
    double zs = world.random.nextFloat() * 0.7F + 0.1500000059604645D;
    loc = loc.clone();
    loc.setX(loc.getX() + xs);
    loc.setY(loc.getY() + ys);
    loc.setZ(loc.getZ() + zs);
    return dropItem(loc, item);
  }
  
  public Arrow spawnArrow(Location loc, Vector velocity, float speed, float spread) {
    Validate.notNull(loc, "Can not spawn arrow with a null location");
    Validate.notNull(velocity, "Can not spawn arrow with a null velocity");
    
    EntityArrow arrow = new EntityArrow(world);
    arrow.setPositionRotation(loc.getX(), loc.getY(), loc.getZ(), loc.getYaw(), loc.getPitch());
    arrow.shoot(velocity.getX(), velocity.getY(), velocity.getZ(), speed, spread);
    world.addEntity(arrow);
    return (Arrow)arrow.getBukkitEntity();
  }
  
  @Deprecated
  public LivingEntity spawnCreature(Location loc, CreatureType creatureType) {
    return spawnCreature(loc, creatureType.toEntityType());
  }
  
  @Deprecated
  public LivingEntity spawnCreature(Location loc, EntityType creatureType) {
    Validate.isTrue(creatureType.isAlive(), "EntityType not instance of LivingEntity");
    return (LivingEntity)spawnEntity(loc, creatureType);
  }
  
  public org.bukkit.entity.Entity spawnEntity(Location loc, EntityType entityType) {
    return spawn(loc, entityType.getEntityClass());
  }
  
  public LightningStrike strikeLightning(Location loc) {
    EntityLightning lightning = new EntityLightning(world, loc.getX(), loc.getY(), loc.getZ());
    world.strikeLightning(lightning);
    return new org.bukkit.craftbukkit.v1_7_R1.entity.CraftLightningStrike(server, lightning);
  }
  
  public LightningStrike strikeLightningEffect(Location loc) {
    EntityLightning lightning = new EntityLightning(world, loc.getX(), loc.getY(), loc.getZ(), true);
    world.strikeLightning(lightning);
    return new org.bukkit.craftbukkit.v1_7_R1.entity.CraftLightningStrike(server, lightning);
  }
  
  public boolean generateTree(Location loc, TreeType type) {
    return generateTree(loc, type, world);
  }
  
  public boolean generateTree(Location loc, TreeType type, org.bukkit.BlockChangeDelegate delegate) {
    net.minecraft.server.v1_7_R1.BlockSapling.TreeGenerator gen;
    switch (1.$SwitchMap$org$bukkit$TreeType[type.ordinal()]) {
    case 1: 
      gen = new net.minecraft.server.v1_7_R1.WorldGenBigTree(true);
      break;
    case 2: 
      gen = new net.minecraft.server.v1_7_R1.WorldGenForest(true, false);
      break;
    case 3: 
      gen = new net.minecraft.server.v1_7_R1.WorldGenTaiga2(true);
      break;
    case 4: 
      gen = new net.minecraft.server.v1_7_R1.WorldGenTaiga1();
      break;
    case 5: 
      gen = new net.minecraft.server.v1_7_R1.WorldGenJungleTree(true, 10, 20, 3, 3);
      break;
    case 6: 
      gen = new net.minecraft.server.v1_7_R1.WorldGenTrees(true, 4 + rand.nextInt(7), 3, 3, false);
      break;
    case 7: 
      gen = new net.minecraft.server.v1_7_R1.WorldGenGroundBush(3, 0);
      break;
    case 8: 
      gen = new net.minecraft.server.v1_7_R1.WorldGenHugeMushroom(1);
      break;
    case 9: 
      gen = new net.minecraft.server.v1_7_R1.WorldGenHugeMushroom(0);
      break;
    case 10: 
      gen = new net.minecraft.server.v1_7_R1.WorldGenSwampTree();
      break;
    case 11: 
      gen = new net.minecraft.server.v1_7_R1.WorldGenAcaciaTree(true);
      break;
    case 12: 
      gen = new net.minecraft.server.v1_7_R1.WorldGenForestTree(true);
      break;
    case 13: 
      gen = new net.minecraft.server.v1_7_R1.WorldGenMegaTree(true, rand.nextBoolean());
      break;
    case 14: 
      gen = new net.minecraft.server.v1_7_R1.WorldGenForest(true, true);
      break;
    case 15: 
    default: 
      gen = new net.minecraft.server.v1_7_R1.WorldGenTrees(true);
    }
    
    
    return gen.generate(new CraftBlockChangeDelegate(delegate), rand, loc.getBlockX(), loc.getBlockY(), loc.getBlockZ());
  }
  
  public net.minecraft.server.v1_7_R1.TileEntity getTileEntityAt(int x, int y, int z) {
    return world.getTileEntity(x, y, z);
  }
  
  public String getName() {
    return world.worldData.getName();
  }
  
  @Deprecated
  public long getId() {
    return world.worldData.getSeed();
  }
  
  public java.util.UUID getUID() {
    return world.getDataManager().getUUID();
  }
  
  public String toString()
  {
    return "CraftWorld{name=" + getName() + '}';
  }
  
  public long getTime() {
    long time = getFullTime() % 24000L;
    if (time < 0L) time += 24000L;
    return time;
  }
  
  public void setTime(long time) {
    long margin = (time - getFullTime()) % 24000L;
    if (margin < 0L) margin += 24000L;
    setFullTime(getFullTime() + margin);
  }
  
  public long getFullTime() {
    return world.getDayTime();
  }
  
  public void setFullTime(long time) {
    world.setDayTime(time);
    

    for (Player p : getPlayers()) {
      CraftPlayer cp = (CraftPlayer)p;
      if (getHandle()playerConnection != null)
      {
        getHandle()playerConnection.sendPacket(new net.minecraft.server.v1_7_R1.PacketPlayOutUpdateTime(getHandle()world.getTime(), cp.getHandle().getPlayerTime(), getHandle()world.getGameRules().getBoolean("doDaylightCycle"))); }
    }
  }
  
  public boolean createExplosion(double x, double y, double z, float power) {
    return createExplosion(x, y, z, power, false, true);
  }
  
  public boolean createExplosion(double x, double y, double z, float power, boolean setFire) {
    return createExplosion(x, y, z, power, setFire, true);
  }
  
  public boolean createExplosion(double x, double y, double z, float power, boolean setFire, boolean breakBlocks) {
    return !world.createExplosion(null, x, y, z, power, setFire, breakBlocks).wasCanceled;
  }
  
  public boolean createExplosion(Location loc, float power) {
    return createExplosion(loc, power, false);
  }
  
  public boolean createExplosion(Location loc, float power, boolean setFire) {
    return createExplosion(loc.getX(), loc.getY(), loc.getZ(), power, setFire);
  }
  
  public World.Environment getEnvironment() {
    return environment;
  }
  
  public void setEnvironment(World.Environment env) {
    if (environment != env) {
      environment = env;
      world.worldProvider = net.minecraft.server.v1_7_R1.WorldProvider.byDimension(environment.getId());
    }
  }
  
  public org.bukkit.block.Block getBlockAt(Location location) {
    return getBlockAt(location.getBlockX(), location.getBlockY(), location.getBlockZ());
  }
  
  public int getBlockTypeIdAt(Location location) {
    return getBlockTypeIdAt(location.getBlockX(), location.getBlockY(), location.getBlockZ());
  }
  
  public int getHighestBlockYAt(Location location) {
    return getHighestBlockYAt(location.getBlockX(), location.getBlockZ());
  }
  
  public org.bukkit.Chunk getChunkAt(Location location) {
    return getChunkAt(location.getBlockX() >> 4, location.getBlockZ() >> 4);
  }
  
  public ChunkGenerator getGenerator() {
    return generator;
  }
  
  public List<org.bukkit.generator.BlockPopulator> getPopulators() {
    return populators;
  }
  
  public org.bukkit.block.Block getHighestBlockAt(int x, int z) {
    return getBlockAt(x, getHighestBlockYAt(x, z), z);
  }
  
  public org.bukkit.block.Block getHighestBlockAt(Location location) {
    return getHighestBlockAt(location.getBlockX(), location.getBlockZ());
  }
  
  public Biome getBiome(int x, int z) {
    return org.bukkit.craftbukkit.v1_7_R1.block.CraftBlock.biomeBaseToBiome(world.getBiome(x, z));
  }
  
  public void setBiome(int x, int z, Biome bio) {
    BiomeBase bb = org.bukkit.craftbukkit.v1_7_R1.block.CraftBlock.biomeToBiomeBase(bio);
    if (world.isLoaded(x, 0, z)) {
      net.minecraft.server.v1_7_R1.Chunk chunk = world.getChunkAtWorldCoords(x, z);
      
      if (chunk != null) {
        byte[] biomevals = chunk.m();
        biomevals[((z & 0xF) << 4 | x & 0xF)] = ((byte)id);
      }
    }
  }
  
  public double getTemperature(int x, int z) {
    return world.getBiome(x, z).temperature;
  }
  
  public double getHumidity(int x, int z) {
    return world.getBiome(x, z).humidity;
  }
  
  public List<org.bukkit.entity.Entity> getEntities() {
    List<org.bukkit.entity.Entity> list = new ArrayList();
    
    for (Object o : world.entityList) {
      if ((o instanceof net.minecraft.server.v1_7_R1.Entity)) {
        net.minecraft.server.v1_7_R1.Entity mcEnt = (net.minecraft.server.v1_7_R1.Entity)o;
        org.bukkit.entity.Entity bukkitEntity = mcEnt.getBukkitEntity();
        

        if (bukkitEntity != null) {
          list.add(bukkitEntity);
        }
      }
    }
    
    return list;
  }
  
  public List<LivingEntity> getLivingEntities() {
    List<LivingEntity> list = new ArrayList();
    
    for (Object o : world.entityList) {
      if ((o instanceof net.minecraft.server.v1_7_R1.Entity)) {
        net.minecraft.server.v1_7_R1.Entity mcEnt = (net.minecraft.server.v1_7_R1.Entity)o;
        org.bukkit.entity.Entity bukkitEntity = mcEnt.getBukkitEntity();
        

        if ((bukkitEntity != null) && ((bukkitEntity instanceof LivingEntity))) {
          list.add((LivingEntity)bukkitEntity);
        }
      }
    }
    
    return list;
  }
  
  @Deprecated
  public <T extends org.bukkit.entity.Entity> Collection<T> getEntitiesByClass(Class<T>... classes)
  {
    return getEntitiesByClasses(classes);
  }
  
  public <T extends org.bukkit.entity.Entity> Collection<T> getEntitiesByClass(Class<T> clazz)
  {
    Collection<T> list = new ArrayList();
    
    for (Object entity : world.entityList) {
      if ((entity instanceof net.minecraft.server.v1_7_R1.Entity)) {
        org.bukkit.entity.Entity bukkitEntity = ((net.minecraft.server.v1_7_R1.Entity)entity).getBukkitEntity();
        
        if (bukkitEntity != null)
        {


          Class<?> bukkitClass = bukkitEntity.getClass();
          
          if (clazz.isAssignableFrom(bukkitClass)) {
            list.add(bukkitEntity);
          }
        }
      }
    }
    return list;
  }
  
  public Collection<org.bukkit.entity.Entity> getEntitiesByClasses(Class<?>... classes) {
    Collection<org.bukkit.entity.Entity> list = new ArrayList();
    
    for (Object entity : world.entityList) {
      if ((entity instanceof net.minecraft.server.v1_7_R1.Entity)) {
        org.bukkit.entity.Entity bukkitEntity = ((net.minecraft.server.v1_7_R1.Entity)entity).getBukkitEntity();
        
        if (bukkitEntity != null)
        {


          Class<?> bukkitClass = bukkitEntity.getClass();
          
          for (Class<?> clazz : classes) {
            if (clazz.isAssignableFrom(bukkitClass)) {
              list.add(bukkitEntity);
              break;
            }
          }
        }
      }
    }
    return list;
  }
  
  public List<Player> getPlayers() {
    List<Player> list = new ArrayList();
    
    for (Object o : world.entityList) {
      if ((o instanceof net.minecraft.server.v1_7_R1.Entity)) {
        net.minecraft.server.v1_7_R1.Entity mcEnt = (net.minecraft.server.v1_7_R1.Entity)o;
        org.bukkit.entity.Entity bukkitEntity = mcEnt.getBukkitEntity();
        
        if ((bukkitEntity != null) && ((bukkitEntity instanceof Player))) {
          list.add((Player)bukkitEntity);
        }
      }
    }
    
    return list;
  }
  
  public void save() {
    server.checkSaveState();
    try {
      boolean oldSave = world.savingDisabled;
      
      world.savingDisabled = false;
      world.save(true, null);
      
      world.savingDisabled = oldSave;
    } catch (ExceptionWorldConflict ex) {
      ex.printStackTrace();
    }
  }
  
  public boolean isAutoSave() {
    return !world.savingDisabled;
  }
  
  public void setAutoSave(boolean value) {
    world.savingDisabled = (!value);
  }
  
  public void setDifficulty(Difficulty difficulty) {
    getHandle()difficulty = net.minecraft.server.v1_7_R1.EnumDifficulty.a(difficulty.getValue());
  }
  
  public Difficulty getDifficulty() {
    return Difficulty.getByValue(getHandle()difficulty.ordinal());
  }
  
  public BlockMetadataStore getBlockMetadata() {
    return blockMetadata;
  }
  
  public boolean hasStorm() {
    return world.worldData.hasStorm();
  }
  
  public void setStorm(boolean hasStorm) {
    CraftServer server = world.getServer();
    
    WeatherChangeEvent weather = new WeatherChangeEvent(this, hasStorm);
    server.getPluginManager().callEvent(weather);
    if (!weather.isCancelled()) {
      world.worldData.setStorm(hasStorm);
      

      if (hasStorm) {
        setWeatherDuration(rand.nextInt(12000) + 12000);
      } else {
        setWeatherDuration(rand.nextInt(168000) + 12000);
      }
    }
  }
  
  public int getWeatherDuration() {
    return world.worldData.getWeatherDuration();
  }
  
  public void setWeatherDuration(int duration) {
    world.worldData.setWeatherDuration(duration);
  }
  
  public boolean isThundering() {
    return (hasStorm()) && (world.worldData.isThundering());
  }
  
  public void setThundering(boolean thundering) {
    if ((thundering) && (!hasStorm())) setStorm(true);
    CraftServer server = world.getServer();
    
    ThunderChangeEvent thunder = new ThunderChangeEvent(this, thundering);
    server.getPluginManager().callEvent(thunder);
    if (!thunder.isCancelled()) {
      world.worldData.setThundering(thundering);
      

      if (thundering) {
        setThunderDuration(rand.nextInt(12000) + 3600);
      } else {
        setThunderDuration(rand.nextInt(168000) + 12000);
      }
    }
  }
  
  public int getThunderDuration() {
    return world.worldData.getThunderDuration();
  }
  
  public void setThunderDuration(int duration) {
    world.worldData.setThunderDuration(duration);
  }
  
  public long getSeed() {
    return world.worldData.getSeed();
  }
  
  public boolean getPVP() {
    return world.pvpMode;
  }
  
  public void setPVP(boolean pvp) {
    world.pvpMode = pvp;
  }
  
  public void playEffect(Player player, Effect effect, int data) {
    playEffect(player.getLocation(), effect, data, 0);
  }
  
  public void playEffect(Location location, Effect effect, int data) {
    playEffect(location, effect, data, 64);
  }
  
  public <T> void playEffect(Location loc, Effect effect, T data) {
    playEffect(loc, effect, data, 64);
  }
  
  public <T> void playEffect(Location loc, Effect effect, T data, int radius) {
    if (data != null) {
      Validate.isTrue(data.getClass().equals(effect.getData()), "Wrong kind of data for this effect!");
    } else {
      Validate.isTrue(effect.getData() == null, "Wrong kind of data for this effect!");
    }
    
    int datavalue = data == null ? 0 : CraftEffect.getDataValue(effect, data);
    playEffect(loc, effect, datavalue, radius);
  }
  
  public void playEffect(Location location, Effect effect, int data, int radius) {
    Validate.notNull(location, "Location cannot be null");
    Validate.notNull(effect, "Effect cannot be null");
    Validate.notNull(location.getWorld(), "World cannot be null");
    int packetData = effect.getId();
    net.minecraft.server.v1_7_R1.PacketPlayOutWorldEvent packet = new net.minecraft.server.v1_7_R1.PacketPlayOutWorldEvent(packetData, location.getBlockX(), location.getBlockY(), location.getBlockZ(), data, false);
    
    radius *= radius;
    
    for (Player player : getPlayers()) {
      if ((getHandle()playerConnection != null) && 
        (location.getWorld().equals(player.getWorld())))
      {
        int distance = (int)player.getLocation().distanceSquared(location);
        if (distance <= radius)
          getHandle()playerConnection.sendPacket(packet);
      }
    }
  }
  
  public <T extends org.bukkit.entity.Entity> T spawn(Location location, Class<T> clazz) throws IllegalArgumentException {
    return spawn(location, clazz, CreatureSpawnEvent.SpawnReason.CUSTOM);
  }
  
  public FallingBlock spawnFallingBlock(Location location, Material material, byte data) throws IllegalArgumentException {
    Validate.notNull(location, "Location cannot be null");
    Validate.notNull(material, "Material cannot be null");
    Validate.isTrue(material.isBlock(), "Material must be a block");
    
    double x = location.getBlockX() + 0.5D;
    double y = location.getBlockY() + 0.5D;
    double z = location.getBlockZ() + 0.5D;
    
    EntityFallingBlock entity = new EntityFallingBlock(world, x, y, z, net.minecraft.server.v1_7_R1.Block.e(material.getId()), data);
    b = 1;
    
    world.addEntity(entity, CreatureSpawnEvent.SpawnReason.CUSTOM);
    return (FallingBlock)entity.getBukkitEntity();
  }
  
  public FallingBlock spawnFallingBlock(Location location, int blockId, byte blockData) throws IllegalArgumentException {
    return spawnFallingBlock(location, Material.getMaterial(blockId), blockData);
  }
  
  public <T extends org.bukkit.entity.Entity> T spawn(Location location, Class<T> clazz, CreatureSpawnEvent.SpawnReason reason) throws IllegalArgumentException
  {
    if ((location == null) || (clazz == null)) {
      throw new IllegalArgumentException("Location or entity class cannot be null");
    }
    
    net.minecraft.server.v1_7_R1.Entity entity = null;
    
    double x = location.getX();
    double y = location.getY();
    double z = location.getZ();
    float pitch = location.getPitch();
    float yaw = location.getYaw();
    

    if (org.bukkit.entity.Boat.class.isAssignableFrom(clazz)) {
      entity = new net.minecraft.server.v1_7_R1.EntityBoat(world, x, y, z);
    } else if (FallingBlock.class.isAssignableFrom(clazz)) {
      x = location.getBlockX();
      y = location.getBlockY();
      z = location.getBlockZ();
      int type = world.getTypeId((int)x, (int)y, (int)z);
      int data = world.getData((int)x, (int)y, (int)z);
      
      entity = new EntityFallingBlock(world, x + 0.5D, y + 0.5D, z + 0.5D, net.minecraft.server.v1_7_R1.Block.e(type), data);
    } else if (org.bukkit.entity.Projectile.class.isAssignableFrom(clazz)) {
      if (org.bukkit.entity.Snowball.class.isAssignableFrom(clazz)) {
        entity = new net.minecraft.server.v1_7_R1.EntitySnowball(world, x, y, z);
      } else if (org.bukkit.entity.Egg.class.isAssignableFrom(clazz)) {
        entity = new net.minecraft.server.v1_7_R1.EntityEgg(world, x, y, z);
      } else if (Arrow.class.isAssignableFrom(clazz)) {
        entity = new EntityArrow(world);
        entity.setPositionRotation(x, y, z, 0.0F, 0.0F);
      } else if (org.bukkit.entity.ThrownExpBottle.class.isAssignableFrom(clazz)) {
        entity = new net.minecraft.server.v1_7_R1.EntityThrownExpBottle(world);
        entity.setPositionRotation(x, y, z, 0.0F, 0.0F);
      } else if (org.bukkit.entity.EnderPearl.class.isAssignableFrom(clazz)) {
        entity = new net.minecraft.server.v1_7_R1.EntityEnderPearl(world);
        entity.setPositionRotation(x, y, z, 0.0F, 0.0F);
      } else if (org.bukkit.entity.ThrownPotion.class.isAssignableFrom(clazz)) {
        entity = new net.minecraft.server.v1_7_R1.EntityPotion(world, x, y, z, org.bukkit.craftbukkit.v1_7_R1.inventory.CraftItemStack.asNMSCopy(new ItemStack(Material.POTION, 1)));
      } else if (org.bukkit.entity.Fireball.class.isAssignableFrom(clazz)) {
        if (org.bukkit.entity.SmallFireball.class.isAssignableFrom(clazz)) {
          entity = new net.minecraft.server.v1_7_R1.EntitySmallFireball(world);
        } else if (org.bukkit.entity.WitherSkull.class.isAssignableFrom(clazz)) {
          entity = new net.minecraft.server.v1_7_R1.EntityWitherSkull(world);
        } else {
          entity = new net.minecraft.server.v1_7_R1.EntityLargeFireball(world);
        }
        entity.setPositionRotation(x, y, z, yaw, pitch);
        Vector direction = location.getDirection().multiply(10);
        ((net.minecraft.server.v1_7_R1.EntityFireball)entity).setDirection(direction.getX(), direction.getY(), direction.getZ());
      }
    } else if (org.bukkit.entity.Minecart.class.isAssignableFrom(clazz)) {
      if (org.bukkit.entity.minecart.PoweredMinecart.class.isAssignableFrom(clazz)) {
        entity = new net.minecraft.server.v1_7_R1.EntityMinecartFurnace(world, x, y, z);
      } else if (org.bukkit.entity.minecart.StorageMinecart.class.isAssignableFrom(clazz)) {
        entity = new net.minecraft.server.v1_7_R1.EntityMinecartChest(world, x, y, z);
      } else if (org.bukkit.entity.minecart.ExplosiveMinecart.class.isAssignableFrom(clazz)) {
        entity = new net.minecraft.server.v1_7_R1.EntityMinecartTNT(world, x, y, z);
      } else if (org.bukkit.entity.minecart.HopperMinecart.class.isAssignableFrom(clazz)) {
        entity = new net.minecraft.server.v1_7_R1.EntityMinecartHopper(world, x, y, z);
      } else if (org.bukkit.entity.minecart.SpawnerMinecart.class.isAssignableFrom(clazz)) {
        entity = new net.minecraft.server.v1_7_R1.EntityMinecartMobSpawner(world, x, y, z);
      } else {
        entity = new net.minecraft.server.v1_7_R1.EntityMinecartRideable(world, x, y, z);
      }
    } else if (org.bukkit.entity.EnderSignal.class.isAssignableFrom(clazz)) {
      entity = new net.minecraft.server.v1_7_R1.EntityEnderSignal(world, x, y, z);
    } else if (org.bukkit.entity.EnderCrystal.class.isAssignableFrom(clazz)) {
      entity = new net.minecraft.server.v1_7_R1.EntityEnderCrystal(world);
      entity.setPositionRotation(x, y, z, 0.0F, 0.0F);
    } else if (LivingEntity.class.isAssignableFrom(clazz)) {
      if (org.bukkit.entity.Chicken.class.isAssignableFrom(clazz)) {
        entity = new net.minecraft.server.v1_7_R1.EntityChicken(world);
      } else if (org.bukkit.entity.Cow.class.isAssignableFrom(clazz)) {
        if (org.bukkit.entity.MushroomCow.class.isAssignableFrom(clazz)) {
          entity = new net.minecraft.server.v1_7_R1.EntityMushroomCow(world);
        } else {
          entity = new net.minecraft.server.v1_7_R1.EntityCow(world);
        }
      } else if (org.bukkit.entity.Golem.class.isAssignableFrom(clazz)) {
        if (org.bukkit.entity.Snowman.class.isAssignableFrom(clazz)) {
          entity = new net.minecraft.server.v1_7_R1.EntitySnowman(world);
        } else if (org.bukkit.entity.IronGolem.class.isAssignableFrom(clazz)) {
          entity = new net.minecraft.server.v1_7_R1.EntityIronGolem(world);
        }
      } else if (org.bukkit.entity.Creeper.class.isAssignableFrom(clazz)) {
        entity = new net.minecraft.server.v1_7_R1.EntityCreeper(world);
      } else if (org.bukkit.entity.Ghast.class.isAssignableFrom(clazz)) {
        entity = new net.minecraft.server.v1_7_R1.EntityGhast(world);
      } else if (org.bukkit.entity.Pig.class.isAssignableFrom(clazz)) {
        entity = new net.minecraft.server.v1_7_R1.EntityPig(world);
      } else if (!Player.class.isAssignableFrom(clazz))
      {
        if (org.bukkit.entity.Sheep.class.isAssignableFrom(clazz)) {
          entity = new net.minecraft.server.v1_7_R1.EntitySheep(world);
        } else if (org.bukkit.entity.Horse.class.isAssignableFrom(clazz)) {
          entity = new net.minecraft.server.v1_7_R1.EntityHorse(world);
        } else if (org.bukkit.entity.Skeleton.class.isAssignableFrom(clazz)) {
          entity = new net.minecraft.server.v1_7_R1.EntitySkeleton(world);
        } else if (org.bukkit.entity.Slime.class.isAssignableFrom(clazz)) {
          if (org.bukkit.entity.MagmaCube.class.isAssignableFrom(clazz)) {
            entity = new net.minecraft.server.v1_7_R1.EntityMagmaCube(world);
          } else {
            entity = new net.minecraft.server.v1_7_R1.EntitySlime(world);
          }
        } else if (org.bukkit.entity.Spider.class.isAssignableFrom(clazz)) {
          if (org.bukkit.entity.CaveSpider.class.isAssignableFrom(clazz)) {
            entity = new net.minecraft.server.v1_7_R1.EntityCaveSpider(world);
          } else {
            entity = new net.minecraft.server.v1_7_R1.EntitySpider(world);
          }
        } else if (org.bukkit.entity.Squid.class.isAssignableFrom(clazz)) {
          entity = new net.minecraft.server.v1_7_R1.EntitySquid(world);
        } else if (org.bukkit.entity.Tameable.class.isAssignableFrom(clazz)) {
          if (org.bukkit.entity.Wolf.class.isAssignableFrom(clazz)) {
            entity = new net.minecraft.server.v1_7_R1.EntityWolf(world);
          } else if (org.bukkit.entity.Ocelot.class.isAssignableFrom(clazz)) {
            entity = new net.minecraft.server.v1_7_R1.EntityOcelot(world);
          }
        } else if (org.bukkit.entity.PigZombie.class.isAssignableFrom(clazz)) {
          entity = new net.minecraft.server.v1_7_R1.EntityPigZombie(world);
        } else if (org.bukkit.entity.Zombie.class.isAssignableFrom(clazz)) {
          entity = new net.minecraft.server.v1_7_R1.EntityZombie(world);
        } else if (org.bukkit.entity.Giant.class.isAssignableFrom(clazz)) {
          entity = new net.minecraft.server.v1_7_R1.EntityGiantZombie(world);
        } else if (org.bukkit.entity.Silverfish.class.isAssignableFrom(clazz)) {
          entity = new net.minecraft.server.v1_7_R1.EntitySilverfish(world);
        } else if (org.bukkit.entity.Enderman.class.isAssignableFrom(clazz)) {
          entity = new net.minecraft.server.v1_7_R1.EntityEnderman(world);
        } else if (org.bukkit.entity.Blaze.class.isAssignableFrom(clazz)) {
          entity = new net.minecraft.server.v1_7_R1.EntityBlaze(world);
        } else if (org.bukkit.entity.Villager.class.isAssignableFrom(clazz)) {
          entity = new net.minecraft.server.v1_7_R1.EntityVillager(world);
        } else if (org.bukkit.entity.Witch.class.isAssignableFrom(clazz)) {
          entity = new net.minecraft.server.v1_7_R1.EntityWitch(world);
        } else if (org.bukkit.entity.Wither.class.isAssignableFrom(clazz)) {
          entity = new net.minecraft.server.v1_7_R1.EntityWither(world);
        } else if (org.bukkit.entity.ComplexLivingEntity.class.isAssignableFrom(clazz)) {
          if (org.bukkit.entity.EnderDragon.class.isAssignableFrom(clazz)) {
            entity = new net.minecraft.server.v1_7_R1.EntityEnderDragon(world);
          }
        } else if ((org.bukkit.entity.Ambient.class.isAssignableFrom(clazz)) && 
          (org.bukkit.entity.Bat.class.isAssignableFrom(clazz))) {
          entity = new net.minecraft.server.v1_7_R1.EntityBat(world);
        }
      }
      
      if (entity != null) {
        entity.setLocation(x, y, z, yaw, pitch);
      }
    } else if (org.bukkit.entity.Hanging.class.isAssignableFrom(clazz)) {
      org.bukkit.block.Block block = getBlockAt(location);
      BlockFace face = BlockFace.SELF;
      if (block.getRelative(BlockFace.EAST).getTypeId() == 0) {
        face = BlockFace.EAST;
      } else if (block.getRelative(BlockFace.NORTH).getTypeId() == 0) {
        face = BlockFace.NORTH;
      } else if (block.getRelative(BlockFace.WEST).getTypeId() == 0) {
        face = BlockFace.WEST;
      } else if (block.getRelative(BlockFace.SOUTH).getTypeId() == 0) {
        face = BlockFace.SOUTH;
      }
      int dir;
      switch (1.$SwitchMap$org$bukkit$block$BlockFace[face.ordinal()]) {
      case 1: 
      default: 
        dir = 0;
        break;
      case 2: 
        dir = 1;
        break;
      case 3: 
        dir = 2;
        break;
      case 4: 
        dir = 3;
      }
      
      
      if (org.bukkit.entity.Painting.class.isAssignableFrom(clazz)) {
        entity = new net.minecraft.server.v1_7_R1.EntityPainting(world, (int)x, (int)y, (int)z, dir);
      } else if (org.bukkit.entity.ItemFrame.class.isAssignableFrom(clazz)) {
        entity = new net.minecraft.server.v1_7_R1.EntityItemFrame(world, (int)x, (int)y, (int)z, dir);
      } else if (org.bukkit.entity.LeashHitch.class.isAssignableFrom(clazz)) {
        entity = new net.minecraft.server.v1_7_R1.EntityLeash(world, (int)x, (int)y, (int)z);
        o = true;
      }
      
      if ((entity != null) && (!((net.minecraft.server.v1_7_R1.EntityHanging)entity).survives())) {
        throw new IllegalArgumentException("Cannot spawn hanging entity for " + clazz.getName() + " at " + location);
      }
    } else if (org.bukkit.entity.TNTPrimed.class.isAssignableFrom(clazz)) {
      entity = new net.minecraft.server.v1_7_R1.EntityTNTPrimed(world, x, y, z, null);
    } else if (org.bukkit.entity.ExperienceOrb.class.isAssignableFrom(clazz)) {
      entity = new net.minecraft.server.v1_7_R1.EntityExperienceOrb(world, x, y, z, 0);
    } else if (org.bukkit.entity.Weather.class.isAssignableFrom(clazz))
    {
      if (LightningStrike.class.isAssignableFrom(clazz)) {
        entity = new EntityLightning(world, x, y, z);
      }
    }
    else if (org.bukkit.entity.Firework.class.isAssignableFrom(clazz)) {
      entity = new net.minecraft.server.v1_7_R1.EntityFireworks(world, x, y, z, null);
    }
    
    if (entity != null) {
      world.addEntity(entity, reason);
      return entity.getBukkitEntity();
    }
    
    throw new IllegalArgumentException("Cannot spawn an entity for " + clazz.getName());
  }
  
  public org.bukkit.ChunkSnapshot getEmptyChunkSnapshot(int x, int z, boolean includeBiome, boolean includeBiomeTempRain) {
    return CraftChunk.getEmptyChunkSnapshot(x, z, this, includeBiome, includeBiomeTempRain);
  }
  
  public void setSpawnFlags(boolean allowMonsters, boolean allowAnimals) {
    world.setSpawnFlags(allowMonsters, allowAnimals);
  }
  
  public boolean getAllowAnimals() {
    return world.allowAnimals;
  }
  
  public boolean getAllowMonsters() {
    return world.allowMonsters;
  }
  
  public int getMaxHeight() {
    return world.getHeight();
  }
  
  public int getSeaLevel() {
    return 64;
  }
  
  public boolean getKeepSpawnInMemory() {
    return world.keepSpawnInMemory;
  }
  
  public void setKeepSpawnInMemory(boolean keepLoaded) {
    world.keepSpawnInMemory = keepLoaded;
    
    ChunkCoordinates chunkcoordinates = world.getSpawn();
    int chunkCoordX = x >> 4;
    int chunkCoordZ = z >> 4;
    
    for (int x = -12; x <= 12; x++) {
      for (int z = -12; z <= 12; z++) {
        if (keepLoaded) {
          loadChunk(chunkCoordX + x, chunkCoordZ + z);
        }
        else if (isChunkLoaded(chunkCoordX + x, chunkCoordZ + z)) {
          if ((getHandle().getChunkAt(chunkCoordX + x, chunkCoordZ + z) instanceof EmptyChunk)) {
            unloadChunk(chunkCoordX + x, chunkCoordZ + z, false);
          } else {
            unloadChunk(chunkCoordX + x, chunkCoordZ + z);
          }
        }
      }
    }
  }
  

  public int hashCode()
  {
    return getUID().hashCode();
  }
  
  public boolean equals(Object obj)
  {
    if (obj == null) {
      return false;
    }
    if (getClass() != obj.getClass()) {
      return false;
    }
    
    CraftWorld other = (CraftWorld)obj;
    
    return getUID() == other.getUID();
  }
  
  public java.io.File getWorldFolder() {
    return ((net.minecraft.server.v1_7_R1.WorldNBTStorage)world.getDataManager()).getDirectory();
  }
  
  public void sendPluginMessage(Plugin source, String channel, byte[] message) {
    org.bukkit.plugin.messaging.StandardMessenger.validatePluginMessage(server.getMessenger(), source, channel, message);
    
    for (Player player : getPlayers()) {
      player.sendPluginMessage(source, channel, message);
    }
  }
  
  public Set<String> getListeningPluginChannels() {
    Set<String> result = new java.util.HashSet();
    
    for (Player player : getPlayers()) {
      result.addAll(player.getListeningPluginChannels());
    }
    
    return result;
  }
  
  public org.bukkit.WorldType getWorldType() {
    return org.bukkit.WorldType.getByName(world.getWorldData().getType().name());
  }
  
  public boolean canGenerateStructures() {
    return world.getWorldData().shouldGenerateMapFeatures();
  }
  
  public long getTicksPerAnimalSpawns() {
    return world.ticksPerAnimalSpawns;
  }
  
  public void setTicksPerAnimalSpawns(int ticksPerAnimalSpawns) {
    world.ticksPerAnimalSpawns = ticksPerAnimalSpawns;
  }
  
  public long getTicksPerMonsterSpawns() {
    return world.ticksPerMonsterSpawns;
  }
  
  public void setTicksPerMonsterSpawns(int ticksPerMonsterSpawns) {
    world.ticksPerMonsterSpawns = ticksPerMonsterSpawns;
  }
  
  public void setMetadata(String metadataKey, MetadataValue newMetadataValue) {
    server.getWorldMetadata().setMetadata(this, metadataKey, newMetadataValue);
  }
  
  public List<MetadataValue> getMetadata(String metadataKey) {
    return server.getWorldMetadata().getMetadata(this, metadataKey);
  }
  
  public boolean hasMetadata(String metadataKey) {
    return server.getWorldMetadata().hasMetadata(this, metadataKey);
  }
  
  public void removeMetadata(String metadataKey, Plugin owningPlugin) {
    server.getWorldMetadata().removeMetadata(this, metadataKey, owningPlugin);
  }
  
  public int getMonsterSpawnLimit() {
    if (monsterSpawn < 0) {
      return server.getMonsterSpawnLimit();
    }
    
    return monsterSpawn;
  }
  
  public void setMonsterSpawnLimit(int limit) {
    monsterSpawn = limit;
  }
  
  public int getAnimalSpawnLimit() {
    if (animalSpawn < 0) {
      return server.getAnimalSpawnLimit();
    }
    
    return animalSpawn;
  }
  
  public void setAnimalSpawnLimit(int limit) {
    animalSpawn = limit;
  }
  
  public int getWaterAnimalSpawnLimit() {
    if (waterAnimalSpawn < 0) {
      return server.getWaterAnimalSpawnLimit();
    }
    
    return waterAnimalSpawn;
  }
  
  public void setWaterAnimalSpawnLimit(int limit) {
    waterAnimalSpawn = limit;
  }
  
  public int getAmbientSpawnLimit() {
    if (ambientSpawn < 0) {
      return server.getAmbientSpawnLimit();
    }
    
    return ambientSpawn;
  }
  
  public void setAmbientSpawnLimit(int limit) {
    ambientSpawn = limit;
  }
  
  public void playSound(Location loc, org.bukkit.Sound sound, float volume, float pitch)
  {
    if ((loc == null) || (sound == null)) { return;
    }
    double x = loc.getX();
    double y = loc.getY();
    double z = loc.getZ();
    
    getHandle().makeSound(x, y, z, CraftSound.getSound(sound), volume, pitch);
  }
  
  public String getGameRuleValue(String rule) {
    return getHandle().getGameRules().get(rule);
  }
  
  public boolean setGameRuleValue(String rule, String value)
  {
    if ((rule == null) || (value == null)) { return false;
    }
    if (!isGameRule(rule)) { return false;
    }
    getHandle().getGameRules().set(rule, value);
    return true;
  }
  
  public String[] getGameRules() {
    return getHandle().getGameRules().b();
  }
  
  public boolean isGameRule(String rule) {
    return getHandle().getGameRules().e(rule);
  }
  
  public void processChunkGC() {
    chunkGCTickCount += 1;
    
    if ((chunkLoadCount >= server.chunkGCLoadThresh) && (server.chunkGCLoadThresh > 0)) {
      chunkLoadCount = 0;
    } else if ((chunkGCTickCount >= server.chunkGCPeriod) && (server.chunkGCPeriod > 0)) {
      chunkGCTickCount = 0;
    } else {
      return;
    }
    
    ChunkProviderServer cps = world.chunkProviderServer;
    for (net.minecraft.server.v1_7_R1.Chunk chunk : chunks.values())
    {
      if ((!isChunkInUse(locX, locZ)) && 
      



        (!unloadQueue.contains(locX, locZ)))
      {



        cps.queueUnload(locX, locZ);
      }
    }
  }
}
