package org.bukkit.craftbukkit.v1_7_R1.block;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import net.minecraft.server.v1_7_R1.BiomeBase;
import net.minecraft.server.v1_7_R1.BlockCocoa;
import net.minecraft.server.v1_7_R1.BlockRedstoneWire;
import net.minecraft.server.v1_7_R1.Blocks;
import net.minecraft.server.v1_7_R1.EnumSkyBlock;
import net.minecraft.server.v1_7_R1.Item;
import net.minecraft.server.v1_7_R1.NBTTagCompound;
import net.minecraft.server.v1_7_R1.TileEntitySkull;
import org.bukkit.Location;
import org.bukkit.block.Biome;
import org.bukkit.block.BlockFace;
import org.bukkit.block.BlockState;
import org.bukkit.block.PistonMoveReaction;
import org.bukkit.craftbukkit.v1_7_R1.CraftChunk;
import org.bukkit.craftbukkit.v1_7_R1.CraftWorld;
import org.bukkit.craftbukkit.v1_7_R1.inventory.CraftItemStack;
import org.bukkit.craftbukkit.v1_7_R1.metadata.BlockMetadataStore;
import org.bukkit.craftbukkit.v1_7_R1.util.CraftMagicNumbers;
import org.bukkit.metadata.MetadataValue;
import org.bukkit.plugin.Plugin;
import org.bukkit.util.BlockVector;





public class CraftBlock
  implements org.bukkit.block.Block
{
  private final CraftChunk chunk;
  private final int x;
  private final int y;
  private final int z;
  
  public CraftBlock(CraftChunk chunk, int x, int y, int z)
  {
    this.x = x;
    this.y = y;
    this.z = z;
    this.chunk = chunk;
  }
  
  private net.minecraft.server.v1_7_R1.Block getNMSBlock() {
    return CraftMagicNumbers.getBlock(this);
  }
  
  private static net.minecraft.server.v1_7_R1.Block getNMSBlock(int type) {
    return CraftMagicNumbers.getBlock(type);
  }
  
  public org.bukkit.World getWorld() {
    return chunk.getWorld();
  }
  
  public Location getLocation() {
    return new Location(getWorld(), x, y, z);
  }
  
  public Location getLocation(Location loc) {
    if (loc != null) {
      loc.setWorld(getWorld());
      loc.setX(x);
      loc.setY(y);
      loc.setZ(z);
      loc.setYaw(0.0F);
      loc.setPitch(0.0F);
    }
    
    return loc;
  }
  
  public BlockVector getVector() {
    return new BlockVector(x, y, z);
  }
  
  public int getX() {
    return x;
  }
  
  public int getY() {
    return y;
  }
  
  public int getZ() {
    return z;
  }
  
  public org.bukkit.Chunk getChunk() {
    return chunk;
  }
  
  public void setData(byte data) {
    chunk.getHandle().world.setData(x, y, z, data, 3);
  }
  
  public void setData(byte data, boolean applyPhysics) {
    if (applyPhysics) {
      chunk.getHandle().world.setData(x, y, z, data, 3);
    } else {
      chunk.getHandle().world.setData(x, y, z, data, 2);
    }
  }
  
  public byte getData() {
    return (byte)chunk.getHandle().getData(x & 0xF, y & 0xFF, z & 0xF);
  }
  
  public void setType(org.bukkit.Material type) {
    setTypeId(type.getId());
  }
  
  public boolean setTypeId(int type) {
    return setTypeId(type, true);
  }
  
  public boolean setTypeId(int type, boolean applyPhysics) {
    return setTypeIdAndData(type, getData(), applyPhysics);
  }
  
  public boolean setTypeIdAndData(int type, byte data, boolean applyPhysics) {
    if (applyPhysics) {
      return chunk.getHandle().world.setTypeAndData(x, y, z, getNMSBlock(type), data, 3);
    }
    boolean success = chunk.getHandle().world.setTypeAndData(x, y, z, getNMSBlock(type), data, 2);
    if (success) {
      chunk.getHandle().world.notify(x, y, z);
    }
    return success;
  }
  
  public org.bukkit.Material getType()
  {
    return org.bukkit.Material.getMaterial(getTypeId());
  }
  
  @Deprecated
  public int getTypeId()
  {
    return CraftMagicNumbers.getId(chunk.getHandle().getType(x & 0xF, y & 0xFF, z & 0xF));
  }
  
  public byte getLightLevel() {
    return (byte)chunk.getHandle().world.getLightLevel(x, y, z);
  }
  
  public byte getLightFromSky() {
    return (byte)chunk.getHandle().getBrightness(EnumSkyBlock.SKY, x & 0xF, y & 0xFF, z & 0xF);
  }
  
  public byte getLightFromBlocks() {
    return (byte)chunk.getHandle().getBrightness(EnumSkyBlock.BLOCK, x & 0xF, y & 0xFF, z & 0xF);
  }
  
  public org.bukkit.block.Block getFace(BlockFace face)
  {
    return getRelative(face, 1);
  }
  
  public org.bukkit.block.Block getFace(BlockFace face, int distance) {
    return getRelative(face, distance);
  }
  
  public org.bukkit.block.Block getRelative(int modX, int modY, int modZ) {
    return getWorld().getBlockAt(getX() + modX, getY() + modY, getZ() + modZ);
  }
  
  public org.bukkit.block.Block getRelative(BlockFace face) {
    return getRelative(face, 1);
  }
  
  public org.bukkit.block.Block getRelative(BlockFace face, int distance) {
    return getRelative(face.getModX() * distance, face.getModY() * distance, face.getModZ() * distance);
  }
  
  public BlockFace getFace(org.bukkit.block.Block block) {
    BlockFace[] values = BlockFace.values();
    
    for (BlockFace face : values) {
      if ((getX() + face.getModX() == block.getX()) && (getY() + face.getModY() == block.getY()) && (getZ() + face.getModZ() == block.getZ()))
      {


        return face;
      }
    }
    
    return null;
  }
  
  public String toString()
  {
    return "CraftBlock{chunk=" + chunk + ",x=" + x + ",y=" + y + ",z=" + z + ",type=" + getType() + ",data=" + getData() + '}';
  }
  





  public static BlockFace notchToBlockFace(int notch)
  {
    switch (notch) {
    case 0: 
      return BlockFace.DOWN;
    case 1: 
      return BlockFace.UP;
    case 2: 
      return BlockFace.NORTH;
    case 3: 
      return BlockFace.SOUTH;
    case 4: 
      return BlockFace.WEST;
    case 5: 
      return BlockFace.EAST;
    }
    return BlockFace.SELF;
  }
  
  public static int blockFaceToNotch(BlockFace face)
  {
    switch (1.$SwitchMap$org$bukkit$block$BlockFace[face.ordinal()]) {
    case 1: 
      return 0;
    case 2: 
      return 1;
    case 3: 
      return 2;
    case 4: 
      return 3;
    case 5: 
      return 4;
    case 6: 
      return 5;
    }
    return 7;
  }
  
  public BlockState getState()
  {
    org.bukkit.Material material = getType();
    
    switch (1.$SwitchMap$org$bukkit$Material[material.ordinal()]) {
    case 1: 
    case 2: 
    case 3: 
      return new CraftSign(this);
    case 4: 
    case 5: 
      return new CraftChest(this);
    case 6: 
    case 7: 
      return new CraftFurnace(this);
    case 8: 
      return new CraftDispenser(this);
    case 9: 
      return new CraftDropper(this);
    case 10: 
      return new CraftHopper(this);
    case 11: 
      return new CraftCreatureSpawner(this);
    case 12: 
      return new CraftNoteBlock(this);
    case 13: 
      return new CraftJukebox(this);
    case 14: 
      return new CraftBrewingStand(this);
    case 15: 
      return new CraftSkull(this);
    case 16: 
      return new CraftCommandBlock(this);
    case 17: 
      return new CraftBeacon(this);
    }
    return new CraftBlockState(this);
  }
  
  public Biome getBiome()
  {
    return getWorld().getBiome(x, z);
  }
  
  public void setBiome(Biome bio) {
    getWorld().setBiome(x, z, bio);
  }
  
  public static Biome biomeBaseToBiome(BiomeBase base) {
    if (base == null) {
      return null;
    }
    
    return BIOME_MAPPING[id];
  }
  
  public static BiomeBase biomeToBiomeBase(Biome bio) {
    if (bio == null) {
      return null;
    }
    return BIOMEBASE_MAPPING[bio.ordinal()];
  }
  
  public double getTemperature() {
    return getWorld().getTemperature(x, z);
  }
  
  public double getHumidity() {
    return getWorld().getHumidity(x, z);
  }
  
  public boolean isBlockPowered() {
    return chunk.getHandle().world.getBlockPower(x, y, z) > 0;
  }
  
  public boolean isBlockIndirectlyPowered() {
    return chunk.getHandle().world.isBlockIndirectlyPowered(x, y, z);
  }
  
  public boolean equals(Object o)
  {
    if (o == this) return true;
    if (!(o instanceof CraftBlock)) return false;
    CraftBlock other = (CraftBlock)o;
    
    return (x == x) && (y == y) && (z == z) && (getWorld().equals(other.getWorld()));
  }
  
  public int hashCode()
  {
    return y << 24 ^ x ^ z ^ getWorld().hashCode();
  }
  
  public boolean isBlockFacePowered(BlockFace face) {
    return chunk.getHandle().world.isBlockFacePowered(x, y, z, blockFaceToNotch(face));
  }
  
  public boolean isBlockFaceIndirectlyPowered(BlockFace face) {
    int power = chunk.getHandle().world.getBlockFacePower(x, y, z, blockFaceToNotch(face));
    
    org.bukkit.block.Block relative = getRelative(face);
    if (relative.getType() == org.bukkit.Material.REDSTONE_WIRE) {
      return Math.max(power, relative.getData()) > 0;
    }
    
    return power > 0;
  }
  
  public int getBlockPower(BlockFace face) {
    int power = 0;
    BlockRedstoneWire wire = Blocks.REDSTONE_WIRE;
    net.minecraft.server.v1_7_R1.World world = chunk.getHandle().world;
    if (((face == BlockFace.DOWN) || (face == BlockFace.SELF)) && (world.isBlockFacePowered(x, y - 1, z, 0))) power = wire.getPower(world, x, y - 1, z, power);
    if (((face == BlockFace.UP) || (face == BlockFace.SELF)) && (world.isBlockFacePowered(x, y + 1, z, 1))) power = wire.getPower(world, x, y + 1, z, power);
    if (((face == BlockFace.EAST) || (face == BlockFace.SELF)) && (world.isBlockFacePowered(x + 1, y, z, 2))) power = wire.getPower(world, x + 1, y, z, power);
    if (((face == BlockFace.WEST) || (face == BlockFace.SELF)) && (world.isBlockFacePowered(x - 1, y, z, 3))) power = wire.getPower(world, x - 1, y, z, power);
    if (((face == BlockFace.NORTH) || (face == BlockFace.SELF)) && (world.isBlockFacePowered(x, y, z - 1, 4))) power = wire.getPower(world, x, y, z - 1, power);
    if (((face == BlockFace.SOUTH) || (face == BlockFace.SELF)) && (world.isBlockFacePowered(x, y, z + 1, 5))) power = wire.getPower(world, x, y, z - 1, power);
    return face == BlockFace.SELF ? isBlockIndirectlyPowered() : isBlockFaceIndirectlyPowered(face) ? 15 : power > 0 ? power : 0;
  }
  
  public int getBlockPower() {
    return getBlockPower(BlockFace.SELF);
  }
  
  public boolean isEmpty() {
    return getType() == org.bukkit.Material.AIR;
  }
  
  public boolean isLiquid() {
    return (getType() == org.bukkit.Material.WATER) || (getType() == org.bukkit.Material.STATIONARY_WATER) || (getType() == org.bukkit.Material.LAVA) || (getType() == org.bukkit.Material.STATIONARY_LAVA);
  }
  
  public PistonMoveReaction getPistonMoveReaction() {
    return PistonMoveReaction.getById(getNMSBlock().getMaterial().getPushReaction());
  }
  
  private boolean itemCausesDrops(org.bukkit.inventory.ItemStack item) {
    net.minecraft.server.v1_7_R1.Block block = getNMSBlock();
    Item itemType = item != null ? Item.d(item.getTypeId()) : null;
    return (block != null) && ((block.getMaterial().isAlwaysDestroyable()) || ((itemType != null) && (itemType.canDestroySpecialBlock(block))));
  }
  
  public boolean breakNaturally()
  {
    net.minecraft.server.v1_7_R1.Block block = getNMSBlock();
    byte data = getData();
    boolean result = false;
    
    if ((block != null) && (block != Blocks.AIR)) {
      block.dropNaturally(chunk.getHandle().world, x, y, z, data, 1.0F, 0);
      result = true;
    }
    
    setTypeId(org.bukkit.Material.AIR.getId());
    return result;
  }
  
  public boolean breakNaturally(org.bukkit.inventory.ItemStack item) {
    if (itemCausesDrops(item)) {
      return breakNaturally();
    }
    return setTypeId(org.bukkit.Material.AIR.getId());
  }
  
  public Collection<org.bukkit.inventory.ItemStack> getDrops()
  {
    List<org.bukkit.inventory.ItemStack> drops = new ArrayList();
    
    net.minecraft.server.v1_7_R1.Block block = getNMSBlock();
    if (block != Blocks.AIR) {
      byte data = getData();
      
      int count = block.getDropCount(0, chunk.getHandle().world.random);
      for (int i = 0; i < count; i++) {
        Item item = block.getDropType(data, chunk.getHandle().world.random, 0);
        if (item != null)
        {
          if (Blocks.SKULL == block) {
            net.minecraft.server.v1_7_R1.ItemStack nmsStack = new net.minecraft.server.v1_7_R1.ItemStack(item, 1, block.getDropData(chunk.getHandle().world, x, y, z));
            TileEntitySkull tileentityskull = (TileEntitySkull)chunk.getHandle().world.getTileEntity(x, y, z);
            
            if ((tileentityskull.getSkullType() == 3) && (tileentityskull.getExtraType() != null) && (tileentityskull.getExtraType().length() > 0)) {
              nmsStack.setTag(new NBTTagCompound());
              nmsStack.getTag().setString("SkullOwner", tileentityskull.getExtraType());
            }
            
            drops.add(CraftItemStack.asBukkitCopy(nmsStack));
          }
          else if (Blocks.COCOA == block) {
            int dropAmount = BlockCocoa.c(data) >= 2 ? 3 : 1;
            for (int j = 0; j < dropAmount; j++) {
              drops.add(new org.bukkit.inventory.ItemStack(org.bukkit.Material.INK_SACK, 1, (short)3));
            }
          } else {
            drops.add(new org.bukkit.inventory.ItemStack(CraftMagicNumbers.getMaterial(item), 1, (short)block.getDropData(data)));
          }
        }
      }
    }
    return drops;
  }
  
  public Collection<org.bukkit.inventory.ItemStack> getDrops(org.bukkit.inventory.ItemStack item) {
    if (itemCausesDrops(item)) {
      return getDrops();
    }
    return Collections.emptyList();
  }
  



  private static final Biome[] BIOME_MAPPING = new Biome[BiomeBase.n().length];
  private static final BiomeBase[] BIOMEBASE_MAPPING = new BiomeBase[Biome.values().length];
  static { BIOME_MAPPING[OCEANid] = Biome.OCEAN;
    BIOME_MAPPING[PLAINSid] = Biome.PLAINS;
    BIOME_MAPPING[DESERTid] = Biome.DESERT;
    BIOME_MAPPING[EXTREME_HILLSid] = Biome.EXTREME_HILLS;
    BIOME_MAPPING[FORESTid] = Biome.FOREST;
    BIOME_MAPPING[TAIGAid] = Biome.TAIGA;
    BIOME_MAPPING[SWAMPLANDid] = Biome.SWAMPLAND;
    BIOME_MAPPING[RIVERid] = Biome.RIVER;
    BIOME_MAPPING[HELLid] = Biome.HELL;
    BIOME_MAPPING[SKYid] = Biome.SKY;
    BIOME_MAPPING[FROZEN_OCEANid] = Biome.FROZEN_OCEAN;
    BIOME_MAPPING[FROZEN_RIVERid] = Biome.FROZEN_RIVER;
    BIOME_MAPPING[ICE_PLAINSid] = Biome.ICE_PLAINS;
    BIOME_MAPPING[ICE_MOUNTAINSid] = Biome.ICE_MOUNTAINS;
    BIOME_MAPPING[MUSHROOM_ISLANDid] = Biome.MUSHROOM_ISLAND;
    BIOME_MAPPING[MUSHROOM_SHOREid] = Biome.MUSHROOM_SHORE;
    BIOME_MAPPING[BEACHid] = Biome.BEACH;
    BIOME_MAPPING[DESERT_HILLSid] = Biome.DESERT_HILLS;
    BIOME_MAPPING[FOREST_HILLSid] = Biome.FOREST_HILLS;
    BIOME_MAPPING[TAIGA_HILLSid] = Biome.TAIGA_HILLS;
    BIOME_MAPPING[SMALL_MOUNTAINSid] = Biome.SMALL_MOUNTAINS;
    BIOME_MAPPING[JUNGLEid] = Biome.JUNGLE;
    BIOME_MAPPING[JUNGLE_HILLSid] = Biome.JUNGLE_HILLS;
    BIOME_MAPPING[JUNGLE_EDGEid] = Biome.JUNGLE_EDGE;
    BIOME_MAPPING[DEEP_OCEANid] = Biome.DEEP_OCEAN;
    BIOME_MAPPING[STONE_BEACHid] = Biome.STONE_BEACH;
    BIOME_MAPPING[COLD_BEACHid] = Biome.COLD_BEACH;
    BIOME_MAPPING[BIRCH_FORESTid] = Biome.BIRCH_FOREST;
    BIOME_MAPPING[BIRCH_FOREST_HILLSid] = Biome.BIRCH_FOREST_HILLS;
    BIOME_MAPPING[ROOFED_FORESTid] = Biome.ROOFED_FOREST;
    BIOME_MAPPING[COLD_TAIGAid] = Biome.COLD_TAIGA;
    BIOME_MAPPING[COLD_TAIGA_HILLSid] = Biome.COLD_TAIGA_HILLS;
    BIOME_MAPPING[MEGA_TAIGAid] = Biome.MEGA_TAIGA;
    BIOME_MAPPING[MEGA_TAIGA_HILLSid] = Biome.MEGA_TAIGA_HILLS;
    BIOME_MAPPING[EXTREME_HILLS_PLUSid] = Biome.EXTREME_HILLS_PLUS;
    BIOME_MAPPING[SAVANNAid] = Biome.SAVANNA;
    BIOME_MAPPING[SAVANNA_PLATEAUid] = Biome.SAVANNA_PLATEAU;
    BIOME_MAPPING[MESAid] = Biome.MESA;
    BIOME_MAPPING[MESA_PLATEAU_Fid] = Biome.MESA_PLATEAU_FOREST;
    BIOME_MAPPING[MESA_PLATEAUid] = Biome.MESA_PLATEAU;
    

    BIOME_MAPPING[(PLAINSid + 128)] = Biome.SUNFLOWER_PLAINS;
    BIOME_MAPPING[(DESERTid + 128)] = Biome.DESERT_MOUNTAINS;
    BIOME_MAPPING[(FORESTid + 128)] = Biome.FLOWER_FOREST;
    BIOME_MAPPING[(TAIGAid + 128)] = Biome.TAIGA_MOUNTAINS;
    BIOME_MAPPING[(SWAMPLANDid + 128)] = Biome.SWAMPLAND_MOUNTAINS;
    BIOME_MAPPING[(ICE_PLAINSid + 128)] = Biome.ICE_PLAINS_SPIKES;
    BIOME_MAPPING[(JUNGLEid + 128)] = Biome.JUNGLE_MOUNTAINS;
    BIOME_MAPPING[(JUNGLE_EDGEid + 128)] = Biome.JUNGLE_EDGE_MOUNTAINS;
    BIOME_MAPPING[(COLD_TAIGAid + 128)] = Biome.COLD_TAIGA_MOUNTAINS;
    BIOME_MAPPING[(SAVANNAid + 128)] = Biome.SAVANNA_MOUNTAINS;
    BIOME_MAPPING[(SAVANNA_PLATEAUid + 128)] = Biome.SAVANNA_PLATEAU_MOUNTAINS;
    BIOME_MAPPING[(MESAid + 128)] = Biome.MESA_BRYCE;
    BIOME_MAPPING[(MESA_PLATEAU_Fid + 128)] = Biome.MESA_PLATEAU_FOREST_MOUNTAINS;
    BIOME_MAPPING[(MESA_PLATEAUid + 128)] = Biome.MESA_PLATEAU_MOUNTAINS;
    BIOME_MAPPING[(BIRCH_FORESTid + 128)] = Biome.BIRCH_FOREST_MOUNTAINS;
    BIOME_MAPPING[(BIRCH_FOREST_HILLSid + 128)] = Biome.BIRCH_FOREST_HILLS_MOUNTAINS;
    BIOME_MAPPING[(ROOFED_FORESTid + 128)] = Biome.ROOFED_FOREST_MOUNTAINS;
    BIOME_MAPPING[(MEGA_TAIGAid + 128)] = Biome.MEGA_SPRUCE_TAIGA;
    BIOME_MAPPING[(EXTREME_HILLSid + 128)] = Biome.EXTREME_HILLS_MOUNTAINS;
    BIOME_MAPPING[(EXTREME_HILLS_PLUSid + 128)] = Biome.EXTREME_HILLS_PLUS_MOUNTAINS;
    BIOME_MAPPING[(MEGA_TAIGA_HILLSid + 128)] = Biome.MEGA_SPRUCE_TAIGA_HILLS;
    


    for (int i = 0; i < BIOME_MAPPING.length; i++) {
      if ((BiomeBase.getBiome(i) != null) && (BIOME_MAPPING[i] == null)) {
        throw new IllegalArgumentException("Missing Biome mapping for BiomeBase[" + i + ", " + BiomeBase.getBiome(i) + "]");
      }
      if (BIOME_MAPPING[i] != null) {
        BIOMEBASE_MAPPING[BIOME_MAPPING[i].ordinal()] = BiomeBase.getBiome(i);
      }
    }
  }
  
  public void setMetadata(String metadataKey, MetadataValue newMetadataValue) {
    chunk.getCraftWorld().getBlockMetadata().setMetadata(this, metadataKey, newMetadataValue);
  }
  
  public List<MetadataValue> getMetadata(String metadataKey) {
    return chunk.getCraftWorld().getBlockMetadata().getMetadata(this, metadataKey);
  }
  
  public boolean hasMetadata(String metadataKey) {
    return chunk.getCraftWorld().getBlockMetadata().hasMetadata(this, metadataKey);
  }
  
  public void removeMetadata(String metadataKey, Plugin owningPlugin) {
    chunk.getCraftWorld().getBlockMetadata().removeMetadata(this, metadataKey, owningPlugin);
  }
}
