package org.bukkit.craftbukkit.v1_7_R1.block;

import com.google.common.base.Strings;
import net.minecraft.server.v1_7_R1.TileEntitySkull;
import org.bukkit.SkullType;
import org.bukkit.block.Block;
import org.bukkit.block.BlockFace;
import org.bukkit.block.Skull;
import org.bukkit.craftbukkit.v1_7_R1.CraftWorld;

public class CraftSkull
  extends CraftBlockState implements Skull
{
  private final TileEntitySkull skull;
  private String player;
  private SkullType skullType;
  private byte rotation;
  private final int MAX_OWNER_LENGTH = 16;
  
  public CraftSkull(Block block) {
    super(block);
    
    CraftWorld world = (CraftWorld)block.getWorld();
    skull = ((TileEntitySkull)world.getTileEntityAt(getX(), getY(), getZ()));
    player = skull.getExtraType();
    skullType = getSkullType(skull.getSkullType());
    rotation = ((byte)skull.getRotation());
  }
  
  static SkullType getSkullType(int id) {
    switch (id) {
    case 0: 
      return SkullType.SKELETON;
    case 1: 
      return SkullType.WITHER;
    case 2: 
      return SkullType.ZOMBIE;
    case 3: 
      return SkullType.PLAYER;
    case 4: 
      return SkullType.CREEPER;
    }
    throw new AssertionError(id);
  }
  
  static int getSkullType(SkullType type)
  {
    switch (1.$SwitchMap$org$bukkit$SkullType[type.ordinal()]) {
    case 1: 
      return 0;
    case 2: 
      return 1;
    case 3: 
      return 2;
    case 4: 
      return 3;
    case 5: 
      return 4;
    }
    throw new AssertionError(type);
  }
  
  static byte getBlockFace(BlockFace rotation)
  {
    switch (1.$SwitchMap$org$bukkit$block$BlockFace[rotation.ordinal()]) {
    case 1: 
      return 0;
    case 2: 
      return 1;
    case 3: 
      return 2;
    case 4: 
      return 3;
    case 5: 
      return 4;
    case 6: 
      return 5;
    case 7: 
      return 6;
    case 8: 
      return 7;
    case 9: 
      return 8;
    case 10: 
      return 9;
    case 11: 
      return 10;
    case 12: 
      return 11;
    case 13: 
      return 12;
    case 14: 
      return 13;
    case 15: 
      return 14;
    case 16: 
      return 15;
    }
    throw new IllegalArgumentException("Invalid BlockFace rotation: " + rotation);
  }
  
  static BlockFace getBlockFace(byte rotation)
  {
    switch (rotation) {
    case 0: 
      return BlockFace.NORTH;
    case 1: 
      return BlockFace.NORTH_NORTH_EAST;
    case 2: 
      return BlockFace.NORTH_EAST;
    case 3: 
      return BlockFace.EAST_NORTH_EAST;
    case 4: 
      return BlockFace.EAST;
    case 5: 
      return BlockFace.EAST_SOUTH_EAST;
    case 6: 
      return BlockFace.SOUTH_EAST;
    case 7: 
      return BlockFace.SOUTH_SOUTH_EAST;
    case 8: 
      return BlockFace.SOUTH;
    case 9: 
      return BlockFace.SOUTH_SOUTH_WEST;
    case 10: 
      return BlockFace.SOUTH_WEST;
    case 11: 
      return BlockFace.WEST_SOUTH_WEST;
    case 12: 
      return BlockFace.WEST;
    case 13: 
      return BlockFace.WEST_NORTH_WEST;
    case 14: 
      return BlockFace.NORTH_WEST;
    case 15: 
      return BlockFace.NORTH_NORTH_WEST;
    }
    throw new AssertionError(rotation);
  }
  
  public boolean hasOwner()
  {
    return !Strings.isNullOrEmpty(player);
  }
  
  public String getOwner() {
    return player;
  }
  
  public boolean setOwner(String name) {
    if ((name == null) || (name.length() > 16)) {
      return false;
    }
    player = name;
    
    if (skullType != SkullType.PLAYER) {
      skullType = SkullType.PLAYER;
    }
    
    return true;
  }
  
  public BlockFace getRotation() {
    return getBlockFace(rotation);
  }
  
  public void setRotation(BlockFace rotation) {
    this.rotation = getBlockFace(rotation);
  }
  
  public SkullType getSkullType() {
    return skullType;
  }
  
  public void setSkullType(SkullType skullType) {
    this.skullType = skullType;
    
    if (skullType != SkullType.PLAYER) {
      player = "";
    }
  }
  
  public boolean update(boolean force, boolean applyPhysics)
  {
    boolean result = super.update(force, applyPhysics);
    
    if (result) {
      skull.setSkullType(getSkullType(skullType), player);
      skull.setRotation(rotation);
      skull.update();
    }
    
    return result;
  }
}
