package org.bukkit.craftbukkit.v1_7_R1.conversations;

import java.util.LinkedList;
import org.bukkit.conversations.Conversation;
import org.bukkit.conversations.ConversationAbandonedEvent;
import org.bukkit.conversations.ManuallyAbandonedConversationCanceller;




public class ConversationTracker
{
  private LinkedList<Conversation> conversationQueue = new LinkedList();
  
  public synchronized boolean beginConversation(Conversation conversation) {
    if (!conversationQueue.contains(conversation)) {
      conversationQueue.addLast(conversation);
      if (conversationQueue.getFirst() == conversation) {
        conversation.begin();
        conversation.outputNextPrompt();
        return true;
      }
    }
    return true;
  }
  
  public synchronized void abandonConversation(Conversation conversation, ConversationAbandonedEvent details) {
    if (!conversationQueue.isEmpty()) {
      if (conversationQueue.getFirst() == conversation) {
        conversation.abandon(details);
      }
      if (conversationQueue.contains(conversation)) {
        conversationQueue.remove(conversation);
      }
      if (!conversationQueue.isEmpty()) {
        ((Conversation)conversationQueue.getFirst()).outputNextPrompt();
      }
    }
  }
  
  public synchronized void abandonAllConversations()
  {
    LinkedList<Conversation> oldQueue = conversationQueue;
    conversationQueue = new LinkedList();
    for (Conversation conversation : oldQueue) {
      conversation.abandon(new ConversationAbandonedEvent(conversation, new ManuallyAbandonedConversationCanceller()));
    }
  }
  
  public synchronized void acceptConversationInput(String input) {
    if (isConversing()) {
      ((Conversation)conversationQueue.getFirst()).acceptInput(input);
    }
  }
  
  public synchronized boolean isConversing() {
    return !conversationQueue.isEmpty();
  }
  
  public synchronized boolean isConversingModaly() {
    return (isConversing()) && (((Conversation)conversationQueue.getFirst()).isModal());
  }
}
