package org.bukkit.craftbukkit.v1_7_R1.entity;

import java.util.Set;
import net.minecraft.server.v1_7_R1.Container;
import net.minecraft.server.v1_7_R1.EntityHuman;
import net.minecraft.server.v1_7_R1.EntityMinecartHopper;
import net.minecraft.server.v1_7_R1.EntityPlayer;
import net.minecraft.server.v1_7_R1.PacketPlayInCloseWindow;
import net.minecraft.server.v1_7_R1.PacketPlayOutOpenWindow;
import net.minecraft.server.v1_7_R1.PlayerConnection;
import net.minecraft.server.v1_7_R1.TileEntityBrewingStand;
import net.minecraft.server.v1_7_R1.TileEntityDispenser;
import net.minecraft.server.v1_7_R1.TileEntityFurnace;
import net.minecraft.server.v1_7_R1.TileEntityHopper;
import org.bukkit.GameMode;
import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.block.Block;
import org.bukkit.craftbukkit.v1_7_R1.CraftServer;
import org.bukkit.craftbukkit.v1_7_R1.event.CraftEventFactory;
import org.bukkit.craftbukkit.v1_7_R1.inventory.CraftContainer;
import org.bukkit.craftbukkit.v1_7_R1.inventory.CraftInventory;
import org.bukkit.craftbukkit.v1_7_R1.inventory.CraftInventoryPlayer;
import org.bukkit.craftbukkit.v1_7_R1.inventory.CraftInventoryView;
import org.bukkit.craftbukkit.v1_7_R1.inventory.CraftItemStack;
import org.bukkit.entity.HumanEntity;
import org.bukkit.event.inventory.InventoryType;
import org.bukkit.inventory.EntityEquipment;
import org.bukkit.inventory.Inventory;
import org.bukkit.inventory.InventoryView;
import org.bukkit.inventory.InventoryView.Property;
import org.bukkit.permissions.PermissibleBase;
import org.bukkit.permissions.Permission;
import org.bukkit.permissions.PermissionAttachment;
import org.bukkit.permissions.PermissionAttachmentInfo;
import org.bukkit.plugin.Plugin;

public class CraftHumanEntity
  extends CraftLivingEntity implements HumanEntity
{
  private CraftInventoryPlayer inventory;
  private final CraftInventory enderChest;
  protected final PermissibleBase perm = new PermissibleBase(this);
  private boolean op;
  private GameMode mode;
  
  public CraftHumanEntity(CraftServer server, EntityHuman entity) {
    super(server, entity);
    mode = server.getDefaultGameMode();
    inventory = new CraftInventoryPlayer(inventory);
    enderChest = new CraftInventory(entity.getEnderChest());
  }
  
  public String getName() {
    return getHandle().getName();
  }
  
  public org.bukkit.inventory.PlayerInventory getInventory() {
    return inventory;
  }
  
  public EntityEquipment getEquipment() {
    return inventory;
  }
  
  public Inventory getEnderChest() {
    return enderChest;
  }
  
  public org.bukkit.inventory.ItemStack getItemInHand() {
    return getInventory().getItemInHand();
  }
  
  public void setItemInHand(org.bukkit.inventory.ItemStack item) {
    getInventory().setItemInHand(item);
  }
  
  public org.bukkit.inventory.ItemStack getItemOnCursor() {
    return CraftItemStack.asCraftMirror(getHandle()inventory.getCarried());
  }
  
  public void setItemOnCursor(org.bukkit.inventory.ItemStack item) {
    net.minecraft.server.v1_7_R1.ItemStack stack = CraftItemStack.asNMSCopy(item);
    getHandle()inventory.setCarried(stack);
    if ((this instanceof CraftPlayer)) {
      ((EntityPlayer)getHandle()).broadcastCarriedItem();
    }
  }
  
  public boolean isSleeping() {
    return getHandle()sleeping;
  }
  
  public int getSleepTicks() {
    return getHandle()sleepTicks;
  }
  
  public boolean isOp() {
    return op;
  }
  
  public boolean isPermissionSet(String name) {
    return perm.isPermissionSet(name);
  }
  
  public boolean isPermissionSet(Permission perm) {
    return this.perm.isPermissionSet(perm);
  }
  
  public boolean hasPermission(String name) {
    return perm.hasPermission(name);
  }
  
  public boolean hasPermission(Permission perm) {
    return this.perm.hasPermission(perm);
  }
  
  public PermissionAttachment addAttachment(Plugin plugin, String name, boolean value) {
    return perm.addAttachment(plugin, name, value);
  }
  
  public PermissionAttachment addAttachment(Plugin plugin) {
    return perm.addAttachment(plugin);
  }
  
  public PermissionAttachment addAttachment(Plugin plugin, String name, boolean value, int ticks) {
    return perm.addAttachment(plugin, name, value, ticks);
  }
  
  public PermissionAttachment addAttachment(Plugin plugin, int ticks) {
    return perm.addAttachment(plugin, ticks);
  }
  
  public void removeAttachment(PermissionAttachment attachment) {
    perm.removeAttachment(attachment);
  }
  
  public void recalculatePermissions() {
    perm.recalculatePermissions();
  }
  
  public void setOp(boolean value) {
    op = value;
    perm.recalculatePermissions();
  }
  
  public Set<PermissionAttachmentInfo> getEffectivePermissions() {
    return perm.getEffectivePermissions();
  }
  
  public GameMode getGameMode() {
    return mode;
  }
  
  public void setGameMode(GameMode mode) {
    if (mode == null) {
      throw new IllegalArgumentException("Mode cannot be null");
    }
    
    this.mode = mode;
  }
  
  public EntityHuman getHandle()
  {
    return (EntityHuman)entity;
  }
  
  public void setHandle(EntityHuman entity) {
    super.setHandle(entity);
    inventory = new CraftInventoryPlayer(inventory);
  }
  
  public String toString()
  {
    return "CraftHumanEntity{id=" + getEntityId() + "name=" + getName() + '}';
  }
  
  public InventoryView getOpenInventory() {
    return getHandle()activeContainer.getBukkitView();
  }
  
  public InventoryView openInventory(Inventory inventory) {
    if (!(getHandle() instanceof EntityPlayer)) return null;
    EntityPlayer player = (EntityPlayer)getHandle();
    InventoryType type = inventory.getType();
    Container formerContainer = getHandle()activeContainer;
    
    CraftInventory craftinv = (CraftInventory)inventory;
    switch (1.$SwitchMap$org$bukkit$event$inventory$InventoryType[type.ordinal()]) {
    case 1: 
    case 2: 
    case 3: 
      getHandle().openContainer(craftinv.getInventory());
      break;
    case 4: 
      if ((craftinv.getInventory() instanceof TileEntityDispenser)) {
        getHandle().openDispenser((TileEntityDispenser)craftinv.getInventory());
      } else {
        openCustomInventory(inventory, player, 3);
      }
      break;
    case 5: 
      if ((craftinv.getInventory() instanceof TileEntityFurnace)) {
        getHandle().openFurnace((TileEntityFurnace)craftinv.getInventory());
      } else {
        openCustomInventory(inventory, player, 2);
      }
      break;
    case 6: 
      openCustomInventory(inventory, player, 1);
      break;
    case 7: 
      if ((craftinv.getInventory() instanceof TileEntityBrewingStand)) {
        getHandle().openBrewingStand((TileEntityBrewingStand)craftinv.getInventory());
      } else {
        openCustomInventory(inventory, player, 5);
      }
      break;
    case 8: 
      openCustomInventory(inventory, player, 4);
      break;
    case 9: 
      if ((craftinv.getInventory() instanceof TileEntityHopper)) {
        getHandle().openHopper((TileEntityHopper)craftinv.getInventory());
      } else if ((craftinv.getInventory() instanceof EntityMinecartHopper)) {
        getHandle().openMinecartHopper((EntityMinecartHopper)craftinv.getInventory());
      } else {
        openCustomInventory(inventory, player, 9);
      }
      break;
    case 10: 
    case 11: 
      throw new IllegalArgumentException("Can't open a " + type + " inventory!");
    }
    if (getHandle()activeContainer == formerContainer) {
      return null;
    }
    getHandle()activeContainer.checkReachable = false;
    return getHandle()activeContainer.getBukkitView();
  }
  
  private void openCustomInventory(Inventory inventory, EntityPlayer player, int windowType) {
    if (playerConnection == null) return;
    Container container = new CraftContainer(inventory, this, player.nextContainerCounter());
    
    container = CraftEventFactory.callInventoryOpenEvent(player, container);
    if (container == null) { return;
    }
    String title = container.getBukkitView().getTitle();
    int size = container.getBukkitView().getTopInventory().getSize();
    
    playerConnection.sendPacket(new PacketPlayOutOpenWindow(windowId, windowType, title, size, true));
    getHandle()activeContainer = container;
    getHandle()activeContainer.addSlotListener(player);
  }
  
  public InventoryView openWorkbench(Location location, boolean force) {
    if (!force) {
      Block block = location.getBlock();
      if (block.getType() != Material.WORKBENCH) {
        return null;
      }
    }
    if (location == null) {
      location = getLocation();
    }
    getHandle().startCrafting(location.getBlockX(), location.getBlockY(), location.getBlockZ());
    if (force) {
      getHandle()activeContainer.checkReachable = false;
    }
    return getHandle()activeContainer.getBukkitView();
  }
  
  public InventoryView openEnchanting(Location location, boolean force) {
    if (!force) {
      Block block = location.getBlock();
      if (block.getType() != Material.ENCHANTMENT_TABLE) {
        return null;
      }
    }
    if (location == null) {
      location = getLocation();
    }
    getHandle().startEnchanting(location.getBlockX(), location.getBlockY(), location.getBlockZ(), null);
    if (force) {
      getHandle()activeContainer.checkReachable = false;
    }
    return getHandle()activeContainer.getBukkitView();
  }
  
  public void openInventory(InventoryView inventory) {
    if (!(getHandle() instanceof EntityPlayer)) return;
    if (getHandle()playerConnection == null) return;
    if (getHandle()activeContainer != getHandle()defaultContainer)
    {
      getHandle()playerConnection.a(new PacketPlayInCloseWindow(getHandle()activeContainer.windowId));
    }
    EntityPlayer player = (EntityPlayer)getHandle();
    Container container;
    if ((inventory instanceof CraftInventoryView)) {
      container = ((CraftInventoryView)inventory).getHandle();
    } else {
      container = new CraftContainer(inventory, player.nextContainerCounter());
    }
    

    Container container = CraftEventFactory.callInventoryOpenEvent(player, container);
    if (container == null) {
      return;
    }
    

    InventoryType type = inventory.getType();
    int windowType = CraftContainer.getNotchInventoryType(type);
    String title = inventory.getTitle();
    int size = inventory.getTopInventory().getSize();
    playerConnection.sendPacket(new PacketPlayOutOpenWindow(windowId, windowType, title, size, false));
    activeContainer = container;
    activeContainer.addSlotListener(player);
  }
  
  public void closeInventory() {
    getHandle().closeInventory();
  }
  
  public boolean isBlocking() {
    return getHandle().isBlocking();
  }
  
  public boolean setWindowProperty(InventoryView.Property prop, int value) {
    return false;
  }
  
  public int getExpToLevel() {
    return getHandle().getExpToLevel();
  }
}
