package org.bukkit.craftbukkit.v1_7_R1.entity;

import net.minecraft.server.v1_7_R1.EntityItemFrame;
import net.minecraft.server.v1_7_R1.EntityTracker;
import net.minecraft.server.v1_7_R1.WorldServer;
import org.bukkit.Rotation;
import org.bukkit.block.BlockFace;
import org.bukkit.craftbukkit.v1_7_R1.CraftServer;
import org.bukkit.craftbukkit.v1_7_R1.CraftWorld;
import org.bukkit.craftbukkit.v1_7_R1.inventory.CraftItemStack;
import org.bukkit.entity.EntityType;
import org.bukkit.inventory.ItemStack;

public class CraftItemFrame extends CraftHanging implements org.bukkit.entity.ItemFrame
{
  public CraftItemFrame(CraftServer server, EntityItemFrame entity)
  {
    super(server, entity);
  }
  
  public boolean setFacingDirection(BlockFace face, boolean force) {
    if (!super.setFacingDirection(face, force)) {
      return false;
    }
    
    WorldServer world = ((CraftWorld)getWorld()).getHandle();
    world.getTracker().untrackEntity(getHandle());
    world.getTracker().track(getHandle());
    return true;
  }
  
  public void setItem(ItemStack item) {
    if ((item == null) || (item.getTypeId() == 0)) {
      getHandle().getDataWatcher().a(2, 5);
      getHandle().getDataWatcher().h(2);
    } else {
      getHandle().setItem(CraftItemStack.asNMSCopy(item));
    }
  }
  
  public ItemStack getItem() {
    return CraftItemStack.asBukkitCopy(getHandle().getItem());
  }
  
  public Rotation getRotation() {
    return toBukkitRotation(getHandle().getRotation());
  }
  
  Rotation toBukkitRotation(int value)
  {
    switch (value) {
    case 0: 
      return Rotation.NONE;
    case 1: 
      return Rotation.CLOCKWISE;
    case 2: 
      return Rotation.FLIPPED;
    case 3: 
      return Rotation.COUNTER_CLOCKWISE;
    }
    throw new AssertionError("Unknown rotation " + value + " for " + getHandle());
  }
  
  public void setRotation(Rotation rotation)
  {
    org.apache.commons.lang.Validate.notNull(rotation, "Rotation cannot be null");
    getHandle().setRotation(toInteger(rotation));
  }
  
  static int toInteger(Rotation rotation)
  {
    switch (1.$SwitchMap$org$bukkit$Rotation[rotation.ordinal()]) {
    case 1: 
      return 0;
    case 2: 
      return 1;
    case 3: 
      return 2;
    case 4: 
      return 3;
    }
    throw new IllegalArgumentException(rotation + " is not applicable to an ItemFrame");
  }
  

  public EntityItemFrame getHandle()
  {
    return (EntityItemFrame)entity;
  }
  
  public String toString()
  {
    return "CraftItemFrame{item=" + getItem() + ", rotation=" + getRotation() + "}";
  }
  
  public EntityType getType() {
    return EntityType.ITEM_FRAME;
  }
}
