package org.bukkit.craftbukkit.v1_7_R1.entity;

import com.google.common.collect.MapMaker;
import java.io.ByteArrayOutputStream;
import java.net.InetSocketAddress;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.logging.Logger;
import net.minecraft.server.v1_7_R1.AttributeInstance;
import net.minecraft.server.v1_7_R1.AttributeMapServer;
import net.minecraft.server.v1_7_R1.AttributeRanged;
import net.minecraft.server.v1_7_R1.BanList;
import net.minecraft.server.v1_7_R1.ChunkCoordinates;
import net.minecraft.server.v1_7_R1.DedicatedPlayerList;
import net.minecraft.server.v1_7_R1.EntityPlayer;
import net.minecraft.server.v1_7_R1.EntityTracker;
import net.minecraft.server.v1_7_R1.EntityTrackerEntry;
import net.minecraft.server.v1_7_R1.FoodMetaData;
import net.minecraft.server.v1_7_R1.NBTTagCompound;
import net.minecraft.server.v1_7_R1.PacketPlayOutCustomPayload;
import net.minecraft.server.v1_7_R1.PacketPlayOutNamedSoundEffect;
import net.minecraft.server.v1_7_R1.PacketPlayOutPlayerInfo;
import net.minecraft.server.v1_7_R1.PlayerAbilities;
import net.minecraft.server.v1_7_R1.PlayerConnection;
import net.minecraft.server.v1_7_R1.ServerStatisticManager;
import net.minecraft.server.v1_7_R1.WorldServer;
import org.apache.commons.lang.Validate;
import org.bukkit.Achievement;
import org.bukkit.Effect;
import org.bukkit.GameMode;
import org.bukkit.Instrument;
import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.Note;
import org.bukkit.OfflinePlayer;
import org.bukkit.Statistic.Type;
import org.bukkit.WeatherType;
import org.bukkit.conversations.Conversation;
import org.bukkit.conversations.ConversationAbandonedEvent;
import org.bukkit.craftbukkit.v1_7_R1.CraftServer;
import org.bukkit.craftbukkit.v1_7_R1.CraftStatistic;
import org.bukkit.craftbukkit.v1_7_R1.CraftWorld;
import org.bukkit.craftbukkit.v1_7_R1.conversations.ConversationTracker;
import org.bukkit.craftbukkit.v1_7_R1.metadata.PlayerMetadataStore;
import org.bukkit.entity.EntityType;
import org.bukkit.entity.Player;
import org.bukkit.event.player.PlayerGameModeChangeEvent;
import org.bukkit.event.player.PlayerTeleportEvent;
import org.bukkit.inventory.InventoryView.Property;
import org.bukkit.map.MapView;
import org.bukkit.metadata.MetadataValue;
import org.bukkit.plugin.Plugin;
import org.bukkit.plugin.PluginManager;

@org.bukkit.configuration.serialization.DelegateDeserialization(org.bukkit.craftbukkit.v1_7_R1.CraftOfflinePlayer.class)
public class CraftPlayer extends CraftHumanEntity implements Player
{
  private long firstPlayed = 0L;
  private long lastPlayed = 0L;
  private boolean hasPlayedBefore = false;
  private final ConversationTracker conversationTracker = new ConversationTracker();
  private final Set<String> channels = new java.util.HashSet();
  private final Map<String, Player> hiddenPlayers = new MapMaker().softValues().makeMap();
  private int hash = 0;
  private double health = 20.0D;
  private boolean scaledHealth = false;
  private double healthScale = 20.0D;
  
  public CraftPlayer(CraftServer server, EntityPlayer entity) {
    super(server, entity);
    
    firstPlayed = System.currentTimeMillis();
  }
  
  public boolean isOp()
  {
    return server.getHandle().isOp(getName());
  }
  
  public void setOp(boolean value)
  {
    if (value == isOp()) { return;
    }
    if (value) {
      server.getHandle().addOp(getName());
    } else {
      server.getHandle().removeOp(getName());
    }
    
    perm.recalculatePermissions();
  }
  
  public boolean isOnline() {
    for (Object obj : server.getHandle().players) {
      EntityPlayer player = (EntityPlayer)obj;
      if (player.getName().equalsIgnoreCase(getName())) {
        return true;
      }
    }
    return false;
  }
  
  public InetSocketAddress getAddress() {
    if (getHandle()playerConnection == null) { return null;
    }
    java.net.SocketAddress addr = getHandle()playerConnection.networkManager.getSocketAddress();
    if ((addr instanceof InetSocketAddress)) {
      return (InetSocketAddress)addr;
    }
    return null;
  }
  

  public double getEyeHeight()
  {
    return getEyeHeight(false);
  }
  
  public double getEyeHeight(boolean ignoreSneaking)
  {
    if (ignoreSneaking) {
      return 1.62D;
    }
    if (isSneaking()) {
      return 1.54D;
    }
    return 1.62D;
  }
  


  public void sendRawMessage(String message)
  {
    if (getHandle()playerConnection == null) { return;
    }
    for (net.minecraft.server.v1_7_R1.IChatBaseComponent component : org.bukkit.craftbukkit.v1_7_R1.util.CraftChatMessage.fromString(message)) {
      getHandle()playerConnection.sendPacket(new net.minecraft.server.v1_7_R1.PacketPlayOutChat(component));
    }
  }
  
  public void sendMessage(String message)
  {
    if (!conversationTracker.isConversingModaly()) {
      sendRawMessage(message);
    }
  }
  
  public void sendMessage(String[] messages)
  {
    for (String message : messages) {
      sendMessage(message);
    }
  }
  
  public String getDisplayName()
  {
    return getHandle()displayName;
  }
  
  public void setDisplayName(String name)
  {
    getHandle()displayName = name;
  }
  
  public String getPlayerListName()
  {
    return getHandle()listName;
  }
  
  public void setPlayerListName(String name)
  {
    String oldName = getHandle()listName;
    
    if (name == null) {
      name = getName();
    }
    
    if (oldName.equals(name)) {
      return;
    }
    
    if (name.length() > 16) {
      throw new IllegalArgumentException("Player list names can only be a maximum of 16 characters long");
    }
    

    for (int i = 0; i < server.getHandle().players.size(); i++) {
      if (server.getHandle().players.get(i)).listName.equals(name)) {
        throw new IllegalArgumentException(name + " is already assigned as a player list name for someone");
      }
    }
    
    getHandle()listName = name;
    

    PacketPlayOutPlayerInfo oldpacket = new PacketPlayOutPlayerInfo(oldName, false, 9999);
    PacketPlayOutPlayerInfo packet = new PacketPlayOutPlayerInfo(name, true, getHandle()ping);
    for (int i = 0; i < server.getHandle().players.size(); i++) {
      EntityPlayer entityplayer = (EntityPlayer)server.getHandle().players.get(i);
      if (playerConnection != null)
      {
        if (entityplayer.getBukkitEntity().canSee(this)) {
          playerConnection.sendPacket(oldpacket);
          playerConnection.sendPacket(packet);
        }
      }
    }
  }
  
  public boolean equals(Object obj) {
    if (!(obj instanceof OfflinePlayer)) {
      return false;
    }
    OfflinePlayer other = (OfflinePlayer)obj;
    if ((getName() == null) || (other.getName() == null)) {
      return false;
    }
    
    boolean nameEquals = getName().equalsIgnoreCase(other.getName());
    boolean idEquals = true;
    
    if ((other instanceof CraftPlayer)) {
      idEquals = getEntityId() == ((CraftPlayer)other).getEntityId();
    }
    
    return (nameEquals) && (idEquals);
  }
  
  public void kickPlayer(String message)
  {
    if (getHandle()playerConnection == null) { return;
    }
    getHandle()playerConnection.disconnect(message == null ? "" : message);
  }
  
  public void setCompassTarget(Location loc)
  {
    if (getHandle()playerConnection == null) { return;
    }
    
    getHandle()playerConnection.sendPacket(new net.minecraft.server.v1_7_R1.PacketPlayOutSpawnPosition(loc.getBlockX(), loc.getBlockY(), loc.getBlockZ()));
  }
  
  public Location getCompassTarget()
  {
    return getHandle()compassTarget;
  }
  
  public void chat(String msg)
  {
    if (getHandle()playerConnection == null) { return;
    }
    getHandle()playerConnection.chat(msg, false);
  }
  
  public boolean performCommand(String command)
  {
    return server.dispatchCommand(this, command);
  }
  
  public void playNote(Location loc, byte instrument, byte note)
  {
    if (getHandle()playerConnection == null) { return;
    }
    String instrumentName = null;
    switch (instrument) {
    case 0: 
      instrumentName = "harp";
      break;
    case 1: 
      instrumentName = "bd";
      break;
    case 2: 
      instrumentName = "snare";
      break;
    case 3: 
      instrumentName = "hat";
      break;
    case 4: 
      instrumentName = "bassattack";
    }
    
    getHandle()playerConnection.sendPacket(new PacketPlayOutNamedSoundEffect("note." + instrumentName, loc.getBlockX(), loc.getBlockY(), loc.getBlockZ(), 3.0F, note));
  }
  
  public void playNote(Location loc, Instrument instrument, Note note)
  {
    if (getHandle()playerConnection == null) { return;
    }
    String instrumentName = null;
    switch (instrument.ordinal()) {
    case 0: 
      instrumentName = "harp";
      break;
    case 1: 
      instrumentName = "bd";
      break;
    case 2: 
      instrumentName = "snare";
      break;
    case 3: 
      instrumentName = "hat";
      break;
    case 4: 
      instrumentName = "bassattack";
    }
    
    getHandle()playerConnection.sendPacket(new PacketPlayOutNamedSoundEffect("note." + instrumentName, loc.getBlockX(), loc.getBlockY(), loc.getBlockZ(), 3.0F, note.getId()));
  }
  
  public void playSound(Location loc, org.bukkit.Sound sound, float volume, float pitch)
  {
    if (sound == null) {
      return;
    }
    playSound(loc, org.bukkit.craftbukkit.v1_7_R1.CraftSound.getSound(sound), volume, pitch);
  }
  
  public void playSound(Location loc, String sound, float volume, float pitch)
  {
    if ((loc == null) || (sound == null) || (getHandle()playerConnection == null)) { return;
    }
    double x = loc.getBlockX() + 0.5D;
    double y = loc.getBlockY() + 0.5D;
    double z = loc.getBlockZ() + 0.5D;
    
    PacketPlayOutNamedSoundEffect packet = new PacketPlayOutNamedSoundEffect(sound, x, y, z, volume, pitch);
    getHandle()playerConnection.sendPacket(packet);
  }
  
  public void playEffect(Location loc, Effect effect, int data)
  {
    if (getHandle()playerConnection == null) { return;
    }
    int packetData = effect.getId();
    net.minecraft.server.v1_7_R1.PacketPlayOutWorldEvent packet = new net.minecraft.server.v1_7_R1.PacketPlayOutWorldEvent(packetData, loc.getBlockX(), loc.getBlockY(), loc.getBlockZ(), data, false);
    getHandle()playerConnection.sendPacket(packet);
  }
  
  public <T> void playEffect(Location loc, Effect effect, T data)
  {
    if (data != null) {
      Validate.isTrue(data.getClass().equals(effect.getData()), "Wrong kind of data for this effect!");
    } else {
      Validate.isTrue(effect.getData() == null, "Wrong kind of data for this effect!");
    }
    
    int datavalue = data == null ? 0 : org.bukkit.craftbukkit.v1_7_R1.CraftEffect.getDataValue(effect, data);
    playEffect(loc, effect, datavalue);
  }
  
  public void sendBlockChange(Location loc, Material material, byte data)
  {
    sendBlockChange(loc, material.getId(), data);
  }
  
  public void sendBlockChange(Location loc, int material, byte data)
  {
    if (getHandle()playerConnection == null) { return;
    }
    net.minecraft.server.v1_7_R1.PacketPlayOutBlockChange packet = new net.minecraft.server.v1_7_R1.PacketPlayOutBlockChange(loc.getBlockX(), loc.getBlockY(), loc.getBlockZ(), ((CraftWorld)loc.getWorld()).getHandle());
    
    block = org.bukkit.craftbukkit.v1_7_R1.util.CraftMagicNumbers.getBlock(material);
    data = data;
    getHandle()playerConnection.sendPacket(packet);
  }
  
  public boolean sendChunkChange(Location loc, int sx, int sy, int sz, byte[] data)
  {
    if (getHandle()playerConnection == null) { return false;
    }
    


























    throw new org.apache.commons.lang.NotImplementedException("Chunk changes do not yet work");
  }
  
  public void sendMap(MapView map)
  {
    if (getHandle()playerConnection == null) { return;
    }
    org.bukkit.craftbukkit.v1_7_R1.map.RenderData data = ((org.bukkit.craftbukkit.v1_7_R1.map.CraftMapView)map).render(this);
    for (int x = 0; x < 128; x++) {
      byte[] bytes = new byte[''];
      bytes[1] = ((byte)x);
      for (int y = 0; y < 128; y++) {
        bytes[(y + 3)] = buffer[(y * 128 + x)];
      }
      net.minecraft.server.v1_7_R1.PacketPlayOutMap packet = new net.minecraft.server.v1_7_R1.PacketPlayOutMap(map.getId(), bytes);
      getHandle()playerConnection.sendPacket(packet);
    }
  }
  
  public boolean teleport(Location location, org.bukkit.event.player.PlayerTeleportEvent.TeleportCause cause)
  {
    EntityPlayer entity = getHandle();
    
    if ((getHealth() == 0.0D) || (dead)) {
      return false;
    }
    
    if ((playerConnection == null) || (playerConnection.isDisconnected())) {
      return false;
    }
    
    if ((vehicle != null) || (passenger != null)) {
      return false;
    }
    

    Location from = getLocation();
    
    Location to = location;
    
    PlayerTeleportEvent event = new PlayerTeleportEvent(this, from, to, cause);
    server.getPluginManager().callEvent(event);
    

    if (event.isCancelled()) {
      return false;
    }
    

    from = event.getFrom();
    
    to = event.getTo();
    
    WorldServer fromWorld = ((CraftWorld)from.getWorld()).getHandle();
    WorldServer toWorld = ((CraftWorld)to.getWorld()).getHandle();
    

    if (getHandle()activeContainer != getHandle()defaultContainer) {
      getHandle().closeInventory();
    }
    

    if (fromWorld == toWorld) {
      playerConnection.teleport(to);
    } else {
      server.getHandle().moveToWorld(entity, dimension, true, to, true);
    }
    return true;
  }
  
  public void setSneaking(boolean sneak)
  {
    getHandle().setSneaking(sneak);
  }
  
  public boolean isSneaking()
  {
    return getHandle().isSneaking();
  }
  
  public boolean isSprinting()
  {
    return getHandle().isSprinting();
  }
  
  public void setSprinting(boolean sprinting)
  {
    getHandle().setSprinting(sprinting);
  }
  
  public void loadData()
  {
    server.getHandle().playerFileData.load(getHandle());
  }
  
  public void saveData()
  {
    server.getHandle().playerFileData.save(getHandle());
  }
  
  @Deprecated
  public void updateInventory()
  {
    getHandle().updateInventory(getHandle()activeContainer);
  }
  
  public void setSleepingIgnored(boolean isSleeping)
  {
    getHandle()fauxSleeping = isSleeping;
    ((CraftWorld)getWorld()).getHandle().checkSleepStatus();
  }
  
  public boolean isSleepingIgnored()
  {
    return getHandle()fauxSleeping;
  }
  
  public void awardAchievement(Achievement achievement)
  {
    Validate.notNull(achievement, "Achievement cannot be null");
    if ((achievement.hasParent()) && (!hasAchievement(achievement.getParent()))) {
      awardAchievement(achievement.getParent());
    }
    getHandle().x().a(getHandle(), CraftStatistic.getNMSAchievement(achievement), 1);
    getHandle().x().b(getHandle());
  }
  
  public void removeAchievement(Achievement achievement)
  {
    Validate.notNull(achievement, "Achievement cannot be null");
    for (Achievement achieve : Achievement.values()) {
      if ((achieve.getParent() == achievement) && (hasAchievement(achieve))) {
        removeAchievement(achieve);
      }
    }
    getHandle().x().a(getHandle(), CraftStatistic.getNMSAchievement(achievement), 0);
  }
  
  public boolean hasAchievement(Achievement achievement)
  {
    Validate.notNull(achievement, "Achievement cannot be null");
    return getHandle().x().a(CraftStatistic.getNMSAchievement(achievement));
  }
  
  public void incrementStatistic(org.bukkit.Statistic statistic)
  {
    incrementStatistic(statistic, 1);
  }
  
  public void decrementStatistic(org.bukkit.Statistic statistic)
  {
    decrementStatistic(statistic, 1);
  }
  
  public int getStatistic(org.bukkit.Statistic statistic)
  {
    Validate.notNull(statistic, "Statistic cannot be null");
    Validate.isTrue(statistic.getType() == Statistic.Type.UNTYPED, "Must supply additional paramater for this statistic");
    return getHandle().x().a(CraftStatistic.getNMSStatistic(statistic));
  }
  
  public void incrementStatistic(org.bukkit.Statistic statistic, int amount)
  {
    Validate.isTrue(amount > 0, "Amount must be greater than 0");
    setStatistic(statistic, getStatistic(statistic) + amount);
  }
  
  public void decrementStatistic(org.bukkit.Statistic statistic, int amount)
  {
    Validate.isTrue(amount > 0, "Amount must be greater than 0");
    setStatistic(statistic, getStatistic(statistic) - amount);
  }
  
  public void setStatistic(org.bukkit.Statistic statistic, int newValue)
  {
    Validate.notNull(statistic, "Statistic cannot be null");
    Validate.isTrue(statistic.getType() == Statistic.Type.UNTYPED, "Must supply additional paramater for this statistic");
    Validate.isTrue(newValue >= 0, "Value must be greater than or equal to 0");
    net.minecraft.server.v1_7_R1.Statistic nmsStatistic = CraftStatistic.getNMSStatistic(statistic);
    getHandle().x().a(getHandle(), nmsStatistic, newValue);
  }
  
  public void incrementStatistic(org.bukkit.Statistic statistic, Material material)
  {
    incrementStatistic(statistic, material, 1);
  }
  
  public void decrementStatistic(org.bukkit.Statistic statistic, Material material)
  {
    decrementStatistic(statistic, material, 1);
  }
  
  public int getStatistic(org.bukkit.Statistic statistic, Material material)
  {
    Validate.notNull(statistic, "Statistic cannot be null");
    Validate.notNull(material, "Material cannot be null");
    Validate.isTrue((statistic.getType() == Statistic.Type.BLOCK) || (statistic.getType() == Statistic.Type.ITEM), "This statistic does not take a Material parameter");
    net.minecraft.server.v1_7_R1.Statistic nmsStatistic = CraftStatistic.getMaterialStatistic(statistic, material);
    Validate.notNull(nmsStatistic, "The supplied Material does not have a corresponding statistic");
    return getHandle().x().a(nmsStatistic);
  }
  
  public void incrementStatistic(org.bukkit.Statistic statistic, Material material, int amount)
  {
    Validate.isTrue(amount > 0, "Amount must be greater than 0");
    setStatistic(statistic, material, getStatistic(statistic, material) + amount);
  }
  
  public void decrementStatistic(org.bukkit.Statistic statistic, Material material, int amount)
  {
    Validate.isTrue(amount > 0, "Amount must be greater than 0");
    setStatistic(statistic, material, getStatistic(statistic, material) - amount);
  }
  
  public void setStatistic(org.bukkit.Statistic statistic, Material material, int newValue)
  {
    Validate.notNull(statistic, "Statistic cannot be null");
    Validate.notNull(material, "Material cannot be null");
    Validate.isTrue(newValue >= 0, "Value must be greater than or equal to 0");
    Validate.isTrue((statistic.getType() == Statistic.Type.BLOCK) || (statistic.getType() == Statistic.Type.ITEM), "This statistic does not take a Material parameter");
    net.minecraft.server.v1_7_R1.Statistic nmsStatistic = CraftStatistic.getMaterialStatistic(statistic, material);
    Validate.notNull(nmsStatistic, "The supplied Material does not have a corresponding statistic");
    getHandle().x().a(getHandle(), nmsStatistic, newValue);
  }
  
  public void incrementStatistic(org.bukkit.Statistic statistic, EntityType entityType)
  {
    incrementStatistic(statistic, entityType, 1);
  }
  
  public void decrementStatistic(org.bukkit.Statistic statistic, EntityType entityType)
  {
    decrementStatistic(statistic, entityType, 1);
  }
  
  public int getStatistic(org.bukkit.Statistic statistic, EntityType entityType)
  {
    Validate.notNull(statistic, "Statistic cannot be null");
    Validate.notNull(entityType, "EntityType cannot be null");
    Validate.isTrue(statistic.getType() == Statistic.Type.ENTITY, "This statistic does not take an EntityType parameter");
    net.minecraft.server.v1_7_R1.Statistic nmsStatistic = CraftStatistic.getEntityStatistic(statistic, entityType);
    Validate.notNull(nmsStatistic, "The supplied EntityType does not have a corresponding statistic");
    return getHandle().x().a(nmsStatistic);
  }
  
  public void incrementStatistic(org.bukkit.Statistic statistic, EntityType entityType, int amount)
  {
    Validate.isTrue(amount > 0, "Amount must be greater than 0");
    setStatistic(statistic, entityType, getStatistic(statistic, entityType) + amount);
  }
  
  public void decrementStatistic(org.bukkit.Statistic statistic, EntityType entityType, int amount)
  {
    Validate.isTrue(amount > 0, "Amount must be greater than 0");
    setStatistic(statistic, entityType, getStatistic(statistic, entityType) - amount);
  }
  
  public void setStatistic(org.bukkit.Statistic statistic, EntityType entityType, int newValue)
  {
    Validate.notNull(statistic, "Statistic cannot be null");
    Validate.notNull(entityType, "EntityType cannot be null");
    Validate.isTrue(newValue >= 0, "Value must be greater than or equal to 0");
    Validate.isTrue(statistic.getType() == Statistic.Type.ENTITY, "This statistic does not take an EntityType parameter");
    net.minecraft.server.v1_7_R1.Statistic nmsStatistic = CraftStatistic.getEntityStatistic(statistic, entityType);
    Validate.notNull(nmsStatistic, "The supplied EntityType does not have a corresponding statistic");
    getHandle().x().a(getHandle(), nmsStatistic, newValue);
  }
  
  public void setPlayerTime(long time, boolean relative)
  {
    getHandle()timeOffset = time;
    getHandle()relativeTime = relative;
  }
  
  public long getPlayerTimeOffset()
  {
    return getHandle()timeOffset;
  }
  
  public long getPlayerTime()
  {
    return getHandle().getPlayerTime();
  }
  
  public boolean isPlayerTimeRelative()
  {
    return getHandle()relativeTime;
  }
  
  public void resetPlayerTime()
  {
    setPlayerTime(0L, true);
  }
  
  public void setPlayerWeather(WeatherType type)
  {
    getHandle().setPlayerWeather(type, true);
  }
  
  public WeatherType getPlayerWeather()
  {
    return getHandle().getPlayerWeather();
  }
  
  public void resetPlayerWeather()
  {
    getHandle().resetPlayerWeather();
  }
  
  public boolean isBanned()
  {
    return server.getHandle().getNameBans().isBanned(getName().toLowerCase());
  }
  
  public void setBanned(boolean value)
  {
    if (value) {
      net.minecraft.server.v1_7_R1.BanEntry entry = new net.minecraft.server.v1_7_R1.BanEntry(getName().toLowerCase());
      server.getHandle().getNameBans().add(entry);
    } else {
      server.getHandle().getNameBans().remove(getName().toLowerCase());
    }
    
    server.getHandle().getNameBans().save();
  }
  
  public boolean isWhitelisted()
  {
    return server.getHandle().getWhitelisted().contains(getName().toLowerCase());
  }
  
  public void setWhitelisted(boolean value)
  {
    if (value) {
      server.getHandle().addWhitelist(getName().toLowerCase());
    } else {
      server.getHandle().removeWhitelist(getName().toLowerCase());
    }
  }
  
  public void setGameMode(GameMode mode)
  {
    if (getHandle()playerConnection == null) { return;
    }
    if (mode == null) {
      throw new IllegalArgumentException("Mode cannot be null");
    }
    
    if (mode != getGameMode()) {
      PlayerGameModeChangeEvent event = new PlayerGameModeChangeEvent(this, mode);
      server.getPluginManager().callEvent(event);
      if (event.isCancelled()) {
        return;
      }
      
      getHandle()playerInteractManager.setGameMode(net.minecraft.server.v1_7_R1.EnumGamemode.a(mode.getValue()));
      getHandle()fallDistance = 0.0F;
      getHandle()playerConnection.sendPacket(new net.minecraft.server.v1_7_R1.PacketPlayOutGameStateChange(3, mode.getValue()));
    }
  }
  
  public GameMode getGameMode()
  {
    return GameMode.getByValue(getHandle()playerInteractManager.getGameMode().a());
  }
  
  public void giveExp(int exp) {
    getHandle().giveExp(exp);
  }
  
  public void giveExpLevels(int levels) {
    getHandle().levelDown(levels);
  }
  
  public float getExp() {
    return getHandle()exp;
  }
  
  public void setExp(float exp) {
    getHandle()exp = exp;
    getHandle()lastSentExp = -1;
  }
  
  public int getLevel() {
    return getHandle()expLevel;
  }
  
  public void setLevel(int level) {
    getHandle()expLevel = level;
    getHandle()lastSentExp = -1;
  }
  
  public int getTotalExperience() {
    return getHandle()expTotal;
  }
  
  public void setTotalExperience(int exp) {
    getHandle()expTotal = exp;
  }
  
  public float getExhaustion() {
    return getHandle()getFoodData()exhaustionLevel;
  }
  
  public void setExhaustion(float value) {
    getHandle()getFoodData()exhaustionLevel = value;
  }
  
  public float getSaturation() {
    return getHandle()getFoodData()saturationLevel;
  }
  
  public void setSaturation(float value) {
    getHandle()getFoodData()saturationLevel = value;
  }
  
  public int getFoodLevel() {
    return getHandle()getFoodData()foodLevel;
  }
  
  public void setFoodLevel(int value) {
    getHandle()getFoodData()foodLevel = value;
  }
  
  public Location getBedSpawnLocation() {
    org.bukkit.World world = getServer().getWorld(getHandle()spawnWorld);
    ChunkCoordinates bed = getHandle().getBed();
    
    if ((world != null) && (bed != null)) {
      bed = net.minecraft.server.v1_7_R1.EntityHuman.getBed(((CraftWorld)world).getHandle(), bed, getHandle().isRespawnForced());
      if (bed != null) {
        return new Location(world, x, y, z);
      }
    }
    return null;
  }
  
  public void setBedSpawnLocation(Location location) {
    setBedSpawnLocation(location, false);
  }
  
  public void setBedSpawnLocation(Location location, boolean override) {
    if (location == null) {
      getHandle().setRespawnPosition(null, override);
    } else {
      getHandle().setRespawnPosition(new ChunkCoordinates(location.getBlockX(), location.getBlockY(), location.getBlockZ()), override);
      getHandle()spawnWorld = location.getWorld().getName();
    }
  }
  
  public void hidePlayer(Player player) {
    Validate.notNull(player, "hidden player cannot be null");
    if (getHandle()playerConnection == null) return;
    if (equals(player)) return;
    if (hiddenPlayers.containsKey(player.getName())) return;
    hiddenPlayers.put(player.getName(), player);
    

    EntityTracker tracker = entity.world).tracker;
    EntityPlayer other = ((CraftPlayer)player).getHandle();
    EntityTrackerEntry entry = (EntityTrackerEntry)trackedEntities.get(other.getId());
    if (entry != null) {
      entry.clear(getHandle());
    }
    

    getHandle()playerConnection.sendPacket(new PacketPlayOutPlayerInfo(player.getPlayerListName(), false, 9999));
  }
  
  public void showPlayer(Player player) {
    Validate.notNull(player, "shown player cannot be null");
    if (getHandle()playerConnection == null) return;
    if (equals(player)) return;
    if (!hiddenPlayers.containsKey(player.getName())) return;
    hiddenPlayers.remove(player.getName());
    
    EntityTracker tracker = entity.world).tracker;
    EntityPlayer other = ((CraftPlayer)player).getHandle();
    EntityTrackerEntry entry = (EntityTrackerEntry)trackedEntities.get(other.getId());
    if ((entry != null) && (!trackedPlayers.contains(getHandle()))) {
      entry.updatePlayer(getHandle());
    }
    
    getHandle()playerConnection.sendPacket(new PacketPlayOutPlayerInfo(player.getPlayerListName(), true, getHandle()ping));
  }
  
  public boolean canSee(Player player) {
    return !hiddenPlayers.containsKey(player.getName());
  }
  
  public Map<String, Object> serialize() {
    Map<String, Object> result = new java.util.LinkedHashMap();
    
    result.put("name", getName());
    
    return result;
  }
  
  public Player getPlayer() {
    return this;
  }
  
  public EntityPlayer getHandle()
  {
    return (EntityPlayer)entity;
  }
  
  public void setHandle(EntityPlayer entity) {
    super.setHandle(entity);
  }
  
  public String toString()
  {
    return "CraftPlayer{name=" + getName() + '}';
  }
  
  public int hashCode()
  {
    if ((hash == 0) || (hash == 485)) {
      hash = (485 + (getName() != null ? getName().toLowerCase().hashCode() : 0));
    }
    return hash;
  }
  
  public long getFirstPlayed() {
    return firstPlayed;
  }
  
  public long getLastPlayed() {
    return lastPlayed;
  }
  
  public boolean hasPlayedBefore() {
    return hasPlayedBefore;
  }
  
  public void setFirstPlayed(long firstPlayed) {
    this.firstPlayed = firstPlayed;
  }
  
  public void readExtraData(NBTTagCompound nbttagcompound) {
    hasPlayedBefore = true;
    if (nbttagcompound.hasKey("bukkit")) {
      NBTTagCompound data = nbttagcompound.getCompound("bukkit");
      
      if (data.hasKey("firstPlayed")) {
        firstPlayed = data.getLong("firstPlayed");
        lastPlayed = data.getLong("lastPlayed");
      }
      
      if (data.hasKey("newExp")) {
        EntityPlayer handle = getHandle();
        newExp = data.getInt("newExp");
        newTotalExp = data.getInt("newTotalExp");
        newLevel = data.getInt("newLevel");
        expToDrop = data.getInt("expToDrop");
        keepLevel = data.getBoolean("keepLevel");
      }
    }
  }
  
  public void setExtraData(NBTTagCompound nbttagcompound) {
    if (!nbttagcompound.hasKey("bukkit")) {
      nbttagcompound.set("bukkit", new NBTTagCompound());
    }
    
    NBTTagCompound data = nbttagcompound.getCompound("bukkit");
    EntityPlayer handle = getHandle();
    data.setInt("newExp", newExp);
    data.setInt("newTotalExp", newTotalExp);
    data.setInt("newLevel", newLevel);
    data.setInt("expToDrop", expToDrop);
    data.setBoolean("keepLevel", keepLevel);
    data.setLong("firstPlayed", getFirstPlayed());
    data.setLong("lastPlayed", System.currentTimeMillis());
  }
  
  public boolean beginConversation(Conversation conversation) {
    return conversationTracker.beginConversation(conversation);
  }
  
  public void abandonConversation(Conversation conversation) {
    conversationTracker.abandonConversation(conversation, new ConversationAbandonedEvent(conversation, new org.bukkit.conversations.ManuallyAbandonedConversationCanceller()));
  }
  
  public void abandonConversation(Conversation conversation, ConversationAbandonedEvent details) {
    conversationTracker.abandonConversation(conversation, details);
  }
  
  public void acceptConversationInput(String input) {
    conversationTracker.acceptConversationInput(input);
  }
  
  public boolean isConversing() {
    return conversationTracker.isConversing();
  }
  
  public void sendPluginMessage(Plugin source, String channel, byte[] message) {
    org.bukkit.plugin.messaging.StandardMessenger.validatePluginMessage(server.getMessenger(), source, channel, message);
    if (getHandle()playerConnection == null) { return;
    }
    if (channels.contains(channel)) {
      PacketPlayOutCustomPayload packet = new PacketPlayOutCustomPayload(channel, message);
      getHandle()playerConnection.sendPacket(packet);
    }
  }
  
  public void setTexturePack(String url) {
    setResourcePack(url);
  }
  
  public void setResourcePack(String url)
  {
    Validate.notNull(url, "Resource pack URL cannot be null");
    
    getHandle().a(url);
  }
  
  public void addChannel(String channel) {
    if (channels.add(channel)) {
      server.getPluginManager().callEvent(new org.bukkit.event.player.PlayerRegisterChannelEvent(this, channel));
    }
  }
  
  public void removeChannel(String channel) {
    if (channels.remove(channel)) {
      server.getPluginManager().callEvent(new org.bukkit.event.player.PlayerUnregisterChannelEvent(this, channel));
    }
  }
  
  public Set<String> getListeningPluginChannels() {
    return com.google.common.collect.ImmutableSet.copyOf(channels);
  }
  
  public void sendSupportedChannels() {
    if (getHandle()playerConnection == null) return;
    Set<String> listening = server.getMessenger().getIncomingChannels();
    
    if (!listening.isEmpty()) {
      ByteArrayOutputStream stream = new ByteArrayOutputStream();
      
      for (String channel : listening) {
        try {
          stream.write(channel.getBytes("UTF8"));
          stream.write(0);
        } catch (java.io.IOException ex) {
          Logger.getLogger(CraftPlayer.class.getName()).log(java.util.logging.Level.SEVERE, "Could not send Plugin Channel REGISTER to " + getName(), ex);
        }
      }
      
      getHandle()playerConnection.sendPacket(new PacketPlayOutCustomPayload("REGISTER", stream.toByteArray()));
    }
  }
  
  public EntityType getType()
  {
    return EntityType.PLAYER;
  }
  
  public void setMetadata(String metadataKey, MetadataValue newMetadataValue)
  {
    server.getPlayerMetadata().setMetadata(this, metadataKey, newMetadataValue);
  }
  
  public List<MetadataValue> getMetadata(String metadataKey)
  {
    return server.getPlayerMetadata().getMetadata(this, metadataKey);
  }
  
  public boolean hasMetadata(String metadataKey)
  {
    return server.getPlayerMetadata().hasMetadata(this, metadataKey);
  }
  
  public void removeMetadata(String metadataKey, Plugin owningPlugin)
  {
    server.getPlayerMetadata().removeMetadata(this, metadataKey, owningPlugin);
  }
  
  public boolean setWindowProperty(InventoryView.Property prop, int value)
  {
    net.minecraft.server.v1_7_R1.Container container = getHandle()activeContainer;
    if (container.getBukkitView().getType() != prop.getType()) {
      return false;
    }
    getHandle().setContainerData(container, prop.getId(), value);
    return true;
  }
  
  public void disconnect(String reason) {
    conversationTracker.abandonAllConversations();
    perm.clearPermissions();
  }
  
  public boolean isFlying() {
    return getHandle()abilities.isFlying;
  }
  
  public void setFlying(boolean value) {
    if ((!getAllowFlight()) && (value)) {
      throw new IllegalArgumentException("Cannot make player fly if getAllowFlight() is false");
    }
    
    getHandle()abilities.isFlying = value;
    getHandle().updateAbilities();
  }
  
  public boolean getAllowFlight() {
    return getHandle()abilities.canFly;
  }
  
  public void setAllowFlight(boolean value) {
    if ((isFlying()) && (!value)) {
      getHandle()abilities.isFlying = false;
    }
    
    getHandle()abilities.canFly = value;
    getHandle().updateAbilities();
  }
  
  public int getNoDamageTicks()
  {
    if (getHandle()invulnerableTicks > 0) {
      return Math.max(getHandle()invulnerableTicks, getHandle()noDamageTicks);
    }
    return getHandle()noDamageTicks;
  }
  
  public void setFlySpeed(float value)
  {
    validateSpeed(value);
    EntityPlayer player = getHandle();
    abilities.flySpeed = (value / 2.0F);
    player.updateAbilities();
  }
  
  public void setWalkSpeed(float value)
  {
    validateSpeed(value);
    EntityPlayer player = getHandle();
    abilities.walkSpeed = (value / 2.0F);
    player.updateAbilities();
  }
  
  public float getFlySpeed() {
    return getHandle()abilities.flySpeed * 2.0F;
  }
  
  public float getWalkSpeed() {
    return getHandle()abilities.walkSpeed * 2.0F;
  }
  
  private void validateSpeed(float value) {
    if (value < 0.0F) {
      if (value < -1.0F) {
        throw new IllegalArgumentException(value + " is too low");
      }
    }
    else if (value > 1.0F) {
      throw new IllegalArgumentException(value + " is too high");
    }
  }
  

  public void setMaxHealth(double amount)
  {
    super.setMaxHealth(amount);
    health = Math.min(health, health);
    getHandle().triggerHealthUpdate();
  }
  
  public void resetMaxHealth()
  {
    super.resetMaxHealth();
    getHandle().triggerHealthUpdate();
  }
  
  public org.bukkit.craftbukkit.v1_7_R1.scoreboard.CraftScoreboard getScoreboard() {
    return server.getScoreboardManager().getPlayerBoard(this);
  }
  
  public void setScoreboard(org.bukkit.scoreboard.Scoreboard scoreboard) {
    Validate.notNull(scoreboard, "Scoreboard cannot be null");
    PlayerConnection playerConnection = getHandle()playerConnection;
    if (playerConnection == null) {
      throw new IllegalStateException("Cannot set scoreboard yet");
    }
    if (playerConnection.isDisconnected()) {
      throw new IllegalStateException("Cannot set scoreboard for invalid CraftPlayer");
    }
    
    server.getScoreboardManager().setPlayerBoard(this, scoreboard);
  }
  
  public void setHealthScale(double value) {
    Validate.isTrue((float)value > 0.0F, "Must be greater than 0");
    healthScale = value;
    scaledHealth = true;
    updateScaledHealth();
  }
  
  public double getHealthScale() {
    return healthScale;
  }
  
  public void setHealthScaled(boolean scale) {
    if (scaledHealth != (this.scaledHealth = scale)) {
      updateScaledHealth();
    }
  }
  
  public boolean isHealthScaled() {
    return scaledHealth;
  }
  
  public float getScaledHealth() {
    return (float)(isHealthScaled() ? getHealth() * getHealthScale() / getMaxHealth() : getHealth());
  }
  
  public double getHealth()
  {
    return health;
  }
  
  public void setRealHealth(double health) {
    this.health = health;
  }
  
  public void updateScaledHealth() {
    AttributeMapServer attributemapserver = (AttributeMapServer)getHandle().bc();
    Set set = attributemapserver.b();
    
    injectScaledMaxHealth(set, true);
    
    getHandle().getDataWatcher().watch(6, Float.valueOf(getScaledHealth()));
    getHandle()playerConnection.sendPacket(new net.minecraft.server.v1_7_R1.PacketPlayOutUpdateHealth(getScaledHealth(), getHandle().getFoodData().a(), getHandle().getFoodData().e()));
    getHandle()playerConnection.sendPacket(new net.minecraft.server.v1_7_R1.PacketPlayOutUpdateAttributes(getHandle().getId(), set));
    
    set.clear();
    getHandle()maxHealthCache = getMaxHealth();
  }
  
  public void injectScaledMaxHealth(Collection collection, boolean force) {
    if ((!scaledHealth) && (!force)) {
      return;
    }
    for (Object genericInstance : collection) {
      net.minecraft.server.v1_7_R1.IAttribute attribute = ((AttributeInstance)genericInstance).a();
      if (attribute.a().equals("generic.maxHealth")) {
        collection.remove(genericInstance);
      }
    }
    

    collection.add(new net.minecraft.server.v1_7_R1.AttributeModifiable(getHandle().bc(), new AttributeRanged("generic.maxHealth", scaledHealth ? healthScale : getMaxHealth(), 0.0D, 3.402823466385289E+038D).a("Max Health").a(true)));
  }
}
