package org.bukkit.craftbukkit.v1_7_R1.event;

import java.util.List;
import net.minecraft.server.v1_7_R1.ChunkCoordinates;
import net.minecraft.server.v1_7_R1.Container;
import net.minecraft.server.v1_7_R1.DamageSource;
import net.minecraft.server.v1_7_R1.EntityArrow;
import net.minecraft.server.v1_7_R1.EntityDamageSourceIndirect;
import net.minecraft.server.v1_7_R1.EntityHuman;
import net.minecraft.server.v1_7_R1.EntityInsentient;
import net.minecraft.server.v1_7_R1.EntityItem;
import net.minecraft.server.v1_7_R1.EntityLiving;
import net.minecraft.server.v1_7_R1.EntityPlayer;
import net.minecraft.server.v1_7_R1.EntityPotion;
import net.minecraft.server.v1_7_R1.Explosion;
import net.minecraft.server.v1_7_R1.InventoryCrafting;
import net.minecraft.server.v1_7_R1.Items;
import net.minecraft.server.v1_7_R1.PlayerInventory;
import net.minecraft.server.v1_7_R1.WorldServer;
import org.bukkit.Bukkit;
import org.bukkit.Material;
import org.bukkit.Server;
import org.bukkit.block.BlockState;
import org.bukkit.craftbukkit.v1_7_R1.CraftServer;
import org.bukkit.craftbukkit.v1_7_R1.CraftStatistic;
import org.bukkit.craftbukkit.v1_7_R1.CraftWorld;
import org.bukkit.craftbukkit.v1_7_R1.block.CraftBlockState;
import org.bukkit.craftbukkit.v1_7_R1.entity.CraftEntity;
import org.bukkit.craftbukkit.v1_7_R1.entity.CraftLivingEntity;
import org.bukkit.craftbukkit.v1_7_R1.entity.CraftPlayer;
import org.bukkit.craftbukkit.v1_7_R1.inventory.CraftItemStack;
import org.bukkit.entity.LivingEntity;
import org.bukkit.entity.Player;
import org.bukkit.entity.Projectile;
import org.bukkit.entity.ThrownPotion;
import org.bukkit.event.Event;
import org.bukkit.event.block.Action;
import org.bukkit.event.block.BlockDamageEvent;
import org.bukkit.event.block.BlockFadeEvent;
import org.bukkit.event.block.BlockGrowEvent;
import org.bukkit.event.block.BlockIgniteEvent;
import org.bukkit.event.block.BlockIgniteEvent.IgniteCause;
import org.bukkit.event.block.BlockPlaceEvent;
import org.bukkit.event.block.BlockSpreadEvent;
import org.bukkit.event.entity.CreeperPowerEvent;
import org.bukkit.event.entity.EntityBreakDoorEvent;
import org.bukkit.event.entity.EntityChangeBlockEvent;
import org.bukkit.event.entity.EntityDamageEvent;
import org.bukkit.event.entity.EntityDamageEvent.DamageCause;
import org.bukkit.event.entity.EntityDeathEvent;
import org.bukkit.event.entity.EntityShootBowEvent;
import org.bukkit.event.entity.EntityTameEvent;
import org.bukkit.event.entity.EntityTargetEvent.TargetReason;
import org.bukkit.event.entity.ExpBottleEvent;
import org.bukkit.event.entity.HorseJumpEvent;
import org.bukkit.event.entity.ItemDespawnEvent;
import org.bukkit.event.entity.ItemSpawnEvent;
import org.bukkit.event.entity.PigZapEvent;
import org.bukkit.event.entity.PlayerDeathEvent;
import org.bukkit.event.entity.PlayerLeashEntityEvent;
import org.bukkit.event.entity.PotionSplashEvent;
import org.bukkit.event.entity.ProjectileHitEvent;
import org.bukkit.event.entity.ProjectileLaunchEvent;
import org.bukkit.event.inventory.InventoryOpenEvent;
import org.bukkit.event.inventory.PrepareItemCraftEvent;
import org.bukkit.event.player.PlayerBucketEmptyEvent;
import org.bukkit.event.player.PlayerBucketFillEvent;
import org.bukkit.event.player.PlayerEditBookEvent;
import org.bukkit.event.player.PlayerExpChangeEvent;
import org.bukkit.event.player.PlayerInteractEvent;
import org.bukkit.event.player.PlayerLevelChangeEvent;
import org.bukkit.event.player.PlayerStatisticIncrementEvent;
import org.bukkit.event.player.PlayerUnleashEntityEvent;
import org.bukkit.plugin.PluginManager;

public class CraftEventFactory
{
  public static final DamageSource MELTING = org.bukkit.craftbukkit.v1_7_R1.util.CraftDamageSource.copyOf(DamageSource.BURN);
  public static final DamageSource POISON = org.bukkit.craftbukkit.v1_7_R1.util.CraftDamageSource.copyOf(DamageSource.MAGIC);
  
  private static boolean canBuild(CraftWorld world, Player player, int x, int z)
  {
    WorldServer worldServer = world.getHandle();
    int spawnSize = Bukkit.getServer().getSpawnRadius();
    
    if (getHandle()dimension != 0) return true;
    if (spawnSize <= 0) return true;
    if (((CraftServer)Bukkit.getServer()).getHandle().getOPs().isEmpty()) return true;
    if (player.isOp()) { return true;
    }
    ChunkCoordinates chunkcoordinates = worldServer.getSpawn();
    
    int distanceFromSpawn = Math.max(Math.abs(x - x), Math.abs(z - z));
    return distanceFromSpawn > spawnSize;
  }
  
  public static <T extends Event> T callEvent(T event) {
    Bukkit.getServer().getPluginManager().callEvent(event);
    return event;
  }
  


  public static BlockPlaceEvent callBlockPlaceEvent(net.minecraft.server.v1_7_R1.World world, EntityHuman who, BlockState replacedBlockState, int clickedX, int clickedY, int clickedZ)
  {
    CraftWorld craftWorld = world.getWorld();
    CraftServer craftServer = world.getServer();
    
    Player player = who == null ? null : (Player)who.getBukkitEntity();
    
    org.bukkit.block.Block blockClicked = craftWorld.getBlockAt(clickedX, clickedY, clickedZ);
    org.bukkit.block.Block placedBlock = replacedBlockState.getBlock();
    
    boolean canBuild = canBuild(craftWorld, player, placedBlock.getX(), placedBlock.getZ());
    
    BlockPlaceEvent event = new BlockPlaceEvent(placedBlock, replacedBlockState, blockClicked, player.getItemInHand(), player, canBuild);
    craftServer.getPluginManager().callEvent(event);
    
    return event;
  }
  


  public static PlayerBucketEmptyEvent callPlayerBucketEmptyEvent(EntityHuman who, int clickedX, int clickedY, int clickedZ, int clickedFace, net.minecraft.server.v1_7_R1.ItemStack itemInHand)
  {
    return (PlayerBucketEmptyEvent)getPlayerBucketEvent(false, who, clickedX, clickedY, clickedZ, clickedFace, itemInHand, Items.BUCKET);
  }
  
  public static PlayerBucketFillEvent callPlayerBucketFillEvent(EntityHuman who, int clickedX, int clickedY, int clickedZ, int clickedFace, net.minecraft.server.v1_7_R1.ItemStack itemInHand, net.minecraft.server.v1_7_R1.Item bucket) {
    return (PlayerBucketFillEvent)getPlayerBucketEvent(true, who, clickedX, clickedY, clickedZ, clickedFace, itemInHand, bucket);
  }
  
  private static org.bukkit.event.player.PlayerEvent getPlayerBucketEvent(boolean isFilling, EntityHuman who, int clickedX, int clickedY, int clickedZ, int clickedFace, net.minecraft.server.v1_7_R1.ItemStack itemstack, net.minecraft.server.v1_7_R1.Item item) {
    Player player = who == null ? null : (Player)who.getBukkitEntity();
    CraftItemStack itemInHand = CraftItemStack.asNewCraftStack(item);
    Material bucket = org.bukkit.craftbukkit.v1_7_R1.util.CraftMagicNumbers.getMaterial(itemstack.getItem());
    
    CraftWorld craftWorld = (CraftWorld)player.getWorld();
    CraftServer craftServer = (CraftServer)player.getServer();
    
    org.bukkit.block.Block blockClicked = craftWorld.getBlockAt(clickedX, clickedY, clickedZ);
    org.bukkit.block.BlockFace blockFace = org.bukkit.craftbukkit.v1_7_R1.block.CraftBlock.notchToBlockFace(clickedFace);
    
    org.bukkit.event.player.PlayerEvent event = null;
    if (isFilling) {
      event = new PlayerBucketFillEvent(player, blockClicked, blockFace, bucket, itemInHand);
      ((PlayerBucketFillEvent)event).setCancelled(!canBuild(craftWorld, player, clickedX, clickedZ));
    } else {
      event = new PlayerBucketEmptyEvent(player, blockClicked, blockFace, bucket, itemInHand);
      ((PlayerBucketEmptyEvent)event).setCancelled(!canBuild(craftWorld, player, clickedX, clickedZ));
    }
    
    craftServer.getPluginManager().callEvent(event);
    
    return event;
  }
  


  public static PlayerInteractEvent callPlayerInteractEvent(EntityHuman who, Action action, net.minecraft.server.v1_7_R1.ItemStack itemstack)
  {
    if ((action != Action.LEFT_CLICK_AIR) && (action != Action.RIGHT_CLICK_AIR)) {
      throw new IllegalArgumentException();
    }
    return callPlayerInteractEvent(who, action, 0, 256, 0, 0, itemstack);
  }
  
  public static PlayerInteractEvent callPlayerInteractEvent(EntityHuman who, Action action, int clickedX, int clickedY, int clickedZ, int clickedFace, net.minecraft.server.v1_7_R1.ItemStack itemstack) {
    Player player = who == null ? null : (Player)who.getBukkitEntity();
    CraftItemStack itemInHand = CraftItemStack.asCraftMirror(itemstack);
    
    CraftWorld craftWorld = (CraftWorld)player.getWorld();
    CraftServer craftServer = (CraftServer)player.getServer();
    
    org.bukkit.block.Block blockClicked = craftWorld.getBlockAt(clickedX, clickedY, clickedZ);
    org.bukkit.block.BlockFace blockFace = org.bukkit.craftbukkit.v1_7_R1.block.CraftBlock.notchToBlockFace(clickedFace);
    
    if (clickedY > 255) {
      blockClicked = null;
      switch (action) {
      case LEFT_CLICK_BLOCK: 
        action = Action.LEFT_CLICK_AIR;
        break;
      case RIGHT_CLICK_BLOCK: 
        action = Action.RIGHT_CLICK_AIR;
      }
      
    }
    
    if ((itemInHand.getType() == Material.AIR) || (itemInHand.getAmount() == 0)) {
      itemInHand = null;
    }
    
    PlayerInteractEvent event = new PlayerInteractEvent(player, action, itemInHand, blockClicked, blockFace);
    craftServer.getPluginManager().callEvent(event);
    
    return event;
  }
  


  public static EntityShootBowEvent callEntityShootBowEvent(EntityLiving who, net.minecraft.server.v1_7_R1.ItemStack itemstack, EntityArrow entityArrow, float force)
  {
    LivingEntity shooter = (LivingEntity)who.getBukkitEntity();
    CraftItemStack itemInHand = CraftItemStack.asCraftMirror(itemstack);
    org.bukkit.entity.Arrow arrow = (org.bukkit.entity.Arrow)entityArrow.getBukkitEntity();
    
    if ((itemInHand != null) && ((itemInHand.getType() == Material.AIR) || (itemInHand.getAmount() == 0))) {
      itemInHand = null;
    }
    
    EntityShootBowEvent event = new EntityShootBowEvent(shooter, itemInHand, arrow, force);
    Bukkit.getPluginManager().callEvent(event);
    
    return event;
  }
  


  public static BlockDamageEvent callBlockDamageEvent(EntityHuman who, int x, int y, int z, net.minecraft.server.v1_7_R1.ItemStack itemstack, boolean instaBreak)
  {
    Player player = who == null ? null : (Player)who.getBukkitEntity();
    CraftItemStack itemInHand = CraftItemStack.asCraftMirror(itemstack);
    
    CraftWorld craftWorld = (CraftWorld)player.getWorld();
    CraftServer craftServer = (CraftServer)player.getServer();
    
    org.bukkit.block.Block blockClicked = craftWorld.getBlockAt(x, y, z);
    
    BlockDamageEvent event = new BlockDamageEvent(player, blockClicked, itemInHand, instaBreak);
    craftServer.getPluginManager().callEvent(event);
    
    return event;
  }
  


  public static org.bukkit.event.entity.CreatureSpawnEvent callCreatureSpawnEvent(EntityLiving entityliving, org.bukkit.event.entity.CreatureSpawnEvent.SpawnReason spawnReason)
  {
    LivingEntity entity = (LivingEntity)entityliving.getBukkitEntity();
    CraftServer craftServer = (CraftServer)entity.getServer();
    
    org.bukkit.event.entity.CreatureSpawnEvent event = new org.bukkit.event.entity.CreatureSpawnEvent(entity, spawnReason);
    craftServer.getPluginManager().callEvent(event);
    return event;
  }
  


  public static EntityTameEvent callEntityTameEvent(EntityInsentient entity, EntityHuman tamer)
  {
    org.bukkit.entity.Entity bukkitEntity = entity.getBukkitEntity();
    org.bukkit.entity.AnimalTamer bukkitTamer = tamer != null ? tamer.getBukkitEntity() : null;
    CraftServer craftServer = (CraftServer)bukkitEntity.getServer();
    
    persistent = true;
    
    EntityTameEvent event = new EntityTameEvent((LivingEntity)bukkitEntity, bukkitTamer);
    craftServer.getPluginManager().callEvent(event);
    return event;
  }
  


  public static ItemSpawnEvent callItemSpawnEvent(EntityItem entityitem)
  {
    org.bukkit.entity.Item entity = (org.bukkit.entity.Item)entityitem.getBukkitEntity();
    CraftServer craftServer = (CraftServer)entity.getServer();
    
    ItemSpawnEvent event = new ItemSpawnEvent(entity, entity.getLocation());
    
    craftServer.getPluginManager().callEvent(event);
    return event;
  }
  


  public static ItemDespawnEvent callItemDespawnEvent(EntityItem entityitem)
  {
    org.bukkit.entity.Item entity = (org.bukkit.entity.Item)entityitem.getBukkitEntity();
    
    ItemDespawnEvent event = new ItemDespawnEvent(entity, entity.getLocation());
    
    entity.getServer().getPluginManager().callEvent(event);
    return event;
  }
  


  public static PotionSplashEvent callPotionSplashEvent(EntityPotion potion, java.util.Map<LivingEntity, Double> affectedEntities)
  {
    ThrownPotion thrownPotion = (ThrownPotion)potion.getBukkitEntity();
    
    PotionSplashEvent event = new PotionSplashEvent(thrownPotion, affectedEntities);
    Bukkit.getPluginManager().callEvent(event);
    return event;
  }
  


  public static BlockFadeEvent callBlockFadeEvent(org.bukkit.block.Block block, net.minecraft.server.v1_7_R1.Block type)
  {
    BlockState state = block.getState();
    state.setTypeId(net.minecraft.server.v1_7_R1.Block.b(type));
    
    BlockFadeEvent event = new BlockFadeEvent(block, state);
    Bukkit.getPluginManager().callEvent(event);
    return event;
  }
  
  public static void handleBlockSpreadEvent(org.bukkit.block.Block block, org.bukkit.block.Block source, net.minecraft.server.v1_7_R1.Block type, int data) {
    BlockState state = block.getState();
    state.setTypeId(net.minecraft.server.v1_7_R1.Block.b(type));
    state.setRawData((byte)data);
    
    BlockSpreadEvent event = new BlockSpreadEvent(block, source, state);
    Bukkit.getPluginManager().callEvent(event);
    
    if (!event.isCancelled()) {
      state.update(true);
    }
  }
  
  public static EntityDeathEvent callEntityDeathEvent(EntityLiving victim) {
    return callEntityDeathEvent(victim, new java.util.ArrayList(0));
  }
  
  public static EntityDeathEvent callEntityDeathEvent(EntityLiving victim, List<org.bukkit.inventory.ItemStack> drops) {
    CraftLivingEntity entity = (CraftLivingEntity)victim.getBukkitEntity();
    EntityDeathEvent event = new EntityDeathEvent(entity, drops, victim.getExpReward());
    CraftWorld world = (CraftWorld)entity.getWorld();
    Bukkit.getServer().getPluginManager().callEvent(event);
    
    expToDrop = event.getDroppedExp();
    
    for (org.bukkit.inventory.ItemStack stack : event.getDrops()) {
      if ((stack != null) && (stack.getType() != Material.AIR) && (stack.getAmount() != 0))
      {
        world.dropItemNaturally(entity.getLocation(), stack);
      }
    }
    return event;
  }
  
  public static PlayerDeathEvent callPlayerDeathEvent(EntityPlayer victim, List<org.bukkit.inventory.ItemStack> drops, String deathMessage) {
    CraftPlayer entity = victim.getBukkitEntity();
    PlayerDeathEvent event = new PlayerDeathEvent(entity, drops, victim.getExpReward(), 0, deathMessage);
    org.bukkit.World world = entity.getWorld();
    Bukkit.getServer().getPluginManager().callEvent(event);
    
    keepLevel = event.getKeepLevel();
    newLevel = event.getNewLevel();
    newTotalExp = event.getNewTotalExp();
    expToDrop = event.getDroppedExp();
    newExp = event.getNewExp();
    
    for (org.bukkit.inventory.ItemStack stack : event.getDrops()) {
      if ((stack != null) && (stack.getType() != Material.AIR))
      {
        world.dropItemNaturally(entity.getLocation(), stack);
      }
    }
    return event;
  }
  


  public static org.bukkit.event.server.ServerListPingEvent callServerListPingEvent(Server craftServer, java.net.InetAddress address, String motd, int numPlayers, int maxPlayers)
  {
    org.bukkit.event.server.ServerListPingEvent event = new org.bukkit.event.server.ServerListPingEvent(address, motd, numPlayers, maxPlayers);
    craftServer.getPluginManager().callEvent(event);
    return event;
  }
  

  public static EntityDamageEvent callEntityDamageEvent(net.minecraft.server.v1_7_R1.Entity damager, net.minecraft.server.v1_7_R1.Entity damagee, EntityDamageEvent.DamageCause cause, double damage)
  {
    EntityDamageEvent event;
    EntityDamageEvent event;
    if (damager != null) {
      event = new org.bukkit.event.entity.EntityDamageByEntityEvent(damager.getBukkitEntity(), damagee.getBukkitEntity(), cause, damage);
    } else {
      event = new EntityDamageEvent(damagee.getBukkitEntity(), cause, damage);
    }
    
    callEvent(event);
    
    if (!event.isCancelled()) {
      event.getEntity().setLastDamageCause(event);
    }
    
    return event;
  }
  
  public static EntityDamageEvent handleEntityDamageEvent(net.minecraft.server.v1_7_R1.Entity entity, DamageSource source, float damage)
  {
    if (source.c())
      return null;
    if ((source instanceof net.minecraft.server.v1_7_R1.EntityDamageSource)) {
      net.minecraft.server.v1_7_R1.Entity damager = source.getEntity();
      EntityDamageEvent.DamageCause cause = EntityDamageEvent.DamageCause.ENTITY_ATTACK;
      
      if ((source instanceof EntityDamageSourceIndirect)) {
        damager = ((EntityDamageSourceIndirect)source).getProximateDamageSource();
        if ((damager.getBukkitEntity() instanceof ThrownPotion)) {
          cause = EntityDamageEvent.DamageCause.MAGIC;
        } else if ((damager.getBukkitEntity() instanceof Projectile)) {
          cause = EntityDamageEvent.DamageCause.PROJECTILE;
        }
      } else if ("thorns".equals(translationIndex)) {
        cause = EntityDamageEvent.DamageCause.THORNS;
      }
      
      return callEntityDamageEvent(damager, entity, cause, damage); }
    if (source == DamageSource.OUT_OF_WORLD) {
      EntityDamageEvent event = (EntityDamageEvent)callEvent(new org.bukkit.event.entity.EntityDamageByBlockEvent(null, entity.getBukkitEntity(), EntityDamageEvent.DamageCause.VOID, damage));
      if (!event.isCancelled()) {
        event.getEntity().setLastDamageCause(event);
      }
      return event;
    }
    
    EntityDamageEvent.DamageCause cause = null;
    if (source == DamageSource.FIRE) {
      cause = EntityDamageEvent.DamageCause.FIRE;
    } else if (source == DamageSource.STARVE) {
      cause = EntityDamageEvent.DamageCause.STARVATION;
    } else if (source == DamageSource.WITHER) {
      cause = EntityDamageEvent.DamageCause.WITHER;
    } else if (source == DamageSource.STUCK) {
      cause = EntityDamageEvent.DamageCause.SUFFOCATION;
    } else if (source == DamageSource.DROWN) {
      cause = EntityDamageEvent.DamageCause.DROWNING;
    } else if (source == DamageSource.BURN) {
      cause = EntityDamageEvent.DamageCause.FIRE_TICK;
    } else if (source == MELTING) {
      cause = EntityDamageEvent.DamageCause.MELTING;
    } else if (source == POISON) {
      cause = EntityDamageEvent.DamageCause.POISON;
    } else if (source == DamageSource.MAGIC) {
      cause = EntityDamageEvent.DamageCause.MAGIC;
    }
    
    if (cause != null) {
      return callEntityDamageEvent(null, entity, cause, damage);
    }
    


    return null;
  }
  
  public static boolean handleNonLivingEntityDamageEvent(net.minecraft.server.v1_7_R1.Entity entity, DamageSource source, float damage)
  {
    if (!(source instanceof net.minecraft.server.v1_7_R1.EntityDamageSource)) {
      return false;
    }
    EntityDamageEvent event = handleEntityDamageEvent(entity, source, damage);
    if (event == null) {
      return false;
    }
    return (event.isCancelled()) || (event.getDamage() == 0.0D);
  }
  
  public static PlayerLevelChangeEvent callPlayerLevelChangeEvent(Player player, int oldLevel, int newLevel) {
    PlayerLevelChangeEvent event = new PlayerLevelChangeEvent(player, oldLevel, newLevel);
    Bukkit.getPluginManager().callEvent(event);
    return event;
  }
  
  public static PlayerExpChangeEvent callPlayerExpChangeEvent(EntityHuman entity, int expAmount) {
    Player player = (Player)entity.getBukkitEntity();
    PlayerExpChangeEvent event = new PlayerExpChangeEvent(player, expAmount);
    Bukkit.getPluginManager().callEvent(event);
    return event;
  }
  
  public static void handleBlockGrowEvent(net.minecraft.server.v1_7_R1.World world, int x, int y, int z, net.minecraft.server.v1_7_R1.Block type, int data) {
    org.bukkit.block.Block block = world.getWorld().getBlockAt(x, y, z);
    CraftBlockState state = (CraftBlockState)block.getState();
    state.setTypeId(net.minecraft.server.v1_7_R1.Block.b(type));
    state.setRawData((byte)data);
    
    BlockGrowEvent event = new BlockGrowEvent(block, state);
    Bukkit.getPluginManager().callEvent(event);
    
    if (!event.isCancelled()) {
      state.update(true);
    }
  }
  
  public static org.bukkit.event.entity.FoodLevelChangeEvent callFoodLevelChangeEvent(EntityHuman entity, int level) {
    org.bukkit.event.entity.FoodLevelChangeEvent event = new org.bukkit.event.entity.FoodLevelChangeEvent(entity.getBukkitEntity(), level);
    entity.getBukkitEntity().getServer().getPluginManager().callEvent(event);
    return event;
  }
  
  public static PigZapEvent callPigZapEvent(net.minecraft.server.v1_7_R1.Entity pig, net.minecraft.server.v1_7_R1.Entity lightning, net.minecraft.server.v1_7_R1.Entity pigzombie) {
    PigZapEvent event = new PigZapEvent((org.bukkit.entity.Pig)pig.getBukkitEntity(), (org.bukkit.entity.LightningStrike)lightning.getBukkitEntity(), (org.bukkit.entity.PigZombie)pigzombie.getBukkitEntity());
    pig.getBukkitEntity().getServer().getPluginManager().callEvent(event);
    return event;
  }
  
  public static HorseJumpEvent callHorseJumpEvent(net.minecraft.server.v1_7_R1.Entity horse, float power) {
    HorseJumpEvent event = new HorseJumpEvent((org.bukkit.entity.Horse)horse.getBukkitEntity(), power);
    horse.getBukkitEntity().getServer().getPluginManager().callEvent(event);
    return event;
  }
  
  public static EntityChangeBlockEvent callEntityChangeBlockEvent(org.bukkit.entity.Entity entity, org.bukkit.block.Block block, Material material) {
    return callEntityChangeBlockEvent(entity, block, material, 0);
  }
  
  public static EntityChangeBlockEvent callEntityChangeBlockEvent(net.minecraft.server.v1_7_R1.Entity entity, org.bukkit.block.Block block, Material material) {
    return callEntityChangeBlockEvent(entity.getBukkitEntity(), block, material, 0);
  }
  
  public static EntityChangeBlockEvent callEntityChangeBlockEvent(net.minecraft.server.v1_7_R1.Entity entity, org.bukkit.block.Block block, Material material, boolean cancelled) {
    return callEntityChangeBlockEvent(entity.getBukkitEntity(), block, material, 0, cancelled);
  }
  
  public static EntityChangeBlockEvent callEntityChangeBlockEvent(net.minecraft.server.v1_7_R1.Entity entity, int x, int y, int z, net.minecraft.server.v1_7_R1.Block type, int data) {
    org.bukkit.block.Block block = world.getWorld().getBlockAt(x, y, z);
    Material material = org.bukkit.craftbukkit.v1_7_R1.util.CraftMagicNumbers.getMaterial(type);
    
    return callEntityChangeBlockEvent(entity.getBukkitEntity(), block, material, data);
  }
  
  public static EntityChangeBlockEvent callEntityChangeBlockEvent(org.bukkit.entity.Entity entity, org.bukkit.block.Block block, Material material, int data) {
    return callEntityChangeBlockEvent(entity, block, material, data, false);
  }
  
  public static EntityChangeBlockEvent callEntityChangeBlockEvent(org.bukkit.entity.Entity entity, org.bukkit.block.Block block, Material material, int data, boolean cancelled) {
    EntityChangeBlockEvent event = new EntityChangeBlockEvent(entity, block, material, (byte)data);
    event.setCancelled(cancelled);
    entity.getServer().getPluginManager().callEvent(event);
    return event;
  }
  
  public static CreeperPowerEvent callCreeperPowerEvent(net.minecraft.server.v1_7_R1.Entity creeper, net.minecraft.server.v1_7_R1.Entity lightning, org.bukkit.event.entity.CreeperPowerEvent.PowerCause cause) {
    CreeperPowerEvent event = new CreeperPowerEvent((org.bukkit.entity.Creeper)creeper.getBukkitEntity(), (org.bukkit.entity.LightningStrike)lightning.getBukkitEntity(), cause);
    creeper.getBukkitEntity().getServer().getPluginManager().callEvent(event);
    return event;
  }
  
  public static org.bukkit.event.entity.EntityTargetEvent callEntityTargetEvent(net.minecraft.server.v1_7_R1.Entity entity, net.minecraft.server.v1_7_R1.Entity target, EntityTargetEvent.TargetReason reason) {
    org.bukkit.event.entity.EntityTargetEvent event = new org.bukkit.event.entity.EntityTargetEvent(entity.getBukkitEntity(), target == null ? null : target.getBukkitEntity(), reason);
    entity.getBukkitEntity().getServer().getPluginManager().callEvent(event);
    return event;
  }
  
  public static org.bukkit.event.entity.EntityTargetLivingEntityEvent callEntityTargetLivingEvent(net.minecraft.server.v1_7_R1.Entity entity, EntityLiving target, EntityTargetEvent.TargetReason reason) {
    org.bukkit.event.entity.EntityTargetLivingEntityEvent event = new org.bukkit.event.entity.EntityTargetLivingEntityEvent(entity.getBukkitEntity(), (LivingEntity)target.getBukkitEntity(), reason);
    entity.getBukkitEntity().getServer().getPluginManager().callEvent(event);
    return event;
  }
  
  public static EntityBreakDoorEvent callEntityBreakDoorEvent(net.minecraft.server.v1_7_R1.Entity entity, int x, int y, int z) {
    org.bukkit.entity.Entity entity1 = entity.getBukkitEntity();
    org.bukkit.block.Block block = entity1.getWorld().getBlockAt(x, y, z);
    
    EntityBreakDoorEvent event = new EntityBreakDoorEvent((LivingEntity)entity1, block);
    entity1.getServer().getPluginManager().callEvent(event);
    
    return event;
  }
  
  public static Container callInventoryOpenEvent(EntityPlayer player, Container container) {
    if (activeContainer != defaultContainer) {
      playerConnection.a(new net.minecraft.server.v1_7_R1.PacketPlayInCloseWindow(activeContainer.windowId));
    }
    
    CraftServer server = world.getServer();
    CraftPlayer craftPlayer = player.getBukkitEntity();
    activeContainer.transferTo(container, craftPlayer);
    
    InventoryOpenEvent event = new InventoryOpenEvent(container.getBukkitView());
    server.getPluginManager().callEvent(event);
    
    if (event.isCancelled()) {
      container.transferTo(activeContainer, craftPlayer);
      return null;
    }
    
    return container;
  }
  
  public static net.minecraft.server.v1_7_R1.ItemStack callPreCraftEvent(InventoryCrafting matrix, net.minecraft.server.v1_7_R1.ItemStack result, org.bukkit.inventory.InventoryView lastCraftView, boolean isRepair) {
    org.bukkit.craftbukkit.v1_7_R1.inventory.CraftInventoryCrafting inventory = new org.bukkit.craftbukkit.v1_7_R1.inventory.CraftInventoryCrafting(matrix, resultInventory);
    inventory.setResult(CraftItemStack.asCraftMirror(result));
    
    PrepareItemCraftEvent event = new PrepareItemCraftEvent(inventory, lastCraftView, isRepair);
    Bukkit.getPluginManager().callEvent(event);
    
    org.bukkit.inventory.ItemStack bitem = event.getInventory().getResult();
    
    return CraftItemStack.asNMSCopy(bitem);
  }
  
  public static ProjectileLaunchEvent callProjectileLaunchEvent(net.minecraft.server.v1_7_R1.Entity entity) {
    Projectile bukkitEntity = (Projectile)entity.getBukkitEntity();
    ProjectileLaunchEvent event = new ProjectileLaunchEvent(bukkitEntity);
    Bukkit.getPluginManager().callEvent(event);
    return event;
  }
  
  public static ProjectileHitEvent callProjectileHitEvent(net.minecraft.server.v1_7_R1.Entity entity) {
    ProjectileHitEvent event = new ProjectileHitEvent((Projectile)entity.getBukkitEntity());
    world.getServer().getPluginManager().callEvent(event);
    return event;
  }
  
  public static ExpBottleEvent callExpBottleEvent(net.minecraft.server.v1_7_R1.Entity entity, int exp) {
    org.bukkit.entity.ThrownExpBottle bottle = (org.bukkit.entity.ThrownExpBottle)entity.getBukkitEntity();
    ExpBottleEvent event = new ExpBottleEvent(bottle, exp);
    Bukkit.getPluginManager().callEvent(event);
    return event;
  }
  
  public static org.bukkit.event.block.BlockRedstoneEvent callRedstoneChange(net.minecraft.server.v1_7_R1.World world, int x, int y, int z, int oldCurrent, int newCurrent) {
    org.bukkit.event.block.BlockRedstoneEvent event = new org.bukkit.event.block.BlockRedstoneEvent(world.getWorld().getBlockAt(x, y, z), oldCurrent, newCurrent);
    world.getServer().getPluginManager().callEvent(event);
    return event;
  }
  
  public static org.bukkit.event.block.NotePlayEvent callNotePlayEvent(net.minecraft.server.v1_7_R1.World world, int x, int y, int z, byte instrument, byte note) {
    org.bukkit.event.block.NotePlayEvent event = new org.bukkit.event.block.NotePlayEvent(world.getWorld().getBlockAt(x, y, z), org.bukkit.Instrument.getByType(instrument), new org.bukkit.Note(note));
    world.getServer().getPluginManager().callEvent(event);
    return event;
  }
  
  public static void callPlayerItemBreakEvent(EntityHuman human, net.minecraft.server.v1_7_R1.ItemStack brokenItem) {
    CraftItemStack item = CraftItemStack.asCraftMirror(brokenItem);
    org.bukkit.event.player.PlayerItemBreakEvent event = new org.bukkit.event.player.PlayerItemBreakEvent((Player)human.getBukkitEntity(), item);
    Bukkit.getPluginManager().callEvent(event);
  }
  
  public static BlockIgniteEvent callBlockIgniteEvent(net.minecraft.server.v1_7_R1.World world, int x, int y, int z, int igniterX, int igniterY, int igniterZ) {
    org.bukkit.World bukkitWorld = world.getWorld();
    org.bukkit.block.Block igniter = bukkitWorld.getBlockAt(igniterX, igniterY, igniterZ);
    BlockIgniteEvent.IgniteCause cause;
    switch (1.$SwitchMap$org$bukkit$Material[igniter.getType().ordinal()]) {
    case 1: 
    case 2: 
      cause = BlockIgniteEvent.IgniteCause.LAVA;
      break;
    case 3: 
      cause = BlockIgniteEvent.IgniteCause.FLINT_AND_STEEL;
      break;
    case 4: 
    default: 
      cause = BlockIgniteEvent.IgniteCause.SPREAD;
    }
    
    BlockIgniteEvent event = new BlockIgniteEvent(bukkitWorld.getBlockAt(x, y, z), cause, igniter);
    world.getServer().getPluginManager().callEvent(event);
    return event;
  }
  
  public static BlockIgniteEvent callBlockIgniteEvent(net.minecraft.server.v1_7_R1.World world, int x, int y, int z, net.minecraft.server.v1_7_R1.Entity igniter) {
    org.bukkit.World bukkitWorld = world.getWorld();
    org.bukkit.entity.Entity bukkitIgniter = igniter.getBukkitEntity();
    BlockIgniteEvent.IgniteCause cause;
    switch (1.$SwitchMap$org$bukkit$entity$EntityType[bukkitIgniter.getType().ordinal()]) {
    case 1: 
      cause = BlockIgniteEvent.IgniteCause.ENDER_CRYSTAL;
      break;
    case 2: 
      cause = BlockIgniteEvent.IgniteCause.LIGHTNING;
      break;
    case 3: 
    case 4: 
      cause = BlockIgniteEvent.IgniteCause.FIREBALL;
      break;
    default: 
      cause = BlockIgniteEvent.IgniteCause.FLINT_AND_STEEL;
    }
    
    BlockIgniteEvent event = new BlockIgniteEvent(bukkitWorld.getBlockAt(x, y, z), cause, bukkitIgniter);
    world.getServer().getPluginManager().callEvent(event);
    return event;
  }
  
  public static BlockIgniteEvent callBlockIgniteEvent(net.minecraft.server.v1_7_R1.World world, int x, int y, int z, Explosion explosion) {
    org.bukkit.World bukkitWorld = world.getWorld();
    org.bukkit.entity.Entity igniter = source == null ? null : source.getBukkitEntity();
    
    BlockIgniteEvent event = new BlockIgniteEvent(bukkitWorld.getBlockAt(x, y, z), BlockIgniteEvent.IgniteCause.EXPLOSION, igniter);
    world.getServer().getPluginManager().callEvent(event);
    return event;
  }
  
  public static BlockIgniteEvent callBlockIgniteEvent(net.minecraft.server.v1_7_R1.World world, int x, int y, int z, BlockIgniteEvent.IgniteCause cause, net.minecraft.server.v1_7_R1.Entity igniter) {
    BlockIgniteEvent event = new BlockIgniteEvent(world.getWorld().getBlockAt(x, y, z), cause, igniter.getBukkitEntity());
    world.getServer().getPluginManager().callEvent(event);
    return event;
  }
  
  public static void handleInventoryCloseEvent(EntityHuman human) {
    org.bukkit.event.inventory.InventoryCloseEvent event = new org.bukkit.event.inventory.InventoryCloseEvent(activeContainer.getBukkitView());
    world.getServer().getPluginManager().callEvent(event);
    activeContainer.transferTo(defaultContainer, human.getBukkitEntity());
  }
  
  public static void handleEditBookEvent(EntityPlayer player, net.minecraft.server.v1_7_R1.ItemStack newBookItem) {
    int itemInHandIndex = inventory.itemInHandIndex;
    
    PlayerEditBookEvent editBookEvent = new PlayerEditBookEvent(player.getBukkitEntity(), inventory.itemInHandIndex, (org.bukkit.inventory.meta.BookMeta)CraftItemStack.getItemMeta(inventory.getItemInHand()), (org.bukkit.inventory.meta.BookMeta)CraftItemStack.getItemMeta(newBookItem), newBookItem.getItem() == Items.WRITTEN_BOOK);
    world.getServer().getPluginManager().callEvent(editBookEvent);
    net.minecraft.server.v1_7_R1.ItemStack itemInHand = inventory.getItem(itemInHandIndex);
    

    if (itemInHand.getItem() == Items.BOOK_AND_QUILL) {
      if (!editBookEvent.isCancelled()) {
        CraftItemStack.setItemMeta(itemInHand, editBookEvent.getNewBookMeta());
        if (editBookEvent.isSigning()) {
          itemInHand.setItem(Items.WRITTEN_BOOK);
        }
      }
      

      net.minecraft.server.v1_7_R1.Slot slot = activeContainer.a(inventory, itemInHandIndex);
      playerConnection.sendPacket(new net.minecraft.server.v1_7_R1.PacketPlayOutSetSlot(activeContainer.windowId, rawSlotIndex, itemInHand));
    }
  }
  
  public static PlayerUnleashEntityEvent callPlayerUnleashEntityEvent(EntityInsentient entity, EntityHuman player) {
    PlayerUnleashEntityEvent event = new PlayerUnleashEntityEvent(entity.getBukkitEntity(), (Player)player.getBukkitEntity());
    world.getServer().getPluginManager().callEvent(event);
    return event;
  }
  
  public static PlayerLeashEntityEvent callPlayerLeashEntityEvent(EntityInsentient entity, net.minecraft.server.v1_7_R1.Entity leashHolder, EntityHuman player) {
    PlayerLeashEntityEvent event = new PlayerLeashEntityEvent(entity.getBukkitEntity(), leashHolder.getBukkitEntity(), (Player)player.getBukkitEntity());
    world.getServer().getPluginManager().callEvent(event);
    return event;
  }
  
  public static org.bukkit.event.Cancellable handleStatisticsIncrease(EntityHuman entityHuman, net.minecraft.server.v1_7_R1.Statistic statistic, int current, int incrementation) {
    Player player = ((EntityPlayer)entityHuman).getBukkitEntity();
    Event event;
    Event event; if ((statistic instanceof net.minecraft.server.v1_7_R1.Achievement)) {
      if (current != 0) {
        return null;
      }
      event = new org.bukkit.event.player.PlayerAchievementAwardedEvent(player, CraftStatistic.getBukkitAchievement((net.minecraft.server.v1_7_R1.Achievement)statistic));
    } else {
      org.bukkit.Statistic stat = CraftStatistic.getBukkitStatistic(statistic);
      switch (1.$SwitchMap$org$bukkit$Statistic[stat.ordinal()])
      {
      case 1: 
      case 2: 
      case 3: 
      case 4: 
      case 5: 
      case 6: 
      case 7: 
      case 8: 
      case 9: 
      case 10: 
      case 11: 
        return null;
      }
      Event event;
      if (stat.getType() == org.bukkit.Statistic.Type.UNTYPED) {
        event = new PlayerStatisticIncrementEvent(player, stat, current, current + incrementation); } else { Event event;
        if (stat.getType() == org.bukkit.Statistic.Type.ENTITY) {
          org.bukkit.entity.EntityType entityType = CraftStatistic.getEntityTypeFromStatistic(statistic);
          event = new PlayerStatisticIncrementEvent(player, stat, current, current + incrementation, entityType);
        } else {
          Material material = CraftStatistic.getMaterialFromStatistic(statistic);
          event = new PlayerStatisticIncrementEvent(player, stat, current, current + incrementation, material);
        }
      } }
    world.getServer().getPluginManager().callEvent(event);
    return (org.bukkit.event.Cancellable)event;
  }
}
