package org.bukkit.craftbukkit.v1_7_R1.help;

import com.google.common.base.Predicates;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import org.bukkit.command.Command;
import org.bukkit.command.PluginCommand;
import org.bukkit.command.PluginIdentifiableCommand;
import org.bukkit.command.SimpleCommandMap;
import org.bukkit.craftbukkit.v1_7_R1.CraftServer;
import org.bukkit.help.HelpTopic;
import org.bukkit.help.HelpTopicFactory;

public class SimpleHelpMap implements org.bukkit.help.HelpMap
{
  private HelpTopic defaultTopic;
  private final Map<String, HelpTopic> helpTopics;
  private final Map<Class, HelpTopicFactory<Command>> topicFactoryMap;
  private final CraftServer server;
  private HelpYamlReader yaml;
  
  public SimpleHelpMap(CraftServer server)
  {
    helpTopics = new java.util.TreeMap(org.bukkit.help.HelpTopicComparator.topicNameComparatorInstance());
    topicFactoryMap = new java.util.HashMap();
    this.server = server;
    yaml = new HelpYamlReader(server);
    
    com.google.common.base.Predicate indexFilter = Predicates.not(Predicates.instanceOf(CommandAliasHelpTopic.class));
    if (!yaml.commandTopicsInMasterIndex()) {
      indexFilter = Predicates.and(indexFilter, Predicates.not(new IsCommandTopicPredicate(null)));
    }
    
    defaultTopic = new org.bukkit.help.IndexHelpTopic("Index", null, null, com.google.common.collect.Collections2.filter(helpTopics.values(), indexFilter), "Use /help [n] to get page n of help.");
    
    registerHelpTopicFactory(org.bukkit.command.MultipleCommandAlias.class, new MultipleCommandAliasHelpTopicFactory());
  }
  
  public synchronized HelpTopic getHelpTopic(String topicName) {
    if (topicName.equals("")) {
      return defaultTopic;
    }
    
    if (helpTopics.containsKey(topicName)) {
      return (HelpTopic)helpTopics.get(topicName);
    }
    
    return null;
  }
  
  public Collection<HelpTopic> getHelpTopics() {
    return helpTopics.values();
  }
  
  public synchronized void addTopic(HelpTopic topic)
  {
    if (!helpTopics.containsKey(topic.getName())) {
      helpTopics.put(topic.getName(), topic);
    }
  }
  
  public synchronized void clear() {
    helpTopics.clear();
  }
  
  public java.util.List<String> getIgnoredPlugins() {
    return yaml.getIgnoredPlugins();
  }
  


  public synchronized void initializeGeneralTopics()
  {
    yaml = new HelpYamlReader(server);
    

    for (HelpTopic topic : yaml.getGeneralTopics()) {
      addTopic(topic);
    }
    

    for (HelpTopic topic : yaml.getIndexTopics()) {
      if (topic.getName().equals("Default")) {
        defaultTopic = topic;
      } else {
        addTopic(topic);
      }
    }
  }
  



  public synchronized void initializeCommands()
  {
    Set<String> ignoredPlugins = new java.util.HashSet(yaml.getIgnoredPlugins());
    

    if (ignoredPlugins.contains("All")) {
      return;
    }
    

    for (Command command : server.getCommandMap().getCommands()) {
      if (!commandInIgnoredPlugin(command, ignoredPlugins))
      {



        Iterator i$ = topicFactoryMap.keySet().iterator(); for (;;) { if (!i$.hasNext()) break label224; Class c = (Class)i$.next();
          if (c.isAssignableFrom(command.getClass())) {
            HelpTopic t = ((HelpTopicFactory)topicFactoryMap.get(c)).createTopic(command);
            if (t == null) break; addTopic(t); break;
          }
          
          if (((command instanceof PluginCommand)) && (c.isAssignableFrom(((PluginCommand)command).getExecutor().getClass()))) {
            HelpTopic t = ((HelpTopicFactory)topicFactoryMap.get(c)).createTopic(command);
            if (t == null) break; addTopic(t); break;
          }
        }
        
        addTopic(new org.bukkit.help.GenericCommandHelpTopic(command));
      }
    }
    label224:
    for (Iterator i$ = server.getCommandMap().getCommands().iterator(); i$.hasNext();) { command = (Command)i$.next();
      if (!commandInIgnoredPlugin(command, ignoredPlugins))
      {

        for (String alias : command.getAliases()) {
          if (!helpTopics.containsKey("/" + alias)) {
            addTopic(new CommandAliasHelpTopic("/" + alias, "/" + command.getLabel(), this));
          }
        }
      }
    }
    Command command;
    for (org.bukkit.command.defaults.VanillaCommand command : server.getCommandMap().getFallbackCommands()) {
      if (!commandInIgnoredPlugin(command, ignoredPlugins)) {
        addTopic(new org.bukkit.help.GenericCommandHelpTopic(command));
      }
    }
    

    Collection<HelpTopic> filteredTopics = com.google.common.collect.Collections2.filter(helpTopics.values(), Predicates.instanceOf(CommandAliasHelpTopic.class));
    if (!filteredTopics.isEmpty()) {
      addTopic(new org.bukkit.help.IndexHelpTopic("Aliases", "Lists command aliases", null, filteredTopics));
    }
    

    Map<String, Set<HelpTopic>> pluginIndexes = new java.util.HashMap();
    fillPluginIndexes(pluginIndexes, server.getCommandMap().getCommands());
    fillPluginIndexes(pluginIndexes, server.getCommandMap().getFallbackCommands());
    
    for (Map.Entry<String, Set<HelpTopic>> entry : pluginIndexes.entrySet()) {
      addTopic(new org.bukkit.help.IndexHelpTopic((String)entry.getKey(), "All commands for " + (String)entry.getKey(), null, (Collection)entry.getValue(), "Below is a list of all " + (String)entry.getKey() + " commands:"));
    }
    

    for (HelpTopicAmendment amendment : yaml.getTopicAmendments()) {
      if (helpTopics.containsKey(amendment.getTopicName())) {
        ((HelpTopic)helpTopics.get(amendment.getTopicName())).amendTopic(amendment.getShortText(), amendment.getFullText());
        if (amendment.getPermission() != null) {
          ((HelpTopic)helpTopics.get(amendment.getTopicName())).amendCanSee(amendment.getPermission());
        }
      }
    }
  }
  
  private void fillPluginIndexes(Map<String, Set<HelpTopic>> pluginIndexes, Collection<? extends Command> commands) {
    for (Command command : commands) {
      String pluginName = getCommandPluginName(command);
      if (pluginName != null) {
        HelpTopic topic = getHelpTopic("/" + command.getLabel());
        if (topic != null) {
          if (!pluginIndexes.containsKey(pluginName)) {
            pluginIndexes.put(pluginName, new java.util.TreeSet(org.bukkit.help.HelpTopicComparator.helpTopicComparatorInstance()));
          }
          ((Set)pluginIndexes.get(pluginName)).add(topic);
        }
      }
    }
  }
  
  private String getCommandPluginName(Command command) {
    if (((command instanceof org.bukkit.command.defaults.BukkitCommand)) || ((command instanceof org.bukkit.command.defaults.VanillaCommand))) {
      return "Bukkit";
    }
    if ((command instanceof PluginIdentifiableCommand)) {
      return ((PluginIdentifiableCommand)command).getPlugin().getName();
    }
    return null;
  }
  
  private boolean commandInIgnoredPlugin(Command command, Set<String> ignoredPlugins) {
    if ((((command instanceof org.bukkit.command.defaults.BukkitCommand)) || ((command instanceof org.bukkit.command.defaults.VanillaCommand))) && (ignoredPlugins.contains("Bukkit"))) {
      return true;
    }
    if (((command instanceof PluginIdentifiableCommand)) && (ignoredPlugins.contains(((PluginIdentifiableCommand)command).getPlugin().getName()))) {
      return true;
    }
    return false;
  }
  
  public void registerHelpTopicFactory(Class commandClass, HelpTopicFactory factory) {
    if ((!Command.class.isAssignableFrom(commandClass)) && (!org.bukkit.command.CommandExecutor.class.isAssignableFrom(commandClass))) {
      throw new IllegalArgumentException("commandClass must implement either Command or CommandExecutor!");
    }
    topicFactoryMap.put(commandClass, factory);
  }
  
  private class IsCommandTopicPredicate implements com.google.common.base.Predicate<HelpTopic> {
    private IsCommandTopicPredicate() {}
    
    public boolean apply(HelpTopic topic) { return topic.getName().charAt(0) == '/'; }
  }
}
