package org.bukkit.craftbukkit.v1_7_R1.inventory;

import com.google.common.collect.ImmutableMap;
import com.google.common.collect.ImmutableMap.Builder;
import java.util.Map;
import net.minecraft.server.v1_7_R1.EnchantmentManager;
import net.minecraft.server.v1_7_R1.Item;
import net.minecraft.server.v1_7_R1.NBTTagCompound;
import net.minecraft.server.v1_7_R1.NBTTagList;
import org.apache.commons.lang.Validate;
import org.bukkit.Material;
import org.bukkit.configuration.serialization.DelegateDeserialization;
import org.bukkit.craftbukkit.v1_7_R1.util.CraftMagicNumbers;
import org.bukkit.enchantments.Enchantment;
import org.bukkit.inventory.meta.ItemMeta;




@DelegateDeserialization(org.bukkit.inventory.ItemStack.class)
public final class CraftItemStack
  extends org.bukkit.inventory.ItemStack
{
  net.minecraft.server.v1_7_R1.ItemStack handle;
  
  public static net.minecraft.server.v1_7_R1.ItemStack asNMSCopy(org.bukkit.inventory.ItemStack original)
  {
    if ((original instanceof CraftItemStack)) {
      CraftItemStack stack = (CraftItemStack)original;
      return handle == null ? null : handle.cloneItemStack();
    }
    if ((original == null) || (original.getTypeId() <= 0)) {
      return null;
    }
    
    Item item = CraftMagicNumbers.getItem(original.getType());
    
    if (item == null) {
      return null;
    }
    
    net.minecraft.server.v1_7_R1.ItemStack stack = new net.minecraft.server.v1_7_R1.ItemStack(item, original.getAmount(), original.getDurability());
    if (original.hasItemMeta()) {
      setItemMeta(stack, original.getItemMeta());
    }
    return stack;
  }
  
  public static net.minecraft.server.v1_7_R1.ItemStack copyNMSStack(net.minecraft.server.v1_7_R1.ItemStack original, int amount) {
    net.minecraft.server.v1_7_R1.ItemStack stack = original.cloneItemStack();
    count = amount;
    return stack;
  }
  


  public static org.bukkit.inventory.ItemStack asBukkitCopy(net.minecraft.server.v1_7_R1.ItemStack original)
  {
    if (original == null) {
      return new org.bukkit.inventory.ItemStack(Material.AIR);
    }
    org.bukkit.inventory.ItemStack stack = new org.bukkit.inventory.ItemStack(CraftMagicNumbers.getMaterial(original.getItem()), count, (short)original.getData());
    if (hasItemMeta(original)) {
      stack.setItemMeta(getItemMeta(original));
    }
    return stack;
  }
  
  public static CraftItemStack asCraftMirror(net.minecraft.server.v1_7_R1.ItemStack original) {
    return new CraftItemStack(original);
  }
  
  public static CraftItemStack asCraftCopy(org.bukkit.inventory.ItemStack original) {
    if ((original instanceof CraftItemStack)) {
      CraftItemStack stack = (CraftItemStack)original;
      return new CraftItemStack(handle == null ? null : handle.cloneItemStack());
    }
    return new CraftItemStack(original);
  }
  
  public static CraftItemStack asNewCraftStack(Item item) {
    return asNewCraftStack(item, 1);
  }
  
  public static CraftItemStack asNewCraftStack(Item item, int amount) {
    return new CraftItemStack(CraftMagicNumbers.getMaterial(item), amount, (short)0, null);
  }
  




  private CraftItemStack(net.minecraft.server.v1_7_R1.ItemStack item)
  {
    handle = item;
  }
  
  private CraftItemStack(org.bukkit.inventory.ItemStack item) {
    this(item.getTypeId(), item.getAmount(), item.getDurability(), item.hasItemMeta() ? item.getItemMeta() : null);
  }
  
  private CraftItemStack(Material type, int amount, short durability, ItemMeta itemMeta) {
    setType(type);
    setAmount(amount);
    setDurability(durability);
    setItemMeta(itemMeta);
  }
  
  private CraftItemStack(int typeId, int amount, short durability, ItemMeta itemMeta) {
    this(Material.getMaterial(typeId), amount, durability, itemMeta);
  }
  

  public int getTypeId()
  {
    return handle != null ? CraftMagicNumbers.getId(handle.getItem()) : 0;
  }
  
  public void setTypeId(int type)
  {
    if (getTypeId() == type)
      return;
    if (type == 0) {
      handle = null;
    } else if (CraftMagicNumbers.getItem(type) == null) {
      handle = null;
    } else if (handle == null) {
      handle = new net.minecraft.server.v1_7_R1.ItemStack(CraftMagicNumbers.getItem(type), 1, 0);
    } else {
      handle.setItem(CraftMagicNumbers.getItem(type));
      if (hasItemMeta())
      {
        setItemMeta(handle, getItemMeta(handle));
      }
    }
    setData(null);
  }
  
  public int getAmount()
  {
    return handle != null ? handle.count : 0;
  }
  
  public void setAmount(int amount)
  {
    if (handle == null) {
      return;
    }
    if (amount == 0) {
      handle = null;
    } else {
      handle.count = amount;
    }
  }
  

  public void setDurability(short durability)
  {
    if (handle != null) {
      handle.setData(durability);
    }
  }
  
  public short getDurability()
  {
    if (handle != null) {
      return (short)handle.getData();
    }
    return -1;
  }
  

  public int getMaxStackSize()
  {
    return handle == null ? Material.AIR.getMaxStackSize() : handle.getItem().getMaxStackSize();
  }
  
  public void addUnsafeEnchantment(Enchantment ench, int level)
  {
    Validate.notNull(ench, "Cannot add null enchantment");
    
    if (!makeTag(handle)) {
      return;
    }
    NBTTagList list = getEnchantmentList(handle);
    if (list == null) {
      list = new NBTTagList();
      handle.tag.set(ENCHANTMENTSNBT, list);
    }
    int size = list.size();
    
    for (int i = 0; i < size; i++) {
      NBTTagCompound tag = list.get(i);
      short id = tag.getShort(ENCHANTMENTS_IDNBT);
      if (id == ench.getId()) {
        tag.setShort(ENCHANTMENTS_LVLNBT, (short)level);
        return;
      }
    }
    NBTTagCompound tag = new NBTTagCompound();
    tag.setShort(ENCHANTMENTS_IDNBT, (short)ench.getId());
    tag.setShort(ENCHANTMENTS_LVLNBT, (short)level);
    list.add(tag);
  }
  
  static boolean makeTag(net.minecraft.server.v1_7_R1.ItemStack item) {
    if (item == null) {
      return false;
    }
    
    if (tag == null) {
      item.setTag(new NBTTagCompound());
    }
    
    return true;
  }
  
  public boolean containsEnchantment(Enchantment ench)
  {
    return getEnchantmentLevel(ench) > 0;
  }
  
  public int getEnchantmentLevel(Enchantment ench)
  {
    Validate.notNull(ench, "Cannot find null enchantment");
    if (handle == null) {
      return 0;
    }
    return EnchantmentManager.getEnchantmentLevel(ench.getId(), handle);
  }
  
  public int removeEnchantment(Enchantment ench)
  {
    Validate.notNull(ench, "Cannot remove null enchantment");
    
    NBTTagList list = getEnchantmentList(handle);
    if (list == null) {
      return 0;
    }
    int index = -2147483648;
    int level = -2147483648;
    int size = list.size();
    
    for (int i = 0; i < size; i++) {
      NBTTagCompound enchantment = list.get(i);
      int id = 0xFFFF & enchantment.getShort(ENCHANTMENTS_IDNBT);
      if (id == ench.getId()) {
        index = i;
        level = 0xFFFF & enchantment.getShort(ENCHANTMENTS_LVLNBT);
        break;
      }
    }
    
    if (index == -2147483648) {
      return 0;
    }
    if (size == 1) {
      handle.tag.remove(ENCHANTMENTSNBT);
      if (handle.tag.isEmpty()) {
        handle.tag = null;
      }
      return level;
    }
    

    NBTTagList listCopy = new NBTTagList();
    for (int i = 0; i < size; i++) {
      if (i != index) {
        listCopy.add(list.get(i));
      }
    }
    handle.tag.set(ENCHANTMENTSNBT, listCopy);
    
    return level;
  }
  
  public Map<Enchantment, Integer> getEnchantments()
  {
    return getEnchantments(handle);
  }
  
  static Map<Enchantment, Integer> getEnchantments(net.minecraft.server.v1_7_R1.ItemStack item) {
    NBTTagList list = (item != null) && (item.hasEnchantments()) ? item.getEnchantments() : null;
    
    if ((list == null) || (list.size() == 0)) {
      return ImmutableMap.of();
    }
    
    ImmutableMap.Builder<Enchantment, Integer> result = ImmutableMap.builder();
    
    for (int i = 0; i < list.size(); i++) {
      int id = 0xFFFF & list.get(i).getShort(ENCHANTMENTS_IDNBT);
      int level = 0xFFFF & list.get(i).getShort(ENCHANTMENTS_LVLNBT);
      
      result.put(Enchantment.getById(id), Integer.valueOf(level));
    }
    
    return result.build();
  }
  
  static NBTTagList getEnchantmentList(net.minecraft.server.v1_7_R1.ItemStack item) {
    return (item != null) && (item.hasEnchantments()) ? item.getEnchantments() : null;
  }
  
  public CraftItemStack clone()
  {
    CraftItemStack itemStack = (CraftItemStack)super.clone();
    if (handle != null) {
      handle = handle.cloneItemStack();
    }
    return itemStack;
  }
  
  public ItemMeta getItemMeta()
  {
    return getItemMeta(handle);
  }
  
  public static ItemMeta getItemMeta(net.minecraft.server.v1_7_R1.ItemStack item) {
    if (!hasItemMeta(item)) {
      return CraftItemFactory.instance().getItemMeta(getType(item));
    }
    switch (1.$SwitchMap$org$bukkit$Material[getType(item).ordinal()]) {
    case 1: 
    case 2: 
      return new CraftMetaBook(tag);
    case 3: 
      return new CraftMetaSkull(tag);
    case 4: 
    case 5: 
    case 6: 
    case 7: 
      return new CraftMetaLeatherArmor(tag);
    case 8: 
      return new CraftMetaPotion(tag);
    case 9: 
      return new CraftMetaMap(tag);
    case 10: 
      return new CraftMetaFirework(tag);
    case 11: 
      return new CraftMetaCharge(tag);
    case 12: 
      return new CraftMetaEnchantedBook(tag);
    }
    return new CraftMetaItem(tag);
  }
  
  static Material getType(net.minecraft.server.v1_7_R1.ItemStack item)
  {
    Material material = Material.getMaterial(item == null ? 0 : CraftMagicNumbers.getId(item.getItem()));
    return material == null ? Material.AIR : material;
  }
  
  public boolean setItemMeta(ItemMeta itemMeta)
  {
    return setItemMeta(handle, itemMeta);
  }
  
  public static boolean setItemMeta(net.minecraft.server.v1_7_R1.ItemStack item, ItemMeta itemMeta) {
    if (item == null) {
      return false;
    }
    if (CraftItemFactory.instance().equals(itemMeta, null)) {
      item.tag = null;
      return true;
    }
    if (!CraftItemFactory.instance().isApplicable(itemMeta, getType(item))) {
      return false;
    }
    
    NBTTagCompound tag = new NBTTagCompound();
    item.setTag(tag);
    
    ((CraftMetaItem)itemMeta).applyToItem(tag);
    return true;
  }
  
  public boolean isSimilar(org.bukkit.inventory.ItemStack stack)
  {
    if (stack == null) {
      return false;
    }
    if (stack == this) {
      return true;
    }
    if (!(stack instanceof CraftItemStack)) {
      return (stack.getClass() == org.bukkit.inventory.ItemStack.class) && (stack.isSimilar(this));
    }
    
    CraftItemStack that = (CraftItemStack)stack;
    if (handle == handle) {
      return true;
    }
    if ((handle == null) || (handle == null)) {
      return false;
    }
    if ((that.getTypeId() != getTypeId()) || (getDurability() != that.getDurability())) {
      return false;
    }
    return (that.hasItemMeta()) && (handle.tag.equals(handle.tag));
  }
  
  public boolean hasItemMeta()
  {
    return hasItemMeta(handle);
  }
  
  static boolean hasItemMeta(net.minecraft.server.v1_7_R1.ItemStack item) {
    return (item != null) && (tag != null) && (!tag.isEmpty());
  }
}
