package org.bukkit.craftbukkit.v1_7_R1.inventory;

import com.google.common.collect.ImmutableList;
import com.google.common.collect.ImmutableMap.Builder;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import net.minecraft.server.v1_7_R1.NBTTagCompound;
import net.minecraft.server.v1_7_R1.NBTTagList;
import org.apache.commons.lang.Validate;
import org.bukkit.Color;
import org.bukkit.FireworkEffect;
import org.bukkit.FireworkEffect.Builder;
import org.bukkit.FireworkEffect.Type;
import org.bukkit.Material;
import org.bukkit.configuration.serialization.DelegateDeserialization;
import org.bukkit.inventory.meta.FireworkMeta;



















@DelegateDeserialization(CraftMetaItem.SerializableMeta.class)
class CraftMetaFirework
  extends CraftMetaItem
  implements FireworkMeta
{
  static final CraftMetaItem.ItemMetaKey FIREWORKS = new CraftMetaItem.ItemMetaKey("Fireworks");
  static final CraftMetaItem.ItemMetaKey FLIGHT = new CraftMetaItem.ItemMetaKey("Flight", "power");
  static final CraftMetaItem.ItemMetaKey EXPLOSIONS = new CraftMetaItem.ItemMetaKey("Explosions", "firework-effects");
  
  static final CraftMetaItem.ItemMetaKey EXPLOSION_COLORS = new CraftMetaItem.ItemMetaKey("Colors");
  
  static final CraftMetaItem.ItemMetaKey EXPLOSION_TYPE = new CraftMetaItem.ItemMetaKey("Type");
  
  static final CraftMetaItem.ItemMetaKey EXPLOSION_TRAIL = new CraftMetaItem.ItemMetaKey("Trail");
  
  static final CraftMetaItem.ItemMetaKey EXPLOSION_FLICKER = new CraftMetaItem.ItemMetaKey("Flicker");
  
  static final CraftMetaItem.ItemMetaKey EXPLOSION_FADE = new CraftMetaItem.ItemMetaKey("FadeColors");
  private List<FireworkEffect> effects;
  private int power;
  
  CraftMetaFirework(CraftMetaItem meta)
  {
    super(meta);
    
    if (!(meta instanceof CraftMetaFirework)) {
      return;
    }
    
    CraftMetaFirework that = (CraftMetaFirework)meta;
    
    power = power;
    
    if (that.hasEffects()) {
      effects = new ArrayList(effects);
    }
  }
  
  CraftMetaFirework(NBTTagCompound tag) {
    super(tag);
    
    if (!tag.hasKey(FIREWORKSNBT)) {
      return;
    }
    
    NBTTagCompound fireworks = tag.getCompound(FIREWORKSNBT);
    
    power = (0xFF & fireworks.getByte(FLIGHTNBT));
    
    if (!fireworks.hasKey(EXPLOSIONSNBT)) {
      return;
    }
    
    NBTTagList fireworkEffects = fireworks.getList(EXPLOSIONSNBT, 10);
    List<FireworkEffect> effects = this.effects = new ArrayList(fireworkEffects.size());
    
    for (int i = 0; i < fireworkEffects.size(); i++) {
      effects.add(getEffect(fireworkEffects.get(i)));
    }
  }
  
  static FireworkEffect getEffect(NBTTagCompound explosion) {
    FireworkEffect.Builder effect = FireworkEffect.builder().flicker(explosion.getBoolean(EXPLOSION_FLICKERNBT)).trail(explosion.getBoolean(EXPLOSION_TRAILNBT)).with(getEffectType(0xFF & explosion.getByte(EXPLOSION_TYPENBT)));
    



    for (int color : explosion.getIntArray(EXPLOSION_COLORSNBT)) {
      effect.withColor(Color.fromRGB(color));
    }
    
    for (int color : explosion.getIntArray(EXPLOSION_FADENBT)) {
      effect.withFade(Color.fromRGB(color));
    }
    
    return effect.build();
  }
  
  static NBTTagCompound getExplosion(FireworkEffect effect) {
    NBTTagCompound explosion = new NBTTagCompound();
    
    if (effect.hasFlicker()) {
      explosion.setBoolean(EXPLOSION_FLICKERNBT, true);
    }
    
    if (effect.hasTrail()) {
      explosion.setBoolean(EXPLOSION_TRAILNBT, true);
    }
    
    addColors(explosion, EXPLOSION_COLORS, effect.getColors());
    addColors(explosion, EXPLOSION_FADE, effect.getFadeColors());
    
    explosion.setByte(EXPLOSION_TYPENBT, (byte)getNBT(effect.getType()));
    
    return explosion;
  }
  
  static int getNBT(FireworkEffect.Type type) {
    switch (1.$SwitchMap$org$bukkit$FireworkEffect$Type[type.ordinal()]) {
    case 1: 
      return 0;
    case 2: 
      return 1;
    case 3: 
      return 2;
    case 4: 
      return 3;
    case 5: 
      return 4;
    }
    throw new AssertionError(type);
  }
  
  static FireworkEffect.Type getEffectType(int nbt)
  {
    switch (nbt) {
    case 0: 
      return FireworkEffect.Type.BALL;
    case 1: 
      return FireworkEffect.Type.BALL_LARGE;
    case 2: 
      return FireworkEffect.Type.STAR;
    case 3: 
      return FireworkEffect.Type.CREEPER;
    case 4: 
      return FireworkEffect.Type.BURST;
    }
    throw new AssertionError(nbt);
  }
  
  CraftMetaFirework(Map<String, Object> map)
  {
    super(map);
    
    Integer power = (Integer)CraftMetaItem.SerializableMeta.getObject(Integer.class, map, FLIGHTBUKKIT, true);
    if (power != null) {
      setPower(power.intValue());
    }
    
    Iterable<?> effects = (Iterable)CraftMetaItem.SerializableMeta.getObject(Iterable.class, map, EXPLOSIONSBUKKIT, true);
    safelyAddEffects(effects);
  }
  
  public boolean hasEffects() {
    return (effects != null) && (!effects.isEmpty());
  }
  
  void safelyAddEffects(Iterable<?> collection) {
    if ((collection == null) || (((collection instanceof Collection)) && (((Collection)collection).isEmpty()))) {
      return;
    }
    
    List<FireworkEffect> effects = this.effects;
    if (effects == null) {
      effects = this.effects = new ArrayList();
    }
    
    for (Object obj : collection) {
      if ((obj instanceof FireworkEffect)) {
        effects.add((FireworkEffect)obj);
      } else {
        throw new IllegalArgumentException(obj + " in " + collection + " is not a FireworkEffect");
      }
    }
  }
  
  void applyToItem(NBTTagCompound itemTag)
  {
    super.applyToItem(itemTag);
    if (isFireworkEmpty()) {
      return;
    }
    
    NBTTagCompound fireworks = itemTag.getCompound(FIREWORKSNBT);
    itemTag.set(FIREWORKSNBT, fireworks);
    
    if (hasEffects()) {
      NBTTagList effects = new NBTTagList();
      for (FireworkEffect effect : this.effects) {
        effects.add(getExplosion(effect));
      }
      
      if (effects.size() > 0) {
        fireworks.set(EXPLOSIONSNBT, effects);
      }
    }
    
    if (hasPower()) {
      fireworks.setByte(FLIGHTNBT, (byte)power);
    }
  }
  
  static void addColors(NBTTagCompound compound, CraftMetaItem.ItemMetaKey key, List<Color> colors) {
    if (colors.isEmpty()) {
      return;
    }
    
    int[] colorArray = new int[colors.size()];
    int i = 0;
    for (Color color : colors) {
      colorArray[(i++)] = color.asRGB();
    }
    
    compound.setIntArray(NBT, colorArray);
  }
  
  boolean applicableTo(Material type)
  {
    switch (type) {
    case FIREWORK: 
      return true;
    }
    return false;
  }
  

  boolean isEmpty()
  {
    return (super.isEmpty()) && (isFireworkEmpty());
  }
  
  boolean isFireworkEmpty() {
    return (!hasEffects()) && (!hasPower());
  }
  
  boolean hasPower() {
    return power != 0;
  }
  
  boolean equalsCommon(CraftMetaItem meta)
  {
    if (!super.equalsCommon(meta)) {
      return false;
    }
    
    if ((meta instanceof CraftMetaFirework)) {
      CraftMetaFirework that = (CraftMetaFirework)meta;
      
      return (hasPower() ? (that.hasPower()) && (power == power) : !that.hasPower()) && (hasEffects() ? (that.hasEffects()) && (effects.equals(effects)) : !that.hasEffects());
    }
    

    return true;
  }
  
  boolean notUncommon(CraftMetaItem meta)
  {
    return (super.notUncommon(meta)) && (((meta instanceof CraftMetaFirework)) || (isFireworkEmpty()));
  }
  
  int applyHash()
  {
    int original;
    int hash = original = super.applyHash();
    if (hasPower()) {
      hash = 61 * hash + power;
    }
    if (hasEffects()) {
      hash = 61 * hash + 13 * effects.hashCode();
    }
    return hash != original ? CraftMetaFirework.class.hashCode() ^ hash : hash;
  }
  
  ImmutableMap.Builder<String, Object> serialize(ImmutableMap.Builder<String, Object> builder)
  {
    super.serialize(builder);
    
    if (hasEffects()) {
      builder.put(EXPLOSIONSBUKKIT, ImmutableList.copyOf(effects));
    }
    
    if (hasPower()) {
      builder.put(FLIGHTBUKKIT, Integer.valueOf(power));
    }
    
    return builder;
  }
  
  public CraftMetaFirework clone()
  {
    CraftMetaFirework meta = (CraftMetaFirework)super.clone();
    
    if (effects != null) {
      effects = new ArrayList(effects);
    }
    
    return meta;
  }
  
  public void addEffect(FireworkEffect effect) {
    Validate.notNull(effect, "Effect cannot be null");
    if (effects == null) {
      effects = new ArrayList();
    }
    effects.add(effect);
  }
  
  public void addEffects(FireworkEffect... effects) {
    Validate.notNull(effects, "Effects cannot be null");
    if (effects.length == 0) {
      return;
    }
    
    List<FireworkEffect> list = this.effects;
    if (list == null) {
      list = this.effects = new ArrayList();
    }
    
    for (FireworkEffect effect : effects) {
      Validate.notNull(effect, "Effect cannot be null");
      list.add(effect);
    }
  }
  
  public void addEffects(Iterable<FireworkEffect> effects) {
    Validate.notNull(effects, "Effects cannot be null");
    safelyAddEffects(effects);
  }
  
  public List<FireworkEffect> getEffects() {
    return effects == null ? ImmutableList.of() : ImmutableList.copyOf(effects);
  }
  
  public int getEffectsSize() {
    return effects == null ? 0 : effects.size();
  }
  
  public void removeEffect(int index) {
    if (effects == null) {
      throw new IndexOutOfBoundsException("Index: " + index + ", Size: 0");
    }
    effects.remove(index);
  }
  
  public void clearEffects()
  {
    effects = null;
  }
  
  public int getPower() {
    return power;
  }
  
  public void setPower(int power) {
    Validate.isTrue(power >= 0, "Power cannot be less than zero: ", power);
    Validate.isTrue(power < 128, "Power cannot be more than 127: ", power);
    this.power = power;
  }
}
