package org.bukkit.craftbukkit.v1_7_R1.inventory;

import com.google.common.base.Strings;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.ImmutableMap;
import com.google.common.collect.ImmutableMap.Builder;
import java.lang.annotation.Annotation;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.NoSuchElementException;
import net.minecraft.server.v1_7_R1.NBTBase;
import net.minecraft.server.v1_7_R1.NBTTagCompound;
import net.minecraft.server.v1_7_R1.NBTTagDouble;
import net.minecraft.server.v1_7_R1.NBTTagInt;
import net.minecraft.server.v1_7_R1.NBTTagList;
import net.minecraft.server.v1_7_R1.NBTTagLong;
import net.minecraft.server.v1_7_R1.NBTTagString;
import org.apache.commons.lang.Validate;
import org.bukkit.Material;
import org.bukkit.configuration.serialization.ConfigurationSerializable;
import org.bukkit.configuration.serialization.DelegateDeserialization;
import org.bukkit.configuration.serialization.SerializableAs;
import org.bukkit.craftbukkit.v1_7_R1.Overridden;
import org.bukkit.enchantments.Enchantment;
import org.bukkit.inventory.meta.ItemMeta;
import org.bukkit.inventory.meta.Repairable;

















@DelegateDeserialization(SerializableMeta.class)
class CraftMetaItem
  implements ItemMeta, Repairable
{
  static class ItemMetaKey
  {
    final String BUKKIT;
    final String NBT;
    
    @Retention(RetentionPolicy.SOURCE)
    @Target({java.lang.annotation.ElementType.FIELD})
    static @interface Specific
    {
      To value();
      
      public static enum To
      {
        BUKKIT, 
        NBT;
        

        private To() {}
      }
    }
    

    ItemMetaKey(String both)
    {
      this(both, both);
    }
    
    ItemMetaKey(String nbt, String bukkit) {
      NBT = nbt;
      BUKKIT = bukkit;
    }
  }
  


  @SerializableAs("ItemMeta")
  public static class SerializableMeta
    implements ConfigurationSerializable
  {
    static final String TYPE_FIELD = "meta-type";
    
    static final ImmutableMap<Class<? extends CraftMetaItem>, String> classMap = ImmutableMap.builder().put(CraftMetaBook.class, "BOOK").put(CraftMetaSkull.class, "SKULL").put(CraftMetaLeatherArmor.class, "LEATHER_ARMOR").put(CraftMetaMap.class, "MAP").put(CraftMetaPotion.class, "POTION").put(CraftMetaEnchantedBook.class, "ENCHANTED").put(CraftMetaFirework.class, "FIREWORK").put(CraftMetaCharge.class, "FIREWORK_EFFECT").put(CraftMetaItem.class, "UNSPECIFIC").build();
    



    static final ImmutableMap<String, Constructor<? extends CraftMetaItem>> constructorMap;
    



    static
    {
      ImmutableMap.Builder<String, Constructor<? extends CraftMetaItem>> classConstructorBuilder = ImmutableMap.builder();
      for (Map.Entry<Class<? extends CraftMetaItem>, String> mapping : classMap.entrySet()) {
        try {
          classConstructorBuilder.put(mapping.getValue(), ((Class)mapping.getKey()).getDeclaredConstructor(new Class[] { Map.class }));
        } catch (NoSuchMethodException e) {
          throw new AssertionError(e);
        }
      }
      constructorMap = classConstructorBuilder.build();
    }
    

    public static ItemMeta deserialize(Map<String, Object> map)
      throws Throwable
    {
      Validate.notNull(map, "Cannot deserialize null map");
      
      String type = getString(map, "meta-type", false);
      Constructor<? extends CraftMetaItem> constructor = (Constructor)constructorMap.get(type);
      
      if (constructor == null) {
        throw new IllegalArgumentException(type + " is not a valid " + "meta-type");
      }
      try
      {
        return (ItemMeta)constructor.newInstance(new Object[] { map });
      } catch (InstantiationException e) {
        throw new AssertionError(e);
      } catch (IllegalAccessException e) {
        throw new AssertionError(e);
      } catch (InvocationTargetException e) {
        throw e.getCause();
      }
    }
    
    public Map<String, Object> serialize() {
      throw new AssertionError();
    }
    
    static String getString(Map<?, ?> map, Object field, boolean nullable) {
      return (String)getObject(String.class, map, field, nullable);
    }
    
    static boolean getBoolean(Map<?, ?> map, Object field) {
      Boolean value = (Boolean)getObject(Boolean.class, map, field, true);
      return (value != null) && (value.booleanValue());
    }
    
    static <T> T getObject(Class<T> clazz, Map<?, ?> map, Object field, boolean nullable) {
      Object object = map.get(field);
      
      if (clazz.isInstance(object)) {
        return clazz.cast(object);
      }
      if (object == null) {
        if (!nullable) {
          throw new NoSuchElementException(map + " does not contain " + field);
        }
        return null;
      }
      throw new IllegalArgumentException(field + "(" + object + ") is not a valid " + clazz);
    }
  }
  
  static final ItemMetaKey NAME = new ItemMetaKey("Name", "display-name");
  
  static final ItemMetaKey DISPLAY = new ItemMetaKey("display");
  static final ItemMetaKey LORE = new ItemMetaKey("Lore", "lore");
  static final ItemMetaKey ENCHANTMENTS = new ItemMetaKey("ench", "enchants");
  
  static final ItemMetaKey ENCHANTMENTS_ID = new ItemMetaKey("id");
  
  static final ItemMetaKey ENCHANTMENTS_LVL = new ItemMetaKey("lvl");
  static final ItemMetaKey REPAIR = new ItemMetaKey("RepairCost", "repair-cost");
  
  static final ItemMetaKey ATTRIBUTES = new ItemMetaKey("AttributeModifiers");
  
  static final ItemMetaKey ATTRIBUTES_IDENTIFIER = new ItemMetaKey("AttributeName");
  
  static final ItemMetaKey ATTRIBUTES_NAME = new ItemMetaKey("Name");
  
  static final ItemMetaKey ATTRIBUTES_VALUE = new ItemMetaKey("Amount");
  
  static final ItemMetaKey ATTRIBUTES_TYPE = new ItemMetaKey("Operation");
  
  static final ItemMetaKey ATTRIBUTES_UUID_HIGH = new ItemMetaKey("UUIDMost");
  
  static final ItemMetaKey ATTRIBUTES_UUID_LOW = new ItemMetaKey("UUIDLeast");
  private String displayName;
  private List<String> lore;
  private Map<Enchantment, Integer> enchantments;
  private int repairCost;
  private final NBTTagList attributes;
  
  CraftMetaItem(CraftMetaItem meta)
  {
    if (meta == null) {
      attributes = null;
      return;
    }
    
    displayName = displayName;
    
    if (meta.hasLore()) {
      lore = new ArrayList(lore);
    }
    
    if (meta.hasEnchants()) {
      enchantments = new HashMap(enchantments);
    }
    
    repairCost = repairCost;
    attributes = attributes;
  }
  
  CraftMetaItem(NBTTagCompound tag) {
    if (tag.hasKey(DISPLAYNBT)) {
      NBTTagCompound display = tag.getCompound(DISPLAYNBT);
      
      if (display.hasKey(NAMENBT)) {
        displayName = display.getString(NAMENBT);
      }
      
      if (display.hasKey(LORENBT)) {
        NBTTagList list = display.getList(LORENBT, 8);
        lore = new ArrayList(list.size());
        
        for (int index = 0; index < list.size(); index++) {
          String line = list.f(index);
          lore.add(line);
        }
      }
    }
    
    enchantments = buildEnchantments(tag, ENCHANTMENTS);
    
    if (tag.hasKey(REPAIRNBT)) {
      repairCost = tag.getInt(REPAIRNBT);
    }
    

    if ((tag.get(ATTRIBUTESNBT) instanceof NBTTagList)) {
      NBTTagList save = null;
      NBTTagList nbttaglist = tag.getList(ATTRIBUTESNBT, 10);
      
      for (int i = 0; i < nbttaglist.size(); i++)
        if ((nbttaglist.get(i) instanceof NBTTagCompound))
        {

          NBTTagCompound nbttagcompound = nbttaglist.get(i);
          
          if ((nbttagcompound.get(ATTRIBUTES_UUID_HIGHNBT) instanceof NBTTagLong))
          {

            if ((nbttagcompound.get(ATTRIBUTES_UUID_LOWNBT) instanceof NBTTagLong))
            {

              if (((nbttagcompound.get(ATTRIBUTES_IDENTIFIERNBT) instanceof NBTTagString)) && (CraftItemFactory.KNOWN_NBT_ATTRIBUTE_NAMES.contains(nbttagcompound.getString(ATTRIBUTES_IDENTIFIERNBT))))
              {

                if (((nbttagcompound.get(ATTRIBUTES_NAMENBT) instanceof NBTTagString)) && (!nbttagcompound.getString(ATTRIBUTES_NAMENBT).isEmpty()))
                {

                  if ((nbttagcompound.get(ATTRIBUTES_VALUENBT) instanceof NBTTagDouble))
                  {

                    if (((nbttagcompound.get(ATTRIBUTES_TYPENBT) instanceof NBTTagInt)) && (nbttagcompound.getInt(ATTRIBUTES_TYPENBT) >= 0) && (nbttagcompound.getInt(ATTRIBUTES_TYPENBT) <= 2))
                    {


                      if (save == null) {
                        save = new NBTTagList();
                      }
                      
                      NBTTagCompound entry = new NBTTagCompound();
                      entry.set(ATTRIBUTES_UUID_HIGHNBT, nbttagcompound.get(ATTRIBUTES_UUID_HIGHNBT));
                      entry.set(ATTRIBUTES_UUID_LOWNBT, nbttagcompound.get(ATTRIBUTES_UUID_LOWNBT));
                      entry.set(ATTRIBUTES_IDENTIFIERNBT, nbttagcompound.get(ATTRIBUTES_IDENTIFIERNBT));
                      entry.set(ATTRIBUTES_NAMENBT, nbttagcompound.get(ATTRIBUTES_NAMENBT));
                      entry.set(ATTRIBUTES_VALUENBT, nbttagcompound.get(ATTRIBUTES_VALUENBT));
                      entry.set(ATTRIBUTES_TYPENBT, nbttagcompound.get(ATTRIBUTES_TYPENBT));
                      save.add(entry);
                    } } } } } }
        }
      attributes = save;
    } else {
      attributes = null;
    }
  }
  
  static Map<Enchantment, Integer> buildEnchantments(NBTTagCompound tag, ItemMetaKey key) {
    if (!tag.hasKey(NBT)) {
      return null;
    }
    
    NBTTagList ench = tag.getList(NBT, 10);
    Map<Enchantment, Integer> enchantments = new HashMap(ench.size());
    
    for (int i = 0; i < ench.size(); i++) {
      int id = 0xFFFF & ench.get(i).getShort(ENCHANTMENTS_IDNBT);
      int level = 0xFFFF & ench.get(i).getShort(ENCHANTMENTS_LVLNBT);
      
      enchantments.put(Enchantment.getById(id), Integer.valueOf(level));
    }
    
    return enchantments;
  }
  
  CraftMetaItem(Map<String, Object> map) {
    setDisplayName(SerializableMeta.getString(map, NAMEBUKKIT, true));
    
    Iterable<?> lore = (Iterable)SerializableMeta.getObject(Iterable.class, map, LOREBUKKIT, true);
    if (lore != null) {
      safelyAdd(lore, this.lore = new ArrayList(), 2147483647);
    }
    
    enchantments = buildEnchantments(map, ENCHANTMENTS);
    
    Integer repairCost = (Integer)SerializableMeta.getObject(Integer.class, map, REPAIRBUKKIT, true);
    if (repairCost != null) {
      setRepairCost(repairCost.intValue());
    }
    
    attributes = null;
  }
  
  static Map<Enchantment, Integer> buildEnchantments(Map<String, Object> map, ItemMetaKey key) {
    Map<?, ?> ench = (Map)SerializableMeta.getObject(Map.class, map, BUKKIT, true);
    if (ench == null) {
      return null;
    }
    
    Map<Enchantment, Integer> enchantments = new HashMap(ench.size());
    for (Map.Entry<?, ?> entry : ench.entrySet()) {
      Enchantment enchantment = Enchantment.getByName(entry.getKey().toString());
      
      if ((enchantment != null) && ((entry.getValue() instanceof Integer))) {
        enchantments.put(enchantment, (Integer)entry.getValue());
      }
    }
    
    return enchantments;
  }
  
  @Overridden
  void applyToItem(NBTTagCompound itemTag) {
    if (hasDisplayName()) {
      setDisplayTag(itemTag, NAMENBT, new NBTTagString(displayName));
    }
    
    if (hasLore()) {
      setDisplayTag(itemTag, LORENBT, createStringList(lore));
    }
    
    applyEnchantments(enchantments, itemTag, ENCHANTMENTS);
    
    if (hasRepairCost()) {
      itemTag.setInt(REPAIRNBT, repairCost);
    }
    
    if (attributes != null) {
      itemTag.set(ATTRIBUTESNBT, attributes);
    }
  }
  
  static NBTTagList createStringList(List<String> list) {
    if ((list == null) || (list.isEmpty())) {
      return null;
    }
    
    NBTTagList tagList = new NBTTagList();
    for (String value : list) {
      tagList.add(new NBTTagString(value));
    }
    
    return tagList;
  }
  
  static void applyEnchantments(Map<Enchantment, Integer> enchantments, NBTTagCompound tag, ItemMetaKey key) {
    if ((enchantments == null) || (enchantments.size() == 0)) {
      return;
    }
    
    NBTTagList list = new NBTTagList();
    
    for (Map.Entry<Enchantment, Integer> entry : enchantments.entrySet()) {
      NBTTagCompound subtag = new NBTTagCompound();
      
      subtag.setShort(ENCHANTMENTS_IDNBT, (short)((Enchantment)entry.getKey()).getId());
      subtag.setShort(ENCHANTMENTS_LVLNBT, ((Integer)entry.getValue()).shortValue());
      
      list.add(subtag);
    }
    
    tag.set(NBT, list);
  }
  
  void setDisplayTag(NBTTagCompound tag, String key, NBTBase value) {
    NBTTagCompound display = tag.getCompound(DISPLAYNBT);
    
    if (!tag.hasKey(DISPLAYNBT)) {
      tag.set(DISPLAYNBT, display);
    }
    
    display.set(key, value);
  }
  
  @Overridden
  boolean applicableTo(Material type) {
    return type != Material.AIR;
  }
  
  @Overridden
  boolean isEmpty() {
    return (!hasDisplayName()) && (!hasEnchants()) && (!hasLore()) && (!hasAttributes()) && (!hasRepairCost());
  }
  
  public String getDisplayName() {
    return displayName;
  }
  
  public final void setDisplayName(String name) {
    displayName = name;
  }
  
  public boolean hasDisplayName() {
    return !Strings.isNullOrEmpty(displayName);
  }
  
  public boolean hasLore() {
    return (lore != null) && (!lore.isEmpty());
  }
  
  public boolean hasAttributes() {
    return attributes != null;
  }
  
  public boolean hasRepairCost() {
    return repairCost > 0;
  }
  
  public boolean hasEnchant(Enchantment ench) {
    return (hasEnchants()) && (enchantments.containsKey(ench));
  }
  
  public int getEnchantLevel(Enchantment ench) {
    Integer level = hasEnchants() ? (Integer)enchantments.get(ench) : null;
    if (level == null) {
      return 0;
    }
    return level.intValue();
  }
  
  public Map<Enchantment, Integer> getEnchants() {
    return hasEnchants() ? ImmutableMap.copyOf(enchantments) : ImmutableMap.of();
  }
  
  public boolean addEnchant(Enchantment ench, int level, boolean ignoreRestrictions) {
    if (enchantments == null) {
      enchantments = new HashMap(4);
    }
    
    if ((ignoreRestrictions) || ((level >= ench.getStartLevel()) && (level <= ench.getMaxLevel()))) {
      Integer old = (Integer)enchantments.put(ench, Integer.valueOf(level));
      return (old == null) || (old.intValue() != level);
    }
    return false;
  }
  
  public boolean removeEnchant(Enchantment ench) {
    return (hasEnchants()) && (enchantments.remove(ench) != null);
  }
  
  public boolean hasEnchants() {
    return (enchantments != null) && (!enchantments.isEmpty());
  }
  
  public boolean hasConflictingEnchant(Enchantment ench) {
    return checkConflictingEnchants(enchantments, ench);
  }
  
  public List<String> getLore() {
    return lore == null ? null : new ArrayList(lore);
  }
  
  public void setLore(List<String> lore) {
    if (lore == null) {
      this.lore = null;
    }
    else if (this.lore == null) {
      safelyAdd(lore, this.lore = new ArrayList(lore.size()), 2147483647);
    } else {
      this.lore.clear();
      safelyAdd(lore, this.lore, 2147483647);
    }
  }
  
  public int getRepairCost()
  {
    return repairCost;
  }
  
  public void setRepairCost(int cost) {
    repairCost = cost;
  }
  
  public final boolean equals(Object object)
  {
    if (object == null) {
      return false;
    }
    if (object == this) {
      return true;
    }
    if (!(object instanceof CraftMetaItem)) {
      return false;
    }
    return CraftItemFactory.instance().equals(this, (ItemMeta)object);
  }
  




  @Overridden
  boolean equalsCommon(CraftMetaItem that)
  {
    return (hasDisplayName() ? (that.hasDisplayName()) && (displayName.equals(displayName)) : !that.hasDisplayName()) && (hasEnchants() ? (that.hasEnchants()) && (enchantments.equals(enchantments)) : !that.hasEnchants()) && (hasLore() ? (that.hasLore()) && (lore.equals(lore)) : !that.hasLore()) && (hasAttributes() ? (that.hasAttributes()) && (attributes.equals(attributes)) : !that.hasAttributes()) && (hasRepairCost() ? (that.hasRepairCost()) && (repairCost == repairCost) : !that.hasRepairCost());
  }
  








  @Overridden
  boolean notUncommon(CraftMetaItem meta)
  {
    return true;
  }
  
  public final int hashCode()
  {
    return applyHash();
  }
  
  @Overridden
  int applyHash() {
    int hash = 3;
    hash = 61 * hash + (hasDisplayName() ? displayName.hashCode() : 0);
    hash = 61 * hash + (hasLore() ? lore.hashCode() : 0);
    hash = 61 * hash + (hasEnchants() ? enchantments.hashCode() : 0);
    hash = 61 * hash + (hasAttributes() ? attributes.hashCode() : 0);
    hash = 61 * hash + (hasRepairCost() ? repairCost : 0);
    return hash;
  }
  
  @Overridden
  public CraftMetaItem clone()
  {
    try {
      CraftMetaItem clone = (CraftMetaItem)super.clone();
      if (lore != null) {
        lore = new ArrayList(lore);
      }
      if (enchantments != null) {
        enchantments = new HashMap(enchantments);
      }
      return clone;
    } catch (CloneNotSupportedException e) {
      throw new Error(e);
    }
  }
  
  public final Map<String, Object> serialize() {
    ImmutableMap.Builder<String, Object> map = ImmutableMap.builder();
    map.put("meta-type", SerializableMeta.classMap.get(getClass()));
    serialize(map);
    return map.build();
  }
  
  @Overridden
  ImmutableMap.Builder<String, Object> serialize(ImmutableMap.Builder<String, Object> builder) {
    if (hasDisplayName()) {
      builder.put(NAMEBUKKIT, displayName);
    }
    
    if (hasLore()) {
      builder.put(LOREBUKKIT, ImmutableList.copyOf(lore));
    }
    
    serializeEnchantments(enchantments, builder, ENCHANTMENTS);
    
    if (hasRepairCost()) {
      builder.put(REPAIRBUKKIT, Integer.valueOf(repairCost));
    }
    
    return builder;
  }
  
  static void serializeEnchantments(Map<Enchantment, Integer> enchantments, ImmutableMap.Builder<String, Object> builder, ItemMetaKey key) {
    if ((enchantments == null) || (enchantments.isEmpty())) {
      return;
    }
    
    ImmutableMap.Builder<String, Integer> enchants = ImmutableMap.builder();
    for (Map.Entry<? extends Enchantment, Integer> enchant : enchantments.entrySet()) {
      enchants.put(((Enchantment)enchant.getKey()).getName(), enchant.getValue());
    }
    
    builder.put(BUKKIT, enchants.build());
  }
  
  static void safelyAdd(Iterable<?> addFrom, Collection<String> addTo, int maxItemLength) {
    if (addFrom == null) {
      return;
    }
    
    for (Object object : addFrom) {
      if (!(object instanceof String)) {
        if (object != null) {
          throw new IllegalArgumentException(addFrom + " cannot contain non-string " + object.getClass().getName());
        }
        
        addTo.add("");
      } else {
        String page = object.toString();
        
        if (page.length() > maxItemLength) {
          page = page.substring(0, maxItemLength);
        }
        
        addTo.add(page);
      }
    }
  }
  
  static boolean checkConflictingEnchants(Map<Enchantment, Integer> enchantments, Enchantment ench) {
    if ((enchantments == null) || (enchantments.isEmpty())) {
      return false;
    }
    
    for (Enchantment enchant : enchantments.keySet()) {
      if (enchant.conflictsWith(ench)) {
        return true;
      }
    }
    
    return false;
  }
  
  public final String toString()
  {
    return (String)SerializableMeta.classMap.get(getClass()) + "_META:" + serialize();
  }
}
