package org.bukkit.craftbukkit.v1_7_R1.inventory;

import com.google.common.collect.ImmutableMap.Builder;
import java.util.Map;
import net.minecraft.server.v1_7_R1.NBTTagCompound;
import net.minecraft.server.v1_7_R1.NBTTagInt;
import org.bukkit.Color;
import org.bukkit.Material;
import org.bukkit.configuration.serialization.DelegateDeserialization;
import org.bukkit.inventory.meta.LeatherArmorMeta;




@DelegateDeserialization(CraftMetaItem.SerializableMeta.class)
class CraftMetaLeatherArmor
  extends CraftMetaItem
  implements LeatherArmorMeta
{
  static final CraftMetaItem.ItemMetaKey COLOR = new CraftMetaItem.ItemMetaKey("color");
  
  private Color color = CraftItemFactory.DEFAULT_LEATHER_COLOR;
  
  CraftMetaLeatherArmor(CraftMetaItem meta) {
    super(meta);
    if (!(meta instanceof CraftMetaLeatherArmor)) {
      return;
    }
    
    CraftMetaLeatherArmor armorMeta = (CraftMetaLeatherArmor)meta;
    color = color;
  }
  
  CraftMetaLeatherArmor(NBTTagCompound tag) {
    super(tag);
    if (tag.hasKey(DISPLAYNBT)) {
      NBTTagCompound display = tag.getCompound(DISPLAYNBT);
      if (display.hasKey(COLORNBT)) {
        color = Color.fromRGB(display.getInt(COLORNBT));
      }
    }
  }
  
  CraftMetaLeatherArmor(Map<String, Object> map) {
    super(map);
    setColor((Color)CraftMetaItem.SerializableMeta.getObject(Color.class, map, COLORBUKKIT, true));
  }
  
  void applyToItem(NBTTagCompound itemTag)
  {
    super.applyToItem(itemTag);
    
    if (hasColor()) {
      setDisplayTag(itemTag, COLORNBT, new NBTTagInt(color.asRGB()));
    }
  }
  
  boolean isEmpty()
  {
    return (super.isEmpty()) && (isLeatherArmorEmpty());
  }
  
  boolean isLeatherArmorEmpty() {
    return !hasColor();
  }
  
  boolean applicableTo(Material type)
  {
    switch (1.$SwitchMap$org$bukkit$Material[type.ordinal()]) {
    case 1: 
    case 2: 
    case 3: 
    case 4: 
      return true;
    }
    return false;
  }
  

  public CraftMetaLeatherArmor clone()
  {
    return (CraftMetaLeatherArmor)super.clone();
  }
  
  public Color getColor() {
    return color;
  }
  
  public void setColor(Color color) {
    this.color = (color == null ? CraftItemFactory.DEFAULT_LEATHER_COLOR : color);
  }
  
  boolean hasColor() {
    return !CraftItemFactory.DEFAULT_LEATHER_COLOR.equals(color);
  }
  
  ImmutableMap.Builder<String, Object> serialize(ImmutableMap.Builder<String, Object> builder)
  {
    super.serialize(builder);
    
    if (hasColor()) {
      builder.put(COLORBUKKIT, color);
    }
    
    return builder;
  }
  
  boolean equalsCommon(CraftMetaItem meta)
  {
    if (!super.equalsCommon(meta)) {
      return false;
    }
    if ((meta instanceof CraftMetaLeatherArmor)) {
      CraftMetaLeatherArmor that = (CraftMetaLeatherArmor)meta;
      
      return color.equals(color);
    }
    return true;
  }
  
  boolean notUncommon(CraftMetaItem meta)
  {
    return (super.notUncommon(meta)) && (((meta instanceof CraftMetaLeatherArmor)) || (isLeatherArmorEmpty()));
  }
  
  int applyHash()
  {
    int original;
    int hash = original = super.applyHash();
    if (hasColor()) {
      hash ^= color.hashCode();
    }
    return original != hash ? CraftMetaSkull.class.hashCode() ^ hash : hash;
  }
}
