package org.bukkit.craftbukkit.v1_7_R1.inventory;

import com.google.common.base.Strings;
import com.google.common.collect.ImmutableMap.Builder;
import java.util.Map;
import net.minecraft.server.v1_7_R1.NBTTagCompound;
import org.bukkit.Material;
import org.bukkit.configuration.serialization.DelegateDeserialization;
import org.bukkit.inventory.meta.SkullMeta;


@DelegateDeserialization(CraftMetaItem.SerializableMeta.class)
class CraftMetaSkull
  extends CraftMetaItem
  implements SkullMeta
{
  static final CraftMetaItem.ItemMetaKey SKULL_OWNER = new CraftMetaItem.ItemMetaKey("SkullOwner", "skull-owner");
  static final int MAX_OWNER_LENGTH = 16;
  private String player;
  
  CraftMetaSkull(CraftMetaItem meta)
  {
    super(meta);
    if (!(meta instanceof CraftMetaSkull)) {
      return;
    }
    CraftMetaSkull skullMeta = (CraftMetaSkull)meta;
    player = player;
  }
  
  CraftMetaSkull(NBTTagCompound tag) {
    super(tag);
    
    if (tag.hasKey(SKULL_OWNERNBT)) {
      player = tag.getString(SKULL_OWNERNBT);
    }
  }
  
  CraftMetaSkull(Map<String, Object> map) {
    super(map);
    setOwner(CraftMetaItem.SerializableMeta.getString(map, SKULL_OWNERBUKKIT, true));
  }
  
  void applyToItem(NBTTagCompound tag)
  {
    super.applyToItem(tag);
    
    if (hasOwner()) {
      tag.setString(SKULL_OWNERNBT, player);
    }
  }
  
  boolean isEmpty()
  {
    return (super.isEmpty()) && (isSkullEmpty());
  }
  
  boolean isSkullEmpty() {
    return !hasOwner();
  }
  
  boolean applicableTo(Material type)
  {
    switch (1.$SwitchMap$org$bukkit$Material[type.ordinal()]) {
    case 1: 
      return true;
    }
    return false;
  }
  

  public CraftMetaSkull clone()
  {
    return (CraftMetaSkull)super.clone();
  }
  
  public boolean hasOwner() {
    return !Strings.isNullOrEmpty(player);
  }
  
  public String getOwner() {
    return player;
  }
  
  public boolean setOwner(String name) {
    if ((name != null) && (name.length() > 16)) {
      return false;
    }
    player = name;
    return true;
  }
  
  int applyHash()
  {
    int original;
    int hash = original = super.applyHash();
    if (hasOwner()) {
      hash = 61 * hash + player.hashCode();
    }
    return original != hash ? CraftMetaSkull.class.hashCode() ^ hash : hash;
  }
  
  boolean equalsCommon(CraftMetaItem meta)
  {
    if (!super.equalsCommon(meta)) {
      return false;
    }
    if ((meta instanceof CraftMetaSkull)) {
      CraftMetaSkull that = (CraftMetaSkull)meta;
      
      return (that.hasOwner()) && (player.equals(player));
    }
    return true;
  }
  
  boolean notUncommon(CraftMetaItem meta)
  {
    return (super.notUncommon(meta)) && (((meta instanceof CraftMetaSkull)) || (isSkullEmpty()));
  }
  
  ImmutableMap.Builder<String, Object> serialize(ImmutableMap.Builder<String, Object> builder)
  {
    super.serialize(builder);
    if (hasOwner()) {
      return builder.put(SKULL_OWNERBUKKIT, player);
    }
    return builder;
  }
}
