package org.bukkit.craftbukkit.v1_7_R1.inventory;

import org.bukkit.inventory.Inventory;
import org.bukkit.inventory.ItemStack;

public class InventoryIterator implements java.util.ListIterator<ItemStack>
{
  private final Inventory inventory;
  private int nextIndex;
  private Boolean lastDirection;
  
  InventoryIterator(Inventory craftInventory)
  {
    inventory = craftInventory;
    nextIndex = 0;
  }
  
  InventoryIterator(Inventory craftInventory, int index) {
    inventory = craftInventory;
    nextIndex = index;
  }
  
  public boolean hasNext() {
    return nextIndex < inventory.getSize();
  }
  
  public ItemStack next() {
    lastDirection = Boolean.valueOf(true);
    return inventory.getItem(nextIndex++);
  }
  
  public int nextIndex() {
    return nextIndex;
  }
  
  public boolean hasPrevious() {
    return nextIndex > 0;
  }
  
  public ItemStack previous() {
    lastDirection = Boolean.valueOf(false);
    return inventory.getItem(--nextIndex);
  }
  
  public int previousIndex() {
    return nextIndex - 1;
  }
  
  public void set(ItemStack item) {
    if (lastDirection == null) {
      throw new IllegalStateException("No current item!");
    }
    int i = lastDirection.booleanValue() ? nextIndex - 1 : nextIndex;
    inventory.setItem(i, item);
  }
  
  public void add(ItemStack item) {
    throw new UnsupportedOperationException("Can't change the size of an inventory!");
  }
  
  public void remove() {
    throw new UnsupportedOperationException("Can't change the size of an inventory!");
  }
}
