package org.bukkit.craftbukkit.v1_7_R1.map;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import net.minecraft.server.v1_7_R1.WorldMap;
import net.minecraft.server.v1_7_R1.WorldServer;
import org.bukkit.World;
import org.bukkit.craftbukkit.v1_7_R1.CraftWorld;
import org.bukkit.craftbukkit.v1_7_R1.entity.CraftPlayer;
import org.bukkit.map.MapCursorCollection;
import org.bukkit.map.MapRenderer;
import org.bukkit.map.MapView;
import org.bukkit.map.MapView.Scale;

public final class CraftMapView implements MapView
{
  private final Map<CraftPlayer, RenderData> renderCache = new HashMap();
  private final List<MapRenderer> renderers = new ArrayList();
  private final Map<MapRenderer, Map<CraftPlayer, CraftMapCanvas>> canvases = new HashMap();
  protected final WorldMap worldMap;
  
  public CraftMapView(WorldMap worldMap) {
    this.worldMap = worldMap;
    addRenderer(new CraftMapRenderer(this, worldMap));
  }
  
  public short getId() {
    String text = worldMap.id;
    if (text.startsWith("map_")) {
      try {
        return Short.parseShort(text.substring("map_".length()));
      }
      catch (NumberFormatException ex) {
        throw new IllegalStateException("Map has non-numeric ID");
      }
    }
    throw new IllegalStateException("Map has invalid ID");
  }
  
  public boolean isVirtual()
  {
    return (renderers.size() > 0) && (!(renderers.get(0) instanceof CraftMapRenderer));
  }
  
  public MapView.Scale getScale() {
    return MapView.Scale.valueOf(worldMap.scale);
  }
  
  public void setScale(MapView.Scale scale) {
    worldMap.scale = scale.getValue();
  }
  
  public World getWorld() {
    byte dimension = worldMap.map;
    for (World world : org.bukkit.Bukkit.getServer().getWorlds()) {
      if (getHandle()dimension == dimension) {
        return world;
      }
    }
    return null;
  }
  
  public void setWorld(World world) {
    worldMap.map = ((byte)getHandle()dimension);
  }
  
  public int getCenterX() {
    return worldMap.centerX;
  }
  
  public int getCenterZ() {
    return worldMap.centerZ;
  }
  
  public void setCenterX(int x) {
    worldMap.centerX = x;
  }
  
  public void setCenterZ(int z) {
    worldMap.centerZ = z;
  }
  
  public List<MapRenderer> getRenderers() {
    return new ArrayList(renderers);
  }
  
  public void addRenderer(MapRenderer renderer) {
    if (!renderers.contains(renderer)) {
      renderers.add(renderer);
      canvases.put(renderer, new HashMap());
      renderer.initialize(this);
    }
  }
  
  public boolean removeRenderer(MapRenderer renderer) {
    if (renderers.contains(renderer)) {
      renderers.remove(renderer);
      for (Map.Entry<CraftPlayer, CraftMapCanvas> entry : ((Map)canvases.get(renderer)).entrySet()) {
        for (int x = 0; x < 128; x++) {
          for (int y = 0; y < 128; y++) {
            ((CraftMapCanvas)entry.getValue()).setPixel(x, y, (byte)-1);
          }
        }
      }
      canvases.remove(renderer);
      return true;
    }
    return false;
  }
  
  private boolean isContextual()
  {
    for (MapRenderer renderer : renderers) {
      if (renderer.isContextual()) return true;
    }
    return false;
  }
  
  public RenderData render(CraftPlayer player) {
    boolean context = isContextual();
    RenderData render = (RenderData)renderCache.get(context ? player : null);
    
    if (render == null) {
      render = new RenderData();
      renderCache.put(context ? player : null, render);
    }
    
    if ((context) && (renderCache.containsKey(null))) {
      renderCache.remove(null);
    }
    
    java.util.Arrays.fill(buffer, (byte)0);
    cursors.clear();
    
    for (MapRenderer renderer : renderers) {
      CraftMapCanvas canvas = (CraftMapCanvas)((Map)canvases.get(renderer)).get(renderer.isContextual() ? player : null);
      if (canvas == null) {
        canvas = new CraftMapCanvas(this);
        ((Map)canvases.get(renderer)).put(renderer.isContextual() ? player : null, canvas);
      }
      
      canvas.setBase(buffer);
      renderer.render(this, canvas, player);
      
      byte[] buf = canvas.getBuffer();
      for (int i = 0; i < buf.length; i++) {
        byte color = buf[i];
        
        if ((color >= 0) || (color <= -113)) { buffer[i] = color;
        }
      }
      for (int i = 0; i < canvas.getCursors().size(); i++) {
        cursors.add(canvas.getCursors().getCursor(i));
      }
    }
    
    return render;
  }
}
