package org.bukkit.craftbukkit.v1_7_R1.projectiles;

import java.util.Random;
import net.minecraft.server.v1_7_R1.BlockDispenser;
import net.minecraft.server.v1_7_R1.Entity;
import net.minecraft.server.v1_7_R1.EntityArrow;
import net.minecraft.server.v1_7_R1.EntityEnderPearl;
import net.minecraft.server.v1_7_R1.EntityFireball;
import net.minecraft.server.v1_7_R1.EntityLargeFireball;
import net.minecraft.server.v1_7_R1.EntityPotion;
import net.minecraft.server.v1_7_R1.EntityProjectile;
import net.minecraft.server.v1_7_R1.EntitySmallFireball;
import net.minecraft.server.v1_7_R1.EntitySnowball;
import net.minecraft.server.v1_7_R1.EntityThrownExpBottle;
import net.minecraft.server.v1_7_R1.EntityWitherSkull;
import net.minecraft.server.v1_7_R1.EnumFacing;
import net.minecraft.server.v1_7_R1.IPosition;
import net.minecraft.server.v1_7_R1.IProjectile;
import net.minecraft.server.v1_7_R1.MathHelper;
import net.minecraft.server.v1_7_R1.SourceBlock;
import net.minecraft.server.v1_7_R1.TileEntityDispenser;
import net.minecraft.server.v1_7_R1.World;
import org.apache.commons.lang.Validate;
import org.bukkit.Material;
import org.bukkit.block.Block;
import org.bukkit.craftbukkit.v1_7_R1.CraftWorld;
import org.bukkit.craftbukkit.v1_7_R1.inventory.CraftItemStack;
import org.bukkit.entity.Arrow;
import org.bukkit.entity.Egg;
import org.bukkit.entity.Projectile;
import org.bukkit.entity.SmallFireball;
import org.bukkit.entity.Snowball;
import org.bukkit.entity.ThrownExpBottle;
import org.bukkit.entity.ThrownPotion;
import org.bukkit.entity.WitherSkull;
import org.bukkit.inventory.ItemStack;
import org.bukkit.projectiles.BlockProjectileSource;
import org.bukkit.util.Vector;

public class CraftBlockProjectileSource implements BlockProjectileSource
{
  private final TileEntityDispenser dispenserBlock;
  
  public CraftBlockProjectileSource(TileEntityDispenser dispenserBlock)
  {
    this.dispenserBlock = dispenserBlock;
  }
  
  public Block getBlock()
  {
    return dispenserBlock.getWorld().getWorld().getBlockAt(dispenserBlock.x, dispenserBlock.y, dispenserBlock.z);
  }
  
  public <T extends Projectile> T launchProjectile(Class<? extends T> projectile)
  {
    return launchProjectile(projectile, null);
  }
  
  public <T extends Projectile> T launchProjectile(Class<? extends T> projectile, Vector velocity)
  {
    Validate.isTrue(getBlock().getType() == Material.DISPENSER, "Block is no longer dispenser");
    
    SourceBlock isourceblock = new SourceBlock(dispenserBlock.getWorld(), dispenserBlock.x, dispenserBlock.y, dispenserBlock.z);
    
    IPosition iposition = BlockDispenser.a(isourceblock);
    EnumFacing enumfacing = BlockDispenser.b(isourceblock.h());
    World world = dispenserBlock.getWorld();
    Entity launch = null;
    
    if (Snowball.class.isAssignableFrom(projectile)) {
      launch = new EntitySnowball(world, iposition.getX(), iposition.getY(), iposition.getZ());
    } else if (Egg.class.isAssignableFrom(projectile)) {
      launch = new net.minecraft.server.v1_7_R1.EntityEgg(world, iposition.getX(), iposition.getY(), iposition.getZ());
    } else if (org.bukkit.entity.EnderPearl.class.isAssignableFrom(projectile)) {
      launch = new EntityEnderPearl(world);
      launch.setPosition(iposition.getX(), iposition.getY(), iposition.getZ());
    } else if (ThrownExpBottle.class.isAssignableFrom(projectile)) {
      launch = new EntityThrownExpBottle(world, iposition.getX(), iposition.getY(), iposition.getZ());
    } else if (ThrownPotion.class.isAssignableFrom(projectile)) {
      launch = new EntityPotion(world, iposition.getX(), iposition.getY(), iposition.getZ(), CraftItemStack.asNMSCopy(new ItemStack(Material.POTION, 1)));
    } else if (Arrow.class.isAssignableFrom(projectile)) {
      launch = new EntityArrow(world, iposition.getX(), iposition.getY(), iposition.getZ());
      fromPlayer = 1;
      projectileSource = this;
    } else if (org.bukkit.entity.Fireball.class.isAssignableFrom(projectile)) {
      double d0 = iposition.getX() + enumfacing.c() * 0.3F;
      double d1 = iposition.getY() + enumfacing.c() * 0.3F;
      double d2 = iposition.getZ() + enumfacing.e() * 0.3F;
      Random random = random;
      double d3 = random.nextGaussian() * 0.05D + enumfacing.c();
      double d4 = random.nextGaussian() * 0.05D + enumfacing.d();
      double d5 = random.nextGaussian() * 0.05D + enumfacing.e();
      
      if (SmallFireball.class.isAssignableFrom(projectile)) {
        launch = new EntitySmallFireball(world, d0, d1, d2, d3, d4, d5);
      } else if (WitherSkull.class.isAssignableFrom(projectile)) {
        launch = new EntityWitherSkull(world);
        launch.setPosition(d0, d1, d2);
        double d6 = MathHelper.sqrt(d3 * d3 + d4 * d4 + d5 * d5);
        
        dirX = (d3 / d6 * 0.1D);
        dirY = (d4 / d6 * 0.1D);
        dirZ = (d5 / d6 * 0.1D);
      } else {
        launch = new EntityLargeFireball(world);
        launch.setPosition(d0, d1, d2);
        double d6 = MathHelper.sqrt(d3 * d3 + d4 * d4 + d5 * d5);
        
        dirX = (d3 / d6 * 0.1D);
        dirY = (d4 / d6 * 0.1D);
        dirZ = (d5 / d6 * 0.1D);
      }
      
      projectileSource = this;
    }
    
    Validate.notNull(launch, "Projectile not supported");
    
    if ((launch instanceof IProjectile)) {
      if ((launch instanceof EntityProjectile)) {
        projectileSource = this;
      }
      
      float a = 6.0F;
      float b = 1.1F;
      if (((launch instanceof EntityPotion)) || ((launch instanceof ThrownExpBottle)))
      {
        a *= 0.5F;
        b *= 1.25F;
      }
      
      ((IProjectile)launch).shoot(enumfacing.c(), enumfacing.d() + 0.1F, enumfacing.e(), b, a);
    }
    
    if (velocity != null) {
      ((Projectile)launch.getBukkitEntity()).setVelocity(velocity);
    }
    
    world.addEntity(launch);
    return (Projectile)launch.getBukkitEntity();
  }
}
