package org.bukkit.craftbukkit.v1_7_R1.scheduler;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;
import org.apache.commons.lang.UnhandledException;
import org.bukkit.plugin.Plugin;
import org.bukkit.plugin.PluginDescriptionFile;
import org.bukkit.scheduler.BukkitWorker;

class CraftAsyncTask
  extends CraftTask
{
  private final LinkedList<BukkitWorker> workers = new LinkedList();
  private final Map<Integer, CraftTask> runners;
  
  CraftAsyncTask(Map<Integer, CraftTask> runners, Plugin plugin, Runnable task, int id, long delay) {
    super(plugin, task, id, delay);
    this.runners = runners;
  }
  
  public boolean isSync()
  {
    return false;
  }
  
  public void run()
  {
    final Thread thread = Thread.currentThread();
    synchronized (this.workers) {
      if (getPeriod() == -2L)
      {

        return;
      }
      this.workers.add(new BukkitWorker()
      {
        public Thread getThread() {
          return thread;
        }
        
        public int getTaskId() {
          return CraftAsyncTask.this.getTaskId();
        }
        
        public Plugin getOwner() {
          return CraftAsyncTask.this.getOwner();
        }
      });
    }
    Throwable thrown = null;
    try {
      super.run(); } catch (Throwable t) { Iterator<BukkitWorker> workers;
      boolean removed;
      thrown = t;
      throw new UnhandledException(String.format("Plugin %s generated an exception while executing task %s", new Object[] { getOwner().getDescription().getFullName(), Integer.valueOf(getTaskId()) }), thrown);


    }
    finally
    {


      synchronized (this.workers) {
        try {
          Iterator<BukkitWorker> workers = this.workers.iterator();
          boolean removed = false;
          while (workers.hasNext()) {
            if (((BukkitWorker)workers.next()).getThread() == thread) {
              workers.remove();
              removed = true;
            }
          }
          
          if (!removed) {
            throw new IllegalStateException(String.format("Unable to remove worker %s on task %s for %s", new Object[] { thread.getName(), Integer.valueOf(getTaskId()), getOwner().getDescription().getFullName() }), thrown);
          }
          






          if ((getPeriod() < 0L) && (this.workers.isEmpty()))
          {

            runners.remove(Integer.valueOf(getTaskId()));
          }
        }
        finally
        {
          if ((getPeriod() < 0L) && (this.workers.isEmpty()))
          {

            runners.remove(Integer.valueOf(getTaskId()));
          }
        }
      }
    }
  }
  
  LinkedList<BukkitWorker> getWorkers() {
    return workers;
  }
  
  boolean cancel0() {
    synchronized (workers)
    {
      setPeriod(-2L);
      if (workers.isEmpty()) {
        runners.remove(Integer.valueOf(getTaskId()));
      }
    }
    return true;
  }
}
