package org.bukkit.craftbukkit.v1_7_R1.scheduler;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Comparator;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.PriorityQueue;
import java.util.concurrent.Callable;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicReference;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.apache.commons.lang.Validate;
import org.bukkit.plugin.IllegalPluginAccessException;
import org.bukkit.plugin.Plugin;
import org.bukkit.plugin.PluginDescriptionFile;
import org.bukkit.scheduler.BukkitScheduler;
import org.bukkit.scheduler.BukkitTask;
import org.bukkit.scheduler.BukkitWorker;


















public class CraftScheduler
  implements BukkitScheduler
{
  private final AtomicInteger ids = new AtomicInteger(1);
  


  private volatile CraftTask head = new CraftTask();
  


  private final AtomicReference<CraftTask> tail = new AtomicReference(head);
  


  private final PriorityQueue<CraftTask> pending = new PriorityQueue(10, new Comparator()
  {
    public int compare(CraftTask o1, CraftTask o2) {
      return (int)(o1.getNextRun() - o2.getNextRun());
    }
  });
  







  private final List<CraftTask> temp = new ArrayList();
  


  private final ConcurrentHashMap<Integer, CraftTask> runners = new ConcurrentHashMap();
  private volatile int currentTick = -1;
  private final Executor executor = Executors.newCachedThreadPool();
  private CraftAsyncDebugger debugHead = new CraftAsyncDebugger(-1, null, null) { StringBuilder debugTo(StringBuilder string) { return string; } };
  private CraftAsyncDebugger debugTail = debugHead;
  


  private static final int RECENT_TICKS = 30;
  
  public int scheduleSyncDelayedTask(Plugin plugin, Runnable task)
  {
    return scheduleSyncDelayedTask(plugin, task, 0L);
  }
  
  public BukkitTask runTask(Plugin plugin, Runnable runnable) {
    return runTaskLater(plugin, runnable, 0L);
  }
  
  public int scheduleAsyncDelayedTask(Plugin plugin, Runnable task) {
    return scheduleAsyncDelayedTask(plugin, task, 0L);
  }
  
  public BukkitTask runTaskAsynchronously(Plugin plugin, Runnable runnable) {
    return runTaskLaterAsynchronously(plugin, runnable, 0L);
  }
  
  public int scheduleSyncDelayedTask(Plugin plugin, Runnable task, long delay) {
    return scheduleSyncRepeatingTask(plugin, task, delay, -1L);
  }
  
  public BukkitTask runTaskLater(Plugin plugin, Runnable runnable, long delay) {
    return runTaskTimer(plugin, runnable, delay, -1L);
  }
  
  public int scheduleAsyncDelayedTask(Plugin plugin, Runnable task, long delay) {
    return scheduleAsyncRepeatingTask(plugin, task, delay, -1L);
  }
  
  public BukkitTask runTaskLaterAsynchronously(Plugin plugin, Runnable runnable, long delay) {
    return runTaskTimerAsynchronously(plugin, runnable, delay, -1L);
  }
  
  public int scheduleSyncRepeatingTask(Plugin plugin, Runnable runnable, long delay, long period) {
    return runTaskTimer(plugin, runnable, delay, period).getTaskId();
  }
  
  public BukkitTask runTaskTimer(Plugin plugin, Runnable runnable, long delay, long period) {
    validate(plugin, runnable);
    if (delay < 0L) {
      delay = 0L;
    }
    if (period == 0L) {
      period = 1L;
    } else if (period < -1L) {
      period = -1L;
    }
    return handle(new CraftTask(plugin, runnable, nextId(), period), delay);
  }
  
  public int scheduleAsyncRepeatingTask(Plugin plugin, Runnable runnable, long delay, long period) {
    return runTaskTimerAsynchronously(plugin, runnable, delay, period).getTaskId();
  }
  
  public BukkitTask runTaskTimerAsynchronously(Plugin plugin, Runnable runnable, long delay, long period) {
    validate(plugin, runnable);
    if (delay < 0L) {
      delay = 0L;
    }
    if (period == 0L) {
      period = 1L;
    } else if (period < -1L) {
      period = -1L;
    }
    return handle(new CraftAsyncTask(runners, plugin, runnable, nextId(), period), delay);
  }
  
  public <T> Future<T> callSyncMethod(Plugin plugin, Callable<T> task) {
    validate(plugin, task);
    CraftFuture<T> future = new CraftFuture(task, plugin, nextId());
    handle(future, 0L);
    return future;
  }
  
  public void cancelTask(final int taskId) {
    if (taskId <= 0) {
      return;
    }
    CraftTask task = (CraftTask)runners.get(Integer.valueOf(taskId));
    if (task != null) {
      task.cancel0();
    }
    task = new CraftTask(new Runnable()
    {
      public void run() {
        if (!check(temp))
          check(pending);
      }
      
      private boolean check(Iterable<CraftTask> collection) {
        Iterator<CraftTask> tasks = collection.iterator();
        while (tasks.hasNext()) {
          CraftTask task = (CraftTask)tasks.next();
          if (task.getTaskId() == taskId) {
            task.cancel0();
            tasks.remove();
            if (task.isSync()) {
              runners.remove(Integer.valueOf(taskId));
            }
            return true;
          }
        }
        return false;
      } });
    handle(task, 0L);
    for (CraftTask taskPending = head.getNext(); taskPending != null; taskPending = taskPending.getNext()) {
      if (taskPending == task) {
        return;
      }
      if (taskPending.getTaskId() == taskId) {
        taskPending.cancel0();
      }
    }
  }
  
  public void cancelTasks(final Plugin plugin) {
    Validate.notNull(plugin, "Cannot cancel tasks of null plugin");
    CraftTask task = new CraftTask(new Runnable()
    {
      public void run() {
        check(pending);
        check(temp);
      }
      
      void check(Iterable<CraftTask> collection) { Iterator<CraftTask> tasks = collection.iterator();
        while (tasks.hasNext()) {
          CraftTask task = (CraftTask)tasks.next();
          if (task.getOwner().equals(plugin)) {
            task.cancel0();
            tasks.remove();
            if (task.isSync()) {
              runners.remove(Integer.valueOf(task.getTaskId()));
            }
          }
        }
      }
    });
    handle(task, 0L);
    for (CraftTask taskPending = head.getNext(); taskPending != null; taskPending = taskPending.getNext()) {
      if (taskPending == task) {
        return;
      }
      if ((taskPending.getTaskId() != -1) && (taskPending.getOwner().equals(plugin))) {
        taskPending.cancel0();
      }
    }
    for (CraftTask runner : runners.values()) {
      if (runner.getOwner().equals(plugin)) {
        runner.cancel0();
      }
    }
  }
  
  public void cancelAllTasks() {
    CraftTask task = new CraftTask(new Runnable()
    {
      public void run() {
        Iterator<CraftTask> it = runners.values().iterator();
        while (it.hasNext()) {
          CraftTask task = (CraftTask)it.next();
          task.cancel0();
          if (task.isSync()) {
            it.remove();
          }
        }
        pending.clear();
        temp.clear();
      }
    });
    handle(task, 0L);
    for (CraftTask taskPending = head.getNext(); (taskPending != null) && 
          (taskPending != task); taskPending = taskPending.getNext())
    {


      taskPending.cancel0();
    }
    for (CraftTask runner : runners.values()) {
      runner.cancel0();
    }
  }
  
  public boolean isCurrentlyRunning(int taskId) {
    CraftTask task = (CraftTask)runners.get(Integer.valueOf(taskId));
    if ((task == null) || (task.isSync())) {
      return false;
    }
    CraftAsyncTask asyncTask = (CraftAsyncTask)task;
    synchronized (asyncTask.getWorkers()) {
      return asyncTask.getWorkers().isEmpty();
    }
  }
  
  public boolean isQueued(int taskId) {
    if (taskId <= 0) {
      return false;
    }
    for (CraftTask task = head.getNext(); task != null; task = task.getNext()) {
      if (task.getTaskId() == taskId) {
        return task.getPeriod() >= -1L;
      }
    }
    CraftTask task = (CraftTask)runners.get(Integer.valueOf(taskId));
    return (task != null) && (task.getPeriod() >= -1L);
  }
  
  public List<BukkitWorker> getActiveWorkers() {
    ArrayList<BukkitWorker> workers = new ArrayList();
    for (CraftTask taskObj : runners.values())
    {
      if (!taskObj.isSync())
      {

        CraftAsyncTask task = (CraftAsyncTask)taskObj;
        synchronized (task.getWorkers())
        {
          workers.addAll(task.getWorkers());
        }
      } }
    return workers;
  }
  
  public List<BukkitTask> getPendingTasks() {
    ArrayList<CraftTask> truePending = new ArrayList();
    for (CraftTask task = head.getNext(); task != null; task = task.getNext()) {
      if (task.getTaskId() != -1)
      {
        truePending.add(task);
      }
    }
    
    ArrayList<BukkitTask> pending = new ArrayList();
    for (CraftTask task : runners.values()) {
      if (task.getPeriod() >= -1L) {
        pending.add(task);
      }
    }
    
    for (CraftTask task : truePending) {
      if ((task.getPeriod() >= -1L) && (!pending.contains(task))) {
        pending.add(task);
      }
    }
    return pending;
  }
  


  public void mainThreadHeartbeat(int currentTick)
  {
    this.currentTick = currentTick;
    List<CraftTask> temp = this.temp;
    parsePending();
    while (isReady(currentTick)) {
      CraftTask task = (CraftTask)pending.remove();
      if (task.getPeriod() < -1L) {
        if (task.isSync()) {
          runners.remove(Integer.valueOf(task.getTaskId()), task);
        }
        parsePending();
      }
      else {
        if (task.isSync()) {
          try {
            task.run();
          } catch (Throwable throwable) {
            task.getOwner().getLogger().log(Level.WARNING, String.format("Task #%s for %s generated an exception", new Object[] { Integer.valueOf(task.getTaskId()), task.getOwner().getDescription().getFullName() }), throwable);
          }
          





          parsePending();
        } else {
          debugTail = debugTail.setNext(new CraftAsyncDebugger(currentTick + RECENT_TICKS, task.getOwner(), task.getTaskClass()));
          executor.execute(task);
        }
        

        long period = task.getPeriod();
        if (period > 0L) {
          task.setNextRun(currentTick + period);
          temp.add(task);
        } else if (task.isSync()) {
          runners.remove(Integer.valueOf(task.getTaskId()));
        }
      } }
    pending.addAll(temp);
    temp.clear();
    debugHead = debugHead.getNextHead(currentTick);
  }
  
  private void addTask(CraftTask task) {
    AtomicReference<CraftTask> tail = this.tail;
    CraftTask tailTask = (CraftTask)tail.get();
    while (!tail.compareAndSet(tailTask, task)) {
      tailTask = (CraftTask)tail.get();
    }
    tailTask.setNext(task);
  }
  
  private CraftTask handle(CraftTask task, long delay) {
    task.setNextRun(currentTick + delay);
    addTask(task);
    return task;
  }
  
  private static void validate(Plugin plugin, Object task) {
    Validate.notNull(plugin, "Plugin cannot be null");
    Validate.notNull(task, "Task cannot be null");
    if (!plugin.isEnabled()) {
      throw new IllegalPluginAccessException("Plugin attempted to register task while disabled");
    }
  }
  
  private int nextId() {
    return ids.incrementAndGet();
  }
  
  private void parsePending() {
    CraftTask head = this.head;
    CraftTask task = head.getNext();
    CraftTask lastTask = head;
    for (; task != null; task = (lastTask = task).getNext()) {
      if (task.getTaskId() == -1) {
        task.run();
      } else if (task.getPeriod() >= -1L) {
        pending.add(task);
        runners.put(Integer.valueOf(task.getTaskId()), task);
      }
    }
    

    for (task = head; task != lastTask; task = head) {
      head = task.getNext();
      task.setNext(null);
    }
    this.head = lastTask;
  }
  
  private boolean isReady(int currentTick) {
    return (!pending.isEmpty()) && (((CraftTask)pending.peek()).getNextRun() <= currentTick);
  }
  
  public String toString()
  {
    int debugTick = currentTick;
    StringBuilder string = new StringBuilder("Recent tasks from ").append(debugTick - RECENT_TICKS).append('-').append(debugTick).append('{');
    debugHead.debugTo(string);
    return '}';
  }
}
