package org.bukkit.craftbukkit.v1_7_R1.scoreboard;

import com.google.common.collect.ImmutableMap.Builder;
import java.util.Map;
import java.util.Map.Entry;
import net.minecraft.server.v1_7_R1.IScoreboardCriteria;
import net.minecraft.server.v1_7_R1.ScoreboardObjective;

final class CraftCriteria
{
  static final Map<String, CraftCriteria> DEFAULTS;
  
  static
  {
    ImmutableMap.Builder<String, CraftCriteria> defaults = com.google.common.collect.ImmutableMap.builder();
    
    for (Map.Entry<?, ?> entry : IScoreboardCriteria.a.entrySet()) {
      String name = entry.getKey().toString();
      IScoreboardCriteria criteria = (IScoreboardCriteria)entry.getValue();
      if (!criteria.getName().equals(name)) {
        throw new AssertionError("Unexpected entry " + name + " to criteria " + criteria + "(" + criteria.getName() + ")");
      }
      
      defaults.put(name, new CraftCriteria(criteria));
    }
    
    DEFAULTS = defaults.build(); }
  static final CraftCriteria DUMMY = (CraftCriteria)DEFAULTS.get("dummy");
  
  final IScoreboardCriteria criteria;
  final String bukkitName;
  
  private CraftCriteria(String bukkitName)
  {
    this.bukkitName = bukkitName;
    criteria = DUMMYcriteria;
  }
  
  private CraftCriteria(IScoreboardCriteria criteria) {
    this.criteria = criteria;
    bukkitName = criteria.getName();
  }
  
  static CraftCriteria getFromNMS(ScoreboardObjective objective) {
    return (CraftCriteria)DEFAULTS.get(objective.getCriteria().getName());
  }
  
  static CraftCriteria getFromBukkit(String name) {
    CraftCriteria criteria = (CraftCriteria)DEFAULTS.get(name);
    if (criteria != null) {
      return criteria;
    }
    return new CraftCriteria(name);
  }
  
  public boolean equals(Object that)
  {
    if (!(that instanceof CraftCriteria)) {
      return false;
    }
    return bukkitName.equals(bukkitName);
  }
  
  public int hashCode()
  {
    return bukkitName.hashCode() ^ CraftCriteria.class.hashCode();
  }
}
