package org.bukkit.craftbukkit.v1_7_R1.scoreboard;

import java.util.Map;
import net.minecraft.server.v1_7_R1.Scoreboard;
import net.minecraft.server.v1_7_R1.ScoreboardObjective;
import org.apache.commons.lang.Validate;
import org.bukkit.OfflinePlayer;
import org.bukkit.scoreboard.DisplaySlot;

final class CraftObjective extends CraftScoreboardComponent implements org.bukkit.scoreboard.Objective
{
  private final ScoreboardObjective objective;
  private final CraftCriteria criteria;
  
  CraftObjective(CraftScoreboard scoreboard, ScoreboardObjective objective)
  {
    super(scoreboard);
    this.objective = objective;
    criteria = CraftCriteria.getFromNMS(objective);
    
    objectives.put(objective.getName(), this);
  }
  
  ScoreboardObjective getHandle() {
    return objective;
  }
  
  public String getName() throws IllegalStateException {
    CraftScoreboard scoreboard = checkState();
    
    return objective.getName();
  }
  
  public String getDisplayName() throws IllegalStateException {
    CraftScoreboard scoreboard = checkState();
    
    return objective.getDisplayName();
  }
  
  public void setDisplayName(String displayName) throws IllegalStateException, IllegalArgumentException {
    Validate.notNull(displayName, "Display name cannot be null");
    Validate.isTrue(displayName.length() <= 32, "Display name '" + displayName + "' is longer than the limit of 32 characters");
    CraftScoreboard scoreboard = checkState();
    
    objective.setDisplayName(displayName);
  }
  
  public String getCriteria() throws IllegalStateException {
    CraftScoreboard scoreboard = checkState();
    
    return criteria.bukkitName;
  }
  
  public boolean isModifiable() throws IllegalStateException {
    CraftScoreboard scoreboard = checkState();
    
    return !criteria.criteria.isReadOnly();
  }
  
  public void setDisplaySlot(DisplaySlot slot) throws IllegalStateException {
    CraftScoreboard scoreboard = checkState();
    Scoreboard board = board;
    ScoreboardObjective objective = this.objective;
    
    for (int i = 0; i < 3; i++) {
      if (board.getObjectiveForSlot(i) == objective) {
        board.setDisplaySlot(i, null);
      }
    }
    if (slot != null) {
      int slotNumber = CraftScoreboardTranslations.fromBukkitSlot(slot);
      board.setDisplaySlot(slotNumber, getHandle());
    }
  }
  
  public DisplaySlot getDisplaySlot() throws IllegalStateException {
    CraftScoreboard scoreboard = checkState();
    Scoreboard board = board;
    ScoreboardObjective objective = this.objective;
    
    for (int i = 0; i < 3; i++) {
      if (board.getObjectiveForSlot(i) == objective) {
        return CraftScoreboardTranslations.toBukkitSlot(i);
      }
    }
    return null;
  }
  
  public org.bukkit.scoreboard.Score getScore(OfflinePlayer player) throws IllegalArgumentException, IllegalStateException {
    Validate.notNull(player, "Player cannot be null");
    CraftScoreboard scoreboard = checkState();
    
    return new CraftScore(this, player.getName());
  }
  
  public void unregister() throws IllegalStateException
  {
    CraftScoreboard scoreboard = checkState();
    
    objectives.remove(getName());
    board.unregisterObjective(objective);
    setUnregistered();
  }
}
