package org.bukkit.craftbukkit.v1_7_R1.scoreboard;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import net.minecraft.server.v1_7_R1.EntityPlayer;
import net.minecraft.server.v1_7_R1.IScoreboardCriteria;
import net.minecraft.server.v1_7_R1.MinecraftServer;
import net.minecraft.server.v1_7_R1.PacketPlayOutScoreboardObjective;
import net.minecraft.server.v1_7_R1.PacketPlayOutScoreboardTeam;
import net.minecraft.server.v1_7_R1.PlayerConnection;
import net.minecraft.server.v1_7_R1.PlayerList;
import net.minecraft.server.v1_7_R1.ScoreboardObjective;
import net.minecraft.server.v1_7_R1.ScoreboardScore;
import net.minecraft.server.v1_7_R1.ScoreboardServer;
import net.minecraft.server.v1_7_R1.ScoreboardTeam;
import org.apache.commons.lang.Validate;
import org.bukkit.craftbukkit.v1_7_R1.entity.CraftPlayer;
import org.bukkit.craftbukkit.v1_7_R1.util.WeakCollection;
import org.bukkit.entity.Player;
import org.bukkit.scoreboard.ScoreboardManager;

public final class CraftScoreboardManager implements ScoreboardManager
{
  private final CraftScoreboard mainScoreboard;
  private final MinecraftServer server;
  private final Collection<CraftScoreboard> scoreboards = new WeakCollection();
  private final Map<CraftPlayer, CraftScoreboard> playerBoards = new HashMap();
  
  public CraftScoreboardManager(MinecraftServer minecraftserver, net.minecraft.server.v1_7_R1.Scoreboard scoreboardServer) {
    mainScoreboard = new CraftScoreboard(scoreboardServer);
    server = minecraftserver;
    scoreboards.add(mainScoreboard);
  }
  
  public CraftScoreboard getMainScoreboard() {
    return mainScoreboard;
  }
  
  public CraftScoreboard getNewScoreboard() {
    CraftScoreboard scoreboard = new CraftScoreboard(new ScoreboardServer(server));
    scoreboards.add(scoreboard);
    return scoreboard;
  }
  
  public CraftScoreboard getPlayerBoard(CraftPlayer player)
  {
    CraftScoreboard board = (CraftScoreboard)playerBoards.get(player);
    return board == null ? getMainScoreboard() : board;
  }
  
  public void setPlayerBoard(CraftPlayer player, org.bukkit.scoreboard.Scoreboard bukkitScoreboard) throws IllegalArgumentException
  {
    Validate.isTrue(bukkitScoreboard instanceof CraftScoreboard, "Cannot set player scoreboard to an unregistered Scoreboard");
    
    CraftScoreboard scoreboard = (CraftScoreboard)bukkitScoreboard;
    net.minecraft.server.v1_7_R1.Scoreboard oldboard = getPlayerBoard(player).getHandle();
    net.minecraft.server.v1_7_R1.Scoreboard newboard = scoreboard.getHandle();
    EntityPlayer entityplayer = player.getHandle();
    
    if (oldboard == newboard) {
      return;
    }
    
    if (scoreboard == mainScoreboard) {
      playerBoards.remove(player);
    } else {
      playerBoards.put(player, scoreboard);
    }
    

    HashSet<ScoreboardObjective> removed = new HashSet();
    for (int i = 0; i < 3; i++) {
      ScoreboardObjective scoreboardobjective = oldboard.getObjectiveForSlot(i);
      if ((scoreboardobjective != null) && (!removed.contains(scoreboardobjective))) {
        playerConnection.sendPacket(new PacketPlayOutScoreboardObjective(scoreboardobjective, 1));
        removed.add(scoreboardobjective);
      }
    }
    

    Iterator<?> iterator = oldboard.getTeams().iterator();
    while (iterator.hasNext()) {
      ScoreboardTeam scoreboardteam = (ScoreboardTeam)iterator.next();
      playerConnection.sendPacket(new PacketPlayOutScoreboardTeam(scoreboardteam, 1));
    }
    

    server.getPlayerList().a((ScoreboardServer)newboard, player.getHandle());
  }
  
  public void removePlayer(Player player)
  {
    playerBoards.remove(player);
  }
  
  public Collection<ScoreboardScore> getScoreboardScores(IScoreboardCriteria criteria, String name, Collection<ScoreboardScore> collection)
  {
    for (CraftScoreboard scoreboard : scoreboards) {
      board = board;
      for (ScoreboardObjective objective : board.getObjectivesForCriteria(criteria))
        collection.add(board.getPlayerScoreForObjective(name, objective));
    }
    net.minecraft.server.v1_7_R1.Scoreboard board;
    return collection;
  }
  
  public void updateAllScoresForList(IScoreboardCriteria criteria, String name, List<EntityPlayer> of)
  {
    for (ScoreboardScore score : getScoreboardScores(criteria, name, new ArrayList())) {
      score.updateForList(of);
    }
  }
}
