package org.bukkit.craftbukkit.v1_7_R1.scoreboard;

import com.google.common.collect.ImmutableSet.Builder;
import java.util.Collection;
import java.util.Map;
import java.util.Set;
import net.minecraft.server.v1_7_R1.Scoreboard;
import net.minecraft.server.v1_7_R1.ScoreboardTeam;
import org.apache.commons.lang.Validate;
import org.bukkit.OfflinePlayer;

final class CraftTeam extends CraftScoreboardComponent implements org.bukkit.scoreboard.Team
{
  private final ScoreboardTeam team;
  
  CraftTeam(CraftScoreboard scoreboard, ScoreboardTeam team)
  {
    super(scoreboard);
    this.team = team;
    teams.put(team.getName(), this);
  }
  
  public String getName() throws IllegalStateException {
    CraftScoreboard scoreboard = checkState();
    
    return team.getName();
  }
  
  public String getDisplayName() throws IllegalStateException {
    CraftScoreboard scoreboard = checkState();
    
    return team.getDisplayName();
  }
  
  public void setDisplayName(String displayName) throws IllegalStateException {
    Validate.notNull(displayName, "Display name cannot be null");
    Validate.isTrue(displayName.length() <= 32, "Display name '" + displayName + "' is longer than the limit of 32 characters");
    CraftScoreboard scoreboard = checkState();
    
    team.setDisplayName(displayName);
  }
  
  public String getPrefix() throws IllegalStateException {
    CraftScoreboard scoreboard = checkState();
    
    return team.getPrefix();
  }
  
  public void setPrefix(String prefix) throws IllegalStateException, IllegalArgumentException {
    Validate.notNull(prefix, "Prefix cannot be null");
    Validate.isTrue(prefix.length() <= 32, "Prefix '" + prefix + "' is longer than the limit of 32 characters");
    CraftScoreboard scoreboard = checkState();
    
    team.setPrefix(prefix);
  }
  
  public String getSuffix() throws IllegalStateException {
    CraftScoreboard scoreboard = checkState();
    
    return team.getSuffix();
  }
  
  public void setSuffix(String suffix) throws IllegalStateException, IllegalArgumentException {
    Validate.notNull(suffix, "Suffix cannot be null");
    Validate.isTrue(suffix.length() <= 32, "Suffix '" + suffix + "' is longer than the limit of 32 characters");
    CraftScoreboard scoreboard = checkState();
    
    team.setSuffix(suffix);
  }
  
  public boolean allowFriendlyFire() throws IllegalStateException {
    CraftScoreboard scoreboard = checkState();
    
    return team.allowFriendlyFire();
  }
  
  public void setAllowFriendlyFire(boolean enabled) throws IllegalStateException {
    CraftScoreboard scoreboard = checkState();
    
    team.setAllowFriendlyFire(enabled);
  }
  
  public boolean canSeeFriendlyInvisibles() throws IllegalStateException {
    CraftScoreboard scoreboard = checkState();
    
    return team.canSeeFriendlyInvisibles();
  }
  
  public void setCanSeeFriendlyInvisibles(boolean enabled) throws IllegalStateException {
    CraftScoreboard scoreboard = checkState();
    
    team.setCanSeeFriendlyInvisibles(enabled);
  }
  
  public Set<OfflinePlayer> getPlayers() throws IllegalStateException {
    CraftScoreboard scoreboard = checkState();
    
    ImmutableSet.Builder<OfflinePlayer> players = com.google.common.collect.ImmutableSet.builder();
    for (Object o : team.getPlayerNameSet()) {
      players.add(org.bukkit.Bukkit.getOfflinePlayer(o.toString()));
    }
    return players.build();
  }
  
  public int getSize() throws IllegalStateException {
    CraftScoreboard scoreboard = checkState();
    
    return team.getPlayerNameSet().size();
  }
  
  public void addPlayer(OfflinePlayer player) throws IllegalStateException, IllegalArgumentException {
    Validate.notNull(player, "OfflinePlayer cannot be null");
    CraftScoreboard scoreboard = checkState();
    
    board.addPlayerToTeam(player.getName(), team.getName());
  }
  
  public boolean removePlayer(OfflinePlayer player) throws IllegalStateException, IllegalArgumentException {
    Validate.notNull(player, "OfflinePlayer cannot be null");
    CraftScoreboard scoreboard = checkState();
    
    if (!team.getPlayerNameSet().contains(player.getName())) {
      return false;
    }
    
    board.removePlayerFromTeam(player.getName(), team);
    return true;
  }
  
  public boolean hasPlayer(OfflinePlayer player) throws IllegalArgumentException, IllegalStateException {
    Validate.notNull(player, "OfflinePlayer cannot be null");
    CraftScoreboard scoreboard = checkState();
    
    return team.getPlayerNameSet().contains(player.getName());
  }
  
  public void unregister() throws IllegalStateException
  {
    CraftScoreboard scoreboard = checkState();
    
    board.removeTeam(team);
    teams.remove(team.getName());
    setUnregistered();
  }
}
