package org.bukkit.craftbukkit.v1_7_R1.util;

import com.google.common.collect.ImmutableMap;
import com.google.common.collect.ImmutableMap.Builder;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import net.minecraft.server.v1_7_R1.ChatClickable;
import net.minecraft.server.v1_7_R1.ChatComponentText;
import net.minecraft.server.v1_7_R1.ChatModifier;
import net.minecraft.server.v1_7_R1.EnumChatFormat;
import net.minecraft.server.v1_7_R1.EnumClickAction;
import net.minecraft.server.v1_7_R1.IChatBaseComponent;

public final class CraftChatMessage
{
  private static class StringMessage
  {
    private static final Map<Character, EnumChatFormat> formatMap;
    private static final Pattern INCREMENTAL_PATTERN = Pattern.compile("(" + String.valueOf('§') + "[0-9a-fk-or])|(\\n)|(?:(https?://[^ ][^ ]*?)(?=[\\.\\?!,;:]?(?:[ \\n]|$)))", 2);
    
    static {
      ImmutableMap.Builder<Character, EnumChatFormat> builder = ImmutableMap.builder();
      for (EnumChatFormat format : EnumChatFormat.values()) {
        builder.put(Character.valueOf(Character.toLowerCase(format.getChar())), format);
      }
      formatMap = builder.build();
    }
    
    private final List<IChatBaseComponent> list = new ArrayList();
    private IChatBaseComponent currentChatComponent = new ChatComponentText("");
    private ChatModifier modifier = new ChatModifier();
    private final IChatBaseComponent[] output;
    private int currentIndex;
    private final String message;
    
    private StringMessage(String message) {
      this.message = message;
      if (message == null) {
        output = new IChatBaseComponent[] { currentChatComponent };
        return;
      }
      list.add(currentChatComponent);
      
      Matcher matcher = INCREMENTAL_PATTERN.matcher(message);
      String match = null;
      while (matcher.find()) {
        int groupId = 0;
        while ((match = matcher.group(++groupId)) == null) {}
        

        appendNewComponent(matcher.start(groupId));
        switch (groupId) {
        case 1: 
          EnumChatFormat format = (EnumChatFormat)formatMap.get(Character.valueOf(match.toLowerCase().charAt(1)));
          if (format == EnumChatFormat.RESET) {
            modifier = new ChatModifier();
          } else if (format.isFormat()) {
            switch (CraftChatMessage.1.$SwitchMap$net$minecraft$server$EnumChatFormat[format.ordinal()]) {
            case 1: 
              modifier.setBold(Boolean.TRUE);
              break;
            case 2: 
              modifier.setItalic(Boolean.TRUE);
              break;
            case 3: 
              modifier.setStrikethrough(Boolean.TRUE);
              break;
            case 4: 
              modifier.setUnderline(Boolean.TRUE);
              break;
            case 5: 
              modifier.setRandom(Boolean.TRUE);
              break;
            default: 
              throw new AssertionError("Unexpected message format");
            }
          } else {
            modifier = new ChatModifier().setColor(format);
          }
          break;
        case 2: 
          currentChatComponent = null;
          break;
        case 3: 
          modifier.a(new ChatClickable(EnumClickAction.OPEN_URL, match));
          appendNewComponent(matcher.end(groupId));
          modifier.a((ChatClickable)null);
        }
        currentIndex = matcher.end(groupId);
      }
      
      if (currentIndex < message.length()) {
        appendNewComponent(message.length());
      }
      
      output = ((IChatBaseComponent[])list.toArray(new IChatBaseComponent[0]));
    }
    
    private void appendNewComponent(int index) {
      if (index <= currentIndex) {
        return;
      }
      IChatBaseComponent addition = new ChatComponentText(message.substring(currentIndex, index)).setChatModifier(modifier);
      currentIndex = index;
      modifier = modifier.clone();
      if (currentChatComponent == null) {
        currentChatComponent = new ChatComponentText("");
        list.add(currentChatComponent);
      }
      currentChatComponent.a(addition);
    }
    
    private IChatBaseComponent[] getOutput() {
      return output;
    }
  }
  
  public static IChatBaseComponent[] fromString(String message) {
    return new StringMessage(message, null).getOutput();
  }
}
