package org.bukkit.craftbukkit.v1_7_R1.util;

import java.util.ConcurrentModificationException;
import java.util.Iterator;
import java.util.NoSuchElementException;

















public class LongHashSet
{
  private static final int INITIAL_SIZE = 3;
  private static final double LOAD_FACTOR = 0.75D;
  private static final long FREE = 0L;
  private static final long REMOVED = -9223372036854775808L;
  private int freeEntries;
  private int elements;
  private long[] values;
  private int modCount;
  
  public LongHashSet()
  {
    this(3);
  }
  
  public LongHashSet(int size) {
    values = new long[size == 0 ? 1 : size];
    elements = 0;
    freeEntries = values.length;
    modCount = 0;
  }
  
  public Iterator iterator() {
    return new Itr();
  }
  
  public int size() {
    return elements;
  }
  
  public boolean isEmpty() {
    return elements == 0;
  }
  
  public boolean contains(int msw, int lsw) {
    return contains(LongHash.toLong(msw, lsw));
  }
  
  public boolean contains(long value) {
    int hash = hash(value);
    int index = (hash & 0x7FFFFFFF) % values.length;
    int offset = 1;
    

    while ((values[index] != 0L) && ((hash(values[index]) != hash) || (values[index] != value))) {
      index = (index + offset & 0x7FFFFFFF) % values.length;
      offset = offset * 2 + 1;
      
      if (offset == -1) {
        offset = 2;
      }
    }
    
    return values[index] != 0L;
  }
  
  public boolean add(int msw, int lsw) {
    return add(LongHash.toLong(msw, lsw));
  }
  
  public boolean add(long value) {
    int hash = hash(value);
    int index = (hash & 0x7FFFFFFF) % values.length;
    int offset = 1;
    int deletedix = -1;
    

    while ((values[index] != 0L) && ((hash(values[index]) != hash) || (values[index] != value)))
    {

      if (values[index] == -9223372036854775808L) {
        deletedix = index;
      }
      
      index = (index + offset & 0x7FFFFFFF) % values.length;
      offset = offset * 2 + 1;
      
      if (offset == -1) {
        offset = 2;
      }
    }
    
    if (values[index] == 0L) {
      if (deletedix != -1) {
        index = deletedix;
      } else {
        freeEntries -= 1;
      }
      
      modCount += 1;
      elements += 1;
      values[index] = value;
      
      if (1.0D - freeEntries / values.length > 0.75D) {
        rehash();
      }
      
      return true;
    }
    return false;
  }
  
  public void remove(int msw, int lsw)
  {
    remove(LongHash.toLong(msw, lsw));
  }
  
  public boolean remove(long value) {
    int hash = hash(value);
    int index = (hash & 0x7FFFFFFF) % values.length;
    int offset = 1;
    

    while ((values[index] != 0L) && ((hash(values[index]) != hash) || (values[index] != value))) {
      index = (index + offset & 0x7FFFFFFF) % values.length;
      offset = offset * 2 + 1;
      
      if (offset == -1) {
        offset = 2;
      }
    }
    
    if (values[index] != 0L) {
      values[index] = -9223372036854775808L;
      modCount += 1;
      elements -= 1;
      return true;
    }
    return false;
  }
  
  public void clear()
  {
    elements = 0;
    for (int ix = 0; ix < values.length; ix++) {
      values[ix] = 0L;
    }
    
    freeEntries = values.length;
    modCount += 1;
  }
  
  public long[] toArray() {
    long[] result = new long[elements];
    long[] values = Java15Compat.Arrays_copyOf(this.values, this.values.length);
    int pos = 0;
    
    for (long value : values) {
      if ((value != 0L) && (value != -9223372036854775808L)) {
        result[(pos++)] = value;
      }
    }
    
    return result;
  }
  
  public long popFirst() {
    for (long value : values) {
      if ((value != 0L) && (value != -9223372036854775808L)) {
        remove(value);
        return value;
      }
    }
    
    return 0L;
  }
  
  public long[] popAll() {
    long[] ret = toArray();
    clear();
    return ret;
  }
  
  private int hash(long value)
  {
    value ^= value >>> 33;
    value *= -49064778989728563L;
    value ^= value >>> 33;
    value *= -4265267296055464877L;
    value ^= value >>> 33;
    return (int)value;
  }
  
  private void rehash() {
    int gargagecells = values.length - (elements + freeEntries);
    if (gargagecells / values.length > 0.05D) {
      rehash(values.length);
    } else {
      rehash(values.length * 2 + 1);
    }
  }
  
  private void rehash(int newCapacity) {
    long[] newValues = new long[newCapacity];
    
    for (long value : values) {
      if ((value != 0L) && (value != -9223372036854775808L))
      {


        int hash = hash(value);
        int index = (hash & 0x7FFFFFFF) % newCapacity;
        int offset = 1;
        

        while (newValues[index] != 0L) {
          index = (index + offset & 0x7FFFFFFF) % newCapacity;
          offset = offset * 2 + 1;
          
          if (offset == -1) {
            offset = 2;
          }
        }
        
        newValues[index] = value;
      }
    }
    values = newValues;
    freeEntries = (values.length - elements);
  }
  
  private class Itr implements Iterator {
    private int index;
    private int lastReturned = -1;
    private int expectedModCount;
    
    public Itr() {
      for (index = 0; (index < values.length) && ((values[index] == 0L) || (values[index] == -9223372036854775808L)); index += 1) {}
      

      expectedModCount = modCount;
    }
    
    public boolean hasNext() {
      return index != values.length;
    }
    
    public Long next() {
      if (modCount != expectedModCount) {
        throw new ConcurrentModificationException();
      }
      
      int length = values.length;
      if (index >= length) {
        lastReturned = -2;
        throw new NoSuchElementException();
      }
      
      lastReturned = index;
      for (index += 1; (index < length) && ((values[index] == 0L) || (values[index] == -9223372036854775808L)); index += 1) {}
      


      if (values[lastReturned] == 0L) {
        return Long.valueOf(0L);
      }
      return Long.valueOf(values[lastReturned]);
    }
    
    public void remove()
    {
      if (modCount != expectedModCount) {
        throw new ConcurrentModificationException();
      }
      
      if ((lastReturned == -1) || (lastReturned == -2)) {
        throw new IllegalStateException();
      }
      
      if ((values[lastReturned] != 0L) && (values[lastReturned] != -9223372036854775808L)) {
        values[lastReturned] = -9223372036854775808L;
        LongHashSet.access$210(LongHashSet.this);
        LongHashSet.access$108(LongHashSet.this);
        expectedModCount = modCount;
      }
    }
  }
}
