package org.bukkit.craftbukkit.v1_7_R1.util;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.AbstractCollection;
import java.util.AbstractSet;
import java.util.Arrays;
import java.util.Collection;
import java.util.ConcurrentModificationException;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;
import java.util.NoSuchElementException;
import java.util.Set;


public class LongObjectHashMap<V>
  implements Cloneable, Serializable
{
  static final long serialVersionUID = 2841537710170573815L;
  private static final long EMPTY_KEY = -9223372036854775808L;
  private static final int BUCKET_SIZE = 4096;
  private transient long[][] keys;
  private transient V[][] values;
  private transient int modCount;
  private transient int size;
  
  public LongObjectHashMap()
  {
    initialize();
  }
  
  public LongObjectHashMap(Map<? extends Long, ? extends V> map) {
    this();
    putAll(map);
  }
  
  public int size() {
    return size;
  }
  
  public boolean isEmpty() {
    return size == 0;
  }
  
  public boolean containsKey(long key) {
    return get(key) != null;
  }
  
  public boolean containsValue(V value) {
    for (V val : values()) {
      if ((val == value) || (val.equals(value))) {
        return true;
      }
    }
    
    return false;
  }
  
  public V get(long key) {
    int index = (int)(keyIndex(key) & 0xFFF);
    long[] inner = keys[index];
    if (inner == null) { return null;
    }
    for (int i = 0; i < inner.length; i++) {
      long innerKey = inner[i];
      if (innerKey == -9223372036854775808L)
        return null;
      if (innerKey == key) {
        return values[index][i];
      }
    }
    
    return null;
  }
  
  public V put(long key, V value) {
    int index = (int)(keyIndex(key) & 0xFFF);
    long[] innerKeys = keys[index];
    V[] innerValues = values[index];
    modCount += 1;
    
    if (innerKeys == null)
    {
      long[] tmp55_53 = new long[8];innerKeys = tmp55_53;keys[index] = tmp55_53;
      Arrays.fill(innerKeys, -9223372036854775808L); Object[] 
        tmp81_78 = ((Object[])new Object[8]);innerValues = tmp81_78;values[index] = tmp81_78;
      innerKeys[0] = key;
      innerValues[0] = value;
      size += 1;
    }
    else {
      for (int i = 0; i < innerKeys.length; i++)
      {
        if (innerKeys[i] == -9223372036854775808L) {
          size += 1;
          innerKeys[i] = key;
          innerValues[i] = value;
          return null;
        }
        

        if (innerKeys[i] == key) {
          V oldValue = innerValues[i];
          innerKeys[i] = key;
          innerValues[i] = value;
          return oldValue;
        }
      }
      

      long[] tmp208_205 = Java15Compat.Arrays_copyOf(innerKeys, i << 1);innerKeys = tmp208_205;keys[index] = tmp208_205;
      Arrays.fill(innerKeys, i, innerKeys.length, -9223372036854775808L); Object[] 
        tmp240_237 = Java15Compat.Arrays_copyOf(innerValues, i << 1);innerValues = tmp240_237;values[index] = tmp240_237;
      innerKeys[i] = key;
      innerValues[i] = value;
      size += 1;
    }
    
    return null;
  }
  
  public V remove(long key) {
    int index = (int)(keyIndex(key) & 0xFFF);
    long[] inner = keys[index];
    if (inner == null) {
      return null;
    }
    
    for (int i = 0; i < inner.length; i++)
    {
      if (inner[i] == -9223372036854775808L) {
        break;
      }
      
      if (inner[i] == key) {
        V value = values[index][i];
        
        for (i++; i < inner.length; i++) {
          if (inner[i] == -9223372036854775808L) {
            break;
          }
          
          inner[(i - 1)] = inner[i];
          values[index][(i - 1)] = values[index][i];
        }
        
        inner[(i - 1)] = -9223372036854775808L;
        values[index][(i - 1)] = null;
        size -= 1;
        modCount += 1;
        return value;
      }
    }
    
    return null;
  }
  
  public void putAll(Map<? extends Long, ? extends V> map) {
    for (Map.Entry entry : map.entrySet()) {
      put(((Long)entry.getKey()).longValue(), entry.getValue());
    }
  }
  
  public void clear() {
    if (size == 0) {
      return;
    }
    
    modCount += 1;
    size = 0;
    Arrays.fill(keys, null);
    Arrays.fill(values, null);
  }
  
  public Set<Long> keySet() {
    return new KeySet(null);
  }
  
  public Collection<V> values() {
    return new ValueCollection(null);
  }
  







  @Deprecated
  public Set<Map.Entry<Long, V>> entrySet()
  {
    HashSet<Map.Entry<Long, V>> set = new HashSet();
    for (Iterator i$ = keySet().iterator(); i$.hasNext();) { long key = ((Long)i$.next()).longValue();
      set.add(new Entry(key, get(key)));
    }
    
    return set;
  }
  
  public Object clone() throws CloneNotSupportedException {
    LongObjectHashMap clone = (LongObjectHashMap)super.clone();
    
    clone.clear();
    
    clone.initialize();
    

    for (Iterator i$ = keySet().iterator(); i$.hasNext();) { long key = ((Long)i$.next()).longValue();
      V value = get(key);
      clone.put(key, value);
    }
    
    return clone;
  }
  
  private void initialize() {
    keys = new long[4096][];
    values = ((Object[][])new Object[4096][]);
  }
  
  private long keyIndex(long key) {
    key ^= key >>> 33;
    key *= -49064778989728563L;
    key ^= key >>> 33;
    key *= -4265267296055464877L;
    key ^= key >>> 33;
    return key;
  }
  
  private void writeObject(ObjectOutputStream outputStream) throws IOException {
    outputStream.defaultWriteObject();
    
    for (Iterator i$ = keySet().iterator(); i$.hasNext();) { long key = ((Long)i$.next()).longValue();
      V value = get(key);
      outputStream.writeLong(key);
      outputStream.writeObject(value);
    }
    
    outputStream.writeLong(-9223372036854775808L);
    outputStream.writeObject(null);
  }
  
  private void readObject(ObjectInputStream inputStream) throws ClassNotFoundException, IOException {
    inputStream.defaultReadObject();
    initialize();
    for (;;)
    {
      long key = inputStream.readLong();
      V value = inputStream.readObject();
      if ((key == -9223372036854775808L) && (value == null)) {
        break;
      }
      
      put(key, value);
    }
  }
  
  private class ValueIterator implements Iterator<V>
  {
    private int count;
    private int index;
    private int innerIndex;
    private int expectedModCount;
    private long lastReturned = -9223372036854775808L;
    
    long prevKey = -9223372036854775808L;
    V prevValue;
    
    ValueIterator() {
      expectedModCount = modCount;
    }
    
    public boolean hasNext() {
      return count < size;
    }
    
    public void remove() {
      if (modCount != expectedModCount) {
        throw new ConcurrentModificationException();
      }
      
      if (lastReturned == -9223372036854775808L) {
        throw new IllegalStateException();
      }
      
      count -= 1;
      remove(lastReturned);
      lastReturned = -9223372036854775808L;
      expectedModCount = modCount;
    }
    
    public V next() {
      if (modCount != expectedModCount) {
        throw new ConcurrentModificationException();
      }
      
      if (!hasNext()) {
        throw new NoSuchElementException();
      }
      
      long[][] keys = LongObjectHashMap.this.keys;
      count += 1;
      
      if (prevKey != -9223372036854775808L) {
        innerIndex += 1;
      }
      for (; 
          index < keys.length; index += 1) {
        if (keys[index] != null) {
          if (innerIndex < keys[index].length) {
            long key = keys[index][innerIndex];
            V value = values[index][innerIndex];
            if (key != -9223372036854775808L)
            {


              lastReturned = key;
              prevKey = key;
              prevValue = value;
              return prevValue;
            } }
          innerIndex = 0;
        }
      }
      
      throw new NoSuchElementException();
    }
  }
  
  private class KeyIterator implements Iterator<Long> {
    final LongObjectHashMap<V>.ValueIterator iterator;
    
    public KeyIterator() {
      iterator = new LongObjectHashMap.ValueIterator(LongObjectHashMap.this);
    }
    
    public void remove() {
      iterator.remove();
    }
    
    public boolean hasNext() {
      return iterator.hasNext();
    }
    
    public Long next() {
      iterator.next();
      return Long.valueOf(iterator.prevKey);
    }
  }
  
  private class KeySet extends AbstractSet<Long> {
    private KeySet() {}
    
    public void clear() { LongObjectHashMap.this.clear(); }
    
    public int size()
    {
      return LongObjectHashMap.this.size();
    }
    
    public boolean contains(Object key) {
      return ((key instanceof Long)) && (containsKey(((Long)key).longValue()));
    }
    
    public boolean remove(Object key)
    {
      return remove(((Long)key).longValue()) != null;
    }
    
    public Iterator<Long> iterator() {
      return new LongObjectHashMap.KeyIterator(LongObjectHashMap.this);
    }
  }
  
  private class ValueCollection extends AbstractCollection<V> {
    private ValueCollection() {}
    
    public void clear() { LongObjectHashMap.this.clear(); }
    
    public int size()
    {
      return LongObjectHashMap.this.size();
    }
    
    public boolean contains(Object value) {
      return containsValue(value);
    }
    
    public Iterator<V> iterator() {
      return new LongObjectHashMap.ValueIterator(LongObjectHashMap.this);
    }
  }
  
  private class Entry implements Map.Entry<Long, V>
  {
    private final Long key;
    private V value;
    
    Entry(V arg3) {
      key = Long.valueOf(k);
      value = v;
    }
    
    public Long getKey() {
      return key;
    }
    
    public V getValue() {
      return value;
    }
    
    public V setValue(V v) {
      V old = value;
      value = v;
      put(key.longValue(), v);
      return old;
    }
  }
}
