package org.bukkit.craftbukkit.v1_7_R1.util;

import java.util.concurrent.ExecutionException;

public abstract class Waitable<T> implements Runnable
{
  private static enum Status {
    WAITING, 
    RUNNING, 
    FINISHED;
    private Status() {} }
  Throwable t = null;
  T value = null;
  Status status = Status.WAITING;
  
  public final void run() {
    synchronized (this) {
      if (status != Status.WAITING) {
        throw new IllegalStateException("Invalid state " + status);
      }
      status = Status.RUNNING;
    }
    try {
      value = evaluate();
    } catch (Throwable t) {
      this.t = t;
    } finally {
      synchronized (this) {
        status = Status.FINISHED;
        notifyAll();
      }
    }
  }
  
  protected abstract T evaluate();
  
  public synchronized T get() throws InterruptedException, ExecutionException {
    while (status != Status.FINISHED) {
      wait();
    }
    if (t != null) {
      throw new ExecutionException(t);
    }
    return value;
  }
}
