package org.bukkit.material;

import java.util.Arrays;
import java.util.EnumSet;
import org.bukkit.Material;
import org.bukkit.block.BlockFace;



public class Vine
  extends MaterialData
{
  private static final int VINE_NORTH = 4;
  private static final int VINE_EAST = 8;
  private static final int VINE_WEST = 2;
  private static final int VINE_SOUTH = 1;
  EnumSet<BlockFace> possibleFaces = EnumSet.of(BlockFace.WEST, BlockFace.NORTH, BlockFace.SOUTH, BlockFace.EAST);
  
  public Vine() {
    super(Material.VINE);
  }
  



  @Deprecated
  public Vine(int type, byte data)
  {
    super(type, data);
  }
  



  @Deprecated
  public Vine(byte data)
  {
    super(Material.VINE, data);
  }
  
  public Vine(BlockFace... faces) {
    this(EnumSet.copyOf(Arrays.asList(faces)));
  }
  
  public Vine(EnumSet<BlockFace> faces) {
    this((byte)0);
    faces.retainAll(possibleFaces);
    
    byte data = 0;
    
    if (faces.contains(BlockFace.WEST)) {
      data = (byte)(data | 0x2);
    }
    
    if (faces.contains(BlockFace.NORTH)) {
      data = (byte)(data | 0x4);
    }
    
    if (faces.contains(BlockFace.SOUTH)) {
      data = (byte)(data | 0x1);
    }
    
    if (faces.contains(BlockFace.EAST)) {
      data = (byte)(data | 0x8);
    }
    
    setData(data);
  }
  







  public boolean isOnFace(BlockFace face)
  {
    switch (1.$SwitchMap$org$bukkit$block$BlockFace[face.ordinal()]) {
    case 1: 
      return (getData() & 0x2) == 2;
    case 2: 
      return (getData() & 0x4) == 4;
    case 3: 
      return (getData() & 0x1) == 1;
    case 4: 
      return (getData() & 0x8) == 8;
    case 5: 
      return (isOnFace(BlockFace.EAST)) && (isOnFace(BlockFace.NORTH));
    case 6: 
      return (isOnFace(BlockFace.WEST)) && (isOnFace(BlockFace.NORTH));
    case 7: 
      return (isOnFace(BlockFace.EAST)) && (isOnFace(BlockFace.SOUTH));
    case 8: 
      return (isOnFace(BlockFace.WEST)) && (isOnFace(BlockFace.SOUTH));
    case 9: 
      return true;
    }
    return false;
  }
  





  public void putOnFace(BlockFace face)
  {
    switch (1.$SwitchMap$org$bukkit$block$BlockFace[face.ordinal()]) {
    case 1: 
      setData((byte)(getData() | 0x2));
      break;
    case 2: 
      setData((byte)(getData() | 0x4));
      break;
    case 3: 
      setData((byte)(getData() | 0x1));
      break;
    case 4: 
      setData((byte)(getData() | 0x8));
      break;
    case 6: 
      putOnFace(BlockFace.WEST);
      putOnFace(BlockFace.NORTH);
      break;
    case 8: 
      putOnFace(BlockFace.WEST);
      putOnFace(BlockFace.SOUTH);
      break;
    case 5: 
      putOnFace(BlockFace.EAST);
      putOnFace(BlockFace.NORTH);
      break;
    case 7: 
      putOnFace(BlockFace.EAST);
      putOnFace(BlockFace.SOUTH);
      break;
    case 9: 
      break;
    default: 
      throw new IllegalArgumentException("Vines can't go on face " + face.toString());
    }
    
  }
  



  public void removeFromFace(BlockFace face)
  {
    switch (1.$SwitchMap$org$bukkit$block$BlockFace[face.ordinal()]) {
    case 1: 
      setData((byte)(getData() & 0xFFFFFFFD));
      break;
    case 2: 
      setData((byte)(getData() & 0xFFFFFFFB));
      break;
    case 3: 
      setData((byte)(getData() & 0xFFFFFFFE));
      break;
    case 4: 
      setData((byte)(getData() & 0xFFFFFFF7));
      break;
    case 6: 
      removeFromFace(BlockFace.WEST);
      removeFromFace(BlockFace.NORTH);
      break;
    case 8: 
      removeFromFace(BlockFace.WEST);
      removeFromFace(BlockFace.SOUTH);
      break;
    case 5: 
      removeFromFace(BlockFace.EAST);
      removeFromFace(BlockFace.NORTH);
      break;
    case 7: 
      removeFromFace(BlockFace.EAST);
      removeFromFace(BlockFace.SOUTH);
      break;
    case 9: 
      break;
    default: 
      throw new IllegalArgumentException("Vines can't go on face " + face.toString());
    }
  }
  
  public String toString()
  {
    return "VINE";
  }
  
  public Vine clone()
  {
    return (Vine)super.clone();
  }
}
