package org.bukkit.plugin;

import com.google.common.collect.ImmutableList;
import com.google.common.collect.ImmutableList.Builder;
import com.google.common.collect.ImmutableMap;
import com.google.common.collect.ImmutableMap.Builder;
import java.io.InputStream;
import java.io.Reader;
import java.io.Writer;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import org.bukkit.permissions.Permission;
import org.bukkit.permissions.PermissionDefault;
import org.yaml.snakeyaml.Yaml;
import org.yaml.snakeyaml.constructor.SafeConstructor;




















































































































































public final class PluginDescriptionFile
{
  private static final Yaml yaml = new Yaml(new SafeConstructor());
  private String name = null;
  private String main = null;
  private String classLoaderOf = null;
  private List<String> depend = null;
  private List<String> softDepend = null;
  private List<String> loadBefore = null;
  private String version = null;
  private Map<String, Map<String, Object>> commands = null;
  private String description = null;
  private List<String> authors = null;
  private String website = null;
  private String prefix = null;
  private boolean database = false;
  private PluginLoadOrder order = PluginLoadOrder.POSTWORLD;
  private List<Permission> permissions = null;
  private Map<?, ?> lazyPermissions = null;
  private PermissionDefault defaultPerm = PermissionDefault.OP;
  
  public PluginDescriptionFile(InputStream stream) throws InvalidDescriptionException {
    loadMap(asMap(yaml.load(stream)));
  }
  





  public PluginDescriptionFile(Reader reader)
    throws InvalidDescriptionException
  {
    loadMap(asMap(yaml.load(reader)));
  }
  






  public PluginDescriptionFile(String pluginName, String pluginVersion, String mainClass)
  {
    name = pluginName;
    version = pluginVersion;
    main = mainClass;
  }
  























  public String getName()
  {
    return name;
  }
  















  public String getVersion()
  {
    return version;
  }
  





















  public String getMain()
  {
    return main;
  }
  














  public String getDescription()
  {
    return description;
  }
  


















  public PluginLoadOrder getLoad()
  {
    return order;
  }
  






























  public List<String> getAuthors()
  {
    return authors;
  }
  














  public String getWebsite()
  {
    return website;
  }
  














  public boolean isDatabaseEnabled()
  {
    return database;
  }
  

























  public List<String> getDepend()
  {
    return depend;
  }
  
























  public List<String> getSoftDepend()
  {
    return softDepend;
  }
  
























  public List<String> getLoadBefore()
  {
    return loadBefore;
  }
  














  public String getPrefix()
  {
    return prefix;
  }
  















































































































  public Map<String, Map<String, Object>> getCommands()
  {
    return commands;
  }
  








































































































  public List<Permission> getPermissions()
  {
    if (permissions == null) {
      if (lazyPermissions == null) {
        permissions = ImmutableList.of();
      } else {
        permissions = ImmutableList.copyOf(Permission.loadPermissions(lazyPermissions, "Permission node '%s' in plugin description file for " + getFullName() + " is invalid", defaultPerm));
        lazyPermissions = null;
      }
    }
    return permissions;
  }
  
















  public PermissionDefault getPermissionDefault()
  {
    return defaultPerm;
  }
  






  public String getFullName()
  {
    return name + " v" + version;
  }
  


  @Deprecated
  public String getClassLoaderOf()
  {
    return classLoaderOf;
  }
  
  public void setDatabaseEnabled(boolean database) {
    this.database = database;
  }
  




  public void save(Writer writer)
  {
    yaml.dump(saveMap(), writer);
  }
  
  private void loadMap(Map<?, ?> map) throws InvalidDescriptionException {
    try {
      name = map.get("name").toString();
      
      if (!name.matches("^[A-Za-z0-9 _.-]+$")) {
        throw new InvalidDescriptionException("name '" + name + "' contains invalid characters.");
      }
    } catch (NullPointerException ex) {
      throw new InvalidDescriptionException(ex, "name is not defined");
    } catch (ClassCastException ex) {
      throw new InvalidDescriptionException(ex, "name is of wrong type");
    }
    try
    {
      version = map.get("version").toString();
    } catch (NullPointerException ex) {
      throw new InvalidDescriptionException(ex, "version is not defined");
    } catch (ClassCastException ex) {
      throw new InvalidDescriptionException(ex, "version is of wrong type");
    }
    try
    {
      main = map.get("main").toString();
      if (main.startsWith("org.bukkit.")) {
        throw new InvalidDescriptionException("main may not be within the org.bukkit namespace");
      }
    } catch (NullPointerException ex) {
      throw new InvalidDescriptionException(ex, "main is not defined");
    } catch (ClassCastException ex) {
      throw new InvalidDescriptionException(ex, "main is of wrong type");
    }
    
    if (map.get("commands") != null) {
      ImmutableMap.Builder<String, Map<String, Object>> commandsBuilder = ImmutableMap.builder();
      try {
        for (Map.Entry<?, ?> command : ((Map)map.get("commands")).entrySet()) {
          ImmutableMap.Builder<String, Object> commandBuilder = ImmutableMap.builder();
          if (command.getValue() != null) {
            for (Map.Entry<?, ?> commandEntry : ((Map)command.getValue()).entrySet()) {
              if ((commandEntry.getValue() instanceof Iterable))
              {
                ImmutableList.Builder<Object> commandSubList = ImmutableList.builder();
                for (Object commandSubListItem : (Iterable)commandEntry.getValue()) {
                  if (commandSubListItem != null) {
                    commandSubList.add(commandSubListItem);
                  }
                }
                commandBuilder.put(commandEntry.getKey().toString(), commandSubList.build());
              } else if (commandEntry.getValue() != null) {
                commandBuilder.put(commandEntry.getKey().toString(), commandEntry.getValue());
              }
            }
          }
          commandsBuilder.put(command.getKey().toString(), commandBuilder.build());
        }
      } catch (ClassCastException ex) {
        throw new InvalidDescriptionException(ex, "commands are of wrong type");
      }
      commands = commandsBuilder.build();
    }
    
    if (map.get("class-loader-of") != null) {
      classLoaderOf = map.get("class-loader-of").toString();
    }
    
    if (map.get("depend") != null) {
      ImmutableList.Builder<String> dependBuilder = ImmutableList.builder();
      try {
        for (Object dependency : (Iterable)map.get("depend")) {
          dependBuilder.add(dependency.toString());
        }
      } catch (ClassCastException ex) {
        throw new InvalidDescriptionException(ex, "depend is of wrong type");
      } catch (NullPointerException e) {
        throw new InvalidDescriptionException(e, "invalid dependency format");
      }
      depend = dependBuilder.build();
    }
    
    if (map.get("softdepend") != null) {
      ImmutableList.Builder<String> softDependBuilder = ImmutableList.builder();
      try {
        for (Object dependency : (Iterable)map.get("softdepend")) {
          softDependBuilder.add(dependency.toString());
        }
      } catch (ClassCastException ex) {
        throw new InvalidDescriptionException(ex, "softdepend is of wrong type");
      } catch (NullPointerException ex) {
        throw new InvalidDescriptionException(ex, "invalid soft-dependency format");
      }
      softDepend = softDependBuilder.build();
    }
    
    if (map.get("loadbefore") != null) {
      ImmutableList.Builder<String> loadBeforeBuilder = ImmutableList.builder();
      try {
        for (Object predependency : (Iterable)map.get("loadbefore")) {
          loadBeforeBuilder.add(predependency.toString());
        }
      } catch (ClassCastException ex) {
        throw new InvalidDescriptionException(ex, "loadbefore is of wrong type");
      } catch (NullPointerException ex) {
        throw new InvalidDescriptionException(ex, "invalid load-before format");
      }
      loadBefore = loadBeforeBuilder.build();
    }
    
    if (map.get("database") != null) {
      try {
        database = ((Boolean)map.get("database")).booleanValue();
      } catch (ClassCastException ex) {
        throw new InvalidDescriptionException(ex, "database is of wrong type");
      }
    }
    
    if (map.get("website") != null) {
      website = map.get("website").toString();
    }
    
    if (map.get("description") != null) {
      description = map.get("description").toString();
    }
    
    if (map.get("load") != null) {
      try {
        order = PluginLoadOrder.valueOf(((String)map.get("load")).toUpperCase().replaceAll("\\W", ""));
      } catch (ClassCastException ex) {
        throw new InvalidDescriptionException(ex, "load is of wrong type");
      } catch (IllegalArgumentException ex) {
        throw new InvalidDescriptionException(ex, "load is not a valid choice");
      }
    }
    
    if (map.get("authors") != null) {
      ImmutableList.Builder<String> authorsBuilder = ImmutableList.builder();
      if (map.get("author") != null) {
        authorsBuilder.add(map.get("author").toString());
      }
      try {
        for (Object o : (Iterable)map.get("authors")) {
          authorsBuilder.add(o.toString());
        }
      } catch (ClassCastException ex) {
        throw new InvalidDescriptionException(ex, "authors are of wrong type");
      } catch (NullPointerException ex) {
        throw new InvalidDescriptionException(ex, "authors are improperly defined");
      }
      authors = authorsBuilder.build();
    } else if (map.get("author") != null) {
      authors = ImmutableList.of(map.get("author").toString());
    } else {
      authors = ImmutableList.of();
    }
    
    if (map.get("default-permission") != null) {
      try {
        defaultPerm = PermissionDefault.getByName(map.get("default-permission").toString());
      } catch (ClassCastException ex) {
        throw new InvalidDescriptionException(ex, "default-permission is of wrong type");
      } catch (IllegalArgumentException ex) {
        throw new InvalidDescriptionException(ex, "default-permission is not a valid choice");
      }
    }
    try
    {
      lazyPermissions = ((Map)map.get("permissions"));
    } catch (ClassCastException ex) {
      throw new InvalidDescriptionException(ex, "permissions are of the wrong type");
    }
    
    if (map.get("prefix") != null) {
      prefix = map.get("prefix").toString();
    }
  }
  
  private Map<String, Object> saveMap() {
    Map<String, Object> map = new HashMap();
    
    map.put("name", name);
    map.put("main", main);
    map.put("version", version);
    map.put("database", Boolean.valueOf(database));
    map.put("order", order.toString());
    map.put("default-permission", defaultPerm.toString());
    
    if (commands != null) {
      map.put("command", commands);
    }
    if (depend != null) {
      map.put("depend", depend);
    }
    if (softDepend != null) {
      map.put("softdepend", softDepend);
    }
    if (website != null) {
      map.put("website", website);
    }
    if (description != null) {
      map.put("description", description);
    }
    
    if (authors.size() == 1) {
      map.put("author", authors.get(0));
    } else if (authors.size() > 1) {
      map.put("authors", authors);
    }
    
    if (classLoaderOf != null) {
      map.put("class-loader-of", classLoaderOf);
    }
    
    if (prefix != null) {
      map.put("prefix", prefix);
    }
    
    return map;
  }
  
  private Map<?, ?> asMap(Object object) throws InvalidDescriptionException {
    if ((object instanceof Map)) {
      return (Map)object;
    }
    throw new InvalidDescriptionException(object + " is not properly structured.");
  }
}
