package org.bukkit.plugin;

import org.bukkit.event.Event;
import org.bukkit.event.EventException;
import org.bukkit.event.EventPriority;
import org.bukkit.event.Listener;


public class TimedRegisteredListener
  extends RegisteredListener
{
  private int count;
  private long totalTime;
  private Class<? extends Event> eventClass;
  private boolean multiple = false;
  
  public TimedRegisteredListener(Listener pluginListener, EventExecutor eventExecutor, EventPriority eventPriority, Plugin registeredPlugin, boolean listenCancelled) {
    super(pluginListener, eventExecutor, eventPriority, registeredPlugin, listenCancelled);
  }
  
  public void callEvent(Event event) throws EventException
  {
    if (event.isAsynchronous()) {
      super.callEvent(event);
      return;
    }
    count += 1;
    Class<? extends Event> newEventClass = event.getClass();
    if (eventClass == null) {
      eventClass = newEventClass;
    } else if (!eventClass.equals(newEventClass)) {
      multiple = true;
      eventClass = getCommonSuperclass(newEventClass, eventClass).asSubclass(Event.class);
    }
    long start = System.nanoTime();
    super.callEvent(event);
    totalTime += System.nanoTime() - start;
  }
  
  private static Class<?> getCommonSuperclass(Class<?> class1, Class<?> class2) {
    while (!class1.isAssignableFrom(class2)) {
      class1 = class1.getSuperclass();
    }
    return class1;
  }
  


  public void reset()
  {
    count = 0;
    totalTime = 0L;
  }
  




  public int getCount()
  {
    return count;
  }
  




  public long getTotalTime()
  {
    return totalTime;
  }
  










  public Class<? extends Event> getEventClass()
  {
    return eventClass;
  }
  





  public boolean hasMultiple()
  {
    return multiple;
  }
}
