package org.bukkit.plugin.messaging;

import org.bukkit.plugin.Plugin;



public final class PluginMessageListenerRegistration
{
  private final Messenger messenger;
  private final Plugin plugin;
  private final String channel;
  private final PluginMessageListener listener;
  
  public PluginMessageListenerRegistration(Messenger messenger, Plugin plugin, String channel, PluginMessageListener listener)
  {
    if (messenger == null) {
      throw new IllegalArgumentException("Messenger cannot be null!");
    }
    if (plugin == null) {
      throw new IllegalArgumentException("Plugin cannot be null!");
    }
    if (channel == null) {
      throw new IllegalArgumentException("Channel cannot be null!");
    }
    if (listener == null) {
      throw new IllegalArgumentException("Listener cannot be null!");
    }
    
    this.messenger = messenger;
    this.plugin = plugin;
    this.channel = channel;
    this.listener = listener;
  }
  




  public String getChannel()
  {
    return channel;
  }
  




  public PluginMessageListener getListener()
  {
    return listener;
  }
  




  public Plugin getPlugin()
  {
    return plugin;
  }
  




  public boolean isValid()
  {
    return messenger.isRegistrationValid(this);
  }
  
  public boolean equals(Object obj)
  {
    if (obj == null) {
      return false;
    }
    if (getClass() != obj.getClass()) {
      return false;
    }
    PluginMessageListenerRegistration other = (PluginMessageListenerRegistration)obj;
    if ((messenger != messenger) && ((messenger == null) || (!messenger.equals(messenger)))) {
      return false;
    }
    if ((plugin != plugin) && ((plugin == null) || (!plugin.equals(plugin)))) {
      return false;
    }
    if (channel == null ? channel != null : !channel.equals(channel)) {
      return false;
    }
    if ((listener != listener) && ((listener == null) || (!listener.equals(listener)))) {
      return false;
    }
    return true;
  }
  
  public int hashCode()
  {
    int hash = 7;
    hash = 53 * hash + (messenger != null ? messenger.hashCode() : 0);
    hash = 53 * hash + (plugin != null ? plugin.hashCode() : 0);
    hash = 53 * hash + (channel != null ? channel.hashCode() : 0);
    hash = 53 * hash + (listener != null ? listener.hashCode() : 0);
    return hash;
  }
}
