package org.bukkit.potion;

import com.google.common.collect.ImmutableMap;
import java.util.Map;
import java.util.NoSuchElementException;
import org.apache.commons.lang.Validate;
import org.bukkit.configuration.serialization.ConfigurationSerializable;
import org.bukkit.configuration.serialization.SerializableAs;
import org.bukkit.entity.LivingEntity;















@SerializableAs("PotionEffect")
public class PotionEffect
  implements ConfigurationSerializable
{
  private static final String AMPLIFIER = "amplifier";
  private static final String DURATION = "duration";
  private static final String TYPE = "effect";
  private static final String AMBIENT = "ambient";
  private final int amplifier;
  private final int duration;
  private final PotionEffectType type;
  private final boolean ambient;
  
  public PotionEffect(PotionEffectType type, int duration, int amplifier, boolean ambient)
  {
    Validate.notNull(type, "effect type cannot be null");
    this.type = type;
    this.duration = duration;
    this.amplifier = amplifier;
    this.ambient = ambient;
  }
  







  public PotionEffect(PotionEffectType type, int duration, int amplifier)
  {
    this(type, duration, amplifier, true);
  }
  




  public PotionEffect(Map<String, Object> map)
  {
    this(getEffectType(map), getInt(map, "duration"), getInt(map, "amplifier"), getBool(map, "ambient"));
  }
  
  private static PotionEffectType getEffectType(Map<?, ?> map) {
    int type = getInt(map, "effect");
    PotionEffectType effect = PotionEffectType.getById(type);
    if (effect != null) {
      return effect;
    }
    throw new NoSuchElementException(map + " does not contain " + "effect");
  }
  
  private static int getInt(Map<?, ?> map, Object key) {
    Object num = map.get(key);
    if ((num instanceof Integer)) {
      return ((Integer)num).intValue();
    }
    throw new NoSuchElementException(map + " does not contain " + key);
  }
  
  private static boolean getBool(Map<?, ?> map, Object key) {
    Object bool = map.get(key);
    if ((bool instanceof Boolean)) {
      return ((Boolean)bool).booleanValue();
    }
    throw new NoSuchElementException(map + " does not contain " + key);
  }
  
  public Map<String, Object> serialize() {
    return ImmutableMap.of("effect", Integer.valueOf(type.getId()), "duration", Integer.valueOf(duration), "amplifier", Integer.valueOf(amplifier), "ambient", Boolean.valueOf(ambient));
  }
  












  public boolean apply(LivingEntity entity)
  {
    return entity.addPotionEffect(this);
  }
  
  public boolean equals(Object obj)
  {
    if (this == obj) {
      return true;
    }
    if (!(obj instanceof PotionEffect)) {
      return false;
    }
    PotionEffect that = (PotionEffect)obj;
    return (type.equals(type)) && (ambient == ambient) && (amplifier == amplifier) && (duration == duration);
  }
  






  public int getAmplifier()
  {
    return amplifier;
  }
  





  public int getDuration()
  {
    return duration;
  }
  




  public PotionEffectType getType()
  {
    return type;
  }
  




  public boolean isAmbient()
  {
    return ambient;
  }
  
  public int hashCode()
  {
    int hash = 1;
    hash = hash * 31 + type.hashCode();
    hash = hash * 31 + amplifier;
    hash = hash * 31 + duration;
    hash ^= 572662306 >> (ambient ? 1 : -1);
    return hash;
  }
  
  public String toString()
  {
    return type.getName() + (ambient ? ":(" : ":") + duration + "t-x" + amplifier + (ambient ? ")" : "");
  }
}
