package org.bukkit.util;

import java.util.Iterator;
import java.util.NoSuchElementException;
import org.bukkit.Location;
import org.bukkit.World;
import org.bukkit.block.Block;
import org.bukkit.block.BlockFace;
import org.bukkit.entity.LivingEntity;








public class BlockIterator
  implements Iterator<Block>
{
  private final World world;
  private final int maxDistance;
  private static final int gridSize = 16777216;
  private boolean end = false;
  
  private Block[] blockQueue = new Block[3];
  private int currentBlock = 0;
  private int currentDistance = 0;
  

  private int maxDistanceInt;
  

  private int secondError;
  

  private int thirdError;
  

  private int secondStep;
  

  private int thirdStep;
  

  private BlockFace mainFace;
  
  private BlockFace secondFace;
  
  private BlockFace thirdFace;
  

  public BlockIterator(World world, Vector start, Vector direction, double yOffset, int maxDistance)
  {
    this.world = world;
    this.maxDistance = maxDistance;
    
    Vector startClone = start.clone();
    
    startClone.setY(startClone.getY() + yOffset);
    
    currentDistance = 0;
    
    double mainDirection = 0.0D;
    double secondDirection = 0.0D;
    double thirdDirection = 0.0D;
    
    double mainPosition = 0.0D;
    double secondPosition = 0.0D;
    double thirdPosition = 0.0D;
    
    Block startBlock = this.world.getBlockAt(NumberConversions.floor(startClone.getX()), NumberConversions.floor(startClone.getY()), NumberConversions.floor(startClone.getZ()));
    
    if (getXLength(direction) > mainDirection) {
      mainFace = getXFace(direction);
      mainDirection = getXLength(direction);
      mainPosition = getXPosition(direction, startClone, startBlock);
      
      secondFace = getYFace(direction);
      secondDirection = getYLength(direction);
      secondPosition = getYPosition(direction, startClone, startBlock);
      
      thirdFace = getZFace(direction);
      thirdDirection = getZLength(direction);
      thirdPosition = getZPosition(direction, startClone, startBlock);
    }
    if (getYLength(direction) > mainDirection) {
      mainFace = getYFace(direction);
      mainDirection = getYLength(direction);
      mainPosition = getYPosition(direction, startClone, startBlock);
      
      secondFace = getZFace(direction);
      secondDirection = getZLength(direction);
      secondPosition = getZPosition(direction, startClone, startBlock);
      
      thirdFace = getXFace(direction);
      thirdDirection = getXLength(direction);
      thirdPosition = getXPosition(direction, startClone, startBlock);
    }
    if (getZLength(direction) > mainDirection) {
      mainFace = getZFace(direction);
      mainDirection = getZLength(direction);
      mainPosition = getZPosition(direction, startClone, startBlock);
      
      secondFace = getXFace(direction);
      secondDirection = getXLength(direction);
      secondPosition = getXPosition(direction, startClone, startBlock);
      
      thirdFace = getYFace(direction);
      thirdDirection = getYLength(direction);
      thirdPosition = getYPosition(direction, startClone, startBlock);
    }
    


    double d = mainPosition / mainDirection;
    double secondd = secondPosition - secondDirection * d;
    double thirdd = thirdPosition - thirdDirection * d;
    



    secondError = NumberConversions.floor(secondd * 16777216.0D);
    secondStep = NumberConversions.round(secondDirection / mainDirection * 16777216.0D);
    thirdError = NumberConversions.floor(thirdd * 16777216.0D);
    thirdStep = NumberConversions.round(thirdDirection / mainDirection * 16777216.0D);
    
    if (secondError + secondStep <= 0) {
      secondError = (-secondStep + 1);
    }
    
    if (thirdError + thirdStep <= 0) {
      thirdError = (-thirdStep + 1);
    }
    


    Block lastBlock = startBlock.getRelative(mainFace.getOppositeFace());
    
    if (secondError < 0) {
      secondError += 16777216;
      lastBlock = lastBlock.getRelative(secondFace.getOppositeFace());
    }
    
    if (thirdError < 0) {
      thirdError += 16777216;
      lastBlock = lastBlock.getRelative(thirdFace.getOppositeFace());
    }
    

    secondError -= 16777216;
    thirdError -= 16777216;
    
    blockQueue[0] = lastBlock;
    currentBlock = -1;
    
    scan();
    
    boolean startBlockFound = false;
    
    for (int cnt = currentBlock; cnt >= 0; cnt--) {
      if (blockEquals(blockQueue[cnt], startBlock)) {
        currentBlock = cnt;
        startBlockFound = true;
        break;
      }
    }
    
    if (!startBlockFound) {
      throw new IllegalStateException("Start block missed in BlockIterator");
    }
    

    maxDistanceInt = NumberConversions.round(maxDistance / (Math.sqrt(mainDirection * mainDirection + secondDirection * secondDirection + thirdDirection * thirdDirection) / mainDirection));
  }
  
  private boolean blockEquals(Block a, Block b)
  {
    return (a.getX() == b.getX()) && (a.getY() == b.getY()) && (a.getZ() == b.getZ());
  }
  
  private BlockFace getXFace(Vector direction) {
    return direction.getX() > 0.0D ? BlockFace.EAST : BlockFace.WEST;
  }
  
  private BlockFace getYFace(Vector direction) {
    return direction.getY() > 0.0D ? BlockFace.UP : BlockFace.DOWN;
  }
  
  private BlockFace getZFace(Vector direction) {
    return direction.getZ() > 0.0D ? BlockFace.SOUTH : BlockFace.NORTH;
  }
  
  private double getXLength(Vector direction) {
    return Math.abs(direction.getX());
  }
  
  private double getYLength(Vector direction) {
    return Math.abs(direction.getY());
  }
  
  private double getZLength(Vector direction) {
    return Math.abs(direction.getZ());
  }
  
  private double getPosition(double direction, double position, int blockPosition) {
    return direction > 0.0D ? position - blockPosition : blockPosition + 1 - position;
  }
  
  private double getXPosition(Vector direction, Vector position, Block block) {
    return getPosition(direction.getX(), position.getX(), block.getX());
  }
  
  private double getYPosition(Vector direction, Vector position, Block block) {
    return getPosition(direction.getY(), position.getY(), block.getY());
  }
  
  private double getZPosition(Vector direction, Vector position, Block block) {
    return getPosition(direction.getZ(), position.getZ(), block.getZ());
  }
  









  public BlockIterator(Location loc, double yOffset, int maxDistance)
  {
    this(loc.getWorld(), loc.toVector(), loc.getDirection(), yOffset, maxDistance);
  }
  







  public BlockIterator(Location loc, double yOffset)
  {
    this(loc.getWorld(), loc.toVector(), loc.getDirection(), yOffset, 0);
  }
  





  public BlockIterator(Location loc)
  {
    this(loc, 0.0D);
  }
  








  public BlockIterator(LivingEntity entity, int maxDistance)
  {
    this(entity.getLocation(), entity.getEyeHeight(), maxDistance);
  }
  





  public BlockIterator(LivingEntity entity)
  {
    this(entity, 0);
  }
  



  public boolean hasNext()
  {
    scan();
    return currentBlock != -1;
  }
  





  public Block next()
  {
    scan();
    if (currentBlock <= -1) {
      throw new NoSuchElementException();
    }
    return blockQueue[(currentBlock--)];
  }
  
  public void remove()
  {
    throw new UnsupportedOperationException("[BlockIterator] doesn't support block removal");
  }
  
  private void scan() {
    if (currentBlock >= 0) {
      return;
    }
    if ((maxDistance != 0) && (currentDistance > maxDistanceInt)) {
      end = true;
      return;
    }
    if (end) {
      return;
    }
    
    currentDistance += 1;
    
    secondError += secondStep;
    thirdError += thirdStep;
    
    if ((secondError > 0) && (thirdError > 0)) {
      blockQueue[2] = blockQueue[0].getRelative(mainFace);
      if (secondStep * thirdError < thirdStep * secondError) {
        blockQueue[1] = blockQueue[2].getRelative(secondFace);
        blockQueue[0] = blockQueue[1].getRelative(thirdFace);
      } else {
        blockQueue[1] = blockQueue[2].getRelative(thirdFace);
        blockQueue[0] = blockQueue[1].getRelative(secondFace);
      }
      thirdError -= 16777216;
      secondError -= 16777216;
      currentBlock = 2;
      return; }
    if (secondError > 0) {
      blockQueue[1] = blockQueue[0].getRelative(mainFace);
      blockQueue[0] = blockQueue[1].getRelative(secondFace);
      secondError -= 16777216;
      currentBlock = 1;
      return; }
    if (thirdError > 0) {
      blockQueue[1] = blockQueue[0].getRelative(mainFace);
      blockQueue[0] = blockQueue[1].getRelative(thirdFace);
      thirdError -= 16777216;
      currentBlock = 1;
      return;
    }
    blockQueue[0] = blockQueue[0].getRelative(mainFace);
    currentBlock = 0;
  }
}
