package org.bukkit.util;

import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Random;
import org.bukkit.Location;
import org.bukkit.World;
import org.bukkit.configuration.serialization.ConfigurationSerializable;
import org.bukkit.configuration.serialization.SerializableAs;






@SerializableAs("Vector")
public class Vector
  implements Cloneable, ConfigurationSerializable
{
  private static final long serialVersionUID = -2657651106777219169L;
  private static Random random = new Random();
  

  private static final double epsilon = 1.0E-006D;
  

  protected double x;
  
  protected double y;
  
  protected double z;
  

  public Vector()
  {
    x = 0.0D;
    y = 0.0D;
    z = 0.0D;
  }
  






  public Vector(int x, int y, int z)
  {
    this.x = x;
    this.y = y;
    this.z = z;
  }
  






  public Vector(double x, double y, double z)
  {
    this.x = x;
    this.y = y;
    this.z = z;
  }
  






  public Vector(float x, float y, float z)
  {
    this.x = x;
    this.y = y;
    this.z = z;
  }
  





  public Vector add(Vector vec)
  {
    x += x;
    y += y;
    z += z;
    return this;
  }
  





  public Vector subtract(Vector vec)
  {
    x -= x;
    y -= y;
    z -= z;
    return this;
  }
  





  public Vector multiply(Vector vec)
  {
    x *= x;
    y *= y;
    z *= z;
    return this;
  }
  





  public Vector divide(Vector vec)
  {
    x /= x;
    y /= y;
    z /= z;
    return this;
  }
  





  public Vector copy(Vector vec)
  {
    x = x;
    y = y;
    z = z;
    return this;
  }
  








  public double length()
  {
    return Math.sqrt(NumberConversions.square(x) + NumberConversions.square(y) + NumberConversions.square(z));
  }
  




  public double lengthSquared()
  {
    return NumberConversions.square(x) + NumberConversions.square(y) + NumberConversions.square(z);
  }
  









  public double distance(Vector o)
  {
    return Math.sqrt(NumberConversions.square(x - x) + NumberConversions.square(y - y) + NumberConversions.square(z - z));
  }
  





  public double distanceSquared(Vector o)
  {
    return NumberConversions.square(x - x) + NumberConversions.square(y - y) + NumberConversions.square(z - z);
  }
  





  public float angle(Vector other)
  {
    double dot = dot(other) / (length() * other.length());
    
    return (float)Math.acos(dot);
  }
  





  public Vector midpoint(Vector other)
  {
    x = ((x + x) / 2.0D);
    y = ((y + y) / 2.0D);
    z = ((z + z) / 2.0D);
    return this;
  }
  





  public Vector getMidpoint(Vector other)
  {
    double x = (this.x + x) / 2.0D;
    double y = (this.y + y) / 2.0D;
    double z = (this.z + z) / 2.0D;
    return new Vector(x, y, z);
  }
  






  public Vector multiply(int m)
  {
    x *= m;
    y *= m;
    z *= m;
    return this;
  }
  






  public Vector multiply(double m)
  {
    x *= m;
    y *= m;
    z *= m;
    return this;
  }
  






  public Vector multiply(float m)
  {
    x *= m;
    y *= m;
    z *= m;
    return this;
  }
  






  public double dot(Vector other)
  {
    return x * x + y * y + z * z;
  }
  











  public Vector crossProduct(Vector o)
  {
    double newX = y * z - y * z;
    double newY = z * x - z * x;
    double newZ = x * y - x * y;
    
    x = newX;
    y = newY;
    z = newZ;
    return this;
  }
  




  public Vector normalize()
  {
    double length = length();
    
    x /= length;
    y /= length;
    z /= length;
    
    return this;
  }
  




  public Vector zero()
  {
    x = 0.0D;
    y = 0.0D;
    z = 0.0D;
    return this;
  }
  









  public boolean isInAABB(Vector min, Vector max)
  {
    return (x >= x) && (x <= x) && (y >= y) && (y <= y) && (z >= z) && (z <= z);
  }
  






  public boolean isInSphere(Vector origin, double radius)
  {
    return NumberConversions.square(x - x) + NumberConversions.square(y - y) + NumberConversions.square(z - z) <= NumberConversions.square(radius);
  }
  




  public double getX()
  {
    return x;
  }
  





  public int getBlockX()
  {
    return NumberConversions.floor(x);
  }
  




  public double getY()
  {
    return y;
  }
  





  public int getBlockY()
  {
    return NumberConversions.floor(y);
  }
  




  public double getZ()
  {
    return z;
  }
  





  public int getBlockZ()
  {
    return NumberConversions.floor(z);
  }
  





  public Vector setX(int x)
  {
    this.x = x;
    return this;
  }
  





  public Vector setX(double x)
  {
    this.x = x;
    return this;
  }
  





  public Vector setX(float x)
  {
    this.x = x;
    return this;
  }
  





  public Vector setY(int y)
  {
    this.y = y;
    return this;
  }
  





  public Vector setY(double y)
  {
    this.y = y;
    return this;
  }
  





  public Vector setY(float y)
  {
    this.y = y;
    return this;
  }
  





  public Vector setZ(int z)
  {
    this.z = z;
    return this;
  }
  





  public Vector setZ(double z)
  {
    this.z = z;
    return this;
  }
  





  public Vector setZ(float z)
  {
    this.z = z;
    return this;
  }
  







  public boolean equals(Object obj)
  {
    if (!(obj instanceof Vector)) {
      return false;
    }
    
    Vector other = (Vector)obj;
    
    return (Math.abs(x - x) < 1.0E-006D) && (Math.abs(y - y) < 1.0E-006D) && (Math.abs(z - z) < 1.0E-006D) && (getClass().equals(obj.getClass()));
  }
  





  public int hashCode()
  {
    int hash = 7;
    
    hash = 79 * hash + (int)(Double.doubleToLongBits(x) ^ Double.doubleToLongBits(x) >>> 32);
    hash = 79 * hash + (int)(Double.doubleToLongBits(y) ^ Double.doubleToLongBits(y) >>> 32);
    hash = 79 * hash + (int)(Double.doubleToLongBits(z) ^ Double.doubleToLongBits(z) >>> 32);
    return hash;
  }
  




  public Vector clone()
  {
    try
    {
      return (Vector)super.clone();
    } catch (CloneNotSupportedException e) {
      throw new Error(e);
    }
  }
  



  public String toString()
  {
    return x + "," + y + "," + z;
  }
  





  public Location toLocation(World world)
  {
    return new Location(world, x, y, z);
  }
  







  public Location toLocation(World world, float yaw, float pitch)
  {
    return new Location(world, x, y, z, yaw, pitch);
  }
  




  public BlockVector toBlockVector()
  {
    return new BlockVector(x, y, z);
  }
  




  public static double getEpsilon()
  {
    return 1.0E-006D;
  }
  






  public static Vector getMinimum(Vector v1, Vector v2)
  {
    return new Vector(Math.min(x, x), Math.min(y, y), Math.min(z, z));
  }
  






  public static Vector getMaximum(Vector v1, Vector v2)
  {
    return new Vector(Math.max(x, x), Math.max(y, y), Math.max(z, z));
  }
  





  public static Vector getRandom()
  {
    return new Vector(random.nextDouble(), random.nextDouble(), random.nextDouble());
  }
  
  public Map<String, Object> serialize() {
    Map<String, Object> result = new LinkedHashMap();
    
    result.put("x", Double.valueOf(getX()));
    result.put("y", Double.valueOf(getY()));
    result.put("z", Double.valueOf(getZ()));
    
    return result;
  }
  
  public static Vector deserialize(Map<String, Object> args) {
    double x = 0.0D;
    double y = 0.0D;
    double z = 0.0D;
    
    if (args.containsKey("x")) {
      x = ((Double)args.get("x")).doubleValue();
    }
    if (args.containsKey("y")) {
      y = ((Double)args.get("y")).doubleValue();
    }
    if (args.containsKey("z")) {
      z = ((Double)args.get("z")).doubleValue();
    }
    
    return new Vector(x, y, z);
  }
}
