package org.bukkit.util.noise;

import java.util.Random;
import org.bukkit.World;







public class SimplexNoiseGenerator
  extends PerlinNoiseGenerator
{
  protected static final double SQRT_3 = Math.sqrt(3.0D);
  protected static final double SQRT_5 = Math.sqrt(5.0D);
  protected static final double F2 = 0.5D * (SQRT_3 - 1.0D);
  protected static final double G2 = (3.0D - SQRT_3) / 6.0D;
  protected static final double G22 = G2 * 2.0D - 1.0D;
  protected static final double F3 = 0.3333333333333333D;
  protected static final double G3 = 0.1666666666666667D;
  protected static final double F4 = (SQRT_5 - 1.0D) / 4.0D;
  protected static final double G4 = (5.0D - SQRT_5) / 20.0D;
  protected static final double G42 = G4 * 2.0D;
  protected static final double G43 = G4 * 3.0D;
  protected static final double G44 = G4 * 4.0D - 1.0D;
  protected static final int[][] grad4 = { { 0, 1, 1, 1 }, { 0, 1, 1, -1 }, { 0, 1, -1, 1 }, { 0, 1, -1, -1 }, { 0, -1, 1, 1 }, { 0, -1, 1, -1 }, { 0, -1, -1, 1 }, { 0, -1, -1, -1 }, { 1, 0, 1, 1 }, { 1, 0, 1, -1 }, { 1, 0, -1, 1 }, { 1, 0, -1, -1 }, { -1, 0, 1, 1 }, { -1, 0, 1, -1 }, { -1, 0, -1, 1 }, { -1, 0, -1, -1 }, { 1, 1, 0, 1 }, { 1, 1, 0, -1 }, { 1, -1, 0, 1 }, { 1, -1, 0, -1 }, { -1, 1, 0, 1 }, { -1, 1, 0, -1 }, { -1, -1, 0, 1 }, { -1, -1, 0, -1 }, { 1, 1, 1, 0 }, { 1, 1, -1, 0 }, { 1, -1, 1, 0 }, { 1, -1, -1, 0 }, { -1, 1, 1, 0 }, { -1, 1, -1, 0 }, { -1, -1, 1, 0 }, { -1, -1, -1, 0 } };
  






  protected static final int[][] simplex = { { 0, 1, 2, 3 }, { 0, 1, 3, 2 }, { 0, 0, 0, 0 }, { 0, 2, 3, 1 }, { 0, 0, 0, 0 }, { 0, 0, 0, 0 }, { 0, 0, 0, 0 }, { 1, 2, 3, 0 }, { 0, 2, 1, 3 }, { 0, 0, 0, 0 }, { 0, 3, 1, 2 }, { 0, 3, 2, 1 }, { 0, 0, 0, 0 }, { 0, 0, 0, 0 }, { 0, 0, 0, 0 }, { 1, 3, 2, 0 }, { 0, 0, 0, 0 }, { 0, 0, 0, 0 }, { 0, 0, 0, 0 }, { 0, 0, 0, 0 }, { 0, 0, 0, 0 }, { 0, 0, 0, 0 }, { 0, 0, 0, 0 }, { 0, 0, 0, 0 }, { 1, 2, 0, 3 }, { 0, 0, 0, 0 }, { 1, 3, 0, 2 }, { 0, 0, 0, 0 }, { 0, 0, 0, 0 }, { 0, 0, 0, 0 }, { 2, 3, 0, 1 }, { 2, 3, 1, 0 }, { 1, 0, 2, 3 }, { 1, 0, 3, 2 }, { 0, 0, 0, 0 }, { 0, 0, 0, 0 }, { 0, 0, 0, 0 }, { 2, 0, 3, 1 }, { 0, 0, 0, 0 }, { 2, 1, 3, 0 }, { 0, 0, 0, 0 }, { 0, 0, 0, 0 }, { 0, 0, 0, 0 }, { 0, 0, 0, 0 }, { 0, 0, 0, 0 }, { 0, 0, 0, 0 }, { 0, 0, 0, 0 }, { 0, 0, 0, 0 }, { 2, 0, 1, 3 }, { 0, 0, 0, 0 }, { 0, 0, 0, 0 }, { 0, 0, 0, 0 }, { 3, 0, 1, 2 }, { 3, 0, 2, 1 }, { 0, 0, 0, 0 }, { 3, 1, 2, 0 }, { 2, 1, 0, 3 }, { 0, 0, 0, 0 }, { 0, 0, 0, 0 }, { 0, 0, 0, 0 }, { 3, 1, 0, 2 }, { 0, 0, 0, 0 }, { 3, 2, 0, 1 }, { 3, 2, 1, 0 } };
  



  protected static double offsetW;
  



  private static final SimplexNoiseGenerator instance = new SimplexNoiseGenerator();
  



  protected SimplexNoiseGenerator() {}
  



  public SimplexNoiseGenerator(World world)
  {
    this(new Random(world.getSeed()));
  }
  




  public SimplexNoiseGenerator(long seed)
  {
    this(new Random(seed));
  }
  




  public SimplexNoiseGenerator(Random rand)
  {
    super(rand);
    offsetW = rand.nextDouble() * 256.0D;
  }
  
  protected static double dot(int[] g, double x, double y) {
    return g[0] * x + g[1] * y;
  }
  
  protected static double dot(int[] g, double x, double y, double z) {
    return g[0] * x + g[1] * y + g[2] * z;
  }
  
  protected static double dot(int[] g, double x, double y, double z, double w) {
    return g[0] * x + g[1] * y + g[2] * z + g[3] * w;
  }
  






  public static double getNoise(double xin)
  {
    return instance.noise(xin);
  }
  







  public static double getNoise(double xin, double yin)
  {
    return instance.noise(xin, yin);
  }
  








  public static double getNoise(double xin, double yin, double zin)
  {
    return instance.noise(xin, yin, zin);
  }
  









  public static double getNoise(double x, double y, double z, double w)
  {
    return instance.noise(x, y, z, w);
  }
  
  public double noise(double xin, double yin, double zin)
  {
    xin += offsetX;
    yin += offsetY;
    zin += offsetZ;
    



    double s = (xin + yin + zin) * 0.3333333333333333D;
    int i = floor(xin + s);
    int j = floor(yin + s);
    int k = floor(zin + s);
    double t = (i + j + k) * 0.1666666666666667D;
    double X0 = i - t;
    double Y0 = j - t;
    double Z0 = k - t;
    double x0 = xin - X0;
    double y0 = yin - Y0;
    double z0 = zin - Z0;
    int k2;
    int i1;
    int j1;
    int k1;
    int i2;
    int j2;
    int k2; if (x0 >= y0) { int k2;
      if (y0 >= z0) {
        int i1 = 1;
        int j1 = 0;
        int k1 = 0;
        int i2 = 1;
        int j2 = 1;
        k2 = 0;
      } else { int k2;
        if (x0 >= z0) {
          int i1 = 1;
          int j1 = 0;
          int k1 = 0;
          int i2 = 1;
          int j2 = 0;
          k2 = 1;
        }
        else {
          int i1 = 0;
          int j1 = 0;
          int k1 = 1;
          int i2 = 1;
          int j2 = 0;
          k2 = 1;
        }
      } } else { int k2;
      if (y0 < z0) {
        int i1 = 0;
        int j1 = 0;
        int k1 = 1;
        int i2 = 0;
        int j2 = 1;
        k2 = 1;
      } else { int k2;
        if (x0 < z0) {
          int i1 = 0;
          int j1 = 1;
          int k1 = 0;
          int i2 = 0;
          int j2 = 1;
          k2 = 1;
        }
        else {
          i1 = 0;
          j1 = 1;
          k1 = 0;
          i2 = 1;
          j2 = 1;
          k2 = 0;
        }
      }
    }
    



    double x1 = x0 - i1 + 0.1666666666666667D;
    double y1 = y0 - j1 + 0.1666666666666667D;
    double z1 = z0 - k1 + 0.1666666666666667D;
    double x2 = x0 - i2 + 0.3333333333333333D;
    double y2 = y0 - j2 + 0.3333333333333333D;
    double z2 = z0 - k2 + 0.3333333333333333D;
    double x3 = x0 - 1.0D + 0.5D;
    double y3 = y0 - 1.0D + 0.5D;
    double z3 = z0 - 1.0D + 0.5D;
    

    int ii = i & 0xFF;
    int jj = j & 0xFF;
    int kk = k & 0xFF;
    int gi0 = perm[(ii + perm[(jj + perm[kk])])] % 12;
    int gi1 = perm[(ii + i1 + perm[(jj + j1 + perm[(kk + k1)])])] % 12;
    int gi2 = perm[(ii + i2 + perm[(jj + j2 + perm[(kk + k2)])])] % 12;
    int gi3 = perm[(ii + 1 + perm[(jj + 1 + perm[(kk + 1)])])] % 12;
    

    double t0 = 0.6D - x0 * x0 - y0 * y0 - z0 * z0;
    double n0; double n0; if (t0 < 0.0D) {
      n0 = 0.0D;
    } else {
      t0 *= t0;
      n0 = t0 * t0 * dot(grad3[gi0], x0, y0, z0);
    }
    
    double t1 = 0.6D - x1 * x1 - y1 * y1 - z1 * z1;
    double n1; double n1; if (t1 < 0.0D) {
      n1 = 0.0D;
    } else {
      t1 *= t1;
      n1 = t1 * t1 * dot(grad3[gi1], x1, y1, z1);
    }
    
    double t2 = 0.6D - x2 * x2 - y2 * y2 - z2 * z2;
    double n2; double n2; if (t2 < 0.0D) {
      n2 = 0.0D;
    } else {
      t2 *= t2;
      n2 = t2 * t2 * dot(grad3[gi2], x2, y2, z2);
    }
    
    double t3 = 0.6D - x3 * x3 - y3 * y3 - z3 * z3;
    double n3; double n3; if (t3 < 0.0D) {
      n3 = 0.0D;
    } else {
      t3 *= t3;
      n3 = t3 * t3 * dot(grad3[gi3], x3, y3, z3);
    }
    


    return 32.0D * (n0 + n1 + n2 + n3);
  }
  
  public double noise(double xin, double yin)
  {
    xin += offsetX;
    yin += offsetY;
    



    double s = (xin + yin) * F2;
    int i = floor(xin + s);
    int j = floor(yin + s);
    double t = (i + j) * G2;
    double X0 = i - t;
    double Y0 = j - t;
    double x0 = xin - X0;
    double y0 = yin - Y0;
    
    int j1;
    
    int i1;
    int j1;
    if (x0 > y0) {
      int i1 = 1;
      j1 = 0;
    }
    else {
      i1 = 0;
      j1 = 1;
    }
    




    double x1 = x0 - i1 + G2;
    double y1 = y0 - j1 + G2;
    double x2 = x0 + G22;
    double y2 = y0 + G22;
    

    int ii = i & 0xFF;
    int jj = j & 0xFF;
    int gi0 = perm[(ii + perm[jj])] % 12;
    int gi1 = perm[(ii + i1 + perm[(jj + j1)])] % 12;
    int gi2 = perm[(ii + 1 + perm[(jj + 1)])] % 12;
    

    double t0 = 0.5D - x0 * x0 - y0 * y0;
    double n0; double n0; if (t0 < 0.0D) {
      n0 = 0.0D;
    } else {
      t0 *= t0;
      n0 = t0 * t0 * dot(grad3[gi0], x0, y0);
    }
    
    double t1 = 0.5D - x1 * x1 - y1 * y1;
    double n1; double n1; if (t1 < 0.0D) {
      n1 = 0.0D;
    } else {
      t1 *= t1;
      n1 = t1 * t1 * dot(grad3[gi1], x1, y1);
    }
    
    double t2 = 0.5D - x2 * x2 - y2 * y2;
    double n2; double n2; if (t2 < 0.0D) {
      n2 = 0.0D;
    } else {
      t2 *= t2;
      n2 = t2 * t2 * dot(grad3[gi2], x2, y2);
    }
    


    return 70.0D * (n0 + n1 + n2);
  }
  









  public double noise(double x, double y, double z, double w)
  {
    x += offsetX;
    y += offsetY;
    z += offsetZ;
    w += offsetW;
    



    double s = (x + y + z + w) * F4;
    int i = floor(x + s);
    int j = floor(y + s);
    int k = floor(z + s);
    int l = floor(w + s);
    
    double t = (i + j + k + l) * G4;
    double X0 = i - t;
    double Y0 = j - t;
    double Z0 = k - t;
    double W0 = l - t;
    double x0 = x - X0;
    double y0 = y - Y0;
    double z0 = z - Z0;
    double w0 = w - W0;
    








    int c1 = x0 > y0 ? 32 : 0;
    int c2 = x0 > z0 ? 16 : 0;
    int c3 = y0 > z0 ? 8 : 0;
    int c4 = x0 > w0 ? 4 : 0;
    int c5 = y0 > w0 ? 2 : 0;
    int c6 = z0 > w0 ? 1 : 0;
    int c = c1 + c2 + c3 + c4 + c5 + c6;
    









    int i1 = simplex[c][0] >= 3 ? 1 : 0;
    int j1 = simplex[c][1] >= 3 ? 1 : 0;
    int k1 = simplex[c][2] >= 3 ? 1 : 0;
    int l1 = simplex[c][3] >= 3 ? 1 : 0;
    

    int i2 = simplex[c][0] >= 2 ? 1 : 0;
    int j2 = simplex[c][1] >= 2 ? 1 : 0;
    int k2 = simplex[c][2] >= 2 ? 1 : 0;
    int l2 = simplex[c][3] >= 2 ? 1 : 0;
    

    int i3 = simplex[c][0] >= 1 ? 1 : 0;
    int j3 = simplex[c][1] >= 1 ? 1 : 0;
    int k3 = simplex[c][2] >= 1 ? 1 : 0;
    int l3 = simplex[c][3] >= 1 ? 1 : 0;
    


    double x1 = x0 - i1 + G4;
    double y1 = y0 - j1 + G4;
    double z1 = z0 - k1 + G4;
    double w1 = w0 - l1 + G4;
    
    double x2 = x0 - i2 + G42;
    double y2 = y0 - j2 + G42;
    double z2 = z0 - k2 + G42;
    double w2 = w0 - l2 + G42;
    
    double x3 = x0 - i3 + G43;
    double y3 = y0 - j3 + G43;
    double z3 = z0 - k3 + G43;
    double w3 = w0 - l3 + G43;
    
    double x4 = x0 + G44;
    double y4 = y0 + G44;
    double z4 = z0 + G44;
    double w4 = w0 + G44;
    

    int ii = i & 0xFF;
    int jj = j & 0xFF;
    int kk = k & 0xFF;
    int ll = l & 0xFF;
    
    int gi0 = perm[(ii + perm[(jj + perm[(kk + perm[ll])])])] % 32;
    int gi1 = perm[(ii + i1 + perm[(jj + j1 + perm[(kk + k1 + perm[(ll + l1)])])])] % 32;
    int gi2 = perm[(ii + i2 + perm[(jj + j2 + perm[(kk + k2 + perm[(ll + l2)])])])] % 32;
    int gi3 = perm[(ii + i3 + perm[(jj + j3 + perm[(kk + k3 + perm[(ll + l3)])])])] % 32;
    int gi4 = perm[(ii + 1 + perm[(jj + 1 + perm[(kk + 1 + perm[(ll + 1)])])])] % 32;
    

    double t0 = 0.6D - x0 * x0 - y0 * y0 - z0 * z0 - w0 * w0;
    double n0; double n0; if (t0 < 0.0D) {
      n0 = 0.0D;
    } else {
      t0 *= t0;
      n0 = t0 * t0 * dot(grad4[gi0], x0, y0, z0, w0);
    }
    
    double t1 = 0.6D - x1 * x1 - y1 * y1 - z1 * z1 - w1 * w1;
    double n1; double n1; if (t1 < 0.0D) {
      n1 = 0.0D;
    } else {
      t1 *= t1;
      n1 = t1 * t1 * dot(grad4[gi1], x1, y1, z1, w1);
    }
    
    double t2 = 0.6D - x2 * x2 - y2 * y2 - z2 * z2 - w2 * w2;
    double n2; double n2; if (t2 < 0.0D) {
      n2 = 0.0D;
    } else {
      t2 *= t2;
      n2 = t2 * t2 * dot(grad4[gi2], x2, y2, z2, w2);
    }
    
    double t3 = 0.6D - x3 * x3 - y3 * y3 - z3 * z3 - w3 * w3;
    double n3; double n3; if (t3 < 0.0D) {
      n3 = 0.0D;
    } else {
      t3 *= t3;
      n3 = t3 * t3 * dot(grad4[gi3], x3, y3, z3, w3);
    }
    
    double t4 = 0.6D - x4 * x4 - y4 * y4 - z4 * z4 - w4 * w4;
    double n4; double n4; if (t4 < 0.0D) {
      n4 = 0.0D;
    } else {
      t4 *= t4;
      n4 = t4 * t4 * dot(grad4[gi4], x4, y4, z4, w4);
    }
    

    return 27.0D * (n0 + n1 + n2 + n3 + n4);
  }
  




  public static SimplexNoiseGenerator getInstance()
  {
    return instance;
  }
}
