package org.sqlite;

import java.sql.Connection;
import java.sql.SQLException;










































public abstract class Function
{
  private Conn conn;
  private DB db;
  long context;
  long value;
  int args;
  
  public Function()
  {
    context = 0L;
    value = 0L;
    args = 0;
  }
  
  public static final void create(Connection conn, String name, Function f)
    throws SQLException
  {
    if ((conn == null) || (!(conn instanceof Conn)))
      throw new SQLException("connection must be to an SQLite db");
    if (conn.isClosed()) {
      throw new SQLException("connection closed");
    }
    conn = ((Conn)conn);
    db = conn.db();
    
    if ((name == null) || (name.length() > 255)) {
      throw new SQLException("invalid function name: '" + name + "'");
    }
    if (db.create_function(name, f) != 0) {
      throw new SQLException("error creating function");
    }
  }
  
  public static final void destroy(Connection conn, String name) throws SQLException
  {
    if ((conn == null) || (!(conn instanceof Conn)))
      throw new SQLException("connection must be to an SQLite db");
    ((Conn)conn).db().destroy_function(name);
  }
  


  protected abstract void xFunc()
    throws SQLException;
  


  protected final synchronized int args()
    throws SQLException
  {
    checkContext();return args;
  }
  
  protected final synchronized void result(byte[] value) throws SQLException {
    checkContext();db.result_blob(context, value);
  }
  
  protected final synchronized void result(double value) throws SQLException {
    checkContext();db.result_double(context, value);
  }
  
  protected final synchronized void result(int value) throws SQLException {
    checkContext();db.result_int(context, value);
  }
  
  protected final synchronized void result(long value) throws SQLException {
    checkContext();db.result_long(context, value);
  }
  
  protected final synchronized void result() throws SQLException {
    checkContext();db.result_null(context);
  }
  
  protected final synchronized void result(String value) throws SQLException {
    checkContext();db.result_text(context, value);
  }
  
  protected final synchronized void error(String err) throws SQLException {
    checkContext();db.result_error(context, err);
  }
  
  protected final synchronized int value_bytes(int arg) throws SQLException {
    checkValue(arg);return db.value_bytes(this, arg);
  }
  
  protected final synchronized String value_text(int arg) throws SQLException {
    checkValue(arg);return db.value_text(this, arg);
  }
  
  protected final synchronized byte[] value_blob(int arg) throws SQLException {
    checkValue(arg);return db.value_blob(this, arg);
  }
  
  protected final synchronized double value_double(int arg) throws SQLException {
    checkValue(arg);return db.value_double(this, arg);
  }
  
  protected final synchronized int value_int(int arg) throws SQLException {
    checkValue(arg);return db.value_int(this, arg);
  }
  
  protected final synchronized long value_long(int arg) throws SQLException {
    checkValue(arg);return db.value_long(this, arg);
  }
  
  protected final synchronized int value_type(int arg) throws SQLException {
    checkValue(arg);return db.value_type(this, arg);
  }
  
  private void checkContext() throws SQLException {
    if ((conn == null) || (conn.db() == null) || (context == 0L))
      throw new SQLException("no context, not allowed to read value");
  }
  
  private void checkValue(int arg) throws SQLException {
    if ((conn == null) || (conn.db() == null) || (value == 0L))
      throw new SQLException("not in value access state");
    if (arg >= args) {
      throw new SQLException("arg " + arg + " out bounds [0," + args + ")");
    }
  }
  
  public static abstract class Aggregate extends Function implements Cloneable
  {
    protected final void xFunc() {}
    
    protected abstract void xStep() throws SQLException;
    
    protected abstract void xFinal() throws SQLException;
    
    public Object clone() throws CloneNotSupportedException {
      return super.clone();
    }
  }
}
