package org.sqlite;

import java.sql.Connection;
import java.sql.DriverPropertyInfo;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashSet;
import java.util.Properties;


































public class SQLiteConfig
{
  private final Properties pragmaTable;
  private int openModeFlag = 0;
  
  public SQLiteConfig() {
    this(new Properties());
  }
  
  public SQLiteConfig(Properties prop) {
    pragmaTable = prop;
    String openMode = pragmaTable.getProperty(OPEN_MODEpragmaName);
    if (openMode != null) {
      openModeFlag = Integer.parseInt(openMode);
    }
    else
    {
      setOpenMode(SQLiteOpenMode.READWRITE);
      setOpenMode(SQLiteOpenMode.CREATE);
    }
  }
  




  public Connection createConnection(String url)
    throws SQLException
  {
    return JDBC.createConnection(url, toProperties());
  }
  





  public void apply(Connection conn)
    throws SQLException
  {
    HashSet<String> pragmaParams = new HashSet();
    for (Pragma each : Pragma.values()) {
      pragmaParams.add(pragmaName);
    }
    
    pragmaParams.remove(OPEN_MODEpragmaName);
    pragmaParams.remove(SHARED_CACHEpragmaName);
    pragmaParams.remove(LOAD_EXTENSIONpragmaName);
    
    Statement stat = conn.createStatement();
    try {
      int count = 0;
      for (Object each : pragmaTable.keySet()) {
        String key = each.toString();
        if (pragmaParams.contains(key))
        {

          String value = pragmaTable.getProperty(key);
          if (value != null) {
            String sql = String.format("pragma %s=%s", new Object[] { key, value });
            stat.addBatch(sql);
            count++;
          }
        } }
      if (count > 0) {
        stat.executeBatch();
      }
    } finally {
      if (stat != null) {
        stat.close();
      }
    }
  }
  
  private void set(Pragma pragma, boolean flag) {
    setPragma(pragma, Boolean.toString(flag));
  }
  
  private void set(Pragma pragma, int num) {
    setPragma(pragma, Integer.toString(num));
  }
  
  private boolean getBoolean(Pragma pragma, String defaultValue) {
    return Boolean.parseBoolean(pragmaTable.getProperty(pragmaName, defaultValue));
  }
  
  public boolean isEnabledSharedCache() {
    return getBoolean(Pragma.SHARED_CACHE, "false");
  }
  
  public boolean isEnabledLoadExtension() {
    return getBoolean(Pragma.LOAD_EXTENSION, "false");
  }
  
  public int getOpenModeFlags() {
    return openModeFlag;
  }
  





  public void setPragma(Pragma pragma, String value)
  {
    pragmaTable.put(pragmaName, value);
  }
  





  public Properties toProperties()
  {
    pragmaTable.setProperty(OPEN_MODEpragmaName, Integer.toString(openModeFlag));
    
    return pragmaTable;
  }
  
  static DriverPropertyInfo[] getDriverPropertyInfo() {
    Pragma[] pragma = Pragma.values();
    DriverPropertyInfo[] result = new DriverPropertyInfo[pragma.length];
    int index = 0;
    for (Pragma p : Pragma.values()) {
      DriverPropertyInfo di = new DriverPropertyInfo(pragmaName, null);
      choices = choices;
      description = description;
      required = false;
      result[(index++)] = di;
    }
    
    return result;
  }
  
  private static final String[] OnOff = { "true", "false" };
  

  private static enum Pragma
  {
    OPEN_MODE("open_mode", "Database open-mode flag", null), 
    SHARED_CACHE("shared_cache", "Enablse SQLite Shared-Cache mode, native driver only", SQLiteConfig.OnOff), 
    LOAD_EXTENSION("enable_load_extension", "Enable SQLite load_extention() function, native driver only", SQLiteConfig.OnOff), 
    

    CACHE_SIZE("cache_size"), 
    CASE_SENSITIVE_LIKE("case_sensitive_like", SQLiteConfig.OnOff), 
    COUNT_CHANGES("count_changes", SQLiteConfig.OnOff), 
    DEFAULT_CACHE_SIZE("default_cache_size"), 
    EMPTY_RESULT_CALLBACKS("empty_result_callback", SQLiteConfig.OnOff), 
    ENCODING("encoding", SQLiteConfig.toStringArray(SQLiteConfig.Encoding.values())), 
    FOREIGN_KEYS("foreign_keys", SQLiteConfig.OnOff), 
    FULL_COLUMN_NAMES("full_column_names", SQLiteConfig.OnOff), 
    FULL_SYNC("fullsync", SQLiteConfig.OnOff), 
    INCREMENTAL_VACUUM("incremental_vacuum"), 
    JOURNAL_MODE("journal_mode", SQLiteConfig.toStringArray(SQLiteConfig.JournalMode.values())), 
    JOURNAL_SIZE_LIMIT("journal_size_limit"), 
    LEGACY_FILE_FORMAT("legacy_file_format", SQLiteConfig.OnOff), 
    LOCKING_MODE("locking_mode", SQLiteConfig.toStringArray(SQLiteConfig.LockingMode.values())), 
    PAGE_SIZE("page_size"), 
    MAX_PAGE_COUNT("max_page_count"), 
    READ_UNCOMMITED("read_uncommited", SQLiteConfig.OnOff), 
    RECURSIVE_TRIGGERS("recursive_triggers", SQLiteConfig.OnOff), 
    REVERSE_UNORDERED_SELECTS("reverse_unordered_selects", SQLiteConfig.OnOff), 
    SHORT_COLUMN_NAMES("short_column_names", SQLiteConfig.OnOff), 
    SYNCHRONOUS("synchronous", SQLiteConfig.toStringArray(SQLiteConfig.SynchronousMode.values())), 
    TEMP_STORE("temp_store", SQLiteConfig.toStringArray(SQLiteConfig.TempStore.values())), 
    TEMP_STORE_DIRECTORY("temp_store_directory"), 
    USER_VERSION("user_version");
    
    public final String pragmaName;
    public final String[] choices;
    public final String description;
    
    private Pragma(String pragmaName) {
      this(pragmaName, null);
    }
    
    private Pragma(String pragmaName, String[] choices) {
      this(pragmaName, null, null);
    }
    
    private Pragma(String pragmaName, String description, String[] choices) {
      this.pragmaName = pragmaName;
      this.description = description;
      this.choices = choices;
    }
  }
  




  public void setOpenMode(SQLiteOpenMode mode)
  {
    openModeFlag |= flag;
  }
  




  public void resetOpenMode(SQLiteOpenMode mode)
  {
    openModeFlag &= (flag ^ 0xFFFFFFFF);
  }
  
  public void setSharedCache(boolean enable) {
    set(Pragma.SHARED_CACHE, enable);
  }
  
  public void enableLoadExtension(boolean enable) {
    set(Pragma.LOAD_EXTENSION, enable);
  }
  
  public void setReadOnly(boolean readOnly) {
    if (readOnly) {
      setOpenMode(SQLiteOpenMode.READONLY);
      resetOpenMode(SQLiteOpenMode.READWRITE);
    }
    else {
      setOpenMode(SQLiteOpenMode.READWRITE);
      resetOpenMode(SQLiteOpenMode.READONLY);
    }
  }
  
  public void setCacheSize(int numberOfPages) {
    set(Pragma.CACHE_SIZE, numberOfPages);
  }
  
  public void enableCaseSensitiveLike(boolean enable) {
    set(Pragma.CASE_SENSITIVE_LIKE, enable);
  }
  
  public void enableCountChanges(boolean enable) {
    set(Pragma.COUNT_CHANGES, enable);
  }
  




  public void setDefaultCacheSize(int numberOfPages)
  {
    set(Pragma.DEFAULT_CACHE_SIZE, numberOfPages);
  }
  
  public void enableEmptyResultCallBacks(boolean enable) {
    set(Pragma.EMPTY_RESULT_CALLBACKS, enable);
  }
  

















  private static String[] toStringArray(PragmaValue[] list)
  {
    String[] result = new String[list.length];
    for (int i = 0; i < list.length; i++) {
      result[i] = list[i].getValue();
    }
    return result; }
  
  private static abstract interface PragmaValue { public abstract String getValue(); }
  
  public static enum Encoding implements SQLiteConfig.PragmaValue { UTF8("UTF-8"),  UTF16("UTF-16"),  UTF16_LITTLE_ENDIAN("UTF-16le"),  UTF16_BIG_ENDIAN("UTF-16be");
    
    public final String typeName;
    
    private Encoding(String typeName) { this.typeName = typeName; }
    
    public String getValue()
    {
      return typeName;
    }
  }
  
  public static enum JournalMode implements SQLiteConfig.PragmaValue {
    DELETE,  TRUNCATE,  PERSIST,  MEMORY,  OFF;
    
    private JournalMode() {}
    public String getValue() { return name(); }
  }
  
  public void setEncoding(Encoding encoding)
  {
    setPragma(Pragma.ENCODING, typeName);
  }
  
  public void enforceForeignKeys(boolean enforce) {
    set(Pragma.FOREIGN_KEYS, enforce);
  }
  
  public void enableFullColumnNames(boolean enable) {
    set(Pragma.FULL_COLUMN_NAMES, enable);
  }
  
  public void enableFullSync(boolean enable) {
    set(Pragma.FULL_SYNC, enable);
  }
  
  public void incrementalVacuum(int numberOfPagesToBeRemoved) {
    set(Pragma.INCREMENTAL_VACUUM, numberOfPagesToBeRemoved);
  }
  
  public void setJournalMode(JournalMode mode) {
    setPragma(Pragma.JOURNAL_MODE, mode.name());
  }
  



  public void setJounalSizeLimit(int limit)
  {
    set(Pragma.JOURNAL_SIZE_LIMIT, limit);
  }
  
  public void useLegacyFileFormat(boolean use) {
    set(Pragma.LEGACY_FILE_FORMAT, use);
  }
  
  public static enum LockingMode implements SQLiteConfig.PragmaValue {
    NORMAL,  EXCLUSIVE;
    private LockingMode() {}
    public String getValue() { return name(); }
  }
  
  public void setLockingMode(LockingMode mode)
  {
    setPragma(Pragma.LOCKING_MODE, mode.name());
  }
  



  public void setPageSize(int numBytes)
  {
    set(Pragma.PAGE_SIZE, numBytes);
  }
  
  public void setMaxPageCount(int numPages) {
    set(Pragma.MAX_PAGE_COUNT, numPages);
  }
  
  public void setReadUncommited(boolean useReadUncommitedIsolationMode) {
    set(Pragma.READ_UNCOMMITED, useReadUncommitedIsolationMode);
  }
  
  public void enableRecursiveTriggers(boolean enable) {
    set(Pragma.RECURSIVE_TRIGGERS, enable);
  }
  
  public void enableReverseUnorderedSelects(boolean enable) {
    set(Pragma.REVERSE_UNORDERED_SELECTS, enable);
  }
  
  public void enableShortColumnNames(boolean enable) {
    set(Pragma.SHORT_COLUMN_NAMES, enable);
  }
  
  public static enum SynchronousMode implements SQLiteConfig.PragmaValue {
    OFF,  NORMAL,  FULL;
    
    private SynchronousMode() {}
    public String getValue() { return name(); }
  }
  
  public void setSynchronous(SynchronousMode mode)
  {
    setPragma(Pragma.SYNCHRONOUS, mode.name());
  }
  
  public static enum TempStore implements SQLiteConfig.PragmaValue {
    DEFAULT,  FILE,  MEMORY;
    
    private TempStore() {}
    public String getValue() { return name(); }
  }
  

  public void setTempStore(TempStore storeType)
  {
    setPragma(Pragma.TEMP_STORE, storeType.name());
  }
  
  public void setTempStoreDirectory(String directoryName) {
    setPragma(Pragma.TEMP_STORE_DIRECTORY, String.format("'%s'", new Object[] { directoryName }));
  }
  
  public void setUserVersion(int version) {
    set(Pragma.USER_VERSION, version);
  }
}
