package com.google.common.base;

import com.google.common.annotations.Beta;
import com.google.common.annotations.GwtCompatible;
import java.io.Serializable;
import javax.annotation.Nullable;



























































@Beta
@GwtCompatible
public abstract class Optional<T>
  implements Serializable
{
  private static final long serialVersionUID = 0L;
  
  public static <T> Optional<T> absent()
  {
    return Absent.INSTANCE;
  }
  


  public static <T> Optional<T> of(T reference)
  {
    return new Present(Preconditions.checkNotNull(reference));
  }
  



  public static <T> Optional<T> fromNullable(@Nullable T nullableReference)
  {
    return nullableReference == null ? absent() : new Present(nullableReference);
  }
  




  public abstract boolean isPresent();
  




  public abstract T get();
  




  public abstract T or(T paramT);
  




  public abstract Optional<T> or(Optional<? extends T> paramOptional);
  



  @Nullable
  public abstract T or(Supplier<? extends T> paramSupplier);
  



  @Nullable
  public abstract T orNull();
  



  public abstract boolean equals(@Nullable Object paramObject);
  



  public abstract int hashCode();
  



  public abstract String toString();
  



  private static final class Present<T>
    extends Optional<T>
  {
    private final T reference;
    


    private static final long serialVersionUID = 0L;
    



    Present(T reference)
    {
      super();
      this.reference = reference;
    }
    
    public boolean isPresent() {
      return true;
    }
    
    public T get() {
      return reference;
    }
    
    public T or(T defaultValue) {
      Preconditions.checkNotNull(defaultValue, "use orNull() instead of or(null)");
      return reference;
    }
    
    public Optional<T> or(Optional<? extends T> secondChoice) {
      Preconditions.checkNotNull(secondChoice);
      return this;
    }
    
    public T or(Supplier<? extends T> supplier) {
      Preconditions.checkNotNull(supplier);
      return reference;
    }
    
    public T orNull() {
      return reference;
    }
    
    public boolean equals(@Nullable Object object) {
      if ((object instanceof Present)) {
        Present<?> other = (Present)object;
        return reference.equals(reference);
      }
      return false;
    }
    
    public int hashCode() {
      return 1502476572 + reference.hashCode();
    }
    
    public String toString() {
      return "Optional.of(" + reference + ")";
    }
  }
  
  private static final class Absent
    extends Optional<Object> {
    private Absent() { super(); }
    private static final Absent INSTANCE = new Absent();
    private static final long serialVersionUID = 0L;
    
    public boolean isPresent() { return false; }
    
    public Object get()
    {
      throw new IllegalStateException("value is absent");
    }
    
    public Object or(Object defaultValue) {
      return Preconditions.checkNotNull(defaultValue, "use orNull() instead of or(null)");
    }
    
    public Optional<Object> or(Optional<?> secondChoice)
    {
      return (Optional)Preconditions.checkNotNull(secondChoice);
    }
    
    @Nullable
    public Object or(Supplier<?> supplier) { return supplier.get(); }
    
    @Nullable
    public Object orNull() {
      return null;
    }
    
    public boolean equals(@Nullable Object object) {
      return object == this;
    }
    
    public int hashCode() {
      return 1502476572;
    }
    
    public String toString() {
      return "Optional.absent()";
    }
    
    private Object readResolve() {
      return INSTANCE;
    }
  }
}
