package com.google.common.cache;

import com.google.common.annotations.Beta;
import com.google.common.util.concurrent.UncheckedExecutionException;
import java.util.concurrent.ConcurrentMap;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.atomic.AtomicLong;

































@Beta
public abstract class AbstractCache<K, V>
  implements Cache<K, V>
{
  public V getUnchecked(K key)
  {
    try
    {
      return get(key);
    } catch (ExecutionException e) {
      throw new UncheckedExecutionException(e.getCause());
    }
  }
  
  public final V apply(K key)
  {
    return getUnchecked(key);
  }
  

  public void cleanUp() {}
  
  public long size()
  {
    throw new UnsupportedOperationException();
  }
  
  public void invalidate(Object key)
  {
    throw new UnsupportedOperationException();
  }
  
  public void invalidateAll()
  {
    throw new UnsupportedOperationException();
  }
  
  public CacheStats stats()
  {
    throw new UnsupportedOperationException();
  }
  
  public ConcurrentMap<K, V> asMap()
  {
    throw new UnsupportedOperationException();
  }
  





























































  @Beta
  public static class SimpleStatsCounter
    implements AbstractCache.StatsCounter
  {
    private final AtomicLong hitCount = new AtomicLong();
    private final AtomicLong missCount = new AtomicLong();
    private final AtomicLong loadSuccessCount = new AtomicLong();
    private final AtomicLong loadExceptionCount = new AtomicLong();
    private final AtomicLong totalLoadTime = new AtomicLong();
    private final AtomicLong evictionCount = new AtomicLong();
    
    public void recordHit()
    {
      hitCount.incrementAndGet();
    }
    
    public void recordLoadSuccess(long loadTime)
    {
      missCount.incrementAndGet();
      loadSuccessCount.incrementAndGet();
      totalLoadTime.addAndGet(loadTime);
    }
    
    public void recordLoadException(long loadTime)
    {
      missCount.incrementAndGet();
      loadExceptionCount.incrementAndGet();
      totalLoadTime.addAndGet(loadTime);
    }
    
    public void recordConcurrentMiss()
    {
      missCount.incrementAndGet();
    }
    
    public void recordEviction()
    {
      evictionCount.incrementAndGet();
    }
    
    public CacheStats snapshot()
    {
      return new CacheStats(hitCount.get(), missCount.get(), loadSuccessCount.get(), loadExceptionCount.get(), totalLoadTime.get(), evictionCount.get());
    }
    








    public void incrementBy(AbstractCache.StatsCounter other)
    {
      CacheStats otherStats = other.snapshot();
      hitCount.addAndGet(otherStats.hitCount());
      missCount.addAndGet(otherStats.missCount());
      loadSuccessCount.addAndGet(otherStats.loadSuccessCount());
      loadExceptionCount.addAndGet(otherStats.loadExceptionCount());
      totalLoadTime.addAndGet(otherStats.totalLoadTime());
      evictionCount.addAndGet(otherStats.evictionCount());
    }
  }
  
  @Beta
  public static abstract interface StatsCounter
  {
    public abstract void recordHit();
    
    public abstract void recordLoadSuccess(long paramLong);
    
    public abstract void recordLoadException(long paramLong);
    
    public abstract void recordConcurrentMiss();
    
    public abstract void recordEviction();
    
    public abstract CacheStats snapshot();
  }
}
