package com.google.common.io;

import com.google.common.annotations.Beta;
import com.google.common.base.Preconditions;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInput;
import java.io.DataInputStream;
import java.io.DataOutput;
import java.io.DataOutputStream;
import java.io.EOFException;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.nio.ByteBuffer;
import java.nio.channels.ReadableByteChannel;
import java.nio.channels.WritableByteChannel;
import java.security.MessageDigest;
import java.util.Arrays;
import java.util.zip.Checksum;


































@Beta
public final class ByteStreams
{
  private static final int BUF_SIZE = 4096;
  
  public static InputSupplier<ByteArrayInputStream> newInputStreamSupplier(byte[] b)
  {
    return newInputStreamSupplier(b, 0, b.length);
  }
  









  public static InputSupplier<ByteArrayInputStream> newInputStreamSupplier(byte[] b, final int off, final int len)
  {
    new InputSupplier()
    {
      public ByteArrayInputStream getInput() {
        return new ByteArrayInputStream(val$b, off, len);
      }
    };
  }
  






  public static void write(byte[] from, OutputSupplier<? extends OutputStream> to)
    throws IOException
  {
    Preconditions.checkNotNull(from);
    boolean threw = true;
    OutputStream out = (OutputStream)to.getOutput();
    try {
      out.write(from);
      threw = false;
    } finally {
      Closeables.close(out, threw);
    }
  }
  
  /* Error */
  public static long copy(InputSupplier<? extends InputStream> from, OutputSupplier<? extends OutputStream> to)
    throws IOException
  {
    // Byte code:
    //   0: iconst_1
    //   1: istore_2
    //   2: aload_0
    //   3: invokeinterface 85 1 0
    //   8: checkcast 87	java/io/InputStream
    //   11: astore_3
    //   12: aload_1
    //   13: invokeinterface 59 1 0
    //   18: checkcast 61	java/io/OutputStream
    //   21: astore 4
    //   23: aload_3
    //   24: aload 4
    //   26: invokestatic 90	com/google/common/io/ByteStreams:copy	(Ljava/io/InputStream;Ljava/io/OutputStream;)J
    //   29: lstore 5
    //   31: iconst_0
    //   32: istore_2
    //   33: lload 5
    //   35: lstore 7
    //   37: aload 4
    //   39: iload_2
    //   40: invokestatic 70	com/google/common/io/Closeables:close	(Ljava/io/Closeable;Z)V
    //   43: aload_3
    //   44: iload_2
    //   45: invokestatic 70	com/google/common/io/Closeables:close	(Ljava/io/Closeable;Z)V
    //   48: lload 7
    //   50: lreturn
    //   51: astore 9
    //   53: aload 4
    //   55: iload_2
    //   56: invokestatic 70	com/google/common/io/Closeables:close	(Ljava/io/Closeable;Z)V
    //   59: aload 9
    //   61: athrow
    //   62: astore 10
    //   64: aload_3
    //   65: iload_2
    //   66: invokestatic 70	com/google/common/io/Closeables:close	(Ljava/io/Closeable;Z)V
    //   69: aload 10
    //   71: athrow
    // Line number table:
    //   Java source line #115	-> byte code offset #0
    //   Java source line #116	-> byte code offset #2
    //   Java source line #118	-> byte code offset #12
    //   Java source line #120	-> byte code offset #23
    //   Java source line #121	-> byte code offset #31
    //   Java source line #122	-> byte code offset #33
    //   Java source line #124	-> byte code offset #37
    //   Java source line #127	-> byte code offset #43
    //   Java source line #124	-> byte code offset #51
    //   Java source line #127	-> byte code offset #62
    // Local variable table:
    //   start	length	slot	name	signature
    //   0	72	0	from	InputSupplier<? extends InputStream>
    //   0	72	1	to	OutputSupplier<? extends OutputStream>
    //   1	65	2	threw	boolean
    //   11	54	3	in	InputStream
    //   21	33	4	out	OutputStream
    //   29	5	5	count	long
    //   35	14	7	l1	long
    //   51	9	9	localObject1	Object
    //   62	8	10	localObject2	Object
    // Exception table:
    //   from	to	target	type
    //   23	37	51	finally
    //   51	53	51	finally
    //   12	43	62	finally
    //   51	64	62	finally
  }
  
  public static long copy(InputSupplier<? extends InputStream> from, OutputStream to)
    throws IOException
  {
    boolean threw = true;
    InputStream in = (InputStream)from.getInput();
    try {
      long count = copy(in, to);
      threw = false;
      return count;
    } finally {
      Closeables.close(in, threw);
    }
  }
  










  public static long copy(InputStream from, OutputSupplier<? extends OutputStream> to)
    throws IOException
  {
    boolean threw = true;
    OutputStream out = (OutputStream)to.getOutput();
    try {
      long count = copy(from, out);
      threw = false;
      return count;
    } finally {
      Closeables.close(out, threw);
    }
  }
  








  public static long copy(InputStream from, OutputStream to)
    throws IOException
  {
    byte[] buf = new byte[4096];
    long total = 0L;
    for (;;) {
      int r = from.read(buf);
      if (r == -1) {
        break;
      }
      to.write(buf, 0, r);
      total += r;
    }
    return total;
  }
  








  public static long copy(ReadableByteChannel from, WritableByteChannel to)
    throws IOException
  {
    ByteBuffer buf = ByteBuffer.allocate(4096);
    long total = 0L;
    while (from.read(buf) != -1) {
      buf.flip();
      while (buf.hasRemaining()) {
        total += to.write(buf);
      }
      buf.clear();
    }
    return total;
  }
  






  public static byte[] toByteArray(InputStream in)
    throws IOException
  {
    ByteArrayOutputStream out = new ByteArrayOutputStream();
    copy(in, out);
    return out.toByteArray();
  }
  





  public static byte[] toByteArray(InputSupplier<? extends InputStream> supplier)
    throws IOException
  {
    boolean threw = true;
    InputStream in = (InputStream)supplier.getInput();
    try {
      byte[] result = toByteArray(in);
      threw = false;
      return result;
    } finally {
      Closeables.close(in, threw);
    }
  }
  



  public static ByteArrayDataInput newDataInput(byte[] bytes)
  {
    return new ByteArrayDataInputStream(bytes);
  }
  






  public static ByteArrayDataInput newDataInput(byte[] bytes, int start)
  {
    Preconditions.checkPositionIndex(start, bytes.length);
    return new ByteArrayDataInputStream(bytes, start);
  }
  
  private static class ByteArrayDataInputStream implements ByteArrayDataInput {
    final DataInput input;
    
    ByteArrayDataInputStream(byte[] bytes) {
      input = new DataInputStream(new ByteArrayInputStream(bytes));
    }
    
    ByteArrayDataInputStream(byte[] bytes, int start) {
      input = new DataInputStream(new ByteArrayInputStream(bytes, start, bytes.length - start));
    }
    
    public void readFully(byte[] b)
    {
      try {
        input.readFully(b);
      } catch (IOException e) {
        throw new IllegalStateException(e);
      }
    }
    
    public void readFully(byte[] b, int off, int len) {
      try {
        input.readFully(b, off, len);
      } catch (IOException e) {
        throw new IllegalStateException(e);
      }
    }
    
    public int skipBytes(int n) {
      try {
        return input.skipBytes(n);
      } catch (IOException e) {
        throw new IllegalStateException(e);
      }
    }
    
    public boolean readBoolean() {
      try {
        return input.readBoolean();
      } catch (IOException e) {
        throw new IllegalStateException(e);
      }
    }
    
    public byte readByte() {
      try {
        return input.readByte();
      } catch (EOFException e) {
        throw new IllegalStateException(e);
      } catch (IOException impossible) {
        throw new AssertionError(impossible);
      }
    }
    
    public int readUnsignedByte() {
      try {
        return input.readUnsignedByte();
      } catch (IOException e) {
        throw new IllegalStateException(e);
      }
    }
    
    public short readShort() {
      try {
        return input.readShort();
      } catch (IOException e) {
        throw new IllegalStateException(e);
      }
    }
    
    public int readUnsignedShort() {
      try {
        return input.readUnsignedShort();
      } catch (IOException e) {
        throw new IllegalStateException(e);
      }
    }
    
    public char readChar() {
      try {
        return input.readChar();
      } catch (IOException e) {
        throw new IllegalStateException(e);
      }
    }
    
    public int readInt() {
      try {
        return input.readInt();
      } catch (IOException e) {
        throw new IllegalStateException(e);
      }
    }
    
    public long readLong() {
      try {
        return input.readLong();
      } catch (IOException e) {
        throw new IllegalStateException(e);
      }
    }
    
    public float readFloat() {
      try {
        return input.readFloat();
      } catch (IOException e) {
        throw new IllegalStateException(e);
      }
    }
    
    public double readDouble() {
      try {
        return input.readDouble();
      } catch (IOException e) {
        throw new IllegalStateException(e);
      }
    }
    
    public String readLine() {
      try {
        return input.readLine();
      } catch (IOException e) {
        throw new IllegalStateException(e);
      }
    }
    
    public String readUTF() {
      try {
        return input.readUTF();
      } catch (IOException e) {
        throw new IllegalStateException(e);
      }
    }
  }
  


  public static ByteArrayDataOutput newDataOutput()
  {
    return new ByteArrayDataOutputStream();
  }
  





  public static ByteArrayDataOutput newDataOutput(int size)
  {
    Preconditions.checkArgument(size >= 0, "Invalid size: %s", new Object[] { Integer.valueOf(size) });
    return new ByteArrayDataOutputStream(size);
  }
  
  /* Error */
  public static long length(InputSupplier<? extends InputStream> supplier)
    throws IOException
  {
    // Byte code:
    //   0: lconst_0
    //   1: lstore_1
    //   2: iconst_1
    //   3: istore_3
    //   4: aload_0
    //   5: invokeinterface 85 1 0
    //   10: checkcast 87	java/io/InputStream
    //   13: astore 4
    //   15: aload 4
    //   17: ldc2_w 188
    //   20: invokevirtual 193	java/io/InputStream:skip	(J)J
    //   23: lstore 5
    //   25: lload 5
    //   27: lconst_0
    //   28: lcmp
    //   29: ifne +33 -> 62
    //   32: aload 4
    //   34: invokevirtual 196	java/io/InputStream:read	()I
    //   37: iconst_m1
    //   38: if_icmpne +17 -> 55
    //   41: iconst_0
    //   42: istore_3
    //   43: lload_1
    //   44: lstore 7
    //   46: aload 4
    //   48: iload_3
    //   49: invokestatic 70	com/google/common/io/Closeables:close	(Ljava/io/Closeable;Z)V
    //   52: lload 7
    //   54: lreturn
    //   55: lload_1
    //   56: lconst_1
    //   57: ladd
    //   58: lstore_1
    //   59: goto +8 -> 67
    //   62: lload_1
    //   63: lload 5
    //   65: ladd
    //   66: lstore_1
    //   67: goto -52 -> 15
    //   70: astore 9
    //   72: aload 4
    //   74: iload_3
    //   75: invokestatic 70	com/google/common/io/Closeables:close	(Ljava/io/Closeable;Z)V
    //   78: aload 9
    //   80: athrow
    // Line number table:
    //   Java source line #573	-> byte code offset #0
    //   Java source line #574	-> byte code offset #2
    //   Java source line #575	-> byte code offset #4
    //   Java source line #579	-> byte code offset #15
    //   Java source line #580	-> byte code offset #25
    //   Java source line #581	-> byte code offset #32
    //   Java source line #582	-> byte code offset #41
    //   Java source line #583	-> byte code offset #43
    //   Java source line #591	-> byte code offset #46
    //   Java source line #585	-> byte code offset #55
    //   Java source line #587	-> byte code offset #62
    //   Java source line #589	-> byte code offset #67
    //   Java source line #591	-> byte code offset #70
    // Local variable table:
    //   start	length	slot	name	signature
    //   0	81	0	supplier	InputSupplier<? extends InputStream>
    //   1	66	1	count	long
    //   3	72	3	threw	boolean
    //   13	60	4	in	InputStream
    //   23	41	5	amt	long
    //   44	9	7	l1	long
    //   70	9	9	localObject	Object
    // Exception table:
    //   from	to	target	type
    //   15	46	70	finally
    //   55	72	70	finally
  }
  
  /* Error */
  public static boolean equal(InputSupplier<? extends InputStream> supplier1, InputSupplier<? extends InputStream> supplier2)
    throws IOException
  {
    // Byte code:
    //   0: sipush 4096
    //   3: newarray byte
    //   5: astore_2
    //   6: sipush 4096
    //   9: newarray byte
    //   11: astore_3
    //   12: iconst_1
    //   13: istore 4
    //   15: aload_0
    //   16: invokeinterface 85 1 0
    //   21: checkcast 87	java/io/InputStream
    //   24: astore 5
    //   26: aload_1
    //   27: invokeinterface 85 1 0
    //   32: checkcast 87	java/io/InputStream
    //   35: astore 6
    //   37: aload 5
    //   39: aload_2
    //   40: iconst_0
    //   41: sipush 4096
    //   44: invokestatic 202	com/google/common/io/ByteStreams:read	(Ljava/io/InputStream;[BII)I
    //   47: istore 7
    //   49: aload 6
    //   51: aload_3
    //   52: iconst_0
    //   53: sipush 4096
    //   56: invokestatic 202	com/google/common/io/ByteStreams:read	(Ljava/io/InputStream;[BII)I
    //   59: istore 8
    //   61: iload 7
    //   63: iload 8
    //   65: if_icmpne +11 -> 76
    //   68: aload_2
    //   69: aload_3
    //   70: invokestatic 208	java/util/Arrays:equals	([B[B)Z
    //   73: ifne +26 -> 99
    //   76: iconst_0
    //   77: istore 4
    //   79: iconst_0
    //   80: istore 9
    //   82: aload 6
    //   84: iload 4
    //   86: invokestatic 70	com/google/common/io/Closeables:close	(Ljava/io/Closeable;Z)V
    //   89: aload 5
    //   91: iload 4
    //   93: invokestatic 70	com/google/common/io/Closeables:close	(Ljava/io/Closeable;Z)V
    //   96: iload 9
    //   98: ireturn
    //   99: iload 7
    //   101: sipush 4096
    //   104: if_icmpeq +26 -> 130
    //   107: iconst_0
    //   108: istore 4
    //   110: iconst_1
    //   111: istore 9
    //   113: aload 6
    //   115: iload 4
    //   117: invokestatic 70	com/google/common/io/Closeables:close	(Ljava/io/Closeable;Z)V
    //   120: aload 5
    //   122: iload 4
    //   124: invokestatic 70	com/google/common/io/Closeables:close	(Ljava/io/Closeable;Z)V
    //   127: iload 9
    //   129: ireturn
    //   130: goto -93 -> 37
    //   133: astore 10
    //   135: aload 6
    //   137: iload 4
    //   139: invokestatic 70	com/google/common/io/Closeables:close	(Ljava/io/Closeable;Z)V
    //   142: aload 10
    //   144: athrow
    //   145: astore 11
    //   147: aload 5
    //   149: iload 4
    //   151: invokestatic 70	com/google/common/io/Closeables:close	(Ljava/io/Closeable;Z)V
    //   154: aload 11
    //   156: athrow
    // Line number table:
    //   Java source line #602	-> byte code offset #0
    //   Java source line #603	-> byte code offset #6
    //   Java source line #605	-> byte code offset #12
    //   Java source line #606	-> byte code offset #15
    //   Java source line #608	-> byte code offset #26
    //   Java source line #611	-> byte code offset #37
    //   Java source line #612	-> byte code offset #49
    //   Java source line #613	-> byte code offset #61
    //   Java source line #614	-> byte code offset #76
    //   Java source line #615	-> byte code offset #79
    //   Java source line #622	-> byte code offset #82
    //   Java source line #625	-> byte code offset #89
    //   Java source line #616	-> byte code offset #99
    //   Java source line #617	-> byte code offset #107
    //   Java source line #618	-> byte code offset #110
    //   Java source line #622	-> byte code offset #113
    //   Java source line #625	-> byte code offset #120
    //   Java source line #620	-> byte code offset #130
    //   Java source line #622	-> byte code offset #133
    //   Java source line #625	-> byte code offset #145
    // Local variable table:
    //   start	length	slot	name	signature
    //   0	157	0	supplier1	InputSupplier<? extends InputStream>
    //   0	157	1	supplier2	InputSupplier<? extends InputStream>
    //   5	64	2	buf1	byte[]
    //   11	59	3	buf2	byte[]
    //   13	137	4	threw	boolean
    //   24	124	5	in1	InputStream
    //   35	101	6	in2	InputStream
    //   47	53	7	read1	int
    //   59	5	8	read2	int
    //   80	48	9	bool1	boolean
    //   133	10	10	localObject1	Object
    //   145	10	11	localObject2	Object
    // Exception table:
    //   from	to	target	type
    //   37	82	133	finally
    //   99	113	133	finally
    //   130	135	133	finally
    //   26	89	145	finally
    //   99	120	145	finally
    //   130	147	145	finally
  }
  
  private static class ByteArrayDataOutputStream
    implements ByteArrayDataOutput
  {
    final DataOutput output;
    final ByteArrayOutputStream byteArrayOutputSteam;
    
    ByteArrayDataOutputStream()
    {
      this(new ByteArrayOutputStream());
    }
    
    ByteArrayDataOutputStream(int size) {
      this(new ByteArrayOutputStream(size));
    }
    
    ByteArrayDataOutputStream(ByteArrayOutputStream byteArrayOutputSteam) {
      this.byteArrayOutputSteam = byteArrayOutputSteam;
      output = new DataOutputStream(byteArrayOutputSteam);
    }
    
    public void write(int b) {
      try {
        output.write(b);
      } catch (IOException impossible) {
        throw new AssertionError(impossible);
      }
    }
    
    public void write(byte[] b) {
      try {
        output.write(b);
      } catch (IOException impossible) {
        throw new AssertionError(impossible);
      }
    }
    
    public void write(byte[] b, int off, int len) {
      try {
        output.write(b, off, len);
      } catch (IOException impossible) {
        throw new AssertionError(impossible);
      }
    }
    
    public void writeBoolean(boolean v) {
      try {
        output.writeBoolean(v);
      } catch (IOException impossible) {
        throw new AssertionError(impossible);
      }
    }
    
    public void writeByte(int v) {
      try {
        output.writeByte(v);
      } catch (IOException impossible) {
        throw new AssertionError(impossible);
      }
    }
    
    public void writeBytes(String s) {
      try {
        output.writeBytes(s);
      } catch (IOException impossible) {
        throw new AssertionError(impossible);
      }
    }
    
    public void writeChar(int v) {
      try {
        output.writeChar(v);
      } catch (IOException impossible) {
        throw new AssertionError(impossible);
      }
    }
    
    public void writeChars(String s) {
      try {
        output.writeChars(s);
      } catch (IOException impossible) {
        throw new AssertionError(impossible);
      }
    }
    
    public void writeDouble(double v) {
      try {
        output.writeDouble(v);
      } catch (IOException impossible) {
        throw new AssertionError(impossible);
      }
    }
    
    public void writeFloat(float v) {
      try {
        output.writeFloat(v);
      } catch (IOException impossible) {
        throw new AssertionError(impossible);
      }
    }
    
    public void writeInt(int v) {
      try {
        output.writeInt(v);
      } catch (IOException impossible) {
        throw new AssertionError(impossible);
      }
    }
    
    public void writeLong(long v) {
      try {
        output.writeLong(v);
      } catch (IOException impossible) {
        throw new AssertionError(impossible);
      }
    }
    
    public void writeShort(int v) {
      try {
        output.writeShort(v);
      } catch (IOException impossible) {
        throw new AssertionError(impossible);
      }
    }
    
    public void writeUTF(String s) {
      try {
        output.writeUTF(s);
      } catch (IOException impossible) {
        throw new AssertionError(impossible);
      }
    }
    
    public byte[] toByteArray() {
      return byteArrayOutputSteam.toByteArray();
    }
  }
  






































































  public static void readFully(InputStream in, byte[] b)
    throws IOException
  {
    readFully(in, b, 0, b.length);
  }
  













  public static void readFully(InputStream in, byte[] b, int off, int len)
    throws IOException
  {
    if (read(in, b, off, len) != len) {
      throw new EOFException();
    }
  }
  










  public static void skipFully(InputStream in, long n)
    throws IOException
  {
    while (n > 0L) {
      long amt = in.skip(n);
      if (amt == 0L)
      {
        if (in.read() == -1) {
          throw new EOFException();
        }
        n -= 1L;
      } else {
        n -= amt;
      }
    }
  }
  







  public static <T> T readBytes(InputSupplier<? extends InputStream> supplier, ByteProcessor<T> processor)
    throws IOException
  {
    byte[] buf = new byte[4096];
    boolean threw = true;
    InputStream in = (InputStream)supplier.getInput();
    try {
      int amt;
      do {
        amt = in.read(buf);
        if (amt == -1) {
          threw = false;
          break;
        }
      } while (processor.processBytes(buf, 0, amt));
      return processor.getResult();
    } finally {
      Closeables.close(in, threw);
    }
  }
  









  public static long getChecksum(InputSupplier<? extends InputStream> supplier, Checksum checksum)
    throws IOException
  {
    ((Long)readBytes(supplier, new ByteProcessor()
    {
      public boolean processBytes(byte[] buf, int off, int len) {
        val$checksum.update(buf, off, len);
        return true;
      }
      
      public Long getResult()
      {
        long result = val$checksum.getValue();
        val$checksum.reset();
        return Long.valueOf(result);
      }
    })).longValue();
  }
  









  public static byte[] getDigest(InputSupplier<? extends InputStream> supplier, MessageDigest md)
    throws IOException
  {
    (byte[])readBytes(supplier, new ByteProcessor()
    {
      public boolean processBytes(byte[] buf, int off, int len) {
        val$md.update(buf, off, len);
        return true;
      }
      
      public byte[] getResult()
      {
        return val$md.digest();
      }
    });
  }
  























  public static int read(InputStream in, byte[] b, int off, int len)
    throws IOException
  {
    if (len < 0) {
      throw new IndexOutOfBoundsException("len is negative");
    }
    int total = 0;
    while (total < len) {
      int result = in.read(b, off + total, len - total);
      if (result == -1) {
        break;
      }
      total += result;
    }
    return total;
  }
  













  public static InputSupplier<InputStream> slice(InputSupplier<? extends InputStream> supplier, final long offset, long length)
  {
    Preconditions.checkNotNull(supplier);
    Preconditions.checkArgument(offset >= 0L, "offset is negative");
    Preconditions.checkArgument(length >= 0L, "length is negative");
    new InputSupplier() {
      public InputStream getInput() throws IOException {
        InputStream in = (InputStream)val$supplier.getInput();
        if (offset > 0L) {
          try {
            ByteStreams.skipFully(in, offset);
          } catch (IOException e) {
            Closeables.closeQuietly(in);
            throw e;
          }
        }
        return new LimitInputStream(in, val$length);
      }
    };
  }
  















  public static InputSupplier<InputStream> join(Iterable<? extends InputSupplier<? extends InputStream>> suppliers)
  {
    new InputSupplier() {
      public InputStream getInput() throws IOException {
        return new MultiInputStream(val$suppliers.iterator());
      }
    };
  }
  

  public static InputSupplier<InputStream> join(InputSupplier<? extends InputStream>... suppliers)
  {
    return join(Arrays.asList(suppliers));
  }
}
