package com.google.common.io;

import com.google.common.annotations.Beta;
import com.google.common.base.Preconditions;
import java.io.Closeable;
import java.io.EOFException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Reader;
import java.io.StringReader;
import java.io.Writer;
import java.nio.CharBuffer;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;








































@Beta
public final class CharStreams
{
  private static final int BUF_SIZE = 2048;
  
  public static InputSupplier<StringReader> newReaderSupplier(String value)
  {
    Preconditions.checkNotNull(value);
    new InputSupplier()
    {
      public StringReader getInput() {
        return new StringReader(val$value);
      }
    };
  }
  








  public static InputSupplier<InputStreamReader> newReaderSupplier(InputSupplier<? extends InputStream> in, final Charset charset)
  {
    Preconditions.checkNotNull(in);
    Preconditions.checkNotNull(charset);
    new InputSupplier()
    {
      public InputStreamReader getInput() throws IOException {
        return new InputStreamReader((InputStream)val$in.getInput(), charset);
      }
    };
  }
  








  public static OutputSupplier<OutputStreamWriter> newWriterSupplier(OutputSupplier<? extends OutputStream> out, final Charset charset)
  {
    Preconditions.checkNotNull(out);
    Preconditions.checkNotNull(charset);
    new OutputSupplier()
    {
      public OutputStreamWriter getOutput() throws IOException {
        return new OutputStreamWriter((OutputStream)val$out.getOutput(), charset);
      }
    };
  }
  







  public static <W extends Appendable,  extends Closeable> void write(CharSequence from, OutputSupplier<W> to)
    throws IOException
  {
    Preconditions.checkNotNull(from);
    boolean threw = true;
    W out = (Appendable)to.getOutput();
    try {
      out.append(from);
      threw = false;
    } finally {
      Closeables.close((Closeable)out, threw);
    }
  }
  
  /* Error */
  public static <R extends Readable,  extends Closeable, W extends Appendable,  extends Closeable> long copy(InputSupplier<R> from, OutputSupplier<W> to)
    throws IOException
  {
    // Byte code:
    //   0: iconst_1
    //   1: istore_2
    //   2: aload_0
    //   3: invokeinterface 92 1 0
    //   8: checkcast 94	java/lang/Readable
    //   11: astore_3
    //   12: aload_1
    //   13: invokeinterface 63 1 0
    //   18: checkcast 65	java/lang/Appendable
    //   21: astore 4
    //   23: aload_3
    //   24: aload 4
    //   26: invokestatic 97	com/google/common/io/CharStreams:copy	(Ljava/lang/Readable;Ljava/lang/Appendable;)J
    //   29: lstore 5
    //   31: iconst_0
    //   32: istore_2
    //   33: lload 5
    //   35: lstore 7
    //   37: aload 4
    //   39: checkcast 71	java/io/Closeable
    //   42: iload_2
    //   43: invokestatic 77	com/google/common/io/Closeables:close	(Ljava/io/Closeable;Z)V
    //   46: aload_3
    //   47: checkcast 71	java/io/Closeable
    //   50: iload_2
    //   51: invokestatic 77	com/google/common/io/Closeables:close	(Ljava/io/Closeable;Z)V
    //   54: lload 7
    //   56: lreturn
    //   57: astore 9
    //   59: aload 4
    //   61: checkcast 71	java/io/Closeable
    //   64: iload_2
    //   65: invokestatic 77	com/google/common/io/Closeables:close	(Ljava/io/Closeable;Z)V
    //   68: aload 9
    //   70: athrow
    //   71: astore 10
    //   73: aload_3
    //   74: checkcast 71	java/io/Closeable
    //   77: iload_2
    //   78: invokestatic 77	com/google/common/io/Closeables:close	(Ljava/io/Closeable;Z)V
    //   81: aload 10
    //   83: athrow
    // Line number table:
    //   Java source line #150	-> byte code offset #0
    //   Java source line #151	-> byte code offset #2
    //   Java source line #153	-> byte code offset #12
    //   Java source line #155	-> byte code offset #23
    //   Java source line #156	-> byte code offset #31
    //   Java source line #157	-> byte code offset #33
    //   Java source line #159	-> byte code offset #37
    //   Java source line #162	-> byte code offset #46
    //   Java source line #159	-> byte code offset #57
    //   Java source line #162	-> byte code offset #71
    // Local variable table:
    //   start	length	slot	name	signature
    //   0	84	0	from	InputSupplier<R>
    //   0	84	1	to	OutputSupplier<W>
    //   1	77	2	threw	boolean
    //   11	63	3	in	R
    //   21	39	4	out	W
    //   29	5	5	count	long
    //   35	20	7	l1	long
    //   57	12	9	localObject1	Object
    //   71	11	10	localObject2	Object
    // Exception table:
    //   from	to	target	type
    //   23	37	57	finally
    //   57	59	57	finally
    //   12	46	71	finally
    //   57	73	71	finally
  }
  
  public static <R extends Readable,  extends Closeable> long copy(InputSupplier<R> from, Appendable to)
    throws IOException
  {
    boolean threw = true;
    R in = (Readable)from.getInput();
    try {
      long count = copy(in, to);
      threw = false;
      return count;
    } finally {
      Closeables.close((Closeable)in, threw);
    }
  }
  







  public static long copy(Readable from, Appendable to)
    throws IOException
  {
    CharBuffer buf = CharBuffer.allocate(2048);
    long total = 0L;
    for (;;) {
      int r = from.read(buf);
      if (r == -1) {
        break;
      }
      buf.flip();
      to.append(buf, 0, r);
      total += r;
    }
    return total;
  }
  






  public static String toString(Readable r)
    throws IOException
  {
    return toStringBuilder(r).toString();
  }
  







  public static <R extends Readable,  extends Closeable> String toString(InputSupplier<R> supplier)
    throws IOException
  {
    return toStringBuilder(supplier).toString();
  }
  






  private static StringBuilder toStringBuilder(Readable r)
    throws IOException
  {
    StringBuilder sb = new StringBuilder();
    copy(r, sb);
    return sb;
  }
  






  private static <R extends Readable,  extends Closeable> StringBuilder toStringBuilder(InputSupplier<R> supplier)
    throws IOException
  {
    boolean threw = true;
    R r = (Readable)supplier.getInput();
    try {
      StringBuilder result = toStringBuilder(r);
      threw = false;
      return result;
    } finally {
      Closeables.close((Closeable)r, threw);
    }
  }
  








  public static <R extends Readable,  extends Closeable> String readFirstLine(InputSupplier<R> supplier)
    throws IOException
  {
    boolean threw = true;
    R r = (Readable)supplier.getInput();
    try {
      String line = new LineReader(r).readLine();
      threw = false;
      return line;
    } finally {
      Closeables.close((Closeable)r, threw);
    }
  }
  








  public static <R extends Readable,  extends Closeable> List<String> readLines(InputSupplier<R> supplier)
    throws IOException
  {
    boolean threw = true;
    R r = (Readable)supplier.getInput();
    try {
      List<String> result = readLines(r);
      threw = false;
      return result;
    } finally {
      Closeables.close((Closeable)r, threw);
    }
  }
  











  public static List<String> readLines(Readable r)
    throws IOException
  {
    List<String> result = new ArrayList();
    LineReader lineReader = new LineReader(r);
    String line;
    while ((line = lineReader.readLine()) != null) {
      result.add(line);
    }
    return result;
  }
  









  public static <R extends Readable,  extends Closeable, T> T readLines(InputSupplier<R> supplier, LineProcessor<T> callback)
    throws IOException
  {
    boolean threw = true;
    R r = (Readable)supplier.getInput();
    try {
      LineReader lineReader = new LineReader(r);
      String line;
      while ((line = lineReader.readLine()) != null) {
        if (!callback.processLine(line)) {
          break;
        }
      }
      threw = false;
    } finally {
      Closeables.close((Closeable)r, threw);
    }
    return callback.getResult();
  }
  















  public static InputSupplier<Reader> join(Iterable<? extends InputSupplier<? extends Reader>> suppliers)
  {
    new InputSupplier() {
      public Reader getInput() throws IOException {
        return new MultiReader(val$suppliers.iterator());
      }
    };
  }
  

  public static InputSupplier<Reader> join(InputSupplier<? extends Reader>... suppliers)
  {
    return join(Arrays.asList(suppliers));
  }
  









  public static void skipFully(Reader reader, long n)
    throws IOException
  {
    while (n > 0L) {
      long amt = reader.skip(n);
      if (amt == 0L)
      {
        if (reader.read() == -1) {
          throw new EOFException();
        }
        n -= 1L;
      } else {
        n -= amt;
      }
    }
  }
  









  public static Writer asWriter(Appendable target)
  {
    if ((target instanceof Writer)) {
      return (Writer)target;
    }
    return new AppendableWriter(target);
  }
}
