package com.mysql.jdbc;

import java.sql.SQLException;
import java.util.Iterator;


























public abstract class IterateBlock
{
  DatabaseMetaData.IteratorWithCleanup iteratorWithCleanup;
  Iterator javaIterator;
  boolean stopIterating = false;
  
  IterateBlock(DatabaseMetaData.IteratorWithCleanup i) {
    iteratorWithCleanup = i;
    javaIterator = null;
  }
  
  IterateBlock(Iterator i) {
    javaIterator = i;
    iteratorWithCleanup = null;
  }
  
  public void doForAll() throws SQLException {
    if (iteratorWithCleanup != null) {
      try {
        while (iteratorWithCleanup.hasNext()) {
          forEach(iteratorWithCleanup.next());
          
          if (stopIterating) {
            break;
          }
        }
      } finally {
        iteratorWithCleanup.close();
      }
    } else {
      while (javaIterator.hasNext()) {
        forEach(javaIterator.next());
        
        if (stopIterating) {
          break;
        }
      }
    }
  }
  
  abstract void forEach(Object paramObject) throws SQLException;
  
  public final boolean fullIteration() {
    return !stopIterating;
  }
}
