package net.minecraft.server.v1_7_R1;

import java.util.List;

public class NBTTagList extends NBTBase
{
  private List list = new java.util.ArrayList();
  private byte type = 0;
  



  void write(java.io.DataOutput paramDataOutput)
  {
    if (!list.isEmpty()) type = ((NBTBase)list.get(0)).getTypeId(); else {
      type = 0;
    }
    paramDataOutput.writeByte(type);
    paramDataOutput.writeInt(list.size());
    for (int i = 0; i < list.size(); i++) {
      ((NBTBase)list.get(i)).write(paramDataOutput);
    }
  }
  
  void load(java.io.DataInput paramDataInput, int paramInt) {
    if (paramInt > 512) {
      throw new RuntimeException("Tried to read NBT tag with too high complexity, depth > 512");
    }
    type = paramDataInput.readByte();
    int i = paramDataInput.readInt();
    
    list = new java.util.ArrayList();
    for (int j = 0; j < i; j++) {
      NBTBase localNBTBase = NBTBase.createTag(type);
      localNBTBase.load(paramDataInput, paramInt + 1);
      list.add(localNBTBase);
    }
  }
  
  public byte getTypeId()
  {
    return 9;
  }
  
  public String toString()
  {
    String str = "[";
    int i = 0;
    for (NBTBase localNBTBase : list) {
      str = str + "" + i + ':' + localNBTBase + ',';
      i++;
    }
    return str + "]";
  }
  











  public void add(NBTBase paramNBTBase)
  {
    if (type == 0) {
      type = paramNBTBase.getTypeId();
    } else if (type != paramNBTBase.getTypeId()) {
      System.err.println("WARNING: Adding mismatching tag types to tag list");
      return;
    }
    list.add(paramNBTBase);
  }
  

















  public NBTTagCompound get(int paramInt)
  {
    if ((paramInt < 0) || (paramInt >= list.size())) {
      return new NBTTagCompound();
    }
    NBTBase localNBTBase = (NBTBase)list.get(paramInt);
    if (localNBTBase.getTypeId() == 10) {
      return (NBTTagCompound)localNBTBase;
    }
    return new NBTTagCompound();
  }
  
  public int[] c(int paramInt) {
    if ((paramInt < 0) || (paramInt >= list.size())) {
      return new int[0];
    }
    NBTBase localNBTBase = (NBTBase)list.get(paramInt);
    if (localNBTBase.getTypeId() == 11) {
      return ((NBTTagIntArray)localNBTBase).c();
    }
    return new int[0];
  }
  
  public double d(int paramInt) {
    if ((paramInt < 0) || (paramInt >= list.size())) {
      return 0.0D;
    }
    NBTBase localNBTBase = (NBTBase)list.get(paramInt);
    if (localNBTBase.getTypeId() == 6) {
      return ((NBTTagDouble)localNBTBase).g();
    }
    return 0.0D;
  }
  
  public float e(int paramInt) {
    if ((paramInt < 0) || (paramInt >= list.size())) {
      return 0.0F;
    }
    NBTBase localNBTBase = (NBTBase)list.get(paramInt);
    if (localNBTBase.getTypeId() == 5) {
      return ((NBTTagFloat)localNBTBase).h();
    }
    return 0.0F;
  }
  
  public String f(int paramInt) {
    if ((paramInt < 0) || (paramInt >= list.size())) {
      return "";
    }
    NBTBase localNBTBase = (NBTBase)list.get(paramInt);
    if (localNBTBase.getTypeId() == 8) {
      return localNBTBase.a_();
    }
    return localNBTBase.toString();
  }
  
  public int size() {
    return list.size();
  }
  
  public NBTBase clone()
  {
    NBTTagList localNBTTagList = new NBTTagList();
    type = type;
    for (NBTBase localNBTBase1 : list) {
      NBTBase localNBTBase2 = localNBTBase1.clone();
      list.add(localNBTBase2);
    }
    return localNBTTagList;
  }
  
  public boolean equals(Object paramObject)
  {
    if (super.equals(paramObject)) {
      NBTTagList localNBTTagList = (NBTTagList)paramObject;
      if (type == type) {
        return list.equals(list);
      }
    }
    return false;
  }
  
  public int hashCode()
  {
    return super.hashCode() ^ list.hashCode();
  }
  
  public int d() {
    return type;
  }
}
