package net.minecraft.server.v1_7_R1;





public class WorldData
{
  private long seed;
  



  private WorldType type = WorldType.NORMAL;
  private String generatorOptions = "";
  
  private int spawnX;
  
  private int spawnY;
  
  private int spawnZ;
  private long time;
  private long dayTime;
  private long lastPlayed;
  private long sizeOnDisk;
  private NBTTagCompound playerData;
  private int dimension;
  private String name;
  private int version;
  private boolean isRaining;
  private int rainTicks;
  private boolean isThundering;
  private int thunderTicks;
  private EnumGamemode gameType;
  private boolean useMapFeatures;
  private boolean hardcore;
  private boolean allowCommands;
  private boolean initialized;
  private GameRules gameRules = new GameRules();
  

  protected WorldData() {}
  

  public WorldData(NBTTagCompound paramNBTTagCompound)
  {
    seed = paramNBTTagCompound.getLong("RandomSeed");
    if (paramNBTTagCompound.hasKeyOfType("generatorName", 8)) {
      String str = paramNBTTagCompound.getString("generatorName");
      type = WorldType.getType(str);
      if (type == null) {
        type = WorldType.NORMAL;
      } else if (type.f()) {
        int i = 0;
        if (paramNBTTagCompound.hasKeyOfType("generatorVersion", 99)) {
          i = paramNBTTagCompound.getInt("generatorVersion");
        }
        type = type.a(i);
      }
      
      if (paramNBTTagCompound.hasKeyOfType("generatorOptions", 8)) generatorOptions = paramNBTTagCompound.getString("generatorOptions");
    }
    gameType = EnumGamemode.a(paramNBTTagCompound.getInt("GameType"));
    if (paramNBTTagCompound.hasKeyOfType("MapFeatures", 99)) {
      useMapFeatures = paramNBTTagCompound.getBoolean("MapFeatures");
    } else {
      useMapFeatures = true;
    }
    spawnX = paramNBTTagCompound.getInt("SpawnX");
    spawnY = paramNBTTagCompound.getInt("SpawnY");
    spawnZ = paramNBTTagCompound.getInt("SpawnZ");
    time = paramNBTTagCompound.getLong("Time");
    if (paramNBTTagCompound.hasKeyOfType("DayTime", 99)) {
      dayTime = paramNBTTagCompound.getLong("DayTime");
    } else {
      dayTime = time;
    }
    lastPlayed = paramNBTTagCompound.getLong("LastPlayed");
    sizeOnDisk = paramNBTTagCompound.getLong("SizeOnDisk");
    name = paramNBTTagCompound.getString("LevelName");
    version = paramNBTTagCompound.getInt("version");
    rainTicks = paramNBTTagCompound.getInt("rainTime");
    isRaining = paramNBTTagCompound.getBoolean("raining");
    thunderTicks = paramNBTTagCompound.getInt("thunderTime");
    isThundering = paramNBTTagCompound.getBoolean("thundering");
    hardcore = paramNBTTagCompound.getBoolean("hardcore");
    
    if (paramNBTTagCompound.hasKeyOfType("initialized", 99)) {
      initialized = paramNBTTagCompound.getBoolean("initialized");
    } else {
      initialized = true;
    }
    
    if (paramNBTTagCompound.hasKeyOfType("allowCommands", 99)) {
      allowCommands = paramNBTTagCompound.getBoolean("allowCommands");
    } else {
      allowCommands = (gameType == EnumGamemode.CREATIVE);
    }
    
    if (paramNBTTagCompound.hasKeyOfType("Player", 10)) {
      playerData = paramNBTTagCompound.getCompound("Player");
      dimension = playerData.getInt("Dimension");
    }
    
    if (paramNBTTagCompound.hasKeyOfType("GameRules", 10)) {
      gameRules.a(paramNBTTagCompound.getCompound("GameRules"));
    }
  }
  
  public WorldData(WorldSettings paramWorldSettings, String paramString) {
    seed = paramWorldSettings.d();
    gameType = paramWorldSettings.e();
    useMapFeatures = paramWorldSettings.g();
    name = paramString;
    hardcore = paramWorldSettings.f();
    type = paramWorldSettings.h();
    generatorOptions = paramWorldSettings.j();
    allowCommands = paramWorldSettings.i();
    initialized = false;
  }
  
  public WorldData(WorldData paramWorldData) {
    seed = seed;
    type = type;
    generatorOptions = generatorOptions;
    gameType = gameType;
    useMapFeatures = useMapFeatures;
    spawnX = spawnX;
    spawnY = spawnY;
    spawnZ = spawnZ;
    time = time;
    dayTime = dayTime;
    lastPlayed = lastPlayed;
    sizeOnDisk = sizeOnDisk;
    playerData = playerData;
    dimension = dimension;
    name = name;
    version = version;
    rainTicks = rainTicks;
    isRaining = isRaining;
    thunderTicks = thunderTicks;
    isThundering = isThundering;
    hardcore = hardcore;
    allowCommands = allowCommands;
    initialized = initialized;
    gameRules = gameRules;
  }
  
  public NBTTagCompound a() {
    NBTTagCompound localNBTTagCompound = new NBTTagCompound();
    
    a(localNBTTagCompound, playerData);
    
    return localNBTTagCompound;
  }
  
  public NBTTagCompound a(NBTTagCompound paramNBTTagCompound) {
    NBTTagCompound localNBTTagCompound = new NBTTagCompound();
    a(localNBTTagCompound, paramNBTTagCompound);
    
    return localNBTTagCompound;
  }
  
  private void a(NBTTagCompound paramNBTTagCompound1, NBTTagCompound paramNBTTagCompound2) {
    paramNBTTagCompound1.setLong("RandomSeed", seed);
    paramNBTTagCompound1.setString("generatorName", type.name());
    paramNBTTagCompound1.setInt("generatorVersion", type.getVersion());
    paramNBTTagCompound1.setString("generatorOptions", generatorOptions);
    paramNBTTagCompound1.setInt("GameType", gameType.a());
    paramNBTTagCompound1.setBoolean("MapFeatures", useMapFeatures);
    paramNBTTagCompound1.setInt("SpawnX", spawnX);
    paramNBTTagCompound1.setInt("SpawnY", spawnY);
    paramNBTTagCompound1.setInt("SpawnZ", spawnZ);
    paramNBTTagCompound1.setLong("Time", time);
    paramNBTTagCompound1.setLong("DayTime", dayTime);
    paramNBTTagCompound1.setLong("SizeOnDisk", sizeOnDisk);
    paramNBTTagCompound1.setLong("LastPlayed", MinecraftServer.ap());
    paramNBTTagCompound1.setString("LevelName", name);
    paramNBTTagCompound1.setInt("version", version);
    paramNBTTagCompound1.setInt("rainTime", rainTicks);
    paramNBTTagCompound1.setBoolean("raining", isRaining);
    paramNBTTagCompound1.setInt("thunderTime", thunderTicks);
    paramNBTTagCompound1.setBoolean("thundering", isThundering);
    paramNBTTagCompound1.setBoolean("hardcore", hardcore);
    paramNBTTagCompound1.setBoolean("allowCommands", allowCommands);
    paramNBTTagCompound1.setBoolean("initialized", initialized);
    paramNBTTagCompound1.set("GameRules", gameRules.a());
    
    if (paramNBTTagCompound2 != null) {
      paramNBTTagCompound1.set("Player", paramNBTTagCompound2);
    }
  }
  
  public long getSeed() {
    return seed;
  }
  
  public int c() {
    return spawnX;
  }
  
  public int d() {
    return spawnY;
  }
  
  public int e() {
    return spawnZ;
  }
  
  public long getTime() {
    return time;
  }
  
  public long getDayTime() {
    return dayTime;
  }
  



  public NBTTagCompound i()
  {
    return playerData;
  }
  
  public int j() {
    return dimension;
  }
  















  public void setTime(long paramLong)
  {
    time = paramLong;
  }
  
  public void setDayTime(long paramLong) {
    dayTime = paramLong;
  }
  











  public void setSpawn(int paramInt1, int paramInt2, int paramInt3)
  {
    spawnX = paramInt1;
    spawnY = paramInt2;
    spawnZ = paramInt3;
  }
  
  public String getName() {
    return name;
  }
  
  public void setName(String paramString) {
    name = paramString;
  }
  
  public int l() {
    return version;
  }
  
  public void e(int paramInt) {
    version = paramInt;
  }
  



  public boolean isThundering()
  {
    return isThundering;
  }
  
  public void setThundering(boolean paramBoolean) {
    isThundering = paramBoolean;
  }
  
  public int getThunderDuration() {
    return thunderTicks;
  }
  
  public void setThunderDuration(int paramInt) {
    thunderTicks = paramInt;
  }
  
  public boolean hasStorm() {
    return isRaining;
  }
  
  public void setStorm(boolean paramBoolean) {
    isRaining = paramBoolean;
  }
  
  public int getWeatherDuration() {
    return rainTicks;
  }
  
  public void setWeatherDuration(int paramInt) {
    rainTicks = paramInt;
  }
  
  public EnumGamemode getGameType() {
    return gameType;
  }
  
  public boolean shouldGenerateMapFeatures() {
    return useMapFeatures;
  }
  
  public void setGameType(EnumGamemode paramEnumGamemode) {
    gameType = paramEnumGamemode;
  }
  
  public boolean isHardcore() {
    return hardcore;
  }
  
  public WorldType getType() {
    return type;
  }
  
  public void setType(WorldType paramWorldType) {
    type = paramWorldType;
  }
  
  public String getGeneratorOptions() {
    return generatorOptions;
  }
  



  public boolean allowCommands()
  {
    return allowCommands;
  }
  



  public boolean isInitialized()
  {
    return initialized;
  }
  
  public void d(boolean paramBoolean) {
    initialized = paramBoolean;
  }
  
  public GameRules getGameRules() {
    return gameRules;
  }
  
  public void a(CrashReportSystemDetails paramCrashReportSystemDetails) {
    paramCrashReportSystemDetails.a("Level seed", new CrashReportLevelSeed(this));
    





    paramCrashReportSystemDetails.a("Level generator", new CrashReportLevelGenerator(this));
    





    paramCrashReportSystemDetails.a("Level generator options", new CrashReportLevelGeneratorOptions(this));
    





    paramCrashReportSystemDetails.a("Level spawn location", new CrashReportLevelSpawnLocation(this));
    





    paramCrashReportSystemDetails.a("Level time", new CrashReportLevelTime(this));
    





    paramCrashReportSystemDetails.a("Level dimension", new CrashReportLevelDimension(this));
    





    paramCrashReportSystemDetails.a("Level storage version", new CrashReportLevelStorageVersion(this));
    




















    paramCrashReportSystemDetails.a("Level weather", new CrashReportLevelWeather(this));
    





    paramCrashReportSystemDetails.a("Level game mode", new CrashReportLevelGameMode(this));
  }
}
