package net.minecraft.util.io.netty.channel.nio;

import java.io.IOException;
import java.nio.channels.SelectableChannel;
import java.nio.channels.SelectionKey;
import java.util.ArrayList;
import java.util.List;
import net.minecraft.util.io.netty.channel.Channel;
import net.minecraft.util.io.netty.channel.ChannelConfig;
import net.minecraft.util.io.netty.channel.ChannelOutboundBuffer;
import net.minecraft.util.io.netty.channel.ChannelPipeline;
import net.minecraft.util.io.netty.channel.ServerChannel;





















public abstract class AbstractNioMessageChannel
  extends AbstractNioChannel
{
  protected AbstractNioMessageChannel(Channel parent, SelectableChannel ch, int readInterestOp)
  {
    super(parent, ch, readInterestOp);
  }
  


  protected AbstractNioChannel.AbstractNioUnsafe newUnsafe() { return new NioMessageUnsafe(null); }
  
  private final class NioMessageUnsafe extends AbstractNioChannel.AbstractNioUnsafe {
    private NioMessageUnsafe() { super(); }
    
    private final List<Object> readBuf = new ArrayList();
    
    public void read()
    {
      assert (eventLoop().inEventLoop());
      SelectionKey key = selectionKey();
      if (!config().isAutoRead()) {
        int interestOps = key.interestOps();
        if ((interestOps & readInterestOp) != 0)
        {
          key.interestOps(interestOps & (readInterestOp ^ 0xFFFFFFFF));
        }
      }
      
      ChannelConfig config = config();
      int maxMessagesPerRead = config.getMaxMessagesPerRead();
      boolean autoRead = config.isAutoRead();
      ChannelPipeline pipeline = pipeline();
      boolean closed = false;
      Throwable exception = null;
      try {
        for (;;) {
          int localRead = doReadMessages(readBuf);
          if (localRead == 0) {
            break;
          }
          if (localRead < 0) {
            closed = true;
          }
          else
          {
            if (((readBuf.size() >= maxMessagesPerRead ? 1 : 0) | (!autoRead ? 1 : 0)) != 0)
              break;
          }
        }
      } catch (Throwable t) {
        exception = t;
      }
      
      int size = readBuf.size();
      for (int i = 0; i < size; i++) {
        pipeline.fireChannelRead(readBuf.get(i));
      }
      readBuf.clear();
      pipeline.fireChannelReadComplete();
      
      if (exception != null) {
        if ((exception instanceof IOException))
        {

          closed = !(AbstractNioMessageChannel.this instanceof ServerChannel);
        }
        
        pipeline.fireExceptionCaught(exception);
      }
      
      if ((closed) && 
        (isOpen())) {
        close(voidPromise());
      }
    }
  }
  
  protected void doWrite(ChannelOutboundBuffer in)
    throws Exception
  {
    SelectionKey key = selectionKey();
    int interestOps = key.interestOps();
    for (;;)
    {
      Object msg = in.current();
      if (msg == null)
      {
        if ((interestOps & 0x4) == 0) break;
        key.interestOps(interestOps & 0xFFFFFFFB); break;
      }
      


      boolean done = false;
      for (int i = config().getWriteSpinCount() - 1; i >= 0; i--) {
        if (doWriteMessage(msg, in)) {
          done = true;
          break;
        }
      }
      
      if (done) {
        in.remove();
      }
      else {
        if ((interestOps & 0x4) != 0) break;
        key.interestOps(interestOps | 0x4); break;
      }
    }
  }
  
  protected abstract int doReadMessages(List<Object> paramList)
    throws Exception;
  
  protected abstract boolean doWriteMessage(Object paramObject, ChannelOutboundBuffer paramChannelOutboundBuffer)
    throws Exception;
}
