package org.bukkit.craftbukkit.libs.jline.console.completer;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;
import org.bukkit.craftbukkit.libs.jline.internal.Log;


















public class ArgumentCompleter
  implements Completer
{
  private final ArgumentDelimiter delimiter;
  private final List<Completer> completers = new ArrayList();
  
  private boolean strict = true;
  





  public ArgumentCompleter(ArgumentDelimiter delimiter, Collection<Completer> completers)
  {
    assert (delimiter != null);
    this.delimiter = delimiter;
    assert (completers != null);
    this.completers.addAll(completers);
  }
  





  public ArgumentCompleter(ArgumentDelimiter delimiter, Completer... completers)
  {
    this(delimiter, Arrays.asList(completers));
  }
  




  public ArgumentCompleter(Completer... completers)
  {
    this(new WhitespaceArgumentDelimiter(), completers);
  }
  




  public ArgumentCompleter(List<Completer> completers)
  {
    this(new WhitespaceArgumentDelimiter(), completers);
  }
  



  public void setStrict(boolean strict)
  {
    this.strict = strict;
  }
  






  public boolean isStrict()
  {
    return strict;
  }
  


  public ArgumentDelimiter getDelimiter()
  {
    return delimiter;
  }
  


  public List<Completer> getCompleters()
  {
    return completers;
  }
  
  public int complete(String buffer, int cursor, List<CharSequence> candidates)
  {
    assert (candidates != null);
    
    ArgumentDelimiter delim = getDelimiter();
    ArgumentList list = delim.delimit(buffer, cursor);
    int argpos = list.getArgumentPosition();
    int argIndex = list.getCursorArgumentIndex();
    
    if (argIndex < 0) {
      return -1;
    }
    
    List<Completer> completers = getCompleters();
    
    Completer completer;
    Completer completer;
    if (argIndex >= completers.size()) {
      completer = (Completer)completers.get(completers.size() - 1);
    }
    else {
      completer = (Completer)completers.get(argIndex);
    }
    

    for (int i = 0; (isStrict()) && (i < argIndex); i++) {
      Completer sub = (Completer)completers.get(i >= completers.size() ? completers.size() - 1 : i);
      String[] args = list.getArguments();
      String arg = (args == null) || (i >= args.length) ? "" : args[i];
      
      List<CharSequence> subCandidates = new LinkedList();
      
      if (sub.complete(arg, arg.length(), subCandidates) == -1) {
        return -1;
      }
      
      if (subCandidates.size() == 0) {
        return -1;
      }
    }
    
    int ret = completer.complete(list.getCursorArgument(), argpos, candidates);
    
    if (ret == -1) {
      return -1;
    }
    
    int pos = ret + list.getBufferPosition() - argpos;
    






    if ((cursor != buffer.length()) && (delim.isDelimiter(buffer, cursor))) {
      for (int i = 0; i < candidates.size(); i++) {
        CharSequence val = (CharSequence)candidates.get(i);
        
        while ((val.length() > 0) && (delim.isDelimiter(val, val.length() - 1))) {
          val = val.subSequence(0, val.length() - 1);
        }
        
        candidates.set(i, val);
      }
    }
    
    Log.trace(new Object[] { "Completing ", buffer, " (pos=", Integer.valueOf(cursor), ") with: ", candidates, ": offset=", Integer.valueOf(pos) });
    
    return pos;
  }
  










  public static abstract interface ArgumentDelimiter
  {
    public abstract ArgumentCompleter.ArgumentList delimit(CharSequence paramCharSequence, int paramInt);
    









    public abstract boolean isDelimiter(CharSequence paramCharSequence, int paramInt);
  }
  









  public static abstract class AbstractArgumentDelimiter
    implements ArgumentCompleter.ArgumentDelimiter
  {
    private char[] quoteChars = { '\'', '"' };
    
    private char[] escapeChars = { '\\' };
    
    public void setQuoteChars(char[] chars) {
      quoteChars = chars;
    }
    
    public char[] getQuoteChars() {
      return quoteChars;
    }
    
    public void setEscapeChars(char[] chars) {
      escapeChars = chars;
    }
    
    public char[] getEscapeChars() {
      return escapeChars;
    }
    
    public ArgumentCompleter.ArgumentList delimit(CharSequence buffer, int cursor) {
      List<String> args = new LinkedList();
      StringBuilder arg = new StringBuilder();
      int argpos = -1;
      int bindex = -1;
      
      for (int i = 0; (buffer != null) && (i <= buffer.length()); i++)
      {

        if (i == cursor) {
          bindex = args.size();
          

          argpos = arg.length();
        }
        
        if ((i == buffer.length()) || (isDelimiter(buffer, i))) {
          if (arg.length() > 0) {
            args.add(arg.toString());
            arg.setLength(0);
          }
        }
        else {
          arg.append(buffer.charAt(i));
        }
      }
      
      return new ArgumentCompleter.ArgumentList((String[])args.toArray(new String[args.size()]), bindex, argpos, cursor);
    }
    








    public boolean isDelimiter(CharSequence buffer, int pos)
    {
      return (!isQuoted(buffer, pos)) && (!isEscaped(buffer, pos)) && (isDelimiterChar(buffer, pos));
    }
    
    public boolean isQuoted(CharSequence buffer, int pos) {
      return false;
    }
    
    public boolean isEscaped(CharSequence buffer, int pos) {
      if (pos <= 0) {
        return false;
      }
      
      for (int i = 0; (escapeChars != null) && (i < escapeChars.length); 
          i++) {
        if (buffer.charAt(pos) == escapeChars[i]) {
          return !isEscaped(buffer, pos - 1);
        }
      }
      
      return false;
    }
    








    public abstract boolean isDelimiterChar(CharSequence paramCharSequence, int paramInt);
  }
  







  public static class WhitespaceArgumentDelimiter
    extends ArgumentCompleter.AbstractArgumentDelimiter
  {
    public boolean isDelimiterChar(CharSequence buffer, int pos)
    {
      return Character.isWhitespace(buffer.charAt(pos));
    }
  }
  



  public static class ArgumentList
  {
    private String[] arguments;
    


    private int cursorArgumentIndex;
    


    private int argumentPosition;
    

    private int bufferPosition;
    


    public ArgumentList(String[] arguments, int cursorArgumentIndex, int argumentPosition, int bufferPosition)
    {
      assert (arguments != null);
      
      this.arguments = arguments;
      this.cursorArgumentIndex = cursorArgumentIndex;
      this.argumentPosition = argumentPosition;
      this.bufferPosition = bufferPosition;
    }
    
    public void setCursorArgumentIndex(int i) {
      cursorArgumentIndex = i;
    }
    
    public int getCursorArgumentIndex() {
      return cursorArgumentIndex;
    }
    
    public String getCursorArgument() {
      if ((cursorArgumentIndex < 0) || (cursorArgumentIndex >= arguments.length)) {
        return null;
      }
      
      return arguments[cursorArgumentIndex];
    }
    
    public void setArgumentPosition(int pos) {
      argumentPosition = pos;
    }
    
    public int getArgumentPosition() {
      return argumentPosition;
    }
    
    public void setArguments(String[] arguments) {
      this.arguments = arguments;
    }
    
    public String[] getArguments() {
      return arguments;
    }
    
    public void setBufferPosition(int pos) {
      bufferPosition = pos;
    }
    
    public int getBufferPosition() {
      return bufferPosition;
    }
  }
}
