package org.bukkit.craftbukkit.libs.org.ibex.nestedvm.util;

import java.io.IOException;

public abstract class Seekable {
  public abstract int read(byte[] paramArrayOfByte, int paramInt1, int paramInt2) throws IOException;
  
  public abstract int write(byte[] paramArrayOfByte, int paramInt1, int paramInt2) throws IOException;
  
  public abstract int length() throws IOException;
  
  public abstract void seek(int paramInt) throws IOException;
  
  public abstract void close() throws IOException;
  
  public abstract int pos() throws IOException;
  
  public void sync() throws IOException { throw new IOException("sync not implemented for " + getClass()); }
  
  public void resize(long paramLong) throws IOException {
    throw new IOException("resize not implemented for " + getClass());
  }
  
  public Lock lock(long paramLong1, long paramLong2, boolean paramBoolean) throws IOException {
    throw new IOException("lock not implemented for " + getClass());
  }
  
  public int read() throws IOException {
    byte[] arrayOfByte = new byte[1];
    int i = read(arrayOfByte, 0, 1);
    return i == -1 ? -1 : arrayOfByte[0] & 0xFF;
  }
  
  public int tryReadFully(byte[] paramArrayOfByte, int paramInt1, int paramInt2) throws IOException {
    int i = 0;
    while (paramInt2 > 0) {
      int j = read(paramArrayOfByte, paramInt1, paramInt2);
      if (j == -1) break;
      paramInt1 += j;
      paramInt2 -= j;
      i += j;
    }
    return i == 0 ? -1 : i;
  }
  
  public static class ByteArray extends Seekable {
    protected byte[] data;
    protected int pos;
    private final boolean writable;
    
    public ByteArray(byte[] paramArrayOfByte, boolean paramBoolean) {
      data = paramArrayOfByte;
      pos = 0;
      writable = paramBoolean;
    }
    
    public int read(byte[] paramArrayOfByte, int paramInt1, int paramInt2) {
      paramInt2 = Math.min(paramInt2, data.length - pos);
      if (paramInt2 <= 0) return -1;
      System.arraycopy(data, pos, paramArrayOfByte, paramInt1, paramInt2);
      pos += paramInt2;
      return paramInt2;
    }
    
    public int write(byte[] paramArrayOfByte, int paramInt1, int paramInt2) throws IOException {
      if (!writable) throw new IOException("read-only data");
      paramInt2 = Math.min(paramInt2, data.length - pos);
      if (paramInt2 <= 0) throw new IOException("no space");
      System.arraycopy(paramArrayOfByte, paramInt1, data, pos, paramInt2);
      pos += paramInt2;
      return paramInt2;
    }
    
    public int length() { return data.length; }
    public int pos() { return pos; }
    public void seek(int paramInt) { pos = paramInt; }
    
    public void close() {}
  }
  
  public static class File extends Seekable { private final java.io.File file;
    private final java.io.RandomAccessFile raf;
    
    public File(String paramString) throws IOException { this(paramString, false); }
    public File(String paramString, boolean paramBoolean) throws IOException { this(new java.io.File(paramString), paramBoolean, false); }
    
    public File(java.io.File paramFile, boolean paramBoolean1, boolean paramBoolean2) throws IOException {
      file = paramFile;
      String str = paramBoolean1 ? "rw" : "r";
      raf = new java.io.RandomAccessFile(paramFile, str);
      if (paramBoolean2) Platform.setFileLength(raf, 0);
    }
    
    public int read(byte[] paramArrayOfByte, int paramInt1, int paramInt2) throws IOException { return raf.read(paramArrayOfByte, paramInt1, paramInt2); }
    public int write(byte[] paramArrayOfByte, int paramInt1, int paramInt2) throws IOException { raf.write(paramArrayOfByte, paramInt1, paramInt2);return paramInt2; }
    public void sync() throws IOException { raf.getFD().sync(); }
    public void seek(int paramInt) throws IOException { raf.seek(paramInt); }
    public int pos() throws IOException { return (int)raf.getFilePointer(); }
    public int length() throws IOException { return (int)raf.length(); }
    public void close() throws IOException { raf.close(); }
    public void resize(long paramLong) throws IOException { Platform.setFileLength(raf, (int)paramLong); }
    
    public boolean equals(Object paramObject) { return (paramObject != null) && ((paramObject instanceof File)) && (file.equals(file)); }
    
    public Seekable.Lock lock(long paramLong1, long paramLong2, boolean paramBoolean)
      throws IOException
    {
      return Platform.lockFile(this, raf, paramLong1, paramLong2, paramBoolean);
    }
  }
  
  public static class InputStream extends Seekable {
    private byte[] buffer = new byte[4096];
    private int bytesRead = 0;
    private boolean eof = false;
    private int pos;
    private java.io.InputStream is;
    
    public InputStream(java.io.InputStream paramInputStream) { is = paramInputStream; }
    
    public int read(byte[] paramArrayOfByte, int paramInt1, int paramInt2) throws IOException {
      if ((pos >= bytesRead) && (!eof)) readTo(pos + 1);
      paramInt2 = Math.min(paramInt2, bytesRead - pos);
      if (paramInt2 <= 0) return -1;
      System.arraycopy(buffer, pos, paramArrayOfByte, paramInt1, paramInt2);
      pos += paramInt2;
      return paramInt2;
    }
    
    private void readTo(int paramInt) throws IOException {
      if (paramInt >= buffer.length) {
        byte[] arrayOfByte = new byte[Math.max(buffer.length + Math.min(buffer.length, 65536), paramInt)];
        System.arraycopy(buffer, 0, arrayOfByte, 0, bytesRead);
        buffer = arrayOfByte;
      }
      while (bytesRead < paramInt) {
        int i = is.read(buffer, bytesRead, buffer.length - bytesRead);
        if (i == -1) {
          eof = true;
          break;
        }
        bytesRead += i;
      }
    }
    
    public int length() throws IOException {
      while (!eof) readTo(bytesRead + 4096);
      return bytesRead;
    }
    
    public int write(byte[] paramArrayOfByte, int paramInt1, int paramInt2) throws IOException { throw new IOException("read-only"); }
    public void seek(int paramInt) { pos = paramInt; }
    public int pos() { return pos; }
    public void close() throws IOException { is.close(); } }
  public static abstract class Lock { public abstract Seekable seekable();
    public abstract boolean isShared();
    public abstract boolean isValid();
    private Object owner = null;
    

    public abstract void release() throws IOException;
    
    public abstract long position();
    
    public abstract long size();
    
    public void setOwner(Object paramObject) { owner = paramObject; }
    public Object getOwner() { return owner; }
    
    public final boolean contains(int paramInt1, int paramInt2) {
      return (paramInt1 >= position()) && (position() + size() >= paramInt1 + paramInt2);
    }
    
    public final boolean contained(int paramInt1, int paramInt2) {
      return (paramInt1 < position()) && (position() + size() < paramInt1 + paramInt2);
    }
    
    public final boolean overlaps(int paramInt1, int paramInt2) {
      return (contains(paramInt1, paramInt2)) || (contained(paramInt1, paramInt2));
    }
  }
}
