package org.bukkit.metadata;

import java.util.Map;
import org.apache.commons.lang.Validate;
import org.bukkit.plugin.Plugin;

public abstract class MetadataStoreBase<T>
{
  private Map<String, Map<Plugin, MetadataValue>> metadataMap = new java.util.HashMap();
  





















  public synchronized void setMetadata(T subject, String metadataKey, MetadataValue newMetadataValue)
  {
    Validate.notNull(newMetadataValue, "Value cannot be null");
    Plugin owningPlugin = newMetadataValue.getOwningPlugin();
    Validate.notNull(owningPlugin, "Plugin cannot be null");
    String key = disambiguate(subject, metadataKey);
    Map<Plugin, MetadataValue> entry = (Map)metadataMap.get(key);
    if (entry == null) {
      entry = new java.util.WeakHashMap(1);
      metadataMap.put(key, entry);
    }
    entry.put(owningPlugin, newMetadataValue);
  }
  









  public synchronized java.util.List<MetadataValue> getMetadata(T subject, String metadataKey)
  {
    String key = disambiguate(subject, metadataKey);
    if (metadataMap.containsKey(key)) {
      java.util.Collection<MetadataValue> values = ((Map)metadataMap.get(key)).values();
      return java.util.Collections.unmodifiableList(new java.util.ArrayList(values));
    }
    return java.util.Collections.emptyList();
  }
  








  public synchronized boolean hasMetadata(T subject, String metadataKey)
  {
    String key = disambiguate(subject, metadataKey);
    return metadataMap.containsKey(key);
  }
  










  public synchronized void removeMetadata(T subject, String metadataKey, Plugin owningPlugin)
  {
    Validate.notNull(owningPlugin, "Plugin cannot be null");
    String key = disambiguate(subject, metadataKey);
    Map<Plugin, MetadataValue> entry = (Map)metadataMap.get(key);
    if (entry == null) {
      return;
    }
    
    entry.remove(owningPlugin);
    if (entry.isEmpty()) {
      metadataMap.remove(key);
    }
  }
  








  public synchronized void invalidateAll(Plugin owningPlugin)
  {
    Validate.notNull(owningPlugin, "Plugin cannot be null");
    for (Map<Plugin, MetadataValue> values : metadataMap.values()) {
      if (values.containsKey(owningPlugin)) {
        ((MetadataValue)values.get(owningPlugin)).invalidate();
      }
    }
  }
  
  protected abstract String disambiguate(T paramT, String paramString);
}
