package org.sqlite;

import java.sql.BatchUpdateException;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;




















abstract class DB
  implements Codes
{
  Conn conn;
  long begin;
  long commit;
  private final Map<Long, Stmt> stmts;
  
  DB()
  {
    conn = null;
    

    begin = 0L;
    commit = 0L;
    

    stmts = new HashMap();
  }
  
  abstract void interrupt() throws SQLException;
  
  abstract void busy_timeout(int paramInt) throws SQLException;
  
  abstract String errmsg() throws SQLException;
  
  abstract String libversion() throws SQLException;
  
  abstract int changes() throws SQLException;
  
  abstract int shared_cache(boolean paramBoolean) throws SQLException;
  
  abstract int enable_load_extension(boolean paramBoolean) throws SQLException;
  
  final synchronized void exec(String sql) throws SQLException
  {
    long pointer = 0L;
    try {
      pointer = prepare(sql);
      switch (step(pointer)) {
      case 101: 
        ensureAutoCommit(); return;
      case 100: 
        return;
      }
      
      throwex();
    }
    finally
    {
      finalize(pointer);
    }
  }
  
  final synchronized void open(Conn conn, String file, int openFlags) throws SQLException {
    this.conn = conn;
    _open(file, openFlags);
  }
  
  final synchronized void close() throws SQLException
  {
    synchronized (stmts) {
      Iterator i = stmts.entrySet().iterator();
      while (i.hasNext()) {
        Map.Entry entry = (Map.Entry)i.next();
        Stmt stmt = (Stmt)entry.getValue();
        finalize(((Long)entry.getKey()).longValue());
        if (stmt != null) {
          pointer = 0L;
        }
        i.remove();
      }
    }
    

    free_functions();
    

    if (begin != 0L) {
      finalize(begin);
      begin = 0L;
    }
    if (commit != 0L) {
      finalize(commit);
      commit = 0L;
    }
    
    _close();
  }
  
  final synchronized void prepare(Stmt stmt) throws SQLException {
    if (pointer != 0L)
      finalize(stmt);
    pointer = prepare(sql);
    stmts.put(new Long(pointer), stmt);
  }
  
  final synchronized int finalize(Stmt stmt) throws SQLException {
    if (pointer == 0L)
      return 0;
    int rc = 1;
    try {
      rc = finalize(pointer);
    }
    finally {
      stmts.remove(new Long(pointer));
      pointer = 0L;
    }
    return rc;
  }
  
  protected abstract void _open(String paramString, int paramInt)
    throws SQLException;
  
  protected abstract void _close()
    throws SQLException;
  
  protected abstract int _exec(String paramString)
    throws SQLException;
  
  protected abstract long prepare(String paramString)
    throws SQLException;
  
  protected abstract int finalize(long paramLong)
    throws SQLException;
  
  protected abstract int step(long paramLong)
    throws SQLException;
  
  protected abstract int reset(long paramLong)
    throws SQLException;
  
  abstract int clear_bindings(long paramLong)
    throws SQLException;
  
  abstract int bind_parameter_count(long paramLong)
    throws SQLException;
  
  abstract int column_count(long paramLong)
    throws SQLException;
  
  abstract int column_type(long paramLong, int paramInt)
    throws SQLException;
  
  abstract String column_decltype(long paramLong, int paramInt) throws SQLException;
  
  abstract String column_table_name(long paramLong, int paramInt) throws SQLException;
  
  abstract String column_name(long paramLong, int paramInt) throws SQLException;
  
  abstract String column_text(long paramLong, int paramInt) throws SQLException;
  
  abstract byte[] column_blob(long paramLong, int paramInt) throws SQLException;
  
  abstract double column_double(long paramLong, int paramInt) throws SQLException;
  
  abstract long column_long(long paramLong, int paramInt) throws SQLException;
  
  abstract int column_int(long paramLong, int paramInt) throws SQLException;
  
  abstract int bind_null(long paramLong, int paramInt) throws SQLException;
  
  abstract int bind_int(long paramLong, int paramInt1, int paramInt2) throws SQLException;
  
  abstract int bind_long(long paramLong1, int paramInt, long paramLong2) throws SQLException;
  
  abstract int bind_double(long paramLong, int paramInt, double paramDouble) throws SQLException;
  
  abstract int bind_text(long paramLong, int paramInt, String paramString) throws SQLException;
  
  abstract int bind_blob(long paramLong, int paramInt, byte[] paramArrayOfByte) throws SQLException;
  
  abstract void result_null(long paramLong) throws SQLException;
  
  abstract void result_text(long paramLong, String paramString) throws SQLException;
  
  abstract void result_blob(long paramLong, byte[] paramArrayOfByte) throws SQLException;
  
  abstract void result_double(long paramLong, double paramDouble) throws SQLException;
  
  abstract void result_long(long paramLong1, long paramLong2) throws SQLException;
  
  abstract void result_int(long paramLong, int paramInt) throws SQLException;
  
  abstract void result_error(long paramLong, String paramString) throws SQLException;
  
  abstract int value_bytes(Function paramFunction, int paramInt) throws SQLException;
  
  abstract String value_text(Function paramFunction, int paramInt) throws SQLException;
  
  abstract byte[] value_blob(Function paramFunction, int paramInt) throws SQLException;
  
  abstract double value_double(Function paramFunction, int paramInt) throws SQLException;
  
  abstract long value_long(Function paramFunction, int paramInt) throws SQLException;
  
  abstract int value_int(Function paramFunction, int paramInt) throws SQLException;
  
  abstract int value_type(Function paramFunction, int paramInt) throws SQLException;
  
  abstract int create_function(String paramString, Function paramFunction) throws SQLException;
  
  abstract int destroy_function(String paramString) throws SQLException;
  
  abstract void free_functions() throws SQLException;
  
  abstract int backup(String paramString1, String paramString2, ProgressObserver paramProgressObserver) throws SQLException;
  
  abstract int restore(String paramString1, String paramString2, ProgressObserver paramProgressObserver) throws SQLException;
  
  abstract boolean[][] column_metadata(long paramLong) throws SQLException;
  
  final synchronized String[] column_names(long stmt) throws SQLException
  {
    String[] names = new String[column_count(stmt)];
    for (int i = 0; i < names.length; i++)
      names[i] = column_name(stmt, i);
    return names;
  }
  
  final synchronized int sqlbind(long stmt, int pos, Object v) throws SQLException {
    
    if (v == null) {
      return bind_null(stmt, pos);
    }
    if ((v instanceof Integer)) {
      return bind_int(stmt, pos, ((Integer)v).intValue());
    }
    if ((v instanceof Short)) {
      return bind_int(stmt, pos, ((Short)v).intValue());
    }
    if ((v instanceof Long)) {
      return bind_long(stmt, pos, ((Long)v).longValue());
    }
    if ((v instanceof Float)) {
      return bind_double(stmt, pos, ((Float)v).doubleValue());
    }
    if ((v instanceof Double)) {
      return bind_double(stmt, pos, ((Double)v).doubleValue());
    }
    if ((v instanceof String)) {
      return bind_text(stmt, pos, (String)v);
    }
    if ((v instanceof byte[])) {
      return bind_blob(stmt, pos, (byte[])v);
    }
    
    throw new SQLException("unexpected param type: " + v.getClass());
  }
  
  final synchronized int[] executeBatch(long stmt, int count, Object[] vals) throws SQLException
  {
    if (count < 1) {
      throw new SQLException("count (" + count + ") < 1");
    }
    int params = bind_parameter_count(stmt);
    

    int[] changes = new int[count];
    try
    {
      for (int i = 0; i < count; i++) {
        reset(stmt);
        for (int j = 0; j < params; j++) {
          if (sqlbind(stmt, j, vals[(i * params + j)]) != 0)
            throwex();
        }
        int rc = step(stmt);
        if (rc != 101) {
          reset(stmt);
          if (rc == 100)
            throw new BatchUpdateException("batch entry " + i + ": query returns results", changes);
          throwex();
        }
        
        changes[i] = changes();
      }
    }
    finally {
      ensureAutoCommit();
    }
    
    reset(stmt);
    return changes;
  }
  
  final synchronized boolean execute(Stmt stmt, Object[] vals) throws SQLException {
    if (vals != null) {
      int params = bind_parameter_count(pointer);
      if (params != vals.length) {
        throw new SQLException("assertion failure: param count (" + params + ") != value count (" + vals.length + ")");
      }
      
      for (int i = 0; i < params; i++) {
        if (sqlbind(pointer, i, vals[i]) != 0)
          throwex();
      }
    }
    int statusCode = step(pointer);
    switch (statusCode) {
    case 101: 
      reset(pointer);
      ensureAutoCommit();
      return false;
    case 100: 
      return true;
    case 5: 
    case 6: 
    case 21: 
      throw newSQLException(statusCode);
    }
    finalize(stmt);
    throw newSQLException(statusCode);
  }
  
  final synchronized boolean execute(String sql)
    throws SQLException
  {
    int statusCode = _exec(sql);
    switch (statusCode) {
    case 0: 
      return false;
    case 101: 
      ensureAutoCommit();
      return false;
    case 100: 
      return true;
    }
    throw newSQLException(statusCode);
  }
  
  final synchronized int executeUpdate(Stmt stmt, Object[] vals) throws SQLException
  {
    if (execute(stmt, vals))
      throw new SQLException("query returns results");
    reset(pointer);
    return changes();
  }
  
  final void throwex() throws SQLException {
    throw new SQLException(errmsg());
  }
  
  final void throwex(int errorCode) throws SQLException {
    throw newSQLException(errorCode);
  }
  
  final void throwex(int errorCode, String errorMessage) throws SQLException {
    throw newSQLException(errorCode, errorMessage);
  }
  
  static SQLException newSQLException(int errorCode, String errorMessage) throws SQLException {
    SQLiteErrorCode code = SQLiteErrorCode.getErrorCode(errorCode);
    return new SQLException(String.format("%s (%s)", new Object[] { code, errorMessage }));
  }
  
  private SQLException newSQLException(int errorCode) throws SQLException {
    return newSQLException(errorCode, errmsg());
  }
  





























  final void ensureAutoCommit()
    throws SQLException
  {
    if (!conn.getAutoCommit()) {
      return;
    }
    if (begin == 0L)
      begin = prepare("begin;");
    if (commit == 0L) {
      commit = prepare("commit;");
    }
    try {
      if (step(begin) != 101)
        return;
      if (step(commit) != 101) {
        reset(commit);
        throwex();
      }
    }
    finally
    {
      reset(begin);
      reset(commit);
    }
  }
  
  public static abstract interface ProgressObserver
  {
    public abstract void progress(int paramInt1, int paramInt2);
  }
}
